(function(ea,sa,Ja,Ka,w,La,s,n,v,p,t,A,Ma,q,G,H,B,E,K,D,O,I,Q,N,L,M,T,P,R,V,u,aa,$,W,ba,ga,ca,ha,X,Y,la,ta,ua,va,ma,wa,xa,ya,za,Aa,Ba,Ca,na,Da,Ea,oa,Fa,Ga,c,j,da,Ha){Jmol.___JmolDate="$Date: 2022-01-05 06:57:40 -0600 (Wed, 05 Jan 2022) $";Jmol.___fullJmolProperties="src/org/jmol/viewer/Jmol.properties";Jmol.___JmolVersion="14.32.9";(function(a){a._Loader.registerPackages("java",["io","lang","lang.reflect","util"]);var b=java.util;a._Loader.ignore("net.sf.j2s.ajax.HttpRequest java.util.MapEntry.Type java.net.UnknownServiceException java.lang.Runtime java.security.AccessController java.security.PrivilegedExceptionAction java.io.File java.io.FileInputStream java.io.FileWriter java.io.OutputStreamWriter java.util.concurrent.Executors".split(" "));
Math.rint=Math.round;Math.log10||(Math.log10=function(a){return Math.log(a)/2.302585092994046});Math.signum||(Math.signum=function(a){return 0==a||isNaN(a)?a:0>a?-1:1});if(a._supportsNativeObject)for(var d=0;d<a._extendedObjectMethods.length-2;d++){var g=a._extendedObjectMethods[d];Array.prototype[g]=a._O.prototype[g];Number.prototype[g]=a._O.prototype[g]}java.lang.Number=Number;Number.__CLASS_NAME__="Number";ha(Number,java.io.Serializable);Number.equals=a._innerFunctions.equals;Number.getName=a._innerFunctions.getName;
Number.prototype.compareTo=function(a){var b=this.value;a=a.value;return b<a?-1:b==a?0:1};c(Number,"shortValue",function(){var a=Math.round(this)&65535;return 0>this&&0<a?a-65536:a});c(Number,"byteValue",function(){var a=Math.round(this)&255;return 0>this&&0<a?a-256:a});c(Number,"intValue",function(){return Math.round(this)&4294967295});c(Number,"longValue",function(){return Math.round(this)});c(Number,"floatValue",function(){return this.valueOf()});c(Number,"doubleValue",function(){return parseFloat(this.valueOf())});
j(Number,"hashCode",function(){return this.valueOf()});java.lang.Integer=Integer=function(){p(this,arguments)};ba(Integer,"Integer",Number,Comparable,null,!0);Integer.prototype.valueOf=function(){return 0};Integer.toString=Integer.prototype.toString=function(){return 0!=arguments.length?""+arguments[0]:this===Integer?"class java.lang.Integer":""+this.valueOf()};Y(Integer,function(a){null==a&&(a=0);"number"!=typeof a&&(a=Integer.parseIntRadix(a,10));this.valueOf=function(){return a}});Integer.MIN_VALUE=
Integer.prototype.MIN_VALUE=-2147483648;Integer.MAX_VALUE=Integer.prototype.MAX_VALUE=2147483647;Integer.TYPE=Integer.prototype.TYPE=Integer;Integer.compare=c(Integer,"compare",function(a,b){return a<b?-1:a>b?1:0},"Number,Number");c(Integer,"bitCount",function(a){a-=a>>>1&1431655765;a=(a&858993459)+(a>>>2&858993459);a=a+(a>>>4)&252645135;a+=a>>>8;return a+(a>>>16)&63},"Number");Integer.bitCount=Integer.prototype.bitCount;c(Integer,"numberOfLeadingZeros",function(a){if(0==a)return 32;var b=1;0==a>>>
16&&(b+=16,a<<=16);0==a>>>24&&(b+=8,a<<=8);0==a>>>28&&(b+=4,a<<=4);0==a>>>30&&(b+=2,a<<=2);return b-(a>>>31)},"Number");Integer.numberOfLeadingZeros=Integer.prototype.numberOfLeadingZeros;c(Integer,"numberOfTrailingZeros",function(a){if(0==a)return 32;var b=31,d=a<<16;0!=d&&(b-=16,a=d);d=a<<8;0!=d&&(b-=8,a=d);d=a<<4;0!=d&&(b-=4,a=d);d=a<<2;0!=d&&(b-=2,a=d);return b-(a<<1>>>31)},"Number");Integer.numberOfTrailingZeros=Integer.prototype.numberOfTrailingZeros;c(Integer,"parseIntRadix",function(a,b){if(null==
a)throw new NumberFormatException("null");if(2>b)throw new NumberFormatException("radix "+b+" less than Character.MIN_RADIX");if(36<b)throw new NumberFormatException("radix "+b+" greater than Character.MAX_RADIX");if(10==b)for(var d=a.length;0<=--d;){var g=a.charCodeAt(d);if(!(48<=g&&57>=g)&&(0<d||43!=g&&45!=g))throw new NumberFormatException("Not a Number : "+a);}d=parseInt(a,b);if(isNaN(d))throw new NumberFormatException("Not a Number : "+a);return d},"String, Number");Integer.parseIntRadix=Integer.prototype.parseIntRadix;
c(Integer,"parseInt",function(a){return Integer.parseIntRadix(a,10)},"String");Integer.parseInt=Integer.prototype.parseInt;j(Integer,"$valueOf",function(a){return new Integer(a)});Integer.$valueOf=Integer.prototype.$valueOf;j(Integer,"equals",function(a){return null==a||!n(a,Integer)?!1:a.valueOf()==this.valueOf()},"Object");Integer.toHexString=Integer.prototype.toHexString=function(a){a.valueOf&&(a=a.valueOf());if(0>a){var b=a&16777215;return(a>>24&255)._numberToString(16)+(b="000000"+b._numberToString(16)).substring(b.length-
6)}return a._numberToString(16)};Integer.toOctalString=Integer.prototype.toOctalString=function(a){a.valueOf&&(a=a.valueOf());return a._numberToString(8)};Integer.toBinaryString=Integer.prototype.toBinaryString=function(a){a.valueOf&&(a=a.valueOf());return a._numberToString(2)};Integer.decodeRaw=c(Integer,"decodeRaw",function(a){0<=a.indexOf(".")&&(a="");var b=a.startsWith("-")?1:0;a=a.replace(/\#/,"0x").toLowerCase();b=a.startsWith("0x",b)?16:a.startsWith("0",b)?8:10;a=Number(a)&4294967295;return 8==
b?parseInt(a,8):a},"~S");Integer.decode=c(Integer,"decode",function(a){a=Integer.decodeRaw(a);if(isNaN(a)||a<Integer.MIN_VALUE||a>Integer.MAX_VALUE)throw new NumberFormatException("Invalid Integer");return new Integer(a)},"~S");j(Integer,"hashCode",function(){return this.valueOf()});java.lang.Long=Long=function(){p(this,arguments)};ba(Long,"Long",Number,Comparable,null,!0);Long.prototype.valueOf=function(){return 0};Long.toString=Long.prototype.toString=function(){return 0!=arguments.length?""+arguments[0]:
this===Long?"class java.lang.Long":""+this.valueOf()};Y(Long,function(a){null==a&&(a=0);a="number"==typeof a?Math.round(a):Integer.parseIntRadix(a,10);this.valueOf=function(){return a}});Long.TYPE=Long.prototype.TYPE=Long;c(Long,"parseLong",function(a,b){return Integer.parseInt(a,b||10)});Long.parseLong=Long.prototype.parseLong;j(Long,"$valueOf",function(a){return new Long(a)});Long.$valueOf=Long.prototype.$valueOf;j(Long,"equals",function(a){return null==a||!n(a,Long)?!1:a.valueOf()==this.valueOf()},
"Object");Long.toHexString=Long.prototype.toHexString=function(a){return a.toString(16)};Long.toOctalString=Long.prototype.toOctalString=function(a){return a.toString(8)};Long.toBinaryString=Long.prototype.toBinaryString=function(a){return a.toString(2)};Long.decode=c(Long,"decode",function(a){a=Integer.decodeRaw(a);if(isNaN(a))throw new NumberFormatException("Invalid Long");return new Long(a)},"~S");java.lang.Short=Short=function(){p(this,arguments)};ba(Short,"Short",Number,Comparable,null,!0);Short.prototype.valueOf=
function(){return 0};Short.toString=Short.prototype.toString=function(){return 0!=arguments.length?""+arguments[0]:this===Short?"class java.lang.Short":""+this.valueOf()};Y(Short,function(a){null==a&&(a=0);"number"!=typeof a&&(a=Integer.parseIntRadix(a,10));a=a.shortValue();this.valueOf=function(){return a}});Short.MIN_VALUE=Short.prototype.MIN_VALUE=-32768;Short.MAX_VALUE=Short.prototype.MAX_VALUE=32767;Short.TYPE=Short.prototype.TYPE=Short;c(Short,"parseShortRadix",function(a,b){return Integer.parseIntRadix(a,
b).shortValue()},"String, Number");Short.parseShortRadix=Short.prototype.parseShortRadix;c(Short,"parseShort",function(a){return Short.parseShortRadix(a,10)},"String");Short.parseShort=Short.prototype.parseShort;j(Short,"$valueOf",function(a){return new Short(a)});Short.$valueOf=Short.prototype.$valueOf;j(Short,"equals",function(a){return null==a||!n(a,Short)?!1:a.valueOf()==this.valueOf()},"Object");Short.toHexString=Short.prototype.toHexString=function(a){return a.toString(16)};Short.toOctalString=
Short.prototype.toOctalString=function(a){return a.toString(8)};Short.toBinaryString=Short.prototype.toBinaryString=function(a){return a.toString(2)};Short.decode=c(Short,"decode",function(a){a=Integer.decodeRaw(a);if(isNaN(a)||-32768>a||32767<a)throw new NumberFormatException("Invalid Short");return new Short(a)},"~S");java.lang.Byte=Byte=function(){p(this,arguments)};ba(Byte,"Byte",Number,Comparable,null,!0);Byte.prototype.valueOf=function(){return 0};Byte.toString=Byte.prototype.toString=function(){return 0!=
arguments.length?""+arguments[0]:this===Byte?"class java.lang.Byte":""+this.valueOf()};q(Byte,function(a){"number"!=typeof a&&(a=Integer.parseIntRadix(a,10));a=a.byteValue();this.valueOf=function(){return a}});Byte.serialVersionUID=Byte.prototype.serialVersionUID=-0x63b19f7b11af0c00;Byte.MIN_VALUE=Byte.prototype.MIN_VALUE=-128;Byte.MAX_VALUE=Byte.prototype.MAX_VALUE=127;Byte.SIZE=Byte.prototype.SIZE=8;Byte.TYPE=Byte.prototype.TYPE=Byte;c(Byte,"parseByteRadix",function(a,b){return Integer.parseIntRadix(a,
b).byteValue()},"String, Number");Byte.parseByteRadix=Byte.prototype.parseByteRadix;c(Byte,"parseByte",function(a){return Byte.parseByte(a,10)},"String");Byte.parseByte=Byte.prototype.parseByte;j(Byte,"$valueOf",function(a){return new Byte(a)});Byte.$valueOf=Byte.prototype.$valueOf;j(Byte,"equals",function(a){return null==a||!n(a,Byte)?!1:a.valueOf()==this.valueOf()},"Object");Byte.toHexString=Byte.prototype.toHexString=function(a){return a.toString(16)};Byte.toOctalString=Byte.prototype.toOctalString=
function(a){return a.toString(8)};Byte.toBinaryString=Byte.prototype.toBinaryString=function(a){return a.toString(2)};Byte.decode=c(Byte,"decode",function(a){a=Integer.decodeRaw(a);if(isNaN(a)||-128>a||127<a)throw new NumberFormatException("Invalid Byte");return new Byte(a)},"~S");a._floatToString=function(a){a=""+a;0>a.indexOf(".")&&0>a.indexOf("e")&&(a+=".0");return a};java.lang.Float=Float=function(){p(this,arguments)};ba(Float,"Float",Number,Comparable,null,!0);Float.prototype.valueOf=function(){return 0};
Float.toString=Float.prototype.toString=function(){return 0!=arguments.length?a._floatToString(arguments[0]):this===Float?"class java.lang.Float":a._floatToString(this.valueOf())};a._a32=null;Float.floatToIntBits=function(b){var d=a._a32||(a._a32=new Float32Array(1));d[0]=b;return(new Int32Array(d.buffer))[0]};Y(Float,function(a){null==a&&(a=0);"number"!=typeof a&&(a=Number(a));this.valueOf=function(){return a}});Float.serialVersionUID=Float.prototype.serialVersionUID=-0x2512365d24c31000;Float.MIN_VALUE=
Float.prototype.MIN_VALUE=1.4E-45;Float.MAX_VALUE=Float.prototype.MAX_VALUE=3.4028235E38;Float.NEGATIVE_INFINITY=Number.NEGATIVE_INFINITY;Float.POSITIVE_INFINITY=Number.POSITIVE_INFINITY;Float.NaN=Number.NaN;Float.TYPE=Float.prototype.TYPE=Float;c(Float,"parseFloat",function(a){if(null==a)throw new NumberFormatException("null");if("number"==typeof a)return a;var b=Number(a);if(isNaN(b))throw new NumberFormatException("Not a Number : "+a);return b},"String");Float.parseFloat=Float.prototype.parseFloat;
j(Float,"$valueOf",function(a){return new Float(a)});Float.$valueOf=Float.prototype.$valueOf;c(Float,"isNaN",function(a){return isNaN(1==arguments.length?a:this.valueOf())},"Number");Float.isNaN=Float.prototype.isNaN;c(Float,"isInfinite",function(a){return!isFinite(1==arguments.length?a:this.valueOf())},"Number");Float.isInfinite=Float.prototype.isInfinite;j(Float,"equals",function(a){return null==a||!n(a,Float)?!1:a.valueOf()==this.valueOf()},"Object");java.lang.Double=Double=function(){p(this,arguments)};
ba(Double,"Double",Number,Comparable,null,!0);Double.prototype.valueOf=function(){return 0};Double.toString=Double.prototype.toString=function(){return 0!=arguments.length?a._floatToString(arguments[0]):this===Double?"class java.lang.Double":a._floatToString(this.valueOf())};Y(Double,function(a){null==a&&(a=0);"number"!=typeof a&&(a=Double.parseDouble(a));this.valueOf=function(){return a}});Double.serialVersionUID=Double.prototype.serialVersionUID=-0x7f4c3db5d6940400;Double.MIN_VALUE=Double.prototype.MIN_VALUE=
4.9E-324;Double.MAX_VALUE=Double.prototype.MAX_VALUE=1.7976931348623157E308;Double.NEGATIVE_INFINITY=Number.NEGATIVE_INFINITY;Double.POSITIVE_INFINITY=Number.POSITIVE_INFINITY;Double.NaN=Number.NaN;Double.TYPE=Double.prototype.TYPE=Double;c(Double,"isNaN",function(a){return isNaN(1==arguments.length?a:this.valueOf())},"Number");Double.isNaN=Double.prototype.isNaN;c(Double,"isInfinite",function(a){return!isFinite(1==arguments.length?a:this.valueOf())},"Number");Double.isInfinite=Double.prototype.isInfinite;
c(Double,"parseDouble",function(a){if(null==a)throw new NumberFormatException("null");if("number"==typeof a)return a;var b=Number(a);if(isNaN(b))throw new NumberFormatException("Not a Number : "+a);return b},"String");Double.parseDouble=Double.prototype.parseDouble;c(Double,"$valueOf",function(a){return new Double(a)},"Number");Double.$valueOf=Double.prototype.$valueOf;j(Double,"equals",function(a){return null==a||!n(a,Double)?!1:a.valueOf()==this.valueOf()},"Object");Boolean=java.lang.Boolean=Boolean||
function(){p(this,arguments)};if(a._supportsNativeObject)for(d=0;d<a._extendedObjectMethods.length;d++)g=a._extendedObjectMethods[d],Boolean.prototype[g]=a._O.prototype[g];Boolean.__CLASS_NAME__="Boolean";ha(Boolean,[java.io.Serializable,java.lang.Comparable]);Boolean.equals=a._innerFunctions.equals;Boolean.getName=a._innerFunctions.getName;Boolean.serialVersionUID=Boolean.prototype.serialVersionUID=-0x32df8d7f2a630600;Y(Boolean,function(a){var b=("string"==typeof a?Boolean.toBoolean(a):a)?!0:!1;
this.valueOf=function(){return b}},"~O");Boolean.parseBoolean=c(Boolean,"parseBoolean",function(a){return Boolean.toBoolean(a)},"~S");c(Boolean,"booleanValue",function(){return this.valueOf()});Boolean.$valueOf=j(Boolean,"$valueOf",function(a){return("string"==typeof a?"true".equalsIgnoreCase(a):a)?Boolean.TRUE:Boolean.FALSE});j(Boolean,"toString",function(){return this.valueOf()?"true":"false"});j(Boolean,"hashCode",function(){return this.valueOf()?1231:1237});j(Boolean,"equals",function(a){return n(a,
Boolean)?this.booleanValue()==a.booleanValue():!1},"~O");Boolean.getBoolean=c(Boolean,"getBoolean",function(a){var b=!1;try{b=Boolean.toBoolean(Zystem.getProperty(a))}catch(d){if(!n(d,IllegalArgumentException)&&!n(d,NullPointerException))throw d;}return b},"~S");j(Boolean,"compareTo",function(a){return a.value==this.value?0:this.value?1:-1},"Boolean");Boolean.toBoolean=c(Boolean,"toBoolean",($fz=function(a){return null!=a&&a.equalsIgnoreCase("true")},$fz.isPrivate=!0,$fz),"~S");Boolean.TRUE=Boolean.prototype.TRUE=
new Boolean(!0);Boolean.FALSE=Boolean.prototype.FALSE=new Boolean(!1);Boolean.TYPE=Boolean.prototype.TYPE=Boolean;a._Encoding={};(function(b){b.UTF8="utf-8";b.UTF16="utf-16";b.ASCII="ascii";b.guessEncoding=function(a){return 239==a.charCodeAt(0)&&187==a.charCodeAt(1)&&191==a.charCodeAt(2)?b.UTF8:255==a.charCodeAt(0)&&254==a.charCodeAt(1)?b.UTF16:b.ASCII};b.guessEncodingArray=function(a){return 239==a[0]&&187==a[1]&&191==a[2]?b.UTF8:255==a[0]&&254==a[1]?b.UTF16:b.ASCII};b.readUTF8=function(a){if("string"!=
typeof a)return b.readUTF8Array(a);var d=b.guessEncoding(a),g=0;d==b.UTF8?g=3:d==b.UTF16&&(g=2);for(d=[];g<a.length;g++){var c=a.charCodeAt(g);if(128>c)d[d.length]=a.charAt(g);else if(192<c&&224>c){c&=31;g++;var e=a.charCodeAt(g)&63,c=(c<<6)+e;d[d.length]=String.fromCharCode(c)}else if(224<=c){c&=15;g++;e=a.charCodeAt(g)&63;g++;var h=a.charCodeAt(g)&63,c=(c<<12)+(e<<6)+h;d[d.length]=String.fromCharCode(c)}}return d.join("")};b.readUTF8Array=function(a){var d=b.guessEncodingArray(a),g=0;d==b.UTF8?
g=3:d==b.UTF16&&(g=2);for(d=[];g<a.length;g++){var c=a[g];if(128>c)d[d.length]=String.fromCharCode(c);else if(192<c&&224>c){var c=c&31,e=a[++g]&63,c=(c<<6)+e;d[d.length]=String.fromCharCode(c)}else if(224<=c){var c=c&15,e=a[++g]&63,h=a[++g]&63,c=(c<<12)+(e<<6)+h;d[d.length]=String.fromCharCode(c)}}return d.join("")};b.convert2UTF8=function(a){var d=this.guessEncoding(a),g=0;if(d==b.UTF8)return a;d==b.UTF16&&(g=2);for(var d=Array(0+a.length-g),c=g;c<a.length;c++){var e=a.charCodeAt(c);if(128>e)d[0+
c-g]=a.charAt(c);else if(2047>=e){var h=192+((e&1984)>>6),k=128+(e&63);d[0+c-g]=String.fromCharCode(h)+String.fromCharCode(k)}else h=224+((e&61440)>>12),k=128+((e&4032)>>6),e=128+(e&63),d[0+c-g]=String.fromCharCode(h)+String.fromCharCode(k)+String.fromCharCode(e)}return d.join("")};b.base64Chars="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".split("");b.encodeBase64=function(a){if(null==a||0==a.length)return a;for(var d=b.base64Chars,g=a.length,c=0,e=[],h,k;c<g;)h=a.charCodeAt(c++),
e[e.length]=d[h>>2],c<g?(k=a.charCodeAt(c++),e[e.length]=d[h<<4&48|k>>4],c<g?(h=a.charCodeAt(c++),e[e.length]=d[k<<2&60|h>>6],e[e.length]=d[h&63]):(e[e.length]=d[k<<2&60],e[e.length]="=")):(e[e.length]=d[h<<4&48],e[e.length]="=",e[e.length]="=");return e.join("")};b.decodeBase64=function(a){if(null==a||0==a.length)return a;var d=b.base64Chars,g=b.xBase64Chars;if(null==b.xBase64Chars){for(var g={},c=0;c<d.length;c++)g[d[c]]=c;b.xBase64Chars=g}for(var d=a.length,c=0,e=[],h,k,j,n,p=0;c<d&&6E4>p++;)h=
g[a.charAt(c++)],k=g[a.charAt(c++)],j=g[a.charAt(c++)],n=g[a.charAt(c++)],e[e.length]=String.fromCharCode(h<<2&255|k>>4),null!=j&&(e[e.length]=String.fromCharCode(k<<4&255|j>>2),null!=n&&(e[e.length]=String.fromCharCode(j<<6&255|n)));return e.join("")};if(null==String.prototype.$replace){java.lang.String=String;if(a._supportsNativeObject)for(var d=0;d<a._extendedObjectMethods.length;d++){var g=a._extendedObjectMethods[d];"to$tring"==g||("toString"==g||"equals"==g||"hashCode"==g)||(String.prototype[g]=
a._O.prototype[g])}ha(String,[java.io.Serializable,CharSequence,Comparable]);String.getName=a._innerFunctions.getName;String.serialVersionUID=String.prototype.serialVersionUID=-0x5f0f5bc785c44c00;(function(a){a.$replace=function(a,b){if(a==b||0>this.indexOf(a))return""+this;1==a.length?0<="\\$.*+|?^{}()[]".indexOf(a)&&(a="\\"+a):a=a.replace(/([\\\$\.\*\+\|\?\^\{\}\(\)\[\]])/g,function(a,b){return"\\"+b});return this.replace(RegExp(a,"gm"),b)};a.$generateExpFunction=function(a){var b=[],d=[],g=0;b[0]=
"";for(var c=0;c<a.length;c++){var f=a.charAt(c);c!=a.length-1&&"\\"==f?(c++,f=a.charAt(c),"\\"==f&&(b[g]+="\\"),b[g]+=f):c!=a.length-1&&"$"==f?(c++,d[g]=parseInt(a.charAt(c)),g++,b[g]=""):b[g]="\r"==f?b[g]+"\\r":"\n"==f?b[g]+"\\n":"\t"==f?b[g]+"\\t":'"'==f?b[g]+'\\"':b[g]+f}a="f = function (";g=Math.max.apply({},d);for(c=0;c<=g;c++)a+="$"+c,c!=g&&(a+=", ");a+=") { return ";for(c=0;c<b.length-1;c++)a+='"'+b[c]+'" + $'+d[c]+" + ";a+='"'+b[c]+'"; }';eval(a);return null};a.replaceAll=function(a,b){return this.replace(RegExp(a,
"gm"),this.$generateExpFunction(b))};a.replaceFirst=function(a,b){return this.replace(RegExp(a,"m"),this.$generateExpFunction(b))};a.matches=function(a){null!=a&&(a="^("+a+")$");a=this.match(RegExp(a,"gm"));return null!=a&&0!=a.length};a.regionMatches=function(a,b,d,g,c){if("number"==typeof a||!0!=a&&!1!=a)c=g,g=d,d=b,b=a,a=!1;if(0>g||0>b||b>this.length-c||g>d.length-c)return!1;b=this.substring(b,b+c);d=d.substring(g,g+c);a&&(b=b.toLowerCase(),d=d.toLowerCase());return b==d};a.$plit=function(a,b){if(!b&&
" "==a)return this.split(a);if(null!=b&&0<b){if(1==b)return this;var d=RegExp("("+a+")","gm"),g=1,c=this.replace(d,function(a){g++;return g==b?"@@_@@":a}),d=RegExp(a,"gm"),d=this.split(d);d.length>b&&(d[b-1]=c.substring(c.indexOf("@@_@@")+5),d.length=b);return d}d=RegExp(a,"gm");return this.split(d)};a.trim||(a.trim=function(){return this.replace(/^\s+/g,"").replace(/\s+$/g,"")});if(!a.startsWith||!a.endsWith){var d=function(a,b,d){var g=d,c=0,f=b.length;if(0>d||d>a.length-f)return!1;for(;0<=--f;)if(a.charAt(g++)!=
b.charAt(c++))return!1;return!0};a.startsWith=function(a){return 1==arguments.length?d(this,arguments[0],0):2==arguments.length?d(this,arguments[0],arguments[1]):!1};a.endsWith=function(a){return d(this,a,this.length-a.length)}}a.equals=function(a){return this.valueOf()==a};a.equalsIgnoreCase=function(a){return null==a?!1:this==a||this.toLowerCase()==a.toLowerCase()};a.hash=0;a.hashCode=function(){var a=this.hash;if(0==a){for(var b=0,d=this.length,g=0;g<d;g++)a=31*a+this.charCodeAt(b++),a&=4294967295;
this.hash=a}return a};a.getBytes=function(){if(4==arguments.length)return this.getChars(arguments[0],arguments[1],arguments[2],arguments[3]);var a=this;if(1==arguments.length){for(var d=arguments[0].toString().toLowerCase(),g="utf-8 UTF8 us-ascii iso-8859-1 8859_1 gb2312 gb18030 gbk".split(" "),c=!1,e=0;e<g.length;e++)if(g[e]==d){c=!0;break}if(!c)throw new java.io.UnsupportedEncodingException;if("utf-8"==d||"utf8"==d)a=b.convert2UTF8(this)}d=Array(a.length);for(e=c=g=0;e<a.length;e++)g=a.charCodeAt(e),
255<g?(d[c]=26,d[c+1]=g&255,d[c+2]=(g&65280)>>8,c+=2):d[c]=g,c++;return Q(d)};a.contains=function(a){return 0<=this.indexOf(a)};a.compareTo=function(a){return this>a?1:this<a?-1:0};a.toCharArray=function(){for(var a=Array(this.length),b=0;b<this.length;b++)a[b]=this.charAt(b);return a};String.value0f=String.valueOf;String.valueOf=function(a){if("undefined"==a)return String.value0f();if(a instanceof Array){if(1==arguments.length)return a.join("");for(var b=arguments[1],d=arguments[2],g=Array(d),c=
0;c<d;c++)g[c]=a[b+c];return g.join("")}return""+a};a.subSequence=function(a,b){return this.substring(a,b)};a.compareToIgnoreCase=function(a){if(null==a)throw new NullPointerException;var b=this.toUpperCase(),d=a.toUpperCase();if(b==d)return 0;b=this.toLowerCase();d=a.toLowerCase();return b==d?0:b>d?1:-1};a.contentEquals=function(a){if(this.length!=a.length())return!1;a=a.getValue();for(var b=0,d=0,g=this.length;0!=g--;)if(this.charCodeAt(b++)!=a[d++])return!1;return!0};a.getChars=function(a,b,d,
g){if(0>a)throw new StringIndexOutOfBoundsException(a);if(b>this.length)throw new StringIndexOutOfBoundsException(b);if(a>b)throw new StringIndexOutOfBoundsException(b-a);if(null==d)throw new NullPointerException;for(var c=0;c<b-a;c++)d[g+c]=this.charAt(a+c)};a.$concat=a.concat;a.concat=function(a){if(null==a)throw new NullPointerException;return this.$concat(a)};a.$lastIndexOf=a.lastIndexOf;a.lastIndexOf=function(a,b){return null!=b&&0>=b+this.length?-1:null!=b?this.$lastIndexOf(a,b):this.$lastIndexOf(a)};
a.intern=function(){return this.valueOf()};String.copyValueOf=a.copyValueOf=function(){return 1==arguments.length?String.instantialize(arguments[0]):String.instantialize(arguments[0],arguments[1],arguments[2])};a.codePointAt||(a.codePointAt=a.charCodeAt)})(String.prototype);var c=new TextDecoder;String.instantialize=function(){switch(arguments.length){case 0:return new String;case 1:var a=arguments[0];if(a.BYTES_PER_ELEMENT)return 0==a.length?"":"number"==typeof a[0]?c.decode(a):a.join("");if(a instanceof
Array)return 0==a.length?"":"number"==typeof a[0]?c.decode(new Uint8Array(a)):a.join("");if("string"==typeof a||a instanceof String)return new String(a);if("StringBuffer"==a.__CLASS_NAME__||"java.lang.StringBuffer"==a.__CLASS_NAME__){for(var b=a.shareValue(),d=a.length(),g=Array(d),a=0;a<d;a++)g[a]=b[a];return g.join("")}return""+a;case 2:return a=arguments[0],d=arguments[1],"string"==typeof d?String.instantialize(a,0,a.length,d):String.instantialize(a,d,0,a.length);case 3:var g=arguments[0],f=arguments[1],
d=arguments[2];arguments[2]instanceof Array&&(g=arguments[2],f=arguments[0],d=arguments[1]);b=Array(d);if(0>f||d+f>g.length)throw new IndexOutOfBoundsException;if(0<d)if(null!=g[f].length)for(a=0;a<d;a++)b[a]=g[f+a];else for(a=0;a<d;a++)b[a]=String.fromCharCode(g[f+a]);return b.join("");case 4:g=arguments[0];b=arguments[3];if("string"==typeof b||b instanceof String){f=arguments[1];d=arguments[2];b=new Uint8Array(d);for(a=0;a<d;a++)b[a]=g[f+a];return c.decode(b)}a=arguments[3];f=arguments[2];d=arguments[1];
b=Array(a);if(0==d)for(;0<a--;)b[a]=String.fromCharCode(g[a+f]&255);else for(d<<=8;0<a--;)b[a]=String.fromCharCode(d|g[a+f]&255);return b.join("");default:b="";for(a=0;a<arguments.length;a++)b+=arguments[a];return b}};-1!=navigator.userAgent.toLowerCase().indexOf("chrome")&&(String.prototype.toString=function(){return this.valueOf()})}})(a._Encoding);c$=t(function(){this.value=0;p(this,arguments)},java.lang,"Character",null,[java.io.Serializable,Comparable]);q(c$,function(a){this.value=a},"~N");c(c$,
"charValue",function(){return this.value});j(c$,"hashCode",function(){return this.value.charCodeAt(0)});j(c$,"equals",function(a){return n(a,Character)?this.value.charCodeAt(0)==a.charValue().charCodeAt(0):!1},"~O");j(c$,"compareTo",function(a){return this.value.charCodeAt(0)-a.value.charCodeAt(0)},"Character");c$.toLowerCase=c(c$,"toLowerCase",function(a){return(""+a).toLowerCase().charAt(0)},"~N");c$.toUpperCase=c(c$,"toUpperCase",function(a){return(""+a).toUpperCase().charAt(0)},"~N");c$.isDigit=
c(c$,"isDigit",function(a){a=a.charCodeAt(0);return 48<=a&&57>=a},"~N");c$.isUpperCase=c(c$,"isUpperCase",function(a){a=a.charCodeAt(0);return 65<=a&&90>=a},"~N");c$.isLowerCase=c(c$,"isLowerCase",function(a){a=a.charCodeAt(0);return 97<=a&&122>=a},"~N");c$.isWhitespace=c(c$,"isWhitespace",function(a){a=a.charCodeAt(0);return 28<=a&&32>=a||9<=a&&13>=a||5760==a||8192<=a&&8199!=a&&(8203>=a||8232==a||8233==a||12288==a)},"~N");c$.isLetter=c(c$,"isLetter",function(a){a=a.charCodeAt(0);return 65<=a&&90>=
a||97<=a&&122>=a},"~N");c$.isLetterOrDigit=c(c$,"isLetterOrDigit",function(a){a=a.charCodeAt(0);return 65<=a&&90>=a||97<=a&&122>=a||48<=a&&57>=a},"~N");c$.isSpaceChar=c(c$,"isSpaceChar",function(a){a=a.charCodeAt(0);return 32==a||160==a||5760==a?!0:8192>a?!1:8203>=a||8232==a||8233==a||8239==a||12288==a},"~N");c$.digit=c(c$,"digit",function(a,b){var d=a.charCodeAt(0);if(2<=b&&36>=b&&128>d){var g=-1;48<=d&&57>=d?g=d-48:97<=d&&122>=d?g=d-87:65<=d&&90>=d&&(g=d-55);return g<b?g:-1}return-1},"~N,~N");j(c$,
"toString",function(){return String.valueOf([this.value])});c$.toString=j(c$,"toString",function(a){return this===Character?"class java.lang.Character":String.valueOf(a)},"~N");E(c$,"MIN_VALUE","\x00","MAX_VALUE","\uffff","MIN_RADIX",2,"MAX_RADIX",36,"TYPE",null);java.lang.Character.TYPE=java.lang.Character.prototype.TYPE=java.lang.Character;a._ArrayWrapper=function(a,b){return{a:a,__CLASS_NAME__:"Array",superClazz:Array,getComponentType:function(){return b},instanceOf:function(a){return n(b,a)},
getName:function(){return this.__CLASS_NAME__}}};c$=D(java.lang.reflect,"Array");c$.newInstance=c(c$,"newInstance",function(b,d){var g=u(d);g.getClass=function(){return new a._ArrayWrapper(this,b)};return g},"Class,~N");b.Date=Date;Date.TYPE="javautil.Date";Date.__CLASS_NAME__="Date";ha(Date,[java.io.Serializable,java.lang.Comparable]);c(b.Date,"clone",function(){return new Date(this.getTime())});c(b.Date,"before",function(a){return this.getTime()<a.getTime()},"javautil.Date");c(b.Date,"after",function(a){return this.getTime()>
a.getTime()},"javautil.Date");c(b.Date,"equals",function(a){return n(a,b.Date)&&this.getTime()==a.getTime()},"Object");c(b.Date,"compareTo",function(a){var b=this.getTime();a=a.getTime();return b<a?-1:b==a?0:1},"javautil.Date");c(b.Date,"compareTo",function(a){return this.compareTo(a)},"Object");j(b.Date,"hashCode",function(){var a=this.getTime();return parseInt(a)^parseInt(a>>32)});c$=t(function(){this.source=null;p(this,arguments)},b,"EventObject",null,java.io.Serializable);q(c$,function(a){if(null!=
a)this.source=a;else throw new IllegalArgumentException;},"~O");c(c$,"getSource",function(){return this.source});j(c$,"toString",function(){return this.getClass().getName()+"[source="+String.valueOf(this.source)+"]"});N(b,"EventListener");c$=t(function(){this.listener=null;p(this,arguments)},b,"EventListenerProxy",null,b.EventListener);q(c$,function(a){this.listener=a},"javautil.EventListener");c(c$,"getListener",function(){return this.listener});N(b,"Iterator");N(b,"ListIterator",b.Iterator);N(b,
"Enumeration");N(b,"Collection",Iterable);N(b,"Set",b.Collection);N(b,"Map");N(b.Map,"Entry");N(b,"List",b.Collection);N(b,"Queue",b.Collection);N(b,"RandomAccess");c$=t(function(){this.stackTrace=this.cause=this.detailMessage=null;p(this,arguments)},java.lang,"Throwable",null,java.io.Serializable);O(c$,function(){this.cause=this});q(c$,function(){this.fillInStackTrace()});q(c$,function(a){this.fillInStackTrace();this.detailMessage=a},"~S");q(c$,function(a,b){this.fillInStackTrace();this.detailMessage=
a;this.cause=b},"~S,Throwable");q(c$,function(a){this.fillInStackTrace();this.detailMessage=null==a?null:a.toString();this.cause=a},"Throwable");c(c$,"getMessage",function(){return this.message||this.detailMessage||this.toString()});c(c$,"getLocalizedMessage",function(){return this.getMessage()});c(c$,"getCause",function(){return this.cause===this?null:this.cause});c(c$,"initCause",function(a){if(this.cause!==this)throw new IllegalStateException("Can't overwrite cause");if(a===this)throw new IllegalArgumentException("Self-causation not permitted");
this.cause=a;return this},"Throwable");j(c$,"toString",function(){var a=this.getClass().getName(),b=this.message||this.detailMessage;return b?a+": "+b:a});c(c$,"printStackTrace",function(){Zystem.err.println(this.getStackTrace?this.getStackTrace():this.message+" "+oa())});c(c$,"getStackTrace",function(){for(var a=""+this+"\n",b=0;b<this.stackTrace.length;b++){var d=this.stackTrace[b],g=d.methodName.indexOf("(");if("construct"!=d.methodName.substring(0,g).replace(/\s+/g,"")||null==d.nativeClazz||0>
ua(d.nativeClazz,Throwable))a+=d+"\n"}return a});c(c$,"printStackTrace",function(){this.printStackTrace()},"java.io.PrintStream");c(c$,"printStackTrace",function(){this.printStackTrace()},"java.io.PrintWriter");c(c$,"fillInStackTrace",function(){this.stackTrace=[];for(var b=arguments.callee.caller,d=null,g=[],c=a._callingStackTraces.length-1,l=!0;-1<c||null!=b;){var j=null;if(!l||b==Ea||b==W||null==b){if(0>c)break;l=!0;d=a._callingStackTraces[c].caller;j=a._callingStackTraces[c].owner;c--}else d=
b,null!=d.claxxOwner?j=d.claxxOwner:null!=d.exClazz&&(j=d.exClazz);b=new StackTraceElement(null!=j&&0!=j.__CLASS_NAME__.length?j.__CLASS_NAME__:"anonymous",(null==d.exName?"anonymous":d.exName)+" ("+va(d.arguments)+")",null,-1);b.nativeClazz=j;this.stackTrace[this.stackTrace.length]=b;for(j=0;j<g.length;j++)g[j]==d&&(b=new StackTraceElement("lost","missing",null,-3),b.nativeClazz=null,this.stackTrace[this.stackTrace.length]=b,l=!1);null!=d&&(g[g.length]=d);b=d.arguments.callee.caller}a._initializingException=
!1;return this});c(c$,"setStackTrace",function(a){a=a.clone();for(var b=0;b<a.length;b++)if(null==a[b])throw new NullPointerException("stackTrace["+b+"]");this.stackTrace=a},"~A");c$=t(function(){this.fileName=this.methodName=this.declaringClass=null;this.lineNumber=0;p(this,arguments)},java.lang,"StackTraceElement",null,java.io.Serializable);q(c$,function(a,b,d,g){if(null==a||null==b)throw new NullPointerException;this.declaringClass=a;this.methodName=b;this.fileName=d;this.lineNumber=g},"~S,~S,~S,~N");
j(c$,"equals",function(a){if(!n(a,StackTraceElement)||(null==this.methodName||null==a.methodName)||!this.getMethodName().equals(a.getMethodName())||!this.getClassName().equals(a.getClassName()))return!1;var b=this.getFileName();if(null==b){if(null!=a.getFileName())return!1}else if(!b.equals(a.getFileName()))return!1;return this.getLineNumber()!=a.getLineNumber()?!1:!0},"~O");c(c$,"getClassName",function(){return null==this.declaringClass?"<unknown class>":this.declaringClass});c(c$,"getFileName",
function(){return this.fileName});c(c$,"getLineNumber",function(){return this.lineNumber});c(c$,"getMethodName",function(){return null==this.methodName?"<unknown method>":this.methodName});j(c$,"hashCode",function(){return null==this.methodName?0:this.methodName.hashCode()^this.declaringClass.hashCode()});c(c$,"isNativeMethod",function(){return-2==this.lineNumber});j(c$,"toString",function(){var a=new StringBuilder(80);a.append(this.getClassName());a.append(".");a.append(this.getMethodName());if(this.isNativeMethod())a.append("(Native Method)");
else{var b=this.getFileName();if(null==b)a.append("(Unknown Source)");else{var d=this.getLineNumber();a.append("(");a.append(b);0<=d&&(a.append(":"),a.append(d));a.append(")")}}return a.toString()});TypeError.prototype.getMessage||(TypeError.prototype.getMessage=function(){return(this.message||this.toString())+(this.getStackTrace?this.getStackTrace():oa())});Clazz_Error=Error;Clazz_declareTypeError=function(a,b,d,g,c){return t(function(){p(this,arguments);return Clazz_Error()},a,b,d,g,c)};a._Error||
(a._Error=Error);t(function(){p(this,arguments);return a._Error()},java.lang,"Error",Throwable);c$=D(java.lang,"LinkageError",Error);c$=D(java.lang,"IncompatibleClassChangeError",LinkageError);c$=D(java.lang,"AbstractMethodError",IncompatibleClassChangeError);c$=D(java.lang,"AssertionError",Error);q(c$,function(a){I(this,AssertionError,[String.valueOf(a),n(a,Throwable)?a:null])},"~O");q(c$,function(a){this.construct(""+a)},"~B");q(c$,function(a){this.construct(""+a)},"~N");c$=D(java.lang,"ClassCircularityError",
LinkageError);c$=D(java.lang,"ClassFormatError",LinkageError);c$=t(function(){this.exception=null;p(this,arguments)},java.lang,"ExceptionInInitializerError",LinkageError);q(c$,function(){I(this,ExceptionInInitializerError);this.initCause(null)});q(c$,function(a){I(this,ExceptionInInitializerError,[a]);this.initCause(null)},"~S");q(c$,function(a){I(this,ExceptionInInitializerError);this.exception=a;this.initCause(a)},"Throwable");c(c$,"getException",function(){return this.exception});j(c$,"getCause",
function(){return this.exception});c$=D(java.lang,"IllegalAccessError",IncompatibleClassChangeError);c$=D(java.lang,"InstantiationError",IncompatibleClassChangeError);c$=D(java.lang,"VirtualMachineError",Error);c$=D(java.lang,"InternalError",VirtualMachineError);c$=D(java.lang,"NoClassDefFoundError",LinkageError);c$=D(java.lang,"NoSuchFieldError",IncompatibleClassChangeError);c$=D(java.lang,"NoSuchMethodError",IncompatibleClassChangeError);c$=D(java.lang,"OutOfMemoryError",VirtualMachineError);c$=
D(java.lang,"StackOverflowError",VirtualMachineError);c$=D(java.lang,"UnknownError",VirtualMachineError);c$=D(java.lang,"UnsatisfiedLinkError",LinkageError);c$=D(java.lang,"UnsupportedClassVersionError",ClassFormatError);c$=D(java.lang,"VerifyError",LinkageError);c$=D(java.lang,"ThreadDeath",Error);q(c$,function(){I(this,ThreadDeath,[])});c$=D(java.lang,"Exception",Throwable);c$=D(java.lang,"RuntimeException",Exception);c$=D(java.lang,"ArithmeticException",RuntimeException);c$=D(java.lang,"IndexOutOfBoundsException",
RuntimeException);c$=D(java.lang,"ArrayIndexOutOfBoundsException",IndexOutOfBoundsException);q(c$,function(a){I(this,ArrayIndexOutOfBoundsException,["Array index out of range: "+a])},"~N");c$=D(java.lang,"ArrayStoreException",RuntimeException);c$=D(java.lang,"ClassCastException",RuntimeException);c$=t(function(){this.ex=null;p(this,arguments)},java.lang,"ClassNotFoundException",Exception);q(c$,function(){I(this,ClassNotFoundException,[aa("Throwable")])});q(c$,function(a){I(this,ClassNotFoundException,
[a,null])},"~S");q(c$,function(a,b){I(this,ClassNotFoundException,[a]);this.ex=b},"~S,Throwable");c(c$,"getException",function(){return this.ex});j(c$,"getCause",function(){return this.ex});c$=D(java.lang,"CloneNotSupportedException",Exception);c$=D(java.lang,"IllegalAccessException",Exception);c$=D(java.lang,"IllegalArgumentException",RuntimeException);q(c$,function(a){I(this,IllegalArgumentException,[null==a?null:a.toString(),a])},"Throwable");c$=D(java.lang,"IllegalMonitorStateException",RuntimeException);
c$=D(java.lang,"IllegalStateException",RuntimeException);q(c$,function(a){I(this,IllegalStateException,[null==a?null:a.toString(),a])},"Throwable");c$=D(java.lang,"IllegalThreadStateException",IllegalArgumentException);c$=D(java.lang,"InstantiationException",Exception);c$=D(java.lang,"InterruptedException",Exception);c$=D(java.lang,"NegativeArraySizeException",RuntimeException);c$=D(java.lang,"NoSuchFieldException",Exception);c$=D(java.lang,"NoSuchMethodException",Exception);c$=D(java.lang,"NullPointerException",
RuntimeException);c$=D(java.lang,"NumberFormatException",IllegalArgumentException);c$=D(java.lang,"SecurityException",RuntimeException);q(c$,function(a){I(this,SecurityException,[null==a?null:a.toString(),a])},"Throwable");c$=D(java.lang,"StringIndexOutOfBoundsException",IndexOutOfBoundsException);q(c$,function(a){I(this,StringIndexOutOfBoundsException,["String index out of range: "+a])},"~N");c$=D(java.lang,"UnsupportedOperationException",RuntimeException);q(c$,function(){I(this,UnsupportedOperationException,
[])});q(c$,function(a){I(this,UnsupportedOperationException,[null==a?null:a.toString(),a])},"Throwable");c$=t(function(){this.target=null;p(this,arguments)},java.lang.reflect,"InvocationTargetException",Exception);q(c$,function(){I(this,java.lang.reflect.InvocationTargetException,[aa("Throwable")])});q(c$,function(a){I(this,java.lang.reflect.InvocationTargetException,[null,a]);this.target=a},"Throwable");q(c$,function(a,b){I(this,java.lang.reflect.InvocationTargetException,[b,a]);this.target=a},"Throwable,~S");
c(c$,"getTargetException",function(){return this.target});j(c$,"getCause",function(){return this.target});c$=t(function(){this.undeclaredThrowable=null;p(this,arguments)},java.lang.reflect,"UndeclaredThrowableException",RuntimeException);q(c$,function(a){I(this,java.lang.reflect.UndeclaredThrowableException);this.undeclaredThrowable=a;this.initCause(a)},"Throwable");q(c$,function(a,b){I(this,java.lang.reflect.UndeclaredThrowableException,[b]);this.undeclaredThrowable=a;this.initCause(a)},"Throwable,~S");
c(c$,"getUndeclaredThrowable",function(){return this.undeclaredThrowable});j(c$,"getCause",function(){return this.undeclaredThrowable});c$=D(java.io,"IOException",Exception);c$=D(java.io,"CharConversionException",java.io.IOException);c$=D(java.io,"EOFException",java.io.IOException);c$=D(java.io,"FileNotFoundException",java.io.IOException);c$=t(function(){this.bytesTransferred=0;p(this,arguments)},java.io,"InterruptedIOException",java.io.IOException);c$=D(java.io,"ObjectStreamException",java.io.IOException);
c$=t(function(){this.classname=null;p(this,arguments)},java.io,"InvalidClassException",java.io.ObjectStreamException);q(c$,function(a,b){I(this,java.io.InvalidClassException,[b]);this.classname=a},"~S,~S");c(c$,"getMessage",function(){var a=W(this,java.io.InvalidClassException,"getMessage",[]);null!=this.classname&&(a=this.classname+"; "+a);return a});c$=D(java.io,"InvalidObjectException",java.io.ObjectStreamException);c$=D(java.io,"NotActiveException",java.io.ObjectStreamException);c$=D(java.io,
"NotSerializableException",java.io.ObjectStreamException);c$=t(function(){this.eof=!1;this.length=0;p(this,arguments)},java.io,"OptionalDataException",java.io.ObjectStreamException);c$=D(java.io,"StreamCorruptedException",java.io.ObjectStreamException);c$=D(java.io,"SyncFailedException",java.io.IOException);c$=D(java.io,"UnsupportedEncodingException",java.io.IOException);c$=D(java.io,"UTFDataFormatException",java.io.IOException);c$=t(function(){this.detail=null;p(this,arguments)},java.io,"WriteAbortedException",
java.io.ObjectStreamException);q(c$,function(a,b){I(this,java.io.WriteAbortedException,[a]);this.detail=b;this.initCause(b)},"~S,Exception");c(c$,"getMessage",function(){var a=W(this,java.io.WriteAbortedException,"getMessage",[]);return this.detail?a+"; "+this.detail.toString():a});j(c$,"getCause",function(){return this.detail});c$=D(b,"ConcurrentModificationException",RuntimeException);q(c$,function(){I(this,b.ConcurrentModificationException,[])});c$=D(b,"EmptyStackException",RuntimeException);c$=
t(function(){this.key=this.className=null;p(this,arguments)},b,"MissingResourceException",RuntimeException);q(c$,function(a,d,g){I(this,b.MissingResourceException,[a]);this.className=d;this.key=g},"~S,~S,~S");c(c$,"getClassName",function(){return this.className});c(c$,"getKey",function(){return this.key});c$=D(b,"NoSuchElementException",RuntimeException);c$=D(b,"TooManyListenersException",Exception);c$=D(java.lang,"Void");E(c$,"TYPE",null);java.lang.Void.TYPE=java.lang.Void;N(java.lang.reflect,"GenericDeclaration");
N(java.lang.reflect,"AnnotatedElement");c$=D(java.lang.reflect,"AccessibleObject",null,java.lang.reflect.AnnotatedElement);q(c$,function(){});c(c$,"isAccessible",function(){return!1});c$.setAccessible=c(c$,"setAccessible",function(){},"~A,~B");c(c$,"setAccessible",function(){},"~B");j(c$,"isAnnotationPresent",function(){return!1},"Class");j(c$,"getDeclaredAnnotations",function(){return[]});j(c$,"getAnnotations",function(){return[]});j(c$,"getAnnotation",function(){return null},"Class");c$.marshallArguments=
c(c$,"marshallArguments",function(){return null},"~A,~A");c(c$,"invokeV",function(){},"~O,~A");c(c$,"invokeL",function(){return null},"~O,~A");c(c$,"invokeI",function(){return 0},"~O,~A");c(c$,"invokeJ",function(){return 0},"~O,~A");c(c$,"invokeF",function(){return 0},"~O,~A");c(c$,"invokeD",function(){return 0},"~O,~A");c$.emptyArgs=c$.prototype.emptyArgs=[];N(java.lang.reflect,"InvocationHandler");c$=N(java.lang.reflect,"Member");E(c$,"PUBLIC",0,"DECLARED",1);c$=D(java.lang.reflect,"Modifier");
q(c$,function(){});c$.isAbstract=c(c$,"isAbstract",function(a){return 0!=(a&1024)},"~N");c$.isFinal=c(c$,"isFinal",function(a){return 0!=(a&16)},"~N");c$.isInterface=c(c$,"isInterface",function(a){return 0!=(a&512)},"~N");c$.isNative=c(c$,"isNative",function(a){return 0!=(a&256)},"~N");c$.isPrivate=c(c$,"isPrivate",function(a){return 0!=(a&2)},"~N");c$.isProtected=c(c$,"isProtected",function(a){return 0!=(a&4)},"~N");c$.isPublic=c(c$,"isPublic",function(a){return 0!=(a&1)},"~N");c$.isStatic=c(c$,
"isStatic",function(a){return 0!=(a&8)},"~N");c$.isStrict=c(c$,"isStrict",function(a){return 0!=(a&2048)},"~N");c$.isSynchronized=c(c$,"isSynchronized",function(a){return 0!=(a&32)},"~N");c$.isTransient=c(c$,"isTransient",function(a){return 0!=(a&128)},"~N");c$.isVolatile=c(c$,"isVolatile",function(a){return 0!=(a&64)},"~N");c$.toString=c(c$,"toString",function(a){var b=[];java.lang.reflect.Modifier.isPublic(a)&&(b[b.length]="public");java.lang.reflect.Modifier.isProtected(a)&&(b[b.length]="protected");
java.lang.reflect.Modifier.isPrivate(a)&&(b[b.length]="private");java.lang.reflect.Modifier.isAbstract(a)&&(b[b.length]="abstract");java.lang.reflect.Modifier.isStatic(a)&&(b[b.length]="static");java.lang.reflect.Modifier.isFinal(a)&&(b[b.length]="final");java.lang.reflect.Modifier.isTransient(a)&&(b[b.length]="transient");java.lang.reflect.Modifier.isVolatile(a)&&(b[b.length]="volatile");java.lang.reflect.Modifier.isSynchronized(a)&&(b[b.length]="synchronized");java.lang.reflect.Modifier.isNative(a)&&
(b[b.length]="native");java.lang.reflect.Modifier.isStrict(a)&&(b[b.length]="strictfp");java.lang.reflect.Modifier.isInterface(a)&&(b[b.length]="interface");return 0<b.length?b.join(" "):""},"~N");E(c$,"PUBLIC",1,"PRIVATE",2,"PROTECTED",4,"STATIC",8,"FINAL",16,"SYNCHRONIZED",32,"VOLATILE",64,"TRANSIENT",128,"NATIVE",256,"INTERFACE",512,"ABSTRACT",1024,"STRICT",2048,"BRIDGE",64,"VARARGS",128,"SYNTHETIC",4096,"ANNOTATION",8192,"ENUM",16384);c$=t(function(){this.exceptionTypes=this.parameterTypes=this.clazz=
null;this.modifiers=0;p(this,arguments)},java.lang.reflect,"Constructor",java.lang.reflect.AccessibleObject,[java.lang.reflect.GenericDeclaration,java.lang.reflect.Member]);q(c$,function(a,b,d,g){I(this,java.lang.reflect.Constructor,[]);this.clazz=a;this.parameterTypes=b;this.exceptionTypes=d;this.modifiers=g},"Class,~A,~A,~N");j(c$,"getTypeParameters",function(){return null});c(c$,"toGenericString",function(){return null});c(c$,"getGenericParameterTypes",function(){return null});c(c$,"getGenericExceptionTypes",
function(){return null});c(c$,"getParameterAnnotations",function(){return null});c(c$,"isVarArgs",function(){return!1});j(c$,"isSynthetic",function(){return!1});j(c$,"equals",function(a){if(null!=a&&n(a,java.lang.reflect.Constructor)&&this.getDeclaringClass()===a.getDeclaringClass()){var b=this.parameterTypes;a=a.parameterTypes;if(b.length==a.length){for(var d=0;d<b.length;d++)if(b[d]!==a[d])return!1;return!0}}return!1},"~O");j(c$,"getDeclaringClass",function(){return this.clazz});c(c$,"getExceptionTypes",
function(){return this.exceptionTypes});j(c$,"getModifiers",function(){return this.modifiers});j(c$,"getName",function(){return this.getDeclaringClass().getName()});c(c$,"getParameterTypes",function(){return this.parameterTypes});j(c$,"hashCode",function(){return this.getDeclaringClass().getName().hashCode()});c(c$,"newInstance",function(a){var b=new this.clazz(Fa);p(b,a);return b},"~A");j(c$,"toString",function(){return null});c$=D(java.lang.reflect,"Field",java.lang.reflect.AccessibleObject,java.lang.reflect.Member);
j(c$,"isSynthetic",function(){return!1});c(c$,"toGenericString",function(){return null});c(c$,"isEnumConstant",function(){return!1});c(c$,"getGenericType",function(){return null});j(c$,"equals",function(){return!1},"~O");j(c$,"getDeclaringClass",function(){return null});j(c$,"getName",function(){return null});c(c$,"getType",function(){return null});j(c$,"hashCode",function(){return 0});j(c$,"toString",function(){return null});c$=t(function(){this.exceptionTypes=this.parameterTypes=this.returnType=
this.name=this.clazz=null;this.modifiers=0;p(this,arguments)},java.lang.reflect,"Method",java.lang.reflect.AccessibleObject,[java.lang.reflect.GenericDeclaration,java.lang.reflect.Member]);q(c$,function(a,b,d,g,c,j){I(this,java.lang.reflect.Method,[]);this.clazz=a;this.name=b;this.parameterTypes=d;this.returnType=g;this.exceptionTypes=c;this.modifiers=j},"Class,~S,~A,Class,~A,~N");j(c$,"getTypeParameters",function(){return null});c(c$,"toGenericString",function(){return null});c(c$,"getGenericParameterTypes",
function(){return null});c(c$,"getGenericExceptionTypes",function(){return null});c(c$,"getGenericReturnType",function(){return null});c(c$,"getParameterAnnotations",function(){return null});c(c$,"isVarArgs",function(){return!1});c(c$,"isBridge",function(){return!1});j(c$,"isSynthetic",function(){return!1});c(c$,"getDefaultValue",function(){return null});j(c$,"equals",function(a){if(null!=a&&n(a,java.lang.reflect.Method)&&this.getDeclaringClass()===a.getDeclaringClass()&&this.getName()===a.getName()){var b=
this.parameterTypes;a=a.parameterTypes;if(b.length==a.length){for(var d=0;d<b.length;d++)if(b[d]!==a[d])return!1;return!0}}return!1},"~O");j(c$,"getDeclaringClass",function(){return this.clazz});c(c$,"getExceptionTypes",function(){return this.exceptionTypes});j(c$,"getModifiers",function(){return this.modifiers});j(c$,"getName",function(){return this.name});c(c$,"getParameterTypes",function(){return this.parameterTypes});c(c$,"getReturnType",function(){return this.returnType});j(c$,"hashCode",function(){return this.getDeclaringClass().getName().hashCode()^
this.getName().hashCode()});c(c$,"invoke",function(a,b){var d=this.clazz.prototype[this.getName()];null==d&&(d=this.clazz[this.getName()]);null!=d&&d.apply(a,b)},"~O,~A");j(c$,"toString",function(){return null})})(ea);if(!Jmol._debugCode){v(null,"java.lang.AbstractStringBuilder","java.io.InvalidObjectException java.lang.ArrayIndexOutOfBoundsException $.IndexOutOfBoundsException $.NegativeArraySizeException $.NullPointerException $.StringIndexOutOfBoundsException".split(" "),function(){c$=t(function(){this.value=
null;this.count=0;this.shared=!1;p(this,arguments)},java.lang,"AbstractStringBuilder");c(c$,"getValue",function(){return this.value});c(c$,"shareValue",function(){this.shared=!0;return this.value});c(c$,"set",function(a,b){null==a&&(a=u(0,"\x00"));if(a.length<b)throw new java.io.InvalidObjectException("K0199");this.shared=!1;this.value=a;this.count=b},"~A,~N");q(c$,function(){this.value=u(16,"\x00")});q(c$,function(a){if(0>a)throw new NegativeArraySizeException;this.value=u(a,"\x00")},"~N");q(c$,
function(a){this.count=a.length;this.shared=!1;this.value=u(this.count+16,"\x00");a.getChars(0,this.count,this.value,0)},"~S");c(c$,"enlargeBuffer",($fz=function(a){var b=(this.value.length<<1)+2;a=u(a>b?a:b,"\x00");Zystem.arraycopy(this.value,0,a,0,this.count);this.value=a;this.shared=!1},$fz.isPrivate=!0,$fz),"~N");c(c$,"appendNull",function(){var a=this.count+4;a>this.value.length?this.enlargeBuffer(a):this.shared&&(this.value=this.value.clone(),this.shared=!1);this.value[this.count++]="n";this.value[this.count++]=
"u";this.value[this.count++]="l";this.value[this.count++]="l"});c(c$,"append0",function(a){var b=this.count+a.length;b>this.value.length?this.enlargeBuffer(b):this.shared&&(this.value=this.value.clone(),this.shared=!1);Zystem.arraycopy(a,0,this.value,this.count,a.length);this.count=b},"~A");c(c$,"append0",function(a,b,d){if(null==a)throw new NullPointerException;if(0<=b&&0<=d&&d<=a.length-b){var g=this.count+d;g>this.value.length?this.enlargeBuffer(g):this.shared&&(this.value=this.value.clone(),this.shared=
!1);Zystem.arraycopy(a,b,this.value,this.count,d);this.count=g}else throw new ArrayIndexOutOfBoundsException;},"~A,~N,~N");c(c$,"append0",function(a){this.count==this.value.length&&this.enlargeBuffer(this.count+1);this.shared&&(this.value=this.value.clone(),this.shared=!1);this.value[this.count++]=a},"~N");c(c$,"append0",function(a){if(null==a)this.appendNull();else{var b=a.length,d=this.count+b;d>this.value.length?this.enlargeBuffer(d):this.shared&&(this.value=this.value.clone(),this.shared=!1);
a.getChars(0,b,this.value,this.count);this.count=d}},"~S");c(c$,"append0",function(a,b,d){null==a&&(a="null");if(0>b||0>d||b>d||d>a.length())throw new IndexOutOfBoundsException;this.append0(a.subSequence(b,d).toString())},"CharSequence,~N,~N");c(c$,"capacity",function(){return this.value.length});c(c$,"charAt",function(a){if(0>a||a>=this.count)throw new StringIndexOutOfBoundsException(a);return this.value[a]},"~N");c(c$,"delete0",function(a,b){if(0<=a){b>this.count&&(b=this.count);if(b==a)return;
if(b>a){var d=this.count-b;if(0<d)if(this.shared){var g=u(this.value.length,"\x00");Zystem.arraycopy(this.value,0,g,0,a);Zystem.arraycopy(this.value,b,g,a,d);this.value=g;this.shared=!1}else Zystem.arraycopy(this.value,b,this.value,a,d);this.count-=b-a;return}}throw new StringIndexOutOfBoundsException;},"~N,~N");c(c$,"deleteCharAt0",function(a){if(0>a||a>=this.count)throw new StringIndexOutOfBoundsException(a);var b=this.count-a-1;if(0<b)if(this.shared){var d=u(this.value.length,"\x00");Zystem.arraycopy(this.value,
0,d,0,a);Zystem.arraycopy(this.value,a+1,d,a,b);this.value=d;this.shared=!1}else Zystem.arraycopy(this.value,a+1,this.value,a,b);this.count--},"~N");c(c$,"ensureCapacity",function(a){a>this.value.length&&this.enlargeBuffer(a)},"~N");c(c$,"getChars",function(a,b,d,g){if(a>this.count||b>this.count||a>b)throw new StringIndexOutOfBoundsException;Zystem.arraycopy(this.value,a,d,g,b-a)},"~N,~N,~A,~N");c(c$,"insert0",function(a,b){if(0>a||a>this.count)throw new StringIndexOutOfBoundsException(a);0!=b.length&&
(this.move(b.length,a),Zystem.arraycopy(b,0,this.value,a,b.length),this.count+=b.length)},"~N,~A");c(c$,"insert0",function(a,b,d,g){if(0<=a&&a<=this.count){if(0<=d&&0<=g&&g<=b.length-d){0!=g&&(this.move(g,a),Zystem.arraycopy(b,d,this.value,a,g),this.count+=g);return}throw new StringIndexOutOfBoundsException("offset "+d+", len "+g+", array.length "+b.length);}throw new StringIndexOutOfBoundsException(a);},"~N,~A,~N,~N");c(c$,"insert0",function(a,b){if(0>a||a>this.count)throw new ArrayIndexOutOfBoundsException(a);
this.move(1,a);this.value[a]=b;this.count++},"~N,~N");c(c$,"insert0",function(a,b){if(0<=a&&a<=this.count){null==b&&(b="null");var d=b.length;0!=d&&(this.move(d,a),b.getChars(0,d,this.value,a),this.count+=d)}else throw new StringIndexOutOfBoundsException(a);},"~N,~S");c(c$,"insert0",function(a,b,d,g){null==b&&(b="null");if(0>a||a>this.count||0>d||0>g||d>g||g>b.length())throw new IndexOutOfBoundsException;this.insert0(a,b.subSequence(d,g).toString())},"~N,CharSequence,~N,~N");c(c$,"length",function(){return this.count});
c(c$,"move",($fz=function(a,b){var d;if(this.value.length-this.count>=a){if(!this.shared){Zystem.arraycopy(this.value,b,this.value,b+a,this.count-b);return}d=this.value.length}else{d=this.count+a;var g=(this.value.length<<1)+2;d=d>g?d:g}d=u(d,"\x00");Zystem.arraycopy(this.value,0,d,0,b);Zystem.arraycopy(this.value,b,d,b+a,this.count-b);this.value=d;this.shared=!1},$fz.isPrivate=!0,$fz),"~N,~N");c(c$,"replace0",function(a,b,d){if(0<=a){b>this.count&&(b=this.count);if(b>a){var g=d.length,c=b-a-g;if(0<
c)if(this.shared){var e=u(this.value.length,"\x00");Zystem.arraycopy(this.value,0,e,0,a);Zystem.arraycopy(this.value,b,e,a+g,this.count-b);this.value=e;this.shared=!1}else Zystem.arraycopy(this.value,b,this.value,a+g,this.count-b);else 0>c?this.move(-c,b):this.shared&&(this.value=this.value.clone(),this.shared=!1);d.getChars(0,g,this.value,a);this.count-=c;return}if(a==b){if(null==d)throw new NullPointerException;this.insert0(a,d);return}}throw new StringIndexOutOfBoundsException;},"~N,~N,~S");c(c$,
"reverse0",function(){if(!(2>this.count))if(this.shared){for(var a=u(this.value.length,"\x00"),b=0,d=this.count;b<this.count;b++)a[--d]=this.value[b];this.value=a;this.shared=!1}else for(var b=0,d=this.count,a=Math.floor(this.count/2);b<a;b++){var g=this.value[--d];this.value[d]=this.value[b];this.value[b]=g}});c(c$,"setCharAt",function(a,b){if(0>a||a>=this.count)throw new StringIndexOutOfBoundsException(a);this.shared&&(this.value=this.value.clone(),this.shared=!1);this.value[a]=b},"~N,~N");c(c$,
"setLength",function(a){if(0>a)throw new StringIndexOutOfBoundsException(a);if(this.count<a)if(a>this.value.length)this.enlargeBuffer(a);else if(this.shared){var b=u(this.value.length,"\x00");Zystem.arraycopy(this.value,0,b,0,this.count);this.value=b;this.shared=!1}else for(b=this.count;b<a;b++)this.value[b]=String.fromCharCode(0);this.count=a},"~N");c(c$,"substring",function(a){if(0<=a&&a<=this.count){if(a==this.count)return"";this.shared=!0;return String.instantialize(a,this.count-a,this.value)}throw new StringIndexOutOfBoundsException(a);
},"~N");c(c$,"substring",function(a,b){if(0<=a&&a<=b&&b<=this.count){if(a==b)return"";this.shared=!0;return String.instantialize(this.value,a,b-a)}throw new StringIndexOutOfBoundsException;},"~N,~N");j(c$,"toString",function(){if(0==this.count)return"";if(256<=this.count&&this.count<=this.value.length>>1)return String.instantialize(this.value,0,this.count);this.shared=!0;return String.instantialize(0,this.count,this.value)});c(c$,"subSequence",function(a,b){return this.substring(a,b)},"~N,~N");c(c$,
"indexOf",function(a){return this.indexOf(a,0)},"~S");c(c$,"indexOf",function(a,b){0>b&&(b=0);var d=a.length;if(0<d){if(d+b>this.count)return-1;for(var g=a.charAt(0);;){for(var c=b,e=!1;c<this.count;c++)if(this.value[c].charCodeAt(0)==g.charCodeAt(0)){e=!0;break}if(!e||d+c>this.count)return-1;for(var e=c,h=0;++h<d&&this.value[++e].charCodeAt(0)==a.charAt(h).charCodeAt(0););if(h==d)return c;b=c+1}}return b<this.count||0==b?b:this.count},"~S,~N");c(c$,"lastIndexOf",function(a){return this.lastIndexOf(a,
this.count)},"~S");c(c$,"lastIndexOf",function(a,b){var d=a.length;if(d<=this.count&&0<=b){if(0<d){b>this.count-d&&(b=this.count-d);for(var g=a.charAt(0);;){for(var c=b,e=!1;0<=c;--c)if(this.value[c].charCodeAt(0)==g.charCodeAt(0)){e=!0;break}if(!e)return-1;for(var e=c,h=0;++h<d&&this.value[++e].charCodeAt(0)==a.charAt(h).charCodeAt(0););if(h==d)return c;b=c-1}}return b<this.count?b:this.count}return-1},"~S,~N");c(c$,"trimToSize",function(){if(this.count<this.value.length){var a=u(this.count,"\x00");
Zystem.arraycopy(this.value,0,a,0,this.count);this.value=a;this.shared=!1}});E(c$,"INITIAL_CAPACITY",16)});v(null,"java.lang.Enum",["java.lang.CloneNotSupportedException","$.IllegalArgumentException","$.NullPointerException"],function(){c$=t(function(){this.$name=null;this.$ordinal=0;p(this,arguments)},java.lang,"Enum",null,[java.io.Serializable,Comparable]);q(c$,function(a,b){this.$name=a;this.$ordinal=b},"~S,~N");c(c$,"name",function(){return this.$name});c(c$,"ordinal",function(){return this.$ordinal});
j(c$,"toString",function(){return this.$name});j(c$,"equals",function(a){return this===a},"~O");j(c$,"hashCode",function(){return this.$ordinal+(null==this.$name?0:this.$name.hashCode())});j(c$,"clone",function(){throw new CloneNotSupportedException("KA004");});j(c$,"compareTo",function(a){return this.$ordinal-a.$ordinal},"~O");c(c$,"getDeclaringClass",function(){var a=this.getClass(),b=a.getSuperclass();return Enum===b?a:b});c$.$valueOf=c(c$,"$valueOf",function(a,b){if(null==a||null==b)throw new NullPointerException("KA001");
var d=Enum.getValues(a);if(null==d)throw new IllegalArgumentException("KA005");for(var g,c=0;c<d.length&&((g=d[c])||1);c++)if(g.$name.equals(b))return g;throw new IllegalArgumentException("KA006");},"Class,~S");c$.getValues=c(c$,"getValues",function(a){return a.values()},"Class")});v(["java.lang.AbstractStringBuilder","$.Appendable"],"java.lang.StringBuffer",["java.lang.Character","$.Double","$.Float","$.Long"],function(){c$=D(java.lang,"StringBuffer",AbstractStringBuilder,[Appendable,java.io.Serializable,
CharSequence]);q(c$,function(a){if(null==a)throw new NullPointerException;I(this,StringBuffer,[a.toString()])},"CharSequence");c(c$,"append",function(a){return this.append(a?"true":"false")},"~B");c(c$,"append",function(a){this.append0(a);return this},"~N");c(c$,"append",function(a){return this.append(Double.toString(a))},"~N");c(c$,"append",function(a){null==a?this.appendNull():this.append0(a.toString());return this},"~O");c(c$,"append",function(a){this.append0(a);return this},"~S");c(c$,"append",
function(a){null==a?this.appendNull():this.append0(a.getValue(),0,a.length());return this},"StringBuffer");c(c$,"append",function(a){this.append0(a);return this},"~A");c(c$,"append",function(a,b,d){this.append0(a,b,d);return this},"~A,~N,~N");c(c$,"append",function(a){null==a?this.appendNull():this.append0(a.toString());return this},"CharSequence");c(c$,"append",function(a,b,d){this.append0(a,b,d);return this},"CharSequence,~N,~N");c(c$,"appendCodePoint",function(a){return this.append(Character.toChars(a))},
"~N");c(c$,"$delete",function(a,b){this.delete0(a,b);return this},"~N,~N");c(c$,"deleteCharAt",function(a){this.deleteCharAt0(a);return this},"~N");c(c$,"insert",function(a,b){this.insert0(a,b);return this},"~N,~N");c(c$,"insert",function(a,b){return this.insert(a,b?"true":"false")},"~N,~B");c(c$,"insert",function(a,b){return this.insert(a,Integer.toString(b))},"~N,~N");c(c$,"insert",function(a,b){return this.insert(a,Long.toString(b))},"~N,~N");c(c$,"insert",function(a,b){return this.insert(a,Double.toString(b))},
"~N,~N");c(c$,"insert",function(a,b){return this.insert(a,Float.toString(b))},"~N,~N");c(c$,"insert",function(a,b){return this.insert(a,null==b?"null":b.toString())},"~N,~O");c(c$,"insert",function(a,b){this.insert0(a,b);return this},"~N,~S");c(c$,"insert",function(a,b){this.insert0(a,b);return this},"~N,~A");c(c$,"insert",function(a,b,d,g){this.insert0(a,b,d,g);return this},"~N,~A,~N,~N");c(c$,"insert",function(a,b){this.insert0(a,null==b?"null":b.toString());return this},"~N,CharSequence");c(c$,
"insert",function(a,b,d,g){this.insert0(a,b,d,g);return this},"~N,CharSequence,~N,~N");c(c$,"replace",function(a,b,d){this.replace0(a,b,d);return this},"~N,~N,~S");c(c$,"reverse",function(){this.reverse0();return this});j(c$,"subSequence",function(a,b){return W(this,StringBuffer,"substring",[a,b])},"~N,~N")});v(["java.lang.AbstractStringBuilder","$.Appendable"],"java.lang.StringBuilder",["java.lang.Double","$.Float","$.Long"],function(){c$=D(java.lang,"StringBuilder",AbstractStringBuilder,[Appendable,
CharSequence,java.io.Serializable]);q(c$,function(a){I(this,StringBuilder,[a.toString()])},"CharSequence");c(c$,"append",function(a){this.append0(a?"true":"false");return this},"~B");c(c$,"append",function(a){this.append0(a);return this},"~N");c(c$,"append",function(a){this.append0(Integer.toString(a));return this},"~N");c(c$,"append",function(a){this.append0(Long.toString(a));return this},"~N");c(c$,"append",function(a){this.append0(Float.toString(a));return this},"~N");c(c$,"append",function(a){this.append0(Double.toString(a));
return this},"~N");c(c$,"append",function(a){null==a?this.appendNull():this.append0(a.toString());return this},"~O");c(c$,"append",function(a){this.append0(a);return this},"~S");c(c$,"append",function(a){null==a?this.appendNull():this.append0(a.getValue(),0,a.length());return this},"StringBuffer");c(c$,"append",function(a){this.append0(a);return this},"~A");c(c$,"append",function(a,b,d){this.append0(a,b,d);return this},"~A,~N,~N");c(c$,"append",function(a){null==a?this.appendNull():this.append0(a.toString());
return this},"CharSequence");c(c$,"append",function(a,b,d){this.append0(a,b,d);return this},"CharSequence,~N,~N");c(c$,"$delete",function(a,b){this.delete0(a,b);return this},"~N,~N");c(c$,"deleteCharAt",function(a){this.deleteCharAt0(a);return this},"~N");c(c$,"insert",function(a,b){this.insert0(a,b?"true":"false");return this},"~N,~B");c(c$,"insert",function(a,b){this.insert0(a,b);return this},"~N,~N");c(c$,"insert",function(a,b){this.insert0(a,Integer.toString(b));return this},"~N,~N");c(c$,"insert",
function(a,b){this.insert0(a,Long.toString(b));return this},"~N,~N");c(c$,"insert",function(a,b){this.insert0(a,Float.toString(b));return this},"~N,~N");c(c$,"insert",function(a,b){this.insert0(a,Double.toString(b));return this},"~N,~N");c(c$,"insert",function(a,b){this.insert0(a,null==b?"null":b.toString());return this},"~N,~O");c(c$,"insert",function(a,b){this.insert0(a,b);return this},"~N,~S");c(c$,"insert",function(a,b){this.insert0(a,b);return this},"~N,~A");c(c$,"insert",function(a,b,d,g){this.insert0(a,
b,d,g);return this},"~N,~A,~N,~N");c(c$,"insert",function(a,b){this.insert0(a,null==b?"null":b.toString());return this},"~N,CharSequence");c(c$,"insert",function(a,b,d,g){this.insert0(a,b,d,g);return this},"~N,CharSequence,~N,~N");c(c$,"replace",function(a,b,d){this.replace0(a,b,d);return this},"~N,~N,~S");c(c$,"reverse",function(){this.reverse0();return this})});v(null,"java.lang.Thread",["java.lang.IllegalArgumentException","$.ThreadGroup","java.util.Date"],function(){c$=t(function(){this.name=
this.group=this.target=null;this.priority=0;p(this,arguments)},java.lang,"Thread",null,Runnable);c$.currentThread=c(c$,"currentThread",function(){null==Thread.J2S_THREAD&&(Thread.J2S_THREAD=new Thread,Thread.prototype.J2S_THREAD=Thread.J2S_THREAD);return Thread.J2S_THREAD});c$.sleep=c(c$,"sleep",function(){Ga("Thread.sleep is not implemented in Java2Script!")},"~N");q(c$,function(){});q(c$,function(a){this.init(null,a,"Thread-"+(new java.util.Date).getTime()+Math.random(),0)},"Runnable");q(c$,function(a,
b){this.init(a,b,"Thread-"+(new java.util.Date).getTime()+Math.random(),0)},"ThreadGroup,Runnable");q(c$,function(a){this.init(null,null,a,0)},"~S");q(c$,function(a,b){this.init(a,null,b,0)},"ThreadGroup,~S");q(c$,function(a,b){this.init(null,a,b,0)},"Runnable,~S");q(c$,function(a,b,d){this.init(a,b,d,0)},"ThreadGroup,Runnable,~S");q(c$,function(a,b,d,g){this.init(a,b,d,g)},"ThreadGroup,Runnable,~S,~N");c(c$,"init",($fz=function(a,b,d){null==a&&(a=new ThreadGroup);this.group=a;this.target=b;this.name=
d;this.priority=5},$fz.isPrivate=!0,$fz),"ThreadGroup,Runnable,~S,~N");c(c$,"start",function(){window.setTimeout(function(a){return function(){a.run()}}(this),0)});c(c$,"run",function(){null!=this.target&&this.target.run()});c(c$,"setPriority",function(a){if(10<a||1>a)throw new IllegalArgumentException;this.priority=a},"~N");c(c$,"getPriority",function(){return this.priority});c(c$,"interrupt",function(){});c(c$,"setName",function(a){this.name=a},"~S");c(c$,"getName",function(){return String.valueOf(this.name)});
c(c$,"getThreadGroup",function(){return this.group});j(c$,"toString",function(){var a=this.getThreadGroup();return null!=a?"Thread["+this.getName()+","+this.getPriority()+","+a.getName()+"]":"Thread["+this.getName()+","+this.getPriority()+",]"});E(c$,"MIN_PRIORITY",1,"NORM_PRIORITY",5,"MAX_PRIORITY",10,"J2S_THREAD",null)});v(null,"java.lang.ThreadGroup",["java.lang.NullPointerException","$.Thread"],function(){c$=t(function(){this.name=this.parent=null;this.maxPriority=0;p(this,arguments)},java.lang,
"ThreadGroup");q(c$,function(){this.name="Zystem";this.maxPriority=10});q(c$,function(a){this.construct(Thread.currentThread().getThreadGroup(),a)},"~S");q(c$,function(a,b){if(null==a)throw new NullPointerException;this.name=b;this.parent=a;this.maxPriority=10},"ThreadGroup,~S");c(c$,"getName",function(){return this.name});c(c$,"getParent",function(){return this.parent});c(c$,"getMaxPriority",function(){return this.maxPriority})});v(["java.io.FilterInputStream"],"java.io.BufferedInputStream",["java.io.IOException",
"java.lang.IndexOutOfBoundsException"],function(){c$=t(function(){this.buf=null;this.pos=this.count=0;this.markpos=-1;this.marklimit=0;p(this,arguments)},java.io,"BufferedInputStream",java.io.FilterInputStream);c(c$,"getInIfOpen",function(){var a=this.$in;if(null==a)throw new java.io.IOException("Stream closed");return a});c(c$,"getBufIfOpen",function(){var a=this.buf;if(null==a)throw new java.io.IOException("Stream closed");return a});j(c$,"resetStream",function(){});q(c$,function(a){I(this,java.io.BufferedInputStream,
[a]);this.buf=Q(8192,0)},"java.io.InputStream");c(c$,"fill",function(){var a=this.getBufIfOpen();if(0>this.markpos)this.pos=0;else if(this.pos>=a.length)if(0<this.markpos){var b=this.pos-this.markpos;Zystem.arraycopy(a,this.markpos,a,0,b);this.pos=b;this.markpos=0}else a.length>=this.marklimit?(this.markpos=-1,this.pos=0):(b=2*this.pos,b>this.marklimit&&(b=this.marklimit),b=Q(b,0),Zystem.arraycopy(a,0,b,0,this.pos),a=this.buf=b);this.count=this.pos;a=this.getInIfOpen().read(a,this.pos,a.length-this.pos);
0<a&&(this.count=a+this.pos)});j(c$,"readByteAsInt",function(){return this.pos>=this.count&&(this.fill(),this.pos>=this.count)?-1:this.getBufIfOpen()[this.pos++]&255});c(c$,"read1",function(a,b,d){var g=this.count-this.pos;if(0>=g){if(d>=this.getBufIfOpen().length&&0>this.markpos)return this.getInIfOpen().read(a,b,d);this.fill();g=this.count-this.pos;if(0>=g)return-1}d=g<d?g:d;Zystem.arraycopy(this.getBufIfOpen(),this.pos,a,b,d);this.pos+=d;return d},"~A,~N,~N");c(c$,"read",function(a,b,d){this.getBufIfOpen();
if(0>(b|d|b+d|a.length-(b+d)))throw new IndexOutOfBoundsException;if(0==d)return 0;for(var g=0;;){var c=this.read1(a,b+g,d-g);if(0>=c)return 0==g?c:g;g+=c;if(g>=d)return g;c=this.$in;if(null!=c&&0>=c.available())return g}},"~A,~N,~N");j(c$,"skip",function(a){this.getBufIfOpen();if(0>=a)return 0;var b=this.count-this.pos;if(0>=b){if(0>this.markpos)return this.getInIfOpen().skip(a);this.fill();b=this.count-this.pos;if(0>=b)return 0}a=b<a?b:a;this.pos+=a;return a},"~N");j(c$,"available",function(){var a=
this.count-this.pos,b=this.getInIfOpen().available();return a>2147483647-b?2147483647:a+b});j(c$,"mark",function(a){this.marklimit=a;this.markpos=this.pos},"~N");j(c$,"reset",function(){this.getBufIfOpen();if(0>this.markpos)throw new java.io.IOException("Resetting to invalid mark");this.pos=this.markpos});j(c$,"markSupported",function(){return!0});j(c$,"close",function(){var a=this.$in;this.$in=null;null!=a&&a.close()});E(c$,"DEFAULT_BUFFER_SIZE",8192)});v(["java.io.Reader"],"java.io.BufferedReader",
["java.io.IOException","java.lang.IllegalArgumentException","$.IndexOutOfBoundsException","JU.SB"],function(){c$=t(function(){this.cb=this.$in=null;this.nextChar=this.nChars=0;this.markedChar=-1;this.readAheadLimit=0;this.markedSkipLF=this.skipLF=!1;p(this,arguments)},java.io,"BufferedReader",java.io.Reader);c(c$,"setSize",function(a){if(0>=a)throw new IllegalArgumentException("Buffer size <= 0");this.cb=ca(a,"\x00");this.nextChar=this.nChars=0},"~N");q(c$,function(a){I(this,java.io.BufferedReader,
[a]);this.$in=a;this.setSize(8192)},"java.io.Reader");c(c$,"ensureOpen",function(){if(null==this.$in)throw new java.io.IOException("Stream closed");});c(c$,"fill",function(){var a;if(-1>=this.markedChar)a=0;else{var b=this.nextChar-this.markedChar;b>=this.readAheadLimit?(this.markedChar=-2,a=this.readAheadLimit=0):(this.readAheadLimit<=this.cb.length?Zystem.arraycopy(this.cb,this.markedChar,this.cb,0,b):(a=ca(this.readAheadLimit,"\x00"),Zystem.arraycopy(this.cb,this.markedChar,a,0,b),this.cb=a),this.markedChar=
0,this.nextChar=this.nChars=a=b)}do b=this.$in.read(this.cb,a,this.cb.length-a);while(0==b);0<b&&(this.nChars=a+b,this.nextChar=a)});c(c$,"read1",function(a,b,d){if(this.nextChar>=this.nChars){if(d>=this.cb.length&&-1>=this.markedChar&&!this.skipLF)return this.$in.read(a,b,d);this.fill()}if(this.nextChar>=this.nChars||this.skipLF&&(this.skipLF=!1,"\n"==this.cb[this.nextChar]&&(this.nextChar++,this.nextChar>=this.nChars&&this.fill(),this.nextChar>=this.nChars)))return-1;d=Math.min(d,this.nChars-this.nextChar);
Zystem.arraycopy(this.cb,this.nextChar,a,b,d);this.nextChar+=d;return d},"~A,~N,~N");c(c$,"read",function(a,b,d){this.ensureOpen();if(0>b||b>a.length||0>d||b+d>a.length||0>b+d)throw new IndexOutOfBoundsException;if(0==d)return 0;var g=this.read1(a,b,d);if(0>=g)return g;for(;g<d&&this.$in.ready();){var c=this.read1(a,b+g,d-g);if(0>=c)break;g+=c}return g},"~A,~N,~N");c(c$,"readLine1",function(a){var b=null,d;this.ensureOpen();for(var g=a||this.skipLF;;){this.nextChar>=this.nChars&&this.fill();if(this.nextChar>=
this.nChars)return null!=b&&0<b.length()?b.toString():null;var c=!1;a=String.fromCharCode(0);var e;g&&"\n"==this.cb[this.nextChar]&&this.nextChar++;g=this.skipLF=!1;e=this.nextChar;a:for(;e<this.nChars;e++)if(a=this.cb[e],"\n"==a||"\r"==a){c=!0;break a}d=this.nextChar;this.nextChar=e;if(c)return null==b?b=String.instantialize(this.cb,d,e-d):(b.appendCB(this.cb,d,e-d),b=b.toString()),this.nextChar++,"\r"==a&&(this.skipLF=!0),b;null==b&&(b=JU.SB.newN(80));b.appendCB(this.cb,d,e-d)}},"~B");c(c$,"readLine",
function(){return this.readLine1(!1)});j(c$,"skip",function(a){if(0>a)throw new IllegalArgumentException("skip value is negative");this.ensureOpen();for(var b=a;0<b;){this.nextChar>=this.nChars&&this.fill();if(this.nextChar>=this.nChars)break;this.skipLF&&(this.skipLF=!1,"\n"==this.cb[this.nextChar]&&this.nextChar++);var d=this.nChars-this.nextChar;if(b<=d){this.nextChar+=b;b=0;break}b-=d;this.nextChar=this.nChars}return a-b},"~N");c(c$,"ready",function(){this.ensureOpen();this.skipLF&&(this.nextChar>=
this.nChars&&this.$in.ready()&&this.fill(),this.nextChar<this.nChars&&("\n"==this.cb[this.nextChar]&&this.nextChar++,this.skipLF=!1));return this.nextChar<this.nChars||this.$in.ready()});j(c$,"markSupported",function(){return!0});j(c$,"mark",function(a){if(0>a)throw new IllegalArgumentException("Read-ahead limit < 0");this.ensureOpen();this.readAheadLimit=a;this.markedChar=this.nextChar;this.markedSkipLF=this.skipLF},"~N");j(c$,"reset",function(){this.ensureOpen();if(0>this.markedChar)throw new java.io.IOException(-2==
this.markedChar?"Mark invalid":"Stream not marked");this.nextChar=this.markedChar;this.skipLF=this.markedSkipLF});c(c$,"close",function(){null!=this.$in&&(this.$in.close(),this.cb=this.$in=null)});E(c$,"INVALIDATED",-2,"UNMARKED",-1,"DEFAULT_CHAR_BUFFER_SIZE",8192,"DEFAULT_EXPECTED_LINE_LENGTH",80)});v(["java.io.Writer"],"java.io.BufferedWriter",["java.io.IOException","java.lang.IllegalArgumentException","$.IndexOutOfBoundsException","$.StringIndexOutOfBoundsException"],function(){c$=t(function(){this.buf=
this.out=null;this.pos=0;this.lineSeparator="\r\n";p(this,arguments)},java.io,"BufferedWriter",java.io.Writer);q(c$,function(a){I(this,java.io.BufferedWriter,[a]);this.out=a;this.buf=u(8192,"\x00")},"java.io.Writer");q(c$,function(a,b){I(this,java.io.BufferedWriter,[a]);if(0<b)this.out=a,this.buf=u(b,"\x00");else throw new IllegalArgumentException("K0058");},"java.io.Writer,~N");c(c$,"close",function(){this.isOpen()&&(this.flush(),this.out.close(),this.out=this.buf=null)});c(c$,"flush",function(){if(this.isOpen())0<
this.pos&&this.out.write(this.buf,0,this.pos),this.pos=0,this.out.flush();else throw new java.io.IOException("K005d");});c(c$,"isOpen",($fz=function(){return null!=this.out},$fz.isPrivate=!0,$fz));c(c$,"newLine",function(){this.write("\r\n",0,2)});c(c$,"write",function(a,b,d){if(!this.isOpen())throw new java.io.IOException("K005d");if(0>b||b>a.length-d||0>d)throw new IndexOutOfBoundsException;if(0==this.pos&&d>=this.buf.length)this.out.write(a,b,d);else{var g=this.buf.length-this.pos;d<g&&(g=d);0<
g&&(Zystem.arraycopy(a,b,this.buf,this.pos,g),this.pos+=g);this.pos==this.buf.length&&(this.out.write(this.buf,0,this.buf.length),this.pos=0,d>g&&(b+=g,g=d-g,g>=this.buf.length?this.out.write(a,b,g):(Zystem.arraycopy(a,b,this.buf,this.pos,g),this.pos+=g)))}},"~A,~N,~N");c(c$,"write",function(a){if(this.isOpen())this.pos>=this.buf.length&&(this.out.write(this.buf,0,this.buf.length),this.pos=0),this.buf[this.pos++]=String.fromCharCode(a);else throw new java.io.IOException("K005d");},"~N");c(c$,"write",
function(a,b,d){if(!this.isOpen())throw new java.io.IOException("K005d");if(!(0>=d)){if(b>a.length-d||0>b)throw new StringIndexOutOfBoundsException;if(0==this.pos&&d>=this.buf.length){var g=u(d,"\x00");a.getChars(b,b+d,g,0);this.out.write(g,0,d)}else{var c=this.buf.length-this.pos;d<c&&(c=d);0<c&&(a.getChars(b,b+c,this.buf,this.pos),this.pos+=c);this.pos==this.buf.length&&(this.out.write(this.buf,0,this.buf.length),this.pos=0,d>c&&(b+=c,c=d-c,c>=this.buf.length?(g=u(d,"\x00"),a.getChars(b,b+c,g,0),
this.out.write(g,0,c)):(a.getChars(b,b+c,this.buf,this.pos),this.pos+=c)))}}},"~S,~N,~N")});v(["java.io.InputStream"],"java.io.ByteArrayInputStream",["java.lang.IndexOutOfBoundsException","$.NullPointerException"],function(){c$=t(function(){this.buf=null;this.count=this.$mark=this.pos=0;p(this,arguments)},java.io,"ByteArrayInputStream",java.io.InputStream);q(c$,function(a){I(this,java.io.ByteArrayInputStream,[]);this.buf=a;this.pos=0;this.count=a.length},"~A");j(c$,"readByteAsInt",function(){return this.pos<
this.count?this.buf[this.pos++]&255:-1});c(c$,"read",function(a,b,d){if(null==a)throw new NullPointerException;if(0>b||0>d||d>a.length-b)throw new IndexOutOfBoundsException;if(this.pos>=this.count)return-1;var g=this.count-this.pos;d>g&&(d=g);if(0>=d)return 0;Zystem.arraycopy(this.buf,this.pos,a,b,d);this.pos+=d;return d},"~A,~N,~N");j(c$,"skip",function(a){var b=this.count-this.pos;a<b&&(b=0>a?0:a);this.pos+=b;return b},"~N");j(c$,"available",function(){return this.count-this.pos});j(c$,"markSupported",
function(){return!0});j(c$,"mark",function(){this.$mark=this.pos},"~N");j(c$,"resetStream",function(){});j(c$,"reset",function(){this.pos=this.$mark});j(c$,"close",function(){})});v(["java.io.OutputStream"],"java.io.ByteArrayOutputStream",["java.lang.IllegalArgumentException","$.IndexOutOfBoundsException","$.OutOfMemoryError"],function(){c$=t(function(){this.buf=null;this.count=0;p(this,arguments)},java.io,"ByteArrayOutputStream",java.io.OutputStream);q(c$,function(){this.construct(32)});q(c$,function(a){I(this,
java.io.ByteArrayOutputStream,[]);if(0>a)throw new IllegalArgumentException("Negative initial size: "+a);this.buf=Q(a,0)},"~N");c(c$,"ensureCapacity",function(a){0<a-this.buf.length&&this.grow(a)},"~N");c(c$,"grow",function(a){var b=this.buf.length<<1;0>b-a&&(b=a);if(0>b){if(0>a)throw new OutOfMemoryError;b=a}this.buf=java.io.ByteArrayOutputStream.arrayCopyByte(this.buf,b)},"~N");c$.arrayCopyByte=c(c$,"arrayCopyByte",function(a,b){var d=Q(b,0);Zystem.arraycopy(a,0,d,0,a.length<b?a.length:b);return d},
"~A,~N");j(c$,"writeByteAsInt",function(a){this.ensureCapacity(this.count+1);this.buf[this.count]=a;this.count+=1},"~N");c(c$,"write",function(a,b,d){if(0>b||b>a.length||0>d||0<b+d-a.length)throw new IndexOutOfBoundsException;this.ensureCapacity(this.count+d);Zystem.arraycopy(a,b,this.buf,this.count,d);this.count+=d},"~A,~N,~N");c(c$,"writeTo",function(a){a.write(this.buf,0,this.count)},"java.io.OutputStream");c(c$,"reset",function(){this.count=0});c(c$,"toByteArray",function(){return this.count==
this.buf.length?this.buf:java.io.ByteArrayOutputStream.arrayCopyByte(this.buf,this.count)});c(c$,"size",function(){return this.count});j(c$,"toString",function(){return String.instantialize(this.buf,0,this.count)});j(c$,"close",function(){})});v(["java.io.InputStream"],"java.io.FilterInputStream",null,function(){c$=t(function(){this.$in=null;p(this,arguments)},java.io,"FilterInputStream",java.io.InputStream);q(c$,function(a){I(this,java.io.FilterInputStream,[]);this.$in=a},"java.io.InputStream");
c(c$,"readByteAsInt",function(){return this.$in.readByteAsInt()});c(c$,"read",function(a,b,d){return this.$in.read(a,b,d)},"~A,~N,~N");c(c$,"skip",function(a){return this.$in.skip(a)},"~N");c(c$,"available",function(){return this.$in.available()});c(c$,"close",function(){this.$in.close()});c(c$,"mark",function(a){this.$in.mark(a)},"~N");c(c$,"reset",function(){this.$in.reset()});c(c$,"markSupported",function(){return this.$in.markSupported()})});v(null,"java.io.InputStream",["java.io.IOException",
"java.lang.IndexOutOfBoundsException","$.NullPointerException"],function(){c$=D(java.io,"InputStream");c(c$,"read",function(a,b,d){if(null==a)throw new NullPointerException;if(0>b||0>d||d>a.length-b)throw new IndexOutOfBoundsException;if(0==d)return 0;var g=this.readByteAsInt();if(-1==g)return-1;a[b]=g;var c=1;try{for(;c<d;c++){g=this.readByteAsInt();if(-1==g)break;a[b+c]=g}}catch(e){if(!H(e,java.io.IOException))throw e;}return c},"~A,~N,~N");c(c$,"skip",function(a){var b=a,d;null==java.io.InputStream.skipBuffer&&
(java.io.InputStream.skipBuffer=Q(2048,0));var g=java.io.InputStream.skipBuffer;if(0>=a)return 0;for(;0<b;){d=this.read(g,0,Math.min(2048,b));if(0>d)break;b-=d}return a-b},"~N");c(c$,"available",function(){return 0});c(c$,"close",function(){});c(c$,"mark",function(){},"~N");c(c$,"reset",function(){throw new java.io.IOException("mark/reset not supported");});c(c$,"markSupported",function(){return!1});c(c$,"resetStream",function(){});E(c$,"SKIP_BUFFER_SIZE",2048,"skipBuffer",null)});v(["java.io.Reader"],
"java.io.InputStreamReader",["java.lang.NullPointerException"],function(){c$=t(function(){this.$in=null;this.isOpen=!0;this.charsetName=null;this.isUTF8=!1;this.bytearr=null;this.pos=0;p(this,arguments)},java.io,"InputStreamReader",java.io.Reader);q(c$,function(a,b){I(this,java.io.InputStreamReader,[a]);this.$in=a;this.charsetName=b;if(!(this.isUTF8="UTF-8".equals(b))&&!"ISO-8859-1".equals(b))throw new NullPointerException("charsetName");},"java.io.InputStream,~S");c(c$,"getEncoding",function(){return this.charsetName});
j(c$,"read",function(a,b,d){if(null==this.bytearr||this.bytearr.length<d)this.bytearr=Q(d,0);var g,c,e=0,h=b,k=this.$in.read(this.bytearr,this.pos,d-this.pos),l=this.$in.available();if(0>k)return-1;for(var j=k;e<j;){d=this.bytearr[e]&255;if(this.isUTF8)switch(d>>4){case 12:case 13:if(e+1>=k){if(1<=l){j=e;continue}}else if(128==((g=this.bytearr[e+1])&192)){a[h++]=String.fromCharCode((d&31)<<6|g&63);e+=2;continue}this.isUTF8=!1;break;case 14:if(e+2>=k){if(2<=l){j=e;continue}}else if(128==((g=this.bytearr[e+
1])&192)&&128==((c=this.bytearr[e+2])&192)){a[h++]=String.fromCharCode((d&15)<<12|(g&63)<<6|c&63);e+=3;continue}this.isUTF8=!1}e++;a[h++]=String.fromCharCode(d)}this.pos=k-e;for(a=0;a<this.pos;a++)this.bytearr[a]=this.bytearr[e++];return h-b},"~A,~N,~N");j(c$,"ready",function(){return this.isOpen});j(c$,"close",function(){this.$in.close();this.isOpen=!1})});v(["java.io.Closeable","$.Flushable"],"java.io.OutputStream",["java.lang.IndexOutOfBoundsException","$.NullPointerException"],function(){c$=D(java.io,
"OutputStream",null,[java.io.Closeable,java.io.Flushable]);c(c$,"write",function(a,b,d){if(null==a)throw new NullPointerException;if(0>b||b>a.length||0>d||b+d>a.length||0>b+d)throw new IndexOutOfBoundsException;if(0!=d)for(var g=0;g<d;g++)this.writeByteAsInt(a[b+g])},"~A,~N,~N");j(c$,"flush",function(){});j(c$,"close",function(){})});v(["java.io.Closeable"],"java.io.Reader",["java.io.IOException","java.lang.IllegalArgumentException","$.NullPointerException"],function(){c$=t(function(){this.skipBuffer=
this.lock=null;p(this,arguments)},java.io,"Reader",null,java.io.Closeable);q(c$,function(a){if(null==a)throw new NullPointerException;this.lock=a},"~O");c(c$,"skip",function(a){if(0>a)throw new IllegalArgumentException("skip value is negative");var b=Math.min(a,8192);if(null==this.skipBuffer||this.skipBuffer.length<b)this.skipBuffer=ca(b,"\x00");for(var d=a;0<d;){var g=this.read(this.skipBuffer,0,Math.min(d,b));if(-1==g)break;d-=g}return a-d},"~N");c(c$,"ready",function(){return!1});c(c$,"markSupported",
function(){return!1});c(c$,"mark",function(){throw new java.io.IOException("mark() not supported");},"~N");c(c$,"reset",function(){throw new java.io.IOException("reset() not supported");});E(c$,"MAX_SKIP_BUFFE_SIZE",8192)});v(["java.io.Reader"],"java.io.StringReader",["java.io.IOException","java.lang.IllegalArgumentException","$.IndexOutOfBoundsException"],function(){c$=t(function(){this.str=null;this.$mark=this.next=this.length=0;p(this,arguments)},java.io,"StringReader",java.io.Reader);q(c$,function(a){I(this,
java.io.StringReader,[a]);this.str=a;this.length=a.length},"~S");c(c$,"ensureOpen",function(){if(null==this.str)throw new java.io.IOException("Stream closed");});j(c$,"read",function(a,b,d){this.ensureOpen();if(0>b||b>a.length||0>d||b+d>a.length||0>b+d)throw new IndexOutOfBoundsException;if(0==d)return 0;if(this.next>=this.length)return-1;d=Math.min(this.length-this.next,d);this.str.getChars(this.next,this.next+d,a,b);this.next+=d;return d},"~A,~N,~N");j(c$,"skip",function(a){this.ensureOpen();if(this.next>=
this.length)return 0;a=Math.min(this.length-this.next,a);a=Math.max(-this.next,a);this.next+=a;return a},"~N");j(c$,"ready",function(){this.ensureOpen();return!0});j(c$,"markSupported",function(){return!0});j(c$,"mark",function(a){if(0>a)throw new IllegalArgumentException("Read-ahead limit < 0");this.ensureOpen();this.$mark=this.next},"~N");j(c$,"reset",function(){this.ensureOpen();this.next=this.$mark});j(c$,"close",function(){this.str=null})});v(["java.io.Closeable","$.Flushable","java.lang.Appendable"],
"java.io.Writer",["java.lang.NullPointerException","$.StringIndexOutOfBoundsException"],function(){c$=t(function(){this.lock=null;p(this,arguments)},java.io,"Writer",null,[Appendable,java.io.Closeable,java.io.Flushable]);q(c$,function(){this.lock=this});q(c$,function(a){if(null!=a)this.lock=a;else throw new NullPointerException;},"~O");c(c$,"write",function(a){this.write(a,0,a.length)},"~A");c(c$,"write",function(a){var b=u(1,"\x00");b[0]=String.fromCharCode(a);this.write(b)},"~N");c(c$,"write",function(a){var b=
u(a.length,"\x00");a.getChars(0,b.length,b,0);this.write(b)},"~S");c(c$,"write",function(a,b,d){if(0<=d){var g=u(d,"\x00");a.getChars(b,b+d,g,0);this.write(g)}else throw new StringIndexOutOfBoundsException;},"~S,~N,~N");c(c$,"append",function(a){this.write(a.charCodeAt(0));return this},"~N");c(c$,"append",function(a){null==a?this.write("null"):this.write(a.toString());return this},"CharSequence");c(c$,"append",function(a,b,d){null==a?this.write("null".substring(b,d)):this.write(a.subSequence(b,d).toString());
return this},"CharSequence,~N,~N");E(c$,"TOKEN_NULL","null")});s("java.net");v(["java.io.IOException"],"java.net.MalformedURLException",null,function(){c$=D(java.net,"MalformedURLException",java.io.IOException);q(c$,function(){I(this,java.net.MalformedURLException,[])})});s("java.net");v(["java.io.IOException"],"java.net.UnknownServiceException",null,function(){c$=D(java.net,"UnknownServiceException",java.io.IOException);q(c$,function(){I(this,java.net.UnknownServiceException,[])})});s("java.net");
v(["java.util.Hashtable"],"java.net.URL",["java.lang.Character","$.Error","java.net.MalformedURLException"],function(){c$=t(function(){this.host=this.protocol=null;this.port=-1;this.handler=this.ref=this.userInfo=this.path=this.authority=this.query=this.file=null;this.$hashCode=-1;p(this,arguments)},java.net,"URL");q(c$,function(a,b,d){switch(arguments.length){case 1:b=a;a=d=null;break;case 2:d=null;break;case 3:if(null==a||n(a,java.net.URL))break;default:alert("java.net.URL constructor format not supported")}a&&
a.valueOf&&null==a.valueOf()&&(a=null);var g=b,c,e,h,k=0,l=null,j=!1,m=!1;try{for(e=b.length;0<e&&" ">=b.charAt(e-1);)e--;for(;k<e&&" ">=b.charAt(k);)k++;b.regionMatches(!0,k,"url:",0,4)&&(k+=4);k<b.length&&"#"==b.charAt(k)&&(j=!0);for(c=k;!j&&c<e&&47!=(h=b.charCodeAt(c));c++)if(58==h){var r=b.substring(k,c).toLowerCase();this.isValidProtocol(r)&&(l=r,k=c+1);break}this.protocol=l;if(null!=a&&(null==l||l.equalsIgnoreCase(a.protocol)))null==d&&(d=a.handler),null!=a.path&&a.path.startsWith("/")&&(l=
null),null==l&&(this.protocol=a.protocol,this.authority=a.authority,this.userInfo=a.userInfo,this.host=a.host,this.port=a.port,this.file=a.file,this.path=a.path,m=!0);if(null==this.protocol)throw new java.net.MalformedURLException("no protocol: "+g);if(null==d&&null==(d=java.net.URL.getURLStreamHandler(this.protocol)))throw new java.net.MalformedURLException("unknown protocol: "+this.protocol);this.handler=d;c=b.indexOf("#",k);0<=c&&(this.ref=b.substring(c+1,e),e=c);m&&k==e&&(this.query=a.query,null==
this.ref&&(this.ref=a.ref));d.parseURL(this,b,k,e)}catch(x){if(H(x,java.net.MalformedURLException))throw x;if(H(x,Exception))throw g=x,c=new java.net.MalformedURLException(g.getMessage()),c.initCause(g),c;throw x;}},"java.net.URL,~S,java.net.URLStreamHandler");c(c$,"isValidProtocol",function(a){var b=a.length;if(1>b)return!1;var d=a.charAt(0);if(!Character.isLetter(d))return!1;for(var g=1;g<b;g++)if(d=a.charAt(g),!Character.isLetterOrDigit(d)&&"."!=d&&"+"!=d&&"-"!=d)return!1;return!0},"~S");c(c$,
"set5",function(a,b,d,g,c){this.protocol=a;this.host=b;this.authority=-1==d?b:b+":"+d;this.port=d;this.file=g;this.ref=c;this.$hashCode=-1;a=g.lastIndexOf("?");-1!=a?(this.query=g.substring(a+1),this.path=g.substring(0,a)):this.path=g},"~S,~S,~N,~S,~S");c(c$,"set",function(a,b,d,g,c,e,h,k){this.protocol=a;this.host=b;this.port=d;this.file=null==h?e:e+"?"+h;this.userInfo=c;this.path=e;this.ref=k;this.$hashCode=-1;this.query=h;this.authority=g},"~S,~S,~N,~S,~S,~S,~S,~S");c(c$,"getQuery",function(){return this.query});
c(c$,"getPath",function(){return this.path});c(c$,"getUserInfo",function(){return this.userInfo});c(c$,"getAuthority",function(){return this.authority});c(c$,"getPort",function(){return this.port});c(c$,"getDefaultPort",function(){return this.handler.getDefaultPort()});c(c$,"getProtocol",function(){return this.protocol});c(c$,"getHost",function(){return this.host});c(c$,"getFile",function(){return this.file});c(c$,"getRef",function(){return this.ref});j(c$,"equals",function(a){return!n(a,java.net.URL)?
!1:this.handler.equals2(this,a)},"~O");j(c$,"hashCode",function(){return-1!=this.$hashCode?this.$hashCode:this.$hashCode=this.handler.hashCode(this)});c(c$,"sameFile",function(a){return this.handler.sameFile(this,a)},"java.net.URL");j(c$,"toString",function(){return this.toExternalForm()});c(c$,"toExternalForm",function(){return this.handler.toExternalForm(this)});c(c$,"openConnection",function(){return this.handler.openConnection(this)});c(c$,"openStream",function(){return this.openConnection().getInputStream()});
c(c$,"getContent",function(){return this.openConnection().getInputStream()});c$.setURLStreamHandlerFactory=c(c$,"setURLStreamHandlerFactory",function(a){if(null!=java.net.URL.factory)throw Error("factory already defined");var b=Zystem.getSecurityManager();null!=b&&b.checkSetFactory();java.net.URL.handlers.clear();java.net.URL.factory=a},"java.net.URLStreamHandlerFactory");c$.getURLStreamHandler=c(c$,"getURLStreamHandler",function(a){var b=java.net.URL.handlers.get(a);null==b&&null!=java.net.URL.factory&&
(b=java.net.URL.factory.createURLStreamHandler(a));return b},"~S");E(c$,"factory",null);c$.handlers=c$.prototype.handlers=new java.util.Hashtable;c$.streamHandlerLock=c$.prototype.streamHandlerLock=new ea._O});s("java.net");v(null,"java.net.URLConnection",["java.lang.IllegalStateException","$.NullPointerException","java.net.UnknownServiceException","JU.Lst"],function(){c$=t(function(){this.url=null;this.doInput=!0;this.connected=this.doOutput=!1;this.requests=null;p(this,arguments)},java.net,"URLConnection");
c(c$,"setDoInput",function(a){if(this.connected)throw new IllegalStateException("Already connected");this.doInput=a},"~B");c(c$,"getDoInput",function(){return this.doInput});c(c$,"setDoOutput",function(a){if(this.connected)throw new IllegalStateException("Already connected");this.doOutput=a},"~B");c(c$,"getDoOutput",function(){return this.doOutput});q(c$,function(a){this.url=a},"java.net.URL");c(c$,"getURL",function(){return this.url});c(c$,"getInputStream",function(){throw new java.net.UnknownServiceException("protocol doesn't support input");
});c(c$,"getOutputStream",function(){throw new java.net.UnknownServiceException("protocol doesn't support output");});c(c$,"setRequestProperty",function(a,b){if(this.connected)throw new IllegalStateException("Already connected");if(null==a)throw new NullPointerException("key is null");null==this.requests&&(this.requests=new JU.Lst);for(var d=this.requests.size();0<=--d;)if(this.requests.get(d)[0].equals(a)){this.requests.get(d)[1]=b;return}this.requests.addLast(u(-1,[a,b]))},"~S,~S")});s("java.net");
c$=D(java.net,"URLEncoder");c$.encode=c(c$,"encode",function(a){return encodeURIComponent(a)},"~S");c$.encode=c(c$,"encode",function(a){return encodeURIComponent(a)},"~S,~S");E(c$,"digits","0123456789ABCDEF");s("java.net");v(null,"java.net.URLStreamHandler",["java.lang.IllegalArgumentException","$.SecurityException","$.UnsupportedOperationException"],function(){c$=D(java.net,"URLStreamHandler");c(c$,"openConnectionProxy",function(){throw new UnsupportedOperationException("Method not implemented.");
},"java.net.URL,java.net.Proxy");c(c$,"parseURL",function(a,b,d,g){var c=a.getProtocol(),e=a.getAuthority(),h=a.getUserInfo(),k=a.getHost(),l=a.getPort(),j=a.getPath(),m=a.getQuery(),r=a.getRef(),x=!1,y=!1;if(d<g){var F=b.indexOf("?"),y=F==d;-1!=F&&F<g&&(m=b.substring(F+1,g),g>F&&(g=F),b=b.substring(0,F))}var C=0;if(!(d<=g-4&&"/"==b.charAt(d)&&"/"==b.charAt(d+1)&&"/"==b.charAt(d+2)&&"/"==b.charAt(d+3))&&d<=g-2&&"/"==b.charAt(d)&&"/"==b.charAt(d+1)){d+=2;C=b.indexOf("/",d);0>C&&(C=b.indexOf("?",d),
0>C&&(C=g));k=e=b.substring(d,C);F=e.indexOf("@");-1!=F?(h=e.substring(0,F),k=e.substring(F+1)):h=null;if(null!=k){if(0<k.length&&"["==k.charAt(0))throw new IllegalArgumentException("Invalid host: "+k);F=k.indexOf(":");l=-1;0<=F&&(k.length>F+1&&(l=Integer.parseInt(k.substring(F+1))),k=k.substring(0,F))}else k="";if(-1>l)throw new IllegalArgumentException("Invalid port number :"+l);d=C;0<e.length&&(j="")}null==k&&(k="");d<g?"/"==b.charAt(d)?j=b.substring(d,g):null!=j&&0<j.length?(x=!0,F=j.lastIndexOf("/"),
y="",-1==F&&null!=e&&(y="/"),j=j.substring(0,F+1)+y+b.substring(d,g)):j=(null!=e?"/":"")+b.substring(d,g):y&&null!=j&&(F=j.lastIndexOf("/"),0>F&&(F=0),j=j.substring(0,F)+"/");null==j&&(j="");if(x){for(;0<=(C=j.indexOf("/./"));)j=j.substring(0,C)+j.substring(C+2);for(C=0;0<=(C=j.indexOf("/../",C));)0<C&&0<=(g=j.lastIndexOf("/",C-1))&&0!=j.indexOf("/../",g)?(j=j.substring(0,g)+j.substring(C+3),C=0):C+=3;for(;j.endsWith("/..");)if(C=j.indexOf("/.."),0<=(g=j.lastIndexOf("/",C-1)))j=j.substring(0,g+1);
else break;j.startsWith("./")&&2<j.length&&(j=j.substring(2));j.endsWith("/.")&&(j=j.substring(0,j.length-1))}this.setURL(a,c,k,l,e,h,j,m,r)},"java.net.URL,~S,~N,~N");c(c$,"getDefaultPort",function(){return-1});c(c$,"equals2",function(a,b){var d=a.getRef(),g=b.getRef();return(d===g||null!=d&&d.equals(g))&&this.sameFile(a,b)},"java.net.URL,java.net.URL");c(c$,"hashCode",function(a){var b=0,d=a.getProtocol();null!=d&&(b+=d.hashCode());b+=a.toString().hashCode();d=a.getFile();null!=d&&(b+=d.hashCode());
b=-1==a.getPort()?b+this.getDefaultPort():b+a.getPort();a=a.getRef();null!=a&&(b+=a.hashCode());return b},"java.net.URL");c(c$,"sameFile",function(a,b){if(!(a.getProtocol()===b.getProtocol()||null!=a.getProtocol()&&a.getProtocol().equalsIgnoreCase(b.getProtocol()))||!(a.getFile()===b.getFile()||null!=a.getFile()&&a.getFile().equals(b.getFile())))return!1;var d,g;d=-1!=a.getPort()?a.getPort():a.handler.getDefaultPort();g=-1!=b.getPort()?b.getPort():b.handler.getDefaultPort();return d!=g||!this.hostsEqual(a,
b)?!1:!0},"java.net.URL,java.net.URL");c(c$,"hostsEqual",function(a,b){return null!=a.getHost()&&null!=b.getHost()?a.getHost().equalsIgnoreCase(b.getHost()):null==a.getHost()&&null==b.getHost()},"java.net.URL,java.net.URL");c(c$,"toExternalForm",function(){return""},"java.net.URL");c(c$,"setURL",function(a,b,d,g,c,e,h,k,l){if(this!==a.handler)throw new SecurityException("handler for url different from this handler");a.set(a.getProtocol(),d,g,c,e,h,k,l)},"java.net.URL,~S,~S,~N,~S,~S,~S,~S,~S");c(c$,
"setURLDeprecated",function(a,b,d,g,c,e){var h=null,k=null;if(null!=d&&0!=d.length){var h=-1==g?d:d+":"+g,l=d.lastIndexOf("@");-1!=l&&(k=d.substring(0,l),d=d.substring(l+1))}var j=null,l=null;null!=c&&(j=c.lastIndexOf("?"),-1!=j?(l=c.substring(j+1),j=c.substring(0,j)):j=c);this.setURL(a,b,d,g,h,k,j,l,e)},"java.net.URL,~S,~S,~N,~S,~S")});s("java.net");N(java.net,"URLStreamHandlerFactory");s("java.text");c$=t(function(){this.pattern=null;p(this,arguments)},java.text,"MessageFormat");q(c$,function(a){this.pattern=
a},"~S");q(c$,function(a){this.pattern=a},"~S,java.util.Locale");c$.format=c(c$,"format",function(a,b){return a.replace(/\{(\d+)\}/g,function(a,g){var c=parseInt(g);return null==b?null:b[c]})},"~S,~A");c(c$,"format",function(a){return java.text.MessageFormat.format(this.pattern,[a])},"~O");v(null,"java.util.Random",["java.lang.IllegalArgumentException"],function(){c$=t(function(){this.haveNextNextGaussian=!1;this.nextNextGaussian=this.seed=0;p(this,arguments)},java.util,"Random",null,java.io.Serializable);
q(c$,function(){this.setSeed(Zystem.currentTimeMillis())});q(c$,function(a){this.setSeed(a)},"~N");c(c$,"next",function(a){this.seed=25214903917*this.seed+11&0xffffffffffff;return this.seed>>>48-a},"~N");c(c$,"nextBoolean",function(){return 0.5<Math.random()});c(c$,"nextBytes",function(){for(var a=0;a<bytes.length;a++)bytes[a]=Math.round(256*Math.random())},"~A");c(c$,"nextDouble",function(){return Math.random()});c(c$,"nextFloat",function(){return Math.random()});c(c$,"nextGaussian",function(){if(this.haveNextNextGaussian)return this.haveNextNextGaussian=
!1,this.nextNextGaussian;var a,b,d;do a=2*this.nextDouble()-1,b=2*this.nextDouble()-1,d=a*a+b*b;while(1<=d);d=Math.sqrt(-2*Math.log(d)/d);this.nextNextGaussian=b*d;this.haveNextNextGaussian=!0;return a*d});c(c$,"nextInt",function(){return Math.ceil(65535*Math.random())-32768});c(c$,"nextInt",function(a){if(0<a)return a=Math.min(a,31),Math.floor((2<<a-1)*Math.random());throw new IllegalArgumentException;},"~N");c(c$,"nextLong",function(){return Math.ceil(4294967295*Math.random())-2147483648});c(c$,
"setSeed",function(a){Math.seedrandom(a)},"~N");E(c$,"multiplier",25214903917)});var ia=[],fa=Math,ja=52,pa=void 0,qa=void 0,Ia=function(a){var b,d,g=this,c=a.length,e=0,h=g.i=g.j=g.m=0;g.S=[];g.c=[];for(c||(a=[c++]);256>e;)g.S[e]=e++;for(e=0;256>e;e++)b=g.S[e],h=h+b+a[e%c]&255,d=g.S[h],g.S[e]=d,g.S[h]=b;g.g=function(a){var b=g.S,d=g.i+1&255,c=b[d],f=g.j+c&255,e=b[f];b[d]=e;b[f]=c;for(var h=b[c+e&255];--a;)d=d+1&255,c=b[d],f=f+c&255,e=b[f],b[d]=e,b[f]=c,h=256*h+b[c+e&255];g.i=d;g.j=f;return h};g.g(256)},
ra=function(a,b,d,g){d=[];if(b&&"object"==typeof a)for(g in a)if(5>g.indexOf("S"))try{d.push(ra(a[g],b-1))}catch(c){}return d.length?d:""+a},ka=function(a,b,d,g){a+="";for(g=d=0;g<a.length;g++){var c=b,e=g&255,h=(d^=19*b[g&255])+a.charCodeAt(g);c[e]=h&255}a="";for(g in b)a+=String.fromCharCode(b[g]);return a};fa.seedrandom=function(a,b){var d=[],g;a=ka(ra(b?[a,ia]:arguments.length?a:[(new Date).getTime(),ia,window],3),d);g=new Ia(d);ka(g.S,ia);fa.random=function(){for(var a=g.g(6),b=qa,d=0;a<ja;)a=
256*(a+d),b*=256,d=g.g(1);for(;a>=pa;)a/=2,b/=2,d>>>=1;return(a+d)/b};return a};qa=fa.pow(256,6);ja=fa.pow(2,ja);pa=2*ja;ka(fa.random(),ia);v(["java.util.Collection"],"java.util.AbstractCollection",["java.lang.StringBuilder","$.UnsupportedOperationException","java.lang.reflect.Array"],function(){c$=D(java.util,"AbstractCollection",null,java.util.Collection);q(c$,function(){});j(c$,"add",function(){throw new UnsupportedOperationException;},"~O");j(c$,"addAll",function(a){var b=!1;for(a=a.iterator();a.hasNext();)this.add(a.next())&&
(b=!0);return b},"java.util.Collection");j(c$,"clear",function(){for(var a=this.iterator();a.hasNext();)a.next(),a.remove()});j(c$,"contains",function(a){var b=this.iterator();if(null!=a)for(;b.hasNext();){if(a.equals(b.next()))return!0}else for(;b.hasNext();)if(null==b.next())return!0;return!1},"~O");j(c$,"containsAll",function(a){for(a=a.iterator();a.hasNext();)if(!this.contains(a.next()))return!1;return!0},"java.util.Collection");j(c$,"isEmpty",function(){return 0==this.size()});j(c$,"remove",
function(a){var b=this.iterator();if(null!=a)for(;b.hasNext();){if(a.equals(b.next()))return b.remove(),!0}else for(;b.hasNext();)if(null==b.next())return b.remove(),!0;return!1},"~O");j(c$,"removeAll",function(a){for(var b=!1,d=this.iterator();d.hasNext();)a.contains(d.next())&&(d.remove(),b=!0);return b},"java.util.Collection");j(c$,"retainAll",function(a){for(var b=!1,d=this.iterator();d.hasNext();)a.contains(d.next())||(d.remove(),b=!0);return b},"java.util.Collection");c(c$,"toArray",function(){for(var a=
this.size(),b=0,d=this.iterator(),g=Array(a);b<a;)g[b++]=d.next();return g});c(c$,"toArray",function(a){var b=this.size(),d=0;b>a.length&&(a=a.getClass().getComponentType(),a=java.lang.reflect.Array.newInstance(a,b));for(var g,b=this.iterator();b.hasNext()&&((g=b.next())||1);)a[d++]=g;d<a.length&&(a[d]=null);return a},"~A");j(c$,"toString",function(){if(this.isEmpty())return"[]";var a=new StringBuilder(16*this.size());a.append("[");for(var b=this.iterator();b.hasNext();){var d=b.next();d!==this?a.append(d):
a.append("(this Collection)");b.hasNext()&&a.append(", ")}a.append("]");return a.toString()})});v("java.util.AbstractCollection $.Iterator $.List $.ListIterator $.RandomAccess $.NoSuchElementException".split(" "),"java.util.AbstractList",["java.lang.IllegalArgumentException","$.IllegalStateException","$.IndexOutOfBoundsException","$.UnsupportedOperationException","java.util.ConcurrentModificationException"],function(){c$=t(function(){this.modCount=0;p(this,arguments)},java.util,"AbstractList",java.util.AbstractCollection,
java.util.List);c(c$,"add",function(){throw new UnsupportedOperationException;},"~N,~O");c(c$,"add",function(a){this.add(this.size(),a);return!0},"~O");c(c$,"addAll",function(a,b){for(var d=b.iterator();d.hasNext();)this.add(a++,d.next());return!b.isEmpty()},"~N,java.util.Collection");j(c$,"clear",function(){this.removeRange(0,this.size())});j(c$,"equals",function(a){if(this===a)return!0;if(n(a,java.util.List)){if(a.size()!=this.size())return!1;var b=this.iterator();for(a=a.iterator();b.hasNext();){var d=
b.next(),g=a.next();if(!(null==d?null==g:d.equals(g)))return!1}return!0}return!1},"~O");j(c$,"hashCode",function(){for(var a=1,b=this.iterator();b.hasNext();)var d=b.next(),a=31*a+(null==d?0:d.hashCode());return a});j(c$,"indexOf",function(a){var b=this.listIterator();if(null!=a)for(;b.hasNext();){if(a.equals(b.next()))return b.previousIndex()}else for(;b.hasNext();)if(null==b.next())return b.previousIndex();return-1},"~O");j(c$,"iterator",function(){return new java.util.AbstractListSimpleListIterator(this)});
j(c$,"lastIndexOf",function(a){var b=this.listIterator(this.size());if(null!=a)for(;b.hasPrevious();){if(a.equals(b.previous()))return b.nextIndex()}else for(;b.hasPrevious();)if(null==b.previous())return b.nextIndex();return-1},"~O");c(c$,"listIterator",function(a){a||(a=0);return new java.util.AbstractListFullListIterator(this,a)},"~N");c(c$,"remove",function(){throw new UnsupportedOperationException;},"~N");c(c$,"removeRange",function(a,b){for(var d=this.listIterator(a),g=a;g<b;g++)d.next(),d.remove()},
"~N,~N");j(c$,"set",function(){throw new UnsupportedOperationException;},"~N,~O");j(c$,"subList",function(a,b){if(0<=a&&b<=this.size()){if(a<=b)return n(this,java.util.RandomAccess)?new java.util.AbstractList.SubAbstractListRandomAccess(this,a,b):new java.util.AbstractList.SubAbstractList(this,a,b);throw new IllegalArgumentException;}throw new IndexOutOfBoundsException;},"~N,~N");M(self.c$);c$=t(function(){this.pos=-1;this.expectedModCount=0;this.lastPosition=-1;p(this,arguments)},java.util,"AbstractListSimpleListIterator",
null,java.util.Iterator);q(c$,function(a){this._list=a;this.expectedModCount=a.modCount},"java.util.AbstractList");j(c$,"hasNext",function(){return this.pos+1<this._list.size()});j(c$,"next",function(){if(this.expectedModCount==this._list.modCount)try{var a=this._list.get(this.pos+1);this.lastPosition=++this.pos;return a}catch(b){if(n(b,IndexOutOfBoundsException))throw new java.util.NoSuchElementException;throw b;}throw new java.util.ConcurrentModificationException;});j(c$,"remove",function(){if(this.expectedModCount==
this._list.modCount){try{this._list.remove(this.lastPosition)}catch(a){if(n(a,IndexOutOfBoundsException))throw new IllegalStateException;throw a;}this._list.modCount!=this.expectedModCount&&this.expectedModCount++;this.pos==this.lastPosition&&this.pos--;this.lastPosition=-1}else throw new java.util.ConcurrentModificationException;});c$=L();M(self.c$);c$=t(function(){p(this,arguments)},java.util,"AbstractListFullListIterator",java.util.AbstractListSimpleListIterator,java.util.ListIterator);q(c$,function(a,
b){I(this,java.util.AbstractListFullListIterator,[a]);if(0<=b&&b<=this._list.size())this.pos=b-1;else throw new IndexOutOfBoundsException;},"java.util.AbstractList,~N");j(c$,"add",function(a){if(this.expectedModCount==this._list.modCount){try{this._list.add(this.pos+1,a)}catch(b){if(n(b,IndexOutOfBoundsException))throw new java.util.NoSuchElementException;throw b;}this.pos++;this.lastPosition=-1;this._list.modCount!=this.expectedModCount&&this.expectedModCount++}else throw new java.util.ConcurrentModificationException;
},"~O");j(c$,"hasPrevious",function(){return 0<=this.pos});j(c$,"nextIndex",function(){return this.pos+1});j(c$,"previous",function(){if(this.expectedModCount==this._list.modCount)try{var a=this._list.get(this.pos);this.lastPosition=this.pos;this.pos--;return a}catch(b){if(n(b,IndexOutOfBoundsException))throw new java.util.NoSuchElementException;throw b;}throw new java.util.ConcurrentModificationException;});j(c$,"previousIndex",function(){return this.pos});j(c$,"set",function(a){if(this.expectedModCount==
this._list.modCount)try{this._list.set(this.lastPosition,a)}catch(b){if(n(b,IndexOutOfBoundsException))throw new IllegalStateException;throw b;}else throw new java.util.ConcurrentModificationException;},"~O");c$=L();M(self.c$);c$=D(java.util.AbstractList,"SubAbstractListRandomAccess",java.util.AbstractList.SubAbstractList,java.util.RandomAccess);c$=L();M(self.c$);c$=t(function(){this.fullList=null;this.$size=this.offset=0;p(this,arguments)},java.util.AbstractList,"SubAbstractList",java.util.AbstractList);
q(c$,function(a,b,d){I(this,java.util.AbstractList.SubAbstractList);this.fullList=a;this.modCount=this.fullList.modCount;this.offset=b;this.$size=d-b},"java.util.AbstractList,~N,~N");c(c$,"add",function(a,b){if(this.modCount==this.fullList.modCount)if(0<=a&&a<=this.$size)this.fullList.add(a+this.offset,b),this.$size++,this.modCount=this.fullList.modCount;else throw new IndexOutOfBoundsException;else throw new java.util.ConcurrentModificationException;},"~N,~O");c(c$,"addAll",function(a,b){if(this.modCount==
this.fullList.modCount){if(0<=a&&a<=this.$size){var d=this.fullList.addAll(a+this.offset,b);d&&(this.$size+=b.size(),this.modCount=this.fullList.modCount);return d}throw new IndexOutOfBoundsException;}throw new java.util.ConcurrentModificationException;},"~N,java.util.Collection");c(c$,"addAll",function(a){if(this.modCount==this.fullList.modCount){var b=this.fullList.addAll(this.offset+this.$size,a);b&&(this.$size+=a.size(),this.modCount=this.fullList.modCount);return b}throw new java.util.ConcurrentModificationException;
},"java.util.Collection");c(c$,"get",function(a){if(this.modCount==this.fullList.modCount){if(0<=a&&a<this.$size)return this.fullList.get(a+this.offset);throw new IndexOutOfBoundsException;}throw new java.util.ConcurrentModificationException;},"~N");j(c$,"iterator",function(){return this.listIterator(0)});c(c$,"listIterator",function(a){if(this.modCount==this.fullList.modCount){if(0<=a&&a<=this.$size)return new java.util.AbstractList.SubAbstractList.SubAbstractListIterator(this.fullList.listIterator(a+
this.offset),this,this.offset,this.$size);throw new IndexOutOfBoundsException;}throw new java.util.ConcurrentModificationException;},"~N");c(c$,"remove",function(a){if(this.modCount==this.fullList.modCount){if(0<=a&&a<this.$size)return a=this.fullList.remove(a+this.offset),this.$size--,this.modCount=this.fullList.modCount,a;throw new IndexOutOfBoundsException;}throw new java.util.ConcurrentModificationException;},"~N");c(c$,"removeRange",function(a,b){if(a!=b)if(this.modCount==this.fullList.modCount)this.fullList.removeRange(a+
this.offset,b+this.offset),this.$size-=b-a,this.modCount=this.fullList.modCount;else throw new java.util.ConcurrentModificationException;},"~N,~N");c(c$,"set",function(a,b){if(this.modCount==this.fullList.modCount){if(0<=a&&a<this.$size)return this.fullList.set(a+this.offset,b);throw new IndexOutOfBoundsException;}throw new java.util.ConcurrentModificationException;},"~N,~O");j(c$,"size",function(){return this.$size});c(c$,"sizeChanged",function(a){a?this.$size++:this.$size--;this.modCount=this.fullList.modCount},
"~B");M(self.c$);c$=t(function(){this.iterator=this.subList=null;this.end=this.start=0;p(this,arguments)},java.util.AbstractList.SubAbstractList,"SubAbstractListIterator",null,java.util.ListIterator);q(c$,function(a,b,d,g){this.iterator=a;this.subList=b;this.start=d;this.end=this.start+g},"java.util.ListIterator,java.util.AbstractList.SubAbstractList,~N,~N");c(c$,"add",function(a){this.iterator.add(a);this.subList.sizeChanged(!0);this.end++},"~O");j(c$,"hasNext",function(){return this.iterator.nextIndex()<
this.end});j(c$,"hasPrevious",function(){return this.iterator.previousIndex()>=this.start});c(c$,"next",function(){if(this.iterator.nextIndex()<this.end)return this.iterator.next();throw new java.util.NoSuchElementException;});c(c$,"nextIndex",function(){return this.iterator.nextIndex()-this.start});c(c$,"previous",function(){if(this.iterator.previousIndex()>=this.start)return this.iterator.previous();throw new java.util.NoSuchElementException;});c(c$,"previousIndex",function(){var a=this.iterator.previousIndex();
return a>=this.start?a-this.start:-1});c(c$,"remove",function(){this.iterator.remove();this.subList.sizeChanged(!1);this.end--});c(c$,"set",function(a){this.iterator.set(a)},"~O");c$=L();c$=L()});v(["java.util.Map"],"java.util.AbstractMap",["java.lang.StringBuilder","$.UnsupportedOperationException","java.util.AbstractCollection","$.AbstractSet","$.Iterator"],function(){c$=t(function(){this.valuesCollection=this.$keySet=null;p(this,arguments)},java.util,"AbstractMap",null,java.util.Map);q(c$,function(){});
j(c$,"clear",function(){this.entrySet().clear()});j(c$,"containsKey",function(a){var b=this.entrySet().iterator();if(null!=a)for(;b.hasNext();){if(a.equals(b.next().getKey()))return!0}else for(;b.hasNext();)if(null==b.next().getKey())return!0;return!1},"~O");j(c$,"containsValue",function(a){var b=this.entrySet().iterator();if(null!=a)for(;b.hasNext();){if(a.equals(b.next().getValue()))return!0}else for(;b.hasNext();)if(null==b.next().getValue())return!0;return!1},"~O");j(c$,"equals",function(a){if(this===
a)return!0;if(n(a,java.util.Map)){if(this.size()!=a.size())return!1;a=a.entrySet();for(var b=this.entrySet().iterator();b.hasNext();)if(!a.contains(b.next()))return!1;return!0}return!1},"~O");j(c$,"get",function(a){var b=this.entrySet().iterator();if(null!=a)for(;b.hasNext();){var d=b.next();if(a.equals(d.getKey()))return d.getValue()}else for(;b.hasNext();)if(d=b.next(),null==d.getKey())return d.getValue();return null},"~O");j(c$,"hashCode",function(){for(var a=0,b=this.entrySet().iterator();b.hasNext();)a+=
b.next().hashCode();return a});j(c$,"isEmpty",function(){return 0==this.size()});j(c$,"keySet",function(){null==this.$keySet&&(this.$keySet=(R("java.util.AbstractMap$1")?0:java.util.AbstractMap.$AbstractMap$1$(),P(java.util.AbstractMap$1,this,null)));return this.$keySet});j(c$,"put",function(){throw new UnsupportedOperationException;},"~O,~O");j(c$,"putAll",function(a){this.putAllAM(a)},"java.util.Map");j(c$,"putAllAM",function(a){var b;for(a=a.entrySet().iterator();a.hasNext()&&((b=a.next())||1);)this.put(b.getKey(),
b.getValue())},"java.util.Map");j(c$,"remove",function(a){var b=this.entrySet().iterator();if(null!=a)for(;b.hasNext();){var d=b.next();if(a.equals(d.getKey()))return b.remove(),d.getValue()}else for(;b.hasNext();)if(d=b.next(),null==d.getKey())return b.remove(),d.getValue();return null},"~O");j(c$,"size",function(){return this.entrySet().size()});j(c$,"toString",function(){if(this.isEmpty())return"{}";var a=new StringBuilder(28*this.size());a.append("{");for(var b=this.entrySet().iterator();b.hasNext();){var d=
b.next(),g=d.getKey();g!==this?a.append(g):a.append("(this Map)");a.append("=");d=d.getValue();d!==this?a.append(d):a.append("(this Map)");b.hasNext()&&a.append(", ")}a.append("}");return a.toString()});j(c$,"values",function(){null==this.valuesCollection&&(this.valuesCollection=(R("java.util.AbstractMap$2")?0:java.util.AbstractMap.$AbstractMap$2$(),P(java.util.AbstractMap$2,this,null)));return this.valuesCollection});c(c$,"clone",function(){return this.cloneAM()});c(c$,"cloneAM",function(){var a=
la(this);a.$keySet=null;a.valuesCollection=null;return a});c$.$AbstractMap$1$=function(){M(self.c$);c$=da(java.util,"AbstractMap$1",java.util.AbstractSet);j(c$,"contains",function(a){return this.b$["java.util.AbstractMap"].containsKey(a)},"~O");j(c$,"size",function(){return this.b$["java.util.AbstractMap"].size()});j(c$,"iterator",function(){return R("java.util.AbstractMap$1$1")?0:java.util.AbstractMap.$AbstractMap$1$1$(),P(java.util.AbstractMap$1$1,this,null)});c$=L()};c$.$AbstractMap$1$1$=function(){M(self.c$);
c$=t(function(){V(this,arguments);this.setIterator=null;p(this,arguments)},java.util,"AbstractMap$1$1",null,java.util.Iterator);O(c$,function(){this.setIterator=this.b$["java.util.AbstractMap"].entrySet().iterator()});j(c$,"hasNext",function(){return this.setIterator.hasNext()});j(c$,"next",function(){return this.setIterator.next().getKey()});j(c$,"remove",function(){this.setIterator.remove()});c$=L()};c$.$AbstractMap$2$=function(){M(self.c$);c$=da(java.util,"AbstractMap$2",java.util.AbstractCollection);
j(c$,"size",function(){return this.b$["java.util.AbstractMap"].size()});j(c$,"contains",function(a){return this.b$["java.util.AbstractMap"].containsValue(a)},"~O");j(c$,"iterator",function(){return R("java.util.AbstractMap$2$1")?0:java.util.AbstractMap.$AbstractMap$2$1$(),P(java.util.AbstractMap$2$1,this,null)});c$=L()};c$.$AbstractMap$2$1$=function(){M(self.c$);c$=t(function(){V(this,arguments);this.setIterator=null;p(this,arguments)},java.util,"AbstractMap$2$1",null,java.util.Iterator);O(c$,function(){this.setIterator=
this.b$["java.util.AbstractMap"].entrySet().iterator()});j(c$,"hasNext",function(){return this.setIterator.hasNext()});j(c$,"next",function(){return this.setIterator.next().getValue()});j(c$,"remove",function(){this.setIterator.remove()});c$=L()}});v(["java.util.AbstractCollection","$.Set"],"java.util.AbstractSet",null,function(){c$=D(java.util,"AbstractSet",java.util.AbstractCollection,java.util.Set);j(c$,"equals",function(a){return this===a?!0:n(a,java.util.Set)?this.size()==a.size()&&this.containsAll(a):
!1},"~O");j(c$,"hashCode",function(){for(var a=0,b=this.iterator();b.hasNext();)var d=b.next(),a=a+(null==d?0:d.hashCode());return a});j(c$,"removeAll",function(a){var b=!1;if(this.size()<=a.size())for(var d=this.iterator();d.hasNext();)a.contains(d.next())&&(d.remove(),b=!0);else for(d=a.iterator();d.hasNext();)b=this.remove(d.next())||b;return b},"java.util.Collection")});v(["java.util.AbstractList","$.List","$.RandomAccess"],"java.util.ArrayList",["java.lang.IllegalArgumentException","$.IndexOutOfBoundsException",
"java.lang.reflect.Array","java.util.Arrays"],function(){c$=t(function(){this.lastIndex=this.firstIndex=0;this.array=null;p(this,arguments)},java.util,"ArrayList",java.util.AbstractList,[java.util.List,Cloneable,java.io.Serializable,java.util.RandomAccess]);Y(c$,function(){this.setup(0)});c(c$,"setup",function(a){this.firstIndex=this.lastIndex=0;try{this.array=this.newElementArray(a)}catch(b){if(n(b,NegativeArraySizeException))throw new IllegalArgumentException;throw b;}},"~N");c(c$,"newElementArray",
($fz=function(a){return Array(a)},$fz.isPrivate=!0,$fz),"~N");j(c$,"add",function(a,b){if(1==arguments.length)return this.add1(a);var d=this.size();if(0<a&&a<d){if(0==this.firstIndex&&this.lastIndex==this.array.length)this.growForInsert(a,1);else if(a<Math.floor(d/2)&&0<this.firstIndex||this.lastIndex==this.array.length)Zystem.arraycopy(this.array,this.firstIndex,this.array,--this.firstIndex,a);else{var g=a+this.firstIndex;Zystem.arraycopy(this.array,g,this.array,g+1,d-a);this.lastIndex++}this.array[a+
this.firstIndex]=b}else if(0==a)0==this.firstIndex&&this.growAtFront(1),this.array[--this.firstIndex]=b;else if(a==d)this.lastIndex==this.array.length&&this.growAtEnd(1),this.array[this.lastIndex++]=b;else throw new IndexOutOfBoundsException;this.modCount++},"~N,~O");j(c$,"add1",function(a){this.lastIndex==this.array.length&&this.growAtEnd(1);this.array[this.lastIndex++]=a;this.modCount++;return!0},"~O");j(c$,"addAll",function(a){var b=a.size();if(0<b){this.lastIndex>this.array.length-b&&this.growAtEnd(b);
a=a.iterator();for(b=this.lastIndex+b;this.lastIndex<b;)this.array[this.lastIndex++]=a.next();this.modCount++;return!0}return!1},"java.util.Collection");j(c$,"clear",function(){this.firstIndex!=this.lastIndex&&(this.fill(this.firstIndex,this.lastIndex),this.firstIndex=this.lastIndex=0,this.modCount++)});c(c$,"fill",function(a,b){for(var d=b;--d>=a;)this.array[d]=null},"~N,~N");c(c$,"clone",function(){try{var a=W(this,java.util.ArrayList,"clone",[]);a.array=this.array.clone();return a}catch(b){if(n(b,
CloneNotSupportedException))return null;throw b;}});j(c$,"contains",function(a){if(null!=a)for(var b=this.firstIndex;b<this.lastIndex;b++){if(a.equals(this.array[b]))return!0}else for(b=this.firstIndex;b<this.lastIndex;b++)if(null==this.array[b])return!0;return!1},"~O");c(c$,"ensureCapacity",function(a){this.array.length<a&&(0<this.firstIndex?this.growAtFront(a-this.array.length):this.growAtEnd(a-this.array.length))},"~N");j(c$,"get",function(a){if(0<=a&&a<this.size())return this.array[this.firstIndex+
a];throw new IndexOutOfBoundsException;},"~N");c(c$,"growAtEnd",($fz=function(a){var b=this.size();if(this.firstIndex>=a-(this.array.length-this.lastIndex))a=this.lastIndex-this.firstIndex,0<b&&(Zystem.arraycopy(this.array,this.firstIndex,this.array,0,b),this.fill(a<this.firstIndex?this.firstIndex:a,this.array.length)),this.firstIndex=0,this.lastIndex=a;else{var d=Math.floor(b/2);a>d&&(d=a);12>d&&(d=12);a=this.newElementArray(b+d);0<b&&Zystem.arraycopy(this.array,this.firstIndex,a,this.firstIndex,
b);this.array=a}},$fz.isPrivate=!0,$fz),"~N");c(c$,"growAtFront",($fz=function(a){var b=this.size();if(this.array.length-this.lastIndex>=a)a=this.array.length-b,0<b&&(Zystem.arraycopy(this.array,this.firstIndex,this.array,a,b),this.fill(this.firstIndex,this.firstIndex+b>a?a:this.firstIndex+b)),this.firstIndex=a,this.lastIndex=this.array.length;else{var d=Math.floor(b/2);a>d&&(d=a);12>d&&(d=12);a=this.newElementArray(b+d);0<b&&Zystem.arraycopy(this.array,this.firstIndex,a,a.length-b,b);this.firstIndex=
a.length-b;this.lastIndex=a.length;this.array=a}},$fz.isPrivate=!0,$fz),"~N");c(c$,"growForInsert",($fz=function(a,b){var d=this.size(),g=Math.floor(d/2);b>g&&(g=b);12>g&&(g=12);var c=this.newElementArray(d+g);if(a<Math.floor(d/2)){var e=c.length-(d+b);Zystem.arraycopy(this.array,a,c,a+g,d-a);Zystem.arraycopy(this.array,this.firstIndex,c,e,a);this.firstIndex=e;this.lastIndex=c.length}else Zystem.arraycopy(this.array,this.firstIndex,c,0,a),Zystem.arraycopy(this.array,a,c,a+b,d-a),this.firstIndex=0,
this.lastIndex+=b;this.array=c},$fz.isPrivate=!0,$fz),"~N,~N");j(c$,"indexOf",function(a){if(null!=a)for(var b=this.firstIndex;b<this.lastIndex;b++){if(a.equals(this.array[b]))return b-this.firstIndex}else for(b=this.firstIndex;b<this.lastIndex;b++)if(null==this.array[b])return b-this.firstIndex;return-1},"~O");j(c$,"isEmpty",function(){return this.lastIndex==this.firstIndex});j(c$,"lastIndexOf",function(a){if(null!=a)for(var b=this.lastIndex-1;b>=this.firstIndex;b--){if(a.equals(this.array[b]))return b-
this.firstIndex}else for(b=this.lastIndex-1;b>=this.firstIndex;b--)if(null==this.array[b])return b-this.firstIndex;return-1},"~O");j(c$,"remove",function(a){return"number"==typeof a?this._removeItemAt(a):this._removeObject(a)},"~N");j(c$,"_removeItemAt",function(a){var b,d=this.size();if(0<=a&&a<d)if(a==d-1)b=this.array[--this.lastIndex],this.array[this.lastIndex]=null;else if(0==a)b=this.array[this.firstIndex],this.array[this.firstIndex++]=null;else{var g=this.firstIndex+a;b=this.array[g];a<Math.floor(d/
2)?(Zystem.arraycopy(this.array,this.firstIndex,this.array,this.firstIndex+1,a),this.array[this.firstIndex++]=null):(Zystem.arraycopy(this.array,g+1,this.array,g,d-a-1),this.array[--this.lastIndex]=null)}else throw new IndexOutOfBoundsException;this.modCount++;return b},"~N");c(c$,"_removeObject",function(a){a=this.indexOf(a);return 0>a?null:this._removeItemAt(a)},"~O");j(c$,"removeRange",function(a,b){if(0<=a&&a<=b&&b<=this.size()){if(a!=b){var d=this.size();b==d?(this.fill(this.firstIndex+a,this.lastIndex),
this.lastIndex=this.firstIndex+a):0==a?(this.fill(this.firstIndex,this.firstIndex+b),this.firstIndex+=b):(Zystem.arraycopy(this.array,this.firstIndex+b,this.array,this.firstIndex+a,d-b),d=this.lastIndex+a-b,this.fill(d,this.lastIndex),this.lastIndex=d);this.modCount++}}else throw new IndexOutOfBoundsException;},"~N,~N");j(c$,"set",function(a,b){if(0<=a&&a<this.size()){var d=this.array[this.firstIndex+a];this.array[this.firstIndex+a]=b;return d}throw new IndexOutOfBoundsException;},"~N,~O");j(c$,"size",
function(){return this.lastIndex-this.firstIndex});j(c$,"toArray",function(a){var b=this.size();if(!a||b>a.length)return this.array.slice(this.firstIndex,this.firstIndex+b);Zystem.arraycopy(this.array,this.firstIndex,a,0,b);b<a.length&&(a[b]=null);return a},"~O");c(c$,"trimToSize",function(){var a=this.size(),b=this.newElementArray(a);Zystem.arraycopy(this.array,this.firstIndex,b,0,a);this.array=b;this.firstIndex=0;this.lastIndex=this.array.length})});v(["java.util.AbstractList","$.RandomAccess"],
"java.util.Arrays",["java.lang.ArrayIndexOutOfBoundsException","$.IllegalArgumentException","$.NullPointerException"],function(){c$=D(java.util,"Arrays");c$.sort=j(c$,"sort",function(a,b,d,g){switch(arguments.length){case 1:for(var c=a.sort(function(a,b){return"string"==typeof a||n(a,Comparable)?a.compareTo(b):a-b}),e=0;e<a.length;e++)a[e]=c[e];break;case 2:c=a.sort(function(a,d){return null!=b?b.compare(a,d):"string"==typeof a||n(a,Comparable)?a.compareTo(d):a-d});for(e=0;e<a.length;e++)a[e]=c[e];
break;case 3:var h=b,k=d;this.rangeCheck(a.length,h,k);c=[];for(e=h;e<k;e++)c[e-h]=a[e];c=c.sort(function(a,b){return"string"==typeof a||n(a,Comparable)?a.compareTo(b):a-b});for(e=h;e<k;e++)a[e]=c[e-h];break;case 4:h=b;k=d;b=g;this.rangeCheck(a.length,h,k);c=[];for(e=h;e<k;e++)c[e-h]=a[e];c=c.sort(function(a,d){return null!=b?b.compare(a,d):"string"==typeof a||n(a,Comparable)?a.compareTo(d):a-d});for(e=h;e<k;e++)a[e]=c[e-h]}});c$.rangeCheck=c(c$,"rangeCheck",($fz=function(a,b,d){if(b>d)throw new IllegalArgumentException("fromIndex("+
b+") > toIndex("+d+")");if(0>b)throw new ArrayIndexOutOfBoundsException(b);if(d>a)throw new ArrayIndexOutOfBoundsException(d);},$fz.isPrivate=!0,$fz),"~N,~N,~N");c$.binarySearch=c(c$,"binarySearch",function(a,b){for(var d=0,g=a.length-1;d<=g;){var c=d+g>>1,e=a[c];if(e<b)d=c+1;else if(e>b)g=c-1;else return c}return-(d+1)},"~A,~N");c$.binarySearch=c(c$,"binarySearch",function(a,b){for(var d=0,g=a.length-1;d<=g;){var c=d+g>>1,e=a[c].compareTo(b);if(0>e)d=c+1;else if(0<e)g=c-1;else return c}return-(d+
1)},"~A,~O");c$.binarySearch=c(c$,"binarySearch",function(a,b,d){if(null==d)return java.util.Arrays.binarySearch(a,b);for(var g=0,c=a.length-1;g<=c;){var e=g+c>>1,h=d.compare(a[e],b);if(0>h)g=e+1;else if(0<h)c=e-1;else return e}return-(g+1)},"~A,~O,java.util.Comparator");c$.equals=c(c$,"equals",function(a,b){if(a===b)return!0;if(null==a||null==b)return!1;var d=a.length;if(b.length!=d)return!1;for(var g=0;g<d;g++){var c=a[g],e=b[g];if(!(null==c?null==e:null==c.equals?c==e:c.equals(e)))return!1}return!0},
"~A,~A");c$.fill=j(c$,"fill",function(a,b,d,g){2==arguments.length&&(g=b,b=0,d=a.length);java.util.Arrays.rangeCheck(a.length,b,d);for(var c=b;c<d;c++)a[c]=g});c$.copyOf=j(c$,"copyOf",function(a,b){for(var d=u(b,null),g=Math.min(a.length,b);0<=--g;)d[g]=a[g];return d});c$.asList=c(c$,"asList",function(a){return new java.util.Arrays.ArrayList(1==arguments.length&&"Array"==sa(a)?a:arguments)},"~A");M(self.c$);c$=t(function(){this.a=null;p(this,arguments)},java.util.Arrays,"ArrayList",java.util.AbstractList,
[java.util.RandomAccess,java.io.Serializable]);q(c$,function(a){I(this,java.util.Arrays.ArrayList,[]);if(null==a)throw new NullPointerException;this.a=a},"~A");j(c$,"size",function(){return this.a.length});c(c$,"toArray",function(){return this.a.clone()});j(c$,"get",function(a){return this.a[a]},"~N");j(c$,"set",function(a,b){var d=this.a[a];this.a[a]=b;return d},"~N,~O");j(c$,"indexOf",function(a){if(null==a)for(var b=0;b<this.a.length;b++){if(null==this.a[b])return b}else for(b=0;b<this.a.length;b++)if(a.equals(this.a[b]))return b;
return-1},"~O");j(c$,"contains",function(a){return-1!=this.indexOf(a)},"~O");c$=L();E(c$,"INSERTIONSORT_THRESHOLD",7)});v("java.util.AbstractList $.AbstractMap $.AbstractSet $.Collection $.Iterator $.List $.ListIterator $.Map $.RandomAccess $.Set $.SortedMap $.SortedSet java.lang.NullPointerException $.UnsupportedOperationException java.lang.reflect.Array".split(" "),"java.util.Collections","java.lang.ArrayIndexOutOfBoundsException $.ClassCastException $.IllegalArgumentException $.IndexOutOfBoundsException java.util.ArrayList $.Arrays $.Enumeration java.util.Map.Entry java.util.NoSuchElementException $.Random".split(" "),
function(){c$=D(java.util,"Collections");c$.binarySearch=c(c$,"binarySearch",function(a,b){if(null==a)throw new NullPointerException;if(a.isEmpty())return-1;if(!n(a,java.util.RandomAccess)){for(var d=a.listIterator();d.hasNext();){var g;if(0>=(g=b.compareTo(d.next())))return 0==g?d.previousIndex():-d.previousIndex()-1}return-a.size()-1}var d=0,c=a.size(),e=c-1;for(g=-1;d<=e;)if(c=d+e>>1,0<(g=b.compareTo(a.get(c))))d=c+1;else{if(0==g)return c;e=c-1}return-c-(0>g?1:2)},"java.util.List,~O");c$.binarySearch=
c(c$,"binarySearch",function(a,b,d){if(null==d)return java.util.Collections.binarySearch(a,b);if(!n(a,java.util.RandomAccess)){for(var g=a.listIterator();g.hasNext();){var c;if(0>=(c=d.compare(b,g.next())))return 0==c?g.previousIndex():-g.previousIndex()-1}return-a.size()-1}var g=0,e=a.size(),h=e-1;for(c=-1;g<=h;)if(e=g+h>>1,0<(c=d.compare(b,a.get(e))))g=e+1;else{if(0==c)return e;h=e-1}return-e-(0>c?1:2)},"java.util.List,~O,java.util.Comparator");c$.copy=c(c$,"copy",function(a,b){if(a.size()<b.size())throw new ArrayIndexOutOfBoundsException;
for(var d=b.iterator(),g=a.listIterator();d.hasNext();){try{g.next()}catch(c){if(n(c,java.util.NoSuchElementException))throw new ArrayIndexOutOfBoundsException;throw c;}g.set(d.next())}},"java.util.List,java.util.List");c$.enumeration=c(c$,"enumeration",function(a){R("java.util.Collections$1")||java.util.Collections.$Collections$1$(a);return P(java.util.Collections$1,this,null)},"java.util.Collection");c$.fill=c(c$,"fill",function(a,b){for(var d=a.listIterator();d.hasNext();)d.next(),d.set(b)},"java.util.List,~O");
c$.max=c(c$,"max",function(a){a=a.iterator();for(var b=a.next();a.hasNext();){var d=a.next();0>b.compareTo(d)&&(b=d)}return b},"java.util.Collection");c$.max=c(c$,"max",function(a,b){for(var d=a.iterator(),g=d.next();d.hasNext();){var c=d.next();0>b.compare(g,c)&&(g=c)}return g},"java.util.Collection,java.util.Comparator");c$.min=c(c$,"min",function(a){a=a.iterator();for(var b=a.next();a.hasNext();){var d=a.next();0<b.compareTo(d)&&(b=d)}return b},"java.util.Collection");c$.min=c(c$,"min",function(a,
b){for(var d=a.iterator(),g=d.next();d.hasNext();){var c=d.next();0<b.compare(g,c)&&(g=c)}return g},"java.util.Collection,java.util.Comparator");c$.nCopies=c(c$,"nCopies",function(a,b){return new java.util.Collections.CopiesList(a,b)},"~N,~O");c$.reverse=c(c$,"reverse",function(a){var b=a.size(),d=a.listIterator();a=a.listIterator(b);for(var g=0;g<Math.floor(b/2);g++){var c=d.next(),e=a.previous();d.set(e);a.set(c)}},"java.util.List");c$.reverseOrder=c(c$,"reverseOrder",function(){return new java.util.Collections.ReverseComparator});
c$.reverseOrder=c(c$,"reverseOrder",function(a){return null==a?java.util.Collections.reverseOrder():new java.util.Collections.ReverseComparatorWithComparator(a)},"java.util.Comparator");c$.shuffle=c(c$,"shuffle",function(a){java.util.Collections.shuffle(a,new java.util.Random)},"java.util.List");c$.shuffle=c(c$,"shuffle",function(a,b){if(n(a,java.util.RandomAccess))for(g=a.size()-1;0<g;g--)c=b.nextInt()%(g+1),0>c&&(c=-c),a.set(c,a.set(g,a.get(c)));else{for(var d=a.toArray(),g=d.length-1;0<g;g--){var c=
b.nextInt()%(g+1);0>c&&(c=-c);var e=d[g];d[g]=d[c];d[c]=e}g=0;for(c=a.listIterator();c.hasNext();)c.next(),c.set(d[g++])}},"java.util.List,java.util.Random");c$.singleton=c(c$,"singleton",function(a){return new java.util.Collections.SingletonSet(a)},"~O");c$.singletonList=c(c$,"singletonList",function(a){return new java.util.Collections.SingletonList(a)},"~O");c$.singletonMap=c(c$,"singletonMap",function(a,b){return new java.util.Collections.SingletonMap(a,b)},"~O,~O");c$.sort=c(c$,"sort",function(a){var b=
a.toArray();java.util.Arrays.sort(b);var d=0;for(a=a.listIterator();a.hasNext();)a.next(),a.set(b[d++])},"java.util.List");c$.sort=c(c$,"sort",function(a,b){var d=a.toArray(Array(a.size()));java.util.Arrays.sort(d,b);for(var g=0,c=a.listIterator();c.hasNext();)c.next(),c.set(d[g++])},"java.util.List,java.util.Comparator");c$.swap=c(c$,"swap",function(a,b,d){if(null==a)throw new NullPointerException;b!=d&&a.set(d,a.set(b,a.get(d)))},"java.util.List,~N,~N");c$.replaceAll=c(c$,"replaceAll",function(a,
b,d){for(var g,c=!1;-1<(g=a.indexOf(b));)c=!0,a.set(g,d);return c},"java.util.List,~O,~O");c$.rotate=c(c$,"rotate",function(a,b){var d=a.size();if(0!=d){var g;g=0<b?b%d:d- -1*(b%d);if(!(0==g||g==d))if(n(a,java.util.RandomAccess))for(var c=a.get(0),e=0,h=0,k=0;k<d;k++)e=(e+g)%d,c=a.set(e,c),e==h&&(e=++h,c=a.get(h));else c=(d-g)%d,g=a.subList(0,c),d=a.subList(c,d),java.util.Collections.reverse(g),java.util.Collections.reverse(d),java.util.Collections.reverse(a)}},"java.util.List,~N");c$.indexOfSubList=
c(c$,"indexOfSubList",function(a,b){var d=a.size(),g=b.size();if(g>d)return-1;if(0==g)return 0;var c=b.get(0),e=a.indexOf(c);if(-1==e)return-1;for(;e<d&&d-e>=g;){var h=a.listIterator(e);if(null==c?null==h.next():c.equals(h.next())){for(var k=b.listIterator(1),l=!1;k.hasNext();){var j=k.next();if(!h.hasNext())return-1;if(null==j?null!=h.next():!j.equals(h.next())){l=!0;break}}if(!l)return e}e++}return-1},"java.util.List,java.util.List");c$.lastIndexOfSubList=c(c$,"lastIndexOfSubList",function(a,b){var d=
b.size(),g=a.size();if(d>g)return-1;if(0==d)return g;for(var g=b.get(d-1),c=a.lastIndexOf(g);-1<c&&c+1>=d;){var e=a.listIterator(c+1);if(null==g?null==e.previous():g.equals(e.previous())){for(var h=b.listIterator(d-1),k=!1;h.hasPrevious();){var l=h.previous();if(!e.hasPrevious())return-1;if(null==l?null!=e.previous():!l.equals(e.previous())){k=!0;break}}if(!k)return e.nextIndex()}c--}return-1},"java.util.List,java.util.List");c$.list=c(c$,"list",function(a){for(var b=new java.util.ArrayList;a.hasMoreElements();)b.add(a.nextElement());
return b},"java.util.Enumeration");c$.synchronizedCollection=c(c$,"synchronizedCollection",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.SynchronizedCollection(a)},"java.util.Collection");c$.synchronizedList=c(c$,"synchronizedList",function(a){if(null==a)throw new NullPointerException;return n(a,java.util.RandomAccess)?new java.util.Collections.SynchronizedRandomAccessList(a):new java.util.Collections.SynchronizedList(a)},"java.util.List");c$.synchronizedMap=
c(c$,"synchronizedMap",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.SynchronizedMap(a)},"java.util.Map");c$.synchronizedSet=c(c$,"synchronizedSet",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.SynchronizedSet(a)},"java.util.Set");c$.synchronizedSortedMap=c(c$,"synchronizedSortedMap",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.SynchronizedSortedMap(a)},"java.util.SortedMap");
c$.synchronizedSortedSet=c(c$,"synchronizedSortedSet",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.SynchronizedSortedSet(a)},"java.util.SortedSet");c$.unmodifiableCollection=c(c$,"unmodifiableCollection",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.UnmodifiableCollection(a)},"java.util.Collection");c$.unmodifiableList=c(c$,"unmodifiableList",function(a){if(null==a)throw new NullPointerException;return n(a,java.util.RandomAccess)?
new java.util.Collections.UnmodifiableRandomAccessList(a):new java.util.Collections.UnmodifiableList(a)},"java.util.List");c$.unmodifiableMap=c(c$,"unmodifiableMap",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.UnmodifiableMap(a)},"java.util.Map");c$.unmodifiableSet=c(c$,"unmodifiableSet",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.UnmodifiableSet(a)},"java.util.Set");c$.unmodifiableSortedMap=c(c$,"unmodifiableSortedMap",
function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.UnmodifiableSortedMap(a)},"java.util.SortedMap");c$.unmodifiableSortedSet=c(c$,"unmodifiableSortedSet",function(a){if(null==a)throw new NullPointerException;return new java.util.Collections.UnmodifiableSortedSet(a)},"java.util.SortedSet");c$.frequency=c(c$,"frequency",function(a,b){if(null==a)throw new NullPointerException;if(a.isEmpty())return 0;for(var d=0,g=a.iterator();g.hasNext();){var c=g.next();(null==b?
null==c:b.equals(c))&&d++}return d},"java.util.Collection,~O");c$.emptyList=c(c$,"emptyList",function(){return java.util.Collections.EMPTY_LIST});c$.emptySet=c(c$,"emptySet",function(){return java.util.Collections.EMPTY_SET});c$.emptyMap=c(c$,"emptyMap",function(){return java.util.Collections.EMPTY_MAP});c$.checkedCollection=c(c$,"checkedCollection",function(a,b){return new java.util.Collections.CheckedCollection(a,b)},"java.util.Collection,Class");c$.checkedMap=c(c$,"checkedMap",function(a,b,d){return new java.util.Collections.CheckedMap(a,
b,d)},"java.util.Map,Class,Class");c$.checkedList=c(c$,"checkedList",function(a,b){return n(a,java.util.RandomAccess)?new java.util.Collections.CheckedRandomAccessList(a,b):new java.util.Collections.CheckedList(a,b)},"java.util.List,Class");c$.checkedSet=c(c$,"checkedSet",function(a,b){return new java.util.Collections.CheckedSet(a,b)},"java.util.Set,Class");c$.checkedSortedMap=c(c$,"checkedSortedMap",function(a,b,d){return new java.util.Collections.CheckedSortedMap(a,b,d)},"java.util.SortedMap,Class,Class");
c$.checkedSortedSet=c(c$,"checkedSortedSet",function(a,b){return new java.util.Collections.CheckedSortedSet(a,b)},"java.util.SortedSet,Class");c$.addAll=c(c$,"addAll",function(a,b){for(var d=!1,g=0;g<b.length;g++)d=(new Boolean(d|a.add(b[g]))).valueOf();return d},"java.util.Collection,~A");c$.disjoint=c(c$,"disjoint",function(a,b){if(n(a,java.util.Set)&&!n(b,java.util.Set)||b.size()>a.size()){var d=a;a=b;b=d}for(d=a.iterator();d.hasNext();)if(b.contains(d.next()))return!1;return!0},"java.util.Collection,java.util.Collection");
c$.checkType=c(c$,"checkType",function(a,b){if(!b.isInstance(a))throw new ClassCastException("Attempt to insert "+a.getClass()+" element into collection with element type "+b);return a},"~O,Class");c$.$Collections$1$=function(a){M(self.c$);c$=t(function(){V(this,arguments);this.it=null;p(this,arguments)},java.util,"Collections$1",null,java.util.Enumeration);O(c$,function(){this.it=a.iterator()});c(c$,"hasMoreElements",function(){return this.it.hasNext()});c(c$,"nextElement",function(){return this.it.next()});
c$=L()};M(self.c$);c$=t(function(){this.n=0;this.element=null;p(this,arguments)},java.util.Collections,"CopiesList",java.util.AbstractList,java.io.Serializable);q(c$,function(a,b){I(this,java.util.Collections.CopiesList,[]);if(0>a)throw new IllegalArgumentException;this.n=a;this.element=b},"~N,~O");j(c$,"contains",function(a){return null==this.element?null==a:this.element.equals(a)},"~O");j(c$,"size",function(){return this.n});j(c$,"get",function(a){if(0<=a&&a<this.n)return this.element;throw new IndexOutOfBoundsException;
},"~N");c$=L();M(self.c$);c$=D(java.util.Collections,"EmptyList",java.util.AbstractList,java.io.Serializable);j(c$,"contains",function(){return!1},"~O");j(c$,"size",function(){return 0});j(c$,"get",function(){throw new IndexOutOfBoundsException;},"~N");c$=L();M(self.c$);c$=D(java.util.Collections,"EmptySet",java.util.AbstractSet,java.io.Serializable);j(c$,"contains",function(){return!1},"~O");j(c$,"size",function(){return 0});j(c$,"iterator",function(){return R("java.util.Collections$EmptySet$1")?
0:java.util.Collections.EmptySet.$Collections$EmptySet$1$(),P(java.util.Collections$EmptySet$1,this,null)});c$.$Collections$EmptySet$1$=function(){M(self.c$);c$=da(java.util,"Collections$EmptySet$1",null,java.util.Iterator);j(c$,"hasNext",function(){return!1});j(c$,"next",function(){throw new java.util.NoSuchElementException;});j(c$,"remove",function(){throw new UnsupportedOperationException;});c$=L()};c$=L();M(self.c$);c$=D(java.util.Collections,"EmptyMap",java.util.AbstractMap,java.io.Serializable);
j(c$,"containsKey",function(){return!1},"~O");j(c$,"containsValue",function(){return!1},"~O");j(c$,"entrySet",function(){return java.util.Collections.EMPTY_SET});j(c$,"get",function(){return null},"~O");j(c$,"keySet",function(){return java.util.Collections.EMPTY_SET});j(c$,"values",function(){return java.util.Collections.EMPTY_LIST});c$=L();M(self.c$);c$=D(java.util.Collections,"ReverseComparator",null,[java.util.Comparator,java.io.Serializable]);j(c$,"compare",function(a,b){return b.compareTo(a)},
"~O,~O");c$=L();M(self.c$);c$=t(function(){this.comparator=null;p(this,arguments)},java.util.Collections,"ReverseComparatorWithComparator",null,[java.util.Comparator,java.io.Serializable]);q(c$,function(a){this.comparator=a},"java.util.Comparator");c(c$,"compare",function(a,b){return this.comparator.compare(b,a)},"~O,~O");c$=L();M(self.c$);c$=t(function(){this.element=null;p(this,arguments)},java.util.Collections,"SingletonSet",java.util.AbstractSet,java.io.Serializable);q(c$,function(a){I(this,java.util.Collections.SingletonSet,
[]);this.element=a},"~O");j(c$,"contains",function(a){return null==this.element?null==a:this.element.equals(a)},"~O");j(c$,"size",function(){return 1});j(c$,"iterator",function(){return R("java.util.Collections$SingletonSet$1")?0:java.util.Collections.SingletonSet.$Collections$SingletonSet$1$(),P(java.util.Collections$SingletonSet$1,this,null)});c$.$Collections$SingletonSet$1$=function(){M(self.c$);c$=t(function(){V(this,arguments);this.$hasNext=!0;p(this,arguments)},java.util,"Collections$SingletonSet$1",
null,java.util.Iterator);j(c$,"hasNext",function(){return this.$hasNext});j(c$,"next",function(){if(this.$hasNext)return this.$hasNext=!1,this.b$["java.util.Collections.SingletonSet"].element;throw new java.util.NoSuchElementException;});j(c$,"remove",function(){throw new UnsupportedOperationException;});c$=L()};c$=L();M(self.c$);c$=t(function(){this.element=null;p(this,arguments)},java.util.Collections,"SingletonList",java.util.AbstractList,java.io.Serializable);q(c$,function(a){I(this,java.util.Collections.SingletonList,
[]);this.element=a},"~O");j(c$,"contains",function(a){return null==this.element?null==a:this.element.equals(a)},"~O");j(c$,"get",function(a){if(0==a)return this.element;throw new IndexOutOfBoundsException;},"~N");j(c$,"size",function(){return 1});c$=L();M(self.c$);c$=t(function(){this.v=this.k=null;p(this,arguments)},java.util.Collections,"SingletonMap",java.util.AbstractMap,java.io.Serializable);q(c$,function(a,b){I(this,java.util.Collections.SingletonMap,[]);this.k=a;this.v=b},"~O,~O");j(c$,"containsKey",
function(a){return null==this.k?null==a:this.k.equals(a)},"~O");j(c$,"containsValue",function(a){return null==this.v?null==a:this.v.equals(a)},"~O");j(c$,"get",function(a){return this.containsKey(a)?this.v:null},"~O");j(c$,"size",function(){return 1});j(c$,"entrySet",function(){return R("java.util.Collections$SingletonMap$1")?0:java.util.Collections.SingletonMap.$Collections$SingletonMap$1$(),P(java.util.Collections$SingletonMap$1,this,null)});c$.$Collections$SingletonMap$1$=function(){M(self.c$);
c$=da(java.util,"Collections$SingletonMap$1",java.util.AbstractSet);j(c$,"contains",function(a){return n(a,java.util.Map.Entry)?this.b$["java.util.Collections.SingletonMap"].containsKey(a.getKey())&&this.b$["java.util.Collections.SingletonMap"].containsValue(a.getValue()):!1},"~O");j(c$,"size",function(){return 1});j(c$,"iterator",function(){return R("java.util.Collections$SingletonMap$1$1")?0:java.util.Collections.$Collections$SingletonMap$1$1$(),P(java.util.Collections$SingletonMap$1$1,this,null)});
c$=L()};c$.$Collections$SingletonMap$1$1$=function(){M(self.c$);c$=t(function(){V(this,arguments);this.$hasNext=!0;p(this,arguments)},java.util,"Collections$SingletonMap$1$1",null,java.util.Iterator);j(c$,"hasNext",function(){return this.$hasNext});j(c$,"next",function(){if(this.$hasNext)return this.$hasNext=!1,R("java.util.Collections$SingletonMap$1$1$1")?0:java.util.Collections.$Collections$SingletonMap$1$1$1$(),P(java.util.Collections$SingletonMap$1$1$1,this,null);throw new java.util.NoSuchElementException;
});j(c$,"remove",function(){throw new UnsupportedOperationException;});c$=L()};c$.$Collections$SingletonMap$1$1$1$=function(){M(self.c$);c$=da(java.util,"Collections$SingletonMap$1$1$1",null,java.util.Map.Entry);j(c$,"equals",function(a){return this.b$["java.util.Collections$SingletonMap$1"].contains(a)},"~O");j(c$,"getKey",function(){return this.b$["java.util.Collections.SingletonMap"].k});j(c$,"getValue",function(){return this.b$["java.util.Collections.SingletonMap"].v});j(c$,"hashCode",function(){return(null==
this.b$["java.util.Collections.SingletonMap"].k?0:this.b$["java.util.Collections.SingletonMap"].k.hashCode())^(null==this.b$["java.util.Collections.SingletonMap"].v?0:this.b$["java.util.Collections.SingletonMap"].v.hashCode())});j(c$,"setValue",function(){throw new UnsupportedOperationException;},"~O");c$=L()};c$=L();M(self.c$);c$=t(function(){this.mutex=this.c=null;p(this,arguments)},java.util.Collections,"SynchronizedCollection",null,[java.util.Collection,java.io.Serializable]);q(c$,function(a){this.c=
a;this.mutex=this},"java.util.Collection");q(c$,function(a,b){this.c=a;this.mutex=b},"java.util.Collection,~O");c(c$,"add",function(a){return this.c.add(a)},"~O");c(c$,"addAll",function(a){return this.c.addAll(a)},"java.util.Collection");c(c$,"clear",function(){this.c.clear()});c(c$,"contains",function(a){return this.c.contains(a)},"~O");c(c$,"containsAll",function(a){return this.c.containsAll(a)},"java.util.Collection");c(c$,"isEmpty",function(){return this.c.isEmpty()});c(c$,"iterator",function(){return this.c.iterator()});
c(c$,"remove",function(a){return this.c.remove(a)},"~O");c(c$,"removeAll",function(a){return this.c.removeAll(a)},"java.util.Collection");c(c$,"retainAll",function(a){return this.c.retainAll(a)},"java.util.Collection");c(c$,"size",function(){return this.c.size()});c(c$,"toArray",function(){return this.c.toArray()});c(c$,"toString",function(){return this.c.toString()});c(c$,"toArray",function(a){return this.c.toArray(a)},"~A");c$=L();M(self.c$);c$=t(function(){this.list=null;p(this,arguments)},java.util.Collections,
"SynchronizedList",java.util.Collections.SynchronizedCollection,java.util.List);q(c$,function(a){I(this,java.util.Collections.SynchronizedList,[a]);this.list=a},"java.util.List");q(c$,function(a,b){I(this,java.util.Collections.SynchronizedList,[a,b]);this.list=a},"java.util.List,~O");c(c$,"add",function(a,b){this.list.add(a,b)},"~N,~O");c(c$,"addAll",function(a,b){return this.list.addAll(a,b)},"~N,java.util.Collection");j(c$,"equals",function(a){return this.list.equals(a)},"~O");c(c$,"get",function(a){return this.list.get(a)},
"~N");j(c$,"hashCode",function(){return this.list.hashCode()});c(c$,"indexOf",function(a){return this.list.indexOf(a)},"~O");c(c$,"lastIndexOf",function(a){return this.list.lastIndexOf(a)},"~O");c(c$,"listIterator",function(a){a||(a=0);return this.list.listIterator(a)},"~N");c(c$,"remove",function(a){return this.list.remove(a)},"~N");c(c$,"set",function(a,b){return this.list.set(a,b)},"~N,~O");c(c$,"subList",function(a,b){return new java.util.Collections.SynchronizedList(this.list.subList(a,b),this.mutex)},
"~N,~N");c$=L();M(self.c$);c$=D(java.util.Collections,"SynchronizedRandomAccessList",java.util.Collections.SynchronizedList,java.util.RandomAccess);j(c$,"subList",function(a,b){return new java.util.Collections.SynchronizedRandomAccessList(this.list.subList(a,b),this.mutex)},"~N,~N");c$=L();M(self.c$);c$=t(function(){this.mutex=this.m=null;p(this,arguments)},java.util.Collections,"SynchronizedMap",null,[java.util.Map,java.io.Serializable]);q(c$,function(a){this.m=a;this.mutex=this},"java.util.Map");
q(c$,function(a,b){this.m=a;this.mutex=b},"java.util.Map,~O");c(c$,"clear",function(){this.m.clear()});c(c$,"containsKey",function(a){return this.m.containsKey(a)},"~O");c(c$,"containsValue",function(a){return this.m.containsValue(a)},"~O");c(c$,"entrySet",function(){return new java.util.Collections.SynchronizedSet(this.m.entrySet(),this.mutex)});j(c$,"equals",function(a){return this.m.equals(a)},"~O");c(c$,"get",function(a){return this.m.get(a)},"~O");j(c$,"hashCode",function(){return this.m.hashCode()});
c(c$,"isEmpty",function(){return this.m.isEmpty()});c(c$,"keySet",function(){return new java.util.Collections.SynchronizedSet(this.m.keySet(),this.mutex)});c(c$,"put",function(a,b){return this.m.put(a,b)},"~O,~O");c(c$,"putAll",function(a){this.m.putAll(a)},"java.util.Map");c(c$,"remove",function(a){return this.m.remove(a)},"~O");c(c$,"size",function(){return this.m.size()});c(c$,"values",function(){return new java.util.Collections.SynchronizedCollection(this.m.values(),this.mutex)});c(c$,"toString",
function(){return this.m.toString()});c$=L();M(self.c$);c$=D(java.util.Collections,"SynchronizedSet",java.util.Collections.SynchronizedCollection,java.util.Set);j(c$,"equals",function(a){return this.c.equals(a)},"~O");j(c$,"hashCode",function(){return this.c.hashCode()});c$=L();M(self.c$);c$=t(function(){this.sm=null;p(this,arguments)},java.util.Collections,"SynchronizedSortedMap",java.util.Collections.SynchronizedMap,java.util.SortedMap);q(c$,function(a){I(this,java.util.Collections.SynchronizedSortedMap,
[a]);this.sm=a},"java.util.SortedMap");q(c$,function(a,b){I(this,java.util.Collections.SynchronizedSortedMap,[a,b]);this.sm=a},"java.util.SortedMap,~O");c(c$,"comparator",function(){return this.sm.comparator()});c(c$,"firstKey",function(){return this.sm.firstKey()});c(c$,"headMap",function(a){return new java.util.Collections.SynchronizedSortedMap(this.sm.headMap(a),this.mutex)},"~O");c(c$,"lastKey",function(){return this.sm.lastKey()});c(c$,"subMap",function(a,b){return new java.util.Collections.SynchronizedSortedMap(this.sm.subMap(a,
b),this.mutex)},"~O,~O");c(c$,"tailMap",function(a){return new java.util.Collections.SynchronizedSortedMap(this.sm.tailMap(a),this.mutex)},"~O");c$=L();M(self.c$);c$=t(function(){this.ss=null;p(this,arguments)},java.util.Collections,"SynchronizedSortedSet",java.util.Collections.SynchronizedSet,java.util.SortedSet);q(c$,function(a){I(this,java.util.Collections.SynchronizedSortedSet,[a]);this.ss=a},"java.util.SortedSet");q(c$,function(a,b){I(this,java.util.Collections.SynchronizedSortedSet,[a,b]);this.ss=
a},"java.util.SortedSet,~O");c(c$,"comparator",function(){return this.ss.comparator()});c(c$,"first",function(){return this.ss.first()});c(c$,"headSet",function(a){return new java.util.Collections.SynchronizedSortedSet(this.ss.headSet(a),this.mutex)},"~O");c(c$,"last",function(){return this.ss.last()});c(c$,"subSet",function(a,b){return new java.util.Collections.SynchronizedSortedSet(this.ss.subSet(a,b),this.mutex)},"~O,~O");c(c$,"tailSet",function(a){return new java.util.Collections.SynchronizedSortedSet(this.ss.tailSet(a),
this.mutex)},"~O");c$=L();M(self.c$);c$=t(function(){this.c=null;p(this,arguments)},java.util.Collections,"UnmodifiableCollection",null,[java.util.Collection,java.io.Serializable]);q(c$,function(a){this.c=a},"java.util.Collection");j(c$,"add",function(){throw new UnsupportedOperationException;},"~O");j(c$,"addAll",function(){throw new UnsupportedOperationException;},"java.util.Collection");j(c$,"clear",function(){throw new UnsupportedOperationException;});c(c$,"contains",function(a){return this.c.contains(a)},
"~O");c(c$,"containsAll",function(a){return this.c.containsAll(a)},"java.util.Collection");c(c$,"isEmpty",function(){return this.c.isEmpty()});c(c$,"iterator",function(){return R("java.util.Collections$UnmodifiableCollection$1")?0:java.util.Collections.UnmodifiableCollection.$Collections$UnmodifiableCollection$1$(),P(java.util.Collections$UnmodifiableCollection$1,this,null)});j(c$,"remove",function(){throw new UnsupportedOperationException;},"~O");j(c$,"removeAll",function(){throw new UnsupportedOperationException;
},"java.util.Collection");j(c$,"retainAll",function(){throw new UnsupportedOperationException;},"java.util.Collection");c(c$,"size",function(){return this.c.size()});c(c$,"toArray",function(){return this.c.toArray()});c(c$,"toArray",function(a){return this.c.toArray(a)},"~A");c(c$,"toString",function(){return this.c.toString()});c$.$Collections$UnmodifiableCollection$1$=function(){M(self.c$);c$=t(function(){V(this,arguments);this.iterator=null;p(this,arguments)},java.util,"Collections$UnmodifiableCollection$1",
null,java.util.Iterator);O(c$,function(){this.iterator=this.b$["java.util.Collections.UnmodifiableCollection"].c.iterator()});c(c$,"hasNext",function(){return this.iterator.hasNext()});c(c$,"next",function(){return this.iterator.next()});j(c$,"remove",function(){throw new UnsupportedOperationException;});c$=L()};c$=L();M(self.c$);c$=t(function(){this.list=null;p(this,arguments)},java.util.Collections,"UnmodifiableList",java.util.Collections.UnmodifiableCollection,java.util.List);q(c$,function(a){I(this,
java.util.Collections.UnmodifiableList,[a]);this.list=a},"java.util.List");c(c$,"add",function(){throw new UnsupportedOperationException;},"~N,~O");c(c$,"addAll",function(){throw new UnsupportedOperationException;},"~N,java.util.Collection");j(c$,"equals",function(a){return this.list.equals(a)},"~O");c(c$,"get",function(a){return this.list.get(a)},"~N");j(c$,"hashcode",function(){return this.list.hashCode()});c(c$,"indexOf",function(a){return this.list.indexOf(a)},"~O");c(c$,"lastIndexOf",function(a){return this.list.lastIndexOf(a)},
"~O");c(c$,"listIterator",function(){return R("java.util.Collections$UnmodifiableList$1")?0:java.util.Collections.UnmodifiableList.$Collections$UnmodifiableList$1$(),P(java.util.Collections$UnmodifiableList$1,this,null)},"~N");c(c$,"remove",function(){throw new UnsupportedOperationException;},"~N");j(c$,"set",function(){throw new UnsupportedOperationException;},"~N,~O");c(c$,"subList",function(a,b){return new java.util.Collections.UnmodifiableList(this.list.subList(a,b))},"~N,~N");c$.$Collections$UnmodifiableList$1$=
function(){M(self.c$);c$=t(function(){V(this,arguments);this.iterator=null;p(this,arguments)},java.util,"Collections$UnmodifiableList$1",null,java.util.ListIterator);O(c$,function(){this.iterator=this.b$["java.util.Collections.UnmodifiableList"].list.listIterator(location)});j(c$,"add",function(){throw new UnsupportedOperationException;},"~O");c(c$,"hasNext",function(){return this.iterator.hasNext()});c(c$,"hasPrevious",function(){return this.iterator.hasPrevious()});c(c$,"next",function(){return this.iterator.next()});
c(c$,"nextIndex",function(){return this.iterator.nextIndex()});c(c$,"previous",function(){return this.iterator.previous()});c(c$,"previousIndex",function(){return this.iterator.previousIndex()});j(c$,"remove",function(){throw new UnsupportedOperationException;});j(c$,"set",function(){throw new UnsupportedOperationException;},"~O");c$=L()};c$=L();M(self.c$);c$=D(java.util.Collections,"UnmodifiableRandomAccessList",java.util.Collections.UnmodifiableList,java.util.RandomAccess);j(c$,"subList",function(a,
b){return new java.util.Collections.UnmodifiableRandomAccessList(this.list.subList(a,b))},"~N,~N");c$=L();M(self.c$);c$=D(java.util.Collections,"UnmodifiableSet",java.util.Collections.UnmodifiableCollection,java.util.Set);j(c$,"equals",function(a){return this.c.equals(a)},"~O");j(c$,"hashCode",function(){return this.c.hashCode()});c$=L();M(self.c$);c$=t(function(){this.m=null;p(this,arguments)},java.util.Collections,"UnmodifiableMap",null,[java.util.Map,java.io.Serializable]);q(c$,function(a){this.m=
a},"java.util.Map");j(c$,"clear",function(){throw new UnsupportedOperationException;});c(c$,"containsKey",function(a){return this.m.containsKey(a)},"~O");c(c$,"containsValue",function(a){return this.m.containsValue(a)},"~O");c(c$,"entrySet",function(){return new java.util.Collections.UnmodifiableMap.UnmodifiableEntrySet(this.m.entrySet())});j(c$,"equals",function(a){return this.m.equals(a)},"~O");c(c$,"get",function(a){return this.m.get(a)},"~O");j(c$,"hashcode",function(){return this.m.hashCode()});
c(c$,"isEmpty",function(){return this.m.isEmpty()});c(c$,"keySet",function(){return new java.util.Collections.UnmodifiableSet(this.m.keySet())});j(c$,"put",function(){throw new UnsupportedOperationException;},"~O,~O");j(c$,"putAll",function(){throw new UnsupportedOperationException;},"java.util.Map");j(c$,"remove",function(){throw new UnsupportedOperationException;},"~O");c(c$,"size",function(){return this.m.size()});c(c$,"values",function(){return new java.util.Collections.UnmodifiableCollection(this.m.values())});
c(c$,"toString",function(){return this.m.toString()});M(self.c$);c$=D(java.util.Collections.UnmodifiableMap,"UnmodifiableEntrySet",java.util.Collections.UnmodifiableSet);j(c$,"iterator",function(){return R("java.util.Collections$UnmodifiableMap$UnmodifiableEntrySet$1")?0:java.util.Collections.UnmodifiableMap.UnmodifiableEntrySet.$Collections$UnmodifiableMap$UnmodifiableEntrySet$1$(),P(java.util.Collections$UnmodifiableMap$UnmodifiableEntrySet$1,this,null)});c(c$,"toArray",function(){for(var a=this.c.size(),
b=Array(a),d=this.iterator();0<=--a;)b[a]=d.next();return b});c(c$,"toArray",function(a){var b=this.c.size(),d=0,g=this.iterator();b>a.length&&(a=a.getClass().getComponentType(),a=java.lang.reflect.Array.newInstance(a,b));for(;d<b;)a[d++]=g.next();d<a.length&&(a[d]=null);return a},"~A");c$.$Collections$UnmodifiableMap$UnmodifiableEntrySet$1$=function(){M(self.c$);c$=t(function(){V(this,arguments);this.iterator=null;p(this,arguments)},java.util,"Collections$UnmodifiableMap$UnmodifiableEntrySet$1",
null,java.util.Iterator);O(c$,function(){this.iterator=this.b$["java.util.Collections.UnmodifiableMap.UnmodifiableEntrySet"].c.iterator()});c(c$,"hasNext",function(){return this.iterator.hasNext()});c(c$,"next",function(){return new java.util.Collections.UnmodifiableMap.UnmodifiableEntrySet.UnmodifiableMapEntry(this.iterator.next())});j(c$,"remove",function(){throw new UnsupportedOperationException;});c$=L()};M(self.c$);c$=t(function(){this.mapEntry=null;p(this,arguments)},java.util.Collections.UnmodifiableMap.UnmodifiableEntrySet,
"UnmodifiableMapEntry",null,java.util.Map.Entry);q(c$,function(a){this.mapEntry=a},"java.util.Map.Entry");j(c$,"equals",function(a){return this.mapEntry.equals(a)},"~O");c(c$,"getKey",function(){return this.mapEntry.getKey()});c(c$,"getValue",function(){return this.mapEntry.getValue()});j(c$,"hashcode",function(){return this.mapEntry.hashCode()});j(c$,"setValue",function(){throw new UnsupportedOperationException;},"~O");c(c$,"toString",function(){return this.mapEntry.toString()});c$=L();c$=L();c$=
L();M(self.c$);c$=t(function(){this.sm=null;p(this,arguments)},java.util.Collections,"UnmodifiableSortedMap",java.util.Collections.UnmodifiableMap,java.util.SortedMap);q(c$,function(a){I(this,java.util.Collections.UnmodifiableSortedMap,[a]);this.sm=a},"java.util.SortedMap");c(c$,"comparator",function(){return this.sm.comparator()});c(c$,"firstKey",function(){return this.sm.firstKey()});c(c$,"headMap",function(a){return new java.util.Collections.UnmodifiableSortedMap(this.sm.headMap(a))},"~O");c(c$,
"lastKey",function(){return this.sm.lastKey()});c(c$,"subMap",function(a,b){return new java.util.Collections.UnmodifiableSortedMap(this.sm.subMap(a,b))},"~O,~O");c(c$,"tailMap",function(a){return new java.util.Collections.UnmodifiableSortedMap(this.sm.tailMap(a))},"~O");c$=L();M(self.c$);c$=t(function(){this.ss=null;p(this,arguments)},java.util.Collections,"UnmodifiableSortedSet",java.util.Collections.UnmodifiableSet,java.util.SortedSet);q(c$,function(a){I(this,java.util.Collections.UnmodifiableSortedSet,
[a]);this.ss=a},"java.util.SortedSet");c(c$,"comparator",function(){return this.ss.comparator()});c(c$,"first",function(){return this.ss.first()});c(c$,"headSet",function(a){return new java.util.Collections.UnmodifiableSortedSet(this.ss.headSet(a))},"~O");c(c$,"last",function(){return this.ss.last()});c(c$,"subSet",function(a,b){return new java.util.Collections.UnmodifiableSortedSet(this.ss.subSet(a,b))},"~O,~O");c(c$,"tailSet",function(a){return new java.util.Collections.UnmodifiableSortedSet(this.ss.tailSet(a))},
"~O");c$=L();M(self.c$);c$=t(function(){this.type=this.c=null;p(this,arguments)},java.util.Collections,"CheckedCollection",null,[java.util.Collection,java.io.Serializable]);q(c$,function(a,b){if(null==a||null==b)throw new NullPointerException;this.c=a;this.type=b},"java.util.Collection,Class");c(c$,"size",function(){return this.c.size()});c(c$,"isEmpty",function(){return this.c.isEmpty()});c(c$,"contains",function(a){return this.c.contains(a)},"~O");c(c$,"iterator",function(){var a=this.c.iterator();
n(a,java.util.ListIterator)&&(a=new java.util.Collections.CheckedListIterator(a,this.type));return a});c(c$,"toArray",function(){return this.c.toArray()});c(c$,"toArray",function(a){return this.c.toArray(a)},"~A");c(c$,"add",function(a){return this.c.add(java.util.Collections.checkType(a,this.type))},"~O");c(c$,"remove",function(a){return this.c.remove(a)},"~O");c(c$,"containsAll",function(a){return this.c.containsAll(a)},"java.util.Collection");j(c$,"addAll",function(a){var b=a.size();if(0==b)return!1;
var d=Array(b);a=a.iterator();for(var g=0;g<b;g++)d[g]=java.util.Collections.checkType(a.next(),this.type);a=!1;for(g=0;g<b;g++)a=(new Boolean(a|this.c.add(d[g]))).valueOf();return a},"java.util.Collection");c(c$,"removeAll",function(a){return this.c.removeAll(a)},"java.util.Collection");c(c$,"retainAll",function(a){return this.c.retainAll(a)},"java.util.Collection");c(c$,"clear",function(){this.c.clear()});c(c$,"toString",function(){return this.c.toString()});c$=L();M(self.c$);c$=t(function(){this.type=
this.i=null;p(this,arguments)},java.util.Collections,"CheckedListIterator",null,java.util.ListIterator);q(c$,function(a,b){this.i=a;this.type=b},"java.util.ListIterator,Class");c(c$,"hasNext",function(){return this.i.hasNext()});c(c$,"next",function(){return this.i.next()});c(c$,"remove",function(){this.i.remove()});c(c$,"hasPrevious",function(){return this.i.hasPrevious()});c(c$,"previous",function(){return this.i.previous()});c(c$,"nextIndex",function(){return this.i.nextIndex()});c(c$,"previousIndex",
function(){return this.i.previousIndex()});c(c$,"set",function(a){this.i.set(java.util.Collections.checkType(a,this.type))},"~O");c(c$,"add",function(a){this.i.add(java.util.Collections.checkType(a,this.type))},"~O");c$=L();M(self.c$);c$=t(function(){this.l=null;p(this,arguments)},java.util.Collections,"CheckedList",java.util.Collections.CheckedCollection,java.util.List);q(c$,function(a,b){I(this,java.util.Collections.CheckedList,[a,b]);this.l=a},"java.util.List,Class");c(c$,"addAll",function(a,b){var d=
b.size();if(0==d)return!1;for(var g=Array(d),c=b.iterator(),e=0;e<d;e++)g[e]=java.util.Collections.checkType(c.next(),this.type);return this.l.addAll(a,java.util.Arrays.asList(g))},"~N,java.util.Collection");c(c$,"get",function(a){return this.l.get(a)},"~N");c(c$,"set",function(a,b){return this.l.set(a,java.util.Collections.checkType(b,this.type))},"~N,~O");c(c$,"add",function(a,b){this.l.add(a,java.util.Collections.checkType(b,this.type))},"~N,~O");c(c$,"remove",function(a){return this.l.remove(a)},
"~N");c(c$,"indexOf",function(a){return this.l.indexOf(a)},"~O");c(c$,"lastIndexOf",function(a){return this.l.lastIndexOf(a)},"~O");c(c$,"listIterator",function(a){a||(a=0);return new java.util.Collections.CheckedListIterator(this.l.listIterator(a),this.type)},"~N");c(c$,"subList",function(a,b){return java.util.Collections.checkedList(this.l.subList(a,b),this.type)},"~N,~N");j(c$,"equals",function(a){return this.l.equals(a)},"~O");j(c$,"hashcode",function(){return this.l.hashCode()});c$=L();M(self.c$);
c$=D(java.util.Collections,"CheckedRandomAccessList",java.util.Collections.CheckedList,java.util.RandomAccess);c$=L();M(self.c$);c$=D(java.util.Collections,"CheckedSet",java.util.Collections.CheckedCollection,java.util.Set);j(c$,"equals",function(a){return this.c.equals(a)},"~O");j(c$,"hashCode",function(){return this.c.hashCode()});c$=L();M(self.c$);c$=t(function(){this.valueType=this.keyType=this.m=null;p(this,arguments)},java.util.Collections,"CheckedMap",null,[java.util.Map,java.io.Serializable]);
q(c$,($fz=function(a,b,d){if(null==a||null==b||null==d)throw new NullPointerException;this.m=a;this.keyType=b;this.valueType=d},$fz.isPrivate=!0,$fz),"java.util.Map,Class,Class");c(c$,"size",function(){return this.m.size()});c(c$,"isEmpty",function(){return this.m.isEmpty()});c(c$,"containsKey",function(a){return this.m.containsKey(a)},"~O");c(c$,"containsValue",function(a){return this.m.containsValue(a)},"~O");c(c$,"get",function(a){return this.m.get(a)},"~O");c(c$,"put",function(a,b){return this.m.put(java.util.Collections.checkType(a,
this.keyType),java.util.Collections.checkType(b,this.valueType))},"~O,~O");c(c$,"remove",function(a){return this.m.remove(a)},"~O");j(c$,"putAll",function(a){var b=a.size();if(0!=b){var d=Array(b);a=a.entrySet().iterator();for(var g=0;g<b;g++){var c=a.next();java.util.Collections.checkType(c.getKey(),this.keyType);java.util.Collections.checkType(c.getValue(),this.valueType);d[g]=c}for(c=0;c<b;c++)this.m.put(d[c].getKey(),d[c].getValue())}},"java.util.Map");c(c$,"clear",function(){this.m.clear()});
c(c$,"keySet",function(){return this.m.keySet()});c(c$,"values",function(){return this.m.values()});c(c$,"entrySet",function(){return new java.util.Collections.CheckedMap.CheckedEntrySet(this.m.entrySet(),this.valueType)});j(c$,"equals",function(a){return this.m.equals(a)},"~O");j(c$,"hashcode",function(){return this.m.hashCode()});c(c$,"toString",function(){return this.m.toString()});M(self.c$);c$=t(function(){this.valueType=this.e=null;p(this,arguments)},java.util.Collections.CheckedMap,"CheckedEntry",
null,java.util.Map.Entry);q(c$,function(a,b){if(null==a)throw new NullPointerException;this.e=a;this.valueType=b},"java.util.Map.Entry,Class");c(c$,"getKey",function(){return this.e.getKey()});c(c$,"getValue",function(){return this.e.getValue()});c(c$,"setValue",function(a){return this.e.setValue(java.util.Collections.checkType(a,this.valueType))},"~O");j(c$,"equals",function(a){return this.e.equals(a)},"~O");j(c$,"hashcode",function(){return this.e.hashCode()});c$=L();M(self.c$);c$=t(function(){this.valueType=
this.s=null;p(this,arguments)},java.util.Collections.CheckedMap,"CheckedEntrySet",null,java.util.Set);q(c$,function(a,b){this.s=a;this.valueType=b},"java.util.Set,Class");c(c$,"iterator",function(){return new java.util.Collections.CheckedMap.CheckedEntrySet.CheckedEntryIterator(this.s.iterator(),this.valueType)});c(c$,"toArray",function(){for(var a=this.size(),b=Array(a),d=this.iterator(),g=0;g<a;g++)b[g]=d.next();return b});c(c$,"toArray",function(a){var b=this.size();if(a.length<b){var d=a.getClass().getComponentType();
a=java.lang.reflect.Array.newInstance(d,b)}for(var d=this.iterator(),g=0;g<b;g++)a[g]=d.next();b<a.length&&(a[b]=null);return a},"~A");c(c$,"retainAll",function(a){return this.s.retainAll(a)},"java.util.Collection");c(c$,"removeAll",function(a){return this.s.removeAll(a)},"java.util.Collection");c(c$,"containsAll",function(a){return this.s.containsAll(a)},"java.util.Collection");j(c$,"addAll",function(){throw new UnsupportedOperationException;},"java.util.Collection");c(c$,"remove",function(a){return this.s.remove(a)},
"~O");c(c$,"contains",function(a){return this.s.contains(a)},"~O");j(c$,"add",function(){throw new UnsupportedOperationException;},"java.util.Map.Entry");c(c$,"isEmpty",function(){return this.s.isEmpty()});c(c$,"clear",function(){this.s.clear()});c(c$,"size",function(){return this.s.size()});j(c$,"hashcode",function(){return this.s.hashCode()});j(c$,"equals",function(a){return this.s.equals(a)},"~O");M(self.c$);c$=t(function(){this.valueType=this.i=null;p(this,arguments)},java.util.Collections.CheckedMap.CheckedEntrySet,
"CheckedEntryIterator",null,java.util.Iterator);q(c$,function(a,b){this.i=a;this.valueType=b},"java.util.Iterator,Class");c(c$,"hasNext",function(){return this.i.hasNext()});c(c$,"remove",function(){this.i.remove()});c(c$,"next",function(){return new java.util.Collections.CheckedMap.CheckedEntry(this.i.next(),this.valueType)});c$=L();c$=L();c$=L();M(self.c$);c$=t(function(){this.ss=null;p(this,arguments)},java.util.Collections,"CheckedSortedSet",java.util.Collections.CheckedSet,java.util.SortedSet);
q(c$,function(a,b){I(this,java.util.Collections.CheckedSortedSet,[a,b]);this.ss=a},"java.util.SortedSet,Class");c(c$,"comparator",function(){return this.ss.comparator()});c(c$,"subSet",function(a,b){return new java.util.Collections.CheckedSortedSet(this.ss.subSet(a,b),this.type)},"~O,~O");c(c$,"headSet",function(a){return new java.util.Collections.CheckedSortedSet(this.ss.headSet(a),this.type)},"~O");c(c$,"tailSet",function(a){return new java.util.Collections.CheckedSortedSet(this.ss.tailSet(a),this.type)},
"~O");c(c$,"first",function(){return this.ss.first()});c(c$,"last",function(){return this.ss.last()});c$=L();M(self.c$);c$=t(function(){this.sm=null;p(this,arguments)},java.util.Collections,"CheckedSortedMap",java.util.Collections.CheckedMap,java.util.SortedMap);q(c$,function(a,b,d){I(this,java.util.Collections.CheckedSortedMap,[a,b,d]);this.sm=a},"java.util.SortedMap,Class,Class");c(c$,"comparator",function(){return this.sm.comparator()});c(c$,"subMap",function(a,b){return new java.util.Collections.CheckedSortedMap(this.sm.subMap(a,
b),this.keyType,this.valueType)},"~O,~O");c(c$,"headMap",function(a){return new java.util.Collections.CheckedSortedMap(this.sm.headMap(a),this.keyType,this.valueType)},"~O");c(c$,"tailMap",function(a){return new java.util.Collections.CheckedSortedMap(this.sm.tailMap(a),this.keyType,this.valueType)},"~O");c(c$,"firstKey",function(){return this.sm.firstKey()});c(c$,"lastKey",function(){return this.sm.lastKey()});c$=L();c$.EMPTY_LIST=c$.prototype.EMPTY_LIST=new java.util.Collections.EmptyList;c$.EMPTY_SET=
c$.prototype.EMPTY_SET=new java.util.Collections.EmptySet;c$.EMPTY_MAP=c$.prototype.EMPTY_MAP=new java.util.Collections.EmptyMap});c$=D(java.util,"Dictionary");q(c$,function(){});v([],"java.util.HashtableIterator",[],function(){c$=t(function(){this.expectedModCount=this.position=0;this.lastEntry=this.type=null;this.lastPosition=0;this.canRemove=!1;p(this,arguments)},java.util,"HashtableIterator",null,java.util.Iterator);q(c$,function(a){this.type=a;this.h$=a.h$;this.position=this.h$.lastSlot;this.expectedModCount=
this.h$.modCount},"java.util.AbstractSet");j(c$,"hasNext",function(){if(this.lastEntry&&this.lastEntry.next)return!0;for(;this.position>=this.h$.firstSlot;)if(null==this.h$.elementData[this.position])this.position--;else return!0;return!1});j(c$,"next",function(){if(this.expectedModCount==this.h$.modCount){this.lastEntry&&(this.lastEntry=this.lastEntry.next);if(null==this.lastEntry){for(;this.position>=this.h$.firstSlot&&null==(this.lastEntry=this.h$.elementData[this.position]);)this.position--;this.lastEntry&&
(this.lastPosition=this.position,this.position--)}if(this.lastEntry)return this.canRemove=!0,this.type.get(this.lastEntry);throw new java.util.NoSuchElementException;}throw new java.util.ConcurrentModificationException;});j(c$,"remove",function(){if(this.expectedModCount==this.h$.modCount)if(this.canRemove){var a=this.canRemove=!1,b=this.h$.elementData[this.lastPosition];if(b===this.lastEntry)this.h$.elementData[this.lastPosition]=b.next,a=!0;else{for(;b&&b.next!==this.lastEntry;)b=b.next;b&&(b.next=
this.lastEntry.next,a=!0)}if(a){this.h$.modCount++;this.h$.elementCount--;this.expectedModCount++;return}}else throw new IllegalStateException;throw new java.util.ConcurrentModificationException;})});v([],"java.util.HashtableEnumerator",[],function(){c$=t(function(){this.key=!1;this.start=0;this.entry=null;p(this,arguments)},java.util,"HashtableEnumerator",null,java.util.Enumeration);q(c$,function(a,b){this.key=a;if(this.h$=b)this.start=this.h$.lastSlot+1},"~B,java.util.Hashtable");j(c$,"hasMoreElements",
function(){if(!this.h$)return!1;if(this.entry)return!0;for(;--this.start>=this.h$.firstSlot;)if(this.h$.elementData[this.start])return this.entry=this.h$.elementData[this.start],!0;return!1});j(c$,"nextElement",function(){if(this.hasMoreElements()){var a=this.key?this.entry.key:this.entry.value;this.entry=this.entry.next;return a}throw new java.util.NoSuchElementException;})});v(["java.util.AbstractSet"],"java.util.HashtableEntrySet",[],function(){c$=t(function(){p(this,arguments)},java.util,"HashtableEntrySet",
java.util.AbstractSet,null);q(c$,function(a){this.h$=a},"java.util.Hashtable");j(c$,"size",function(){return this.h$.elementCount});j(c$,"clear",function(){this.h$.clear()});j(c$,"remove",function(a){return this.contains(a)?(this.h$.remove(a.getKey()),!0):!1},"~O");c(c$,"contains",function(a){var b=this.h$.getEntry(a.getKey());return a.equals(b)},"~O");j(c$,"get",function(a){return a},"java.util.MapEntry");c(c$,"iterator",function(){return new java.util.HashtableIterator(this)})});v(["java.util.AbstractSet"],
"java.util.HashtableKeySet",[],function(){c$=t(function(){p(this,arguments)},java.util,"HashtableKeySet",java.util.AbstractSet,null);q(c$,function(a){this.h$=a},"java.util.Hashtable");j(c$,"contains",function(a){return this.h$.containsKey(a)},"~O");j(c$,"size",function(){return this.h$.elementCount});j(c$,"clear",function(){this.h$.clear()});j(c$,"remove",function(a){return this.h$.containsKey(a)?(this.h$.remove(a),!0):!1},"~O");j(c$,"get",function(a){return a.key},"java.util.MapEntry");j(c$,"iterator",
function(){return new java.util.HashtableIterator(this)})});v(["java.util.AbstractCollection"],"java.util.HashtableValueCollection",[],function(){c$=t(function(){p(this,arguments)},java.util,"HashtableValueCollection",java.util.AbstractCollection,null);q(c$,function(a){this.h$=a},"java.util.Hashtable");j(c$,"contains",function(a){return this.h$.contains(a)},"~O");j(c$,"size",function(){return this.h$.elementCount});j(c$,"clear",function(){this.h$.clear()});j(c$,"get",function(a){return a.value},"java.util.MapEntry");
j(c$,"iterator",function(){return new java.util.HashtableIterator(this)})});v(["java.util.MapEntry"],"java.util.HashtableEntry",[],function(){c$=t(function(){this.next=null;this.hashcode=0;p(this,arguments)},java.util,"HashtableEntry",java.util.MapEntry);Y(c$,function(a,b){this.key=a;this.value=b;this.hashcode=a.hashCode()});c(c$,"clone",function(){var a=W(this,java.util.HashtableEntry,"clone",[]);null!=this.next&&(a.next=this.next.clone());return a});j(c$,"setValue",function(a){if(null==a)throw new NullPointerException;
var b=this.value;this.value=a;return b},"~O");c(c$,"getKeyHash",function(){return this.key.hashCode()});c(c$,"equalsKey",function(a){return this.hashcode==(!a.hashCode||a.hashCode())&&this.key.equals(a)},"~O,~N");j(c$,"toString",function(){return this.key+"="+this.value})});v("java.util.Dictionary $.Enumeration $.HashtableEnumerator $.Iterator $.Map $.MapEntry $.NoSuchElementException".split(" "),"java.util.Hashtable","java.lang.IllegalArgumentException $.IllegalStateException $.NullPointerException $.StringBuilder java.util.AbstractCollection $.AbstractSet $.Arrays $.Collections $.ConcurrentModificationException java.util.MapEntry.Type java.util.HashtableEntry".split(" "),
function(){c$=t(function(){this.elementCount=0;this.elementData=null;this.firstSlot=this.threshold=this.loadFactor=0;this.lastSlot=-1;this.modCount=0;p(this,arguments)},java.util,"Hashtable",java.util.Dictionary,[java.util.Map,Cloneable,java.io.Serializable]);c$.newEntry=c(c$,"newEntry",($fz=function(a,b){return new java.util.HashtableEntry(a,b)},$fz.isPrivate=!0,$fz),"~O,~O,~N");Y(c$,function(){this.elementCount=0;this.elementData=this.newElementArray(11);this.firstSlot=this.elementData.length;this.loadFactor=
0.75;this.computeMaxSize()});c(c$,"newElementArray",($fz=function(a){return Array(a)},$fz.isPrivate=!0,$fz),"~N");j(c$,"clear",function(){this.elementCount=0;for(var a=this.elementData.length;0<=--a;)this.elementData[a]=null;this.modCount++});c(c$,"clone",function(){try{var a=W(this,java.util.Hashtable,"clone",[]);a.elementData=Array(this.elementData.length);for(var b=this.elementData.length;0<=--b;)null!=this.elementData[b]&&(a.elementData[b]=this.elementData[b].clone());return a}catch(d){if(n(d,
CloneNotSupportedException))return null;throw d;}});c(c$,"computeMaxSize",($fz=function(){this.threshold=Math.round(this.elementData.length*this.loadFactor)},$fz.isPrivate=!0,$fz));c(c$,"contains",function(a){if(null==a)throw new NullPointerException;for(var b=this.elementData.length;0<=--b;)for(var d=this.elementData[b];d;){if(a.equals(d.value))return!0;d=d.next}return!1},"~O");j(c$,"containsKey",function(a){a.hashCode||(a.hashCode=function(){return 1},a.equals||(a.equals=function(a){return this==
a}));return null!=this.getEntry(a)},"~O");j(c$,"containsValue",function(a){return this.contains(a)},"~O");j(c$,"elements",function(){return 0==this.elementCount?java.util.Hashtable.EMPTY_ENUMERATION:new java.util.HashtableEnumerator(!1,this)});j(c$,"entrySet",function(){return new java.util.HashtableEntrySet(this)});j(c$,"equals",function(a){if(this===a)return!0;if(n(a,java.util.Map)){if(this.size()!=a.size())return!1;var b=this.entrySet(),d;for(a=a.entrySet().iterator();a.hasNext()&&((d=a.next())||
1);)if(!b.contains(d))return!1;return!0}return!1},"~O");j(c$,"get",function(a){a.hashCode||(a.hashCode=function(){return 1},a.equals||(a.equals=function(a){return this==a}));for(var b=a.hashCode(),d=this.elementData[(b&2147483647)%this.elementData.length];d;){if(d.equalsKey(a,b))return d.value;d=d.next}return null},"~O");c(c$,"getEntry",function(a){for(var b=a.hashCode(),d=this.elementData[(b&2147483647)%this.elementData.length];d;){if(d.equalsKey(a,b))return d;d=d.next}return null},"~O");j(c$,"hashCode",
function(){for(var a=0,b=this.entrySet().iterator();b.hasNext();)var d=b.next(),g=d.getKey(),d=d.getValue(),g=(g!==this?g.hashCode():0)^(d!==this?null!=d?d.hashCode():0:0),a=a+g;return a});j(c$,"isEmpty",function(){return 0==this.elementCount});j(c$,"keys",function(){return 0==this.elementCount?java.util.Hashtable.EMPTY_ENUMERATION:new java.util.HashtableEnumerator(!0,this)});j(c$,"keySet",function(){return new java.util.HashtableKeySet(this)});j(c$,"put",function(a,b){if(null!=a&&null!=b){a.hashCode||
(a.hashCode=function(){return 1},a.equals||(a.equals=function(a){return this==a}));for(var d=a.hashCode(),g=(d&2147483647)%this.elementData.length,c=this.elementData[g];null!=c&&!c.equalsKey(a,d);)c=c.next;if(null==c)return this.modCount++,++this.elementCount>this.threshold&&(this.rehash(),g=(d&2147483647)%this.elementData.length),g<this.firstSlot&&(this.firstSlot=g),g>this.lastSlot&&(this.lastSlot=g),c=java.util.Hashtable.newEntry(a,b,d),c.next=this.elementData[g],this.elementData[g]=c,null;d=c.value;
c.value=b;return d}throw new NullPointerException;},"~O,~O");j(c$,"putAll",function(a){var b;for(a=a.entrySet().iterator();a.hasNext()&&((b=a.next())||1);)this.put(b.getKey(),b.getValue())},"java.util.Map");c(c$,"rehash",function(){var a=(this.elementData.length<<1)+1;0==a&&(a=1);for(var b=a,d=-1,g=this.newElementArray(a),c=this.lastSlot+1;--c>=this.firstSlot;)for(var e=this.elementData[c];null!=e;){var h=(e.getKeyHash()&2147483647)%a;h<b&&(b=h);h>d&&(d=h);var k=e.next;e.next=g[h];g[h]=e;e=k}this.firstSlot=
b;this.lastSlot=d;this.elementData=g;this.computeMaxSize()});j(c$,"remove",function(a){for(var b=a.hashCode(),d=(b&2147483647)%this.elementData.length,g=null,c=this.elementData[d];null!=c&&!c.equalsKey(a,b);)g=c,c=c.next;return null!=c?(this.modCount++,null==g?this.elementData[d]=c.next:g.next=c.next,this.elementCount--,a=c.value,c.value=null,a):null},"~O");j(c$,"size",function(){return this.elementCount});j(c$,"toString",function(){if(this.isEmpty())return"{}";var a=new StringBuilder(28*this.size());
a.append("{");for(var b=this.lastSlot;b>=this.firstSlot;b--)for(var d=this.elementData[b];null!=d;)d.key!==this?a.append(d.key):a.append("(this Map)"),a.append("="),d.value!==this?a.append(d.value):a.append("(this Map)"),a.append(", "),d=d.next;0<this.elementCount&&a.setLength(a.length()-2);a.append("}");return a.toString()});j(c$,"values",function(){return new java.util.HashtableValueCollection(this)});java.util.Hashtable.EMPTY_ENUMERATION=new java.util.HashtableEnumerator});v(["java.util.Map"],
"java.util.MapEntry",null,function(){c$=t(function(){this.value=this.key=null;p(this,arguments)},java.util,"MapEntry",null,[java.util.Map.Entry,Cloneable]);q(c$,function(a){this.key=a},"~O");q(c$,function(a,b){this.key=a;this.value=b},"~O,~O");c(c$,"clone",function(){try{return W(this,java.util.MapEntry,"clone",[])}catch(a){if(n(a,CloneNotSupportedException))return null;throw a;}});j(c$,"equals",function(a){return this===a?!0:n(a,java.util.Map.Entry)?(null==this.key?null==a.getKey():this.key.equals(a.getKey()))&&
(null==this.value?null==a.getValue():this.value.equals(a.getValue())):!1},"~O");j(c$,"getKey",function(){return this.key});j(c$,"getValue",function(){return this.value});j(c$,"hashCode",function(){return(null==this.key?0:this.key.hashCode())^(null==this.value?0:this.value.hashCode())});j(c$,"setValue",function(a){var b=this.value;this.value=a;return b},"~O");j(c$,"toString",function(){return this.key+"="+this.value});N(java.util.MapEntry,"Type")});v(["java.util.Hashtable"],"java.util.Properties",
["java.lang.NullPointerException","$.StringBuffer"],function(){c$=t(function(){this.defaults=this.builder=null;p(this,arguments)},java.util,"Properties",java.util.Hashtable);q(c$,function(a){I(this,java.util.Properties,[]);this.defaults=a},"java.util.Properties");c(c$,"dumpString",($fz=function(a,b,d){var g=0;!d&&(g<b.length&&32==b.charAt(g).charCodeAt(0))&&(a.append("\\ "),g++);for(;g<b.length;g++){var c=b.charAt(g);switch(c){case "\t":a.append("\\t");break;case "\n":a.append("\\n");break;case "\f":a.append("\\f");
break;case "\r":a.append("\\r");break;default:(0<="\\#!=:".indexOf(c)||d&&32==c.charCodeAt(0))&&a.append("\\");if(!(32<=c.charCodeAt(0)&&126>=c.charCodeAt(0))){c=Integer.toHexString(c.charCodeAt(0));a.append("\\u");for(var e=0;e<4-c.length;e++)a.append("0")}a.append(c)}}},$fz.isPrivate=!0,$fz),"StringBuilder,~S,~B");c(c$,"getProperty",function(a){var b=this.get(a),b=n(b,String)?b:null;null==b&&null!=this.defaults&&(b=this.defaults.getProperty(a));return b},"~S");c(c$,"getProperty",function(a,b){var d=
this.get(a),d=n(d,String)?d:null;null==d&&null!=this.defaults&&(d=this.defaults.getProperty(a));return null==d?b:d},"~S,~S");c(c$,"list",function(a){if(null==a)throw new NullPointerException;for(var b=new StringBuffer(80),d=this.propertyNames();d.hasMoreElements();){var g=d.nextElement();b.append(g);b.append("=");for(var c=this.get(g),e=this.defaults;null==c;)c=e.get(g),e=e.defaults;40<c.length?(b.append(c.substring(0,37)),b.append("...")):b.append(c);a.println(b.toString());b.setLength(0)}},"java.io.PrintStream");
c(c$,"list",function(a){if(null==a)throw new NullPointerException;for(var b=new StringBuffer(80),d=this.propertyNames();d.hasMoreElements();){var g=d.nextElement();b.append(g);b.append("=");for(var c=this.get(g),e=this.defaults;null==c;)c=e.get(g),e=e.defaults;40<c.length?(b.append(c.substring(0,37)),b.append("...")):b.append(c);a.println(b.toString());b.setLength(0)}},"java.io.PrintWriter");c(c$,"load",function(){},"java.io.InputStream");c(c$,"propertyNames",function(){if(null==this.defaults)return this.keys();
for(var a=new java.util.Hashtable(this.defaults.size()+this.size()),b=this.defaults.propertyNames();b.hasMoreElements();)a.put(b.nextElement(),a);for(b=this.keys();b.hasMoreElements();)a.put(b.nextElement(),a);return a.keys()});c(c$,"save",function(a,b){try{this.store(a,b)}catch(d){if(!n(d,java.io.IOException))throw d;}},"java.io.OutputStream,~S");c(c$,"setProperty",function(a,b){return this.put(a,b)},"~S,~S");c(c$,"store",function(){},"java.io.OutputStream,~S");c(c$,"loadFromXML",function(){},"java.io.InputStream");
c(c$,"storeToXML",function(){},"java.io.OutputStream,~S");c(c$,"storeToXML",function(){},"java.io.OutputStream,~S,~S");c(c$,"substitutePredefinedEntries",($fz=function(a){return a.replaceAll("&","&amp;").replaceAll("<","&lt;").replaceAll(">","&gt;").replaceAll("'","&apos;").replaceAll('"',"&quot;")},$fz.isPrivate=!0,$fz),"~S");E(c$,"PROP_DTD_NAME","http://java.sun.com/dtd/properties.dtd","PROP_DTD",'<?xml version="1.0" encoding="UTF-8"?>    <!ELEMENT properties (comment?, entry*) >    <!ATTLIST properties version CDATA #FIXED "1.0" >    <!ELEMENT comment (#PCDATA) >    <!ELEMENT entry (#PCDATA) >    <!ATTLIST entry key CDATA #REQUIRED >',
"NONE",0,"SLASH",1,"UNICODE",2,"CONTINUE",3,"KEY_DONE",4,"IGNORE",5,"lineSeparator",null)});v(["java.util.Map"],"java.util.SortedMap",null,function(){N(java.util,"SortedMap",java.util.Map)});v(["java.util.Set"],"java.util.SortedSet",null,function(){N(java.util,"SortedSet",java.util.Set)});v(["java.util.Enumeration"],"java.util.StringTokenizer",["java.lang.NullPointerException","java.util.NoSuchElementException"],function(){c$=t(function(){this.delimiters=this.string=null;this.returnDelimiters=!1;
this.position=0;p(this,arguments)},java.util,"StringTokenizer",null,java.util.Enumeration);q(c$,function(a){this.construct(a," \t\n\r\f",!1)},"~S");q(c$,function(a,b){this.construct(a,b,!1)},"~S,~S");q(c$,function(a,b,d){if(null!=a)this.string=a,this.delimiters=b,this.returnDelimiters=d,this.position=0;else throw new NullPointerException;},"~S,~S,~B");c(c$,"countTokens",function(){for(var a=0,b=!1,d=this.position,g=this.string.length;d<g;d++)0<=this.delimiters.indexOf(this.string.charAt(d),0)?(this.returnDelimiters&&
a++,b&&(a++,b=!1)):b=!0;b&&a++;return a});j(c$,"hasMoreElements",function(){return this.hasMoreTokens()});c(c$,"hasMoreTokens",function(){var a=this.string.length;if(this.position<a){if(this.returnDelimiters)return!0;for(var b=this.position;b<a;b++)if(-1==this.delimiters.indexOf(this.string.charAt(b),0))return!0}return!1});j(c$,"nextElement",function(){return this.nextToken()});c(c$,"nextToken",function(){var a=this.position,b=this.string.length;if(a<b){if(this.returnDelimiters){if(0<=this.delimiters.indexOf(this.string.charAt(this.position),
0))return String.valueOf(this.string.charAt(this.position++));for(this.position++;this.position<b;this.position++)if(0<=this.delimiters.indexOf(this.string.charAt(this.position),0))return this.string.substring(a,this.position);return this.string.substring(a)}for(;a<b&&0<=this.delimiters.indexOf(this.string.charAt(a),0);)a++;this.position=a;if(a<b){for(this.position++;this.position<b;this.position++)if(0<=this.delimiters.indexOf(this.string.charAt(this.position),0))return this.string.substring(a,this.position);
return this.string.substring(a)}}throw new java.util.NoSuchElementException;});c(c$,"nextToken",function(a){this.delimiters=a;return this.nextToken()},"~S")});s("javajs.api");N(javajs.api,"BytePoster");s("javajs.api");N(javajs.api,"GenericColor");s("J.api");N(J.api,"GenericFileInterface");s("javajs.api");N(javajs.api,"GenericOutputChannel");s("javajs.api");N(javajs.api,"JSInterface");s("javajs.api");N(javajs.api,"JSONEncodable");s("javajs.api");N(javajs.api,"ZInputStream");s("javajs.api.js");N(javajs.api.js,
"J2SObjectInterface");s("J.api");N(J.api,"GenericMouseInterface");s("J.api");v(["J.api.FontManager"],"J.api.GenericPlatform",null,function(){c$=N(J.api,"GenericPlatform",J.api.FontManager);E(c$,"CURSOR_DEFAULT",0,"CURSOR_CROSSHAIR",1,"CURSOR_WAIT",3,"CURSOR_ZOOM",8,"CURSOR_HAND",12,"CURSOR_MOVE",13)});s("J.api");N(J.api,"PlatformViewer");s("J.api");N(J.api,"EventManager");s("J.api");N(J.api,"FontManager");s("JU");v(null,"JU.Font",["JU.AU"],function(){c$=t(function(){this.fid=0;this.fontStyle=this.fontFace=
null;this.fontSize=this.idFontStyle=this.idFontFace=this.fontSizeNominal=0;this.manager=this.fontMetrics=this.font=null;this.descent=this.ascent=0;this.isItalic=this.isBold=!1;p(this,arguments)},JU,"Font");q(c$,function(a,b,d,g,c,e,h){this.manager=a;this.fid=b;this.fontFace=JU.Font.fontFaces[d];this.fontStyle=JU.Font.fontStyles[g];this.idFontFace=d;this.idFontStyle=g;this.fontSize=c;this.isBold=1==(g&1);this.isItalic=2==(g&2);this.fontSizeNominal=e;this.font=a.newFont(JU.Font.fontFaces[d],this.isBold,
this.isItalic,c);this.fontMetrics=a.getFontMetrics(this,h);this.descent=a.getFontDescent(this.fontMetrics);this.ascent=a.getFontAscent(this.fontMetrics)},"J.api.FontManager,~N,~N,~N,~N,~N,~O");c$.getFont3D=c(c$,"getFont3D",function(a){return JU.Font.font3ds[a&255]},"~N");c$.createFont3D=c(c$,"createFont3D",function(a,b,d,g,c,e){255<d&&(d=255);for(var h=A(d)<<4,h=a&3|(b&3)<<2|h<<4,k=JU.Font.fontkeyCount;0<--k;)if(h==JU.Font.fontkeys[k]&&JU.Font.font3ds[k].fontSizeNominal==g)return JU.Font.font3ds[k];
k=JU.Font.fontkeyCount++;k==JU.Font.fontkeys.length&&(JU.Font.fontkeys=JU.AU.arrayCopyI(JU.Font.fontkeys,k+8));JU.Font.font3ds=JU.AU.arrayCopyObject(JU.Font.font3ds,k+8);a=new JU.Font(c,k,a,b,d,g,e);JU.Font.font3ds[k]=a;JU.Font.fontkeys[k]=h;return a},"~N,~N,~N,~N,J.api.FontManager,~O");c$.getFontFaceID=c(c$,"getFontFaceID",function(a){return"Monospaced".equalsIgnoreCase(a)?2:"Serif".equalsIgnoreCase(a)?1:0},"~S");c$.getFontStyleID=c(c$,"getFontStyleID",function(a){for(var b=4;0<=--b;)if(JU.Font.fontStyles[b].equalsIgnoreCase(a))return b;
return-1},"~S");c(c$,"getAscent",function(){return this.ascent});c(c$,"getDescent",function(){return this.descent});c(c$,"getHeight",function(){return this.getAscent()+this.getDescent()});c(c$,"getFontMetrics",function(){return this.fontMetrics});c(c$,"stringWidth",function(a){return this.manager.fontStringWidth(this,a)},"~S");c(c$,"getInfo",function(){return this.fontSizeNominal+" "+this.fontFace+" "+this.fontStyle});j(c$,"toString",function(){return"["+this.getInfo()+"]"});E(c$,"FONT_ALLOCATION_UNIT",
8,"fontkeyCount",1,"fontkeys",B(8,0));c$.font3ds=c$.prototype.font3ds=Array(8);E(c$,"FONT_FACE_SANS",0,"FONT_FACE_SERIF",1,"FONT_FACE_MONO",2,"fontFaces",u(-1,["SansSerif","Serif","Monospaced",""]),"FONT_STYLE_PLAIN",0,"FONT_STYLE_BOLD",1,"FONT_STYLE_ITALIC",2,"FONT_STYLE_BOLDITALIC",3,"fontStyles",u(-1,["Plain","Bold","Italic","BoldItalic"]))});s("JS");v(["javajs.api.GenericColor"],"JS.Color",null,function(){c$=t(function(){this.argb=0;p(this,arguments)},JS,"Color",null,javajs.api.GenericColor);
j(c$,"getRGB",function(){return this.argb&16777215});j(c$,"getOpacity255",function(){return this.argb>>24&255});j(c$,"setOpacity255",function(a){this.argb=this.argb&16777215|(a&255)<<24},"~N");c$.get1=c(c$,"get1",function(a){var b=new JS.Color;b.argb=a|4278190080;return b},"~N");c$.get3=c(c$,"get3",function(a,b,d){return(new JS.Color).set4(a,b,d,255)},"~N,~N,~N");c$.get4=c(c$,"get4",function(a,b,d,g){return(new JS.Color).set4(a,b,d,g)},"~N,~N,~N,~N");c(c$,"set4",function(a,b,d,g){this.argb=(g<<24|
a<<16|b<<8|d)&4294967295;return this},"~N,~N,~N,~N");j(c$,"toString",function(){var a="00000000"+Integer.toHexString(this.argb);return"[0x"+a.substring(a.length-8,a.length)+"]"})});s("JS");c$=t(function(){this.height=this.width=0;p(this,arguments)},JS,"Dimension");q(c$,function(a,b){this.set(a,b)},"~N,~N");c(c$,"set",function(a,b){this.width=a;this.height=b;return this},"~N,~N");s("J.awtjs");c$=D(J.awtjs,"Event");E(c$,"MOUSE_LEFT",16,"MOUSE_MIDDLE",8,"MOUSE_RIGHT",4,"MOUSE_WHEEL",32,"MAC_COMMAND",
20,"BUTTON_MASK",28,"MOUSE_DOWN",501,"MOUSE_UP",502,"MOUSE_MOVE",503,"MOUSE_ENTER",504,"MOUSE_EXIT",505,"MOUSE_DRAG",506,"SHIFT_MASK",1,"ALT_MASK",8,"CTRL_MASK",2,"CTRL_ALT",10,"CTRL_SHIFT",3,"META_MASK",4,"VK_SHIFT",16,"VK_ALT",18,"VK_CONTROL",17,"VK_META",157,"VK_LEFT",37,"VK_RIGHT",39,"VK_PERIOD",46,"VK_SPACE",32,"VK_DOWN",40,"VK_UP",38,"VK_ESCAPE",27,"VK_DELETE",127,"VK_BACK_SPACE",8,"VK_PAGE_DOWN",34,"VK_PAGE_UP",33,"MOVED",0,"DRAGGED",1,"CLICKED",2,"WHEELED",3,"PRESSED",4,"RELEASED",5);s("J.api");
N(J.api,"GenericMenuInterface");s("JU");v(["javajs.api.JSONEncodable"],"JU.A4",["JU.T3"],function(){c$=t(function(){this.angle=this.z=this.y=this.x=0;p(this,arguments)},JU,"A4",null,[javajs.api.JSONEncodable,java.io.Serializable]);q(c$,function(){this.z=1});c$.new4=c(c$,"new4",function(a,b,d,g){var c=new JU.A4;c.set4(a,b,d,g);return c},"~N,~N,~N,~N");c$.newAA=c(c$,"newAA",function(a){var b=new JU.A4;b.set4(a.x,a.y,a.z,a.angle);return b},"JU.A4");c$.newVA=c(c$,"newVA",function(a,b){var d=new JU.A4;
d.setVA(a,b);return d},"JU.V3,~N");c(c$,"setVA",function(a,b){this.x=a.x;this.y=a.y;this.z=a.z;this.angle=b},"JU.V3,~N");c(c$,"set4",function(a,b,d,g){this.x=a;this.y=b;this.z=d;this.angle=g},"~N,~N,~N,~N");c(c$,"setAA",function(a){this.x=a.x;this.y=a.y;this.z=a.z;this.angle=a.angle},"JU.A4");c(c$,"setM",function(a){this.setFromMat(a.m00,a.m01,a.m02,a.m10,a.m11,a.m12,a.m20,a.m21,a.m22)},"JU.M3");c(c$,"setFromMat",function(a,b,d,g,c,e,h,k,l){a=0.5*(a+c+l-1);this.x=k-e;this.y=d-h;this.z=g-b;b=0.5*Math.sqrt(this.x*
this.x+this.y*this.y+this.z*this.z);0==b&&1==a?(this.x=this.y=0,this.z=1,this.angle=0):this.angle=Math.atan2(b,a)},"~N,~N,~N,~N,~N,~N,~N,~N,~N");j(c$,"hashCode",function(){return JU.T3.floatToIntBits(this.x)^JU.T3.floatToIntBits(this.y)^JU.T3.floatToIntBits(this.z)^JU.T3.floatToIntBits(this.angle)});j(c$,"equals",function(a){return!n(a,JU.A4)?!1:this.x==a.x&&this.y==a.y&&this.z==a.z&&this.angle==a.angle},"~O");j(c$,"toString",function(){return"("+this.x+", "+this.y+", "+this.z+", "+this.angle+")"});
j(c$,"toJSON",function(){return"["+this.x+","+this.y+","+this.z+","+180*this.angle/3.141592653589793+"]"})});s("JU");v(["java.net.URLConnection"],"JU.AjaxURLConnection",["JU.AU","$.Rdr","$.SB"],function(){c$=t(function(){this.bytesOut=null;this.postOut="";p(this,arguments)},JU,"AjaxURLConnection",java.net.URLConnection);c(c$,"doAjax",function(){var a=null,a=Jmol;return a.doAjax(this.url,this.postOut,this.bytesOut,!1)});j(c$,"connect",function(){});c(c$,"outputBytes",function(a){this.bytesOut=a},"~A");
c(c$,"outputString",function(a){this.postOut=a},"~S");j(c$,"getInputStream",function(){var a=null,b=this.doAjax();JU.AU.isAB(b)?a=JU.Rdr.getBIS(b):n(b,JU.SB)?a=JU.Rdr.getBIS(JU.Rdr.getBytesFromSB(b)):n(b,String)&&(a=JU.Rdr.getBIS(b.getBytes()));return a});c(c$,"getContents",function(){return this.doAjax()})});s("JU");v(["java.net.URLStreamHandler"],"JU.AjaxURLStreamHandler",["JU.AjaxURLConnection","$.SB"],function(){c$=t(function(){this.protocol=null;p(this,arguments)},JU,"AjaxURLStreamHandler",java.net.URLStreamHandler);
q(c$,function(a){I(this,JU.AjaxURLStreamHandler,[]);this.protocol=a},"~S");j(c$,"openConnection",function(a){return new JU.AjaxURLConnection(a)},"java.net.URL");j(c$,"toExternalForm",function(a){var b=new JU.SB;b.append(a.getProtocol());b.append(":");null!=a.getAuthority()&&0<a.getAuthority().length&&(b.append("//"),b.append(a.getAuthority()));null!=a.getPath()&&b.append(a.getPath());null!=a.getQuery()&&(b.append("?"),b.append(a.getQuery()));null!=a.getRef()&&(b.append("#"),b.append(a.getRef()));
return b.toString()},"java.net.URL")});s("JU");v(["java.net.URLStreamHandlerFactory","java.util.Hashtable"],"JU.AjaxURLStreamHandlerFactory",["JU.AjaxURLStreamHandler"],function(){c$=t(function(){this.htFactories=null;p(this,arguments)},JU,"AjaxURLStreamHandlerFactory",null,java.net.URLStreamHandlerFactory);O(c$,function(){this.htFactories=new java.util.Hashtable});j(c$,"createURLStreamHandler",function(a){var b=this.htFactories.get(a);null==b&&this.htFactories.put(a,b=new JU.AjaxURLStreamHandler(a));
return null==b.protocol?null:b},"~S")});s("JU");v(null,"JU.AU",["java.lang.reflect.Array","java.util.Arrays","JU.Lst"],function(){c$=D(JU,"AU");c$.ensureLength=c(c$,"ensureLength",function(a,b){return null!=a&&JU.AU.getLength(a)>=b?a:JU.AU.arrayCopyObject(a,b)},"~O,~N");c$.ensureLengthS=c(c$,"ensureLengthS",function(a,b){return null!=a&&a.length>=b?a:JU.AU.arrayCopyS(a,b)},"~A,~N");c$.ensureLengthA=c(c$,"ensureLengthA",function(a,b){return null!=a&&a.length>=b?a:JU.AU.arrayCopyF(a,b)},"~A,~N");c$.ensureLengthI=
c(c$,"ensureLengthI",function(a,b){return null!=a&&a.length>=b?a:JU.AU.arrayCopyI(a,b)},"~A,~N");c$.ensureLengthShort=c(c$,"ensureLengthShort",function(a,b){return null!=a&&a.length>=b?a:JU.AU.arrayCopyShort(a,b)},"~A,~N");c$.ensureLengthByte=c(c$,"ensureLengthByte",function(a,b){return null!=a&&a.length>=b?a:JU.AU.arrayCopyByte(a,b)},"~A,~N");c$.doubleLength=c(c$,"doubleLength",function(a){return JU.AU.arrayCopyObject(a,null==a?16:2*JU.AU.getLength(a))},"~O");c$.doubleLengthS=c(c$,"doubleLengthS",
function(a){return JU.AU.arrayCopyS(a,null==a?16:2*a.length)},"~A");c$.doubleLengthF=c(c$,"doubleLengthF",function(a){return JU.AU.arrayCopyF(a,null==a?16:2*a.length)},"~A");c$.doubleLengthI=c(c$,"doubleLengthI",function(a){return JU.AU.arrayCopyI(a,null==a?16:2*a.length)},"~A");c$.doubleLengthShort=c(c$,"doubleLengthShort",function(a){return JU.AU.arrayCopyShort(a,null==a?16:2*a.length)},"~A");c$.doubleLengthByte=c(c$,"doubleLengthByte",function(a){return JU.AU.arrayCopyByte(a,null==a?16:2*a.length)},
"~A");c$.doubleLengthBool=c(c$,"doubleLengthBool",function(a){return JU.AU.arrayCopyBool(a,null==a?16:2*a.length)},"~A");c$.deleteElements=c(c$,"deleteElements",function(a,b,d){if(0==d||null==a)return a;var g=JU.AU.getLength(a);if(b>=g)return a;g-=b+d;0>g&&(g=0);var c=JU.AU.newInstanceO(a,b+g);0<b&&Zystem.arraycopy(a,0,c,0,b);0<g&&Zystem.arraycopy(a,b+d,c,b,g);return c},"~O,~N,~N");c$.arrayCopyObject=c(c$,"arrayCopyObject",function(a,b){var d=null==a?-1:JU.AU.getLength(a);0>b&&(b=d);if(b==d)return a;
if(b<d)return u(-1,a,0,b);var g=JU.AU.newInstanceO(a,b);0<d&&Zystem.arraycopy(a,0,g,0,d<b?d:b);return g},"~O,~N");c$.newInstanceO=c(c$,"newInstanceO",function(a,b){return JU.AU.isAI(a)?B(b,0):!a.getClass||!a.getClass().getComponentType?Array(b):java.lang.reflect.Array.newInstance(a.getClass().getComponentType(),b)},"~O,~N");c$.getLength=c(c$,"getLength",function(a){return a.length},"~O");c$.arrayCopyS=c(c$,"arrayCopyS",function(a,b){var d=null==a?-1:a.length;0>b&&(b=d);if(b<d)return u(-1,a,0,b);var g=
Array(b);null!=a&&Zystem.arraycopy(a,0,g,0,d<b?d:b);return g},"~A,~N");c$.arrayCopyII=c(c$,"arrayCopyII",function(a,b){var d=JU.AU.newInt2(b);if(null!=a){var g=a.length;Zystem.arraycopy(a,0,d,0,g<b?g:b)}return d},"~A,~N");c$.arrayCopyPt=c(c$,"arrayCopyPt",function(a,b){0>b&&(b=a.length);var d=Array(b);if(null!=a){var g=a.length;Zystem.arraycopy(a,0,d,0,g<b?g:b)}return d},"~A,~N");c$.arrayCopyF=c(c$,"arrayCopyF",function(a,b){var d=null==a?-1:a.length;0>b&&(b=d);if(b<d)return u(-1,a,0,b);var g=K(b,
0);null!=a&&Zystem.arraycopy(a,0,g,0,d<b?d:b);return g},"~A,~N");c$.arrayCopyI=c(c$,"arrayCopyI",function(a,b){var d=null==a?-1:a.length;0>b&&(b=d);if(b<d)return u(-1,a,0,b);var g=B(b,0);null!=a&&Zystem.arraycopy(a,0,g,0,d<b?d:b);return g},"~A,~N");c$.arrayCopyRangeI=c(c$,"arrayCopyRangeI",function(a,b,d){if(null==a)return null;var g=a.length;-1==d&&(d=g);-2==d&&(d=w(g/2));return u(-1,a,b,d)},"~A,~N,~N");c$.arrayCopyRangeRevI=c(c$,"arrayCopyRangeRevI",function(a,b,d){return null==a?null:u(-1,a,b,
d).reverse()},"~A,~N,~N");c$.arrayCopyShort=c(c$,"arrayCopyShort",function(a,b){var d=null==a?-1:a.length;0>b&&(b=d);if(b<d)return u(-1,a,0,b);var g=T(b,0);null!=a&&Zystem.arraycopy(a,0,g,0,d<b?d:b);return g},"~A,~N");c$.arrayCopyByte=c(c$,"arrayCopyByte",function(a,b){var d=null==a?-1:a.length;0>b&&(b=d);if(b<d)return u(-1,a,0,b);var g=Q(b,0);null!=a&&Zystem.arraycopy(a,0,g,0,d<b?d:b);return g},"~A,~N");c$.arrayCopyBool=c(c$,"arrayCopyBool",function(a,b){var d=null==a?-1:a.length;0>b&&(b=d);if(b<
d)return u(-1,a,0,b);var g=ga(b,!1);null!=a&&Zystem.arraycopy(a,0,g,0,d<b?d:b);return g},"~A,~N");c$.swapInt=c(c$,"swapInt",function(a,b,d){var g=a[b];a[b]=a[d];a[d]=g},"~A,~N,~N");c$.dumpArray=c(c$,"dumpArray",function(a,b,d,g,c,e){a="dumpArray: "+a+"\n";for(var h=d;h<=g;h++)a+="\t*"+h+"*";for(;e>=c;e--){a+="\n*"+e+"*";for(h=d;h<=g;h++)a+="\t"+(h<b.length&&e<b[h].length?b[h][e]:NaN)}return a},"~S,~A,~N,~N,~N,~N");c$.dumpIntArray=c(c$,"dumpIntArray",function(a,b){for(var d="",g=0;g<b;g++)d+=" "+a[g];
return d},"~A,~N");c$.sortedItem=c(c$,"sortedItem",function(a,b){if(0==a.size())return null;if(1==a.size())return a.get(0);var d=a.toArray(Array(a.size()));java.util.Arrays.sort(d);return d[b%d.length]},"JU.Lst,~N");c$.createArrayOfArrayList=c(c$,"createArrayOfArrayList",function(a){return Array(a)},"~N");c$.createArrayOfHashtable=c(c$,"createArrayOfHashtable",function(a){return Array(a)},"~N");c$.swap=c(c$,"swap",function(a,b,d){var g=a[b];a[b]=a[d];a[d]=g},"~A,~N,~N");c$.newFloat2=c(c$,"newFloat2",
function(a){return u(a,null)},"~N");c$.newInt2=c(c$,"newInt2",function(a){return u(a,null)},"~N");c$.newInt3=c(c$,"newInt3",function(a){return u(a,null)},"~N,~N");c$.newFloat3=c(c$,"newFloat3",function(a){return u(a,null)},"~N,~N");c$.newInt4=c(c$,"newInt4",function(a){return u(a,null)},"~N");c$.newShort2=c(c$,"newShort2",function(a){return u(a,null)},"~N");c$.newByte2=c(c$,"newByte2",function(a){return u(a,null)},"~N");c$.newDouble2=c(c$,"newDouble2",function(a){return u(a,null)},"~N");c$.removeMapKeys=
c(c$,"removeMapKeys",function(a,b){for(var d=new JU.Lst,g,c=a.keySet().iterator();c.hasNext()&&((g=c.next())||1);)g.startsWith(b)&&d.addLast(g);for(g=d.size();0<=--g;)a.remove(d.get(g));return d.size()},"java.util.Map,~S");c$.isAS=c(c$,"isAS",function(a){return ya(a)},"~O");c$.isASS=c(c$,"isASS",function(a){return za(a)},"~O");c$.isAP=c(c$,"isAP",function(a){return Aa(a)},"~O");c$.isAF=c(c$,"isAF",function(a){return ma(a)},"~O");c$.isAFloat=c(c$,"isAFloat",function(a){return Ba(a)},"~O");c$.isAD=
c(c$,"isAD",function(a){return ma(a)},"~O");c$.isADD=c(c$,"isADD",function(a){return na(a)},"~O");c$.isAB=c(c$,"isAB",function(a){return wa(a)},"~O");c$.isAI=c(c$,"isAI",function(a){return xa(a)},"~O");c$.isAII=c(c$,"isAII",function(a){return Ca(a)},"~O");c$.isAFF=c(c$,"isAFF",function(a){return na(a)},"~O");c$.isAFFF=c(c$,"isAFFF",function(a){return Da(a)},"~O");c$.ensureSignedBytes=c(c$,"ensureSignedBytes",function(a){if(null!=a)for(var b=a.length;0<=--b;){var d=a[b]&255;128<=d&&(d-=256);a[b]=d}return a},
"~A")});s("JU");v(null,"JU.Base64",["JU.SB"],function(){c$=D(JU,"Base64");c$.getBytes64=c(c$,"getBytes64",function(a){return JU.Base64.getBase64(a).toBytes(0,-1)},"~A");c$.getBase64=c(c$,"getBase64",function(a){var b=a.length,d=new JU.SB;if(0==b)return d;for(var g=0,c=0;g<b&&0==c;){0==g%75&&0!=g&&d.append("\r\n");var c=g+2==b?1:g+1==b?2:0,e=a[g++]<<16&16711680|(2==c?0:a[g++]<<8)&65280|(1<=c?0:a[g++])&255;d.appendC(JU.Base64.base64.charAt(e>>18&63));d.appendC(JU.Base64.base64.charAt(e>>12&63));d.appendC(2==
c?"=":JU.Base64.base64.charAt(e>>6&63));d.appendC(1<=c?"=":JU.Base64.base64.charAt(e&63))}return d},"~A");c$.decodeBase64=c(c$,"decodeBase64",function(a){var b=0,d,g=a.indexOf(";base64,")+1;0<g&&(g+=7);a=a.toCharArray();var c=a.length;if(0==c)return Q(0,0);for(var e=c;--e>=g;)b+=65==(d=a[e].charCodeAt(0)&127)||0<JU.Base64.decode64[d]?3:0;for(var b=b>>2,h=Q(b,0),k=18,e=g,l=g=0;e<c;e++)if(0<JU.Base64.decode64[d=a[e].charCodeAt(0)&127]||65==d||61==d)l|=JU.Base64.decode64[d]<<k,k-=6,0>k&&(h[g++]=(l&16711680)>>
16,g<b&&(h[g++]=(l&65280)>>8),g<b&&(h[g++]=l&255),k=18,l=0);return h},"~S");E(c$,"base64","ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/","decode64",B(-1,[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,62,0,62,0,63,52,53,54,55,56,57,58,59,60,61,0,0,0,0,0,0,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,0,0,0,0,63,0,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,0,0,0,0,0]))});s("JU");v(["javajs.api.JSONEncodable"],
"JU.BS",["java.lang.IndexOutOfBoundsException","$.NegativeArraySizeException","JU.PT","$.SB"],function(){c$=t(function(){this.words=null;this.wordsInUse=0;this.sizeIsSticky=!1;p(this,arguments)},JU,"BS",null,[Cloneable,javajs.api.JSONEncodable]);c$.wordIndex=c(c$,"wordIndex",function(a){return a>>5},"~N");c(c$,"recalculateWordsInUse",function(){var a;for(a=this.wordsInUse-1;0<=a&&0==this.words[a];a--);this.wordsInUse=a+1});q(c$,function(){this.initWords(32);this.sizeIsSticky=!1});c$.newN=c(c$,"newN",
function(a){var b=new JU.BS;b.init(a);return b},"~N");c(c$,"init",function(a){if(0>a)throw new NegativeArraySizeException("nbits < 0: "+a);this.initWords(a);this.sizeIsSticky=!0},"~N");c(c$,"initWords",function(a){this.words=B(JU.BS.wordIndex(a-1)+1,0)},"~N");c(c$,"ensureCapacity",function(a){this.words.length<a&&(a=Math.max(2*this.words.length,a),this.setLength(a),this.sizeIsSticky=!1)},"~N");c(c$,"expandTo",function(a){a+=1;this.wordsInUse<a&&(this.ensureCapacity(a),this.wordsInUse=a)},"~N");c(c$,
"set",function(a){if(0>a)throw new IndexOutOfBoundsException("bitIndex < 0: "+a);var b=JU.BS.wordIndex(a);this.expandTo(b);this.words[b]|=1<<a},"~N");c(c$,"setBitTo",function(a,b){b?this.set(a):this.clear(a)},"~N,~B");c(c$,"setBits",function(a,b){if(a!=b){var d=JU.BS.wordIndex(a),g=JU.BS.wordIndex(b-1);this.expandTo(g);var c=-1<<a,e=-1>>>-b;if(d==g)this.words[d]|=c&e;else{this.words[d]|=c;for(d+=1;d<g;d++)this.words[d]=-1;this.words[g]|=e}}},"~N,~N");c(c$,"clear",function(a){if(0>a)throw new IndexOutOfBoundsException("bitIndex < 0: "+
a);var b=JU.BS.wordIndex(a);b>=this.wordsInUse||(this.words[b]&=~(1<<a),this.recalculateWordsInUse())},"~N");c(c$,"clearBits",function(a,b){if(a!=b){var d=JU.BS.wordIndex(a);if(!(d>=this.wordsInUse)){var g=JU.BS.wordIndex(b-1);g>=this.wordsInUse&&(b=this.length(),g=this.wordsInUse-1);var c=-1<<a,e=-1>>>-b;if(d==g)this.words[d]&=~(c&e);else{this.words[d]&=~c;for(d+=1;d<g;d++)this.words[d]=0;this.words[g]&=~e}this.recalculateWordsInUse()}}},"~N,~N");c(c$,"clearAll",function(){for(;0<this.wordsInUse;)this.words[--this.wordsInUse]=
0});c(c$,"get",function(a){if(0>a)throw new IndexOutOfBoundsException("bitIndex < 0: "+a);var b=JU.BS.wordIndex(a);return b<this.wordsInUse&&0!=(this.words[b]&1<<a)},"~N");c(c$,"nextSetBit",function(a){if(0>a)throw new IndexOutOfBoundsException("fromIndex < 0: "+a);var b=JU.BS.wordIndex(a);if(b>=this.wordsInUse)return-1;for(a=this.words[b]&-1<<a;;){if(0!=a)return 32*b+Integer.numberOfTrailingZeros(a);if(++b==this.wordsInUse)return-1;a=this.words[b]}},"~N");c(c$,"nextClearBit",function(a){if(0>a)throw new IndexOutOfBoundsException("fromIndex < 0: "+
a);var b=JU.BS.wordIndex(a);if(b>=this.wordsInUse)return a;for(a=~this.words[b]&-1<<a;;){if(0!=a)return 32*b+Integer.numberOfTrailingZeros(a);if(++b==this.wordsInUse)return 32*this.wordsInUse;a=~this.words[b]}},"~N");c(c$,"length",function(){return 0==this.wordsInUse?0:32*(this.wordsInUse-1)+(32-Integer.numberOfLeadingZeros(this.words[this.wordsInUse-1]))});c(c$,"isEmpty",function(){return 0==this.wordsInUse});c(c$,"intersects",function(a){for(var b=Math.min(this.wordsInUse,a.wordsInUse)-1;0<=b;b--)if(0!=
(this.words[b]&a.words[b]))return!0;return!1},"JU.BS");c(c$,"cardinality",function(){for(var a=0,b=0;b<this.wordsInUse;b++)a+=Integer.bitCount(this.words[b]);return a});c(c$,"and",function(a){if(this!==a){for(;this.wordsInUse>a.wordsInUse;)this.words[--this.wordsInUse]=0;for(var b=0;b<this.wordsInUse;b++)this.words[b]&=a.words[b];this.recalculateWordsInUse()}},"JU.BS");c(c$,"or",function(a){if(this!==a){var b=Math.min(this.wordsInUse,a.wordsInUse);this.wordsInUse<a.wordsInUse&&(this.ensureCapacity(a.wordsInUse),
this.wordsInUse=a.wordsInUse);for(var d=0;d<b;d++)this.words[d]|=a.words[d];b<a.wordsInUse&&Zystem.arraycopy(a.words,b,this.words,b,this.wordsInUse-b)}},"JU.BS");c(c$,"xor",function(a){var b=Math.min(this.wordsInUse,a.wordsInUse);this.wordsInUse<a.wordsInUse&&(this.ensureCapacity(a.wordsInUse),this.wordsInUse=a.wordsInUse);for(var d=0;d<b;d++)this.words[d]^=a.words[d];b<a.wordsInUse&&Zystem.arraycopy(a.words,b,this.words,b,a.wordsInUse-b);this.recalculateWordsInUse()},"JU.BS");c(c$,"andNot",function(a){for(var b=
Math.min(this.wordsInUse,a.wordsInUse)-1;0<=b;b--)this.words[b]&=~a.words[b];this.recalculateWordsInUse()},"JU.BS");j(c$,"hashCode",function(){for(var a=1234,b=this.wordsInUse;0<=--b;)a^=this.words[b]*(b+1);return a>>32^a});c(c$,"size",function(){return 32*this.words.length});j(c$,"equals",function(a){if(!n(a,JU.BS))return!1;if(this===a)return!0;if(this.wordsInUse!=a.wordsInUse)return!1;for(var b=0;b<this.wordsInUse;b++)if(this.words[b]!=a.words[b])return!1;return!0},"~O");j(c$,"clone",function(){!this.sizeIsSticky&&
this.wordsInUse!=this.words.length&&this.setLength(this.wordsInUse);return JU.BS.copy(this)});c(c$,"setLength",function(a){a!=this.words.length&&(a==this.wordsInUse?this.words=u(-1,this.words,0,a):(a=B(a,0),Zystem.arraycopy(this.words,0,a,0,this.wordsInUse),this.words=a))},"~N");j(c$,"toString",function(){return JU.BS.escape(this,"(",")")});c$.copy=c(c$,"copy",function(a){var b;b=la(a);var d=a.wordsInUse;b.words=0==d?JU.BS.emptyBitmap:u(-1,a.words,0,b.wordsInUse=d);return b},"JU.BS");c(c$,"cardinalityN",
function(a){for(var b=this.cardinality(),d=this.length();--d>=a;)this.get(d)&&b--;return b},"~N");j(c$,"toJSON",function(){var a=128<this.wordsInUse?this.cardinality():32*this.wordsInUse,a=JU.SB.newN(6*a+2);a.appendC("[");var b=this.nextSetBit(0);if(-1!=b){a.appendI(b);for(b=this.nextSetBit(b+1);0<=b;b=this.nextSetBit(b+1)){var d=this.nextClearBit(b);do a.append(", ").appendI(b);while(++b<d)}}a.appendC("]");return a.toString()});c$.escape=c(c$,"escape",function(a,b,d){if(null==a)return b+"{}"+d;var g=
new JU.SB;g.append(b+"{");b=a.length();for(var c=-1,e=-2,h=-1;++h<=b;){var k=a.get(h);if(h==b||0<=c&&!k){0<=c&&e!=c&&g.append((e==c-1?" ":":")+c);if(h==b)break;c=-1}a.get(h)&&(0>c&&(g.append((-2==e?"":" ")+h),e=h),c=h)}g.append("}").appendC(d);return g.toString()},"JU.BS,~S,~S");c$.unescape=c(c$,"unescape",function(a){var b,d;if(null==a||4>(d=(a=a.trim()).length)||a.equalsIgnoreCase("({null})")||"("!=(b=a.charAt(0))&&"["!=b||a.charAt(d-1)!=("("==b?")":"]")||"{"!=a.charAt(1)||a.indexOf("}")!=d-2)return null;
for(var g=d-=2;2<=--g;)if(!JU.PT.isDigit(b=a.charAt(g))&&" "!=b&&"\t"!=b&&":"!=b)return null;for(var c=d;JU.PT.isDigit(a.charAt(--c)););if(++c==d)c=0;else try{c=Integer.parseInt(a.substring(c,d))}catch(e){if(H(e,NumberFormatException))return null;throw e;}for(var h=JU.BS.newN(c),k=c=-1,l=-2,g=2;g<=d;g++)switch(b=a.charAt(g)){case "\t":case " ":case "}":if(0>l)break;if(l<c)return null;c=l;0>k&&(k=l);h.setBits(k,l+1);k=-1;l=-2;break;case ":":k=c=l;l=-2;break;default:JU.PT.isDigit(b)&&(0>l&&(l=0),l=
10*l+(b.charCodeAt(0)-48))}return 0<=k?null:h},"~S");E(c$,"ADDRESS_BITS_PER_WORD",5,"BITS_PER_WORD",32,"WORD_MASK",4294967295,"emptyBitmap",B(0,0))});s("JU");v(["java.util.Hashtable"],"JU.CU",["JU.P3","$.PT"],function(){c$=D(JU,"CU");c$.toRGBHexString=c(c$,"toRGBHexString",function(a){var d=a.getRGB();if(0==d)return"000000";a="00"+Integer.toHexString(d>>16&255);a=a.substring(a.length-2);var g="00"+Integer.toHexString(d>>8&255),g=g.substring(g.length-2),d="00"+Integer.toHexString(d&255),d=d.substring(d.length-
2);return a+g+d},"javajs.api.GenericColor");c$.toCSSString=c(c$,"toCSSString",function(a){var d=a.getOpacity255();if(255==d)return"#"+JU.CU.toRGBHexString(a);a=a.getRGB();return"rgba("+(a>>16&255)+","+(a>>8&255)+","+(a&255)+","+d/255+")"},"javajs.api.GenericColor");c$.getArgbFromString=c(c$,"getArgbFromString",function(a){var d=0;if(null==a||0==(d=a.length))return 0;a=a.toLowerCase();if("["==a.charAt(0)&&"]"==a.charAt(d-1)){var g;if(0<=a.indexOf(",")){g=JU.PT.split(a.substring(1,a.length-1),",");
if(3!=g.length)return 0;a=JU.PT.parseFloat(g[0]);d=JU.PT.parseFloat(g[1]);g=JU.PT.parseFloat(g[2]);return JU.CU.colorTriadToFFRGB(a,d,g)}switch(d){case 9:g="x";break;case 10:g="0x";break;default:return 0}if(1!=a.indexOf(g))return 0;a="#"+a.substring(d-7,d-1);d=7}if(7==d&&"#"==a.charAt(0))try{return JU.PT.parseIntRadix(a.substring(1,7),16)|4278190080}catch(c){if(H(c,Exception))return 0;throw c;}a=JU.CU.mapJavaScriptColors.get(a);return null==a?0:a.intValue()},"~S");c$.colorTriadToFFRGB=c(c$,"colorTriadToFFRGB",
function(a,d,g){1>=a&&(1>=d&&1>=g)&&(0<a&&(a=256*a-1),0<d&&(d=256*d-1),0<g&&(g=256*g-1));return JU.CU.rgb(A(a),A(d),A(g))},"~N,~N,~N");c$.rgb=c(c$,"rgb",function(a,d,g){return 4278190080|a<<16|d<<8|g},"~N,~N,~N");c$.colorPtFromString=c(c$,"colorPtFromString",function(a){return JU.CU.colorPtFromInt(JU.CU.getArgbFromString(a),null)},"~S");c$.colorPtFromInt=c(c$,"colorPtFromInt",function(a,d){null==d&&(d=new JU.P3);d.set(a>>16&255,a>>8&255,a&255);return d},"~N,JU.P3");c$.colorPtToFFRGB=c(c$,"colorPtToFFRGB",
function(a){return JU.CU.colorTriadToFFRGB(a.x,a.y,a.z)},"JU.T3");c$.toRGB3f=c(c$,"toRGB3f",function(a,d){d[0]=(a>>16&255)/255;d[1]=(a>>8&255)/255;d[2]=(a&255)/255},"~N,~A");c$.toFFGGGfromRGB=c(c$,"toFFGGGfromRGB",function(a){a=w((2989*(a>>16&255)+5870*(a>>8&255)+1140*(a&255)+5E3)/1E4)&16777215;return JU.CU.rgb(a,a,a)},"~N");c$.rgbToHSL=c(c$,"rgbToHSL",function(a,d){var g=a.x/255,c=a.y/255,e=a.z/255,h=Math.min(g,Math.min(c,e)),k=Math.max(g,Math.max(c,e)),l=k+h,h=k-h,g=60*(0==h?0:k==g?(c-e)/h+6:k==
c?(e-g)/h+2:(g-c)/h+4)%360,c=h/(0==h?1:1>=l?l:2-l);return d?JU.P3.new3(Math.round(10*g)/10,Math.round(1E3*c)/10,Math.round(500*l)/10):JU.P3.new3(g,100*c,50*l)},"JU.P3,~B");c$.hslToRGB=c(c$,"hslToRGB",function(a){var d=Math.max(0,Math.min(360,a.x))/60,g=Math.max(0,Math.min(100,a.y))/100;a=Math.max(0,Math.min(100,a.z))/100;var g=a-(0.5>a?a:1-a)*g,c=2*(a-g);a=JU.CU.toRGB(g,c,d+2);var e=JU.CU.toRGB(g,c,d),d=JU.CU.toRGB(g,c,d-2);return JU.P3.new3(Math.round(255*a),Math.round(255*e),Math.round(255*d))},
"JU.P3");c$.toRGB=c(c$,"toRGB",function(a,d,g){return 1>(g+=0>g?6:6<g?-6:0)?a+d*g:3>g?a+d:4>g?a+d*(4-g):a},"~N,~N,~N");E(c$,"colorNames",u(-1,"black pewhite pecyan pepurple pegreen peblue peviolet pebrown pepink peyellow pedarkgreen peorange pelightblue pedarkcyan pedarkgray aliceblue antiquewhite aqua aquamarine azure beige bisque blanchedalmond blue blueviolet brown burlywood cadetblue chartreuse chocolate coral cornflowerblue cornsilk crimson cyan darkblue darkcyan darkgoldenrod darkgray darkgreen darkkhaki darkmagenta darkolivegreen darkorange darkorchid darkred darksalmon darkseagreen darkslateblue darkslategray darkturquoise darkviolet deeppink deepskyblue dimgray dodgerblue firebrick floralwhite forestgreen fuchsia gainsboro ghostwhite gold goldenrod gray green greenyellow honeydew hotpink indianred indigo ivory khaki lavender lavenderblush lawngreen lemonchiffon lightblue lightcoral lightcyan lightgoldenrodyellow lightgreen lightgrey lightgray lightpink lightsalmon lightseagreen lightskyblue lightslategray lightsteelblue lightyellow lime limegreen linen magenta maroon mediumaquamarine mediumblue mediumorchid mediumpurple mediumseagreen mediumslateblue mediumspringgreen mediumturquoise mediumvioletred midnightblue mintcream mistyrose moccasin navajowhite navy oldlace olive olivedrab orange orangered orchid palegoldenrod palegreen paleturquoise palevioletred papayawhip peachpuff peru pink plum powderblue purple red rosybrown royalblue saddlebrown salmon sandybrown seagreen seashell sienna silver skyblue slateblue slategray snow springgreen steelblue tan teal thistle tomato turquoise violet wheat white whitesmoke yellow yellowgreen bluetint greenblue greentint grey gray pinktint redorange yellowtint".split(" ")),
"colorArgbs",B(-1,[4278190080,4294967295,4278255615,4291830015,4278255360,4284506367,4294934720,4288946216,4294957272,4294967040,4278239232,4294946816,4289769727,4278231200,4284506208,4293982463,4294634455,4278255615,4286578644,4293984255,4294309340,4294960324,4294962125,4278190335,4287245282,4289014314,4292786311,4284456608,4286578432,4291979550,4294934352,4284782061,4294965468,4292613180,4278255615,4278190219,4278225803,4290283019,4289309097,4278215680,4290623339,4287299723,4283788079,4294937600,
4288230092,4287299584,4293498490,4287609999,4282924427,4281290575,4278243025,4287889619,4294907027,4278239231,4285098345,4280193279,4289864226,4294966E3,4280453922,4294902015,4292664540,4294506751,4294956800,4292519200,4286611584,4278222848,4289593135,4293984240,4294928820,4291648604,4283105410,4294967280,4293977740,4293322490,4294963445,4286381056,4294965965,4289583334,4293951616,4292935679,4294638290,4287688336,4292072403,4292072403,4294948545,4294942842,4280332970,4287090426,4286023833,4289774814,
4294967264,4278255360,4281519410,4294635750,4294902015,4286578688,4284927402,4278190285,4290401747,4287852763,4282168177,4286277870,4278254234,4282962380,4291237253,4279834992,4294311930,4294960353,4294960309,4294958765,4278190208,4294833638,4286611456,4285238819,4294944E3,4294919424,4292505814,4293847210,4288215960,4289720046,4292571283,4294963157,4294957753,4291659071,4294951115,4292714717,4289781990,4286578816,4294901760,4290547599,4282477025,4287317267,4294606962,4294222944,4281240407,4294964718,
4288696877,4290822336,4287090411,4285160141,4285563024,4294966010,4278255487,4282811060,4291998860,4278222976,4292394968,4294927175,4282441936,4293821166,4294303411,4294967295,4294309365,4294967040,4288335154,4289714175,4281240407,4288217011,4286611584,4286611584,4294945723,4294919424,4294375029]));c$.mapJavaScriptColors=c$.prototype.mapJavaScriptColors=new java.util.Hashtable;for(var a=JU.CU.colorNames.length;0<=--a;)JU.CU.mapJavaScriptColors.put(JU.CU.colorNames[a],Integer.$valueOf(JU.CU.colorArgbs[a]))});
s("JU");v(["java.lang.Boolean"],"JU.DF",["java.lang.Double","$.Float","JU.PT","$.SB"],function(){c$=D(JU,"DF");c$.setUseNumberLocalization=c(c$,"setUseNumberLocalization",function(a){JU.DF.useNumberLocalization[0]=a?Boolean.TRUE:Boolean.FALSE},"~B");c$.formatDecimalDbl=c(c$,"formatDecimalDbl",function(a,b){return 2147483647==b||-Infinity==a||Infinity==a||Double.isNaN(a)?""+a:JU.DF.formatDecimal(a,b)},"~N,~N");c$.formatDecimal=c(c$,"formatDecimal",function(a,b){if(2147483647==b||-Infinity==a||Infinity==
a||Float.isNaN(a))return""+a;var d=0>a;d&&(a=-a);var g;if(0>b){b=-b;b>JU.DF.formattingStrings.length&&(b=JU.DF.formattingStrings.length);if(0==a)return JU.DF.formattingStrings[b-1]+"E+0";var c;1>Math.abs(a)?(g=100,c=1E-100*a):(g=-100,c=1E100*a);c=(""+c).toUpperCase();var e=c.indexOf("E");0>e?c=""+a:(g=JU.PT.parseInt(c.substring(e+(c.indexOf("E+")==e?2:1)))+g,c=JU.PT.parseFloat(c.substring(0,e)),c=JU.DF.formatDecimal(c,b-1),c.startsWith("10.")&&(c=JU.DF.formatDecimal(1,b-1),g++));return(d?"-":"")+
c+"E"+(0<=g?"+":"")+g}b>=JU.DF.formattingStrings.length&&(b=JU.DF.formattingStrings.length-1);c=(""+a).toUpperCase();g=c.indexOf(".");if(0>g)return c+JU.DF.formattingStrings[b].substring(1);e=c.indexOf("E-");0<e&&(g=JU.PT.parseInt(c.substring(e+1)),c="0."+"0000000000000000000000000000000000000000".substring(0,-g-1)+c.substring(0,1)+c.substring(2,e),g=1);e=c.indexOf("E");0<e&&(g=JU.PT.parseInt(c.substring(e+1)),c=c.substring(0,1)+c.substring(2,e)+"0000000000000000000000000000000000000000",c=c.substring(0,
g+1)+"."+c.substring(g+1),g=c.indexOf("."));var e=c.length,h=b+g+1;if(h<e&&"5"<=c.charAt(h))return JU.DF.formatDecimal((d?-1:1)*(a+JU.DF.formatAdds[b]),b);var k=c.substring(0,0==b?g:++g),h=JU.SB.newS(k);d&&(k.equals("0.")&&b+2<=e&&c.substring(2,2+b).equals("0000000000000000000000000000000000000000".substring(0,b)))&&(d=!1);for(k=0;k<b;k++,g++)g<e?h.appendC(c.charAt(g)):h.appendC("0");c=(d?"-":"")+h;return Boolean.TRUE.equals(JU.DF.useNumberLocalization[0])?c:c.$replace(",",".")},"~N,~N");c$.formatDecimalTrimmed=
c(c$,"formatDecimalTrimmed",function(a,b){for(var d=JU.DF.formatDecimalDbl(a,b),g=d.length-1;0<=g&&"0"==d.charAt(g);)g--;return d.substring(0,g+1)},"~N,~N");c$.formatDecimalTrimmed0=c(c$,"formatDecimalTrimmed0",function(a,b){for(var d=JU.DF.formatDecimalDbl(a,b),g=d.length-1,c=d.indexOf(".")+1;g>c&&"0"==d.charAt(g);)g--;return d.substring(0,g+1)},"~N,~N");E(c$,"formattingStrings",u(-1,"0 0.0 0.00 0.000 0.0000 0.00000 0.000000 0.0000000 0.00000000 0.000000000".split(" ")),"zeros","0000000000000000000000000000000000000000",
"formatAdds",K(-1,[0.5,0.05,0.005,5E-4,5E-5,5E-6,5E-7,5E-8,5E-9,5E-10]));c$.useNumberLocalization=c$.prototype.useNumberLocalization=u(-1,[Boolean.TRUE])});s("JU");v(["java.lang.Enum"],"JU.Encoding",null,function(){c$=D(JU,"Encoding",Enum);G(c$,"NONE",0,[]);G(c$,"UTF8",1,[]);G(c$,"UTF_16BE",2,[]);G(c$,"UTF_16LE",3,[]);G(c$,"UTF_32BE",4,[]);G(c$,"UTF_32LE",5,[])});s("JU");v(["java.util.ArrayList"],"JU.Lst",null,function(){c$=D(JU,"Lst",java.util.ArrayList);c(c$,"addLast",function(a){return this.add1(a)},
"~O");c(c$,"removeItemAt",function(a){return this._removeItemAt(a)},"~N");c(c$,"removeObj",function(a){return this._removeObject(a)},"~O")});s("JU");v(null,"JU.M34",["java.lang.ArrayIndexOutOfBoundsException"],function(){c$=t(function(){this.m22=this.m21=this.m20=this.m12=this.m11=this.m10=this.m02=this.m01=this.m00=0;p(this,arguments)},JU,"M34");c(c$,"setAA33",function(a){var b=a.x,d=a.y,g=a.z;a=a.angle;var c=Math.sqrt(b*b+d*d+g*g),c=1/c,b=b*c,d=d*c,g=g*c,e=Math.cos(a);a=Math.sin(a);c=1-e;this.m00=
e+b*b*c;this.m11=e+d*d*c;this.m22=e+g*g*c;var e=b*d*c,h=g*a;this.m01=e-h;this.m10=e+h;e=b*g*c;h=d*a;this.m02=e+h;this.m20=e-h;e=d*g*c;h=b*a;this.m12=e-h;this.m21=e+h},"JU.A4");c(c$,"rotate",function(a){this.rotate2(a,a)},"JU.T3");c(c$,"rotate2",function(a,b){b.set(this.m00*a.x+this.m01*a.y+this.m02*a.z,this.m10*a.x+this.m11*a.y+this.m12*a.z,this.m20*a.x+this.m21*a.y+this.m22*a.z)},"JU.T3,JU.T3");c(c$,"setM33",function(a){this.m00=a.m00;this.m01=a.m01;this.m02=a.m02;this.m10=a.m10;this.m11=a.m11;this.m12=
a.m12;this.m20=a.m20;this.m21=a.m21;this.m22=a.m22},"JU.M34");c(c$,"clear33",function(){this.m00=this.m01=this.m02=this.m10=this.m11=this.m12=this.m20=this.m21=this.m22=0});c(c$,"set33",function(a,b,d){switch(a){case 0:switch(b){case 0:this.m00=d;return;case 1:this.m01=d;return;case 2:this.m02=d;return}break;case 1:switch(b){case 0:this.m10=d;return;case 1:this.m11=d;return;case 2:this.m12=d;return}break;case 2:switch(b){case 0:this.m20=d;return;case 1:this.m21=d;return;case 2:this.m22=d;return}}this.err()},
"~N,~N,~N");c(c$,"get33",function(a,b){switch(a){case 0:switch(b){case 0:return this.m00;case 1:return this.m01;case 2:return this.m02}break;case 1:switch(b){case 0:return this.m10;case 1:return this.m11;case 2:return this.m12}break;case 2:switch(b){case 0:return this.m20;case 1:return this.m21;case 2:return this.m22}}this.err();return 0},"~N,~N");c(c$,"setRow33",function(a,b){switch(a){case 0:this.m00=b[0];this.m01=b[1];this.m02=b[2];break;case 1:this.m10=b[0];this.m11=b[1];this.m12=b[2];break;case 2:this.m20=
b[0];this.m21=b[1];this.m22=b[2];break;default:this.err()}},"~N,~A");c(c$,"getRow33",function(a,b){switch(a){case 0:b[0]=this.m00;b[1]=this.m01;b[2]=this.m02;return;case 1:b[0]=this.m10;b[1]=this.m11;b[2]=this.m12;return;case 2:b[0]=this.m20;b[1]=this.m21;b[2]=this.m22;return}this.err()},"~N,~A");c(c$,"setColumn33",function(a,b){switch(a){case 0:this.m00=b[0];this.m10=b[1];this.m20=b[2];break;case 1:this.m01=b[0];this.m11=b[1];this.m21=b[2];break;case 2:this.m02=b[0];this.m12=b[1];this.m22=b[2];break;
default:this.err()}},"~N,~A");c(c$,"getColumn33",function(a,b){switch(a){case 0:b[0]=this.m00;b[1]=this.m10;b[2]=this.m20;break;case 1:b[0]=this.m01;b[1]=this.m11;b[2]=this.m21;break;case 2:b[0]=this.m02;b[1]=this.m12;b[2]=this.m22;break;default:this.err()}},"~N,~A");c(c$,"add33",function(a){this.m00+=a.m00;this.m01+=a.m01;this.m02+=a.m02;this.m10+=a.m10;this.m11+=a.m11;this.m12+=a.m12;this.m20+=a.m20;this.m21+=a.m21;this.m22+=a.m22},"JU.M34");c(c$,"sub33",function(a){this.m00-=a.m00;this.m01-=a.m01;
this.m02-=a.m02;this.m10-=a.m10;this.m11-=a.m11;this.m12-=a.m12;this.m20-=a.m20;this.m21-=a.m21;this.m22-=a.m22},"JU.M34");c(c$,"mul33",function(a){this.m00*=a;this.m01*=a;this.m02*=a;this.m10*=a;this.m11*=a;this.m12*=a;this.m20*=a;this.m21*=a;this.m22*=a},"~N");c(c$,"transpose33",function(){var a=this.m01;this.m01=this.m10;this.m10=a;a=this.m02;this.m02=this.m20;this.m20=a;a=this.m12;this.m12=this.m21;this.m21=a});c(c$,"setXRot",function(a){var b=Math.cos(a);a=Math.sin(a);this.m00=1;this.m10=this.m02=
this.m01=0;this.m11=b;this.m12=-a;this.m20=0;this.m21=a;this.m22=b},"~N");c(c$,"setYRot",function(a){var b=Math.cos(a);a=Math.sin(a);this.m00=b;this.m01=0;this.m02=a;this.m10=0;this.m11=1;this.m12=0;this.m20=-a;this.m21=0;this.m22=b},"~N");c(c$,"setZRot",function(a){var b=Math.cos(a);a=Math.sin(a);this.m00=b;this.m01=-a;this.m02=0;this.m10=a;this.m11=b;this.m21=this.m20=this.m12=0;this.m22=1},"~N");c(c$,"determinant3",function(){return this.m00*(this.m11*this.m22-this.m21*this.m12)-this.m01*(this.m10*
this.m22-this.m20*this.m12)+this.m02*(this.m10*this.m21-this.m20*this.m11)});c(c$,"err",function(){throw new ArrayIndexOutOfBoundsException("matrix column/row out of bounds");})});s("JU");v(["JU.M34"],"JU.M3",["JU.T3"],function(){c$=D(JU,"M3",JU.M34,java.io.Serializable);c$.newA9=c(c$,"newA9",function(a){var b=new JU.M3;b.setA(a);return b},"~A");c$.newM3=c(c$,"newM3",function(a){var b=new JU.M3;if(null==a)return b.setScale(1),b;b.m00=a.m00;b.m01=a.m01;b.m02=a.m02;b.m10=a.m10;b.m11=a.m11;b.m12=a.m12;
b.m20=a.m20;b.m21=a.m21;b.m22=a.m22;return b},"JU.M3");c(c$,"setScale",function(a){this.clear33();this.m00=this.m11=this.m22=a},"~N");c(c$,"setM3",function(a){this.setM33(a)},"JU.M34");c(c$,"setA",function(a){this.m00=a[0];this.m01=a[1];this.m02=a[2];this.m10=a[3];this.m11=a[4];this.m12=a[5];this.m20=a[6];this.m21=a[7];this.m22=a[8]},"~A");c(c$,"setElement",function(a,b,d){this.set33(a,b,d)},"~N,~N,~N");c(c$,"getElement",function(a,b){return this.get33(a,b)},"~N,~N");c(c$,"setRow",function(a,b,d,
g){switch(a){case 0:this.m00=b;this.m01=d;this.m02=g;break;case 1:this.m10=b;this.m11=d;this.m12=g;break;case 2:this.m20=b;this.m21=d;this.m22=g;break;default:this.err()}},"~N,~N,~N,~N");c(c$,"setRowV",function(a,b){switch(a){case 0:this.m00=b.x;this.m01=b.y;this.m02=b.z;break;case 1:this.m10=b.x;this.m11=b.y;this.m12=b.z;break;case 2:this.m20=b.x;this.m21=b.y;this.m22=b.z;break;default:this.err()}},"~N,JU.T3");c(c$,"setRowA",function(a,b){this.setRow33(a,b)},"~N,~A");j(c$,"getRow",function(a,b){this.getRow33(a,
b)},"~N,~A");c(c$,"setColumn3",function(a,b,d,g){switch(a){case 0:this.m00=b;this.m10=d;this.m20=g;break;case 1:this.m01=b;this.m11=d;this.m21=g;break;case 2:this.m02=b;this.m12=d;this.m22=g;break;default:this.err()}},"~N,~N,~N,~N");c(c$,"setColumnV",function(a,b){switch(a){case 0:this.m00=b.x;this.m10=b.y;this.m20=b.z;break;case 1:this.m01=b.x;this.m11=b.y;this.m21=b.z;break;case 2:this.m02=b.x;this.m12=b.y;this.m22=b.z;break;default:this.err()}},"~N,JU.T3");c(c$,"getColumnV",function(a,b){switch(a){case 0:b.x=
this.m00;b.y=this.m10;b.z=this.m20;break;case 1:b.x=this.m01;b.y=this.m11;b.z=this.m21;break;case 2:b.x=this.m02;b.y=this.m12;b.z=this.m22;break;default:this.err()}},"~N,JU.T3");c(c$,"setColumnA",function(a,b){this.setColumn33(a,b)},"~N,~A");c(c$,"getColumn",function(a,b){this.getColumn33(a,b)},"~N,~A");c(c$,"add",function(a){this.add33(a)},"JU.M3");c(c$,"sub",function(a){this.sub33(a)},"JU.M3");c(c$,"transpose",function(){this.transpose33()});c(c$,"transposeM",function(a){this.setM33(a);this.transpose33()},
"JU.M3");c(c$,"invertM",function(a){this.setM33(a);this.invert()},"JU.M3");c(c$,"invert",function(){var a=this.determinant3();0!=a&&(a=1/a,this.set9(this.m11*this.m22-this.m12*this.m21,this.m02*this.m21-this.m01*this.m22,this.m01*this.m12-this.m02*this.m11,this.m12*this.m20-this.m10*this.m22,this.m00*this.m22-this.m02*this.m20,this.m02*this.m10-this.m00*this.m12,this.m10*this.m21-this.m11*this.m20,this.m01*this.m20-this.m00*this.m21,this.m00*this.m11-this.m01*this.m10),this.scale(a))});c(c$,"setAsXRotation",
function(a){this.setXRot(a);return this},"~N");c(c$,"setAsYRotation",function(a){this.setYRot(a);return this},"~N");c(c$,"setAsZRotation",function(a){this.setZRot(a);return this},"~N");c(c$,"scale",function(a){this.mul33(a)},"~N");c(c$,"mul",function(a){this.mul2(this,a)},"JU.M3");c(c$,"mul2",function(a,b){this.set9(a.m00*b.m00+a.m01*b.m10+a.m02*b.m20,a.m00*b.m01+a.m01*b.m11+a.m02*b.m21,a.m00*b.m02+a.m01*b.m12+a.m02*b.m22,a.m10*b.m00+a.m11*b.m10+a.m12*b.m20,a.m10*b.m01+a.m11*b.m11+a.m12*b.m21,a.m10*
b.m02+a.m11*b.m12+a.m12*b.m22,a.m20*b.m00+a.m21*b.m10+a.m22*b.m20,a.m20*b.m01+a.m21*b.m11+a.m22*b.m21,a.m20*b.m02+a.m21*b.m12+a.m22*b.m22)},"JU.M3,JU.M3");j(c$,"equals",function(a){return!n(a,JU.M3)?!1:this.m00==a.m00&&this.m01==a.m01&&this.m02==a.m02&&this.m10==a.m10&&this.m11==a.m11&&this.m12==a.m12&&this.m20==a.m20&&this.m21==a.m21&&this.m22==a.m22},"~O");j(c$,"hashCode",function(){return JU.T3.floatToIntBits(this.m00)^JU.T3.floatToIntBits(this.m01)^JU.T3.floatToIntBits(this.m02)^JU.T3.floatToIntBits(this.m10)^
JU.T3.floatToIntBits(this.m11)^JU.T3.floatToIntBits(this.m12)^JU.T3.floatToIntBits(this.m20)^JU.T3.floatToIntBits(this.m21)^JU.T3.floatToIntBits(this.m22)});c(c$,"setZero",function(){this.clear33()});c(c$,"set9",function(a,b,d,g,c,e,h,k,l){this.m00=a;this.m01=b;this.m02=d;this.m10=g;this.m11=c;this.m12=e;this.m20=h;this.m21=k;this.m22=l},"~N,~N,~N,~N,~N,~N,~N,~N,~N");j(c$,"toString",function(){return"[\n  ["+this.m00+"\t"+this.m01+"\t"+this.m02+"]\n  ["+this.m10+"\t"+this.m11+"\t"+this.m12+"]\n  ["+
this.m20+"\t"+this.m21+"\t"+this.m22+"] ]"});c(c$,"setAA",function(a){this.setAA33(a);return this},"JU.A4");c(c$,"setAsBallRotation",function(a,b,d){var g=Math.sqrt(b*b+d*d),c=g*a;if(0==c)return this.setScale(1),!1;a=Math.cos(c);c=Math.sin(c);d=-d/g;b/=g;g=a-1;this.m00=1+g*d*d;this.m01=this.m10=g*d*b;this.m20=-(this.m02=c*d);this.m11=1+g*b*b;this.m21=-(this.m12=c*b);this.m22=a;return!0},"~N,~N,~N");c(c$,"isRotation",function(){return 0.001>Math.abs(this.determinant3()-1)})});s("JU");v(["JU.M34"],
"JU.M4",["JU.T3"],function(){c$=t(function(){this.m33=this.m32=this.m31=this.m30=this.m23=this.m13=this.m03=0;p(this,arguments)},JU,"M4",JU.M34);c$.newA16=c(c$,"newA16",function(a){var b=new JU.M4;b.m00=a[0];b.m01=a[1];b.m02=a[2];b.m03=a[3];b.m10=a[4];b.m11=a[5];b.m12=a[6];b.m13=a[7];b.m20=a[8];b.m21=a[9];b.m22=a[10];b.m23=a[11];b.m30=a[12];b.m31=a[13];b.m32=a[14];b.m33=a[15];return b},"~A");c$.newM4=c(c$,"newM4",function(a){var b=new JU.M4;if(null==a)return b.setIdentity(),b;b.setToM3(a);b.m03=a.m03;
b.m13=a.m13;b.m23=a.m23;b.m30=a.m30;b.m31=a.m31;b.m32=a.m32;b.m33=a.m33;return b},"JU.M4");c$.newMV=c(c$,"newMV",function(a,b){var d=new JU.M4;d.setMV(a,b);return d},"JU.M3,JU.T3");c(c$,"setZero",function(){this.clear33();this.m03=this.m13=this.m23=this.m30=this.m31=this.m32=this.m33=0});c(c$,"setIdentity",function(){this.setZero();this.m00=this.m11=this.m22=this.m33=1});c(c$,"setM4",function(a){this.setM33(a);this.m03=a.m03;this.m13=a.m13;this.m23=a.m23;this.m30=a.m30;this.m31=a.m31;this.m32=a.m32;
this.m33=a.m33;return this},"JU.M4");c(c$,"setMV",function(a,b){this.setM33(a);this.setTranslation(b);this.m33=1},"JU.M3,JU.T3");c(c$,"setToM3",function(a){this.setM33(a);this.m03=this.m13=this.m23=this.m30=this.m31=this.m32=0;this.m33=1},"JU.M34");c(c$,"setToAA",function(a){this.setIdentity();this.setAA33(a)},"JU.A4");c(c$,"setA",function(a){this.m00=a[0];this.m01=a[1];this.m02=a[2];this.m03=a[3];this.m10=a[4];this.m11=a[5];this.m12=a[6];this.m13=a[7];this.m20=a[8];this.m21=a[9];this.m22=a[10];this.m23=
a[11];this.m30=a[12];this.m31=a[13];this.m32=a[14];this.m33=a[15]},"~A");c(c$,"setTranslation",function(a){this.m03=a.x;this.m13=a.y;this.m23=a.z},"JU.T3");c(c$,"setElement",function(a,b,d){if(3>a&&3>b)this.set33(a,b,d);else{(3<a||3<b)&&this.err();switch(a){case 0:this.m03=d;return;case 1:this.m13=d;return;case 2:this.m23=d;return}switch(b){case 0:this.m30=d;break;case 1:this.m31=d;break;case 2:this.m32=d;break;case 3:this.m33=d}}},"~N,~N,~N");c(c$,"getElement",function(a,b){if(3>a&&3>b)return this.get33(a,
b);if(3<a||3<b)return this.err(),0;switch(a){case 0:return this.m03;case 1:return this.m13;case 2:return this.m23;default:switch(b){case 0:return this.m30;case 1:return this.m31;case 2:return this.m32;default:return this.m33}}},"~N,~N");c(c$,"getTranslation",function(a){a.x=this.m03;a.y=this.m13;a.z=this.m23},"JU.T3");c(c$,"getRotationScale",function(a){a.m00=this.m00;a.m01=this.m01;a.m02=this.m02;a.m10=this.m10;a.m11=this.m11;a.m12=this.m12;a.m20=this.m20;a.m21=this.m21;a.m22=this.m22},"JU.M3");
c(c$,"setRotationScale",function(a){this.m00=a.m00;this.m01=a.m01;this.m02=a.m02;this.m10=a.m10;this.m11=a.m11;this.m12=a.m12;this.m20=a.m20;this.m21=a.m21;this.m22=a.m22},"JU.M3");c(c$,"setRowA",function(a,b){3>a&&this.setRow33(a,b);switch(a){case 0:this.m03=b[3];return;case 1:this.m13=b[3];return;case 2:this.m23=b[3];return;case 3:this.m30=b[0];this.m31=b[1];this.m32=b[2];this.m33=b[3];return}this.err()},"~N,~A");j(c$,"getRow",function(a,b){3>a&&this.getRow33(a,b);switch(a){case 0:b[3]=this.m03;
return;case 1:b[3]=this.m13;return;case 2:b[3]=this.m23;return;case 3:b[0]=this.m30;b[1]=this.m31;b[2]=this.m32;b[3]=this.m33;return}this.err()},"~N,~A");c(c$,"setColumn4",function(a,b,d,g,c){0==a?(this.m00=b,this.m10=d,this.m20=g,this.m30=c):1==a?(this.m01=b,this.m11=d,this.m21=g,this.m31=c):2==a?(this.m02=b,this.m12=d,this.m22=g,this.m32=c):3==a?(this.m03=b,this.m13=d,this.m23=g,this.m33=c):this.err()},"~N,~N,~N,~N,~N");c(c$,"setColumnA",function(a,b){3>a&&this.setColumn33(a,b);switch(a){case 0:this.m30=
b[3];break;case 1:this.m31=b[3];break;case 2:this.m32=b[3];break;case 3:this.m03=b[0];this.m13=b[1];this.m23=b[2];this.m33=b[3];break;default:this.err()}},"~N,~A");c(c$,"getColumn",function(a,b){3>a&&this.getColumn33(a,b);switch(a){case 0:b[3]=this.m30;break;case 1:b[3]=this.m31;break;case 2:b[3]=this.m32;break;case 3:b[0]=this.m03;b[1]=this.m13;b[2]=this.m23;b[3]=this.m33;break;default:this.err()}},"~N,~A");c(c$,"sub",function(a){this.sub33(a);this.m03-=a.m03;this.m13-=a.m13;this.m23-=a.m23;this.m30-=
a.m30;this.m31-=a.m31;this.m32-=a.m32;this.m33-=a.m33},"JU.M4");c(c$,"add",function(a){this.m03+=a.x;this.m13+=a.y;this.m23+=a.z},"JU.T3");c(c$,"transpose",function(){this.transpose33();var a=this.m03;this.m03=this.m30;this.m30=a;a=this.m13;this.m13=this.m31;this.m31=a;a=this.m23;this.m23=this.m32;this.m32=a});c(c$,"invert",function(){var a=this.determinant4();if(0==a)return this;a=1/a;this.set(this.m11*(this.m22*this.m33-this.m23*this.m32)+this.m12*(this.m23*this.m31-this.m21*this.m33)+this.m13*
(this.m21*this.m32-this.m22*this.m31),this.m21*(this.m02*this.m33-this.m03*this.m32)+this.m22*(this.m03*this.m31-this.m01*this.m33)+this.m23*(this.m01*this.m32-this.m02*this.m31),this.m31*(this.m02*this.m13-this.m03*this.m12)+this.m32*(this.m03*this.m11-this.m01*this.m13)+this.m33*(this.m01*this.m12-this.m02*this.m11),this.m01*(this.m13*this.m22-this.m12*this.m23)+this.m02*(this.m11*this.m23-this.m13*this.m21)+this.m03*(this.m12*this.m21-this.m11*this.m22),this.m12*(this.m20*this.m33-this.m23*this.m30)+
this.m13*(this.m22*this.m30-this.m20*this.m32)+this.m10*(this.m23*this.m32-this.m22*this.m33),this.m22*(this.m00*this.m33-this.m03*this.m30)+this.m23*(this.m02*this.m30-this.m00*this.m32)+this.m20*(this.m03*this.m32-this.m02*this.m33),this.m32*(this.m00*this.m13-this.m03*this.m10)+this.m33*(this.m02*this.m10-this.m00*this.m12)+this.m30*(this.m03*this.m12-this.m02*this.m13),this.m02*(this.m13*this.m20-this.m10*this.m23)+this.m03*(this.m10*this.m22-this.m12*this.m20)+this.m00*(this.m12*this.m23-this.m13*
this.m22),this.m13*(this.m20*this.m31-this.m21*this.m30)+this.m10*(this.m21*this.m33-this.m23*this.m31)+this.m11*(this.m23*this.m30-this.m20*this.m33),this.m23*(this.m00*this.m31-this.m01*this.m30)+this.m20*(this.m01*this.m33-this.m03*this.m31)+this.m21*(this.m03*this.m30-this.m00*this.m33),this.m33*(this.m00*this.m11-this.m01*this.m10)+this.m30*(this.m01*this.m13-this.m03*this.m11)+this.m31*(this.m03*this.m10-this.m00*this.m13),this.m03*(this.m11*this.m20-this.m10*this.m21)+this.m00*(this.m13*this.m21-
this.m11*this.m23)+this.m01*(this.m10*this.m23-this.m13*this.m20),this.m10*(this.m22*this.m31-this.m21*this.m32)+this.m11*(this.m20*this.m32-this.m22*this.m30)+this.m12*(this.m21*this.m30-this.m20*this.m31),this.m20*(this.m02*this.m31-this.m01*this.m32)+this.m21*(this.m00*this.m32-this.m02*this.m30)+this.m22*(this.m01*this.m30-this.m00*this.m31),this.m30*(this.m02*this.m11-this.m01*this.m12)+this.m31*(this.m00*this.m12-this.m02*this.m10)+this.m32*(this.m01*this.m10-this.m00*this.m11),this.m00*(this.m11*
this.m22-this.m12*this.m21)+this.m01*(this.m12*this.m20-this.m10*this.m22)+this.m02*(this.m10*this.m21-this.m11*this.m20));this.scale(a);return this});c(c$,"set",function(a,b,d,g,c,e,h,k,l,j,m,r,x,y,F,C){this.m00=a;this.m01=b;this.m02=d;this.m03=g;this.m10=c;this.m11=e;this.m12=h;this.m13=k;this.m20=l;this.m21=j;this.m22=m;this.m23=r;this.m30=x;this.m31=y;this.m32=F;this.m33=C},"~N,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N");c(c$,"determinant4",function(){return(this.m00*this.m11-this.m01*this.m10)*
(this.m22*this.m33-this.m23*this.m32)-(this.m00*this.m12-this.m02*this.m10)*(this.m21*this.m33-this.m23*this.m31)+(this.m00*this.m13-this.m03*this.m10)*(this.m21*this.m32-this.m22*this.m31)+(this.m01*this.m12-this.m02*this.m11)*(this.m20*this.m33-this.m23*this.m30)-(this.m01*this.m13-this.m03*this.m11)*(this.m20*this.m32-this.m22*this.m30)+(this.m02*this.m13-this.m03*this.m12)*(this.m20*this.m31-this.m21*this.m30)});c(c$,"scale",function(a){this.mul33(a);this.m03*=a;this.m13*=a;this.m23*=a;this.m30*=
a;this.m31*=a;this.m32*=a;this.m33*=a},"~N");c(c$,"mul",function(a){this.mul2(this,a)},"JU.M4");c(c$,"mul2",function(a,b){this.set(a.m00*b.m00+a.m01*b.m10+a.m02*b.m20+a.m03*b.m30,a.m00*b.m01+a.m01*b.m11+a.m02*b.m21+a.m03*b.m31,a.m00*b.m02+a.m01*b.m12+a.m02*b.m22+a.m03*b.m32,a.m00*b.m03+a.m01*b.m13+a.m02*b.m23+a.m03*b.m33,a.m10*b.m00+a.m11*b.m10+a.m12*b.m20+a.m13*b.m30,a.m10*b.m01+a.m11*b.m11+a.m12*b.m21+a.m13*b.m31,a.m10*b.m02+a.m11*b.m12+a.m12*b.m22+a.m13*b.m32,a.m10*b.m03+a.m11*b.m13+a.m12*b.m23+
a.m13*b.m33,a.m20*b.m00+a.m21*b.m10+a.m22*b.m20+a.m23*b.m30,a.m20*b.m01+a.m21*b.m11+a.m22*b.m21+a.m23*b.m31,a.m20*b.m02+a.m21*b.m12+a.m22*b.m22+a.m23*b.m32,a.m20*b.m03+a.m21*b.m13+a.m22*b.m23+a.m23*b.m33,a.m30*b.m00+a.m31*b.m10+a.m32*b.m20+a.m33*b.m30,a.m30*b.m01+a.m31*b.m11+a.m32*b.m21+a.m33*b.m31,a.m30*b.m02+a.m31*b.m12+a.m32*b.m22+a.m33*b.m32,a.m30*b.m03+a.m31*b.m13+a.m32*b.m23+a.m33*b.m33)},"JU.M4,JU.M4");c(c$,"transform",function(a){this.transform2(a,a)},"JU.T4");c(c$,"transform2",function(a,
b){b.set4(this.m00*a.x+this.m01*a.y+this.m02*a.z+this.m03*a.w,this.m10*a.x+this.m11*a.y+this.m12*a.z+this.m13*a.w,this.m20*a.x+this.m21*a.y+this.m22*a.z+this.m23*a.w,this.m30*a.x+this.m31*a.y+this.m32*a.z+this.m33*a.w)},"JU.T4,JU.T4");c(c$,"rotTrans",function(a){this.rotTrans2(a,a)},"JU.T3");c(c$,"rotTrans2",function(a,b){b.set(this.m00*a.x+this.m01*a.y+this.m02*a.z+this.m03,this.m10*a.x+this.m11*a.y+this.m12*a.z+this.m13,this.m20*a.x+this.m21*a.y+this.m22*a.z+this.m23);return b},"JU.T3,JU.T3");c(c$,
"setAsXYRotation",function(a){this.setIdentity();var b=Math.cos(a);a=Math.sin(a);this.m22=b;this.m23=-a;this.m32=a;this.m33=b;return this},"~N");c(c$,"setAsYZRotation",function(a){this.setIdentity();var b=Math.cos(a);a=Math.sin(a);this.m00=b;this.m03=-a;this.m30=a;this.m33=b;return this},"~N");c(c$,"setAsXZRotation",function(a){this.setIdentity();var b=Math.cos(a);a=Math.sin(a);this.m11=b;this.m13=-a;this.m31=a;this.m33=b;return this},"~N");j(c$,"equals",function(a){return!n(a,JU.M4)?!1:this.m00==
a.m00&&this.m01==a.m01&&this.m02==a.m02&&this.m03==a.m03&&this.m10==a.m10&&this.m11==a.m11&&this.m12==a.m12&&this.m13==a.m13&&this.m20==a.m20&&this.m21==a.m21&&this.m22==a.m22&&this.m23==a.m23&&this.m30==a.m30&&this.m31==a.m31&&this.m32==a.m32&&this.m33==a.m33},"~O");j(c$,"hashCode",function(){return JU.T3.floatToIntBits(this.m00)^JU.T3.floatToIntBits(this.m01)^JU.T3.floatToIntBits(this.m02)^JU.T3.floatToIntBits(this.m03)^JU.T3.floatToIntBits(this.m10)^JU.T3.floatToIntBits(this.m11)^JU.T3.floatToIntBits(this.m12)^
JU.T3.floatToIntBits(this.m13)^JU.T3.floatToIntBits(this.m20)^JU.T3.floatToIntBits(this.m21)^JU.T3.floatToIntBits(this.m22)^JU.T3.floatToIntBits(this.m23)^JU.T3.floatToIntBits(this.m30)^JU.T3.floatToIntBits(this.m31)^JU.T3.floatToIntBits(this.m32)^JU.T3.floatToIntBits(this.m33)});j(c$,"toString",function(){return"[\n  ["+this.m00+"\t"+this.m01+"\t"+this.m02+"\t"+this.m03+"]\n  ["+this.m10+"\t"+this.m11+"\t"+this.m12+"\t"+this.m13+"]\n  ["+this.m20+"\t"+this.m21+"\t"+this.m22+"\t"+this.m23+"]\n  ["+
this.m30+"\t"+this.m31+"\t"+this.m32+"\t"+this.m33+"] ]"});c(c$,"round",function(a){this.m00=this.rnd(this.m00,a);this.m01=this.rnd(this.m01,a);this.m02=this.rnd(this.m02,a);this.m03=this.rnd(this.m03,a);this.m10=this.rnd(this.m10,a);this.m11=this.rnd(this.m11,a);this.m12=this.rnd(this.m12,a);this.m13=this.rnd(this.m13,a);this.m20=this.rnd(this.m20,a);this.m21=this.rnd(this.m21,a);this.m22=this.rnd(this.m22,a);this.m23=this.rnd(this.m23,a);this.m30=this.rnd(this.m30,a);this.m31=this.rnd(this.m31,
a);this.m32=this.rnd(this.m32,a);this.m33=this.rnd(this.m33,a);return this},"~N");c(c$,"rnd",function(a,b){return Math.abs(a)<b?0:a},"~N,~N")});s("JU");v(["java.io.OutputStream","javajs.api.GenericOutputChannel"],"JU.OC","java.io.BufferedWriter $.ByteArrayOutputStream $.OutputStreamWriter java.lang.Float JU.Base64 $.SB".split(" "),function(){c$=t(function(){this.bw=this.fileName=this.bytePoster=null;this.isLocalFile=!1;this.byteCount=0;this.closed=this.isCanceled=!1;this.type=this.sb=this.os=null;
this.$isBase64=!1;this.bytes=this.os0=null;this.bigEndian=!0;p(this,arguments)},JU,"OC",java.io.OutputStream,javajs.api.GenericOutputChannel);q(c$,function(){I(this,JU.OC,[])});q(c$,function(a){I(this,JU.OC,[]);this.setParams(null,a,!1,null)},"~S");c(c$,"setParams",function(a,b,d,g){this.bytePoster=a;if(this.$isBase64=";base64,".equals(b))b=null,this.os0=g,g=null;this.fileName=b;this.os=g;this.isLocalFile=null!=b&&!JU.OC.isRemote(b);d&&(!this.$isBase64&&null!=g)&&(this.bw=new java.io.BufferedWriter(new java.io.OutputStreamWriter(g)));
return this},"javajs.api.BytePoster,~S,~B,java.io.OutputStream");j(c$,"isBigEndian",function(){return this.bigEndian});c(c$,"setBigEndian",function(a){this.bigEndian=a},"~B");c(c$,"setBytes",function(a){this.bytes=a;return this},"~A");c(c$,"getFileName",function(){return this.fileName});c(c$,"getName",function(){return null==this.fileName?null:this.fileName.substring(this.fileName.lastIndexOf("/")+1)});c(c$,"getByteCount",function(){return this.byteCount});c(c$,"setType",function(a){this.type=a},
"~S");c(c$,"getType",function(){return this.type});c(c$,"append",function(a){try{if(null!=this.bw)this.bw.write(a);else if(null==this.os)null==this.sb&&(this.sb=new JU.SB),this.sb.append(a);else{var b=a.getBytes();this.os.write(b,0,b.length);this.byteCount+=b.length;return this}}catch(d){if(!H(d,java.io.IOException))throw d;}this.byteCount+=a.length;return this},"~S");j(c$,"reset",function(){this.sb=null;this.initOS()});c(c$,"initOS",function(){if(null!=this.sb){var a=this.sb.toString();this.reset();
this.append(a)}else{try{this.os=null,null==this.os&&(this.os=new java.io.ByteArrayOutputStream),null!=this.bw&&(this.bw.close(),this.bw=new java.io.BufferedWriter(new java.io.OutputStreamWriter(this.os)))}catch(b){if(H(b,Exception))Zystem.out.println(b.toString());else throw b;}this.byteCount=0}});j(c$,"writeByteAsInt",function(a){null==this.os&&this.initOS();this.os.writeByteAsInt(a);this.byteCount++},"~N");j(c$,"write",function(a,b,d){null==this.os&&this.initOS();0>d&&(d=a.length-b);try{this.os.write(a,
b,d)}catch(g){if(!H(g,java.io.IOException))throw g;}this.byteCount+=d},"~A,~N,~N");j(c$,"writeShort",function(a){this.isBigEndian()?(this.writeByteAsInt(a>>8),this.writeByteAsInt(a)):(this.writeByteAsInt(a),this.writeByteAsInt(a>>8))},"~N");j(c$,"writeLong",function(a){this.isBigEndian()?(this.writeInt(a>>32&4294967295),this.writeInt(a&4294967295)):(this.writeByteAsInt(a>>56),this.writeByteAsInt(a>>48),this.writeByteAsInt(a>>40),this.writeByteAsInt(a>>32),this.writeByteAsInt(a>>24),this.writeByteAsInt(a>>
16),this.writeByteAsInt(a>>8),this.writeByteAsInt(a))},"~N");c(c$,"cancel",function(){this.isCanceled=!0;this.closeChannel()});j(c$,"closeChannel",function(){if(this.closed)return null;try{null!=this.bw?(this.bw.flush(),this.bw.close()):null!=this.os&&(this.os.flush(),this.os.close()),null!=this.os0&&this.isCanceled&&(this.os0.flush(),this.os0.close())}catch(a){if(!H(a,Exception))throw a;}if(this.isCanceled)return this.closed=!0,null;if(null==this.fileName){if(this.$isBase64){var b=this.getBase64();
null!=this.os0&&(this.os=this.os0,this.append(b));this.sb=new JU.SB;this.sb.append(b);this.$isBase64=!1;return this.closeChannel()}return null==this.sb?null:this.sb.toString()}this.closed=!0;if(!this.isLocalFile){b=this.postByteArray();if(null==b||b.startsWith("java.net"))this.byteCount=-1;return b}var d=b=null,b=self.J2S||Jmol,d="function"==typeof this.fileName?this.fileName:null;if(null!=b){var g=null==this.sb?this.toByteArray():this.sb.toString();null==d?b.doAjax(this.fileName,null,g,null==this.sb):
b.applyFunc(this.fileName,g)}return null});c(c$,"isBase64",function(){return this.$isBase64});c(c$,"getBase64",function(){return JU.Base64.getBase64(this.toByteArray()).toString()});c(c$,"toByteArray",function(){return null!=this.bytes?this.bytes:n(this.os,java.io.ByteArrayOutputStream)?this.os.toByteArray():null});c(c$,"close",function(){this.closeChannel()});j(c$,"toString",function(){if(null!=this.bw)try{this.bw.flush()}catch(a){if(!H(a,java.io.IOException))throw a;}return null!=this.sb?this.closeChannel():
this.byteCount+" bytes"});c(c$,"postByteArray",function(){var a=null==this.sb?this.toByteArray():this.sb.toString().getBytes();return this.bytePoster.postByteArray(this.fileName,a)});c$.isRemote=c(c$,"isRemote",function(a){if(null==a)return!1;a=JU.OC.urlTypeIndex(a);return 0<=a&&4>a},"~S");c$.isLocal=c(c$,"isLocal",function(a){return null!=a&&!JU.OC.isRemote(a)},"~S");c$.urlTypeIndex=c(c$,"urlTypeIndex",function(a){if(null==a)return-2;for(var b=0;b<JU.OC.urlPrefixes.length;++b)if(a.startsWith(JU.OC.urlPrefixes[b]))return b;
return-1},"~S");j(c$,"writeInt",function(a){this.bigEndian?(this.writeByteAsInt(a>>24),this.writeByteAsInt(a>>16),this.writeByteAsInt(a>>8),this.writeByteAsInt(a)):(this.writeByteAsInt(a),this.writeByteAsInt(a>>8),this.writeByteAsInt(a>>16),this.writeByteAsInt(a>>24))},"~N");c(c$,"writeFloat",function(a){this.writeInt(0==a?0:Float.floatToIntBits(a))},"~N");E(c$,"urlPrefixes",u(-1,"http: https: sftp: ftp: file: cache:".split(" ")),"URL_LOCAL",4,"URL_CACHE",5)});s("JU");v(["JU.T3"],"JU.P3",null,function(){c$=
D(JU,"P3",JU.T3);c$.newP=c(c$,"newP",function(a){var b=new JU.P3;b.x=a.x;b.y=a.y;b.z=a.z;return b},"JU.T3");c$.getUnlikely=c(c$,"getUnlikely",function(){return null==JU.P3.unlikely?JU.P3.unlikely=JU.P3.new3(3.141592653589793,2.718281828459045,8.539734222673566):JU.P3.unlikely});c$.new3=c(c$,"new3",function(a,b,d){var g=new JU.P3;g.x=a;g.y=b;g.z=d;return g},"~N,~N,~N");c$.newA=c(c$,"newA",function(a){return JU.P3.new3(a[0],a[1],a[2])},"~A");E(c$,"unlikely",null)});s("JU");v(["JU.T3i"],"JU.P3i",null,
function(){c$=D(JU,"P3i",JU.T3i);c$.new3=c(c$,"new3",function(a,b,d){var g=new JU.P3i;g.x=a;g.y=b;g.z=d;return g},"~N,~N,~N")});s("JU");v(["JU.T4"],"JU.P4",null,function(){c$=D(JU,"P4",JU.T4);c$.new4=c(c$,"new4",function(a,b,d,g){var c=new JU.P4;c.set4(a,b,d,g);return c},"~N,~N,~N,~N");c$.newPt=c(c$,"newPt",function(a){var b=new JU.P4;b.set4(a.x,a.y,a.z,a.w);return b},"JU.P4");c(c$,"distance4",function(a){var b=this.x-a.x,d=this.y-a.y,g=this.z-a.z;a=this.w-a.w;return Math.sqrt(b*b+d*d+g*g+a*a)},"JU.P4")});
s("JU");v(null,"JU.PT","java.lang.Boolean $.Double $.Float $.Number java.util.Map javajs.api.JSONEncodable JU.AU $.DF $.Lst $.M34 $.M4 $.SB".split(" "),function(){c$=D(JU,"PT");c$.parseInt=c(c$,"parseInt",function(a){return JU.PT.parseIntNext(a,B(-1,[0]))},"~S");c$.parseIntNext=c(c$,"parseIntNext",function(a,b){var d=a.length;return 0>b[0]||b[0]>=d?-2147483648:JU.PT.parseIntChecked(a,d,b)},"~S,~A");c$.parseIntChecked=c(c$,"parseIntChecked",function(a,b,d){var g=!1,c=0,e=d[0];if(0>e)return-2147483648;
for(var h;e<b&&JU.PT.isWhiteSpace(a,e);)++e;var k=!1;e<b&&45==a.charCodeAt(e)&&(k=!0,++e);for(;e<b&&48<=(h=a.charCodeAt(e))&&57>=h;)c=10*c+(h-48),g=!0,++e;g?k&&(c=-c):c=-2147483648;d[0]=e;return c},"~S,~N,~A");c$.isWhiteSpace=c(c$,"isWhiteSpace",function(a,b){var d;return 0<=b&&(" "==(d=a.charAt(b))||"\t"==d||"\n"==d)},"~S,~N");c$.parseFloatChecked=c(c$,"parseFloatChecked",function(a,b,d,g){var c=!1,e=d[0];if(g&&a.indexOf("\n")!=a.lastIndexOf("\n"))return NaN;for(;e<b&&JU.PT.isWhiteSpace(a,e);)++e;
var h=!1;e<b&&"-"==a.charAt(e)&&(++e,h=!0);for(var k=0,l=0,j=0;e<b&&48<=(k=a.charCodeAt(e))&&57>=k;)l=10*l+1*(k-48),++e,c=!0;var m=!1,r=0,x=0==l?-1:0;if(46==k)for(m=!0;++e<b&&48<=(k=a.charCodeAt(e))&&57>=k;){c=!0;if(0>x){if(48==k){x--;continue}x=-x}r<JU.PT.decimalScale.length&&(j=10*j+1*(k-48),r++)}c?0<j?(c=j*JU.PT.decimalScale[r-1],c=1<x?x-2<JU.PT.decimalScale.length?c*JU.PT.decimalScale[x-2]:c*Math.pow(10,1-x):c+l):c=l:c=NaN;l=!1;if(e<b&&(69==k||101==k||68==k)){l=!0;if(++e>=b)return NaN;k=a.charCodeAt(e);
if(43==k&&++e>=b)return NaN;d[0]=e;e=JU.PT.parseIntChecked(a,b,d);if(-2147483648==e)return NaN;0<e&&e<=JU.PT.tensScale.length?c*=JU.PT.tensScale[e-1]:0>e&&-e<=JU.PT.decimalScale.length?c*=JU.PT.decimalScale[-e-1]:0!=e&&(c*=Math.pow(10,e))}else d[0]=e;h&&(c=-c);Infinity==c&&(c=3.4028235E38);return!g||(!l||m)&&JU.PT.checkTrailingText(a,d[0],b)?c:NaN},"~S,~N,~A,~B");c$.checkTrailingText=c(c$,"checkTrailingText",function(a,b,d){for(var g;b<d&&(JU.PT.isWhitespace(g=a.charAt(b))||";"==g);)++b;return b==
d},"~S,~N,~N");c$.parseFloatArray=c(c$,"parseFloatArray",function(a){return JU.PT.parseFloatArrayNext(a,B(1,0),null,null,null)},"~S");c$.parseFloatArrayInfested=c(c$,"parseFloatArrayInfested",function(a,b){for(var d=b.length,g=a.length,c=0,e=0,h=0;0<=h&&h<d&&c<g;h++){for(var k;Float.isNaN(k=JU.PT.parseFloat(a[c++]))&&c<g;);Float.isNaN(k)||(b[e=h]=k);if(c==g)break}return e+1},"~A,~A");c$.parseFloatArrayNext=c(c$,"parseFloatArrayNext",function(a,b,d,g,c){var e=0,h=b[0];0<=h&&(null!=g&&(e=a.indexOf(g,
h),0<=e&&(b[0]=e+g.length)),a=a.substring(b[0]),h=null==c?-1:a.indexOf(c),0>h?h=a.length:a=a.substring(0,h),b[0]+=h+1,a=JU.PT.getTokens(a),null==d&&(d=K(a.length,0)),e=JU.PT.parseFloatArrayInfested(a,d));if(null==d)return K(0,0);for(a=e;a<d.length;a++)d[a]=NaN;return d},"~S,~A,~A,~S,~S");c$.parseFloatRange=c(c$,"parseFloatRange",function(a,b,d){var g=a.length;b>g&&(b=g);return 0>d[0]||d[0]>=b?NaN:JU.PT.parseFloatChecked(a,b,d,!1)},"~S,~N,~A");c$.parseFloatNext=c(c$,"parseFloatNext",function(a,b){var d=
null==a?-1:a.length;return 0>b[0]||b[0]>=d?NaN:JU.PT.parseFloatChecked(a,d,b,!1)},"~S,~A");c$.parseFloatStrict=c(c$,"parseFloatStrict",function(a){var b=a.length;return 0==b?NaN:JU.PT.parseFloatChecked(a,b,B(-1,[0]),!0)},"~S");c$.parseFloat=c(c$,"parseFloat",function(a){return JU.PT.parseFloatNext(a,B(-1,[0]))},"~S");c$.parseIntRadix=c(c$,"parseIntRadix",function(a,b){return Integer.parseIntRadix(a,b)},"~S,~N");c$.getTokens=c(c$,"getTokens",function(a){return JU.PT.getTokensAt(a,0)},"~S");c$.parseToken=
c(c$,"parseToken",function(a){return JU.PT.parseTokenNext(a,B(-1,[0]))},"~S");c$.parseTrimmed=c(c$,"parseTrimmed",function(a){return JU.PT.parseTrimmedRange(a,0,a.length)},"~S");c$.parseTrimmedAt=c(c$,"parseTrimmedAt",function(a,b){return JU.PT.parseTrimmedRange(a,b,a.length)},"~S,~N");c$.parseTrimmedRange=c(c$,"parseTrimmedRange",function(a,b,d){var g=a.length;d<g&&(g=d);return g<b?"":JU.PT.parseTrimmedChecked(a,b,g)},"~S,~N,~N");c$.getTokensAt=c(c$,"getTokensAt",function(a,b){if(null==a)return null;
var d=a.length;if(0>b||b>d)return null;var g=JU.PT.countTokens(a,b),c=Array(g),e=B(1,0);e[0]=b;for(var h=0;h<g;++h)c[h]=JU.PT.parseTokenChecked(a,d,e);return c},"~S,~N");c$.countChar=c(c$,"countChar",function(a,b){for(var d=0,g=a.lastIndexOf(b)+1;0<=--g;)a.charAt(g)==b&&d++;return d},"~S,~S");c$.countTokens=c(c$,"countTokens",function(a,b){var d=0;if(null!=a)for(var g=a.length;;){for(;b<g&&JU.PT.isWhiteSpace(a,b);)++b;if(b==g)break;++d;do++b;while(b<g&&!JU.PT.isWhiteSpace(a,b))}return d},"~S,~N");
c$.parseTokenNext=c(c$,"parseTokenNext",function(a,b){var d=a.length;return 0>b[0]||b[0]>=d?null:JU.PT.parseTokenChecked(a,d,b)},"~S,~A");c$.parseTokenRange=c(c$,"parseTokenRange",function(a,b,d){var g=a.length;b>g&&(b=g);return 0>d[0]||d[0]>=b?null:JU.PT.parseTokenChecked(a,b,d)},"~S,~N,~A");c$.parseTokenChecked=c(c$,"parseTokenChecked",function(a,b,d){for(var g=d[0];g<b&&JU.PT.isWhiteSpace(a,g);)++g;for(var c=g;g<b&&!JU.PT.isWhiteSpace(a,g);)++g;d[0]=g;return c==g?null:a.substring(c,g)},"~S,~N,~A");
c$.parseTrimmedChecked=c(c$,"parseTrimmedChecked",function(a,b,d){for(;b<d&&JU.PT.isWhiteSpace(a,b);)++b;for(d-=1;d>=b&&JU.PT.isWhiteSpace(a,d);)--d;return d<b?"":a.substring(b,d+1)},"~S,~N,~N");c$.dVal=c(c$,"dVal",function(a){if(null==a)throw new NumberFormatException("null");var b=parseFloat(a);if(isNaN(b))throw new NumberFormatException("Not a Number : "+a);return b},"~S");c$.fVal=c(c$,"fVal",function(a){return this.dVal(a)},"~S");c$.parseIntRange=c(c$,"parseIntRange",function(a,b,d){var g=a.length;
b>g&&(b=g);return 0>d[0]||d[0]>=b?-2147483648:JU.PT.parseIntChecked(a,b,d)},"~S,~N,~A");c$.parseFloatArrayData=c(c$,"parseFloatArrayData",function(a,b){JU.PT.parseFloatArrayDataN(a,b,b.length)},"~A,~A");c$.parseFloatArrayDataN=c(c$,"parseFloatArrayDataN",function(a,b,d){for(;0<=--d;)b[d]=d>=a.length?NaN:JU.PT.parseFloat(a[d])},"~A,~A,~N");c$.split=c(c$,"split",function(a,b){if(0==a.length)return[];var d=1,g=a.indexOf(b),c,e=b.length;if(0>g||0==e)return c=Array(1),c[0]=a,c;for(var h=a.length-e;0<=
g&&g<h;d++)g=a.indexOf(b,g+e);c=Array(d);for(var k=g=0,l=0;0<=(k=a.indexOf(b,g))&&l+1<d;)c[l++]=a.substring(g,k),g=k+e;a.indexOf(b,h)!=h&&(h+=e);c[l]=a.substring(g,h);return c},"~S,~S");c$.getQuotedStringAt=c(c$,"getQuotedStringAt",function(a,b){var d=B(-1,[b]);return JU.PT.getQuotedStringNext(a,d)},"~S,~N");c$.getQuotedStringNext=c(c$,"getQuotedStringNext",function(a,b){var d=b[0];if(0>d||0>(d=a.indexOf('"',d)))return"";for(var g=d+1,c=a.length;++d<c&&'"'!=a.charAt(d);)"\\"==a.charAt(d)&&d++;b[0]=
d+1;return a.substring(g,d)},"~S,~A");c$.getQuotedOrUnquotedAttribute=c(c$,"getQuotedOrUnquotedAttribute",function(a,b){if(null==a||null==b)return null;var d=a.toLowerCase().indexOf(b.toLowerCase()+"=");if(0>d||(d=d+b.length+1)>=a.length)return"";var g=a.charAt(d);switch(g){case "'":case '"':d++;break;default:g=" ",a+=" "}g=a.indexOf(g,d);return 0>g?null:a.substring(d,g)},"~S,~S");c$.getCSVString=c(c$,"getCSVString",function(a,b){var d=b[1];if(0>d||0>(d=a.indexOf('"',d)))return null;for(var g=b[0]=
d,c=a.length,e=!1,h=!1;++d<c&&('"'!=a.charAt(d)||(e=d+1<c&&'"'==a.charAt(d+1)));)e&&(e=!1,h=!0,d++);if(d>=c)return b[1]=-1,null;b[1]=d+1;d=a.substring(g+1,d);return h?JU.PT.rep(JU.PT.rep(d,'""',"\x00"),"\x00",'"'):d},"~S,~A");c$.isOneOf=c(c$,"isOneOf",function(a,b){if(0==b.length)return!1;";"!=b.charAt(0)&&(b=";"+b+";");return 0>a.indexOf(";")&&0<=b.indexOf(";"+a+";")},"~S,~S");c$.getQuotedAttribute=c(c$,"getQuotedAttribute",function(a,b){var d=a.indexOf(b+"=");return 0>d?null:JU.PT.getQuotedStringAt(a,
d)},"~S,~S");c$.approx=c(c$,"approx",function(a,b){return Math.round(a*b)/b},"~N,~N");c$.rep=c(c$,"rep",function(a,b,d){if(null==a||0==b.length||0>a.indexOf(b))return a;var g=0<=d.indexOf(b);do a=a.$replace(b,d);while(!g&&0<=a.indexOf(b));return a},"~S,~S,~S");c$.formatF=c(c$,"formatF",function(a,b,d,g,c){return JU.PT.formatS(JU.DF.formatDecimal(a,d),b,0,g,c)},"~N,~N,~N,~B,~B");c$.formatD=c(c$,"formatD",function(a,b,d,g,c){return JU.PT.formatS(JU.DF.formatDecimal(a,-1-d),b,0,g,c)},"~N,~N,~N,~B,~B,~B");
c$.formatS=c(c$,"formatS",function(a,b,d,g,c){if(null==a)return"";var e=a.length;2147483647!=d&&0<d&&d<e?a=a.substring(0,d):0>d&&0<=e+d&&(a=a.substring(e+d+1));d=b-a.length;if(0>=d)return a;b=c&&!g&&"-"==a.charAt(0);c=c?"0":" ";var h=b?"-":c,e=new JU.SB;g&&e.append(a);for(e.appendC(h);0<--d;)e.appendC(c);g||e.append(b?c+a.substring(1):a);return e.toString()},"~S,~N,~N,~B,~B");c$.replaceWithCharacter=c(c$,"replaceWithCharacter",function(a,b,d){if(null==a)return null;for(var g=b.length;0<=--g;)a=a.$replace(b.charAt(g),
d);return a},"~S,~S,~S");c$.replaceAllCharacters=c(c$,"replaceAllCharacters",function(a,b,d){for(var g=b.length;0<=--g;){var c=b.substring(g,g+1);a=JU.PT.rep(a,c,d)}return a},"~S,~S,~S");c$.trim=c(c$,"trim",function(a,b){if(null==a||0==a.length)return a;if(0==b.length)return a.trim();for(var d=a.length,g=0;g<d&&0<=b.indexOf(a.charAt(g));)g++;for(d=a.length-1;d>g&&0<=b.indexOf(a.charAt(d));)d--;return a.substring(g,d+1)},"~S,~S");c$.trimQuotes=c(c$,"trimQuotes",function(a){return null!=a&&1<a.length&&
a.startsWith('"')&&a.endsWith('"')?a.substring(1,a.length-1):a},"~S");c$.isNonStringPrimitive=c(c$,"isNonStringPrimitive",function(a){return"number"==typeof a||"boolean"==typeof a?!0:n(a,Number)||n(a,Boolean)},"~O");c$.toJSON=c(c$,"toJSON",function(a,b){if(null==b)return JU.PT.packageJSON(a,null);if(JU.PT.isNonStringPrimitive(b))return JU.PT.packageJSON(a,b.toString());for(var d=null,g=null;;){if(n(b,String)){d=b;"undefined"==typeof d&&(d="null");0!=d.indexOf('{"')&&(d=JU.PT.esc(d));break}if(n(b,
javajs.api.JSONEncodable)){if(null==(d=b.toJSON()))d="null";break}g=new JU.SB;if(n(b,java.util.Map)){g.append("{ ");for(var c="",e,h=b.keySet().iterator();h.hasNext()&&((e=h.next())||1);)null==e&&(e="null"),g.append(c).append(JU.PT.packageJSON(e,JU.PT.toJSON(null,b.get(e)))),c=",";g.append(" }");break}if(n(b,JU.Lst)){g.append("[ ");e=b.size();for(c=0;c<e;c++)0<c&&g.appendC(","),g.append(JU.PT.toJSON(null,b.get(c)));g.append(" ]");break}if(n(b,JU.M34)){e=n(b,JU.M4)?4:3;var h=K(e,0),k=b;g.appendC("[");
for(c=0;c<e;c++)0<c&&g.appendC(","),k.getRow(c,h),g.append(JU.PT.toJSON(null,h));g.appendC("]");break}d=JU.PT.nonArrayString(b);if(null==d){g.append("[");e=JU.AU.getLength(b);c=null;c=b[0];"number"!=typeof c&&(c=null);if(null!=c)g.appendO(b);else for(c=0;c<e;c++)0<c&&g.appendC(","),g.append(JU.PT.toJSON(null,java.lang.reflect.Array.get(b,c)));g.append("]");break}b=b.toString()}return JU.PT.packageJSON(a,null==d?g.toString():d)},"~S,~O");c$.nonArrayString=c(c$,"nonArrayString",function(a){return a.constructor==
Array||a.BYTES_PER_ELEMENT?null:a.toString()},"~O");c$.byteArrayToJSON=c(c$,"byteArrayToJSON",function(a){var b=new JU.SB;b.append("[");for(var d=a.length,g=0;g<d;g++)0<g&&b.appendC(","),b.appendI(a[g]&255);b.append("]");return b.toString()},"~A");c$.packageJSON=c(c$,"packageJSON",function(a,b){return null==a?b:'"'+a+'": '+b},"~S,~S");c$.escapeUrl=c(c$,"escapeUrl",function(a){a=JU.PT.rep(a,"\n","");a=JU.PT.rep(a,"%","%25");a=JU.PT.rep(a,"#","%23");a=JU.PT.rep(a,"[","%5B");a=JU.PT.rep(a,"\\","%5C");
a=JU.PT.rep(a,"]","%5D");return a=JU.PT.rep(a," ","%20")},"~S");c$.esc=c(c$,"esc",function(a){if(null==a||0==a.length)return'""';for(var b=!1,d=0;10>d;d+=2)if(0<=a.indexOf('\\\\\tt\rr\nn""'.charAt(d))){b=!0;break}if(b)for(;10>d;){for(var b=-1,g='\\\\\tt\rr\nn""'.charAt(d++),c='\\\\\tt\rr\nn""'.charAt(d++),e=new JU.SB,h=0;0<=(b=a.indexOf(g,b+1));)e.append(a.substring(h,b)).appendC("\\").appendC(c),h=b+1;e.append(a.substring(h,a.length));a=e.toString()}return'"'+JU.PT.escUnicode(a)+'"'},"~S");c$.escUnicode=
c(c$,"escUnicode",function(a){for(var b=a.length;0<=--b;)if(127<a.charCodeAt(b)){var d="0000"+Integer.toHexString(a.charCodeAt(b));a=a.substring(0,b)+"\\u"+d.substring(d.length-4)+a.substring(b+1)}return a},"~S");c$.escF=c(c$,"escF",function(a){a=""+a;0>a.indexOf(".")&&0>a.indexOf("e")&&(a+=".0");return a},"~N");c$.join=c(c$,"join",function(a,b,d){if(a.length<d)return null;var g=new JU.SB;for(g.append(a[d++]);d<a.length;d++)g.appendC(b).append(a[d]);return g.toString()},"~A,~S,~N");c$.isLike=c(c$,
"isLike",function(a,b){var d=a.equals(b);if(d)return!0;var g=b.startsWith("*"),c=b.endsWith("*");return!g&&!c?d:g&&c?1==b.length||a.contains(b.substring(1,b.length-1)):g?a.endsWith(b.substring(1)):a.startsWith(b.substring(0,b.length-1))},"~S,~S");c$.getMapValueNoCase=c(c$,"getMapValueNoCase",function(a,b){if("this".equals(b))return a;var d=a.get(b);if(null==d)for(var g,c=a.entrySet().iterator();c.hasNext()&&((g=c.next())||1);)if(g.getKey().equalsIgnoreCase(b))return g.getValue();return d},"java.util.Map,~S");
c$.clean=c(c$,"clean",function(a){return JU.PT.rep(JU.PT.replaceAllCharacters(a," \t\n\r"," "),"  "," ").trim()},"~S");c$.fdup=c(c$,"fdup",function(a,b,d){for(var g,c=0,e=b;1<=--e;)if(!JU.PT.isDigit(g=a.charAt(e)))switch(g){case ".":if(0!=c++)return a;continue;case "-":if(1!=e&&"."!=a.charAt(e-1))return a;continue;default:return a}g=a.substring(0,b+1);c=new JU.SB;for(e=0;e<d;e++)c.append(g);c.append(a.substring(b+1));return c.toString()},"~S,~N,~N");c$.formatString=c(c$,"formatString",function(a,
b,d,g,c,e){if(null==a)return null;if("".equals(a))return"";var h=b.length;if(0>a.indexOf("%")||0==h||0>a.indexOf(b))return a;var k="",l,j,m;for(l=0;0<=(j=a.indexOf("%",l))&&0<=(m=a.indexOf(b,j+1));)if(l!=j&&(k+=a.substring(l,j)),l=j+1,m>j+6)k+="%";else try{var r=!1;"-"==a.charAt(l)&&(r=!0,++l);var x=!1;"0"==a.charAt(l)&&(x=!0,++l);for(var y,F=0;"0"<=(y=a.charAt(l))&&"9">=y;)F=10*F+(y.charCodeAt(0)-48),++l;var C=2147483647,n=!1;if("."==a.charAt(l)){++l;if("-"==(y=a.charAt(l)))n=null==d,++l;if("0"<=
(y=a.charAt(l))&&"9">=y)C=y.charCodeAt(0)-48,++l;n&&(C=-C)}if(a.substring(l,l+h).equals(b)){if(l+=h,Float.isNaN(g)?null!=d?k+=JU.PT.formatS(d,F,C,r,x):Double.isNaN(c)||(k+=JU.PT.formatD(c,F,C-1,r,x,!0)):k+=JU.PT.formatF(g,F,C,r,x),e)break}else l=j+1,k+="%"}catch(p){if(H(p,IndexOutOfBoundsException)){l=j;break}else throw p;}return k+=a.substring(l)},"~S,~S,~S,~N,~N,~B");c$.formatStringS=c(c$,"formatStringS",function(a,b,d){return JU.PT.formatString(a,b,d,NaN,NaN,!1)},"~S,~S,~S");c$.formatStringF=c(c$,
"formatStringF",function(a,b,d){return JU.PT.formatString(a,b,null,d,NaN,!1)},"~S,~S,~N");c$.formatStringI=c(c$,"formatStringI",function(a,b,d){return JU.PT.formatString(a,b,""+d,NaN,NaN,!1)},"~S,~S,~N");c$.sprintf=c(c$,"sprintf",function(a,b,d){if(null==d)return a;var g=b.length;if(g==d.length)try{for(var c=0;c<g;c++)if(null!=d[c])switch(b.charAt(c)){case "s":a=JU.PT.formatString(a,"s",d[c],NaN,NaN,!0);break;case "f":a=JU.PT.formatString(a,"f",null,d[c].floatValue(),NaN,!0);break;case "i":a=JU.PT.formatString(a,
"d",""+d[c],NaN,NaN,!0);a=JU.PT.formatString(a,"i",""+d[c],NaN,NaN,!0);break;case "d":a=JU.PT.formatString(a,"e",null,NaN,d[c].doubleValue(),!0);break;case "p":var e=d[c];a=JU.PT.formatString(a,"p",null,e.x,NaN,!0);a=JU.PT.formatString(a,"p",null,e.y,NaN,!0);a=JU.PT.formatString(a,"p",null,e.z,NaN,!0);break;case "q":var h=d[c];a=JU.PT.formatString(a,"q",null,h.x,NaN,!0);a=JU.PT.formatString(a,"q",null,h.y,NaN,!0);a=JU.PT.formatString(a,"q",null,h.z,NaN,!0);a=JU.PT.formatString(a,"q",null,h.w,NaN,
!0);break;case "S":for(var k=d[c],l=0;l<k.length;l++)a=JU.PT.formatString(a,"s",k[l],NaN,NaN,!0);break;case "F":for(var j=d[c],l=0;l<j.length;l++)a=JU.PT.formatString(a,"f",null,j[l],NaN,!0);break;case "I":for(var m=d[c],l=0;l<m.length;l++)a=JU.PT.formatString(a,"d",""+m[l],NaN,NaN,!0);for(l=0;l<m.length;l++)a=JU.PT.formatString(a,"i",""+m[l],NaN,NaN,!0);break;case "D":for(var r=d[c],l=0;l<r.length;l++)a=JU.PT.formatString(a,"e",null,NaN,r[l],!0)}return JU.PT.rep(a,"%%","%")}catch(x){if(!H(x,Exception))throw x;
}Zystem.out.println("TextFormat.sprintf error "+b+" "+a);return JU.PT.rep(a,"%","?")},"~S,~S,~A");c$.formatCheck=c(c$,"formatCheck",function(a){if(null==a||0>a.indexOf("p")&&0>a.indexOf("q"))return a;a=JU.PT.rep(a,"%%","\u0001");a=JU.PT.rep(a,"%p","%6.2p");a=JU.PT.rep(a,"%q","%6.2q");a=JU.PT.split(a,"%");var b=new JU.SB;b.append(a[0]);for(var d=1;d<a.length;d++){var g="%"+a[d],c;if(3<=g.length){if(0<=(c=g.indexOf("p")))g=JU.PT.fdup(g,c,3);if(0<=(c=g.indexOf("q")))g=JU.PT.fdup(g,c,4)}b.append(g)}return b.toString().$replace("\u0001",
"%")},"~S");c$.leftJustify=c(c$,"leftJustify",function(a,b,d){a.append(d);d=b.length-d.length;0<d&&a.append(b.substring(0,d))},"JU.SB,~S,~S");c$.rightJustify=c(c$,"rightJustify",function(a,b,d){var g=b.length-d.length;0<g&&a.append(b.substring(0,g));a.append(d)},"JU.SB,~S,~S");c$.safeTruncate=c(c$,"safeTruncate",function(a,b){-0.001<a&&0.001>a&&(a=0);return(a+"         ").substring(0,b)},"~N,~N");c$.isWild=c(c$,"isWild",function(a){return null!=a&&(0<=a.indexOf("*")||0<=a.indexOf("?"))},"~S");c$.isMatch=
c(c$,"isMatch",function(a,b,d,g){if(a.equals(b))return!0;var c=b.length;if(0==c)return!1;var e=d&&g?"*"==b.charAt(0):!1;if(1==c&&e)return!0;var h=d&&b.endsWith("*");if(!(0<=b.indexOf("?"))){if(e)return h?3>c||0<=a.indexOf(b.substring(1,c-1)):a.endsWith(b.substring(1));if(h)return a.startsWith(b.substring(0,c-1))}for(var k=a.length,l="????",j=4;j<k;)l+=l,j+=4;d&&(e&&(b=l+b.substring(1),c+=j-1),h&&(b=b.substring(0,c-1)+l,c+=j-1));if(c<k)return!1;for(d=0;c>k;){if(g&&"?"==b.charAt(d))++d;else if("?"!=
b.charAt(d+c-1))return!1;--c}for(g=k;0<=--g;)if(c=b.charAt(d+g),"?"!=c&&(k=a.charAt(g),c!=k&&("\u0001"!=c||"?"!=k)))return!1;return!0},"~S,~S,~B,~B");c$.replaceQuotedStrings=c(c$,"replaceQuotedStrings",function(a,b,d){for(var g=b.size(),c=0;c<g;c++){var e=b.get(c),h=d.get(c);h.equals(e)||(a=JU.PT.rep(a,'"'+e+'"','"'+h+'"'))}return a},"~S,JU.Lst,JU.Lst");c$.replaceStrings=c(c$,"replaceStrings",function(a,b,d){for(var g=b.size(),c=0;c<g;c++){var e=b.get(c),h=d.get(c);h.equals(e)||(a=JU.PT.rep(a,e,h))}return a},
"~S,JU.Lst,JU.Lst");c$.isDigit=c(c$,"isDigit",function(a){a=a.charCodeAt(0);return 48<=a&&57>=a},"~S");c$.isUpperCase=c(c$,"isUpperCase",function(a){a=a.charCodeAt(0);return 65<=a&&90>=a},"~S");c$.isLowerCase=c(c$,"isLowerCase",function(a){a=a.charCodeAt(0);return 97<=a&&122>=a},"~S");c$.isLetter=c(c$,"isLetter",function(a){a=a.charCodeAt(0);return 65<=a&&90>=a||97<=a&&122>=a},"~S");c$.isLetterOrDigit=c(c$,"isLetterOrDigit",function(a){a=a.charCodeAt(0);return 65<=a&&90>=a||97<=a&&122>=a||48<=a&&
57>=a},"~S");c$.isWhitespace=c(c$,"isWhitespace",function(a){a=a.charCodeAt(0);return 28<=a&&32>=a||9<=a&&13>=a},"~S");c$.fixPtFloats=c(c$,"fixPtFloats",function(a,b){a.x=Math.round(a.x*b)/b;a.y=Math.round(a.y*b)/b;a.z=Math.round(a.z*b)/b},"JU.T3,~N");c$.fixDouble=c(c$,"fixDouble",function(a,b){return Math.round(a*b)/b},"~N,~N");c$.parseFloatFraction=c(c$,"parseFloatFraction",function(a){var b=a.indexOf("/");return 0>b?JU.PT.parseFloat(a):JU.PT.parseFloat(a.substring(0,b))/JU.PT.parseFloat(a.substring(b+
1))},"~S");E(c$,"tensScale",K(-1,[10,100,1E3,1E4,1E5,1E6]),"decimalScale",K(-1,[0.1,0.01,0.001,1E-4,1E-5,1E-6,1E-7,1E-8,1E-9]),"FLOAT_MIN_SAFE",2E-45,"escapable",'\\\\\tt\rr\nn""',"FRACTIONAL_PRECISION",1E5,"CARTESIAN_PRECISION",1E4)});s("JU");c$=t(function(){this.s=this.sb=null;p(this,arguments)},JU,"SB");q(c$,function(){this.s=""});c$.newN=c(c$,"newN",function(){return new JU.SB},"~N");c$.newS=c(c$,"newS",function(a){var b=new JU.SB;b.s=a;return b},"~S");c(c$,"append",function(a){this.s+=a;return this},
"~S");c(c$,"appendC",function(a){this.s+=a;return this},"~S");c(c$,"appendI",function(a){this.s+=a;return this},"~N");c(c$,"appendB",function(a){this.s+=a;return this},"~B");c(c$,"appendF",function(a){a=""+a;0>a.indexOf(".")&&0>a.indexOf("e")&&(a+=".0");this.s+=a;return this},"~N");c(c$,"appendD",function(a){a=""+a;0>a.indexOf(".")&&0>a.indexOf("e")&&(a+=".0");this.s+=a;return this},"~N");c(c$,"appendSB",function(a){this.s+=a.s;return this},"JU.SB");c(c$,"appendO",function(a){null!=a&&(this.s+=a.toString());
return this},"~O");c(c$,"appendCB",function(a,b,d){this.s+=a.slice(b,b+d).join("")},"~A,~N,~N");j(c$,"toString",function(){return this.s});c(c$,"length",function(){return this.s.length});c(c$,"indexOf",function(a){return this.s.indexOf(a)},"~S");c(c$,"charAt",function(a){return this.s.charAt(a)},"~N");c(c$,"charCodeAt",function(a){return this.s.charCodeAt(a)},"~N");c(c$,"setLength",function(a){this.s=this.s.substring(0,a)},"~N");c(c$,"lastIndexOf",function(a){return this.s.lastIndexOf(a)},"~S");c(c$,
"indexOf2",function(a,b){return this.s.indexOf(a,b)},"~S,~N");c(c$,"substring",function(a){return this.s.substring(a)},"~N");c(c$,"substring2",function(a,b){return this.s.substring(a,b)},"~N,~N");c(c$,"toBytes",function(a,b){return 0==b?Q(0,0):(0<b?this.substring2(a,a+b):0==a?this.toString():this.substring2(a,this.length()-a)).getBytes("UTF-8")},"~N,~N");c(c$,"replace",function(a,b,d){this.s=this.s.substring(0,a)+d+this.s.substring(b)},"~N,~N,~S");c(c$,"insert",function(a,b){this.replace(a,a,b)},
"~N,~S");s("JU");v(["javajs.api.JSONEncodable"],"JU.T3",["java.lang.Float"],function(){c$=t(function(){this.z=this.y=this.x=0;p(this,arguments)},JU,"T3",null,[javajs.api.JSONEncodable,java.io.Serializable]);c(c$,"set",function(a,b,d){this.x=a;this.y=b;this.z=d},"~N,~N,~N");c(c$,"setA",function(a){this.x=a[0];this.y=a[1];this.z=a[2]},"~A");c(c$,"setT",function(a){this.x=a.x;this.y=a.y;this.z=a.z},"JU.T3");c(c$,"add2",function(a,b){this.x=a.x+b.x;this.y=a.y+b.y;this.z=a.z+b.z},"JU.T3,JU.T3");c(c$,"add",
function(a){this.x+=a.x;this.y+=a.y;this.z+=a.z},"JU.T3");c(c$,"distanceSquared",function(a){var b=this.x-a.x,d=this.y-a.y;a=this.z-a.z;return b*b+d*d+a*a},"JU.T3");c(c$,"distance",function(a){return Math.sqrt(this.distanceSquared(a))},"JU.T3");c(c$,"sub2",function(a,b){this.x=a.x-b.x;this.y=a.y-b.y;this.z=a.z-b.z},"JU.T3,JU.T3");c(c$,"sub",function(a){this.x-=a.x;this.y-=a.y;this.z-=a.z},"JU.T3");c(c$,"scale",function(a){this.x*=a;this.y*=a;this.z*=a},"~N");c(c$,"add3",function(a,b,d){this.x+=a;
this.y+=b;this.z+=d},"~N,~N,~N");c(c$,"scaleT",function(a){this.x*=a.x;this.y*=a.y;this.z*=a.z},"JU.T3");c(c$,"scaleAdd2",function(a,b,d){this.x=a*b.x+d.x;this.y=a*b.y+d.y;this.z=a*b.z+d.z},"~N,JU.T3,JU.T3");c(c$,"ave",function(a,b){this.x=(a.x+b.x)/2;this.y=(a.y+b.y)/2;this.z=(a.z+b.z)/2},"JU.T3,JU.T3");c(c$,"dot",function(a){return this.x*a.x+this.y*a.y+this.z*a.z},"JU.T3");c(c$,"lengthSquared",function(){return this.x*this.x+this.y*this.y+this.z*this.z});c(c$,"length",function(){return Math.sqrt(this.lengthSquared())});
c(c$,"normalize",function(){var a=this.length();this.x/=a;this.y/=a;this.z/=a});c(c$,"cross",function(a,b){this.set(a.y*b.z-a.z*b.y,a.z*b.x-a.x*b.z,a.x*b.y-a.y*b.x)},"JU.T3,JU.T3");j(c$,"hashCode",function(){var a;a=31+JU.T3.floatToIntBits(this.x);a=31*a+JU.T3.floatToIntBits(this.y);a=31*a+JU.T3.floatToIntBits(this.z);return a^a>>32});c$.floatToIntBits=c(c$,"floatToIntBits",function(a){return 0==a?0:Float.floatToIntBits(a)},"~N");j(c$,"equals",function(a){return!n(a,JU.T3)?!1:this.x==a.x&&this.y==
a.y&&this.z==a.z},"~O");j(c$,"toString",function(){return"{"+this.x+", "+this.y+", "+this.z+"}"});j(c$,"toJSON",function(){return"["+this.x+","+this.y+","+this.z+"]"})});s("JU");c$=t(function(){this.z=this.y=this.x=0;p(this,arguments)},JU,"T3i",null,java.io.Serializable);q(c$,function(){});c(c$,"set",function(a,b,d){this.x=a;this.y=b;this.z=d},"~N,~N,~N");c(c$,"setT",function(a){this.x=a.x;this.y=a.y;this.z=a.z},"JU.T3i");c(c$,"add",function(a){this.x+=a.x;this.y+=a.y;this.z+=a.z},"JU.T3i");c(c$,
"scaleAdd",function(a,b,d){this.x=a*b.x+d.x;this.y=a*b.y+d.y;this.z=a*b.z+d.z},"~N,JU.T3i,JU.T3i");j(c$,"hashCode",function(){return this.x^this.y^this.z});j(c$,"equals",function(a){return!n(a,JU.T3i)?!1:this.x==a.x&&this.y==a.y&&this.z==a.z},"~O");j(c$,"toString",function(){return"("+this.x+", "+this.y+", "+this.z+")"});s("JU");v(["JU.T3"],"JU.T4",null,function(){c$=t(function(){this.w=0;p(this,arguments)},JU,"T4",JU.T3);c(c$,"set4",function(a,b,d,g){this.x=a;this.y=b;this.z=d;this.w=g},"~N,~N,~N,~N");
c(c$,"scale4",function(a){this.scale(a);this.w*=a},"~N");j(c$,"hashCode",function(){return JU.T3.floatToIntBits(this.x)^JU.T3.floatToIntBits(this.y)^JU.T3.floatToIntBits(this.z)^JU.T3.floatToIntBits(this.w)});j(c$,"equals",function(a){return!n(a,JU.T4)?!1:this.x==a.x&&this.y==a.y&&this.z==a.z&&this.w==a.w},"~O");j(c$,"toString",function(){return"("+this.x+", "+this.y+", "+this.z+", "+this.w+")"});j(c$,"toJSON",function(){return"["+this.x+", "+this.y+", "+this.z+", "+this.w+"]"})});s("JU");v(["JU.T3"],
"JU.V3",null,function(){c$=D(JU,"V3",JU.T3);q(c$,function(){});c$.newV=c(c$,"newV",function(a){return JU.V3.new3(a.x,a.y,a.z)},"JU.T3");c$.newVsub=c(c$,"newVsub",function(a,b){return JU.V3.new3(a.x-b.x,a.y-b.y,a.z-b.z)},"JU.T3,JU.T3");c$.new3=c(c$,"new3",function(a,b,d){var g=new JU.V3;g.x=a;g.y=b;g.z=d;return g},"~N,~N,~N");c(c$,"angle",function(a){var b=this.y*a.z-this.z*a.y,d=this.z*a.x-this.x*a.z,g=this.x*a.y-this.y*a.x,b=Math.sqrt(b*b+d*d+g*g);return Math.abs(Math.atan2(b,this.dot(a)))},"JU.V3")});
s("javajs.api");v(["javajs.api.GenericBinaryDocumentReader"],"javajs.api.GenericBinaryDocument",null,function(){N(javajs.api,"GenericBinaryDocument",javajs.api.GenericBinaryDocumentReader)});s("javajs.api");N(javajs.api,"GenericBinaryDocumentReader");s("javajs.api");N(javajs.api,"GenericZipTools");s("javajs.api");N(javajs.api,"GenericLineReader");s("javajs.api");c$=N(javajs.api,"GenericCifDataParser");E(c$,"NONE",-1);s("javajs.api");c$=D(javajs.api,"Interface");c$.getInterface=c(c$,"getInterface",
function(a){try{var b=ea._4Name(a);return null==b?null:b.newInstance()}catch(d){if(H(d,Exception))return Zystem.out.println("Interface.java Error creating instance for "+a+": \n"+d),null;throw d;}},"~S");s("JU");c$=t(function(){this.data=null;p(this,arguments)},JU,"BArray");q(c$,function(a){this.data=a},"~A");j(c$,"equals",function(a){if(n(a,JU.BArray)&&(a=a.data,a.length==this.data.length)){for(var b=0;b<a.length;b++)if(a[b]!=this.data[b])return!1;return!0}return!1},"~O");c(c$,"hashCode",function(){return this.data.hashCode()});
j(c$,"toString",function(){return String.instantialize(this.data)});s("JU");c$=t(function(){this.buf=null;this.ichCurrent=this.cchBuf=0;p(this,arguments)},JU,"LimitedLineReader");q(c$,function(a,b){a.mark(b+1);this.buf=ca(b,"\x00");this.cchBuf=Math.max(a.read(this.buf,0,b),0);this.ichCurrent=0;a.reset()},"java.io.BufferedReader,~N");c(c$,"getHeader",function(a){return 0==a?String.instantialize(this.buf):String.instantialize(this.buf,0,Math.min(this.cchBuf,a))},"~N");c(c$,"readLineWithNewline",function(){for(;this.ichCurrent<
this.cchBuf;){for(var a=this.ichCurrent,b=String.fromCharCode(0);this.ichCurrent<this.cchBuf&&"\r"!=(b=this.buf[this.ichCurrent++])&&"\n"!=b;);"\r"==b&&(this.ichCurrent<this.cchBuf&&"\n"==this.buf[this.ichCurrent])&&++this.ichCurrent;b=this.ichCurrent-a;if("#"!=this.buf[a])return String.instantialize(this.buf,a,b)}return""});s("JU");c$=t(function(){this.a=null;this.n=this.m=0;R("JU.Matrix.LUDecomp")||JU.Matrix.$Matrix$LUDecomp$();p(this,arguments)},JU,"Matrix",null,Cloneable);q(c$,function(a,b,d){this.a=
null==a?X(b,d,0):a;this.m=b;this.n=d},"~A,~N,~N");c(c$,"getRowDimension",function(){return this.m});c(c$,"getColumnDimension",function(){return this.n});c(c$,"getArray",function(){return this.a});c(c$,"getArrayCopy",function(){for(var a=X(this.m,this.n,0),b=this.m;0<=--b;)for(var d=this.n;0<=--d;)a[b][d]=this.a[b][d];return a});c(c$,"copy",function(){for(var a=new JU.Matrix(null,this.m,this.n),b=a.a,d=this.m;0<=--d;)for(var g=this.n;0<=--g;)b[d][g]=this.a[d][g];return a});j(c$,"clone",function(){return this.copy()});
c(c$,"getSubmatrix",function(a,b,d,g){for(var c=new JU.Matrix(null,d,g),e=c.a;0<=--d;)for(var h=g;0<=--h;)e[d][h]=this.a[a+d][b+h];return c},"~N,~N,~N,~N");c(c$,"getMatrixSelected",function(a,b){for(var d=new JU.Matrix(null,a.length,b),g=d.a,c=a.length;0<=--c;)for(var e=this.a[a[c]],h=b;0<=--h;)g[c][h]=e[h];return d},"~A,~N");c(c$,"transpose",function(){for(var a=new JU.Matrix(null,this.n,this.m),b=a.a,d=this.m;0<=--d;)for(var g=this.n;0<=--g;)b[g][d]=this.a[d][g];return a});c(c$,"add",function(a){return this.scaleAdd(a,
1)},"JU.Matrix");c(c$,"sub",function(a){return this.scaleAdd(a,-1)},"JU.Matrix");c(c$,"scaleAdd",function(a,b){for(var d=new JU.Matrix(null,this.m,this.n),g=d.a,c=a.a,e=this.m;0<=--e;)for(var h=this.n;0<=--h;)g[e][h]=c[e][h]*b+this.a[e][h];return d},"JU.Matrix,~N");c(c$,"mul",function(a){if(a.m!=this.n)return null;var b=new JU.Matrix(null,this.m,a.n),d=b.a,g=a.a;for(a=a.n;0<=--a;)for(var c=this.m;0<=--c;){for(var e=this.a[c],h=0,k=this.n;0<=--k;)h+=e[k]*g[k][a];d[c][a]=h}return b},"JU.Matrix");c(c$,
"inverse",function(){return P(JU.Matrix.LUDecomp,this,null,this.m,this.n).solve(JU.Matrix.identity(this.m,this.m),this.n)});c(c$,"trace",function(){for(var a=0,b=Math.min(this.m,this.n);0<=--b;)a+=this.a[b][b];return a});c$.identity=c(c$,"identity",function(a,b){for(var d=new JU.Matrix(null,a,b),g=d.a,c=Math.min(a,b);0<=--c;)g[c][c]=1;return d},"~N,~N");c(c$,"getRotation",function(){return this.getSubmatrix(0,0,this.m-1,this.n-1)});c(c$,"getTranslation",function(){return this.getSubmatrix(0,this.n-
1,this.m-1,1)});c$.newT=c(c$,"newT",function(a,b){return b?new JU.Matrix(u(-1,[X(-1,[a.x]),X(-1,[a.y]),X(-1,[a.z])]),3,1):new JU.Matrix(u(-1,[X(-1,[a.x,a.y,a.z])]),1,3)},"JU.T3,~B");j(c$,"toString",function(){for(var a="[\n",b=0;b<this.m;b++){for(var a=a+"  [",d=0;d<this.n;d++)a+=" "+this.a[b][d];a+="]\n"}return a+"]"});c$.$Matrix$LUDecomp$=function(){M(self.c$);c$=t(function(){V(this,arguments);this.piv=this.LU=null;this.pivsign=0;p(this,arguments)},JU.Matrix,"LUDecomp");q(c$,function(a,b){this.LU=
this.b$["JU.Matrix"].getArrayCopy();this.piv=B(a,0);for(var d=a;0<=--d;)this.piv[d]=d;this.pivsign=1;for(var g,d=X(a,0),c=0;c<b;c++){for(var e=a;0<=--e;)d[e]=this.LU[e][c];for(var h=a;0<=--h;){g=this.LU[h];for(var e=Math.min(h,c),k=0,l=e;0<=--l;)k+=g[l]*d[l];g[c]=d[h]-=k}e=c;for(k=a;--k>c;)Math.abs(d[k])>Math.abs(d[e])&&(e=k);if(e!=c){for(l=b;0<=--l;)g=this.LU[e][l],this.LU[e][l]=this.LU[c][l],this.LU[c][l]=g;g=this.piv[e];this.piv[e]=this.piv[c];this.piv[c]=g;this.pivsign=-this.pivsign}if((new Boolean(c<
a&0!=this.LU[c][c])).valueOf())for(l=a;--l>c;)this.LU[l][c]/=this.LU[c][c]}},"~N,~N");c(c$,"solve",function(a,b){for(var d=0;d<b;d++)if(0==this.LU[d][d])return null;for(var d=a.n,g=a.getMatrixSelected(this.piv,d),c=g.a,e=0;e<b;e++)for(var h=e+1;h<b;h++)for(var k=0;k<d;k++)c[h][k]-=c[e][k]*this.LU[h][e];for(e=b;0<=--e;){for(h=d;0<=--h;)c[e][h]/=this.LU[e][e];for(h=e;0<=--h;)for(k=d;0<=--k;)c[h][k]-=c[e][k]*this.LU[h][e]}return g},"JU.Matrix,~N");c$=L()};s("JU");v(["JU.P4"],"JU.Quat",["java.lang.Float",
"JU.A4","$.M3","$.V3"],function(){c$=t(function(){this.q3=this.q2=this.q1=this.q0=0;this.mat=null;p(this,arguments)},JU,"Quat");q(c$,function(){this.q0=1});c$.newQ=c(c$,"newQ",function(a){var b=new JU.Quat;b.set(a);return b},"JU.Quat");c$.newVA=c(c$,"newVA",function(a,b){var d=new JU.Quat;d.setTA(a,b);return d},"JU.T3,~N");c$.newM=c(c$,"newM",function(a){var b=new JU.Quat;b.setM(JU.M3.newM3(a));return b},"JU.M3");c$.newAA=c(c$,"newAA",function(a){var b=new JU.Quat;b.setAA(a);return b},"JU.A4");c$.newP4=
c(c$,"newP4",function(a){var b=new JU.Quat;b.setP4(a);return b},"JU.P4");c$.new4=c(c$,"new4",function(a,b,d,g){var c=new JU.Quat;if(-1>g)return c.q0=-1,c;if(1<g)return c.q0=1,c;c.q0=g;c.q1=a;c.q2=b;c.q3=d;return c},"~N,~N,~N,~N");c(c$,"set",function(a){this.q0=a.q0;this.q1=a.q1;this.q2=a.q2;this.q3=a.q3},"JU.Quat");c(c$,"setP4",function(a){var b=null==a?0:a.distance4(JU.Quat.qZero);0==b?this.q0=1:(this.q0=a.w/b,this.q1=a.x/b,this.q2=a.y/b,this.q3=a.z/b)},"JU.P4");c(c$,"setTA",function(a,b){if(0==
a.x&&0==a.y&&0==a.z)this.q0=1;else{var d=Math.sin(0.017453292519943295*(b/2))/Math.sqrt(a.x*a.x+a.y*a.y+a.z*a.z);this.q0=Math.cos(0.017453292519943295*(b/2));this.q1=a.x*d;this.q2=a.y*d;this.q3=a.z*d}},"JU.T3,~N");c(c$,"setAA",function(a){a=JU.A4.newAA(a);0==a.angle&&(a.y=1);this.setM((new JU.M3).setAA(a))},"JU.A4");c(c$,"setM",function(a){this.mat=a;var b=a.m00+a.m11+a.m22,d,g,c;0.5<=b?(b=Math.sqrt(1+b),d=(a.m21-a.m12)/b,g=(a.m02-a.m20)/b,c=(a.m10-a.m01)/b):0.5<=(d=a.m00+a.m00-b)?(d=Math.sqrt(1+
d),b=(a.m21-a.m12)/d,g=(a.m10+a.m01)/d,c=(a.m20+a.m02)/d):0.5<=(d=a.m11+a.m11-b)||a.m11>a.m22?(g=Math.sqrt(1+d),b=(a.m02-a.m20)/g,d=(a.m10+a.m01)/g,c=(a.m21+a.m12)/g):(c=Math.sqrt(1+a.m22+a.m22-b),b=(a.m10-a.m01)/c,d=(a.m20+a.m02)/c,g=(a.m21+a.m12)/c);this.q0=0.5*b;this.q1=0.5*d;this.q2=0.5*g;this.q3=0.5*c},"JU.M3");c(c$,"setRef",function(a){null==a?this.mul(this.getFixFactor()):0<=this.dot(a)||(this.q0*=-1,this.q1*=-1,this.q2*=-1,this.q3*=-1)},"JU.Quat");c$.getQuaternionFrame=c(c$,"getQuaternionFrame",
function(a,b,d){b=JU.V3.newV(b);d=JU.V3.newV(d);null!=a&&(b.sub(a),d.sub(a));return JU.Quat.getQuaternionFrameV(b,d,null,!1)},"JU.P3,JU.T3,JU.T3");c$.getQuaternionFrameV=c(c$,"getQuaternionFrameV",function(a,b,d,g){null==d&&(d=new JU.V3,d.cross(a,b),g&&a.cross(b,d));b=new JU.V3;b.cross(d,a);a.normalize();b.normalize();d.normalize();g=new JU.M3;g.setColumnV(0,a);g.setColumnV(1,b);g.setColumnV(2,d);return JU.Quat.newM(g)},"JU.V3,JU.V3,JU.V3,~B");c(c$,"getMatrix",function(){null==this.mat&&this.setMatrix();
return this.mat});c(c$,"setMatrix",function(){this.mat=new JU.M3;this.mat.m00=this.q0*this.q0+this.q1*this.q1-this.q2*this.q2-this.q3*this.q3;this.mat.m01=2*this.q1*this.q2-2*this.q0*this.q3;this.mat.m02=2*this.q1*this.q3+2*this.q0*this.q2;this.mat.m10=2*this.q1*this.q2+2*this.q0*this.q3;this.mat.m11=this.q0*this.q0-this.q1*this.q1+this.q2*this.q2-this.q3*this.q3;this.mat.m12=2*this.q2*this.q3-2*this.q0*this.q1;this.mat.m20=2*this.q1*this.q3-2*this.q0*this.q2;this.mat.m21=2*this.q2*this.q3+2*this.q0*
this.q1;this.mat.m22=this.q0*this.q0-this.q1*this.q1-this.q2*this.q2+this.q3*this.q3});c(c$,"add",function(a){return JU.Quat.newVA(this.getNormal(),this.getTheta()+a)},"~N");c(c$,"mul",function(a){return 1==a?JU.Quat.new4(this.q1,this.q2,this.q3,this.q0):JU.Quat.newVA(this.getNormal(),this.getTheta()*a)},"~N");c(c$,"mulQ",function(a){return JU.Quat.new4(this.q0*a.q1+this.q1*a.q0+this.q2*a.q3-this.q3*a.q2,this.q0*a.q2+this.q2*a.q0+this.q3*a.q1-this.q1*a.q3,this.q0*a.q3+this.q3*a.q0+this.q1*a.q2-this.q2*
a.q1,this.q0*a.q0-this.q1*a.q1-this.q2*a.q2-this.q3*a.q3)},"JU.Quat");c(c$,"div",function(a){return this.mulQ(a.inv())},"JU.Quat");c(c$,"divLeft",function(a){return this.inv().mulQ(a)},"JU.Quat");c(c$,"dot",function(a){return this.q0*a.q0+this.q1*a.q1+this.q2*a.q2+this.q3*a.q3},"JU.Quat");c(c$,"inv",function(){return JU.Quat.new4(-this.q1,-this.q2,-this.q3,this.q0)});c(c$,"negate",function(){return JU.Quat.new4(-this.q1,-this.q2,-this.q3,-this.q0)});c(c$,"getFixFactor",function(){return 0>this.q0||
0==this.q0&&(0>this.q1||0==this.q1&&(0>this.q2||0==this.q2&&0>this.q3))?-1:1});c(c$,"getVector",function(a){return this.getVectorScaled(a,1)},"~N");c(c$,"getVectorScaled",function(a,b){if(-1==a)return b*=this.getFixFactor(),JU.V3.new3(this.q1*b,this.q2*b,this.q3*b);null==this.mat&&this.setMatrix();var d=new JU.V3;this.mat.getColumnV(a,d);1!=b&&d.scale(b);return d},"~N,~N");c(c$,"getNormal",function(){var a=JU.Quat.getRawNormal(this);a.scale(this.getFixFactor());return a});c$.getRawNormal=c(c$,"getRawNormal",
function(a){a=JU.V3.new3(a.q1,a.q2,a.q3);if(0==a.length())return JU.V3.new3(0,0,1);a.normalize();return a},"JU.Quat");c(c$,"getTheta",function(){return 360*Math.acos(Math.abs(this.q0))/3.141592653589793});c(c$,"getThetaRadians",function(){return 2*Math.acos(Math.abs(this.q0))});c(c$,"getNormalDirected",function(a){var b=this.getNormal();0>b.x*a.x+b.y*a.y+b.z*a.z&&b.scale(-1);return b},"JU.V3");c(c$,"get3dProjection",function(a){a.set(this.q1,this.q2,this.q3);return a},"JU.V3");c(c$,"getThetaDirected",
function(a){var b=this.getTheta(),d=this.getNormal();0>a.x*this.q1+a.y*this.q2+a.z*this.q3&&(d.scale(-1),b=-b);a.set4(d.x,d.y,d.z,b);return a},"JU.P4");c(c$,"getThetaDirectedV",function(a){var b=this.getTheta(),d=this.getNormal();0>a.x*this.q1+a.y*this.q2+a.z*this.q3&&(d.scale(-1),b=-b);return b},"JU.V3");c(c$,"toPoint4f",function(){return JU.P4.new4(this.q1,this.q2,this.q3,this.q0)});c(c$,"toAxisAngle4f",function(){var a=2*Math.acos(Math.abs(this.q0)),b=Math.sin(a/2),d=this.getNormal();0>b&&(d.scale(-1),
a=3.141592653589793-a);return JU.A4.newVA(d,a)});c(c$,"transform2",function(a,b){null==this.mat&&this.setMatrix();this.mat.rotate2(a,b);return b},"JU.T3,JU.T3");c(c$,"leftDifference",function(a){a=0>this.dot(a)?a.negate():a;return this.inv().mulQ(a)},"JU.Quat");c(c$,"rightDifference",function(a){a=0>this.dot(a)?a.negate():a;return this.mulQ(a.inv())},"JU.Quat");j(c$,"toString",function(){return"{"+this.q1+" "+this.q2+" "+this.q3+" "+this.q0+"}"});c$.div=c(c$,"div",function(a,b,d,g){var c;if(null==
a||null==b||0==(c=Math.min(a.length,b.length)))return null;0<d&&c>d&&(c=d);d=Array(c);for(var e=0;e<c;e++){if(null==a[e]||null==b[e])return null;d[e]=g?a[e].divLeft(b[e]):a[e].div(b[e])}return d},"~A,~A,~N,~B");c$.sphereMean=c(c$,"sphereMean",function(a,b,d){if(null==a||0==a.length)return new JU.Quat;null==b&&(b=K(1,0));if(1==a.length)return b[0]=0,JU.Quat.newQ(a[0]);for(var g=3.4028235E38,c=3.4028235E38,e=JU.Quat.simpleAverage(a);g>d&&0!=c;)e=JU.Quat.newMean(a,e),b[0]=JU.Quat.stdDev(a,e),g=Math.abs(b[0]-
c),c=b[0];return e},"~A,~A,~N");c$.simpleAverage=c(c$,"simpleAverage",function(a){var b=JU.V3.new3(0,0,1),d=a[0].getNormal();b.add(d);for(var g=a.length;0<=--g;)b.add(a[g].getNormalDirected(b));b.sub(d);b.normalize();for(var c=0,g=a.length;0<=--g;)c+=Math.abs(a[g].get3dProjection(d).dot(b));0!=c&&b.scale(c/a.length);c=Math.sqrt(1-b.lengthSquared());Float.isNaN(c)&&(c=0);return JU.Quat.newP4(JU.P4.new4(b.x,b.y,b.z,c))},"~A");c$.newMean=c(c$,"newMean",function(a,b){for(var d=new JU.V3,g,c,e=a.length;0<=
--e;)g=a[e],c=g.div(b),g=c.getNormal(),g.scale(c.getTheta()),d.add(g);d.scale(1/a.length);return JU.Quat.newVA(d,d.length()).mulQ(b)},"~A,JU.Quat");c$.stdDev=c(c$,"stdDev",function(a,b){for(var d=0,g=a.length,c=g;0<=--c;)var e=a[c].div(b).getTheta(),d=d+e*e;return Math.sqrt(d/g)},"~A,JU.Quat");c(c$,"getEulerZYZ",function(){var a,b,d;if(0==this.q1&&0==this.q2)return a=this.getTheta(),K(-1,[0>this.q3?-a:a,0,0]);a=Math.atan2(2*(this.q2*this.q3+this.q0*this.q1),2*(-this.q1*this.q3+this.q0*this.q2));b=
Math.acos(this.q3*this.q3-this.q2*this.q2-this.q1*this.q1+this.q0*this.q0);d=Math.atan2(2*(this.q2*this.q3-this.q0*this.q1),2*(this.q0*this.q2+this.q1*this.q3));return K(-1,[a/0.017453292519943295,b/0.017453292519943295,d/0.017453292519943295])});c(c$,"getEulerZXZ",function(){var a,b,d;if(0==this.q1&&0==this.q2)return a=this.getTheta(),K(-1,[0>this.q3?-a:a,0,0]);a=Math.atan2(2*(this.q1*this.q3-this.q0*this.q2),2*(this.q0*this.q1+this.q2*this.q3));b=Math.acos(this.q3*this.q3-this.q2*this.q2-this.q1*
this.q1+this.q0*this.q0);d=Math.atan2(2*(this.q1*this.q3+this.q0*this.q2),2*(-this.q2*this.q3+this.q0*this.q1));return K(-1,[a/0.017453292519943295,b/0.017453292519943295,d/0.017453292519943295])});c$.qZero=c$.prototype.qZero=new JU.P4;E(c$,"RAD_PER_DEG",0.017453292519943295)});s("JU");v(["java.io.BufferedReader","javajs.api.GenericLineReader"],"JU.Rdr","java.io.BufferedInputStream $.ByteArrayInputStream $.InputStreamReader $.StringReader JU.AU $.Base64 $.Encoding $.SB".split(" "),function(){c$=t(function(){this.reader=
null;p(this,arguments)},JU,"Rdr",null,javajs.api.GenericLineReader);q(c$,function(a){this.reader=a},"java.io.BufferedReader");j(c$,"readNextLine",function(){return this.reader.readLine()});c$.readCifData=c(c$,"readCifData",function(a,b){return a.set(null,b,!1).getAllCifData()},"javajs.api.GenericCifDataParser,java.io.BufferedReader");c$.fixUTF=c(c$,"fixUTF",function(a){var b=JU.Rdr.getUTFEncoding(a);if(b!==JU.Encoding.NONE)try{var d=String.instantialize(a,b.name().$replace("_","-"));switch(b){case JU.Encoding.UTF8:case JU.Encoding.UTF_16BE:case JU.Encoding.UTF_16LE:d=
d.substring(1)}return d}catch(g){if(H(g,java.io.UnsupportedEncodingException))Zystem.out.println(g);else throw g;}return String.instantialize(a)},"~A");c$.getUTFEncoding=c(c$,"getUTFEncoding",function(a){return 3<=a.length&&239==(a[0]&255)&&187==(a[1]&255)&&191==(a[2]&255)?JU.Encoding.UTF8:4<=a.length&&0==(a[0]&255)&&0==(a[1]&255)&&254==(a[2]&255)&&255==(a[3]&255)?JU.Encoding.UTF_32BE:4<=a.length&&255==(a[0]&255)&&254==(a[1]&255)&&0==(a[2]&255)&&0==(a[3]&255)?JU.Encoding.UTF_32LE:2<=a.length&&255==
(a[0]&255)&&254==(a[1]&255)?JU.Encoding.UTF_16LE:2<=a.length&&254==(a[0]&255)&&255==(a[1]&255)?JU.Encoding.UTF_16BE:JU.Encoding.NONE},"~A");c$.getUTFEncodingForStream=c(c$,"getUTFEncodingForStream",function(a){a.resetStream();var b=Q(4,0);b[3]=1;try{a.mark(5)}catch(d){if(H(d,Exception))return JU.Encoding.NONE;throw d;}a.read(b,0,4);a.reset();return JU.Rdr.getUTFEncoding(b)},"java.io.BufferedInputStream");c$.isBase64=c(c$,"isBase64",function(a){return 0==a.indexOf(";base64,")},"JU.SB");c$.isCompoundDocumentS=
c(c$,"isCompoundDocumentS",function(a){return JU.Rdr.isCompoundDocumentB(JU.Rdr.getMagic(a,8))},"java.io.InputStream");c$.isCompoundDocumentB=c(c$,"isCompoundDocumentB",function(a){return 8<=a.length&&208==(a[0]&255)&&207==(a[1]&255)&&17==(a[2]&255)&&224==(a[3]&255)&&161==(a[4]&255)&&177==(a[5]&255)&&26==(a[6]&255)&&225==(a[7]&255)},"~A");c$.isBZip2S=c(c$,"isBZip2S",function(a){return JU.Rdr.isBZip2B(JU.Rdr.getMagic(a,3))},"java.io.InputStream");c$.isGzipS=c(c$,"isGzipS",function(a){return JU.Rdr.isGzipB(JU.Rdr.getMagic(a,
2))},"java.io.InputStream");c$.isBZip2B=c(c$,"isBZip2B",function(a){return null!=a&&3<=a.length&&66==(a[0]&255)&&90==(a[1]&255)&&104==(a[2]&255)},"~A");c$.isGzipB=c(c$,"isGzipB",function(a){return null!=a&&2<=a.length&&31==(a[0]&255)&&139==(a[1]&255)},"~A");c$.isPickleS=c(c$,"isPickleS",function(a){return JU.Rdr.isPickleB(JU.Rdr.getMagic(a,2))},"java.io.InputStream");c$.isPickleB=c(c$,"isPickleB",function(a){return null!=a&&2<=a.length&&125==(a[0]&255)&&113==(a[1]&255)},"~A");c$.isMessagePackS=c(c$,
"isMessagePackS",function(a){return JU.Rdr.isMessagePackB(JU.Rdr.getMagic(a,2))},"java.io.InputStream");c$.isMessagePackB=c(c$,"isMessagePackB",function(a){var b;return null!=a&&1<=a.length&&(222==(b=a[0]&255)||128==(b&224)&&80!=a[1])},"~A");c$.isPngZipStream=c(c$,"isPngZipStream",function(a){return JU.Rdr.isPngZipB(JU.Rdr.getMagic(a,55))},"java.io.InputStream");c$.isPngZipB=c(c$,"isPngZipB",function(a){return 0==a[50]&&80==a[51]&&78==a[52]&&71==a[53]&&74==a[54]},"~A");c$.isZipS=c(c$,"isZipS",function(a){return JU.Rdr.isZipB(JU.Rdr.getMagic(a,
4))},"java.io.InputStream");c$.isZipB=c(c$,"isZipB",function(a){return 4<=a.length&&80==a[0]&&75==a[1]&&3==a[2]&&4==a[3]},"~A");c$.getMagic=c(c$,"getMagic",function(a,b){var d=264<b?Q(b,0):null==JU.Rdr.b264?JU.Rdr.b264=Q(264,0):JU.Rdr.b264;a.resetStream();try{a.mark(b+1),a.read(d,0,b)<b&&(d[0]=d[257]=0)}catch(g){if(!H(g,java.io.IOException))throw g;}try{a.reset()}catch(c){if(!H(c,java.io.IOException))throw c;}return d},"java.io.InputStream,~N");c$.guessMimeTypeForBytes=c(c$,"guessMimeTypeForBytes",
function(a){switch(2>a.length?-1:a[1]){case 0:return"image/jpg";case 73:return"image/gif";case 77:return"image/BMP";case 80:return"image/png";default:return"image/unknown"}},"~A");c$.getBIS=c(c$,"getBIS",function(a){return new java.io.BufferedInputStream(new java.io.ByteArrayInputStream(a))},"~A");c$.getBR=c(c$,"getBR",function(a){return new java.io.BufferedReader(new java.io.StringReader(a))},"~S");c$.getUnzippedInputStream=c(c$,"getUnzippedInputStream",function(a,b){for(;JU.Rdr.isGzipS(b);)b=new java.io.BufferedInputStream(a.newGZIPInputStream(b));
return b},"javajs.api.GenericZipTools,java.io.BufferedInputStream");c$.getUnzippedInputStreamBZip2=c(c$,"getUnzippedInputStreamBZip2",function(a,b){for(;JU.Rdr.isBZip2S(b);)b=new java.io.BufferedInputStream(a.newBZip2InputStream(b));return b},"javajs.api.GenericZipTools,java.io.BufferedInputStream");c$.getBytesFromSB=c(c$,"getBytesFromSB",function(a){return JU.Rdr.isBase64(a)?JU.Base64.decodeBase64(a.substring(8)):a.toBytes(0,-1)},"JU.SB");c$.getStreamAsBytes=c(c$,"getStreamAsBytes",function(a,b){for(var d=
Q(1024,0),g=null==b?Q(4096,0):null,c=0,e=0;0<(c=a.read(d,0,1024));)e+=c,null==b?(e>=g.length&&(g=JU.AU.ensureLengthByte(g,2*e)),Zystem.arraycopy(d,0,g,e-c,c)):b.write(d,0,c);a.close();return null==b?JU.AU.arrayCopyByte(g,e):e+" bytes"},"java.io.BufferedInputStream,JU.OC");c$.getBufferedReader=c(c$,"getBufferedReader",function(a,b){if(JU.Rdr.getUTFEncodingForStream(a)===JU.Encoding.NONE)return new JU.Rdr.StreamReader(a,b);var d=JU.Rdr.getLimitedStreamBytes(a,-1);a.close();return JU.Rdr.getBR(null==
b?JU.Rdr.fixUTF(d):String.instantialize(d,b))},"java.io.BufferedInputStream,~S");c$.getLimitedStreamBytes=c(c$,"getLimitedStreamBytes",function(a,b){var d=0<b&&1024>b?b:1024,g=Q(d,0),c=Q(0>b?4096:b,0),e=0,h=0;for(0>b&&(b=2147483647);h<b&&0<(e=a.read(g,0,d));)h+=e,h>c.length&&(c=JU.AU.ensureLengthByte(c,2*h)),Zystem.arraycopy(g,0,c,h-e,e),2147483647!=b&&h+d>c.length&&(d=c.length-h);if(h==c.length)return c;g=Q(h,0);Zystem.arraycopy(c,0,g,0,h);return g},"java.io.InputStream,~N");c$.streamToUTF8String=
c(c$,"streamToUTF8String",function(a){var b=Array(1);try{JU.Rdr.readAllAsString(JU.Rdr.getBufferedReader(a,"UTF-8"),-1,!0,b,0)}catch(d){if(!H(d,java.io.IOException))throw d;}return b[0]},"java.io.BufferedInputStream");c$.readAllAsString=c(c$,"readAllAsString",function(a,b,d,g,c){try{var e=JU.SB.newN(8192),h;if(0>b){if(h=a.readLine(),d||null!=h&&0>h.indexOf("\x00")&&(4!=h.length||65533!=h.charCodeAt(0)||1!=h.indexOf("PNG")))for(e.append(h).appendC("\n");null!=(h=a.readLine());)e.append(h).appendC("\n")}else{d=
0;for(var k;d<b&&null!=(h=a.readLine());){if(b-d<(k=h.length)+1)h=h.substring(0,b-d-1);e.append(h).appendC("\n");d+=k+1}}a.close();g[c]=e.toString();return!0}catch(l){if(H(l,Exception))return g[c]=l.toString(),!1;throw l;}},"java.io.BufferedReader,~N,~B,~A,~N");c$.getPngZipPointAndCount=c(c$,"getPngZipPointAndCount",function(a,b){a.mark(75);try{var d=JU.Rdr.getLimitedStreamBytes(a,74);a.reset();for(var g=0,c=64,e=1;54<--c;e*=10)g+=(d[c]-48)*e;for(var h=0,c=74,e=1;64<--c;e*=10)h+=(d[c]-48)*e;b[0]=
g;b[1]=h}catch(k){b[1]=0}},"java.io.BufferedInputStream,~A");c$.getPngZipStream=c(c$,"getPngZipStream",function(a,b){if(!JU.Rdr.isPngZipStream(a))return a;var d=Q(0,0);a.mark(75);try{var g=B(2,0);JU.Rdr.getPngZipPointAndCount(a,g);if(0!=g[1]){for(var c=g[0];0<c;)c-=a.skip(c);if(!b)return a;d=JU.Rdr.getLimitedStreamBytes(a,g[1])}}catch(e){}finally{try{b&&a.close()}catch(h){if(!H(h,Exception))throw h;}}return JU.Rdr.getBIS(d)},"java.io.BufferedInputStream,~B");c$.getZipRoot=c(c$,"getZipRoot",function(a){var b=
a.indexOf("|");return 0>b?a:a.substring(0,b)},"~S");c$.isTar=c(c$,"isTar",function(a){a=JU.Rdr.getMagic(a,264);return 0!=a[0]&&117==(a[257]&255)&&115==(a[258]&255)&&116==(a[259]&255)&&97==(a[260]&255)&&114==(a[261]&255)},"java.io.BufferedInputStream");c$.streamToBytes=c(c$,"streamToBytes",function(a){var b=JU.Rdr.getLimitedStreamBytes(a,-1);a.close();return b},"java.io.InputStream");c$.streamToString=c(c$,"streamToString",function(a){return String.instantialize(JU.Rdr.streamToBytes(a))},"java.io.InputStream");
M(self.c$);c$=t(function(){this.stream=null;p(this,arguments)},JU.Rdr,"StreamReader",java.io.BufferedReader);q(c$,function(a,b){I(this,JU.Rdr.StreamReader,[new java.io.InputStreamReader(a,null==b?"UTF-8":b)]);this.stream=a},"java.io.BufferedInputStream,~S");c(c$,"getStream",function(){try{this.stream.reset()}catch(a){if(!H(a,java.io.IOException))throw a;}return this.stream});c$=L();E(c$,"b264",null)});s("JU");v(null,"JU.T3d",["java.lang.Double"],function(){c$=t(function(){this.z=this.y=this.x=0;p(this,
arguments)},JU,"T3d",null,java.io.Serializable);q(c$,function(){});c(c$,"set",function(a,b,d){this.x=a;this.y=b;this.z=d},"~N,~N,~N");c(c$,"setA",function(a){this.x=a[0];this.y=a[1];this.z=a[2]},"~A");c(c$,"setT",function(a){this.x=a.x;this.y=a.y;this.z=a.z},"JU.T3d");c(c$,"add2",function(a,b){this.x=a.x+b.x;this.y=a.y+b.y;this.z=a.z+b.z},"JU.T3d,JU.T3d");c(c$,"add",function(a){this.x+=a.x;this.y+=a.y;this.z+=a.z},"JU.T3d");c(c$,"sub2",function(a,b){this.x=a.x-b.x;this.y=a.y-b.y;this.z=a.z-b.z},"JU.T3d,JU.T3d");
c(c$,"sub",function(a){this.x-=a.x;this.y-=a.y;this.z-=a.z},"JU.T3d");c(c$,"scale",function(a){this.x*=a;this.y*=a;this.z*=a},"~N");c(c$,"scaleAdd",function(a,b,d){this.x=a*b.x+d.x;this.y=a*b.y+d.y;this.z=a*b.z+d.z},"~N,JU.T3d,JU.T3d");j(c$,"hashCode",function(){var a=JU.T3d.doubleToLongBits0(this.x),b=JU.T3d.doubleToLongBits0(this.y),d=JU.T3d.doubleToLongBits0(this.z);return a^a>>32^b^b>>32^d^d>>32});c$.doubleToLongBits0=c(c$,"doubleToLongBits0",function(a){return 0==a?0:Double.doubleToLongBits(a)},
"~N");j(c$,"equals",function(a){return!n(a,JU.T3d)?!1:this.x==a.x&&this.y==a.y&&this.z==a.z},"~O");j(c$,"toString",function(){return"{"+this.x+", "+this.y+", "+this.z+"}"})});s("JU");v(["JU.T3d"],"JU.V3d",null,function(){c$=D(JU,"V3d",JU.T3d);c(c$,"cross",function(a,b){this.set(a.y*b.z-a.z*b.y,a.z*b.x-a.x*b.z,a.x*b.y-a.y*b.x)},"JU.V3d,JU.V3d");c(c$,"normalize",function(){var a=this.length();this.x/=a;this.y/=a;this.z/=a});c(c$,"angle",function(a){var b=this.y*a.z-this.z*a.y,d=this.z*a.x-this.x*a.z,
g=this.x*a.y-this.y*a.x,b=Math.sqrt(b*b+d*d+g*g);return Math.abs(Math.atan2(b,this.dot(a)))},"JU.V3d");c(c$,"dot",function(a){return this.x*a.x+this.y*a.y+this.z*a.z},"JU.V3d");c(c$,"lengthSquared",function(){return this.x*this.x+this.y*this.y+this.z*this.z});c(c$,"length",function(){return Math.sqrt(this.lengthSquared())})});s("J.adapter.readers.molxyz");v(["J.adapter.smarter.AtomSetCollectionReader"],"J.adapter.readers.molxyz.MolReader","java.lang.Exception java.util.Hashtable JU.BS $.Lst $.PT J.adapter.smarter.Atom J.api.JmolAdapter JU.Logger".split(" "),
function(){c$=t(function(){this.haveAtomSerials=this.optimize2D=!1;this.allow2D=!0;this.iatom0=0;this.vr=null;this.atomCount=0;this.atomData=null;this.is2D=!1;p(this,arguments)},J.adapter.readers.molxyz,"MolReader",J.adapter.smarter.AtomSetCollectionReader);j(c$,"initializeReader",function(){this.optimize2D=this.checkFilterKey("2D")});j(c$,"checkLine",function(){var a=this.line.startsWith("$MDL");if(a){if(this.discardLinesUntilStartsWith("$HDR"),this.rd(),null==this.line)return JU.Logger.warn("$HDR not found in MDL RG file"),
this.continuing=!1}else if(this.line.equals("M  END"))return!0;if(this.doGetModel(++this.modelNumber,null)&&(this.iatom0=this.asc.ac,this.processMolSdHeader(),this.processCtab(a),this.vr=null,this.isLastModel(this.modelNumber)))return this.continuing=!1;null!=this.line&&0>this.line.indexOf("$$$$")&&this.discardLinesUntilStartsWith("$$$$");return!0});j(c$,"finalizeSubclassReader",function(){this.finalizeReaderMR()});c(c$,"finalizeReaderMR",function(){this.is2D&&!this.optimize2D&&this.appendLoadNote('This model is 2D. Its 3D structure has not been generated; use LOAD "" FILTER "2D" to optimize 3D.');
if(this.optimize2D){this.set2D();null==this.asc.bsAtoms&&(this.asc.bsAtoms=new JU.BS,this.asc.bsAtoms.setBits(0,this.asc.ac));for(var a=this.asc.bondCount;0<=--a;){var b=this.asc.bonds[a];this.asc.atoms[b.atomIndex2].elementSymbol.equals("H")&&(1025!=b.order&&1041!=b.order)&&this.asc.bsAtoms.clear(b.atomIndex2)}}this.isTrajectory=!1;this.finalizeReaderASCR()});c(c$,"processMolSdHeader",function(){var a="",b=this.line.trim();this.asc.setCollectionName(b);a+=this.line+"\n";this.rd();if(null!=this.line){a+=
this.line+"\n";if((this.is2D=22<=this.line.length&&this.line.substring(20,22).equals("2D"))&&!this.allow2D)throw new Exception("File is 2D, not 3D");this.rd();null!=this.line&&(this.line=this.line.trim(),a+=this.line+"\n",JU.Logger.info(a),this.checkCurrentLineForScript(),this.asc.setInfo("fileHeader",a),this.newAtomSet(b))}});c(c$,"processCtab",function(a){a&&this.discardLinesUntilStartsWith("$CTAB");null!=this.rd()&&(0<=this.line.indexOf("V3000")?(this.optimize2D=this.is2D,this.vr=this.getInterface("J.adapter.readers.molxyz.V3000Rdr").set(this),
this.discardLinesUntilContains("COUNTS"),this.vr.readAtomsAndBonds(this.getTokens())):this.readAtomsAndBonds(this.parseIntRange(this.line,0,3),this.parseIntRange(this.line,3,6)),this.applySymmetryAndSetTrajectory())},"~B");c(c$,"readAtomsAndBonds",function(a,b){this.atomCount=a;for(var d=0;d<a;++d){this.rd();var g=this.line.length,c,e,h,k,l=0,j=0,m=-2147483648;e=this.parseFloatRange(this.line,0,10);h=this.parseFloatRange(this.line,10,20);k=this.parseFloatRange(this.line,20,30);this.is2D&&0!=k&&(this.is2D=
this.optimize2D=!1);34>g?c=this.line.substring(31).trim():(c=this.line.substring(31,34).trim(),c.equals("H1")&&(c="H",j=1),39<=g&&(g=this.parseIntRange(this.line,36,39),1<=g&&7>=g&&(l=4-g),g=this.parseIntRange(this.line,34,36),0!=g&&(-3<=g&&4>=g)&&(j=J.api.JmolAdapter.getNaturalIsotope(J.api.JmolAdapter.getElementNumber(c))+g),-2147483648==m&&this.haveAtomSerials&&(m=d+1)));this.addMolAtom(m,j,c,l,e,h,k)}this.asc.setModelInfoForSet("dimension",this.is2D?"2D":"3D",this.asc.iSet);this.rd();this.line.startsWith("V  ")&&
this.readAtomValues();0==b&&this.asc.setNoAutoBond();for(d=0;d<b;++d)0<d&&this.rd(),h=0,c=this.line.substring(0,3).trim(),e=this.line.substring(3,6).trim(),k=this.parseIntRange(this.line,6,9),this.is2D&&(1==k&&12<=this.line.length)&&(h=this.parseIntRange(this.line,9,12)),k=this.fixOrder(k,h),this.haveAtomSerials?this.asc.addNewBondFromNames(c,e,k):this.asc.addNewBondWithOrder(this.iatom0+this.parseIntStr(c)-1,this.iatom0+this.parseIntStr(e)-1,k);d=new java.util.Hashtable;c=new JU.Lst;for(this.rd();null!=
this.line&&0!=this.line.indexOf("$$$$");)0==this.line.indexOf(">")?this.readMolData(d,c):this.line.startsWith("M  ISO")?this.readIsotopes():this.rd();null!=this.atomData&&(e=d.get("atom_value_name"),d.put(null==e?"atom_values":e.toString(),this.atomData));d.isEmpty()||(this.asc.setCurrentModelInfo("molDataKeys",c),this.asc.setCurrentModelInfo("molData",d))},"~N,~N");c(c$,"readAtomValues",function(){this.atomData=Array(this.atomCount);for(var a=this.atomData.length;0<=--a;)this.atomData[a]="";for(;0==
this.line.indexOf("V  ");){a=this.parseIntAt(this.line,3);if(1>a||a>this.atomCount){JU.Logger.error("V  nnn does not evalute to a valid atom number: "+a);break}var b=this.line.substring(6).trim();this.atomData[a-1]=b;this.rd()}});c(c$,"readIsotopes",function(){var a=this.parseIntAt(this.line,6);try{for(var b=this.asc.getLastAtomSetAtomIndex(),d=0,g=9;d<a;d++){var c=this.parseIntAt(this.line,g),e=this.asc.atoms[c+b-1],h=this.parseIntAt(this.line,g+4),g=g+8;e.elementSymbol=""+h+JU.PT.replaceAllCharacters(e.elementSymbol,
"0123456789","")}}catch(k){}this.rd()});c(c$,"readMolData",function(a,b){for(var d=this.asc.atoms,g=JU.PT.trim(this.line,"> <").toLowerCase(),c="",e=null;null!=this.rd()&&!this.line.equals("$$$$")&&0<this.line.length;)c+=81==this.line.length&&"+"==this.line.charAt(80)?this.line.substring(0,80):this.line+"\n";c=JU.PT.trim(c,"\n");JU.Logger.info(g+":"+JU.PT.esc(c));a.put(g,c);b.addLast(g);var h=0;if(g.toUpperCase().contains("_PARTIAL_CHARGES")){try{for(var e=JU.PT.parseFloatArray(c),k=this.asc.getLastAtomSetAtomIndex(),
l=this.asc.ac;k<l;k++)d[k].partialCharge=0;for(var j=0,k=A(e[j++]);0<=--k;){var m=A(e[j++])+this.iatom0-1,r=e[j++];d[m].partialCharge=r;h++}}catch(x){k=this.asc.getLastAtomSetAtomIndex();for(l=this.asc.ac;k<l;k++)d[k].partialCharge=0;JU.Logger.error("error reading "+g+" field -- partial charges cleared")}JU.Logger.info(h+" partial charges read")}else if(g.toUpperCase().contains("ATOM_NAMES")){h=0;try{for(var y=JU.PT.getTokens(c),j=0,k=this.parseIntStr(y[j++]);0<=--k;){for(var F;-2147483648==(F=this.parseIntStr(y[j++])););
var m=F+this.iatom0-1,C=y[j++];C.equals(".")||(d[m].atomName=C);h++}}catch(n){JU.Logger.error("error reading "+g+" field")}JU.Logger.info(h+" atom names read")}},"java.util.Map,JU.Lst");c(c$,"addMolAtom",function(a,b,d,g,c,e,h){switch(b){case 0:break;case 1:d="1H";break;case 2:d="2H";break;case 3:d="3H";break;default:d=b+d}this.optimize2D&&0!=h&&(this.optimize2D=!1);b=new J.adapter.smarter.Atom;b.elementSymbol=d;b.formalCharge=g;this.setAtomCoordXYZ(b,c,e,h);-2147483648==a?this.asc.addAtom(b):(this.haveAtomSerials=
!0,b.atomSerial=a,this.asc.addAtomWithMappedSerialNumber(b))},"~N,~N,~S,~N,~N,~N,~N");c(c$,"fixOrder",function(a,b){switch(a){default:case 0:case -10:return 1;case 1:switch(b){case 1:return 1025;case 3:case 6:return 1041}break;case 2:case 3:break;case 4:return 515;case 5:return 66;case 6:return 513;case 7:return 514;case 8:case 9:return 33;case 14:return 4;case 15:return 5;case 16:return 6}return a},"~N,~N");c(c$,"addMolBond",function(a,b,d,g){d=this.fixOrder(d,g);this.haveAtomSerials?this.asc.addNewBondFromNames(a,
b,d):this.asc.addNewBondWithOrder(this.iatom0+this.parseIntStr(a)-1,this.iatom0+this.parseIntStr(b)-1,d)},"~S,~S,~N,~N")});s("J.adapter.readers.molxyz");v(["J.adapter.smarter.AtomSetCollectionReader"],"J.adapter.readers.molxyz.XyzReader",["java.lang.Float","JU.Logger"],function(){c$=D(J.adapter.readers.molxyz,"XyzReader",J.adapter.smarter.AtomSetCollectionReader);j(c$,"checkLine",function(){var a=this.parseIntStr(this.line);if(-2147483648==a)return this.continuing=!1;this.vibrationNumber=++this.modelNumber;
if(0>=this.desiredVibrationNumber?this.doGetModel(this.modelNumber,null):this.doGetVibration(this.vibrationNumber)){this.rd();this.checkCurrentLineForScript();this.asc.newAtomSet();var b=this.line.trim();this.readAtoms(a);this.applySymmetryAndSetTrajectory();this.asc.setAtomSetName(b);if(this.isLastModel(this.modelNumber))return this.continuing=!1}else this.skipAtomSet(a);this.discardLinesUntilNonBlank();return!1});j(c$,"finalizeSubclassReader",function(){this.isTrajectory=!1;this.finalizeReaderASCR()});
c(c$,"skipAtomSet",function(a){for(this.rd();0<=--a;)this.rd()},"~N");c(c$,"readAtoms",function(a){for(var b=0;b<a;++b){this.rd();var d=this.getTokens();if(4>d.length)JU.Logger.warn("line cannot be read for XYZ atom data: "+this.line);else{var g=this.addAtomXYZSymName(d,1,null,null);this.setElementAndIsotope(g,d[0]);var c=4;switch(d.length){case 4:continue;case 5:case 6:case 8:case 9:if(0<=d[4].indexOf("."))g.partialCharge=this.parseFloatStr(d[4]);else{var e=this.parseIntStr(d[4]);-2147483648!=e&&
(g.formalCharge=e)}switch(d.length){case 5:continue;case 6:g.radius=this.parseFloatStr(d[5]);continue;case 9:g.atomSerial=this.parseIntStr(d[8])}c++;default:var e=this.parseFloatStr(d[c++]),h=this.parseFloatStr(d[c++]),d=this.parseFloatStr(d[c++]);if(Float.isNaN(e)||Float.isNaN(h)||Float.isNaN(d))continue;this.asc.addVibrationVector(g.index,e,h,d)}}}},"~N")});s("J.adapter.smarter");v(["JU.P3"],"J.adapter.smarter.Atom",["JU.AU","$.Lst","$.V3","JU.Vibration"],function(){c$=t(function(){this.index=this.atomSetIndex=
0;this.bsSymmetry=null;this.atomSite=0;this.elementSymbol=null;this.elementNumber=-1;this.atomName=null;this.formalCharge=-2147483648;this.partialCharge=NaN;this.vib=null;this.bfactor=NaN;this.foccupancy=1;this.radius=NaN;this.isHetero=!1;this.atomSerial=-2147483648;this.chainID=0;this.bondRadius=NaN;this.altLoc="\x00";this.group3=null;this.sequenceNumber=-2147483648;this.insertionCode="\x00";this.tensors=this.anisoBorU=null;this.ignoreSymmetry=!1;this.typeSymbol=null;p(this,arguments)},J.adapter.smarter,
"Atom",JU.P3,Cloneable);c(c$,"addTensor",function(a,b,d){if(null==a)return null;if(d||null==this.tensors)this.tensors=new JU.Lst;this.tensors.addLast(a);null!=b&&a.setType(b);return a},"JU.Tensor,~S,~B");q(c$,function(){I(this,J.adapter.smarter.Atom,[]);this.set(NaN,NaN,NaN)});c(c$,"getClone",function(){var a;try{a=this.clone()}catch(b){if(H(b,CloneNotSupportedException))return null;throw b;}null!=this.vib&&(a.vib=n(this.vib,JU.Vibration)?this.vib.clone():JU.V3.newV(a.vib));null!=this.anisoBorU&&
(a.anisoBorU=JU.AU.arrayCopyF(this.anisoBorU,-1));if(null!=this.tensors){a.tensors=new JU.Lst;for(var d=this.tensors.size();0<=--d;)a.tensors.addLast(this.tensors.get(d).copyTensor())}return a});c(c$,"getElementSymbol",function(){if(null==this.elementSymbol&&null!=this.atomName){for(var a=this.atomName.length,b=0,d=String.fromCharCode(0);b<a&&!J.adapter.smarter.Atom.isValidSymChar1(d=this.atomName.charAt(b));)++b;switch(a-b){case 0:break;default:if(a=this.atomName.charAt(b+1),J.adapter.smarter.Atom.isValidSymNoCase(d,
a)){this.elementSymbol=""+d+a;break}case 1:J.adapter.smarter.Atom.isValidSym1(d)&&(this.elementSymbol=""+d)}}return this.elementSymbol});c$.isValidSym1=c(c$,"isValidSym1",function(a){return"A"<=a&&"Z">=a&&0>J.adapter.smarter.Atom.elementCharMasks[a.charCodeAt(0)-65]},"~S");c$.isValidSym2=c(c$,"isValidSym2",function(a,b){return"A"<=a&&"Z">=a&&"a"<=b&&"z">=b&&0!=(J.adapter.smarter.Atom.elementCharMasks[a.charCodeAt(0)-65]>>b.charCodeAt(0)-97&1)},"~S,~S");c$.isValidSymNoCase=c(c$,"isValidSymNoCase",
function(a,b){return J.adapter.smarter.Atom.isValidSym2(a,"a">b?String.fromCharCode(b.charCodeAt(0)+32):b)},"~S,~S");c$.isValidSymChar1=c(c$,"isValidSymChar1",function(a){return"A"<=a&&"Z">=a&&0!=J.adapter.smarter.Atom.elementCharMasks[a.charCodeAt(0)-65]},"~S");c(c$,"copyTo",function(a,b){var d=b.newCloneAtom(this);d.setT(a);return d},"JU.P3,J.adapter.smarter.AtomSetCollection");E(c$,"elementCharMasks",B(-1,[1972292,-2147351151,-2146019271,-2130706430,1441792,-2147348464,25,-2147205008,-2147344384,
0,-2147352576,1179905,548936,-2147434213,-2147221504,-2145759221,0,1056947,-2147339946,-2147477097,-2147483648,-2147483648,-2147483648,8388624,-2147483646,139264]))});s("J.adapter.smarter");v(["J.api.JmolAdapterAtomIterator"],"J.adapter.smarter.AtomIterator",["java.lang.Float","J.api.JmolAdapter"],function(){c$=t(function(){this.iatom=0;this.atom=null;this.ac=0;this.bsAtoms=this.atoms=null;p(this,arguments)},J.adapter.smarter,"AtomIterator",null,J.api.JmolAdapterAtomIterator);q(c$,function(a){this.ac=
a.ac;this.atoms=a.atoms;this.bsAtoms=a.bsAtoms;this.iatom=0},"J.adapter.smarter.AtomSetCollection");j(c$,"hasNext",function(){if(this.iatom==this.ac)return!1;for(;null==(this.atom=this.atoms[this.iatom++])||null!=this.bsAtoms&&!this.bsAtoms.get(this.atom.index);)if(this.iatom==this.ac)return!1;this.atoms[this.iatom-1]=null;return!0});j(c$,"getAtomSetIndex",function(){return this.atom.atomSetIndex});j(c$,"getSymmetry",function(){return this.atom.bsSymmetry});j(c$,"getAtomSite",function(){return this.atom.atomSite+
1});j(c$,"getUniqueID",function(){return Integer.$valueOf(this.atom.index)});j(c$,"getElementNumber",function(){return 0<this.atom.elementNumber?this.atom.elementNumber:J.api.JmolAdapter.getElementNumber(this.atom.getElementSymbol())});j(c$,"getAtomName",function(){return this.atom.atomName});j(c$,"getFormalCharge",function(){return this.atom.formalCharge});j(c$,"getPartialCharge",function(){return this.atom.partialCharge});j(c$,"getTensors",function(){return this.atom.tensors});j(c$,"getRadius",
function(){return this.atom.radius});j(c$,"getBondRadius",function(){return this.atom.bondRadius});j(c$,"getVib",function(){return null==this.atom.vib||Float.isNaN(this.atom.vib.z)?null:this.atom.vib});j(c$,"getSeqID",function(){return null==this.atom.vib||!Float.isNaN(this.atom.vib.y)||1094713365!=this.atom.vib.z?0:A(this.atom.vib.x)});j(c$,"getBfactor",function(){return this.atom.bfactor});j(c$,"getOccupancy",function(){return 100*this.atom.foccupancy});j(c$,"getIsHetero",function(){return this.atom.isHetero});
j(c$,"getSerial",function(){return this.atom.atomSerial});j(c$,"getChainID",function(){return this.atom.chainID});j(c$,"getAltLoc",function(){return J.api.JmolAdapter.canonizeAlternateLocationID(this.atom.altLoc)});j(c$,"getGroup3",function(){return this.atom.group3});j(c$,"getSequenceNumber",function(){return this.atom.sequenceNumber});j(c$,"getInsertionCode",function(){return J.api.JmolAdapter.canonizeInsertionCode(this.atom.insertionCode)});j(c$,"getXYZ",function(){return this.atom})});s("J.adapter.smarter");
v(["java.util.Hashtable"],"J.adapter.smarter.AtomSetCollection","java.lang.Boolean $.Float java.util.Collections $.Properties JU.AU $.BS $.Lst $.P3 $.V3 J.adapter.smarter.Atom $.Bond $.SmarterJmolAdapter J.api.Interface JU.BSUtil $.Logger".split(" "),function(){c$=t(function(){this.atoms=this.atomSetInfo=this.collectionName=this.fileTypeName=this.bsAtoms=this.reader=null;this.ac=0;this.bonds=null;this.bondCount=0;this.structures=null;this.atomSetCount=this.structureCount=0;this.iSet=-1;this.errorMessage=
this.atomSetAuxiliaryInfo=this.atomSetBondCounts=this.atomSetAtomCounts=this.atomSetAtomIndexes=this.atomSetNumbers=null;this.isTrajectory=this.coordinatesAreFractional=!1;this.trajectoryStepCount=0;this.trajectoryNames=this.vibrationSteps=this.trajectorySteps=null;this.allowMultiple=this.doFixPeriodic=!1;this.readerList=null;this.atomMapAnyCase=!1;this.bsStructuredModels=null;this.haveAnisou=!1;this.baseSymmetryAtomCount=0;this.checkLatticeOnly=!1;this.xtalSymmetry=null;this.bondIndex0=0;this.checkSpecial=
!0;this.atomSymbolicMap=null;this.haveUnitCell=!1;this.vibScale=0;p(this,arguments)},J.adapter.smarter,"AtomSetCollection");O(c$,function(){this.atomSetInfo=new java.util.Hashtable;this.atoms=Array(256);this.bonds=Array(256);this.structures=Array(16);this.atomSetNumbers=B(16,0);this.atomSetAtomIndexes=B(16,0);this.atomSetAtomCounts=B(16,0);this.atomSetBondCounts=B(16,0);this.atomSetAuxiliaryInfo=Array(16);this.atomSymbolicMap=new java.util.Hashtable});c(c$,"setCollectionName",function(a){if(null!=
a&&0<(a=a.trim()).length)this.collectionName=a},"~S");c(c$,"clearGlobalBoolean",function(a){this.atomSetInfo.remove(J.adapter.smarter.AtomSetCollection.globalBooleans[a])},"~N");c(c$,"setGlobalBoolean",function(a){this.setInfo(J.adapter.smarter.AtomSetCollection.globalBooleans[a],Boolean.TRUE)},"~N");c(c$,"getGlobalBoolean",function(a){return this.atomSetInfo.get(J.adapter.smarter.AtomSetCollection.globalBooleans[a])===Boolean.TRUE},"~N");q(c$,function(a,b,d,g){this.fileTypeName=a;this.reader=b;this.allowMultiple=
null==b||0>b.desiredVibrationNumber;a=new java.util.Properties;a.put("PATH_KEY",".PATH");a.put("PATH_SEPARATOR",J.adapter.smarter.SmarterJmolAdapter.PATH_SEPARATOR);this.setInfo("properties",a);b=null==b?null:b.htParams.get("appendToModelIndex");null!=b&&this.setInfo("appendToModelIndex",b);if(null!=d){g=0;this.readerList=new JU.Lst;for(b=0;b<d.length;b++)null!=d[b]&&(0<d[b].ac||null!=d[b].reader&&d[b].reader.mustFinalizeModelSet)&&this.appendAtomSetCollection(g++,d[b]);1<g&&this.setInfo("isMultiFile",
Boolean.TRUE)}else null!=g&&(this.setInfo("isMultiFile",Boolean.TRUE),this.appendAtomSetCollectionList(g))},"~S,J.adapter.smarter.AtomSetCollectionReader,~A,JU.Lst");c(c$,"appendAtomSetCollectionList",function(a){var b=a.size();if(0==b)this.errorMessage="No file found!";else for(var d=0;d<b;d++){var g=a.get(d);n(g,JU.Lst)?this.appendAtomSetCollectionList(g):this.appendAtomSetCollection(d,g)}},"JU.Lst");c(c$,"setTrajectory",function(){this.isTrajectory||(this.trajectorySteps=new JU.Lst);this.isTrajectory=
!0;var a=null==this.bsAtoms?this.ac:this.bsAtoms.cardinality();if(!(1>=a)){for(var b=Array(a),d=0<a&&null!=this.atoms[0].vib&&!Float.isNaN(this.atoms[0].vib.z),a=d?Array(a):null,g=0==this.trajectoryStepCount?null:this.trajectorySteps.get(this.trajectoryStepCount-1),c=0,e=0;c<this.ac;c++)if(null==this.bsAtoms||this.bsAtoms.get(c)){var h=JU.P3.newP(this.atoms[c]);this.doFixPeriodic&&null!=g&&(h=J.adapter.smarter.AtomSetCollection.fixPeriodic(h,g[c]));b[e]=h;d&&(a[e]=this.atoms[c].vib);e++}if(d){if(null==
this.vibrationSteps){this.vibrationSteps=new JU.Lst;for(c=0;c<this.trajectoryStepCount;c++)this.vibrationSteps.addLast(null)}this.vibrationSteps.addLast(a)}this.trajectorySteps.addLast(b);this.trajectoryStepCount++}});c(c$,"appendAtomSetCollection",function(a,b){null!=b.reader&&b.reader.mustFinalizeModelSet&&this.readerList.addLast(b.reader);var d=this.ac;this.setInfo("loadState",b.atomSetInfo.get("loadState"));if(null!=b.bsAtoms){null==this.bsAtoms&&(this.bsAtoms=new JU.BS);for(var g=b.bsAtoms.nextSetBit(0);0<=
g;g=b.bsAtoms.nextSetBit(g+1))this.bsAtoms.set(d+g)}for(var g=0,c=this.atomSetCount,e=0;e<b.atomSetCount;e++){this.newAtomSet();var h=(this.atomSetAuxiliaryInfo[this.iSet]=b.atomSetAuxiliaryInfo[e]).get("PDB_CONECT_firstAtom_count_max");null!=h&&(h[0]+=d);this.setCurrentModelInfo("title",b.collectionName);this.setAtomSetName(b.getAtomSetName(e));for(h=0;h<b.atomSetAtomCounts[e];h++)null!=this.bsAtoms&&this.bsAtoms.set(this.ac),this.newCloneAtom(b.atoms[g]),g++;this.atomSetNumbers[this.iSet]=0>a?this.iSet+
1:1E6*(a+1)+b.atomSetNumbers[e]}for(g=0;g<b.bondCount;g++)e=b.bonds[g],this.addNewBondWithOrder(e.atomIndex1+d,e.atomIndex2+d,e.order);for(g=J.adapter.smarter.AtomSetCollection.globalBooleans.length;0<=--g;)b.getGlobalBoolean(g)&&this.setGlobalBoolean(g);for(g=0;g<b.structureCount;g++)d=b.structures[g],this.addStructure(d),d.modelStartEnd[0]+=c,d.modelStartEnd[1]+=c},"~N,J.adapter.smarter.AtomSetCollection");c(c$,"setNoAutoBond",function(){this.setInfo("noAutoBond",Boolean.TRUE)});c(c$,"freeze",function(a){1==
this.atomSetCount&&null==this.collectionName&&(this.collectionName=this.getAtomSetAuxiliaryInfoValue(0,"name"));a&&this.reverseAtomSets();1<this.trajectoryStepCount&&this.finalizeTrajectory();this.getList(!0);this.getList(!1);for(a=0;a<this.atomSetCount;a++)this.setModelInfoForSet("initialAtomCount",Integer.$valueOf(this.atomSetAtomCounts[a]),a),this.setModelInfoForSet("initialBondCount",Integer.$valueOf(this.atomSetBondCounts[a]),a)},"~B");c(c$,"reverseAtomSets",function(){this.reverseArray(this.atomSetAtomIndexes);
this.reverseArray(this.atomSetNumbers);this.reverseArray(this.atomSetAtomCounts);this.reverseArray(this.atomSetBondCounts);J.adapter.smarter.AtomSetCollection.reverseList(this.trajectorySteps);J.adapter.smarter.AtomSetCollection.reverseList(this.trajectoryNames);J.adapter.smarter.AtomSetCollection.reverseList(this.vibrationSteps);this.reverseObject(this.atomSetAuxiliaryInfo);for(var a=0;a<this.ac;a++)this.atoms[a].atomSetIndex=this.atomSetCount-1-this.atoms[a].atomSetIndex;for(a=0;a<this.structureCount;a++){var b=
this.structures[a].modelStartEnd[0];0<=b&&(this.structures[a].modelStartEnd[0]=this.atomSetCount-1-this.structures[a].modelStartEnd[1],this.structures[a].modelStartEnd[1]=this.atomSetCount-1-b)}for(a=0;a<this.bondCount;a++)this.bonds[a].atomSetIndex=this.atomSetCount-1-this.atoms[this.bonds[a].atomIndex1].atomSetIndex;this.reverseSets(this.bonds,this.bondCount);for(var d=JU.AU.createArrayOfArrayList(this.atomSetCount),a=0;a<this.atomSetCount;a++)d[a]=new JU.Lst;for(a=0;a<this.ac;a++)d[this.atoms[a].atomSetIndex].addLast(this.atoms[a]);
for(var b=B(this.ac,0),g=this.ac,a=this.atomSetCount;0<=--a;)for(var c=d[a].size();0<=--c;){var e=this.atoms[--g]=d[a].get(c);b[e.index]=g;e.index=g}for(a=0;a<this.bondCount;a++)this.bonds[a].atomIndex1=b[this.bonds[a].atomIndex1],this.bonds[a].atomIndex2=b[this.bonds[a].atomIndex2];for(a=0;a<this.atomSetCount;a++)d=this.getAtomSetAuxiliaryInfoValue(a,"PDB_CONECT_firstAtom_count_max"),null!=d&&(d[0]=b[d[0]],d[1]=this.atomSetAtomCounts[a])});c(c$,"reverseSets",function(a,b){for(var d=JU.AU.createArrayOfArrayList(this.atomSetCount),
g=0;g<this.atomSetCount;g++)d[g]=new JU.Lst;for(g=0;g<b;g++){if(0>a[g].atomSetIndex)return;d[a[g].atomSetIndex].addLast(a[g])}for(g=this.atomSetCount;0<=--g;)for(var c=d[g].size();0<=--c;)a[--b]=d[g].get(c)},"~A,~N");c(c$,"reverseObject",function(a){for(var b=this.atomSetCount,d=w(b/2);0<=--d;)JU.AU.swap(a,d,b-1-d)},"~A");c$.reverseList=c(c$,"reverseList",function(a){null!=a&&java.util.Collections.reverse(a)},"JU.Lst");c(c$,"reverseArray",function(a){for(var b=this.atomSetCount,d=w(b/2);0<=--d;)JU.AU.swapInt(a,
d,b-1-d)},"~A");c(c$,"getList",function(a){var b;for(b=this.ac;0<=--b&&!(null!=this.atoms[b]&&"\x00"!=(a?this.atoms[b].altLoc:this.atoms[b].insertionCode)););if(!(0>b)){var d=Array(this.atomSetCount);for(b=0;b<this.atomSetCount;b++)d[b]="";var g;for(b=0;b<this.ac;b++)if(null!=this.atoms[b]){var c=a?this.atoms[b].altLoc:this.atoms[b].insertionCode;if("\x00"!=c&&0>d[g=this.atoms[b].atomSetIndex].indexOf(c))d[g]+=c}a=a?"altLocs":"insertionCodes";for(b=0;b<this.atomSetCount;b++)0<d[b].length&&this.setModelInfoForSet(a,
d[b],b)}},"~B");c(c$,"finish",function(){if(null!=this.reader)this.reader.finalizeModelSet();else if(null!=this.readerList)for(var a=0;a<this.readerList.size();a++)this.readerList.get(a).finalizeModelSet();this.atoms=null;this.atomSetAtomCounts=B(16,0);this.atomSetAuxiliaryInfo=Array(16);this.atomSetInfo=new java.util.Hashtable;this.atomSetCount=0;this.atomSetNumbers=B(16,0);this.atomSymbolicMap=new java.util.Hashtable;this.bonds=null;this.iSet=-1;this.xtalSymmetry=this.readerList=null;this.structures=
Array(16);this.structureCount=0;this.vibrationSteps=this.trajectorySteps=null});c(c$,"discardPreviousAtoms",function(){for(var a=this.ac;0<=--a;)this.atoms[a]=null;this.ac=0;this.atomSymbolicMap.clear();this.atomMapAnyCase=!1;this.atomSetCount=0;this.iSet=-1;for(a=this.atomSetAuxiliaryInfo.length;0<=--a;)this.atomSetAtomCounts[a]=0,this.atomSetBondCounts[a]=0,this.atomSetAuxiliaryInfo[a]=null});c(c$,"removeCurrentAtomSet",function(){if(!(0>this.iSet)){var a=this.atomSetAtomIndexes[this.iSet];null!=
this.bsAtoms&&this.bsAtoms.clearBits(a,this.ac);this.ac=a;this.atomSetAtomCounts[this.iSet]=0;this.iSet--;this.atomSetCount--;this.reader.doCheckUnitCell=!1}});c(c$,"getHydrogenAtomCount",function(){for(var a=0,b=0;b<this.ac;b++)(1==this.atoms[b].elementNumber||this.atoms[b].elementSymbol.equals("H"))&&a++;return a});c(c$,"newCloneAtom",function(a){a=a.getClone();this.addAtom(a);return a},"J.adapter.smarter.Atom");c(c$,"cloneFirstAtomSet",function(a){if(!this.allowMultiple)return 0;this.newAtomSet();
0==a&&(a=this.atomSetAtomCounts[0]);for(var b=0;b<a;++b)this.newCloneAtom(this.atoms[b]);return this.ac},"~N");c(c$,"cloneAtomSetWithBonds",function(a){var b=this.atomSetBondCounts[a?this.iSet:0];a=a?this.cloneLastAtomSet():this.cloneFirstAtomSet(0);if(0<a)for(var d=0;d<b;d++){var g=this.bonds[this.bondCount-b];this.addNewBondWithOrder(g.atomIndex1+a,g.atomIndex2+a,g.order)}},"~B");c(c$,"cloneLastAtomSet",function(){return this.cloneLastAtomSetFromPoints(0,null)});c(c$,"cloneLastAtomSetFromPoints",
function(a,b){if(!this.allowMultiple)return 0;var d=0<a?a:this.getLastAtomSetAtomCount(),g=this.getLastAtomSetAtomIndex();this.newAtomSet();for(var c=0;c<d;++c){var e=this.newCloneAtom(this.atoms[g++]);null!=b&&e.setT(b[c])}return d},"~N,~A");c(c$,"getLastAtomSetAtomCount",function(){return this.atomSetAtomCounts[this.iSet]});c(c$,"getLastAtomSetAtomIndex",function(){return this.ac-this.atomSetAtomCounts[this.iSet]});c(c$,"addNewAtom",function(){return this.addAtom(new J.adapter.smarter.Atom)});c(c$,
"addAtom",function(a){this.ac==this.atoms.length&&(this.atoms=2E5<this.ac?JU.AU.ensureLength(this.atoms,this.ac+5E4):JU.AU.doubleLength(this.atoms));0==this.atomSetCount&&this.newAtomSet();a.index=this.ac;this.atoms[this.ac++]=a;a.atomSetIndex=this.iSet;a.atomSite=this.atomSetAtomCounts[this.iSet]++;return a},"J.adapter.smarter.Atom");c(c$,"addAtomWithMappedName",function(a){var b=this.addAtom(a).atomName;null!=b&&this.atomSymbolicMap.put(b,a)},"J.adapter.smarter.Atom");c(c$,"addAtomWithMappedSerialNumber",
function(a){var b=this.addAtom(a).atomSerial;-2147483648!=b&&this.atomSymbolicMap.put(""+b,a)},"J.adapter.smarter.Atom");c(c$,"getAtomFromName",function(a){return this.atomSymbolicMap.get(a)},"~S");c(c$,"setAtomMapAnyCase",function(){this.atomMapAnyCase=!0;var a=new java.util.Hashtable;a.putAll(this.atomSymbolicMap);for(var b,d=this.atomSymbolicMap.entrySet().iterator();d.hasNext()&&((b=d.next())||1);){var g=b.getKey(),c=g.toUpperCase();c.equals(g)||a.put(c,b.getValue())}this.atomSymbolicMap=a});
c(c$,"getAtomIndex",function(a){var b=this.atomSymbolicMap.get(a);null==b&&this.atomMapAnyCase&&(b=this.atomSymbolicMap.get(a.toUpperCase()));return null==b?-1:b.index},"~S");c(c$,"addNewBondWithOrder",function(a,b,d){var g=null;0<=a&&(a<this.ac&&0<=b&&b<this.ac&&a!=b)&&(g=new J.adapter.smarter.Bond(a,b,d),this.addBond(g));return g},"~N,~N,~N");c(c$,"addNewBondFromNames",function(a,b,d){this.addNewBondWithOrderA(this.getAtomFromName(a),this.getAtomFromName(b),d)},"~S,~S,~N");c(c$,"addNewBondWithOrderA",
function(a,b,d){null!=a&&null!=b&&this.addNewBondWithOrder(a.index,b.index,d)},"J.adapter.smarter.Atom,J.adapter.smarter.Atom,~N");c(c$,"addBond",function(a){0<this.trajectoryStepCount||(0>a.atomIndex1||0>a.atomIndex2||0>a.order||a.atomIndex1==a.atomIndex2||this.atoms[a.atomIndex1].atomSetIndex!=this.atoms[a.atomIndex2].atomSetIndex?JU.Logger.debugging&&JU.Logger.debug(">>>>>>BAD BOND:"+a.atomIndex1+"-"+a.atomIndex2+" order="+a.order):(this.bondCount==this.bonds.length&&(this.bonds=JU.AU.arrayCopyObject(this.bonds,
this.bondCount+1024)),this.bonds[this.bondCount++]=a,this.atomSetBondCounts[this.iSet]++))},"J.adapter.smarter.Bond");c(c$,"finalizeStructures",function(){if(0!=this.structureCount){this.bsStructuredModels=new JU.BS;for(var a=new java.util.Hashtable,b=0;b<this.structureCount;b++){var d=this.structures[b];-1==d.modelStartEnd[0]&&(d.modelStartEnd[0]=0,d.modelStartEnd[1]=this.atomSetCount-1);this.bsStructuredModels.setBits(d.modelStartEnd[0],d.modelStartEnd[1]+1);if(0!=d.strandCount){var d=d.structureID+
" "+d.modelStartEnd[0],g=a.get(d),g=(null==g?0:g.intValue())+1;a.put(d,Integer.$valueOf(g))}}for(b=0;b<this.structureCount;b++)d=this.structures[b],1==d.strandCount&&(d.strandCount=a.get(d.structureID+" "+d.modelStartEnd[0]).intValue())}});c(c$,"addStructure",function(a){this.structureCount==this.structures.length&&(this.structures=JU.AU.arrayCopyObject(this.structures,this.structureCount+32));this.structures[this.structureCount++]=a},"J.adapter.smarter.Structure");c(c$,"addVibrationVectorWithSymmetry",
function(a,b,d,g,c){if(c){c=this.atoms[a].atomSite;for(var e=this.atoms[a].atomSetIndex;a<this.ac&&this.atoms[a].atomSetIndex==e;a++)this.atoms[a].atomSite==c&&this.addVibrationVector(a,b,d,g)}else this.addVibrationVector(a,b,d,g)},"~N,~N,~N,~N,~B");c(c$,"addVibrationVector",function(a,b,d,g){this.allowMultiple||(a%=this.ac);return this.atoms[a].vib=JU.V3.new3(b,d,g)},"~N,~N,~N,~N");c(c$,"setCoordinatesAreFractional",function(a){this.coordinatesAreFractional=a;this.setCurrentModelInfo("coordinatesAreFractional",
Boolean.$valueOf(a));a&&this.setGlobalBoolean(0)},"~B");c(c$,"setAnisoBorU",function(a,b,d){this.haveAnisou=!0;a.anisoBorU=b;b[6]=d},"J.adapter.smarter.Atom,~A,~N");c(c$,"setU",function(a,b,d){var g=a.anisoBorU;null==g&&this.setAnisoBorU(a,g=K(8,0),8);g[b]=d},"J.adapter.smarter.Atom,~N,~N");c(c$,"getXSymmetry",function(){null==this.xtalSymmetry&&(this.xtalSymmetry=J.api.Interface.getOption("adapter.smarter.XtalSymmetry",this.reader.vwr,"file").set(this.reader));return this.xtalSymmetry});c(c$,"getSymmetry",
function(){return this.getXSymmetry().getSymmetry()});c(c$,"setSymmetry",function(a){return null==a?null:this.getXSymmetry().setSymmetry(a)},"J.api.SymmetryInterface");c(c$,"setTensors",function(){this.haveAnisou&&this.getXSymmetry().setTensors()});c(c$,"setInfo",function(a,b){null==b?this.atomSetInfo.remove(a):this.atomSetInfo.put(a,b)},"~S,~O");c(c$,"setAtomSetCollectionPartialCharges",function(a){if(!this.atomSetInfo.containsKey(a))return!1;for(var b=this.atomSetInfo.get(a),d=b.size(),g=this.ac;0<=
--g;)this.atoms[g].partialCharge=b.get(g%d).floatValue();JU.Logger.info("Setting partial charges type "+a);return!0},"~S");c(c$,"mapPartialCharge",function(a,b){this.getAtomFromName(a).partialCharge=b},"~S,~N");c$.fixPeriodic=c(c$,"fixPeriodic",function(a,b){a.x=J.adapter.smarter.AtomSetCollection.fixPoint(a.x,b.x);a.y=J.adapter.smarter.AtomSetCollection.fixPoint(a.y,b.y);a.z=J.adapter.smarter.AtomSetCollection.fixPoint(a.z,b.z);return a},"JU.P3,JU.P3");c$.fixPoint=c(c$,"fixPoint",function(a,b){for(;0.9<
a-b;)a-=1;for(;-0.9>a-b;)a+=1;return a},"~N,~N");c(c$,"finalizeTrajectoryAs",function(a,b){this.trajectorySteps=a;this.vibrationSteps=b;this.trajectoryStepCount=a.size();this.finalizeTrajectory()},"JU.Lst,JU.Lst");c(c$,"finalizeTrajectory",function(){if(0!=this.trajectoryStepCount){var a=this.trajectorySteps.get(0),b=null==this.vibrationSteps?null:this.vibrationSteps.get(0),d=null==this.bsAtoms?this.ac:this.bsAtoms.cardinality();if(null!=this.vibrationSteps&&null!=b&&b.length<d||a.length<d)this.errorMessage=
"File cannot be loaded as a trajectory";else{for(var d=new JU.V3,g=0,c=0;g<this.ac;g++)if(null==this.bsAtoms||this.bsAtoms.get(g))null!=this.vibrationSteps&&(this.atoms[g].vib=null==b?d:b[c]),null!=a[c]&&this.atoms[g].setT(a[c]),c++;this.setInfo("trajectorySteps",this.trajectorySteps);null!=this.vibrationSteps&&this.setInfo("vibrationSteps",this.vibrationSteps);0==this.ac&&(this.ac=a.length)}}});c(c$,"newAtomSet",function(){this.newAtomSetClear(!0)});c(c$,"newAtomSetClear",function(a){!this.allowMultiple&&
0<=this.iSet&&this.reader.discardPreviousAtoms();this.bondIndex0=this.bondCount;this.isTrajectory&&this.reader.discardPreviousAtoms();this.iSet=this.atomSetCount++;this.atomSetCount>this.atomSetNumbers.length&&(this.atomSetAtomIndexes=JU.AU.doubleLengthI(this.atomSetAtomIndexes),this.atomSetAtomCounts=JU.AU.doubleLengthI(this.atomSetAtomCounts),this.atomSetBondCounts=JU.AU.doubleLengthI(this.atomSetBondCounts),this.atomSetAuxiliaryInfo=JU.AU.doubleLength(this.atomSetAuxiliaryInfo));this.atomSetAtomIndexes[this.iSet]=
this.ac;this.atomSetCount+this.trajectoryStepCount>this.atomSetNumbers.length&&(this.atomSetNumbers=JU.AU.doubleLengthI(this.atomSetNumbers));this.isTrajectory?this.atomSetNumbers[this.iSet+this.trajectoryStepCount]=this.atomSetCount+this.trajectoryStepCount:this.atomSetNumbers[this.iSet]=this.atomSetCount;a&&(this.atomSymbolicMap.clear(),this.atomMapAnyCase=!1);this.setCurrentModelInfo("title",this.collectionName)},"~B");c(c$,"getAtomSetAtomIndex",function(a){0>a&&Zystem.out.println("??");return this.atomSetAtomIndexes[a]},
"~N");c(c$,"getAtomSetAtomCount",function(a){return this.atomSetAtomCounts[a]},"~N");c(c$,"getAtomSetBondCount",function(a){return this.atomSetBondCounts[a]},"~N");c(c$,"setAtomSetName",function(a){if(null!=a)if(this.isTrajectory)this.setTrajectoryName(a);else{var b=0>this.iSet?null:this.getAtomSetName(this.iSet);this.setModelInfoForSet("name",a,this.iSet);null!=this.reader&&(0<a.length&&!a.equals(b))&&this.reader.appendLoadNote(a);this.allowMultiple||this.setCollectionName(a)}},"~S");c(c$,"setTrajectoryName",
function(a){if(0!=this.trajectoryStepCount){null==this.trajectoryNames&&(this.trajectoryNames=new JU.Lst);for(var b=this.trajectoryNames.size();b<this.trajectoryStepCount;b++)this.trajectoryNames.addLast(null);this.trajectoryNames.set(this.trajectoryStepCount-1,a)}},"~S");c(c$,"setCurrentAtomSetNumber",function(a){this.setAtomSetNumber(this.iSet+(this.isTrajectory?this.trajectoryStepCount:0),a)},"~N");c(c$,"setAtomSetNumber",function(a,b){this.atomSetNumbers[a]=b},"~N,~N");c(c$,"setAtomSetModelProperty",
function(a,b){this.setAtomSetModelPropertyForSet(a,b,this.iSet)},"~S,~S");c(c$,"setAtomSetModelPropertyForSet",function(a,b,d){var g=this.getAtomSetAuxiliaryInfoValue(d,"modelProperties");null==g&&this.setModelInfoForSet("modelProperties",g=new java.util.Properties,d);g.put(a,b);a.startsWith(".")&&g.put(a.substring(1),b)},"~S,~S,~N");c(c$,"setAtomProperties",function(a,b,d){n(b,String)&&!b.endsWith("\n")&&(b+="\n");0>d&&(d=this.iSet);var g=this.getAtomSetAuxiliaryInfoValue(d,"atomProperties");null==
g&&this.setModelInfoForSet("atomProperties",g=new java.util.Hashtable,d);g.put(a,b)},"~S,~O,~N,~B");c(c$,"setAtomSetPartialCharges",function(a){if(!this.atomSetAuxiliaryInfo[this.iSet].containsKey(a))return!1;a=this.getAtomSetAuxiliaryInfoValue(this.iSet,a);for(var b=a.size();0<=--b;)this.atoms[b].partialCharge=a.get(b).floatValue();return!0},"~S");c(c$,"getAtomSetAuxiliaryInfoValue",function(a,b){return this.atomSetAuxiliaryInfo[0<=a?a:this.iSet].get(b)},"~N,~S");c(c$,"setCurrentModelInfo",function(a,
b){this.setModelInfoForSet(a,b,this.iSet)},"~S,~O");c(c$,"setModelInfoForSet",function(a,b,d){0>d||(null==this.atomSetAuxiliaryInfo[d]&&(this.atomSetAuxiliaryInfo[d]=new java.util.Hashtable),null==b?this.atomSetAuxiliaryInfo[d].remove(a):this.atomSetAuxiliaryInfo[d].put(a,b))},"~S,~O,~N");c(c$,"getAtomSetNumber",function(a){return this.atomSetNumbers[a>=this.atomSetCount?0:a]},"~N");c(c$,"getAtomSetName",function(a){if(null!=this.trajectoryNames&&a<this.trajectoryNames.size())return this.trajectoryNames.get(a);
a>=this.atomSetCount&&(a=this.atomSetCount-1);return this.getAtomSetAuxiliaryInfoValue(a,"name")},"~N");c(c$,"getAtomSetAuxiliaryInfo",function(a){a=a>=this.atomSetCount?this.atomSetCount-1:a;return 0>a?null:this.atomSetAuxiliaryInfo[a]},"~N");c(c$,"setAtomSetEnergy",function(a,b){0>this.iSet||(JU.Logger.info("Energy for model "+(this.iSet+1)+" = "+a),this.setCurrentModelInfo("EnergyString",a),this.setCurrentModelInfo("Energy",Float.$valueOf(b)),this.setAtomSetModelProperty("Energy",""+b))},"~S,~N");
c(c$,"setAtomSetFrequency",function(a,b,d,g,c){this.setAtomSetModelProperty("FreqValue",g);g+=" "+(null==c?"cm^-1":c);c=(null==d?"":d+" ")+g;this.setAtomSetName(c);this.setAtomSetModelProperty("Frequency",g);this.setAtomSetModelProperty("Mode",""+a);this.setModelInfoForSet("vibrationalMode",Integer.$valueOf(a),this.iSet);null!=d&&this.setAtomSetModelProperty("FrequencyLabel",d);this.setAtomSetModelProperty(".PATH",(null==b?"":b+J.adapter.smarter.SmarterJmolAdapter.PATH_SEPARATOR+"Frequencies")+"Frequencies");
return c},"~N,~S,~S,~S,~S");c(c$,"getBondList",function(){for(var a=Array(this.bondCount),b=0;b<this.bondCount;b++)a[b]=u(-1,[this.atoms[this.bonds[b].atomIndex1].atomName,this.atoms[this.bonds[b].atomIndex2].atomName,""+this.bonds[b].order]);return a});c(c$,"centralize",function(){for(var a=new JU.P3,b=0;b<this.atomSetCount;b++){var d=this.atomSetAtomCounts[b],g=this.atomSetAtomIndexes[b];a.set(0,0,0);for(var c=g+d;--c>=g;)a.add(this.atoms[c]);a.scale(1/d);for(c=g+d;--c>=g;)this.atoms[c].sub(a)}});
c(c$,"mergeTrajectories",function(a){if(this.isTrajectory&&a.isTrajectory&&null==this.vibrationSteps){for(var b=0;b<a.trajectoryStepCount;b++)this.trajectorySteps.add(this.trajectoryStepCount++,a.trajectorySteps.get(b));this.setInfo("trajectorySteps",this.trajectorySteps);this.setInfo("ignoreUnitCell",a.atomSetInfo.get("ignoreUnitCell"))}},"J.adapter.smarter.AtomSetCollection");c(c$,"removeAtomSet",function(a){null==this.bsAtoms&&(this.bsAtoms=JU.BSUtil.newBitSet2(0,this.ac));var b=this.atomSetAtomIndexes[a],
d=b+this.atomSetAtomCounts[a];this.bsAtoms.clearBits(b,d);for(b=d;b<this.ac;b++)this.atoms[b].atomSetIndex--;for(b=a+1;b<this.atomSetCount;b++)this.atomSetAuxiliaryInfo[b-1]=this.atomSetAuxiliaryInfo[b],this.atomSetAtomIndexes[b-1]=this.atomSetAtomIndexes[b],this.atomSetBondCounts[b-1]=this.atomSetBondCounts[b],this.atomSetAtomCounts[b-1]=this.atomSetAtomCounts[b],this.atomSetNumbers[b-1]=this.atomSetNumbers[b];for(b=0;b<this.bondCount;b++)this.bonds[b].atomSetIndex=this.atoms[this.bonds[b].atomIndex1].atomSetIndex;
this.atomSetAuxiliaryInfo[--this.atomSetCount]=null;for(b=d=0;b<this.structureCount;b++){var g=this.structures[b];g.modelStartEnd[0]==a&&g.modelStartEnd[1]==a&&(this.structures[b]=null,d++)}if(0<d){a=Array(this.structureCount-d);for(d=b=0;b<this.structureCount;b++)null!=this.structures[b]&&(a[d++]=this.structures[b]);this.structures=a}},"~N");c(c$,"removeLastUnselectedAtoms",function(){for(var a=this.ac,b=0,d=this.getLastAtomSetAtomIndex(),g=0,c=d;c<a;c++)this.bsAtoms.get(c)?(0<b&&(this.atoms[this.atoms[c].index=
c-b]=this.atoms[c],this.atoms[c]=null),g++):(b++,this.ac--,this.atoms[c]=null);this.atomSetAtomCounts[this.iSet]=g;0==g?(this.iSet--,this.atomSetCount--):this.bsAtoms.setBits(d,d+g)});c(c$,"checkNoEmptyModel",function(){for(;0<this.atomSetCount&&0==this.atomSetAtomCounts[this.atomSetCount-1];)this.atomSetCount--});E(c$,"globalBooleans",u(-1,"someModelsHaveFractionalCoordinates someModelsHaveSymmetry someModelsHaveUnitcells someModelsHaveCONECT isPDB someModelsHaveDomains someModelsHaveValidations".split(" ")),
"GLOBAL_FRACTCOORD",0,"GLOBAL_SYMMETRY",1,"GLOBAL_UNITCELLS",2,"GLOBAL_CONECT",3,"GLOBAL_ISPDB",4,"GLOBAL_DOMAINS",5,"GLOBAL_VALIDATIONS",6)});s("J.adapter.smarter");v(["javajs.api.GenericLineReader","JU.SB"],"J.adapter.smarter.AtomSetCollectionReader","java.io.BufferedReader java.lang.Boolean $.Double $.Float $.NullPointerException javajs.api.GenericBinaryDocument JU.BS $.Lst $.M3 $.P3 $.PT $.Quat $.T4 $.V3 J.adapter.smarter.Atom $.AtomSetCollection J.api.Interface $.JmolAdapter JU.BSUtil $.Logger".split(" "),
function(){c$=t(function(){this.requiresBSFilter=this.debugging=this.isBinary=!1;this.dssr=this.validation=this.domains=this.trajectorySteps=this.htParams=this.readerName=this.binaryDoc=this.reader=this.asc=this.primitiveToCrystal=null;this.isConcatenated=!1;this.addedDataKey=this.addedData=null;this.fixJavaFloat=!0;this.next=this.prevline=this.line=this.lstNCS=this.thisBiomolecule=null;this.ptLine=0;this.fillRange=this.latticeCells=this.latticeType=null;this.iHaveSymmetryOperators=this.iHaveUnitCell=
this.doProcessLines=!1;this.continuing=!0;this.vwr=null;this.isMolecular=this.isSequential=this.getHeader=this.doCheckUnitCell=this.applySymmetryToBonds=this.isTrajectory=this.ignoreFileSymmetryOperators=this.doApplySymmetry=!1;this.vibrationNumber=this.modelNumber=this.templateAtomCount=0;this.desiredVibrationNumber=-2147483648;this.bsModels=null;this.havePartialChargeFilter=this.useFileModelNumbers=!1;this.calculationType="?";this.sgName=null;this.ignoreFileSpaceGroupName=this.ignoreFileUnitCell=
!1;this.unitCellParams=null;this.desiredModelNumber=-2147483648;this.out=this.symmetry=null;this.doPackUnitCell=this.iHaveFractionalCoordinates=!1;this.ptSupercell=null;this.forcePacked=this.mustFinalizeModelSet=!1;this.packingError=0.02;this.isPrimitive=this.rotateHexCell=!1;this.modDim=0;this.loadNote=null;this.merging=this.fileCoordinatesAreFractional=this.doConvertToFractional=!1;this.symmetryRange=0;this.firstLastStep=null;this.lastModelNumber=2147483647;this.desiredSpaceGroupIndex=-1;this.fileScaling=
null;this.latticeScaling=NaN;this.unitCellOffset=this.fileOffsetFractional=this.fileOffset=null;this.unitCellOffsetFractional=!1;this.paramsLattice=this.moreUnitCellInfo=null;this.paramsPacked=this.paramsCentroid=!1;this.fileName=this.filePath=null;this.baseBondIndex=this.baseAtomIndex=0;this.stateScriptVersionInt=2147483647;this.haveModel=this.isFinalized=!1;this.previousUnitCell=this.previousSpaceGroup=null;this.nMatrixElements=0;this.filter=this.bsFilter=this.matUnitCellOrientation=this.ucItems=
null;this.filterAtomType=this.filterAtomName=this.filterChain=this.filterGroup3=this.filterAltLoc=this.haveAtomFilter=!1;this.filterAtomTypeStr=null;this.filterAtomNameTerminator=";";this.filterEveryNth=this.filterHetero=this.filterElement=!1;this.filterSymop=null;this.nFiltered=this.filterN=0;this.reverseModels=this.doReadMolecularOrbitals=this.allowPDBFilter=this.isDSSP1=this.ignoreStructure=this.useAltNames=this.addVibrations=this.doCentralize=this.doSetOrientation=!1;this.nameRequired=null;this.centroidPacked=
this.doCentroidUnitCell=!1;this.strSupercell=null;this.allow_a_len_1=!1;this.ms=this.matRot=this.filter2=this.filter1=null;this.vibsFractional=!1;this.siteScript=this.previousScript=null;p(this,arguments)},J.adapter.smarter,"AtomSetCollectionReader",null,javajs.api.GenericLineReader);O(c$,function(){this.next=B(1,0);this.loadNote=new JU.SB});c(c$,"setup",function(a,b,d){this.setupASCR(a,b,d)},"~S,java.util.Map,~O");c(c$,"setupASCR",function(a,b,d){null!=a&&(this.debugging=JU.Logger.debugging,this.htParams=
b,this.filePath=""+b.get("fullPathName"),a=this.filePath.lastIndexOf("/"),this.fileName=this.filePath.substring(a+1),n(d,java.io.BufferedReader)?this.reader=d:n(d,javajs.api.GenericBinaryDocument)&&(this.binaryDoc=d))},"~S,java.util.Map,~O");c(c$,"readData",function(){this.initialize();this.asc=new J.adapter.smarter.AtomSetCollection(this.readerName,this,null,null);try{this.initializeReader();if(null==this.binaryDoc)for(null==this.line&&this.continuing&&this.rd();null!=this.line&&this.continuing;)this.checkLine()&&
this.rd();else this.binaryDoc.setOutputChannel(this.out),this.processBinaryDocument();this.finalizeSubclassReader();this.isFinalized||this.finalizeReaderASCR()}catch(a){JU.Logger.info("Reader error: "+a),a.printStackTrace(),this.setError(a)}null!=this.reader&&this.reader.close();null!=this.binaryDoc&&this.binaryDoc.close();return this.finish()});c(c$,"fixBaseIndices",function(){try{var a=this.htParams.get("baseModelIndex").intValue();this.baseAtomIndex+=this.asc.ac;this.baseBondIndex+=this.asc.bondCount;
a+=this.asc.atomSetCount;this.htParams.put("baseAtomIndex",Integer.$valueOf(this.baseAtomIndex));this.htParams.put("baseBondIndex",Integer.$valueOf(this.baseBondIndex));this.htParams.put("baseModelIndex",Integer.$valueOf(a))}catch(b){if(!H(b,Exception))throw b;}});c(c$,"readDataObject",function(a){this.initialize();this.asc=new J.adapter.smarter.AtomSetCollection(this.readerName,this,null,null);this.initializeReader();this.processDOM(a);return this.finish()},"~O");c(c$,"processDOM",function(){},"~O");
c(c$,"processBinaryDocument",function(){});c(c$,"initializeReader",function(){});c(c$,"checkLine",function(){return!0});c(c$,"checkLastModel",function(){if(this.isLastModel(this.modelNumber)&&this.doProcessLines)return this.continuing=this.doProcessLines=!1;this.doProcessLines=!1;return!0});c(c$,"isLastModel",function(a){return 0<this.desiredModelNumber||a>=this.lastModelNumber},"~N");c(c$,"appendLoadNote",function(a){null==a?this.loadNote=new JU.SB:(this.loadNote.append(a).append("\n"),JU.Logger.info(a))},
"~S");c(c$,"initializeTrajectoryFile",function(){this.asc.addAtom(new J.adapter.smarter.Atom);this.trajectorySteps=this.htParams.get("trajectorySteps");null==this.trajectorySteps&&this.htParams.put("trajectorySteps",this.trajectorySteps=new JU.Lst)});c(c$,"finalizeSubclassReader",function(){});c(c$,"finalizeReaderASCR",function(){this.isFinalized=!0;if(0<this.asc.atomSetCount){1==this.asc.atomSetCount&&(this.asc.setCurrentModelInfo("dbName",this.htParams.get("dbName")),this.asc.setCurrentModelInfo("auxFiles",
this.htParams.get("auxFiles")));this.applySymmetryAndSetTrajectory();this.asc.finalizeStructures();this.doCentralize&&this.asc.centralize();null!=this.fillRange&&this.asc.setInfo("boundbox",this.fillRange);var a=this.asc.getAtomSetAuxiliaryInfo(0);if(null!=a){if(null!=this.domains){this.asc.setGlobalBoolean(5);var b=this.domains.getMapKeys(2,!0),d=b.indexOf("{ ",2);0<=d&&(b=b.substring(d+2));d=b.indexOf("_metadata");0>d&&(d=b.indexOf("metadata"));0<=d&&(b=b.substring(0,d));b=JU.PT.rep(JU.PT.replaceAllCharacters(b,
"{}","").trim(),"\n","\n  ")+"\n\nUse SHOW DOMAINS for details.";this.appendLoadNote("\nDomains loaded:\n   "+b);for(b=this.asc.atomSetCount;0<=--b;)a=this.asc.getAtomSetAuxiliaryInfo(b),a.put("domains",this.domains)}if(null!=this.validation)for(b=this.asc.atomSetCount;0<=--b;)a=this.asc.getAtomSetAuxiliaryInfo(b),a.put("validation",this.validation);if(null!=this.dssr){a.put("dssrJSON",Boolean.TRUE);for(b=this.asc.atomSetCount;0<=--b;)a=this.asc.getAtomSetAuxiliaryInfo(b),a.put("dssr",this.dssr)}}}this.fixJavaFloat||
this.asc.setInfo("legacyJavaFloat",Boolean.TRUE);this.setLoadNote()});c(c$,"setLoadNote",function(){var a=this.loadNote.toString();0<this.loadNote.length()&&this.asc.setInfo("modelLoadNote",a);return a});c(c$,"setIsPDB",function(){this.asc.setGlobalBoolean(4);null!=this.htParams.get("pdbNoHydrogens")&&this.asc.setInfo("pdbNoHydrogens",this.htParams.get("pdbNoHydrogens"));this.checkFilterKey("ADDHYDROGENS")&&this.asc.setInfo("pdbAddHydrogens",Boolean.TRUE)});c(c$,"setModelPDB",function(a){a?this.asc.setGlobalBoolean(4):
this.asc.clearGlobalBoolean(4);this.asc.setCurrentModelInfo("isPDB",a?Boolean.TRUE:null)},"~B");c(c$,"finish",function(){var a=this.htParams.get("loadState");this.asc.setInfo("loadState",null==a?"":a);a=this.htParams.get("smilesString");null!=a&&this.asc.setInfo("smilesString",a);this.htParams.containsKey("templateAtomCount")||this.htParams.put("templateAtomCount",Integer.$valueOf(this.asc.ac));null!=this.bsFilter&&(this.htParams.put("filteredAtomCount",Integer.$valueOf(JU.BSUtil.cardinalityOf(this.bsFilter))),
this.htParams.put("bsFilter",this.bsFilter));this.calculationType.equals("?")||this.asc.setInfo("calculationType",this.calculationType);var b=a=this.asc.fileTypeName;0<=b.indexOf("(")&&(b=b.substring(0,b.indexOf("(")));for(var d=this.asc.atomSetCount;0<=--d;)this.asc.setModelInfoForSet("fileName",this.filePath,d),this.asc.setModelInfoForSet("fileType",b,d);this.asc.freeze(this.reverseModels);if(null!=this.asc.errorMessage)return this.asc.errorMessage+"\nfor file "+this.filePath+"\ntype "+a;if((null==
this.asc.bsAtoms?0==this.asc.ac:0>this.asc.bsAtoms.nextSetBit(0))&&0>b.indexOf("DataOnly")&&null==this.asc.atomSetInfo.get("dataOnly"))return"No atoms found\nfor file "+this.filePath+"\ntype "+a;this.fixBaseIndices();return this.asc});c(c$,"setError",function(a){var b=a.getMessage();this.asc.errorMessage=null==this.line?"Error reading file at end of file \n"+b:"Error reading file at line "+this.ptLine+":\n"+this.line+"\n"+b;a.printStackTrace()},"Throwable");c(c$,"initialize",function(){this.htParams.containsKey("baseAtomIndex")&&
(this.baseAtomIndex=this.htParams.get("baseAtomIndex").intValue());this.htParams.containsKey("baseBondIndex")&&(this.baseBondIndex=this.htParams.get("baseBondIndex").intValue());this.initializeSymmetry();this.vwr=this.htParams.remove("vwr");this.htParams.containsKey("stateScriptVersionInt")&&(this.stateScriptVersionInt=this.htParams.get("stateScriptVersionInt").intValue());var a=this.htParams.get("packingError");null!=a?this.packingError=a.floatValue():null!=this.htParams.get("legacyJavaFloat")&&
(this.fixJavaFloat=!1);this.merging=this.htParams.containsKey("merging");this.getHeader=this.htParams.containsKey("getHeader");this.isSequential=this.htParams.containsKey("isSequential");this.readerName=this.htParams.get("readerName");this.htParams.containsKey("outputChannel")&&(this.out=this.htParams.get("outputChannel"));this.htParams.containsKey("vibrationNumber")?this.desiredVibrationNumber=this.htParams.get("vibrationNumber").intValue():this.htParams.containsKey("modelNumber")&&(this.desiredModelNumber=
this.htParams.get("modelNumber").intValue());this.applySymmetryToBonds=this.htParams.containsKey("applySymmetryToBonds");this.bsFilter=this.requiresBSFilter?this.htParams.get("bsFilter"):null;this.setFilter(null);this.fillRange=this.htParams.get("fillRange");null!=this.strSupercell&&!this.checkFilterKey("NOPACK")&&(this.forcePacked=!0);a=this.htParams.get("supercell");n(a,JU.P3)?(a=this.ptSupercell=a,this.strSupercell=A(a.x)+"a,"+A(a.y)+"b,"+A(a.z)+"c"):n(a,String)&&(this.strSupercell=a);a=this.htParams.containsKey("ptFile")?
this.htParams.get("ptFile").intValue():-1;this.isTrajectory=this.htParams.containsKey("isTrajectory");0<a&&this.htParams.containsKey("firstLastSteps")?(a=this.htParams.get("firstLastSteps").get(a-1),n(a,JU.BS)?this.bsModels=a:this.firstLastStep=a):this.htParams.containsKey("firstLastStep")?this.firstLastStep=this.htParams.get("firstLastStep"):this.htParams.containsKey("bsModels")&&(this.bsModels=this.htParams.get("bsModels"));this.useFileModelNumbers=this.htParams.containsKey("useFileModelNumbers")||
this.checkFilterKey("USEFILEMODELNUMBERS");this.htParams.containsKey("templateAtomCount")&&(this.templateAtomCount=this.htParams.get("templateAtomCount").intValue());if(null!=this.bsModels||null!=this.firstLastStep)this.desiredModelNumber=-2147483648;if(null==this.bsModels&&null!=this.firstLastStep){0>this.firstLastStep[0]&&(this.firstLastStep[0]=0);if(0==this.firstLastStep[2]||this.firstLastStep[1]<this.firstLastStep[0])this.firstLastStep[1]=-1;1>this.firstLastStep[2]&&(this.firstLastStep[2]=1);
this.bsModels=JU.BSUtil.newAndSetBit(this.firstLastStep[0]);if(this.firstLastStep[1]>this.firstLastStep[0])for(a=this.firstLastStep[0];a<=this.firstLastStep[1];a+=this.firstLastStep[2])this.bsModels.set(a)}if(null!=this.bsModels&&(null==this.firstLastStep||-1!=this.firstLastStep[1]))this.lastModelNumber=this.bsModels.length();this.symmetryRange=this.htParams.containsKey("symmetryRange")?this.htParams.get("symmetryRange").floatValue():0;this.paramsLattice=this.htParams.get("lattice");this.paramsCentroid=
this.htParams.containsKey("centroid");this.paramsPacked=this.htParams.containsKey("packed");this.initializeSymmetryOptions();this.htParams.containsKey("spaceGroupIndex")&&(this.desiredSpaceGroupIndex=this.htParams.get("spaceGroupIndex").intValue(),-2==this.desiredSpaceGroupIndex&&(this.sgName=this.htParams.get("spaceGroupName")),this.ignoreFileSpaceGroupName=-2==this.desiredSpaceGroupIndex||0<=this.desiredSpaceGroupIndex,this.ignoreFileSymmetryOperators=-1!=this.desiredSpaceGroupIndex);this.htParams.containsKey("unitCellOffset")&&
(this.fileScaling=JU.P3.new3(1,1,1),this.fileOffset=this.htParams.get("unitCellOffset"),this.fileOffsetFractional=JU.P3.newP(this.fileOffset),this.unitCellOffsetFractional=this.htParams.containsKey("unitCellOffsetFractional"));this.htParams.containsKey("unitcell")&&(a=this.htParams.get("unitcell"),this.merging&&this.setFractionalCoordinates(!0),9==a.length?(this.addExplicitLatticeVector(0,a,0),this.addExplicitLatticeVector(1,a,3),this.addExplicitLatticeVector(2,a,6)):this.setUnitCell(a[0],a[1],a[2],
a[3],a[4],a[5]),this.ignoreFileUnitCell=this.iHaveUnitCell,this.merging&&!this.iHaveUnitCell&&this.setFractionalCoordinates(!1));this.domains=this.htParams.get("domains");this.validation=this.htParams.get("validation");this.dssr=this.htParams.get("dssr");this.isConcatenated=this.htParams.containsKey("concatenate")});c(c$,"initializeSymmetryOptions",function(){this.latticeCells=B(4,0);this.doApplySymmetry=!1;var a=this.paramsLattice;if(null==a||0==a.length()){if(!this.forcePacked&&null==this.strSupercell)return;
a=JU.P3.new3(1,1,1)}this.latticeCells[0]=A(a.x);this.latticeCells[1]=A(a.y);this.latticeCells[2]=A(a.z);n(a,JU.T4)&&(this.latticeCells[3]=A(a.w));if((this.doCentroidUnitCell=this.paramsCentroid)&&(-1==this.latticeCells[2]||0==this.latticeCells[2]))this.latticeCells[2]=1;a=this.forcePacked||this.paramsPacked;this.centroidPacked=this.doCentroidUnitCell&&a;this.doPackUnitCell=!this.doCentroidUnitCell&&(a||0>this.latticeCells[2]);this.doApplySymmetry=0<this.latticeCells[0]&&0<this.latticeCells[1];this.doApplySymmetry||
(this.latticeCells=B(3,0))});c(c$,"doGetModel",function(a,b){if(null!=b&&null!=this.nameRequired&&0<this.nameRequired.length&&0>b.toUpperCase().indexOf(this.nameRequired))return!1;var d=null==this.bsModels?1>this.desiredModelNumber||a==this.desiredModelNumber:a>this.lastModelNumber?!1:0<a&&this.bsModels.get(a-1)||this.haveModel&&null!=this.firstLastStep&&0>this.firstLastStep[1]&&(2>this.firstLastStep[2]||0==(a-1-this.firstLastStep[0])%this.firstLastStep[2]);d&&0==this.desiredModelNumber&&this.discardPreviousAtoms();
this.haveModel=(new Boolean(this.haveModel|d)).valueOf();d&&(this.doProcessLines=!0);return d},"~N,~S");c(c$,"discardPreviousAtoms",function(){this.asc.discardPreviousAtoms()});c(c$,"initializeSymmetry",function(){this.previousSpaceGroup=this.sgName;this.previousUnitCell=this.unitCellParams;this.iHaveUnitCell=this.ignoreFileUnitCell;if(!this.ignoreFileUnitCell){this.unitCellParams=K(26,0);for(var a=25;0<=--a;)this.unitCellParams[a]=NaN;this.unitCellParams[25]=this.latticeScaling;this.symmetry=null}this.ignoreFileSpaceGroupName||
(this.sgName="unspecified!");this.doCheckUnitCell=!1});c(c$,"newAtomSet",function(a){0<=this.asc.iSet?(this.asc.newAtomSet(),this.asc.setCollectionName("<collection of "+(this.asc.iSet+1)+" models>")):this.asc.setCollectionName(a);this.asc.setModelInfoForSet("name",a,Math.max(0,this.asc.iSet));this.asc.setAtomSetName(a)},"~S");c(c$,"cloneLastAtomSet",function(a,b){var d=this.asc.getLastAtomSetAtomCount();this.asc.cloneLastAtomSetFromPoints(a,b);this.asc.haveUnitCell&&(this.doCheckUnitCell=this.iHaveUnitCell=
!0,this.sgName=this.previousSpaceGroup,this.unitCellParams=this.previousUnitCell);return d},"~N,~A");c(c$,"setSpaceGroupName",function(a){this.ignoreFileSpaceGroupName||null==a||(a=a.trim(),0==a.length||(a.equals("HM:")||a.equals(this.sgName))||(a.equals("P1")||JU.Logger.info("Setting space group name to "+a),this.sgName=a))},"~S");c(c$,"setSymmetryOperator",function(a){if(this.ignoreFileSymmetryOperators)return-1;var b=this.asc.getXSymmetry().addSpaceGroupOperation(a,!0);0>b&&JU.Logger.warn("Skippings symmetry operation "+
a);this.iHaveSymmetryOperators=!0;return b},"~S");c(c$,"initializeCartesianToFractional",function(){for(var a=0;16>a;a++)if(!Float.isNaN(this.unitCellParams[6+a]))return;for(a=0;16>a;a++)this.unitCellParams[6+a]=0==a%5?1:0;this.nMatrixElements=0});c(c$,"clearUnitCell",function(){if(!this.ignoreFileUnitCell){for(var a=6;22>a;a++)this.unitCellParams[a]=NaN;this.checkUnitCell(6)}});c(c$,"setUnitCellItem",function(a,b){this.ignoreFileUnitCell||(0==a&&1==b&&!this.allow_a_len_1||3==a&&0==b?(null==this.ucItems&&
(this.ucItems=K(6,0)),this.ucItems[a]=b):(null!=this.ucItems&&6>a&&(this.ucItems[a]=b),!Float.isNaN(b)&&(6<=a&&Float.isNaN(this.unitCellParams[6]))&&this.initializeCartesianToFractional(),this.unitCellParams[a]=b,this.debugging&&JU.Logger.debug("setunitcellitem "+a+" "+b),6>a||Float.isNaN(b)?this.iHaveUnitCell=this.checkUnitCell(6):12==++this.nMatrixElements&&(this.iHaveUnitCell=this.checkUnitCell(22))))},"~N,~N");c(c$,"setUnitCell",function(a,b,d,g,c,e){this.ignoreFileUnitCell||(this.clearUnitCell(),
this.unitCellParams[0]=a,this.unitCellParams[1]=b,this.unitCellParams[2]=d,0!=g&&(this.unitCellParams[3]=g),0!=c&&(this.unitCellParams[4]=c),0!=e&&(this.unitCellParams[5]=e),this.iHaveUnitCell=this.checkUnitCell(6))},"~N,~N,~N,~N,~N,~N");c(c$,"addExplicitLatticeVector",function(a,b,d){if(!this.ignoreFileUnitCell){if(0==a)for(var g=0;6>g;g++)this.unitCellParams[g]=0;a=6+3*a;this.unitCellParams[a++]=b[d++];this.unitCellParams[a++]=b[d++];this.unitCellParams[a]=b[d];if(Float.isNaN(this.unitCellParams[0]))for(a=
0;6>a;a++)this.unitCellParams[a]=-1;this.iHaveUnitCell=this.checkUnitCell(15)}},"~N,~A,~N");c(c$,"checkUnitCell",function(a){for(var b=0;b<a;b++)if(Float.isNaN(this.unitCellParams[b]))return!1;if(22==a&&1==this.unitCellParams[0]&&1==this.unitCellParams[1]&&1==this.unitCellParams[2]&&1==this.unitCellParams[6]&&1==this.unitCellParams[11]&&1==this.unitCellParams[16])return!1;this.doApplySymmetry&&(this.getSymmetry(),this.doConvertToFractional=!this.fileCoordinatesAreFractional);return!0},"~N");c(c$,
"getSymmetry",function(){if(!this.iHaveUnitCell)return null;null==this.symmetry&&(this.getNewSymmetry().setUnitCell(this.unitCellParams,!1),this.checkUnitCellOffset());null==this.symmetry?this.iHaveUnitCell=!1:this.symmetry.setSpaceGroupName(this.sgName);return this.symmetry});c(c$,"checkUnitCellOffset",function(){null==this.fileOffsetFractional||null==this.symmetry||(this.fileOffset.setT(this.fileOffsetFractional),this.unitCellOffsetFractional!=this.fileCoordinatesAreFractional&&(this.unitCellOffsetFractional?
this.symmetry.toCartesian(this.fileOffset,!1):this.symmetry.toFractional(this.fileOffset,!1)))});c(c$,"fractionalizeCoordinates",function(a){if(null!=this.getSymmetry()){var b=this.asc.atoms;if(a)for(var d=this.asc.ac;0<=--d;)this.symmetry.toFractional(b[d],!1);else for(d=this.asc.ac;0<=--d;)this.symmetry.toCartesian(b[d],!1);this.setFractionalCoordinates(a)}},"~B");c(c$,"getNewSymmetry",function(){return this.symmetry=this.getInterface("JS.Symmetry")});c(c$,"setFractionalCoordinates",function(a){this.iHaveFractionalCoordinates=
this.fileCoordinatesAreFractional=a;this.checkUnitCellOffset()},"~B");c(c$,"setFilterAtomTypeStr",function(a){this.filterAtomTypeStr=a;this.filterAtomNameTerminator="\x00"},"~S");c(c$,"setFilter",function(a){null==a?a=this.htParams.get("filter"):this.bsFilter=null;null!=a&&(a=a.toUpperCase());this.filter=a;this.doSetOrientation=!this.checkFilterKey("NOORIENT");this.doCentralize=!this.checkFilterKey("NOCENTER")&&this.checkFilterKey("CENTER");this.addVibrations=!this.checkFilterKey("NOVIB");this.ignoreStructure=
this.checkFilterKey("DSSP");this.isDSSP1=this.checkFilterKey("DSSP1");this.doReadMolecularOrbitals=!this.checkFilterKey("NOMO");this.useAltNames=this.checkFilterKey("ALTNAME");this.reverseModels=this.checkFilterKey("REVERSEMODELS");this.allow_a_len_1=this.checkFilterKey("TOPOS");if(null!=this.filter){this.checkFilterKey("HETATM")&&(this.filterHetero=!0,this.filter=JU.PT.rep(this.filter,"HETATM","HETATM-Y"));this.checkFilterKey("ATOM")&&(this.filterHetero=!0,this.filter=JU.PT.rep(this.filter,"ATOM",
"HETATM-N"));this.checkFilterKey("CELL=")&&(this.strSupercell=this.filter.substring(this.filter.indexOf("CELL=")+5).toLowerCase());this.nameRequired=JU.PT.getQuotedAttribute(this.filter,"NAME");null!=this.nameRequired&&(this.nameRequired.startsWith("'")?this.nameRequired=JU.PT.split(this.nameRequired,"'")[1]:this.nameRequired.startsWith('"')&&(this.nameRequired=JU.PT.split(this.nameRequired,'"')[1]),this.filter=JU.PT.rep(this.filter,this.nameRequired,""),this.filter=JU.PT.rep(this.filter,"NAME=",
""));this.filterAtomName=this.checkFilterKey("*.")||this.checkFilterKey("!.");if(this.filter.startsWith("_")||0<=this.filter.indexOf(";_"))this.filterElement=this.checkFilterKey("_");this.filterGroup3=this.checkFilterKey("[");this.filterChain=this.checkFilterKey(":");this.filterAltLoc=this.checkFilterKey("%");if(this.filterEveryNth=this.checkFilterKey("/="))this.filterN=this.parseIntAt(this.filter,this.filter.indexOf("/=")+2);else if(this.filter.startsWith("=")||0<=this.filter.indexOf(";="))this.filterAtomType=
this.checkFilterKey("=");-2147483648==this.filterN&&(this.filterEveryNth=!1);this.haveAtomFilter=this.filterAtomName||this.filterAtomType||this.filterElement||this.filterGroup3||this.filterChain||this.filterAltLoc||this.filterHetero||this.filterEveryNth||this.checkFilterKey("/=");if(null==this.bsFilter&&(this.bsFilter=new JU.BS,this.htParams.put("bsFilter",this.bsFilter),this.filter=(";"+this.filter+";").$replace(",",";"),a=this.getFilter("LATTICESCALING="),null!=a&&25<this.unitCellParams.length&&
(this.unitCellParams[25]=this.latticeScaling=this.parseFloatStr(a)),a=this.getFilter("SYMOP="),null!=a&&(this.filterSymop=" "+a+" "),JU.Logger.info("filtering with "+this.filter),this.haveAtomFilter&&(this.filter1=this.filter,0<=(a=this.filter.indexOf("|")))))this.filter1=this.filter.substring(0,a).trim()+";",this.filter2=";"+this.filter.substring(a).trim()}},"~S");c(c$,"getFilter",function(a){var b=null==this.filter?-1:this.filter.indexOf(a);return 0>b?null:this.filter.substring(b+a.length,this.filter.indexOf(";",
b))},"~S");c(c$,"checkFilterKey",function(a){return null!=this.filter&&0<=this.filter.indexOf(a)},"~S");c(c$,"checkAndRemoveFilterKey",function(a){if(!this.checkFilterKey(a))return!1;this.filter=JU.PT.rep(this.filter,a,"");3>this.filter.length&&(this.filter=null);return!0},"~S");c(c$,"filterAtom",function(a,b){if(!this.haveAtomFilter)return!0;var d=this.checkFilter(a,this.filter1);null!=this.filter2&&(d=(new Boolean(d|this.checkFilter(a,this.filter2))).valueOf());d&&this.filterEveryNth&&(d=0==this.nFiltered++%
this.filterN);this.bsFilter.setBitTo(0<=b?b:this.asc.ac,d);return d},"J.adapter.smarter.Atom,~N");c(c$,"checkFilter",function(a,b){return(!this.filterGroup3||null==a.group3||!this.filterReject(b,"[",a.group3.toUpperCase()+"]"))&&(!this.filterAtomName||this.allowAtomName(a.atomName,b))&&(null==this.filterAtomTypeStr||null==a.atomName||0<=a.atomName.toUpperCase().indexOf("\x00"+this.filterAtomTypeStr))&&(!this.filterElement||null==a.elementSymbol||!this.filterReject(b,"_",a.elementSymbol.toUpperCase()+
";"))&&(!this.filterChain||0==a.chainID||!this.filterReject(b,":",""+this.vwr.getChainIDStr(a.chainID)))&&(!this.filterAltLoc||"\x00"==a.altLoc||!this.filterReject(b,"%",""+a.altLoc))&&(!this.filterHetero||!this.allowPDBFilter||!this.filterReject(b,"HETATM",a.isHetero?"-Y":"-N"))},"J.adapter.smarter.Atom,~S");c(c$,"rejectAtomName",function(a){return this.filterAtomName&&!this.allowAtomName(a,this.filter)},"~S");c(c$,"allowAtomName",function(a,b){return null==a||!this.filterReject(b,".",a.toUpperCase()+
this.filterAtomNameTerminator)},"~S,~S");c(c$,"filterReject",function(a,b,d){return 0<=a.indexOf(b)&&0<=a.indexOf("!"+b)==0<=a.indexOf(b+d)},"~S,~S,~S");c(c$,"set2D",function(){this.asc.setInfo("is2D",Boolean.TRUE);this.checkFilterKey("NOMIN")||this.asc.setInfo("doMinimize",Boolean.TRUE);this.appendLoadNote("This model is 2D. Its 3D structure will be generated.")});c(c$,"doGetVibration",function(a){return this.addVibrations&&(0>=this.desiredVibrationNumber||a==this.desiredVibrationNumber)},"~N");
c(c$,"setTransform",function(a,b,d,g,c,e,h,k,l){null==this.matRot&&this.doSetOrientation&&(this.matRot=new JU.M3,a=JU.V3.new3(a,b,d),a.normalize(),this.matRot.setColumnV(0,a),a.set(g,c,e),a.normalize(),this.matRot.setColumnV(1,a),a.set(h,k,l),a.normalize(),this.matRot.setColumnV(2,a),this.asc.setInfo("defaultOrientationMatrix",JU.M3.newM3(this.matRot)),g=JU.Quat.newM(this.matRot),this.asc.setInfo("defaultOrientationQuaternion",g),JU.Logger.info("defaultOrientationMatrix = "+this.matRot))},"~N,~N,~N,~N,~N,~N,~N,~N,~N");
c(c$,"setAtomCoordXYZ",function(a,b,d,g){a.set(b,d,g);this.setAtomCoord(a)},"J.adapter.smarter.Atom,~N,~N,~N");c(c$,"setAtomCoordScaled",function(a,b,d,g){null==a&&(a=this.asc.addNewAtom());this.setAtomCoordXYZ(a,this.parseFloatStr(b[d])*g,this.parseFloatStr(b[d+1])*g,this.parseFloatStr(b[d+2])*g);return a},"J.adapter.smarter.Atom,~A,~N,~N");c(c$,"setAtomCoordTokens",function(a,b,d){this.setAtomCoordXYZ(a,this.parseFloatStr(b[d]),this.parseFloatStr(b[d+1]),this.parseFloatStr(b[d+2]))},"J.adapter.smarter.Atom,~A,~N");
c(c$,"addAtomXYZSymName",function(a,b,d,g){var c=this.asc.addNewAtom();null!=d&&(c.elementSymbol=d);null!=g&&(c.atomName=g);this.setAtomCoordTokens(c,a,b);return c},"~A,~N,~S,~S");c(c$,"setAtomCoord",function(a){var b=this.doConvertToFractional&&!this.fileCoordinatesAreFractional&&null!=this.getSymmetry();null!=this.fileScaling&&(a.x=a.x*this.fileScaling.x+this.fileOffset.x,a.y=a.y*this.fileScaling.y+this.fileOffset.y,a.z=a.z*this.fileScaling.z+this.fileOffset.z);b&&(this.symmetry.haveUnitCell()||
this.symmetry.setUnitCell(this.unitCellParams,!1),this.symmetry.toFractional(a,!1),this.iHaveFractionalCoordinates=!0);this.fixJavaFloat&&this.fileCoordinatesAreFractional&&JU.PT.fixPtFloats(a,1E5);this.doCheckUnitCell=!0},"J.adapter.smarter.Atom");c(c$,"addSites",function(a){this.asc.setCurrentModelInfo("pdbSites",a);var b="",d;for(a=a.entrySet().iterator();a.hasNext()&&((d=a.next())||1);){for(var g=d.getKey(),c=d.getValue(),e,h=g.length;0<=--h;)if(!JU.PT.isLetterOrDigit(e=g.charAt(h))&&"'"!=e)g=
g.substring(0,h)+"_"+g.substring(h+1);c=c.get("groups");0!=c.length&&(this.addSiteScript("@site_"+g+" "+c),this.addSiteScript("site_"+g+' = ["'+JU.PT.rep(c,",",'","')+'"]'),b+=',"site_'+g+'"')}0<b.length&&this.addSiteScript("site_list = ["+b.substring(1)+"]")},"java.util.Map");c(c$,"applySymmetryAndSetTrajectory",function(){this.applySymTrajASCR()});c(c$,"applySymTrajASCR",function(){this.forcePacked&&this.initializeSymmetryOptions();var a=this.iHaveUnitCell&&this.doCheckUnitCell?this.asc.getXSymmetry().applySymmetryFromReader(this.getSymmetry()):
null;null==a&&this.asc.setTensors();this.isTrajectory&&this.asc.setTrajectory();null!=this.moreUnitCellInfo&&(this.asc.setCurrentModelInfo("moreUnitCellInfo",this.moreUnitCellInfo),this.moreUnitCellInfo=null);this.finalizeSubclassSymmetry(null!=a);this.initializeSymmetry();return a});c(c$,"finalizeSubclassSymmetry",function(){},"~B");c(c$,"doPreSymmetry",function(){});c(c$,"finalizeMOData",function(a){this.asc.setCurrentModelInfo("moData",a);null!=a&&(a=a.get("mos"),null!=a&&JU.Logger.info(a.size()+
" molecular orbitals read in model "+this.asc.atomSetCount))},"java.util.Map");c$.getElementSymbol=c(c$,"getElementSymbol",function(a){return J.api.JmolAdapter.getElementSymbol(a)},"~N");c(c$,"fillDataBlock",function(a,b){for(var d=a.length,g=0;g<d;g++)a[g]=JU.PT.getTokens(this.discardLinesUntilNonBlank()),a[g].length<b&&--g},"~A,~N");c(c$,"fill3x3",function(a,b){for(var d=X(3,3,0),g=null==a,c=b,e=0;3>e;e++){if(g||b>=a.length){for(;3>(a=JU.PT.getTokens(this.rd())).length;);b=0>c?a.length+c:c}for(var h=
0;3>h;h++)d[e][h]=Double.$valueOf(a[b++]).doubleValue()}return d},"~A,~N");c(c$,"fillFloatArray",function(a,b,d){for(var g=[],c=0,e=0;e<d.length;e++){for(;null!=g&&c>=g.length;){null==a&&(a=this.rd());if(0==b)g=JU.PT.getTokens(a);else{g=Array(w(a.length/b));for(c=0;c<g.length;c++)g[c]=a.substring(c*b,(c+1)*b)}a=null;c=0}if(null==g)break;d[e]=this.parseFloatStr(g[c++])}return d},"~S,~N,~A");c(c$,"fillFrequencyData",function(a,b,d,g,c,e,h,k,l,j){var m=0!=b&&d!=b&&null==j;0==b&&null!=k&&(b=k.length);
b=c?b:3*b;var r=g.length;if(null==j)j=Array(b),this.fillDataBlockFixed(j,e,h,l);else if(!c){this.fillDataBlockFixed(j,e,h,-l);if(null==j[0])return;a+=this.parseIntAt(this.line,l-5)-1}for(h=e=0;e<b;e++,h++){l=j[e];for(var x=c?null:j[++e],y=c?null:j[++e],F=l.length-(c?3*r:r)-1,C=0,n=0;n<r;n++){++F;var p=l[F];")"==p.charAt(0)&&(p=p.substring(1));var p=this.parseFloatStr(p),q=this.parseFloatStr(c?l[++F]:x[F]),t=this.parseFloatStr(c?l[++F]:y[F]);if(!g[n]){var s=null==k?h:k[h];0>s||(s+=a+d*C++,this.debugging&&
JU.Logger.debug("atom "+s+" vib"+C+": "+p+" "+q+" "+t),this.asc.addVibrationVectorWithSymmetry(s,p,q,t,m))}}}},"~N,~N,~N,~A,~B,~N,~N,~A,~N,~A");c(c$,"fillDataBlockFixed",function(a,b,d,g){if(0==d)this.fillDataBlock(a,g);else for(var c=a.length,e=0;e<c;e++){this.discardLinesUntilNonBlank();if(0>g&&" "==this.line.charAt(-g)){a[0]=null;break}var h=w((this.line.length-b+1)/d);a[e]=Array(h);for(var k=0,l=b;k<h;k++,l+=d)a[e][k]=this.line.substring(l,Math.min(this.line.length,l+d))}},"~A,~N,~N,~N");c(c$,
"readLines",function(a){for(;0<=--a;)this.rd();return this.line},"~N");c(c$,"discardLinesUntilStartsWith",function(a){for(;null!=this.rd()&&!this.line.startsWith(a););return this.line},"~S");c(c$,"discardLinesUntilContains",function(a){for(;null!=this.rd()&&0>this.line.indexOf(a););return this.line},"~S");c(c$,"discardLinesUntilContains2",function(a,b){for(;null!=this.rd()&&0>this.line.indexOf(a)&&0>this.line.indexOf(b););return this.line},"~S,~S");c(c$,"discardLinesUntilBlank",function(){for(;null!=
this.rd()&&0!=this.line.trim().length;);return this.line});c(c$,"discardLinesUntilNonBlank",function(){for(;null!=this.rd()&&0==this.line.trim().length;);return this.line});c(c$,"checkLineForScript",function(a){this.line=a;this.checkCurrentLineForScript()},"~S");c(c$,"checkCurrentLineForScript",function(){this.line.endsWith("#noautobond")&&(this.line=this.line.substring(0,this.line.lastIndexOf("#")).trim(),this.asc.setNoAutoBond());var a=this.line.indexOf("jmolscript:");if(0<=a){var b=this.line.substring(a+
11,this.line.length);0<=b.indexOf("#")&&(b=b.substring(0,b.indexOf("#")));this.addJmolScript(b);this.line=this.line.substring(0,a).trim()}});c(c$,"addJmolScript",function(a){JU.Logger.info("#jmolScript: "+a);null==this.previousScript?this.previousScript="":this.previousScript.endsWith(";")||(this.previousScript+=";");this.previousScript+=a;this.asc.setInfo("jmolscript",this.previousScript)},"~S");c(c$,"addSiteScript",function(a){null==this.siteScript?this.siteScript="":this.siteScript.endsWith(";")||
(this.siteScript+=";");this.siteScript+=a;this.asc.setInfo("sitescript",this.siteScript)},"~S");c(c$,"rd",function(){return this.RL()});c(c$,"RL",function(){this.prevline=this.line;this.line=this.reader.readLine();null!=this.out&&null!=this.line&&this.out.append(this.line).append("\n");this.ptLine++;this.debugging&&null!=this.line&&JU.Logger.info(this.line);return this.line});c$.getStrings=c(c$,"getStrings",function(a,b,d){for(var g=Array(b),c=0,e=0;c<b;c++,e+=d)g[c]=a.substring(e,e+d);return g},
"~S,~N,~N");c(c$,"getTokens",function(){return JU.PT.getTokens(this.line)});c$.getTokensFloat=c(c$,"getTokensFloat",function(a,b,d){null==b&&(b=K(d,0));JU.PT.parseFloatArrayDataN(JU.PT.getTokens(a),b,d);return b},"~S,~A,~N");c(c$,"parseFloat",function(){return JU.PT.parseFloatNext(this.line,this.next)});c(c$,"parseFloatStr",function(a){this.next[0]=0;return JU.PT.parseFloatNext(a,this.next)},"~S");c(c$,"parseFloatRange",function(a,b,d){this.next[0]=b;return JU.PT.parseFloatRange(a,d,this.next)},"~S,~N,~N");
c(c$,"parseInt",function(){return JU.PT.parseIntNext(this.line,this.next)});c(c$,"parseIntStr",function(a){this.next[0]=0;return JU.PT.parseIntNext(a,this.next)},"~S");c(c$,"parseIntAt",function(a,b){this.next[0]=b;return JU.PT.parseIntNext(a,this.next)},"~S,~N");c(c$,"parseIntRange",function(a,b,d){this.next[0]=b;return JU.PT.parseIntRange(a,d,this.next)},"~S,~N,~N");c(c$,"parseToken",function(){return JU.PT.parseTokenNext(this.line,this.next)});c(c$,"parseTokenStr",function(a){this.next[0]=0;return JU.PT.parseTokenNext(a,
this.next)},"~S");c(c$,"parseTokenNext",function(a){return JU.PT.parseTokenNext(a,this.next)},"~S");c(c$,"parseTokenRange",function(a,b,d){this.next[0]=b;return JU.PT.parseTokenRange(a,d,this.next)},"~S,~N,~N");c$.getFortranFormatLengths=c(c$,"getFortranFormatLengths",function(a){var b=new JU.Lst,d=0,g=0,c=1,e=!1,h=!0;a+=",";for(var k=0;k<a.length;k++){var l=a.charAt(k);switch(l){case ".":e=!1;continue;case ",":for(e=0;e<g;e++)b.addLast(Integer.$valueOf(d*c));e=!1;h=!0;g=0;continue;case "X":d=g;g=
1;c=-1;continue}JU.PT.isDigit(l)?e?d=10*d+l.charCodeAt(0)-48:h&&(g=10*g+l.charCodeAt(0)-48):JU.PT.isLetter(l)?(d=0,e=!0,h=!1,c=1):e=!1}return b},"~S");c(c$,"read3Vectors",function(a){for(var b=Array(3),d=K(3,0),g=0;3>g;g++){if(0<g||Float.isNaN(this.parseFloatStr(this.line)))if(this.rd(),0==g&&null!=this.line){g=-1;continue}this.fillFloatArray(this.line,0,d);b[g]=new JU.V3;b[g].setA(d);a&&b[g].scale(0.5291772)}return b},"~B");c(c$,"setElementAndIsotope",function(a,b){var d=this.parseIntStr(b);-2147483648==
d?a.elementSymbol=b:(b=b.substring((""+d).length),a.elementNumber=0==b.length?d:(d<<7)+J.api.JmolAdapter.getElementNumber(b))},"J.adapter.smarter.Atom,~S");c(c$,"finalizeModelSet",function(){});c(c$,"setChainID",function(a,b){a.chainID=this.vwr.getChainID(b,!0)},"J.adapter.smarter.Atom,~S");j(c$,"readNextLine",function(){null!=this.rd()&&0<=this.line.indexOf("#jmolscript:")&&this.checkCurrentLineForScript();return this.line});c(c$,"appendUunitCellInfo",function(a){null==this.moreUnitCellInfo&&(this.moreUnitCellInfo=
new JU.Lst);this.moreUnitCellInfo.addLast(a);this.appendLoadNote(a)},"~S");c(c$,"getInterface",function(a){a=J.api.Interface.getInterface(a,this.vwr,"file");if(null==a)throw new NullPointerException("Interface");return a},"~S");c(c$,"forceSymmetry",function(a){a&&(this.doPackUnitCell=a);this.doApplySymmetry||(this.doApplySymmetry=!0,this.latticeCells[0]=1,this.latticeCells[1]=1,this.latticeCells[2]=1)},"~B");E(c$,"ANGSTROMS_PER_BOHR",0.5291772)});s("J.adapter.smarter");c$=t(function(){this.atomSetIndex=
0;p(this,arguments)},J.adapter.smarter,"AtomSetObject");s("J.adapter.smarter");v(["J.adapter.smarter.AtomSetObject"],"J.adapter.smarter.Bond",null,function(){c$=t(function(){this.order=this.atomIndex2=this.atomIndex1=0;this.uniqueID=this.colix=this.radius=-1;this.distance=0;p(this,arguments)},J.adapter.smarter,"Bond",J.adapter.smarter.AtomSetObject);q(c$,function(a,b,d){I(this,J.adapter.smarter.Bond,[]);this.atomIndex1=a;this.atomIndex2=b;this.order=d},"~N,~N,~N");q(c$,function(){I(this,J.adapter.smarter.Bond,
[])});j(c$,"toString",function(){return"[Bond "+this.atomIndex1+" "+this.atomIndex2+" "+this.order+"]"})});s("J.adapter.smarter");v(["J.api.JmolAdapterBondIterator"],"J.adapter.smarter.BondIterator",null,function(){c$=t(function(){this.bonds=this.bsAtoms=null;this.ibond=0;this.bond=null;this.bondCount=0;p(this,arguments)},J.adapter.smarter,"BondIterator",J.api.JmolAdapterBondIterator);q(c$,function(a){I(this,J.adapter.smarter.BondIterator,[]);this.bsAtoms=a.bsAtoms;this.bonds=a.bonds;this.bondCount=
a.bondCount;this.ibond=0},"J.adapter.smarter.AtomSetCollection");j(c$,"hasNext",function(){if(this.ibond==this.bondCount)return!1;for(;null==(this.bond=this.bonds[this.ibond++])||null!=this.bsAtoms&&(!this.bsAtoms.get(this.bond.atomIndex1)||!this.bsAtoms.get(this.bond.atomIndex2));)if(this.ibond==this.bondCount)return!1;return!0});j(c$,"getAtomUniqueID1",function(){return Integer.$valueOf(this.bond.atomIndex1)});j(c$,"getAtomUniqueID2",function(){return Integer.$valueOf(this.bond.atomIndex2)});j(c$,
"getEncodedOrder",function(){return this.bond.order});j(c$,"getRadius",function(){return this.bond.radius});j(c$,"getColix",function(){return this.bond.colix})});s("J.adapter.smarter");v(null,"J.adapter.smarter.Resolver","java.io.BufferedInputStream $.InputStream java.lang.Float java.util.StringTokenizer javajs.api.GenericBinaryDocument JU.LimitedLineReader $.PT $.Rdr J.adapter.smarter.AtomSetCollectionReader $.SmarterJmolAdapter J.api.Interface JU.Logger JV.FileManager $.JC".split(" "),function(){c$=
D(J.adapter.smarter,"Resolver");c$.getReaderClassBase=c(c$,"getReaderClassBase",function(a){var b=a+"Reader";if(a.startsWith("Xml"))return"J.adapter.readers.xml."+b;a=";"+a+";";for(var d=1;d<J.adapter.smarter.Resolver.readerSets.length;d+=2)if(0<=J.adapter.smarter.Resolver.readerSets[d].indexOf(a))return"J.adapter.readers."+J.adapter.smarter.Resolver.readerSets[d-1]+b;return"J.adapter.readers.???."+b},"~S");c$.getFileType=c(c$,"getFileType",function(a){try{return J.adapter.smarter.Resolver.determineAtomSetCollectionReader(a,
!1)}catch(b){if(H(b,Exception))return null;throw b;}},"java.io.BufferedReader");c$.getAtomCollectionReader=c(c$,"getAtomCollectionReader",function(a,b,d,g,c){var e;a=JV.FileManager.fixDOSName(a);var h=null;null==b&&(b=g.get("filter"),e=null==b?-1:b.toLowerCase().indexOf("filetype"),b=0>e?null:b.substring(e+8,(b+";").indexOf(";",e)).$replace("="," ").trim());null!=b?(e=J.adapter.smarter.Resolver.getReaderFromType(b),null==e&&(e=J.adapter.smarter.Resolver.getReaderFromType("Xml"+b)),null==e?h="unrecognized file format type "+
b:JU.Logger.info("The Resolver assumes "+e)):(e=J.adapter.smarter.Resolver.determineAtomSetCollectionReader(d,!0),"\n"==e.charAt(0)&&(b=g.get("defaultType"),null!=b&&(b=J.adapter.smarter.Resolver.getReaderFromType(b),null!=b&&(e=b))),"\n"==e.charAt(0)?h="unrecognized file format for file\n"+a+"\n"+J.adapter.smarter.Resolver.split(e,50):e.equals("spt")?h="NOTE: file recognized as a script file: "+a+"\n":a.equals("ligand")||JU.Logger.info("The Resolver thinks "+e));if(null!=h)return J.adapter.smarter.SmarterJmolAdapter.close(d),
h;g.put("ptFile",Integer.$valueOf(c));0>=c&&g.put("readerName",e);0==e.indexOf("Xml")&&(e="Xml");return J.adapter.smarter.Resolver.getReader(e,g)},"~S,~S,~O,java.util.Map,~N");c$.getReader=c(c$,"getReader",function(a,b){var d=null,g=null,d=null,g=J.adapter.smarter.Resolver.getReaderClassBase(a);if(null==(d=J.api.Interface.getInterface(g,b.get("vwr"),"reader")))d=JV.JC.READER_NOT_FOUND+g,JU.Logger.error(d);return d},"~S,java.util.Map");c$.getReaderFromType=c(c$,"getReaderFromType",function(a){a=";"+
a.toLowerCase()+";";if(0<=";zmatrix;cfi;c;vfi;v;mnd;jag;adf;gms;g;gau;mp;nw;orc;pqs;qc;".indexOf(a))return"Input";for(var b,d,g=J.adapter.smarter.Resolver.readerSets.length;0<=--g;)if(0<=(d=(b=J.adapter.smarter.Resolver.readerSets[g--]).toLowerCase().indexOf(a)))return b.substring(d+1,b.indexOf(";",d+2));return null},"~S");c$.split=c(c$,"split",function(a,b){for(var d="",g=a.length,c=0,e=0;c<g;c=e)d+=a.substring(c,e=Math.min(c+b,g))+"\n";return d},"~S,~N");c$.DOMResolve=c(c$,"DOMResolve",function(a){var b=
J.adapter.smarter.Resolver.getXmlType(a.get("nameSpaceInfo"));JU.Logger.debugging&&JU.Logger.debug("The Resolver thinks "+b);a.put("readerName",b);return J.adapter.smarter.Resolver.getReader("XmlReader",a)},"java.util.Map");c$.determineAtomSetCollectionReader=c(c$,"determineAtomSetCollectionReader",function(a,b){var d;if(n(a,javajs.api.GenericBinaryDocument))return d=J.adapter.smarter.Resolver.getBinaryType(a.getInputStream()),null==d?"binary file type not recognized":d;if(n(a,java.io.InputStream)){d=
J.adapter.smarter.Resolver.getBinaryType(a);if(null!=d)return d;a=JU.Rdr.getBufferedReader(new java.io.BufferedInputStream(a),null)}var g=new JU.LimitedLineReader(a,16384),c=g.getHeader(64).trim();if(1==c.indexOf("PNG")&&0<=c.indexOf("PNGJ"))return"pngj";if(1==c.indexOf("PNG")||1==c.indexOf("JPG")||6==c.indexOf("JFIF"))return"spt";if(0<=c.indexOf('"num_pairs"'))return"dssr";if(0<=c.indexOf("output.31\n"))return"GenNBO|output.31";if(null!=(d=J.adapter.smarter.Resolver.checkFileStart(c)))return d.equals("Xml")?
J.adapter.smarter.Resolver.getXmlType(g.getHeader(0)):d;for(var e=Array(16),h=0,k=0;k<e.length;++k)e[k]=g.readLineWithNewline(),0<e[k].length&&h++;return null!=(d=J.adapter.smarter.Resolver.checkSpecial1(h,e,c))||null!=(d=J.adapter.smarter.Resolver.checkLineStarts(e))||null!=(d=J.adapter.smarter.Resolver.checkHeaderContains(g.getHeader(0)))||null!=(d=J.adapter.smarter.Resolver.checkSpecial2(e))?d:b?"\n"+e[0]+"\n"+e[1]+"\n"+e[2]+"\n":null},"~O,~B");c$.getBinaryType=c(c$,"getBinaryType",function(a){return JU.Rdr.isPickleS(a)?
"PyMOL":222==(JU.Rdr.getMagic(a,1)[0]&222)?"MMTF":null},"java.io.InputStream");c$.checkFileStart=c(c$,"checkFileStart",function(a){for(var b=0;b<J.adapter.smarter.Resolver.fileStartsWithRecords.length;++b)for(var d=J.adapter.smarter.Resolver.fileStartsWithRecords[b],g=1;g<d.length;++g)if(a.startsWith(d[g]))return d[0];return null},"~S");c$.checkSpecial1=c(c$,"checkSpecial1",function(a,b,d){if(1==a&&0<b[0].length&&JU.PT.isDigit(b[0].charAt(0)))return"Jme";if(J.adapter.smarter.Resolver.checkMopacGraphf(b))return"MopacGraphf";
if(J.adapter.smarter.Resolver.checkOdyssey(b))return"Odyssey";switch(J.adapter.smarter.Resolver.checkMol(b)){case 1:case 3:case 2E3:case 3E3:return"Mol"}switch(J.adapter.smarter.Resolver.checkXyz(b)){case 1:return"Xyz";case 2:return"Bilbao";case 3:return"PWmat"}return J.adapter.smarter.Resolver.checkAlchemy(b[0])?"Alchemy":J.adapter.smarter.Resolver.checkFoldingXyz(b)?"FoldingXyz":J.adapter.smarter.Resolver.checkCube(b)?"Cube":J.adapter.smarter.Resolver.checkWien2k(b)?"Wien2k":J.adapter.smarter.Resolver.checkAims(b)?
"Aims":J.adapter.smarter.Resolver.checkGenNBO(b,d)?"GenNBO":null},"~N,~A,~S");c$.checkAims=c(c$,"checkAims",function(a){for(var b=0;b<a.length&&!a[b].startsWith("mol 1");b++){var d=JU.PT.getTokens(a[b]);if(0!=d.length&&(d[0].startsWith("atom")&&4<d.length&&Float.isNaN(JU.PT.parseFloat(d[4]))||d[0].startsWith("multipole")&&6<=d.length||d[0].startsWith("lattice_vector")&&4<=d.length))return!0}return!1},"~A");c$.checkAlchemy=c(c$,"checkAlchemy",function(a){var b;return 0<=(b=a.indexOf("ATOMS"))&&a.indexOf("BONDS")>
b?0<JU.PT.parseInt(a.substring(0,b).trim()):!1},"~S");c$.isInt=c(c$,"isInt",function(a){J.adapter.smarter.Resolver.n[0]=0;a=a.trim();return 0<a.length&&-2147483648!=JU.PT.parseIntNext(a,J.adapter.smarter.Resolver.n)&&J.adapter.smarter.Resolver.n[0]==a.length},"~S");c$.isFloat=c(c$,"isFloat",function(a){return!Float.isNaN(JU.PT.parseFloat(a))},"~S");c$.checkCube=c(c$,"checkCube",function(a){for(var b=2;5>=b;b++){var d=new java.util.StringTokenizer(a[b]),g=d.countTokens();if(!(4==g||2==b&&5==g)||!J.adapter.smarter.Resolver.isInt(d.nextToken()))return!1;
for(var c=3;0<=--c;)if(!J.adapter.smarter.Resolver.isFloat(d.nextToken()))return!1;if(5==g&&!J.adapter.smarter.Resolver.isInt(d.nextToken()))return!1}return!0},"~A");c$.checkFoldingXyz=c(c$,"checkFoldingXyz",function(a){var b=new java.util.StringTokenizer(a[0].trim()," \t");if(2>b.countTokens()||!J.adapter.smarter.Resolver.isInt(b.nextToken().trim()))return!1;b=a[1].trim();0==b.length&&(b=a[2].trim());b=new java.util.StringTokenizer(b," \t");return 0<b.countTokens()&&J.adapter.smarter.Resolver.isInt(b.nextToken().trim())},
"~A");c$.checkGenNBO=c(c$,"checkGenNBO",function(a,b){return 0<=b.indexOf("$GENNBO")||a[1].startsWith(" Basis set information needed for plotting orbitals")||0<=a[1].indexOf("s in the AO basis:")||0<=a[1].indexOf("***** NBO ")||0<=a[2].indexOf(" N A T U R A L   A T O M I C   O R B I T A L")},"~A,~S");c$.checkMol=c(c$,"checkMol",function(a){var b=("X"+a[3]).trim().toUpperCase();if(7>b.length||0<=b.indexOf(".")||a[0].startsWith("data_"))return 0;if(b.endsWith("V2000"))return 2E3;if(b.endsWith("V3000"))return 3E3;
var b=JU.PT.parseInt(a[3].substring(0,3).trim()),d=JU.PT.parseInt(a[3].substring(3,6).trim());return 0<b&&0<=d&&0!=a[0].indexOf("@<TRIPOS>")&&0!=a[1].indexOf("@<TRIPOS>")&&0!=a[2].indexOf("@<TRIPOS>")?3:0},"~A");c$.checkMopacGraphf=c(c$,"checkMopacGraphf",function(a){return 2<a[0].indexOf("MOPAC-Graphical data")},"~A");c$.checkOdyssey=c(c$,"checkOdyssey",function(a){var b;for(b=0;b<a.length&&(a[b].startsWith("C ")||0==a[b].length);b++);if(b>=a.length||" "!=a[b].charAt(0)||(b+=2)+1>=a.length)return!1;
var d=a[b];if(3>d.length)return!1;var g=JU.PT.parseInt(d.substring(2).trim()),c=JU.PT.parseInt(d.substring(0,2).trim());if(2>(d=a[b+1]).length)return!1;a=JU.PT.parseInt(d.substring(0,2).trim());if(0>g||5<g||0>=a||-2147483648==c||5<c)return!1;d=J.adapter.smarter.AtomSetCollectionReader.getTokensFloat(d,null,5);return!Float.isNaN(d[1])&&!Float.isNaN(d[2])&&!Float.isNaN(d[3])&&Float.isNaN(d[4])},"~A");c$.checkWien2k=c(c$,"checkWien2k",function(a){return a[2].startsWith("MODE OF CALC=")||a[2].startsWith("             RELA")||
a[2].startsWith("             NREL")},"~A");c$.checkXyz=c(c$,"checkXyz",function(a){return J.adapter.smarter.Resolver.isInt(a[0].trim())?J.adapter.smarter.Resolver.isInt(a[2])?2:5<a.length&&8<a[5].length&&a[5].substring(0,8).equalsIgnoreCase("position")?3:1:0<=a[0].indexOf("Bilabao Crys")?2:0},"~A");c$.checkLineStarts=c(c$,"checkLineStarts",function(a){for(var b=0;b<J.adapter.smarter.Resolver.lineStartsWithRecords.length;++b)for(var d=J.adapter.smarter.Resolver.lineStartsWithRecords[b],g=1;g<d.length;++g)for(var c=
d[g],e=0;e<a.length;e++)if(a[e].startsWith(c))return d[0];return null},"~A");c$.checkHeaderContains=c(c$,"checkHeaderContains",function(a){for(var b=0;b<J.adapter.smarter.Resolver.headerContainsRecords.length;++b)for(var d=J.adapter.smarter.Resolver.headerContainsRecords[b],g=1;g<d.length;++g)if(!(0>a.indexOf(d[g])))return b=d[0],!b.equals("Xml")?b:0<=a.indexOf("/AFLOWDATA/")||0<=a.indexOf("-- Structure PRE --")?"AFLOW":0>a.indexOf("<!DOCTYPE HTML PUBLIC")&&0>a.indexOf("XHTML")&&(0>a.indexOf("xhtml")||
0<=a.indexOf("<cml"))?J.adapter.smarter.Resolver.getXmlType(a):null;return null},"~S");c$.getXmlType=c(c$,"getXmlType",function(a){return 0<=a.indexOf("http://www.molpro.net/")?"XmlMolpro":0<=a.indexOf("odyssey")?"XmlOdyssey":0<=a.indexOf("C3XML")?"XmlChem3d":0<=a.indexOf("arguslab")?"XmlArgus":0<=a.indexOf("jvxl")||0<=a.indexOf("http://www.xml-cml.org/schema")||0<=a.indexOf("cml:")||0<=a.indexOf("<cml>")?"XmlCml":0<=a.indexOf("XSD")?"XmlXsd":0<=a.indexOf(">vasp")?"XmlVasp":0<=a.indexOf("<GEOMETRY_INFO>")?
"XmlQE":"XmlCml(unidentified)"},"~S");c$.checkSpecial2=c(c$,"checkSpecial2",function(a){if(J.adapter.smarter.Resolver.checkGromacs(a))return"Gromacs";if(J.adapter.smarter.Resolver.checkCrystal(a))return"Crystal";if(J.adapter.smarter.Resolver.checkFAH(a))return"FAH";a=J.adapter.smarter.Resolver.checkCastepVaspSiesta(a);return null!=a?a:null},"~A");c$.checkFAH=c(c$,"checkFAH",function(a){return(a[0].trim()+a[2].trim()).equals('{"atoms": [')},"~A");c$.checkCrystal=c(c$,"checkCrystal",function(a){var b=
a[1].trim();if(b.equals("SLAB")||b.equals("MOLECULE")||b.equals("CRYSTAL")||b.equals("POLYMER")||(b=a[3]).equals("SLAB")||b.equals("MOLECULE")||b.equals("POLYMER"))return!0;for(b=0;b<a.length;b++)if(a[b].trim().equals("OPTGEOM")||a[b].trim().equals("FREQCALC")||a[b].contains("DOVESI")||a[b].contains("TORINO")||a[b].contains("http://www.crystal.unito.it")||a[b].contains("Pcrystal")||a[b].contains("MPPcrystal")||a[b].contains("crystal executable"))return!0;return!1},"~A");c$.checkGromacs=c(c$,"checkGromacs",
function(a){if(-2147483648==JU.PT.parseInt(a[1]))return!1;for(var b=-1,d=2;16>d&&0!=b;d++)if(69!=(b=a[d].length)&&45!=b&&0!=b)return!1;return!0},"~A");c$.checkCastepVaspSiesta=c(c$,"checkCastepVaspSiesta",function(a){for(var b=0;b<a.length;b++){var d=a[b].toUpperCase();if(1==d.indexOf("FREQUENCIES IN         CM-1")||d.contains("CASTEP")||d.startsWith("%BLOCK LATTICE_ABC")||d.startsWith("%BLOCK LATTICE_CART")||d.startsWith("%BLOCK POSITIONS_FRAC")||d.startsWith("%BLOCK POSITIONS_ABS")||d.contains("<-- E"))return"Castep";
if(d.contains("%BLOCK"))return"Siesta";if(6<=b&&10>b&&(d.startsWith("DIRECT")||d.startsWith("CARTESIAN")))return"VaspPoscar"}return null},"~A");E(c$,"classBase","J.adapter.readers.");c$.readerSets=c$.prototype.readerSets=u(-1,"cif. ;Cif;Cif2;MMCif;MMTF;MagCif molxyz. ;Mol3D;Mol;Xyz; more. ;AFLOW;BinaryDcd;Gromacs;Jcampdx;MdCrd;MdTop;Mol2;TlsDataOnly; quantum. ;Adf;Csf;Dgrid;GamessUK;GamessUS;Gaussian;GaussianFchk;GaussianWfn;Jaguar;Molden;MopacGraphf;GenNBO;NWChem;Psi;Qchem;QCJSON;WebMO;MO; pdb. ;Pdb;Pqr;P2n;JmolData; pymol. ;PyMOL; simple. ;Alchemy;Ampac;Cube;FoldingXyz;GhemicalMM;HyperChem;Jme;JSON;Mopac;MopacArchive;Tinker;Input;FAH; spartan. ;Spartan;SpartanSmol;Odyssey; xtal. ;Abinit;Aims;Bilbao;Castep;Cgd;Crystal;Dmol;Espresso;Gulp;Jana;Magres;Shelx;Siesta;VaspOutcar;VaspPoscar;Wien2k;Xcrysden;PWmat; xml. ;XmlArgus;XmlCml;XmlChem3d;XmlMolpro;XmlOdyssey;XmlXsd;XmlVasp;XmlQE;".split(" "));
E(c$,"CML_NAMESPACE_URI","http://www.xml-cml.org/schema","LEADER_CHAR_MAX",64,"sptRecords",u(-1,["spt","# Jmol state","# Jmol script","JmolManifest"]),"m3dStartRecords",u(-1,["Alchemy","STRUCTURE  1.00     1"]),"cubeFileStartRecords",u(-1,["Cube","JVXL","#JVXL"]),"mol2Records",u(-1,["Mol2","mol2","@<TRIPOS>"]),"webmoFileStartRecords",u(-1,["WebMO","[HEADER]"]),"moldenFileStartRecords",u(-1,["Molden","[Molden","MOLDEN","[MOLDEN"]),"dcdFileStartRecords",u(-1,["BinaryDcd","T\x00\x00\x00CORD","\x00\x00\x00TCORD"]),
"tlsDataOnlyFileStartRecords",u(-1,["TlsDataOnly","REFMAC\n\nTL","REFMAC\r\n\r\n","REFMAC\r\rTL"]),"inputFileStartRecords",u(-1,["Input","#ZMATRIX","%mem=","AM1","$rungauss"]),"magresFileStartRecords",u(-1,["Magres","#$magres","# magres"]),"pymolStartRecords",u(-1,["PyMOL","}q"]),"janaStartRecords",u(-1,["Jana","Version Jana"]),"jsonStartRecords",u(-1,["JSON",'{"mol":']),"jcampdxStartRecords",u(-1,["Jcampdx","##TITLE"]),"jmoldataStartRecords",u(-1,["JmolData","REMARK   6 Jmol"]),"pqrStartRecords",
u(-1,["Pqr","REMARK   1 PQR","REMARK    The B-factors"]),"p2nStartRecords",u(-1,["P2n","REMARK   1 P2N"]),"cif2StartRecords",u(-1,["Cif2","#\\#CIF_2","\u00ef\u00bb\u00bf#\\#CIF_2"]),"xmlStartRecords",u(-1,["Xml","<?xml"]),"cfiStartRecords",u(-1,["Input","$CFI"]));c$.fileStartsWithRecords=c$.prototype.fileStartsWithRecords=u(-1,[J.adapter.smarter.Resolver.xmlStartRecords,J.adapter.smarter.Resolver.sptRecords,J.adapter.smarter.Resolver.m3dStartRecords,J.adapter.smarter.Resolver.cubeFileStartRecords,
J.adapter.smarter.Resolver.mol2Records,J.adapter.smarter.Resolver.webmoFileStartRecords,J.adapter.smarter.Resolver.moldenFileStartRecords,J.adapter.smarter.Resolver.dcdFileStartRecords,J.adapter.smarter.Resolver.tlsDataOnlyFileStartRecords,J.adapter.smarter.Resolver.inputFileStartRecords,J.adapter.smarter.Resolver.magresFileStartRecords,J.adapter.smarter.Resolver.pymolStartRecords,J.adapter.smarter.Resolver.janaStartRecords,J.adapter.smarter.Resolver.jsonStartRecords,J.adapter.smarter.Resolver.jcampdxStartRecords,
J.adapter.smarter.Resolver.jmoldataStartRecords,J.adapter.smarter.Resolver.pqrStartRecords,J.adapter.smarter.Resolver.p2nStartRecords,J.adapter.smarter.Resolver.cif2StartRecords,J.adapter.smarter.Resolver.cfiStartRecords]);E(c$,"n",B(1,0),"mmcifLineStartRecords",u(-1,"MMCif _entry.id _database_PDB_ _pdbx_ _chem_comp.pdbx_type _audit_author.name _atom_site.".split(" ")),"cifLineStartRecords",u(-1,["Cif","data_","_publ"]),"pdbLineStartRecords",u(-1,"Pdb;HEADER;OBSLTE;TITLE ;CAVEAT;COMPND;SOURCE;KEYWDS;EXPDTA;AUTHOR;REVDAT;SPRSDE;JRNL  ;REMARK ;DBREF ;SEQADV;SEQRES;MODRES;HELIX ;SHEET ;TURN  ;CRYST1;ORIGX1;ORIGX2;ORIGX3;SCALE1;SCALE2;SCALE3;ATOM  ;HETATM;MODEL ;LINK  ;USER  MOD ".split(";")),
"cgdLineStartRecords",u(-1,["Cgd","EDGE ","edge "]),"shelxLineStartRecords",u(-1,"Shelx;TITL ;ZERR ;LATT ;SYMM ;CELL ".split(";")),"ghemicalMMLineStartRecords",u(-1,["GhemicalMM","!Header mm1gp","!Header gpr"]),"jaguarLineStartRecords",u(-1,["Jaguar","  |  Jaguar version"]),"mdlLineStartRecords",u(-1,["Mol","$MDL "]),"spartanSmolLineStartRecords",u(-1,["SpartanSmol","INPUT="]),"csfLineStartRecords",u(-1,["Csf","local_transform"]),"mdTopLineStartRecords",u(-1,["MdTop","%FLAG TITLE"]),"hyperChemLineStartRecords",
u(-1,["HyperChem","mol 1"]),"vaspOutcarLineStartRecords",u(-1,["VaspOutcar"," vasp."," INCAR:"]));c$.lineStartsWithRecords=c$.prototype.lineStartsWithRecords=u(-1,[J.adapter.smarter.Resolver.mmcifLineStartRecords,J.adapter.smarter.Resolver.cifLineStartRecords,J.adapter.smarter.Resolver.pdbLineStartRecords,J.adapter.smarter.Resolver.cgdLineStartRecords,J.adapter.smarter.Resolver.shelxLineStartRecords,J.adapter.smarter.Resolver.ghemicalMMLineStartRecords,J.adapter.smarter.Resolver.jaguarLineStartRecords,
J.adapter.smarter.Resolver.mdlLineStartRecords,J.adapter.smarter.Resolver.spartanSmolLineStartRecords,J.adapter.smarter.Resolver.csfLineStartRecords,J.adapter.smarter.Resolver.mol2Records,J.adapter.smarter.Resolver.mdTopLineStartRecords,J.adapter.smarter.Resolver.hyperChemLineStartRecords,J.adapter.smarter.Resolver.vaspOutcarLineStartRecords]);E(c$,"bilbaoContainsRecords",u(-1,["Bilbao",">Bilbao Crystallographic Server<"]),"xmlContainsRecords",u(-1,'Xml <?xml <atom <molecule <reaction <cml <bond .dtd" <list> <entry <identifier http://www.xml-cml.org/schema/cml2/core'.split(" ")),
"gaussianContainsRecords",u(-1,["Gaussian","Entering Gaussian Zystem","Entering Link 1","1998 Gaussian, Inc."]),"ampacContainsRecords",u(-1,["Ampac","AMPAC Version"]),"mopacContainsRecords",u(-1,"Mopac;MOPAC 93 (c) Fujitsu;MOPAC FOR LINUX (PUBLIC DOMAIN VERSION);MOPAC:  VERSION  6;MOPAC   7;MOPAC2;MOPAC (PUBLIC".split(";")),"qchemContainsRecords",u(-1,["Qchem","Welcome to Q-Chem","A Quantum Leap Into The Future Of Chemistry"]),"gamessUKContainsRecords",u(-1,["GamessUK","GAMESS-UK","G A M E S S - U K"]),
"gamessUSContainsRecords",u(-1,["GamessUS","GAMESS","$CONTRL"]),"spartanBinaryContainsRecords",u(-1,["SpartanSmol","|PropertyArchive","_spartan","spardir","BEGIN Directory Entry Molecule"]),"spartanContainsRecords",u(-1,["Spartan","Spartan","converted archive file"]),"adfContainsRecords",u(-1,["Adf","Amsterdam Density Functional"]),"psiContainsRecords",u(-1,["Psi","    PSI  3","PSI3:"]),"nwchemContainsRecords",u(-1,["NWChem"," argument  1 = "]),"uicrcifContainsRecords",u(-1,["Cif","Crystallographic Information File"]),
"dgridContainsRecords",u(-1,["Dgrid","BASISFILE   created by DGrid"]),"crystalContainsRecords",u(-1,["Crystal","*                                CRYSTAL","TORINO","DOVESI"]),"dmolContainsRecords",u(-1,["Dmol","DMol^3"]),"gulpContainsRecords",u(-1,["Gulp","GENERAL UTILITY LATTICE PROGRAM"]),"espressoContainsRecords",u(-1,["Espresso","Program PWSCF","Program PHONON"]),"siestaContainsRecords",u(-1,["Siesta","MD.TypeOfRun","SolutionMethod","MeshCutoff","WELCOME TO SIESTA"]),"xcrysDenContainsRecords",
u(-1,["Xcrysden","PRIMVEC","CONVVEC","PRIMCOORD","ANIMSTEP"]),"mopacArchiveContainsRecords",u(-1,["MopacArchive","SUMMARY OF PM"]),"abinitContainsRecords",u(-1,["Abinit","http://www.abinit.org","Catholique","Louvain"]),"qcJsonContainsRecords",u(-1,["QCJSON",'"QCJSON']),"gaussianFchkContainsRecords",u(-1,["GaussianFchk","Number of point charges in /Mol/"]),"inputContainsRecords",u(-1,"Input; ATOMS cartesian;$molecule;&zmat;geometry={;$DATA;%coords;GEOM=PQS;geometry units angstroms".split(";")),"aflowContainsRecords",
u(-1,["AFLOW","/AFLOWDATA/"]),"magCifContainsRecords",u(-1,["MagCif","_space_group_magn"]));c$.headerContainsRecords=c$.prototype.headerContainsRecords=u(-1,[J.adapter.smarter.Resolver.sptRecords,J.adapter.smarter.Resolver.bilbaoContainsRecords,J.adapter.smarter.Resolver.xmlContainsRecords,J.adapter.smarter.Resolver.gaussianContainsRecords,J.adapter.smarter.Resolver.ampacContainsRecords,J.adapter.smarter.Resolver.mopacContainsRecords,J.adapter.smarter.Resolver.gamessUKContainsRecords,J.adapter.smarter.Resolver.gamessUSContainsRecords,
J.adapter.smarter.Resolver.qchemContainsRecords,J.adapter.smarter.Resolver.spartanBinaryContainsRecords,J.adapter.smarter.Resolver.spartanContainsRecords,J.adapter.smarter.Resolver.mol2Records,J.adapter.smarter.Resolver.adfContainsRecords,J.adapter.smarter.Resolver.psiContainsRecords,J.adapter.smarter.Resolver.nwchemContainsRecords,J.adapter.smarter.Resolver.uicrcifContainsRecords,J.adapter.smarter.Resolver.dgridContainsRecords,J.adapter.smarter.Resolver.crystalContainsRecords,J.adapter.smarter.Resolver.dmolContainsRecords,
J.adapter.smarter.Resolver.gulpContainsRecords,J.adapter.smarter.Resolver.espressoContainsRecords,J.adapter.smarter.Resolver.siestaContainsRecords,J.adapter.smarter.Resolver.xcrysDenContainsRecords,J.adapter.smarter.Resolver.mopacArchiveContainsRecords,J.adapter.smarter.Resolver.abinitContainsRecords,J.adapter.smarter.Resolver.gaussianFchkContainsRecords,J.adapter.smarter.Resolver.inputContainsRecords,J.adapter.smarter.Resolver.aflowContainsRecords,J.adapter.smarter.Resolver.magCifContainsRecords,
J.adapter.smarter.Resolver.qcJsonContainsRecords])});s("J.adapter.smarter");v(["J.api.JmolAdapter"],"J.adapter.smarter.SmarterJmolAdapter","java.io.BufferedReader $.InputStream java.lang.UnsupportedOperationException javajs.api.GenericBinaryDocument JU.PT $.Rdr J.adapter.smarter.AtomIterator $.AtomSetCollection $.AtomSetCollectionReader $.BondIterator $.Resolver $.StructureIterator JS.SV JU.Logger JV.Viewer".split(" "),function(){c$=D(J.adapter.smarter,"SmarterJmolAdapter",J.api.JmolAdapter);q(c$,
function(){I(this,J.adapter.smarter.SmarterJmolAdapter,[])});j(c$,"getFileTypeName",function(a){return n(a,J.adapter.smarter.AtomSetCollection)?a.fileTypeName:n(a,java.io.BufferedReader)?J.adapter.smarter.Resolver.getFileType(a):n(a,java.io.InputStream)?J.adapter.smarter.Resolver.getBinaryType(a):null},"~O");j(c$,"getAtomSetCollectionReader",function(a,b,d,g){return J.adapter.smarter.SmarterJmolAdapter.staticGetAtomSetCollectionReader(a,b,d,g)},"~S,~S,~O,java.util.Map");c$.staticGetAtomSetCollectionReader=
c(c$,"staticGetAtomSetCollectionReader",function(a,b,d,g){try{var c=J.adapter.smarter.Resolver.getAtomCollectionReader(a,b,d,g,-1);if(n(c,String))try{J.adapter.smarter.SmarterJmolAdapter.close(d)}catch(e){if(!H(e,Exception))throw e;}else c.setup(a,g,d);return c}catch(h){try{J.adapter.smarter.SmarterJmolAdapter.close(d)}catch(k){if(!H(k,Exception))throw k;}JU.Logger.error(""+h);return""+h}},"~S,~S,~O,java.util.Map");j(c$,"getAtomSetCollectionFromReader",function(a,b,d){var g=J.adapter.smarter.Resolver.getAtomCollectionReader(a,
null,b,d,-1);return n(g,J.adapter.smarter.AtomSetCollectionReader)?(g.setup(a,d,b),g.readData()):""+g},"~S,~O,java.util.Map");j(c$,"getAtomSetCollection",function(a){return J.adapter.smarter.SmarterJmolAdapter.staticGetAtomSetCollection(a)},"~O");c$.staticGetAtomSetCollection=c(c$,"staticGetAtomSetCollection",function(a){var b=null;try{var b=a.reader,d=a.readData();return!n(d,J.adapter.smarter.AtomSetCollection)?d:null!=d.errorMessage?d.errorMessage:d}catch(g){try{JU.Logger.info(g.toString())}catch(c){if(H(c,
Exception))JU.Logger.error(g.toString());else throw c;}try{b.close()}catch(e){if(!H(e,Exception))throw e;}JU.Logger.error(""+g);return""+g}},"J.adapter.smarter.AtomSetCollectionReader");j(c$,"getAtomSetCollectionReaders",function(a,b,d,g,c){var e=g.get("vwr"),h=b.length,k=null;if(g.containsKey("concatenate")){for(var k="",l=0;l<h;l++){var j=b[l],m=e.getFileAsString3(j,!1,null);if(0<l&&3>=h&&m.startsWith("{"))j=m.contains('version":"DSSR')?"dssr":m.contains("/outliers/")?"validation":"domains",m=e.parseJSONMap(m),
null!=m&&g.put(j,j.equals("dssr")?m:JS.SV.getVariableMap(m));else{0<=j.indexOf("|")&&(j=JU.PT.rep(j,"_","/"));if(1==l){if(0<=j.indexOf("/rna3dhub/")){k+="\n_rna3d \n;"+m+"\n;\n";continue}if(0<=j.indexOf("/dssr/")){k+="\n_dssr \n;"+m+"\n;\n";continue}}k+=m;k.endsWith("\n")||(k+="\n")}}h=1;k=JU.Rdr.getBR(k)}for(var m=c?Array(h):null,j=c?null:Array(h),r=null,l=0;l<h;l++)try{g.put("vwr",e);null==k&&(k=a.getBufferedReaderOrBinaryDocument(l,!1));if(!n(k,java.io.BufferedReader)&&!n(k,javajs.api.GenericBinaryDocument))return k;
g.put("fullPathName",b[l]);var x=J.adapter.smarter.Resolver.getAtomCollectionReader(b[l],null==d?null:d[l],k,g,l);if(!n(x,J.adapter.smarter.AtomSetCollectionReader))return x;r=x;r.setup(null,null,null);r.isBinary?r.setup(b[l],g,a.getBufferedReaderOrBinaryDocument(l,!0)):r.setup(b[l],g,k);k=null;if(c)m[l]=r;else{x=r.readData();if(!n(x,J.adapter.smarter.AtomSetCollection))return x;j[l]=x;if(null!=j[l].errorMessage)return j[l].errorMessage}}catch(y){return JU.Logger.error(""+y),JV.Viewer.isJS||y.printStackTrace(),
""+y}return c?m:this.getAtomSetCollectionFromSet(m,j,g)},"J.api.JmolFilesReaderInterface,~A,~A,java.util.Map,~B");j(c$,"getAtomSetCollectionFromSet",function(a,b,d){var g=null==b?Array(a.length):b;if(null==b)for(b=0;b<a.length;b++)if(null!=a[b])try{var c=a[b].readData();if(!n(c,J.adapter.smarter.AtomSetCollection))return c;g[b]=c;if(null!=g[b].errorMessage)return g[b].errorMessage}catch(e){return JU.Logger.error(""+e),""+e}if(d.containsKey("trajectorySteps")){a=g[0];try{1<g.length&&g[0].setInfo("ignoreUnitCell",
g[1].atomSetInfo.get("ignoreUnitCell")),a.finalizeTrajectoryAs(d.get("trajectorySteps"),d.get("vibrationSteps"))}catch(h){if(H(h,Exception))null==a.errorMessage&&(a.errorMessage=""+h);else throw h;}}else if(g[0].isTrajectory){a=g[0];for(b=1;b<g.length;b++)g[0].mergeTrajectories(g[b])}else a=1==g.length?g[0]:new J.adapter.smarter.AtomSetCollection("Array",null,g,null);return null==a.errorMessage?a:a.errorMessage},"~O,~O,java.util.Map");j(c$,"getAtomSetCollectionFromDOM",function(){throw new UnsupportedOperationException;
},"~O,java.util.Map");j(c$,"finish",function(a){a.finish()},"~O");j(c$,"getAtomSetCollectionName",function(a){return a.collectionName},"~O");j(c$,"getAtomSetCollectionAuxiliaryInfo",function(a){return a.atomSetInfo},"~O");j(c$,"getAtomSetCount",function(a){return a.atomSetCount},"~O");j(c$,"getAtomSetNumber",function(a,b){return a.getAtomSetNumber(b)},"~O,~N");j(c$,"getAtomSetName",function(a,b){return a.getAtomSetName(b)},"~O,~N");j(c$,"getAtomSetAuxiliaryInfo",function(a,b){return a.getAtomSetAuxiliaryInfo(b)},
"~O,~N");j(c$,"getHydrogenAtomCount",function(a){return a.getHydrogenAtomCount()},"~O");j(c$,"getBondList",function(a){return a.getBondList()},"~O");j(c$,"getAtomCount",function(a){return null==a.bsAtoms?a.ac:a.bsAtoms.cardinality()},"~O");j(c$,"coordinatesAreFractional",function(a){return a.coordinatesAreFractional},"~O");j(c$,"getAtomIterator",function(a){return new J.adapter.smarter.AtomIterator(a)},"~O");j(c$,"getBondIterator",function(a){return new J.adapter.smarter.BondIterator(a)},"~O");j(c$,
"getStructureIterator",function(a){return 0==a.structureCount?null:new J.adapter.smarter.StructureIterator(a)},"~O");c$.close=c(c$,"close",function(a){n(a,java.io.BufferedReader);a.close()},"~O");E(c$,"PATH_KEY",".PATH");c$.PATH_SEPARATOR=c$.prototype.PATH_SEPARATOR=Zystem.getProperty("path.separator","/")});s("J.adapter.smarter");v(["J.api.JmolAdapterStructureIterator"],"J.adapter.smarter.StructureIterator",["J.api.JmolAdapter"],function(){c$=t(function(){this.structureCount=0;this.structure=this.structures=
null;this.istructure=0;this.bsModelsDefined=null;p(this,arguments)},J.adapter.smarter,"StructureIterator",J.api.JmolAdapterStructureIterator);q(c$,function(a){I(this,J.adapter.smarter.StructureIterator,[]);this.structureCount=a.structureCount;this.structures=a.structures;this.istructure=0;this.bsModelsDefined=a.bsStructuredModels},"J.adapter.smarter.AtomSetCollection");j(c$,"hasNext",function(){if(this.istructure==this.structureCount)return!1;this.structure=this.structures[this.istructure++];return!0});
j(c$,"getStructureType",function(){return this.structure.structureType});j(c$,"getSubstructureType",function(){return this.structure.substructureType});j(c$,"getStructureID",function(){return this.structure.structureID});j(c$,"getSerialID",function(){return this.structure.serialID});j(c$,"getStartChainID",function(){return this.structure.startChainID});j(c$,"getStartSequenceNumber",function(){return this.structure.startSequenceNumber});j(c$,"getStartInsertionCode",function(){return J.api.JmolAdapter.canonizeInsertionCode(this.structure.startInsertionCode)});
j(c$,"getEndChainID",function(){return this.structure.endChainID});j(c$,"getEndSequenceNumber",function(){return this.structure.endSequenceNumber});j(c$,"getEndInsertionCode",function(){return this.structure.endInsertionCode});j(c$,"getStrandCount",function(){return this.structure.strandCount});j(c$,"getStructuredModels",function(){return this.bsModelsDefined});j(c$,"getAtomIndices",function(){return this.structure.atomStartEnd});j(c$,"getModelIndices",function(){return this.structure.modelStartEnd});
j(c$,"getBSAll",function(){return this.structure.bsAll})});s("J.api");N(J.api,"AtomIndexIterator");s("J.api");v(null,"J.api.Interface",["JU.Logger"],function(){c$=D(J.api,"Interface");c$.getInterface=c(c$,"getInterface",function(a){try{var b=null,b=ea._4Name(a);return null==b?null:b.newInstance()}catch(d){if(H(d,Exception))return JU.Logger.error("Interface.java Error creating instance for "+a+": \n"+d),null;throw d;}},"~S,JV.Viewer,~S");c$.getOption=c(c$,"getOption",function(a,b,d){return J.api.Interface.getInterface("J."+
a,b,d)},"~S,JV.Viewer,~S");c$.getUtil=c(c$,"getUtil",function(a,b,d){return J.api.Interface.getInterface("JU."+a,b,d)},"~S,JV.Viewer,~S");c$.getSymmetry=c(c$,"getSymmetry",function(a,b){return J.api.Interface.getInterface("JS.Symmetry",a,b)},"JV.Viewer,~S")});s("J.api");v(null,"J.api.JmolAdapter",["JU.PT","J.api.JmolViewer","JU.Elements"],function(){c$=D(J.api,"JmolAdapter");c$.getElementSymbol=c(c$,"getElementSymbol",function(a){return JU.Elements.elementSymbolFromNumber(a)},"~N");c$.getElementNumber=
c(c$,"getElementNumber",function(a){return JU.Elements.elementNumberFromSymbol(a,!1)},"~S");c$.getNaturalIsotope=c(c$,"getNaturalIsotope",function(a){return JU.Elements.getNaturalIsotope(a)},"~N");c$.getBondingRadius=c(c$,"getBondingRadius",function(a,b){return JU.Elements.getBondingRadius(a,b)},"~N,~N");c(c$,"getAtomSetCollectionFromReaderType",function(a,b,d,g){a=this.getAtomSetCollectionReader(a,b,d,J.api.JmolViewer.allocateViewer(null,this).setLoadParameters(g,!1));return n(a,String)?a:this.getAtomSetCollection(a)},
"~S,~S,~O,java.util.Map");c(c$,"openBufferedReader",function(a,b){return this.getAtomSetCollectionFromReaderType(a,null,b,null)},"~S,java.io.BufferedReader");c(c$,"openBufferedReader",function(a,b,d){return this.getAtomSetCollectionFromReaderType(a,null,b,d)},"~S,java.io.BufferedReader,java.util.Map");c(c$,"openBufferedReader",function(a,b,d){return this.getAtomSetCollectionFromReaderType(a,b,d,null)},"~S,~S,java.io.BufferedReader");c$.canonizeAlphaDigit=c(c$,"canonizeAlphaDigit",function(a){return JU.PT.isLetterOrDigit(a)?
a:"\x00"},"~S");c$.canonizeInsertionCode=c(c$,"canonizeInsertionCode",function(a){return J.api.JmolAdapter.canonizeAlphaDigit(a)},"~S");c$.canonizeAlternateLocationID=c(c$,"canonizeAlternateLocationID",function(a){return J.api.JmolAdapter.canonizeAlphaDigit(a)},"~S");E(c$,"ORDER_COVALENT_SINGLE",1,"ORDER_COVALENT_DOUBLE",2,"ORDER_COVALENT_TRIPLE",3,"ORDER_COVALENT_QUAD",4,"ORDER_COVALENT_QUINT",5,"ORDER_COVALENT_HEX",6,"ORDER_AROMATIC",515,"ORDER_AROMATIC_SINGLE",513,"ORDER_AROMATIC_DOUBLE",514,"ORDER_HBOND",
2048,"ORDER_STEREO_NEAR",1025,"ORDER_STEREO_FAR",1041,"ORDER_PARTIAL01",33,"ORDER_PARTIAL12",66,"ORDER_PARTIAL23",97,"ORDER_PARTIAL32",100,"ORDER_UNSPECIFIED",17,"ORDER_PYMOL_SINGLE",65536,"ORDER_PYMOL_MULT",98304,"cellParamNames",u(-1,"_cell_length_a _cell_length_b _cell_length_c _cell_angle_alpha _cell_angle_beta _cell_angle_gamma".split(" ")))});s("J.api");N(J.api,"JmolAdapterAtomIterator");s("J.api");c$=D(J.api,"JmolAdapterBondIterator");s("J.api");c$=D(J.api,"JmolAdapterStructureIterator");s("J.api");
N(J.api,"JmolAppConsoleInterface");s("J.api");v(["J.api.JmolSyncInterface"],"J.api.JmolAppletInterface",null,function(){N(J.api,"JmolAppletInterface",J.api.JmolSyncInterface)});s("J.api");N(J.api,"JmolCallbackListener");s("J.api");N(J.api,"JmolAnnotationParser");s("J.api");N(J.api,"JmolGraphicsInterface");s("J.api");N(J.api,"JmolMeasurementClient");s("J.api");N(J.api,"JmolModulationSet");s("J.api");N(J.api,"JmolNavigatorInterface",Runnable);s("J.api");v(["J.api.JmolGraphicsInterface"],"J.api.JmolRendererInterface",
null,function(){N(J.api,"JmolRendererInterface",J.api.JmolGraphicsInterface)});s("J.api");N(J.api,"JmolRepaintManager");s("J.api");v(["J.api.JmolCallbackListener"],"J.api.JmolStatusListener",null,function(){N(J.api,"JmolStatusListener",J.api.JmolCallbackListener)});s("J.api");N(J.api,"JmolSyncInterface");s("J.api");v(null,"J.api.JmolViewer",["java.lang.Boolean","java.util.Hashtable"],function(){c$=t(function(){this.apiPlatform=this.menuStructure=null;p(this,arguments)},J.api,"JmolViewer");c$.allocateViewer=
c(c$,"allocateViewer",function(a,b,d,g,c,e,h,k){var l=new java.util.Hashtable;null!=a&&l.put("display",a);null!=b&&l.put("adapter",b);null!=h&&l.put("statuslistener",h);null!=k&&l.put("platform",k);null!=e&&l.put("options",e);null!=d&&l.put("fullname",d);null!=g&&l.put("documentbase",g);null!=c&&l.put("codebase",c);l.put("isApp",Boolean.TRUE);return new JV.Viewer(l)},"~O,J.api.JmolAdapter,~S,java.net.URL,java.net.URL,~S,J.api.JmolStatusListener,J.api.GenericPlatform");c$.allocateViewer=c(c$,"allocateViewer",
function(a,b){return J.api.JmolViewer.allocateViewer(a,b,null,null,null,null,null,null)},"~O,J.api.JmolAdapter");c$.allocateViewer=c(c$,"allocateViewer",function(a,b,d,g,c,e,h){return J.api.JmolViewer.allocateViewer(a,b,d,g,c,e,h,null)},"~O,J.api.JmolAdapter,~S,java.net.URL,java.net.URL,~S,J.api.JmolStatusListener");c(c$,"setConsole",function(a){this.getProperty("DATA_API","getAppConsole",a)},"J.api.JmolAppConsoleInterface");c$.getJmolVersion=c(c$,"getJmolVersion",function(){return JV.Viewer.getJmolVersion()});
c(c$,"openReader",function(a,b){return this.openReader(null==a?"String":a,null,b)},"~S,~O");c(c$,"openFileAsync",function(a){this.openFileAsyncSpecial(a,0)},"~S");c(c$,"renderScreenImage",function(a,b){this.apiPlatform.renderScreenImage(a,b)},"~O,~O,~O");c(c$,"runScriptCautiously",function(){return null},"~S");c(c$,"dispose",function(){})});s("J.api");N(J.api,"Translator");s("J.appletjs");v(["javajs.api.JSInterface","JU.GenericApplet"],"J.appletjs.Jmol",["java.util.Hashtable","JU.PT","J.awtjs2d.Platform",
"JU.Logger","$.Parser"],function(){c$=t(function(){this.htParams=null;p(this,arguments)},J.appletjs,"Jmol",JU.GenericApplet,javajs.api.JSInterface);q(c$,function(a){I(this,J.appletjs.Jmol,[]);this.htParams=new java.util.Hashtable;null==a&&(a=new java.util.Hashtable);this.vwrOptions=a;for(var b,d=a.entrySet().iterator();d.hasNext()&&((b=d.next())||1);)this.htParams.put(b.getKey().toLowerCase(),b.getValue());this.documentBase=""+a.get("documentBase");this.codeBase=""+a.get("codePath");this.isJS=!0;
this.init(this)},"java.util.Map");j(c$,"setStereoGraphics",function(a){return a?vwr.apiPlatform.context:null},"~B");j(c$,"initOptions",function(){this.vwrOptions.remove("debug");this.vwrOptions.put("fullName",this.fullName);this.haveDocumentAccess="true".equalsIgnoreCase(""+this.getValue("allowjavascript","true"));this.mayScript=!0});j(c$,"getJmolParameter",function(a){a=this.htParams.get(a.toLowerCase());return null==a?null:""+a},"~S");j(c$,"doSendJsTextStatus",function(a){Zystem.out.println(a)},
"~S");j(c$,"doSendJsTextareaStatus",function(a){Zystem.out.println(a)},"~S");j(c$,"doFunctionXY",function(a,b,d){var g=K(Math.abs(b),Math.abs(d),0);if(!this.mayScript||!this.haveDocumentAccess||0==b||0==d)return g;try{if(0<b&&0<d)for(var c=0;c<b;c++)for(var e=0;e<d;e++)g[c][e]=window.eval(a)(this.htmlName,c,e);else if(0<d){var h;h=window.eval(a)(this.htmlName,b,d);b=Math.abs(b);var k=K(b*d,0);JU.Parser.parseStringInfestedFloatArray(h,null,k);for(a=c=0;c<b;c++)for(e=0;e<d;e++,a++)g[c][e]=k[a]}else window.eval(a)(this.htmlName,
b,d,g)}catch(l){if(H(l,Exception))JU.Logger.error("Exception "+l+" with nX, nY: "+b+" "+d);else throw l;}return g},"~S,~N,~N");j(c$,"doFunctionXYZ",function(a,b,d,g){var c=K(Math.abs(b),Math.abs(d),Math.abs(g),0);if(!this.mayScript||!this.haveDocumentAccess||0==b||0==d||0==g)return c;try{window.eval(a)(this.htmlName,b,d,g,c)}catch(e){if(H(e,Exception))JU.Logger.error("Exception "+e+" for "+a+" with nX, nY, nZ: "+b+" "+d+" "+g);else throw e;}return c},"~S,~N,~N,~N");j(c$,"doShowDocument",function(a){var b=
JU.PT.split(a.toString(),"?POST?");if(1==b.length)window.open(b[0]);else{a="<form id=f method=POST action='"+b[0]+"'>";a+="<input type='hidden' name='name' value='nmr-1h-prediction' id='name'><input type='submit' value='working...'>";for(var b=b[1].$plit("&"),d=0;d<b.length;d++){var g=b[d],c=g.indexOf("="),e=g.substring(0,c),g=g.substring(c);a=0<=g.indexOf("\n")?a+("<textarea style='display:none' name="+e+">"+g+"</textarea>"):a+("<input type=hidden name="+e+' value="'+g+'">')}a+="</form>";b=window.open("");
b.document.write(a);b.document.getElementById("f").submit()}},"java.net.URL");j(c$,"doSendCallback",function(a,b,d){if(!(null==a||0==a.length))if(a.equals("alert"))alert(d);else{d=JU.PT.split(a,".");try{for(var g=window[d[0]],c=1;c<d.length;c++)g=g[d[c]];for(c=0;c<b.length;c++)b[c]&&b[c].booleanValue&&(b[c]=b[c].booleanValue());return g.apply(null,b)}catch(e){Zystem.out.println(a+" failed "+e)}}return""},"~S,~A,~S");j(c$,"doEval",function(a){try{return window.eval(a)}catch(b){if(H(b,Exception))JU.Logger.error("# error evaluating "+
a+":"+b.toString());else throw b;}return""},"~S");j(c$,"doShowStatus",function(a){try{Zystem.out.println(a)}catch(b){if(!H(b,Exception))throw b;}},"~S");c(c$,"getGLmolView",function(){return this.viewer.getGLmolView()});c(c$,"openFile",function(a){return this.viewer.openFile(a)},"~S");j(c$,"cacheFileByName",function(a,b){return this.viewer.cacheFileByName(a,b)},"~S,~B");j(c$,"cachePut",function(a,b){this.viewer.cachePut(a,b)},"~S,~O");j(c$,"getFullName",function(){return this.fullName});j(c$,"processMouseEvent",
function(a,b,d,g,c){return this.viewer.processMouseEvent(a,b,d,g,c)},"~N,~N,~N,~N,~N");j(c$,"setDisplay",function(a){this.viewer.setDisplay(a)},"~O");j(c$,"setStatusDragDropped",function(a,b,d,g){return this.viewer.setStatusDragDropped(a,b,d,g)},"~N,~N,~N,~S");j(c$,"startHoverWatcher",function(a){this.viewer.startHoverWatcher(a)},"~B");j(c$,"update",function(){this.viewer.updateJS()});j(c$,"openFileAsyncSpecial",function(a,b){this.viewer.openFileAsyncSpecial(a,b)},"~S,~N");j(c$,"processTwoPointGesture",
function(a){this.viewer.processTwoPointGesture(a)},"~A");j(c$,"setScreenDimension",function(a,b){this.viewer.setScreenDimension(a,b)},"~N,~N");j(c$,"resizeInnerPanel",function(a){var b=K(2,0);JU.Parser.parseStringInfestedFloatArray(a,null,b);this.resizeDisplay(A(b[0]),A(b[1]));return B(-1,[A(b[0]),A(b[1])])},"~S");c(c$,"resizeDisplay",function(a,b){J.awtjs2d.Platform.Jmol().resizeApplet(this.viewer.html5Applet,B(-1,[a,b]))},"~N,~N")});s("J.atomdata");v(null,"J.atomdata.AtomData",["JU.P3","JU.BSUtil"],
function(){c$=t(function(){this.modelName=this.fileName=this.programInfo=null;this.modelIndex=0;this.radiusData=this.bsMolecules=this.bsIgnored=this.bsSelected=null;this.hAtomRadius=this.lastModelIndex=this.firstModelIndex=this.firstAtomIndex=0;this.hAtoms=this.atomMolecule=this.atomicNumber=this.atomRadius=this.xyz=this.atoms=this.atomIndex=null;this.adpMode=this.hydrogenAtomCount=this.ac=0;p(this,arguments)},J.atomdata,"AtomData");q(c$,function(){});c(c$,"transformXYZ",function(a,b){var d=Array(this.xyz.length);
null==b&&(b=JU.BSUtil.newBitSet2(0,this.xyz.length));for(var g=b.nextSetBit(0);0<=g;g=b.nextSetBit(g+1))null!=this.xyz[g]&&(d[g]=JU.P3.newP(this.xyz[g]),a.rotTrans(d[g]));this.xyz=d},"JU.M4,JU.BS");E(c$,"MODE_FILL_MODEL",0,"MODE_FILL_COORDS",1,"MODE_FILL_RADII",2,"MODE_FILL_MOLECULES",4,"MODE_GET_ATTACHED_HYDROGENS",8,"MODE_FILL_MULTIMODEL",16)});s("J.atomdata");N(J.atomdata,"AtomDataServer");s("J.atomdata");v(["java.lang.Enum","J.c.VDW"],"J.atomdata.RadiusData",["java.lang.Float","JU.SB"],function(){c$=
t(function(){this.vdwType=this.factorType=this.info=null;this.value=NaN;this.valueExtended=0;this.values=null;p(this,arguments)},J.atomdata,"RadiusData");O(c$,function(){this.factorType=J.atomdata.RadiusData.EnumType.ABSOLUTE;this.vdwType=J.c.VDW.AUTO});q(c$,function(a,b,d,g){null!=a?(this.values=a,this.value=2147483647):null!=d&&(this.factorType=d,this.value=b,null!=g&&(this.vdwType=g))},"~A,~N,J.atomdata.RadiusData.EnumType,J.c.VDW");j(c$,"toString",function(){if(Float.isNaN(this.value))return"";
var a=new JU.SB;switch(this.factorType){case J.atomdata.RadiusData.EnumType.ABSOLUTE:a.appendF(this.value);break;case J.atomdata.RadiusData.EnumType.OFFSET:a.append(0<this.value?"+":"").appendF(this.value);break;case J.atomdata.RadiusData.EnumType.FACTOR:a.appendI(A(100*this.value)).append("%");this.vdwType!==J.c.VDW.AUTO&&a.append(this.vdwType.getVdwLabel());break;case J.atomdata.RadiusData.EnumType.SCREEN:a.appendI(A(this.value))}return a.toString()});M(self.c$);c$=D(J.atomdata.RadiusData,"EnumType",
Enum);G(c$,"ABSOLUTE",0,[]);G(c$,"OFFSET",1,[]);G(c$,"FACTOR",2,[]);G(c$,"SCREEN",3,[]);c$=L()});s("J.awtjs");v(["J.awtjs2d.Platform"],"J.awtjs.Platform",null,function(){c$=D(J.awtjs,"Platform",J.awtjs2d.Platform);j(c$,"drawImage",function(){},"~O,~O,~N,~N,~N,~N,~B");j(c$,"getTextPixels",function(){return null},"~S,JU.Font,~O,~O,~N,~N,~N");j(c$,"getGraphics",function(){return null},"~O");j(c$,"getStaticGraphics",function(){return null},"~O,~B");j(c$,"newBufferedImage",function(){return null},"~O,~N,~N");
j(c$,"newOffScreenImage",function(){return null},"~N,~N")});s("J.awtjs2d");c$=D(J.awtjs2d,"Display");c$.getFullScreenDimensions=c(c$,"getFullScreenDimensions",function(a,b){b[0]=a.width;b[1]=a.height},"~O,~A");c$.hasFocus=c(c$,"hasFocus",function(){return!0},"~O");c$.requestFocusInWindow=c(c$,"requestFocusInWindow",function(){},"~O");c$.renderScreenImage=c(c$,"renderScreenImage",function(){},"J.api.PlatformViewer,~O,~O");c$.prompt=c(c$,"prompt",function(a,b){var d=null==b?alert(a):prompt(a,b);return null!=
d?d:"null"},"~S,~S,~A,~B");c$.convertPointFromScreen=c(c$,"convertPointFromScreen",function(){},"~O,JU.P3");c$.drawImage=c(c$,"drawImage",function(a,b,d,g,c,e,h){g=b.buf8;e=b.buf32;var k=b.width*b.height,l=1;if(h){var j=c%2;c=Math.floor(c/2);l=Math.floor(b.width/c)}h=4*(b.width-c||d);var m=0,r=0;for(d*=4;m<k;)g[d++]=e[m]>>16&255,g[d++]=e[m]>>8&255,g[d++]=e[m]&255,g[d++]=255,m+=l,0==++r%c&&(j&&(m+=1,g[d]=0,g[d+1]=0,g[d+2]=0,g[d+3]=0),d+=h);a.putImageData(b.imgdata,0,0)},"~O,~O,~N,~N,~N,~N,~B");s("J.awtjs2d");
v(null,"J.awtjs2d.Image",["J.awtjs2d.Platform"],function(){c$=D(J.awtjs2d,"Image");c$.getWidth=c(c$,"getWidth",function(a){return a.imageWidth?a.imageWidth:a.width},"~O");c$.getHeight=c(c$,"getHeight",function(a){return a.imageHeight?a.imageHeight:a.height},"~O");c$.grabPixels=c(c$,"grabPixels",function(a,b,d){var g=null,g=a.getImageData(0,0,b,d).data;return J.awtjs2d.Image.toIntARGB(g)},"~O,~N,~N");c$.toIntARGB=c(c$,"toIntARGB",function(a){for(var b=w(a.length/4),d=B(b,0),g=0,c=0;g<b;)d[g++]=a[c++]<<
16|a[c++]<<8|a[c++]|a[c++]<<24;return d},"~A");c$.getTextPixels=c(c$,"getTextPixels",function(a,b,d,g,c,e){d.fillStyle="#000000";d.fillRect(0,0,g,c);d.fillStyle="#FFFFFF";d.font=b.font;d.fillText(a,0,e);return J.awtjs2d.Image.grabPixels(d,g,c)},"~S,JU.Font,~O,~N,~N,~N");c$.allocateRgbImage=c(c$,"allocateRgbImage",function(a,b,d,g,c,e){null==e&&(e={width:a,height:b});e.buf32=d;return e},"~N,~N,~A,~N,~B,~O");c$.getImageDialog=c(c$,"getImageDialog",function(a,b,d){return J.awtjs2d.Platform.Jmol().consoleGetImageDialog(a,
b,d)},"JV.Viewer,~S,java.util.Map")});s("J.awtjs2d");v(["J.api.GenericFileInterface"],"J.awtjs2d.JSFile",["JU.OC","$.PT","JV.Viewer"],function(){c$=t(function(){this.fullName=this.name=null;p(this,arguments)},J.awtjs2d,"JSFile",null,J.api.GenericFileInterface);c$.newFile=c(c$,"newFile",function(a){return new J.awtjs2d.JSFile(a)},"~S");q(c$,function(a){this.name=a.$replace("\\","/");this.fullName=a;!this.fullName.startsWith("/")&&0>JU.OC.urlTypeIndex(a)&&(this.fullName=JV.Viewer.jsDocumentBase+"/"+
this.fullName);this.fullName=JU.PT.rep(this.fullName,"/./","/");a.substring(a.lastIndexOf("/")+1)},"~S");j(c$,"getParentAsFile",function(){var a=this.fullName.lastIndexOf("/");return 0>a?null:new J.awtjs2d.JSFile(this.fullName.substring(0,a))});j(c$,"getFullPath",function(){return this.fullName});j(c$,"getName",function(){return this.name});j(c$,"isDirectory",function(){return this.fullName.endsWith("/")});j(c$,"length",function(){return 0});c$.getURLContents=c(c$,"getURLContents",function(a,b,d){try{var g=
a.openConnection();null!=b?g.outputBytes(b):null!=d&&g.outputString(d);return g.getContents()}catch(c){if(H(c,Exception))return c.toString();throw c;}},"java.net.URL,~A,~S")});s("J.awtjs2d");c$=D(J.awtjs2d,"JSFont");c$.newFont=c(c$,"newFont",function(a,b,d,g,c){a=a.equals("Monospaced")?"Courier":a.startsWith("Sans")?"Helvetica Neue, Sans-serif":"Serif";return(b?"bold ":"")+(d?"italic ":"")+g+c+" "+a},"~S,~B,~B,~N,~S");c$.getFontMetrics=c(c$,"getFontMetrics",function(a,b){b.font!=a.font&&(b.font=a.font,
a.font=b.font,b._fontAscent=Math.ceil(a.fontSize),b._fontDescent=Math.ceil(0.25*a.fontSize));return b},"JU.Font,~O");c$.getAscent=c(c$,"getAscent",function(a){return Math.ceil(a._fontAscent)},"~O");c$.getDescent=c(c$,"getDescent",function(a){return Math.ceil(a._fontDescent)},"~O");c$.stringWidth=c(c$,"stringWidth",function(a,b,d){b.font=a.font;return Math.ceil(b.measureText(d).width)},"JU.Font,~O,~S");s("J.awtjs2d");v(["J.api.GenericMouseInterface"],"J.awtjs2d.Mouse",["java.lang.Character","JU.PT",
"$.V3","JU.Logger"],function(){c$=t(function(){this.manager=this.vwr=null;this.keyBuffer="";this.wheeling=this.isMouseDown=!1;this.modifiersWhenPressed10=this.yWhenPressed=this.xWhenPressed=this.modifiersDown=0;p(this,arguments)},J.awtjs2d,"Mouse",null,J.api.GenericMouseInterface);q(c$,function(a,b){this.vwr=b;this.manager=this.vwr.acm},"~N,JV.Viewer,~O");j(c$,"clear",function(){});j(c$,"dispose",function(){});j(c$,"processEvent",function(a,b,d,g,c){507!=a&&(g=J.awtjs2d.Mouse.applyLeftMouse(g));switch(a){case 507:this.wheeled(c,
b,g);break;case 501:this.xWhenPressed=b;this.yWhenPressed=d;this.modifiersWhenPressed10=g;this.pressed(c,b,d,g,!1);break;case 506:this.dragged(c,b,d,g);break;case 504:this.entry(c,b,d,!1);break;case 505:this.entry(c,b,d,!0);break;case 503:this.moved(c,b,d,g);break;case 502:this.released(c,b,d,g);b==this.xWhenPressed&&(d==this.yWhenPressed&&g==this.modifiersWhenPressed10)&&this.clicked(c,b,d,g,1);break;default:return!1}return!0},"~N,~N,~N,~N,~N");j(c$,"processTwoPointGesture",function(a){if(!(2>a[0].length)){var b=
a[0],d=a[1],g=b[0],c=b[d.length-1];a=g[0];var e=c[0],g=g[1],c=c[1],h=JU.V3.new3(e-a,c-g,0),k=h.length(),l=d[0],j=d[d.length-1],d=l[0],m=j[0],l=l[1],j=j[1],r=JU.V3.new3(m-d,j-l,0),x=r.length();1>k||1>x||(h.normalize(),r.normalize(),h=h.dot(r),0.8<h?(a=A(e-b[b.length-2][0]),b=A(c-b[b.length-2][1]),this.vwr.translateXYBy(a,b)):-0.8>h&&(h=JU.V3.new3(d-a,l-g,0),r=JU.V3.new3(m-e,j-c,0),b=r.length()-h.length(),this.wheeled(Zystem.currentTimeMillis(),0>b?-1:1,32)))}},"~A");c(c$,"mouseClicked",function(a){this.clicked(a.getWhen(),
a.getX(),a.getY(),a.getModifiers(),a.getClickCount())},"java.awt.event.MouseEvent");c(c$,"mouseEntered",function(a){this.entry(a.getWhen(),a.getX(),a.getY(),!1)},"java.awt.event.MouseEvent");c(c$,"mouseExited",function(a){this.entry(a.getWhen(),a.getX(),a.getY(),!0)},"java.awt.event.MouseEvent");c(c$,"mousePressed",function(a){this.pressed(a.getWhen(),a.getX(),a.getY(),a.getModifiers(),a.isPopupTrigger())},"java.awt.event.MouseEvent");c(c$,"mouseReleased",function(a){this.released(a.getWhen(),a.getX(),
a.getY(),a.getModifiers())},"java.awt.event.MouseEvent");c(c$,"mouseDragged",function(a){var b=a.getModifiers();0==(b&28)&&(b|=16);this.dragged(a.getWhen(),a.getX(),a.getY(),b)},"java.awt.event.MouseEvent");c(c$,"mouseMoved",function(a){this.moved(a.getWhen(),a.getX(),a.getY(),a.getModifiers())},"java.awt.event.MouseEvent");c(c$,"mouseWheelMoved",function(a){a.consume();this.wheeled(a.getWhen(),a.getWheelRotation(),a.getModifiers())},"java.awt.event.MouseWheelEvent");c(c$,"keyTyped",function(a){a.consume();
if(this.vwr.menuEnabled()){var b=a.getKeyChar(),d=a.getModifiers();JU.Logger.debuggingHigh&&JU.Logger.debug("MouseManager keyTyped: "+b+" "+(0+b.charCodeAt(0))+" "+d);if(0!=d&&1!=d)switch(b){case String.fromCharCode(11):case "k":a=!this.vwr.getBooleanProperty("allowKeyStrokes");switch(d){case 2:this.vwr.setBooleanProperty("allowKeyStrokes",a);this.vwr.setBooleanProperty("showKeyStrokes",!0);break;case 10:case 1:this.vwr.setBooleanProperty("allowKeyStrokes",a),this.vwr.setBooleanProperty("showKeyStrokes",
!1)}this.clearKeyBuffer();this.vwr.refresh(3,"showkey");break;case 26:case "z":switch(d){case 2:this.vwr.undoMoveAction(4165,1);break;case 3:this.vwr.undoMoveAction(4139,1)}break;case 25:case "y":switch(d){case 2:this.vwr.undoMoveAction(4139,1)}}else this.vwr.getBooleanProperty("allowKeyStrokes")&&this.addKeyBuffer(1==a.getModifiers()?Character.toUpperCase(b):b)}},"java.awt.event.KeyEvent");c(c$,"keyPressed",function(a){this.vwr.isApplet&&a.consume();this.manager.keyPressed(a.getKeyCode(),a.getModifiers())},
"java.awt.event.KeyEvent");c(c$,"keyReleased",function(a){a.consume();this.manager.keyReleased(a.getKeyCode())},"java.awt.event.KeyEvent");c(c$,"clearKeyBuffer",function(){0!=this.keyBuffer.length&&(this.keyBuffer="",this.vwr.getBooleanProperty("showKeyStrokes")&&this.vwr.evalStringQuietSync('!set echo _KEYSTROKES; set echo bottom left;echo ""',!0,!0))});c(c$,"addKeyBuffer",function(a){10==a.charCodeAt(0)?this.sendKeyBuffer():(8==a.charCodeAt(0)?0<this.keyBuffer.length&&(this.keyBuffer=this.keyBuffer.substring(0,
this.keyBuffer.length-1)):this.keyBuffer+=a,this.vwr.getBooleanProperty("showKeyStrokes")&&this.vwr.evalStringQuietSync("!set echo _KEYSTROKES; set echo bottom left;echo "+JU.PT.esc("\u0001"+this.keyBuffer),!0,!0))},"~S");c(c$,"sendKeyBuffer",function(){var a=this.keyBuffer;this.vwr.getBooleanProperty("showKeyStrokes")&&this.vwr.evalStringQuietSync("!set echo _KEYSTROKES; set echo bottom left;echo "+JU.PT.esc(this.keyBuffer),!0,!0);this.clearKeyBuffer();this.vwr.evalStringQuietSync(a,!1,!0)});c(c$,
"entry",function(a,b,d,g){this.isMouseDown=this.wheeling=!1;this.modifiersDown=0;this.manager.mouseEnterExit(a,b,d,g)},"~N,~N,~N,~B");c(c$,"clicked",function(a,b,d,g){this.clearKeyBuffer();this.manager.mouseAction(2,a,b,d,1,g)},"~N,~N,~N,~N,~N");c(c$,"moved",function(a,b,d,g){this.clearKeyBuffer();this.isMouseDown?this.manager.mouseAction(1,a,b,d,0,this.modifiersDown):this.manager.mouseAction(0,a,b,d,0,g)},"~N,~N,~N,~N");c(c$,"wheeled",function(a,b,d){this.clearKeyBuffer();this.wheeling=!0;this.manager.mouseAction(3,
a,0,b,0,d&-29|32)},"~N,~N,~N");c(c$,"pressed",function(a,b,d,g){this.clearKeyBuffer();this.isMouseDown=!0;this.modifiersDown=g;this.wheeling=!1;this.manager.mouseAction(4,a,b,d,0,g)},"~N,~N,~N,~N,~B");c(c$,"released",function(a,b,d,g){this.isMouseDown=!1;this.modifiersDown=0;this.wheeling=!1;this.manager.mouseAction(5,a,b,d,0,g)},"~N,~N,~N,~N");c(c$,"dragged",function(a,b,d){this.wheeling||(20==(this.modifiersDown&20)&&(this.modifiersDown=this.modifiersDown&-5|2),this.manager.mouseAction(1,a,b,d,
0,this.modifiersDown))},"~N,~N,~N,~N");c$.applyLeftMouse=c(c$,"applyLeftMouse",function(a){return 0==(a&28)?a|16:a},"~N")});s("J.awtjs2d");v(["J.api.GenericPlatform"],"J.awtjs2d.Platform","java.net.URL JU.AjaxURLStreamHandlerFactory $.Rdr $.SB J.api.Interface J.awtjs2d.Display $.Image $.JSFile $.JSFont $.Mouse".split(" "),function(){c$=t(function(){this.context=this.vwr=this.canvas=null;p(this,arguments)},J.awtjs2d,"Platform",null,J.api.GenericPlatform);j(c$,"setViewer",function(a,b){this.vwr=a;null==
b?(b=document.createElement("canvas"),this.context=b.getContext("2d")):(this.context=b.getContext("2d"),b.imgdata=this.context.getImageData(0,0,b.width,b.height),b.buf8=b.imgdata.data);this.canvas=b;try{java.net.URL.setURLStreamHandlerFactory(new JU.AjaxURLStreamHandlerFactory)}catch(d){}},"J.api.PlatformViewer,~O");j(c$,"isSingleThreaded",function(){return!0});j(c$,"getJsObjectInfo",function(a,b,d){return null==b?null:"localName"==b?a[0].nodeName:null==d?a[0][b]:a[0][b](d[0])},"~A,~S,~A");j(c$,"isHeadless",
function(){return!1});j(c$,"getMouseManager",function(a,b){return new J.awtjs2d.Mouse(a,this.vwr,b)},"~N,~O");j(c$,"convertPointFromScreen",function(a,b){J.awtjs2d.Display.convertPointFromScreen(a,b)},"~O,JU.P3");j(c$,"getFullScreenDimensions",function(a,b){J.awtjs2d.Display.getFullScreenDimensions(a,b)},"~O,~A");j(c$,"getMenuPopup",function(a,b){var d="j"==b?"awtjs2d.JSJmolPopup":"awtjs2d.JSModelKitPopup",g=J.api.Interface.getOption(d,this.vwr,"popup");try{null!=g&&g.jpiInitialize(this.vwr,a)}catch(c){if(H(c,
Exception))return Zystem.out.println("Exception creating "+d+":"+c),null;throw c;}return g},"~S,~S");j(c$,"hasFocus",function(a){return J.awtjs2d.Display.hasFocus(a)},"~O");j(c$,"prompt",function(a,b,d,g){return J.awtjs2d.Display.prompt(a,b,d,g)},"~S,~S,~A,~B");j(c$,"renderScreenImage",function(a,b){J.awtjs2d.Display.renderScreenImage(this.vwr,a,b)},"~O,~O");j(c$,"drawImage",function(a,b,d,g,c,e,h){J.awtjs2d.Display.drawImage(a,b,d,g,c,e,h)},"~O,~O,~N,~N,~N,~N,~B");j(c$,"requestFocusInWindow",function(a){J.awtjs2d.Display.requestFocusInWindow(a)},
"~O");j(c$,"repaint",function(){var a=null,a=self.Jmol&&Jmol.repaint?Jmol:null;null!=a&&a.repaint(this.vwr.html5Applet,!0)},"~O");j(c$,"setTransparentCursor",function(){},"~O");j(c$,"setCursor",function(a){J.awtjs2d.Platform.Jmol().setCursor(this.vwr.html5Applet,a)},"~N,~O");j(c$,"allocateRgbImage",function(a,b,d,g,c,e){null==d&&(d=this.grabPixels(null,0,0,null,0,0),a=this.canvas.width,b=this.canvas.height);return J.awtjs2d.Image.allocateRgbImage(a,b,d,g,c,e?null:this.canvas)},"~N,~N,~A,~N,~B,~B");
j(c$,"notifyEndOfRendering",function(){});j(c$,"disposeGraphics",function(){},"~O");j(c$,"grabPixels",function(a,b,d){var g=null,c=null==a;c&&(this.canvas=a=Jmol.loadImage(this,"webgl",""+Zystem.currentTimeMillis(),this.vwr.html5Applet._canvas.toDataURL(),null,null),b=a.imageWidth,d=a.imageHeight,a.imageWidth=0);a.image&&(b!=a.width||d!=a.height)&&Jmol.setCanvasImage(a,b,d);if(a.buf32)return a.buf32;g=a.getContext("2d");b=J.awtjs2d.Image.grabPixels(g,b,d);a.buf32=b;if(c)for(a=b.length;0<=--a;)0==
b[a]&&(b[a]=-1);return b},"~O,~N,~N,~A,~N,~N");j(c$,"drawImageToBuffer",function(a,b,d,g,c){return this.grabPixels(d,g,c,null,0,0)},"~O,~O,~O,~N,~N,~N");j(c$,"getTextPixels",function(a,b,d,g,c,e,h){return J.awtjs2d.Image.getTextPixels(a,b,d,c,e,h)},"~S,JU.Font,~O,~O,~N,~N,~N");j(c$,"flushImage",function(){},"~O");j(c$,"getGraphics",function(a){return null==a?this.context:a.getContext("2d")},"~O");j(c$,"getImageHeight",function(a){return null==a?-1:J.awtjs2d.Image.getHeight(a)},"~O");j(c$,"getImageWidth",
function(a){return null==a?-1:J.awtjs2d.Image.getWidth(a)},"~O");j(c$,"getStaticGraphics",function(a){return this.getGraphics(a)},"~O,~B");j(c$,"newBufferedImage",function(a,b,d){return J.awtjs2d.Platform.Jmol().getHiddenCanvas(this.vwr.html5Applet,"stereoImage",b,d)},"~O,~N,~N");j(c$,"newOffScreenImage",function(a,b){return J.awtjs2d.Platform.Jmol().getHiddenCanvas(this.vwr.html5Applet,"textImage",a,b)},"~N,~N");j(c$,"waitForDisplay",function(){return!1},"~O,~O");j(c$,"createImage",function(a){var b=
a[0],d=a[1];a=a[2];var g=this.vwr,c=null==a?g.getEvalContextAndHoldQueue(g.eval):null,e=null,e=function(a,d){g.loadImageData(a,d,b,c)};return J.awtjs2d.Platform.Jmol().loadImage(this,b,d,a,e)},"~O");j(c$,"fontStringWidth",function(a,b){return J.awtjs2d.JSFont.stringWidth(a,this.context,b)},"JU.Font,~S");j(c$,"getFontAscent",function(a){return J.awtjs2d.JSFont.getAscent(a)},"~O");j(c$,"getFontDescent",function(a){return J.awtjs2d.JSFont.getDescent(a)},"~O");j(c$,"getFontMetrics",function(a,b){return J.awtjs2d.JSFont.getFontMetrics(a,
null==b?this.context:b)},"JU.Font,~O");j(c$,"newFont",function(a,b,d,g){return J.awtjs2d.JSFont.newFont(a,b,d,g,"px")},"~S,~B,~B,~N");j(c$,"getDateFormat",function(a){if(null!=a){if(0<=a.indexOf("8824")){var b=new Date;a=b.toString().split(" ");var d="0"+(1+b.getMonth()),d=d.substring(d.length-2),b="0"+b.getDate(),b=b.substring(b.length-2);return a[3]+d+b+a[4].replace(/\:/g,"")+a[5].substring(3,6)+"'"+a[5].substring(6,8)+"'"}if(0<=a.indexOf("8601")){b=new Date;a=b.toString().split(" ");if(1==a.length)return a;
d="0"+(1+b.getMonth());d=d.substring(d.length-2);b="0"+b.getDate();b=b.substring(b.length-2);return a[3]+"-"+d+"-"+b+"T"+a[4]}}return(""+new Date).split(" (")[0]},"~S");j(c$,"newFile",function(a){return new J.awtjs2d.JSFile(a)},"~S");j(c$,"getBufferedFileInputStream",function(){return null},"~S");j(c$,"getURLContents",function(a,b,d,g){return J.awtjs2d.Platform.getURLContentsStatic(a,b,d,g)},"java.net.URL,~A,~S,~B");c$.getURLContentsStatic=c(c$,"getURLContentsStatic",function(a,b,d,g){a=J.awtjs2d.JSFile.getURLContents(a,
b,d);try{return!g?a:n(a,String)?a:n(a,JU.SB)?a.toString():n(a,Array)?String.instantialize(a):String.instantialize(JU.Rdr.getStreamAsBytes(a,null))}catch(c){if(H(c,Exception))return""+c;throw c;}},"java.net.URL,~A,~S,~B");j(c$,"getLocalUrl",function(){return null},"~S");j(c$,"getImageDialog",function(a,b){return J.awtjs2d.Image.getImageDialog(this.vwr,a,b)},"~S,java.util.Map");c$.Jmol=c(c$,"Jmol",function(){return Jmol});j(c$,"forceAsyncLoad",function(a){return J.awtjs2d.Platform.Jmol().isBinaryUrl(a)},
"~S");j(c$,"getInChI",function(){return null==J.awtjs2d.Platform.inchi?J.awtjs2d.Platform.inchi=J.api.Interface.getInterface("J.inchi.InChIJS",this.vwr,"platform"):J.awtjs2d.Platform.inchi});j(c$,"confirm",function(a,b){var d=!1;null!=b&&(d=!1);return d?1:2},"~S,~S");E(c$,"inchi",null)});s("J.bspt");v(null,"J.bspt.Bspf",["JU.AU","J.bspt.Bspt"],function(){c$=t(function(){this.dimMax=0;this.bspts=null;this.isValid=!1;this.cubeIterators=this.bsptsValid=null;p(this,arguments)},J.bspt,"Bspf");c(c$,"validateModel",
function(a,b){this.bsptsValid[a]=b},"~N,~B");c(c$,"isInitializedIndex",function(a){return this.bspts.length>a&&null!=this.bspts[a]&&this.bsptsValid[a]},"~N");q(c$,function(a){this.dimMax=a;this.bspts=Array(1);this.bsptsValid=ga(1,!1);this.cubeIterators=[]},"~N");c(c$,"addTuple",function(a,b){a>=this.bspts.length&&(this.bspts=JU.AU.arrayCopyObject(this.bspts,a+1),this.bsptsValid=JU.AU.arrayCopyBool(this.bsptsValid,a+1));var d=this.bspts[a];null==d&&(d=this.bspts[a]=new J.bspt.Bspt(this.dimMax,a));
d.addTuple(b)},"~N,JU.P3");c(c$,"stats",function(){for(var a=0;a<this.bspts.length;++a)null!=this.bspts[a]&&this.bspts[a].stats()});c(c$,"getCubeIterator",function(a){if(0>a)return this.getNewCubeIterator(-1-a);a>=this.cubeIterators.length&&(this.cubeIterators=JU.AU.arrayCopyObject(this.cubeIterators,a+1));null==this.cubeIterators[a]&&null!=this.bspts[a]&&(this.cubeIterators[a]=this.getNewCubeIterator(a));this.cubeIterators[a].set(this.bspts[a]);return this.cubeIterators[a]},"~N");c(c$,"getNewCubeIterator",
function(a){return this.bspts[a].allocateCubeIterator()},"~N");c(c$,"initialize",function(a,b,d){null!=this.bspts[a]&&this.bspts[a].reset();for(var g=d.nextSetBit(0);0<=g;g=d.nextSetBit(g+1))this.addTuple(a,b[g]);this.bsptsValid[a]=!0},"~N,~A,JU.BS")});s("J.bspt");v(null,"J.bspt.Bspt",["J.bspt.CubeIterator","$.Leaf"],function(){c$=t(function(){this.index=this.dimMax=this.treeDepth=0;this.eleRoot=null;p(this,arguments)},J.bspt,"Bspt");q(c$,function(a,b){this.dimMax=a;this.index=b;this.reset()},"~N,~N");
c(c$,"reset",function(){this.eleRoot=new J.bspt.Leaf(this,null,0);this.treeDepth=1});c(c$,"addTuple",function(a){this.eleRoot=this.eleRoot.addTuple(0,a)},"JU.T3");c(c$,"stats",function(){});c(c$,"allocateCubeIterator",function(){return new J.bspt.CubeIterator(this)});E(c$,"leafCountMax",2,"MAX_TREE_DEPTH",100)});s("J.bspt");v(null,"J.bspt.CubeIterator",["J.bspt.Node"],function(){c$=t(function(){this.stack=this.bspt=null;this.leafIndex=this.sp=0;this.leaf=null;this.dz=this.dy=this.dx=this.cz=this.cy=
this.cx=this.radius=0;this.tHemisphere=!1;p(this,arguments)},J.bspt,"CubeIterator");q(c$,function(a){this.set(a)},"J.bspt.Bspt");c(c$,"set",function(a){this.bspt=a;this.stack=Array(a.treeDepth)},"J.bspt.Bspt");c(c$,"initialize",function(a,b,d){this.radius=b;this.tHemisphere=!1;this.cx=a.x;this.cy=a.y;this.cz=a.z;this.leaf=null;this.stack.length<this.bspt.treeDepth&&this.set(this.bspt);this.stack[0]=this.bspt.eleRoot;this.sp=1;this.findLeftLeaf();this.tHemisphere=d},"JU.T3,~N,~B");c(c$,"release",function(){this.set(this.bspt)});
c(c$,"hasMoreElements",function(){for(;null!=this.leaf;){for(;this.leafIndex<this.leaf.count;++this.leafIndex)if(this.isWithinRadius(this.leaf.tuples[this.leafIndex]))return!0;this.findLeftLeaf()}return!1});c(c$,"nextElement",function(){return this.leaf.tuples[this.leafIndex++]});c(c$,"foundDistance2",function(){return this.dx*this.dx+this.dy*this.dy+this.dz*this.dz});c(c$,"findLeftLeaf",function(){this.leaf=null;if(0!=this.sp){for(var a=this.stack[--this.sp];n(a,J.bspt.Node);){var b;switch(a.dim){case 0:b=
this.cx;break;case 1:b=this.cy;break;default:b=this.cz}var d=b+this.radius;if(!this.tHemisphere||0!=a.dim)b-=this.radius;if(b<=a.maxLeft&&d>=a.minLeft)d>=a.minRight&&b<=a.maxRight&&(this.stack[this.sp++]=a.eleRight),a=a.eleLeft;else if(d>=a.minRight&&b<=a.maxRight)a=a.eleRight;else{if(0==this.sp)return;a=this.stack[--this.sp]}}this.leaf=a;this.leafIndex=0}});c(c$,"isWithinRadius",function(a){this.dx=a.x-this.cx;return(!this.tHemisphere||0<=this.dx)&&(this.dx=Math.abs(this.dx))<=this.radius&&(this.dy=
Math.abs(a.y-this.cy))<=this.radius&&(this.dz=Math.abs(a.z-this.cz))<=this.radius},"JU.T3")});s("J.bspt");c$=t(function(){this.bspt=null;this.count=0;p(this,arguments)},J.bspt,"Element");s("J.bspt");v(["J.bspt.Element"],"J.bspt.Leaf",["J.bspt.Node"],function(){c$=t(function(){this.tuples=null;p(this,arguments)},J.bspt,"Leaf",J.bspt.Element);q(c$,function(a,b,d){this.bspt=a;this.count=0;this.tuples=Array(2);if(null!=b){for(a=d;2>a;++a)this.tuples[this.count++]=b.tuples[a],b.tuples[a]=null;b.count=
d}},"J.bspt.Bspt,J.bspt.Leaf,~N");c(c$,"sort",function(a){for(var b=this.count;0<--b;)for(var d=this.tuples[b],g=J.bspt.Node.getDimensionValue(d,a),c=b;0<=--c;){var e=this.tuples[c],h=J.bspt.Node.getDimensionValue(e,a);h>g&&(this.tuples[b]=e,this.tuples[c]=d,d=e,g=h)}},"~N");j(c$,"addTuple",function(a,b){return 2>this.count?(this.tuples[this.count++]=b,this):(new J.bspt.Node(this.bspt,a,this)).addTuple(a,b)},"~N,JU.T3")});s("J.bspt");v(["J.bspt.Element"],"J.bspt.Node",["java.lang.NullPointerException",
"J.bspt.Leaf"],function(){c$=t(function(){this.maxLeft=this.minLeft=this.dim=0;this.eleLeft=null;this.maxRight=this.minRight=0;this.eleRight=null;p(this,arguments)},J.bspt,"Node",J.bspt.Element);q(c$,function(a,b,d){this.bspt=a;b==a.treeDepth&&(a.treeDepth=b+1);if(2!=d.count)throw new NullPointerException;this.dim=b%a.dimMax;d.sort(this.dim);a=new J.bspt.Leaf(a,d,1);this.minLeft=J.bspt.Node.getDimensionValue(d.tuples[0],this.dim);this.maxLeft=J.bspt.Node.getDimensionValue(d.tuples[d.count-1],this.dim);
this.minRight=J.bspt.Node.getDimensionValue(a.tuples[0],this.dim);this.maxRight=J.bspt.Node.getDimensionValue(a.tuples[a.count-1],this.dim);this.eleLeft=d;this.eleRight=a;this.count=2},"J.bspt.Bspt,~N,J.bspt.Leaf");c(c$,"addTuple",function(a,b){var d=J.bspt.Node.getDimensionValue(b,this.dim);++this.count;d<this.maxLeft||(d>this.minRight?0:d==this.maxLeft?d==this.minRight?this.eleLeft.count<this.eleRight.count:1:d==this.minRight?0:this.eleLeft.count<this.eleRight.count)?(d<this.minLeft?this.minLeft=
d:d>this.maxLeft&&(this.maxLeft=d),this.eleLeft=this.eleLeft.addTuple(a+1,b)):(d<this.minRight?this.minRight=d:d>this.maxRight&&(this.maxRight=d),this.eleRight=this.eleRight.addTuple(a+1,b));return this},"~N,JU.T3");c$.getDimensionValue=c(c$,"getDimensionValue",function(a,b){null==a&&Zystem.out.println("bspt.Node ???");switch(b){case 0:return a.x;case 1:return a.y;default:return a.z}},"JU.T3,~N")});s("J.c");v(["java.lang.Enum"],"J.c.CBK",["JU.SB"],function(){c$=D(J.c,"CBK",Enum);c$.getCallback=c(c$,
"getCallback",function(a){a=a.toUpperCase();a=a.substring(0,Math.max(a.indexOf("CALLBACK"),0));for(var b,d=0,g=J.c.CBK.values();d<g.length&&((b=g[d])||1);d++)if(b.name().equalsIgnoreCase(a))return b;return null},"~S");c$.getNameList=c(c$,"getNameList",function(){if(null==J.c.CBK.nameList){for(var a=new JU.SB,b,d=0,g=J.c.CBK.values();d<g.length&&((b=g[d])||1);d++)a.append(b.name().toLowerCase()).append("Callback;");J.c.CBK.nameList=a.toString()}return J.c.CBK.nameList});c$.nameList=null;G(c$,"ANIMFRAME",
0,[]);G(c$,"APPLETREADY",1,[]);G(c$,"ATOMMOVED",2,[]);G(c$,"AUDIO",3,[]);G(c$,"CLICK",4,[]);G(c$,"DRAGDROP",5,[]);G(c$,"ECHO",6,[]);G(c$,"ERROR",7,[]);G(c$,"EVAL",8,[]);G(c$,"HOVER",9,[]);G(c$,"IMAGE",10,[]);G(c$,"LOADSTRUCT",11,[]);G(c$,"MEASURE",12,[]);G(c$,"MESSAGE",13,[]);G(c$,"MINIMIZATION",14,[]);G(c$,"MODELKIT",15,[]);G(c$,"SERVICE",16,[]);G(c$,"PICK",17,[]);G(c$,"RESIZE",18,[]);G(c$,"SCRIPT",19,[]);G(c$,"SYNC",20,[]);G(c$,"STRUCTUREMODIFIED",21,[])});s("J.c");v(["java.lang.Enum"],"J.c.FIL",
null,function(){c$=t(function(){this.code=0;p(this,arguments)},J.c,"FIL",Enum);c(c$,"getCode",function(){return this.code});q(c$,function(a){this.code=a},"~N");G(c$,"DELETED",0,[5]);G(c$,"CREATED",1,[3]);G(c$,"CREATING_MODELSET",2,[2]);G(c$,"ZAPPED",3,[0]);G(c$,"NOT_LOADED",4,[-1])});s("J.c");v(["java.lang.Enum"],"J.c.PAL",["java.lang.Byte"],function(){c$=t(function(){this.$$name=null;this.id=0;p(this,arguments)},J.c,"PAL",Enum);q(c$,function(a,b){this.$$name=a;this.id=b},"~S,~N");c$.pidOf=c(c$,"pidOf",
function(a){return n(a,J.c.PAL)?a.id:n(a,Byte)?a.byteValue():J.c.PAL.UNKNOWN.id},"~O");c$.isPaletteVariable=c(c$,"isPaletteVariable",function(a){return 0!=(a&64)},"~N");c$.getPalette=c(c$,"getPalette",function(a){if(0>a.indexOf("_"))for(var b,d=0,g=J.c.PAL.values();d<g.length&&((b=g[d])||1);d++)if(a.equalsIgnoreCase(b.$$name))return b;return 0==a.indexOf("property_")?J.c.PAL.PROPERTY:J.c.PAL.UNKNOWN},"~S");c$.getPaletteID=c(c$,"getPaletteID",function(a){if(0>a.indexOf("_"))for(var b,d=0,g=J.c.PAL.values();d<
g.length&&((b=g[d])||1);d++)if(a.equalsIgnoreCase(b.$$name))return b.id;return 0==a.indexOf("property_")?J.c.PAL.PROPERTY.id:J.c.PAL.UNKNOWN.id},"~S");c$.getPaletteName=c(c$,"getPaletteName",function(a){for(var b,d=0,g=J.c.PAL.values();d<g.length&&((b=g[d])||1);d++)if(b.id==a)return b.$$name;return null},"~N");c$.PALETTE_VOLATILE=64;c$.PALETTE_NONE=0;c$.PALETTE_CPK=1;c$.PALETTE_PARTIAL_CHARGE=2;c$.PALETTE_FORMAL_CHARGE=3;c$.PALETTE_TEMP=68;c$.PALETTE_FIXEDTEMP=5;c$.PALETTE_SURFACE=70;c$.PALETTE_STRUCTURE=
7;c$.PALETTE_AMINO=8;c$.PALETTE_SHAPELY=9;c$.PALETTE_CHAIN=10;c$.PALETTE_GROUP=75;c$.PALETTE_MONOMER=76;c$.PALETTE_MOLECULE=77;c$.PALETTE_ALTLOC=14;c$.PALETTE_INSERTION=15;c$.PALETTE_JMOL=16;c$.PALETTE_RASMOL=17;c$.PALETTE_TYPE=18;c$.PALETTE_ENERGY=19;c$.PALETTE_PROPERTY=84;c$.PALETTE_VARIABLE=85;c$.PALETTE_STRAIGHTNESS=86;c$.PALETTE_POLYMER=87;c$.PALETTE_NUCLEIC=24;c$.argbsCpkRasmol=B(-1,[16716947,33554431,50315467,62005794,83951360,113821896,126849023,149946368,165324064,184549631,203590434,226525328,
249210144,268412160,285198386,285277952,343965840,377520272,411074704,427851920,452961536,480586282,497363498,514140714,598026794,796950672,899686640,956278016,1339729184]);c$.argbsCpk=B(-1,[4294907027,4294967295,4292476927,4291592447,4290969344,4294948277,4287664272,4281356536,4294905101,4287684688,4289979381,4289420530,4287299328,4290750118,4293970080,4294934528,4294967088,4280283167,4286632419,4287578324,4282253056,4293322470,4290757319,4289111723,4287273415,4288445127,4292896307,4293955744,4283486288,
4291330099,4286415024,4290940815,4284911503,4290609379,4294942976,4289079593,4284266705,4285542064,4278255360,4287954943,4287946976,4285776585,4283741621,4282097310,4280586127,4278877580,4278217093,4290822336,4294957455,4289099123,4284907648,4288570293,4292114944,4287889556,4282556080,4283897743,4278241536,4285584639,4294967239,4292476871,4291297223,4288937927,4287627207,4284612551,4282777543,4281401287,4280287175,4278255516,4278249077,4278244434,4278239032,4278233892,4283286271,4283279103,4280390870,
4280712619,4280706710,4279719047,4291875040,4294955299,4290296016,4289090637,4283914593,4288565173,4289420288,4285878085,4282548886,4282515558,4278222080,4285574138,4278237951,4278231551,4278226943,4278223103,4278217727,4283718898,4286078179,4287254499,4288755412,4289929172,4289929146,4289924518,4290579847,4291231846,4291559513,4291887183,4292411461,4292870200,4293263406,4293591078]);G(c$,"UNKNOWN",0,[null,255]);G(c$,"NONE",1,["none",0]);G(c$,"CPK",2,["cpk",1]);G(c$,"PARTIAL_CHARGE",3,["partialcharge",
2]);G(c$,"FORMAL_CHARGE",4,["formalcharge",3]);G(c$,"TEMP",5,["temperature",68]);G(c$,"FIXEDTEMP",6,["fixedtemperature",5]);G(c$,"SURFACE",7,["surfacedistance",70]);G(c$,"STRUCTURE",8,["structure",7]);G(c$,"AMINO",9,["amino",8]);G(c$,"SHAPELY",10,["shapely",9]);G(c$,"CHAIN",11,["chain",10]);G(c$,"GROUP",12,["group",75]);G(c$,"MONOMER",13,["monomer",76]);G(c$,"MOLECULE",14,["molecule",77]);G(c$,"ALTLOC",15,["altloc",14]);G(c$,"INSERTION",16,["insertion",15]);G(c$,"JMOL",17,["jmol",16]);G(c$,"RASMOL",
18,["rasmol",17]);G(c$,"TYPE",19,["type",18]);G(c$,"ENERGY",20,["energy",19]);G(c$,"PROPERTY",21,["property",84]);G(c$,"VARIABLE",22,["variable",85]);G(c$,"STRAIGHTNESS",23,["straightness",86]);G(c$,"POLYMER",24,["polymer",87]);G(c$,"NUCLEIC",25,["nucleic",24])});s("J.c");v(["java.lang.Enum"],"J.c.STER",null,function(){c$=t(function(){this.$$name=null;this.$isBiColor=!1;p(this,arguments)},J.c,"STER",Enum);q(c$,function(a,b){this.$$name=a;this.$isBiColor=b},"~S,~B");c(c$,"getName",function(){return this.$$name});
c(c$,"isBiColor",function(){return this.$isBiColor});c$.getStereoMode=c(c$,"getStereoMode",function(a){for(var b,d=0,g=J.c.STER.values();d<g.length&&((b=g[d])||1);d++)if(b.$$name.equalsIgnoreCase(a))return b;return null},"~S");G(c$,"NONE",0,["OFF",!1]);G(c$,"DOUBLE",1,["",!1]);G(c$,"REDCYAN",2,["REDCYAN",!0]);G(c$,"REDBLUE",3,["REDBLUE",!0]);G(c$,"REDGREEN",4,["REDGREEN",!0]);G(c$,"DTI",5,["DTI",!1]);G(c$,"CUSTOM",6,["",!0])});s("J.c");v(["java.lang.Enum"],"J.c.STR",null,function(){c$=t(function(){this.color=
this.id=0;p(this,arguments)},J.c,"STR",Enum);q(c$,function(a,b){this.id=a;this.color=b},"~N,~N");c(c$,"getId",function(){return this.id});c(c$,"getColor",function(){return this.color});c$.getProteinStructureType=c(c$,"getProteinStructureType",function(a){for(var b,d=0,g=J.c.STR.values();d<g.length&&((b=g[d])||1);d++)if(a.equalsIgnoreCase(b.name()))return b.isProtein()?b:J.c.STR.NOT;return J.c.STR.NOT},"~S");c(c$,"getBioStructureTypeName",function(a){return 0>this.id?"":a&&this.isProtein()?"protein":
this.name()},"~B");c(c$,"isProtein",function(){return 0<=this.id&&3>=this.id||7<=this.id});G(c$,"NOT",0,[-1,4286611584]);G(c$,"NONE",1,[0,4294967295]);G(c$,"TURN",2,[1,4284514559]);G(c$,"SHEET",3,[2,4294952960]);G(c$,"HELIX",4,[3,4294901888]);G(c$,"DNA",5,[4,4289593598]);G(c$,"RNA",6,[5,4294771042]);G(c$,"CARBOHYDRATE",7,[6,4289111802]);G(c$,"HELIX310",8,[7,4288675968]);G(c$,"HELIXALPHA",9,[8,4294901888]);G(c$,"HELIXPI",10,[9,4284481664]);G(c$,"ANNOTATION",11,[-2,0])});s("J.c");v(["java.lang.Enum"],
"J.c.VDW",null,function(){c$=t(function(){this.pt=0;this.type2=this.type=null;p(this,arguments)},J.c,"VDW",Enum);q(c$,function(a,b,d){this.pt=a;this.type=b;this.type2=d},"~N,~S,~S");c(c$,"getVdwLabel",function(){return null==this.type?this.type2:this.type});c$.getVdwType=c(c$,"getVdwType",function(a){if(null!=a)for(var b,d=0,g=J.c.VDW.values();d<g.length&&((b=g[d])||1);d++)if(a.equalsIgnoreCase(b.type))return b;return null},"~S");c$.getVdwType2=c(c$,"getVdwType2",function(a){if(null!=a)for(var b,
d=0,g=J.c.VDW.values();d<g.length&&((b=g[d])||1);d++)if(a.equalsIgnoreCase(b.type2))return b;return null},"~S");G(c$,"JMOL",0,[0,"Jmol",null]);G(c$,"BABEL",1,[1,"Babel",null]);G(c$,"RASMOL",2,[2,"RasMol",null]);G(c$,"BABEL21",3,[3,"Babel21",null]);G(c$,"AUTO_JMOL",4,[0,null,"Jmol"]);G(c$,"AUTO_BABEL",5,[1,null,"Babel"]);G(c$,"AUTO_RASMOL",6,[2,null,"RasMol"]);G(c$,"AUTO",7,[0,"Auto",null]);G(c$,"USER",8,[-1,"User",null]);G(c$,"ADPMAX",9,[-1,null,"adpmax"]);G(c$,"ADPMIN",10,[-1,null,"adpmin"]);G(c$,
"HYDRO",11,[-1,null,"hydrophobic"]);G(c$,"BONDING",12,[-1,null,"bondingradius"]);G(c$,"TEMP",13,[-1,null,"temperature"]);G(c$,"NOJMOL",14,[-1,null,null]);G(c$,"NADA",15,[-1,null,null])});s("J.g3d");v(["JU.P3i"],"J.g3d.CylinderRenderer",["JU.AU","$.P3"],function(){c$=t(function(){this.shader=this.line3d=this.g3d=null;this.colixB=this.colixA=0;this.shadesB=this.shadesA=null;this.dzBf=this.dyBf=this.dxBf=this.zAf=this.yAf=this.xAf=this.dzB=this.dyB=this.dxB=this.zA=this.yA=this.xA=0;this.tEvenDiameter=
!1;this.endcaps=this.diameter=0;this.endCapHidden=!1;this.sinPhi=this.cosPhi=this.cosTheta=this.radius2=this.radius=this.endcapShadeIndex=this.colixEndcap=this.argbEndcap=this.zEndcap=this.yEndcap=this.xEndcap=0;this.clipped=!1;this.rasterCount=0;this.ptB0i=this.ptA0i=this.ptB0=this.ptA0=this.xyzfRaster=this.xyztRaster=null;this.zTip=this.yTip=this.xTip=0;p(this,arguments)},J.g3d,"CylinderRenderer");O(c$,function(){this.xyztRaster=u(-1,[K(32,0),K(32,0),K(32,0),K(32,0)]);this.xyzfRaster=u(-1,[B(32,
0),B(32,0),B(32,0),B(32,0)]);this.ptA0i=new JU.P3i;this.ptB0i=new JU.P3i});q(c$,function(a){this.g3d=a;this.line3d=a.line3d;this.shader=a.shader},"J.g3d.Graphics3D");c(c$,"renderOld",function(a,b,d,g,c,e,h,k,l,j,m){var r=w(c/2)+1,x=this.g3d,y=x.clipCode3(e-r,h-r,k-r),F=x.clipCode3(e+r,h+r,k+r),C=x.clipCode3(l-r,j-r,m-r),r=x.clipCode3(l+r,j+r,m+r),n=y|F|C|r;this.clipped=0!=n;if(!(-1==n||0!=(y&r&F&C)))if(this.dxB=l-e,this.dyB=j-h,this.dzB=m-k,1>=c)this.line3d.plotLineDeltaOld(x.getColorArgbOrGray(a),
x.getColorArgbOrGray(b),e,h,k,this.dxB,this.dyB,this.dzB,this.clipped);else{l=0==d&&(this.clipped||2==g||0==g);this.diameter=c;this.xA=e;this.yA=h;this.zA=k;this.endcaps=g;this.shadesA=x.getShades(this.colixA=a);this.shadesB=x.getShades(this.colixB=b);this.calcArgbEndcap(!0,!1);this.calcCosSin(this.dxB,this.dyB,this.dzB);this.calcPoints(3,!1);this.interpolate(0,1,this.xyzfRaster,this.xyztRaster);this.interpolate(1,2,this.xyzfRaster,this.xyztRaster);k=this.xyzfRaster;2==g&&this.renderFlatEndcap(!0,
!1,k);x.setZMargin(5);a=x.width;b=x.zbuf;c=k[0];e=k[1];h=k[2];k=k[3];j=x.pixel;for(m=this.rasterCount;0<=--m;)y=k[m]>>8,F=y>>1,C=c[m],r=e[m],n=h[m],this.endCapHidden&&0!=this.argbEndcap&&(this.clipped?(x.plotPixelClippedArgb(this.argbEndcap,this.xEndcap+C,this.yEndcap+r,this.zEndcap-n-1,a,b,j),x.plotPixelClippedArgb(this.argbEndcap,this.xEndcap-C,this.yEndcap-r,this.zEndcap+n-1,a,b,j)):(x.plotPixelUnclipped(this.argbEndcap,this.xEndcap+C,this.yEndcap+r,this.zEndcap-n-1,a,b,j),x.plotPixelUnclipped(this.argbEndcap,
this.xEndcap-C,this.yEndcap-r,this.zEndcap+n-1,a,b,j))),this.line3d.plotLineDeltaAOld(this.shadesA,this.shadesB,d,y,this.xA+C,this.yA+r,this.zA-n,this.dxB,this.dyB,this.dzB,this.clipped),l&&this.line3d.plotLineDeltaOld(this.shadesA[F],this.shadesB[F],this.xA-C,this.yA-r,this.zA+n,this.dxB,this.dyB,this.dzB,this.clipped);x.setZMargin(0);3==g&&this.renderSphericalEndcaps()}},"~N,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N");c(c$,"renderBitsFloat",function(a,b,d,g,c,e,h){var k=this.g3d;null==this.ptA0&&(this.ptA0=
new JU.P3,this.ptB0=new JU.P3);this.ptA0.setT(e);var l=w(c/2)+1,j=Math.round(e.x),m=Math.round(e.y),r=Math.round(e.z),x=Math.round(h.x),y=Math.round(h.y),F=Math.round(h.z),C=k.clipCode3(j-l,m-l,r-l),j=k.clipCode3(j+l,m+l,r+l),m=k.clipCode3(x-l,y-l,F-l),l=k.clipCode3(x+l,y+l,F+l),x=C|j|m|l;this.clipped=0!=x;if(!(-1==x||0!=(C&l&j&m))){this.dxBf=h.x-e.x;this.dyBf=h.y-e.y;this.dzBf=h.z-e.z;0<c&&(this.diameter=c,this.xAf=e.x,this.yAf=e.y,this.zAf=e.z);e=0==d&&(this.clipped||2==g||0==g);this.xA=A(this.xAf);
this.yA=A(this.yAf);this.zA=A(this.zAf);this.dxB=A(this.dxBf);this.dyB=A(this.dyBf);this.dzB=A(this.dzBf);this.shadesA=k.getShades(this.colixA=a);this.shadesB=k.getShades(this.colixB=b);this.endcaps=g;this.calcArgbEndcap(!0,!0);x=this.xyzfRaster;0<c&&this.generateBaseEllipsePrecisely(!1);2==g&&this.renderFlatEndcap(!0,!0,x);this.line3d.setLineBits(this.dxBf,this.dyBf);k.setZMargin(5);c=k.pixel;a=k.width;b=k.zbuf;h=x[0];C=x[1];l=x[2];x=x[3];for(y=this.rasterCount;0<=--y;){var F=x[y]>>8,j=F>>1,m=h[y],
r=C[y],n=l[y];this.endCapHidden&&0!=this.argbEndcap&&(this.clipped?(k.plotPixelClippedArgb(this.argbEndcap,this.xEndcap+m,this.yEndcap+r,this.zEndcap-n-1,a,b,c),k.plotPixelClippedArgb(this.argbEndcap,this.xEndcap-m,this.yEndcap-r,this.zEndcap+n-1,a,b,c)):(k.plotPixelUnclipped(this.argbEndcap,this.xEndcap+m,this.yEndcap+r,this.zEndcap-n-1,a,b,c),k.plotPixelUnclipped(this.argbEndcap,this.xEndcap-m,this.yEndcap-r,this.zEndcap+n-1,a,b,c)));this.ptA0.set(this.xA+m,this.yA+r,this.zA-n);this.ptB0.setT(this.ptA0);
this.ptB0.x+=this.dxB;this.ptB0.y+=this.dyB;this.ptB0.z+=this.dzB;this.line3d.plotLineDeltaABitsFloat(this.shadesA,this.shadesB,F,this.ptA0,this.ptB0,d,this.clipped);e&&(this.ptA0.set(this.xA-m,this.yA-r,this.zA+n),this.ptB0.setT(this.ptA0),this.ptB0.x+=this.dxB,this.ptB0.y+=this.dyB,this.ptB0.z+=this.dzB,this.line3d.plotLineDeltaABitsFloat(this.shadesA,this.shadesB,j,this.ptA0,this.ptB0,d,this.clipped))}k.setZMargin(0);3==g&&this.renderSphericalEndcaps();this.xAf+=this.dxBf;this.yAf+=this.dyBf;this.zAf+=
this.dzBf}},"~N,~N,~N,~N,~N,JU.P3,JU.P3");c(c$,"renderBits",function(a,b,d,g,c,e,h){var k=this.g3d;if(0==c||1==c)this.line3d.plotLineBits(k.getColorArgbOrGray(a),k.getColorArgbOrGray(b),e,h,0,0,!1);else{this.ptA0i.setT(e);var l=w(c/2)+1,j=e.x,m=e.y,r=e.z,x=h.x,y=h.y,F=h.z,C=k.clipCode3(j-l,m-l,r-l),j=k.clipCode3(j+l,m+l,r+l),m=k.clipCode3(x-l,y-l,F-l),l=k.clipCode3(x+l,y+l,F+l),x=C|j|m|l;this.clipped=0!=x;if(!(-1==x||0!=(C&l&j&m))){this.dxBf=h.x-e.x;this.dyBf=h.y-e.y;this.dzBf=h.z-e.z;0<c&&(this.diameter=
c,this.xAf=e.x,this.yAf=e.y,this.zAf=e.z);e=0==d&&(this.clipped||2==g||0==g);this.xA=A(this.xAf);this.yA=A(this.yAf);this.zA=A(this.zAf);this.dxB=A(this.dxBf);this.dyB=A(this.dyBf);this.dzB=A(this.dzBf);this.shadesA=k.getShades(this.colixA=a);this.shadesB=k.getShades(this.colixB=b);this.endcaps=g;this.calcArgbEndcap(!0,!0);x=this.xyzfRaster;0<c&&this.generateBaseEllipsePrecisely(!1);2==g&&this.renderFlatEndcap(!0,!0,x);this.line3d.setLineBits(this.dxBf,this.dyBf);k.setZMargin(5);c=k.pixel;a=k.width;
b=k.zbuf;h=x[0];C=x[1];l=x[2];x=x[3];for(y=this.rasterCount;0<=--y;){var F=x[y]>>8,j=F>>1,m=h[y],r=C[y],n=l[y];this.endCapHidden&&0!=this.argbEndcap&&(this.clipped?(k.plotPixelClippedArgb(this.argbEndcap,this.xEndcap+m,this.yEndcap+r,this.zEndcap-n-1,a,b,c),k.plotPixelClippedArgb(this.argbEndcap,this.xEndcap-m,this.yEndcap-r,this.zEndcap+n-1,a,b,c)):(k.plotPixelUnclipped(this.argbEndcap,this.xEndcap+m,this.yEndcap+r,this.zEndcap-n-1,a,b,c),k.plotPixelUnclipped(this.argbEndcap,this.xEndcap-m,this.yEndcap-
r,this.zEndcap+n-1,a,b,c)));this.ptA0i.set(this.xA+m,this.yA+r,this.zA-n);this.ptB0i.setT(this.ptA0i);this.ptB0i.x+=this.dxB;this.ptB0i.y+=this.dyB;this.ptB0i.z+=this.dzB;this.line3d.plotLineDeltaABitsInt(this.shadesA,this.shadesB,F,this.ptA0i,this.ptB0i,d,this.clipped);e&&(this.ptA0i.set(this.xA-m,this.yA-r,this.zA+n),this.ptB0i.setT(this.ptA0i),this.ptB0i.x+=this.dxB,this.ptB0i.y+=this.dyB,this.ptB0i.z+=this.dzB,this.line3d.plotLineDeltaABitsInt(this.shadesA,this.shadesB,j,this.ptA0i,this.ptB0i,
d,this.clipped))}k.setZMargin(0);3==g&&this.renderSphericalEndcaps();this.xAf+=this.dxBf;this.yAf+=this.dyBf;this.zAf+=this.dzBf}}},"~N,~N,~N,~N,~N,JU.P3i,JU.P3i");c(c$,"renderConeOld",function(a,b,d,g,c,e,h,k,l,j,m){this.dxBf=h-(this.xAf=g);this.dyBf=k-(this.yAf=c);this.dzBf=l-(this.zAf=e);this.xA=w(Math.floor(this.xAf));this.yA=w(Math.floor(this.yAf));this.zA=w(Math.floor(this.zAf));this.dxB=w(Math.floor(this.dxBf));this.dyB=w(Math.floor(this.dyBf));this.dzB=w(Math.floor(this.dzBf));this.xTip=h;
this.yTip=k;this.zTip=l;this.shadesA=this.g3d.getShades(this.colixA=a);var r=this.shader.getShadeIndex(this.dxB,this.dyB,-this.dzB);a=this.g3d;g=a.pixel;c=a.width;e=a.zbuf;a.plotPixelClippedArgb(this.shadesA[r],A(h),A(k),A(l),c,e,g);this.diameter=d;if(1>=d)1==d&&this.line3d.plotLineDeltaOld(this.colixA,this.colixA,this.xA,this.yA,this.zA,this.dxB,this.dyB,this.dzB,this.clipped);else{this.endcaps=b;this.calcArgbEndcap(!1,!0);this.generateBaseEllipsePrecisely(m);!m&&2==this.endcaps&&this.renderFlatEndcap(!1,
!0,this.xyzfRaster);a.setZMargin(5);b=this.xyztRaster[0];d=this.xyztRaster[1];h=this.xyztRaster[2];k=this.xyzfRaster[3];l=this.shadesA;for(var r=this.endCapHidden&&0!=this.argbEndcap,x=this.rasterCount;0<=--x;){var y=b[x],F=d[x],C=h[x],n=k[x]>>8,p=this.xAf+y,q=this.yAf+F,s=this.zAf-C,y=this.xAf-y,F=this.yAf-F,C=this.zAf+C,t=l[0];r&&(a.plotPixelClippedArgb(this.argbEndcap,A(p),A(q),A(s),c,e,g),a.plotPixelClippedArgb(this.argbEndcap,A(y),A(F),A(C),c,e,g));0!=t&&(this.line3d.plotLineDeltaAOld(l,l,0,
n,A(p),A(q),A(s),w(Math.ceil(this.xTip-p)),w(Math.ceil(this.yTip-q)),w(Math.ceil(this.zTip-s)),!0),j&&(this.line3d.plotLineDeltaAOld(l,l,0,n,A(p),A(q)+1,A(s),w(Math.ceil(this.xTip-p)),w(Math.ceil(this.yTip-q))+1,w(Math.ceil(this.zTip-s)),!0),this.line3d.plotLineDeltaAOld(l,l,0,n,A(p)+1,A(q),A(s),w(Math.ceil(this.xTip-p))+1,w(Math.ceil(this.yTip-q)),w(Math.ceil(this.zTip-s)),!0)),!m&&!(2!=this.endcaps&&0<this.dzB)&&this.line3d.plotLineDeltaOld(t,t,A(y),A(F),A(C),w(Math.ceil(this.xTip-y)),w(Math.ceil(this.yTip-
F)),w(Math.ceil(this.zTip-C)),!0))}a.setZMargin(0)}},"~N,~N,~N,~N,~N,~N,~N,~N,~N,~B,~B");c(c$,"generateBaseEllipsePrecisely",function(a){this.calcCosSin(this.dxBf,this.dyBf,this.dzBf);this.calcPoints(a?2:3,!0);this.interpolatePrecisely(0,1,this.xyzfRaster,this.xyztRaster);a||this.interpolatePrecisely(1,2,this.xyzfRaster,this.xyztRaster);for(a=3;0<=--a;)for(var b=this.rasterCount;0<=--b;)this.xyzfRaster[a][b]=w(Math.floor(this.xyztRaster[a][b]))},"~B");c(c$,"calcPoints",function(a,b){this.calcRotatedPoint(0,
0,b,this.xyzfRaster,this.xyztRaster);this.calcRotatedPoint(0.5,1,b,this.xyzfRaster,this.xyztRaster);3==(this.rasterCount=a)&&this.calcRotatedPoint(1,2,b,this.xyzfRaster,this.xyztRaster)},"~N,~B");c(c$,"calcCosSin",function(a,b,d){var g=a*a+b*b;if(0==g)this.cosPhi=this.cosTheta=1,this.sinPhi=0;else{var c=Math.sqrt(g),g=Math.sqrt(g+d*d);this.cosTheta=d/g;this.cosPhi=a/c;this.sinPhi=b/c}},"~N,~N,~N");c(c$,"calcRotatedPoint",function(a,b,d,g,c){c[3][b]=a;a*=3.141592653589793;var e=Math.sin(a)*this.cosTheta,
h=Math.cos(a);a=this.radius*(e*this.cosPhi-h*this.sinPhi);e=this.radius*(e*this.sinPhi+h*this.cosPhi);h=this.radius2-(a*a+e*e);h=0<h?Math.sqrt(h):0;d?(c[0][b]=a,c[1][b]=e,c[2][b]=h):(this.tEvenDiameter?(g[0][b]=w(a-0.5),g[1][b]=w(e-0.5)):(g[0][b]=w(a),g[1][b]=w(e)),g[2][b]=w(h+0.5));g[3][b]=this.shader.getShadeFp8(a,e,h)},"~N,~N,~B,~A,~A");c(c$,"allocRaster",function(a,b,d){if(this.rasterCount>=b[0].length)for(;this.rasterCount>=b[0].length;){for(var g=4;0<=--g;)b[g]=JU.AU.doubleLengthI(b[g]);d[3]=
JU.AU.doubleLengthF(d[3])}if(a)for(;this.rasterCount>=d[0].length;)for(g=3;0<=--g;)d[g]=JU.AU.doubleLengthF(d[g]);return this.rasterCount++},"~B,~A,~A");c(c$,"interpolate",function(a,b,d,g){var c=d[0],e=d[1],c=c[b]-c[a];0>c&&(c=-c);e=e[b]-e[a];0>e&&(e=-e);if(!(1>=c+e)){for(var h=this.allocRaster(!1,d,g),c=d[0],e=d[1],k=d[3],l=g[3][a],j=g[3][b],m=4;0<=--m;){var r=(l+j)/2;this.calcRotatedPoint(r,h,!1,d,g);if(c[h]==c[a]&&e[h]==e[a])k[a]=k[a]+k[h]>>>1,l=r;else if(c[h]==c[b]&&e[h]==e[b])k[b]=k[b]+k[h]>>>
1,j=r;else{this.interpolate(a,h,d,g);this.interpolate(h,b,d,g);return}}c[h]=c[a];e[h]=e[b]}},"~N,~N,~A,~A");c(c$,"interpolatePrecisely",function(a,b,d,g){var c=g[0],e=g[1],h=w(Math.floor(c[b]))-w(Math.floor(c[a]));0>h&&(h=-h);e=w(Math.floor(e[b]))-w(Math.floor(e[a]));0>e&&(e=-e);if(!(1>=h+e)){for(var e=g[3],h=e[a],k=e[b],l=this.allocRaster(!0,d,g),c=g[0],e=g[1],j=d[3],m=4;0<=--m;){var r=(h+k)/2;this.calcRotatedPoint(r,l,!0,d,g);if(w(Math.floor(c[l]))==w(Math.floor(c[a]))&&w(Math.floor(e[l]))==w(Math.floor(e[a])))j[a]=
j[a]+j[l]>>>1,h=r;else if(w(Math.floor(c[l]))==w(Math.floor(c[b]))&&w(Math.floor(e[l]))==w(Math.floor(e[b])))j[b]=j[b]+j[l]>>>1,k=r;else{this.interpolatePrecisely(a,l,d,g);this.interpolatePrecisely(l,b,d,g);return}}c[l]=c[a];e[l]=e[b]}},"~N,~N,~A,~A");c(c$,"renderFlatEndcap",function(a,b,d){var g,c;if(b){if(0==this.dzBf||!this.g3d.setC(this.colixEndcap))return;b=this.xAf;g=this.yAf;c=this.zAf;a&&0>this.dzBf&&(b+=this.dxBf,g+=this.dyBf,c+=this.dzBf);b=A(b);g=A(g);c=A(c)}else{if(0==this.dzB||!this.g3d.setC(this.colixEndcap))return;
b=this.xA;g=this.yA;c=this.zA;a&&0>this.dzB&&(b+=this.dxB,g+=this.dyB,c+=this.dzB)}var e=d[1][0];a=d[1][0];var h=0,k=0,l=d[0],j=d[1];d=d[2];for(var m=this.rasterCount;0<--m;){var r=j[m];r<e?e=r:r>a?a=r:(r=-r,r<e?e=r:r>a&&(a=r))}for(r=e;r<=a;++r){for(var e=2147483647,x=-2147483648,m=this.rasterCount;0<=--m;){if(j[m]==r){var y=l[m];y<e&&(e=y,h=d[m]);y>x&&(x=y,k=d[m])}j[m]==-r&&(y=-l[m],y<e&&(e=y,h=-d[m]),y>x&&(x=y,k=-d[m]))}m=x-e+1;this.g3d.setColorNoisy(this.endcapShadeIndex);this.g3d.plotPixelsClippedRaster(m,
b+e,g+r,c-h-1,c-k-1,null,null)}},"~B,~B,~A");c(c$,"renderSphericalEndcaps",function(){0!=this.colixA&&this.g3d.setC(this.colixA)&&this.g3d.fillSphereXYZ(this.diameter,this.xA,this.yA,this.zA+1);0!=this.colixB&&this.g3d.setC(this.colixB)&&this.g3d.fillSphereXYZ(this.diameter,this.xA+this.dxB,this.yA+this.dyB,this.zA+this.dzB+1)});c(c$,"calcArgbEndcap",function(a,b){this.tEvenDiameter=0==(this.diameter&1);this.radius=this.diameter/2;this.radius2=this.radius*this.radius;this.endCapHidden=!1;var d=b?
this.dzBf:this.dzB;if(!(3==this.endcaps||0==d)){this.xEndcap=this.xA;this.yEndcap=this.yA;this.zEndcap=this.zA;var g=b?this.dxBf:this.dxB,c=b?this.dyBf:this.dyB;0<=d||!a?(this.endcapShadeIndex=this.shader.getShadeIndex(-g,-c,d),this.colixEndcap=this.colixA,d=this.shadesA):(this.endcapShadeIndex=this.shader.getShadeIndex(g,c,-d),this.colixEndcap=this.colixB,d=this.shadesB,this.xEndcap+=this.dxB,this.yEndcap+=this.dyB,this.zEndcap+=this.dzB);56<this.endcapShadeIndex&&(this.endcapShadeIndex=56);this.argbEndcap=
d[this.endcapShadeIndex];this.endCapHidden=1==this.endcaps}},"~B,~B")});s("J.g3d");N(J.g3d,"G3DRenderer");s("J.g3d");v(["J.api.JmolRendererInterface","JU.GData","JU.P3i","$.V3"],"J.g3d.Graphics3D","java.lang.NullPointerException java.util.Arrays JU.AU J.api.Interface J.c.STER J.g3d.CylinderRenderer $.LineRenderer $.Pixelator $.PixelatorScreened $.PixelatorShaded $.PixelatorT $.Platform3D $.SphereRenderer $.TextRenderer $.TextString JU.C $.Font $.Normix".split(" "),function(){c$=t(function(){this.hermite3d=
this.circle3d=this.triangle3d=this.cylinder3d=this.sphere3d=this.line3d=this.platform=null;this.antialias2=this.isFullSceneAntialiasingEnabled=!1;this.strings=null;this.stringCount=0;this.anaglyphChannelBytes=null;this.$haveTranslucentObjects=this.twoPass=!1;this.zbufT=this.zbuf=this.pbufT=this.pbuf=null;this.translucencyMask=0;this.renderLow=!1;this.shadesCurrent=null;this.anaglyphLength=0;this.pixelShaded=this.pixelScreened=this.pixelT0=this.pixel0=this.pixel=null;this.zMargin=0;this.aobuf=null;
this.translucencyLog=this.lastRawColor=this.currentShadeIndex=0;this.wasScreened=!1;this.saveDiffuse=this.saveAmbient=0;this.shadeIndexes2Sided=this.shadeIndexes=this.vectorNormal=this.vectorAC=this.vectorAB=this.sC=this.sB=this.sA=null;this.pass2Flag01=0;p(this,arguments)},J.g3d,"Graphics3D",JU.GData,J.api.JmolRendererInterface);O(c$,function(){this.sA=new JU.P3i;this.sB=new JU.P3i;this.sC=new JU.P3i;this.vectorAB=new JU.V3;this.vectorAC=new JU.V3;this.vectorNormal=new JU.V3;this.shadeIndexes=Q(JU.GData.normixCount,
0);this.shadeIndexes2Sided=Q(JU.GData.normixCount,0)});j(c$,"isWebGL",function(){return!1});j(c$,"clear",function(){this.stringCount=0;this.strings=null;J.g3d.TextRenderer.clearFontCache()});j(c$,"destroy",function(){this.releaseBuffers();this.graphicsForMetrics=this.pixelScreened=this.pixelT0=this.pixel=this.pixel0=this.pixelShaded=this.platform=null});c(c$,"setZMargin",function(a){this.zMargin=a},"~N");q(c$,function(){I(this,J.g3d.Graphics3D,[]);for(var a=JU.GData.normixCount;0<=--a;)this.transformedVectors[a]=
new JU.V3});j(c$,"initialize",function(a,b){this.vwr=a;this.apiPlatform=b;this.platform=new J.g3d.Platform3D(b);this.pixel=this.pixel0=new J.g3d.Pixelator(this);this.graphicsForMetrics=this.platform.getGraphicsForMetrics();this.line3d=new J.g3d.LineRenderer(this);this.sphere3d=new J.g3d.SphereRenderer(this);this.cylinder3d=new J.g3d.CylinderRenderer(this)},"JV.Viewer,J.api.GenericPlatform");j(c$,"addRenderer",function(a){switch(a){case 1073741880:null==this.circle3d&&(this.circle3d=this.getRenderer("Circle"));
break;case 553648145:null==this.hermite3d&&(this.hermite3d=this.getRenderer("Hermite"));case 1073742182:null==this.triangle3d&&(this.triangle3d=this.getRenderer("Triangle"),this.triangle3d.isOrthographic=!this.vwr.tm.perspectiveDepth)}},"~N");c(c$,"getRenderer",function(a){a=J.api.Interface.getOption("g3d."+a+"Renderer",this.vwr,"render");if(null==a)throw new NullPointerException("Interface");a.set(this,this);return a},"~S");j(c$,"setWindowParameters",function(a,b,d){this.setWinParams(a,b,d);this.currentlyRendering&&
this.endRendering()},"~N,~N,~B");j(c$,"checkTranslucent",function(a){a&&(this.$haveTranslucentObjects=!0);return!this.twoPass||this.twoPass&&this.isPass2==a},"~B");j(c$,"beginRendering",function(a,b,d,g){this.currentlyRendering&&this.endRendering();this.renderLow=g;if(this.windowWidth!=this.newWindowWidth||this.windowHeight!=this.newWindowHeight||this.newAntialiasing!=this.isFullSceneAntialiasingEnabled)this.windowWidth=this.newWindowWidth,this.windowHeight=this.newWindowHeight,this.isFullSceneAntialiasingEnabled=
this.newAntialiasing,this.releaseBuffers();this.setRotationMatrix(a);this.line3d.isOrthographic=!this.vwr.tm.perspectiveDepth;null!=this.triangle3d&&(this.triangle3d.isOrthographic=!this.vwr.tm.perspectiveDepth);this.antialiasEnabled=this.antialiasThisFrame=this.newAntialiasing;this.currentlyRendering=!0;if(null!=this.strings)for(a=Math.min(this.strings.length,this.stringCount);0<=--a;)this.strings[a]=null;this.stringCount=0;this.twoPass=!0;this.isPass2=!1;this.colixCurrent=this.pass2Flag01=0;this.$haveTranslucentObjects=
this.wasScreened=!1;this.pixel=this.pixel0;this.pixel.bgcolor=this.bgcolor;this.translucentCoverOnly=!b;null==this.pbuf&&(this.platform.allocateBuffers(this.windowWidth,this.windowHeight,this.antialiasThisFrame,d),this.pbuf=this.platform.pBuffer,this.zbuf=this.platform.zBuffer,this.aobuf=null,this.pixel0.setBuf(),null!=this.pixelT0&&this.pixelT0.setBuf(),null!=this.pixelShaded&&this.pixelShaded.setBuf());this.setWidthHeight(this.antialiasThisFrame);null!=this.pixelScreened&&(this.pixelScreened.width=
this.width);this.platform.clearBuffer();null!=this.backgroundImage&&this.plotImage(-2147483648,0,-2147483648,this.backgroundImage,null,0,0,0);this.textY=0},"JU.M3,~B,~B,~B");j(c$,"setBackgroundTransparent",function(a){null!=this.platform&&this.platform.setBackgroundTransparent(a)},"~B");c(c$,"releaseBuffers",function(){this.aobuf=this.zbufT=this.pbufT=this.zbuf=this.pbuf=null;this.platform.releaseBuffers();this.line3d.clearLineCache()});j(c$,"setPass2",function(a){if(!this.$haveTranslucentObjects||
!this.currentlyRendering)return!1;this.isPass2=!0;this.pass2Flag01=1;this.colixCurrent=0;if(null==this.pbufT||this.antialias2!=a)this.platform.allocateTBuffers(a),this.pbufT=this.platform.pBufferT,this.zbufT=this.platform.zBufferT;this.antialias2=a;this.antialiasThisFrame&&!this.antialias2&&this.downsampleFullSceneAntialiasing(!0);this.platform.clearTBuffer();null==this.pixelT0&&(this.pixelT0=new J.g3d.PixelatorT(this));null==this.pixel.p0?this.pixel=this.pixelT0:this.pixel.p0=this.pixelT0;return!0},
"~B");j(c$,"endRendering",function(){if(this.currentlyRendering){if(null!=this.pbuf){if(this.isPass2&&null!=this.pbufT)for(var a=this.pbufT.length;0<=--a;)this.pbuf[a]=J.g3d.Graphics3D.mergeBufferPixel(this.pbuf[a],this.pbufT[a],this.bgcolor);this.pixel===this.pixelShaded&&0==this.pixelShaded.zShadePower&&this.pixelShaded.showZBuffer();this.antialiasThisFrame&&this.downsampleFullSceneAntialiasing(!1)}this.platform.setBackgroundColor(this.bgcolor);this.platform.notifyEndOfRendering();this.currentlyRendering=
this.isPass2=!1}});c$.mergeBufferPixel=c(c$,"mergeBufferPixel",function(a,b,d){if(0==b||a==b)return a;0==a&&(a=d);d=a&16711935;a&=65280;var g=b&16711935,c=b&65280;switch(b>>24&15){case 0:d=g;a=c;break;case 1:d=(g<<2)+(g<<1)+g+d>>3&16711935;a=(c<<2)+ +(c<<1)+c+a>>3&65280;break;case 2:d=(g<<1)+g+d>>2&16711935;a=(c<<1)+c+a>>2&65280;break;case 3:d=(g<<2)+g+(d<<1)+d>>3&16711935;a=(c<<2)+c+(a<<1)+a>>3&65280;break;case 4:d=d+g>>1&16711935;a=a+c>>1&65280;break;case 5:d=(g<<1)+g+(d<<2)+d>>3&16711935;a=(c<<
1)+c+(a<<2)+a>>3&65280;break;case 6:d=(d<<1)+d+g>>2&16711935;a=(a<<1)+a+c>>2&65280;break;case 7:d=(d<<2)+(d<<1)+d+g>>3&16711935,a=(a<<2)+(a<<1)+a+c>>3&65280}return 4278190080|d|a},"~N,~N,~N");j(c$,"getScreenImage",function(a){var b=this.platform.bufferedImage;a&&this.releaseBuffers();return b},"~B");j(c$,"applyAnaglygh",function(a,b){switch(a){case J.c.STER.REDCYAN:for(var d=this.anaglyphLength;0<=--d;){var g=this.anaglyphChannelBytes[d]&255;this.pbuf[d]=this.pbuf[d]&4294901760|g<<8|g}break;case J.c.STER.CUSTOM:for(var g=
b[0],c=b[1]&16777215,d=this.anaglyphLength;0<=--d;){var e=this.anaglyphChannelBytes[d]&255,e=(e|(e|e<<8)<<8)&c;this.pbuf[d]=this.pbuf[d]&g|e}break;case J.c.STER.REDBLUE:for(d=this.anaglyphLength;0<=--d;)g=this.anaglyphChannelBytes[d]&255,this.pbuf[d]=this.pbuf[d]&4294901760|g;break;case J.c.STER.REDGREEN:for(d=this.anaglyphLength;0<=--d;)this.pbuf[d]=this.pbuf[d]&4294901760|(this.anaglyphChannelBytes[d]&255)<<8}},"J.c.STER,~A");j(c$,"snapshotAnaglyphChannelBytes",function(){if(this.currentlyRendering)throw new NullPointerException;
this.anaglyphLength=this.windowWidth*this.windowHeight;if(null==this.anaglyphChannelBytes||this.anaglyphChannelBytes.length!=this.anaglyphLength)this.anaglyphChannelBytes=Q(this.anaglyphLength,0);for(var a=this.anaglyphLength;0<=--a;)this.anaglyphChannelBytes[a]=this.pbuf[a]});j(c$,"releaseScreenImage",function(){this.platform.clearScreenBufferThreaded()});j(c$,"haveTranslucentObjects",function(){return this.$haveTranslucentObjects});j(c$,"setSlabAndZShade",function(a,b,d,g,c){this.setSlab(a);this.setDepth(b);
d<g?(null==this.pixelShaded&&(this.pixelShaded=new J.g3d.PixelatorShaded(this)),this.pixel=this.pixelShaded.set(d,g,c)):this.pixel=this.pixel0},"~N,~N,~N,~N,~N");c(c$,"downsampleFullSceneAntialiasing",function(a){var b=this.bgcolor;a&&(b+=255==(b&255)?-1:1);J.g3d.Graphics3D.downsample2d(this.pbuf,this.windowWidth,this.windowHeight,b);a&&(J.g3d.Graphics3D.downsample2dZ(this.pbuf,this.zbuf,this.windowWidth,this.windowHeight,b),this.antialiasThisFrame=!1,this.setWidthHeight(!1))},"~B");c$.downsample2d=
c(c$,"downsample2d",function(a,b,d,g){var c=b<<1;if(0!=g){g&=16777215;for(var e=a.length;0<=--e;)0==a[e]&&(a[e]=g)}for(var h=(g>>2&1061109567)<<2,h=h+((h&3233857728)>>6),k=0,l=0,e=d;0<=--e;l+=c)for(d=b;0<=--d;++k){var j=(a[l]>>2&1061109567)+(a[l++ +c]>>2&1061109567)+(a[l]>>2&1061109567)+(a[l++ +c]>>2&1061109567),j=j+((j&3233857728)>>6);j==h&&(j=g);a[k]=j&16777215|4278190080}},"~A,~N,~N,~N");c$.downsample2dZ=c(c$,"downsample2dZ",function(a,b,d,g,c){for(var e=d<<1,h=0,k=0;0<=--g;k+=e)for(var l=d;0<=
--l;++h,++k){var j=Math.min(b[k],b[k+e]),j=Math.min(j,b[++k]),j=Math.min(j,b[k+e]);2147483647!=j&&(j>>=1);b[h]=a[h]==c?2147483647:j}},"~A,~A,~N,~N,~N");c(c$,"hasContent",function(){return this.platform.hasContent()});j(c$,"setC",function(a){var b=JU.C.isColixLastAvailable(a);if(!b&&a==this.colixCurrent&&-1==this.currentShadeIndex)return!0;var d=a&30720;if(16384==d)return!1;this.renderLow&&(d=0);var g=0!=d,c=g&&30720==d;this.setScreened(c);if(!this.checkTranslucent(g&&!c))return!1;this.isPass2?(this.translucencyMask=
d<<13|16777215,this.translucencyLog=d>>11):this.translucencyLog=0;this.colixCurrent=a;b&&this.argbCurrent!=this.lastRawColor&&(0==this.argbCurrent&&(this.argbCurrent=4294967295),this.lastRawColor=this.argbCurrent,this.shader.setLastColix(this.argbCurrent,this.inGreyscaleMode));this.shadesCurrent=this.getShades(a);this.currentShadeIndex=-1;this.setColor(this.getColorArgbOrGray(a));return!0},"~N");c(c$,"setScreened",function(a){this.wasScreened!=a&&((this.wasScreened=a)?(null==this.pixelScreened&&(this.pixelScreened=
new J.g3d.PixelatorScreened(this,this.pixel0)),null==this.pixel.p0?this.pixel=this.pixelScreened:this.pixel.p0=this.pixelScreened):null==this.pixel.p0||this.pixel===this.pixelScreened?this.pixel=this.isPass2?this.pixelT0:this.pixel0:this.pixel.p0=this.isPass2?this.pixelT0:this.pixel0);return this.pixel},"~B");j(c$,"drawFilledCircle",function(a,b,d,g,c,e){if(!this.isClippedZ(e)){var h=w((d+1)/2),h=g<h||g+h>=this.width||c<h||c+h>=this.height;if(!h||!this.isClippedXY(d,g,c))0!=a&&this.setC(a)&&(h?this.isClippedXY(d,
g,c)||this.circle3d.plotCircleCenteredClipped(g,c,e,d):this.circle3d.plotCircleCenteredUnclipped(g,c,e,d)),0!=b&&this.setC(b)&&(h?this.circle3d.plotFilledCircleCenteredClipped(g,c,e,d):this.circle3d.plotFilledCircleCenteredUnclipped(g,c,e,d))}},"~N,~N,~N,~N,~N,~N");j(c$,"volumeRender4",function(a,b,d,g){if(1==a)this.plotPixelClippedArgb(this.argbCurrent,b,d,g,this.width,this.zbuf,this.pixel);else if(!this.isClippedZ(g)){var c=w((a+1)/2),c=b<c||b+c>=this.width||d<c||d+c>=this.height;if(!c||!this.isClippedXY(a,
b,d))c?this.circle3d.plotFilledCircleCenteredClipped(b,d,g,a):this.circle3d.plotFilledCircleCenteredUnclipped(b,d,g,a)}},"~N,~N,~N,~N");j(c$,"fillSphereXYZ",function(a,b,d,g){switch(a){case 1:this.plotPixelClippedArgb(this.argbCurrent,b,d,g,this.width,this.zbuf,this.pixel);return;case 0:return}a<=(this.antialiasThisFrame?2E3:1E3)&&this.sphere3d.render(this.shadesCurrent,a,b,d,g,null,null,null,-1,null)},"~N,~N,~N,~N");j(c$,"volumeRender",function(a){a?(this.saveAmbient=this.getAmbientPercent(),this.saveDiffuse=
this.getDiffusePercent(),this.setAmbientPercent(100),this.setDiffusePercent(0),this.addRenderer(1073741880)):(this.setAmbientPercent(this.saveAmbient),this.setDiffusePercent(this.saveDiffuse))},"~B");j(c$,"fillSphereI",function(a,b){this.fillSphereXYZ(a,b.x,b.y,b.z)},"~N,JU.P3i");j(c$,"fillSphereBits",function(a,b){this.fillSphereXYZ(a,Math.round(b.x),Math.round(b.y),Math.round(b.z))},"~N,JU.P3");j(c$,"fillEllipsoid",function(a,b,d,g,c,e,h,k,l,j,m){switch(e){case 1:this.plotPixelClippedArgb(this.argbCurrent,
d,g,c,this.width,this.zbuf,this.pixel);return;case 0:return}e<=(this.antialiasThisFrame?2E3:1E3)&&this.sphere3d.render(this.shadesCurrent,e,d,g,c,h,k,l,j,m)},"JU.P3,~A,~N,~N,~N,~N,JU.M3,~A,JU.M4,~N,~A");j(c$,"drawRect",function(a,b,d,g,c,e){if(!(0!=g&&this.isClippedZ(g))){g=c-1;e-=1;c=a+g;var h=b+e;0<=b&&b<this.height&&this.drawHLine(a,b,d,g);0<=h&&h<this.height&&this.drawHLine(a,h,d,g);0<=a&&a<this.width&&this.drawVLine(a,b,d,e);0<=c&&c<this.width&&this.drawVLine(c,b,d,e)}},"~N,~N,~N,~N,~N,~N");
c(c$,"drawHLine",function(a,b,d,g){0>g&&(a+=g,g=-g);0>a&&(g+=a,a=0);a+g>=this.width&&(g=this.width-1-a);var c=this.pixel,e=this.argbCurrent;a+=this.width*b;for(b=0;b<=g;b++)d<this.zbuf[a]&&c.addPixel(a,d,e),a++},"~N,~N,~N,~N");c(c$,"drawVLine",function(a,b,d,c){0>c&&(b+=c,c=-c);0>b&&(c+=b,b=0);b+c>=this.height&&(c=this.height-1-b);a+=this.width*b;b=this.pixel;for(var f=this.argbCurrent,e=0;e<=c;e++)d<this.zbuf[a]&&b.addPixel(a,d,f),a+=this.width},"~N,~N,~N,~N");j(c$,"fillTextRect",function(a,b,d,
c,f,e){if(!this.isClippedZ(c)){c=this.width;if(0>a){f+=a;if(0>=f)return;a=0}if(a+f>c&&(f=c-a,0>=f))return;if(0>b){e+=b;if(0>=e)return;b=0}b+e>this.height&&(e=this.height-b);for(var h=this.argbCurrent,k=this.zbuf,l=this.pixel;0<=--e;)this.plotPixelsUnclippedCount(h,f,a,b++,d,c,k,l)}},"~N,~N,~N,~N,~N,~N");j(c$,"drawString",function(a,b,d,c,f,e,h){this.currentShadeIndex=0;null!=a&&(this.isClippedZ(e)||this.drawStringNoSlab(a,b,d,c,f,h))},"~S,JU.Font,~N,~N,~N,~N,~N");j(c$,"drawStringNoSlab",function(a,
b,d,c,f,e){if(null!=a){null==this.strings&&(this.strings=Array(10));this.stringCount==this.strings.length&&(this.strings=JU.AU.doubleLength(this.strings));var h=new J.g3d.TextString;h.setText(a,null==b?this.currentFont:this.currentFont=b,this.argbCurrent,JU.C.isColixTranslucent(e)?this.getColorArgbOrGray(e)&16777215|(e&30720)<<13:0,d,c,f);this.strings[this.stringCount++]=h}},"~S,JU.Font,~N,~N,~N,~N");j(c$,"renderAllStrings",function(a){if(null!=this.strings){2<=this.stringCount&&(null==J.g3d.Graphics3D.sort&&
(J.g3d.Graphics3D.sort=new J.g3d.TextString),java.util.Arrays.sort(this.strings,J.g3d.Graphics3D.sort));for(var b=0;b<this.stringCount;b++){var d=this.strings[b];this.plotText(d.x,d.y,d.z,d.argb,d.bgargb,d.text,d.font,a)}this.strings=null;this.stringCount=0}},"~O");j(c$,"plotText",function(a,b,d,c,f,e,h,k){J.g3d.TextRenderer.plot(a,b,d,c,f,e,h,this,k,this.antialiasThisFrame)},"~N,~N,~N,~N,~N,~S,JU.Font,J.api.JmolRendererInterface");j(c$,"drawImage",function(a,b,d,c,f,e,h,k){null!=a&&(0<h&&0<k&&!this.isClippedZ(f))&&
this.plotImage(b,d,c,a,null,e,h,k)},"~O,~N,~N,~N,~N,~N,~N,~N");j(c$,"plotImage",function(a,b,d,c,f,e,h,k){this.setC(e);this.isPass2||(this.translucencyMask=-1);0==e&&(this.argbCurrent=0);var l=-2147483648==a;e=l?this.bgcolor:this.argbCurrent;l&&(a=0,d=2147483646,h=this.width,k=this.height);if(!(0>=a+h||a>=this.width||0>=b+k||b>=this.height))if(c=this.apiPlatform.drawImageToBuffer(null,this.platform.offscreenImage,c,h,k,l?e:0),null!=c){var l=this.zbuf,j=this.width,m=this.pixel,r=this.height,x=this.translucencyLog;
if(null==f&&0<=a&&a+h<=j&&0<=b&&b+k<=r){var y=0,F=0;for(a=b*j+a;y<k;y++,a+=j-h)for(var C=0;C<h;C++,F++,a++)if(d<l[a]){var n=c[F];-16777216==(n&4278190080)&&m.addPixel(a,d,n)}}else{null==f&&(f=this);for(F=y=0;y<k;y++)for(C=0;C<h;C++)n=c[F++],-16777216==(n&4278190080)&&f.plotImagePixel(n,a+C,b+y,d,8,e,j,r,l,m,x)}}},"~N,~N,~N,~O,J.api.JmolRendererInterface,~N,~N,~N");j(c$,"setFontFid",function(a){this.currentFont=JU.Font.getFont3D(a)},"~N");j(c$,"setFont",function(a){this.currentFont=a},"JU.Font");j(c$,
"drawPixel",function(a,b,d){this.plotPixelClippedArgb(this.argbCurrent,a,b,d,this.width,this.zbuf,this.pixel)},"~N,~N,~N");j(c$,"drawPoints",function(a,b,d){if(1<d)for(var c=0.8*d*d,f=-d;f<d;f++)for(var e=-d;e<d;e++)f*f+e*e>c||(this.plotPoints(a,b,f,e),this.plotPoints(a,b,f,e));else this.plotPoints(a,b,0,0)},"~N,~A,~N");j(c$,"drawDashedLineBits",function(a,b,d,c){this.isAntialiased()&&(a+=a,b+=b);this.setScreeni(d,this.sA);this.setScreeni(c,this.sB);this.line3d.plotLineBits(this.argbCurrent,this.argbCurrent,
this.sA,this.sB,a,b,!0);this.isAntialiased()&&(Math.abs(d.x-c.x)<Math.abs(d.y-c.y)?(this.sA.x+=1,this.sB.x+=1):(this.sA.y+=1,this.sB.y+=1),this.line3d.plotLineBits(this.argbCurrent,this.argbCurrent,this.sA,this.sB,a,b,!0))},"~N,~N,JU.P3,JU.P3");c(c$,"setScreeni",function(a,b){b.x=Math.round(a.x);b.y=Math.round(a.y);b.z=Math.round(a.z)},"JU.P3,JU.P3i");j(c$,"drawLineXYZ",function(a,b,d,c,f,e){this.line3d.plotLineOld(this.argbCurrent,this.argbCurrent,a,b,d,c,f,e)},"~N,~N,~N,~N,~N,~N");j(c$,"drawLine",
function(a,b,d,c,f,e,h,k){this.setC(a)||(a=0);var l=this.argbCurrent;this.setC(b)||(b=0);(0!=a||0!=b)&&this.line3d.plotLineOld(l,this.argbCurrent,d,c,f,e,h,k)},"~N,~N,~N,~N,~N,~N,~N,~N");j(c$,"drawLineBits",function(a,b,d,c){this.setC(a)||(a=0);var f=this.argbCurrent;this.setC(b)||(b=0);if(0!=a||0!=b)this.setScreeni(d,this.sA),this.setScreeni(c,this.sB),this.line3d.plotLineBits(f,this.argbCurrent,this.sA,this.sB,0,0,!1)},"~N,~N,JU.P3,JU.P3");j(c$,"drawLineAB",function(a,b){this.setScreeni(a,this.sA);
this.setScreeni(b,this.sB);this.line3d.plotLineBits(this.argbCurrent,this.argbCurrent,this.sA,this.sB,0,0,!1)},"JU.P3,JU.P3");j(c$,"fillCylinderXYZ",function(a,b,d,c,f,e,h,k,l,j){if(!(c>this.ht3)){var m=0;this.currentShadeIndex=0;this.setC(b)||(b=0);this.wasScreened&&(m=2);this.setC(a)||(a=0);this.wasScreened&&(m+=1);0==a&&0==b||this.cylinder3d.renderOld(a,b,m,d,c,f,e,h,k,l,j)}},"~N,~N,~N,~N,~N,~N,~N,~N,~N,~N");j(c$,"fillCylinderScreen3I",function(a,b,d,c){b<=this.ht3&&this.cylinder3d.renderOld(this.colixCurrent,
this.colixCurrent,0,a,b,A(d.x),A(d.y),A(d.z),A(c.x),A(c.y),A(c.z))},"~N,~N,JU.P3,JU.P3,JU.P3,JU.P3,~N");j(c$,"fillCylinder",function(a,b,d,c){b<=this.ht3&&this.cylinder3d.renderOld(this.colixCurrent,this.colixCurrent,0,a,b,d.x,d.y,d.z,c.x,c.y,c.z)},"~N,~N,JU.P3i,JU.P3i");j(c$,"fillCylinderBits",function(a,b,d,c){b<=this.ht3&&(1!=d.z&&1!=c.z)&&(0==b||1==b?(this.setScreeni(d,this.sA),this.setScreeni(c,this.sB),this.line3d.plotLineBits(this.getColorArgbOrGray(this.colixCurrent),this.getColorArgbOrGray(this.colixCurrent),
this.sA,this.sB,0,0,!1)):this.cylinder3d.renderBitsFloat(this.colixCurrent,this.colixCurrent,0,a,b,d,c))},"~N,~N,JU.P3,JU.P3");j(c$,"fillCylinderBits2",function(a,b,d,c,f,e){if(!(c>this.ht3)){var h=0;this.currentShadeIndex=0;this.setC(b)||(b=0);this.wasScreened&&(h=2);this.setC(a)||(a=0);this.wasScreened&&(h+=1);0==a&&0==b||(this.setScreeni(f,this.sA),this.setScreeni(e,this.sB),this.cylinder3d.renderBits(a,b,h,d,c,this.sA,this.sB))}},"~N,~N,~N,~N,JU.P3,JU.P3");j(c$,"fillConeScreen3f",function(a,b,
d,c,f){b<=this.ht3&&this.cylinder3d.renderConeOld(this.colixCurrent,a,b,d.x,d.y,d.z,c.x,c.y,c.z,!0,f)},"~N,~N,JU.P3,JU.P3,~B");j(c$,"drawHermite4",function(a,b,d,c,f){this.hermite3d.renderHermiteRope(!1,a,0,0,0,b,d,c,f)},"~N,JU.P3,JU.P3,JU.P3,JU.P3");j(c$,"drawHermite7",function(a,b,d,c,f,e,h,k,l,j,m,r,x){if(0==x)this.hermite3d.renderHermiteRibbon(a,b,d,c,f,e,h,k,l,j,m,r,0);else{this.hermite3d.renderHermiteRibbon(a,b,d,c,f,e,h,k,l,j,m,r,1);var y=this.colixCurrent;this.setC(x);this.hermite3d.renderHermiteRibbon(a,
b,d,c,f,e,h,k,l,j,m,r,-1);this.setC(y)}},"~B,~B,~N,JU.P3,JU.P3,JU.P3,JU.P3,JU.P3,JU.P3,JU.P3,JU.P3,~N,~N");j(c$,"fillHermite",function(a,b,d,c,f,e,h,k){this.hermite3d.renderHermiteRope(!0,a,b,d,c,f,e,h,k)},"~N,~N,~N,~N,JU.P3,JU.P3,JU.P3,JU.P3");j(c$,"drawTriangle3C",function(a,b,d,c,f,e,h){1==(h&1)&&this.drawLine(b,c,a.x,a.y,a.z,d.x,d.y,d.z);2==(h&2)&&this.drawLine(c,e,d.x,d.y,d.z,f.x,f.y,f.z);4==(h&4)&&this.drawLine(b,e,a.x,a.y,a.z,f.x,f.y,f.z)},"JU.P3i,~N,JU.P3i,~N,JU.P3i,~N,~N");j(c$,"fillTriangleTwoSided",
function(a,b,d,c){this.setColorNoisy(this.getShadeIndex(a));this.fillTriangleP3f(b,d,c,!1)},"~N,JU.P3,JU.P3,JU.P3");c(c$,"fillTriangleP3f",function(a,b,d,c){this.setScreeni(a,this.sA);this.setScreeni(b,this.sB);this.setScreeni(d,this.sC);this.triangle3d.fillTriangle(this.sA,this.sB,this.sC,c)},"JU.P3,JU.P3,JU.P3,~B");j(c$,"fillTriangle3f",function(a,b,d,c){var f=this.getShadeIndexP3(a,b,d,c);0>f||(c?this.setColorNoisy(f):this.setColor(this.shadesCurrent[f]),this.fillTriangleP3f(a,b,d,!1))},"JU.P3,JU.P3,JU.P3,~B");
j(c$,"fillTriangle3i",function(a,b,d,c,f,e,h){h&&(c=this.vectorAB,c.set(b.x-a.x,b.y-a.y,b.z-a.z),null==d?c=this.shader.getShadeIndex(-c.x,-c.y,c.z):(this.vectorAC.set(d.x-a.x,d.y-a.y,d.z-a.z),c.cross(c,this.vectorAC),c=0<=c.z?this.shader.getShadeIndex(-c.x,-c.y,c.z):this.shader.getShadeIndex(c.x,c.y,-c.z)),56<c&&(c=56),this.setColorNoisy(c));this.fillTriangleP3f(a,b,d,!1)},"JU.P3,JU.P3,JU.P3,JU.T3,JU.T3,JU.T3,~B");j(c$,"fillTriangle3CN",function(a,b,d,c,f,e,h,k,l){this.triangle3d.fillTriangle(a,c,
h,this.checkGouraud(b,f,k,d,e,l))},"JU.P3i,~N,~N,JU.P3i,~N,~N,JU.P3i,~N,~N");j(c$,"fillTriangle3CNBits",function(a,b,d,c,f,e,h,k,l){this.fillTriangleP3f(a,c,h,this.checkGouraud(b,f,k,d,e,l))},"JU.P3,~N,~N,JU.P3,~N,~N,JU.P3,~N,~N,~B");c(c$,"checkGouraud",function(a,b,d,c,f,e){if(!this.isPass2&&c==f&&c==e&&a==b&&a==d){b=this.getShadeIndex(c);if(a!=this.colixCurrent||this.currentShadeIndex!=b)this.currentShadeIndex=-1,this.setC(a),this.setColorNoisy(b);return!1}this.setTriangleTranslucency(a,b,d);this.triangle3d.setGouraud(this.getShades(a)[this.getShadeIndex(c)],
this.getShades(b)[this.getShadeIndex(f)],this.getShades(d)[this.getShadeIndex(e)]);return!0},"~N,~N,~N,~N,~N,~N");c(c$,"getShadeIndex",function(a){return-1E4==a||9999==a?J.g3d.Graphics3D.nullShadeIndex:0>a?this.shadeIndexes2Sided[~a]:this.shadeIndexes[a]},"~N");c(c$,"setTriangleTranslucency",function(a,b,d){this.isPass2&&(a&=14336,b&=14336,d&=14336,this.translucencyMask=(JU.GData.roundInt(w((a+b+d)/3))&30720)<<13|16777215)},"~N,~N,~N");j(c$,"fillQuadrilateral",function(a,b,d,c,f){f=this.getShadeIndexP3(a,
b,d,f);0>f||(this.setColorNoisy(f),this.fillTriangleP3f(a,b,d,!1),this.fillTriangleP3f(a,d,c,!1))},"JU.P3,JU.P3,JU.P3,JU.P3,~B");j(c$,"drawSurface",function(){},"JU.MeshSurface,~N");j(c$,"plotPixelClippedP3i",function(a){this.plotPixelClippedArgb(this.argbCurrent,a.x,a.y,a.z,this.width,this.zbuf,this.pixel)},"JU.P3i");c(c$,"plotPixelClippedArgb",function(a,b,d,c,f,e,h){this.isClipped3(b,d,c)||(b=d*f+b,c<e[b]&&h.addPixel(b,c,a))},"~N,~N,~N,~N,~N,~A,J.g3d.Pixelator");c(c$,"plotPixelUnclipped",function(a,
b,d,c,f,e,h){b=d*f+b;c<e[b]&&h.addPixel(b,c,a)},"~N,~N,~N,~N,~N,~A,J.g3d.Pixelator");c(c$,"plotImagePixel",function(a,b,d,c,f,e,h,k,l,j,m){0>b||(b>=h||0>d||d>=k)||j.addImagePixel(f,m,d*h+b,c,a,e)},"~N,~N,~N,~N,~N,~N,~N,~N,~A,~O,~N");c(c$,"plotPixelsClippedRaster",function(a,b,d,c,f,e,h){var k,l;if(!(0>=a||0>d||d>=this.height||b>=this.width||c<(l=this.slab)&&f<l||c>(k=this.depth)&&f>k)){var j=this.zbuf,m=(b<<16)+(d<<1)^858993459,r=(c<<10)+512;c=f-c;f=w(a/2);c=JU.GData.roundInt(w(((c<<10)+(0<=c?f:-f))/
a));if(0>b){b=-b;r+=c*b;a-=b;if(0>=a)return;b=0}a+b>this.width&&(a=this.width-b);b=d*this.width+b;d=this.pixel;if(null==e){e=this.argbNoisyDn;f=this.argbNoisyUp;for(var x=this.argbCurrent;0<=--a;){h=r>>10;if(h>=l&&h<=k&&h<j[b]){var m=(m<<16)+(m<<1)+m&2147483647,y=m>>16&7;d.addPixel(b,h,0==y?e:1==y?f:x)}++b;r+=c}}else{m=e.r<<8;f=w((h.r-e.r<<8)/a);x=e.g;y=w((h.g-x)/a);e=e.b;for(var F=w((h.b-e)/a);0<=--a;)h=r>>10,h>=l&&(h<=k&&h<j[b])&&d.addPixel(b,h,4278190080|m&16711680|x&65280|e>>8&255),++b,r+=c,m+=
f,x+=y,e+=F}}},"~N,~N,~N,~N,~N,JU.Rgb16,JU.Rgb16");c(c$,"plotPixelsUnclippedRaster",function(a,b,d,c,f,e,h){if(!(0>=a)){var k=((b<<16)+(d<<1)^858993459)&2147483647,l=(c<<10)+512;c=f-c;f=w(a/2);c=JU.GData.roundInt(w(((c<<10)+(0<=c?f:-f))/a));b=d*this.width+b;d=this.zbuf;f=this.pixel;if(null==e){e=this.argbNoisyDn;for(var j=this.argbNoisyUp,m=this.argbCurrent;0<=--a;){h=l>>10;if(h<d[b]){var k=(k<<16)+(k<<1)+k&2147483647,r=k>>16&7;f.addPixel(b,h,0==r?e:1==r?j:m)}++b;l+=c}}else{k=e.r<<8;j=JU.GData.roundInt(w((h.r-
e.r<<8)/a));m=e.g;r=JU.GData.roundInt(w((h.g-m)/a));e=e.b;for(var x=JU.GData.roundInt(w((h.b-e)/a));0<=--a;)h=l>>10,h<d[b]&&f.addPixel(b,h,4278190080|k&16711680|m&65280|e>>8&255),++b,l+=c,k+=j,m+=r,e+=x}}},"~N,~N,~N,~N,~N,JU.Rgb16,JU.Rgb16");c(c$,"plotPixelsClippedRasterBits",function(a,b,d,c,f,e,h,k,l){var j,m;if(!(0>=a||0>d||d>=this.height||b>=this.width||c<(m=this.slab)&&f<m||c>(j=this.depth)&&f>j)){c=this.zbuf;var r=(b<<16)+(d<<1)^858993459;if(0>b){a-=-b;if(0>=a)return;b=0}a+b>this.width&&(a=
this.width-b);d=d*this.width+b;f=this.pixel;if(null==e){e=this.argbNoisyDn;for(var x=this.argbNoisyUp,y=this.argbCurrent;0<=--a;){h=this.line3d.getZCurrent(k,l,b++);if(h>=m&&h<=j&&h<c[d]){var r=(r<<16)+(r<<1)+r&2147483647,F=r>>16&7;f.addPixel(d,h,2>F?e:6>F?x:y)}++d}}else{r=e.r<<8;x=w((h.r-e.r<<8)/a);y=e.g;F=w((h.g-y)/a);e=e.b;for(var C=w((h.b-e)/a);0<=--a;)h=this.line3d.getZCurrent(k,l,b++),h>=m&&(h<=j&&h<c[d])&&f.addPixel(d,h,4278190080|r&16711680|y&65280|e>>8&255),++d,r+=x,y+=F,e+=C}}},"~N,~N,~N,~N,~N,JU.Rgb16,JU.Rgb16,~N,~N");
c(c$,"plotPixelsUnclippedRasterBits",function(a,b,d,c,f,e,h){if(!(0>=a)){var k=((b<<16)+(d<<1)^858993459)&2147483647;d=d*this.width+b;var l=this.zbuf,j=this.pixel;if(null==c)for(var m=this.argbNoisyDn,r=this.argbNoisyUp,x=this.argbCurrent;0<=--a;)f=this.line3d.getZCurrent(e,h,b++),f<l[d]&&(k=(k<<16)+(k<<1)+k&2147483647,c=k>>16&7,j.addPixel(d,f,0==c?m:1==c?r:x)),++d;else{k=c.r<<8;m=JU.GData.roundInt(w((f.r-c.r<<8)/a));r=c.g;x=JU.GData.roundInt(w((f.g-r)/a));c=c.b;for(var y=JU.GData.roundInt(w((f.b-
c)/a));0<=--a;)f=this.line3d.getZCurrent(e,h,b++),f<l[d]&&j.addPixel(d,f,4278190080|k&16711680|r&65280|c>>8&255),++d,k+=m,r+=x,c+=y}}},"~N,~N,~N,JU.Rgb16,JU.Rgb16,~N,~N");c(c$,"plotPixelsUnclippedCount",function(a,b,d,c,f,e,h,k){for(d=c*e+d;0<=--b;)f<h[d]&&k.addPixel(d,f,a),++d},"~N,~N,~N,~N,~N,~N,~A,J.g3d.Pixelator");c(c$,"plotPoints",function(a,b,d,c){var f=this.pixel,e=this.argbCurrent,h=this.zbuf,k=this.width,l=this.antialiasThisFrame;for(a*=3;0<a;){var j=b[--a],m=b[--a]+c,r=b[--a]+d;if(!this.isClipped3(r,
m,j)){var x=m*k+r++;j<h[x]&&f.addPixel(x,j,e);l&&(x=m*k+r,!this.isClipped3(r,m,j)&&j<h[x]&&f.addPixel(x,j,e),x=++m*k+r,!this.isClipped3(r,m,j)&&j<h[x]&&f.addPixel(x,j,e),x=m*k+--r,!this.isClipped3(r,m,j)&&j<h[x]&&f.addPixel(x,j,e))}}},"~N,~A,~N,~N");c(c$,"setColorNoisy",function(a){this.currentShadeIndex=a;this.argbCurrent=this.shadesCurrent[a];this.argbNoisyUp=this.shadesCurrent[63>a?a+1:63];this.argbNoisyDn=this.shadesCurrent[0<a?a-1:0]},"~N");c(c$,"getShadeIndexP3",function(a,b,d,c){this.vectorAB.sub2(b,
a);this.vectorAC.sub2(d,a);a=this.vectorNormal;a.cross(this.vectorAB,this.vectorAC);return 0>a.z?this.shader.getShadeIndex(a.x,a.y,-a.z):c?-1:this.shader.getShadeIndex(-a.x,-a.y,a.z)},"JU.P3,JU.P3,JU.P3,~B");j(c$,"renderBackground",function(a){null!=this.backgroundImage&&this.plotImage(-2147483648,0,-2147483648,this.backgroundImage,a,0,0,0)},"J.api.JmolRendererInterface");j(c$,"drawAtom",function(a){this.fillSphereXYZ(a.sD,a.sX,a.sY,a.sZ)},"JM.Atom,~N");j(c$,"getExportType",function(){return 0});
j(c$,"getExportName",function(){return null});c(c$,"canDoTriangles",function(){return!0});c(c$,"isCartesianExport",function(){return!1});j(c$,"initializeExporter",function(){return null},"JV.Viewer,~N,JU.GData,java.util.Map");j(c$,"finalizeOutput",function(){return null});j(c$,"drawBond",function(){},"JU.P3,JU.P3,~N,~N,~N,~N,~N");j(c$,"drawEllipse",function(){return!1},"JU.P3,JU.P3,JU.P3,~B,~B");c(c$,"getPrivateKey",function(){return 0});j(c$,"clearFontCache",function(){J.g3d.TextRenderer.clearFontCache()});
c(c$,"setRotationMatrix",function(a){for(var b=JU.Normix.getVertexVectors(),d=JU.GData.normixCount;0<=--d;){var c=this.transformedVectors[d];a.rotate2(b[d],c);this.shadeIndexes[d]=this.shader.getShadeB(c.x,-c.y,c.z);this.shadeIndexes2Sided[d]=0<=c.z?this.shadeIndexes[d]:this.shader.getShadeB(-c.x,c.y,-c.z)}},"JU.M3");j(c$,"renderCrossHairs",function(a,b,d,c,f){b=this.isAntialiased();this.setC(0>f?10:100<f?11:23);f=Math.max(Math.min(this.width,Math.round(c.x)),0);d=Math.max(Math.min(this.height,Math.round(c.y)),
0);var e=Math.round(c.z)+1,h=b?8:4,k=b?20:10;b=b?2:1;this.drawRect(f-h,d,e,0,k,b);this.drawRect(f,d-h,e,0,b,k);this.drawRect(f-h,d-h,e,0,k,k);h=k;k>>=1;this.setC(a[1]<c.x?21:11);this.drawRect(f-h,d,e,0,k,b);this.setC(a[0]>c.x?21:11);this.drawRect(f+k,d,e,0,k,b);this.setC(a[3]<c.y?21:11);this.drawRect(f,d-h,e,0,b,k);this.setC(a[2]>c.y?21:11);this.drawRect(f,d+k,e,0,b,k)},"~A,~N,~N,JU.P3,~N");j(c$,"initializeOutput",function(){return!1},"JV.Viewer,~N,java.util.Map");E(c$,"sort",null,"nullShadeIndex",
50)});s("J.g3d");v(["J.g3d.PrecisionRenderer","java.util.Hashtable"],"J.g3d.LineRenderer",["java.lang.Float","JU.BS"],function(){c$=t(function(){this.lineBits=this.shader=this.g3d=null;this.slope=0;this.lineTypeX=!1;this.nBits=0;this.slopeKey=this.lineCache=null;this.z2t=this.y2t=this.x2t=this.z1t=this.y1t=this.x1t=0;p(this,arguments)},J.g3d,"LineRenderer",J.g3d.PrecisionRenderer);O(c$,function(){this.lineCache=new java.util.Hashtable});q(c$,function(a){I(this,J.g3d.LineRenderer,[]);this.g3d=a;this.shader=
a.shader},"J.g3d.Graphics3D");c(c$,"setLineBits",function(a,b){this.slope=0!=a?b/a:0<=b?3.4028235E38:-3.4028235E38;this.nBits=(this.lineTypeX=1>=this.slope&&-1<=this.slope)?this.g3d.width:this.g3d.height;this.slopeKey=Float.$valueOf(this.slope);if(this.lineCache.containsKey(this.slopeKey))this.lineBits=this.lineCache.get(this.slopeKey);else{this.lineBits=JU.BS.newN(this.nBits);b=Math.abs(b);a=Math.abs(a);if(b>a){var d=a;a=b;b=d}for(var d=0,c=a+a,f=b+b,e=0;e<this.nBits;e++)d+=f,d>a&&(this.lineBits.set(e),
d-=c);this.lineCache.put(this.slopeKey,this.lineBits)}},"~N,~N");c(c$,"clearLineCache",function(){this.lineCache.clear()});c(c$,"plotLineOld",function(a,b,d,c,f,e,h,k){this.x1t=d;this.x2t=e;this.y1t=c;this.y2t=h;this.z1t=f;this.z2t=k;var l=!0;switch(this.getTrimmedLineImpl()){case 0:l=!1;break;case 2:return}this.plotLineClippedOld(a,b,d,c,f,e-d,h-c,k-f,l,0,0)},"~N,~N,~N,~N,~N,~N,~N,~N");c(c$,"plotLineDeltaOld",function(a,b,d,c,f,e,h,k,l){this.x1t=d;this.x2t=d+e;this.y1t=c;this.y2t=c+h;this.z1t=f;
this.z2t=f+k;if(l)switch(this.getTrimmedLineImpl()){case 2:return;case 0:l=!1}this.plotLineClippedOld(a,b,d,c,f,e,h,k,l,0,0)},"~N,~N,~N,~N,~N,~N,~N,~N,~B");c(c$,"plotLineDeltaAOld",function(a,b,d,c,f,e,h,k,l,j,m){this.x1t=f;this.x2t=f+k;this.y1t=e;this.y2t=e+l;this.z1t=h;this.z2t=h+j;if(m)switch(this.getTrimmedLineImpl()){case 2:return;case 0:m=!1}var r=this.g3d.zbuf,x=this.g3d.width,y=0,F=e*x+f,C=this.g3d.bufferSize,n=63>c?c+1:c,p=0<c?c-1:c,q=a[n],s=a[p],t=b[c],n=b[n];b=b[p];a=a[c];c=this.g3d.pixel;
0!=d&&(c=this.g3d.setScreened(1==(d&1)),this.g3d.currentShadeIndex=0);0!=a&&(!m&&0<=F&&F<C&&h<r[F])&&c.addPixel(F,h,a);if(!(0==k&&0==l)){m=1;p=x;f+=k;var u=e+l;0>k&&(k=-k,m=-1);0>l&&(l=-l,p=-x);x=k+k;e=l+l;h<<=10;if(l<=k){var v=k-1;0>j&&(v=-v);j=w(((j<<10)+v)/k);l=0;v=Math.abs(f-this.x2t)-1;f=Math.abs(f-this.x1t)-1;for(var u=k-1,A=w(u/2);--u>=v;){if(u==A){a=t;if(0==a)break;q=n;s=b;0!=d%3&&(c=this.g3d.setScreened(2==(d&2)),this.g3d.currentShadeIndex=0)}F+=m;h+=j;l+=e;l>k&&(F+=p,l-=x);if(0!=a&&u<f&&
0<=F&&F<C&&2147483647>y){var B=h>>10;if(B<r[F]){var D=this.shader.nextRandom8Bit();c.addPixel(F,B,85>D?s:170<D?q:a)}}y=(y+1)%1}}else{v=l-1;0>j&&(v=-v);j=w(((j<<10)+v)/l);k=0;v=Math.abs(u-this.y2t)-1;f=Math.abs(u-this.y1t)-1;u=l-1;for(A=w(u/2);--u>=v;){if(u==A){a=t;if(0==a)break;q=n;s=b;0!=d%3&&(c=this.g3d.setScreened(2==(d&2)),this.g3d.currentShadeIndex=0)}F+=p;h+=j;k+=x;k>l&&(F+=m,k-=e);0!=a&&(u<f&&0<=F&&F<C&&2147483647>y)&&(B=h>>10,B<r[F]&&(D=this.g3d.shader.nextRandom8Bit(),c.addPixel(F,B,85>D?
s:170<D?q:a)));y=(y+1)%1}}}},"~A,~A,~N,~N,~N,~N,~N,~N,~N,~N,~B");c(c$,"plotLineDeltaABitsFloat",function(a,b,d,c,f,e,h){var k=Math.round(c.x),l=Math.round(c.y),j=Math.round(c.z),m=Math.round(f.x),r=Math.round(f.y),x=Math.round(f.z),y=m-k,F=r-l;this.x1t=k;this.x2t=m;this.y1t=l;this.y2t=r;this.z1t=j;this.z2t=x;if(!(h&&2==this.getTrimmedLineImpl())){h=this.g3d.zbuf;var C=this.g3d.width,m=0,x=63>d?d+1:d,r=0<d?d-1:d,n=a[d],p=a[x],q=a[r];d=b[d];a=b[x];b=b[r];var r=l*C+k,x=this.g3d.bufferSize,s,t,u,v;this.lineTypeX?
(s=k,t=this.x1t,u=this.x2t,k+=w(y/2),v=l=0<=y?1:-1,y=0<=F?C:-C,this.setRastABFloat(c.x,c.z,f.x,f.z)):(s=l,t=this.y1t,u=this.y2t,k=l+w(F/2),l=0<=F?1:-1,v=0<=F?C:-C,y=0<=y?1:-1,this.setRastABFloat(c.y,c.z,f.y,f.z));c=j;f=n;j=p;F=q;p=!1;q=this.g3d.pixel;0!=e&&(q=this.g3d.setScreened(1==(e&1)),this.g3d.currentShadeIndex=0);for(n=s;;n+=l,s+=l){n==t&&(p=!0);if(n==k){f=d;if(0==f)break;j=a;F=b;0!=e%3&&(q=this.g3d.setScreened(2==(e&2)),this.g3d.currentShadeIndex=0)}0!=f&&(p&&0<=r&&r<x&&2147483647>m)&&(c=this.getZCurrent(this.a,
this.b,n),c<h[r]&&(C=this.shader.nextRandom8Bit(),q.addPixel(r,A(c),85>C?F:170<C?j:f)));if(n==u)break;m=(m+1)%1;for(r+=v;0>s;)s+=this.nBits;this.lineBits.get(s%this.nBits)&&(r+=y)}}},"~A,~A,~N,JU.P3,JU.P3,~N,~B");c(c$,"plotLineDeltaABitsInt",function(a,b,d,c,f,e,h){var k=c.x,l=c.y,j=c.z,m=f.x,r=f.y,x=f.z,y=m-k,F=r-l;this.x1t=k;this.x2t=m;this.y1t=l;this.y2t=r;this.z1t=j;this.z2t=x;if(!(h&&2==this.getTrimmedLineImpl())){h=this.g3d.zbuf;var C=this.g3d.width,m=0,x=63>d?d+1:d,r=0<d?d-1:d,n=a[d],p=a[x],
q=a[r];d=b[d];a=b[x];b=b[r];var r=l*C+k,x=this.g3d.bufferSize,s,t,u,v;this.lineTypeX?(s=k,t=this.x1t,u=this.x2t,k+=w(y/2),v=l=0<=y?1:-1,y=0<=F?C:-C,this.setRastAB(c.x,c.z,f.x,f.z)):(s=l,t=this.y1t,u=this.y2t,k=l+w(F/2),l=0<=F?1:-1,v=0<=F?C:-C,y=0<=y?1:-1,this.setRastAB(c.y,c.z,f.y,f.z));c=j;f=n;j=p;F=q;p=!1;q=this.g3d.pixel;0!=e&&(q=this.g3d.setScreened(1==(e&1)),this.g3d.currentShadeIndex=0);for(n=s;;n+=l,s+=l){n==t&&(p=!0);if(n==k){f=d;if(0==f)break;j=a;F=b;0!=e%3&&(q=this.g3d.setScreened(2==(e&
2)),this.g3d.currentShadeIndex=0)}0!=f&&(p&&0<=r&&r<x&&2147483647>m)&&(c=this.getZCurrent(this.a,this.b,n),c<h[r]&&(C=this.shader.nextRandom8Bit(),q.addPixel(r,A(c),85>C?F:170<C?j:f)));if(n==u)break;m=(m+1)%1;for(r+=v;0>s;)s+=this.nBits;this.lineBits.get(s%this.nBits)&&(r+=y)}}},"~A,~A,~N,JU.P3i,JU.P3i,~N,~B");c(c$,"plotLineBits",function(a,b,d,c,f,e,h){if(!(1>=d.z||1>=c.z)){var k=!0;this.x1t=d.x;this.y1t=d.y;this.z1t=d.z;this.x2t=c.x;this.y2t=c.y;this.z2t=c.z;switch(this.getTrimmedLineImpl()){case 2:return;
case 0:k=!1;break;default:h&&(d.set(this.x1t,this.y1t,this.z1t),c.set(this.x2t,this.y2t,this.z2t))}h=this.g3d.zbuf;var l=this.g3d.width,j=0;0==f&&(e=2147483647,f=1);var m=d.x,r=d.y,x=d.z,y=c.x-m,F=m+y,C=c.y-r,n=r+C,p=r*l+m,s=this.g3d.bufferSize,q=this.g3d.pixel;0!=a&&(!k&&0<=p&&p<s&&x<h[p])&&q.addPixel(p,x,a);if(!(0==y&&0==C)){var t=k=1,x=l;0>y&&(y=-y,k=-1);0>C&&(C=-C,x=-l,t=-1);var l=y+y,u=C+C;if(C<=y){this.setRastAB(d.x,d.z,c.x,c.z);r=0;d=Math.abs(F-this.x2t)-1;F=Math.abs(F-this.x1t)-1;n=y-1;for(c=
w(n/2);--n>=d&&!(n==c&&(a=b,0==a));){p+=k;m+=k;r+=u;r>y&&(p+=x,r-=l);if(0!=a&&n<F&&0<=p&&p<s&&j<e){var v=this.getZCurrent(this.a,this.b,m);v<h[p]&&q.addPixel(p,v,a)}j=(j+1)%f}}else{this.setRastAB(d.y,d.z,c.y,c.z);m=0;d=Math.abs(n-this.y2t)-1;F=Math.abs(n-this.y1t)-1;n=C-1;for(c=w(n/2);--n>=d&&!(n==c&&(a=b,0==a));)p+=x,r+=t,m+=l,m>C&&(p+=k,m-=u),0!=a&&(n<F&&0<=p&&p<s&&j<e)&&(v=this.getZCurrent(this.a,this.b,r),v<h[p]&&q.addPixel(p,v,a)),j=(j+1)%f}}}},"~N,~N,JU.P3i,JU.P3i,~N,~N,~B");c(c$,"getTrimmedLineImpl",
function(){var a=this.g3d.clipCode3(this.x1t,this.y1t,this.z1t),b=this.g3d.clipCode3(this.x2t,this.y2t,this.z2t);if(0==(a|b))return 0;if(-1==(a|b))return 2;var d=this.g3d.xLast,c=this.g3d.yLast,f=this.g3d.slab,e=this.g3d.depth;do{if(0!=(a&b))return 2;var h=this.x2t-this.x1t,k=this.y2t-this.y1t,l=this.z2t-this.z1t;0!=a?(0!=(a&8)?(this.y1t+=A(-this.x1t*k/h),this.z1t+=A(-this.x1t*l/h),this.x1t=0):0!=(a&4)?(this.y1t+=A((d-this.x1t)*k/h),this.z1t+=A((d-this.x1t)*l/h),this.x1t=d):0!=(a&2)?(this.x1t+=A(-this.y1t*
h/k),this.z1t+=A(-this.y1t*l/k),this.y1t=0):0!=(a&1)?(this.x1t+=A((c-this.y1t)*h/k),this.z1t+=A((c-this.y1t)*l/k),this.y1t=c):0!=(a&32)?(this.x1t+=A((f-this.z1t)*h/l),this.y1t+=A((f-this.z1t)*k/l),this.z1t=f):(this.x1t+=A((e-this.z1t)*h/l),this.y1t+=A((e-this.z1t)*k/l),this.z1t=e),a=this.g3d.clipCode3(this.x1t,this.y1t,this.z1t)):(0!=(b&8)?(this.y2t+=A(-this.x2t*k/h),this.z2t+=A(-this.x2t*l/h),this.x2t=0):0!=(b&4)?(this.y2t+=A((d-this.x2t)*k/h),this.z2t+=A((d-this.x2t)*l/h),this.x2t=d):0!=(b&2)?(this.x2t+=
A(-this.y2t*h/k),this.z2t+=A(-this.y2t*l/k),this.y2t=0):0!=(b&1)?(this.x2t+=A((c-this.y2t)*h/k),this.z2t+=A((c-this.y2t)*l/k),this.y2t=c):0!=(b&32)?(this.x2t+=A((f-this.z2t)*h/l),this.y2t+=A((f-this.z2t)*k/l),this.z2t=f):(this.x2t+=A((e-this.z2t)*h/l),this.y2t+=A((e-this.z2t)*k/l),this.z2t=e),b=this.g3d.clipCode3(this.x2t,this.y2t,this.z2t))}while(0!=(a|b));return 1});c(c$,"plotLineClippedOld",function(a,b,d,c,f,e,h,k,l,j,m){var r=this.g3d.zbuf,x=this.g3d.width,y=0;0==j&&(m=2147483647,j=1);var F=
c*x+d,C=this.g3d.bufferSize,n=this.g3d.pixel;0!=a&&(!l&&0<=F&&F<C&&f<r[F])&&n.addPixel(F,f,a);if(!(0==e&&0==h)){l=1;var p=x;d+=e;var s=c+h;0>e&&(e=-e,l=-1);0>h&&(h=-h,p=-x);x=e+e;c=h+h;f<<=10;if(h<=e){var q=e-1;0>k&&(q=-q);k=w(((k<<10)+q)/e);h=0;q=Math.abs(d-this.x2t)-1;d=Math.abs(d-this.x1t)-1;for(var s=e-1,t=w(s/2);--s>=q&&!(s==t&&(a=b,0==a));){F+=l;f+=k;h+=c;h>e&&(F+=p,h-=x);if(0!=a&&s<d&&0<=F&&F<C&&y<m){var u=f>>10;u<r[F]&&n.addPixel(F,u,a)}y=(y+1)%j}}else{q=h-1;0>k&&(q=-q);k=w(((k<<10)+q)/h);
e=0;q=Math.abs(s-this.y2t)-1;d=Math.abs(s-this.y1t)-1;s=h-1;for(t=w(s/2);--s>=q&&!(s==t&&(a=b,0==a));)F+=p,f+=k,e+=x,e>h&&(F+=l,e-=c),0!=a&&(s<d&&0<=F&&F<C&&y<m)&&(u=f>>10,u<r[F]&&n.addPixel(F,u,a)),y=(y+1)%j}}},"~N,~N,~N,~N,~N,~N,~N,~N,~B,~N,~N");E(c$,"VISIBILITY_UNCLIPPED",0,"VISIBILITY_CLIPPED",1,"VISIBILITY_OFFSCREEN",2)});s("J.g3d");v(null,"J.g3d.Pixelator",["J.g3d.Graphics3D"],function(){c$=t(function(){this.pb=this.zb=this.p0=this.g=null;this.bgcolor=this.width=0;p(this,arguments)},J.g3d,"Pixelator");
q(c$,function(a){this.g=a;this.bgcolor=this.g.bgcolor;this.setBuf()},"J.g3d.Graphics3D");c(c$,"setBuf",function(){this.zb=this.g.zbuf;this.pb=this.g.pbuf});c(c$,"clearPixel",function(a,b){this.zb[a]>b&&(this.zb[a]=2147483647)},"~N,~N");c(c$,"addPixel",function(a,b,d){this.zb[a]=b;this.pb[a]=d},"~N,~N,~N");c(c$,"addImagePixel",function(a,b,d,c,f,e){if(c<this.zb[d])switch(a){case 0:break;case 8:this.addPixel(d,c,f);break;default:a+=b,7>=a&&(b=this.pb[d],0!=e&&(b=J.g3d.Graphics3D.mergeBufferPixel(b,
e,e)),b=J.g3d.Graphics3D.mergeBufferPixel(b,f&16777215|a<<24,this.bgcolor),this.addPixel(d,c,b))}},"~N,~N,~N,~N,~N,~N")});s("J.g3d");v(["J.g3d.Pixelator"],"J.g3d.PixelatorT",["J.g3d.Graphics3D"],function(){c$=D(J.g3d,"PixelatorT",J.g3d.Pixelator);j(c$,"clearPixel",function(){},"~N,~N");j(c$,"addPixel",function(a,b,d){var c=this.g.zbufT[a];if(b<c){var f=this.g.pbufT[a];!this.g.translucentCoverOnly&&(0!=f&&c-b>this.g.zMargin)&&(this.pb[a]=J.g3d.Graphics3D.mergeBufferPixel(this.pb[a],f,this.g.bgcolor));
this.g.zbufT[a]=b;this.g.pbufT[a]=d&this.g.translucencyMask}else b!=c&&!this.g.translucentCoverOnly&&b-c>this.g.zMargin&&(this.pb[a]=J.g3d.Graphics3D.mergeBufferPixel(this.pb[a],d&this.g.translucencyMask,this.g.bgcolor))},"~N,~N,~N")});s("J.g3d");v(["J.g3d.Pixelator"],"J.g3d.PixelatorShaded",null,function(){c$=t(function(){this.tmp=this.bgRGB=null;this.zShadePower=this.zDepth=this.zSlab=0;p(this,arguments)},J.g3d,"PixelatorShaded",J.g3d.Pixelator);q(c$,function(a){I(this,J.g3d.PixelatorShaded,[a]);
this.tmp=B(3,0)},"J.g3d.Graphics3D");c(c$,"set",function(a,b,d){this.bgcolor=this.g.bgcolor;this.bgRGB=B(-1,[this.bgcolor&255,this.bgcolor>>8&255,this.g.bgcolor>>16&255]);this.zSlab=0>a?0:a;this.zDepth=0>b?0:b;this.zShadePower=d;this.p0=this.g.pixel0;return this},"~N,~N,~N");j(c$,"addPixel",function(a,b,d){if(!(b>this.zDepth)){if(b>=this.zSlab&&0<this.zShadePower){var c=this.tmp,f=this.bgRGB;c[0]=d;c[1]=d>>8;c[2]=d>>16;var e=(this.zDepth-b)/(this.zDepth-this.zSlab);if(1<this.zShadePower)for(var h=
0;h<this.zShadePower;h++)e*=e;for(h=0;3>h;h++)c[h]=f[h]+A(e*((c[h]&255)-f[h]));d=c[2]<<16|c[1]<<8|c[0]|d&4278190080}this.p0.addPixel(a,b,d)}},"~N,~N,~N");c(c$,"showZBuffer",function(){for(var a=this.p0.zb.length;0<=--a;)if(0!=this.p0.pb[a]){var b=A(Math.min(255,Math.max(0,255*(this.zDepth-this.p0.zb[a])/(this.zDepth-this.zSlab))));this.p0.pb[a]=4278190080|b|b<<8|b<<16}})});s("J.g3d");v(["J.g3d.Pixelator"],"J.g3d.PixelatorScreened",null,function(){c$=D(J.g3d,"PixelatorScreened",J.g3d.Pixelator);q(c$,
function(a,b){I(this,J.g3d.PixelatorScreened,[a]);this.width=a.width;this.p0=b},"J.g3d.Graphics3D,J.g3d.Pixelator");j(c$,"addPixel",function(a,b,d){a%this.width%2==w(a/this.width)%2&&this.p0.addPixel(a,b,d)},"~N,~N,~N")});s("J.g3d");c$=t(function(){this.bufferSizeT=this.bufferSize=this.bufferHeight=this.bufferWidth=this.windowSize=this.windowHeight=this.windowWidth=0;this.zBufferT=this.zBuffer=this.pBufferT=this.pBuffer=this.bufferedImage=null;this.heightOffscreen=this.widthOffscreen=0;this.apiPlatform=
this.graphicsForTextOrImage=this.offscreenImage=null;p(this,arguments)},J.g3d,"Platform3D");q(c$,function(a){this.apiPlatform=a},"J.api.GenericPlatform");c(c$,"getGraphicsForMetrics",function(){return this.apiPlatform.getGraphics(this.allocateOffscreenImage(1,1))});c(c$,"allocateTBuffers",function(a){this.bufferSizeT=a?this.bufferSize:this.windowSize;this.zBufferT=B(this.bufferSizeT,0);this.pBufferT=B(this.bufferSizeT,0)},"~B");c(c$,"allocateBuffers",function(a,b,d,c){this.windowWidth=a;this.windowHeight=
b;this.windowSize=a*b;d&&(a*=2,b*=2);this.bufferWidth=a;this.bufferHeight=b;this.bufferSize=this.bufferWidth*this.bufferHeight;this.zBuffer=B(this.bufferSize,0);this.pBuffer=B(this.bufferSize,0);this.bufferedImage=this.apiPlatform.allocateRgbImage(this.windowWidth,this.windowHeight,this.pBuffer,this.windowSize,J.g3d.Platform3D.backgroundTransparent,c)},"~N,~N,~B,~B");c(c$,"releaseBuffers",function(){this.windowWidth=this.windowHeight=this.bufferWidth=this.bufferHeight=this.bufferSize=-1;null!=this.bufferedImage&&
(this.apiPlatform.flushImage(this.bufferedImage),this.bufferedImage=null);this.zBufferT=this.pBufferT=this.zBuffer=this.pBuffer=null});c(c$,"hasContent",function(){for(var a=this.bufferSize;0<=--a;)if(2147483647!=this.zBuffer[a])return!0;return!1});c(c$,"clearScreenBuffer",function(){for(var a=this.bufferSize;0<=--a;)this.zBuffer[a]=2147483647,this.pBuffer[a]=0});c(c$,"setBackgroundColor",function(a){if(null!=this.pBuffer)for(var b=this.bufferSize;0<=--b;)0==this.pBuffer[b]&&(this.pBuffer[b]=a)},
"~N");c(c$,"clearTBuffer",function(){for(var a=this.bufferSizeT;0<=--a;)this.zBufferT[a]=2147483647,this.pBufferT[a]=0});c(c$,"clearBuffer",function(){this.clearScreenBuffer()});c(c$,"clearScreenBufferThreaded",function(){});c(c$,"notifyEndOfRendering",function(){this.apiPlatform.notifyEndOfRendering()});c(c$,"getGraphicsForTextOrImage",function(a,b){if(a>this.widthOffscreen||b>this.heightOffscreen)null!=this.offscreenImage&&(this.apiPlatform.disposeGraphics(this.graphicsForTextOrImage),this.apiPlatform.flushImage(this.offscreenImage)),
a>this.widthOffscreen&&(this.widthOffscreen=a),b>this.heightOffscreen&&(this.heightOffscreen=b),this.offscreenImage=this.allocateOffscreenImage(this.widthOffscreen,this.heightOffscreen),this.graphicsForTextOrImage=this.apiPlatform.getStaticGraphics(this.offscreenImage,J.g3d.Platform3D.backgroundTransparent);return this.graphicsForTextOrImage},"~N,~N");c(c$,"allocateOffscreenImage",function(a,b){return this.apiPlatform.newOffScreenImage(a,b)},"~N,~N");c(c$,"setBackgroundTransparent",function(a){J.g3d.Platform3D.backgroundTransparent=
a},"~B");E(c$,"backgroundTransparent",!1);s("J.g3d");c$=t(function(){this.b=this.a=0;this.isOrthographic=!1;p(this,arguments)},J.g3d,"PrecisionRenderer");c(c$,"getZCurrent",function(a,b,d){return Math.round(1.4E-45==a?b:this.isOrthographic?a*d+b:a/(b-d))},"~N,~N,~N");c(c$,"setRastABFloat",function(a,b,d,c){var f=c-b,e=d-a;0==f||0==e?(this.a=1.4E-45,this.b=b):this.isOrthographic?(this.a=f/e,this.b=b-this.a*a):(this.a=e*b*(c/f),this.b=(d*c-a*b)/f)},"~N,~N,~N,~N");c(c$,"setRastAB",function(a,b,d,c){var f=
c-b,e=d-a;1.4E-45==a||0==f||0==e?(this.a=1.4E-45,this.b=c):this.isOrthographic?(this.a=f/e,this.b=b-this.a*a):(this.a=e*b*(c/f),this.b=(d*c-a*b)/f)},"~N,~N,~N,~N");s("J.g3d");v(["JU.P3"],"J.g3d.SphereRenderer",null,function(){c$=t(function(){this.mDeriv=this.coef=this.mat=this.zroot=this.shader=this.g3d=null;this.planeShade=this.selectedOctant=0;this.zbuf=null;this.offsetPbufBeginLine=this.slab=this.depth=this.height=this.width=0;this.dxyz=this.planeShades=this.ptTemp=null;p(this,arguments)},J.g3d,
"SphereRenderer");O(c$,function(){this.zroot=X(2,0);this.ptTemp=new JU.P3;this.planeShades=B(3,0);this.dxyz=K(3,3,0)});q(c$,function(a){this.g3d=a;this.shader=a.shader},"J.g3d.Graphics3D");c(c$,"render",function(a,b,d,c,f,e,h,k,l,j){if(1!=f&&(49<b&&(b&=-2),!this.g3d.isClippedXY(b,d,c))){this.slab=this.g3d.slab;this.depth=this.g3d.depth;var m=b+1>>1,r=f-m;if(!(f+m<this.slab||r>this.depth)){var x=d-m,y=d+m,F=c-m,C=c+m;this.shader.nOut=this.shader.nIn=0;this.zbuf=this.g3d.zbuf;this.height=this.g3d.height;
this.width=this.g3d.width;this.offsetPbufBeginLine=this.width*c+d;var n=this.shader;this.mat=e;if(null!=e&&(this.coef=h,this.mDeriv=k,this.selectedOctant=l,null==n.ellipsoidShades&&n.createEllipsoidShades(),null!=j)){this.planeShade=-1;for(h=0;3>h;h++)if(m=this.dxyz[h][0]=j[h].x-d,k=this.dxyz[h][1]=j[h].y-c,l=this.dxyz[h][2]=j[h].z-f,this.planeShades[h]=n.getShadeIndex(m,k,-l),0==m&&0==k){this.planeShade=this.planeShades[h];break}}if(null!=e||128<b)this.renderQuadrant(-1,-1,d,c,f,b,a),this.renderQuadrant(-1,
1,d,c,f,b,a),this.renderQuadrant(1,-1,d,c,f,b,a),this.renderQuadrant(1,1,d,c,f,b,a),null!=e&&(this.mDeriv=this.coef=this.mat=null);else{var p=n.sphereShapeCache[b-1];if(null==p){p=0;e=0!=(b&1);j=b/2;k=j*j;m=w((b+1)/2);l=e?0:0.5;for(h=0;h<m;++h,++l)for(var s=l*l,q=e?0:0.5,t=0;t<m;++t,++q){var u=q*q,u=k-s-u;0<=u&&++p}var p=B(p,0),v=0;l=e?0:0.5;for(h=0;h<m;++h,++l){s=l*l;q=e?0:0.5;for(t=0;t<m;++t,++q)if(u=q*q,u=k-s-u,0<=u){var D=Math.sqrt(u),u=A(D),G=n.getShadeN(q,l,D,j),E=n.getShadeN(-q,l,D,j),H=n.getShadeN(q,
-l,D,j),D=n.getShadeN(-q,-l,D,j),u=u|G<<7|E<<13|H<<19|D<<25;p[v++]=u}p[v-1]|=2147483648}n.sphereShapeCache[b-1]=p}0>x||y>=this.width||0>F||C>=this.height||r<this.slab||f>this.depth?this.renderSphereClipped(p,d,c,f,b,a):this.renderSphereUnclipped(p,f,b,a)}this.zbuf=null}}},"~A,~N,~N,~N,~N,JU.M3,~A,JU.M4,~N,~A");c(c$,"renderSphereUnclipped",function(a,b,d,c){var f=0,e=1-(d&1),h=this.offsetPbufBeginLine,k=h-e*this.width;d=w((d+1)/2);var l=this.zbuf,j=this.width,m=this.g3d.pixel;do{var r=h,x=h-e,y=k,
n=k-e,C;do{C=a[f++];var p=b-(C&127);p<l[r]&&m.addPixel(r,p,c[C>>7&63]);p<l[x]&&m.addPixel(x,p,c[C>>13&63]);p<l[y]&&m.addPixel(y,p,c[C>>19&63]);p<l[n]&&m.addPixel(n,p,c[C>>25&63]);++r;--x;++y;--n}while(0<=C);h+=j;k-=j}while(0<--d)},"~A,~N,~N,~A");c(c$,"renderSphereClipped",function(a,b,d,c,f,e){var h=this.width,k=this.height,l=0,j=1-(f&1),m=this.offsetPbufBeginLine,r=m-j*h;f=w((f+1)/2);var x=d,y=d-j;d=(b<<16)+(d<<1)^858993459;var n=this.zbuf,C=this.g3d.pixel,p=this.slab,q=this.depth;do{var s=0<=x&&
x<k,t=0<=y&&y<k,u=m,v=m-j,A=r,B=r-j,D,G=b,E=b-j;do{var H=0<=E&&E<h,I=0<=G&&G<h;D=a[l++];var K;K=D&127;var L;c<p?(L=c+K,K=L>=p):(L=c-K,K=L<p);K&&(L=p);if(L>=p&&L<=q){if(s){if(I&&L<n[u]){var M=K?44+(d>>7&7):D>>7&63;C.addPixel(u,L,e[M])}H&&L<n[v]&&(M=K?44+(d>>13&7):D>>13&63,C.addPixel(v,L,e[M]))}t&&(I&&L<n[A]&&(M=K?44+(d>>19&7):D>>19&63,C.addPixel(A,L,e[M])),H&&L<n[B]&&(M=K?44+(d>>25&7):D>>25&63,C.addPixel(B,L,e[M])))}++u;--v;++A;--B;++G;--E;K&&(d=(d<<16)+(d<<1)+d&2147483647)}while(0<=D);m+=h;r-=h;++x;
--y}while(0<--f)},"~A,~N,~N,~N,~N,~A");c(c$,"renderQuadrant",function(a,b,d,c,f,e,h){e=w(e/2);var k=d+e*a,l=(0>d?-1:d<this.width?0:1)+(0>k?-2:k<this.width?0:2);-3==l||3==l||(k=c+e*b,k=(0>c?-1:c<this.height?0:1)+(0>k?-2:k<this.height?0:2),-3==k||3==k||(null==this.mat&&0==l&&0==k&&f-e>=this.slab&&f<=this.depth?this.renderQuadrantUnclipped(e,a,b,f,h):this.renderQuadrantClipped(e,a,b,d,c,f,h)))},"~N,~N,~N,~N,~N,~N,~A");c(c$,"renderQuadrantUnclipped",function(a,b,d,c,f){for(var e=a*a,h=2*a+1,k=0>d?-this.width:
this.width,l=this.offsetPbufBeginLine,j=this.zbuf,m=this.g3d.pixel,r=this.shader.sphereShadeIndexes,x=0,y=0;y<=e;y+=x+ ++x,l+=k)for(var n=l,C=e-y,p=c-a,q=w((x*d+a<<8)/h),s=0,u=0;u<=C;u+=s+ ++s,n+=b)if(!(j[n]<=p)&&(p=w(Math.sqrt(C-u)),p=c-p,!(j[n]<=p))){var t=w((s*b+a<<8)/h);m.addPixel(n,p,f[r[(q<<8)+t]])}},"~N,~N,~N,~N,~A");c(c$,"renderQuadrantClipped",function(a,b,d,c,f,e,h){for(var k=null!=this.mat,l=0<=this.selectedOctant,j=a*a,m=2*a+1,r=0>d?-this.width:this.width,x=this.offsetPbufBeginLine,y=
(c<<16)+(f<<1)^858993459,n=0,C=0,p=this.g3d.pixel,q=0,s=this.height,u=this.width,t=this.zbuf,v=this.dxyz,A=this.slab,B=this.depth,D=this.ptTemp,G=this.coef,E=this.zroot,H=this.selectedOctant,K=this.shader,I=this.planeShades,L=K.sphereShadeIndexes,M=this.planeShade,N=this.mat,O=0,Q=0,P=f;Q<=j;Q+=O+ ++O,x+=r,P+=d)if(0>P){if(0>d)break}else if(P>=s){if(0<d)break}else{var T=j-(k?0:Q);k||(n=w((O*d+a<<8)/m));for(var y=(y<<16)+(y<<1)+y&2147483647,R=c,W=0,V=0,X=-1,Z=1,Y=x;V<=T;V+=W+ ++W,Y+=b,R+=b)if(0>R){if(0>
b)break}else if(R>=u){if(0<b)break}else{var S;if(k){S=(G[4]*R+G[5]*P+G[8])/G[2]/2;var U=S*S-(G[0]*R*R+G[1]*P*P+G[3]*R*P+G[6]*R+G[7]*P-1)/G[2];if(0>U){if(0<=X)break;continue}U=Math.sqrt(U);E[0]=-S-U;E[1]=-S+U;X=e<A?1:0;if(0==(S=w(E[X])))S=e;Z=2;q=S;if(l){D.set(R-c,P-f,S-e);N.rotate(D);U=0;0>D.x&&(U|=1);0>D.y&&(U|=2);0>D.z&&(U|=4);if(U==H){if(0<=M)C=M;else{Z=3;C=3.4028235E38;for(S=0;3>S;S++)if(0!=(U=v[S][2]))U=e+(-v[S][0]*(R-c)-v[S][1]*(P-f))/U,U<C&&(C=U,Z=S);3==Z&&(Z=0,C=e);E[0]=C;C=I[Z]}S=w(E[0]);
Z=3}if(U=e<A?S>=A:S<A)q=S=A,Z=0}if(S<A||S>B||t[Y]<=q)continue}else{S=w(Math.sqrt(T-V));S=e+(e<A?S:-S);if(U=e<A?S>=A:S<A)S=A,Z=0;if(S<A||S>B||t[Y]<=S)continue}switch(Z){case 0:C=44+(y>>8&7);y=(y<<16)+(y<<1)+y&2147483647;Z=1;break;case 2:C=K.getEllipsoidShade(R,P,E[X],a,this.mDeriv);break;case 3:p.clearPixel(Y,q);break;default:C=w((W*b+a<<8)/m),C=L[(n<<8)+C]}p.addPixel(Y,S,h[C])}y=(y+R+P|1)&2147483647}},"~N,~N,~N,~N,~N,~N,~A");E(c$,"maxOddSizeSphere",49,"maxSphereDiameter",1E3,"maxSphereDiameter2",
2E3,"SHADE_SLAB_CLIPPED",47)});s("J.g3d");v(["java.util.Hashtable"],"J.g3d.TextRenderer",["JU.CU"],function(){c$=t(function(){this.size=this.mapWidth=this.width=this.ascent=this.height=0;this.tmap=null;this.isInvalid=!1;p(this,arguments)},J.g3d,"TextRenderer");c$.clearFontCache=c(c$,"clearFontCache",function(){J.g3d.TextRenderer.working||(J.g3d.TextRenderer.htFont3d.clear(),J.g3d.TextRenderer.htFont3dAntialias.clear())});c$.plot=c(c$,"plot",function(a,b,d,c,f,e,h,k,l,j){if(0==e.length)return 0;if(0<=
e.indexOf("<su")||0<=e.indexOf("<color"))return J.g3d.TextRenderer.plotByCharacter(a,b,d,c,f,e,h,k,l,j);var m=h.getAscent();b-=m;e=J.g3d.TextRenderer.getPlotText3D(a,b,k,e,h,j);if(e.isInvalid)return e.width;j&&0==(c&12632256)&&(c|=263172);j=e.height;h=e.width;var m=e.tmap,r=k.width,x=k.height,y=k.zbuf,n=k.pixel,C=k.translucencyLog;if(null!=l||(0>a||a+e.width>r||0>b||b+e.height>x)&&null!=(l=k))for(var p=k=0;p<j;p++)for(var q=0;q<h;q++){var s=m[k++];0!=s&&l.plotImagePixel(c,a+q,b+p,d,s,f,r,x,y,n,C)}else{k=
p=0;for(a=b*r+a;p<j;p++,a+=r-h)for(q=0;q<h;q++)n.addImagePixel(m[k++],C,a++,d,c,f)}return e.width},"~N,~N,~N,~N,~N,~S,JU.Font,J.g3d.Graphics3D,J.api.JmolRendererInterface,~B");c$.plotByCharacter=c(c$,"plotByCharacter",function(a,b,d,c,f,e,h,k,l,j){for(var m=0,r=e.length,x=Math.round(0.25*h.getHeight()),y=-Math.round(0.3*h.getHeight()),n=0,C=0;C<r;C++){if("<"==e.charAt(C)){if(C+5<r&&e.substring(C,C+6).equals("<color")){var n=c,p=e.indexOf(">",C);if(0>p)continue;c=JU.CU.getArgbFromString(e.substring(C+
7,p).trim());C=p;continue}if(C+7<r&&e.substring(C,C+8).equals("</color>")){C+=7;c=n;continue}if(C+4<r&&e.substring(C,C+5).equals("<sub>")){C+=4;b+=x;continue}if(C+4<r&&e.substring(C,C+5).equals("<sup>")){C+=4;b+=y;continue}if(C+5<r&&e.substring(C,C+6).equals("</sub>")){C+=5;b-=x;continue}if(C+5<r&&e.substring(C,C+6).equals("</sup>")){C+=5;b-=y;continue}}p=J.g3d.TextRenderer.plot(a+m,b,d,c,f,e.substring(C,C+1),h,k,l,j);m+=p}return m},"~N,~N,~N,~N,~N,~S,JU.Font,J.g3d.Graphics3D,J.api.JmolRendererInterface,~B");
q(c$,function(a,b){this.ascent=b.getAscent();this.height=b.getHeight();this.width=b.stringWidth(a);0!=this.width&&(this.mapWidth=this.width,this.size=this.mapWidth*this.height)},"~S,JU.Font");c$.getPlotText3D=c(c$,"getPlotText3D",function(a,b,d,c,f,e){J.g3d.TextRenderer.working=!0;e=e?J.g3d.TextRenderer.htFont3dAntialias:J.g3d.TextRenderer.htFont3d;var h=e.get(f),k=null,l=!1,j=!1;null!=h?k=h.get(c):(h=new java.util.Hashtable,l=!0);null==k&&(k=new J.g3d.TextRenderer(c,f),j=!0);k.isInvalid=0==k.width||
0>=a+k.width||a>=d.width||0>=b+k.height||b>=d.height;if(k.isInvalid)return k;l&&e.put(f,h);j&&(k.setTranslucency(c,f,d),h.put(c,k));J.g3d.TextRenderer.working=!1;return k},"~N,~N,J.g3d.Graphics3D,~S,JU.Font,~B");c(c$,"setTranslucency",function(a,b,d){a=d.apiPlatform.getTextPixels(a,b,d.platform.getGraphicsForTextOrImage(this.mapWidth,this.height),d.platform.offscreenImage,this.mapWidth,this.height,this.ascent);if(null!=a){this.tmap=Q(this.size,0);for(b=a.length;0<=--b;)d=a[b]&255,0!=d&&(this.tmap[b]=
J.g3d.TextRenderer.translucency[d>>5])}},"~S,JU.Font,J.g3d.Graphics3D");E(c$,"translucency",Q(-1,[7,6,5,4,3,2,1,8]),"working",!1);c$.htFont3d=c$.prototype.htFont3d=new java.util.Hashtable;c$.htFont3dAntialias=c$.prototype.htFont3dAntialias=new java.util.Hashtable});s("J.g3d");v(["JU.P3i"],"J.g3d.TextString",null,function(){c$=t(function(){this.font=this.text=null;this.bgargb=this.argb=0;p(this,arguments)},J.g3d,"TextString",JU.P3i,java.util.Comparator);c(c$,"setText",function(a,b,d,c,f,e,h){this.text=
a;this.font=b;this.argb=d;this.bgargb=c;this.x=f;this.y=e;this.z=h},"~S,JU.Font,~N,~N,~N,~N,~N");j(c$,"compare",function(a,b){return null==a||null==b?0:a.z>b.z?-1:a.z<b.z?1:0},"J.g3d.TextString,J.g3d.TextString");c(c$,"toString",function(){return W(this,J.g3d.TextString,"toString",[])+" "+this.text})});s("J.g3d");v(["J.g3d.G3DRenderer","$.PrecisionRenderer","JU.Rgb16"],"J.g3d.TriangleRenderer",["JU.GData"],function(){c$=t(function(){this.rgb16t2=this.rgb16t1=this.rgb16sGouraud=this.rgb16sE=this.rgb16sW=
this.azE=this.axE=this.azW=this.axW=this.bb=this.aa=this.az=this.ay=this.ax=this.g3d=null;p(this,arguments)},J.g3d,"TriangleRenderer",J.g3d.PrecisionRenderer,J.g3d.G3DRenderer);O(c$,function(){this.ax=B(3,0);this.ay=B(3,0);this.az=B(3,0);this.aa=K(64,0);this.bb=K(64,0);this.axW=B(64,0);this.azW=B(64,0);this.axE=B(64,0);this.azE=B(64,0);this.rgb16t1=new JU.Rgb16;this.rgb16t2=new JU.Rgb16});q(c$,function(){I(this,J.g3d.TriangleRenderer,[])});j(c$,"set",function(a){try{this.rgb16sW=Array(64);this.rgb16sE=
Array(64);for(var b=64;0<=--b;)this.rgb16sW[b]=new JU.Rgb16,this.rgb16sE[b]=new JU.Rgb16;this.g3d=a;this.rgb16sGouraud=Array(3);for(b=3;0<=--b;)this.rgb16sGouraud[b]=new JU.Rgb16}catch(d){if(!H(d,Exception))throw d;}return this},"J.api.JmolRendererInterface,JU.GData");c(c$,"reallocRgb16s",function(a,b){var d=Array(b);Zystem.arraycopy(a,0,d,0,a.length);for(var c=a.length;c<b;++c)d[c]=new JU.Rgb16;return d},"~A,~N");c(c$,"setGouraud",function(a,b,d){this.rgb16sGouraud[0].setInt(a);this.rgb16sGouraud[1].setInt(b);
this.rgb16sGouraud[2].setInt(d)},"~N,~N,~N");c(c$,"fillTriangle",function(a,b,d,c){this.ax[0]=a.x;this.ax[1]=b.x;this.ax[2]=d.x;this.ay[0]=a.y;this.ay[1]=b.y;this.ay[2]=d.y;this.az[0]=a.z;this.az[1]=b.z;this.az[2]=d.z;if(!(1>=this.az[0]||1>=this.az[1]||1>=this.az[2])){b=this.g3d.clipCode3(this.ax[0],this.ay[0],this.az[0]);d=this.g3d.clipCode3(this.ax[1],this.ay[1],this.az[1]);var f=this.g3d.clipCode3(this.ax[2],this.ay[2],this.az[2]),e=b|d|f;a=0!=e;if(!a||!(-1==e||0!=(b&d&f))){f=0;this.ay[1]<this.ay[f]&&
(f=1);this.ay[2]<this.ay[f]&&(f=2);var e=(f+1)%3,h=(f+2)%3;if(this.ay[e]>this.ay[h])var k=e,e=h,h=k;b=this.ay[f];var l=this.ay[e],j=this.ay[h];d=j-b+1;if(!(d>3*this.g3d.height)){if(d>this.axW.length){var m=d+31&-32;this.axW=B(m,0);this.azW=B(m,0);this.axE=B(m,0);this.azE=B(m,0);this.aa=K(m,0);this.bb=K(m,0);this.rgb16sW=this.reallocRgb16s(this.rgb16sW,m);this.rgb16sE=this.reallocRgb16s(this.rgb16sE,m)}var r;c?(m=this.rgb16sW,r=this.rgb16sE):m=r=null;k=l-b;0==k?(this.ax[e]<this.ax[f]&&(k=e,e=f,f=k),
this.generateRaster(d,f,h,this.axW,this.azW,0,m),this.generateRaster(d,e,h,this.axE,this.azE,0,r)):l==j?(this.ax[h]<this.ax[e]&&(k=e,e=h,h=k),this.generateRaster(d,f,e,this.axW,this.azW,0,m),this.generateRaster(d,f,h,this.axE,this.azE,0,r)):(l=this.ax[h]-this.ax[f],j=JU.GData.roundInt(w(d/2)),0>l&&(j=-j),this.ax[f]+w((l*k+j)/d)<this.ax[e]?(this.generateRaster(d,f,h,this.axW,this.azW,0,m),this.generateRaster(k+1,f,e,this.axE,this.azE,0,r),this.generateRaster(d-k,e,h,this.axE,this.azE,k,r)):(this.generateRaster(k+
1,f,e,this.axW,this.azW,0,m),this.generateRaster(d-k,e,h,this.axW,this.azW,k,m),this.generateRaster(d,f,h,this.axE,this.azE,0,r)));this.g3d.setZMargin(5);f=this.g3d.pass2Flag01;e=1-f;h=0;0>b&&(d+=b,h-=b,b=0);b+d>this.g3d.height&&(d=this.g3d.height-b);if(c)if(a)for(;--d>=f;++b,++h)a=this.axE[h]-(c=this.axW[h])+e,0<a&&this.g3d.plotPixelsClippedRaster(a,c,b,this.azW[h],this.azE[h],this.rgb16sW[h],this.rgb16sE[h]);else for(;--d>=f;++b,++h)a=this.axE[h]-(c=this.axW[h])+e,1==f&&0>a&&(a=1,c--),0<a&&this.g3d.plotPixelsUnclippedRaster(a,
c,b,this.azW[h],this.azE[h],this.rgb16sW[h],this.rgb16sE[h]);else if(a)for(;--d>=f;++b,++h)a=this.axE[h]-(c=this.axW[h])+e,0<a&&this.g3d.plotPixelsClippedRasterBits(a,c,b,this.azW[h],this.azE[h],null,null,this.aa[h],this.bb[h]);else for(;--d>=f;++b,++h)a=this.axE[h]-(c=this.axW[h])+e,1==f&&0>a&&(a=1,c--),0<a&&this.g3d.plotPixelsUnclippedRasterBits(a,c,b,null,null,this.aa[h],this.bb[h]);this.g3d.setZMargin(0)}}}},"JU.P3i,JU.P3i,JU.P3i,~B");c(c$,"generateRaster",function(a,b,d,c,f,e,h){var k=this.ax[b],
l=this.ax[d]-k,j,m,r;0<=l?(j=1,m=l,r=0):(j=-1,m=-l,r=1-a);var x;m<=a?x=0:(x=JU.GData.roundInt(w(l/a)),m%=a);this.setRastAB(this.ay[b],this.az[b],this.ay[d],this.az[d]);for(var y=this.a,n=this.b,C=c===this.axE,p=0,q=this.ay[b],s=a-1,l=e;p<=s;++l,++p,++q){if(0==l||l>e)c[l]=p==s?this.ax[d]:k,f[l]=this.getZCurrent(y,n,q),C&&(this.setRastAB(this.axW[l],this.azW[l],c[l],f[l]),this.aa[l]=this.a,this.bb[l]=this.b);k+=x;r+=m;0<r&&(k+=j,r-=a)}if(null!=h){c=this.rgb16t1;c.setRgb(this.rgb16sGouraud[b]);b=this.rgb16t2;
b.diffDiv(this.rgb16sGouraud[d],c,a);l=e;for(a=e+a;l<a;++l)h[l].setAndIncrement(c,b)}},"~N,~N,~N,~A,~A,~N,~A");E(c$,"DEFAULT",64)});s("J.i18n");v("J.api.Translator java.text.MessageFormat java.util.Hashtable JU.PT J.i18n.Language $.Resource".split(" "),"J.i18n.GT",["JU.Logger"],function(){c$=t(function(){this.resources=null;this.resourceCount=0;this.doTranslate=!0;this.language=null;p(this,arguments)},J.i18n,"GT",null,J.api.Translator);q(c$,function(){});j(c$,"translate",function(a){return J.i18n.GT.$(a)},
"~S");q(c$,function(a,b){this.resources=null;this.resourceCount=0;J.i18n.GT.$getTextWrapper=this;null!=b&&0==b.length&&(b="none");null!=b&&(this.language=b);"none".equals(this.language)&&(this.language=null);null==this.language&&(this.language=J.i18n.Resource.getLanguage());null==this.language&&(this.language="en");var d=this.language,c=null,f=null,e=this.language.indexOf("_");if(0<=e&&(d=d.substring(0,e),c=this.language,0<=(e=c.indexOf("_",++e))))c=c.substring(0,e),f=this.language;if(null==(this.language=
this.getSupported(f))&&null==(this.language=this.getSupported(c))&&null==(this.language=this.getSupported(d)))this.language="en",Zystem.out.println(this.language+" not supported -- using en");else{c=f=null;switch(this.language.length){default:f=this.language;c=this.language.substring(0,5);d=this.language.substring(0,2);break;case 5:c=this.language;d=this.language.substring(0,2);break;case 2:d=this.language}c=this.getSupported(c);d=this.getSupported(d);if(d===c||"en_US".equals(d))d=null;c===f&&(c=
null);"en_US".equals(c)||(J.i18n.GT.allowDebug&&JU.Logger.debugging&&JU.Logger.debug("Instantiating gettext wrapper for "+this.language+" using files for language:"+d+" country:"+c+" variant:"+f),J.i18n.GT.$ignoreApplicationBundle||this.addBundles(a,"Jmol",f,null,null),this.addBundles(a,"JmolApplet",f,null,null),J.i18n.GT.$ignoreApplicationBundle||this.addBundles(a,"Jmol",null,c,null),this.addBundles(a,"JmolApplet",null,c,null),J.i18n.GT.$ignoreApplicationBundle||this.addBundles(a,"Jmol",null,null,
d),this.addBundles(a,"JmolApplet",null,null,d))}},"JV.Viewer,~S");c$.getLanguageList=c(c$,"getLanguageList",function(a){null==J.i18n.GT.languageList&&(null==a&&(a=J.i18n.GT.getTextWrapper()),a.createLanguageList());return J.i18n.GT.languageList},"J.i18n.GT");c$.getLanguage=c(c$,"getLanguage",function(){return J.i18n.GT.getTextWrapper().language});c$.ignoreApplicationBundle=c(c$,"ignoreApplicationBundle",function(){J.i18n.GT.$ignoreApplicationBundle=!0});c$.setDoTranslate=c(c$,"setDoTranslate",function(a){var b=
J.i18n.GT.getDoTranslate();J.i18n.GT.getTextWrapper().doTranslate=a;return b},"~B");c$.getDoTranslate=c(c$,"getDoTranslate",function(){return J.i18n.GT.getTextWrapper().doTranslate});c$.$=c(c$,"$",function(a){return J.i18n.GT.getTextWrapper().getString(a)},"~S");c$.o=c(c$,"o",function(a,b){if(n(b,Array)){if(1!=b.length)return java.text.MessageFormat.format(a,b);b=b[0]}return JU.PT.rep(a,"{0}",b.toString())},"~S,~O");c$.i=c(c$,"i",function(a,b){return JU.PT.rep(a,"{0}",""+b)},"~S,~N");c$.escapeHTML=
c(c$,"escapeHTML",function(a){for(var b,d=a.length;0<=--d;)if(127<(b=a.charAt(d)).charCodeAt(0))a=a.substring(0,d)+"&#"+b.charCodeAt(0)+";"+a.substring(d+1);return a},"~S");c$.getTextWrapper=c(c$,"getTextWrapper",function(){return null==J.i18n.GT.$getTextWrapper?J.i18n.GT.$getTextWrapper=new J.i18n.GT(null,null):J.i18n.GT.$getTextWrapper});c(c$,"createLanguageList",function(){var a=this.doTranslate;this.doTranslate=!1;J.i18n.GT.languageList=J.i18n.Language.getLanguageList();this.doTranslate=a});c(c$,
"getSupported",function(a){if(null==a)return null;var b=J.i18n.GT.htLanguages.get(a);if(null!=b)return 0==b.length?null:b;b=J.i18n.Language.getSupported(J.i18n.GT.getLanguageList(this),a);J.i18n.GT.htLanguages.put(a,null==b?"":b);return b},"~S");c(c$,"addBundles",function(a,b,d,c,f){try{b="J.translation."+b+".",null!=d&&this.addBundle(a,b,d),null!=c&&this.addBundle(a,b,c),null!=f&&this.addBundle(a,b,f)}catch(e){if(H(e,Exception))J.i18n.GT.allowDebug&&JU.Logger.errorEx("Some exception occurred!",e),
this.resources=null,this.resourceCount=0;else throw e;}},"JV.Viewer,~S,~S,~S,~S");c(c$,"addBundle",function(a,b,d){a=J.i18n.Resource.getResource(a,b,d);null!=a&&(null==this.resources&&(this.resources=Array(8),this.resourceCount=0),this.resources[this.resourceCount]=a,this.resourceCount++,J.i18n.GT.allowDebug&&JU.Logger.debug("GT adding "+b))},"JV.Viewer,~S,~S");c(c$,"getString",function(a){var b=null;if(this.doTranslate){for(var d=0;d<this.resourceCount;d++)if(b=this.resources[d].getString(a),null!=
b){a=b;break}0<this.resourceCount&&(null==b&&J.i18n.GT.allowDebug&&JU.Logger.debugging)&&JU.Logger.debug("No trans, using default: "+a)}null==b&&(a.startsWith("[")?a=a.substring(a.indexOf("]")+1):a.endsWith("]")&&(a=a.substring(0,a.indexOf("["))));return a},"~S");E(c$,"$ignoreApplicationBundle",!1,"$getTextWrapper",null,"languageList",null,"allowDebug",!1,"vwr",null);c$.htLanguages=c$.prototype.htLanguages=new java.util.Hashtable});s("J.i18n");v(null,"J.i18n.Language",["J.i18n.GT"],function(){c$=
t(function(){this.nativeLanguage=this.language=this.code=null;this.display=!1;p(this,arguments)},J.i18n,"Language");c$.getLanguageList=c(c$,"getLanguageList",function(){return u(-1,[new J.i18n.Language("ar",J.i18n.GT.$("Arabic"),"\u0627\u0644\u0639\u0631\u0628\u064a\u0629",!1),new J.i18n.Language("ast",J.i18n.GT.$("Asturian"),"Asturian",!1),new J.i18n.Language("az",J.i18n.GT.$("Azerbaijani"),"az\u0259rbaycan dili",!1),new J.i18n.Language("bs",J.i18n.GT.$("Bosnian"),"bosanski jezik",!1),new J.i18n.Language("ca",
J.i18n.GT.$("Catalan"),"Catal\u00e0",!0),new J.i18n.Language("cs",J.i18n.GT.$("Czech"),"\u010ce\u0161tina",!0),new J.i18n.Language("da",J.i18n.GT.$("Danish"),"Dansk",!0),new J.i18n.Language("de",J.i18n.GT.$("German"),"Deutsch",!0),new J.i18n.Language("el",J.i18n.GT.$("Greek"),"\u0395\u03bb\u03bb\u03b7\u03bd\u03b9\u03ba\u03ac",!1),new J.i18n.Language("en_AU",J.i18n.GT.$("Australian English"),"Australian English",!1),new J.i18n.Language("en_GB",J.i18n.GT.$("British English"),"British English",!0),new J.i18n.Language("en_US",
J.i18n.GT.$("American English"),"American English",!0),new J.i18n.Language("es",J.i18n.GT.$("Spanish"),"Espa\u00f1ol",!0),new J.i18n.Language("et",J.i18n.GT.$("Estonian"),"Eesti",!1),new J.i18n.Language("eu",J.i18n.GT.$("Basque"),"Euskara",!0),new J.i18n.Language("fi",J.i18n.GT.$("Finnish"),"Suomi",!0),new J.i18n.Language("fo",J.i18n.GT.$("Faroese"),"F\u00f8royskt",!1),new J.i18n.Language("fr",J.i18n.GT.$("French"),"Fran\u00e7ais",!0),new J.i18n.Language("fy",J.i18n.GT.$("Frisian"),"Frysk",!1),new J.i18n.Language("gl",
J.i18n.GT.$("Galician"),"Galego",!1),new J.i18n.Language("hr",J.i18n.GT.$("Croatian"),"Hrvatski",!1),new J.i18n.Language("hu",J.i18n.GT.$("Hungarian"),"Magyar",!0),new J.i18n.Language("hy",J.i18n.GT.$("Armenian"),"\u0540\u0561\u0575\u0565\u0580\u0565\u0576",!1),new J.i18n.Language("id",J.i18n.GT.$("Indonesian"),"Indonesia",!0),new J.i18n.Language("it",J.i18n.GT.$("Italian"),"Italiano",!0),new J.i18n.Language("ja",J.i18n.GT.$("Japanese"),"\u65e5\u672c\u8a9e",!0),new J.i18n.Language("jv",J.i18n.GT.$("Javanese"),
"Basa Jawa",!1),new J.i18n.Language("ko",J.i18n.GT.$("Korean"),"\ud55c\uad6d\uc5b4",!0),new J.i18n.Language("ms",J.i18n.GT.$("Malay"),"Bahasa Melayu",!0),new J.i18n.Language("nb",J.i18n.GT.$("Norwegian Bokmal"),"Norsk Bokm\u00e5l",!1),new J.i18n.Language("nl",J.i18n.GT.$("Dutch"),"Nederlands",!0),new J.i18n.Language("oc",J.i18n.GT.$("Occitan"),"Occitan",!1),new J.i18n.Language("pl",J.i18n.GT.$("Polish"),"Polski",!1),new J.i18n.Language("pt",J.i18n.GT.$("Portuguese"),"Portugu\u00eas",!1),new J.i18n.Language("pt_BR",
J.i18n.GT.$("Brazilian Portuguese"),"Portugu\u00eas brasileiro",!0),new J.i18n.Language("ru",J.i18n.GT.$("Russian"),"\u0420\u0443\ufffd?\ufffd?\u043a\u0438\u0439",!0),new J.i18n.Language("sl",J.i18n.GT.$("Slovenian"),"Sloven\u0161\ufffd?ina",!1),new J.i18n.Language("sr",J.i18n.GT.$("Serbian"),"\ufffd?\u0440\u043f\ufffd?\u043a\u0438 \u0458\u0435\u0437\u0438\u043a",!1),new J.i18n.Language("sv",J.i18n.GT.$("Swedish"),"Svenska",!0),new J.i18n.Language("ta",J.i18n.GT.$("Tamil"),"\u0ba4\u0bae\u0bbf\u0bb4\ufffd?",
!1),new J.i18n.Language("te",J.i18n.GT.$("Telugu"),"\u0c24\u0c46\u0c32\ufffd?\u0c17\ufffd?",!1),new J.i18n.Language("tr",J.i18n.GT.$("Turkish"),"T\u00fcrk\u00e7e",!0),new J.i18n.Language("ug",J.i18n.GT.$("Uyghur"),"Uy\u01a3urq\u0259",!1),new J.i18n.Language("uk",J.i18n.GT.$("Ukrainian"),"\u0423\u043a\u0440\u0430\u0457\u043d\ufffd?\u044c\u043a\u0430",!0),new J.i18n.Language("uz",J.i18n.GT.$("Uzbek"),"O'zbek",!1),new J.i18n.Language("zh_CN",J.i18n.GT.$("Simplified Chinese"),"\u7b80\u4f53\u4e2d\u6587",
!0),new J.i18n.Language("zh_TW",J.i18n.GT.$("Traditional Chinese"),"\ufffd?\u9ad4\u4e2d\u6587",!0)])});q(c$,function(a,b,d,c){this.code=a;this.language=b;this.nativeLanguage=d;this.display=c},"~S,~S,~S,~B");c$.getSupported=c(c$,"getSupported",function(a,b){for(var d=a.length;0<=--d;)if(a[d].code.equalsIgnoreCase(b))return a[d].code;for(d=a.length;0<=--d;)if(a[d].code.startsWith(b))return a[d].code;return null},"~A,~S")});s("J.i18n");v(null,"J.i18n.Resource",["java.util.Hashtable","JU.PT","JU.Logger",
"JV.Viewer"],function(){c$=t(function(){this.resourceMap=this.resource=null;p(this,arguments)},J.i18n,"Resource");q(c$,function(a,b){null==b?this.resourceMap=a:this.resource=a},"~O,~S");c$.getResource=c(c$,"getResource",function(a,b,d){b=null;if(null!=a&&(JV.Viewer.isJS||a.isApplet))d=JV.Viewer.appletIdiomaBase+"/"+d+".po",JU.Logger.info("Loading language resource "+d),b=a.getFileAsString3(d,!1,"gt");return J.i18n.Resource.getResourceFromPO(b)},"JV.Viewer,~S,~S");c(c$,"getString",function(a){try{return null==
this.resource?this.resourceMap.get(a):this.resource.getString(a)}catch(b){if(H(b,Exception))return null;throw b;}},"~S");c$.getLanguage=c(c$,"getLanguage",function(){var a=null;return a=Jmol.featureDetection.getDefaultLanguage().replace(/-/g,"_")});c$.getResourceFromPO=c(c$,"getResourceFromPO",function(a){if(null==a||0==a.length)return null;var b=new java.util.Hashtable;try{var d=JU.PT.split(a,"\n");a=0;for(var c="",f="",e=0;e<d.length;e++){var h=d[e];2>=h.length?(2==a&&(0!=c.length&&0!=f.length)&&
b.put(f,c),a=0):0==h.indexOf("msgid")?(a=1,f=J.i18n.Resource.fix(h)):0==h.indexOf("msgstr")?(a=2,c=J.i18n.Resource.fix(h)):1==a?f+=J.i18n.Resource.fix(h):2==a&&(c+=J.i18n.Resource.fix(h))}}catch(k){if(!H(k,Exception))throw k;}JU.Logger.info(b.size()+" translations loaded");return 0==b.size()?null:new J.i18n.Resource(b,null)},"~S");c$.fix=c(c$,"fix",function(a){0<=a.indexOf('\\"')&&(a=JU.PT.rep(a,'\\"','"'));return JU.PT.rep(a.substring(a.indexOf('"')+1,a.lastIndexOf('"')),"\\n","\n")},"~S")});s("J.io");
v(null,"J.io.FileReader","java.io.BufferedInputStream $.BufferedReader $.Reader java.util.zip.ZipInputStream javajs.api.GenericBinaryDocument JU.AU $.PT $.Rdr J.api.Interface JU.Logger".split(" "),function(){c$=t(function(){this.htParams=this.readerOrDocument=this.atomSetCollection=this.fileTypeIn=this.nameAsGivenIn=this.fullPathNameIn=this.fileNameIn=this.vwr=null;this.isAppend=!1;this.bytesOrStream=null;p(this,arguments)},J.io,"FileReader");q(c$,function(a,b,d,c,f,e,h,k){this.vwr=a;this.fileNameIn=
null==b?d:b;this.fullPathNameIn=null==d?this.fileNameIn:d;this.nameAsGivenIn=null==c?this.fileNameIn:c;this.fileTypeIn=f;null!=e&&(JU.AU.isAB(e)||n(e,java.io.BufferedInputStream)?(this.bytesOrStream=e,e=null):n(e,java.io.Reader)&&!n(e,java.io.BufferedReader)&&(e=new java.io.BufferedReader(e)));this.readerOrDocument=e;this.htParams=h;this.isAppend=k},"JV.Viewer,~S,~S,~S,~S,~O,java.util.Map,~B");c(c$,"run",function(){!this.isAppend&&this.vwr.displayLoadErrors&&this.vwr.zap(!1,!0,!1);var a=null,b=null;
this.fullPathNameIn.contains("#_DOCACHE_")&&(this.readerOrDocument=J.io.FileReader.getChangeableReader(this.vwr,this.nameAsGivenIn,this.fullPathNameIn));if(null==this.readerOrDocument){b=this.vwr.fm.getUnzippedReaderOrStreamFromName(this.fullPathNameIn,this.bytesOrStream,!0,!1,!1,!0,this.htParams);if(null==b||n(b,String)){a=null==b?"error opening:"+this.nameAsGivenIn:b;a.startsWith("NOTE:")||JU.Logger.error("file ERROR: "+this.fullPathNameIn+"\n"+a);this.atomSetCollection=a;return}if(n(b,java.io.BufferedReader))this.readerOrDocument=
b;else if(n(b,java.util.zip.ZipInputStream)){var a=this.fullPathNameIn,d=null,a=a.$replace("\\","/");0<=a.indexOf("|")&&!a.endsWith(".zip")&&(d=JU.PT.split(a,"|"),a=d[0]);null!=d&&this.htParams.put("subFileList",d);d=b;b=this.vwr.fm.getZipDirectory(a,!0,!0);this.atomSetCollection=b=this.vwr.fm.getJzu().getAtomSetCollectionOrBufferedReaderFromZip(this.vwr,d,a,b,this.htParams,1,!1);try{d.close()}catch(c){if(!H(c,Exception))throw c;}}}n(b,java.io.BufferedInputStream)&&(this.readerOrDocument=J.api.Interface.getInterface("JU.BinaryDocument",
this.vwr,"file").setStream(b,!this.htParams.containsKey("isLittleEndian")));if(null!=this.readerOrDocument){this.atomSetCollection=this.vwr.getModelAdapter().getAtomSetCollectionReader(this.fullPathNameIn,this.fileTypeIn,this.readerOrDocument,this.htParams);n(this.atomSetCollection,String)||(this.atomSetCollection=this.vwr.getModelAdapter().getAtomSetCollection(this.atomSetCollection));try{n(this.readerOrDocument,java.io.BufferedReader)?this.readerOrDocument.close():n(this.readerOrDocument,javajs.api.GenericBinaryDocument)&&
this.readerOrDocument.close()}catch(f){if(!H(f,java.io.IOException))throw f;}}n(this.atomSetCollection,String)||(!this.isAppend&&!this.vwr.displayLoadErrors&&this.vwr.zap(!1,!0,!1),this.vwr.fm.setFileInfo(u(-1,[this.fullPathNameIn,this.fileNameIn,this.nameAsGivenIn])))});c$.getChangeableReader=c(c$,"getChangeableReader",function(a,b,d){return JU.Rdr.getBR(a.getLigandModel(b,d,"_file",null))},"JV.Viewer,~S,~S");c(c$,"getAtomSetCollection",function(){return this.atomSetCollection})});s("J.render");
v(["J.render.ShapeRenderer"],"J.render.BallsRenderer",["J.shape.Shape"],function(){c$=D(J.render,"BallsRenderer",J.render.ShapeRenderer);j(c$,"render",function(){var a=!1;if(this.isExport||this.vwr.checkMotionRendering(1140850689))for(var b=this.ms.at,d=this.shape.colixes,c=this.vwr.shm.bsRenderableAtoms,f=c.nextSetBit(0);0<=f;f=c.nextSetBit(f+1)){var e=b[f];0<e.sD&&0!=(e.shapeVisibilityFlags&this.myVisibilityFlag)&&(this.g3d.setC(null==d?e.colixAtom:J.shape.Shape.getColix(d,f,e))?this.g3d.drawAtom(e,
0):a=!0)}return a})});s("J.render");v(["J.render.ShapeRenderer","JU.P3","$.P3i","$.V3"],"J.render.FontLineShapeRenderer",["java.lang.Float","JU.PT"],function(){c$=t(function(){this.imageFontScaling=0;this.tickInfo=this.vectorT3=this.vectorT2=this.vectorT=this.pointT3=this.pointT2=this.pointT=this.s2=this.s1=this.pt2i=this.pt0i=this.font3d=this.tickBs=this.tickAs=this.tickB=this.tickA=null;this.draw000=!0;this.width=0;this.endcap=3;this.pt1=this.pt0=null;this.colixB=this.colixA=0;this.dotsOrDashes=
!1;this.dashDots=null;this.asLineOnly=!1;p(this,arguments)},J.render,"FontLineShapeRenderer",J.render.ShapeRenderer);O(c$,function(){this.pt0i=new JU.P3i;this.pt2i=new JU.P3i;this.s1=new JU.P3i;this.s2=new JU.P3i;this.pointT=new JU.P3;this.pointT2=new JU.P3;this.pointT3=new JU.P3;this.vectorT=new JU.V3;this.vectorT2=new JU.V3;this.vectorT3=new JU.V3;this.pt0=new JU.P3;this.pt1=new JU.P3});c(c$,"getDiameter",function(a,b){var d;d=20<b;switch(this.exportType){case 1:d=d?b:w(Math.floor(1E3*this.vwr.tm.unscaleToScreen(a,
2*b/10)));break;default:d?d=A(this.vwr.tm.scaleToScreen(a,w(b/10))):(this.g3d.isAntialiased()&&(b+=b),d=b)}return d},"~N,~N");c(c$,"renderLine",function(a,b,d,c){0>d?this.g3d.drawDashedLineBits(8,4,a,b):this.g3d.fillCylinderBits(this.endcap,d,a,b);c&&null!=this.tickInfo&&(this.checkTickTemps(),this.tickAs.setT(a),this.tickBs.setT(b),this.drawTicks(d,!0))},"JU.P3,JU.P3,~N,~B");c(c$,"checkTickTemps",function(){null==this.tickA&&(this.tickA=new JU.P3,this.tickB=new JU.P3,this.tickAs=new JU.P3,this.tickBs=
new JU.P3)});c(c$,"drawTicks",function(a,b){Float.isNaN(this.tickInfo.first)&&(this.tickInfo.first=0);this.drawTicks2(this.tickInfo.ticks.x,8,a,!b?null:null==this.tickInfo.tickLabelFormats?u(-1,["%0.2f"]):this.tickInfo.tickLabelFormats);this.drawTicks2(this.tickInfo.ticks.y,4,a,null);this.drawTicks2(this.tickInfo.ticks.z,2,a,null)},"~N,~B");c(c$,"drawTicks2",function(a,b,d,c){if(0!=a&&(this.g3d.isAntialiased()&&(b*=2),this.vectorT2.set(this.tickBs.x,this.tickBs.y,0),this.vectorT.set(this.tickAs.x,
this.tickAs.y,0),this.vectorT2.sub(this.vectorT),!(50>this.vectorT2.length()))){var f=this.tickInfo.signFactor;this.vectorT.sub2(this.tickB,this.tickA);var e=this.vectorT.length();if(null!=this.tickInfo.scale)if(Float.isNaN(this.tickInfo.scale.x)){var h=this.vwr.getUnitCellInfo(0);Float.isNaN(h)||this.vectorT.set(this.vectorT.x/h,this.vectorT.y/this.vwr.getUnitCellInfo(1),this.vectorT.z/this.vwr.getUnitCellInfo(2))}else this.vectorT.set(this.vectorT.x*this.tickInfo.scale.x,this.vectorT.y*this.tickInfo.scale.y,
this.vectorT.z*this.tickInfo.scale.z);h=this.vectorT.length()+1E-4*a;if(!(h<a)){this.vectorT.scale(a/h*e/h);var e=(this.tickBs.z-this.tickAs.z)/(h/a),h=h+this.tickInfo.first,k=w(Math.floor(this.tickInfo.first/a))*a-this.tickInfo.first;this.pointT.scaleAdd2(k/a,this.vectorT,this.tickA);var k=k+this.tickInfo.first,l=this.tickAs.z;0>d&&(d=1);this.vectorT2.set(-this.vectorT2.y,this.vectorT2.x,0);this.vectorT2.scale(b/this.vectorT2.length());b=this.tickInfo.reference;null==b?(this.pointT3.setT(this.vwr.getBoundBoxCenter()),
603979809==this.vwr.g.axesMode&&this.pointT3.add3(1,1,1)):this.pointT3.setT(b);this.tm.transformPtScr(this.pointT3,this.pt2i);b=0.2>Math.abs(this.vectorT2.x/this.vectorT2.y);for(var j=!b,m=!b&&0>this.vectorT2.x,r=null!=c&&0<c.length,x,y,n=Array(1),C=this.draw000?0:-1;k<h;){if(k>=this.tickInfo.first&&(this.pointT2.setT(this.pointT),this.tm.transformPt3f(this.pointT2,this.pointT2),this.drawLine(w(Math.floor(this.pointT2.x)),w(Math.floor(this.pointT2.y)),A(l),x=w(Math.floor(this.pointT2.x+this.vectorT2.x)),
y=w(Math.floor(this.pointT2.y+this.vectorT2.y)),A(l),d),r&&(this.draw000||0!=k))){n[0]=Float.$valueOf(0==k?0:k*f);var p=JU.PT.sprintf(c[C%c.length],"f",n);this.drawString(x,y,A(l),4,m,b,j,w(Math.floor(this.pointT2.y)),p)}this.pointT.add(this.vectorT);k+=a;l+=e;C++}}}},"~N,~N,~N,~A");c(c$,"drawLine",function(a,b,d,c,f,e,h){return this.drawLine2(a,b,d,c,f,e,h)},"~N,~N,~N,~N,~N,~N,~N");c(c$,"drawLine2",function(a,b,d,c,f,e,h){this.pt0.set(a,b,d);this.pt1.set(c,f,e);if(this.dotsOrDashes)null!=this.dashDots&&
this.drawDashed(a,b,d,c,f,e,this.dashDots);else{if(0>h)return this.g3d.drawDashedLineBits(8,4,this.pt0,this.pt1),1;this.g3d.fillCylinderBits(2,h,this.pt0,this.pt1)}return w((h+1)/2)},"~N,~N,~N,~N,~N,~N,~N");c(c$,"drawString",function(a,b,d,c,f,e,h,k,l){if(null!=l){var j=this.font3d.stringWidth(l),m=this.font3d.getAscent();a=f?a-(w(c/2)+2+j):e?a-(w(c/2)+2+w(j/2)):a+(w(c/2)+2);f=b;f=h?f+w(m/2):0==k||k<b?f+m:f-w(c/2);b=d-c-2;1>b&&(b=1);this.g3d.drawString(l,this.font3d,a,f,b,b,0)}},"~N,~N,~N,~N,~B,~B,~B,~N,~S");
c(c$,"drawDashed",function(a,b,d,c,f,e,h){if(!(null==h||0>this.width)){var k=h[0];c-=a;f-=b;e-=d;var l=0,j=h===J.render.FontLineShapeRenderer.ndots,m=j||h===J.render.FontLineShapeRenderer.sixdots;if(m){var r=(c*c+f*f)/(this.width*this.width);j?(k=Math.sqrt(r)/1.5,l=A(k)+2):8>r?h=J.render.FontLineShapeRenderer.twodots:32>r&&(h=J.render.FontLineShapeRenderer.fourdots)}var r=h[1],x=h[2],y=this.colixA,n=0==x?this.colixB:this.colixA;0==l&&(l=h.length);for(var C=0,p=3;p<l;p++){var C=j?C+1:h[p],q=w(Math.floor(a+
c*C/k)),s=w(Math.floor(b+f*C/k)),u=w(Math.floor(d+e*C/k));if(m)this.s1.set(q,s,u),p==r?this.g3d.setC(this.colixA):p==x&&this.g3d.setC(this.colixB),this.g3d.fillSphereI(this.width,this.s1);else{p==r&&(y=this.colixB);C=h[++p];p==x&&(n=this.colixB);var t=w(Math.floor(a+c*C/k)),v=w(Math.floor(b+f*C/k)),B=w(Math.floor(d+e*C/k));this.fillCylinder(y,n,2,this.width,q,s,u,t,v,B)}}}},"~N,~N,~N,~N,~N,~N,~A");c(c$,"fillCylinder",function(a,b,d,c,f,e,h,k,l,j){this.asLineOnly?this.g3d.drawLine(a,b,f,e,h,k,l,j):
this.g3d.fillCylinderXYZ(a,b,d,!this.isExport||1==this.mad?c:this.mad,f,e,h,k,l,j)},"~N,~N,~N,~N,~N,~N,~N,~N,~N,~N");E(c$,"dashes",B(-1,[12,0,0,2,5,7,10]),"hDashes",B(-1,[10,7,6,1,3,4,6,7,9]),"ndots",B(-1,[0,3,1E3]),"sixdots",B(-1,[12,3,6,1,3,5,7,9,11]),"fourdots",B(-1,[13,3,5,2,5,8,11]),"twodots",B(-1,[12,3,4,3,9]))});s("J.render");v(["J.render.ShapeRenderer"],"J.render.FrankRenderer",["JV.Viewer"],function(){c$=D(J.render,"FrankRenderer",J.render.ShapeRenderer);j(c$,"render",function(){var a=this.shape,
b=this.vwr.getBooleanProperty("allowKeyStrokes"),d=this.vwr.getBoolean(603983903);this.colix=d?20:this.vwr.isSignedApplet?b||(JV.Viewer.isJS||JV.Viewer.isSwingJS)&&!this.vwr.isWebGL?5:10:b?7:12;if(this.isExport||!this.vwr.getShowFrank()||!this.g3d.setC(this.colix))return!1;if(this.vwr.frankOn&&!this.vwr.noFrankEcho)return this.vwr.noFrankEcho;this.vwr.noFrankEcho=!0;b=this.vwr.imageFontScaling;a.getFont(b);b=A(a.frankWidth+4*b);this.g3d.drawStringNoSlab(a.frankString,a.font3d,this.vwr.gdata.width-
b,this.vwr.gdata.height-a.frankDescent,0,0);a=d?this.vwr.getModelkit(!1):null;d&&!a.isHidden()&&(this.g3d.setC(12),this.g3d.fillTextRect(0,0,1,0,10,104),d=a.getActiveMenu(),null!=d&&("atomMenu".equals(d)?(this.g3d.setC(21),this.g3d.fillTextRect(0,0,0,0,10,26)):"bondMenu".equals(d)?(this.g3d.setC(7),this.g3d.fillTextRect(0,26,0,0,10,26)):"xtalMenu".equals(d)&&(this.g3d.setC(8),this.g3d.fillTextRect(0,52,0,0,10,26))));return!1})});s("J.render");v(["J.api.JmolRepaintManager","JU.BS"],"J.render.RepaintManager",
"java.lang.NullPointerException $.Thread J.api.Interface JU.Logger JV.JC $.Viewer".split(" "),function(){c$=t(function(){this.bsTranslucent=this.renderers=this.shapeManager=this.vwr=null;this.holdRepaint=0;this.repaintPending=!1;p(this,arguments)},J.render,"RepaintManager",null,J.api.JmolRepaintManager);O(c$,function(){this.bsTranslucent=JU.BS.newN(37)});q(c$,function(){});j(c$,"set",function(a,b){this.vwr=a;this.shapeManager=b},"JV.Viewer,JV.ShapeManager");j(c$,"isRepaintPending",function(){return this.repaintPending});
j(c$,"pushHoldRepaint",function(){++this.holdRepaint},"~S");j(c$,"popHoldRepaint",function(a,b){--this.holdRepaint;0>=this.holdRepaint&&(this.holdRepaint=0,a&&(this.repaintPending=!0,this.repaintNow(b)))},"~B,~S");j(c$,"requestRepaintAndWait",function(a){var b=null;JV.Viewer.isJS&&!JV.Viewer.isSwingJS&&(b=self.Jmol&&Jmol.repaint?Jmol:null);if(null==b)try{this.repaintNow(a),JV.Viewer.isJS||this.wait(this.vwr.g.repaintWaitMs),this.repaintPending&&(JU.Logger.error("repaintManager requestRepaintAndWait timeout"),
this.repaintDone())}catch(d){if(H(d,InterruptedException))Zystem.out.println("repaintManager requestRepaintAndWait interrupted thread="+Thread.currentThread().getName());else throw d;}else b.repaint(this.vwr.html5Applet,!1),this.repaintDone()},"~S");j(c$,"repaintIfReady",function(a){if(this.repaintPending)return!1;this.repaintPending=!0;0==this.holdRepaint&&this.repaintNow(a);return!0},"~S");c(c$,"repaintNow",function(){this.vwr.haveDisplay&&this.vwr.apiPlatform.repaint(this.vwr.display)},"~S");j(c$,
"repaintDone",function(){this.repaintPending=!1});j(c$,"clear",function(a){if(null!=this.renderers)if(0<=a)this.renderers[a]=null;else for(a=0;37>a;++a)this.renderers[a]=null},"~N");c(c$,"getRenderer",function(a){if(null!=this.renderers[a])return this.renderers[a];var b=JV.JC.getShapeClassName(a,!0)+"Renderer";if(null==(b=J.api.Interface.getInterface(b,this.vwr,"render")))return null;b.setViewerG3dShapeID(this.vwr,a);return this.renderers[a]=b},"~N");j(c$,"render",function(a,b,d,c){null==this.renderers&&
(this.renderers=Array(37));this.getAllRenderers();try{var f=this.vwr.getBoolean(603979934);a.renderBackground(null);if(d){this.bsTranslucent.clearAll();null!=c&&a.renderCrossHairs(c,this.vwr.getScreenWidth(),this.vwr.getScreenHeight(),this.vwr.tm.getNavigationOffset(),this.vwr.tm.navigationDepthPercent);var e=this.vwr.getRubberBandSelection();null!=e&&a.setC(this.vwr.cm.colixRubberband)&&a.drawRect(e.x,e.y,0,0,e.width,e.height);this.vwr.noFrankEcho=!0}c=null;for(e=0;37>e&&a.currentlyRendering;++e){var h=
this.shapeManager.getShape(e);null!=h&&(f&&(c="rendering "+JV.JC.getShapeClassName(e,!1),JU.Logger.startTimer(c)),(d||this.bsTranslucent.get(e))&&this.getRenderer(e).renderShape(a,b,h)&&this.bsTranslucent.set(e),f&&JU.Logger.checkTimer(c,!1))}a.renderAllStrings(null)}catch(k){if(H(k,Exception)){k.printStackTrace();if(this.vwr.async&&"Interface".equals(k.getMessage()))throw new NullPointerException;JU.Logger.error("rendering error? "+k)}else throw k;}},"JU.GData,JM.ModelSet,~B,~A");c(c$,"getAllRenderers",
function(){for(var a=!0,b=0;37>b;++b)null==this.shapeManager.getShape(b)||null!=this.getRenderer(b)||(a=this.repaintPending=!this.vwr.async);if(!a)throw new NullPointerException;});j(c$,"renderExport",function(a,b,d){this.shapeManager.finalizeAtoms(null,!0);a=this.vwr.initializeExporter(d);if(null==a)return JU.Logger.error("Cannot export "+d.get("type")),null;null==this.renderers&&(this.renderers=Array(37));this.getAllRenderers();d=null;try{var c=this.vwr.getBoolean(603979934);a.renderBackground(a);
for(var f=0;37>f;++f){var e=this.shapeManager.getShape(f);null!=e&&(c&&(d="rendering "+JV.JC.getShapeClassName(f,!1),JU.Logger.startTimer(d)),this.getRenderer(f).renderShape(a,b,e),c&&JU.Logger.checkTimer(d,!1))}a.renderAllStrings(a);d=a.finalizeOutput()}catch(h){if(H(h,Exception))h.printStackTrace(),JU.Logger.error("rendering error? "+h);else throw h;}return d},"JU.GData,JM.ModelSet,java.util.Map")});s("J.render");v(null,"J.render.ShapeRenderer",["JV.JC"],function(){c$=t(function(){this.shape=this.ms=
this.g3d=this.tm=this.vwr=null;this.exportType=this.mad=this.colix=this.shapeID=this.myVisibilityFlag=0;this.isExport=!1;p(this,arguments)},J.render,"ShapeRenderer");c(c$,"initRenderer",function(){});c(c$,"setViewerG3dShapeID",function(a,b){this.vwr=a;this.tm=a.tm;this.shapeID=b;this.myVisibilityFlag=JV.JC.getShapeVisibilityFlag(b);this.initRenderer()},"JV.Viewer,~N");c(c$,"renderShape",function(a,b,d){this.setup(a,b,d);a=this.render();this.exportType=0;this.isExport=!1;return a},"J.api.JmolRendererInterface,JM.ModelSet,J.shape.Shape");
c(c$,"setup",function(a,b,d){this.g3d=a;this.ms=b;this.shape=d;this.exportType=a.getExportType();this.isExport=0!=this.exportType},"J.api.JmolRendererInterface,JM.ModelSet,J.shape.Shape");c(c$,"isVisibleForMe",function(a){return a.isVisible(this.myVisibilityFlag|9)},"JM.Atom")});s("J.render");v(["J.render.FontLineShapeRenderer","JU.BS","$.P3","$.V3"],"J.render.SticksRenderer","java.lang.Float JU.A4 $.M3 J.c.PAL JU.C $.Edge".split(" "),function(){c$=t(function(){this.showMultipleBonds=!1;this.multipleBondRadiusFactor=
this.multipleBondSpacing=0;this.useBananas=this.bondsPerp=!1;this.modeMultipleBond=0;this.isCartesian=!1;this.endcaps=0;this.hbondsSolid=this.bondsBackbone=this.hbondsBackbone=this.ssbondsBackbone=!1;this.bond=this.b=this.a=null;this.bondOrder=this.mag2d=this.dy=this.dx=this.zB=this.yB=this.xB=this.zA=this.yA=this.xA=0;this.slabByAtom=this.slabbing=this.isAntialiased=this.wireframeOnly=!1;this.bsForPass2=this.p2=this.p1=this.z=this.y=this.x=null;this.isPass2=!1;this.dyStep=this.dxStep=this.yAxis2=
this.xAxis2=this.yAxis1=this.xAxis1=this.rTheta=0;this.a4=this.rot=null;p(this,arguments)},J.render,"SticksRenderer",J.render.FontLineShapeRenderer);O(c$,function(){this.x=new JU.V3;this.y=new JU.V3;this.z=new JU.V3;this.p1=new JU.P3;this.p2=new JU.P3;this.bsForPass2=JU.BS.newN(64)});j(c$,"render",function(){var a=this.ms.bo;if(null==a)return!1;(this.isPass2=this.vwr.gdata.isPass2)||this.bsForPass2.clearAll();this.slabbing=this.tm.slabEnabled;this.slabByAtom=this.vwr.getBoolean(603979939);this.endcaps=
3;this.dashDots=this.vwr.getBoolean(603979893)?J.render.FontLineShapeRenderer.sixdots:J.render.FontLineShapeRenderer.dashes;this.isCartesian=1==this.exportType;this.getMultipleBondSettings(!1);this.wireframeOnly=!this.vwr.checkMotionRendering(1677721602);this.ssbondsBackbone=this.vwr.getBoolean(603979952);this.hbondsBackbone=this.vwr.getBoolean(603979852);this.bondsBackbone=(new Boolean(this.hbondsBackbone|this.ssbondsBackbone)).valueOf();this.hbondsSolid=this.vwr.getBoolean(603979854);this.isAntialiased=
this.g3d.isAntialiased();var b=!1;if(this.isPass2){if(!this.isExport)for(var d=this.bsForPass2.nextSetBit(0);0<=d;d=this.bsForPass2.nextSetBit(d+1))this.bond=a[d],this.renderBond()}else for(d=this.ms.bondCount;0<=--d;)this.bond=a[d],0!=(this.bond.shapeVisibilityFlags&this.myVisibilityFlag)&&this.renderBond()&&(b=!0,this.bsForPass2.set(d));return b});c(c$,"getMultipleBondSettings",function(a){this.useBananas=this.vwr.getBoolean(603979886)&&!a;this.multipleBondSpacing=a?0.15:this.vwr.getFloat(570425370);
this.multipleBondRadiusFactor=a?0.4:this.vwr.getFloat(570425369);this.bondsPerp=this.useBananas||0<this.multipleBondSpacing&&0>this.multipleBondRadiusFactor;this.useBananas&&(this.multipleBondSpacing=0>this.multipleBondSpacing?0.4*-this.multipleBondSpacing:this.multipleBondSpacing);this.multipleBondRadiusFactor=Math.abs(this.multipleBondRadiusFactor);0==this.multipleBondSpacing&&this.isCartesian&&(this.multipleBondSpacing=0.2);this.modeMultipleBond=this.vwr.g.modeMultipleBond;this.showMultipleBonds=
0!=this.multipleBondSpacing&&0!=this.modeMultipleBond&&this.vwr.getBoolean(603979928)},"~B");c(c$,"renderBond",function(){var a,b;this.a=a=this.bond.atom1;this.b=b=this.bond.atom2;var d=this.bond.order&-131073;this.bondsBackbone&&(this.ssbondsBackbone&&0!=(d&256)?(this.a=this.a.group.getLeadAtomOr(this.a),this.b=this.b.group.getLeadAtomOr(this.b)):this.hbondsBackbone&&JU.Edge.isOrderH(d)&&(this.a=this.a.group.getLeadAtomOr(this.a),this.b=this.b.group.getLeadAtomOr(this.b)));if(!this.isPass2&&(!this.a.isVisible(9)||
!this.b.isVisible(9)||!this.g3d.isInDisplayRange(this.a.sX,this.a.sY)||!this.g3d.isInDisplayRange(this.b.sX,this.b.sY)))return!1;if(this.slabbing){var c=this.vwr.gdata.isClippedZ(this.a.sZ);if(c&&this.vwr.gdata.isClippedZ(this.b.sZ)||this.slabByAtom&&(c||this.vwr.gdata.isClippedZ(this.b.sZ)))return!1}this.zA=this.a.sZ;this.zB=this.b.sZ;if(1==this.zA||1==this.zB)return!1;this.colixA=a.colixAtom;this.colixB=b.colixAtom;2==((this.colix=this.bond.colix)&-30721)?(this.colix&=30720,this.colixA=JU.C.getColixInherited(this.colix|
this.vwr.cm.getColixAtomPalette(a,J.c.PAL.CPK.id),this.colixA),this.colixB=JU.C.getColixInherited(this.colix|this.vwr.cm.getColixAtomPalette(b,J.c.PAL.CPK.id),this.colixB)):(this.colixA=JU.C.getColixInherited(this.colix,this.colixA),this.colixB=JU.C.getColixInherited(this.colix,this.colixB));a=!1;if(!this.isExport&&!this.isPass2&&(b=!JU.C.renderPass2(this.colixA),c=!JU.C.renderPass2(this.colixB),!b||!c)){if(!b&&!c&&!a)return this.g3d.setC(!b?this.colixA:this.colixB),!0;a=!0}this.bondOrder=d&-131073;
if(0==(this.bondOrder&224)&&(0!=(this.bondOrder&256)&&(this.bondOrder&=-257),0!=(this.bondOrder&1023)&&(!this.showMultipleBonds||2==this.modeMultipleBond&&500<this.mad||65536==(this.bondOrder&98304))))this.bondOrder=1;b=0;switch(this.bondOrder){case 1025:case 1041:this.bondOrder=1;case 1:case 2:case 3:case 4:case 5:case 6:break;case 17:case 513:this.bondOrder=1;b=513==d?0:1;break;case 515:case 514:this.bondOrder=2;b=515==d?this.getAromaticDottedBondMask():0;break;default:0!=(this.bondOrder&224)?(this.bondOrder=
JU.Edge.getPartialBondOrder(d),b=JU.Edge.getPartialBondDotted(d)):JU.Edge.isOrderH(this.bondOrder)?(this.bondOrder=1,this.hbondsSolid||(b=-1)):32768==this.bondOrder?this.bondOrder=1:98304==(this.bondOrder&98304)&&(this.getMultipleBondSettings(!0),this.bondOrder&=3,b=-2)}this.xA=this.a.sX;this.yA=this.a.sY;this.xB=this.b.sX;this.yB=this.b.sY;this.mad=this.bond.mad;0<this.multipleBondRadiusFactor&&1<this.bondOrder&&(this.mad*=this.multipleBondRadiusFactor);this.dx=this.xB-this.xA;this.dy=this.yB-this.yA;
this.width=A(this.vwr.tm.scaleToScreen(w((this.zA+this.zB)/2),this.mad));this.wireframeOnly&&0<this.width&&(this.width=1);if(!this.isCartesian&&(this.asLineOnly=1>=this.width)&&this.isAntialiased)this.width=3,this.asLineOnly=!1;switch(b){case -2:this.drawBond(0);this.getMultipleBondSettings(!1);break;case -1:this.drawDashed(this.xA,this.yA,this.zA,this.xB,this.yB,this.zB,J.render.FontLineShapeRenderer.hDashes);break;default:switch(this.bondOrder){case 4:this.bondOrder=2;d=this.multipleBondRadiusFactor;
0==d&&1<this.width&&(this.width=w(0.5*this.width));d=this.multipleBondSpacing;0>d&&(this.multipleBondSpacing=0.3);this.drawBond(b);this.bondsPerp=!this.bondsPerp;this.bondOrder=2;this.drawBond(b>>2);this.bondsPerp=!this.bondsPerp;this.multipleBondSpacing=d;break;case 5:this.bondOrder=3;d=this.multipleBondRadiusFactor;0==d&&1<this.width&&(this.width=w(0.5*this.width));d=this.multipleBondSpacing;0>d&&(this.multipleBondSpacing=0.2);this.drawBond(b);this.bondsPerp=!this.bondsPerp;this.bondOrder=2;this.multipleBondSpacing*=
1.5;this.drawBond(b>>3);this.bondsPerp=!this.bondsPerp;this.multipleBondSpacing=d;break;case 6:this.bondOrder=4;d=this.multipleBondRadiusFactor;0==d&&1<this.width&&(this.width=w(0.5*this.width));d=this.multipleBondSpacing;0>d&&(this.multipleBondSpacing=0.15);this.drawBond(b);this.bondsPerp=!this.bondsPerp;this.bondOrder=2;this.multipleBondSpacing*=1.5;this.drawBond(b>>4);this.bondsPerp=!this.bondsPerp;this.multipleBondSpacing=d;break;default:this.drawBond(b)}}return a});c(c$,"drawBond",function(a){var b=
0!=(a&1);if(this.isCartesian&&1==this.bondOrder&&!b)this.g3d.drawBond(this.a,this.b,this.colixA,this.colixB,this.endcaps,this.mad,-1);else{var d=0==this.dx&&0==this.dy;if(!d||!this.asLineOnly||this.isCartesian){var c=1<this.bondOrder&&0<this.multipleBondSpacing,f=c&&(null!=this.vwr.getHybridizationAndAxes(this.a.i,this.z,this.x,"pz")||null!=this.vwr.getHybridizationAndAxes(this.b.i,this.z,this.x,"pz"))&&!Float.isNaN(this.x.x);if(d&&!c){a=w(this.width/8)+3;a=this.width+a;b=this.yA-w((this.bondOrder-
1)*a/2);do this.fillCylinder(this.colixA,this.colixB,this.endcaps,this.width,this.xA,b,this.zA,this.xB,b,this.zB),b+=a;while(0<--this.bondOrder)}else if(1==this.bondOrder)b?this.drawDashed(this.xA,this.yA,this.zA,this.xB,this.yB,this.zB,this.dashDots):this.fillCylinder(this.colixA,this.colixB,this.endcaps,this.width,this.xA,this.yA,this.zA,this.xB,this.yB,this.zB);else if(c)if(f||this.z.setT(JU.P3.getUnlikely()),this.x.sub2(this.b,this.a),this.y.cross(this.x,this.z),this.y.normalize(),Float.isNaN(this.y.x)&&
(this.z.setT(JU.P3.getUnlikely()),this.y.cross(this.x,this.z),this.y.cross(this.y,this.x),this.y.normalize()),this.bondsPerp&&this.y.cross(this.y,this.x),this.y.scale(this.multipleBondSpacing),this.x.setT(this.y),this.x.scale((this.bondOrder-1)/2),this.useBananas)switch(this.drawBanana(this.a,this.b,this.x,0),this.bondOrder){case 4:this.drawBanana(this.a,this.b,this.x,90),this.drawBanana(this.a,this.b,this.x,-90);default:this.drawBanana(this.a,this.b,this.x,180);break;case 3:this.drawBanana(this.a,
this.b,this.x,120),this.drawBanana(this.a,this.b,this.x,-120)}else{this.p1.sub2(this.a,this.x);for(this.p2.sub2(this.b,this.x);;){this.isCartesian?this.g3d.drawBond(this.p1,this.p2,this.colixA,this.colixB,this.endcaps,this.mad,-2):(this.tm.transformPtScr(this.p1,this.s1),this.tm.transformPtScr(this.p2,this.s2),b?this.drawDashed(this.s1.x,this.s1.y,this.s1.z,this.s2.x,this.s2.y,this.s2.z,this.dashDots):this.fillCylinder(this.colixA,this.colixB,this.endcaps,this.width,this.s1.x,this.s1.y,this.s1.z,
this.s2.x,this.s2.y,this.s2.z));a>>=1;b=0!=(a&1);if(0>=--this.bondOrder)break;this.p1.add(this.y);this.p2.add(this.y);this.stepAxisCoordinates()}}else if(this.mag2d=Math.round(Math.sqrt(this.dx*this.dx+this.dy*this.dy)),this.resetAxisCoordinates(),this.isCartesian&&3==this.bondOrder)this.fillCylinder(this.colixA,this.colixB,this.endcaps,this.width,this.xAxis1,this.yAxis1,this.zA,this.xAxis2,this.yAxis2,this.zB),this.stepAxisCoordinates(),this.x.sub2(this.b,this.a),this.x.scale(0.05),this.p1.sub2(this.a,
this.x),this.p2.add2(this.b,this.x),this.g3d.drawBond(this.p1,this.p2,this.colixA,this.colixB,this.endcaps,this.mad,-2),this.stepAxisCoordinates(),this.fillCylinder(this.colixA,this.colixB,this.endcaps,this.width,this.xAxis1,this.yAxis1,this.zA,this.xAxis2,this.yAxis2,this.zB);else for(;;){0!=(a&1)?this.drawDashed(this.xAxis1,this.yAxis1,this.zA,this.xAxis2,this.yAxis2,this.zB,this.dashDots):this.fillCylinder(this.colixA,this.colixB,this.endcaps,this.width,this.xAxis1,this.yAxis1,this.zA,this.xAxis2,
this.yAxis2,this.zB);a>>=1;if(0>=--this.bondOrder)break;this.stepAxisCoordinates()}}}},"~N");c(c$,"resetAxisCoordinates",function(){var a=this.mag2d>>3;-1!=this.multipleBondSpacing&&0>this.multipleBondSpacing&&(a*=-this.multipleBondSpacing);a=this.width+a;this.dxStep=w(a*this.dy/this.mag2d);this.dyStep=w(a*-this.dx/this.mag2d);this.xAxis1=this.xA;this.yAxis1=this.yA;this.xAxis2=this.xB;this.yAxis2=this.yB;a=this.bondOrder-1;this.xAxis1-=w(this.dxStep*a/2);this.yAxis1-=w(this.dyStep*a/2);this.xAxis2-=
w(this.dxStep*a/2);this.yAxis2-=w(this.dyStep*a/2)});c(c$,"stepAxisCoordinates",function(){this.xAxis1+=this.dxStep;this.yAxis1+=this.dyStep;this.xAxis2+=this.dxStep;this.yAxis2+=this.dyStep});c(c$,"getAromaticDottedBondMask",function(){var a=this.b.findAromaticNeighbor(this.a.i);return null==a?1:0>this.dx*(a.sY-this.yA)-this.dy*(a.sX-this.xA)?2:1});c(c$,"drawBanana",function(a,b,d,c){this.g3d.addRenderer(553648145);this.vectorT.sub2(b,a);null==this.rot&&(this.rot=new JU.M3,this.a4=new JU.A4);this.a4.setVA(this.vectorT,
3.141592653589793*c/180);this.rot.setAA(this.a4);this.pointT.setT(a);this.pointT3.setT(b);this.pointT2.ave(a,b);this.rot.rotate2(d,this.vectorT);this.pointT2.add(this.vectorT);this.tm.transformPtScrT3(a,this.pointT);this.tm.transformPtScrT3(this.pointT2,this.pointT2);this.tm.transformPtScrT3(b,this.pointT3);a=Math.max(this.width,1);this.g3d.setC(this.colixA);this.g3d.fillHermite(5,a,a,a,this.pointT,this.pointT,this.pointT2,this.pointT3);this.g3d.setC(this.colixB);this.g3d.fillHermite(5,a,a,a,this.pointT,
this.pointT2,this.pointT3,this.pointT3)},"JM.Atom,JM.Atom,JU.V3,~N")});s("JS");v(["JS.T"],"JS.ContextToken",["java.util.Hashtable","JS.SV"],function(){c$=t(function(){this.name0=this.forVars=this.contextVariables=null;p(this,arguments)},JS,"ContextToken",JS.T);c$.newContext=c(c$,"newContext",function(a){a=a?JS.ContextToken.newCmd(1275335685,"{"):JS.ContextToken.newCmd(1275334681,"}");a.intValue=0;return a},"~B");c$.newCmd=c(c$,"newCmd",function(a,b){var d=new JS.ContextToken;d.tok=a;d.value=b;return d},
"~N,~O");c(c$,"addName",function(a){null==this.contextVariables&&(this.contextVariables=new java.util.Hashtable);this.contextVariables.put(a,JS.SV.newS("").setName(a))},"~S")});s("JS");v(null,"JS.ScriptContext",["java.util.Hashtable","JS.SV"],function(){c$=t(function(){this.aatoken=null;this.chk=this.allowJSThreads=!1;this.contextPath=" >> ";this.vars=null;this.displayLoadErrorsSave=!1;this.errorType=this.errorMessageUntranslated=this.errorMessage=null;this.executionStepping=this.executionPaused=
!1;this.functionName=null;this.iCommandError=-1;this.id=0;this.isComplete=!0;this.isTryCatch=this.isStateScript=this.isJSThread=this.isFunction=!1;this.forVars=null;this.iToken=0;this.lineEnd=2147483647;this.lineNumbers=this.lineIndices=null;this.mustResumeEval=!1;this.parentContext=this.parallelProcessor=this.outputBuffer=null;this.pc0=this.pc=0;this.pcEnd=2147483647;this.scriptFileName=this.scriptExtensions=this.script=null;this.scriptLevel=0;this.htFileCache=this.statement=null;this.statementLength=
0;this.token=null;this.tryPt=0;this.theToken=null;this.theTok=0;this.privateFuncs=this.why=this.pointers=null;p(this,arguments)},JS,"ScriptContext");q(c$,function(){this.id=++JS.ScriptContext.contextCount});c(c$,"setMustResume",function(){for(var a=this;null!=a;)a.mustResumeEval=!0,a.pc=a.pc0,a=a.parentContext});c(c$,"getVariable",function(a){for(var b=this;null!=b&&!b.isFunction;){if(null!=b.vars&&b.vars.containsKey(a))return b.vars.get(a);b=b.parentContext}return null},"~S");c(c$,"getFullMap",function(){var a=
new java.util.Hashtable,b=this;for(null!=this.contextPath&&a.put("_path",JS.SV.newS(this.contextPath));null!=b&&!b.isFunction;){if(null!=b.vars)for(var d,c=b.vars.keySet().iterator();c.hasNext()&&((d=c.next())||1);)if(!a.containsKey(d)){var f=b.vars.get(d);(2!=f.tok||2147483647!=f.intValue)&&a.put(d,f)}b=b.parentContext}return a});c(c$,"saveTokens",function(a){this.aatoken=a;if(null==a)this.pointers=null;else{this.pointers=B(a.length,0);for(var b=this.pointers.length;0<=--b;)this.pointers[b]=null==
a[b]?-1:a[b][0].intValue}},"~A");c(c$,"restoreTokens",function(){if(null!=this.pointers)for(var a=this.pointers.length;0<=--a;)null!=this.aatoken[a]&&(this.aatoken[a][0].intValue=this.pointers[a]);return this.aatoken});c(c$,"getTokenCount",function(){return null==this.aatoken?-1:this.aatoken.length});c(c$,"getToken",function(a){return this.aatoken[a]},"~N");E(c$,"contextCount",0)});s("JS");v(["java.lang.Exception"],"JS.ScriptException",null,function(){c$=t(function(){this.untranslated=this.message=
this.eval=null;this.isError=!1;p(this,arguments)},JS,"ScriptException",Exception);q(c$,function(a,b,d,c){this.eval=a;this.message=b;(this.isError=c)&&this.eval.setException(this,b,d)},"JS.ScriptError,~S,~S,~B");c(c$,"getErrorMessageUntranslated",function(){return this.untranslated});j(c$,"getMessage",function(){return this.message});j(c$,"toString",function(){return this.message})});s("JS");v(["javajs.api.JSONEncodable","JS.T","JU.P3"],"JS.SV","java.lang.Boolean $.Float java.util.Arrays $.Collections $.Hashtable $.Map JU.AU $.BArray $.BS $.Base64 $.Lst $.M3 $.M34 $.M4 $.Measure $.P4 $.PT $.Quat $.SB $.T3 $.V3 JM.BondSet JS.ScriptContext JU.BSUtil $.Escape JV.Viewer".split(" "),
function(){c$=t(function(){this.index=2147483647;this.myName=null;R("JS.SV.Sort")||JS.SV.$SV$Sort$();p(this,arguments)},JS,"SV",JS.T,javajs.api.JSONEncodable);c$.newV=c(c$,"newV",function(a,b){var d=new JS.SV;d.tok=a;d.value=b;return d},"~N,~O");c$.newI=c(c$,"newI",function(a){var b=new JS.SV;b.tok=2;b.intValue=a;return b},"~N");c$.newF=c(c$,"newF",function(a){var b=new JS.SV;b.tok=3;b.value=Float.$valueOf(a);return b},"~N");c$.newS=c(c$,"newS",function(a){return JS.SV.newV(4,a)},"~S");c$.newT=c(c$,
"newT",function(a){return JS.SV.newSV(a.tok,a.intValue,a.value)},"JS.T");c$.newSV=c(c$,"newSV",function(a,b,d){a=JS.SV.newV(a,d);a.intValue=b;return a},"~N,~N,~O");c(c$,"setv",function(a){this.index=a.index;this.intValue=a.intValue;this.tok=a.tok;this.value=a.value;return this},"JS.SV");c$.sizeOf=c(c$,"sizeOf",function(a){switch(null==a?0:a.tok){case 10:return JS.SV.bsSelectToken(a).cardinality();case 1073742335:case 1073742334:return-1;case 2:return-2;case 3:return-4;case 8:return-8;case 9:return-16;
case 11:return-32;case 12:return-64;case 15:return a.value.data.length;case 4:return a.value.length;case 7:return 2147483647==a.intValue?a.getList().size():JS.SV.sizeOf(JS.SV.selectItemTok(a,-2147483648));case 6:return a.value.size();case 14:return a.value.getFullMap().size();default:return 0}},"JS.T");c$.isVariableType=c(c$,"isVariableType",function(a){return n(a,JS.SV)||n(a,Boolean)||n(a,Integer)||n(a,Float)||n(a,String)||n(a,JU.T3)||n(a,JU.BS)||n(a,JU.P4)||n(a,JU.Quat)||n(a,JU.M34)||n(a,java.util.Map)||
n(a,JU.Lst)||n(a,JU.BArray)||n(a,JS.ScriptContext)||JS.SV.isArray(a)},"~O");c$.isArray=c(c$,"isArray",function(a){return n(a,Array)},"~O");c$.getVariable=c(c$,"getVariable",function(a){return null==a?JS.SV.newS(""):n(a,JS.SV)?a:n(a,Boolean)?JS.SV.getBoolean(a.booleanValue()):n(a,Integer)?JS.SV.newI(a.intValue()):n(a,Float)?JS.SV.newV(3,a):n(a,String)?(a=JS.SV.unescapePointOrBitsetAsVariable(a),n(a,JS.SV)?a:JS.SV.newV(4,a)):n(a,JU.P3)?JS.SV.newV(8,a):n(a,JU.V3)?JS.SV.newV(8,JU.P3.newP(a)):n(a,JU.BS)?
JS.SV.newV(10,a):n(a,JU.P4)?JS.SV.newV(9,a):n(a,JU.Quat)?JS.SV.newV(9,a.toPoint4f()):n(a,JU.M34)?JS.SV.newV(n(a,JU.M4)?12:11,a):n(a,java.util.Map)?JS.SV.getVariableMap(a):n(a,JU.Lst)?JS.SV.getVariableList(a):n(a,JU.BArray)?JS.SV.newV(15,a):n(a,JS.ScriptContext)?JS.SV.newV(14,a):JS.SV.isASV(a)?JS.SV.getVariableAV(a):JU.AU.isAI(a)?JS.SV.getVariableAI(a):JU.AU.isAB(a)?JS.SV.getVariableAB(a):JU.AU.isAF(a)?JS.SV.getVariableAF(a):JU.AU.isAD(a)?JS.SV.getVariableAD(a):JU.AU.isAS(a)?JS.SV.getVariableAS(a):
JU.AU.isAP(a)?JS.SV.getVariableAP(a):JU.AU.isAII(a)?JS.SV.getVariableAII(a):JU.AU.isAFF(a)?JS.SV.getVariableAFF(a):JU.AU.isASS(a)?JS.SV.getVariableASS(a):JU.AU.isADD(a)?JS.SV.getVariableADD(a):JU.AU.isAFloat(a)?JS.SV.newV(13,a):JS.SV.newJSVar(a)},"~O");c$.isASV=c(c$,"isASV",function(a){return!JV.Viewer.isSwingJS?a&&a[0]&&"JS.SV"==a[0].__CLASS_NAME__:n(a,Array)},"~O");c$.newJSVar=c(c$,"newJSVar",function(a){var b,d,c,f,e;switch(a.BYTES_PER_ELEMENT?Array:a.constructor){case Boolean:b=0;d=a;break;case Number:b=
1;c=a;break;case Array:b=2;f=a;break;case Object:b=3,f=a,e=Object.keys(a)}switch(b){case 0:return d?JS.SV.vT:JS.SV.vF;case 1:return 2147483647<c||c!=Math.floor(c)?JS.SV.newF(c):JS.SV.newI(A(c));case 2:e=new JU.Lst;a=0;for(b=f.length;a<b;a++)e.addLast(JS.SV.newJSVar(f[a]));return JS.SV.getVariableList(e);case 3:b=new java.util.Hashtable;for(a=e.length;0<=--a;)d=null,d=f[e[a]],b.put(e[a],JS.SV.newJSVar(d));return JS.SV.getVariableMap(b)}return JS.SV.newS(a.toString())},"~O");c$.getVariableMap=c(c$,
"getVariableMap",function(a){for(var b=a,d=null,c,f=b.values().iterator();f.hasNext()&&((c=f.next())||1);){d=c;break}if(!n(d,JS.SV)){a=new java.util.Hashtable;for(var e,b=b.entrySet().iterator();b.hasNext()&&((e=b.next())||1);)a.put(e.getKey(),JS.SV.getVariable(e.getValue()))}return JS.SV.newV(6,a)},"java.util.Map");c$.getVariableList=c(c$,"getVariableList",function(a){var b=a.size();if(0<b&&n(a.get(0),JS.SV))return JS.SV.newV(7,a);for(var d=new JU.Lst,c=0;c<b;c++)d.addLast(JS.SV.getVariable(a.get(c)));
return JS.SV.newV(7,d)},"JU.Lst");c$.getVariableAV=c(c$,"getVariableAV",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(a[d]);return JS.SV.newV(7,b)},"~A");c$.getVariableAD=c(c$,"getVariableAD",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.newV(3,Float.$valueOf(a[d])));return JS.SV.newV(7,b)},"~A");c$.getVariableAO=c(c$,"getVariableAO",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.getVariable(a[d]));return JS.SV.newV(7,b)},"~A");c$.getVariableAS=
c(c$,"getVariableAS",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.newV(4,a[d]));return JS.SV.newV(7,b)},"~A");c$.getVariableAP=c(c$,"getVariableAP",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.newV(8,a[d]));return JS.SV.newV(7,b)},"~A");c$.getVariableAFF=c(c$,"getVariableAFF",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.getVariableAF(a[d]));return JS.SV.newV(7,b)},"~A");c$.getVariableADD=c(c$,"getVariableADD",function(a){for(var b=
new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.getVariableAD(a[d]));return JS.SV.newV(7,b)},"~A");c$.getVariableASS=c(c$,"getVariableASS",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.getVariableAS(a[d]));return JS.SV.newV(7,b)},"~A");c$.getVariableAII=c(c$,"getVariableAII",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.getVariableAI(a[d]));return JS.SV.newV(7,b)},"~A");c$.getVariableAF=c(c$,"getVariableAF",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.newV(3,
Float.$valueOf(a[d])));return JS.SV.newV(7,b)},"~A");c$.getVariableAI=c(c$,"getVariableAI",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.newI(a[d]));return JS.SV.newV(7,b)},"~A");c$.getVariableAB=c(c$,"getVariableAB",function(a){for(var b=new JU.Lst,d=0;d<a.length;d++)b.addLast(JS.SV.newI(a[d]));return JS.SV.newV(7,b)},"~A");c(c$,"setName",function(a){this.myName=a;return this},"~S");c(c$,"canIncrement",function(){switch(this.tok){case 2:case 3:return!0;default:return!1}});c(c$,
"increment",function(a){switch(this.tok){case 2:return this.intValue+=a,!0;case 3:return this.value=Float.$valueOf(this.value.floatValue()+a),!0;default:return!1}},"~N");c(c$,"asBoolean",function(){return JS.SV.bValue(this)});c(c$,"asInt",function(){return JS.SV.iValue(this)});c(c$,"asFloat",function(){return JS.SV.fValue(this)});c(c$,"asString",function(){return JS.SV.sValue(this)});c$.oValue=c(c$,"oValue",function(a){if(!n(a,JS.SV))return a;switch(a.tok){case 1073742335:return Boolean.TRUE;case 0:case 1073742334:return Boolean.FALSE;
case 2:return Integer.$valueOf(a.intValue);case 10:case 1275068418:return JS.SV.selectItemVar(a).value;default:return a.value}},"~O");c$.nValue=c(c$,"nValue",function(a){switch(null==a?0:a.tok){case 3:return a.value;case 2:a=a.intValue;break;case 4:if(0<=a.value.indexOf("."))return Float.$valueOf(JS.SV.toFloat(a.value));a=A(JS.SV.toFloat(a.value));break;case 8:return Float.$valueOf(a.value.length());default:a=0}return Integer.$valueOf(a)},"JS.T");c$.bValue=c(c$,"bValue",function(a){switch(null==a?
0:a.tok){case 1073742335:case 14:return!0;case 1073742334:return!1;case 2:return 0!=a.intValue;case 3:case 4:case 7:return 0!=JS.SV.fValue(a);case 10:case 15:return 0!=JS.SV.iValue(a);case 8:case 9:case 11:case 12:return 1E-4<Math.abs(JS.SV.fValue(a));case 6:return!a.getMap().isEmpty();default:return!1}},"JS.T");c$.iValue=c(c$,"iValue",function(a){switch(null==a?0:a.tok){case 1073742335:return 1;case 1073742334:return 0;case 2:return a.intValue;case 3:case 7:case 4:case 8:case 9:case 11:case 12:case 134221850:return A(JS.SV.fValue(a));
case 10:return JS.SV.bsSelectToken(a).cardinality();case 15:return a.value.data.length;default:return 0}},"JS.T");c$.fValue=c(c$,"fValue",function(a){switch(null==a?0:a.tok){case 1073742335:return 1;case 1073742334:return 0;case 2:return a.intValue;case 3:return a.value.floatValue();case 7:if(2147483647==a.intValue)return a.getList().size();case 4:return JS.SV.toFloat(JS.SV.sValue(a));case 10:case 15:return JS.SV.iValue(a);case 8:return a.value.length();case 9:return JU.Measure.distanceToPlane(a.value,
JS.SV.pt0);case 11:var b=new JU.P3;a.value.rotate(b);return b.length();case 12:return b=new JU.P3,a.value.rotTrans(b),b.length();default:return 0}},"JS.T");c$.sValue=c(c$,"sValue",function(a){if(null==a)return"";var b;switch(a.tok){case 1073742335:return"true";case 1073742334:return"false";case 2:return""+a.intValue;case 10:return b=JS.SV.bsSelectToken(a),n(a.value,JM.BondSet)?JU.Escape.eBond(b):JU.Escape.eBS(b);case 7:var d=a.getList();b=a.intValue;0>=b&&(b=d.size()-b);if(2147483647!=b)return 1>
b||b>d.size()?"":JS.SV.sValue(d.get(b-1));case 6:case 14:if(n(a.value,String))return a.value;b=new JU.SB;JS.SV.sValueArray(b,a,"","",!1,!0,!0,2147483647,!1);return JU.PT.rep(b.toString(),"\n\x00"," ");case 4:return d=a.value,b=a.intValue,0>=b&&(b=d.length-b),2147483647==b?d:1>b||b>d.length?"":""+d.charAt(b-1);case 8:return JU.Escape.eP(a.value);case 9:return JU.Escape.eP4(a.value);case 11:case 12:return JU.Escape.e(a.value);default:return a.value.toString()}},"JS.T");c$.sValueArray=c(c$,"sValueArray",
function(a,b,d,c,f,e,h,k,l){switch(b.tok){case 6:case 14:case 7:var j=";"+b.hashCode()+";";if(0<=d.indexOf(j)){a.append(f?7==b.tok?"[ ]":"{ }":(7==b.tok?"":"\x00")+'"<'+(null==b.myName?"circular reference":b.myName)+'>"');break}d+=j;if(7==b.tok){if(!h)break;e||a.append(f?"[ ":c+"[\n");b=b.getList();for(j=0;j<b.size();j++){f&&0<j&&a.append(",");var m=b.get(j);JS.SV.sValueArray(a,m,d,c+"  ",f,0==c.length&&!f&&JS.SV.isRawType(m.tok),h,k,l);f||a.append("\n")}e||a.append(f?" ]":c+"]")}else 0<=--k&&(e=
14==b.tok?b.value.getFullMap():b.getMap(),JS.SV.sValueAddKeys(a,d,e,c,f,h,k,l));break;default:if(!h)break;!e&&!f&&a.append(c);a.append(f?b.escape():JS.SV.sValue(b))}},"JU.SB,JS.SV,~S,~S,~B,~B,~B,~N,~B");c$.sValueAddKeys=c(c$,"sValueAddKeys",function(a,b,d,c,f,e,h,k){if(!(0>h)){var l=d.keySet(),l=d.keySet().toArray(Array(l.size()));java.util.Arrays.sort(l);if(f){a.append("{ ");for(var j="",m=0;m<l.length;m++){f=l[m];var r=d.get(f);if(!k||!(7==r.tok&&0==r.getList().size()||6==r.tok&&r.getMap().isEmpty()))e?
a.append(j).append(JU.PT.esc(f)).append(":"):a.appendC(" ").append(f),JS.SV.sValueArray(a,r,b,c+"  ",!0,!1,e,h,k),j=","}a.append(" }");e||a.append("\n")}else{a.append(c).append("{\n");c+="  ";for(m=0;m<l.length;m++)a.append(c),f=l[m],a.append(JU.PT.esc(f)).append("  :"),j=new JU.SB,n(d.get(f),JS.SV)||d.put(f,JS.SV.getVariable(d.get(f))),r=d.get(f),f=JS.SV.isRawType(r.tok),JS.SV.sValueArray(j,r,b,c,f,!1,e,h,k),j=j.toString(),f&&e?a.append("  "):a.append("\n"),a.append(j).append("\n");a.append(c.substring(1)).append("}")}}},
"JU.SB,~S,java.util.Map,~S,~B,~B,~N,~B");c$.isRawType=c(c$,"isRawType",function(a){switch(a){case 4:case 3:case 2:case 8:case 9:case 10:case 15:case 1073742335:case 1073742334:return!0}return!1},"~N");c$.ptValue=c(c$,"ptValue",function(a){switch(a.tok){case 8:return a.value;case 4:if(a=JU.Escape.uP(a.value),n(a,JU.P3))return a}return null},"JS.SV");c$.pt4Value=c(c$,"pt4Value",function(a){switch(a.tok){case 9:return a.value;case 4:a=JU.Escape.uP(a.value);if(!n(a,JU.P4))break;return a}return null},
"JS.SV");c$.toFloat=c(c$,"toFloat",function(a){return a.equalsIgnoreCase("true")?1:0==a.length||a.equalsIgnoreCase("false")?0:JU.PT.parseFloatStrict(JU.PT.trim(a," \t\n"))},"~S");c$.concatList=c(c$,"concatList",function(a,b,d){var c=a.getList(),f=b.getList();if(!d){if(null==f)c.addLast(JS.SV.newT(b));else for(d=0;d<f.size();d++)c.addLast(f.get(d));return a}var e=new JU.Lst;if(null==c)e.addLast(a);else for(d=0;d<c.size();d++)e.addLast(c.get(d));if(null==f)e.addLast(b);else for(d=0;d<f.size();d++)e.addLast(f.get(d));
return JS.SV.getVariableList(e)},"JS.SV,JS.SV,~B");c$.bsSelectToken=c(c$,"bsSelectToken",function(a){return JS.SV.selectItemTok(a,-2147483648).value},"JS.T");c$.bsSelectRange=c(c$,"bsSelectRange",function(a,b){a=JS.SV.selectItemTok(a,-2147483648);a=JS.SV.selectItemTok(a,0>=b?b:1);a=JS.SV.selectItemTok(a,0>=b?2147483646:b);return a.value},"JS.T,~N");c$.selectItemVar=c(c$,"selectItemVar",function(a){return 2147483647!=a.index||(7==a.tok||15==a.tok)&&2147483647==a.intValue?a:JS.SV.selectItemTok(a,-2147483648)},
"JS.SV");c$.selectItemTok=c(c$,"selectItemTok",function(a,b){switch(a.tok){case 11:case 12:case 10:case 7:case 15:case 4:break;default:return n(a,JS.SV)&&null!=a.myName?JS.SV.newI(0).setv(a):a}var d=null,c=null,f=a.intValue,e=-2147483648==b;if(2147483647==f)return JS.SV.newSV(a.tok,e?f:b,a.value);var h=0,k=n(a,JS.SV)&&2147483647!=a.index,l=JS.SV.newSV(a.tok,2147483647,null);switch(a.tok){case 10:n(a.value,JM.BondSet)?(d=JM.BondSet.newBS(a.value,a.value.associatedAtoms),h=d.cardinality()):(d=JU.BSUtil.copy(a.value),
h=k?1:d.cardinality());break;case 15:h=a.value.data.length;break;case 7:h=a.getList().size();break;case 4:c=a.value;h=c.length;break;case 11:h=-3;break;case 12:h=-4}if(0>h){h=-h;if(0<f&&Math.abs(f)>h)return l=f%10,f=w((f-l)/10),0<l&&l<=h&&f<=h?JS.SV.newV(3,Float.$valueOf(a.value.getElement(f-1,l-1))):JS.SV.newV(4,"");if(Math.abs(f)>h)return JS.SV.newV(4,"");d=K(h,0);0>f?a.value.getColumn(-1-f,d):a.value.getRow(f-1,d);return e?JS.SV.getVariableAF(d):1>b||b>h?JS.SV.newV(4,""):JS.SV.newV(3,Float.$valueOf(d[b-
1]))}0>=f&&(f=h+f);e||(1>f&&(f=1),0==b?b=h:0>b&&(b=h+b),b<f&&(b=f));switch(a.tok){case 10:l.value=d;if(k){1<f&&d.clearAll();break}if(e){f==h?b=d.length()-1:1==f&&(b=d.nextSetBit(0));if(-1<=b){d.clearAll();0<=b&&d.set(b);break}b=f}h=0;for(e=d.nextSetBit(0);0<=e;e=d.nextSetBit(e+1))(++h<f||h>b)&&d.clear(e);break;case 4:l.value=0>--f||f>=h?"":e?c.substring(f,f+1):c.substring(f,Math.min(b,h));break;case 7:if(0>--f||f>=h)return JS.SV.newV(4,"");if(e)return a.getList().get(f);d=new JU.Lst;e=a.getList();
c=Math.min(b,h)-f;for(h=0;h<c;h++)d.addLast(JS.SV.newT(e.get(h+f)));l.value=d;break;case 15:if(0>--f||f>=h)return JS.SV.newV(4,"");d=a.value.data;if(e)return JS.SV.newI(d[f]);e=Q(Math.min(b,h)-f,0);for(h=e.length;0<=--h;)e[h]=d[f+h];l.value=new JU.BArray(e)}return l},"JS.T,~N");c(c$,"setSelectedValue",function(a,b,d){if(2147483647!=a){var c;switch(this.tok){case 11:case 12:c=11==this.tok?3:4;if(2147483647!=b){var f=a;if(0<b&&b<=c&&f<=c){this.value.setElement(f-1,b-1,JS.SV.fValue(d));break}}if(0!=
a&&(Math.abs(a)<=c&&7==d.tok)&&(d=d.getList(),d.size()==c)){f=K(c,0);for(b=0;b<c;b++)f[b]=JS.SV.fValue(d.get(b));0<a?this.value.setRowA(a-1,f):this.value.setColumnA(-1-a,f);break}break;case 4:c=this.value;f=c.length;0>=a&&(a=f+a);for(0>--a&&(a=0);a>=c.length;)c+=" ";if(2147483647==b)b=a;else for(0>--b&&(b=f+b);b>=c.length;)c+=" ";b>=a&&(this.value=c.substring(0,a)+JS.SV.sValue(d)+c.substring(++b));this.intValue=this.index=2147483647;break;case 7:f=this.value;c=f.size();0>=a&&(a=c+a);0>--a&&(a=0);
if(c<=a)for(b=c;b<=a;b++)f.addLast(JS.SV.newV(4,""));f.set(a,d)}}},"~N,~N,JS.SV");c(c$,"escape",function(){switch(this.tok){case 4:return JU.PT.esc(this.value);case 11:case 12:return JU.PT.toJSON(null,this.value);case 7:case 6:case 14:var a=new JU.SB;JS.SV.sValueArray(a,this,"","",!0,!1,!0,2147483647,!1);return a.toString();default:return JS.SV.sValue(this)}});c$.unescapePointOrBitsetAsVariable=c(c$,"unescapePointOrBitsetAsVariable",function(a){if(null==a)return a;var b=null,d=null;if(n(a,JS.SV))switch(a.tok){case 8:case 9:case 11:case 12:case 10:b=
a.value;break;case 4:d=a.value;break;default:d=JS.SV.sValue(a)}else n(a,String)&&(d=a);if(null!=d&&0==d.length)return d;null==b&&(b=JU.Escape.uABsM(d));return n(b,JU.P3)?JS.SV.newV(8,b):n(b,JU.P4)?JS.SV.newV(9,b):n(b,JU.BS)?(null!=d&&0==d.indexOf("[{")&&(b=JM.BondSet.newBS(b,null)),JS.SV.newV(10,b)):n(b,JU.M34)?JS.SV.newV(n(b,JU.M3)?11:12,b):a},"~O");c$.getBoolean=c(c$,"getBoolean",function(a){return JS.SV.newT(a?JS.SV.vT:JS.SV.vF)},"~B");c$.sprintf=c(c$,"sprintf",function(a,b){if(null==b)return a;
var d=7==b.tok,c=0<=a.indexOf("d")||0<=a.indexOf("i")?B(1,0):null,f=0<=a.indexOf("f")?K(1,0):null,e=0<=a.indexOf("e")?X(1,0):null,h=0<=a.indexOf("s"),k=0<=a.indexOf("p")&&(d||8==b.tok),l=0<=a.indexOf("q")&&(d||9==b.tok),j=u(-1,[c,f,e,null,null,null]);if(!d)return JS.SV.sprintf(a,b,j,c,f,e,h,k,l);for(var d=b.getList(),m=Array(d.size()),r=0;r<m.length;r++)m[r]=JS.SV.sprintf(a,d.get(r),j,c,f,e,h,k,l);return m},"~S,JS.SV");c$.sprintf=c(c$,"sprintf",function(a,b,d,c,f,e,h,k,l){if(6==b.tok){var j=a.indexOf("[");
if(0<=j){var m;b=b.getMap().get(a.substring(j+1,m=a.indexOf("]")));a=a.substring(0,j)+a.substring(m+1)}}null!=c&&(c[0]=JS.SV.iValue(b));null!=f&&(f[0]=JS.SV.fValue(b));null!=e&&(e[0]=JS.SV.fValue(b));h&&(d[3]=JS.SV.sValue(b));k&&(d[4]=b.value);l&&(d[5]=b.value);return JU.PT.sprintf(a,"IFDspq",d)},"~S,JS.SV,~A,~A,~A,~A,~B,~B,~B");c$.getFormatType=c(c$,"getFormatType",function(a){return 0<=a.indexOf(";")?-1:";json;base64;bytearray;array;".indexOf(";"+a.toLowerCase()+";")},"~S");c$.format=c(c$,"format",
function(a,b){switch(a.length){case 0:return"";case 1:return JS.SV.sValue(a[0]);case 2:switch(2147483647==b&&(b=JS.SV.getFormatType(a[0].asString())),b){case 0:var d=a[1].myName;a[1].myName=null;var c=a[1].toJSON();a[1].myName=d;return c;case 5:case 12:case 22:var f;switch(a[1].tok){case 15:f=JU.AU.arrayCopyByte(a[1].value.data,-1);break;case 7:c=a[1].getList();if(22==b){f=new JU.Lst;for(d=c.size();0<=--d;)f.addLast(c.get(d));return f}f=Q(c.size(),0);for(d=f.length;0<=--d;)f[d]=c.get(d).asInt();break;
default:if(d=a[1].asString(),d.startsWith(";base64,")){if(5==b)return d;f=JU.Base64.decodeBase64(d)}else f=d.getBytes()}return 22==b?JS.SV.getVariable(f):12==b?new JU.BArray(f):";base64,"+JU.Base64.getBase64(f).toString()}}c=JU.PT.split(JU.PT.rep(JS.SV.sValue(a[0]),"%%","\u0001"),"%");if(0==c.length)return"";f=new JU.SB;f.append(c[0]);for(d=1;d<c.length;d++){var e=JS.SV.sprintf(JU.PT.formatCheck("%"+c[d]),6==a[1].tok?a[1]:7==a[1].tok?a[1].getList().get(d-1):d<a.length?a[d]:null);if(JU.AU.isAS(e))for(var h=
0;h<e.length;h++)f.append(e[h]).append("\n");else f.append(e)}return f.toString()},"~A,~N");c$.getBitSet=c(c$,"getBitSet",function(a,b){switch(a.tok){case 10:return(2147483647==a.index?JS.SV.selectItemTok(a,-2147483648):a).value;case 7:return JS.SV.unEscapeBitSetArray(a.getList(),b);default:return b?null:new JU.BS}},"JS.SV,~B");c$.unEscapeBitSetArray=c(c$,"unEscapeBitSetArray",function(a,b){for(var d=new JU.BS,c=0;c<a.size();c++){var f=a.get(c);if(2==f.tok&&0<=f.intValue)d.set(f.intValue);else if(7==
f.tok){f=JS.SV.unEscapeBitSetArray(f.getList(),!0);if(null==f)return b?null:new JU.BS;d.or(f)}else if(!JS.SV.unEscapeBitSet(f,d))return b?null:new JU.BS}return d},"JU.Lst,~B");c$.areEqual=c(c$,"areEqual",function(a,b){if(null==a||null==b)return!1;if(a.tok==b.tok)switch(a.tok){case 2:if(2==b.tok)return a.intValue==b.intValue;break;case 4:return a.value.equalsIgnoreCase(b.value);case 10:case 15:case 6:case 7:case 14:return a.equals(b);case 8:return 1E-6>a.value.distance(b.value);case 9:return 1E-6>
a.value.distance4(b.value);case 11:return a.value.equals(b.value);case 12:return a.value.equals(b.value)}return 1E-6>Math.abs(JS.SV.fValue(a)-JS.SV.fValue(b))},"JS.SV,JS.SV");c$.isLike=c(c$,"isLike",function(a,b){return null!=a&&null!=b&&4==a.tok&&4==b.tok&&JU.PT.isLike(a.value,b.value)},"JS.SV,JS.SV");c(c$,"sortOrReverse",function(a){var b=this.getList();if(null!=b&&1<b.size())if(-2147483648==a){a=b.size();for(var d=0;d<a;d++){var c=b.get(d);b.set(d,b.get(--a));b.set(a,c)}}else java.util.Collections.sort(this.getList(),
P(JS.SV.Sort,this,null,--a,null));return this},"~N");c(c$,"pushPop",function(a,b){if(null==a){var d=this.getMap();if(null==d){var c=this.getList();if(null==b||null==c)return null==c||0==c.size()?JS.SV.newS(""):c.removeItemAt(c.size()-1);c.addLast(JS.SV.newI(0).setv(b))}else null==b?d.clear():(c=b.getMap(),null!=c&&d.putAll(c))}else{d=this.getMap();if(null==b){c=null;if(null==d){var d=this.getList(),f=d.size(),e=JS.SV.iValue(a)-1;0>e&&(e+=f);0<=e&&e<f&&(c=d.removeItemAt(e))}else c=d.remove(a.asString());
return null==c?JS.SV.newS(""):c}null!=d&&d.put(a.asString(),JS.SV.newI(0).setv(b))}return this},"JS.SV,JS.SV");c$.unEscapeBitSet=c(c$,"unEscapeBitSet",function(a,b){switch(a.tok){case 4:var d=JU.BS.unescape(a.value);if(null==d)break;b.or(d);return!0;case 10:return b.or(a.value),!0}return!1},"JS.SV,JU.BS");c$.strListValue=c(c$,"strListValue",function(a){if(7!=a.tok)return u(-1,[JS.SV.sValue(a)]);a=a.getList();for(var b=Array(a.size()),d=a.size();0<=--d;)b[d]=JS.SV.sValue(a.get(d));return b},"JS.T");
c$.getArrayDepth=c(c$,"getArrayDepth",function(a){for(var b=0,d;7==a.tok&&0<(d=a.getList()).size();)b++,a=d.get(0);return b},"JS.T");c$.fflistValue=c(c$,"fflistValue",function(a){if(7!=a.tok)return u(-1,[K(-1,[JS.SV.fValue(a)])]);a=a.getList();var b=a.size(),d;d=JU.AU.newFloat2(b);for(var c=d.length;0<=--c;)d[c]=JS.SV.flistValue(c>=b?null:a.get(c),0);return d},"JS.T,~N");c$.flistValue=c(c$,"flistValue",function(a,b){if(null==a||7!=a.tok)return K(-1,[JS.SV.fValue(a)]);var d=a.getList(),c;c=K(Math.max(b,
d.size()),0);0==b&&(b=c.length);for(var f=Math.min(d.size(),b);0<=--f;)c[f]=JS.SV.fValue(d.get(f));return c},"JS.T,~N");c(c$,"toArray",function(){var a,b,d=null,c=null;switch(this.tok){case 11:d=this.value;a=3;break;case 12:c=this.value;a=4;break;case 7:return this;default:return b=new JU.Lst,b.addLast(this),JS.SV.newV(7,b)}b=new JU.Lst;for(var f=0;f<a;f++){var e=K(a,0);null==d?c.getRow(f,e):d.getRow(f,e);b.addLast(JS.SV.getVariableAF(e))}return JS.SV.newV(7,b)});c(c$,"mapValue",function(a){switch(this.tok){case 6:return this.value.get(a);
case 14:var b=this.value;return a.equals("_path")?JS.SV.newS(b.contextPath):b.getVariable(a)}return null},"~S");c(c$,"getList",function(){return 7==this.tok?this.value:null});c$.isScalar=c(c$,"isScalar",function(a){switch(a.tok){case 7:case 11:case 12:return!1;case 4:return 0>a.value.indexOf("\n");default:return!0}},"JS.SV");j(c$,"toJSON",function(){switch(this.tok){case 1073742335:case 1073742334:case 2:case 3:return JS.SV.sValue(this);case 15:return JU.PT.byteArrayToJSON(this.value.data);case 14:return JU.PT.toJSON(null,
this.value.getFullMap());case 7:case 6:if(null!=this.myName)return this.myName=null,6==this.tok?"{  }":"[  ]";this.myName="x";var a=JU.PT.toJSON(null,this.value);this.myName=null;return a;default:return JU.PT.toJSON(null,this.value)}});c(c$,"mapGet",function(a){return this.getMap().get(a)},"~S");c(c$,"mapPut",function(a,b){this.getMap().put(a,b)},"~S,JS.SV");c(c$,"getMap",function(){switch(this.tok){case 6:return this.value;case 14:return this.value.vars}return null});c(c$,"getMapKeys",function(a,
b){if(6!=this.tok)return"";var d=new JU.SB;JS.SV.sValueArray(d,this,"","",!0,!1,!1,a+1,b);return d.toString()},"~N,~B");j(c$,"toString",function(){return this.toString2()+"["+this.myName+" index ="+this.index+" intValue="+this.intValue+"]"});c(c$,"getKeys",function(a){switch(this.tok){case 6:case 14:case 7:break;default:return null}var b=new JU.Lst;this.getKeyList(a,b,"");a=b.toArray(Array(b.size()));java.util.Arrays.sort(a);return a},"~B");c(c$,"getKeyList",function(a,b,d){var c=this.getMap();if(null==
c){if(a){var f,e;null!=(f=this.getList())&&0<(e=f.size())&&f.get(e-1).getKeyList(!0,b,d+"."+e+".")}}else for(var h,c=c.entrySet().iterator();c.hasNext()&&((h=c.next())||1);){f=h.getKey();if(a&&(0==f.length||!JU.PT.isLetter(f.charAt(0))))d.endsWith(".")&&(d=d.substring(0,d.length-1)),f="["+JU.PT.esc(f)+"]";b.addLast(d+f);a&&h.getValue().getKeyList(!0,b,d+f+".")}},"~B,JU.Lst,~S");c$.deepCopy=c(c$,"deepCopy",function(a,b,d){if(b){b=new java.util.Hashtable;var c;for(a=a.entrySet().iterator();a.hasNext()&&
((c=a.next())||1);){var f=c.getValue();b.put(c.getKey(),d?JS.SV.deepCopySV(f):f)}return b}c=new JU.Lst;b=0;for(f=a.size();b<f;b++){var e=a.get(b);c.addLast(d?JS.SV.deepCopySV(e):e)}return c},"~O,~B,~B");c$.deepCopySV=c(c$,"deepCopySV",function(a){switch(a.tok){case 6:case 7:if("\r".equals(a.myName))a.myName=null,a=JS.SV.newV(a.tok,6==a.tok?new java.util.Hashtable:new JU.Lst);else{var b=a.myName;a.myName="\r";var d=a;a=JS.SV.newV(a.tok,JS.SV.deepCopy(a.value,6==a.tok,!0));d.myName=b}}return a},"JS.SV");
c(c$,"sortMapArray",function(a){null!=this.getList()&&java.util.Collections.sort(this.getList(),P(JS.SV.Sort,this,null,0,a));return this},"~S");c$.safeJSON=c(c$,"safeJSON",function(a,b){return"{"+(n(b,JS.SV)?JU.PT.esc(a)+" : "+JS.SV.format(u(-1,[null,b]),0):JU.PT.toJSON(a,b))+"}"},"~S,~O");c$.$SV$Sort$=function(){M(self.c$);c$=t(function(){V(this,arguments);this.arrayPt=0;this.myKey=null;p(this,arguments)},JS.SV,"Sort",null,java.util.Comparator);q(c$,function(a,b){this.arrayPt=a;this.myKey=b},"~N,~S");
j(c$,"compare",function(a,b){if(a.tok!=b.tok){if(3==a.tok||2==a.tok||3==b.tok||2==b.tok){var d=JS.SV.fValue(a),c=JS.SV.fValue(b);return d<c?-1:d>c?1:0}if(4==a.tok||4==b.tok)return JS.SV.sValue(a).compareTo(JS.SV.sValue(b))}switch(a.tok){case 2:return a.intValue<b.intValue?-1:a.intValue>b.intValue?1:0;case 4:return JS.SV.sValue(a).compareTo(JS.SV.sValue(b));case 7:d=a.getList();c=b.getList();if(d.size()!=c.size())return d.size()<c.size()?-1:1;var f=this.arrayPt;0>f&&(f+=d.size());return 0>f||f>=d.size()?
0:this.compare(d.get(f),c.get(f));case 6:if(null!=this.myKey)return this.compare(a.getMap().get(this.myKey),b.getMap().get(this.myKey));default:return d=JS.SV.fValue(a),c=JS.SV.fValue(b),d<c?-1:d>c?1:0}},"JS.SV,JS.SV");c$=L()};c$.vT=c$.prototype.vT=JS.SV.newSV(1073742335,1,"true");c$.vF=c$.prototype.vF=JS.SV.newSV(1073742334,0,"false");c$.pt0=c$.prototype.pt0=new JU.P3});s("JS");v(["java.util.Hashtable"],"JS.T",["java.lang.Boolean","java.util.Arrays","JU.AU","$.Lst","JU.Logger"],function(){c$=t(function(){this.tok=
0;this.value=null;this.intValue=2147483647;p(this,arguments)},JS,"T");c$.t=c(c$,"t",function(a){var b=new JS.T;b.tok=a;return b},"~N");c$.tv=c(c$,"tv",function(a,b,d){a=JS.T.t(a);a.intValue=b;a.value=d;return a},"~N,~N,~O");c$.o=c(c$,"o",function(a,b){var d=JS.T.t(a);d.value=b;return d},"~N,~O");c$.n=c(c$,"n",function(a,b){var d=JS.T.t(a);d.intValue=b;return d},"~N,~N");c$.i=c(c$,"i",function(a){var b=JS.T.t(2);b.intValue=a;return b},"~N");c$.tokAttr=c(c$,"tokAttr",function(a,b){return(a&b)==(b&b)},
"~N,~N");c$.tokAttrOr=c(c$,"tokAttrOr",function(a,b,d){return(a&b)==(b&b)||(a&d)==(d&d)},"~N,~N,~N");c$.getPrecedence=c(c$,"getPrecedence",function(a){return a>>4&15},"~N");c$.getMaxMathParams=c(c$,"getMaxMathParams",function(a){return a>>9&3},"~N");c$.addToken=c(c$,"addToken",function(a,b){JS.T.tokenMap.put(a,b)},"~S,JS.T");c$.getTokenFromName=c(c$,"getTokenFromName",function(a){return JS.T.tokenMap.get(a)},"~S");c$.getTokFromName=c(c$,"getTokFromName",function(a){a=JS.T.getTokenFromName(a.toLowerCase());
return null==a?0:a.tok},"~S");c$.nameOf=c(c$,"nameOf",function(a){for(var b,d=JS.T.tokenMap.values().iterator();d.hasNext()&&((b=d.next())||1);)if(b.tok==a)return""+b.value;return"0x"+Integer.toHexString(a)},"~N");j(c$,"toString",function(){return this.toString2()});c(c$,"toString2",function(){return"Token["+JS.T.astrType[16>this.tok?this.tok:16]+"("+this.tok%512+"/0x"+Integer.toHexString(this.tok)+")"+(2147483647==this.intValue?"":" intValue="+this.intValue+"(0x"+Integer.toHexString(this.intValue)+
")")+(null==this.value?"":n(this.value,String)?' value="'+this.value+'"':" value="+this.value)+"]"});c$.getCommandSet=c(c$,"getCommandSet",function(a){var b="",d=new java.util.Hashtable,c=0;a=null==a||0==a.length?null:a.toLowerCase();for(var g=null!=a&&1<a.length,e,f=JS.T.tokenMap.entrySet().iterator();f.hasNext()&&((e=f.next())||1);){var h=e.getKey(),k=e.getValue();0!=(k.tok&4096)&&((null==a||0==h.indexOf(a))&&(g||k.value.equals(h)))&&d.put(h,Boolean.TRUE)}for(f=d.entrySet().iterator();f.hasNext()&&
((e=f.next())||1);)if(h=e.getKey(),"s"!=h.charAt(h.length-1)||!d.containsKey(h.substring(0,h.length-1)))b+=(0==c++?"":";")+h;return b},"~S");c$.getAtomPropertiesLike=c(c$,"getAtomPropertiesLike",function(a){a=a.toLowerCase();for(var b=new JU.Lst,d=0==a.length,c,g=JS.T.tokenMap.entrySet().iterator();g.hasNext()&&((c=g.next())||1);){var e=c.getKey();if("_"!=e.charAt(0)){var f=c.getValue();if(JS.T.tokAttr(f.tok,1077936128)&&(d||e.toLowerCase().startsWith(a))){if(d||!f.value.toLowerCase().startsWith(a))f=
JS.T.o(f.tok,e);b.addLast(f)}}}return 0==b.size()?null:b},"~S");c$.getTokensLike=c(c$,"getTokensLike",function(a){a=a.equals("setparam")?536870912:a.equals("misc")?1073741824:a.equals("mathfunc")?134217728:4096;for(var b=536870912==a?1610612736:0,d=new JU.Lst,c,g=JS.T.tokenMap.entrySet().iterator();g.hasNext()&&((c=g.next())||1);){var e=c.getKey(),f=c.getValue();JS.T.tokAttr(f.tok,a)&&(0==b||!JS.T.tokAttr(f.tok,b))&&d.addLast(e)}c=d.toArray(Array(d.size()));java.util.Arrays.sort(c);return c},"~S");
c$.getSettableTokFromString=c(c$,"getSettableTokFromString",function(a){a=JS.T.getTokFromName(a);return 0!=a&&JS.T.tokAttr(a,2048)&&!JS.T.tokAttr(a,1140850688)?a:0},"~S");c$.completeCommand=c(c$,"completeCommand",function(a,b,d,c,g){null==a?a=JS.T.tokenMap:d=!1;var e=new JU.Lst;c=c.toLowerCase();var f;for(a=a.keySet().iterator();a.hasNext()&&((f=a.next())||1);)if(f.startsWith(c)){var h=JS.T.getTokFromName(f);(d?JS.T.tokAttr(h,4096):b?JS.T.tokAttr(h,536870912)&&!JS.T.tokAttr(h,1610612736):1)&&e.addLast(f)}return JU.AU.sortedItem(e,
g)},"java.util.Map,~B,~B,~S,~N");c$.getParamType=c(c$,"getParamType",function(a){return!JS.T.tokAttr(a,536870912)?0:a&662700032},"~N");c$.getTokensType=c(c$,"getTokensType",function(a,b){for(var d,c=JS.T.tokenMap.entrySet().iterator();c.hasNext()&&((d=c.next())||1);){var g=d.getValue();JS.T.tokAttr(g.tok,b)&&a.put(d.getKey(),d.getValue())}},"java.util.Map,~N");c$.isIDcmd=c(c$,"isIDcmd",function(a){switch(a){case 135180:case 135176:case 135174:case 135188:case 134353926:return!0;default:return!1}},
"~N");c(c$,"equals",function(a){if(!n(a,JS.T))return!1;if(this.tok==a.tok)return a.intValue==this.intValue&&(2==this.tok||1073742335==this.tok||1073742334==this.tok||a.value.equals(this.value));switch(this.tok){case 2:return 3==a.tok&&a.value.floatValue()==this.intValue;case 3:return 2==a.tok&&this.value.floatValue()==a.intValue;default:return!1}},"~O");E(c$,"astrType",null);JS.T.astrType="nada identifier integer decimal string inscode hash array point point4 bitset matrix3f matrix4f array hash bytearray keyword".$plit(" ");
E(c$);c$.tokenSpaceBeforeSquare=c$.prototype.tokenSpaceBeforeSquare=JS.T.o(1073742195," ");c$.tokenOn=
c$.prototype.tokenOn=JS.T.tv(1073742335,1,"on");c$.tokenOff=c$.prototype.tokenOff=JS.T.tv(1073742334,0,"off");c$.tokenAll=c$.prototype.tokenAll=JS.T.o(1073742327,"all");c$.tokenIf=c$.prototype.tokenIf=JS.T.o(134320649,"if");c$.tokenAnd=c$.prototype.tokenAnd=JS.T.o(268435552,"and");c$.tokenAndSpec=c$.prototype.tokenAndSpec=JS.T.o(268435584,"");c$.tokenOr=c$.prototype.tokenOr=JS.T.o(268435536,"or");c$.tokenAndFALSE=c$.prototype.tokenAndFALSE=JS.T.o(268435552,"and");c$.tokenOrTRUE=c$.prototype.tokenOrTRUE=
JS.T.o(268435536,"or");c$.tokenOpIf=c$.prototype.tokenOpIf=JS.T.o(805306401,"?");c$.tokenComma=c$.prototype.tokenComma=JS.T.o(268435504,",");c$.tokenDefineString=c$.prototype.tokenDefineString=JS.T.tv(12290,4,"@");c$.tokenPlus=c$.prototype.tokenPlus=JS.T.o(268435617,"+");c$.tokenMinus=c$.prototype.tokenMinus=JS.T.o(268435616,"-");c$.tokenMul3=c$.prototype.tokenMul3=JS.T.o(1275068930,"mul3");c$.tokenTimes=c$.prototype.tokenTimes=JS.T.o(268435633,"*");c$.tokenDivide=c$.prototype.tokenDivide=JS.T.o(268435632,
"/");c$.tokenLeftParen=c$.prototype.tokenLeftParen=JS.T.o(268435472,"(");c$.tokenRightParen=c$.prototype.tokenRightParen=JS.T.o(268435473,")");c$.tokenArraySquare=c$.prototype.tokenArraySquare=JS.T.o(1275068418,"[");c$.tokenArrayOpen=c$.prototype.tokenArrayOpen=JS.T.o(268435520,"[");c$.tokenArrayClose=c$.prototype.tokenArrayClose=JS.T.o(268435521,"]");c$.tokenLeftBrace=c$.prototype.tokenLeftBrace=JS.T.o(1073742332,"{");c$.tokenExpressionBegin=c$.prototype.tokenExpressionBegin=JS.T.o(1073742325,"expressionBegin");
c$.tokenExpressionEnd=c$.prototype.tokenExpressionEnd=JS.T.o(1073742326,"expressionEnd");c$.tokenConnected=c$.prototype.tokenConnected=JS.T.o(134217736,"connected");c$.tokenCoordinateBegin=c$.prototype.tokenCoordinateBegin=JS.T.o(1073742332,"{");c$.tokenRightBrace=c$.prototype.tokenRightBrace=JS.T.o(1073742338,"}");c$.tokenCoordinateEnd=c$.prototype.tokenCoordinateEnd=JS.T.tokenRightBrace;c$.tokenColon=c$.prototype.tokenColon=JS.T.o(268435490,":");c$.tokenSetCmd=c$.prototype.tokenSetCmd=JS.T.o(36867,
"set");c$.tokenSet=c$.prototype.tokenSet=JS.T.tv(36867,61,"");c$.tokenSetArray=c$.prototype.tokenSetArray=JS.T.tv(36867,91,"");c$.tokenSetProperty=c$.prototype.tokenSetProperty=JS.T.tv(36867,46,"");c$.tokenSetVar=c$.prototype.tokenSetVar=JS.T.tv(36868,61,"var");c$.tokenEquals=c$.prototype.tokenEquals=JS.T.o(268435860,"=");c$.tokenScript=c$.prototype.tokenScript=JS.T.o(134222850,"script");c$.tokenSwitch=c$.prototype.tokenSwitch=JS.T.o(102410,"switch");c$.tokenMap=c$.prototype.tokenMap=new java.util.Hashtable;
for(var a=u(-1,["(",JS.T.tokenLeftParen,")",JS.T.tokenRightParen,"and",JS.T.tokenAnd,"&",null,"&&",null,"or",JS.T.tokenOr,"|",null,"||",null,"?",JS.T.tokenOpIf,",",JS.T.tokenComma,"=",JS.T.tokenEquals,"==",null,":",JS.T.tokenColon,"+",JS.T.tokenPlus,"-",JS.T.tokenMinus,"*",JS.T.tokenTimes,"/",JS.T.tokenDivide,"script",JS.T.tokenScript,"source",null,"set",JS.T.tokenSetCmd,"switch",JS.T.tokenSwitch,"all",JS.T.tokenAll,"off",JS.T.tokenOff,"false",null,"on",JS.T.tokenOn,"true",null]),b,d=null,g,f,e=a.length-
1,h=0;h<e;h+=2)g=a[h],f=g.toLowerCase(),b=a[h+1],null==b?b=d:null==b.value&&(b.value=g),JS.T.tokenMap.put(f,b),d=b;var a=null,a=u(-1,"+= -= *= /= \\= &= |= not ! xor tog < <= >= > != <> LIKE within . .. [ ] { } $ % ; ++ -- ** \\ animation anim assign axes backbone background bind bondorder boundbox boundingBox break calculate capture cartoon cartoons case catch cd center centre centerat cgo color colour compare configuration conformation config connect console contact contacts continue data default define @ delay delete density depth dipole dipoles display dot dots draw echo ellipsoid ellipsoids else elseif end endif exit eval file files font for format frame frames frank function functions geosurface getProperty goto halo halos helix helixalpha helix310 helixpi hbond hbonds help hide history hover if in initialize invertSelected isosurface javascript label labels lcaoCartoon lcaoCartoons load log loop measure measures monitor monitors meshribbon meshribbons message minimize minimization mo model models modulation move moveTo mutate navigate navigation nbo origin out parallel pause wait plot private plot3d pmesh polygon polyhedra polyhedron print process prompt quaternion quaternions quit ramachandran rama refresh reset unset restore restrict return ribbon ribbons rocket rockets rotate rotateSelected save select selectionHalos selectionHalo showSelections sheet show slab spacefill cpk spin ssbond ssbonds star stars step steps stereo strand strands structure _structure strucNo struts strut subset subZystem synchronize sync trace translate translateSelected try unbind unitcell var vector vectors vibration while wireframe write zap zoom zoomTo atom atoms axisangle basepair basepairs orientation orientations pdbheader polymer polymers residue residues rotation row sequence seqcode shape state symbol symmetry spaceGroup transform translation url _ abs absolute acos add adpmax adpmin align altloc altlocs ambientOcclusion amino angle array as atomID _atomID _a atomIndex atomName atomno atomType atomX atomY atomZ average babel babel21 back backlit backshell balls baseModel best beta bin bondCount bonded bottom branch brillouin bzone wignerSeitz cache carbohydrate cell chain chains chainNo chemicalShift cs clash clear clickable clipboard connected context constraint contourLines coord coordinates coords cos cross covalentRadius covalent direction displacement displayed distance div DNA domains dotted DSSP DSSR element elemno _e error exportScale fill find fixedTemperature forcefield formalCharge charge eta front frontlit frontOnly fullylit fx fy fz fxyz fux fuy fuz fuxyz group groups group1 groupID _groupID _g groupIndex hidden highlight hkl hydrophobicity hydrophobic hydro id identify ident image info infoFontSize inline insertion insertions intramolecular intra intermolecular inter bondingRadius ionicRadius ionic isAromatic Jmol JSON join keys last left length lines list magneticShielding ms mass max mep mesh middle min mlp mode modify modifyOrCreate modt modt1 modt2 modt3 modx mody modz modo modxyz molecule molecules modelIndex monomer morph movie mouse mul mul3 nboCharges nci next noDelay noDots noFill noMesh none null inherit normal noBackshell noContourLines notFrontOnly noTriangles now nucleic occupancy omega only opaque options partialCharge phi pivot plane planar play playRev point points pointGroup polymerLength pop previous prev probe property properties protein psi purine push PyMOL pyrimidine random range rasmol replace resno resume rewind reverse right rmsd RNA rna3d rock rubberband saSurface saved scale scene search smarts selected seqid shapely sidechain sin site size smiles substructure solid sort specialPosition sqrt split starWidth starScale stddev straightness structureId supercell sub sum sum2 surface surfaceDistance symop sx sy sz sxyz temperature relativeTemperature tensor theta thisModel ticks top torsion trajectory trajectories translucent transparent triangles trim type ux uy uz uxyz user valence vanderWaals vdw vdwRadius visible volume vx vy vz vxyz xyz w x y z addHydrogens allConnected angstroms anisotropy append arc area aromatic arrow async audio auto axis barb binary blockData cancel cap cavity centroid check checkCIR chemical circle collapsed col colorScheme command commands contour contours corners count criterion create crossed curve cutoff cylinder diameter discrete distanceFactor downsample drawing dynamicMeasurements eccentricity ed edges edgesOnly energy exitJmol faceCenterOffset filter first fixed fix flat fps from frontEdges full fullPlane functionXY functionXYZ gridPoints hiddenLinesDashed homo ignore InChI InChIKey increment insideout interior intersection intersect internal lattice line lineData link lobe lonePair lp lumo macro manifest mapProperty maxSet menu minSet modelBased molecular mrc msms name nmr noCross noDebug noEdges noHead noLoad noPlane object obj offset offsetSide once orbital atomicOrbital packed palindrome parameters path pdb period periodic perpendicular perp phase planarParam pocket pointsPerAngstrom radical rad reference remove resolution reverseColor rotate45 selection sigma sign silent sphere squared stdInChI stdInChIKey stop title titleFormat to validation value variable variables vertices width wigner backgroundModel celShading celShadingPower debug debugHigh defaultLattice measurements measurement scale3D toggleLabel userColorScheme throw timeout timeouts window animationMode appletProxy atomTypes axesColor axis1Color axis2Color axis3Color backgroundColor bondmode boundBoxColor boundingBoxColor chirality cipRule currentLocalPath dataSeparator defaultAngleLabel defaultColorScheme defaultColors defaultDirectory defaultDistanceLabel defaultDropScript defaultLabelPDB defaultLabelXYZ defaultLoadFilter defaultLoadScript defaults defaultTorsionLabel defaultVDW drawFontSize eds edsDiff energyUnits fileCacheDirectory fontsize helpPath hoverLabel language loadFormat loadLigandFormat logFile measurementUnits nihResolverFormat nmrPredictFormat nmrUrlFormat pathForAllFiles picking pickingStyle pickLabel platformSpeed propertyColorScheme quaternionFrame smilesUrlFormat smiles2dImageFormat unitCellColor axesOffset axisOffset axesScale axisScale bondTolerance cameraDepth defaultDrawArrowScale defaultTranslucent dipoleScale ellipsoidAxisDiameter gestureSwipeFactor hbondsAngleMinimum hbondHXDistanceMaximum hbondsDistanceMaximum hbondNODistanceMaximum hoverDelay loadAtomDataTolerance minBondDistance minimizationCriterion minimizationMaxAtoms modulationScale mouseDragFactor mouseWheelFactor navFPS navigationDepth navigationSlab navigationSpeed navX navY navZ particleRadius pointGroupDistanceTolerance pointGroupLinearTolerance radius rotationRadius scaleAngstromsPerInch sheetSmoothing slabRange solventProbeRadius spinFPS spinX spinY spinZ stereoDegrees strutDefaultRadius strutLengthMaximum vectorScale vectorsCentered vectorSymmetry vectorTrail vibrationPeriod vibrationScale visualRange ambientPercent ambient animationFps axesMode bondRadiusMilliAngstroms bondingVersion delayMaximumMs diffusePercent diffuse dotDensity dotScale ellipsoidDotCount helixStep hermiteLevel historyLevel lighting logLevel meshScale minimizationSteps minPixelSelRadius percentVdwAtom perspectiveModel phongExponent pickingSpinRate propertyAtomNumberField propertyAtomNumberColumnCount propertyDataColumnCount propertyDataField repaintWaitMs ribbonAspectRatio contextDepthMax scriptReportingLevel showScript smallMoleculeMaxAtoms specular specularExponent specularPercent specPercent specularPower specpower strandCount strandCountForMeshRibbon strandCountForStrands strutSpacing zDepth zSlab zshadePower allowEmbeddedScripts allowGestures allowKeyStrokes allowModelKit allowMoveAtoms allowMultiTouch allowRotateSelected antialiasDisplay antialiasImages antialiasTranslucent appendNew applySymmetryToBonds atomPicking allowAudio autobond autoFPS autoplayMovie axesMolecular axesOrientationRasmol axesUnitCell axesWindow bondModeOr bondPicking bonds bond cartoonBaseEdges cartoonBlocks cartoonBlockHeight cartoonsFancy cartoonFancy cartoonLadders cartoonRibose cartoonRockets cartoonSteps chainCaseSensitive cipRule6Full colorRasmol debugScript defaultStructureDssp disablePopupMenu displayCellParameters showUnitcellInfo dotsSelectedOnly dotSurface dragSelected drawHover drawPicking dsspCalculateHydrogenAlways ellipsoidArcs ellipsoidArrows ellipsoidAxes ellipsoidBall ellipsoidDots ellipsoidFill fileCaching fontCaching fontScaling forceAutoBond fractionalRelative greyscaleRendering hbondsBackbone hbondsRasmol hbondsSolid hetero hideNameInPopup hideNavigationPoint hideNotSelected highResolution hydrogen hydrogens imageState isKiosk isosurfaceKey isosurfacePropertySmoothing isosurfacePropertySmoothingPower jmolInJSpecView justifyMeasurements languageTranslation leadAtom leadAtoms legacyAutoBonding legacyHAddition legacyJavaFloat logCommands logGestures macroDirectory measureAllModels measurementLabels measurementNumbers messageStyleChime minimizationRefresh minimizationSilent modelkitMode modelkit modulateOccupancy monitorEnergy multiplebondbananas multipleBondRadiusFactor multipleBondSpacing multiProcessor navigateSurface navigationMode navigationPeriodic partialDots pdbAddHydrogens pdbGetHeader pdbSequential perspectiveDepth preserveState rangeSelected redoMove refreshing ribbonBorder rocketBarrels saveProteinStructureState scriptQueue selectAllModels selectHetero selectHydrogen showAxes showBoundBox showBoundingBox showFrank showHiddenSelectionHalos showHydrogens showKeyStrokes showMeasurements showModulationVectors showMultipleBonds showNavigationPointAlways showTiming showUnitcell showUnitcellDetails slabByAtom slabByMolecule slabEnabled smartAromatic solvent solventProbe ssBondsBackbone statusReporting strutsMultiple syncMouse syncScript testFlag1 testFlag2 testFlag3 testFlag4 traceAlpha twistedSheets undo undoMove useMinimizationThread useNumberLocalization waitForMoveTo windowCentered wireframeRotation zeroBasedXyzRasmol zoomEnabled zoomHeight zoomLarge zShade".split(" ")),
k=B(-1,[268435666,-1,-1,-1,-1,-1,-1,268435568,-1,268435537,268435538,268435859,268435858,268435857,268435856,268435861,-1,268435862,134217759,1073742336,1073742337,268435520,268435521,1073742332,1073742338,1073742330,268435634,1073742339,268435650,268435649,268435651,268435635,4097,-1,4098,1611272194,1114249217,1610616835,4100,4101,1678381065,-1,102407,4102,4103,1112152066,-1,102411,102412,20488,12289,-1,4105,135174,1765808134,-1,134221831,1094717448,-1,-1,4106,528395,134353926,-1,102408,134221834,
102413,12290,-1,528397,12291,1073741914,554176526,135175,-1,1610625028,1275069444,1112150019,135176,537022465,1112150020,-1,364547,102402,102409,364548,266255,134218759,1228935687,-1,4114,134320648,1287653388,4115,-1,1611272202,134320141,-1,1112150021,1275072526,20500,1112152070,-1,136314895,2097159,2097160,2097162,1613238294,-1,20482,12294,1610616855,544771,134320649,1275068432,4121,4122,135180,134238732,1825200146,-1,135182,-1,134222849,36869,528411,1745489939,-1,-1,-1,1112152071,-1,20485,4126,
-1,1073877010,1094717454,-1,1275072532,4128,4129,4130,4131,-1,1073877011,1073742078,1073742079,102436,20487,-1,4133,4134,135190,135188,1073742106,1275203608,-1,36865,102439,134256131,134221850,-1,266281,4138,-1,266284,4141,-1,4142,12295,36866,1112152073,-1,1112152074,-1,528432,4145,4146,1275082245,1611141171,-1,-1,2097184,134222350,554176565,1112152075,-1,1611141175,1611141176,-1,1112152076,-1,266298,-1,528443,1649022989,-1,1639976963,-1,1094713367,659482,-1,2109448,1094713368,4156,-1,1112152078,
4160,4162,364558,4164,1814695966,36868,135198,-1,4166,102406,659488,134221856,12297,4168,4170,1140850689,-1,134217731,1073741863,-1,1073742077,-1,1073742088,1094713362,-1,1073742120,-1,1073742132,1275068935,1086324744,1086324747,1086324748,1073742158,1086326798,1088421903,134217764,1073742176,1073742178,1073742184,1275068449,134218250,1073741826,134218242,1275069441,1111490561,1111490562,1073741832,1086324739,-1,553648129,2097154,134217729,1275068418,1073741848,1094713346,-1,-1,1094713347,1086326786,
1094715393,1086326785,1111492609,1111492610,1111492611,96,1073741856,1073741857,1073741858,1073741861,1073741862,1073741859,2097200,1073741864,1073741865,1275068420,1228931587,2097155,1073741871,1073742328,1073741872,-1,-1,134221829,2097188,1094713349,1086326788,-1,1094713351,1111490563,-1,1073741881,1073741882,2097190,1073741884,134217736,14,1073741894,1073741898,1073742329,-1,-1,134218245,1275069442,1111490564,-1,1073741918,1073741922,2097192,1275069443,1275068928,2097156,1073741925,1073741926,
1073741915,1111490587,1086326789,1094715402,1094713353,1073741936,570425357,1073741938,1275068427,1073741946,545259560,1631586315,-1,1111490565,1073741954,1073741958,1073741960,1073741964,1111492612,1111492613,1111492614,1145047051,1111492615,1111492616,1111492617,1145047053,1086324742,-1,1086324743,1094713356,-1,-1,1094713357,2097194,536870920,134219265,1113589786,-1,-1,1073741974,1086324745,-1,4120,1073741982,553648147,1073741984,1086324746,-1,1073741989,-1,1073741990,-1,1111492618,-1,-1,1073742331,
1073741991,1073741992,1275069446,1140850706,1073741993,1073741996,1140850691,1140850692,1073742001,1111490566,-1,1111490567,64,1073742016,1073742018,1073742019,32,1073742022,1073742024,1073742025,1073742026,1111490580,-1,1111490581,1111490582,1111490583,1111490584,1111490585,1111490586,1145045008,1094713360,-1,1094713359,1094713361,1073742029,1073742031,1073742030,1275068929,1275068930,603979891,1073742036,1073742037,603979892,1073742042,1073742046,1073742052,1073742333,-1,-1,1073742056,1073742057,
1073742039,1073742058,1073742060,134218760,2097166,1128269825,1111490568,1073742072,1073742074,1073742075,1111492619,1111490569,1275068437,134217750,-1,1073742096,1073742098,134217751,-1,134217762,1094713363,1275334681,1073742108,-1,1073742109,1715472409,-1,2097168,1111490570,2097170,1275335685,1073742110,2097172,134219268,1073742114,1073742116,1275068443,1094715412,4143,1073742125,1140850693,1073742126,1073742127,2097174,1073742128,1073742129,1073742134,1073742135,1073742136,1073742138,1073742139,
134218756,-1,1113589787,1094713365,1073742144,2097178,134218244,1094713366,1140850694,134218757,1237320707,1073742150,1275068444,2097196,134218246,1275069447,570425403,-1,192,1111490574,1086324749,1073742163,1275068931,128,160,2097180,1111490575,1296041986,1111490571,1111490572,1111490573,1145047052,1111492620,-1,1275068445,1111490576,2097182,1073742164,1073742172,1073742174,536870926,-1,603979967,-1,1073742182,1275068932,1140850696,1111490577,1111490578,1111490579,1145045006,1073742186,1094715417,
1648363544,-1,-1,2097198,1312817669,1111492626,1111492627,1111492628,1145047055,1145047050,1140850705,1111492629,1111492630,1111492631,1073741828,1073741834,1073741836,1073741837,1073741839,1073741840,1073741842,1075838996,1073741846,1073741849,1073741851,1073741852,1073741854,1073741860,1073741866,1073741868,1073741874,1073741875,1073741876,1094713350,1073741877,603979821,1073741879,1073741880,1073741886,1275068934,1073741888,1073741890,1073741892,1073741896,1073741900,1073741902,1275068425,1073741905,
1073741904,1073741906,1073741908,1073741910,1073741912,1073741917,1073741920,1073741924,1073741928,1073741929,603979835,1073741931,1073741932,1073741933,1073741934,1073741935,266256,1073741937,1073741940,1073741942,12293,-1,1073741948,1073741950,1073741952,1073741956,1073741961,1073741962,1073741966,1073741968,1073741970,603979856,1073741973,1073741976,1275068433,1073741978,1073741981,1073741985,1073741986,134217763,-1,1073741988,1073741994,1073741998,1073742E3,1073741999,1073742002,1073742004,1073742006,
1073742008,4124,1073742010,4125,1073742014,1073742015,1073742020,1073742027,1073742028,1073742032,1073742033,1073742034,1073742038,1073742040,1073742041,1073742044,1073742048,1073742050,1073742054,1073742064,1073742062,1073742066,1073742068,1073742070,1073742076,1073741850,1073742080,1073742082,1073742083,1073742084,1073742086,1073742090,-1,1073742092,-1,1073742094,1073742099,1073742100,1073742104,1073742112,1073742111,1073742118,1073742119,1073742122,1073742124,1073742130,1073742140,1073742146,1073742147,
1073742148,1073742154,1073742156,1073742159,1073742160,1073742162,1073742166,1073742168,1073742170,1073742189,1073742188,1073742190,1073742192,1073742194,1073742196,1073742197,536870914,603979820,553648137,536870916,536870917,536870918,537006096,-1,1610612740,1610612741,536870930,36870,536875070,-1,536870932,545259521,545259522,545259524,545259526,545259528,545259530,545259532,545259534,1610612737,545259536,-1,1086324752,1086324753,545259538,545259540,545259542,545259545,-1,545259546,545259547,545259548,
545259543,545259544,545259549,545259550,545259552,545259554,545259555,570425355,545259556,545259557,545259558,545259559,1610612738,545259561,545259562,545259563,545259564,545259565,545259566,545259568,545259570,545259569,545259571,545259572,545259573,545259574,545259576,553648158,545259578,545259580,545259582,545259584,545259586,570425345,-1,570425346,-1,570425348,570425350,570425352,570425353,570425354,570425356,570425358,570425359,570425361,570425360,-1,570425362,570425363,570425364,570425365,553648152,
570425366,570425367,570425368,570425371,570425372,570425373,570425374,570425376,570425378,570425380,570425381,570425382,570425384,1665140738,570425388,570425390,570425392,570425393,570425394,570425396,570425398,570425400,570425402,570425404,570425406,570425408,1648361473,603979972,603979973,553648185,570425412,570425414,570425416,553648130,-1,553648132,553648134,553648136,553648138,553648139,553648140,-1,553648141,553648142,553648143,553648144,553648145,553648146,1073741995,553648149,553648150,553648151,
553648153,553648154,553648155,553648156,553648157,553648159,553648160,553648162,553648164,553648165,553648166,553648167,553648168,536870922,553648170,536870924,553648172,553648174,-1,553648176,-1,553648178,553648180,553648182,553648184,553648186,553648188,553648190,603979778,603979780,603979781,603979782,603979783,603979784,603979785,603979786,603979788,603979790,603979792,603979794,603979796,603979797,603979798,603979800,603979802,603979804,603979806,603979808,603979809,603979812,603979814,1677721602,
-1,603979815,603979810,570425347,603979816,-1,603979817,603979818,603979819,603979811,603979822,603979823,603979824,603979825,603979826,603979827,603979828,-1,603979829,603979830,603979831,603979832,603979833,603979834,603979836,603979837,603979838,603979839,603979840,603979841,603979842,603979844,603979845,603979846,603979848,603979850,603979852,603979853,603979854,1612709894,603979858,603979860,603979862,603979864,1612709900,-1,603979865,603979866,603979867,603979868,553648148,603979869,603979870,
603979871,2097165,-1,603979872,603979873,603979874,603979875,603979876,545259567,603979877,603979878,1610612739,603979879,603979880,603979881,603983903,-1,603979884,603979885,603979886,570425369,570425370,603979887,603979888,603979889,603979890,603979893,603979894,603979895,603979896,603979897,603979898,603979899,4139,603979900,603979901,603979902,603979903,603979904,603979906,603979908,603979910,603979914,603979916,-1,603979918,603979920,603979922,603979924,603979926,603979927,603979928,603979930,
603979934,603979936,603979937,603979939,603979940,603979942,603979944,1612709912,603979948,603979952,603979954,603979955,603979956,603979958,603979960,603979962,603979964,603979965,603979966,603979968,536870928,4165,603979970,603979971,603979975,603979976,603979977,603979978,603979980,603979982,603979983,603979984]);a.length!=k.length&&(JU.Logger.error("sTokens.length ("+a.length+") != iTokens.length! ("+k.length+")"),Zystem.exit(1));e=a.length;for(h=0;h<e;h++)g=a[h],f=g.toLowerCase(),b=k[h],b=d=
-1==b?d:JS.T.o(b,g),null!=JS.T.tokenMap.get(f)&&JU.Logger.error("duplicate token definition:"+f),JS.T.tokenMap.put(f,b);k=a=null});s("J.shape");v(["J.shape.Shape"],"J.shape.AtomShape","JU.AU $.BS J.atomdata.RadiusData J.c.PAL $.VDW JU.BSUtil $.C".split(" "),function(){c$=t(function(){this.mad=-1;this.paletteIDs=this.colixes=this.mads=null;this.ac=0;this.atoms=null;this.isActive=!1;this.monomerCount=0;this.rd=this.bsSizeDefault=null;p(this,arguments)},J.shape,"AtomShape",J.shape.Shape);j(c$,"initShape",
function(){});j(c$,"initModelSet",function(){this.atoms=this.ms.at;this.ac=this.ms.ac;null!=this.mads&&(this.mads=JU.AU.arrayCopyShort(this.mads,this.ac));null!=this.colixes&&(this.colixes=JU.AU.arrayCopyShort(this.colixes,this.ac));null!=this.paletteIDs&&(this.paletteIDs=JU.AU.arrayCopyByte(this.paletteIDs,this.ac))});j(c$,"getSize",function(a){return null==this.mads?0:this.mads[a]},"~N");j(c$,"setSize",function(a,b){this.setSize2(a,b)},"~N,JU.BS");c(c$,"setSize2",function(a,b){0==a?this.setSizeRD(null,
b):(null==this.rd?this.rd=new J.atomdata.RadiusData(null,a,J.atomdata.RadiusData.EnumType.SCREEN,null):this.rd.value=a,this.setSizeRD(this.rd,b))},"~N,JU.BS");j(c$,"setSizeRD",function(a,b){if(null!=this.atoms){this.isActive=!0;var d=null!=a&&0!=a.value,c=null==b,f=c?this.ac-1:b.nextSetBit(0);null==this.bsSizeSet&&(this.bsSizeSet=JU.BS.newN(this.ac));null==this.mads&&0<=f&&(this.mads=T(this.ac,0));for(;0<=f;f=c?f-1:b.nextSetBit(f+1))this.setSizeRD2(f,a,d)}},"J.atomdata.RadiusData,JU.BS");c(c$,"setSizeRD2",
function(a,b,d){var c=this.atoms[a];this.mads[a]=c.calculateMad(this.vwr,b);this.bsSizeSet.setBitTo(a,d);c.setShapeVisibility(this.vf,d)},"~N,J.atomdata.RadiusData,~B");c(c$,"setPropAS",function(a,b,d){if("color"===a){this.isActive=!0;var c=JU.C.getColixO(b),f=J.c.PAL.pidOf(b);b=this.checkColixLength(c,d.length());for(a=d.nextSetBit(0);0<=a&&a<b;a=d.nextSetBit(a+1))this.setColixAndPalette(c,f,a)}else if("params"===a){this.isActive=!0;var f=b[0],e=b[1],h=b[2],k=new J.atomdata.RadiusData(null,0,J.atomdata.RadiusData.EnumType.FACTOR,
J.c.VDW.AUTO);null==this.bsColixSet&&(this.bsColixSet=new JU.BS);null==this.bsSizeSet&&(this.bsSizeSet=new JU.BS);a=d.nextSetBit(0);null==this.mads&&0<=a&&(this.mads=T(this.ac,0));b=this.checkColixLength(null==f?0:4,d.length());for(var j=0;0<=a&&a<b;a=d.nextSetBit(a+1),j++){var c=null==f?0:f[j],z=null==e?0:e[j];0.01<z&&(c=JU.C.getColixTranslucent3(c,!0,z));this.setColixAndPalette(c,J.c.PAL.UNKNOWN.id,a);null!=h&&(c=0<(k.value=h[j]),this.setSizeRD2(a,k,c))}}else if("translucency"===a){this.isActive=
!0;b=b.equals("translucent");this.checkColixLength(4,this.ac);for(a=d.nextSetBit(0);0<=a;a=d.nextSetBit(a+1))this.colixes[a]=JU.C.getColixTranslucent3(this.colixes[a],b,this.translucentLevel),b&&this.bsColixSet.set(a)}else"deleteModelAtoms"===a?(this.atoms=b[1],a=b[2],this.ac=this.ms.ac,b=a[1],a=a[2],this.mads=JU.AU.deleteElements(this.mads,b,a),this.colixes=JU.AU.deleteElements(this.colixes,b,a),this.paletteIDs=JU.AU.deleteElements(this.paletteIDs,b,a),JU.BSUtil.deleteBits(this.bsSizeSet,d),JU.BSUtil.deleteBits(this.bsColixSet,
d)):this.setPropS(a,b,d)},"~S,~O,JU.BS");c(c$,"checkColixLength",function(a,b){b=Math.min(this.ac,b);if(0==a)return null==this.colixes?0:this.colixes.length;if(null==this.colixes||b>this.colixes.length)this.colixes=JU.AU.ensureLengthShort(this.colixes,b),this.paletteIDs=JU.AU.ensureLengthByte(this.paletteIDs,b);null==this.bsColixSet&&(this.bsColixSet=JU.BS.newN(this.ac));return b},"~N,~N");c(c$,"setColixAndPalette",function(a,b,d){null==this.colixes&&this.checkColixLength(-1,this.ac);this.colixes[d]=
a=this.getColixI(a,b,d);this.bsColixSet.setBitTo(d,0!=a||0==this.shapeID);this.paletteIDs[d]=b},"~N,~N,~N");j(c$,"setAtomClickability",function(){if(this.isActive)for(var a=this.ac;0<=--a;){var b=this.atoms[a];null==b||(0==(b.shapeVisibilityFlags&this.vf)||this.ms.isAtomHidden(a))||b.setClickable(this.vf)}});c(c$,"getInfoAsString",function(){return null},"~N");j(c$,"getShapeState",function(){return null})});s("J.shape");v(["J.shape.AtomShape"],"J.shape.Balls",["JU.BS","J.c.PAL","JU.C"],function(){c$=
D(J.shape,"Balls",J.shape.AtomShape);j(c$,"setSize",function(a,b){2147483647==a?(this.isActive=!0,null==this.bsSizeSet&&(this.bsSizeSet=new JU.BS),this.bsSizeSet.or(b)):this.setSize2(a,b)},"~N,JU.BS");j(c$,"setSizeRD",function(a,b){this.isActive=!0;null==this.bsSizeSet&&(this.bsSizeSet=new JU.BS);for(var d=Math.min(this.atoms.length,b.length()),c=b.nextSetBit(0);0<=c&&c<d;c=b.nextSetBit(c+1))this.atoms[c].setMadAtom(this.vwr,a),this.bsSizeSet.set(c)},"J.atomdata.RadiusData,JU.BS");j(c$,"setProperty",
function(a,b,d){if("color"===a){var c=JU.C.getColixO(b);0==c&&(c=2);null==this.bsColixSet&&(this.bsColixSet=new JU.BS);var f=J.c.PAL.pidOf(b);for(a=d.nextSetBit(0);0<=a;a=d.nextSetBit(a+1)){var e=this.atoms[a];e.colixAtom=this.getColixA(c,f,e);this.bsColixSet.setBitTo(a,2!=c||f!=J.c.PAL.NONE.id);e.paletteID=f}}else if("colorValues"===a){if(0!=b.length){null==this.bsColixSet&&(this.bsColixSet=new JU.BS);var h=0,f=null;for(a=d.nextSetBit(0);0<=a&&!(h>=b.length);a=d.nextSetBit(a+1))f=Integer.$valueOf(b[h++]),
c=JU.C.getColixO(f),0==c&&(c=2),f=J.c.PAL.pidOf(f),e=this.atoms[a],e.colixAtom=this.getColixA(c,f,e),this.bsColixSet.setBitTo(a,2!=c||f!=J.c.PAL.NONE.id),e.paletteID=f}}else if("colors"===a){b=b[0];null==this.bsColixSet&&(this.bsColixSet=new JU.BS);for(a=d.nextSetBit(0);0<=a;a=d.nextSetBit(a+1))if(!(a>=b.length||0==(c=b[a])))this.atoms[a].colixAtom=c,this.atoms[a].paletteID=J.c.PAL.UNKNOWN.id,this.bsColixSet.set(a)}else if("translucency"===a){b=b.equals("translucent");null==this.bsColixSet&&(this.bsColixSet=
new JU.BS);for(a=d.nextSetBit(0);0<=a;a=d.nextSetBit(a+1))this.atoms[a].setTranslucent(b,this.translucentLevel),b&&this.bsColixSet.set(a)}else a.startsWith("ball")&&(a=a.substring(4).intern()),this.setPropAS(a,b,d)},"~S,~O,JU.BS");j(c$,"setAtomClickability",function(){for(var a=this.vwr.slm.bsDeleted,b=this.ac;0<=--b;){var d=this.atoms[b];null!=d&&(d.setClickable(0),null!=a&&a.get(b)||(0==(d.shapeVisibilityFlags&this.vf)||this.ms.isAtomHidden(b))||d.setClickable(this.vf))}})});s("J.shape");v(["J.shape.Shape"],
"J.shape.FontLineShape",null,function(){c$=t(function(){this.font3d=this.tickInfos=null;p(this,arguments)},J.shape,"FontLineShape",J.shape.Shape);O(c$,function(){this.tickInfos=Array(4)});j(c$,"initShape",function(){this.translucentAllowed=!1});c(c$,"setPropFLS",function(a,b){"tickInfo"===a?null==b.ticks?b.type.equals(" ")?this.tickInfos[0]=this.tickInfos[1]=this.tickInfos[2]=this.tickInfos[3]=null:this.tickInfos["xyz".indexOf(b.type)+1]=null:this.tickInfos["xyz".indexOf(b.type)+1]=b:"font"===a&&
(this.font3d=b)},"~S,~O");j(c$,"getShapeState",function(){return null})});s("J.shape");v(["J.shape.Shape"],"J.shape.Frank",["J.i18n.GT","JV.Viewer"],function(){c$=t(function(){this.frankString="Jmol";this.baseFont3d=this.currentMetricsFont3d=null;this.scaling=this.dy=this.dx=this.y=this.x=this.frankDescent=this.frankAscent=this.frankWidth=0;this.font3d=null;p(this,arguments)},J.shape,"Frank",J.shape.Shape);j(c$,"initShape",function(){this.myType="frank";this.baseFont3d=this.font3d=this.vwr.gdata.getFont3DFSS("SansSerif",
"Plain",16);this.calcMetrics()});j(c$,"setProperty",function(a,b){"font"===a&&10<=b.fontSize&&(this.baseFont3d=b,this.scaling=0)},"~S,~O,JU.BS");j(c$,"wasClicked",function(a,b){var d=this.vwr.getScreenWidth(),c=this.vwr.getScreenHeight();return 0<d&&0<c&&a>d-this.frankWidth-4&&b>c-this.frankAscent-4},"~N,~N");j(c$,"checkObjectHovered",function(a,b){if(!this.vwr.getShowFrank()||!this.wasClicked(a,b)||!this.vwr.menuEnabled())return!1;this.vwr.gdata.antialiasEnabled&&!this.vwr.isSingleThreaded&&(a<<=
1,b<<=1);this.vwr.hoverOnPt(a,b,J.i18n.GT.$("Click for menu..."),null,null);return!0},"~N,~N,JU.BS");c(c$,"calcMetrics",function(){JV.Viewer.isJS?this.frankString="JSmol":this.vwr.isSignedApplet&&(this.frankString="Jmol_S");this.font3d!==this.currentMetricsFont3d&&(this.currentMetricsFont3d=this.font3d,this.frankWidth=this.font3d.stringWidth(this.frankString),this.frankDescent=this.font3d.getDescent(),this.frankAscent=this.font3d.getAscent())});c(c$,"getFont",function(a){a!=this.scaling&&(this.scaling=
a,this.font3d=this.vwr.gdata.getFont3DScaled(this.baseFont3d,a),this.calcMetrics())},"~N");j(c$,"getShapeState",function(){return null});E(c$,"defaultFontName","SansSerif","defaultFontStyle","Plain","defaultFontSize",16,"frankMargin",4)});s("J.shape");v(null,"J.shape.Shape",["J.c.PAL","JU.C","$.Logger","JV.JC"],function(){c$=t(function(){this.ms=this.vwr=this.myType=null;this.translucentLevel=this.vf=this.shapeID=0;this.translucentAllowed=!0;this.isBioShape=!1;this.bsColixSet=this.bsSizeSet=null;
p(this,arguments)},J.shape,"Shape");c(c$,"initializeShape",function(a,b,d){this.vwr=a;this.shapeID=d;this.vf=JV.JC.getShapeVisibilityFlag(d);this.setModelSet(b);this.initShape()},"JV.Viewer,JM.ModelSet,~N");c(c$,"setModelVisibilityFlags",function(){},"JU.BS");c(c$,"getSize",function(){return 0},"~N");c(c$,"getSizeG",function(){return 0},"JM.Group");c(c$,"replaceGroup",function(){},"JM.Group,JM.Group");c(c$,"setModelSet",function(a){this.ms=a;this.initModelSet()},"JM.ModelSet");c(c$,"initModelSet",
function(){});c(c$,"setShapeSizeRD",function(a,b,d){null==b?this.setSize(a,d):this.setSizeRD(b,d)},"~N,J.atomdata.RadiusData,JU.BS");c(c$,"setSize",function(){},"~N,JU.BS");c(c$,"setSizeRD",function(){},"J.atomdata.RadiusData,JU.BS");c(c$,"setPropS",function(a,b,d){if("setProperties"===a)for(null==d&&(d=this.vwr.bsA());0<b.size();)a=b.removeItemAt(0),this.setProperty(a[0].intern(),a[1],d);else"translucentLevel"===a?this.translucentLevel=b.floatValue():"refreshTrajectories"!==a&&JU.Logger.warn("unassigned "+
JV.JC.shapeClassBases[this.shapeID]+" + shape setProperty:"+a+":"+b)},"~S,~O,JU.BS");c(c$,"getPropertyData",function(a,b){return this.getPropShape(a,b)},"~S,~A");c(c$,"getPropShape",function(a,b){if(n(b[1],Integer)){var d=b[1].intValue();b[1]=this.getProperty(a,d);return null!=b[1]}return!1},"~S,~A");c(c$,"getProperty",function(){return null},"~S,~N");c(c$,"getIndexFromName",function(){return-1},"~S");c(c$,"wasClicked",function(){return!1},"~N,~N");c(c$,"findNearestAtomIndex",function(){},"~N,~N,~A,JU.BS");
c(c$,"checkBoundsMinMax",function(){},"JU.P3,JU.P3");c(c$,"setAtomClickability",function(){});c(c$,"checkObjectClicked",function(){return null},"~N,~N,~N,JU.BS,~B");c(c$,"checkObjectHovered",function(){return!1},"~N,~N,JU.BS");c(c$,"checkObjectDragged",function(){return!1},"~N,~N,~N,~N,~N,JU.BS");c(c$,"coordinateInRange",function(a,b,d,c,f){this.vwr.tm.transformPtScr(d,f);a=(a-f.x)*(a-f.x)+(b-f.y)*(b-f.y);return a<c?a:-1},"~N,~N,JU.T3,~N,JU.P3i");c(c$,"getColixI",function(a,b,d){return this.getColixA(a,
b,this.ms.at[d])},"~N,~N,~N");c(c$,"getColixA",function(a,b,d){return 2==a?this.vwr.cm.getColixAtomPalette(d,b):a},"~N,~N,JM.Atom");c(c$,"getColixB",function(a,b,d){return 2==a?this.vwr.cm.getColixBondPalette(d,b):a},"~N,~N,JM.Bond");c(c$,"getShapeDetail",function(){return null});c$.getColix=c(c$,"getColix",function(a,b,d){return JU.C.getColixInherited(null==a||b>=a.length?0:a[b],d.colixAtom)},"~A,~N,JM.Atom");c$.getFontCommand=c(c$,"getFontCommand",function(a,b){return null==b?"":"font "+a+" "+b.getInfo()},
"~S,JU.Font");c$.getColorCommandUnk=c(c$,"getColorCommandUnk",function(a,b,d){return J.shape.Shape.getColorCommand(a,J.c.PAL.UNKNOWN.id,b,d)},"~S,~N,~B");c$.getColorCommand=c(c$,"getColorCommand",function(a,b,d,c){if(b==J.c.PAL.UNKNOWN.id&&0==d)return"";b=b==J.c.PAL.UNKNOWN.id&&0==d?"":(c?J.shape.Shape.getTranslucentLabel(d)+" ":"")+(b!=J.c.PAL.UNKNOWN.id&&!J.c.PAL.isPaletteVariable(b)?J.c.PAL.getPaletteName(b):J.shape.Shape.encodeColor(d));return"color "+a+" "+b},"~S,~N,~N,~B");c$.encodeColor=c(c$,
"encodeColor",function(a){return JU.C.isColixColorInherited(a)?"none":JU.C.getHexCode(a)},"~N");c$.getTranslucentLabel=c(c$,"getTranslucentLabel",function(a){return JU.C.isColixTranslucent(a)?JU.C.getColixTranslucencyLabel(a):"opaque"},"~N");c$.appendCmd=c(c$,"appendCmd",function(a,b){0!=b.length&&a.append("  ").append(b).append(";\n")},"JU.SB,~S");E(c$,"RADIUS_MAX",4)});s("J.shape");v(["J.shape.Shape","JU.P3i"],"J.shape.Sticks","java.util.Hashtable JU.BS $.P3 J.c.PAL JU.BSUtil $.C $.Edge $.Escape".split(" "),
function(){c$=t(function(){this.myMask=0;this.reportAll=!1;this.ptXY=this.closestAtom=this.selectedBonds=this.bsOrderSet=null;p(this,arguments)},J.shape,"Sticks",J.shape.Shape);O(c$,function(){this.closestAtom=B(1,0);this.ptXY=new JU.P3i});j(c$,"initShape",function(){this.myMask=1023;this.reportAll=!1});j(c$,"setSize",function(a,b){if(2147483647==a)this.selectedBonds=JU.BSUtil.copy(b);else if(-2147483648==a)null==this.bsOrderSet&&(this.bsOrderSet=new JU.BS),this.bsOrderSet.or(b);else{null==this.bsSizeSet&&
(this.bsSizeSet=new JU.BS);for(var d=null!=this.selectedBonds?this.ms.getBondIterator(this.selectedBonds):this.ms.getBondIteratorForType(this.myMask,b);d.hasNext();)this.bsSizeSet.set(d.nextIndex()),d.next().setMad(a)}},"~N,JU.BS");j(c$,"setProperty",function(a,b,d){if("type"===a)this.myMask=b.intValue();else if("reportAll"===a)this.reportAll=!0;else if("reset"===a)this.selectedBonds=this.bsColixSet=this.bsSizeSet=this.bsOrderSet=null;else if("bondOrder"===a){null==this.bsOrderSet&&(this.bsOrderSet=
new JU.BS);a=b.intValue();for(d=null!=this.selectedBonds?this.ms.getBondIterator(this.selectedBonds):this.ms.getBondIteratorForType(65535,d);d.hasNext();)this.bsOrderSet.set(d.nextIndex()),d.next().setOrder(a)}else if("color"===a)if(null==this.bsColixSet&&(this.bsColixSet=new JU.BS),a=JU.C.getColixO(b),b=n(b,J.c.PAL)?b:null,b===J.c.PAL.TYPE||b===J.c.PAL.ENERGY){var c=b===J.c.PAL.ENERGY;for(d=null!=this.selectedBonds?this.ms.getBondIterator(this.selectedBonds):this.ms.getBondIteratorForType(this.myMask,
d);d.hasNext();){this.bsColixSet.set(d.nextIndex());var f=d.next();f.colix=c?this.getColixB(a,b.id,f):JU.C.getColix(JU.Edge.getArgbHbondType(f.order))}}else{if(!(2==a&&b!==J.c.PAL.CPK))for(d=null!=this.selectedBonds?this.ms.getBondIterator(this.selectedBonds):this.ms.getBondIteratorForType(this.myMask,d);d.hasNext();)b=d.nextIndex(),d.next().colix=a,this.bsColixSet.setBitTo(b,0!=a&&2!=a)}else if("translucency"===a){null==this.bsColixSet&&(this.bsColixSet=new JU.BS);a=b.equals("translucent");for(d=
null!=this.selectedBonds?this.ms.getBondIterator(this.selectedBonds):this.ms.getBondIteratorForType(this.myMask,d);d.hasNext();)this.bsColixSet.set(d.nextIndex()),d.next().setTranslucent(a,this.translucentLevel)}else"deleteModelAtoms"!==a&&this.setPropS(a,b,d)},"~S,~O,JU.BS");j(c$,"getProperty",function(a){return a.equals("selectionState")?null!=this.selectedBonds?"select BONDS "+JU.Escape.eBS(this.selectedBonds)+"\n":"":a.equals("sets")?u(-1,[this.bsOrderSet,this.bsSizeSet,this.bsColixSet]):null},
"~S,~N");j(c$,"setAtomClickability",function(){for(var a=this.ms.bo,b=this.ms.bondCount;0<=--b;){var d=a[b];0==(d.shapeVisibilityFlags&this.vf)||(this.ms.isAtomHidden(d.atom1.i)||this.ms.isAtomHidden(d.atom2.i))||(d.atom1.setClickable(this.vf),d.atom2.setClickable(this.vf))}});j(c$,"getShapeState",function(){return null});j(c$,"checkObjectHovered",function(a,b,d){var c=new JU.P3;d=this.findPickedBond(a,b,d,c,this.closestAtom);if(null==d)return!1;this.vwr.highlightBond(d.index,this.closestAtom[0],
a,b);return!0},"~N,~N,JU.BS");j(c$,"checkObjectClicked",function(a,b,d,c){d=new JU.P3;a=this.findPickedBond(a,b,c,d,this.closestAtom);if(null==a)return null;b=a.atom1.mi;c=a.getIdentity();var f=new java.util.Hashtable;f.put("pt",d);f.put("index",Integer.$valueOf(a.index));f.put("modelIndex",Integer.$valueOf(b));f.put("model",this.vwr.getModelNumberDotted(b));f.put("type","bond");f.put("info",c);this.vwr.setStatusAtomPicked(-3,'["bond","'+a.getIdentity()+'",'+d.x+","+d.y+","+d.z+"]",f,!1);return f},
"~N,~N,~N,JU.BS,~B");c(c$,"findPickedBond",function(a,b,d,c,f){d=100;this.vwr.gdata.isAntialiased()&&(a<<=1,b<<=1,d<<=1);for(var e=null,h=new JU.P3,k=this.ms.bo,j=this.ms.bondCount;0<=--j;){var z=k[j];if(0!=z.shapeVisibilityFlags){var m=z.atom1,r=z.atom2;if(m.checkVisible()&&r.checkVisible()){h.ave(m,r);var x=this.coordinateInRange(a,b,h,d,this.ptXY);if(0<=x&&40<Math.abs(m.sY-r.sY)+Math.abs(m.sX-r.sX)){var y=1*(this.ptXY.x-m.sX)/(r.sX-m.sX);0.4>y||0.6<y||(d=x,e=z,null!=f&&(f[0]=0.5>y?m.i:r.i),c.setT(h))}}}}return e},
"~N,~N,JU.BS,JU.P3,~A");E(c$,"MAX_BOND_CLICK_DISTANCE_SQUARED",100,"XY_THREASHOLD",40)});s("J.thread");v(["J.thread.JmolThread"],"J.thread.HoverWatcherThread",["java.lang.Thread"],function(){c$=t(function(){this.moved=this.current=this.actionManager=null;this.hoverDelay=0;p(this,arguments)},J.thread,"HoverWatcherThread",J.thread.JmolThread);q(c$,function(a,b,d,c){this.setViewer(c,"HoverWatcher");this.actionManager=a;this.current=b;this.moved=d;this.start()},"JV.ActionManager,JV.MouseState,JV.MouseState,JV.Viewer");
j(c$,"run1",function(a){for(;;)switch(a){case -1:this.isJS||Thread.currentThread().setPriority(1);a=0;break;case 0:this.hoverDelay=this.vwr.getHoverDelay();if(this.stopped||0>=this.hoverDelay||!this.runSleep(this.hoverDelay,1))return;a=1;break;case 1:this.moved.is(this.current)&&(this.currentTime=Zystem.currentTimeMillis(),this.currentTime-this.moved.time>(this.vwr.acm.zoomTrigger?100:this.hoverDelay)&&!this.stopped&&this.actionManager.checkHover()),a=0}},"~N")});s("J.thread");v(["java.lang.Thread"],
"J.thread.JmolThread",["JU.Logger","JV.Viewer"],function(){c$=t(function(){this.$name="JmolThread";this.sc=this.eval=this.vwr=null;this.hoverEnabled=this.haveReference=!1;this.sleepTime=this.currentTime=this.lastRepaintTime=this.targetTime=this.startTime=0;this.isReset=this.stopped=this.isJS=!1;this.useTimeout=!0;this.junk=0;p(this,arguments)},J.thread,"JmolThread",Thread);c(c$,"setManager",function(){return 0},"~O,JV.Viewer,~O");c(c$,"setViewer",function(a,b){this.setName(b);this.$name=b+"_"+ ++J.thread.JmolThread.threadIndex;
this.vwr=a;this.isJS=a.isSingleThreaded},"JV.Viewer,~S");c(c$,"setEval",function(a){this.eval=a;this.sc=this.vwr.getEvalContextAndHoldQueue(a);null!=this.sc&&(this.useTimeout=a.getAllowJSThreads())},"J.api.JmolScriptEvaluator");c(c$,"resumeEval",function(){if(!(null==this.eval||!this.isJS&&!this.vwr.testAsync||!this.useTimeout)){this.sc.mustResumeEval=!this.stopped;var a=this.eval,b=this.sc;this.sc=this.eval=null;setTimeout(function(){a.resumeEval(b)},1)}});c(c$,"start",function(){this.isJS?this.run():
W(this,J.thread.JmolThread,"start",[])});j(c$,"run",function(){this.startTime=Zystem.currentTimeMillis();try{this.run1(-1)}catch(a){if(H(a,InterruptedException)){var b=a;JU.Logger.debugging&&!n(this,J.thread.HoverWatcherThread)&&this.oops(b)}else if(H(a,Exception))this.oops(a);else throw a;}});c(c$,"oops",function(a){JU.Logger.debug(this.$name+" exception "+a);JV.Viewer.isJS||a.printStackTrace();this.vwr.queueOnHold=!1},"Exception");c(c$,"runSleep",function(a,b){if(this.isJS&&!this.useTimeout)return!0;
var d=this;setTimeout(function(){d.run1(b)},Math.max(a,0));return!1},"~N,~N");c(c$,"interrupt",function(){this.stopped=!0;this.vwr.startHoverWatcher(!0);this.isJS||W(this,J.thread.JmolThread,"interrupt",[])});c(c$,"checkInterrupted",function(a){return this.haveReference&&(null==a||!a.$name.equals(this.$name))?!0:this.stopped},"J.thread.JmolThread");c(c$,"reset",function(){this.isReset=!0;this.interrupt()});E(c$,"threadIndex",0,"INIT",-1,"MAIN",0,"FINISH",-2,"CHECK1",1,"CHECK2",2,"CHECK3",3)});s("J.thread");
v(["J.thread.JmolThread"],"J.thread.TimeoutThread",["java.lang.Thread","JU.SB"],function(){c$=t(function(){this.script=null;this.status=0;this.triggered=!0;p(this,arguments)},J.thread,"TimeoutThread",J.thread.JmolThread);q(c$,function(a,b,d,c){this.setViewer(a,b);this.$name=b;this.set(d,c)},"JV.Viewer,~S,~N,~S");c(c$,"set",function(a,b){this.sleepTime=a;null!=b&&(this.script=b)},"~N,~S");j(c$,"toString",function(){return"timeout name="+this.$name+" executions="+this.status+" mSec="+this.sleepTime+
" secRemaining="+(this.targetTime-Zystem.currentTimeMillis())/1E3+" script="+this.script});j(c$,"run1",function(a){for(;;)switch(a){case -1:this.isJS||Thread.currentThread().setPriority(1);this.targetTime=Zystem.currentTimeMillis()+Math.abs(this.sleepTime);a=0;break;case 0:if(this.checkInterrupted(null)||(null==this.script||0==this.script.length)||!this.runSleep(26,1))return;a=1;break;case 1:a=Zystem.currentTimeMillis()<this.targetTime?0:2;break;case 2:this.currentTime=Zystem.currentTimeMillis();
if(null==this.vwr.timeouts.get(this.$name))return;this.status++;(a=0>this.sleepTime)?this.targetTime=Zystem.currentTimeMillis()+Math.abs(this.sleepTime):this.vwr.timeouts.remove(this.$name);this.triggered&&(this.triggered=!1,this.$name.equals("_SET_IN_MOTION_")?this.vwr.checkInMotion(2):this.vwr.evalStringQuiet(a?this.script+';\ntimeout ID "'+this.$name+'";':this.script));a=a?0:-2;break;case -2:this.vwr.timeouts.remove(this.$name);return}},"~N");c$.clear=c(c$,"clear",function(a){for(var b,d=a.values().iterator();d.hasNext()&&
((b=d.next())||1);){var c=b;c.script.equals("exitJmol")||c.interrupt()}a.clear()},"java.util.Map");c$.setTimeout=c(c$,"setTimeout",function(a,b,d,c,f){var e=b.get(d);0==c?null!=e&&(e.interrupt(),b.remove(d)):null!=e?e.set(c,f):(e=new J.thread.TimeoutThread(a,d,c,f),b.put(d,e),e.start())},"JV.Viewer,java.util.Map,~S,~N,~S");c$.trigger=c(c$,"trigger",function(a,b){var d=a.get(b);null!=d&&(d.triggered=0>d.sleepTime)},"java.util.Map,~S");c$.showTimeout=c(c$,"showTimeout",function(a,b){var d=new JU.SB;
if(null!=a)for(var c,f=a.values().iterator();f.hasNext()&&((c=f.next())||1);){var e=c;(null==b||e.$name.equalsIgnoreCase(b))&&d.append(e.toString()).append("\n")}return 0<d.length()?d.toString():"<no timeouts set>"},"java.util.Map,~S")});s("JM");v(["JU.Node","$.Point3fi","J.c.PAL"],"JM.Atom","java.lang.Float JU.BS $.CU $.P3 $.PT $.SB J.atomdata.RadiusData J.c.VDW JM.Group JU.C $.Elements JV.JC".split(" "),function(){c$=t(function(){this.altloc="\x00";this.atomSite=this.atomID=0;this.group=null;this.atomicAndIsotopeNumber=
this.valence=this.userDefinedVanDerWaalRadius=0;this.atomSymmetry=null;this.paletteID=this.colixAtom=this.madAtom=this.formalChargeAndFlags=0;this.bonds=null;this.shapeVisibilityFlags=this.clickabilityFlags=this.nBackbonesDisplayed=this.nBondsDisplayed=0;p(this,arguments)},JM,"Atom",JU.Point3fi,JU.Node);O(c$,function(){this.paletteID=J.c.PAL.CPK.id});j(c$,"setAtom",function(a,b,d,c,f,e,h,k,j){this.mi=a;this.atomSymmetry=f;this.atomSite=e;this.i=b;this.atomicAndIsotopeNumber=h;j&&(this.formalChargeAndFlags=
2);0!=k&&-2147483648!=k&&this.setFormalCharge(k);this.userDefinedVanDerWaalRadius=c;this.setT(d);return this},"~N,~N,JU.P3,~N,JU.BS,~N,~N,~N,~B");c(c$,"setShapeVisibility",function(a,b){this.shapeVisibilityFlags=b?this.shapeVisibilityFlags|a:this.shapeVisibilityFlags&~a},"~N,~B");c(c$,"isCovalentlyBonded",function(a){if(null!=this.bonds)for(var b=this.bonds.length;0<=--b;)if(this.bonds[b].isCovalent()&&this.bonds[b].getOtherAtom(this)===a)return!0;return!1},"JM.Atom");c(c$,"isBonded",function(a){if(null!=
this.bonds)for(var b=this.bonds.length;0<=--b;)if(this.bonds[b].getOtherAtom(this)===a)return!0;return!1},"JM.Atom");c(c$,"getBond",function(a){if(null!=this.bonds)for(var b=this.bonds.length;0<=--b;)if(null!=this.bonds[b].getOtherAtom(a))return this.bonds[b];return null},"JM.Atom");c(c$,"addDisplayedBond",function(a,b){this.nBondsDisplayed+=b?1:-1;this.setShapeVisibility(a,0<this.nBondsDisplayed)},"~N,~B");c(c$,"deleteBond",function(a){if(null!=this.bonds)for(var b=this.bonds.length;0<=--b;)if(this.bonds[b]===
a){this.deleteBondAt(b);break}},"JM.Bond");c(c$,"deleteBondAt",function(a){this.setCIPChirality(0);var b=this.bonds.length-1;if(0==b)this.bonds=null;else{for(var d=Array(b),c=0;c<a;++c)d[c]=this.bonds[c];for(;c<b;++c)d[c]=this.bonds[c+1];this.bonds=d}},"~N");j(c$,"getBondedAtomIndex",function(a){return this.bonds[a].getOtherAtom(this).i},"~N");c(c$,"setMadAtom",function(a,b){this.madAtom=this.calculateMad(a,b)},"JV.Viewer,J.atomdata.RadiusData");c(c$,"calculateMad",function(a,b){if(null==b)return 0;
var d=b.value;if(0==d)return 0;switch(b.factorType){case J.atomdata.RadiusData.EnumType.SCREEN:return $(d);case J.atomdata.RadiusData.EnumType.FACTOR:case J.atomdata.RadiusData.EnumType.OFFSET:var c=0;switch(b.vdwType){case J.c.VDW.TEMP:c=a.ms.getBfactor100Hi();c=0<c?this.getBfactor100()/c:0;break;case J.c.VDW.HYDRO:c=Math.abs(this.getHydrophobicity());break;case J.c.VDW.BONDING:c=this.getBondingRadius();break;case J.c.VDW.ADPMIN:case J.c.VDW.ADPMAX:c=this.getADPMinMax(b.vdwType===J.c.VDW.ADPMAX);
break;default:c=this.getVanderwaalsRadiusFloat(a,b.vdwType)}d=b.factorType===J.atomdata.RadiusData.EnumType.FACTOR?d*c:d+c;break;case J.atomdata.RadiusData.EnumType.ABSOLUTE:if(16.1==d)return JM.Atom.MAD_GLOBAL}c=$(0>d?d:2E3*d);0>c&&0<d&&(c=0);return c},"JV.Viewer,J.atomdata.RadiusData");c(c$,"getADPMinMax",function(a){var b=this.getTensors();if(null==b)return 0;var d=b[0];if(null==d||1!=d.iType)return 0;this.group.chain.model.ms.isModulated(this.i)&&d.isUnmodulated&&(d=b[1]);return d.getFactoredValue(a?
2:1)},"~B");c(c$,"getTensors",function(){return this.group.chain.model.ms.getAtomTensorList(this.i)});c(c$,"getRasMolRadius",function(){return Math.abs(w(this.madAtom/8))});j(c$,"getEdges",function(){return null==this.bonds?[]:this.bonds});j(c$,"getBondCount",function(){return null==this.bonds?0:this.bonds.length});c(c$,"setTranslucent",function(a,b){this.colixAtom=JU.C.getColixTranslucent3(this.colixAtom,a,b)},"~B,~N");j(c$,"getElementNumber",function(){return JU.Elements.getElementNumber(this.atomicAndIsotopeNumber)});
j(c$,"getIsotopeNumber",function(){return JU.Elements.getIsotopeNumber(this.atomicAndIsotopeNumber)});j(c$,"getAtomicAndIsotopeNumber",function(){return this.atomicAndIsotopeNumber});c(c$,"setAtomicAndIsotopeNumber",function(a){if(0>a||(a&127)>=JU.Elements.elementNumberMax||32767<a)a=0;this.atomicAndIsotopeNumber=a},"~N");c(c$,"getElementSymbolIso",function(a){return JU.Elements.elementSymbolFromNumber(a?this.atomicAndIsotopeNumber:this.atomicAndIsotopeNumber&127)},"~B");c(c$,"getElementSymbol",function(){return this.getElementSymbolIso(!0)});
c(c$,"isHetero",function(){return 0!=(this.formalChargeAndFlags&2)});c(c$,"hasVibration",function(){return 0!=(this.formalChargeAndFlags&1)});c(c$,"setFormalCharge",function(a){this.formalChargeAndFlags=this.formalChargeAndFlags&15|(-2147483648==a?0:7<a?7:-3>a?-3:a)<<24},"~N");c(c$,"setVibrationVector",function(){this.formalChargeAndFlags|=1});j(c$,"getFormalCharge",function(){return this.formalChargeAndFlags>>24});c(c$,"getOccupancy100",function(){var a=this.group.chain.model.ms.occupancies;return null==
a?100:Math.round(a[this.i])});c(c$,"isOccupied",function(){var a=this.group.chain.model.ms.occupancies;return null==a||50<=a[this.i]});c(c$,"getBfactor100",function(){var a=this.group.chain.model.ms.bfactor100s;return null==a?0:a[this.i]});c(c$,"getHydrophobicity",function(){var a=this.group.chain.model.ms.hydrophobicities;return null==a?JU.Elements.getHydrophobicity(this.group.groupID):a[this.i]});c(c$,"setRadius",function(a){return!Float.isNaN(this.userDefinedVanDerWaalRadius=0<a?a:NaN)},"~N");
c(c$,"$delete",function(a){this.valence=-1;if(null!=this.bonds)for(var b=this.bonds.length;0<=--b;){var d=this.bonds[b];d.getOtherAtom(this).deleteBond(d);a.set(d.index)}this.bonds=null},"JU.BS");j(c$,"isDeleted",function(){return 0>this.valence});c(c$,"setValence",function(a){this.isDeleted()||(this.valence=0>a?0:127>=a?a:127)},"~N");j(c$,"getValence",function(){if(this.isDeleted())return-1;var a=this.valence;if(0==a&&null!=this.bonds)for(var b=this.bonds.length;0<=--b;)a+=this.bonds[b].getValence();
return a});j(c$,"getCovalentBondCount",function(){if(null==this.bonds)return 0;for(var a=0,b,d=this.bonds.length;0<=--d;)0!=((b=this.bonds[d]).order&1023)&&!b.getOtherAtom(this).isDeleted()&&++a;return a});j(c$,"getCovalentHydrogenCount",function(){if(null==this.bonds)return 0;for(var a=0,b=this.bonds.length;0<=--b;)if(0!=(this.bonds[b].order&1023)){var d=this.bonds[b].getOtherAtom(this);0<=d.valence&&1==d.getElementNumber()&&++a}return a});j(c$,"getImplicitHydrogenCount",function(){return this.group.chain.model.ms.getMissingHydrogenCount(this,
!1)});j(c$,"getTotalHydrogenCount",function(){return this.getCovalentHydrogenCount()+this.getImplicitHydrogenCount()});j(c$,"getTotalValence",function(){var a=this.getValence();if(0>a)return a;var b=this.getImplicitHydrogenCount();return a+b+this.group.chain.model.ms.aaRet[4]});j(c$,"getCovalentBondCountPlusMissingH",function(){return this.getCovalentBondCount()+this.getImplicitHydrogenCount()});c(c$,"getTargetValence",function(){switch(this.getElementNumber()){case 6:case 14:case 32:return 4;case 5:case 7:case 15:return 3;
case 8:case 16:return 2;case 1:case 9:case 17:case 35:case 53:return 1}return-1});c(c$,"getDimensionValue",function(a){return 0==a?this.x:1==a?this.y:this.z},"~N");c(c$,"getVanderwaalsRadiusFloat",function(a,b){return Float.isNaN(this.userDefinedVanDerWaalRadius)?a.getVanderwaalsMarType(this.atomicAndIsotopeNumber,this.getVdwType(b))/1E3:this.userDefinedVanDerWaalRadius},"JV.Viewer,J.c.VDW");c(c$,"getVdwType",function(a){switch(a){case J.c.VDW.AUTO:a=this.group.chain.model.ms.getDefaultVdwType(this.mi);
break;case J.c.VDW.NOJMOL:a=this.group.chain.model.ms.getDefaultVdwType(this.mi),a===J.c.VDW.AUTO_JMOL&&(a=J.c.VDW.AUTO_BABEL)}return a},"J.c.VDW");c(c$,"getBondingRadius",function(){var a=this.group.chain.model.ms.bondingRadii,a=null==a||this.i>=a.length?0:a[this.i];return 0==a?JU.Elements.getBondingRadius(this.atomicAndIsotopeNumber,this.getFormalCharge()):a});c(c$,"getVolume",function(a,b){var d=null==b?this.userDefinedVanDerWaalRadius:NaN;Float.isNaN(d)&&(d=a.getVanderwaalsMarType(this.getElementNumber(),
this.getVdwType(b))/1E3);var c=0;if(null!=this.bonds)for(var f=0;f<this.bonds.length;f++)if(this.bonds[f].isCovalent()){var e=this.bonds[f].getOtherAtom(this),h=null==b?e.userDefinedVanDerWaalRadius:NaN;Float.isNaN(h)&&(h=a.getVanderwaalsMarType(e.getElementNumber(),e.getVdwType(b))/1E3);e=this.distance(e);if(!(e>d+h)){if(e+d<=h)return 0;h=d-(d*d+e*e-h*h)/(2*e);c-=1.0471975511965976*h*h*(3*d-h)}}return c+4.1887902047863905*d*d*d},"JV.Viewer,J.c.VDW");c(c$,"getCurrentBondCount",function(){return null==
this.bonds?0:this.bonds.length});c(c$,"getRadius",function(){return Math.abs(this.madAtom/2E3)});j(c$,"getIndex",function(){return this.i});j(c$,"getAtomSite",function(){return this.atomSite});j(c$,"getGroupBits",function(a){this.group.setAtomBits(a)},"JU.BS");j(c$,"getAtomName",function(){return 0<this.atomID?JM.Group.specialAtomNames[this.atomID]:null==this.group.chain.model.ms.atomNames?"":this.group.chain.model.ms.atomNames[this.i]});j(c$,"getAtomType",function(){var a=this.group.chain.model.ms.atomTypes,
a=null==a?null:a[this.i];return null==a?this.getAtomName():a});j(c$,"getAtomNumber",function(){var a=this.group.chain.model.ms.atomSerials;return null==a?this.i:a[this.i]});c(c$,"getSeqID",function(){var a=this.group.chain.model.ms.atomSeqIDs;return null==a?0:a[this.i]});c(c$,"isVisible",function(a){return(this.shapeVisibilityFlags&a)==a},"~N");c(c$,"getPartialCharge",function(){var a=this.group.chain.model.ms.partialCharges;return null==a?0:a[this.i]});c(c$,"getSymmetryTranslation",function(a,b,
d){for(var c=0;c<b.length;c++)if(this.atomSymmetry.get(a+=d))return b[c];return 0},"~N,~A,~N");c(c$,"getCellTranslation",function(a,b,d){for(var c=d,f=0;f<b.length;f++)for(var e=0;e<d;e++,c++)if(this.atomSymmetry.get(c)&&b[f]==a)return b[f];return 0},"~N,~A,~N");c(c$,"getSymmetryOperatorList",function(a){var b="",d=this.group.chain.model.ms,c=d.getModelSymmetryCount(this.mi);if(0==c||null==this.atomSymmetry)return"";for(var d=d.getModelCellRange(this.mi),f=c,e=null==d?1:d.length,h=a?null:new JU.BS,
k=0;k<e;k++)for(var j=0;j<c;j++)this.atomSymmetry.get(f++)&&(a?b+=","+(j+1)+d[k]:h.set(j+1));if(!a)for(k=h.nextSetBit(0);0<=k;k=h.nextSetBit(k+1))b+=","+k;return 0==b.length?"":b.substring(1)},"~B");j(c$,"getModelIndex",function(){return this.mi});j(c$,"getMoleculeNumber",function(a){return this.group.chain.model.ms.getMoleculeIndex(this.i,a)+1},"~B");c(c$,"getFractionalCoord",function(a,b,d,c){c=this.getFractionalCoordPt(a,d,c);return"X"==b?c.x:"Y"==b?c.y:c.z},"~B,~S,~B,JU.P3");j(c$,"getXYZ",function(){return this});
c(c$,"getFractionalCoordPt",function(a,b,d){var c=this.getUnitCell();if(null==c)return this;null==d?d=JU.P3.newP(this):d.setT(this);c.toFractional(d,b);a&&JU.PT.fixPtFloats(d,1E5);return d},"~B,~B,JU.P3");c(c$,"getUnitCell",function(){return this.group.chain.model.ms.getUnitCellForAtom(this.i)});c(c$,"getFractionalUnitCoord",function(a,b,d){d=this.getFractionalUnitCoordPt(a,!1,d);return"X"==b?d.x:"Y"==b?d.y:d.z},"~B,~S,JU.P3");c(c$,"getFractionalUnitCoordPt",function(a,b,d){var c=this.getUnitCell();
if(null==c)return this;null==d?d=JU.P3.newP(this):d.setT(this);this.group.chain.model.isJmolDataFrame?(c.toFractional(d,!1),b&&c.toCartesian(d,!1)):(c.toUnitCell(d,null),b||c.toFractional(d,!1));a&&JU.PT.fixPtFloats(d,b?1E4:1E5);return d},"~B,~B,JU.P3");c(c$,"getFractionalUnitDistance",function(a,b,d){var c=this.getUnitCell();if(null==c)return this.distance(a);b.setT(this);d.setT(a);this.group.chain.model.isJmolDataFrame?(c.toFractional(b,!0),c.toFractional(d,!0)):(c.toUnitCell(b,null),c.toUnitCell(d,
null));return b.distance(d)},"JU.T3,JU.T3,JU.T3");c(c$,"setFractionalCoord",function(a,b,d){var c=this.getUnitCell();null!=c&&c.toFractional(this,d);switch(a){case 1111492615:case 1111492612:this.x=b;break;case 1111492616:case 1111492613:this.y=b;break;case 1111492617:case 1111492614:this.z=b}null!=c&&c.toCartesian(this,d)},"~N,~N,~B");c(c$,"setFractionalCoordTo",function(a,b){this.setFractionalCoordPt(this,a,b)},"JU.P3,~B");c(c$,"setFractionalCoordPt",function(a,b,d){a.setT(b);b=this.getUnitCell();
null!=b&&b.toCartesian(a,d&&!this.group.chain.model.isJmolDataFrame)},"JU.P3,JU.P3,~B");c(c$,"isCursorOnTopOf",function(a,b,d,c){var f=w(this.sD/2);f<d&&(f=d);d=f*f;f=this.sX-a;f*=f;if(f>d)return!1;var e=this.sY-b;d-=f+e*e;if(0>d)return!1;if(null==c)return!0;var f=this.sZ,e=c.sZ,h=w(c.sD/2);if(f<e-h)return!0;a=c.sX-a;b=c.sY-b;b=h*h-(a*a+b*b);return f-Math.sqrt(d)<e-Math.sqrt(b)},"~N,~N,~N,JM.Atom");c(c$,"getInfo",function(){return this.getIdentity(!0)});c(c$,"getIdentityXYZ",function(a,b){b=this.group.chain.model.isJmolDataFrame?
this.getFractionalCoordPt(!this.group.chain.model.ms.vwr.g.legacyJavaFloat,!1,b):this;return this.getIdentity(a)+" "+JU.PT.formatF(b.x,0,3,!0,!0)+" "+JU.PT.formatF(b.y,0,3,!0,!0)+" "+JU.PT.formatF(b.z,0,3,!0,!0)},"~B,JU.P3");c(c$,"getIdentity",function(a){var b=new JU.SB,d=this.getGroup3(!0);if(null!=d&&0<d.length&&(!d.equals("UNK")||this.group.chain.model.isBioModel)){b.append("[");b.append(d);b.append("]");d=this.group.getSeqcodeString();null!=d&&b.append(d);d=this.group.chain.chainID;if(0!=d&&
32!=d){b.append(":");var c=this.getChainIDStr();256<=d&&(c=JU.PT.esc(c));b.append(c)}if(!a)return b.toString();b.append(".")}b.append(this.getAtomName());0==b.length()&&(b.append(this.getElementSymbolIso(!1)),b.append(" "),b.appendI(this.getAtomNumber()));"\x00"!=this.altloc&&(b.append("%"),b.appendC(this.altloc));1<this.group.chain.model.ms.mc&&!this.group.chain.model.isJmolDataFrame&&(b.append("/"),b.append(this.getModelNumberForLabel()));b.append(" #");b.appendI(this.getAtomNumber());return b.toString()},
"~B");j(c$,"getGroup3",function(a){var b=this.group.getGroup3();return a||null!=b&&0<b.length?b:"UNK"},"~B");j(c$,"getGroup1",function(a){var b=this.group.getGroup1();return"\x00"!=b?""+b:"\x00"!=a?""+a:""},"~S");j(c$,"getBioSmilesType",function(){return this.group.isProtein()?"p":this.group.isDna()?"d":this.group.isRna()?"r":this.group.isCarbohydrate()?"c":" "});j(c$,"isPurine",function(){return this.group.isPurine()});j(c$,"isPyrimidine",function(){return this.group.isPyrimidine()});j(c$,"getResno",
function(){return this.group.getResno()});c(c$,"isClickable",function(){return this.checkVisible()&&0!=this.clickabilityFlags&&0!=((this.shapeVisibilityFlags|this.group.shapeVisibilityFlags)&this.clickabilityFlags)});c(c$,"setClickable",function(a){0==a?this.clickabilityFlags=0:(this.clickabilityFlags|=a,1040384!=a&&(this.shapeVisibilityFlags|=a))},"~N");c(c$,"checkVisible",function(){if(this.isVisible(2))return this.isVisible(4);var a=this.isVisible(9);if(a){a=this.shapeVisibilityFlags;if(0!=this.group.shapeVisibilityFlags&&
(8192!=this.group.shapeVisibilityFlags||this.isLeadAtom()))a|=this.group.shapeVisibilityFlags;a&=-10;32==a&&0==this.clickabilityFlags&&(a=0);if(a=0!=a)this.shapeVisibilityFlags|=4}this.shapeVisibilityFlags|=2;return a});j(c$,"isLeadAtom",function(){return this.group.isLeadAtom(this.i)});j(c$,"getChainID",function(){return this.group.chain.chainID});j(c$,"getChainIDStr",function(){return this.group.chain.getIDStr()});c(c$,"getSurfaceDistance100",function(){return this.group.chain.model.ms.getSurfaceDistance100(this.i)});
c(c$,"getVibrationVector",function(){return this.group.chain.model.ms.getVibration(this.i,!1)});c(c$,"getModulation",function(){return this.group.chain.model.ms.getModulation(this.i)});c(c$,"getModelNumberForLabel",function(){return this.group.chain.model.ms.getModelNumberForAtomLabel(this.mi)});c(c$,"getModelNumber",function(){return this.group.chain.model.ms.getModelNumber(this.mi)%1E6});j(c$,"getBioStructureTypeName",function(){return this.group.getProteinStructureType().getBioStructureTypeName(!0)});
j(c$,"equals",function(a){return this===a},"~O");j(c$,"hashCode",function(){return this.i});c(c$,"findAromaticNeighbor",function(a){if(null==this.bonds)return null;for(var b=this.bonds.length;0<=--b;){var d=this.bonds[b],c=d.getOtherAtom(this);if(d.isAromatic()&&c.i!=a)return c}return null},"~N");c(c$,"atomPropertyInt",function(a){switch(a){case 1094715393:return this.getAtomNumber();case 1094713365:return this.getSeqID();case 1094713346:return this.atomID;case 1094713368:return Math.max(0,this.altloc.charCodeAt(0)-
32);case 1094713347:return this.i;case 1228931587:return this.getCovalentBondCount();case 1094713351:return this.group.chain.chainNo;case 1765808134:return this.group.chain.model.ms.vwr.gdata.getColorArgbOrGray(this.colixAtom);case 1086326789:case 1094715402:return this.getElementNumber();case 1094713353:return this.atomicAndIsotopeNumber;case 1228935687:return this.group.chain.model.fileIndex+1;case 1631586315:return this.getFormalCharge();case 1094713356:return this.group.groupID;case 1094713357:return this.group.groupIndex;
case 1094717454:return this.getModelNumber();case -1094717454:return this.group.chain.model.ms.modelFileNumbers[this.mi];case 1094713359:return this.mi;case 1094713360:return this.getMoleculeNumber(!0);case 1094713361:return this.group.getMonomerIndex()+1;case 1128269825:return this.getOccupancy100();case 1094713362:return this.group.getBioPolymerIndexInModel()+1;case 1094713363:return this.group.getBioPolymerLength();case 1665140738:return this.getRasMolRadius();case 1094715412:return this.getResno();
case 1094713366:return this.getAtomSite();case 1639976963:return this.group.getProteinStructureType().getId();case 1237320707:return this.group.getProteinStructureSubType().getId();case 1094713367:return this.group.getStrucNo();case 1296041986:return this.getSymOp();case 1094715417:return this.getValence()}return 0},"~N");c(c$,"getSymOp",function(){return null==this.atomSymmetry?0:this.atomSymmetry.nextSetBit(0)+1});c(c$,"atomPropertyFloat",function(a,b,d){switch(b){case 1111490561:return this.getADPMinMax(!0);
case 1111490562:return this.getADPMinMax(!1);case 1111492609:case 1111492629:return this.x;case 1111492610:case 1111492630:return this.y;case 1111492611:case 1111492631:return this.z;case 1111490587:return this.group.chain.model.ms.getAtomicDSSRData(this.i);case 1114249217:case 1112152066:case 1112150019:case 1112150020:case 1112150021:case 1112152070:case 1112152071:case 1112152073:case 1112152074:case 1112152076:case 1649022989:case 1112152078:return a.shm.getAtomShapeValue(b,this.group,this.i);
case 1111492618:return this.getBondingRadius();case 1111490563:return a.getNMRCalculation().getChemicalShift(this);case 1111490564:return JU.Elements.getCovalentRadius(this.atomicAndIsotopeNumber);case 1111490565:case 1111490576:case 1111490574:return this.group.getGroupParameter(b);case 1111492615:case 1111492612:return this.getFractionalCoord(!a.g.legacyJavaFloat,"X",!1,d);case 1111492616:case 1111492613:return this.getFractionalCoord(!a.g.legacyJavaFloat,"Y",!1,d);case 1111492617:case 1111492614:return this.getFractionalCoord(!a.g.legacyJavaFloat,
"Z",!1,d);case 1113589786:return this.getHydrophobicity();case 1111490566:return a.getNMRCalculation().getMagneticShielding(this);case 1111490567:return this.getMass();case 1128269825:return this.getOccupancy100()/100;case 1111492619:return this.getPartialCharge();case 1111490569:case 1111490570:case 1111490568:if(this.group.chain.model.isJmolDataFrame&&this.group.chain.model.jmolFrameType.startsWith("plot ramachandran"))switch(b){case 1111490569:return this.getFractionalCoord(!a.g.legacyJavaFloat,
"X",!1,d);case 1111490570:return this.getFractionalCoord(!a.g.legacyJavaFloat,"Y",!1,d);case 1111490568:if(this.group.chain.model.isJmolDataFrame&&this.group.chain.model.jmolFrameType.equals("plot ramachandran"))return a=this.getFractionalCoord(!a.g.legacyJavaFloat,"Z",!1,d)-180,-180>a?360+a:a}return this.group.getGroupParameter(b);case 1665140738:case 1112152075:return this.getRadius();case 1111490571:return a.antialiased?w(this.sX/2):this.sX;case 1111490572:return a.getScreenHeight()-(a.antialiased?
w(this.sY/2):this.sY);case 1111490573:return a.antialiased?w(this.sZ/2):this.sZ;case 1113589787:return a.slm.isAtomSelected(this.i)?1:0;case 1111490575:return a.ms.getSurfaceDistanceMax(),this.getSurfaceDistance100()/100;case 1111492620:return this.getBfactor100()/100;case 1111490577:return this.getFractionalUnitCoord(!a.g.legacyJavaFloat,"X",d);case 1111490578:return this.getFractionalUnitCoord(!a.g.legacyJavaFloat,"Y",d);case 1111490579:return this.getFractionalUnitCoord(!a.g.legacyJavaFloat,"Z",
d);case 1648363544:return this.getVanderwaalsRadiusFloat(a,J.c.VDW.AUTO);case 1648361473:return b=this.getVibrationVector(),null==b?0:b.length()*a.getFloat(1648361473);case 1111492626:return this.getVib("x");case 1111492627:return this.getVib("y");case 1111492628:return this.getVib("z");case 1111490583:return this.getVib("X");case 1111490584:return this.getVib("Y");case 1111490585:return this.getVib("Z");case 1111490586:return this.getVib("O");case 1111490580:return this.getVib("1");case 1111490581:return this.getVib("2");
case 1111490582:return this.getVib("3");case 1312817669:return this.getVolume(a,J.c.VDW.AUTO);case 1145047051:case 1145047053:case 1145045006:case 1145047052:case 1145047055:case 1145045008:case 1145047050:return a=this.atomPropertyTuple(a,b,d),null==a?-1:a.length()}return this.atomPropertyInt(b)},"JV.Viewer,~N,JU.P3");c(c$,"getVib",function(a){return this.group.chain.model.ms.getVibCoord(this.i,a)},"~S");c(c$,"getNominalMass",function(){var a=this.getIsotopeNumber();return 0<a?a:JU.Elements.getNaturalIsotope(this.getElementNumber())});
j(c$,"getMass",function(){var a=this.getIsotopeNumber();return 0<a?a:JU.Elements.getAtomicMass(this.getElementNumber())});c(c$,"atomPropertyString",function(a,b){var d;switch(b){case 1086324739:return d=this.altloc,"\x00"==d?"":""+d;case 1086326786:return this.getAtomName();case 1086326785:return this.getAtomType();case 1086326788:return this.getChainIDStr();case 1086324752:return this.getCIPChirality(!0);case 1086324753:return this.getCIPChiralityRule();case 1086324744:return this.getGroup1("?");
case 1086324747:return d=this.group.getSeqcodeString(),null==d?"":d;case 1086324743:return this.getGroup1("\x00");case 1086324742:return this.getGroup3(!1);case 1086326789:return this.getElementSymbolIso(!0);case 1086324745:return this.getIdentity(!0);case 1086324746:return d=this.group.getInsertionCode(),"\x00"==d?"":""+d;case 1825200146:case 1287653388:return d=a.shm.getShapePropertyIndex(5,"label",this.i),null==d&&(d=""),d;case 1639976963:return this.group.getProteinStructureType().getBioStructureTypeName(!1);
case 1237320707:return this.group.getProteinStructureSubType().getBioStructureTypeName(!1);case 1086324749:return this.group.getStructureId();case 1086324748:return a.getHybridizationAndAxes(this.i,null,null,"d");case 1086326798:return this.getElementSymbolIso(!1);case 1088421903:return this.getSymmetryOperatorList(!0)}return""},"JV.Viewer,~N");j(c$,"getCIPChirality",function(a){var b=(this.formalChargeAndFlags&496)>>4;0==b&&(1<this.atomicAndIsotopeNumber&&a)&&(b=this.group.chain.model.ms.getAtomCIPChiralityCode(this),
this.formalChargeAndFlags|=(0==b?3:b)<<4);return JV.JC.getCIPChiralityName(b)},"~B");c(c$,"getCIPChiralityRule",function(){var a=0==this.getCIPChirality(!0).length?-1:(this.formalChargeAndFlags&3584)>>9;return JV.JC.getCIPRuleName(a+1)});j(c$,"setCIPChirality",function(a){this.formalChargeAndFlags=this.formalChargeAndFlags&-4081|a<<4},"~N");j(c$,"getCIPChiralityCode",function(){return(this.formalChargeAndFlags&496)>>4});j(c$,"getInsertionCode",function(){return this.group.getInsertionCode()});c(c$,
"atomPropertyTuple",function(a,b,d){switch(b){case 1073742329:return JU.P3.newP(this);case 1145047051:return this.getFractionalCoordPt(!a.g.legacyJavaFloat,!1,d);case 1145047053:return this.getFractionalCoordPt(!a.g.legacyJavaFloat,!1,d);case 1145045006:return this.group.chain.model.isJmolDataFrame?this.getFractionalCoordPt(!a.g.legacyJavaFloat,!1,d):this.getFractionalUnitCoordPt(!a.g.legacyJavaFloat,!1,d);case 1145047052:return JU.P3.new3(a.antialiased?w(this.sX/2):this.sX,a.getScreenHeight()-(a.antialiased?
w(this.sY/2):this.sY),a.antialiased?w(this.sZ/2):this.sZ);case 1145047055:return this.getVibrationVector();case 1145045008:return a=this.getModulation(),null==a?null:a.getV3();case 1145047050:return this;case 1765808134:return JU.CU.colorPtFromInt(this.group.chain.model.ms.vwr.gdata.getColorArgbOrGray(this.colixAtom),d)}return null},"JV.Viewer,~N,JU.P3");j(c$,"getOffsetResidueAtom",function(a,b){return this.group.getAtomIndex(a,b)},"~S,~N");j(c$,"isCrossLinked",function(a){return this.group.isCrossLinked(a.group)},
"JU.Node");j(c$,"getCrossLinkVector",function(a,b,d){return this.group.getCrossLinkVector(a,b,d)},"JU.Lst,~B,~B");j(c$,"toString",function(){return this.getInfo()});j(c$,"findAtomsLike",function(a){return this.group.chain.model.ms.vwr.getAtomBitSet(a)},"~S");c(c$,"getUnitID",function(a){var b=this.group.chain.model;return b.isBioModel?b.getUnitID(this,a):""},"~N");j(c$,"getFloatProperty",function(a){a=this.group.chain.model.ms.vwr.getDataObj(a,null,1);var b=NaN;if(null!=a)try{b=a[this.i]}catch(d){if(!H(d,
Exception))throw d;}return b},"~S");j(c$,"modelIsRawPDB",function(){var a=this.group.chain.model;return a.isBioModel&&!a.isPdbWithMultipleBonds&&0==a.hydrogenCount});E(c$,"ATOM_INFRAME",1,"ATOM_VISSET",2,"ATOM_VISIBLE",4,"ATOM_NOTHIDDEN",8,"ATOM_NOFLAGS",-64,"ATOM_INFRAME_NOTHIDDEN",9,"ATOM_SHAPE_VIS_MASK",-10,"RADIUS_MAX",16,"RADIUS_GLOBAL",16.1,"MAD_GLOBAL",32200,"CHARGE_OFFSET",24,"FLAG_MASK",15,"VIBRATION_VECTOR_FLAG",1,"IS_HETERO_FLAG",2,"CIP_CHIRALITY_OFFSET",4,"CIP_CHIRALITY_MASK",496,"CIP_CHIRALITY_RULE_OFFSET",
9,"CIP_CHIRALITY_RULE_MASK",3584,"CIP_MASK",4080)});s("JM");v(["JU.V3"],"JM.AtomCollection","java.lang.Float java.util.Arrays $.Hashtable JU.A4 $.AU $.BS $.Lst $.M3 $.Measure $.P3 $.PT J.api.Interface $.JmolModulationSet J.atomdata.RadiusData J.c.PAL $.VDW JM.Group JS.T JU.BSUtil $.Elements $.Logger $.Parser $.Vibration".split(" "),function(){c$=t(function(){this.at=this.bioModelset=this.g3d=this.vwr=null;this.ac=0;this.labeler=this.pointGroup=this.trajectory=null;this.maxVanderwaalsRadius=this.maxBondingRadius=
1.4E-45;this.hasBfactorRange=!1;this.bfactor100Hi=this.bfactor100Lo=0;this.haveBSClickable=this.haveBSVisible=!1;this.bsSurface=null;this.surfaceDistanceMax=this.nSurfaceAtoms=0;this.haveChirality=!1;this.bspf=null;this.canSkipLoad=this.preserveState=!0;this.haveStraightness=!1;this.aaRet=this.bsPartialCharges=this.hydrophobicities=this.bondingRadii=this.partialCharges=this.bfactor100s=this.occupancies=this.vibrations=this.dssrData=this.atomSeqIDs=this.atomResnos=this.atomSerials=this.atomTypes=this.atomNames=
this.tainted=this.surfaceDistance100s=this.atomTensors=this.atomTensorList=this.bsModulated=this.bsClickable=this.bsVisible=this.bsHidden=null;R("JM.AtomCollection.AtomSorter")||JM.AtomCollection.$AtomCollection$AtomSorter$();this.atomCapacity=0;p(this,arguments)},JM,"AtomCollection");c(c$,"getAtom",function(a){return 0<=a&&a<this.at.length?this.at[a]:null},"~N");c(c$,"setupAC",function(){this.bsHidden=new JU.BS;this.bsVisible=new JU.BS;this.bsClickable=new JU.BS;null==JM.AtomCollection.userSettableValues&&
(JM.AtomCollection.userSettableValues="atomName atomType coord element formalCharge hydrophobicity ionic occupancy partialCharge temperature valence vanderWaals vibrationVector atomNo seqID resNo chain".$plit(" "))});c(c$,"releaseModelSetAC",function(){this.bfactor100s=this.occupancies=this.vibrations=this.atomSeqIDs=this.atomSerials=this.dssrData=this.atomResnos=this.atomTypes=this.atomNames=this.tainted=this.bsSurface=this.surfaceDistance100s=this.bspf=this.g3d=this.vwr=this.at=null;this.resetPartialCharges();
this.atomTensors=this.bondingRadii=null});c(c$,"mergeAtomArrays",function(a){this.tainted=a.tainted;this.atomNames=a.atomNames;this.atomTypes=a.atomTypes;this.atomResnos=a.atomResnos;this.dssrData=a.dssrData;this.atomSerials=a.atomSerials;this.atomSeqIDs=a.atomSeqIDs;this.vibrations=a.vibrations;this.occupancies=a.occupancies;this.bfactor100s=a.bfactor100s;this.bondingRadii=a.bondingRadii;this.partialCharges=a.partialCharges;this.bsPartialCharges=a.bsPartialCharges;this.atomTensors=a.atomTensors;
this.atomTensorList=a.atomTensorList;this.bsModulated=a.bsModulated;this.haveStraightness=!1;this.surfaceDistance100s=null},"JM.AtomCollection");c(c$,"getAtomPointVector",function(a){var b=new JU.Lst,d=this.ac;if(null!=a)for(var c=a.nextSetBit(0);0<=c&&c<d;c=a.nextSetBit(c+1))b.addLast(this.at[c]);return b},"JU.BS");c(c$,"modelSetHasVibrationVectors",function(){return null!=this.vibrations});c(c$,"getAtomTypes",function(){return this.atomTypes});c(c$,"getPartialCharges",function(){return this.partialCharges});
c(c$,"getBondingRadii",function(){return this.bondingRadii});c(c$,"getBFactors",function(){return this.bfactor100s});c(c$,"getHydrophobicity",function(){return this.hydrophobicities});c(c$,"setBsHidden",function(a){this.bsHidden=a},"JU.BS");c(c$,"isAtomHidden",function(a){return this.bsHidden.get(a)},"~N");c(c$,"getLabeler",function(){return null==this.labeler?this.labeler=J.api.Interface.getInterface("JM.LabelToken",this.vwr,"ms"):this.labeler});c(c$,"getAtomInfo",function(a,b,d){return null==b?
this.at[a].getInfo():this.getLabeler().formatLabel(this.vwr,this.at[a],b,d)},"~N,~S,JU.P3");c(c$,"getElementName",function(a){return JU.Elements.elementNameFromNumber(this.at[a].getAtomicAndIsotopeNumber())},"~N");c(c$,"getQuaternion",function(a,b){return 0>a?null:this.at[a].group.getQuaternion(b)},"~N,~S");c(c$,"getFirstAtomIndexFromAtomNumber",function(a,b){for(var d=0;d<this.ac;d++){var c=this.at[d];if(null!=c&&c.getAtomNumber()==a&&b.get(c.mi))return d}return-1},"~N,JU.BS");c(c$,"setFormalCharges",
function(a,b){if(null!=a){this.resetPartialCharges();for(var d=a.nextSetBit(0);0<=d;d=a.nextSetBit(d+1))this.at[d].setFormalCharge(b),this.taintAtom(d,4)}},"JU.BS,~N");c(c$,"getAtomicCharges",function(){for(var a=K(this.ac,0),b=this.ac;0<=--b;)a[b]=null==this.at[b]?0:this.at[b].getElementNumber();return a});c(c$,"getRadiusVdwJmol",function(a){return JU.Elements.getVanderwaalsMar(a.getElementNumber(),J.c.VDW.JMOL)/1E3},"JM.Atom");c(c$,"getMaxVanderwaalsRadius",function(){1.4E-45==this.maxVanderwaalsRadius&&
this.findMaxRadii();return this.maxVanderwaalsRadius});c(c$,"findMaxRadii",function(){for(var a,b=this.ac;0<=--b;){var d=this.at[b];if(null!=d){if((a=d.getBondingRadius())>this.maxBondingRadius)this.maxBondingRadius=a;if((a=d.getVanderwaalsRadiusFloat(this.vwr,J.c.VDW.AUTO))>this.maxVanderwaalsRadius)this.maxVanderwaalsRadius=a}}});c(c$,"clearBfactorRange",function(){this.hasBfactorRange=!1});c(c$,"calcBfactorRange",function(a){if(!this.hasBfactorRange){this.bfactor100Lo=2147483647;this.bfactor100Hi=
-2147483648;if(null==a)for(var b=0;b<this.ac;b++)this.setBf(b);else for(b=a.nextSetBit(0);0<=b;b=a.nextSetBit(b+1))this.setBf(b);this.hasBfactorRange=!0}},"JU.BS");c(c$,"setBf",function(a){null!=this.at[a]&&(a=this.at[a].getBfactor100(),a<this.bfactor100Lo?this.bfactor100Lo=a:a>this.bfactor100Hi&&(this.bfactor100Hi=a))},"~N");c(c$,"getBfactor100Lo",function(){this.hasBfactorRange||(this.vwr.g.rangeSelected?this.calcBfactorRange(this.vwr.bsA()):this.calcBfactorRange(null));return this.bfactor100Lo});
c(c$,"getBfactor100Hi",function(){this.getBfactor100Lo();return this.bfactor100Hi});c(c$,"getSurfaceDistanceMax",function(){null==this.surfaceDistance100s&&this.calcSurfaceDistances();return this.surfaceDistanceMax});c(c$,"calculateVolume",function(a,b){var d=0;if(null!=a)for(var c=a.nextSetBit(0);0<=c;c=a.nextSetBit(c+1))d+=this.at[c].getVolume(this.vwr,b);return d},"JU.BS,J.c.VDW");c(c$,"getSurfaceDistance100",function(a){if(0==this.nSurfaceAtoms)return-1;null==this.surfaceDistance100s&&this.calcSurfaceDistances();
return this.surfaceDistance100s[a]},"~N");c(c$,"calcSurfaceDistances",function(){this.calculateSurface(null,-1)});c(c$,"calculateSurface",function(a,b){0>b&&(b=3);var d=J.api.Interface.getOption("geodesic.EnvelopeCalculation",this.vwr,"ms").set(this.vwr,this.ac,null);d.calculate(new J.atomdata.RadiusData(null,b,J.atomdata.RadiusData.EnumType.ABSOLUTE,null),3.4028235E38,a,JU.BSUtil.copyInvert(a,this.ac),!1,!1,!1,!0);var c=d.getPoints();this.surfaceDistanceMax=0;this.bsSurface=d.getBsSurfaceClone();
this.surfaceDistance100s=B(this.ac,0);this.nSurfaceAtoms=JU.BSUtil.cardinalityOf(this.bsSurface);if(0==this.nSurfaceAtoms||null==c||0==c.length)return c;for(var d=3.4028235E38==b?0:b,f=0;f<this.ac;f++)if(this.bsSurface.get(f)||null==this.at[f])this.surfaceDistance100s[f]=0;else{for(var e=3.4028235E38,h=this.at[f],k=c.length;0<=--k;)e=Math.min(Math.abs(c[k].distance(h)-d),e);e=this.surfaceDistance100s[f]=w(Math.floor(100*e));this.surfaceDistanceMax=Math.max(this.surfaceDistanceMax,e)}return c},"JU.BS,~N");
c(c$,"setAtomCoord2",function(a,b,d){var c=null,f=null,e=null,h=0,k=1;if(n(d,JU.P3))c=d;else if(n(d,JU.Lst)){e=d;if(0==(k=e.size()))return;h=1}else if(JU.AU.isAP(d)){f=d;if(0==(k=f.length))return;h=2}else return;d=0;if(null!=a)for(var j=a.nextSetBit(0);0<=j;j=a.nextSetBit(j+1)){switch(h){case 1:if(d>=k)return;c=e.get(d++);break;case 2:if(d>=k)return;c=f[d++]}if(null!=c)switch(b){case 1145047050:this.setAtomCoord(j,c.x,c.y,c.z);break;case 1145047051:this.at[j].setFractionalCoordTo(c,!0);this.taintAtom(j,
2);break;case 1145047053:this.at[j].setFractionalCoordTo(c,!1);this.taintAtom(j,2);break;case 1145047055:this.setAtomVibrationVector(j,c)}}},"JU.BS,~N,~O");c(c$,"setAtomVibrationVector",function(a,b){this.setVibrationVector(a,b);this.taintAtom(a,12)},"~N,JU.T3");c(c$,"setAtomCoord",function(a,b,d,c){if(!(0>a||a>=this.ac)){var f=this.at[a];f.set(b,d,c);this.fixTrajectory(f);this.taintAtom(a,2)}},"~N,~N,~N,~N");c(c$,"fixTrajectory",function(a){this.isTrajectory(a.mi)&&this.trajectory.fixAtom(a)},"JM.Atom");
c(c$,"setAtomCoordRelative",function(a,b,d,c){if(!(0>a||a>=this.ac)){var f=this.at[a];f.add3(b,d,c);this.fixTrajectory(f);this.taintAtom(a,2)}},"~N,~N,~N,~N");c(c$,"setAtomsCoordRelative",function(a,b,d,c){if(null!=a)for(var f=a.nextSetBit(0);0<=f;f=a.nextSetBit(f+1))this.setAtomCoordRelative(f,b,d,c)},"JU.BS,~N,~N,~N");c(c$,"setAPa",function(a,b,d,c,f,e,h){var k=0;if(!(null!=e&&0==e.length||null==a)){for(var j=null!=e&&e.length==this.ac||null!=h&&h.length==this.ac,z=a.nextSetBit(0);0<=z;z=a.nextSetBit(z+
1)){j&&(k=z);if(null!=e){if(k>=e.length)return;c=e[k++];if(Float.isNaN(c))continue;d=A(c)}else if(null!=h){if(k>=h.length)return;f=h[k++]}var m=this.at[z],r;switch(b){case 1086326786:this.setAtomName(z,f,!0);break;case 1086326785:this.setAtomType(z,f);break;case 1086326788:this.setChainID(z,f);break;case 1094715393:this.setAtomNumber(z,d,!0);break;case 1094713365:this.setAtomSeqID(z,d);break;case 1111492609:case 1111492629:this.setAtomCoord(z,c,m.y,m.z);break;case 1111492610:case 1111492630:this.setAtomCoord(z,
m.x,c,m.z);break;case 1111492611:case 1111492631:this.setAtomCoord(z,m.x,m.y,c);break;case 1111492626:case 1111492627:case 1111492628:this.setVibrationVector2(z,b,c);break;case 1111492612:case 1111492613:case 1111492614:m.setFractionalCoord(b,c,!0);this.taintAtom(z,2);break;case 1111492615:case 1111492616:case 1111492617:m.setFractionalCoord(b,c,!1);this.taintAtom(z,2);break;case 1094715402:case 1086326789:this.setElement(m,d,!0);break;case 1631586315:this.resetPartialCharges();m.setFormalCharge(d);
this.taintAtom(z,4);break;case 1113589786:this.setHydrophobicity(z,c);break;case 1128269825:r=2>c&&0.01<=c?100*c:c;this.setOccupancy(z,r,!0);break;case 1111492619:this.setPartialCharge(z,c,!0);break;case 1111492618:this.setBondingRadius(z,c);break;case 1111492620:this.setBFactor(z,c,!0);break;case 1094715412:this.setAtomResno(z,d);break;case 1825200146:case 1287653388:this.vwr.shm.setAtomLabel(f,z);break;case 1665140738:case 1112152075:r=c;0>r?r=0:16<r&&(r=16.1);m.madAtom=$(2E3*r);break;case 1113589787:this.vwr.slm.setSelectedAtom(m.i,
0!=c);break;case 1094715417:m.setValence(d);this.taintAtom(z,10);break;case 1648363544:m.setRadius(c)?this.taintAtom(z,11):this.untaint(z,11);break;default:JU.Logger.error("unsettable atom property: "+JS.T.nameOf(b));return}}switch(b){case 1113589787:this.vwr.slm.setSelectedAtom(-1,!1);break;case 1665140738:case 1112152075:this.vwr.setShapeSize(0,2147483647,a)}}},"JU.BS,~N,~N,~N,~S,~A,~A");c(c$,"getVibCoord",function(a,b){var d=null,c=null;switch(b){case "x":case "y":case "z":c=this.getVibration(a,
!1);break;default:d=this.getModulation(a),null!=d&&(c=d.getVibration(!1),null==c&&(c=d))}if(null==c&&null==d)return NaN;switch(b){case "x":case "X":return c.x;case "y":case "Y":return c.y;case "z":case "Z":return c.z;case "O":return d.getModulation("O",null).floatValue();case "1":case "2":case "3":return d=d.getModulation("T",null),d="1"==b?d.x:"2"==b?d.y:d.z,d-Math.floor(d);default:return NaN}},"~N,~S");c(c$,"getVibration",function(a,b){var d=null==this.vibrations?null:this.vibrations[a];return n(d,
J.api.JmolModulationSet)?d.getVibration(b):null==d&&b?new JU.Vibration:d},"~N,~B");c(c$,"getModulation",function(a){a=null==this.vibrations?null:this.vibrations[a];return null!=a&&0<a.modDim?a:null},"~N");c(c$,"setVibrationVector",function(a,b){if(!Float.isNaN(b.x)&&!Float.isNaN(b.y)&&!Float.isNaN(b.z)){if(null==this.vibrations||this.vibrations.length<a)this.vibrations=Array(this.at.length);n(b,JU.Vibration)?this.vibrations[a]=b:(null==this.vibrations[a]&&(this.vibrations[a]=new JU.Vibration),this.vibrations[a].setXYZ(b));
this.at[a].setVibrationVector()}},"~N,JU.T3");c(c$,"setVibrationVector2",function(a,b,d){var c=this.getVibration(a,!0);if(null!=c){switch(b){case 1111492626:c.x=d;break;case 1111492627:c.y=d;break;case 1111492628:c.z=d}this.setAtomVibrationVector(a,c)}},"~N,~N,~N");c(c$,"setAtomName",function(a,b,d){if(!d||!b.equals(this.at[a].getAtomName())){var c=this.am[this.at[a].mi].isBioModel?this.vwr.getJBR().lookupSpecialAtomID(b):0;this.at[a].atomID=c;0>=c&&(null==this.atomNames&&(this.atomNames=Array(this.at.length)),
this.atomNames[a]=b);d&&this.taintAtom(a,0)}},"~N,~S,~B");c(c$,"setAtomType",function(a,b){b.equals(this.at[a].getAtomType())||(null==this.atomTypes&&(this.atomTypes=Array(this.at.length)),this.atomTypes[a]=b)},"~N,~S");c(c$,"setChainID",function(a,b){if(!b.equals(this.at[a].getChainIDStr())){var d=this.at[a].getChainID(),d=this.getChainBits(d);this.at[a].group.chain.chainID=this.vwr.getChainID(b,!0);for(var c=d.nextSetBit(0);0<=c;c=d.nextSetBit(c+1))this.taintAtom(c,16)}},"~N,~S");c(c$,"setAtomNumber",
function(a,b,d){d&&b==this.at[a].getAtomNumber()||(null==this.atomSerials&&(this.atomSerials=B(this.at.length,0)),this.atomSerials[a]=b,d&&this.taintAtom(a,13))},"~N,~N,~B");c(c$,"setElement",function(a,b,d){d&&a.getElementNumber()==b||(a.setAtomicAndIsotopeNumber(b),a.paletteID=J.c.PAL.CPK.id,a.colixAtom=this.vwr.cm.getColixAtomPalette(a,J.c.PAL.CPK.id),this.resetPartialCharges(),d&&this.taintAtom(a.i,3))},"JM.Atom,~N,~B");c(c$,"resetPartialCharges",function(){this.bsPartialCharges=this.partialCharges=
null});c(c$,"setAtomResno",function(a,b){b!=this.at[a].getResno()&&(this.at[a].group.setResno(b),null==this.atomResnos&&(this.atomResnos=B(this.at.length,0)),this.atomResnos[a]=b,this.taintAtom(a,15))},"~N,~N");c(c$,"setAtomSeqID",function(a,b){b!=this.at[a].getSeqID()&&(null==this.atomSeqIDs&&(this.atomSeqIDs=B(this.at.length,0)),this.atomSeqIDs[a]=b,this.taintAtom(a,14))},"~N,~N");c(c$,"setOccupancy",function(a,b,d){if(!(d&&b==this.at[a].getOccupancy100())){if(null==this.occupancies){if(100==b)return;
this.occupancies=K(this.at.length,0);for(var c=this.at.length;0<=--c;)this.occupancies[c]=100}this.occupancies[a]=b;d&&this.taintAtom(a,7)}},"~N,~N,~B");c(c$,"setPartialCharge",function(a,b,d){if(!Float.isNaN(b)){if(null==this.partialCharges){this.bsPartialCharges=new JU.BS;if(0==b)return;this.partialCharges=K(this.at.length,0)}this.bsPartialCharges.set(a);this.partialCharges[a]=b;d&&this.taintAtom(a,8)}},"~N,~N,~B");c(c$,"setBondingRadius",function(a,b){Float.isNaN(b)||b==this.at[a].getBondingRadius()||
(null==this.bondingRadii?this.bondingRadii=K(this.at.length,0):this.bondingRadii.length<this.at.length&&(this.bondingRadii=JU.AU.ensureLength(this.bondingRadii,this.at.length)),this.bondingRadii[a]=b,this.taintAtom(a,6))},"~N,~N");c(c$,"setBFactor",function(a,b,d){if(!(Float.isNaN(b)||d&&b==this.at[a].getBfactor100())){if(null==this.bfactor100s){if(0==b)return;this.bfactor100s=T(this.at.length,0)}this.bfactor100s[a]=ta(100*(-327.68>b?-327.68:327.67<b?327.67:b)+(0>b?-0.5:0.5));d&&this.taintAtom(a,
9)}},"~N,~N,~B");c(c$,"setHydrophobicity",function(a,b){if(!(Float.isNaN(b)||b==this.at[a].getHydrophobicity())){if(null==this.hydrophobicities){this.hydrophobicities=K(this.at.length,0);for(var d=0;d<this.at.length;d++)this.hydrophobicities[d]=JU.Elements.getHydrophobicity(this.at[d].group.groupID)}this.hydrophobicities[a]=b;this.taintAtom(a,5)}},"~N,~N");c(c$,"setAtomData",function(a,b,d,c){var f=null,e=null;switch(a){case 2:this.loadCoordinates(d,!1,!c);return;case 12:this.loadCoordinates(d,!0,
!0);return;case 17:f=K(this.ac,0),e=JU.BS.newN(this.ac)}c=JU.Parser.markLines(d,";");var h=0;try{for(var k=JU.PT.parseInt(d.substring(0,c[0]-1)),j=1;j<=k;j++){var z=JU.PT.getTokens(JU.PT.parseTrimmed(d.substring(c[j],c[j+1]-1))),m=JU.PT.parseInt(z[0])-1;if(!(0>m||m>=this.ac)){var r=this.at[m];h++;var x=z.length-1,y=JU.PT.parseFloat(z[x]);switch(a){case 17:f[m]=y;e.set(m);continue;case 0:this.setAtomName(m,z[x],!0);break;case 13:this.setAtomNumber(m,A(y),!0);break;case 15:this.setAtomResno(m,A(y));
break;case 14:this.setAtomSeqID(m,A(y));break;case 1:this.setAtomType(m,z[x]);break;case 16:this.setChainID(m,z[x]);break;case 3:r.setAtomicAndIsotopeNumber(A(y));r.paletteID=J.c.PAL.CPK.id;r.colixAtom=this.vwr.cm.getColixAtomPalette(r,J.c.PAL.CPK.id);break;case 4:r.setFormalCharge(A(y));break;case 5:this.setHydrophobicity(m,y);break;case 6:this.setBondingRadius(m,y);break;case 8:this.setPartialCharge(m,y,!0);break;case 9:this.setBFactor(m,y,!0);break;case 10:r.setValence(A(y));break;case 11:r.setRadius(y)}this.taintAtom(m,
a)}}17==a&&0<h&&this.vwr.setData(b,u(-1,[b,f,e,Integer.$valueOf(1)]),0,0,0,0,0)}catch(n){if(H(n,Exception))JU.Logger.error("AtomCollection.loadData error: "+n);else throw n;}},"~N,~S,~S,~B");c(c$,"loadCoordinates",function(a,b,d){var c=JU.Parser.markLines(a,";"),f=b?new JU.V3:null;try{for(var e=JU.PT.parseInt(a.substring(0,c[0]-1)),h=1;h<=e;h++){var k=JU.PT.getTokens(JU.PT.parseTrimmed(a.substring(c[h],c[h+1]))),j=JU.PT.parseInt(k[0])-1,z=k[3].equalsIgnoreCase("1.4E-45")?1.4E-45:JU.PT.parseFloat(k[3]),
m=k[4].equalsIgnoreCase("1.4E-45")?1.4E-45:JU.PT.parseFloat(k[4]),r=JU.PT.parseFloat(k[5]);b?(f.set(z,m,r),this.setAtomVibrationVector(j,f)):(this.setAtomCoord(j,z,m,r),d||this.untaint(j,2))}}catch(x){if(H(x,Exception))JU.Logger.error("Frame.loadCoordinate error: "+x);else throw x;}},"~S,~B,~B");c(c$,"validateBspf",function(a){null!=this.bspf&&(this.bspf.isValid=a)},"~B");c(c$,"validateBspfForModel",function(a,b){null!=this.bspf&&this.bspf.validateModel(a,b)},"~N,~B");c(c$,"setPreserveState",function(a){this.preserveState=
a},"~B");c$.getUserSettableType=c(c$,"getUserSettableType",function(a){var b=0==a.indexOf("property_");a=b?a.substring(9):a;for(var d=0;17>d;d++)if(JM.AtomCollection.userSettableValues[d].equalsIgnoreCase(a))return d;return b?17:-1},"~S");c(c$,"getTaintedAtoms",function(a){return null==this.tainted?null:this.tainted[a]},"~N");c(c$,"taintAtoms",function(a,b){this.canSkipLoad=!1;if(this.preserveState)for(var d=a.nextSetBit(0);0<=d;d=a.nextSetBit(d+1))this.taintAtom(d,b)},"JU.BS,~N");c(c$,"taintAtom",
function(a,b){this.preserveState&&(null==this.tainted&&(this.tainted=Array(17)),null==this.tainted[b]&&(this.tainted[b]=JU.BS.newN(this.ac)),this.tainted[b].set(a));2==b&&this.taintModelCoord(a)},"~N,~N");c(c$,"taintModelCoord",function(a){a=this.am[this.at[a].mi];this.validateBspfForModel(a.trajectoryBaseIndex,!1);a.isBioModel&&a.resetDSSR(!0);this.pointGroup=null},"~N");c(c$,"untaint",function(a,b){this.preserveState&&(null==this.tainted||null==this.tainted[b]||this.tainted[b].clear(a))},"~N,~N");
c(c$,"setTaintedAtoms",function(a,b){if(this.preserveState){if(null==a){if(null==this.tainted)return;this.tainted[b]=null;return}null==this.tainted&&(this.tainted=Array(17));null==this.tainted[b]&&(this.tainted[b]=JU.BS.newN(this.ac));JU.BSUtil.copy2(a,this.tainted[b])}if(2==b){var d=a.nextSetBit(0);0<=d&&this.taintModelCoord(d)}},"JU.BS,~N");c(c$,"unTaintAtoms",function(a,b){if(!(null==this.tainted||null==this.tainted[b])){for(var d=a.nextSetBit(0);0<=d;d=a.nextSetBit(d+1))this.tainted[b].clear(d);
0>this.tainted[b].nextSetBit(0)&&(this.tainted[b]=null)}},"JU.BS,~N");c(c$,"findNearest2",function(a,b,d,c,f){for(var e=null,h,k=this.ac;0<=--k;)if(!(null!=c&&c.get(k)||null==(h=this.at[k])))h.isClickable()&&this.isCursorOnTopOf(h,a,b,f,e)&&(e=h);d[0]=e},"~N,~N,~A,JU.BS,~N");c(c$,"isCursorOnTopOf",function(a,b,d,c,f){return 1<a.sZ&&!this.g3d.isClippedZ(a.sZ)&&this.g3d.isInDisplayRange(a.sX,a.sY)&&a.isCursorOnTopOf(b,d,c,f)},"JM.Atom,~N,~N,~N,JM.Atom");c(c$,"fillADa",function(a,b){a.xyz=a.atoms=this.at;
a.ac=this.ac;a.atomicNumber=B(this.ac,0);var d=0!=(b&2);d&&(a.atomRadius=K(this.ac,0));for(var c=0!=(b&16),f=0;f<this.ac;f++){var e=this.at[f];null==e||e.isDeleted()||!c&&0<=a.modelIndex&&e.mi!=a.firstModelIndex?(null==a.bsIgnored&&(a.bsIgnored=new JU.BS),a.bsIgnored.set(f)):(a.atomicNumber[f]=e.getElementNumber(),a.lastModelIndex=e.mi,d&&(a.atomRadius[f]=this.getWorkingRadius(e,a)))}},"J.atomdata.AtomData,~N");c(c$,"getWorkingRadius",function(a,b){var d=0,c=b.radiusData;switch(c.factorType){case J.atomdata.RadiusData.EnumType.ABSOLUTE:d=
c.value;break;case J.atomdata.RadiusData.EnumType.FACTOR:case J.atomdata.RadiusData.EnumType.OFFSET:switch(c.vdwType){case J.c.VDW.BONDING:d=a.getBondingRadius();break;case J.c.VDW.ADPMAX:d=a.getADPMinMax(!0);break;case J.c.VDW.ADPMIN:d=a.getADPMinMax(!1);break;default:d=a.getVanderwaalsRadiusFloat(this.vwr,b.radiusData.vdwType)}d=c.factorType===J.atomdata.RadiusData.EnumType.FACTOR?d*c.value:d+c.value}return d+c.valueExtended},"JM.Atom,J.atomdata.AtomData");c(c$,"calculateHydrogens",function(a,b,
d,c){var f=256==(c&256),e=512==(c&512),h=4==(c&4),k=2048==(c&2048);c=4096==(c&4096);var j=new JU.V3,z=new JU.V3,m=Array(this.ac),r=this.vwr.slm.bsDeleted,x,y=0;if(null!=a)for(var n=a.nextSetBit(0);0<=n;n=a.nextSetBit(n+1))if(!(null!=r&&r.get(n))){var p=this.at[n],q=p.getElementNumber();if(!(e&&6!=q)){var s=6>=q?1.1:10>=q?1:1.3;switch(q){case 7:case 8:s=1}x=f||k||c?p.getCovalentHydrogenCount():0;if(!(f&&0<x||k&&0==x)){var u=this.getMissingHydrogenCount(p,!1);if(!(f&&0==u)){k||(x=u);var u=this.aaRet[0],
t=this.aaRet[2],v=this.aaRet[3]-(k?x:0);if(!(0==v&&p.isHetero())){m[n]=Array(x);var w=0;if(0==v)switch(x){case 4:j.set(0.635,0.635,0.635),x=JU.P3.newP(j),x.add(p),m[n][w++]=x,null!=d&&d.addLast(p);case 3:j.set(-0.635,-0.635,0.635),x=JU.P3.newP(j),x.add(p),m[n][w++]=x,null!=d&&d.addLast(p);case 2:j.set(-0.635,0.635,-0.635),x=JU.P3.newP(j),x.add(p),m[n][w++]=x,null!=d&&d.addLast(p);case 1:j.set(0.635,-0.635,-0.635),x=JU.P3.newP(j),x.add(p),m[n][w++]=x,null!=d&&d.addLast(p)}else switch(x){case 3:this.getHybridizationAndAxes(n,
q,j,z,"sp3b",!1,!0,h);x=new JU.P3;x.scaleAdd2(s,j,p);m[n][w++]=x;null!=d&&d.addLast(p);this.getHybridizationAndAxes(n,q,j,z,"sp3c",!1,!0,h);x=new JU.P3;x.scaleAdd2(s,j,p);m[n][w++]=x;null!=d&&d.addLast(p);this.getHybridizationAndAxes(n,q,j,z,"sp3d",!1,!0,h);x=new JU.P3;x.scaleAdd2(s,j,p);m[n][w++]=x;null!=d&&d.addLast(p);break;case 2:t=2==t||5==q||1==v&&4==u||7==q&&this.isAdjacentSp2(p);this.getHybridizationAndAxes(n,q,j,z,t?"sp2b":3==u?"sp3c":"lpa",!1,!0,h);x=JU.P3.newP(j);x.scaleAdd2(s,j,p);m[n][w++]=
x;null!=d&&d.addLast(p);this.getHybridizationAndAxes(n,q,j,z,t?"sp2c":3==u?"sp3d":"lpb",!1,!0,h);x=JU.P3.newP(j);x.scaleAdd2(s,j,p);m[n][w++]=x;null!=d&&d.addLast(p);break;case 1:switch(u-v){case 1:if(8==q&&p===p.group.getCarbonylOxygenAtom()){m[n]=null;continue}null!=this.getHybridizationAndAxes(n,q,j,z,2==t||5==q||6==q&&1==this.aaRet[1]||7==q&&(new Boolean((new Boolean(p.group.getNitrogenAtom()===p&0==p.getFormalCharge())).valueOf()||this.isAdjacentSp2(p))).valueOf()?"sp2c":"sp3d",!0,!1,h)?(x=JU.P3.newP(j),
x.scaleAdd2(s,j,p),m[n][w++]=x,null!=d&&d.addLast(p)):m[n]=[];break;case 2:this.getHybridizationAndAxes(n,q,j,z,4==u?"sp2c":"sp2b",!1,!1,h);x=JU.P3.newP(j);x.scaleAdd2(s,j,p);m[n][w++]=x;null!=d&&d.addLast(p);break;case 3:this.getHybridizationAndAxes(n,q,j,z,"spb",!1,!0,h),x=JU.P3.newP(j),x.scaleAdd2(s,j,p),m[n][w++]=x,null!=d&&d.addLast(p)}}y+=w}}}}}b[0]=y;return m},"JU.BS,~A,JU.Lst,~N");c(c$,"isAdjacentSp2",function(a){for(var b=a.bonds,d=0;d<b.length;d++)for(var c=b[d].getOtherAtom(a).bonds,f=
0;f<c.length;f++)switch(c[f].getCovalentOrder()){case 2:case 3:return!0}return!1},"JM.Atom");c(c$,"getMissingHydrogenCount",function(a,b){var d=a.getTargetValence();if(0>d)return 0;var c=a.getFormalCharge(),f=a.getValence(),e=this.am[a.mi],e=e.isBioModel&&!e.isPdbWithMultipleBonds?a.group.getGroup3():null;null==this.aaRet&&(this.aaRet=B(5,0));this.aaRet[0]=d;this.aaRet[1]=c;this.aaRet[2]=0;this.aaRet[3]=a.getCovalentBondCount();this.aaRet[4]=null==e?0:f;null!=e&&0==c&&this.bioModelset.getAminoAcidValenceAndCharge(e,
a.getAtomName(),this.aaRet)&&(d=this.aaRet[0],c=this.aaRet[1]);0!=c&&(d+=4==d?-Math.abs(c):c,this.aaRet[0]=d);d-=f;return 0>d&&!b?0:d},"JM.Atom,~B");c(c$,"fixFormalCharges",function(a){for(var b=0,d=a.nextSetBit(0);0<=d;d=a.nextSetBit(d+1)){var c=this.at[d],f=this.getMissingHydrogenCount(c,!0);if(0!=f){var e=c.getFormalCharge(),f=e-f;c.setFormalCharge(f);this.taintAtom(d,4);JU.Logger.debugging&&JU.Logger.debug("atom "+c+" formal charge "+e+" -> "+f);b++}}return b},"JU.BS");c(c$,"getHybridizationAndAxes",
function(a,b,d,c,f,e,h,k){var j=0<f.length&&"-"==f.charAt(0)?f.substring(1):f;if(0<=f.indexOf("d")&&!f.endsWith("sp3d"))return this.getHybridizationAndAxesD(a,d,c,j);f=this.at[a];0==b&&(b=f.getElementNumber());var z=this.getAttached(f,4,e,k),m=z.length,r=j.charCodeAt(j.length-1)-97;if(0>r||6<r)r=0;d.set(0,0,0);c.set(0,0,0);for(var n=Array(4),y=0;y<m;y++){var p=z[y];if(null==p){m=y;break}n[y]=JU.V3.newVsub(f,p);n[y].normalize();d.add(n[y])}0<m&&c.setT(n[0]);p=!1;a=new JU.V3;if(3<=m){2.984513>c.angle(n[1])?
a.cross(c,n[1]):a.cross(c,n[2]);a.normalize();var q=new JU.V3;2.984513>n[1].angle(n[2])?q.cross(n[1],n[2]):q.cross(c,n[2]);q.normalize();p=0.95<=Math.abs(q.dot(a))}var y=0==j.indexOf("sp3"),s=!y&&0==j.indexOf("sp2"),u=!y&&!s&&0==j.indexOf("sp"),t=0==j.indexOf("p"),v=0==j.indexOf("lp"),q=null;if(e){if(0==m)return null;if(y){if(3<r||4<m)return null}else if(s){if(2<r||3<m)return null}else if(u&&(1<r||2<m))return null;switch(m){case 1:if(1==b&&!y)return null;if(y){q="sp3";break}switch(z[0].getCovalentBondCount()){case 1:if(2!=
z[0].getValence()){q="sp";break}case 2:q=u?"sp":"sp2";break;case 3:if(!s&&!t)return null;q="sp2"}break;case 2:if(0.1>d.length()){if(0<=j.indexOf("2")||0<=j.indexOf("3"))return null;q="sp";break}q=y?"sp3":"sp2";if(0==j.indexOf("sp"))break;if(v){q="lp";break}q=j;break;default:if(p&&!k)q="sp2";else{p&&d.setT(a);if(v&&3==m){q="lp";break}q="sp3"}}if(null==q)return null;if(0==j.indexOf("p")){if("sp3"===q)return null}else if(0>j.indexOf(q))return null}if(r<m&&!j.startsWith("p")&&!j.startsWith("l"))return d.sub2(z[r],
f),d.normalize(),q;switch(m){case 0:j.equals("sp3c")||j.equals("sp2d")||j.equals("lpa")?d.set(-0.5,-0.7,1):j.equals("sp3b")||j.equals("lpb")?d.set(0.5,-0.7,-1):j.equals("sp3a")?d.set(0,1,0):d.set(0,0,1);c.set(1,0,0);break;case 1:a.setT(JM.AtomCollection.vRef);c.cross(a,d);if(y){for(y=z[0].getBondCount();0<=--y;)if(z[0].bonds[y].isCovalent()&&z[0].getBondedAtomIndex(y)!=f.i&&(c.sub2(z[0],z[0].bonds[y].getOtherAtom(z[0])),c.cross(d,c),0!=c.length())){c.cross(c,d);break}c.normalize();Float.isNaN(c.x)&&
(c.setT(JM.AtomCollection.vRef),c.cross(c,d));a.cross(d,c);a.normalize();d.normalize();c.scaleAdd2(2.828,c,d);3!=r&&(c.normalize(),(new JU.M3).setAA(JU.A4.new4(d.x,d.y,d.z,2.09439507*(2==r?1:-1))).rotate(c));d.setT(c);c.cross(a,d);break}a.cross(c,d);switch(z[0].getCovalentBondCount()){case 1:if(2!=z[0].getValence())break;case 2:h=z[0];c.setT(d);for(a.setT(JM.AtomCollection.vRef);null!=h&&2==h.getCovalentBondCount();){k=h.bonds;p=null;for(y=0;y<k.length;y++)if(k[y].isCovalent()&&(p=k[y].getOtherAtom(h),
p!==f)){a.sub2(p,h);break}a.cross(a,c);if(0.1<a.length()||2!=p.getCovalentBondCount())break;f=h;h=p}u||(0.1<a.length()?(d.cross(a,c),d.normalize(),1==r&&d.scale(-1),d.scale(JM.AtomCollection.sqrt3_2),d.scaleAdd2(0.5,c,d),t&&(a.cross(d,c),d.setT(a),a.setT(c)),c.cross(a,d)):(d.setT(c),c.cross(JM.AtomCollection.vRef,c)));break;case 3:this.getHybridizationAndAxes(z[0].i,0,c,a,"pz",!1,h,k),a.setT(c),s?(c.cross(c,d),1==r&&c.scale(-1),c.scale(JM.AtomCollection.sqrt3_2),d.scaleAdd2(0.5,d,c)):u||(a.setT(d),
d.setT(c)),c.cross(a,d)}break;case 2:if(0.1>d.length()){if(!j.equals("pz")){p=z[0];(b=3==p.getCovalentBondCount())||(b=3==(p=z[1]).getCovalentBondCount());if(b){this.getHybridizationAndAxes(p.i,0,c,d,"pz",!1,h,k);j.equals("px")&&c.scale(-1);d.setT(n[0]);break}a.setT(JM.AtomCollection.vRef);d.cross(a,c);a.cross(d,c)}d.setT(c);c.cross(a,d);break}a.cross(d,c);if(s){c.cross(d,a);break}if(y||v){a.normalize();d.normalize();j.equals("lp")||(0==r||2==r?d.scaleAdd2(-1.2,a,d):d.scaleAdd2(1.2,a,d));c.cross(d,
a);break}c.cross(d,a);d.setT(a);0>d.z&&(d.scale(-1),c.scale(-1));break;default:if(y)break;if(!p){c.cross(d,c);break}d.setT(a);0>d.z&&h&&(d.scale(-1),c.scale(-1))}c.normalize();d.normalize();return q},"~N,~N,JU.V3,JU.V3,~S,~B,~B,~B");c(c$,"getHybridizationAndAxesD",function(a,b,d,c){c.startsWith("sp3d2")&&(c="d2sp3"+(5==c.length?"a":c.substring(5)));c.startsWith("sp3d")&&(c="dsp3"+(4==c.length?"a":c.substring(4)));if(c.equals("d2sp3")||c.equals("dsp3"))c+="a";var f=c.startsWith("dsp3"),e=c.charCodeAt(c.length-
1)-97;if(null!=b&&(!f&&(5<e||!c.startsWith("d2sp3"))||f&&4<e))return null;a=this.at[a];c=this.getAttached(a,6,!0,!1);if(null==c)return null==b?null:"?";var h=c.length;if(3>h&&null!=b)return null;for(var k=e>=h,j=w(h*(h-1)/2),z=JU.AU.newInt2(j),m=B(3,0),j=B(3,j,0),r=0,n=0,y=0;y<h-1;y++)for(var p=y+1;p<h;p++){var q=JU.Measure.computeAngleABC(c[y],a,c[p],!0),q=105>q?0:150<=q?2:1;j[q][m[q]]=r;m[q]++;z[r++]=B(-1,[y,p]);0==y&&1==q&&n++}r=100*m[0]+10*m[1]+m[2];if(null==b)switch(r){default:return"";case 0:return"";
case 1:return"linear";case 100:case 10:return"bent";case 111:case 201:return"T-shaped";case 30:case 120:case 210:case 300:return 162<Math.abs(JU.Measure.computeTorsion(c[0],a,c[1],c[2],!0))?"trigonal planar":"trigonal pyramidal";case 330:return 1==n%2?"tetrahedral":"uncapped trigonal pyramid";case 60:case 150:case 240:return"tetrahedral";case 402:return"square planar";case 411:case 501:return"see-saw";case 631:return"trigonal bipyramidal";case 802:return"uncapped square pyramid";case 1203:return"octahedral"}switch(r){default:return null;
case 201:break;case 210:case 330:case 411:case 631:if(!f)return null;break;case 300:case 402:case 501:case 802:case 1203:if(f)return null}if(k)if(f)switch(m[1]){case 0:b.sub2(c[z[j[0][0]][0]],a);d.sub2(c[z[j[0][0]][1]],a);b.cross(b,d);b.normalize();4==e&&b.scale(-1);m=this.findNotAttached(h,z,j[2],m[2]);y=m.nextSetBit(0);d.sub2(c[y],a);d.normalize();d.scale(0.5);b.scaleAdd2(JM.AtomCollection.sqrt3_2,b,d);e=-1;break;case 1:4==e?(d=z[j[1][0]],b.add2(c[d[0]],c[d[1]]),b.scaleAdd2(-2,a,b),e=-1):(m=this.findNotAttached(h,
z,j[1],m[1]),e=m.nextSetBit(0));break;default:m=this.findNotAttached(h,z,j[1],m[1]),e=m.nextSetBit(0)}else{r=!1;if(4==h)switch(m[2]){case 1:m=this.findNotAttached(h,z,j[2],m[2]);y=m.nextSetBit(0);e=4==e?y:m.nextSetBit(y+1);break;default:r=!0}else{m=this.findNotAttached(h,z,j[2],m[2]);y=m.nextSetBit(0);for(p=h;p<e&&0<=y;p++)y=m.nextSetBit(y+1);-1==y?r=!0:e=y}r&&(b.sub2(c[z[j[0][0]][0]],a),d.sub2(c[z[j[0][0]][1]],a),b.cross(b,d),4==e&&b.scale(-1),e=-1)}0<=e&&b.sub2(c[e],a);k&&b.scale(-1);b.normalize();
return f?"dsp3":"d2sp3"},"~N,JU.V3,JU.V3,~S");c(c$,"getAttached",function(a,b,d,c){var f=a.getCovalentBondCount();if(f>b)return null;b=Array(f);if(0<f){for(var f=a.bonds,e=0,h=0;h<f.length;h++)if(f[h].isCovalent()){var k=f[h].getOtherAtom(a);if(!c||1!=k.getAtomicAndIsotopeNumber())b[e++]=k}d&&!c&&java.util.Arrays.sort(b,P(JM.AtomCollection.AtomSorter,this,null))}return b},"JM.Atom,~N,~B,~B");c(c$,"findNotAttached",function(a,b,d,c){var f=JU.BS.newN(a);f.setBits(0,a);for(var e=0;e<a;e++)for(var h=
0;h<c;h++){var k=b[d[h]];(k[0]==e||k[1]==e)&&f.clear(e)}return f},"~N,~A,~A,~N");c(c$,"getAtomBitsMDa",function(a,b,d){var c=n(b,Integer)?b.intValue():0;switch(a){case 1086326786:case 1086326785:var f=1086326785==a,e=","+b+",";for(a=this.ac;0<=--a;)null!=this.at[a]&&(b=f?this.at[a].getAtomType():this.at[a].getAtomName(),0<=e.indexOf(","+b+",")&&d.set(a));return d;case 1094715393:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].getAtomNumber()==c&&d.set(a);return d;case 2097155:for(a=this.ac;0<=
--a;)null!=this.at[a]&&0<this.at[a].getCovalentBondCount()&&d.set(a);return d;case 2097188:case 2097156:case 136314895:case 2097166:case 2097168:case 2097170:case 2097172:case 2097174:case 2097184:return this.haveBioModels?this.bioModelset.getAtomBitsBS(a,null,d):d;case 1612709900:c=1;case 1094715402:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].getElementNumber()==c&&d.set(a);return d;case 1612709894:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].isHetero()&&d.set(a);return d;case 1073741824:return this.getIdentifierOrNull(b);
case 2097165:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].isLeadAtom()&&d.set(a);return d;case 1094713362:case 1639976963:return this.haveBioModels?this.bioModelset.getAtomBitsBS(a,b,d):d;case 1094715412:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].getResno()==c&&d.set(a);return d;case 1612709912:b=B(2,0);for(a=this.ac;0<=--a;)if(null!=this.at[a])if(c=this.at[a].group.groupID,42<=c&&45>c)d.set(a);else if(8==(f=this.at[a]).getElementNumber()&&2==f.getCovalentBondCount()){for(var c=f.bonds,
h=0,k=c.length;0<=--k&&3>h;)if(c[k].isCovalent()&&1==(e=c[k].getOtherAtom(f)).getElementNumber())b[h++%2]=e.i;2==h&&(d.set(b[1]),d.set(b[0]),d.set(a))}return d;case 1073742355:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.isAltLoc(this.at[a].altloc,b)&&d.set(a);return d;case 1073742356:f=b.toUpperCase();0<=f.indexOf("\\?")&&(f=JU.PT.rep(f,"\\?","\u0001"));(e=f.startsWith("?*"))&&(f=f.substring(1));for(a=this.ac;0<=--a;)null!=this.at[a]&&this.isAtomNameMatch(this.at[a],f,e,e)&&d.set(a);return d;case 1073742357:return JU.BSUtil.copy(this.getChainBits(c));
case 1073742360:return this.getSpecName(b);case 1073742361:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].group.groupID==c&&d.set(a);return d;case 1073742362:return JU.BSUtil.copy(this.getSeqcodeBits(c,!0));case 5:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].group.getInsCode()==c&&d.set(a);return d;case 1296041986:for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].getSymOp()==c&&d.set(a);return d}e=b.nextSetBit(0);if(0>e)return d;switch(a){case 1094717454:f=JU.BSUtil.copy(b);for(a=e;0<=
a;a=f.nextSetBit(a+1))d.or(this.am[this.at[a].mi].bsAtoms),f.andNot(d);return d;case 1086326788:f=JU.BSUtil.copy(b);for(a=e;0<=a;a=f.nextSetBit(a+1))this.at[a].group.chain.setAtomBits(d),f.andNot(d);return d;case 1086326789:f=new JU.BS;for(a=e;0<=a;a=b.nextSetBit(a+1))f.set(this.at[a].getElementNumber());for(a=this.ac;0<=--a;)null!=this.at[a]&&f.get(this.at[a].getElementNumber())&&d.set(a);return d;case 1086324742:f=JU.BSUtil.copy(b);for(a=e;0<=a;a=f.nextSetBit(a+1))this.at[a].group.setAtomBits(d),
f.andNot(d);return d;case 1094713366:f=new JU.BS;for(a=e;0<=a;a=b.nextSetBit(a+1))f.set(this.at[a].atomSite);for(a=this.ac;0<=--a;)null!=this.at[a]&&f.get(this.at[a].atomSite)&&d.set(a);return d}JU.Logger.error("MISSING getAtomBits entry for "+JS.T.nameOf(a));return d},"~N,~O,JU.BS");c(c$,"getChainBits",function(a){var b=this.vwr.getBoolean(603979822);b?97<=a&&122>=a&&(a+=159):0<=a&&300>a&&(a=this.chainToUpper(a));for(var d=new JU.BS,c=JU.BS.newN(this.ac),f,e=c.nextClearBit(0);e<this.ac;e=c.nextClearBit(e+
1))if(null!=this.at[e]){var h=this.at[e].group.chain;a==(f=h.chainID)||!b&&0<=f&&300>f&&a==this.chainToUpper(f)?(h.setAtomBits(d),c.or(d)):h.setAtomBits(c)}return d},"~N");c(c$,"chainToUpper",function(a){return 97<=a&&122>=a?a-32:256<=a&&300>a?a-191:a},"~N");c(c$,"isAltLoc",function(a,b){if(null==b)return"\x00"==a;if(1!=b.length)return!1;var d=b.charAt(0);return"*"==d||"?"==d&&"\x00"!=a||a==d},"~S,~S");c(c$,"getSeqcodeBits",function(a,b){var d=new JU.BS,c=JM.Group.getSeqNumberFor(a),f=2147483647!=
c,e=!0,h=JM.Group.getInsertionCodeChar(a);switch(h){case "?":for(var k=this.ac;0<=--k;)if(null!=this.at[k]){var j=this.at[k].group.seqcode;if((!f||c==JM.Group.getSeqNumberFor(j))&&0!=JM.Group.getInsertionCodeFor(j))d.set(k),e=!1}break;default:for(k=this.ac;0<=--k;)if(null!=this.at[k]&&(j=this.at[k].group.seqcode,a==j||!f&&a==JM.Group.getInsertionCodeFor(j)||"*"==h&&c==JM.Group.getSeqNumberFor(j)))d.set(k),e=!1}return!e||b?d:null},"~N,~B");c(c$,"getIdentifierOrNull",function(a){var b=this.getSpecNameOrNull(a,
!1);0<=a.indexOf("\\?")&&(a=JU.PT.rep(a,"\\?","\u0001"));return null!=b||0<a.indexOf("?")?b:0<a.indexOf("*")?this.getSpecNameOrNull(a,!0):this.haveBioModels?this.bioModelset.getIdentifierOrNull(a):null},"~S");c(c$,"getSpecName",function(a){var b=this.getSpecNameOrNull(a,!1);if(null!=b)return b;0<a.indexOf("*")&&(b=this.getSpecNameOrNull(a,!0));return null==b?new JU.BS:b},"~S");c(c$,"getSpecNameOrNull",function(a,b){var d=null;a=a.toUpperCase();0<=a.indexOf("\\?")&&(a=JU.PT.rep(a,"\\?","\u0001"));
var c=a.startsWith("?*");c&&(a=a.substring(1));for(var f=this.ac;0<=--f;)if(null!=this.at[f]){var e=this.at[f].getGroup3(!0);if(null!=e&&0<e.length){if(JU.PT.isMatch(e,a,b,!0)){null==d&&(d=JU.BS.newN(f+1));for(d.set(f);0<=--f&&this.at[f].getGroup3(!0).equals(e);)d.set(f);f++}}else this.isAtomNameMatch(this.at[f],a,b,c)&&(null==d&&(d=JU.BS.newN(f+1)),d.set(f))}return d},"~S,~B");c(c$,"isAtomNameMatch",function(a,b,d,c){return JU.PT.isMatch(a.getAtomName().toUpperCase(),b,d,c)},"JM.Atom,~S,~B,~B");
c(c$,"getAtomIndices",function(a){for(var b=0,d=B(this.ac,0),c=a.nextSetBit(0);0<=c&&c<this.ac;c=a.nextSetBit(c+1))d[c]=++b;return d},"JU.BS");c(c$,"getAtomsNearPlane",function(a,b){for(var d=new JU.BS,c=this.ac;0<=--c;){var f=this.at[c];if(null!=f){var e=JU.Measure.distanceToPlane(b,f);(0<a&&-0.1<=e&&e<=a||0>a&&0.1>=e&&e>=a||0==a&&0.01>Math.abs(e))&&d.set(f.i)}}return d},"~N,JU.P4");c(c$,"clearVisibleSets",function(){this.haveBSClickable=this.haveBSVisible=!1});c(c$,"getAtomsInFrame",function(a){this.clearVisibleSets();
a.clearAll();for(var b=this.ac;0<=--b;)null!=this.at[b]&&this.at[b].isVisible(1)&&a.set(b)},"JU.BS");c(c$,"getVisibleSet",function(a){if(a)this.vwr.setModelVisibility(),this.vwr.shm.finalizeAtoms(null,!0);else if(this.haveBSVisible)return this.bsVisible;this.bsVisible.clearAll();for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].checkVisible()&&this.bsVisible.set(a);null!=this.vwr.shm.bsSlabbedInternal&&this.bsVisible.andNot(this.vwr.shm.bsSlabbedInternal);this.haveBSVisible=!0;return this.bsVisible},
"~B");c(c$,"getClickableSet",function(a){if(a)this.vwr.setModelVisibility();else if(this.haveBSClickable)return this.bsClickable;this.bsClickable.clearAll();for(a=this.ac;0<=--a;)null!=this.at[a]&&this.at[a].isClickable()&&this.bsClickable.set(a);this.haveBSClickable=!0;return this.bsClickable},"~B");c(c$,"isModulated",function(a){return null!=this.bsModulated&&this.bsModulated.get(a)},"~N");c(c$,"deleteModelAtoms",function(a,b,d){this.at=JU.AU.deleteElements(this.at,a,b);this.ac=this.at.length;for(var c=
a;c<this.ac;c++)this.at[c].i=c,this.at[c].mi--;null!=this.bsModulated&&JU.BSUtil.deleteBits(this.bsModulated,d);this.deleteAtomTensors(d);this.atomNames=JU.AU.deleteElements(this.atomNames,a,b);this.atomTypes=JU.AU.deleteElements(this.atomTypes,a,b);this.atomResnos=JU.AU.deleteElements(this.atomResnos,a,b);this.atomSerials=JU.AU.deleteElements(this.atomSerials,a,b);this.atomSeqIDs=JU.AU.deleteElements(this.atomSeqIDs,a,b);this.dssrData=JU.AU.deleteElements(this.dssrData,a,b);this.bfactor100s=JU.AU.deleteElements(this.bfactor100s,
a,b);this.hasBfactorRange=!1;this.occupancies=JU.AU.deleteElements(this.occupancies,a,b);this.resetPartialCharges();this.atomTensorList=JU.AU.deleteElements(this.atomTensorList,a,b);this.vibrations=JU.AU.deleteElements(this.vibrations,a,b);this.nSurfaceAtoms=0;this.surfaceDistance100s=this.bsSurface=null;if(null!=this.tainted)for(a=0;17>a;a++)JU.BSUtil.deleteBits(this.tainted[a],d)},"~N,~N,JU.BS");c(c$,"getAtomIdentityInfo",function(a,b,d){b.put("_ipt",Integer.$valueOf(a));b.put("atomIndex",Integer.$valueOf(a));
b.put("atomno",Integer.$valueOf(this.at[a].getAtomNumber()));b.put("info",this.getAtomInfo(a,null,d));b.put("sym",this.at[a].getElementSymbol())},"~N,java.util.Map,JU.P3");c(c$,"getAtomTensorList",function(a){return 0>a||null==this.atomTensorList||a>=this.atomTensorList.length?null:this.atomTensorList[a]},"~N");c(c$,"deleteAtomTensors",function(a){if(null!=this.atomTensors){for(var b=new JU.Lst,d,c=this.atomTensors.keySet().iterator();c.hasNext()&&((d=c.next())||1);){for(var f=this.atomTensors.get(d),
e=f.size();0<=--e;){var h=f.get(e);(a.get(h.atomIndex1)||0<=h.atomIndex2&&a.get(h.atomIndex2))&&f.removeItemAt(e)}0==f.size()&&b.addLast(d)}for(e=b.size();0<=--e;)this.atomTensors.remove(b.get(e))}},"JU.BS");c(c$,"setCapacity",function(a){this.atomCapacity+=a},"~N");c(c$,"setAtomTensors",function(a,b){if(!(null==b||0==b.size())){null==this.atomTensors&&(this.atomTensors=new java.util.Hashtable);null==this.atomTensorList&&(this.atomTensorList=Array(this.at.length));this.atomTensorList=JU.AU.ensureLength(this.atomTensorList,
this.at.length);this.atomTensorList[a]=JM.AtomCollection.getTensorList(b);for(var d=b.size();0<=--d;){var c=b.get(d);c.atomIndex1=a;c.atomIndex2=-1;c.modelIndex=this.at[a].mi;this.addTensor(c,c.type);null!=c.altType&&this.addTensor(c,c.altType)}}},"~N,JU.Lst");c(c$,"addTensor",function(a,b){b=b.toLowerCase();var d=this.atomTensors.get(b);null==d&&(this.atomTensors.put(b,d=new JU.Lst),d.ensureCapacity(this.atomCapacity));d.addLast(a)},"JU.Tensor,~S");c$.getTensorList=c(c$,"getTensorList",function(a){for(var b=
-1,d=!1,c=a.size(),f=c;0<=--f;){var e=a.get(f);e.forThermalEllipsoid?b=f:2==e.iType&&(d=!0)}var h=Array((0<=b||!d?0:1)+c);if(0<=b&&(h[0]=a.get(b),1==a.size()))return h;if(d){b=0;for(f=c;0<=--f;)e=a.get(f),e.forThermalEllipsoid||(h[++b]=e)}else for(f=0;f<c;f++)h[f]=a.get(f);return h},"JU.Lst");c(c$,"getAtomTensor",function(a,b){var d=this.getAtomTensorList(a);if(null!=d&&null!=b){b=b.toLowerCase();for(var c=0;c<d.length;c++){var f=d[c];if(null!=f&&(b.equals(f.type)||b.equals(f.altType)))return f}}return null},
"~N,~S");c(c$,"getAllAtomTensors",function(a){if(null==this.atomTensors)return null;if(null!=a)return this.atomTensors.get(a.toLowerCase());a=new JU.Lst;for(var b,d=this.atomTensors.entrySet().iterator();d.hasNext()&&((b=d.next())||1);)a.addAll(b.getValue());return a},"~S");c(c$,"scaleVectorsToMax",function(a){if(null!=this.vibrations){for(var b=0,d=JU.BS.newN(this.ac),c=this.vibrations.length;0<=--c;){var f=this.getVibration(c,!1);if(null!=f&&(-1==f.modDim||-2==f.modDim))b=Math.max(b,f.length()),
d.set(c)}if(!(b==a||0==b)){b=a/b;a=!1;for(c=d.nextSetBit(0);0<=c;c=d.nextSetBit(c+1)){var f=this.getVibration(c,!1),e=this.getModulation(c);if(null==e){if(0==b)return;f.scale(b)}else e.scaleVibration(b);a||(this.taintAtom(c,12),a=!0)}this.tainted[12].or(d)}}},"~N");c(c$,"getAtomsFromAtomNumberInFrame",function(a){for(var b=this.vwr.getFrameAtoms(),d=b.nextSetBit(0);0<=d;d=b.nextSetBit(d+1))this.at[d].getAtomNumber()!=a&&b.clear(d);return b},"~N");c(c$,"generateCrystalClass",function(a,b){var d=0>
a||a>=this.ac?null:this.at[a].getUnitCell(),c=null!=b&&Float.isNaN(b.x);return null==d?new JU.Lst:d.generateCrystalClass(c?null:null!=b?b:this.at[a])},"~N,JU.P3");c$.$AtomCollection$AtomSorter$=function(){M(self.c$);c$=t(function(){V(this,arguments);p(this,arguments)},JM.AtomCollection,"AtomSorter",null,java.util.Comparator);j(c$,"compare",function(a,b){return a.i>b.i?1:a.i<b.i?-1:0},"JM.Atom,JM.Atom");c$=L()};E(c$,"almost180",2.984513);c$.sqrt3_2=c$.prototype.sqrt3_2=Math.sqrt(3)/2;c$.vRef=c$.prototype.vRef=
JU.V3.new3(3.14159,2.71828,1.41421);E(c$,"userSettableValues",null,"TAINT_ATOMNAME",0,"TAINT_ATOMTYPE",1,"TAINT_COORD",2,"TAINT_ELEMENT",3,"TAINT_FORMALCHARGE",4,"TAINT_HYDROPHOBICITY",5,"TAINT_BONDINGRADIUS",6,"TAINT_OCCUPANCY",7,"TAINT_PARTIALCHARGE",8,"TAINT_TEMPERATURE",9,"TAINT_VALENCE",10,"TAINT_VANDERWAALS",11,"TAINT_VIBRATION",12,"TAINT_ATOMNO",13,"TAINT_SEQID",14,"TAINT_RESNO",15,"TAINT_CHAIN",16,"TAINT_MAX",17,"CALC_H_DOALL",256,"CALC_H_JUSTC",512,"CALC_H_HAVEH",1024,"CALC_H_QUICK",4,"CALC_H_IGNORE_H",
2048,"CALC_H_ALLOW_H",4096)});s("JM");v(["J.api.AtomIndexIterator"],"JM.AtomIteratorWithinModel",["J.atomdata.RadiusData"],function(){c$=t(function(){this.bspf=this.cubeIterator=null;this.isZeroBased=this.hemisphereOnly=this.threadSafe=!1;this.modelIndex=2147483647;this.atomIndex=-1;this.distanceSquared=this.zeroBase=0;this.bsSelected=null;this.checkGreater=this.isGreaterOnly=!1;this.radiusData=null;this.vdw1=0;this.isVdw=!1;this.vwr=this.atoms=null;this.iNext=0;p(this,arguments)},JM,"AtomIteratorWithinModel",
null,J.api.AtomIndexIterator);q(c$,function(){});c(c$,"initialize",function(a,b,d,c,f,e){this.bspf=a;this.bsSelected=b;this.isGreaterOnly=d;this.isZeroBased=c;this.hemisphereOnly=f;this.threadSafe=e;this.cubeIterator=null},"J.bspt.Bspf,JU.BS,~B,~B,~B,~B");j(c$,"setModel",function(a,b,d,c,f,e,h){this.threadSafe&&(b=-1-b);if(b!=this.modelIndex||null==this.cubeIterator)this.cubeIterator=this.bspf.getCubeIterator(b),this.modelIndex=b;this.zeroBase=this.isZeroBased?d:0;if(-2147483648!=e){this.atomIndex=
0>e?-1:c;if(this.isVdw=null!=h)this.radiusData=h,this.atoms=a.at,this.vwr=a.vwr,e=h.factorType===J.atomdata.RadiusData.EnumType.OFFSET?5+h.value:5*h.value,this.vdw1=this.atoms[c].getVanderwaalsRadiusFloat(this.vwr,h.vdwType);this.checkGreater=this.isGreaterOnly&&2147483647!=c;this.setCenter(f,e)}},"JM.ModelSet,~N,~N,~N,JU.T3,~N,J.atomdata.RadiusData");j(c$,"setCenter",function(a,b){this.setCenter2(a,b)},"JU.T3,~N");c(c$,"setCenter2",function(a,b){null!=this.cubeIterator&&(this.cubeIterator.initialize(a,
b,this.hemisphereOnly),this.distanceSquared=b*b)},"JU.T3,~N");j(c$,"hasNext",function(){return this.hasNext2()});c(c$,"hasNext2",function(){if(0<=this.atomIndex)for(;this.cubeIterator.hasMoreElements();){var a=this.cubeIterator.nextElement();if((this.iNext=a.i)!=this.atomIndex&&(!this.checkGreater||this.iNext>this.atomIndex)&&(null==this.bsSelected||this.bsSelected.get(this.iNext)))return!0}else if(this.cubeIterator.hasMoreElements())return a=this.cubeIterator.nextElement(),this.iNext=a.i,!0;this.iNext=
-1;return!1});j(c$,"next",function(){return this.iNext-this.zeroBase});j(c$,"foundDistance2",function(){return null==this.cubeIterator?-1:this.cubeIterator.foundDistance2()});j(c$,"addAtoms",function(a){for(var b;this.hasNext();)if(0<=(b=this.next())){var d;if(this.isVdw){d=this.atoms[b].getVanderwaalsRadiusFloat(this.vwr,this.radiusData.vdwType)+this.vdw1;switch(this.radiusData.factorType){case J.atomdata.RadiusData.EnumType.OFFSET:d+=2*this.radiusData.value;break;case J.atomdata.RadiusData.EnumType.FACTOR:d*=
this.radiusData.value}d*=d}else d=this.distanceSquared;this.foundDistance2()<=d&&a.set(b)}},"JU.BS");j(c$,"release",function(){null!=this.cubeIterator&&(this.cubeIterator.release(),this.cubeIterator=null)});j(c$,"getPosition",function(){return null})});s("JM");v(["JM.AtomIteratorWithinModel"],"JM.AtomIteratorWithinModelSet",null,function(){c$=t(function(){this.center=this.bsModels=null;this.distance=0;p(this,arguments)},JM,"AtomIteratorWithinModelSet",JM.AtomIteratorWithinModel);q(c$,function(a){I(this,
JM.AtomIteratorWithinModelSet,[]);this.bsModels=a},"JU.BS");j(c$,"setCenter",function(a,b){this.center=a;this.distance=b;this.set(0)},"JU.T3,~N");c(c$,"set",function(a){if(0>(this.modelIndex=this.bsModels.nextSetBit(a))||null==(this.cubeIterator=this.bspf.getCubeIterator(this.modelIndex)))return!1;this.setCenter2(this.center,this.distance);return!0},"~N");j(c$,"hasNext",function(){return this.hasNext2()?!0:!this.set(this.modelIndex+1)?!1:this.hasNext()})});s("JM");v(["JU.Edge","JV.JC"],"JM.Bond",
["JU.C"],function(){c$=t(function(){this.atom2=this.atom1=null;this.shapeVisibilityFlags=this.colix=this.mad=0;p(this,arguments)},JM,"Bond",JU.Edge);q(c$,function(a,b,d,c,f){I(this,JM.Bond,[]);this.atom1=a;this.atom2=b;this.colix=f;this.setOrder(d);this.setMad(c)},"JM.Atom,JM.Atom,~N,~N,~N");c(c$,"setMad",function(a){this.mad=a;this.setShapeVisibility(0!=a)},"~N");c(c$,"setShapeVisibility",function(a){0!=(this.shapeVisibilityFlags&JM.Bond.myVisibilityFlag)!=a&&(this.atom1.addDisplayedBond(JM.Bond.myVisibilityFlag,
a),this.atom2.addDisplayedBond(JM.Bond.myVisibilityFlag,a),this.shapeVisibilityFlags=a?this.shapeVisibilityFlags|JM.Bond.myVisibilityFlag:this.shapeVisibilityFlags&~JM.Bond.myVisibilityFlag)},"~B");c(c$,"getIdentity",function(){return this.index+1+" "+JU.Edge.getBondOrderNumberFromOrder(this.order)+" "+this.atom1.getInfo()+" -- "+this.atom2.getInfo()+" "+this.atom1.distance(this.atom2)});j(c$,"isCovalent",function(){return 0!=(this.order&1023)});j(c$,"isHydrogen",function(){return JU.Edge.isOrderH(this.order)});
c(c$,"isStereo",function(){return 0!=(this.order&1024)});c(c$,"isPartial",function(){return 0!=(this.order&224)});c(c$,"isAromatic",function(){return 0!=(this.order&512)});c(c$,"getEnergy",function(){return 0});c(c$,"getValence",function(){return!this.isCovalent()?0:this.isPartial()||this.is(515)?1:this.order&7});c(c$,"deleteAtomReferences",function(){null!=this.atom1&&this.atom1.deleteBond(this);null!=this.atom2&&this.atom2.deleteBond(this);this.atom1=this.atom2=null});c(c$,"setTranslucent",function(a,
b){this.colix=JU.C.getColixTranslucent3(this.colix,a,b)},"~B,~N");c(c$,"setOrder",function(a){16==this.atom1.getElementNumber()&&16==this.atom2.getElementNumber()&&(a|=256);512==a&&(a=515);this.order=a|this.order&131072},"~N");j(c$,"getAtomIndex1",function(){return this.atom1.i});j(c$,"getAtomIndex2",function(){return this.atom2.i});j(c$,"getCovalentOrder",function(){return JU.Edge.getCovalentBondOrder(this.order)});c(c$,"getOtherAtom",function(a){return this.atom1===a?this.atom2:this.atom2===a?this.atom1:
null},"JM.Atom");c(c$,"is",function(a){return(this.order&-131073)==a},"~N");j(c$,"getOtherNode",function(a){return this.atom1===a?this.atom2:this.atom2===a||null==a?this.atom1:null},"JU.SimpleNode");c(c$,"setAtropisomerOptions",function(a,b){b.get(this.atom1.i);var d,c=2147483647,f=this.atom1.bonds;for(d=0;d<f.length;d++){var e=f[d].getOtherAtom(this.atom1);if(e!==this.atom2)break}if(d<f.length){f=this.atom2.bonds;for(c=0;c<f.length&&!(e=f[c].getOtherAtom(this.atom2),e!==this.atom1);c++);}this.order=
2<d||c>=f.length||2<c?1:JU.Edge.getAtropismOrder(d+1,c+1)},"JU.BS,JU.BS");j(c$,"getCIPChirality",function(){return""},"~B");j(c$,"setCIPChirality",function(){},"~N");j(c$,"toString",function(){return this.atom1+" - "+this.atom2});j(c$,"getAtom",function(a){return 1==a?this.atom2:this.atom1},"~N");c$.myVisibilityFlag=c$.prototype.myVisibilityFlag=JV.JC.getShapeVisibilityFlag(1)});s("JM");v(["JM.AtomCollection"],"JM.BondCollection","JU.AU $.BS JM.Bond $.BondIteratorSelected $.BondSet $.HBond JU.BSUtil $.C $.Edge".split(" "),
function(){c$=t(function(){this.bo=null;this.bondCount=0;this.molecules=this.freeBonds=this.numCached=null;this.defaultCovalentMad=this.moleculeCount=0;this.bsAromatic=this.bsAromaticDouble=this.bsAromaticSingle=null;this.haveAtropicBonds=this.haveHiddenBonds=!1;p(this,arguments)},JM,"BondCollection",JM.AtomCollection);c(c$,"setupBC",function(){this.bsAromatic=new JU.BS;this.numCached=B(5,0);this.freeBonds=Array(5);for(var a=5;0<--a;)this.freeBonds[a]=Array(200);this.setupAC()});c(c$,"releaseModelSetBC",
function(){this.freeBonds=this.bo=null;this.releaseModelSetAC()});c(c$,"getBondIteratorForType",function(a,b){return new JM.BondIteratorSelected(this.bo,this.bondCount,a,b,this.vwr.getBoolean(603979812))},"~N,JU.BS");c(c$,"getBondIterator",function(a){return new JM.BondIteratorSelected(this.bo,this.bondCount,131071,a,!1)},"JU.BS");c(c$,"getBondColix1",function(a){return JU.C.getColixInherited(this.bo[a].colix,this.bo[a].atom1.colixAtom)},"~N");c(c$,"getBondColix2",function(a){return JU.C.getColixInherited(this.bo[a].colix,
this.bo[a].atom2.colixAtom)},"~N");c(c$,"getBondCountInModel",function(a){for(var b=0,d=this.bondCount;0<=--d;)this.bo[d].atom1.mi==a&&b++;return b},"~N");c(c$,"getBondsForSelectedAtoms",function(a,b){for(var d=new JU.BS,c=0;c<this.bondCount;++c){var f=this.bo[c],e=a.get(f.atom1.i),f=a.get(f.atom2.i);(b?e||f:e&&f)&&d.set(c)}return d},"JU.BS,~B");c(c$,"bondAtoms",function(a,b,d,c,f,e,h,k){d=this.getOrAddBond(a,b,d,c,f,e,!0);k&&(d.order|=131072,h&&(a.group=b.group,a.group.addAtoms(a.i)));return d},
"JM.Atom,JM.Atom,~N,~N,JU.BS,~N,~B,~B");c(c$,"getOrAddBond",function(a,b,d,c,f,e,h){if(131071==d||65535==d)d=1;a.isBonded(b)?(a=a.getBond(b).index,h&&(this.bo[a].setOrder(d),this.bo[a].setMad(c),n(this.bo[a],JM.HBond)&&(this.bo[a].energy=e))):(this.bondCount==this.bo.length&&(this.bo=JU.AU.arrayCopyObject(this.bo,this.bondCount+250)),a=this.setBond(this.bondCount++,this.bondMutually(a,b,d,c,e)).index);null!=f&&f.set(a);return this.bo[a]},"JM.Atom,JM.Atom,~N,~N,JU.BS,~N,~B");c(c$,"setBond",function(a,
b){return this.bo[b.index=a]=b},"~N,JM.Bond");c(c$,"bondMutually",function(a,b,d,c,f){d=JU.Edge.isOrderH(d)?new JM.HBond(a,b,d,c,0,f):new JM.Bond(a,b,d,c,0);this.addBondToAtom(a,d);this.addBondToAtom(b,d);return d},"JM.Atom,JM.Atom,~N,~N,~N");c(c$,"addBondToAtom",function(a,b){null==a.bonds?(a.bonds=Array(1),a.bonds[0]=b):a.bonds=this.addToBonds(b,a.bonds)},"JM.Atom,JM.Bond");c(c$,"addToBonds",function(a,b){var d;if(null==b)d=0<this.numCached[1]?this.freeBonds[1][--this.numCached[1]]:Array(1),d[0]=
a;else{var c=b.length;d=c+1;d=5>d&&0<this.numCached[d]?this.freeBonds[d][--this.numCached[d]]:Array(d);d[c]=a;for(var f=c;0<=--f;)d[f]=b[f];5>c&&200>this.numCached[c]&&(this.freeBonds[c][this.numCached[c]++]=b)}return d},"JM.Bond,~A");c(c$,"addHBond",function(a,b,d,c){this.bondCount==this.bo.length&&(this.bo=JU.AU.arrayCopyObject(this.bo,this.bondCount+250));return this.setBond(this.bondCount++,this.bondMutually(a,b,d,1,c)).index},"JM.Atom,JM.Atom,~N,~N");c(c$,"deleteAllBonds2",function(){this.vwr.setShapeProperty(1,
"reset",null);for(var a=this.bondCount;0<=--a;)this.bo[a].deleteAtomReferences(),this.bo[a]=null;this.bondCount=0});c(c$,"getDefaultMadFromOrder",function(a){return JU.Edge.isOrderH(a)?1:32768==a?w(Math.floor(2E3*this.vwr.getFloat(570425406))):this.defaultCovalentMad},"~N");c(c$,"deleteConnections",function(a,b,d,c,f,e,h){var k=0>a,j=0>b,z=k||j;a=this.fixD(a,k);b=this.fixD(b,j);var m=new JU.BS,r=0,n=d|=131072;!h&&JU.Edge.isOrderH(d)&&(d=30720);if(e)e=c;else{e=new JU.BS;for(var y=c.nextSetBit(0);0<=
y;y=c.nextSetBit(y+1)){var p=this.at[y];if(null!=p.bonds)for(var q=p.bonds.length;0<=--q;)f.get(p.getBondedAtomIndex(q))&&e.set(p.bonds[q].index)}}for(y=e.nextSetBit(0);y<this.bondCount&&0<=y;y=e.nextSetBit(y+1))if(c=this.bo[y],this.isInRange(c.atom1,c.atom2,a,b,k,j,z)&&(h||n==(c.order&-257|131072)||0!=(d&c.order&30720)))m.set(y),r++;0<r&&this.deleteBonds(m,!1);return B(-1,[0,r])},"~N,~N,~N,JU.BS,JU.BS,~B,~B");c(c$,"fixD",function(a,b){return b?-a:a*a},"~N,~B");c(c$,"isInRange",function(a,b,d,c,f,
e,h){var k=a.distanceSquared(b);return h?(h=Math.sqrt(k),a=a.getBondingRadius()+b.getBondingRadius(),(f?h>=a*d:k>=d)&&(e?h<=a*c:k<=c)):k>=d&&k<=c},"JM.Atom,JM.Atom,~N,~N,~B,~B,~B");c(c$,"dBb",function(a,b){var d=a.nextSetBit(0);if(!(0>d)){this.resetMolecules();for(var c=-1,f=a.cardinality(),e=d;e<this.bondCount;++e){var h=this.bo[e];if(0<f&&a.get(e)){f--;if(!b){var k=h.atom1.mi;k!=c&&this.am[c=k].resetBoundCount()}h.deleteAtomReferences()}else this.setBond(d++,h)}for(c=this.bondCount;--c>=d;)this.bo[c]=
null;this.bondCount=d;d=this.vwr.getShapeProperty(1,"sets");if(null!=d)for(c=0;c<d.length;c++)JU.BSUtil.deleteBits(d[c],a);JU.BSUtil.deleteBits(this.bsAromatic,a)}},"JU.BS,~B");c(c$,"resetAromatic",function(){for(var a=this.bondCount;0<=--a;){var b=this.bo[a];b.isAromatic()&&b.setOrder(515)}});c(c$,"assignAromaticBondsBs",function(a,b){a||(this.bsAromatic=new JU.BS);this.bsAromaticSingle=new JU.BS;this.bsAromaticDouble=new JU.BS;for(var d=null==b,c=d?this.bondCount-1:b.nextSetBit(0),f=c;0<=f;f=d?
f-1:b.nextSetBit(f+1)){var e=this.bo[f];this.bsAromatic.get(f)&&e.setOrder(515);switch(e.order&-131073){case 515:if(!this.assignAromaticMustBeSingle(e.atom1)&&!this.assignAromaticMustBeSingle(e.atom2)){this.bsAromatic.set(f);break}e.order=513;case 513:this.bsAromaticSingle.set(f);break;case 514:this.bsAromaticDouble.set(f)}}for(var c=(d=null==b)?this.bondCount-1:b.nextSetBit(0),h=new JU.BS,f=c;0<=f;f=d?f-1:b.nextSetBit(f+1))e=this.bo[f],e.is(515)&&(!this.bsAromaticDouble.get(f)&&!this.bsAromaticSingle.get(f))&&
(h.set(f),8==e.atom1.getElementNumber()||8==e.atom2.getElementNumber()?this.assignAromaticDouble(e)||this.assignAromaticSingle(e):h.set(f));for(f=h.nextSetBit(0);0<=f;f=h.nextSetBit(f+1))this.assignAromaticDouble(e=this.bo[f])||this.assignAromaticSingle(e);h=new JU.BS;for(f=c;0<=f;f=d?f-1:b.nextSetBit(f+1))if(e=this.bo[f],this.bsAromaticDouble.get(f))e.is(514)||(this.bsAromatic.set(f),h.set(e.atom1.mi),e.setOrder(514));else if((this.bsAromaticSingle.get(f)||e.isAromatic())&&!e.is(513))this.bsAromatic.set(f),
e.setOrder(513);d=this.am;for(f=h.nextSetBit(0);0<=f;f=h.nextSetBit(f+1))d[f].isBioModel&&(d[f].isPdbWithMultipleBonds=!0);this.assignAromaticNandO(b);this.bsAromaticDouble=this.bsAromaticSingle=null},"~B,JU.BS");c(c$,"assignAromaticDouble",function(a){var b=a.index;if(this.bsAromaticSingle.get(b))return!1;if(this.bsAromaticDouble.get(b))return!0;this.bsAromaticDouble.set(b);return!this.assignAromaticSingleForAtom(a.atom1,b)||!this.assignAromaticSingleForAtom(a.atom2,b)?(this.bsAromaticDouble.clear(b),
!1):!0},"JM.Bond");c(c$,"assignAromaticSingle",function(a){var b=a.index;if(this.bsAromaticDouble.get(b))return!1;if(this.bsAromaticSingle.get(b))return!0;this.bsAromaticSingle.set(b);return!this.assignAromaticDoubleForAtom(a.atom1)||!this.assignAromaticDoubleForAtom(a.atom2)?(this.bsAromaticSingle.clear(b),!1):!0},"JM.Bond");c(c$,"assignAromaticSingleForAtom",function(a,b){var d=a.bonds;if(null==d)return!1;for(var c=d.length;0<=--c;){var f=d[c],e=f.index;if(!(e==b||!f.isAromatic()||this.bsAromaticSingle.get(e)))if(this.bsAromaticDouble.get(e)||
!this.assignAromaticSingle(f))return!1}return!0},"JM.Atom,~N");c(c$,"assignAromaticDoubleForAtom",function(a){a=a.bonds;if(null==a)return!1;for(var b=!1,d=-1,c=a.length;0<=--c;)this.bsAromaticDouble.get(a[c].index)&&(b=!0),a[c].isAromatic()&&(d=c);for(c=a.length;0<=--c;){var f=a[c],e=f.index;if(f.isAromatic()&&!this.bsAromaticDouble.get(e)&&!this.bsAromaticSingle.get(e))if(!b&&this.assignAromaticDouble(f))b=!0;else if((b||c<d)&&!this.assignAromaticSingle(f))return!1}return b},"JM.Atom");c(c$,"allowAromaticBond",
function(a){if(this.assignAromaticMustBeSingle(a.atom1)||this.assignAromaticMustBeSingle(a.atom2))return!1;switch(a.getCovalentOrder()){case 1:case 2:return 3>=a.atom1.getCovalentBondCount()&&3>=a.atom2.getCovalentBondCount();default:return!1}},"JM.Bond");c(c$,"assignAromaticMustBeSingle",function(a){var b=a.getElementNumber();switch(b){case 6:case 7:case 8:case 16:break;default:return!0}var d=a.getValence();switch(b){case 6:return 4==d;case 7:return a.group.getNitrogenAtom()===a||3==d&&1>a.getFormalCharge();
case 8:return a.group.getCarbonylOxygenAtom()!==a&&2==d&&1>a.getFormalCharge();case 16:return 5==a.group.groupID||2==d&&1>a.getFormalCharge()}return!1},"JM.Atom");c(c$,"assignAromaticNandO",function(a){for(var b,d=null==a,c=d?this.bondCount-1:a.nextSetBit(0);0<=c;c=d?c-1:a.nextSetBit(c+1))if(b=this.bo[c],b.is(513)){var f,e=b.atom2,h,k=e.getElementNumber();7==k||8==k?(h=k,f=e,e=b.atom1,k=e.getElementNumber()):(f=b.atom1,h=f.getElementNumber());if(!(7!=h&&8!=h)){var j=f.getValence();if(!(0>j)){var z=
f.getCovalentBondCount();f=f.getFormalCharge();switch(h){case 7:3==j&&(3==z&&1>f&&6==k&&3==e.getValence())&&b.setOrder(514);break;case 8:1==j&&(0==f&&(14==k||16==k))&&b.setOrder(514)}}}}},"JU.BS");c(c$,"getAtomBitsMDb",function(a,b){var d=new JU.BS;switch(a){default:return this.getAtomBitsMDa(a,b,d);case 1677721602:for(var c=b.nextSetBit(0);0<=c;c=b.nextSetBit(c+1))d.set(this.bo[c].atom1.i),d.set(this.bo[c].atom2.i);return d;case 1073742331:for(c=this.bondCount;0<=--c;)this.bo[c].isAromatic()&&(d.set(this.bo[c].atom1.i),
d.set(this.bo[c].atom2.i));return d}},"~N,~O");c(c$,"removeUnnecessaryBonds",function(a,b){var d=new JU.BS,c=new JU.BS,f=a.bonds;if(null!=f){for(var e=0;e<f.length;e++)f[e].isCovalent()?1==f[e].getOtherAtom(a).getElementNumber()&&d.set(f[e].getOtherAtom(a).i):c.set(f[e].index);0<=c.nextSetBit(0)&&this.deleteBonds(c,!1);b&&d.set(a.i);0<=d.nextSetBit(0)&&this.vwr.deleteAtoms(d,!1)}},"JM.Atom,~B");c(c$,"displayBonds",function(a,b){b||(this.haveHiddenBonds=!0);for(var d=a.nextSetBit(0);0<=d;d=a.nextSetBit(d+
1))d<this.bondCount&&0!=this.bo[d].mad&&this.bo[d].setShapeVisibility(b)},"JM.BondSet,~B");c(c$,"getAtomsConnected",function(a,b,d,c){for(var f=n(c,JM.BondSet),e=f?new JM.BondSet:new JU.BS,h=B(this.ac,0),k,j=30720==d,z=65535==d,m=0;m<this.bondCount;m++){var r=this.bo[m];if(z||r.is(d)||j&&r.isHydrogen())f?e.set(m):(c.get(r.atom1.i)&&(h[k=r.atom2.i]++,e.set(k)),c.get(r.atom2.i)&&(h[k=r.atom1.i]++,e.set(k)))}if(f)return e;d=0==a;for(k=this.ac;0<=--k;)c=h[k],c<a||c>b?e.clear(k):d&&0==c&&e.set(k);return e},
"~N,~N,~N,JU.BS");E(c$,"BOND_GROWTH_INCREMENT",250,"MAX_BONDS_LENGTH_TO_CACHE",5,"MAX_NUM_TO_CACHE",200)});s("JM");N(JM,"BondIterator");s("JM");v(["JM.BondIterator"],"JM.BondIteratorSelected",null,function(){c$=t(function(){this.bonds=null;this.iBond=this.bondType=this.bondCount=0;this.bsSelected=null;this.bondSelectionModeOr=!1;p(this,arguments)},JM,"BondIteratorSelected",null,JM.BondIterator);q(c$,function(a,b,d,c,f){this.bonds=a;this.bondCount=b;this.bondType=d;this.bsSelected=c;this.bondSelectionModeOr=
f},"~A,~N,~N,JU.BS,~B");j(c$,"hasNext",function(){if(131071==this.bondType)return this.iBond=this.bsSelected.nextSetBit(this.iBond),0<=this.iBond&&this.iBond<this.bondCount;for(;this.iBond<this.bondCount;++this.iBond){var a=this.bonds[this.iBond];if(!(65535!=this.bondType&&0==(a.order&this.bondType))&&!(65535==this.bondType&&32768==a.order)){var b=this.bsSelected.get(a.atom1.i),a=this.bsSelected.get(a.atom2.i);if(!this.bondSelectionModeOr&&b&&a||this.bondSelectionModeOr&&(b||a))return!0}}return!1});
j(c$,"nextIndex",function(){return this.iBond});j(c$,"next",function(){return this.bonds[this.iBond++]})});s("JM");v(["JU.BS"],"JM.BondSet",["JU.BSUtil"],function(){c$=t(function(){this.associatedAtoms=null;p(this,arguments)},JM,"BondSet",JU.BS);q(c$,function(){I(this,JM.BondSet,[])});c$.newBS=c(c$,"newBS",function(a,b){var d=new JM.BondSet;JU.BSUtil.copy2(a,d);d.associatedAtoms=b;return d},"JU.BS,~A")});s("JM");v(["JM.Structure"],"JM.Chain",null,function(){c$=t(function(){this.model=null;this.chainNo=
this.chainID=0;this.groups=null;this.selectedGroupCount=this.groupCount=0;p(this,arguments)},JM,"Chain",null,JM.Structure);q(c$,function(a,b,d){this.model=a;this.chainID=b;this.chainNo=d;this.groups=Array(16)},"JM.Model,~N,~N");c(c$,"getIDStr",function(){return 0==this.chainID?"":256>this.chainID?""+String.fromCharCode(this.chainID):this.model.ms.vwr.getChainIDStr(this.chainID)});c(c$,"calcSelectedGroupsCount",function(a){for(var b=this.selectedGroupCount=0;b<this.groupCount;b++)this.groups[b].selectedIndex=
this.groups[b].isSelected(a)?this.selectedGroupCount++:-1},"JU.BS");c(c$,"fixIndices",function(a,b){for(var d=0;d<this.groupCount;d++)this.groups[d].fixIndices(a,b)},"~N,JU.BS");j(c$,"setAtomBits",function(a){for(var b=0;b<this.groupCount;b++)this.groups[b].setAtomBits(a)},"JU.BS");j(c$,"setAtomBitsAndClear",function(a,b){for(var d=0;d<this.groupCount;d++)this.groups[d].setAtomBitsAndClear(a,b)},"JU.BS,JU.BS")});s("JM");v(["JM.Structure","java.lang.Short","JV.JC"],"JM.Group","java.util.Hashtable JU.BS $.Quat J.c.STR JU.BSUtil $.Escape $.Logger".split(" "),
function(){c$=t(function(){this.chain=null;this.groupIndex=0;this.group1="\x00";this.leadAtomIndex=this.firstAtomIndex=-1;this.lastAtomIndex=0;this.bsAdded=null;this.shapeVisibilityFlags=this.selectedIndex=this.groupID=this.seqcode=0;this.dssrNT=null;p(this,arguments)},JM,"Group",null,JM.Structure);q(c$,function(){});c(c$,"setGroup",function(a,b,d,c,f){this.chain=a;this.seqcode=d;this.firstAtomIndex=c;this.lastAtomIndex=f;null!=b&&0<b.length&&this.setGroupID(b);return this},"JM.Chain,~S,~N,~N,~N");
c(c$,"setGroupID",function(){},"~S");c(c$,"isAdded",function(a){return null!=this.bsAdded&&this.bsAdded.get(a)},"~N");c(c$,"addAtoms",function(a){null==this.bsAdded&&(this.bsAdded=new JU.BS);this.bsAdded.set(a)},"~N");j(c$,"setAtomBits",function(a){a.setBits(this.firstAtomIndex,this.lastAtomIndex+1);null!=this.bsAdded&&a.or(this.bsAdded)},"JU.BS");j(c$,"setAtomBitsAndClear",function(a,b){a.setBits(this.firstAtomIndex,this.lastAtomIndex+1);b.clearBits(this.firstAtomIndex,this.lastAtomIndex+1);null!=
this.bsAdded&&(a.or(this.bsAdded),b.andNot(this.bsAdded))},"JU.BS,JU.BS");c(c$,"isSelected",function(a){var b=a.nextSetBit(this.firstAtomIndex);return 0<=b&&b<=this.lastAtomIndex||null!=this.bsAdded&&this.bsAdded.intersects(a)},"JU.BS");c(c$,"setShapeVisibility",function(a,b){this.shapeVisibilityFlags=b?this.shapeVisibilityFlags|a:this.shapeVisibilityFlags&~a},"~N,~B");c(c$,"getGroup3",function(){return 1>this.groupID?"":JM.Group.group3Names[this.groupID]});c(c$,"getGroup1",function(){return"\x00"==
this.group1?"?":this.group1});c(c$,"getBioPolymerLength",function(){return 0});c(c$,"getMonomerIndex",function(){return-1});c(c$,"getStructure",function(){return null});c(c$,"getStrucNo",function(){return 0});c(c$,"getProteinStructureType",function(){return J.c.STR.NOT});c(c$,"getProteinStructureSubType",function(){return this.getProteinStructureType()});c(c$,"setProteinStructureType",function(){return-1},"J.c.STR,~N");c(c$,"isProtein",function(){return 1<=this.groupID&&24>this.groupID});c(c$,"isNucleic",
function(){return 24<=this.groupID&&42>this.groupID});c(c$,"isDna",function(){return this.isDnaByID()});c(c$,"isRna",function(){return this.isRnaByID()});c(c$,"isPurine",function(){return this.isPurineByID()});c(c$,"isPurineByID",function(){return this.isNucleic()&&0!=(153957&1<<this.groupID-24)||0<="AGag".indexOf(this.getGroup1())});c(c$,"isPyrimidine",function(){return this.isPyrimidineByID()});c(c$,"isPyrimidineByID",function(){return this.isNucleic()&&0!=(108186&1<<this.groupID-24)});c(c$,"isRnaByID",
function(){return this.isNucleic()&&0!=(196663&1<<this.groupID-24)});c(c$,"isDnaByID",function(){return this.isNucleic()&&0!=(65480&1<<this.groupID-24)});c(c$,"isCarbohydrate",function(){return!1});c(c$,"getResno",function(){return-2147483648==this.seqcode?0:this.seqcode>>8});c(c$,"setResno",function(a){this.seqcode=JM.Group.getSeqcodeFor(a,this.getInsertionCode())},"~N");c$.getSeqNumberFor=c(c$,"getSeqNumberFor",function(a){return JM.Group.haveSequenceNumber(a)?a>>8:2147483647},"~N");c$.haveSequenceNumber=
c(c$,"haveSequenceNumber",function(a){return 0!=(a&128)},"~N");c(c$,"getSeqcodeString",function(){return JM.Group.getSeqcodeStringFor(this.seqcode)});c$.getSeqcodeFor=c(c$,"getSeqcodeFor",function(a,b){if(-2147483648==a)return a;"A"<=b&&"Z">=b||("a"<=b&&"z">=b||"0"<=b&&"9">=b||"?"==b||"*"==b)||(" "!=b&&"\x00"!=b&&JU.Logger.warn("unrecognized insertionCode:"+b),b="\x00");return(2147483647==a?0:a<<8|128)+b.charCodeAt(0)},"~N,~S");c$.getSeqcodeStringFor=c(c$,"getSeqcodeStringFor",function(a){if(-2147483648==
a)return null;var b=""+(a>>8);0!=(a&127)&&(b+="^"+String.fromCharCode(a&127));return b},"~N");c(c$,"getInsertionCode",function(){return-2147483648==this.seqcode?"\x00":String.fromCharCode(this.seqcode&127)});c(c$,"getInsCode",function(){return this.seqcode&127});c$.getInsertionCodeFor=c(c$,"getInsertionCodeFor",function(a){return a&127},"~N");c$.getInsertionCodeChar=c(c$,"getInsertionCodeChar",function(a){return-2147483648==a?"\x00":String.fromCharCode(a&127)},"~N");c(c$,"scaleToScreen",function(a,
b){return this.chain.model.ms.vwr.tm.scaleToScreen(a,b)},"~N,~N");c(c$,"isCursorOnTopOf",function(a,b,d,c,f){return this.chain.model.ms.isCursorOnTopOf(a,b,d,c,f)},"JM.Atom,~N,~N,~N,JM.Atom");c(c$,"getModel",function(){return this.chain.model});c(c$,"getSelectedMonomerCount",function(){return 0});c(c$,"getSelectedMonomerIndex",function(){return-1});c(c$,"isLeadAtom",function(){return!1},"~N");c(c$,"getLeadAtomOr",function(a){var b=this.getLeadAtom();return null==b?a:b},"JM.Atom");c(c$,"getLeadAtom",
function(){return null});c(c$,"getQuaternion",function(){return null},"~S");c(c$,"getQuaternionFrame",function(a){if(3>this.lastAtomIndex-this.firstAtomIndex)return null;var b=this.firstAtomIndex;return JU.Quat.getQuaternionFrame(a[b],a[++b],a[++b])},"~A");c(c$,"setStrucNo",function(){},"~N");c(c$,"getHelixData",function(a){return JU.Escape.escapeHelical(null,a,null,null,null)},"~N,~S,~N");c(c$,"isWithinStructure",function(){return!1},"J.c.STR");c(c$,"getProteinStructureTag",function(){return null});
c(c$,"getStructureId",function(){return""});c(c$,"getBioPolymerIndexInModel",function(){return-1});c(c$,"isCrossLinked",function(){return!1},"JM.Group");c(c$,"getCrossLinkVector",function(){return!1},"JU.Lst,~B,~B");c(c$,"isConnectedPrevious",function(){return!1});c(c$,"getNitrogenAtom",function(){return null});c(c$,"getCarbonylOxygenAtom",function(){return null});c(c$,"fixIndices",function(a,b){this.firstAtomIndex-=a;this.leadAtomIndex-=a;this.lastAtomIndex-=a;null!=this.bsAdded&&JU.BSUtil.deleteBits(this.bsAdded,
b)},"~N,JU.BS");c(c$,"getGroupInfo",function(a,b){var d=new java.util.Hashtable;d.put("groupIndex",Integer.$valueOf(a));d.put("groupID",Short.$valueOf(this.groupID));var c=this.getSeqcodeString();null!=c&&d.put("seqCode",c);d.put("_apt1",Integer.$valueOf(this.firstAtomIndex));d.put("_apt2",Integer.$valueOf(this.lastAtomIndex));null!=this.bsAdded&&d.put("addedAtoms",this.bsAdded);d.put("atomInfo1",this.chain.model.ms.getAtomInfo(this.firstAtomIndex,null,b));d.put("atomInfo2",this.chain.model.ms.getAtomInfo(this.lastAtomIndex,
null,b));d.put("visibilityFlags",Integer.$valueOf(this.shapeVisibilityFlags));return d},"~N,JU.P3");c(c$,"getMinZ",function(a,b){b[0]=2147483647;for(var d=this.firstAtomIndex;d<=this.lastAtomIndex;d++)this.checkMinZ(a[d],b);if(null!=this.bsAdded)for(d=this.bsAdded.nextSetBit(0);0<=d;d=this.bsAdded.nextSetBit(d+1))this.checkMinZ(a[d],b)},"~A,~A");c(c$,"checkMinZ",function(a,b){var d=a.sZ-w(a.sD/2)-2;d<b[0]&&(b[0]=Math.max(1,d))},"JM.Atom,~A");c(c$,"getGroupParameter",function(){return NaN},"~N");c(c$,
"getAtomIndex",function(){return-1},"~S,~N");c(c$,"getBSSideChain",function(){return new JU.BS});j(c$,"toString",function(){return"["+this.getGroup3()+"-"+this.getSeqcodeString()+"]"});c(c$,"isNucleicMonomer",function(){return!1});E(c$,"standardGroupList",null);c$.group3Names=c$.prototype.group3Names=Array(128);E(c$,"specialAtomNames",null,"SEQUENCE_NUMBER_FLAG",128,"INSERTION_CODE_MASK",127,"SEQUENCE_NUMBER_SHIFT",8)});s("JM");v(["JM.Bond"],"JM.HBond",["JU.Logger"],function(){c$=t(function(){this.energy=
0;p(this,arguments)},JM,"HBond",JM.Bond);q(c$,function(a,b,d,c,f,e){I(this,JM.HBond,[a,b,d,c,f]);this.energy=e;JU.Logger.debugging&&JU.Logger.debug("HBond energy = "+e+" #"+this.getIdentity())},"JM.Atom,JM.Atom,~N,~N,~N,~N");c(c$,"getEnergy",function(){return this.energy});c$.getEnergy=c(c$,"getEnergy",function(a,b,d,c){return Math.round(-27888/a- -27888/c+-27888/d- -27888/b)},"~N,~N,~N,~N");E(c$,"QConst",-27888)});s("JM");v(null,"JM.LabelToken","java.lang.Float java.util.Hashtable JU.AU $.Lst $.PT $.SB $.T3 JS.SV $.T JU.Edge JV.JC".split(" "),
function(){c$=t(function(){this.data=this.key=this.text=null;this.tok=0;this.pt=-1;this.ch1="\x00";this.width=0;this.precision=2147483647;this.intAsFloat=this.zeroPad=this.alignLeft=!1;p(this,arguments)},JM,"LabelToken");q(c$,function(){});c(c$,"set",function(a,b){this.text=a;this.pt=b;return this},"~S,~N");c$.isLabelPropertyTok=c(c$,"isLabelPropertyTok",function(a){for(var b=JM.LabelToken.labelTokenIds.length;0<=--b;)if(JM.LabelToken.labelTokenIds[b]==a)return!0;return!1},"~N");c$.compile=c(c$,"compile",
function(a,b,d,c){if(null==b||0==b.length)return null;if(0>b.indexOf("%")||2>b.length)return u(-1,[(new JM.LabelToken).set(b,-1)]);for(var f=0,e=-1,h=b.length;++e<h&&0<=(e=b.indexOf("%",e));)f++;for(var f=Array(2*f+1),k,j=0,e=0;0<=(k=b.indexOf("%",e));)e!=k&&(f[j++]=(new JM.LabelToken).set(b.substring(e,k),-1)),e=f[j++]=(new JM.LabelToken).set(null,k),a.autoCalculate(e.tok,null),e=JM.LabelToken.setToken(a,b,e,h,d.charCodeAt(0),c);e<h&&(f[j++]=(new JM.LabelToken).set(b.substring(e),-1));return f},
"JV.Viewer,~S,~S,java.util.Map");c(c$,"formatLabel",function(a,b,d,c){return null==d||0==d.length?null:JM.LabelToken.formatLabelAtomArray(a,b,JM.LabelToken.compile(a,d,"\x00",null),"\x00",null,c)},"JV.Viewer,JM.Atom,~S,JU.P3");c$.formatLabelAtomArray=c(c$,"formatLabelAtomArray",function(a,b,d,c,f,e){if(null==b)return null;var h="0"<c?null:new JU.SB;if(null!=d)for(var k=0;k<d.length;k++){var j=d[k];if(null==j)break;"0"<c&&j.ch1!=c||(0>=j.tok||null!=j.key?null!=h&&(h.append(j.text),"\x00"!=j.ch1&&h.appendC(j.ch1)):
JM.LabelToken.appendAtomTokenValue(a,b,j,h,f,e))}return null==h?null:h.toString().intern()},"JV.Viewer,JM.Atom,~A,~S,~A,JU.P3");c$.getBondLabelValues=c(c$,"getBondLabelValues",function(){var a=new java.util.Hashtable;a.put("#","");a.put("ORDER","");a.put("TYPE","");a.put("LENGTH",Float.$valueOf(0));a.put("ENERGY",Float.$valueOf(0));return a});c$.formatLabelBond=c(c$,"formatLabelBond",function(a,b,d,c,f,e){c.put("#",""+(b.index+1));c.put("ORDER",""+JU.Edge.getBondOrderNumberFromOrder(b.order));c.put("TYPE",
JU.Edge.getBondOrderNameFromOrder(b.order));c.put("LENGTH",Float.$valueOf(b.atom1.distance(b.atom2)));c.put("ENERGY",Float.$valueOf(b.getEnergy()));JM.LabelToken.setValues(d,c);JM.LabelToken.formatLabelAtomArray(a,b.atom1,d,"1",f,e);JM.LabelToken.formatLabelAtomArray(a,b.atom2,d,"2",f,e);return JM.LabelToken.getLabel(d)},"JV.Viewer,JM.Bond,~A,java.util.Map,~A,JU.P3");c$.formatLabelMeasure=c(c$,"formatLabelMeasure",function(a,b,d,c,f){var e=new java.util.Hashtable;e.put("#",""+(b.index+1));e.put("VALUE",
Float.$valueOf(c));e.put("UNITS",f);d=JM.LabelToken.compile(a,d,"\u0001",e);if(null==d)return"";JM.LabelToken.setValues(d,e);e=b.ms.at;b=b.countPlusIndices;for(c=b[0];1<=c;--c)0<=b[c]&&JM.LabelToken.formatLabelAtomArray(a,e[b[c]],d,String.fromCharCode(48+c),null,null);d=JM.LabelToken.getLabel(d);return null==d?"":d},"JV.Viewer,JM.Measurement,~S,~N,~S");c$.setValues=c(c$,"setValues",function(a,b){for(var d=0;d<a.length;d++){var c=a[d];if(null==c)break;if(null!=c.key){var f=b.get(c.key);c.text=n(f,
Float)?c.format(f.floatValue(),null,null):c.format(NaN,f,null)}}},"~A,java.util.Map");c$.getLabel=c(c$,"getLabel",function(a){for(var b=new JU.SB,d=0;d<a.length;d++){var c=a[d];if(null==c)break;b.append(c.text)}return b.toString()},"~A");c$.setToken=c(c$,"setToken",function(a,b,d,c,f,e){var h=d.pt+1;if(h>=c)return d.text="%",h;var k;"-"==b.charAt(h)&&(d.alignLeft=!0,++h);h<c&&"0"==b.charAt(h)&&(d.zeroPad=!0,++h);for(;h<c&&JU.PT.isDigit(k=b.charAt(h));)d.width=10*d.width+(k.charCodeAt(0)-48),++h;d.precision=
2147483647;var j=!1;if(h<c&&"."==b.charAt(h)){++h;if(h<c&&"-"==(k=b.charAt(h)))j=!0,++h;if(h<c&&JU.PT.isDigit(k=b.charAt(h)))d.precision=k.charCodeAt(0)-48,j&&(d.precision=-1-d.precision),++h}if(h<c&&null!=e){var z;for(e=e.keySet().iterator();e.hasNext()&&((z=e.next())||1);)if(b.indexOf(z)==h)return h+(d.key=z).length}if(h<c)switch(k=b.charAt(h++)){case "%":return d.text="%",h;case "[":z=b.indexOf("]",h);if(z<h){h=c;break}h=b.substring(h,z).toLowerCase();h.startsWith("property_")?(d.tok=134221834,
d.data=a.getDataObj(h,null,1)):h.startsWith("validation.")?(d.tok=1073742189,d.data=a.getDataObj("property_"+h.substring(11),null,1)):h.startsWith("unitid")?(d.tok=1073741974,d.data=Integer.$valueOf(JV.JC.getUnitIDFlags(h.substring(6)))):(a=JS.T.getTokenFromName(h),null!=a&&JM.LabelToken.isLabelPropertyTok(a.tok)&&(d.tok=a.tok));h=z+1;break;case "{":z=b.indexOf("}",h);if(z<h){h=c;break}h=b.substring(h,z);d.data=a.getDataObj(h,null,1);null==d.data?(d.data=a.getDataObj(h,null,-1),null!=d.data&&(d.data=
d.data[1],n(d.data,String)&&(d.data=JU.PT.split(d.data,"\n")),JU.AU.isAS(d.data)||(d.data=null)),null==d.data?(d.tok=1715472409,d.data=h):d.tok=1275068418):d.tok=134221834;h=z+1;break;default:var m,r;if(h<c&&0<=(m="fuv".indexOf(k))&&0<=(r="xyz".indexOf(b.charAt(h))))d.tok=JM.LabelToken.twoCharLabelTokenIds[3*m+r],h++;else if(0<=(m="AaBbCcDEefGgIiLlMmNnOoPpQqRrSsTtUuVvWXxxYyyZzz%%%gqW".indexOf(k)))d.tok=JM.LabelToken.labelTokenIds[m]}d.text=b.substring(d.pt,h);if(h<c&&0!=f&&JU.PT.isDigit(k=b.charAt(h)))h++,
d.ch1=k,k.charCodeAt(0)!=f&&1!=f&&(d.tok=0);return h},"JV.Viewer,~S,JM.LabelToken,~N,~N,java.util.Map");c$.appendAtomTokenValue=c(c$,"appendAtomTokenValue",function(a,b,d,c,f,e){var h=null,k=NaN,j=null;try{switch(d.tok){case 1094713347:h=""+(null==f?b.i:f[b.i]);break;case 1765808134:j=b.atomPropertyTuple(a,d.tok,e);break;case 1073741974:h=b.getUnitID(d.data.intValue());break;case 134221834:case 1073742189:if(null!=d.data&&(k=d.data[b.i],1073742189==d.tok&&1!=k&&0!=k)){var z=a.getAtomValidation(d.text.substring(13,
d.text.length-1),b);if(null==z)Zystem.out.println("?? o is null ??");else if(1==z.size())k=z.get(0).floatValue();else{k=NaN;h="";a=0;for(var m=z.size();a<m;a++)h+=","+z.get(a);1<h.length&&(h=h.substring(1))}}break;case 1715472409:var r=a.ms.getInfo(b.mi,d.data),p=b.i-a.ms.am[b.mi].firstAtomIndex,z=null;if(0<=p)if(n(r,Array))var y=r,z=p<y.length?y[p]:null;else n(r,JU.Lst)&&(z=p<r.size()?JS.SV.oValue(r.get(p)):null);null==z?h="":n(z,Float)?k=z.floatValue():n(z,Integer)?k=z.intValue():n(z,JU.T3)?j=z:
h=z.toString();break;case 1275068418:null!=d.data&&(y=d.data,h=b.i<y.length?y[b.i]:"");break;case 1631586315:var q=b.getFormalCharge(),h=0<q?""+q+"+":0>q?""+-q+"-":"";break;case 1094717454:h=b.getModelNumberForLabel();break;case 1128269825:h=""+b.atomPropertyInt(d.tok);break;case 1665140738:k=b.atomPropertyFloat(a,d.tok,e);break;case 1086324749:h=b.group.getStructureId();break;case 1094713367:var s=b.group.getStrucNo(),h=0>=s?"":""+s;break;case 1111490574:if(Float.isNaN(k=b.group.getGroupParameter(1111490574)))h=
"null";break;case 1111492626:case 1111492627:case 1111492628:case 1111490583:case 1111490584:case 1111490585:case 1111490586:k=b.atomPropertyFloat(a,d.tok,e);Float.isNaN(k)&&(h="");break;case 1073877011:h=a.getNBOAtomLabel(b);break;case 1086324747:case 1639976963:case 1237320707:h=b.atomPropertyString(a,d.tok);break;case 1140850705:h=b.getIdentityXYZ(!1,e);break;case 79:h=b.getSymmetryOperatorList(!1);break;case 81:k=b.getOccupancy100()/100;break;default:switch(d.tok&1136656384){case 1094713344:d.intAsFloat?
k=b.atomPropertyInt(d.tok):h=""+b.atomPropertyInt(d.tok);break;case 1111490560:k=b.atomPropertyFloat(a,d.tok,e);break;case 1086324736:h=b.atomPropertyString(a,d.tok);break;case 1077936128:j=b.atomPropertyTuple(a,d.tok,e),null==j&&(h="")}}}catch(u){if(H(u,IndexOutOfBoundsException))k=NaN,j=h=null;else throw u;}h=d.format(k,h,j);null==c?d.text=h:c.append(h)},"JV.Viewer,JM.Atom,JM.LabelToken,JU.SB,~A,JU.P3");c(c$,"format",function(a,b,d){return Float.isNaN(a)?null!=b?JU.PT.formatS(b,this.width,this.precision,
this.alignLeft,this.zeroPad):null!=d?(0==this.width&&2147483647==this.precision&&(this.width=6,this.precision=2),JU.PT.formatF(d.x,this.width,this.precision,!1,!1)+JU.PT.formatF(d.y,this.width,this.precision,!1,!1)+JU.PT.formatF(d.z,this.width,this.precision,!1,!1)):this.text:JU.PT.formatF(a,this.width,this.precision,this.alignLeft,this.zeroPad)},"~N,~S,JU.T3");E(c$,"labelTokenParams","AaBbCcDEefGgIiLlMmNnOoPpQqRrSsTtUuVvWXxxYyyZzz%%%gqW","labelTokenIds",B(-1,[1086324739,1086326786,1086326785,1111492620,
1631586315,1086326788,1094713347,1086324746,1086326789,1111490569,1094713357,1094713361,1111492618,1094715393,1094713363,1094715402,1094717454,1086324743,1094713360,1086324742,79,1088421903,1111492619,1111490570,81,1128269825,1094715412,1086324747,1094713366,1086326788,1111490574,1111492620,1086324745,1111490575,1648363544,1145047055,1140850705,1111492612,1111492609,1111492629,1111492613,1111492610,1111492630,1111492614,1111492611,1111492631,1114249217,1112152066,1112150019,1112150020,1112150021,
1112152070,1112152071,1112152073,1112152074,1112152076,1649022989,1112152078,1111490561,1111490562,1094713346,1228931587,1765808134,1094713356,1111490564,1228935687,1287653388,1825200146,1111490567,1094713359,1111490565,1111490568,1094713362,1715472409,1665140738,1113589787,1086324748,1086324744,1112152075,1639976963,1237320707,1094713367,1086324749,1086326798,1111490576,1111490577,1111490578,1111490579,1094715417,1648361473,1111492626,1111492627,1111492628,1312817669,1145045006,1145047051,1145047050,
1145047053,1111492615,1111492616,1111492617,1113589786,1111490571,1111490572,1111490573,1145047052,1111490566,1111490563,1094713351,1094713365,1111490583,1111490584,1111490585,1111490586,1145045008,1296041986,1073877011,1086324752,1086324753]),"STANDARD_LABEL","%[identify]","twoCharLabelTokenParams","fuv","twoCharLabelTokenIds",B(-1,[1111492612,1111492613,1111492614,1111490577,1111490578,1111490579,1111492626,1111492627,1111492628]))});s("JM");v(null,"JM.Measurement","java.lang.Float JU.Measure $.PT $.SB J.atomdata.RadiusData J.c.VDW JM.LabelToken JU.Escape".split(" "),
function(){c$=t(function(){this.ms=this.thisID=null;this.index=0;this.isVisible=!0;this.isTrajectory=this.isHidden=!1;this.$isValid=!0;this.colix=0;this.labelColix=-1;this.mad=0;this.tickInfo=null;this.traceX=-2147483648;this.count=this.traceY=0;this.pts=this.countPlusIndices=null;this.value=0;this.type=this.strMeasurement=this.vwr=this.text=this.units=this.property=this.strFormat=null;this.tainted=!1;this.newUnits=this.renderArc=this.renderAxis=null;this.fixedValue=NaN;this.inFront=this.isPending=
!1;p(this,arguments)},JM,"Measurement");O(c$,function(){this.countPlusIndices=B(5,0)});c(c$,"isTainted",function(){return this.tainted&&!(this.tainted=!1)});c(c$,"setM",function(a,b,d,c,f,e){this.ms=a;this.index=e;this.vwr=a.vwr;this.colix=c;this.strFormat=f;null!=b&&(this.tickInfo=b.tickInfo,this.pts=b.pts,this.mad=b.mad,this.thisID=b.thisID,this.text=b.text,this.property=b.property,this.units=b.units,null==this.property&&"+hz".equals(this.units)&&(this.property="property_J"),null!=this.thisID&&
null!=this.text&&(this.labelColix=this.text.colix));null==this.pts&&(this.pts=Array(4));b=null==b?null:b.countPlusIndices;this.count=null==b?0:b[0];0<this.count&&(Zystem.arraycopy(b,0,this.countPlusIndices,0,this.count+1),this.isTrajectory=a.isTrajectoryMeasurement(this.countPlusIndices));this.value=(this.isPending=Float.isNaN(d))||this.isTrajectory?this.getMeasurement(null):d;this.formatMeasurement(null);return this},"JM.ModelSet,JM.Measurement,~N,~N,~S,~N");c(c$,"setPoints",function(a,b,d,c){this.ms=
a;this.countPlusIndices=b;this.count=b[0];this.pts=null==d?Array(4):d;this.vwr=a.vwr;this.tickInfo=c;return this},"JM.ModelSet,~A,~A,JM.TickInfo");c(c$,"setCount",function(a){this.setCountM(a)},"~N");c(c$,"setCountM",function(a){this.count=this.countPlusIndices[0]=a},"~N");c(c$,"getAtomIndex",function(a){return 0<a&&a<=this.count?this.countPlusIndices[a]:-1},"~N");c(c$,"getAtom",function(a){a=this.countPlusIndices[a];return-1>a?this.pts[-2-a]:this.ms.at[a]},"~N");c(c$,"getLastIndex",function(){return 0<
this.count?this.countPlusIndices[this.count]:-1});c(c$,"getString",function(){return this.strMeasurement});j(c$,"toString",function(){return this.getString()});c(c$,"getStringUsing",function(a,b,d){this.vwr=a;this.value=this.getMeasurement(null);this.formatMeasurementAs(b,d,!0);return null==b?this.getInfoAsString(d):this.strMeasurement},"JV.Viewer,~S,~S");c(c$,"getStringDetail",function(){return(2==this.count?"Distance":3==this.count?"Angle":"Torsion")+this.getMeasurementScript(" - ",!1)+" : "+this.value});
c(c$,"refresh",function(a){this.value=this.getMeasurement(a);this.isTrajectory=this.ms.isTrajectoryMeasurement(this.countPlusIndices);this.formatMeasurement(null)},"~A");c(c$,"getMeasurementScript",function(a,b){for(var d=new JU.SB,c=a.equals(" "),f=1;f<=this.count;f++)d.append(1<f?a:" ").append(this.getLabel(f,c,b));return d.toString()},"~S,~B");c(c$,"formatMeasurementAs",function(a,b,d){null!=a&&0==a.length&&(a=null);if(d||!(null!=a&&0!=a.indexOf(this.countPlusIndices[0]+":")))this.strFormat=a,
this.formatMeasurement(b)},"~S,~S,~B");c(c$,"formatMeasurement",function(a){this.tainted=!0;switch(Float.isNaN(this.value)?0:this.count){default:this.strMeasurement=null;break;case 2:this.strMeasurement=this.formatDistance(a);break;case 3:case 4:this.strMeasurement=this.formatAngle(this.value)}},"~S");c(c$,"reformatDistanceIfSelected",function(){2==this.count&&this.vwr.slm.isSelected(this.countPlusIndices[1])&&this.vwr.slm.isSelected(this.countPlusIndices[2])&&this.formatMeasurement(null)});c(c$,
"formatDistance",function(a){var b=this.getLabelString();if(null==b)return"";var d=this.strFormat.indexOf("//");null==a?(a=this.units,null==a&&(0<=d?(a=this.strFormat.substring(d+2),this.strFormat=this.strFormat.substring(0,d)):a=null==this.property?this.vwr.g.measureDistanceUnits:"")):0<=d&&(this.strFormat=this.strFormat.substring(0,d));this.strFormat+="//"+a;a=JM.Measurement.fixUnits(a);d=b.indexOf("//");0<=d&&(b=b.substring(0,d),0==b.length&&(b="%VALUE"));a=this.fixValue(a,0<=b.indexOf("%V"));
return this.formatString(a,this.newUnits,b)},"~S");c$.fixUnits=c(c$,"fixUnits",function(a){return a.equals("nanometers")?"nm":a.equals("picometers")?"pm":a.equals("angstroms")?"\u00c5":a.equals("vanderwaals")||a.equals("vdw")?"%":a},"~S");c(c$,"fixValue",function(a,b){this.checkJ(a);null!=a&&a.startsWith("+")&&(this.isPending||(this.value=Math.abs(this.value)),a=a.substring(1));this.newUnits=a;if(2!=this.count)return this.value;var d=this.value;null==a&&null!=this.property&&(a="");if(null!=a){var c=
a.equals("%");if(null==this.property&&(c||a.endsWith("hz"))){var f=this.getAtomIndex(1),e=this.getAtomIndex(2);if(0<=f&&0<=e){var f=this.getAtom(1),e=this.getAtom(2),h=JM.Measurement.nmrType(a),k=!c&&1==h;this.type=c?"percent":k?"dipoleCouplingConstant":3==h?"NOE or 3JHH":"J-CouplingConstant";3==h?(f=this.vwr.getNMRCalculation().getNOEorJHH(u(-1,[f,null,null,e]),11),null==f?(d=NaN,this.newUnits=a=""):(d=f[1],a=this.newUnits=2==f.length?"noe":"hz")):d=c?d/(f.getVanderwaalsRadiusFloat(this.vwr,J.c.VDW.AUTO)+
e.getVanderwaalsRadiusFloat(this.vwr,J.c.VDW.AUTO)):k?this.vwr.getNMRCalculation().getDipolarConstantHz(f,e):this.vwr.getNMRCalculation().getIsoOrAnisoHz(!0,f,e,a,null);this.$isValid=!Float.isNaN(d);c&&(a="pm")}}if(Float.isNaN(d))return NaN;if(a.equals("hz"))return b?Math.round(10*d)/10:d;if(a.equals("noe"))return b?Math.round(100*d)/100:d;if(a.equals("nm"))return b?Math.round(100*d)/1E3:d/10;if(a.equals("pm"))return b?Math.round(1E3*d)/10:100*d;if(a.equals("au"))return b?Math.round(1E3*(d/0.5291772))/
1E3:d/0.5291772;if(a.endsWith("khz"))return b?Math.round(d/10)/100:d/1E3}return b?Math.round(100*d)/100:d},"~S,~B");c(c$,"checkJ",function(a){null!=this.property||(null!=a||null!=this.units)||(a=this.vwr.g.measureDistanceUnits,"+hz".equals(a)&&(this.property="property_J",this.units=a))},"~S");c$.nmrType=c(c$,"nmrType",function(a){return 0>a.indexOf("hz")?0:a.equals("noe_hz")?3:a.startsWith("dc_")||a.equals("khz")?1:2},"~S");c(c$,"formatAngle",function(a){var b=this.getLabelString();0<=b.indexOf("%V")&&
(a=Math.round(10*a)/10);return this.formatString(a,"\u00b0",b)},"~N");c(c$,"getLabelString",function(){var a=this.countPlusIndices[0]+":",b=null;if(null!=this.strFormat){if(0==this.strFormat.length)return null;b=2<this.strFormat.length&&0==this.strFormat.indexOf(a)?this.strFormat:null}null==b&&(this.strFormat=null,b=this.vwr.getDefaultMeasurementLabel(this.countPlusIndices[0]));0==b.indexOf(a)&&(b=b.substring(2));null==this.strFormat&&(this.strFormat=a+b);return b});c(c$,"formatString",function(a,
b,d){return JM.LabelToken.formatLabelMeasure(this.vwr,this,d,a,b)},"~N,~S,~S");c(c$,"sameAsPoints",function(a,b){if(this.count!=a[0])return!1;for(var d=!0,c=1;c<=this.count&&d;c++)d=this.countPlusIndices[c]==a[c];if(d)for(c=0;c<this.count&&d;c++)null!=b[c]&&(d=0.01>this.pts[c].distance(b[c]));if(d)return!0;switch(this.count){default:return!0;case 2:return this.sameAsIJ(a,b,1,2)&&this.sameAsIJ(a,b,2,1);case 3:return this.sameAsIJ(a,b,1,3)&&this.sameAsIJ(a,b,2,2)&&this.sameAsIJ(a,b,3,1);case 4:return this.sameAsIJ(a,
b,1,4)&&this.sameAsIJ(a,b,2,3)&&this.sameAsIJ(a,b,3,2)&&this.sameAsIJ(a,b,4,1)}},"~A,~A");c(c$,"sameAsIJ",function(a,b,d,c){d=this.countPlusIndices[d];a=a[c];return 0<=d||0<=a?d==a:0.01>this.pts[-2-d].distance(b[-2-a])},"~A,~A,~N,~N");c(c$,"sameAs",function(a,b){return this.sameAsIJ(this.countPlusIndices,this.pts,a,b)},"~N,~N");c(c$,"getPropMeasurement",function(a){if(null==this.countPlusIndices||2!=this.count)return NaN;for(var b=this.count;0<=--b;)if(0>this.countPlusIndices[b+1])return NaN;try{var d=
null==a?this.getAtom(1):a[0],c=null==a?this.getAtom(2):a[1],f=this.vwr.getDataObj(this.property,null,2),e=d.i,h=c.i;return null==f||h>=f.length||e>=f.length?NaN:f[e][h]}catch(k){return NaN}},"~A");c(c$,"getMeasurement",function(a){this.checkJ(null);if(!Float.isNaN(this.fixedValue))return this.fixedValue;if(null!=this.property)return this.getPropMeasurement(a);if(null==this.countPlusIndices||2>this.count)return NaN;for(var b=this.count;0<=--b;)if(-1==this.countPlusIndices[b+1])return NaN;var b=null==
a?this.getAtom(1):a[0],d=null==a?this.getAtom(2):a[1],c;switch(this.count){case 2:return b.distance(d);case 3:return c=null==a?this.getAtom(3):a[2],JU.Measure.computeAngleABC(b,d,c,!0);case 4:return c=null==a?this.getAtom(3):a[2],a=null==a?this.getAtom(4):a[3],JU.Measure.computeTorsion(b,d,c,a,!0);default:return NaN}},"~A");c(c$,"getLabel",function(a,b,d){var c=this.countPlusIndices[a];return 0>c?(d?"modelIndex "+this.getAtom(a).mi+" ":"")+JU.Escape.eP(this.getAtom(a)):b?"({"+c+"})":this.vwr.getAtomInfo(c)},
"~N,~B,~B");c(c$,"setModelIndex",function(a){if(null!=this.pts)for(var b=0;b<this.count;b++)null!=this.pts[b]&&(this.pts[b].mi=a)},"~N");c(c$,"isValid",function(){return!(this.sameAs(1,2)||2<this.count&&this.sameAs(1,3)||4==this.count&&this.sameAs(2,4))});c$.find=c(c$,"find",function(a,b){for(var d=b.countPlusIndices,c=b.pts,f=a.size();0<=--f;)if(a.get(f).sameAsPoints(d,c))return f;return-1},"JU.Lst,JM.Measurement");c(c$,"isConnected",function(a,b){for(var d=-1,c=1;c<=b;c++){var f=this.getAtomIndex(c);
if(!(0>f)){if(0<=d&&!a[f].isBonded(a[d]))return!1;d=f}}return!0},"~A,~N");c(c$,"getInfoAsString",function(a){var b=this.fixValue(a,!0);a=new JU.SB;a.append(2==this.count?null!=this.property?this.property:null==this.type?"distance":this.type:3==this.count?"angle":"dihedral");a.append(" \t").appendF(b);a.append(" \t").append(JU.PT.esc(this.strMeasurement));for(b=1;b<=this.count;b++)a.append(" \t").append(this.getLabel(b,!1,!1));null!=this.thisID&&a.append(" \t").append(this.thisID);return a.toString()},
"~S");c(c$,"isInRange",function(a,b){if(a.factorType===J.atomdata.RadiusData.EnumType.FACTOR){var d=this.getAtom(1),c=this.getAtom(2),d=(d.getVanderwaalsRadiusFloat(this.vwr,a.vdwType)+c.getVanderwaalsRadiusFloat(this.vwr,a.vdwType))*a.value;return b<=d}return 3.4028235E38==a.values[0]||b>=a.values[0]&&b<=a.values[1]},"J.atomdata.RadiusData,~N");c(c$,"isIntramolecular",function(a,b){for(var d=-1,c=1;c<=b;c++){var f=this.getAtomIndex(c);if(!(0>f))if(f=a[f].getMoleculeNumber(!1),0>d)d=f;else if(f!=
d)return!1}return!0},"~A,~N");c(c$,"isMin",function(a){var b=this.getAtom(1),d=this.getAtom(2),c=A(100*d.distanceSquared(b)),b=b.getAtomName(),d=d.getAtomName(),d=0>b.compareTo(d)?b+d:d+b;a=a.get(d);return null!=a&&c==a.intValue()},"java.util.Map");c$.isUnits=c(c$,"isUnits",function(a){return JU.PT.isOneOf((a.startsWith("+")?a.substring(1):a).toLowerCase(),";nm;nanometers;pm;picometers;angstroms;angstroms;ang;\u00c5;au;vanderwaals;vdw;%;noe;")||0>a.indexOf(" ")&&a.endsWith("hz")},"~S");E(c$,"NMR_NOT",
0,"NMR_DC",1,"NMR_JC",2,"NMR_NOE_OR_J",3)});s("JM");v(["J.api.JmolMeasurementClient"],"JM.MeasurementData",["java.lang.Float","JU.BS","$.Lst","JM.Measurement","JU.BSUtil"],function(){c$=t(function(){this.points=this.measurements=this.measurementStrings=this.client=null;this.mustNotBeConnected=this.mustBeConnected=!1;this.tickInfo=null;this.tokAction=12290;this.note=this.property=this.strFormat=this.radiusData=null;this.isAll=!1;this.colix=0;this.intramolecular=null;this.mad=0;this.units=this.text=
this.thisID=null;this.fixedValue=0;this.ms=this.minArray=this.atoms=null;this.allowSelf=!1;this.vwr=null;this.iFirstAtom=0;this.justOneModel=!0;this.htMin=null;p(this,arguments)},JM,"MeasurementData",null,J.api.JmolMeasurementClient);q(c$,function(){});c(c$,"init",function(a,b,d){this.vwr=b;this.points=d;this.thisID=a;return this},"~S,JV.Viewer,JU.Lst");c(c$,"setModelSet",function(a){this.ms=a;return this},"JM.ModelSet");c(c$,"set",function(a,b,d,c,f,e,h,k,j,z,m,r,p,y,q){this.ms=this.vwr.ms;this.tokAction=
a;2<=this.points.size()&&(n(this.points.get(0),JU.BS)&&n(this.points.get(1),JU.BS))&&(this.justOneModel=JU.BSUtil.haveCommon(this.vwr.ms.getModelBS(this.points.get(0),!1),this.vwr.ms.getModelBS(this.points.get(1),!1)));this.htMin=b;this.radiusData=d;this.property=c;this.strFormat=f;this.units=e;this.tickInfo=h;this.mustBeConnected=k;this.mustNotBeConnected=j;this.intramolecular=z;this.isAll=m;this.mad=r;this.colix=p;this.text=y;this.fixedValue=q;return this},"~N,java.util.Map,J.atomdata.RadiusData,~S,~S,~S,JM.TickInfo,~B,~B,Boolean,~B,~N,~N,JM.Text,~N");
c(c$,"processNextMeasure",function(a){var b=a.getMeasurement(null);if(!(null!=this.htMin&&!a.isMin(this.htMin)||null!=this.radiusData&&!a.isInRange(this.radiusData,b)))if(null==this.measurementStrings&&null==this.measurements){var d=this.minArray[this.iFirstAtom];a.value=b;b=a.fixValue(this.units,!1);this.minArray[this.iFirstAtom]=-Infinity==1/d?b:Math.min(d,b)}else null!=this.measurementStrings?this.measurementStrings.addLast(a.getStringUsing(this.vwr,this.strFormat,this.units)):this.measurements.addLast(Float.$valueOf(a.getMeasurement(null)))},
"JM.Measurement");c(c$,"getMeasurements",function(a,b){if(b){this.minArray=K(this.points.get(0).cardinality(),0);for(var d=0;d<this.minArray.length;d++)this.minArray[d]=-0;this.define(null,this.ms);return this.minArray}if(a)return this.allowSelf=!0,this.measurements=new JU.Lst,this.define(null,this.ms),this.measurements;this.measurementStrings=new JU.Lst;this.define(null,this.ms);return this.measurementStrings},"~B,~B");c(c$,"define",function(a,b){this.client=null==a?this:a;this.atoms=b.at;var d=
this.points.size();if(!(2>d)){var c=-1,f=Array(4),e=B(5,0),h=(new JM.Measurement).setPoints(b,e,f,null);h.setCount(d);h.property=this.property;h.strFormat=this.strFormat;h.units=this.units;h.fixedValue=this.fixedValue;for(var k=-1,j=0;j<d;j++){var z=this.points.get(j);if(n(z,JU.BS)){k=z.cardinality();if(0==k)return;1<k&&(c=0);k=j;0==j&&(this.iFirstAtom=0);e[j+1]=z.nextSetBit(0)}else f[j]=z,e[j+1]=-2-j}this.nextMeasure(0,k,h,c)}},"J.api.JmolMeasurementClient,JM.ModelSet");c(c$,"nextMeasure",function(a,
b,d,c){if(a>b)(this.allowSelf&&!this.mustBeConnected&&!this.mustNotBeConnected||d.isValid())&&((!this.mustBeConnected||d.isConnected(this.atoms,a))&&(!this.mustNotBeConnected||!d.isConnected(this.atoms,a))&&(null==this.intramolecular||d.isIntramolecular(this.atoms,a)==this.intramolecular.booleanValue()))&&this.client.processNextMeasure(d);else{var f=this.points.get(a),e=d.countPlusIndices,h=0==a?2147483647:e[a];if(0>h)this.nextMeasure(a+1,b,d,c);else{for(var k=!1,j=f.nextSetBit(0),z=0;0<=j;j=f.nextSetBit(j+
1),z++)if(j!=h||this.allowSelf){var m=this.atoms[j].mi;if(0<=c&&this.justOneModel)if(0==a)c=m;else if(c!=m)continue;e[a+1]=j;0==a&&(this.iFirstAtom=z);k=!0;this.nextMeasure(a+1,b,d,c)}k||this.nextMeasure(a+1,b,d,c)}}},"~N,~N,JM.Measurement,~N")});s("JM");v(["JM.Measurement"],"JM.MeasurementPending",null,function(){c$=t(function(){this.haveModified=this.haveTarget=!1;this.numSet=0;this.lastIndex=-1;p(this,arguments)},JM,"MeasurementPending",JM.Measurement);c(c$,"set",function(a){return this.setM(a,
null,NaN,0,null,0)},"JM.ModelSet");c(c$,"checkPoint",function(a){for(var b=1;b<=this.numSet;b++)if(this.countPlusIndices[b]==-1-b&&0.01>this.pts[b-1].distance(a))return!1;return!0},"JU.Point3fi");c(c$,"getIndexOf",function(a){for(var b=1;b<=this.numSet;b++)if(this.countPlusIndices[b]==a)return b;return 0},"~N");j(c$,"setCount",function(a){this.setCountM(a);this.numSet=a},"~N");c(c$,"addPoint",function(a,b,d){this.haveModified=a!=this.lastIndex;this.lastIndex=a;if(null==b){if(0<this.getIndexOf(a))return d&&
(this.numSet=this.count),this.count;this.haveTarget=0<=a;if(!this.haveTarget)return this.count=this.numSet;this.count=this.numSet+1;this.countPlusIndices[this.count]=a}else{if(!this.checkPoint(b))return d&&(this.numSet=this.count),this.count;a=this.numSet;this.haveModified=this.haveTarget=!0;this.count=this.numSet+1;this.pts[a]=b;this.countPlusIndices[this.count]=-2-a}this.countPlusIndices[0]=this.count;d&&(this.numSet=this.count);this.value=this.getMeasurement(null);this.strFormat=null;this.formatMeasurement(null);
return this.count},"~N,JU.Point3fi,~B")});s("JM");v(["JU.BS","$.SB"],"JM.Model",["java.util.Hashtable","JU.AU","JU.BSUtil","JV.FileManager"],function(){c$=t(function(){this.mat4=this.ms=null;this.fileIndex=this.modelIndex=0;this.isModelKit=this.isPdbWithMultipleBonds=this.isBioModel=!1;this.translation=this.dataFrames=this.biosymmetry=this.properties=this.auxiliaryInfo=this.orientation=this.dssrCache=this.simpleCage=this.chains=null;this.dataSourceFrame=-1;this.loadState="";this.loadScript=null;this.isTrajectory=
this.isJmolDataFrame=this.structureTainted=this.hasRasmolHBonds=!1;this.act=this.insertionCount=this.altLocCount=this.trajectoryBaseIndex=0;this.bondCount=-1;this.chainCount=0;this.groupCount=-1;this.firstMoleculeIndex=this.firstAtomIndex=this.biosymmetryCount=this.moleculeCount=this.hydrogenCount=0;this.bsAtomsDeleted=this.bsAtoms=null;this.frameDelay=this.defaultRotationRadius=0;this.selectedTrajectory=-1;this.bsCheck=this.pdbID=this.jmolFrameType=this.jmolData=null;this.hasChirality=!1;this.isOrderly=
!0;p(this,arguments)},JM,"Model");O(c$,function(){this.chains=Array(8);this.loadScript=new JU.SB;this.bsAtoms=new JU.BS;this.bsAtomsDeleted=new JU.BS});q(c$,function(){});c(c$,"set",function(a,b,d,c,f,e){this.ms=a;this.dataSourceFrame=this.modelIndex=b;this.trajectoryBaseIndex=(this.isTrajectory=0<=d)?d:b;null==e&&(e=new java.util.Hashtable);this.auxiliaryInfo=e;a=e.get("biosymmetryCount");null!=a&&(this.biosymmetryCount=a.intValue(),this.biosymmetry=e.get("biosymmetry"));a=e.get("fileName");null!=
a&&e.put("fileName",JV.FileManager.stripTypePrefix(a));this.properties=f;null==c?this.jmolFrameType="modelSet":(this.jmolData=c,this.isJmolDataFrame=!0,e.put("jmolData",c),e.put("title",c),this.jmolFrameType=0<=c.indexOf("ramachandran")?"ramachandran":0<=c.indexOf("quaternion")?"quaternion":"data");return this},"JM.ModelSet,~N,~N,~S,java.util.Properties,java.util.Map");c(c$,"getTrueAtomCount",function(){return JU.BSUtil.andNot(this.bsAtoms,this.bsAtomsDeleted).cardinality()});c(c$,"isContainedIn",
function(a){null==this.bsCheck&&(this.bsCheck=new JU.BS);this.bsCheck.clearAll();this.bsCheck.or(a);a=JU.BSUtil.andNot(this.bsAtoms,this.bsAtomsDeleted);this.bsCheck.and(a);return this.bsCheck.equals(a)},"JU.BS");c(c$,"resetBoundCount",function(){this.bondCount=-1});c(c$,"getBondCount",function(){if(0<=this.bondCount)return this.bondCount;var a=this.ms.bo;this.bondCount=0;for(var b=this.ms.bondCount;0<=--b;)a[b].atom1.mi==this.modelIndex&&this.bondCount++;return this.bondCount});c(c$,"getChainCount",
function(a){if(1<this.chainCount&&!a)for(a=0;a<this.chainCount;a++)if(0==this.chains[a].chainID)return this.chainCount-1;return this.chainCount},"~B");c(c$,"calcSelectedGroupsCount",function(a){for(var b=this.chainCount;0<=--b;)this.chains[b].calcSelectedGroupsCount(a)},"JU.BS");c(c$,"getGroupCount",function(){if(0>this.groupCount){this.groupCount=0;for(var a=this.chainCount;0<=--a;)this.groupCount+=this.chains[a].groupCount}return this.groupCount});c(c$,"getChainAt",function(a){return a<this.chainCount?
this.chains[a]:null},"~N");c(c$,"getChain",function(a){for(var b=this.chainCount;0<=--b;){var d=this.chains[b];if(d.chainID==a)return d}return null},"~N");c(c$,"resetDSSR",function(a){this.dssrCache=null;a&&this.auxiliaryInfo.remove("dssr")},"~B");c(c$,"fixIndices",function(a,b,d){this.fixIndicesM(a,b,d)},"~N,~N,JU.BS");c(c$,"fixIndicesM",function(a,b,d){this.dataSourceFrame>a&&this.dataSourceFrame--;this.trajectoryBaseIndex>a&&this.trajectoryBaseIndex--;this.firstAtomIndex-=b;for(a=0;a<this.chainCount;a++)this.chains[a].fixIndices(b,
d);JU.BSUtil.deleteBits(this.bsAtoms,d);JU.BSUtil.deleteBits(this.bsAtomsDeleted,d)},"~N,~N,JU.BS");c(c$,"freeze",function(){this.freezeM();return!1});c(c$,"freezeM",function(){for(var a=0;a<this.chainCount;a++)if(0==this.chains[a].groupCount){for(var b=a+1;b<this.chainCount;b++)this.chains[b-1]=this.chains[b];this.chainCount--}this.chains=JU.AU.arrayCopyObject(this.chains,this.chainCount);this.groupCount=-1;this.getGroupCount();for(a=0;a<this.chainCount;++a)this.chains[a].groups=JU.AU.arrayCopyObject(this.chains[a].groups,
this.chains[a].groupCount)})});s("JM");v(["java.util.Hashtable","JU.BS"],"JM.ModelLoader","java.lang.Boolean $.Float $.NullPointerException java.util.Arrays JU.AU $.Lst $.P3 $.PT $.SB $.V3 J.api.Interface JM.Chain $.Group $.Model $.ModelSet JS.T JU.BSUtil $.Elements $.JmolMolecule $.Logger JV.JC".split(" "),function(){c$=t(function(){this.modelSet0=this.ms=this.vwr=null;this.appendNew=this.merging=!1;this.specialAtomIndexes=this.group3Counts=this.group3Lists=this.jmolData=null;this.doAddHydrogens=
this.doMinimize=this.isPyMOLsession=this.isTrajectory=this.isMutate=this.isMOL2D=this.is2D=this.someModelsAreModulated=this.someModelsHaveUnitcells=!1;this.groups=this.jbr=this.fileHeader=null;this.groupCount=0;this.firstAtomIndexes=this.seqcodes=this.group3Of=this.chainOf=this.htAtomMap=this.modulationTUV=null;this.iModel=0;this.model=null;this.currentChainID=0;this.isNewChain=!1;this.currentChain=null;this.currentGroupSequenceNumber=0;this.currentGroupInsertionCode="\x00";this.nullGroup=this.currentGroup3=
null;this.adapterTrajectoryCount=this.adapterModelCount=this.baseTrajectoryCount=this.baseGroupIndex=this.baseAtomIndex=this.baseModelCount=this.baseModelIndex=0;this.modulationOn=this.noAutoBond=!1;this.$mergeGroups=this.appendToModelIndex=this.htGroup1=null;this.iChain=0;this.vStereo=null;this.lastModel=-1;this.structuresDefinedInFile=null;this.stereodir=1;p(this,arguments)},JM,"ModelLoader");O(c$,function(){this.htAtomMap=new java.util.Hashtable;this.structuresDefinedInFile=new JU.BS});q(c$,function(a,
b,d,c,f,e){this.vwr=a;this.ms=new JM.ModelSet(a,b);var h=a.getModelAdapter();this.modelSet0=f;(this.merging=null!=f&&0<f.ac)?this.ms.canSkipLoad=!1:a.resetShapes(!1);this.ms.preserveState=a.getPreserveState();this.ms.showRebondTimes=a.getBoolean(603979934);null==e?(this.initializeInfo(b,null),this.createModelSet(null,null,null),a.setStringProperty("_fileType","")):(this.ms.preserveState||(this.ms.canSkipLoad=!1),a=h.getAtomSetCollectionAuxiliaryInfo(c),a.put("loadScript",d),this.initializeInfo(h.getFileTypeName(c).toLowerCase().intern(),
a),this.createModelSet(h,c,e),null!=this.jbr&&this.jbr.setLoader(null),this.jbr=null)},"JV.Viewer,~S,JU.SB,~O,JM.ModelSet,JU.BS");c(c$,"initializeInfo",function(a,b){this.ms.g3d=this.vwr.gdata;this.ms.modelSetTypeName=a;this.ms.isXYZ="xyz"===a;this.ms.msInfo=b;this.ms.modelSetProperties=this.ms.getInfoM("properties");this.ms.haveBioModels=this.ms.getMSInfoB("isPDB");this.isMutate=this.ms.getMSInfoB("isMutate");this.ms.haveBioModels&&(this.jbr=this.vwr.getJBR().setLoader(this));this.jmolData=0==this.adapterModelCount?
this.ms.getInfoM("jmolData"):null;this.fileHeader=this.ms.getInfoM("fileHeader");var d=this.ms.getInfoM("trajectorySteps");if(this.isTrajectory=null!=d)this.ms.trajectory=this.newTrajectory(this.ms,d);this.isPyMOLsession=this.ms.getMSInfoB("isPyMOL");this.doAddHydrogens=null!=this.jbr&&!this.isTrajectory&&!this.isPyMOLsession&&!this.ms.getMSInfoB("pdbNoHydrogens")&&(this.ms.getMSInfoB("pdbAddHydrogens")||this.vwr.getBoolean(603979894));null!=b&&(b.remove("pdbNoHydrogens"),b.remove("pdbAddHydrogens"),
b.remove("trajectorySteps"),this.isTrajectory&&(this.ms.vibrationSteps=b.remove("vibrationSteps")),b.containsKey("legacyJavaFloat")&&this.vwr.setBooleanProperty("legacyJavaFloat",!0));this.htGroup1=this.ms.getInfoM("htGroup1");d=this.ms.getInfoM("modulationOn");null!=d&&(this.modulationOn=!0,this.modulationTUV=d===Boolean.TRUE?null:d);this.noAutoBond=this.ms.getMSInfoB("noAutoBond");this.doMinimize=(this.is2D=this.ms.getMSInfoB("is2D"))&&this.ms.getMSInfoB("doMinimize");this.adapterTrajectoryCount=
this.isTrajectory?this.ms.trajectory.steps.size():0;this.ms.someModelsHaveSymmetry=this.ms.getMSInfoB("someModelsHaveSymmetry");this.someModelsHaveUnitcells=this.ms.getMSInfoB("someModelsHaveUnitcells");this.someModelsAreModulated=this.ms.getMSInfoB("someModelsAreModulated");this.ms.someModelsHaveFractionalCoordinates=this.ms.getMSInfoB("someModelsHaveFractionalCoordinates");this.merging&&(this.ms.haveBioModels=(new Boolean(this.ms.haveBioModels|this.modelSet0.haveBioModels)).valueOf(),this.ms.bioModelset=
this.modelSet0.bioModelset,null!=this.ms.bioModelset&&this.ms.bioModelset.set(this.vwr,this.ms),this.ms.someModelsHaveSymmetry=(new Boolean(this.ms.someModelsHaveSymmetry|this.modelSet0.getMSInfoB("someModelsHaveSymmetry"))).valueOf(),this.someModelsHaveUnitcells=(new Boolean(this.someModelsHaveUnitcells|this.modelSet0.getMSInfoB("someModelsHaveUnitcells"))).valueOf(),this.ms.someModelsHaveFractionalCoordinates=(new Boolean(this.ms.someModelsHaveFractionalCoordinates|this.modelSet0.getMSInfoB("someModelsHaveFractionalCoordinates"))).valueOf(),
this.ms.someModelsHaveAromaticBonds=(new Boolean(this.ms.someModelsHaveAromaticBonds|this.modelSet0.someModelsHaveAromaticBonds)).valueOf(),this.ms.msInfo.put("someModelsHaveSymmetry",Boolean.$valueOf(this.ms.someModelsHaveSymmetry)),this.ms.msInfo.put("someModelsHaveUnitcells",Boolean.$valueOf(this.someModelsHaveUnitcells)),this.ms.msInfo.put("someModelsHaveFractionalCoordinates",Boolean.$valueOf(this.ms.someModelsHaveFractionalCoordinates)),this.ms.msInfo.put("someModelsHaveAromaticBonds",Boolean.$valueOf(this.ms.someModelsHaveAromaticBonds)))},
"~S,java.util.Map");c(c$,"newTrajectory",function(a,b){return J.api.Interface.getInterface("JM.Trajectory",this.vwr,"load").set(this.vwr,a,b)},"JM.ModelSet,JU.Lst");c(c$,"getGroup3",function(a){return a>=this.group3Of.length?null:this.group3Of[a]},"~N");c(c$,"getFirstAtomIndex",function(a){return this.firstAtomIndexes[a]},"~N");c(c$,"getAtomCount",function(){return this.ms.ac});c(c$,"createModelSet",function(a,b,d){var c=null==a?0:a.getAtomCount(b);0<c&&JU.Logger.info("reading "+c+" atoms");this.adapterModelCount=
null==a?1:a.getAtomSetCount(b);this.appendToModelIndex=null==this.ms.msInfo?null:this.ms.msInfo.get("appendToModelIndex");this.appendNew=!this.isMutate&&(!this.merging||null==a||1<this.adapterModelCount||this.isTrajectory||this.vwr.getBoolean(603979792)&&null==this.appendToModelIndex);this.htAtomMap.clear();this.chainOf=Array(32);this.group3Of=Array(32);this.seqcodes=B(32,0);this.firstAtomIndexes=B(32,0);this.currentChainID=2147483647;this.currentChain=null;this.currentGroupInsertionCode="\uffff";
this.currentGroup3="xxxxx";this.iModel=-1;this.model=null;this.merging&&this.mergeTrajAndVib(this.modelSet0,this.ms);this.initializeAtomBondModelCounts(c);null!=d&&(this.doMinimize||this.is2D)&&d.setBits(this.baseAtomIndex,this.baseAtomIndex+c);null==a?this.setModelNameNumberProperties(0,-1,"",1,null,null,null):(0<this.adapterModelCount&&(JU.Logger.info("ModelSet: haveSymmetry:"+this.ms.someModelsHaveSymmetry+" haveUnitcells:"+this.someModelsHaveUnitcells+" haveFractionalCoord:"+this.ms.someModelsHaveFractionalCoordinates),
JU.Logger.info(this.adapterModelCount+" model"+(1==this.ms.mc?"":"s")+' in this collection. Use getProperty "modelInfo" or getProperty "auxiliaryInfo" to inspect them.')),d=this.ms.getInfoM("defaultOrientationQuaternion"),null!=d&&(JU.Logger.info("defaultOrientationQuaternion = "+d),JU.Logger.info('Use "set autoLoadOrientation TRUE" before loading or "restore orientation DEFAULT" after loading to view this orientation.')),this.iterateOverAllNewModels(a,b),this.iterateOverAllNewAtoms(a,b),this.iterateOverAllNewBonds(a,
b),this.merging&&!this.appendNew&&(d=a.getAtomSetAuxiliaryInfo(b,0),this.ms.setInfo(this.baseModelIndex,"initialAtomCount",d.get("initialAtomCount")),this.ms.setInfo(this.baseModelIndex,"initialBondCount",d.get("initialBondCount"))),this.initializeUnitCellAndSymmetry(),this.initializeBonding());this.finalizeGroupBuild();this.is2D&&this.doMinimize&&this.applyStereochemistry();this.doAddHydrogens&&this.jbr.finalizeHydrogens();null!=a&&(this.ms.calculatePolymers(this.groups,this.groupCount,this.baseGroupIndex,
null),null!=this.jbr&&this.jbr.iterateOverAllNewStructures(a,b));this.setDefaultRendering(this.vwr.getInt(553648170));var f=this.vwr.rd;d=this.ms.ac;for(var c=this.ms.at,e=this.baseAtomIndex;e<d;e++)c[e].setMadAtom(this.vwr,f);f=this.ms.am;for(e=f[this.baseModelIndex].firstAtomIndex;e<d;e++)null!=c[e]&&f[c[e].mi].bsAtoms.set(e);this.freeze();this.finalizeShapes();this.vwr.setModelSet(this.ms);this.setAtomProperties();null!=a&&a.finish(b);null!=this.modelSet0&&this.modelSet0.releaseModelSet();this.modelSet0=
null},"J.api.JmolAdapter,~O,JU.BS");c(c$,"mergeTrajAndVib",function(a,b){this.baseModelCount=a.mc;this.baseTrajectoryCount=0;null==a.trajectory&&this.isTrajectory&&this.newTrajectory(a,new JU.Lst);if(!(null==a.trajectory||0==a.mc)){this.baseTrajectoryCount=a.mc;for(var d=a.trajectory.steps.size(),c=d;c<this.baseTrajectoryCount;c++)a.trajectory.steps.addLast(null);if(this.isTrajectory){if(null==a.vibrationSteps){a.vibrationSteps=new JU.Lst;for(c=d;0<=--c;)a.vibrationSteps.addLast(null)}d=b.trajectory.steps.size();
for(c=0;c<d;c++)a.trajectory.steps.addLast(b.trajectory.steps.get(c)),a.vibrationSteps.addLast(null==b.vibrationSteps?null:b.vibrationSteps.get(c))}else b.trajectory=this.newTrajectory(b,null);b.vibrationSteps=a.vibrationSteps;b.trajectory.steps=a.trajectory.steps;a.trajectory=null}},"JM.ModelSet,JM.ModelSet");c(c$,"setDefaultRendering",function(a){if(!this.isPyMOLsession){for(var b=new JU.SB,d=this.ms.mc,c=this.ms.am,f=this.baseModelIndex;f<d;f++)c[f].isBioModel&&c[f].getDefaultLargePDBRendering(b,
a);0!=b.length()&&(b.append("select *;"),a=this.ms.getInfoM("jmolscript"),null==a&&(a=""),b.append(a),this.ms.msInfo.put("jmolscript",b.toString()))}},"~N");c(c$,"setAtomProperties",function(){for(var a=this.ms.mc,b=this.baseModelIndex;b<a;b++){var d=this.ms.getInfo(b,"atomProperties");null!=this.jmolData&&this.addJmolDataProperties(this.ms.am[b],this.ms.getInfo(b,"jmolDataProperties"));var c=this.ms.getInfo(b,"groupPropertyList");this.ms.am[b].isBioModel&&null!=this.ms.getInfo(b,"dssr")&&this.vwr.getAnnotationParser(!0).setGroup1(this.ms,
b);if(null!=d)for(var f,d=d.entrySet().iterator();d.hasNext()&&((f=d.next())||1);){var e=f.getKey(),h=f.getValue(),k=this.ms.getModelAtomBitSetIncludingDeleted(b,!0);if(this.doAddHydrogens)var j=null!=c&&JU.PT.isOneOf(e,c),h=this.jbr.fixPropertyValue(k,h,j);e="property_"+e.toLowerCase();JU.Logger.info("creating "+e+" for model "+this.ms.getModelName(b));this.vwr.setData(e,u(-1,[e,h,k,Integer.$valueOf(-1),Boolean.FALSE]),this.ms.ac,0,0,2147483647,0)}}});c(c$,"initializeAtomBondModelCounts",function(a){var b=
this.adapterTrajectoryCount;if(this.merging){if(this.appendNew)this.baseModelIndex=this.baseModelCount,this.ms.mc=this.baseModelCount+this.adapterModelCount;else{this.baseModelIndex=null==this.appendToModelIndex?this.vwr.am.cmi:this.appendToModelIndex.intValue();if(0>this.baseModelIndex||this.baseModelIndex>=this.baseModelCount)this.baseModelIndex=this.baseModelCount-1;this.ms.mc=this.baseModelCount}this.ms.ac=this.baseAtomIndex=this.modelSet0.ac;this.ms.bondCount=this.modelSet0.bondCount;this.$mergeGroups=
this.modelSet0.getGroups();this.groupCount=this.baseGroupIndex=this.$mergeGroups.length;this.ms.mergeModelArrays(this.modelSet0);this.ms.growAtomArrays(this.ms.ac+a)}else this.ms.mc=this.adapterModelCount,this.ms.ac=0,this.ms.bondCount=0,this.ms.at=Array(a),this.ms.bo=Array(250+a);this.doAddHydrogens&&this.jbr.initializeHydrogenAddition();1<b&&(this.ms.mc+=b-1);this.ms.am=JU.AU.arrayCopyObject(this.ms.am,this.ms.mc);this.ms.modelFileNumbers=JU.AU.arrayCopyI(this.ms.modelFileNumbers,this.ms.mc);this.ms.modelNumbers=
JU.AU.arrayCopyI(this.ms.modelNumbers,this.ms.mc);this.ms.modelNumbersForAtomLabel=JU.AU.arrayCopyS(this.ms.modelNumbersForAtomLabel,this.ms.mc);this.ms.modelNames=JU.AU.arrayCopyS(this.ms.modelNames,this.ms.mc);this.ms.frameTitles=JU.AU.arrayCopyS(this.ms.frameTitles,this.ms.mc);if(this.merging)for(a=0;a<this.modelSet0.mc;a++)(this.ms.am[a]=this.modelSet0.am[a]).ms=this.ms},"~N");c(c$,"mergeGroups",function(){var a=this.modelSet0.getAuxiliaryInfo(null),b=a.get("group3Lists"),a=a.get("group3Counts");
if(null!=b){for(var d=0;d<this.baseModelCount;d++)this.group3Lists[d+1]=b[d+1],this.group3Counts[d+1]=a[d+1],this.structuresDefinedInFile.set(d);this.group3Lists[0]=b[0];this.group3Counts[0]=a[0]}!this.appendNew&&this.ms.haveBioModels&&this.structuresDefinedInFile.clear(this.baseModelIndex)});c(c$,"iterateOverAllNewModels",function(a,b){this.group3Lists=Array(this.ms.mc+1);this.group3Counts=JU.AU.newInt2(this.ms.mc+1);this.structuresDefinedInFile=new JU.BS;this.merging&&this.mergeGroups();for(var d=
this.isTrajectory?this.baseTrajectoryCount:-1,c=this.baseModelIndex,f=0;f<this.adapterModelCount;++f,++c){var e=a.getAtomSetNumber(b,f),h=a.getAtomSetName(b,f),k=a.getAtomSetAuxiliaryInfo(b,f);k.containsKey("modelID")&&k.put("modelID0",k.get("modelID"));var j=k.get("modelProperties");this.vwr.setStringProperty("_fileType",k.get("fileType"));null==h&&(h=null!=this.jmolData&&2<this.jmolData.indexOf(";")?this.jmolData.substring(this.jmolData.indexOf(":")+2,this.jmolData.indexOf(";")):this.appendNew?
""+e%1E6:"");this.setModelNameNumberProperties(c,d,h,e,j,k,this.jmolData)}d=this.ms.am[null==this.appendToModelIndex?this.baseModelIndex:this.ms.mc-1];this.vwr.setSmilesString(this.ms.msInfo.get("smilesString"));f=this.ms.msInfo.remove("loadState");e=this.ms.msInfo.remove("loadScript");if(0>e.indexOf("Viewer.AddHydrogens")||!d.isModelKit){h=JU.PT.split(f,"\n");k=new JU.SB;for(f=0;f<h.length;f++)j=d.loadState.indexOf(h[f]),(0>j||j!=d.loadState.lastIndexOf(h[f]))&&k.append(h[f]).appendC("\n");d.loadState+=
d.loadScript.toString()+k.toString();d.loadScript=new JU.SB;if(0<=e.indexOf("load append ")||0<=e.indexOf('data "append '))e.insert(0,";var anew = appendNew;"),e.append(";set appendNew anew");d.loadScript.append("  ").appendSB(e).append(";\n")}if(this.isTrajectory){f=this.ms.mc-c+1;JU.Logger.info(f+" trajectory steps read");this.ms.setInfo(this.baseModelCount,"trajectoryStepCount",Integer.$valueOf(f));d=this.adapterModelCount;for(f=c;f<this.ms.mc;f++,d++)this.ms.am[f]=this.ms.am[this.baseModelCount],
this.ms.modelNumbers[f]=a.getAtomSetNumber(b,d),this.ms.modelNames[f]=a.getAtomSetName(b,d),this.structuresDefinedInFile.set(f)}this.finalizeModels(this.baseModelCount)},"J.api.JmolAdapter,~O");c(c$,"setModelNameNumberProperties",function(a,b,d,c,f,e,h){if(this.appendNew){var k=null!=e&&Boolean.TRUE===e.get("isPDB");this.ms.am[a]=k?this.jbr.getBioModel(a,b,h,f,e):(new JM.Model).set(this.ms,a,b,h,f,e);this.ms.modelNumbers[a]=c;this.ms.modelNames[a]=d;k&&this.jbr.setGroupLists(a)}else b=e.get("PDB_CONECT_firstAtom_count_max"),
null!=b&&this.ms.setInfo(a,"PDB_CONECT_firstAtom_count_max",b);b=this.ms.am;c=this.ms.at;b[a].bsAtoms.set(c.length+1);b[a].bsAtoms.clear(c.length+1);c=this.ms.getInfo(a,"altLocs");b[a].altLocCount=null==c?0:c.length;null!=c&&(c=c.toCharArray(),java.util.Arrays.sort(c),c=String.valueOf(c),this.ms.setInfo(a,"altLocs",c));c=this.ms.getInfo(a,"insertionCodes");b[a].insertionCount=null==c?0:c.length;d=null!=this.ms.modelSetName&&this.ms.modelSetName.startsWith("Jmol Model Kit")||d.startsWith("Jmol Model Kit")||
"Jme".equals(this.ms.getInfo(a,"fileType"));b[a].isModelKit=d},"~N,~N,~S,~N,java.util.Properties,java.util.Map,~S");c(c$,"finalizeModels",function(a){var b=this.ms.mc;if(b!=a){var d,c=0,f=-1,e=this.ms.modelNumbers,h=this.ms.modelNames;if(this.isTrajectory)for(var k=a;++k<this.ms.mc;)e[k]=e[k-1]+1;if(0<a){if(1E6>e[0])for(k=0;k<a;k++)0==h[k].length&&(h[k]=""+e[k]),e[k]+=1E6,this.ms.modelNumbersForAtomLabel[k]="1."+(k+1);var j=e[a-1],j=j-j%1E6;1E6>e[a]&&(j+=1E6);for(k=a;k<b;k++)e[k]+=j}for(var z=this.ms.am,
k=a;k<b;++k)if(this.ms.setInfo(k,"fileType",this.ms.modelSetTypeName),null!=this.fileHeader&&this.ms.setInfo(k,"fileHeader",this.fileHeader),j=w(e[k]/1E6),j!=f&&(c=0,f=j),c++,0==j?(d=""+this.ms.getModelNumber(k),j=1):d=j+"."+c,this.ms.modelNumbersForAtomLabel[k]=d,z[k].fileIndex=j-1,this.ms.modelFileNumbers[k]=1E6*j+c,null==h[k]||0==h[k].length)h[k]=d;if(this.merging)for(k=0;k<a;k++)z[k].ms=this.ms;for(k=0;k<b;k++)this.ms.setInfo(k,"modelName",h[k]),this.ms.setInfo(k,"modelNumber",Integer.$valueOf(e[k]%
1E6)),this.ms.setInfo(k,"modelFileNumber",Integer.$valueOf(this.ms.modelFileNumbers[k])),this.ms.setInfo(k,"modelNumberDotted",this.ms.getModelNumberDotted(k)),a=this.ms.getInfo(k,"altLocs"),null!=a&&JU.Logger.info("model "+this.ms.getModelNumberDotted(k)+" alternative locations: "+a)}},"~N");c(c$,"iterateOverAllNewAtoms",function(a,b){var d=-1,c=!1,f=!1,e=a.getAtomIterator(b);this.ms.setCapacity(a.getAtomCount(b));var h=0,k=this.ms.am;0<this.ms.mc&&(this.nullGroup=(new JM.Group).setGroup(new JM.Chain(this.ms.am[this.baseModelIndex],
32,0),"",0,-1,-1));for(;e.hasNext();){h++;var j=e.getAtomSetIndex()+this.baseModelIndex;j!=d&&(this.iChain=0,this.iModel=j,this.model=k[j],this.currentChainID=2147483647,this.isNewChain=!0,this.model.bsAtoms.clearAll(),this.model.isOrderly=null==this.appendToModelIndex,c=this.model.isBioModel,d=j,f=c&&this.doAddHydrogens,null!=this.jbr&&this.jbr.setHaveHsAlready(!1));var j=e.getGroup3(),z=e.getChainID();this.checkNewGroup(a,z,j,e.getSequenceNumber(),e.getInsertionCode(),f,!1);z=e.getElementNumber();
f&&1==JU.Elements.getElementNumber(z)&&this.jbr.setHaveHsAlready(!0);var m=e.getAtomName(),r=f?this.getPdbCharge(j,m):e.getFormalCharge();this.addAtom(c,e.getSymmetry(),e.getAtomSite(),e.getUniqueID(),z,m,r,e.getPartialCharge(),e.getTensors(),e.getOccupancy(),e.getBfactor(),e.getXYZ(),e.getIsHetero(),e.getSerial(),e.getSeqID(),j,e.getVib(),e.getAltLoc(),e.getRadius(),e.getBondRadius())}0<this.groupCount&&f&&this.jbr.addImplicitHydrogenAtoms(a,this.groupCount-1,this.isNewChain?1:0);d=-1;c=null;f=this.ms.at;
for(e=k[0].firstAtomIndex=0;e<this.ms.ac;e++)null!=f[e]&&f[e].mi>d&&(d=f[e].mi,j=k[d],j.firstAtomIndex=e,j.isOrderly=j.act==j.bsAtoms.length(),j=this.ms.getDefaultVdwType(d),j!==c&&(JU.Logger.info("Default Van der Waals type for model set to "+j.getVdwLabel()),c=j));JU.Logger.info(h+" atoms created")},"J.api.JmolAdapter,~O");c(c$,"addJmolDataProperties",function(a,b){if(null!=b)for(var d=a.bsAtoms,c=d.cardinality(),f,e=b.entrySet().iterator();e.hasNext()&&((f=e.next())||1);){var h=f.getKey(),k=f.getValue();
if(k.length!=c)break;var j=h.startsWith("property_")?1715472409:JS.T.getTokFromName(h);switch(j){default:if(JS.T.tokAttr(j,2048)){this.vwr.setAtomProperty(d,j,0,0,null,k,null);break}case 1111492629:case 1111492630:case 1111492631:h="property_"+h;case 1715472409:this.vwr.setData(h,u(-1,[h,k,d,Integer.$valueOf(1)]),0,0,0,0,0)}}},"JM.Model,java.util.Map");c(c$,"getPdbCharge",function(a,b){return a.equals("ARG")&&b.equals("NH1")||a.equals("LYS")&&b.equals("NZ")||a.equals("HIS")&&b.equals("ND1")?1:0},
"~S,~S");c(c$,"addAtom",function(a,b,d,c,f,e,h,k,j,z,m,r,n,p,q,s,u,t,v,w){var A=0,B=null;if(null!=e){var D;if(0<=(D=e.indexOf("\x00")))B=e.substring(D+1),e=e.substring(0,D);a&&(0<=e.indexOf("*")&&(e=e.$replace("*","'")),A=this.vwr.getJBR().lookupSpecialAtomID(e),2==A&&"CA".equalsIgnoreCase(s)&&(A=0))}a=this.ms.addAtom(this.iModel,this.nullGroup,f,e,B,p,q,d,r,v,u,h,k,z,m,j,n,A,b,w);a.altloc=t;this.htAtomMap.put(c,a)},"~B,JU.BS,~N,~O,~N,~S,~N,~N,JU.Lst,~N,~N,JU.P3,~B,~N,~N,~S,JU.V3,~S,~N,~N");c(c$,
"checkNewGroup",function(a,b,d,c,f,e,h){var k=null==d?null:d.intern();b!=this.currentChainID&&(this.currentChainID=b,this.currentChain=this.getOrAllocateChain(this.model,b),this.currentGroupInsertionCode="\uffff",this.currentGroupSequenceNumber=-1,this.currentGroup3="xxxx",this.isNewChain=!0);if(c!=this.currentGroupSequenceNumber||f!=this.currentGroupInsertionCode||k!==this.currentGroup3){0<this.groupCount&&e&&(this.jbr.addImplicitHydrogenAtoms(a,this.groupCount-1,this.isNewChain&&!h?1:0),this.jbr.setHaveHsAlready(!1));
this.currentGroupSequenceNumber=c;this.currentGroupInsertionCode=f;for(this.currentGroup3=k;this.groupCount>=this.group3Of.length;)this.chainOf=JU.AU.doubleLength(this.chainOf),this.group3Of=JU.AU.doubleLengthS(this.group3Of),this.seqcodes=JU.AU.doubleLengthI(this.seqcodes),this.firstAtomIndexes=JU.AU.doubleLengthI(this.firstAtomIndexes);this.firstAtomIndexes[this.groupCount]=this.ms.ac;this.chainOf[this.groupCount]=this.currentChain;this.group3Of[this.groupCount]=d;this.seqcodes[this.groupCount]=
JM.Group.getSeqcodeFor(c,f);++this.groupCount}},"J.api.JmolAdapter,~N,~S,~N,~S,~B,~B");c(c$,"getOrAllocateChain",function(a,b){var d=a.getChain(b);if(null!=d)return d;a.chainCount==a.chains.length&&(a.chains=JU.AU.doubleLength(a.chains));return a.chains[a.chainCount++]=new JM.Chain(a,b,0==b||32==b?0:++this.iChain)},"JM.Model,~N");c(c$,"iterateOverAllNewBonds",function(a,b){var d=a.getBondIterator(b);if(null!=d){var c=this.vwr.getMadBond(),f=this.isMutate&&!this.vwr.getBoolean(603979894);this.ms.defaultCovalentMad=
null==this.jmolData?c:0;for(var e=!1;d.hasNext();){var h=d.getEncodedOrder(),k=f?1:h,j=this.bondAtoms(d.getAtomUniqueID1(),d.getAtomUniqueID2(),k);null!=j&&(1<k&&(1025!=k&&1041!=k)&&(e=!0),k=d.getRadius(),0<k&&j.setMad($(2E3*k)),k=d.getColix(),0<=k&&(j.colix=k),j.order|=h&98304)}e&&(this.ms.someModelsHaveSymmetry&&!this.vwr.getBoolean(603979794))&&JU.Logger.info('ModelSet: use "set appletSymmetryToBonds TRUE " to apply the file-based multiple bonds to symmetry-generated atoms.');this.ms.defaultCovalentMad=
c}},"J.api.JmolAdapter,~O");c(c$,"bondAtoms",function(a,b,d){var c=this.htAtomMap.get(a);if(null==c)return JU.Logger.error("bondAtoms cannot find atomUid1?:"+a),null;a=this.htAtomMap.get(b);if(null==a)return JU.Logger.error("bondAtoms cannot find atomUid2?:"+b),null;if(c.isBonded(a))return null;1025==d||1041==d?(b=c.getModelIndex(),b!=this.lastModel&&(this.lastModel=b,b=this.ms.getModelAuxiliaryInfo(b),this.isMOL2D=null!=b&&"2D".equals(b.get("dimension"))),d=this.ms.bondMutually(c,a,this.isMOL2D?
d:1,this.ms.getDefaultMadFromOrder(1),0),null==this.vStereo&&(this.vStereo=new JU.Lst),this.vStereo.addLast(d)):(d=this.ms.bondMutually(c,a,d,this.ms.getDefaultMadFromOrder(d),0),d.isAromatic()&&(this.ms.someModelsHaveAromaticBonds=!0));this.ms.bondCount==this.ms.bo.length&&(this.ms.bo=JU.AU.arrayCopyObject(this.ms.bo,this.ms.bondCount+250));this.ms.setBond(this.ms.bondCount++,d);return d},"~O,~O,~N");c(c$,"initializeUnitCellAndSymmetry",function(){this.someModelsAreModulated&&null==this.ms.bsModulated&&
(this.ms.bsModulated=new JU.BS);if(this.someModelsHaveUnitcells){this.ms.unitCells=Array(this.ms.mc);this.ms.haveUnitCells=!0;for(var a=null!=this.modelSet0&&null!=this.modelSet0.unitCells,b=0,d=0;b<this.ms.mc;b++)if(a&&b<this.baseModelCount)this.ms.unitCells[b]=this.modelSet0.unitCells[b];else{this.ms.unitCells[b]=J.api.Interface.getSymmetry(this.vwr,"file");var c=null;if(this.isTrajectory){var f=this.ms.getInfoM("unitCells");null!=f&&(c=f.get(d++))}this.ms.unitCells[b].setSymmetryInfo(b,this.ms.getModelAuxiliaryInfo(b),
c)}}if(this.appendNew&&this.ms.someModelsHaveSymmetry){this.ms.getAtoms(1088421903,null);a=this.ms.at;b=this.baseAtomIndex;d=-1;for(c=0;b<this.ms.ac;b++)a[b].mi!=d&&(d=a[b].mi,c=this.baseAtomIndex+this.ms.getInfoI(d,"presymmetryAtomIndex")+this.ms.getInfoI(d,"presymmetryAtomCount")),b>=c&&this.ms.bsSymmetry.set(b)}if(this.appendNew&&this.ms.someModelsHaveFractionalCoordinates){for(var a=this.ms.at,d=-1,c=null,f=!1,e=!this.vwr.g.legacyJavaFloat,b=this.baseAtomIndex;b<this.ms.ac;b++)a[b].mi!=d&&(d=
a[b].mi,c=this.ms.getUnitCell(d),f=null!=c&&c.getCoordinatesAreFractional()),f&&(c=a[b].getUnitCell(),c.toCartesian(c.toSupercell(a[b]),!1),e&&JU.PT.fixPtFloats(a[b],1E4));for(b=this.baseModelIndex;b<this.ms.mc;b++)this.ms.isTrajectory(b)&&this.ms.trajectory.setUnitCell(b)}});c(c$,"initializeBonding",function(){var a=this.ms.mc,b=this.ms.am,d=0,c=this.ms.getInfoM("bsExcludeBonding");null==c&&(c=null==this.ms.getInfoM("someModelsHaveCONECT")?null:new JU.BS,null!=c&&this.ms.setPdbConectBonding(this.baseAtomIndex,
this.baseModelIndex,c));var f=this.vwr.getBoolean(603979794),e=this.vwr.getBoolean(603979798),h=this.vwr.getBoolean(603979846),k=null,j=!1;if(!this.noAutoBond)for(var z=this.baseModelIndex;z<a;z++){var d=b[z].bsAtoms.cardinality(),m=this.ms.getInfoI(z,"initialBondCount"),r=b[z].isBioModel;0>m&&(m=this.ms.bondCount);if(h||e&&(0==m||r&&null==this.jmolData&&(this.ms.getMSInfoB("havePDBHeaderName")||m<w(d/2))||this.ms.getInfoB(z,"hasSymmetry")&&!f&&!this.ms.getInfoB(z,"hasBonds")))if(j=!0,this.merging||
1<a)null==k&&(k=JU.BS.newN(this.ms.ac)),(z==this.baseModelIndex||!this.isTrajectory)&&k.or(b[z].bsAtoms)}this.modulationOn&&this.ms.setModulation(null,!0,this.modulationTUV,!1);j?(this.ms.autoBondBs4(k,k,c,null,this.ms.defaultCovalentMad,this.vwr.getBoolean(603979872)),JU.Logger.info("ModelSet: autobonding; use  autobond=false  to not generate bonds automatically")):JU.Logger.info("ModelSet: not autobonding; use  forceAutobond=true  to force automatic bond creation")});c(c$,"finalizeGroupBuild",function(){this.groups=
Array(this.groupCount);if(this.merging)for(var a=0;a<this.$mergeGroups.length;a++)(this.groups[a]=this.$mergeGroups[a]).chain.model.ms=this.ms;for(a=this.baseGroupIndex;a<this.groupCount;++a)this.distinguishAndPropagateGroup(a,this.chainOf[a],this.group3Of[a],this.seqcodes[a],this.firstAtomIndexes[a],(a==this.groupCount-1?this.ms.ac:this.firstAtomIndexes[a+1])-1);if(null!=this.group3Lists){this.ms.msInfo.put("group3Lists",this.group3Lists);this.ms.msInfo.put("group3Counts",this.group3Counts);for(a=
0;a<this.group3Counts.length;a++)null==this.group3Counts[a]&&(this.group3Counts[a]=B(0,0))}});c(c$,"distinguishAndPropagateGroup",function(a,b,d,c,f,e){if(e<f)throw new NullPointerException;var h=null==d||null==this.jbr?null:this.jbr.distinguishAndPropagateGroup(b,d,c,f,e,this.specialAtomIndexes,this.ms.at);null==h?(h=(new JM.Group).setGroup(b,d,c,f,e),null!=this.jbr&&(h.groupID=this.jbr.getGroupID(d)),c="o>"):c=h.isProtein()?"p>":h.isNucleic()?"n>":h.isCarbohydrate()?"c>":"o>";null!=d&&(this.countGroup(this.ms.at[f].mi,
c,d),h.isNucleic()&&(d=null==this.htGroup1?null:this.htGroup1.get(d),null!=d&&(h.group1=d.charAt(0))));this.addGroup(b,h);this.groups[a]=h;h.groupIndex=a;for(a=e+1;--a>=f;)this.ms.at[a].group=h},"~N,JM.Chain,~S,~N,~N,~N");c(c$,"addGroup",function(a,b){a.groupCount==a.groups.length&&(a.groups=JU.AU.doubleLength(a.groups));a.groups[a.groupCount++]=b},"JM.Chain,JM.Group");c(c$,"countGroup",function(a,b,d){var c=a+1;if(!(null==this.group3Lists||null==this.group3Lists[c])){var f=(d+"   ").substring(0,
3),e=this.group3Lists[c].indexOf(f);0>e&&(this.group3Lists[c]+=",["+f+"]",e=this.group3Lists[c].indexOf(f),this.group3Counts[c]=JU.AU.arrayCopyI(this.group3Counts[c],this.group3Counts[c].length+10));this.group3Counts[c][w(e/6)]++;e=this.group3Lists[c].indexOf(",["+f);0<=e&&(this.group3Lists[c]=this.group3Lists[c].substring(0,e)+b+this.group3Lists[c].substring(e+2));0<=a&&this.countGroup(-1,b,d)}},"~N,~S,~S");c(c$,"freeze",function(){this.htAtomMap.clear();this.ms.ac<this.ms.at.length&&this.ms.growAtomArrays(this.ms.ac);
this.ms.bondCount<this.ms.bo.length&&(this.ms.bo=JU.AU.arrayCopyObject(this.ms.bo,this.ms.bondCount));for(var a=5;0<--a;){this.ms.numCached[a]=0;for(var b=this.ms.freeBonds[a],d=b.length;0<=--d;)b[d]=null}this.ms.setAtomNamesAndNumbers(0,this.baseAtomIndex,this.modelSet0);this.findElementsPresent();this.ms.resetMolecules();this.currentChain=this.model=null;!this.ms.haveBioModels||this.isPyMOLsession||this.isMutate?this.ms.freezeModels():(a=this.vwr.getBoolean(603979826),a=this.ms.calculateStructuresAllExcept(this.structuresDefinedInFile,
a,!1,!0,!0,a,1405E3<=JV.JC.versionInt&&null==this.ms.getInfoM("DSSP1")?2:1),0<a.length&&JU.Logger.info(a))});c(c$,"findElementsPresent",function(){this.ms.elementsPresent=Array(this.ms.mc);for(var a=0;a<this.ms.mc;a++)this.ms.elementsPresent[a]=JU.BS.newN(64);for(a=this.ms.ac;0<=--a;){var b=this.ms.at[a];if(null!=b){var d=b.getAtomicAndIsotopeNumber();d>=JU.Elements.elementNumberMax&&(d=JU.Elements.elementNumberMax+JU.Elements.altElementIndexFromNumber(d));this.ms.elementsPresent[b.mi].set(d)}}});
c(c$,"applyStereochemistry",function(){this.set2DLengths(this.baseAtomIndex,this.ms.ac);var a=new JU.V3;if(null!=this.vStereo){var b=this.vStereo.size();a:for(;0<=--b;)for(var d=this.vStereo.get(b),c=d.atom1,f=c.bonds,e=c.getBondCount();0<=--e;){var h=f[e];if(h!==d&&(h=h.getOtherAtom(c),a.sub2(h,c),0.1>Math.abs(a.x))){1025==d.order==0>a.y&&(this.stereodir=-1);break a}}}this.set2dZ(this.baseAtomIndex,this.ms.ac,a);if(null!=this.vStereo){a=new JU.BS;a.setBits(this.baseAtomIndex,this.ms.ac);for(b=this.vStereo.size();0<=
--b;){d=this.vStereo.get(b);c=1025==d.order?3:-3;d.order=1;d.atom2.z!=d.atom1.z&&0>c==d.atom2.z<d.atom1.z&&(c/=3);f=JU.JmolMolecule.getBranchBitSet(this.ms.at,d.atom2.i,a,null,d.atom1.i,!1,!0);f.set(d.atom2.i);for(e=f.nextSetBit(0);0<=e;e=f.nextSetBit(e+1))this.ms.at[e].z+=c;d.atom2.x=(d.atom1.x+d.atom2.x)/2;d.atom2.y=(d.atom1.y+d.atom2.y)/2}this.vStereo=null}this.is2D=!1});c(c$,"set2DLengths",function(a,b){for(var d=0,c=0,f=a;f<b;f++){var e=this.ms.at[f],h=e.bonds;if(null!=h)for(var k=h.length;0<=
--k;)if(null!=h[k]){var j=h[k].getOtherAtom(e);1!=j.getAtomNumber()&&j.getIndex()>f&&(d+=j.distance(e),c++)}}if(0!=c){d=1.45/(d/c);for(f=a;f<b;f++)this.ms.at[f].scale(d)}},"~N,~N");c(c$,"set2dZ",function(a,b,d){var c=JU.BS.newN(b),f=new JU.BS,e=JU.V3.new3(0,1,0),h=new JU.V3,k=new JU.BS;for(k.setBits(a,b);a<b;a++)!c.get(a)&&!f.get(a)&&(f=this.getBranch2dZ(a,-1,k,f,d,e,h,this.stereodir),c.or(f))},"~N,~N,JU.V3");c(c$,"getBranch2dZ",function(a,b,d,c,f,e,h,k){c=JU.BS.newN(this.ms.ac);if(0>a)return c;var j=
new JU.BS;j.or(d);0<=b&&j.clear(b);JM.ModelLoader.setBranch2dZ(this.ms.at[a],c,j,f,e,h,k);return c},"~N,~N,JU.BS,JU.BS,JU.V3,JU.V3,JU.V3,~N");c$.setBranch2dZ=c(c$,"setBranch2dZ",function(a,b,d,c,f,e,h){var k=a.i;if(d.get(k)&&(d.clear(k),b.set(k),null!=a.bonds))for(k=a.bonds.length;0<=--k;){var j=a.bonds[k];j.isHydrogen()||(j=j.getOtherAtom(a),JM.ModelLoader.setAtom2dZ(a,j,c,f,e,h),JM.ModelLoader.setBranch2dZ(j,b,d,c,f,e,h))}},"JM.Atom,JU.BS,JU.BS,JU.V3,JU.V3,JU.V3,~N");c$.setAtom2dZ=c(c$,"setAtom2dZ",
function(a,b,d,c,f,e){d.sub2(b,a);d.z=0;d.normalize();f.cross(c,d);d=Math.acos(d.dot(c));e=0.4*-e*Math.sin(4*d);b.z=a.z+e},"JM.Atom,JM.Atom,JU.V3,JU.V3,JU.V3,~N");c(c$,"finalizeShapes",function(){this.ms.sm=this.vwr.shm;this.ms.sm.setModelSet(this.ms);this.ms.setBsHidden(this.vwr.slm.getHiddenSet());this.merging||this.ms.sm.resetShapes();this.ms.sm.loadDefaultShapes(this.ms);this.ms.someModelsHaveAromaticBonds&&this.vwr.getBoolean(603979944)&&this.ms.assignAromaticBondsBs(!1,null);this.merging&&1==
this.baseModelCount&&this.ms.sm.setShapePropertyBs(6,"clearModelIndex",null,null)});c(c$,"undeleteAtom",function(a){this.ms.at[a].valence=0},"~N");c$.createAtomDataSet=c(c$,"createAtomDataSet",function(a,b,d,c,f){if(null==c)return null;var e=a.getModelAdapter(),h=new JU.P3,k=b.at,j=a.getFloat(570425363);if(null!=b.unitCells)for(var z=f.nextSetBit(0);0<=z;z=f.nextSetBit(z+1))if(null!=k[z].atomSymmetry){j=-j;break}for(var z=-1,m=0,r=JU.BSUtil.cardinalityOf(f)==a.ms.ac,n=e.getAtomIterator(c);n.hasNext();){var p=
n.getXYZ();if(!Float.isNaN(p.x+p.y+p.z))if(1145047050==d){z=f.nextSetBit(z+1);if(0>z)break;m++;JU.Logger.debugging&&JU.Logger.debug("atomIndex = "+z+": "+k[z]+" --\x3e ("+p.x+","+p.y+","+p.z);b.setAtomCoord(z,p.x,p.y,p.z)}else{h.setT(p);p=JU.BS.newN(b.ac);b.getAtomsWithin(j,h,p,-1);p.and(f);if(r)if(m=JU.BSUtil.cardinalityOf(p),0==m){JU.Logger.warn("createAtomDataSet: no atom found at position "+h);continue}else 1<m&&JU.Logger.debugging&&JU.Logger.debug("createAtomDataSet: "+m+" atoms found at position "+
h);switch(d){case 1145047055:var q=n.getVib();if(null==q)continue;JU.Logger.debugging&&JU.Logger.debug("xyz: "+h+" vib: "+q);b.setAtomCoords(p,1145047055,q);break;case 1128269825:b.setAtomProperty(p,d,0,n.getOccupancy(),null,null,null);break;case 1111492619:b.setAtomProperty(p,d,0,n.getPartialCharge(),null,null,null);break;case 1111492620:b.setAtomProperty(p,d,0,n.getBfactor(),null,null,null)}}}switch(d){case 1145047055:b=e.getAtomSetName(c,0);JU.Logger.info("_vibrationName = "+b);a.setStringProperty("_vibrationName",
b);break;case 1145047050:if(JU.Logger.info(m+" atom positions read"),b.recalculateLeadMidpointsAndWingVectors(-1),m==b.ac)return"boundbox {*};reset"}return null},"JV.Viewer,JM.ModelSet,~N,~O,JU.BS");E(c$,"defaultGroupCount",32)});s("JM");v(["JM.BondCollection"],"JM.ModelSet","java.lang.Boolean $.Float java.util.Hashtable JU.A4 $.AU $.BS $.Lst $.M3 $.M4 $.Measure $.P3 $.P4 $.PT $.Quat $.SB $.V3 J.api.Interface $.JmolModulationSet J.atomdata.RadiusData J.bspt.Bspf J.c.PAL $.VDW JM.Atom $.AtomIteratorWithinModel $.AtomIteratorWithinModelSet $.HBond $.Model $.StateScript JS.ScriptCompiler JU.BSUtil $.BoxInfo $.Edge $.Elements $.Escape $.JmolMolecule $.Logger $.SimpleUnitCell JV.JC".split(" "),
function(){c$=t(function(){this.haveBioModels=!1;this.am=this.modelSetName=this.bsSymmetry=null;this.mc=0;this.unitCells=null;this.haveUnitCells=!1;this.elementsPresent=this.frameTitles=this.modelNames=this.modelNumbersForAtomLabel=this.modelFileNumbers=this.modelNumbers=this.closest=null;this.isXYZ=!1;this.msInfo=this.modelSetProperties=null;this.isBbcageDefault=this.someModelsHaveFractionalCoordinates=this.someModelsHaveAromaticBonds=this.someModelsHaveSymmetry=!1;this.stateScripts=this.boxInfo=
this.bboxAtoms=this.bboxModels=null;this.thisStateModel=0;this.selectedMolecules=this.vibrationSteps=null;this.showRebondTimes=!0;this.sm=this.bsAll=null;this.proteinStructureTainted=!1;this.vTemp=this.mat4t=this.mat4=this.matInv=this.matTemp=this.ptTemp2=this.ptTemp1=this.ptTemp=this.vOrientations=this.htPeaks=null;this.echoShapeActive=!1;this.defaultBBox=this.translations=this.modelSetTypeName=null;this.maxBondWarned=!1;p(this,arguments)},JM,"ModelSet",JM.BondCollection);c(c$,"getBoxInfo",function(){return this.boxInfo});
q(c$,function(a,b){I(this,JM.ModelSet,[]);this.vwr=a;this.modelSetName=b;this.selectedMolecules=new JU.BS;this.stateScripts=new JU.Lst;this.boxInfo=new JU.BoxInfo;this.boxInfo.addBoundBoxPoint(JU.P3.new3(-10,-10,-10));this.boxInfo.addBoundBoxPoint(JU.P3.new3(10,10,10));this.am=Array(1);this.modelNumbers=B(1,0);this.modelFileNumbers=B(1,0);this.modelNumbersForAtomLabel=Array(1);this.modelNames=Array(1);this.frameTitles=Array(1);this.closest=Array(1);this.ptTemp=new JU.P3;this.ptTemp1=new JU.P3;this.ptTemp2=
new JU.P3;this.matTemp=new JU.M3;this.matInv=new JU.M3;this.mat4=new JU.M4;this.mat4t=new JU.M4;this.vTemp=new JU.V3;this.setupBC()},"JV.Viewer,~S");c(c$,"releaseModelSet",function(){this.am=null;this.unitCells=this.bsAll=this.bsSymmetry=this.am=this.closest[0]=null;this.releaseModelSetBC()});c(c$,"getEchoStateActive",function(){return this.echoShapeActive});c(c$,"setEchoStateActive",function(a){this.echoShapeActive=a},"~B");c(c$,"getModelSetTypeName",function(){return this.modelSetTypeName});c(c$,
"getModelNumberIndex",function(a,b,d){if(b){for(b=0;b<this.mc;b++)if(this.modelNumbers[b]==a||1E6>a&&this.modelNumbers[b]==1E6+a)return b;return-1}if(1E6>a)return a;for(b=0;b<this.mc;b++)if(this.modelFileNumbers[b]==a)return d&&this.isTrajectory(b)&&this.setTrajectory(b),b;return-1},"~N,~B,~B");c(c$,"getModelDataBaseName",function(a){for(var b=0;b<this.mc;b++)if(a.equals(this.am[b].bsAtoms))return this.getInfo(b,"dbName");return null},"JU.BS");c(c$,"setTrajectory",function(a){0<=a&&(this.isTrajectory(a)&&
this.at[this.am[a].firstAtomIndex].mi!=a)&&this.trajectory.setModel(a)},"~N");c(c$,"getBitSetTrajectories",function(){return null==this.trajectory?null:this.trajectory.getModelsSelected()});c(c$,"setTrajectoryBs",function(a){if(null!=this.trajectory)for(var b=0;b<this.mc;b++)a.get(b)&&this.setTrajectory(b)},"JU.BS");c(c$,"morphTrajectories",function(a,b,d){0<=a&&(0<=b&&this.isTrajectory(a)&&this.isTrajectory(b))&&this.trajectory.morph(a,b,d)},"~N,~N,~N");c(c$,"getTranslation",function(a){return null==
this.translations||a>=this.translations.length?null:this.translations[a]},"~N");c(c$,"translateModel",function(a,b){if(null==b){var d=this.getTranslation(a);null!=d&&(b=JU.P3.newP(d),b.scale(-1),this.translateModel(a,b),this.translations[a]=null)}else{if(null==this.translations||this.translations.length<=a)this.translations=Array(this.mc);null==this.translations[a]&&(this.translations[a]=new JU.P3);this.translations[a].add(b);for(var d=this.am[a].bsAtoms,c=d.nextSetBit(0);0<=c;c=d.nextSetBit(c+1))this.at[c].add(b)}},
"~N,JU.T3");c(c$,"getFrameOffsets",function(a,b){if(null==a){if(b)for(var d=this.mc;0<=--d;){var c=this.am[d];!c.isJmolDataFrame&&!c.isTrajectory&&this.translateModel(c.modelIndex,null)}return null}if(0>a.nextSetBit(0))return null;if(b){for(var f=JU.BSUtil.copy(a),e=null,h=new JU.P3,d=0;d<this.mc;d++){c=this.am[d];if(!c.isJmolDataFrame&&!c.isTrajectory){var k=f.nextSetBit(0);c.bsAtoms.get(k)&&(null==e?e=JU.P3.newP(this.at[k]):(h.sub2(e,this.at[k]),this.translateModel(d,h)))}f.andNot(c.bsAtoms)}return null}f=
Array(this.mc);for(d=this.mc;0<=--d;)f[d]=new JU.P3;var h=e=0,k=f[0],j=null!=this.trajectory&&this.trajectory.steps.size()==this.mc,z=j?this.mc:1;f[0].set(0,0,0);for(c=0;c<z;c++){j&&this.setTrajectory(c);for(d=0;d<=this.ac;d++){if(d==this.ac||this.at[d].mi!=e){0<h&&(k.scale(-1/h),0!=e&&k.sub(f[0]),h=0);if(d==this.ac)break;e=this.at[d].mi;k=f[e]}a.get(d)&&(k.add(this.at[d]),h++)}}return f},"JU.BS,~B");c(c$,"getAtoms",function(a,b){switch(a){default:return JU.BSUtil.andNot(this.getAtomBitsMaybeDeleted(a,
b),this.vwr.slm.bsDeleted);case 1073742358:var d=b.intValue(),c=this.getModelNumberIndex(d,!0,!0);return 0>c&&0<d?new JU.BS:this.vwr.getModelUndeletedAtomsBitSet(c);case 1275203608:return d=u(-1,[null,null,null]),this.vwr.shm.getShapePropertyData(21,"getCenters",d),null==d[1]?new JU.BS:d[1]}},"~N,~O");c(c$,"findNearestAtomIndex",function(a,b,d,c){if(0==this.ac)return-1;this.closest[0]=null;this.g3d.isAntialiased()&&(a<<=1,b<<=1);this.findNearest2(a,b,this.closest,d,c);this.sm.findNearestShapeAtomIndex(a,
b,this.closest,d);a=null==this.closest[0]?-1:this.closest[0].i;this.closest[0]=null;return a},"~N,~N,JU.BS,~N");c(c$,"calculatePointGroup",function(a){return this.calculatePointGroupForFirstModel(a,!1,!1,null,0,0,null,null,null)},"JU.BS");c(c$,"getPointGroupInfo",function(a){return this.calculatePointGroupForFirstModel(a,!1,!0,null,0,0,null,null,null)},"JU.BS");c(c$,"getPointGroupAsString",function(a,b,d,c,f,e,h){return this.calculatePointGroupForFirstModel(a,!0,!1,b,d,c,f,e,h)},"JU.BS,~S,~N,~N,~A,JU.P3,~S");
c(c$,"calculatePointGroupForFirstModel",function(a,b,d,c,f,e,h,k,j){var z=this.pointGroup,m=J.api.Interface.getSymmetry(this.vwr,"ms"),r=null,p=!1,n=!1,q=!1,s=null!=h,t=this.vwr.am.cmi;if(!s)if(h=null==a?-1:a.nextSetBit(0),0>t&&0<=h&&(t=this.at[h].mi),0>t&&(t=this.vwr.getVisibleFramesBitSet().nextSetBit(0),a=null),r=this.vwr.getModelUndeletedAtomsBitSet(t),q=null!=a&&r.cardinality()!=a.cardinality(),null!=a&&r.and(a),h=r.nextSetBit(0),0>h&&(r=this.vwr.getModelUndeletedAtomsBitSet(t),h=r.nextSetBit(0)),
n=this.vwr.shm.getShapePropertyIndex(18,"mad",h),p=null!=n&&0!=n.intValue()||this.vwr.tm.vibrationOn,n=null!=c&&0<=c.toUpperCase().indexOf(":POLY")){z=u(-1,[Integer.$valueOf(h),null]);this.vwr.shm.getShapePropertyData(21,"points",z);h=z[1];if(null==h)return null;r=null;p=!1;z=null}else h=this.at;var v;if(null!=c&&0<=(v=c.indexOf(":")))c=c.substring(0,v);if(null!=c&&0<=(v=c.indexOf(".")))f=JU.PT.parseInt(c.substring(v+1)),0>f&&(f=0),c=c.substring(0,v);z=m.setPointGroup(z,k,h,r,p,s?0:this.vwr.getFloat(570425382),
this.vwr.getFloat(570425384),q);!n&&!s&&(this.pointGroup=z);if(!b&&!d)return z.getPointGroupName();b=z.getPointGroupInfo(t,j,d,c,f,e);return d?b:(1<this.mc?"frame "+this.getModelNumberDotted(t)+"; ":"")+b},"JU.BS,~B,~B,~S,~N,~N,~A,JU.P3,~S");c(c$,"getDefaultStructure",function(a,b){return this.haveBioModels?this.bioModelset.getAllDefaultStructures(a,b):""},"JU.BS,JU.BS");c(c$,"deleteModelBonds",function(a){a=this.getModelAtomBitSetIncludingDeleted(a,!1);this.makeConnections(0,3.4028235E38,131071,
12291,a,a,null,!1,!1,0)},"~N");c(c$,"makeConnections",function(a,b,d,c,f,e,h,k,j,z){if(1073741852==c&&2048!=d){var m="connect ";0.1!=a&&(m+=a+" ");1E8!=b&&(m+=b+" ");this.addStateScript(m,k?f:null,k?null:f,k?null:e," auto",!1,!0)}this.moleculeCount=0;return this.makeConnections2(a,b,d,c,f,e,h,k,j,z)},"~N,~N,~N,~N,JU.BS,JU.BS,JU.BS,~B,~B,~N");c(c$,"setPdbConectBonding",function(a,b,d){for(var c=this.vwr.getMadBond();b<this.mc;b++){var f=this.getInfo(b,"PDB_CONECT_bonds");if(null!=f){var e=f.size();
this.setInfo(b,"initialBondCount",Integer.$valueOf(e));for(var h=this.getInfo(b,"PDB_CONECT_firstAtom_count_max"),k=h[0]+a,j=k+h[1],h=h[2],z=B(h+1,0),m=k;m<j;m++)if(0<(k=this.atomSerials[m]))z[k]=m+1;for(j=0;j<e;j++){var k=f.get(j),m=k[0],r=k[1],k=k[2];if(!(0>m||0>r||m>h||r>h))if(m=z[m]-1,r=z[r]-1,!(0>m||0>r)){var p=this.at[m],n=this.at[r];null!=d&&(p.isHetero()&&d.set(m),n.isHetero()&&d.set(r));if(p.altloc==n.altloc||"\x00"==p.altloc||"\x00"==n.altloc)this.getOrAddBond(p,n,k,2048==k?1:c,null,0,!1)}}}}},
"~N,~N,JU.BS");c(c$,"deleteAllBonds",function(){this.moleculeCount=0;for(var a=this.stateScripts.size();0<=--a;)this.stateScripts.get(a).isConnect()&&this.stateScripts.removeItemAt(a);this.deleteAllBonds2()});c(c$,"includeAllRelatedFrames",function(a){for(var b=0,d=0;d<this.mc;d++){var c=this.isTrajectory(d),f=c&&a.get(b=this.am[d].trajectoryBaseIndex);if(a.get(d)){if(c&&!f){a.set(b);this.includeAllRelatedFrames(a);break}}else(c||this.isJmolDataFrameForModel(d)&&a.get(this.am[d].dataSourceFrame))&&
a.set(d)}},"JU.BS");c(c$,"deleteModels",function(a){this.includeAllRelatedFrames(a);var b=a.cardinality();if(0==b)return null;this.moleculeCount=0;null!=this.msInfo&&this.msInfo.remove("models");for(var d=a.nextSetBit(0);0<=d;d=a.nextSetBit(d+1))this.clearDataFrameReference(d);var c;if(b==this.mc)return c=this.getModelAtomBitSetIncludingDeleted(-1,!0),this.vwr.zap(!0,!1,!1),c;this.validateBspf(!1);c=new JU.BS;for(var f=!0,e=!1,h=new JU.BS,d=0;d<this.mc;d++){var k=this.am[d],f=(new Boolean(f&k.isOrderly)).valueOf();
a.get(d)?(0<=k.fileIndex&&h.set(k.fileIndex),c.or(this.getModelAtomBitSetIncludingDeleted(d,!1))):0<=k.fileIndex&&h.get(k.fileIndex)&&(e=!0)}if(!f||e)return this.vwr.deleteAtoms(c,!1),null;b=Array(this.mc-b);f=this.am;for(e=d=0;d<this.mc;d++)a.get(d)||(k=this.am[d],k.modelIndex=e,b[e++]=k);this.am=b;k=this.mc;h=this.getBondsForSelectedAtoms(c,!0);this.deleteBonds(h,!0);for(e=d=0;d<k;d++)if(a.get(d)){var j=f[d],z=j.act;if(0!=z){var m=j.bsAtoms,j=j.firstAtomIndex;JU.BSUtil.deleteBits(this.bsSymmetry,
m);this.deleteModel(e,m,h);this.deleteModelAtoms(j,z,m);this.vwr.deleteModelAtoms(e,j,z,m);for(var r=k;--r>d;)f[r].fixIndices(e,z,m);this.vwr.shm.deleteShapeAtoms(u(-1,[b,this.at,B(-1,[e,j,z])]),m);this.mc--}}else e++;this.haveBioModels=!1;for(d=this.mc;0<=--d;)this.am[d].isBioModel&&(this.haveBioModels=!0,this.bioModelset.set(this.vwr,this));this.validateBspf(!1);this.bsAll=null;this.resetMolecules();this.isBbcageDefault=!1;this.calcBoundBoxDimensions(null,1);return c},"JU.BS");c(c$,"resetMolecules",
function(){this.molecules=this.bsAll=null;this.moleculeCount=0;this.resetChirality()});c(c$,"resetChirality",function(){if(this.haveChirality)for(var a=-1,b=this.ac;0<=--b;){var d=this.at[b];null!=d&&(d.setCIPChirality(0),d.mi!=a&&d.mi<this.am.length&&(this.am[a=d.mi].hasChirality=!1))}});c(c$,"deleteModel",function(a,b,d){if(!(0>a)){this.modelNumbers=JU.AU.deleteElements(this.modelNumbers,a,1);this.modelFileNumbers=JU.AU.deleteElements(this.modelFileNumbers,a,1);this.modelNumbersForAtomLabel=JU.AU.deleteElements(this.modelNumbersForAtomLabel,
a,1);this.modelNames=JU.AU.deleteElements(this.modelNames,a,1);this.frameTitles=JU.AU.deleteElements(this.frameTitles,a,1);this.thisStateModel=-1;var c=this.getInfoM("group3Lists"),f=this.getInfoM("group3Counts"),e=a+1;if(null!=c&&null!=c[e])for(var h=w(c[e].length/6);0<=--h;)0<f[e][h]&&(f[0][h]-=f[e][h],0==f[0][h]&&(c[0]=c[0].substring(0,6*h)+",["+c[0].substring(6*h+2)));null!=c&&(this.msInfo.put("group3Lists",JU.AU.deleteElements(c,a,1)),this.msInfo.put("group3Counts",JU.AU.deleteElements(f,a,1)));
null!=this.unitCells&&(this.unitCells=JU.AU.deleteElements(this.unitCells,a,1));for(h=this.stateScripts.size();0<=--h;)this.stateScripts.get(h).deleteAtoms(a,d,b)||this.stateScripts.removeItemAt(h)}},"~N,JU.BS,JU.BS");c(c$,"setAtomProperty",function(a,b,d,c,f,e,h){switch(b){case 1114249217:case 1112152066:case 1112152071:case 1112152073:case 1112152074:case 1649022989:case 1112152078:if(4<c&&(c=4),null!=e){d=K(this.ac,0);try{var k=a.nextSetBit(0);for(f=0;0<=k;k=a.nextSetBit(k+1))d[k]=e[f++]}catch(j){if(H(j,
Exception))return;throw j;}e=d}case 1112152070:case 1112152076:k=null;d=0;null==e?(16<c&&(c=16.1),0>c&&(c=0),d=w(Math.floor(2E3*c))):k=new J.atomdata.RadiusData(e,0,null,null);this.sm.setShapeSizeBs(JV.JC.shapeTokenIndex(b),d,k,a);return}this.setAPm(a,b,d,c,f,e,h)},"JU.BS,~N,~N,~N,~S,~A,~A");c(c$,"getFileData",function(a){if(0>a)return"";var b=this.getInfo(a,"fileData");if(null!=b)return b;if(!this.getInfoB(a,"isCIF"))return this.getPDBHeader(a);b=this.vwr.getCifData(a);this.setInfo(a,"fileData",
b);return b},"~N");c(c$,"addHydrogens",function(a,b){var d=this.mc-1,c=new JU.BS;if(this.isTrajectory(d)||1<this.am[d].getGroupCount())return c;this.growAtomArrays(this.ac+b.length);var f=this.vwr.rd,e=this.getDefaultMadFromOrder(1);this.am[d].resetDSSR(!1);for(var h=0,k=this.am[d].act+1;h<a.size();h++,k++){var j=a.get(h),z=this.addAtom(d,j.group,1,"H"+k,null,k,j.getSeqID(),k,b[h],NaN,null,0,0,100,NaN,null,!1,0,null,NaN);z.setMadAtom(this.vwr,f);c.set(z.i);this.bondAtoms(j,z,1,e,null,0,!1,!1)}this.sm.loadDefaultShapes(this);
return c},"JU.Lst,~A");c(c$,"mergeModelArrays",function(a){this.at=a.at;this.bo=a.bo;this.stateScripts=a.stateScripts;this.proteinStructureTainted=a.proteinStructureTainted;this.thisStateModel=-1;this.bsSymmetry=a.bsSymmetry;this.modelFileNumbers=a.modelFileNumbers;this.modelNumbersForAtomLabel=a.modelNumbersForAtomLabel;this.modelNames=a.modelNames;this.modelNumbers=a.modelNumbers;this.frameTitles=a.frameTitles;this.haveChirality=a.haveChirality;null!=this.msInfo&&this.msInfo.remove("models");this.mergeAtomArrays(a)},
"JM.ModelSet");c(c$,"getUnitCell",function(a){return 0>a||a>=this.mc?null:null!=this.am[a].simpleCage?this.am[a].simpleCage:null!=this.unitCells&&a<this.unitCells.length&&null!=this.unitCells[a]&&this.unitCells[a].haveUnitCell()?this.unitCells[a]:null!=this.getInfo(a,"unitCellParams")?(null==this.unitCells&&(this.unitCells=Array(this.mc)),this.haveUnitCells=!0,this.unitCells[a]=this.vwr.getSymTemp().setSymmetryInfo(a,this.am[a].auxiliaryInfo,null)):null},"~N");c(c$,"setModelCage",function(a,b){0<=
a&&a<this.mc&&(this.am[a].simpleCage=b,this.haveUnitCells=!0)},"~N,J.api.SymmetryInterface");c(c$,"getModelName",function(a){return 1>this.mc?"":0<=a?this.modelNames[a]:this.modelNumbersForAtomLabel[-1-a]},"~N");c(c$,"getModelTitle",function(a){return this.getInfo(a,"title")},"~N");c(c$,"getModelFileName",function(a){return this.getInfo(a,"fileName")},"~N");c(c$,"getModelFileType",function(a){return this.getInfo(a,"fileType")},"~N");c(c$,"setFrameTitle",function(a,b){if(n(b,String))for(var d=a.nextSetBit(0);0<=
d;d=a.nextSetBit(d+1))this.frameTitles[d]=b;else for(var d=a.nextSetBit(0),c=0;0<=d;d=a.nextSetBit(d+1))c<b.length&&(this.frameTitles[d]=b[c++])},"JU.BS,~O");c(c$,"getFrameTitle",function(a){return 0<=a&&a<this.mc?this.frameTitles[a]:""},"~N");c(c$,"getModelNumberForAtomLabel",function(a){return this.modelNumbersForAtomLabel[a]},"~N");c(c$,"getGroups",function(){for(var a=0,b=0;b<this.mc;b++)a+=this.am[b].getGroupCount();for(var a=Array(a),d=b=0;b<this.mc;b++)for(var c=0;c<this.am[b].chainCount;c++)for(var f=
0;f<this.am[b].chains[c].groupCount;f++)a[d]=this.am[b].chains[c].groups[f],a[d].groupIndex=d,d++;return a});c(c$,"getUnitCellParams",function(){var a=this.getUnitCell(0);return null==a?null:a.getUnitCellParams()});c(c$,"setCrystallographicDefaults",function(){return!this.haveBioModels&&this.someModelsHaveSymmetry&&this.someModelsHaveFractionalCoordinates});c(c$,"getBoundBoxCenter",function(a){return this.isJmolDataFrameForModel(a)?new JU.P3:(null==this.getDefaultBoundBox()?this.boxInfo:this.defaultBBox).getBoundBoxCenter()},
"~N");c(c$,"getBoundBoxCornerVector",function(){return this.boxInfo.getBoundBoxCornerVector()});c(c$,"getBBoxVertices",function(){return this.boxInfo.getBoundBoxVertices()});c(c$,"setBoundBox",function(a,b,d,c){this.isBbcageDefault=!1;this.bboxAtoms=this.bboxModels=null;this.boxInfo.setBoundBox(a,b,d,c)},"JU.T3,JU.T3,~B,~N");c(c$,"getBoundBoxCommand",function(a){if(!a&&null!=this.bboxAtoms)return"boundbox "+JU.Escape.eBS(this.bboxAtoms);this.ptTemp.setT(this.boxInfo.getBoundBoxCenter());var b=this.boxInfo.getBoundBoxCornerVector();
a=a?"boundbox "+JU.Escape.eP(this.ptTemp)+" "+JU.Escape.eP(b)+"\n#or\n":"";this.ptTemp.sub(b);a+="boundbox corners "+JU.Escape.eP(this.ptTemp)+" ";this.ptTemp.scaleAdd2(2,b,this.ptTemp);b=Math.abs(8*b.x*b.y*b.z);return a+=JU.Escape.eP(this.ptTemp)+" # volume = "+b},"~B");c(c$,"findAtomsInRectangle",function(a){for(var b=this.vwr.getVisibleFramesBitSet(),d=new JU.BS,c=this.ac;0<=--c;){var f=this.at[c];null!=f&&(b.get(f.mi)?f.checkVisible()&&a.contains(f.sX,f.sY)&&d.set(c):c=this.am[f.mi].firstAtomIndex)}return d},
"JU.Rectangle");c(c$,"getDefaultVdwType",function(a){return!this.am[a].isBioModel?J.c.VDW.AUTO_BABEL:0==this.am[a].hydrogenCount?J.c.VDW.AUTO_JMOL:J.c.VDW.AUTO_BABEL},"~N");c(c$,"setRotationRadius",function(a,b){return this.isJmolDataFrameForModel(a)?(this.am[a].defaultRotationRadius=b,!1):!0},"~N,~N");c(c$,"calcRotationRadius",function(a,b,d){if(this.isJmolDataFrameForModel(a))return b=this.am[a].defaultRotationRadius,0==b?10:b;if(d&&null!=this.getDefaultBoundBox())return 1.2*(this.defaultBBox.getMaxDim()/
2);d=0;for(var c=this.ac;0<=--c;)if(a=this.at[c],null!=a)if(this.isJmolDataFrameForAtom(a))for(a=a.mi;0<=c&&null!=this.at[c]&&this.at[c].mi==a;)c--;else a=this.at[c],null!=a&&(a=b.distance(a)+this.getRadiusVdwJmol(a),a>d&&(d=a));return 0==d?10:d},"~N,JU.P3,~B");c(c$,"calcBoundBoxDimensions",function(a,b){null!=a&&0>a.nextSetBit(0)&&(a=null);if(!(null==a&&this.isBbcageDefault||0==this.ac)){if(null==this.getDefaultBoundBox())this.bboxModels=this.getModelBS(this.bboxAtoms=JU.BSUtil.copy(a),!1),this.calcAtomsMinMax(a,
this.boxInfo)==this.ac&&(this.isBbcageDefault=!0),null==a&&null!=this.unitCells&&this.calcUnitCellMinMax();else{var d=this.defaultBBox.getBoundBoxVertices();this.boxInfo.reset();for(var c=0;8>c;c++)this.boxInfo.addBoundBoxPoint(d[c])}this.boxInfo.setBbcage(b)}},"JU.BS,~N");c(c$,"getDefaultBoundBox",function(){var a=this.getInfoM("boundbox");null==a?this.defaultBBox=null:(null==this.defaultBBox&&(this.defaultBBox=new JU.BoxInfo),this.defaultBBox.setBoundBoxFromOABC(a));return this.defaultBBox});c(c$,
"getBoxInfo",function(a,b){if(null==a)return this.boxInfo;var d=new JU.BoxInfo;this.calcAtomsMinMax(a,d);d.setBbcage(b);return d},"JU.BS,~N");c(c$,"calcAtomsMinMax",function(a,b){b.reset();for(var d=0,c=null==a,f=c?this.ac-1:a.nextSetBit(0);0<=f;f=c?f-1:a.nextSetBit(f+1)){d++;var e=this.at[f];null!=e&&!this.isJmolDataFrameForAtom(e)&&b.addBoundBoxPoint(e)}return d},"JU.BS,JU.BoxInfo");c(c$,"calcUnitCellMinMax",function(){for(var a=new JU.P3,b=0;b<this.mc;b++)if(this.unitCells[b].getCoordinatesAreFractional())for(var d=
this.unitCells[b].getUnitCellVerticesNoOffset(),c=this.unitCells[b].getCartesianOffset(),f=0;8>f;f++)a.add2(c,d[f]),this.boxInfo.addBoundBoxPoint(a)});c(c$,"calcRotationRadiusBs",function(a){for(var b=this.getAtomSetCenter(a),d=0,c=a.nextSetBit(0);0<=c;c=a.nextSetBit(c+1)){var f=this.at[c],f=b.distance(f)+this.getRadiusVdwJmol(f);f>d&&(d=f)}return 0==d?10:d},"JU.BS");c(c$,"getCenterAndPoints",function(a,b){for(var d,c,f=b?1:0,e=a.size();0<=--e;){var h=a.get(e);d=h[0];n(h[1],JU.BS)?(c=h[1],f+=Math.min(d.cardinality(),
c.cardinality())):f+=Math.min(d.cardinality(),h[1].length)}var k=u(2,f,null);b&&(k[0][0]=new JU.P3,k[1][0]=new JU.P3);for(e=a.size();0<=--e;)if(h=a.get(e),d=h[0],n(h[1],JU.BS)){c=h[1];for(var h=d.nextSetBit(0),j=c.nextSetBit(0);0<=h&&0<=j;h=d.nextSetBit(h+1),j=c.nextSetBit(j+1))k[0][--f]=this.at[h],k[1][f]=this.at[j],b&&(k[0][0].add(this.at[h]),k[1][0].add(this.at[j]))}else{c=h[1];h=d.nextSetBit(0);for(j=0;0<=h&&j<c.length;h=d.nextSetBit(h+1),j++)k[0][--f]=this.at[h],k[1][f]=c[j],b&&(k[0][0].add(this.at[h]),
k[1][0].add(c[j]))}b&&(k[0][0].scale(1/(k[0].length-1)),k[1][0].scale(1/(k[1].length-1)));return k},"JU.Lst,~B");c(c$,"getAtomSetCenter",function(a){for(var b=new JU.P3,d=0,c=a.nextSetBit(0);0<=c;c=a.nextSetBit(c+1))this.isJmolDataFrameForAtom(this.at[c])||(d++,b.add(this.at[c]));1<d&&b.scale(1/d);return b},"JU.BS");c(c$,"getAverageAtomPoint",function(){return this.getAtomSetCenter(this.vwr.bsA())});c(c$,"setAPm",function(a,b,d,c,f,e,h){this.setAPa(a,b,d,c,f,e,h);switch(b){case 1094715417:case 1631586315:this.vwr.getBoolean(603979944)&&
this.assignAromaticBondsBs(!0,null)}},"JU.BS,~N,~N,~N,~S,~A,~A");c(c$,"addStateScript",function(a,b,d,c,f,e,h){var k=this.vwr.am.cmi;e?(this.thisStateModel!=k&&(a="frame "+(0>k?"all #"+k:this.getModelNumberDotted(k))+";\n  "+a),this.thisStateModel=k):this.thisStateModel=-1;a=new JM.StateScript(this.thisStateModel,a,b,d,c,f,h);a.isValid()&&this.stateScripts.addLast(a);return a},"~S,JU.BS,JU.BS,JU.BS,~S,~B,~B");c(c$,"freezeModels",function(){this.haveBioModels=!1;for(var a=this.mc;0<=--a;)this.haveBioModels=
(new Boolean(this.haveBioModels|this.am[a].freeze())).valueOf()});c(c$,"getStructureList",function(){return this.vwr.getStructureList()});c(c$,"getInfoM",function(a){return null==this.msInfo?null:this.msInfo.get(a)},"~S");c(c$,"getMSInfoB",function(a){a=this.getInfoM(a);return n(a,Boolean)&&a.booleanValue()},"~S");c(c$,"isTrajectory",function(a){return this.am[a].isTrajectory},"~N");c(c$,"isTrajectorySubFrame",function(a){return this.am[a].trajectoryBaseIndex!=a},"~N");c(c$,"isTrajectoryMeasurement",
function(a){return null!=this.trajectory&&this.trajectory.hasMeasure(a)},"~A");c(c$,"getModelBS",function(a,b){var d=new JU.BS,c=0,f=null==a;b=(new Boolean(b&null!=this.trajectory)).valueOf();for(var e=f?0:a.nextSetBit(0);0<=e&&e<this.ac;e=f?e+1:a.nextSetBit(e+1))null!=this.at[e]&&(d.set(c=this.at[e].mi),b&&this.trajectory.getModelBS(c,d),c=this.am[c],c.isOrderly&&(e=c.firstAtomIndex+c.act-1));return d},"JU.BS,~B");c(c$,"getIterativeModels",function(a){for(var b=new JU.BS,d=0;d<this.mc;d++)if(a||
!this.isJmolDataFrameForModel(d))this.isTrajectorySubFrame(d)||b.set(d);return b},"~B");c(c$,"fillAtomData",function(a,b){if(0!=(b&4)){this.getMolecules();a.bsMolecules=Array(this.molecules.length);a.atomMolecule=B(this.ac,0);for(var d,c=0;c<this.molecules.length;c++){d=a.bsMolecules[c]=this.molecules[c].atomList;for(var f=d.nextSetBit(0);0<=f;f=d.nextSetBit(f+1))a.atomMolecule[f]=c}}0!=(b&8)?(d=B(1,0),a.hAtomRadius=this.vwr.getVanderwaalsMar(1)/1E3,a.hAtoms=this.calculateHydrogens(a.bsSelected,d,
null,512),a.hydrogenAtomCount=d[0]):(a.firstAtomIndex=0>a.modelIndex?null==a.bsSelected?0:Math.max(0,a.bsSelected.nextSetBit(0)):this.am[a.modelIndex].firstAtomIndex,a.lastModelIndex=a.firstModelIndex=0==this.ac?0:this.at[a.firstAtomIndex].mi,a.modelName=this.getModelNumberDotted(a.firstModelIndex),this.fillADa(a,b))},"J.atomdata.AtomData,~N");c(c$,"getModelNumberDotted",function(a){return 1>this.mc||a>=this.mc||0>a?"":JU.Escape.escapeModelFileNumber(this.modelFileNumbers[a])},"~N");c(c$,"getModelNumber",
function(a){return this.modelNumbers[2147483647==a?this.mc-1:a]},"~N");c(c$,"getModelProperty",function(a,b){var d=this.am[a].properties;return null==d?null:d.getProperty(b)},"~N,~S");c(c$,"getModelAuxiliaryInfo",function(a){return 0>a?null:this.am[a].auxiliaryInfo},"~N");c(c$,"setInfo",function(a,b,d){0<=a&&a<this.mc&&this.am[a].auxiliaryInfo.put(b,d)},"~N,~O,~O");c(c$,"getInfo",function(a,b){return 0>a?null:this.am[a].auxiliaryInfo.get(b)},"~N,~S");c(c$,"getInfoB",function(a,b){var d=this.am[a].auxiliaryInfo;
return null!=d&&d.containsKey(b)&&d.get(b).booleanValue()},"~N,~S");c(c$,"getInfoI",function(a,b){var d=this.am[a].auxiliaryInfo;return null!=d&&d.containsKey(b)?d.get(b).intValue():-2147483648},"~N,~S");c(c$,"getInsertionCountInModel",function(a){return this.am[a].insertionCount},"~N");c$.modelFileNumberFromFloat=c(c$,"modelFileNumberFromFloat",function(a){var b=w(Math.floor(a));for(a=w(Math.floor(1E4*(a-b+1E-5)));0!=a&&0==a%10;)a/=10;return 1E6*b+a},"~N");c(c$,"getChainCountInModelWater",function(a,
b){if(0>a){for(var d=0,c=this.mc;0<=--c;)d+=this.am[c].getChainCount(b);return d}return this.am[a].getChainCount(b)},"~N,~B");c(c$,"getGroupCountInModel",function(a){if(0>a){a=0;for(var b=this.mc;0<=--b;)a+=this.am[b].getGroupCount();return a}return this.am[a].getGroupCount()},"~N");c(c$,"calcSelectedGroupsCount",function(){for(var a=this.vwr.bsA(),b=this.mc;0<=--b;)this.am[b].calcSelectedGroupsCount(a)});c(c$,"isJmolDataFrameForModel",function(a){return 0<=a&&a<this.mc&&this.am[a].isJmolDataFrame},
"~N");c(c$,"isJmolDataFrameForAtom",function(a){return this.am[a.mi].isJmolDataFrame},"JM.Atom");c(c$,"setJmolDataFrame",function(a,b,d){var c=this.am[null==a?this.am[d].dataSourceFrame:b];null==a&&(a=this.am[d].jmolFrameType);0<=b&&(null==c.dataFrames&&(c.dataFrames=new java.util.Hashtable),this.am[d].dataSourceFrame=b,this.am[d].jmolFrameType=a,c.dataFrames.put(a,Integer.$valueOf(d)));a.startsWith("quaternion")&&0>a.indexOf("deriv")&&(a=a.substring(0,a.indexOf(" ")),c.dataFrames.put(a,Integer.$valueOf(d)))},
"~S,~N,~N");c(c$,"getJmolDataFrameIndex",function(a,b){if(null==this.am[a].dataFrames)return-1;var d=this.am[a].dataFrames.get(b);return null==d?-1:d.intValue()},"~N,~S");c(c$,"clearDataFrameReference",function(a){for(var b=0;b<this.mc;b++){var d=this.am[b].dataFrames;if(null!=d)for(d=d.values().iterator();d.hasNext();)d.next().intValue()==a&&d.remove()}},"~N");c(c$,"getJmolFrameType",function(a){return 0<=a&&a<this.mc?this.am[a].jmolFrameType:"modelSet"},"~N");c(c$,"getJmolDataSourceFrame",function(a){return 0<=
a&&a<this.mc?this.am[a].dataSourceFrame:-1},"~N");c(c$,"saveModelOrientation",function(a,b){this.am[a].orientation=b},"~N,JM.Orientation");c(c$,"getModelOrientation",function(a){return this.am[a].orientation},"~N");c(c$,"getPDBHeader",function(a){return this.am[a].isBioModel?this.am[a].getFullPDBHeader():this.getFileHeader(a)},"~N");c(c$,"getFileHeader",function(a){if(0>a)return"";if(this.am[a].isBioModel)return this.getPDBHeader(a);a=this.getInfo(a,"fileHeader");null==a&&(a=this.modelSetName);return null!=
a?a:"no header information found"},"~N");c(c$,"getAltLocCountInModel",function(a){return this.am[a].altLocCount},"~N");c(c$,"getAltLocIndexInModel",function(a,b){if("\x00"==b)return 0;var d=this.getAltLocListInModel(a);return 0==d.length?0:d.indexOf(b)+1},"~N,~S");c(c$,"getInsertionCodeIndexInModel",function(a,b){if("\x00"==b)return 0;var d=this.getInsertionListInModel(a);return 0==d.length?0:d.indexOf(b)+1},"~N,~S");c(c$,"getAltLocListInModel",function(a){a=this.getInfo(a,"altLocs");return null==
a?"":a},"~N");c(c$,"getInsertionListInModel",function(a){a=this.getInfo(a,"insertionCodes");return null==a?"":a},"~N");c(c$,"getModelSymmetryCount",function(a){return 0<this.am[a].biosymmetryCount?this.am[a].biosymmetryCount:null==this.unitCells||null==this.unitCells[a]?0:this.unitCells[a].getSpaceGroupOperationCount()},"~N");c(c$,"getModelCellRange",function(a){return null==this.unitCells?null:this.unitCells[a].getCellRange()},"~N");c(c$,"getLastVibrationVector",function(a,b){if(null!=this.vibrations&&
a<this.vwr.ms.mc)for(var d,c=0>a||this.isTrajectory(a)||a>=this.mc-1?this.ac:this.am[a+1].firstAtomIndex,f=0>=a?0:this.am[a].firstAtomIndex;--c>=f;)if((0>a||this.at[c].mi==a)&&((1275072532==b||0==b)&&null!=(d=this.getModulation(c))||(4166==b||0==b)&&null!=(d=this.getVibration(c,!1)))&&d.isNonzero())return c;return-1},"~N,~N");c(c$,"getModulationList",function(a,b,d){var c=new JU.Lst;if(null!=this.vibrations)for(var f=a.nextSetBit(0);0<=f;f=a.nextSetBit(f+1))n(this.vibrations[f],J.api.JmolModulationSet)?
c.addLast(this.vibrations[f].getModulation(b,d)):c.addLast(Float.$valueOf("O"==b?NaN:-1));return c},"JU.BS,~S,JU.P3");c(c$,"getElementsPresentBitSet",function(a){if(0<=a)return this.elementsPresent[a];a=new JU.BS;for(var b=0;b<this.mc;b++)a.or(this.elementsPresent[b]);return a},"~N");c(c$,"getMoleculeIndex",function(a,b){0==this.moleculeCount&&this.getMolecules();for(var d=0;d<this.moleculeCount;d++)if(this.molecules[d].atomList.get(a))return b?this.molecules[d].indexInModel:d;return 0},"~N,~B");
c(c$,"getMoleculeBitSet",function(a){0==this.moleculeCount&&this.getMolecules();var b=JU.BSUtil.copy(a);a=JU.BSUtil.copy(a);for(var d=0,c=new JU.BS;0<=(d=a.length()-1);)c=this.getMoleculeBitSetForAtom(d),null==c?(a.clear(d),b.clear(d)):(a.andNot(c),b.or(c));return b},"JU.BS");c(c$,"getMoleculeBitSetForAtom",function(a){0==this.moleculeCount&&this.getMolecules();for(var b=0;b<this.moleculeCount;b++)if(this.molecules[b].atomList.get(a))return this.molecules[b].atomList;return null},"~N");c(c$,"getModelDipole",
function(a){if(0>a)return null;var b=this.getInfo(a,"dipole");null==b&&(b=this.getInfo(a,"DIPOLE_VEC"));return b},"~N");c(c$,"calculateMolecularDipole",function(a,b){if(null!=b){var d=b.nextSetBit(0);if(0>d)return null;a=this.at[d].mi}if(0>a)return null;var c=d=0,f=0,e=0,h=new JU.V3,k=new JU.V3;null==b&&(b=this.getModelAtomBitSetIncludingDeleted(-1,!1));this.vwr.getOrCalcPartialCharges(this.am[a].bsAtoms,null);for(var j=b.nextSetBit(0);0<=j;j=b.nextSetBit(j+1))if(!(this.at[j].mi!=a||this.at[j].isDeleted())){var z=
this.partialCharges[j];0>z?(c++,e+=z,k.scaleAdd2(z,this.at[j],k)):0<z&&(d++,f+=z,h.scaleAdd2(z,this.at[j],h))}if(0.015<Math.abs(f+e))return JU.Logger.info("Dipole calculation requires balanced charges: "+f+" "+e),null;if(0==c||0==d)return null;h.add(k);h.scale(4.8);return h},"~N,JU.BS");c(c$,"getMoleculeCountInModel",function(a){var b=0;0==this.moleculeCount&&this.getMolecules();if(0>a)return this.moleculeCount;for(var d=0;d<this.mc;d++)a==d&&(b+=this.am[d].moleculeCount);return b},"~N");c(c$,"calcSelectedMoleculesCount",
function(){var a=this.vwr.bsA();0==this.moleculeCount&&this.getMolecules();this.selectedMolecules.xor(this.selectedMolecules);for(var b=new JU.BS,d=0;d<this.moleculeCount;d++)JU.BSUtil.copy2(a,b),b.and(this.molecules[d].atomList),0<b.length()&&this.selectedMolecules.set(d)});c(c$,"setCentroid",function(a,b){var d=this.getNotInCentroid(a,b);null!=d&&0<=d.nextSetBit(0)&&this.vwr.deleteAtoms(d,!1)},"JU.BS,~A");c(c$,"getNotInCentroid",function(a,b){var d=a.nextSetBit(0);if(0>d)return null;d=this.getUnitCell(this.at[d].mi);
return null==d?null:d.notInCentroid(this,a,b)},"JU.BS,~A");c(c$,"getMolecules",function(){if(0<this.moleculeCount)return this.molecules;null==this.molecules&&(this.molecules=Array(4));this.moleculeCount=0;for(var a=null,b=Array(this.mc),d=null,c=0;c<this.mc;c++)b[c]=this.vwr.getModelUndeletedAtomsBitSet(c),a=this.am[c],a.moleculeCount=0,d=a.isBioModel?a.getBioBranches(d):null;this.molecules=JU.JmolMolecule.getMolecules(this.at,b,d,null);for(c=this.moleculeCount=this.molecules.length;0<=--c;)a=this.am[this.molecules[c].modelIndex],
a.firstMoleculeIndex=c,a.moleculeCount++;return this.molecules});c(c$,"initializeBspf",function(){if(!(null!=this.bspf&&this.bspf.isValid)){this.showRebondTimes&&JU.Logger.startTimer("build bspf");var a=new J.bspt.Bspf(3);JU.Logger.debugging&&JU.Logger.debug("sequential bspt order");for(var b=JU.BS.newN(this.mc),d=this.ac;0<=--d;){var c=this.at[d];null!=c&&(!c.isDeleted()&&!this.isTrajectorySubFrame(c.mi))&&(a.addTuple(this.am[c.mi].trajectoryBaseIndex,c),b.set(c.mi))}this.showRebondTimes&&(JU.Logger.checkTimer("build bspf",
!1),a.stats());for(d=b.nextSetBit(0);0<=d;d=b.nextSetBit(d+1))a.validateModel(d,!0);a.isValid=!0;this.bspf=a}});c(c$,"initializeBspt",function(a){this.initializeBspf();this.bspf.isInitializedIndex(a)||this.bspf.initialize(a,this.at,this.vwr.getModelUndeletedAtomsBitSet(a))},"~N");c(c$,"setIteratorForPoint",function(a,b,d,c){0>b?a.setCenter(d,c):(this.initializeBspt(b),a.setModel(this,b,this.am[b].firstAtomIndex,2147483647,d,c,null))},"J.api.AtomIndexIterator,~N,JU.T3,~N");c(c$,"setIteratorForAtom",
function(a,b,d,c,f){0>b&&(b=this.at[d].mi);b=this.am[b].trajectoryBaseIndex;this.initializeBspt(b);a.setModel(this,b,this.am[b].firstAtomIndex,d,this.at[d],c,f)},"J.api.AtomIndexIterator,~N,~N,~N,J.atomdata.RadiusData");c(c$,"getSelectedAtomIterator",function(a,b,d,c,f){this.initializeBspf();if(f){f=this.getModelBS(a,!1);for(var e=f.nextSetBit(0);0<=e;e=f.nextSetBit(e+1))this.initializeBspt(e);f=new JM.AtomIteratorWithinModelSet(f)}else f=new JM.AtomIteratorWithinModel;f.initialize(this.bspf,a,b,
d,c,this.vwr.isParallel());return f},"JU.BS,~B,~B,~B,~B");j(c$,"getBondCountInModel",function(a){return 0>a?this.bondCount:this.am[a].getBondCount()},"~N");c(c$,"getAtomCountInModel",function(a){return 0>a?this.ac:this.am[a].act},"~N");c(c$,"getModelAtomBitSetIncludingDeletedBs",function(a){var b=new JU.BS;null==a&&null==this.bsAll&&(this.bsAll=JU.BSUtil.setAll(this.ac));if(null==a)b.or(this.bsAll);else for(var d=a.nextSetBit(0);0<=d;d=a.nextSetBit(d+1))b.or(this.getModelAtomBitSetIncludingDeleted(d,
!1));return b},"JU.BS");c(c$,"getModelAtomBitSetIncludingDeleted",function(a,b){var d=0>a?this.bsAll:this.am[a].bsAtoms;null==d&&(d=this.bsAll=JU.BSUtil.setAll(this.ac));return b?JU.BSUtil.copy(d):d},"~N,~B");c(c$,"getAtomBitsMaybeDeleted",function(a,b){var d;switch(a){default:return this.getAtomBitsMDa(a,b,new JU.BS);case 1073741925:case 1073742189:case 1111490587:case 1073742128:case 1073741863:case 1086324744:return d=new JU.BS,this.haveBioModels?this.bioModelset.getAtomBitsStr(a,b,d):d;case 1677721602:case 1073742331:return this.getAtomBitsMDb(a,
b);case 1678381065:var c=this.getBoxInfo(b,1);d=this.getAtomsWithin(c.getBoundBoxCornerVector().length()+1E-4,c.getBoundBoxCenter(),null,-1);for(var f=d.nextSetBit(0);0<=f;f=d.nextSetBit(f+1))c.isWithin(this.at[f])||d.clear(f);return d;case 1094713349:d=new JU.BS;for(f=this.ac;0<=--f;)null!=this.at[f]&&this.isInLatticeCell(f,b,this.ptTemp2,!1)&&d.set(f);return d;case 1094713350:d=JU.BSUtil.newBitSet2(0,this.ac);c=B(-1,[A(b.x)-1,A(b.y)-1,A(b.z)-1,A(b.x),A(b.y),A(b.z),0]);for(f=this.mc;0<=--f;){var e=
this.getUnitCell(f);null==e?JU.BSUtil.andNot(d,this.am[f].bsAtoms):d.andNot(e.notInCentroid(this,this.am[f].bsAtoms,c))}return d;case 1094713360:return this.getMoleculeBitSet(b);case 1073742363:return this.getSelectCodeRange(b);case 2097196:d=JU.BS.newN(this.ac);c=-1;e=0;for(f=this.ac;0<=--f;){var h=this.at[f];if(null!=h){var j=h.atomSymmetry;if(null!=j){if(h.mi!=c){c=h.mi;if(null==this.getModelCellRange(c))continue;e=this.getModelSymmetryCount(c)}for(var h=0,l=e;0<=--l;)if(j.get(l)&&1<++h){d.set(f);
break}}}}return d;case 1088421903:return JU.BSUtil.copy(null==this.bsSymmetry?this.bsSymmetry=JU.BS.newN(this.ac):this.bsSymmetry);case 1814695966:d=new JU.BS;if(null==this.vwr.getCurrentUnitCell())return d;this.ptTemp1.set(1,1,1);for(f=this.ac;0<=--f;)null!=this.at[f]&&this.isInLatticeCell(f,this.ptTemp1,this.ptTemp2,!1)&&d.set(f);return d}},"~N,~O");c(c$,"getSelectCodeRange",function(a){var b=new JU.BS,d=a[0],c=a[1];a=a[2];var f=this.vwr.getBoolean(603979822);0<=a&&(300>a&&!f)&&(a=this.chainToUpper(a));
for(var e=this.mc;0<=--e;)if(this.am[e].isBioModel)for(var h=this.am[e],j,l=h.chainCount;0<=--l;){var z=h.chains[l];if(-1==a||a==(j=z.chainID)||!f&&0<j&&300>j&&a==this.chainToUpper(j))for(var m=z.groups,z=z.groupCount,r=0;0<=r;)r=JM.ModelSet.selectSeqcodeRange(m,z,r,d,c,b)}return b},"~A");c$.selectSeqcodeRange=c(c$,"selectSeqcodeRange",function(a,b,d,c,f,e){var h,j,l,z=!1;for(j=d;j<b&&a[j].seqcode!=c;j++);if(j==b){if(0<d)return-1;z=!0;l=2147483647;for(var m=b;0<=--m;)if((h=a[m].seqcode)>c&&h-c<l)j=
m,l=h-c;if(2147483647==l)return-1}if(2147483647==f)c=b-1,z=!0;else{for(c=j;c<b&&a[c].seqcode!=f;c++);if(c==b){if(0<d)return-1;z=!0;l=2147483647;for(m=j;m<b;m++)if((h=a[m].seqcode)<f&&f-h<l)c=m,l=f-h;if(2147483647==l)return-1}}for(m=j;m<=c;++m)a[m].setAtomBits(e);return z?-1:c+1},"~A,~N,~N,~N,~N,JU.BS");c(c$,"isInLatticeCell",function(a,b,d,c){var f=this.getUnitCell(this.at[a].mi);d.setT(this.at[a]);return null!=f&&f.checkUnitCell(f,b,d,c)},"~N,JU.P3,JU.P3,~B");c(c$,"getAtomsWithinRadius",function(a,
b,d,c){var f=new JU.BS,e=this.getIterativeModels(!1);b=JU.BSUtil.andNot(b,this.vwr.slm.bsDeleted);var h=this.getSelectedAtomIterator(null,!1,!1,!1,!1);if(d){d=!this.vwr.g.legacyJavaFloat;for(var j=new JU.P3,l=b.nextSetBit(0);0<=l;l=b.nextSetBit(l+1))for(var z=this.mc;0<=--z;)e.get(z)&&(0>a?this.getAtomsWithin(a,this.at[l].getFractionalUnitCoordPt(d,!0,j),f,-1):(this.setIteratorForAtom(h,z,l,a,c),h.addAtoms(f)))}else{f.or(b);for(l=b.nextSetBit(0);0<=l;l=b.nextSetBit(l+1))0>a?this.getAtomsWithin(a,
this.at[l],f,this.at[l].mi):(this.setIteratorForAtom(h,-1,l,a,c),h.addAtoms(f))}h.release();return f},"~N,JU.BS,~B,J.atomdata.RadiusData");c(c$,"getAtomsWithin",function(a,b,d,c){null==d&&(d=new JU.BS);if(0>a){a=-a;for(var f=this.ac;0<=--f;){var e=this.at[f];null==e||0<=c&&e.mi!=c||!d.get(f)&&e.getFractionalUnitDistance(b,this.ptTemp1,this.ptTemp2)<=a&&d.set(e.i)}return d}c=this.getIterativeModels(!0);f=this.getSelectedAtomIterator(null,!1,!1,!1,!1);for(e=this.mc;0<=--e;)c.get(e)&&!this.am[e].bsAtoms.isEmpty()&&
(this.setIteratorForAtom(f,-1,this.am[e].firstAtomIndex,-1,null),f.setCenter(b,a),f.addAtoms(d));f.release();return d},"~N,JU.T3,JU.BS,~N");c(c$,"deleteBonds",function(a,b){if(!b)for(var d=new JU.BS,c=new JU.BS,f=a.nextSetBit(0);0<=f;f=a.nextSetBit(f+1)){var e=this.bo[f].atom1;this.am[e.mi].isModelKit||(d.clearAll(),c.clearAll(),d.set(e.i),c.set(this.bo[f].getAtomIndex2()),this.addStateScript("connect ",null,d,c,"delete",!1,!0))}this.dBb(a,b)},"JU.BS,~B");c(c$,"makeConnections2",function(a,b,d,c,
f,e,h,j,l,z){null==h&&(h=new JU.BS);var m=65535==d,r=131071==d,p=65537==d;r&&(d=1);var n=JU.Edge.isOrderH(d),q=!1,s=!1,u=!1,t=!1;switch(c){case 12291:return this.deleteConnections(a,b,d,f,e,j,r);case 603979872:case 1073741852:if(515!=d){if(j){a=f;f=new JU.BS;e=new JU.BS;for(var v=a.nextSetBit(0);0<=v;v=a.nextSetBit(v+1))f.set(this.bo[v].atom1.i),e.set(this.bo[v].atom2.i)}return B(-1,[n?this.autoHbond(f,e,!1):this.autoBondBs4(f,e,null,h,this.vwr.getMadBond(),603979872==c),0])}s=t=!0;break;case 1086324745:q=
s=!0;break;case 1073742025:s=!0;break;case 1073741904:u=!0}c=!q||m;m=!q&&!m;this.defaultCovalentMad=this.vwr.getMadBond();var r=0>a,w=0>b,A=r||w,D=!j||0.1!=a||1E8!=b;D&&(a=this.fixD(a,r),b=this.fixD(b,w));var G=this.getDefaultMadFromOrder(d),E=0,K=0,I=null,L=null,M=null,N="\x00",O=d|131072,Q=0!=(d&512);try{for(v=f.nextSetBit(0);0<=v;v=f.nextSetBit(v+1)){if(j)I=this.bo[v],L=I.atom1,M=I.atom2;else{L=this.at[v];if(L.isDeleted())continue;N=this.isModulated(v)?"\x00":L.altloc}for(var P=j?0:e.nextSetBit(0);0<=
P;P=e.nextSetBit(P+1)){if(j)P=2147483646;else{if(P==v)continue;M=this.at[P];if(null==M||L.mi!=M.mi||M.isDeleted())continue;if("\x00"!=N&&N!=M.altloc&&"\x00"!=M.altloc)continue;I=L.getBond(M)}if(!(null==I?s:u)&&!(D&&!this.isInRange(L,M,a,b,r,w,A)||Q&&!this.allowAromaticBond(I)))if(null==I)h.set(this.bondAtoms(L,M,d,G,h,z,l,!0).index),E++;else if(m&&(I.setOrder(d),p&&(this.haveAtropicBonds=!0,I.setAtropisomerOptions(f,e)),this.bsAromatic.clear(I.index)),c||d==I.order||O==I.order||n&&I.isHydrogen())h.set(I.index),
K++}}}catch(R){if(!H(R,Exception))throw R;}t&&this.assignAromaticBondsBs(!0,h);q||this.sm.setShapeSizeBs(1,-2147483648,null,h);return B(-1,[E,K])},"~N,~N,~N,~N,JU.BS,JU.BS,JU.BS,~B,~B,~N");c(c$,"autoBondBs4",function(a,b,d,c,f,e){if(e)return this.autoBond_Pre_11_9_24(a,b,d,c,f);if(0==this.ac)return 0;0==f&&(f=1);1.4E-45==this.maxBondingRadius&&this.findMaxRadii();e=this.vwr.getFloat(570425348);var h=this.vwr.getFloat(570425364),h=h*h,j=0;this.showRebondTimes&&JU.Logger.startTimer("autobond");var l=
-1,z=null==a,m,r;z?(r=0,m=null):(a.equals(b)?m=a:(m=JU.BSUtil.copy(a),m.or(b)),r=m.nextSetBit(0));for(var p=this.getSelectedAtomIterator(null,!1,!1,!0,!1),n=!1;0<=r&&r<this.ac;r=z?r+1:m.nextSetBit(r+1)){var q=z||a.get(r),s=z||b.get(r),u=this.at[r];if(!(null==u||u.isDeleted())){var t=u.mi;if(t!=l){l=t;if(this.isJmolDataFrameForModel(t)){r=this.am[t].firstAtomIndex+this.am[t].act-1;continue}n=this.getInfoB(t,"autoBondUsingOccupation")}t=u.getBondingRadius();if(0!=t){var v=u.getFormalCharge(),w=0!=v;
w&&(v=Math.signum(v));var A=null!=d&&d.get(r);for(this.setIteratorForAtom(p,-1,r,t+this.maxBondingRadius+e,null);p.hasNext();){var B=this.at[p.next()];if(!B.isDeleted()){var D=B.i,G=z||a.get(D),E=z||b.get(D);!G&&!E||(!(q&&E||s&&G)||A&&d.get(D)||n&&null!=this.occupancies&&50>this.occupancies[r]!=50>this.occupancies[D]||w&&Math.signum(B.getFormalCharge())==v)||(D=this.isBondable(t,B.getBondingRadius(),p.foundDistance2(),h,e)?1:0,0<D&&this.autoBondCheck(u,B,D,f,c)&&j++)}}p.release()}}}this.showRebondTimes&&
JU.Logger.checkTimer("autoBond",!1);return j},"JU.BS,JU.BS,JU.BS,JU.BS,~N,~B");c(c$,"isBondable",function(a,b,d,c,f){if(0==a||0==b||d<c)return!1;a=a+b+f;return d<=a*a},"~N,~N,~N,~N,~N");c(c$,"autoBondCheck",function(a,b,d,c,f){if(20<a.getCurrentBondCount()||20<b.getCurrentBondCount())return this.maxBondWarned||JU.Logger.warn("maximum auto bond count reached"),this.maxBondWarned=!0,!1;var e=a.getFormalCharge();if(0!=e){var h=b.getFormalCharge();if(0>e&&0>h||0<e&&0<h)return!1}if(a.altloc!=b.altloc&&
"\x00"!=a.altloc&&"\x00"!=b.altloc&&null==this.getModulation(a.i))return!1;this.getOrAddBond(a,b,d,c,f,0,!1);return!0},"JM.Atom,JM.Atom,~N,~N,JU.BS");c(c$,"autoBond_Pre_11_9_24",function(a,b,d,c,f){if(0==this.ac)return 0;0==f&&(f=1);1.4E-45==this.maxBondingRadius&&this.findMaxRadii();var e=this.vwr.getFloat(570425348),h=this.vwr.getFloat(570425364),h=h*h,j=0;this.initializeBspf();for(var l=-1,z=this.ac;0<=--z;){var m=this.at[z];if(null!=m){var r=null==a||a.get(z),p=null==b||b.get(z);if((r||p)&&!m.isDeleted()){var n=
m.mi;if(n!=l&&(l=n,this.isJmolDataFrameForModel(n))){for(;0<=--z&&!(null==this.at[z]||this.at[z].mi!=n););z++;continue}var q=m.getBondingRadius();if(0!=q){var s=q+this.maxBondingRadius+e;this.initializeBspt(n);n=this.bspf.getCubeIterator(n);for(n.initialize(m,s,!0);n.hasMoreElements();)if(s=n.nextElement(),!(s===m||s.isDeleted())){var t=s.i,u=null==a||a.get(t),v=null==b||b.get(t);if(!(!u&&!v||null!=d&&d.get(t)&&d.get(z)))if(r&&v||p&&u)t=this.isBondable(q,s.getBondingRadius(),n.foundDistance2(),h,
e)?1:0,0<t&&this.autoBondCheck(m,s,t,f,c)&&j++}n.release()}}}}return j},"JU.BS,JU.BS,JU.BS,JU.BS,~N");c(c$,"autoHbond",function(a,b,d){if(d){for(var c=this.getModelBS(a,!1),f=c.nextSetBit(0);0<=f&&d;f=c.nextSetBit(f+1))d=!this.am[f].hasRasmolHBonds;if(d)return 0}d=!1;for(f=a.nextSetBit(0);0<=f;f=a.nextSetBit(f+1))if(1==this.at[f].getElementNumber()){d=!0;break}c=new JU.BS;f=this.vwr.getBoolean(603979853);if(null==b||f&&!d)return JU.Logger.info((null==b?"DSSP/DSSR ":"RasMol")+" pseudo-hbond calculation"),
this.calcRasmolHydrogenBonds(a,b,null,!1,2147483647,!1,c),-c.cardinality();JU.Logger.info(d?"Standard Hbond calculation":"Jmol pseudo-hbond calculation");var e=null;if(!d){e=new JU.BS;for(f=a.nextSetBit(0);0<=f;f=a.nextSetBit(f+1))switch(this.at[f].atomID){case 64:case 4:case 14:case 15:case 16:case 17:e.set(f)}}var h,j;d?(h=this.vwr.getFloat(570425361),j=1):(h=this.vwr.getFloat(570425360),j=JM.ModelSet.hbondMinRasmol*JM.ModelSet.hbondMinRasmol);var l=h*h,z=3.141592653589793*this.vwr.getFloat(570425359)/
180,m=0,r=0,n=new JU.V3,p=new JU.V3;this.showRebondTimes&&JU.Logger.debugging&&JU.Logger.startTimer("hbond");var q=null,s=null;b=this.getSelectedAtomIterator(b,!1,!1,!1,!1);for(f=a.nextSetBit(0);0<=f;f=a.nextSetBit(f+1)){var t=this.at[f],u=t.getElementNumber(),v=1==u;if(!(v?!d:d||7!=u&&8!=u)){if(v){var u=!1,w=t.bonds;if(null==w)continue;for(var A=!1,B=0;!A&&B<w.length;B++)A=w[B].getOtherAtom(t).getElementNumber(),A=7==A||8==A;if(!A)continue}else u=e.get(f);for(this.setIteratorForAtom(b,-1,t.i,h,null);b.hasNext();)if(w=
this.at[b.next()],B=w.getElementNumber(),!(w===t||(v?1==B:7!=B&&8!=B)||(r=b.foundDistance2())<j||r>l||u&&e.get(w.i)||t.isBonded(w))){if(0<z){n.sub2(t,w);if(null==(s=JM.ModelSet.checkMinAttachedAngle(t,z,n,p,d)))continue;n.scale(-1);if(null==(q=JM.ModelSet.checkMinAttachedAngle(w,z,n,p,d)))continue}B=0;v&&!Float.isNaN(q.x)&&!Float.isNaN(s.x)?(A=4096,B=JM.HBond.getEnergy(Math.sqrt(r),q.distance(t),q.distance(s),w.distance(s))/1E3):A=2048;c.set(this.addHBond(t,w,A,B));m++}}}b.release();this.sm.setShapeSizeBs(1,
-2147483648,null,c);this.showRebondTimes&&JU.Logger.checkTimer("hbond",!1);return d?m:-m},"JU.BS,JU.BS,~B");c$.checkMinAttachedAngle=c(c$,"checkMinAttachedAngle",function(a,b,d,c,f){var e=a.bonds,h=!0,j=null;if(null!=e&&0<e.length)for(var l=3.4028235E38,z=e.length;0<=--z;)if(e[z].isCovalent()){var h=!1,m=e[z].getOtherAtom(a);if(f||1!=m.getElementNumber()){c.sub2(a,m);var r=c.angle(d);if(r<b)return null;r<l&&(j=m,l=r)}}return h?JU.P3.new3(NaN,0,0):j},"JM.Atom,~N,JU.V3,JU.V3,~B");c(c$,"setStructureIndexes",
function(){for(var a,b=0,d=-1,c=-1,f=-1,e=0;e<this.ac;e++)if(null!=this.at[e]){if((c=this.at[e].mi)!=f)b=0,f=c,d=-1;if((a=this.at[e].group.getStrucNo())!=d&&0!=a)this.at[e].group.setStrucNo(++b),d=b}});c(c$,"getModelInfoAsString",function(){var a=(new JU.SB).append('<models count="');a.appendI(this.mc).append('" modelSetHasVibrationVectors="').append(this.modelSetHasVibrationVectors()+'">\n<properties>');if(null!=this.modelSetProperties){for(var b=this.modelSetProperties.propertyNames();b.hasMoreElements();){var d=
b.nextElement();a.append('\n <property name="').append(d).append('" value=').append(JU.PT.esc(this.modelSetProperties.getProperty(d))).append(" />")}a.append("\n</properties>")}for(b=0;b<this.mc;++b)a.append('\n<model index="').appendI(b).append('" n="').append(this.getModelNumberDotted(b)).append('" id=').append(JU.PT.esc(""+this.getInfo(b,"modelID"))),d=this.vwr.getJDXBaseModelIndex(b),d!=b&&a.append(" baseModelId=").append(JU.PT.esc(this.getInfo(d,"jdxModelID"))),a.append(" name=").append(JU.PT.esc(this.getModelName(b))).append(" title=").append(JU.PT.esc(this.getModelTitle(b))).append(' hasVibrationVectors="').appendB(this.vwr.modelHasVibrationVectors(b)).append('" />');
a.append("\n</models>");return a.toString()});c(c$,"getSymmetryInfoAsString",function(){for(var a=(new JU.SB).append("Symmetry Information:"),b=0;b<this.mc;++b){a.append("\nmodel #").append(this.getModelNumberDotted(b)).append("; name=").append(this.getModelName(b)).append("\n");var d=this.getUnitCell(b);a.append(null==d?"no symmetry information":d.getSymmetryInfoStr())}return a.toString()});c(c$,"createModels",function(a){a=this.mc+a;var b=JU.AU.arrayCopyObject(this.am,a);this.validateBspf(!1);this.modelNumbers=
JU.AU.arrayCopyI(this.modelNumbers,a);this.modelFileNumbers=JU.AU.arrayCopyI(this.modelFileNumbers,a);this.modelNumbersForAtomLabel=JU.AU.arrayCopyS(this.modelNumbersForAtomLabel,a);this.modelNames=JU.AU.arrayCopyS(this.modelNames,a);this.frameTitles=JU.AU.arrayCopyS(this.frameTitles,a);for(var d=w(this.modelFileNumbers[this.mc-1]/1E6)+1,c=this.mc,f=0;c<a;c++)this.modelNumbers[c]=c+this.mc,this.modelFileNumbers[c]=1E6*d+ ++f,this.modelNumbersForAtomLabel[c]=this.modelNames[c]=d+"."+f;this.thisStateModel=
-1;d=this.getInfoM("group3Lists");null!=d&&(c=this.getInfoM("group3Counts"),d=JU.AU.arrayCopyS(d,a),c=JU.AU.arrayCopyII(c,a),this.msInfo.put("group3Lists",d),this.msInfo.put("group3Counts",c));this.unitCells=JU.AU.arrayCopyObject(this.unitCells,a);for(c=this.mc;c<a;c++)b[c]=(new JM.Model).set(this,c,-1,null,null,null),b[c].loadState=" model create #"+c+";";this.am=b;this.mc=a},"~N");c(c$,"deleteAtoms",function(a){if(null!=a){for(var b=new JU.BS,d=a.nextSetBit(0);0<=d&&d<this.ac;d=a.nextSetBit(d+1))this.at[d].$delete(b);
for(d=0;d<this.mc;d++){var c=this.am[d];c.resetDSSR(!1);c.bsAtomsDeleted.or(a);c.bsAtomsDeleted.and(c.bsAtoms);a=JU.BSUtil.andNot(c.bsAtoms,c.bsAtomsDeleted);c.firstAtomIndex=a.nextSetBit(0);c.act=a.cardinality();c.isOrderly=c.act==c.bsAtoms.length()}this.deleteBonds(b,!1);this.validateBspf(!1)}},"JU.BS");c(c$,"clearDB",function(a){this.getModelAuxiliaryInfo(this.at[a].mi).remove("dbName")},"~N");c(c$,"adjustAtomArrays",function(a,b,d){this.ac=d;for(var c=b;c<d;c++){this.at[c]=this.at[a[c]];this.at[c].i=
c;var f=this.am[this.at[c].mi];f.firstAtomIndex==a[c]&&(f.firstAtomIndex=c);f.bsAtoms.set(c)}if(null!=this.vibrations)for(c=b;c<d;c++)this.vibrations[c]=this.vibrations[a[c]];if(null!=this.atomTensorList)for(c=b;c<d;c++)if(f=this.atomTensorList[c]=this.atomTensorList[a[c]],null!=f)for(var e=f.length;0<=--e;){var h=f[e];null!=h&&(h.atomIndex1=c)}if(null!=this.atomNames)for(c=b;c<d;c++)this.atomNames[c]=this.atomNames[a[c]];if(null!=this.atomTypes)for(c=b;c<d;c++)this.atomTypes[c]=this.atomTypes[a[c]];
if(null!=this.atomResnos)for(c=b;c<d;c++)this.atomResnos[c]=this.atomResnos[a[c]];if(null!=this.atomSerials)for(c=b;c<d;c++)this.atomSerials[c]=this.atomSerials[a[c]];if(null!=this.atomSeqIDs)for(c=b;c<d;c++)this.atomSeqIDs[c]=this.atomSeqIDs[a[c]];if(null!=this.bfactor100s)for(c=b;c<d;c++)this.bfactor100s[c]=this.bfactor100s[a[c]];if(null!=this.occupancies)for(c=b;c<d;c++)this.occupancies[c]=this.occupancies[a[c]];if(null!=this.partialCharges)for(c=b;c<d;c++)this.partialCharges[c]=this.partialCharges[a[c]]},
"~A,~N,~N");c(c$,"growAtomArrays",function(a){this.at=JU.AU.arrayCopyObject(this.at,a);null!=this.vibrations&&(this.vibrations=JU.AU.arrayCopyObject(this.vibrations,a));null!=this.occupancies&&(this.occupancies=JU.AU.arrayCopyF(this.occupancies,a));null!=this.bfactor100s&&(this.bfactor100s=JU.AU.arrayCopyShort(this.bfactor100s,a));null!=this.partialCharges&&(this.partialCharges=JU.AU.arrayCopyF(this.partialCharges,a));null!=this.atomTensorList&&(this.atomTensorList=JU.AU.arrayCopyObject(this.atomTensorList,
a));null!=this.atomNames&&(this.atomNames=JU.AU.arrayCopyS(this.atomNames,a));null!=this.atomTypes&&(this.atomTypes=JU.AU.arrayCopyS(this.atomTypes,a));null!=this.atomResnos&&(this.atomResnos=JU.AU.arrayCopyI(this.atomResnos,a));null!=this.atomSerials&&(this.atomSerials=JU.AU.arrayCopyI(this.atomSerials,a));null!=this.atomSeqIDs&&(this.atomSeqIDs=JU.AU.arrayCopyI(this.atomSeqIDs,a))},"~N");c(c$,"addAtom",function(a,b,d,c,f,e,h,j,l,z,m,r,n,p,q,s,t,u,v,w){j=(new JM.Atom).setAtom(a,this.ac,l,z,v,j,d,
r,t);this.am[a].act++;this.am[a].bsAtoms.set(this.ac);JU.Elements.isElement(d,1)&&this.am[a].hydrogenCount++;this.ac>=this.at.length&&this.growAtomArrays(this.ac+100);this.at[this.ac]=j;this.setBFactor(this.ac,q,!1);this.setOccupancy(this.ac,p,!1);this.setPartialCharge(this.ac,n,!1);null!=s&&this.setAtomTensors(this.ac,s);j.group=b;j.colixAtom=this.vwr.cm.getColixAtomPalette(j,J.c.PAL.CPK.id);null!=c&&(null!=f&&(null==this.atomTypes&&(this.atomTypes=Array(this.at.length)),this.atomTypes[this.ac]=
f),j.atomID=u,0==u&&(null==this.atomNames&&(this.atomNames=Array(this.at.length)),this.atomNames[this.ac]=c.intern()));-2147483648!=e&&(null==this.atomSerials&&(this.atomSerials=B(this.at.length,0)),this.atomSerials[this.ac]=e);0!=h&&(null==this.atomSeqIDs&&(this.atomSeqIDs=B(this.at.length,0)),this.atomSeqIDs[this.ac]=h);null!=m&&this.setVibrationVector(this.ac,m);Float.isNaN(w)||this.setBondingRadius(this.ac,w);this.ac++;return j},"~N,JM.Group,~N,~S,~S,~N,~N,~N,JU.P3,~N,JU.V3,~N,~N,~N,~N,JU.Lst,~B,~N,JU.BS,~N");
c(c$,"getInlineData",function(a){var b=null;if(0<=a)b=this.am[a].loadScript;else for(a=this.mc;0<=--a&&!(0<(b=this.am[a].loadScript).length()););a=b.lastIndexOf('data "');if(0>a)return b=JU.PT.getQuotedStringAt(b.toString(),0),JS.ScriptCompiler.unescapeString(b,0,b.length);a=b.indexOf2('"',a+7);var d=b.lastIndexOf('end "');return d<a||0>a?null:b.substring2(a+2,d)},"~N");c(c$,"isAtomPDB",function(a){return 0<=a&&this.am[this.at[a].mi].isBioModel},"~N");c(c$,"isAtomInLastModel",function(a){return 0<=
a&&this.at[a].mi==this.mc-1},"~N");c(c$,"haveModelKit",function(){for(var a=0;a<this.mc;a++)if(this.am[a].isModelKit)return!0;return!1});c(c$,"getModelKitStateBitset",function(a,b){for(var d=JU.BSUtil.copy(b),c=0;c<this.mc;c++)this.am[c].isModelKit||d.andNot(this.am[c].bsAtoms);return JU.BSUtil.deleteBits(a,d)},"JU.BS,JU.BS");c(c$,"setAtomNamesAndNumbers",function(a,b,d){0>b&&(a=this.am[this.at[a].mi].firstAtomIndex);null==this.atomSerials&&(this.atomSerials=B(this.ac,0));null==this.atomNames&&(this.atomNames=
Array(this.ac));for(var c=this.isXYZ&&this.vwr.getBoolean(603979978),f=2147483647,e=1;a<this.ac;++a){var h=this.at[a];if(null!=h){h.mi!=f&&(f=h.mi,e=c?0:1);var j=this.atomSerials[a];if(a>=-b){if(0==j||0>b)this.atomSerials[a]=a<b?d.atomSerials[a]:e;if(null==this.atomNames[a]||0>b)this.atomNames[a]=(h.getElementSymbol()+this.atomSerials[a]).intern()}else j>e&&(e=j);(!this.am[f].isModelKit||0<h.getElementNumber()&&!h.isDeleted())&&e++}}},"~N,~N,JM.AtomCollection");c(c$,"setUnitCellOffset",function(a,
b,d){null!=a&&(null==b?a.setOffset(d):a.setOffsetPt(b))},"J.api.SymmetryInterface,JU.T3,~N");c(c$,"connect",function(a){this.resetMolecules();for(var b=new JU.BS,d=0;d<a.length;d++){var c=a[d];if(!(null==c||2>c.length)){var f=A(c[0]),e=0>f;e&&(f=-1-f);var h=A(c[1]);if(!(0>h||f>=this.ac||h>=this.ac)){var j=2<c.length?A(c[2]):1;0>j&&(j&=65535);var l=3<c.length?$(1E3*a[d][3]):this.getDefaultMadFromOrder(j);0==j||0==l&&32768!=j&&!JU.Edge.isOrderH(j)?(c=this.at[f].getBond(this.at[h]),null!=c&&b.set(c.index)):
this.bondAtoms(this.at[f],this.at[h],j,l,null,4<c.length?c[4]:0,e,!0)}}}0<=b.nextSetBit(0)&&this.deleteBonds(b,!1)},"~A");c(c$,"setFrameDelayMs",function(a,b){for(var d=b.nextSetBit(0);0<=d;d=b.nextSetBit(d+1))this.am[this.am[d].trajectoryBaseIndex].frameDelay=a},"~N,JU.BS");c(c$,"getFrameDelayMs",function(a){return a<this.am.length&&0<=a?this.am[this.am[a].trajectoryBaseIndex].frameDelay:0},"~N");c(c$,"getModelIndexFromId",function(a){var b=0<=a.indexOf("#"),d=a.toLowerCase().endsWith(".basemodel");
d&&(a=a.substring(0,a.length-10));for(var c=-1,f=null,e=0;e<this.mc;e++){var h=this.getInfo(e,"modelID"),j=a.startsWith("~")?"~"+this.getModelNumberDotted(e):null;if(!(null==j&&null==h&&null==(h=this.getModelTitle(e)))){b&&(f=this.getModelFileName(e),f.endsWith("#molfile")?h=f:(f+="#",h=f+h));if(a.equalsIgnoreCase(h)||a.equalsIgnoreCase(j))return d?this.vwr.getJDXBaseModelIndex(e):e;null!=f&&a.startsWith(f)&&(c=-2)}}return null==f&&!b?-2:c},"~S");c(c$,"getAuxiliaryInfo",function(a){var b=this.msInfo;
null==b&&(b=new java.util.Hashtable);if(null!=a||!b.containsKey("models")){for(var d=new JU.Lst,c=0;c<this.mc;++c)if(null==a||a.get(c)){var f=this.getModelAuxiliaryInfo(c);f.put("modelIndex",Integer.$valueOf(c));d.addLast(f)}b.put("models",d)}return b},"JU.BS");c(c$,"getDihedralMap",function(a){for(var b=new JU.Lst,d=a.length,c=null,f=null,e=null,h=null,j=d-1;0<=--j;)for(var l=d;--l>j;)if(c=this.at[a[j]],f=this.at[a[l]],c.isBonded(f))for(var n=d;0<=--n;)if(n!=j&&n!=l&&(e=this.at[a[n]]).isBonded(c))for(var m=
d;0<=--m;)if(m!=j&&m!=l&&m!=n&&(h=this.at[a[m]]).isBonded(f)){var r=B(4,0);r[0]=e.i;r[1]=c.i;r[2]=f.i;r[3]=h.i;b.addLast(r)}d=b.size();a=JU.AU.newInt2(d);for(j=d;0<=--j;)a[d-j-1]=b.get(j);return a},"~A");c(c$,"setModulation",function(a,b,d,c){if(null==this.bsModulated)if(b)this.bsModulated=new JU.BS;else if(null==a)return;null==a&&(a=this.getModelAtomBitSetIncludingDeleted(-1,!1));for(var f=this.vwr.getFloat(1275072532),e=!1,h=a.nextSetBit(0);0<=h;h=a.nextSetBit(h+1)){var j=this.getModulation(h);
null!=j&&(j.setModTQ(this.at[h],b,d,c,f),null!=this.bsModulated&&this.bsModulated.setBitTo(h,b),e=!0)}e||(this.bsModulated=null)},"JU.BS,~B,JU.P3,~B");c(c$,"getBoundBoxOrientation",function(a,b){var d=0,c=0,f=0,e=null,h=null,j=b.nextSetBit(0),l=0;if(0<=j){c=null==this.vOrientations?0:this.vOrientations.length;if(0==c){e=Array(3375);c=0;l=new JU.P4;for(d=-7;7>=d;d++)for(f=-7;7>=f;f++)for(var n=0;14>=n;n++,c++)1<(e[c]=JU.V3.new3(d/7,f/7,n/14)).length()&&--c;this.vOrientations=Array(c);for(d=c;0<=--d;)n=
Math.sqrt(1-e[d].lengthSquared()),Float.isNaN(n)&&(n=0),l.set4(e[d].x,e[d].y,e[d].z,n),this.vOrientations[d]=JU.Quat.newP4(l)}var n=new JU.P3,l=3.4028235E38,m=null,r=new JU.BoxInfo;r.setMargin(1312817669==a?0:0.1);for(d=0;d<c;d++){e=this.vOrientations[d];r.reset();for(f=j;0<=f;f=b.nextSetBit(f+1))r.addBoundBoxPoint(e.transform2(this.at[f],n));switch(a){default:case 1312817669:case 1073741864:case 1814695966:f=(r.bbCorner1.x-r.bbCorner0.x)*(r.bbCorner1.y-r.bbCorner0.y)*(r.bbCorner1.z-r.bbCorner0.z);
break;case 1111492629:f=r.bbCorner1.x-r.bbCorner0.x;break;case 1111492630:f=r.bbCorner1.y-r.bbCorner0.y;break;case 1111492631:f=r.bbCorner1.z-r.bbCorner0.z}f<l&&(h=e,m=r,r=new JU.BoxInfo,r.setMargin(0.1),l=f)}switch(a){default:return h;case 1814695966:j=m.getBoundBoxVertices();j=u(-1,[j[0],j[4],j[2],j[1]]);h=h.inv();for(d=0;4>d;d++)h.transform2(j[d],j[d]),0<d&&j[d].sub(j[0]);return j;case 1312817669:case 1073741864:e=JU.Quat.newQ(h),d=m.bbCorner1.x-m.bbCorner0.x,c=m.bbCorner1.y-m.bbCorner0.y,f=m.bbCorner1.z-
m.bbCorner0.z,d<c&&(n.set(0,0,1),e=JU.Quat.newVA(n,90).mulQ(e),h=d,d=c,c=h),c<f&&(f>d&&(n.set(0,1,0),e=JU.Quat.newVA(n,90).mulQ(e),h=d,d=f,f=h),n.set(1,0,0),e=JU.Quat.newVA(n,90).mulQ(e),h=c,c=f,f=h)}}return 1312817669==a?l+"\t{"+d+" "+c+" "+f+"}\t"+b:1814695966==a?null:null==e||0==e.getTheta()?new JU.Quat:e},"~N,JU.BS");c(c$,"getUnitCellForAtom",function(a){if(0>a||a>this.ac)return null;if(null!=this.bsModulated){var b=this.getModulation(a),b=null==b?null:b.getSubZystemUnitCell();if(null!=b)return b}return this.getUnitCell(this.at[a].mi)},
"~N");c(c$,"clearCache",function(){for(var a=this.mc;0<=--a;)this.am[a].resetDSSR(!1)});c(c$,"getSymMatrices",function(a){var b=this.getModelSymmetryCount(a);if(0==b)return null;var d=Array(b),c=this.am[a].biosymmetry;null==c&&(c=this.getUnitCell(a));for(a=b;0<=--a;)d[a]=c.getSpaceGroupOperation(a);return d},"~N");c(c$,"getBsBranches",function(a){for(var b=w(a.length/6),d=Array(b),c=new java.util.Hashtable,f=0,e=0;f<b;f++,e+=6)if(!(1>Math.abs(a[e+5]-a[e+4]))){var h=A(a[e+1]),j=A(a[e+2]),l=""+h+"_"+
j;if(!c.containsKey(l)){c.put(l,Boolean.TRUE);for(var l=this.vwr.getBranchBitSet(j,h,!0),n=this.at[h].bonds,h=this.at[h],m=0;m<n.length;m++){var r=n[m];if(r.isCovalent()&&(r=r.getOtherAtom(h).i,r!=j&&l.get(r))){l=null;break}}d[f]=l}}return d},"~A");c(c$,"recalculatePositionDependentQuantities",function(a,b){null!=this.vwr.shm.getShape(21)&&this.vwr.shm.getShapePropertyData(21,"move",u(-1,[a,b]));this.haveStraightness&&this.calculateStraightnessAll();this.recalculateLeadMidpointsAndWingVectors(-1);
for(var d=this.getModelBS(a,!1),c=d.nextSetBit(0);0<=c;c=d.nextSetBit(c+1))if(this.sm.notifyAtomPositionsChanged(c,a,b),null!=b){var f=this.am[c];f.isContainedIn(a)&&(null==f.mat4&&(f.mat4=JU.M4.newM4(null)),f.mat4.mul2(b,f.mat4))}},"JU.BS,JU.M4");c(c$,"moveAtoms",function(a,b,d,c,f,e,h,j){if(null!=a){for(b=f.nextSetBit(0);0<=b;b=f.nextSetBit(b+1))a.rotTrans(this.at[b]),this.taintAtom(b,2);this.mat4.setM4(a);c=null}else if(!j){null==b?this.matTemp.setM3(d):(this.ptTemp.set(0,0,0),this.matInv.setM3(d),
this.matInv.invert(),this.matTemp.mul2(b,d),this.matTemp.mul2(this.matInv,this.matTemp));h?(this.vTemp.setT(e),this.mat4.setIdentity(),this.mat4.setTranslation(this.vTemp),this.mat4t.setToM3(this.matTemp),this.mat4.mul(this.mat4t),this.mat4t.setIdentity(),this.vTemp.scale(-1),this.mat4t.setTranslation(this.vTemp),this.mat4.mul(this.mat4t)):this.mat4.setToM3(this.matTemp);for(b=f.nextSetBit(0);0<=b;b=f.nextSetBit(b+1))h?this.mat4.rotTrans(this.at[b]):(this.ptTemp.add(this.at[b]),this.mat4.rotTrans(this.at[b]),
this.ptTemp.sub(this.at[b])),this.taintAtom(b,2);h||(this.ptTemp.scale(1/f.cardinality()),null==c&&(c=new JU.V3),c.add(this.ptTemp))}if(null!=c){for(b=f.nextSetBit(0);0<=b;b=f.nextSetBit(b+1))this.at[b].add(c),this.taintAtom(b,2);j||(this.mat4t.setIdentity(),this.mat4t.setTranslation(c),this.mat4.mul2(this.mat4t,this.mat4))}this.recalculatePositionDependentQuantities(f,this.mat4)},"JU.M4,JU.M3,JU.M3,JU.V3,JU.BS,JU.P3,~B,~B");c(c$,"setDihedrals",function(a,b,d){var c=w(a.length/6);1<d&&(d=1);for(var f=
0,e=0;f<c;f++,e+=6){var h=b[f];if(!(null==h||h.isEmpty())){var j=this.at[A(a[e+1])],l=JU.V3.newVsub(this.at[A(a[e+2])],j),l=JU.A4.newVA(l,-((a[e+5]-a[e+4])*d)/57.29577951308232);this.matTemp.setAA(l);this.ptTemp.setT(j);for(j=h.nextSetBit(0);0<=j;j=h.nextSetBit(j+1))this.at[j].sub(this.ptTemp),this.matTemp.rotate(this.at[j]),this.at[j].add(this.ptTemp),this.taintAtom(j,2)}}},"~A,~A,~N");c(c$,"setAtomCoordsRelative",function(a,b){this.setAtomsCoordRelative(b,a.x,a.y,a.z);this.mat4.setIdentity();this.vTemp.setT(a);
this.mat4.setTranslation(this.vTemp);this.recalculatePositionDependentQuantities(b,this.mat4)},"JU.T3,JU.BS");c(c$,"setAtomCoords",function(a,b,d){this.setAtomCoord2(a,b,d);switch(b){case 1111492626:case 1111492627:case 1111492628:case 1145047055:break;default:this.recalculatePositionDependentQuantities(a,null)}},"JU.BS,~N,~O");c(c$,"invertSelected",function(a,b,d,c){this.resetChirality();if(null!=a)for(var f=c.nextSetBit(0);0<=f;f=c.nextSetBit(f+1)){var e=2*(a.x-this.at[f].x);d=2*(a.y-this.at[f].y);
var h=2*(a.z-this.at[f].z);this.setAtomCoordRelative(f,e,d,h)}else if(null!=b){a=JU.V3.new3(b.x,b.y,b.z);a.normalize();for(var j=Math.sqrt(b.x*b.x+b.y*b.y+b.z*b.z),f=c.nextSetBit(0);0<=f;f=c.nextSetBit(f+1))h=2*-JU.Measure.distanceToPlaneD(b,j,this.at[f]),e=a.x*h,d=a.y*h,h*=a.z,this.setAtomCoordRelative(f,e,d,h)}else if(0<=d&&(b=this.at[d],a=b.bonds,null!=a)){e=new JU.BS;h=new JU.Lst;j=this.vwr.getModelUndeletedAtomsBitSet(b.mi);for(f=0;f<a.length;f++){var l=a[f].getOtherAtom(b);c.get(l.i)?e.or(JU.JmolMolecule.getBranchBitSet(this.at,
l.i,j,null,d,!0,!0)):h.addLast(l)}0!=h.size()&&(a=JU.Measure.getCenterAndPoints(h)[0],c=JU.V3.newVsub(b,a),c=JU.Quat.newVA(c,180),this.moveAtoms(null,null,c.getMatrix(),null,e,b,!0,!1))}},"JU.P3,JU.P4,~N,JU.BS");c(c$,"getCellWeights",function(a){var b=null,d=a.nextSetBit(0),c=-1;if(0<=d&&null!=this.getUnitCell(c=this.at[d].mi)){var f=new JU.P3;this.getModelAtomBitSetIncludingDeleted(c,!0).and(a);b=K(a.cardinality(),0);for(c=0;0<=d;d=a.nextSetBit(d+1))b[c++]=JU.SimpleUnitCell.getCellWeight(this.at[d].getFractionalUnitCoordPt(!0,
!1,f))}return b},"JU.BS");c(c$,"getAtomGroupQuaternions",function(a,b,d){var c=0,f=new JU.Lst;a=JU.BSUtil.copy(a);for(var e=new JU.BS,h=a.nextSetBit(0);0<=h&&c<b;h=a.nextSetBit(h+1)){var j=this.at[h].group;j.setAtomBits(e);a.andNot(e);var l=j.getQuaternion(d);if(null==l&&(this.am[this.at[h].mi].isBioModel||(l=j.getQuaternionFrame(this.at)),null==l))continue;c++;f.addLast(l)}return f.toArray(Array(f.size()))},"JU.BS,~N,~S");c(c$,"getConformation",function(a,b,d,c){var f=new JU.BS;if(0<=b)for(var e=
this.mc;0<=--e;)if(!(e!=a&&0<=a)){var h=this.am[e],j=this.vwr.getModelUndeletedAtomsBitSet(a);null!=c&&j.and(c);if(!(0>j.nextSetBit(0)))if(b>=h.altLocCount)0==b&&f.or(j);else if(!this.am[e].isBioModel||!this.am[e].getConformation(b,d,j,f)){for(var l=this.getAltLocCountInModel(e),h=this.getAltLocListInModel(e),n=new JU.BS;0<=--l;)l!=b&&j.andNot(this.getAtomBitsMDa(1073742355,h.substring(l,l+1),n));0<=j.nextSetBit(0)&&f.or(j)}}return f},"~N,~N,~B,JU.BS");c(c$,"getSequenceBits",function(a,b,d){return this.haveBioModels?
this.bioModelset.getAllSequenceBits(a,b,d):d},"~S,JU.BS,JU.BS");c(c$,"getBioPolymerCountInModel",function(a){return this.haveBioModels?this.bioModelset.getBioPolymerCountInModel(a):0},"~N");c(c$,"getPolymerPointsAndVectors",function(a,b,d,c){this.haveBioModels&&this.bioModelset.getAllPolymerPointsAndVectors(a,b,d,c)},"JU.BS,JU.Lst,~B,~N");c(c$,"recalculateLeadMidpointsAndWingVectors",function(a){this.haveBioModels&&this.bioModelset.recalculatePoints(a)},"~N");c(c$,"calcRasmolHydrogenBonds",function(a,
b,d,c,f,e,h){this.haveBioModels&&this.bioModelset.calcAllRasmolHydrogenBonds(a,b,d,c,f,e,h,2)},"JU.BS,JU.BS,JU.Lst,~B,~N,~B,JU.BS");c(c$,"calculateStraightnessAll",function(){this.haveBioModels&&!this.haveStraightness&&this.bioModelset.calculateStraightnessAll()});c(c$,"calculateStruts",function(a,b){return this.haveBioModels?this.bioModelset.calculateStruts(a,b):0},"JU.BS,JU.BS");c(c$,"getGroupsWithin",function(a,b){return this.haveBioModels?this.bioModelset.getGroupsWithinAll(a,b):new JU.BS},"~N,JU.BS");
c(c$,"getProteinStructureState",function(a,b){return this.haveBioModels?this.bioModelset.getFullProteinStructureState(a,b):""},"JU.BS,~N");c(c$,"calculateStructures",function(a,b,d,c,f,e){return this.haveBioModels?this.bioModelset.calculateAllStuctures(a,b,d,c,f,e):""},"JU.BS,~B,~B,~B,~B,~N");c(c$,"calculateStructuresAllExcept",function(a,b,d,c,f,e,h){this.freezeModels();return this.haveBioModels?this.bioModelset.calculateAllStructuresExcept(a,b,d,c,f,e,h):""},"JU.BS,~B,~B,~B,~B,~B,~N");c(c$,"recalculatePolymers",
function(a){this.bioModelset.recalculateAllPolymers(a,this.getGroups())},"JU.BS");c(c$,"calculatePolymers",function(a,b,d,c){null!=this.bioModelset&&this.bioModelset.calculateAllPolymers(a,b,d,c)},"~A,~N,~N,JU.BS");c(c$,"calcSelectedMonomersCount",function(){this.haveBioModels&&this.bioModelset.calcSelectedMonomersCount()});c(c$,"setProteinType",function(a,b){this.haveBioModels&&this.bioModelset.setAllProteinType(a,b)},"JU.BS,J.c.STR");c(c$,"setStructureList",function(a){this.haveBioModels&&this.bioModelset.setAllStructureList(a)},
"java.util.Map");c(c$,"setConformation",function(a){this.haveBioModels&&this.bioModelset.setAllConformation(a);return a},"JU.BS");c(c$,"getHeteroList",function(a){a=this.haveBioModels?this.bioModelset.getAllHeteroList(a):null;return null==a?this.getInfoM("hetNames"):a},"~N");c(c$,"getUnitCellPointsWithin",function(a,b,d,c){var f=new JU.Lst,e=null,h=null;c&&(e=new java.util.Hashtable,h=new JU.Lst,e.put("atoms",h),e.put("points",f));var j=null==b?-1:b.nextSetBit(0);b=this.vwr.getModelUndeletedAtomsBitSet(0>
j?this.vwr.am.cmi:this.at[j].mi);0>j&&(j=b.nextSetBit(0));if(0<=j){var l=this.getUnitCellForAtom(j);if(null!=l){b=l.getIterator(this.vwr,this.at[j],this.at,b,a);for(null!=d&&b.setCenter(d,a);b.hasNext();)j=b.next(),d=b.getPosition(),f.addLast(d),c&&h.addLast(Integer.$valueOf(j))}}return c?e:f},"~N,JU.BS,JU.P3,~B");c(c$,"calculateDssrProperty",function(a){if(null!=a){if(null==this.dssrData||this.dssrData.length<this.ac)this.dssrData=K(this.ac,0);for(var b=0;b<this.ac;b++)this.dssrData[b]=NaN;for(b=
this.mc;0<=--b;)this.am[b].isBioModel&&this.am[b].getAtomicDSSRData(this.dssrData,a)}},"~S");c(c$,"getAtomicDSSRData",function(a){return null==this.dssrData||this.dssrData.length<=a?NaN:this.dssrData[a]},"~N");c(c$,"getAtomCIPChiralityCode",function(a){this.haveChirality=!0;var b=this.am[a.mi];b.hasChirality||(this.calculateChiralityForAtoms(b.bsAtoms,!1),b.hasChirality=!0);return a.getCIPChiralityCode()},"JM.Atom");c(c$,"calculateChiralityForAtoms",function(a,b){this.haveChirality=!0;for(var d=a.nextSetBit(0);0<=
d;d=a.nextSetBit(d+1))this.at[d].setCIPChirality(0);J.api.Interface.getSymmetry(this.vwr,"ms").calculateCIPChiralityForAtoms(this.vwr,a);if(!b)return null;for(var c="",d=a.nextSetBit(0);0<=d;d=a.nextSetBit(d+1))c+=this.at[d].getCIPChirality(!1);return c},"JU.BS,~B");E(c$,"hbondMinRasmol",2.5)});s("JM");v(["JU.M3","$.P3"],"JM.Orientation",["JU.PT","JU.Escape"],function(){c$=t(function(){this.rotationMatrix=this.saveName=null;this.rotationRadius=this.zoom=this.yTrans=this.xTrans=0;this.navCenter=this.center=
null;this.cameraY=this.cameraX=this.cameraDepth=this.navDepth=this.yNav=this.xNav=NaN;this.navigationMode=this.windowCenteredFlag=!1;this.vwr=this.pymolView=this.moveToText=null;p(this,arguments)},JM,"Orientation");O(c$,function(){this.rotationMatrix=new JU.M3;this.center=new JU.P3;this.navCenter=new JU.P3});q(c$,function(a,b,d){this.vwr=a;null!=d?(this.pymolView=d,this.moveToText="moveTo -1.0 PyMOL "+JU.Escape.eAF(d)):(a.finalizeTransformParameters(),b?(b=a.ms.getInfoM("defaultOrientationMatrix"),
null==b?this.rotationMatrix.setScale(1):this.rotationMatrix.setM3(b)):a.tm.getRotation(this.rotationMatrix),this.xTrans=a.tm.getTranslationXPercent(),this.yTrans=a.tm.getTranslationYPercent(),this.zoom=a.tm.getZoomSetting(),this.center.setT(a.tm.fixedRotationCenter),this.windowCenteredFlag=a.tm.isWindowCentered(),this.rotationRadius=a.getFloat(570425388),this.navigationMode=a.getBoolean(603979889),this.moveToText=a.tm.getMoveToText(-1,!1),this.navigationMode&&(this.xNav=a.tm.getNavigationOffsetPercent("X"),
this.yNav=a.tm.getNavigationOffsetPercent("Y"),this.navDepth=a.tm.navigationDepthPercent,this.navCenter=JU.P3.newP(a.tm.navigationCenter)),0!=a.tm.camera.z&&(this.cameraDepth=a.tm.getCameraDepth(),this.cameraX=a.tm.camera.x,this.cameraY=a.tm.camera.y))},"JV.Viewer,~B,~A");c(c$,"getMoveToText",function(a){return a?"   "+this.moveToText+"\n  save orientation "+JU.PT.esc(this.saveName.substring(12))+";\n":this.moveToText},"~B");c(c$,"restore",function(a,b){b?(this.vwr.setBooleanProperty("windowCentered",
this.windowCenteredFlag),this.vwr.setBooleanProperty("navigationMode",this.navigationMode),null==this.pymolView?this.vwr.moveTo(this.vwr.eval,a,this.center,null,NaN,this.rotationMatrix,this.zoom,this.xTrans,this.yTrans,this.rotationRadius,this.navCenter,this.xNav,this.yNav,this.navDepth,this.cameraDepth,this.cameraX,this.cameraY):this.vwr.tm.moveToPyMOL(this.vwr.eval,a,this.pymolView)):this.vwr.tm.setRotation(this.rotationMatrix);return!0},"~N,~B")});s("JM");v(null,"JM.StateScript",["JU.SB","JU.BSUtil",
"$.Escape"],function(){c$=t(function(){this.modelIndex=0;this.script2=this.script1=this.bsAtoms2=this.bsAtoms1=this.bsBonds=null;this.inDefinedStateBlock=!1;p(this,arguments)},JM,"StateScript");q(c$,function(a,b,d,c,f,e,h){this.modelIndex=a;this.script1=b;this.bsBonds=JU.BSUtil.copy(d);this.bsAtoms1=JU.BSUtil.copy(c);this.bsAtoms2=JU.BSUtil.copy(f);this.script2=e;this.inDefinedStateBlock=h},"~N,~S,JU.BS,JU.BS,JU.BS,~S,~B");c(c$,"isValid",function(){return null!=this.script1&&0<this.script1.length&&
(null==this.bsBonds||0<=this.bsBonds.nextSetBit(0))&&(null==this.bsAtoms1||0<=this.bsAtoms1.nextSetBit(0))&&(null==this.bsAtoms2||0<=this.bsAtoms2.nextSetBit(0))});j(c$,"toString",function(){if(!this.isValid())return"";var a=JU.SB.newS(this.script1);null!=this.bsBonds&&a.append(" ").append(JU.Escape.eBond(this.bsBonds));null!=this.bsAtoms1&&a.append(" ").append(JU.Escape.eBS(this.bsAtoms1));null!=this.bsAtoms2&&a.append(" ").append(JU.Escape.eBS(this.bsAtoms2));null!=this.script2&&a.append(" ").append(this.script2);
a=a.toString();a.endsWith(";")||(a+=";");return a});c(c$,"isConnect",function(){return 0<=this.script1.indexOf("connect")});c(c$,"deleteAtoms",function(a,b,d){if(a==this.modelIndex)return!1;if(a>this.modelIndex)return!0;JU.BSUtil.deleteBits(this.bsBonds,b);JU.BSUtil.deleteBits(this.bsAtoms1,d);JU.BSUtil.deleteBits(this.bsAtoms2,d);return this.isValid()},"~N,JU.BS,JU.BS");c(c$,"setModelIndex",function(a){this.modelIndex=a},"~N")});s("JM");N(JM,"Structure");s("JM");c$=t(function(){this.id="";this.type=
" ";this.scale=this.tickLabelFormats=this.ticks=null;this.first=0;this.signFactor=1;this.reference=null;p(this,arguments)},JM,"TickInfo");q(c$,function(a){this.ticks=a},"JU.P3");s("JU");v(["JU.P3","$.V3"],"JU.BoxInfo",["JU.Point3fi"],function(){c$=t(function(){this.bbVertices=this.bbVector=this.bbCenter=this.bbCorner1=this.bbCorner0=null;this.isScaleSet=!1;this.margin=0;p(this,arguments)},JU,"BoxInfo");O(c$,function(){this.bbCorner0=new JU.P3;this.bbCorner1=new JU.P3;this.bbCenter=new JU.P3;this.bbVector=
new JU.V3;this.bbVertices=Array(8);for(var a=0;8>a;a++)JU.BoxInfo.unitBboxPoints[a]=JU.P3.new3(-1,-1,-1),JU.BoxInfo.unitBboxPoints[a].scaleAdd2(2,JU.BoxInfo.unitCubePoints[a],JU.BoxInfo.unitBboxPoints[a])});q(c$,function(){for(var a=8;0<=--a;)this.bbVertices[a]=new JU.Point3fi;this.reset()});c(c$,"reset",function(){this.isScaleSet=!1;this.bbCorner0.set(3.4028235E38,3.4028235E38,3.4028235E38);this.bbCorner1.set(-3.4028235E38,-3.4028235E38,-3.4028235E38)});c$.scaleBox=c(c$,"scaleBox",function(a,b){if(!(0==
b||1==b)){for(var d=new JU.P3,c=new JU.V3,f=0;8>f;f++)d.add(a[f]);d.scale(0.125);for(f=0;8>f;f++)c.sub2(a[f],d),c.scale(b),a[f].add2(d,c)}},"~A,~N");c$.getVerticesFromOABC=c(c$,"getVerticesFromOABC",function(a){for(var b=Array(8),d=0;7>=d;d++)b[d]=JU.P3.newP(a[0]),4==(d&4)&&b[d].add(a[1]),2==(d&2)&&b[d].add(a[2]),1==(d&1)&&b[d].add(a[3]);return b},"~A");c$.getCanonicalCopy=c(c$,"getCanonicalCopy",function(a,b){for(var d=Array(8),c=0;8>c;c++)d[JU.BoxInfo.toCanonical[c]]=JU.P3.newP(a[c]);JU.BoxInfo.scaleBox(d,
b);return d},"~A,~N");c$.toOABC=c(c$,"toOABC",function(a,b){var d=JU.P3.newP(a[0]),c=JU.P3.newP(a[4]),f=JU.P3.newP(a[2]),e=JU.P3.newP(a[1]);c.sub(d);f.sub(d);e.sub(d);null!=b&&d.add(b);return u(-1,[d,c,f,e])},"~A,JU.T3");c(c$,"getBoundBoxCenter",function(){this.isScaleSet||this.setBbcage(1);return this.bbCenter});c(c$,"getBoundBoxCornerVector",function(){this.isScaleSet||this.setBbcage(1);return this.bbVector});c(c$,"getBoundBoxPoints",function(a){this.isScaleSet||this.setBbcage(1);return a?u(-1,
[this.bbCenter,JU.P3.newP(this.bbVector),this.bbCorner0,this.bbCorner1]):u(-1,[this.bbCorner0,this.bbCorner1])},"~B");c(c$,"getBoundBoxVertices",function(){this.isScaleSet||this.setBbcage(1);return this.bbVertices});c(c$,"setBoundBoxFromOABC",function(a){for(var b=JU.P3.newP(a[0]),d=new JU.P3,c=0;4>c;c++)d.add(a[c]);this.setBoundBox(b,d,!0,1)},"~A");c(c$,"setBoundBox",function(a,b,d,c){if(null!=a){if(0==c)return;if(d){if(0==a.distance(b))return;this.bbCorner0.set(Math.min(a.x,b.x),Math.min(a.y,b.y),
Math.min(a.z,b.z));this.bbCorner1.set(Math.max(a.x,b.x),Math.max(a.y,b.y),Math.max(a.z,b.z))}else{if(0==b.x||0==b.y&&0==b.z)return;this.bbCorner0.set(a.x-b.x,a.y-b.y,a.z-b.z);this.bbCorner1.set(a.x+b.x,a.y+b.y,a.z+b.z)}}this.setBbcage(c)},"JU.T3,JU.T3,~B,~N");c(c$,"setMargin",function(a){this.margin=a},"~N");c(c$,"addBoundBoxPoint",function(a){this.isScaleSet=!1;JU.BoxInfo.addPoint(a,this.bbCorner0,this.bbCorner1,this.margin)},"JU.T3");c$.addPoint=c(c$,"addPoint",function(a,b,d,c){a.x-c<b.x&&(b.x=
a.x-c);a.x+c>d.x&&(d.x=a.x+c);a.y-c<b.y&&(b.y=a.y-c);a.y+c>d.y&&(d.y=a.y+c);a.z-c<b.z&&(b.z=a.z-c);a.z+c>d.z&&(d.z=a.z+c)},"JU.T3,JU.T3,JU.T3,~N");c$.addPointXYZ=c(c$,"addPointXYZ",function(a,b,d,c,f,e){a-e<c.x&&(c.x=a-e);a+e>f.x&&(f.x=a+e);b-e<c.y&&(c.y=b-e);b+e>f.y&&(f.y=b+e);d-e<c.z&&(c.z=d-e);d+e>f.z&&(f.z=d+e)},"~N,~N,~N,JU.P3,JU.P3,~N");c(c$,"setBbcage",function(a){this.isScaleSet=!0;this.bbCenter.add2(this.bbCorner0,this.bbCorner1);this.bbCenter.scale(0.5);this.bbVector.sub2(this.bbCorner1,
this.bbCenter);0<a?this.bbVector.scale(a):(this.bbVector.x-=a/2,this.bbVector.y-=a/2,this.bbVector.z-=a/2);for(a=8;0<=--a;){var b=this.bbVertices[a];b.setT(JU.BoxInfo.unitBboxPoints[a]);b.x*=this.bbVector.x;b.y*=this.bbVector.y;b.z*=this.bbVector.z;b.add(this.bbCenter)}this.bbCorner0.setT(this.bbVertices[0]);this.bbCorner1.setT(this.bbVertices[7])},"~N");c(c$,"isWithin",function(a){this.isScaleSet||this.setBbcage(1);return a.x>=this.bbCorner0.x&&a.x<=this.bbCorner1.x&&a.y>=this.bbCorner0.y&&a.y<=
this.bbCorner1.y&&a.z>=this.bbCorner0.z&&a.z<=this.bbCorner1.z},"JU.P3");c(c$,"getMaxDim",function(){return 2*this.bbVector.length()});E(c$,"X",4,"Y",2,"Z",1,"XYZ",7,"bbcageTickEdges",ca(-1,"z\x00\x00yx\x00\x00\x00\x00\x00\x00\x00".split("")),"uccageTickEdges",ca(-1,"zyx\x00\x00\x00\x00\x00\x00\x00\x00\x00".split("")),"edges",Q(-1,[0,1,0,2,0,4,1,3,1,5,2,3,2,6,3,7,4,5,4,6,5,7,6,7]));c$.unitCubePoints=c$.prototype.unitCubePoints=u(-1,[JU.P3.new3(0,0,0),JU.P3.new3(0,0,1),JU.P3.new3(0,1,0),JU.P3.new3(0,
1,1),JU.P3.new3(1,0,0),JU.P3.new3(1,0,1),JU.P3.new3(1,1,0),JU.P3.new3(1,1,1)]);E(c$,"facePoints",u(-1,[B(-1,[4,0,6]),B(-1,[4,6,5]),B(-1,[5,7,1]),B(-1,[1,3,0]),B(-1,[6,2,7]),B(-1,[1,0,5]),B(-1,[0,2,6]),B(-1,[6,7,5]),B(-1,[7,3,1]),B(-1,[3,2,0]),B(-1,[2,3,7]),B(-1,[0,4,5])]),"toCanonical",B(-1,[0,3,4,7,1,2,5,6]));c$.unitBboxPoints=c$.prototype.unitBboxPoints=Array(8)});s("JU");v(["JU.BS"],"JU.BSUtil",null,function(){c$=D(JU,"BSUtil");c$.newAndSetBit=c(c$,"newAndSetBit",function(a){var b=JU.BS.newN(a+
1);b.set(a);return b},"~N");c$.areEqual=c(c$,"areEqual",function(a,b){return null==a||null==b?null==a&&null==b:a.equals(b)},"JU.BS,JU.BS");c$.haveCommon=c(c$,"haveCommon",function(a,b){return null==a||null==b?!1:a.intersects(b)},"JU.BS,JU.BS");c$.cardinalityOf=c(c$,"cardinalityOf",function(a){return null==a?0:a.cardinality()},"JU.BS");c$.newBitSet2=c(c$,"newBitSet2",function(a,b){var d=JU.BS.newN(b);d.setBits(a,b);return d},"~N,~N");c$.setAll=c(c$,"setAll",function(a){var b=JU.BS.newN(a);b.setBits(0,
a);return b},"~N");c$.andNot=c(c$,"andNot",function(a,b){null!=b&&null!=a&&a.andNot(b);return a},"JU.BS,JU.BS");c$.copy=c(c$,"copy",function(a){return null==a?null:a.clone()},"JU.BS");c$.copy2=c(c$,"copy2",function(a,b){if(null==a||null==b)return null;b.clearAll();b.or(a);return b},"JU.BS,JU.BS");c$.copyInvert=c(c$,"copyInvert",function(a,b){return null==a?null:JU.BSUtil.andNot(JU.BSUtil.setAll(b),a)},"JU.BS,~N");c$.invertInPlace=c(c$,"invertInPlace",function(a,b){return JU.BSUtil.copy2(JU.BSUtil.copyInvert(a,
b),a)},"JU.BS,~N");c$.toggleInPlace=c(c$,"toggleInPlace",function(a,b){a.equals(b)?a.clearAll():0==JU.BSUtil.andNot(JU.BSUtil.copy(b),a).length()?JU.BSUtil.andNot(a,b):a.or(b);return a},"JU.BS,JU.BS");c$.deleteBits=c(c$,"deleteBits",function(a,b){if(null==a||null==b)return a;var d=b.nextSetBit(0);if(0>d)return a;var c=a.length(),f=Math.min(c,b.length()),e;for(e=b.nextClearBit(d);e<f&&0<=e;e=b.nextClearBit(e+1))a.setBitTo(d++,a.get(e));for(e=f;e<c;e++)a.setBitTo(d++,a.get(e));d<c&&a.clearBits(d,c);
return a},"JU.BS,JU.BS");c$.shiftBits=c(c$,"shiftBits",function(a,b,d,c){if(!(null==a||null==b)){for(var f=b.length(),e=JU.BS.newN(f),h=!1,j=!1,l=d,n=0,m=0;n<f&&m<c;n++)b.get(n)?(j&&e.set(n),l=d,h=!1):a.get(m++)?(e.set(n),l&&(l=!1,j=h=!0)):l&&!h&&(j=!1);a.clearAll();a.or(e)}},"JU.BS,JU.BS,~B,~N");c$.offset=c(c$,"offset",function(a,b,d){if(null!=a){for(var c=JU.BS.newN(a.length()+d),f=a.nextSetBit(0);f>=b;f=a.nextSetBit(f+1))c.set(f+d);JU.BSUtil.copy2(c,a)}},"JU.BS,~N,~N");c$.setMapBitSet=c(c$,"setMapBitSet",
function(a,b,d,c){var f;a.containsKey(c)?f=a.get(c):a.put(c,f=new JU.BS);f.setBits(b,d+1)},"java.util.Map,~N,~N,~S");c$.emptySet=c$.prototype.emptySet=new JU.BS});s("JU");v(["JU.Int2IntHash"],"JU.C","java.lang.Byte $.Float JU.AU $.CU $.PT $.SB J.c.PAL JU.Escape $.Logger".split(" "),function(){c$=D(JU,"C");q(c$,function(){});c$.getColix=c(c$,"getColix",function(a){if(0==a)return 0;var b=0;-16777216!=(a&4278190080)&&(b=JU.C.getTranslucentFlag(a>>24&255),a|=4278190080);var c=JU.C.colixHash.get(a);3==
(c&3)&&(b=0);return(0<c?c:JU.C.allocateColix(a,!1))|b},"~N");c$.allocateColix=c(c$,"allocateColix",function(a,b){var c;if(b)c=2047;else{for(c=JU.C.colixMax;4<=--c;)if((a&16777215)==(JU.C.argbs[c]&16777215))return c;c=JU.C.colixMax}if(c>=JU.C.argbs.length){var e=b?c+1:2*JU.C.colixMax;2048<e&&(e=2048);JU.C.argbs=JU.AU.arrayCopyI(JU.C.argbs,e);null!=JU.C.argbsGreyscale&&(JU.C.argbsGreyscale=JU.AU.arrayCopyI(JU.C.argbsGreyscale,e))}JU.C.argbs[c]=a;null!=JU.C.argbsGreyscale&&(JU.C.argbsGreyscale[c]=JU.CU.toFFGGGfromRGB(a));
JU.C.colixHash.put(a,c);return 2047>c?JU.C.colixMax++:JU.C.colixMax},"~N,~B");c$.setLastGrey=c(c$,"setLastGrey",function(a){JU.C.calcArgbsGreyscale();JU.C.argbsGreyscale[2047]=JU.CU.toFFGGGfromRGB(a)},"~N");c$.calcArgbsGreyscale=c(c$,"calcArgbsGreyscale",function(){if(null==JU.C.argbsGreyscale){for(var a=B(JU.C.argbs.length,0),b=JU.C.argbs.length;4<=--b;)a[b]=JU.CU.toFFGGGfromRGB(JU.C.argbs[b]);JU.C.argbsGreyscale=a}});c$.getArgbGreyscale=c(c$,"getArgbGreyscale",function(a){null==JU.C.argbsGreyscale&&
JU.C.calcArgbsGreyscale();return JU.C.argbsGreyscale[a&-30721]},"~N");c$.getColixO=c(c$,"getColixO",function(a){if(null==a)return 0;if(n(a,J.c.PAL))return a===J.c.PAL.NONE?0:2;if(n(a,Integer))return JU.C.getColix(a.intValue());if(n(a,String))return JU.C.getColixS(a);if(n(a,Byte))return 0==a.byteValue()?0:2;JU.Logger.debugging&&JU.Logger.debug("?? getColix("+a+")");return 22},"~O");c$.getTranslucentFlag=c(c$,"getTranslucentFlag",function(a){return 0==a?0:0>a?30720:Float.isNaN(a)||255<=a||1==a?16384:
(w(Math.floor(1>a?256*a:15<=a?a:9>=a?w(Math.floor(a-1))<<5:256))>>5&15)<<11},"~N");c$.isColixLastAvailable=c(c$,"isColixLastAvailable",function(a){return 0<a&&2047==(a&2047)},"~N");c$.getArgb=c(c$,"getArgb",function(a){return JU.C.argbs[a&-30721]},"~N");c$.isColixColorInherited=c(c$,"isColixColorInherited",function(a){switch(a){case 0:case 1:return!0;default:return 1==(a&-30721)}},"~N");c$.getColixInherited=c(c$,"getColixInherited",function(a,b){switch(a){case 0:return b;case 1:return b&-30721;default:return 1==
(a&-30721)?b&-30721|a&30720:a}},"~N,~N");c$.renderPass2=c(c$,"renderPass2",function(a){a&=30720;return 0!=a&&30720!=a},"~N");c$.isColixTranslucent=c(c$,"isColixTranslucent",function(a){return 0!=(a&30720)},"~N");c$.getChangeableColixIndex=c(c$,"getChangeableColixIndex",function(a){return 0<=a?-1:a&2047},"~N");c$.getColixTranslucent3=c(c$,"getColixTranslucent3",function(a,b,c){a&=-30721;0==a&&(a=1);return b?a|JU.C.getTranslucentFlag(c):a},"~N,~B,~N");c$.copyColixTranslucency=c(c$,"copyColixTranslucency",
function(a,b){return JU.C.getColixTranslucent3(b,JU.C.isColixTranslucent(a),JU.C.getColixTranslucencyLevel(a))},"~N,~N");c$.getColixTranslucencyFractional=c(c$,"getColixTranslucencyFractional",function(a){a=JU.C.getColixTranslucencyLevel(a);return-1==a?0.5:0==a?0:255==a?1:a/256},"~N");c$.getColixTranslucencyLabel=c(c$,"getColixTranslucencyLabel",function(a){return"translucent "+(30720==(a&30720)?-1:JU.C.getColixTranslucencyFractional(a))},"~N");c$.getColixTranslucencyLevel=c(c$,"getColixTranslucencyLevel",
function(a){a=a>>11&15;switch(a){case 0:return 0;case 1:case 2:case 3:case 4:case 5:case 6:case 7:return a<<5;case 15:return-1;default:return 255}},"~N");c$.getColixS=c(c$,"getColixS",function(a){var b=JU.CU.getArgbFromString(a);return 0!=b?JU.C.getColix(b):"none".equalsIgnoreCase(a)?0:"opaque".equalsIgnoreCase(a)?1:2},"~S");c$.getColixArray=c(c$,"getColixArray",function(a){if(null==a||0==a.length)return null;a=JU.PT.getTokens(a);for(var b=T(a.length,0),c=0;c<a.length;c++)if(b[c]=JU.C.getColix(JU.CU.getArgbFromString(a[c])),
0==b[c])return null;return b},"~S");c$.getHexCode=c(c$,"getHexCode",function(a){return JU.Escape.escapeColor(JU.C.getArgb(a))},"~N");c$.getHexCodes=c(c$,"getHexCodes",function(a){if(null==a)return null;for(var b=new JU.SB,c=0;c<a.length;c++)b.append(0==c?"":" ").append(JU.C.getHexCode(a[c]));return b.toString()},"~A");c$.getColixTranslucent=c(c$,"getColixTranslucent",function(a){var b=a>>24&255;return 255==b?JU.C.getColix(a):JU.C.getColixTranslucent3(JU.C.getColix(a),!0,b/255)},"~N");c$.getBgContrast=
c(c$,"getBgContrast",function(a){return 128>(JU.CU.toFFGGGfromRGB(a)&255)?8:4},"~N");E(c$,"INHERIT_ALL",0,"INHERIT_COLOR",1,"USE_PALETTE",2,"RAW_RGB",3,"SPECIAL_COLIX_MAX",4,"colixMax",4,"argbs",B(128,0),"argbsGreyscale",null);c$.colixHash=c$.prototype.colixHash=new JU.Int2IntHash(256);E(c$,"RAW_RGB_INT",3,"UNMASK_CHANGEABLE_TRANSLUCENT",2047,"CHANGEABLE_MASK",32768,"LAST_AVAILABLE_COLIX",2047,"TRANSLUCENT_SHIFT",11,"ALPHA_SHIFT",13,"TRANSLUCENT_MASK",30720,"TRANSLUCENT_SCREENED",30720,"TRANSPARENT",
16384,"OPAQUE_MASK",-30721,"BLACK",4,"ORANGE",5,"PINK",6,"BLUE",7,"WHITE",8,"CYAN",9,"RED",10,"GREEN",11,"GRAY",12,"SILVER",13,"LIME",14,"MAROON",15,"NAVY",16,"OLIVE",17,"PURPLE",18,"TEAL",19,"MAGENTA",20,"YELLOW",21,"HOTPINK",22,"GOLD",23);for(var a=B(-1,[4278190080,4294944E3,4294951115,4278190335,4294967295,4278255615,4294901760,4278222848,4286611584,4290822336,4278255360,4286578688,4278190208,4286611456,4286578816,4278222976,4294902015,4294967040,4294928820,4294956800]),b=0;b<a.length;++b)JU.C.getColix(a[b])});
s("JU");v(null,"JU.ColorEncoder","java.lang.Boolean $.Float java.util.Hashtable JU.AU $.CU $.Lst $.PT J.c.PAL JU.C $.Escape $.Logger JV.JC".split(" "),function(){c$=t(function(){this.argbsNucleic=this.argbsAmino=this.argbsShapely=this.argbsRwb=this.argbsRoygb=this.argbsCpk=this.paletteFriendly=this.paletteWB=this.paletteBW=this.vwr=null;this.ihalf=0;this.schemes=null;this.currentPalette=0;this.currentSegmentCount=1;this.isTranslucent=!1;this.hi=this.lo=0;this.isReversed=!1;this.thisScale=this.userScale=
null;this.thisName="scheme";this.isColorIndex=!1;this.ce=null;p(this,arguments)},JU,"ColorEncoder");O(c$,function(){this.userScale=B(-1,[-8355712]);this.thisScale=B(-1,[-8355712])});q(c$,function(a,b){null==a?(this.vwr=b,this.schemes=new java.util.Hashtable,this.argbsCpk=J.c.PAL.argbsCpk,this.argbsRoygb=JV.JC.argbsRoygbScale,this.argbsRwb=JV.JC.argbsRwbScale,this.argbsAmino=this.argbsNucleic=this.argbsShapely=null,this.ihalf=w(JV.JC.argbsRoygbScale.length/3),this.ce=this):(this.ce=a,this.vwr=a.vwr,
this.schemes=a.schemes)},"JU.ColorEncoder,JV.Viewer");c$.getSchemeIndex=c(c$,"getSchemeIndex",function(a){for(var b=0;b<JU.ColorEncoder.colorSchemes.length;b++)if(JU.ColorEncoder.colorSchemes[b].equalsIgnoreCase(a))return 16<=b?b-16:13>b?b:-b;return-1},"~S");c$.fixName=c(c$,"fixName",function(a){if(a.equalsIgnoreCase("byelement"))return"byelement_jmol";var b=JU.ColorEncoder.getSchemeIndex(a);return 0<=b?JU.ColorEncoder.colorSchemes[b]:a.toLowerCase()},"~S");c(c$,"makeColorScheme",function(a,b,d){a=
JU.ColorEncoder.fixName(a);if(null==b){this.schemes.remove(a);a=this.createColorScheme(a,!1,d);if(d)switch(a){case 2147483647:return 0;case 12:this.paletteFriendly=this.getPaletteAC();break;case 10:this.paletteBW=this.getPaletteBW();break;case 11:this.paletteWB=this.getPaletteWB();break;case 0:case 1:this.argbsRoygb=JV.JC.argbsRoygbScale;break;case 6:case 7:this.argbsRwb=JV.JC.argbsRwbScale;break;case 2:this.argbsCpk=J.c.PAL.argbsCpk;break;case 3:JU.ColorEncoder.getRasmolScale();break;case 17:this.getNucleic();
break;case 5:this.getAmino();break;case 4:this.getShapely()}return a}this.schemes.put(a,b);this.setThisScheme(a,b);a=this.createColorScheme(a,!1,d);if(d)switch(a){case 10:this.paletteBW=this.thisScale;break;case 11:this.paletteWB=this.thisScale;break;case 0:case 1:this.argbsRoygb=this.thisScale;this.ihalf=w(this.argbsRoygb.length/3);break;case 6:case 7:this.argbsRwb=this.thisScale;break;case 2:this.argbsCpk=this.thisScale;break;case 5:this.argbsAmino=this.thisScale;break;case 17:this.argbsNucleic=
this.thisScale;break;case 4:this.argbsShapely=this.thisScale}return-1},"~S,~A,~B");c(c$,"getShapely",function(){return null==this.argbsShapely?this.argbsShapely=this.vwr.getJBR().getArgbs(1073742144):this.argbsShapely});c(c$,"getAmino",function(){return null==this.argbsAmino?this.argbsAmino=this.vwr.getJBR().getArgbs(2097154):this.argbsAmino});c(c$,"getNucleic",function(){return null==this.argbsNucleic?this.argbsNucleic=this.vwr.getJBR().getArgbs(2097166):this.argbsNucleic});c(c$,"createColorScheme",
function(a,b,d){a=a.toLowerCase();if(a.equals("inherit"))return 15;var c=Math.max(a.indexOf("="),a.indexOf("["));if(0<=c){b=JU.PT.replaceAllCharacters(a.substring(0,c)," =","");0<b.length&&(d=!0);var f=0;a.length>c+1&&!a.contains("[")&&(a="["+a.substring(c+1).trim()+"]",a=JU.PT.rep(a.$replace("\n"," "),"  "," "),a=JU.PT.rep(a,", ",",").$replace(" ",","),a=JU.PT.rep(a,",","]["));for(c=-1;0<=(c=a.indexOf("[",c+1));)f++;if(0==f)return this.makeColorScheme(b,null,d);for(var e=B(f,0),f=0;0<=(c=a.indexOf("[",
c+1));){var h=a.indexOf("]",c);0>h&&(h=a.length-1);var j=JU.CU.getArgbFromString(a.substring(c,h+1));0==j&&(j=JU.CU.getArgbFromString(a.substring(c+1,h).trim()));if(0==j)return JU.Logger.error("error in color value: "+a.substring(c,h+1)),0;e[f++]=j}return b.equals("user")?(this.setUserScale(e),-13):this.makeColorScheme(b,e,d)}a=JU.ColorEncoder.fixName(a);d=JU.ColorEncoder.getSchemeIndex(a);return this.schemes.containsKey(a)?(this.setThisScheme(a,this.schemes.get(a)),d):-1!=d?d:b?0:2147483647},"~S,~B,~B");
c(c$,"setUserScale",function(a){this.ce.userScale=a;this.makeColorScheme("user",a,!1)},"~A");c(c$,"getColorSchemeArray",function(a){switch(a){case -1:return this.thisScale;case 0:return this.ce.argbsRoygb;case 1:return JU.AU.arrayCopyRangeRevI(this.ce.argbsRoygb,0,-1);case 8:return JU.AU.arrayCopyRangeI(this.ce.argbsRoygb,0,this.ce.ihalf);case 9:var b=JU.AU.arrayCopyRangeI(this.ce.argbsRoygb,this.ce.argbsRoygb.length-2*this.ce.ihalf,-1);a=B(this.ce.ihalf,0);for(var d=a.length,c=b.length;0<=--d&&0<=
--c;)a[d]=b[c--];return a;case 12:return this.getPaletteAC();case 10:return this.getPaletteBW();case 11:return this.getPaletteWB();case 6:return this.ce.argbsRwb;case 7:return JU.AU.arrayCopyRangeRevI(this.ce.argbsRwb,0,-1);case 2:return this.ce.argbsCpk;case 3:return JU.ColorEncoder.getRasmolScale();case 4:return this.ce.getShapely();case 17:return this.ce.getNucleic();case 5:return this.ce.getAmino();case -13:return this.ce.userScale;case -14:return JU.AU.arrayCopyRangeRevI(this.ce.userScale,0,
-1);default:return null}},"~N");c(c$,"getColorIndexFromPalette",function(a,b,d,c,f){c=JU.C.getColix(this.getArgbFromPalette(a,b,d,c));f&&(a=(d-a)/(d-b),1<a?a=1:0.125>a&&(a=0.125),c=JU.C.getColixTranslucent3(c,!0,a));return c},"~N,~N,~N,~N,~B");c(c$,"getPaletteColorCount",function(a){switch(a){case -1:return this.thisScale.length;case 10:case 11:return this.getPaletteBW().length;case 0:case 1:return this.ce.argbsRoygb.length;case 8:case 9:return this.ce.ihalf;case 6:case 7:return this.ce.argbsRwb.length;
case -13:case -14:return this.ce.userScale.length;case 2:return this.ce.argbsCpk.length;case 3:return JU.ColorEncoder.getRasmolScale().length;case 4:return this.ce.getShapely().length;case 17:return this.ce.getNucleic().length;case 5:return this.ce.getAmino().length;case 12:return this.getPaletteAC().length;default:return 0}},"~N");c(c$,"getArgbFromPalette",function(a,b,d,c){if(Float.isNaN(a))return-8355712;var f=this.getPaletteColorCount(c);switch(c){case -1:return this.isColorIndex&&(b=0,d=this.thisScale.length),
this.thisScale[JU.ColorEncoder.quantize4(a,b,d,f)];case 10:return this.getPaletteBW()[JU.ColorEncoder.quantize4(a,b,d,f)];case 11:return this.getPaletteWB()[JU.ColorEncoder.quantize4(a,b,d,f)];case 0:return this.ce.argbsRoygb[JU.ColorEncoder.quantize4(a,b,d,f)];case 1:return this.ce.argbsRoygb[JU.ColorEncoder.quantize4(-a,-d,-b,f)];case 8:return this.ce.argbsRoygb[JU.ColorEncoder.quantize4(a,b,d,f)];case 9:return this.ce.argbsRoygb[this.ce.ihalf+2*JU.ColorEncoder.quantize4(a,b,d,f)];case 6:return this.ce.argbsRwb[JU.ColorEncoder.quantize4(a,
b,d,f)];case 7:return this.ce.argbsRwb[JU.ColorEncoder.quantize4(-a,-d,-b,f)];case -13:return 0==this.ce.userScale.length?-8355712:this.ce.userScale[JU.ColorEncoder.quantize4(a,b,d,f)];case -14:return 0==this.ce.userScale.length?-8355712:this.ce.userScale[JU.ColorEncoder.quantize4(-a,-d,-b,f)];case 2:return this.ce.argbsCpk[JU.ColorEncoder.colorIndex(a,f)];case 3:return JU.ColorEncoder.getRasmolScale()[JU.ColorEncoder.colorIndex(a,f)];case 4:return this.ce.getShapely()[JU.ColorEncoder.colorIndex(a,
f)];case 5:return this.ce.getAmino()[JU.ColorEncoder.colorIndex(a,f)];case 17:return this.ce.getNucleic()[JU.ColorEncoder.colorIndex(a-24+2,f)];case 12:return this.getPaletteAC()[JU.ColorEncoder.colorIndexRepeat(a,f)];default:return-8355712}},"~N,~N,~N,~N");c(c$,"setThisScheme",function(a,b){this.thisName=a;this.thisScale=b;a.equals("user")&&(this.userScale=b);this.isColorIndex=0==a.indexOf("byelement")||0==a.indexOf("byresidue")},"~S,~A");c(c$,"getArgb",function(a){return this.isReversed?this.getArgbFromPalette(-a,
-this.hi,-this.lo,this.currentPalette):this.getArgbFromPalette(a,this.lo,this.hi,this.currentPalette)},"~N");c(c$,"getArgbMinMax",function(a,b,d){return this.isReversed?this.getArgbFromPalette(-a,-d,-b,this.currentPalette):this.getArgbFromPalette(a,b,d,this.currentPalette)},"~N,~N,~N");c(c$,"getColorIndex",function(a){return this.isReversed?this.getColorIndexFromPalette(-a,-this.hi,-this.lo,this.currentPalette,this.isTranslucent):this.getColorIndexFromPalette(a,this.lo,this.hi,this.currentPalette,
this.isTranslucent)},"~N");c(c$,"getColorKey",function(){for(var a=new java.util.Hashtable,b=this.getPaletteColorCount(this.currentPalette),d=new JU.Lst,c=K(b+1,0),f=(this.hi-this.lo)/b,e=f*(this.isReversed?-0.5:0.5),h=0;h<b;h++)c[h]=this.isReversed?this.hi-h*f:this.lo+h*f,d.addLast(JU.CU.colorPtFromInt(this.getArgb(c[h]+e),null));c[b]=this.isReversed?this.lo:this.hi;a.put("values",c);a.put("colors",d);a.put("min",Float.$valueOf(this.lo));a.put("max",Float.$valueOf(this.hi));a.put("reversed",Boolean.$valueOf(this.isReversed));
a.put("name",this.getCurrentColorSchemeName());return a});c(c$,"getColorScheme",function(){return(this.isTranslucent?"translucent ":"")+(0>this.currentPalette?JU.ColorEncoder.getColorSchemeList(this.getColorSchemeArray(this.currentPalette)):this.getColorSchemeName(this.currentPalette))});c(c$,"setColorScheme",function(a,b){this.isTranslucent=b;null!=a&&(this.currentPalette=this.createColorScheme(a,!0,!1))},"~S,~B");c(c$,"setRange",function(a,b,d){3.4028235E38==b&&(a=1,b=this.getPaletteColorCount(this.currentPalette)+
1);this.lo=Math.min(a,b);this.hi=Math.max(a,b);this.isReversed=d},"~N,~N,~B");c(c$,"getCurrentColorSchemeName",function(){return this.getColorSchemeName(this.currentPalette)});c(c$,"getColorSchemeName",function(a){var b=Math.abs(a);return-1==a?this.thisName:b<JU.ColorEncoder.colorSchemes.length&&0<=b?JU.ColorEncoder.colorSchemes[b]:null},"~N");c$.getColorSchemeList=c(c$,"getColorSchemeList",function(a){if(null==a)return"";for(var b="",d=0;d<a.length;d++)b+=(0==d?"":" ")+JU.Escape.escapeColor(a[d]);
return b},"~A");c$.getRasmolScale=c(c$,"getRasmolScale",function(){if(null!=JU.ColorEncoder.rasmolScale)return JU.ColorEncoder.rasmolScale;JU.ColorEncoder.rasmolScale=B(J.c.PAL.argbsCpk.length,0);for(var a=J.c.PAL.argbsCpkRasmol[0]|4278190080,b=JU.ColorEncoder.rasmolScale.length;0<=--b;)JU.ColorEncoder.rasmolScale[b]=a;for(b=J.c.PAL.argbsCpkRasmol.length;0<=--b;)a=J.c.PAL.argbsCpkRasmol[b],JU.ColorEncoder.rasmolScale[a>>24]=a|4278190080;return JU.ColorEncoder.rasmolScale});c(c$,"getPaletteAC",function(){return null==
this.ce.paletteFriendly?this.ce.paletteFriendly=B(-1,[8421504,1067945,11141282,13235712,16753152,2640510,8331387,10467374,12553008,339310,7209065,8626176,10906112,4488148,13907405,14219839,16759360,6984660,13918415,14809713,16764019]):this.ce.paletteFriendly});c(c$,"getPaletteWB",function(){if(null!=this.ce.paletteWB)return this.ce.paletteWB;for(var a=B(JV.JC.argbsRoygbScale.length,0),b=0;b<a.length;b++){var d=1/a.length*(a.length-b);a[b]=JU.CU.colorTriadToFFRGB(d,d,d)}return this.ce.paletteWB=a});
c$.getPaletteAtoB=c(c$,"getPaletteAtoB",function(a,b,d){2>d&&(d=JV.JC.argbsRoygbScale.length);var c=B(d,0),f=K(3,0),e=K(3,0);JU.CU.toRGB3f(a,f);JU.CU.toRGB3f(b,e);a=(e[0]-f[0])/(d-1);b=(e[1]-f[1])/(d-1);for(var e=(e[2]-f[2])/(d-1),h=0;h<d;h++)c[h]=JU.CU.colorTriadToFFRGB(f[0]+a*h,f[1]+b*h,f[2]+e*h);return c},"~N,~N,~N");c(c$,"getPaletteBW",function(){if(null!=this.ce.paletteBW)return this.ce.paletteBW;for(var a=B(JV.JC.argbsRoygbScale.length,0),b=0;b<a.length;b++){var d=1/a.length*b;a[b]=JU.CU.colorTriadToFFRGB(d,
d,d)}return this.ce.paletteBW=a});c(c$,"quantize",function(a,b){var d=this.getPaletteColorCount(this.currentPalette);a=(A(a*d)+(b?0:1))/d;return 0>=a?this.lo:1<=a?this.hi:this.lo+(this.hi-this.lo)*a},"~N,~B");c$.quantize4=c(c$,"quantize4",function(a,b,d,c){d-=b;if(0>=d||Float.isNaN(a))return w(c/2);a-=b;if(0>=a)return 0;a=A(a/(d/c)+1E-4);a>=c&&(a=c-1);return a},"~N,~N,~N,~N");c$.colorIndex=c(c$,"colorIndex",function(a,b){return w(Math.floor(0>=a||a>=b?0:a))},"~N,~N");c$.colorIndexRepeat=c(c$,"colorIndexRepeat",
function(a,b){return w(Math.floor(0>=a?0:a))%b},"~N,~N");E(c$,"GRAY",4286611584,"BYELEMENT_PREFIX","byelement","BYRESIDUE_PREFIX","byresidue");c$.BYELEMENT_JMOL=c$.prototype.BYELEMENT_JMOL="byelement_jmol";c$.BYELEMENT_RASMOL=c$.prototype.BYELEMENT_RASMOL="byelement_rasmol";c$.BYRESIDUE_SHAPELY=c$.prototype.BYRESIDUE_SHAPELY="byresidue_shapely";c$.BYRESIDUE_AMINO=c$.prototype.BYRESIDUE_AMINO="byresidue_amino";c$.BYRESIDUE_NUCLEIC=c$.prototype.BYRESIDUE_NUCLEIC="byresidue_nucleic";E(c$,"CUSTOM",-1,
"ROYGB",0,"BGYOR",1,"JMOL",2,"RASMOL",3,"SHAPELY",4,"AMINO",5,"RWB",6,"BWR",7,"LOW",8,"HIGH",9,"BW",10,"WB",11,"FRIENDLY",12,"USER",-13,"RESU",-14,"INHERIT",15,"ALT",16,"NUCLEIC",17);c$.colorSchemes=c$.prototype.colorSchemes=u(-1,"roygb bgyor byelement_jmol byelement_rasmol byresidue_shapely byresidue_amino rwb bwr low high bw wb friendly user resu inherit rgb bgr jmol rasmol byresidue byresidue_nucleic".split(" "));E(c$,"rasmolScale",null,"argbsChainAtom",null,"argbsChainHetero",null)});s("JU");
v(null,"JU.CommandHistory",["JU.Lst"],function(){c$=t(function(){this.commandList=null;this.maxSize=100;this.cursorPos=this.nextCommand=0;this.isOn=!0;this.lstStates=null;p(this,arguments)},JU,"CommandHistory");q(c$,function(){this.reset(100)});c(c$,"clear",function(){this.reset(this.maxSize)});c(c$,"reset",function(a){this.maxSize=a;this.commandList=new JU.Lst;this.nextCommand=0;this.commandList.addLast("");this.cursorPos=0},"~N");c(c$,"setMaxSize",function(a){if(a!=this.maxSize){for(2>a&&(a=2);this.nextCommand>
a;)this.commandList.removeItemAt(0),this.nextCommand--;this.nextCommand>a&&(this.nextCommand=a-1);this.cursorPos=this.nextCommand;this.maxSize=a}},"~N");c(c$,"getCommandUp",function(){if(0>=this.cursorPos)return null;this.cursorPos--;var a=this.getCommand();a.endsWith("#??")&&this.removeCommand(this.cursorPos--);0>this.cursorPos&&(this.cursorPos=0);return a});c(c$,"getCommandDown",function(){if(this.cursorPos>=this.nextCommand)return null;this.cursorPos++;return this.getCommand()});c(c$,"getCommand",
function(){return this.commandList.get(this.cursorPos)});c(c$,"addCommand",function(a){if((this.isOn||a.endsWith("#??"))&&!a.endsWith("#----")){for(var b;0<=(b=a.indexOf("\n"));){var d=a.substring(0,b);0<d.length&&this.addCommandLine(d);a=a.substring(b+1)}0<a.length&&this.addCommandLine(a)}},"~S");c(c$,"getSetHistory",function(a){this.isOn=-2==a?this.isOn:!0;switch(a){case 0:return this.isOn=!1,this.clear(),"";case -2147483648:case -2:return this.clear(),"";case -1:return this.getCommandUp();case 1:return this.getCommandDown();
default:if(0>a)return this.setMaxSize(-2-a),"";a=Math.max(this.nextCommand-a,0)}for(var b="";a<this.nextCommand;a++)this.commandList.get(a).toUpperCase().startsWith("WRITE HISTORY")||(b+=this.commandList.get(a)+"\n");return b},"~N");c(c$,"find",function(a,b){for(var d=this.cursorPos,c=a;null!=c;){c=this.getSetHistory(b);if(null==c)break;if(c.startsWith(a))return c}this.cursorPos=d;return null},"~S,~N");c(c$,"removeCommand",function(){return this.removeCommand(this.nextCommand-1)});c(c$,"removeCommand",
function(a){if(0>a||a>=this.nextCommand)return"";a=this.commandList.removeItemAt(a);this.nextCommand--;return a},"~N");c(c$,"addCommandLine",function(a){if(!(null==a||0==a.length)&&!a.endsWith("#--"))this.nextCommand>=this.maxSize&&(this.commandList.removeItemAt(0),this.nextCommand=this.maxSize-1),this.commandList.add(this.nextCommand,a),this.nextCommand++,this.cursorPos=this.nextCommand,this.commandList.add(this.nextCommand,"")},"~S");c(c$,"pushState",function(a){null==this.lstStates&&(this.lstStates=
new JU.Lst);this.lstStates.addLast(a)},"~S");c(c$,"popState",function(){return null==this.lstStates||0==this.lstStates.size()?null:this.lstStates.removeItemAt(this.lstStates.size()-1)});E(c$,"ERROR_FLAG","#??","NOHISTORYLINE_FLAG","#--","NOHISTORYATALL_FLAG","#----","DEFAULT_MAX_SIZE",100)});s("JU");v(["JU.LoggerInterface"],"JU.DefaultLogger",["JU.Logger"],function(){c$=D(JU,"DefaultLogger",null,JU.LoggerInterface);c(c$,"log",function(a,b,d,c){a===Zystem.err&&Zystem.out.flush();if(null!=a&&(null!=
d||null!=c))if(a.println((JU.Logger.logLevel()?"["+JU.Logger.getLevel(b)+"] ":"")+(null!=d?d:"")+(null!=c?": "+c.toString():"")),null!=c&&(b=c.getStackTrace(),null!=b))for(d=0;d<b.length;d++)a.println(b[d].getClassName()+" - "+b[d].getLineNumber()+" - "+b[d].getMethodName());a===Zystem.err&&Zystem.err.flush()},"java.io.PrintStream,~N,~S,Throwable");j(c$,"debug",function(a){this.log(Zystem.out,5,a,null)},"~S");j(c$,"info",function(a){this.log(Zystem.out,4,a,null)},"~S");j(c$,"warn",function(a){this.log(Zystem.out,
3,a,null)},"~S");j(c$,"warnEx",function(a,b){this.log(Zystem.out,3,a,b)},"~S,Throwable");j(c$,"error",function(a){this.log(Zystem.err,2,a,null)},"~S");j(c$,"errorEx",function(a,b){this.log(Zystem.err,2,a,b)},"~S,Throwable");j(c$,"fatal",function(a){this.log(Zystem.err,1,a,null)},"~S");j(c$,"fatalEx",function(a,b){this.log(Zystem.err,1,a,b)},"~S,Throwable")});s("JU");v(["JU.BS"],"JU.Elements",["java.util.Hashtable","JU.PT","JU.Logger"],function(){c$=D(JU,"Elements");c$.getNaturalIsotope=c(c$,"getNaturalIsotope",
function(a){return JU.Elements.isotopeMass[a&127]},"~N");c$.getAtomicMass=c(c$,"getAtomicMass",function(a){return 1>a||a>=JU.Elements.atomicMass.length?0:JU.Elements.atomicMass[a]},"~N");c$.elementNumberFromSymbol=c(c$,"elementNumberFromSymbol",function(a,d){if(null==JU.Elements.htElementMap){for(var c=new java.util.Hashtable,f=JU.Elements.elementNumberMax;0<=--f;){var e=JU.Elements.elementSymbols[f],h=Integer.$valueOf(f);c.put(e,h);2==e.length&&c.put(e.toUpperCase(),h)}for(f=JU.Elements.altElementMax;4<=
--f;)e=JU.Elements.altElementSymbols[f],h=Integer.$valueOf(JU.Elements.altElementNumbers[f]),c.put(e,h),2==e.length&&c.put(e.toUpperCase(),h);c.put("Z",Integer.$valueOf(0));JU.Elements.htElementMap=c}if(null==a)return 0;c=JU.Elements.htElementMap.get(a);if(null!=c)return c.intValue();if(JU.PT.isDigit(a.charAt(0))&&(e=a.length-2,0<=e&&JU.PT.isDigit(a.charAt(e))&&e++,c=0<e?JU.PT.parseInt(a.substring(0,e)):0,0<c&&(e=JU.Elements.elementNumberFromSymbol(a.substring(e),!0),0<e)))return c=JU.Elements.getAtomicAndIsotopeNumber(e,
c),JU.Elements.htElementMap.put(a.toUpperCase(),Integer.$valueOf(c)),c;d||JU.Logger.error("'"+a+"' is not a recognized symbol");return 0},"~S,~B");c$.elementSymbolFromNumber=c(c$,"elementSymbolFromNumber",function(a){var d=0;if(a>=JU.Elements.elementNumberMax){for(d=JU.Elements.altElementMax;0<=--d;)if(a==JU.Elements.altElementNumbers[d])return JU.Elements.altElementSymbols[d];d=JU.Elements.getIsotopeNumber(a);return""+d+JU.Elements.getElementSymbol(a%128)}return JU.Elements.getElementSymbol(a)},
"~N");c$.getElementSymbol=c(c$,"getElementSymbol",function(a){if(0>a||a>=JU.Elements.elementNumberMax)a=0;return JU.Elements.elementSymbols[a]},"~N");c$.elementNameFromNumber=c(c$,"elementNameFromNumber",function(a){if(a>=JU.Elements.elementNumberMax){for(var d=JU.Elements.altElementMax;0<=--d;)if(a==JU.Elements.altElementNumbers[d])return JU.Elements.altElementNames[d];a%=128}if(0>a||a>=JU.Elements.elementNumberMax)a=0;return JU.Elements.elementNames[a]},"~N");c$.elementNumberFromName=c(c$,"elementNumberFromName",
function(a){for(var d=1;d<JU.Elements.elementNumberMax;d++)if(JU.Elements.elementNames[d].equalsIgnoreCase(a))return d;return-1},"~S");c$.altElementNameFromIndex=c(c$,"altElementNameFromIndex",function(a){return JU.Elements.altElementNames[a]},"~N");c$.altElementNumberFromIndex=c(c$,"altElementNumberFromIndex",function(a){return JU.Elements.altElementNumbers[a]},"~N");c$.altElementSymbolFromIndex=c(c$,"altElementSymbolFromIndex",function(a){return JU.Elements.altElementSymbols[a]},"~N");c$.altIsotopeSymbolFromIndex=
c(c$,"altIsotopeSymbolFromIndex",function(a){a=JU.Elements.altElementNumbers[a];return(a>>7)+JU.Elements.elementSymbolFromNumber(a&127)},"~N");c$.altIsotopeSymbolFromIndex2=c(c$,"altIsotopeSymbolFromIndex2",function(a){a=JU.Elements.altElementNumbers[a];return JU.Elements.elementSymbolFromNumber(a&127)+(a>>7)},"~N");c$.getElementNumber=c(c$,"getElementNumber",function(a){return a&127},"~N");c$.getIsotopeNumber=c(c$,"getIsotopeNumber",function(a){return a>>7},"~N");c$.getAtomicAndIsotopeNumber=c(c$,
"getAtomicAndIsotopeNumber",function(a,d){return(0>a?0:a)+(0>=d?0:d<<7)},"~N,~N");c$.altElementIndexFromNumber=c(c$,"altElementIndexFromNumber",function(a){for(var d=0;d<JU.Elements.altElementMax;d++)if(JU.Elements.altElementNumbers[d]==a)return d;return 0},"~N");c$.isNaturalIsotope=c(c$,"isNaturalIsotope",function(a){return 0<="1H,12C,14N".indexOf(a+",")},"~S");c$.getBondingRadius=c(c$,"getBondingRadius",function(a,d){var c=a&127;return 0<d&&JU.Elements.bsCations.get(c)?JU.Elements.getBondingRadFromTable(c,
d,JU.Elements.cationLookupTable):0>d&&JU.Elements.bsAnions.get(c)?JU.Elements.getBondingRadFromTable(c,d,JU.Elements.anionLookupTable):JU.Elements.defaultBondingMars[(c<<1)+JU.Elements.bondingVersion]/1E3},"~N,~N");c$.getCovalentRadius=c(c$,"getCovalentRadius",function(a){return JU.Elements.defaultBondingMars[((a&127)<<1)+JU.Elements.covalentVersion]/1E3},"~N");c$.getBondingRadFromTable=c(c$,"getBondingRadFromTable",function(a,d,c){d=(a<<4)+(d+4);for(var f=0,e=0,h=0,j=w(c.length/2);h!=j;)if(e=w((h+
j)/2),f=c[e<<1],f>d)j=e;else if(f<d)h=e+1;else return c[(e<<1)+1]/1E3;f>d&&e--;f=c[e<<1];a!=f>>4&&e++;return c[(e<<1)+1]/1E3},"~N,~N,~A");c$.getVanderwaalsMar=c(c$,"getVanderwaalsMar",function(a,d){return JU.Elements.vanderwaalsMars[((a&127)<<2)+d.pt%4]},"~N,J.c.VDW");c$.getHydrophobicity=c(c$,"getHydrophobicity",function(a){return 1>a||a>=JU.Elements.hydrophobicities.length?0:JU.Elements.hydrophobicities[a]},"~N");c$.getAllredRochowElectroNeg=c(c$,"getAllredRochowElectroNeg",function(a){return 0<
a&&a<JU.Elements.electroNegativities.length?JU.Elements.electroNegativities[a]:0},"~N");c$.isElement=c(c$,"isElement",function(a,d){return(a&127)==d},"~N,~N");E(c$,"elementSymbols",u(-1,"Xx H He Li Be B C N O F Ne Na Mg Al Si P S Cl Ar K Ca Sc Ti V Cr Mn Fe Co Ni Cu Zn Ga Ge As Se Br Kr Rb Sr Y Zr Nb Mo Tc Ru Rh Pd Ag Cd In Sn Sb Te I Xe Cs Ba La Ce Pr Nd Pm Sm Eu Gd Tb Dy Ho Er Tm Yb Lu Hf Ta W Re Os Ir Pt Au Hg Tl Pb Bi Po At Rn Fr Ra Ac Th Pa U Np Pu Am Cm Bk Cf Es Fm Md No Lr Rf Db Sg Bh Hs Mt".split(" ")),
"atomicMass",K(-1,[0,1.008,4.002,6.9675,9.012,10.8135,12.0106,14.006,15.999,18.998,20.1797,22.989,24.307,26.981,28.084,30.973,32.059,35.4515,39.948,39.0983,40.078,44.955,47.867,50.9415,51.9961,54.938,55.845,58.933,58.6934,63.546,65.38,69.723,72.63,74.921,78.971,79.904,83.798,85.4678,87.62,88.905,91.224,92.906,95.95,98.91,101.07,102.905,106.42,107.8682,112.414,114.818,118.71,121.76,127.6,126.904,131.293,132.905,137.327,138.905,140.116,140.907,144.242,144.9,150.36,151.964,157.25,158.925,162.5,164.93,
167.259,168.934,173.054,174.9668,178.49,180.947,183.84,186.207,190.23,192.217,195.084,196.966,200.592,204.3835,207.2,208.98,210,210,222,223,226.03,227.03,232.0377,231.035,238.028,237.05,239.1,243.1,247.1,247.1,252.1,252.1,257.1,256.1,259.1,260.1,261,262,263,262,265,268]),"isotopeMass",B(-1,[0,1,4,7,9,11,12,14,16,19,20,23,24,27,28,31,32,35,40,39,40,45,48,51,52,55,56,59,59,64,65,70,73,75,79,80,84,85,88,89,91,93,96,98,101,103,106,108,112,115,119,122,128,127,131,133,137,139,140,141,144,145,150,152,157,
159,163,165,167,169,173,175,179,181,184,186,190,192,195,197,201,204,207,209,209,210,222,223,226,227,232,231,238,237,244,243,247,247,251,252,257,258,259,260,261,262,263,262,265,268]));c$.elementNumberMax=c$.prototype.elementNumberMax=JU.Elements.elementSymbols.length;E(c$,"htElementMap",null,"elementNames",u(-1,"unknown hydrogen helium lithium beryllium boron carbon nitrogen oxygen fluorine neon sodium magnesium aluminum silicon phosphorus sulfur chlorine argon potassium calcium scandium titanium vanadium chromium manganese iron cobalt nickel copper zinc gallium germanium arsenic selenium bromine krypton rubidium strontium yttrium zirconium niobium molybdenum technetium ruthenium rhodium palladium silver cadmium indium tin antimony tellurium iodine xenon cesium barium lanthanum cerium praseodymium neodymium promethium samarium europium gadolinium terbium dysprosium holmium erbium thulium ytterbium lutetium hafnium tantalum tungsten rhenium osmium iridium platinum gold mercury thallium lead bismuth polonium astatine radon francium radium actinium thorium protactinium uranium neptunium plutonium americium curium berkelium californium einsteinium fermium mendelevium nobelium lawrencium rutherfordium dubnium seaborgium bohrium hassium meitnerium".split(" ")),
"naturalIsotopes","1H,12C,14N","firstIsotope",4,"altElementNumbers",B(-1,[0,13,16,55,257,385,1414,1670,1798,1927]));c$.altElementMax=c$.prototype.altElementMax=JU.Elements.altElementNumbers.length;E(c$,"altElementSymbols",u(-1,"Xx Al S Cs D T 11C 13C 14C 15N".split(" ")),"altElementNames",u(-1,"dummy aluminium sulphur caesium deuterium tritium    ".split(" ")),"VdwPROBE","#VDW radii for PROBE;{_H}.vdw = 1.0;{_H and connected(_C) and not connected(within(smiles,'[a]'))}.vdw = 1.17;{_C}.vdw = 1.75;{_C and connected(3) and connected(_O)}.vdw = 1.65;{_N}.vdw = 1.55;{_O}.vdw = 1.4;{_P}.vdw = 1.8;{_S}.vdw = 1.8;message VDW radii for H, C, N, O, P, and S set according to Word, et al., J. Mol. Biol. (1999) 285, 1711-1733",
"vanderwaalsMars",T(-1,[1E3,1E3,1E3,1E3,1200,1100,1100,1200,1400,1400,2200,1400,1820,1810,1220,2200,1700,1530,628,1900,2080,1920,1548,1800,1950,1700,1548,1700,1850,1550,1400,1600,1700,1520,1348,1550,1730,1470,1300,1500,1540,1540,2020,1540,2270,2270,2200,2400,1730,1730,1500,2200,2050,1840,1500,2100,2100,2100,2200,2100,2080,1800,1880,1950,2E3,1800,1808,1800,1970,1750,1748,1800,1880,1880,2768,1880,2750,2750,2388,2800,1973,2310,1948,2400,1700,2300,1320,2300,1700,2150,1948,2150,1700,2050,1060,2050,1700,
2050,1128,2050,1700,2050,1188,2050,1700,2050,1948,2050,1700,2E3,1128,2E3,1630,2E3,1240,2E3,1400,2E3,1148,2E3,1390,2100,1148,2100,1870,1870,1548,2100,1700,2110,3996,2100,1850,1850,828,2050,1900,1900,900,1900,2100,1830,1748,1900,2020,2020,1900,2020,1700,3030,2648,2900,1700,2490,2020,2550,1700,2400,1608,2400,1700,2300,1420,2300,1700,2150,1328,2150,1700,2100,1748,2100,1700,2050,1800,2050,1700,2050,1200,2050,1700,2E3,1220,2E3,1630,2050,1440,2050,1720,2100,1548,2100,1580,2200,1748,2200,1930,2200,1448,2200,
2170,1930,1668,2250,2200,2170,1120,2200,2060,2060,1260,2100,2150,1980,1748,2100,2160,2160,2100,2160,1700,3430,3008,3E3,1700,2680,2408,2700,1700,2500,1828,2500,1700,2480,1860,2480,1700,2470,1620,2470,1700,2450,1788,2450,1700,2430,1760,2430,1700,2420,1740,2420,1700,2400,1960,2400,1700,2380,1688,2380,1700,2370,1660,2370,1700,2350,1628,2350,1700,2330,1608,2330,1700,2320,1588,2320,1700,2300,1568,2300,1700,2280,1540,2280,1700,2270,1528,2270,1700,2250,1400,2250,1700,2200,1220,2200,1700,2100,1260,2100,1700,
2050,1300,2050,1700,2E3,1580,2E3,1700,2E3,1220,2E3,1720,2050,1548,2050,1660,2100,1448,2100,1550,2050,1980,2050,1960,1960,1708,2200,2020,2020,2160,2300,1700,2070,1728,2300,1700,1970,1208,2E3,1700,2020,1120,2E3,1700,2200,2300,2E3,1700,3480,3240,2E3,1700,2830,2568,2E3,1700,2E3,2120,2E3,1700,2400,1840,2400,1700,2E3,1600,2E3,1860,2300,1748,2300,1700,2E3,1708,2E3,1700,2E3,1668,2E3,1700,2E3,1660,2E3,1700,2E3,1648,2E3,1700,2E3,1640,2E3,1700,2E3,1628,2E3,1700,2E3,1620,2E3,1700,2E3,1608,2E3,1700,2E3,1600,2E3,
1700,2E3,1588,2E3,1700,2E3,1580,2E3,1700,2E3,1600,2E3,1700,2E3,1600,2E3,1700,2E3,1600,2E3,1700,2E3,1600,2E3,1700,2E3,1600,2E3,1700,2E3,1600,2E3]),"RAD_COV_IONIC_OB1_100_1",0,"RAD_COV_BODR_2014_02_22",1,"covalentVersion",1,"bondingVersion",0,"defaultBondingMars",T(-1,[0,0,230,320,930,460,680,1330,350,1020,830,850,680,750,680,710,680,630,640,640,1120,670,970,1550,1100,1390,1350,1260,1200,1160,750,1110,1020,1030,990,990,1570,960,1330,1960,990,1710,1440,1480,1470,1360,1330,1340,1350,1220,1350,1190,1340,
1160,1330,1110,1500,1100,1520,1120,1450,1180,1220,1240,1170,1210,1210,1210,1220,1160,1210,1140,1910,1170,1470,2100,1120,1850,1780,1630,1560,1540,1480,1470,1470,1380,1350,1280,1400,1250,1450,1250,1500,1200,1590,1280,1690,1360,1630,1420,1460,1400,1460,1400,1470,1360,1400,1330,1980,1310,1670,2320,1340,1960,1870,1800,1830,1630,1820,1760,1810,1740,1800,1730,1800,1720,1990,1680,1790,1690,1760,1680,1750,1670,1740,1660,1730,1650,1720,1640,1940,1700,1720,1620,1570,1520,1430,1460,1370,1370,1350,1310,1370,1290,
1320,1220,1500,1230,1500,1240,1700,1330,1550,1440,1540,1440,1540,1510,1680,1450,1700,1470,2400,1420,2E3,2230,1900,2010,1880,1860,1790,1750,1610,1690,1580,1700,1550,1710,1530,1720,1510,1660,1500,1660,1500,1680,1500,1680,1500,1650,1500,1670,1500,1730,1500,1760,1500,1610,1600,1570,1600,1490,1600,1430,1600,1410,1600,1340,1600,1290]),"FORMAL_CHARGE_MIN",-4,"FORMAL_CHARGE_MAX",7,"cationLookupTable",T(-1,[53,680,69,440,70,350,85,350,87,230,104,160,117,680,119,160,121,130,133,220,138,90,155,80,165,1120,181,
970,197,820,198,660,215,510,229,650,232,420,247,440,249,350,262,2190,264,370,266,300,281,340,283,270,293,1540,309,1330,325,1180,326,990,343,732,357,960,358,940,359,760,360,680,374,880,375,740,376,630,377,590,389,810,390,890,391,630,394,520,406,800,407,660,408,600,411,460,422,740,423,640,438,720,439,630,454,690,469,960,470,720,485,880,486,740,501,810,503,620,518,730,520,530,535,580,537,460,549,660,552,500,554,420,569,470,571,390,597,1470,614,1120,631,893,645,1090,648,790,661,1E3,664,740,665,690,677,
930,680,700,682,620,699,979,712,670,727,680,742,800,744,650,757,1260,758,890,773,1140,774,970,791,810,806,930,808,710,823,760,825,620,837,820,840,700,842,560,857,620,859,500,885,1670,901,1530,902,1340,917,1390,919,1016,933,1270,935,1034,936,920,951,1013,952,900,967,995,983,979,999,964,1014,1090,1015,950,1031,938,1047,923,1048,840,1063,908,1079,894,1095,881,1111,870,1126,930,1127,858,1143,850,1160,780,1177,680,1192,700,1194,620,1208,720,1211,560,1224,880,1226,690,1240,680,1254,800,1256,650,1269,1370,
1271,850,1285,1270,1286,1100,1301,1470,1303,950,1318,1200,1320,840,1333,980,1335,960,1337,740,1354,670,1371,620,1397,1800,1414,1430,1431,1180,1448,1020,1463,1130,1464,980,1465,890,1480,970,1482,800,1495,1100,1496,950,1499,710,1511,1080,1512,930,1527,1070,1528,920]),"anionLookupTable",T(-1,[19,1540,96,2600,113,1710,130,1360,131,680,147,1330,241,2120,258,1840,275,1810,512,2720,529,2220,546,1980,563,1960,800,2940,803,3700,817,2450,834,2110,835,2500,851,2200]));c$.bsCations=c$.prototype.bsCations=new JU.BS;
c$.bsAnions=c$.prototype.bsAnions=new JU.BS;for(var a=0;a<JU.Elements.anionLookupTable.length;a+=2)JU.Elements.bsAnions.set(JU.Elements.anionLookupTable[a]>>4);for(a=0;a<JU.Elements.cationLookupTable.length;a+=2)JU.Elements.bsCations.set(JU.Elements.cationLookupTable[a]>>4);E(c$,"hydrophobicities",K(-1,[0,0.62,-2.53,-0.78,-0.9,0.29,-0.85,-0.74,0.48,-0.4,1.38,1.06,-1.5,0.64,1.19,0.12,-0.18,-0.05,0.81,0.26,1.08]));(JU.Elements.elementNames.length!=JU.Elements.elementNumberMax||JU.Elements.vanderwaalsMars.length>>
2!=JU.Elements.elementNumberMax||JU.Elements.defaultBondingMars.length>>1!=JU.Elements.elementNumberMax)&&JU.Logger.error("ERROR!!! Element table length mismatch:\n elementSymbols.length="+JU.Elements.elementSymbols.length+"\n elementNames.length="+JU.Elements.elementNames.length+"\n vanderwaalsMars.length="+JU.Elements.vanderwaalsMars.length+"\n covalentMars.length="+JU.Elements.defaultBondingMars.length);E(c$,"electroNegativities",K(-1,[0,2.2,0,0.97,1.47,2.01,2.5,3.07,3.5,4.1,0,1.01,1.23,1.47,1.74,
2.06,2.44,2.83,0,0.91,1.04,1.2,1.32,1.45,1.56,1.6,1.64,1.7,1.75,1.75,1.66,1.82,2.02,2.2,2.48,2.74,0,0.89,0.99,1.11,1.22,1.23,1.3,1.36,1.42,1.45,1.35,1.42,1.46,1.49,1.72,1.82,2.01,2.21]))});s("JU");v(null,"JU.Escape","java.lang.Float java.util.Map JU.A4 $.AU $.BS $.Lst $.M3 $.M34 $.M4 $.P3 $.P4 $.PT $.Quat $.SB $.T3 $.V3 JS.SV".split(" "),function(){c$=D(JU,"Escape");c$.escapeColor=c(c$,"escapeColor",function(a){return 0==a?null:"[x"+JU.Escape.getHexColorFromRGB(a)+"]"},"~N");c$.getHexColorFromRGB=
c(c$,"getHexColorFromRGB",function(a){if(0==a)return null;var b="00"+Integer.toHexString(a>>16&255),b=b.substring(b.length-2),d="00"+Integer.toHexString(a>>8&255),d=d.substring(d.length-2);a="00"+Integer.toHexString(a&255);a=a.substring(a.length-2);return b+d+a},"~N");c$.eP=c(c$,"eP",function(a){return null==a?"null":"{"+a.x+" "+a.y+" "+a.z+"}"},"JU.T3");c$.matrixToScript=c(c$,"matrixToScript",function(a){return JU.PT.replaceAllCharacters(a.toString(),"\n\r ","").$replace("\t"," ")},"~O");c$.eP4=
c(c$,"eP4",function(a){return"{"+a.x+" "+a.y+" "+a.z+" "+a.w+"}"},"JU.P4");c$.drawQuat=c(c$,"drawQuat",function(a,b,d,c,f){c=" VECTOR "+JU.Escape.eP(c)+" ";0==f&&(f=1);return"draw "+b+"x"+d+c+JU.Escape.eP(a.getVectorScaled(0,f))+" color red\ndraw "+b+"y"+d+c+JU.Escape.eP(a.getVectorScaled(1,f))+" color green\ndraw "+b+"z"+d+c+JU.Escape.eP(a.getVectorScaled(2,f))+" color blue\n"},"JU.Quat,~S,~S,JU.P3,~N");c$.e=c(c$,"e",function(a){if(null==a)return"null";if(JU.PT.isNonStringPrimitive(a))return a.toString();
if(n(a,String))return JU.PT.esc(a);if(n(a,JU.Lst))return JU.Escape.eV(a);if(n(a,java.util.Map))return JU.Escape.escapeMap(a);if(n(a,JU.BS))return JU.Escape.eBS(a);if(n(a,JU.P4))return JU.Escape.eP4(a);if(n(a,JU.T3))return JU.Escape.eP(a);if(JU.AU.isAP(a))return JU.Escape.eAP(a);if(JU.AU.isAS(a))return JU.Escape.eAS(a,!0);if(n(a,JU.M34))return JU.PT.rep(JU.PT.rep(a.toString(),"[\n  ","["),"] ]","]]");if(JU.AU.isAFF(a)){for(var b=(new JU.SB).append("["),d="",c=0;c<a.length;c++)b.append(d).append(JU.Escape.eAF(a[c])),
d=",";b.append("]");return b.toString()}if(n(a,JU.A4))return"{"+a.x+" "+a.y+" "+a.z+" "+180*a.angle/3.141592653589793+"}";if(n(a,JU.Quat))return a.toString();b=JU.PT.nonArrayString(a);return null==b?JU.PT.toJSON(null,a):b},"~O");c$.eV=c(c$,"eV",function(a){if(null==a)return JU.PT.esc("");var b=new JU.SB;b.append("[");for(var d=0;d<a.size();d++)0<d&&b.append(", "),b.append(JU.Escape.escapeNice(a.get(d).asString()));b.append("]");return b.toString()},"JU.Lst");c$.escapeMap=c(c$,"escapeMap",function(a){var b=
new JU.SB;b.append("{ ");var d="",c;for(a=a.entrySet().iterator();a.hasNext()&&((c=a.next())||1);){var f=c.getKey();b.append(d).append(JU.PT.esc(f)).appendC(":");d=c.getValue();n(d,JS.SV)||(d=JS.SV.getVariable(d));b.append(d.escape());d=","}b.append(" }");return b.toString()},"java.util.Map");c$.escapeFloatA=c(c$,"escapeFloatA",function(a,b){if(b)return JU.PT.toJSON(null,a);for(var d=new JU.SB,c=0;c<a.length;c++)0<c&&d.appendC("\n"),d.appendF(a[c]);return d.toString()},"~A,~B");c$.escapeFloatAA=c(c$,
"escapeFloatAA",function(a,b){for(var d=new JU.SB,c=b?";\n":"\n",f=0;f<a.length;f++)if(null!=a[f]){0<f&&d.append(c);for(var e=0;e<a[f].length;e++)d.appendF(a[f][e]).appendC("\t")}return d.toString()},"~A,~B");c$.escapeFloatAAA=c(c$,"escapeFloatAAA",function(a,b){var d=new JU.SB,c=b?";\n":"\n";if(null==a[0]||null==a[0][0])return"0 0 0"+c;d.appendI(a.length).append(" ").appendI(a[0].length).append(" ").appendI(a[0][0].length);for(var f=0;f<a.length;f++)if(null!=a[f]){d.append(c);for(var e=0;e<a[f].length;e++)if(null!=
a[f][e]){d.append(c);for(var h=0;h<a[f][e].length;h++)d.appendF(a[f][e][h]).appendC("\t")}}return d.toString()},"~A,~B");c$.eAS=c(c$,"eAS",function(a,b){if(null==a)return JU.PT.esc("");var d=new JU.SB;d.append("[");for(var c=0;c<a.length;c++)0<c&&d.append(", "),d.append(b?JU.Escape.escapeNice(a[c]):JU.PT.esc(a[c]));d.append("]");return d.toString()},"~A,~B");c$.eAI=c(c$,"eAI",function(a){if(null==a)return JU.PT.esc("");var b=new JU.SB;b.append("[");for(var d=0;d<a.length;d++)0<d&&b.append(", "),b.appendI(a[d]);
return b.append("]").toString()},"~A");c$.eAD=c(c$,"eAD",function(a){if(null==a)return JU.PT.esc("");var b=new JU.SB;b.append("[");for(var d=0;d<a.length;d++)0<d&&b.append(", "),b.appendD(a[d]);return b.append("]").toString()},"~A");c$.eAF=c(c$,"eAF",function(a){if(null==a)return JU.PT.esc("");var b=new JU.SB;b.append("[");for(var d=0;d<a.length;d++)0<d&&b.append(", "),b.appendF(a[d]);return b.append("]").toString()},"~A");c$.eAP=c(c$,"eAP",function(a){if(null==a)return JU.PT.esc("");var b=new JU.SB;
b.append("[");for(var d=0;d<a.length;d++)0<d&&b.append(", "),b.append(JU.Escape.eP(a[d]));return b.append("]").toString()},"~A");c$.escapeNice=c(c$,"escapeNice",function(a){if(null==a)return"null";var b=JU.PT.parseFloatStrict(a);return Float.isNaN(b)?JU.PT.esc(a):a},"~S");c$.uABsM=c(c$,"uABsM",function(a){return"{"==a.charAt(0)?JU.Escape.uP(a):(JU.Escape.isStringArray(a)||a.startsWith("[{")&&a.indexOf("[{")==a.lastIndexOf("[{"))&&0>a.indexOf(",")&&0>a.indexOf(".")&&0>a.indexOf("-")?JU.BS.unescape(a):
a.startsWith("[[")?JU.Escape.unescapeMatrix(a):a},"~S");c$.isStringArray=c(c$,"isStringArray",function(a){return a.startsWith("({")&&0==a.lastIndexOf("({")&&a.indexOf("})")==a.length-2},"~S");c$.uP=c(c$,"uP",function(a){if(null==a||0==a.length)return a;var b=a.$replace("\n"," ").trim();if("{"!=b.charAt(0)||"}"!=b.charAt(b.length-1))return a;for(var d=K(5,0),c=0,b=b.substring(1,b.length-1),f=B(1,0);5>c;c++)if(d[c]=JU.PT.parseFloatNext(b,f),Float.isNaN(d[c])){if(f[0]>=b.length||","!=b.charAt(f[0]))break;
f[0]++;c--}return 3==c?JU.P3.new3(d[0],d[1],d[2]):4==c?JU.P4.new4(d[0],d[1],d[2],d[3]):a},"~S");c$.unescapeMatrix=c(c$,"unescapeMatrix",function(a){if(null==a||0==a.length)return a;var b=a.$replace("\n"," ").trim();if(0!=b.lastIndexOf("[[")||b.indexOf("]]")!=b.length-2)return a;for(var d=K(16,0),b=b.substring(2,b.length-2).$replace("["," ").$replace("]"," ").$replace(","," "),c=B(1,0),f=0;16>f&&!(d[f]=JU.PT.parseFloatNext(b,c),Float.isNaN(d[f]));f++);return!Float.isNaN(JU.PT.parseFloatNext(b,c))?
a:9==f?JU.M3.newA9(d):16==f?JU.M4.newA16(d):a},"~S");c$.eBS=c(c$,"eBS",function(a){return JU.BS.escape(a,"(",")")},"JU.BS");c$.eBond=c(c$,"eBond",function(a){return JU.BS.escape(a,"[","]")},"JU.BS");c$.toReadable=c(c$,"toReadable",function(a,b){var d=new JU.SB,c="";if(null==b)return"null";if(JU.PT.isNonStringPrimitive(b))return JU.Escape.packageReadable(a,null,b.toString());if(n(b,String))return JU.Escape.packageReadable(a,null,JU.PT.esc(b));if(n(b,JS.SV))return JU.Escape.packageReadable(a,null,b.escape());
if(JU.AU.isAS(b)){d.append("[");for(var f=b.length,e=0;e<f;e++)d.append(c).append(JU.Escape.toReadable(null,b[e])),c=",";d.append("]");return JU.Escape.packageReadableSb(a,"String["+f+"]",d)}if(JU.AU.isAI(b)){d.append("[");f=b.length;for(e=0;e<f;e++)d.append(c).appendI(b[e]),c=",";d.append("]");return JU.Escape.packageReadableSb(a,"int["+f+"]",d)}if(JU.AU.isAF(b)){d.append("[");f=b.length;for(e=0;e<f;e++)d.append(c).appendF(b[e]),c=",";d.append("]");return JU.Escape.packageReadableSb(a,"float["+f+
"]",d)}if(JU.AU.isAD(b)){d.append("[");f=b.length;for(e=0;e<f;e++)d.append(c).appendD(b[e]),c=",";d.append("]");return JU.Escape.packageReadableSb(a,"double["+f+"]",d)}if(JU.AU.isAP(b)){d.append("[");f=b.length;for(e=0;e<f;e++)d.append(c).append(JU.Escape.eP(b[e])),c=",";d.append("]");return JU.Escape.packageReadableSb(a,"point3f["+f+"]",d)}if(JU.AU.isASS(b)){d.append("[");f=b.length;for(e=0;e<f;e++)d.append(c).append(JU.Escape.toReadable(null,b[e])),c=",\n";d.append("]");return JU.Escape.packageReadableSb(a,
"String["+f+"][]",d)}if(JU.AU.isAII(b)){d.append("[");f=b.length;for(e=0;e<f;e++)d.append(c).append(JU.Escape.toReadable(null,b[e])),c=",";d.append("]");return JU.Escape.packageReadableSb(a,"int["+f+"][]",d)}if(JU.AU.isAFF(b)){d.append("[\n");f=b.length;for(e=0;e<f;e++)d.append(c).append(JU.Escape.toReadable(null,b[e])),c=",\n";d.append("]");return JU.Escape.packageReadableSb(a,"float[][]",d)}if(JU.AU.isADD(b)){d.append("[\n");f=b.length;for(e=0;e<f;e++)d.append(c).append(JU.Escape.toReadable(null,
b[e])),c=",\n";d.append("]");return JU.Escape.packageReadableSb(a,"double[][]",d)}if(n(b,JU.Lst)){f=b.size();for(e=0;e<f;e++)d.append(JU.Escape.toReadable(a+"["+(e+1)+"]",b.get(e)));return JU.Escape.packageReadableSb(a,"List["+f+"]",d)}if(n(b,JU.M34)||n(b,JU.T3)||n(b,JU.P4)||n(b,JU.A4))return d.append(JU.Escape.e(b)),JU.Escape.packageReadableSb(a,null,d);if(n(b,java.util.Map)){for(c=b.keySet().iterator();c.hasNext();)f=c.next(),d.append(JU.Escape.toReadable((null==a?"":a+".")+f,b.get(f)));return d.toString()}return JU.Escape.packageReadable(a,
null,JU.PT.toJSON(null,b))},"~S,~O");c$.packageReadableSb=c(c$,"packageReadableSb",function(a,b,d){return JU.Escape.packageReadable(a,b,d.toString())},"~S,~S,JU.SB");c$.packageReadable=c(c$,"packageReadable",function(a,b,d){return null==a?(null==b?"":b+"\t")+d:"\n"+a+"\t"+(null==b?"":"*"+b+"\t")+d},"~S,~S,~S");c$.escapeModelFileNumber=c(c$,"escapeModelFileNumber",function(a){return""+w(a/1E6)+"."+a%1E6},"~N");c$.encapsulateData=c(c$,"encapsulateData",function(a,b,d){switch(d){case 1:b=JU.Escape.escapeFloatA(b,
!1)+";\n";break;case 2:b=JU.Escape.escapeFloatAA(b,!0)+";\n";break;case 3:b=JU.Escape.escapeFloatAAA(b,!0)+";\n";break;default:b=b.toString()}return'  DATA "'+a+'"\n'+b+'    END "'+a+'";\n'},"~S,~O,~N");c$.unescapeUnicode=c(c$,"unescapeUnicode",function(a){for(var b=a.length,d=JU.SB.newN(b),c=0;c<b;){var f=a.charAt(c++);if("\\"==f&&c<b)switch(f=a.charAt(c++),f){case "u":if(c<b){for(var f=0,e=4;0<=--e&&c<b;){var h=a.charAt(c),h=JU.Escape.getHexitValue(h);if(0>h)break;f<<=4;f+=h;++c}f=String.fromCharCode(f)}}d.appendC(f)}return d.toString()},
"~S");c$.getHexitValue=c(c$,"getHexitValue",function(a){return 48<=a.charCodeAt(0)&&57>=a.charCodeAt(0)?a.charCodeAt(0)-48:97<=a.charCodeAt(0)&&102>=a.charCodeAt(0)?10+a.charCodeAt(0)-97:65<=a.charCodeAt(0)&&70>=a.charCodeAt(0)?10+a.charCodeAt(0)-65:-1},"~S");c$.unescapeStringArray=c(c$,"unescapeStringArray",function(a){if(null==a||!a.startsWith("[")||!a.endsWith("]"))return null;var b=new JU.Lst,d=B(1,0);for(d[0]=1;d[0]<a.length;){var c=JU.PT.getQuotedStringNext(a,d);if(null==c)return null;for(b.addLast(JU.PT.rep(c,
'\\"','"'));d[0]<a.length&&'"'!=a.charAt(d[0]);)d[0]++}return b.toArray(Array(b.size()))},"~S");c$.isAV=c(c$,"isAV",function(a){return n(a[0],JS.SV)},"~O");c$.escapeHelical=c(c$,"escapeHelical",function(a,b,d,c,f){switch(b){case 134217751:return null==f?new JU.P3:f[0];case 1073741854:case 1665140738:return null==f?new JU.V3:f[1073741854==b?1:2];case 134217729:return Float.$valueOf(null==f?NaN:f[3].x);case 135176:return null==f?"":'draw ID "'+a+'" VECTOR '+JU.Escape.eP(f[0])+" "+JU.Escape.eP(f[1])+
" color "+(0>f[3].x?"{255.0 200.0 0.0}":"{255.0 0.0 128.0}");case 1745489939:return(null==f?"":"measure "+JU.Escape.eP(d)+JU.Escape.eP(f[0])+JU.Escape.eP(f[4]))+JU.Escape.eP(c);default:return null==f?[]:f}},"~S,~N,JU.P3,JU.P3,~A")});s("JU");v(["J.api.JmolGraphicsInterface","JU.Normix"],"JU.GData","JU.AU $.P3 $.V3 JU.C $.Font $.Shader".split(" "),function(){c$=t(function(){this.apiPlatform=null;this.antialiasEnabled=this.currentlyRendering=this.translucentCoverOnly=!1;this.displayMaxY2=this.displayMinY2=
this.displayMaxX2=this.displayMinX2=this.displayMaxY=this.displayMinY=this.displayMaxX=this.displayMinX=this.windowHeight=this.windowWidth=0;this.inGreyscaleMode=this.antialiasThisFrame=!1;this.backgroundImage=this.changeableColixMap=null;this.newWindowHeight=this.newWindowWidth=0;this.newAntialiasing=!1;this.ht3=this.argbCurrent=this.colixCurrent=this.ambientOcclusion=this.height=this.width=this.depth=this.slab=this.yLast=this.xLast=this.bgcolor=0;this.isPass2=!1;this.bufferSize=this.textY=0;this.graphicsForMetrics=
this.vwr=this.shader=null;this.argbNoisyDn=this.argbNoisyUp=0;this.currentFont=this.transformedVectors=null;p(this,arguments)},JU,"GData",null,J.api.JmolGraphicsInterface);O(c$,function(){this.changeableColixMap=T(16,0);this.transformedVectors=Array(JU.GData.normixCount)});q(c$,function(){this.shader=new JU.Shader});c(c$,"initialize",function(a,b){this.vwr=a;this.apiPlatform=b},"JV.Viewer,J.api.GenericPlatform");c(c$,"setDepth",function(a){this.depth=0>a?0:a},"~N");j(c$,"setSlab",function(a){this.slab=
Math.max(0,a)},"~N");j(c$,"setSlabAndZShade",function(a,b){this.setSlab(a);this.setDepth(b)},"~N,~N,~N,~N,~N");c(c$,"setAmbientOcclusion",function(a){this.ambientOcclusion=a},"~N");j(c$,"isAntialiased",function(){return this.antialiasThisFrame});c(c$,"getChangeableColix",function(a,b){a>=this.changeableColixMap.length&&(this.changeableColixMap=JU.AU.arrayCopyShort(this.changeableColixMap,a+16));0==this.changeableColixMap[a]&&(this.changeableColixMap[a]=JU.C.getColix(b));return a|-32768},"~N,~N");
c(c$,"changeColixArgb",function(a,b){a<this.changeableColixMap.length&&0!=this.changeableColixMap[a]&&(this.changeableColixMap[a]=JU.C.getColix(b))},"~N,~N");c(c$,"getColorArgbOrGray",function(a){0>a&&(a=this.changeableColixMap[a&2047]);return this.inGreyscaleMode?JU.C.getArgbGreyscale(a):JU.C.getArgb(a)},"~N");c(c$,"getShades",function(a){0>a&&(a=this.changeableColixMap[a&2047]);return this.inGreyscaleMode?this.shader.getShadesG(a):this.shader.getShades(a)},"~N");c(c$,"setGreyscaleMode",function(a){this.inGreyscaleMode=
a},"~B");c(c$,"getSpecularPower",function(){return this.shader.specularPower});c(c$,"setSpecularPower",function(a){0>a?this.setSpecularExponent(-a):this.shader.specularPower!=a&&(this.shader.specularPower=a,this.shader.intenseFraction=a/100,this.shader.flushCaches())},"~N");c(c$,"getSpecularPercent",function(){return this.shader.specularPercent});c(c$,"setSpecularPercent",function(a){this.shader.specularPercent!=a&&(this.shader.specularPercent=a,this.shader.specularFactor=a/100,this.shader.flushCaches())},
"~N");c(c$,"getSpecularExponent",function(){return this.shader.specularExponent});c(c$,"setSpecularExponent",function(a){this.shader.specularExponent!=a&&(this.shader.specularExponent=a,this.shader.phongExponent=w(Math.pow(2,a)),this.shader.usePhongExponent=!1,this.shader.flushCaches())},"~N");c(c$,"getPhongExponent",function(){return this.shader.phongExponent});c(c$,"setPhongExponent",function(a){this.shader.phongExponent==a&&this.shader.usePhongExponent||(this.shader.phongExponent=a,a=Math.log(a)/
Math.log(2),this.shader.usePhongExponent=a!=A(a),this.shader.usePhongExponent||(this.shader.specularExponent=A(a)),this.shader.flushCaches())},"~N");c(c$,"getDiffusePercent",function(){return this.shader.diffusePercent});c(c$,"setDiffusePercent",function(a){this.shader.diffusePercent!=a&&(this.shader.diffusePercent=a,this.shader.diffuseFactor=a/100,this.shader.flushCaches())},"~N");c(c$,"getAmbientPercent",function(){return this.shader.ambientPercent});c(c$,"setAmbientPercent",function(a){this.shader.ambientPercent!=
a&&(this.shader.ambientPercent=a,this.shader.ambientFraction=a/100,this.shader.flushCaches())},"~N");c(c$,"getSpecular",function(){return this.shader.specularOn});c(c$,"setSpecular",function(a){this.shader.specularOn!=a&&(this.shader.specularOn=a,this.shader.flushCaches())},"~B");c(c$,"setCel",function(a){this.shader.setCel(a,this.shader.celPower,this.bgcolor)},"~B");c(c$,"getCel",function(){return this.shader.celOn});c(c$,"getCelPower",function(){return this.shader.celPower});c(c$,"setCelPower",
function(a){this.shader.setCel(this.shader.celOn||0==this.shader.celPower,a,this.bgcolor)},"~N");c(c$,"getLightSource",function(){return this.shader.lightSource});c(c$,"isClipped3",function(a,b,d){return 0>a||a>=this.width||0>b||b>=this.height||d<this.slab||d>this.depth},"~N,~N,~N");c(c$,"isClipped",function(a,b){return 0>a||a>=this.width||0>b||b>=this.height},"~N,~N");j(c$,"isInDisplayRange",function(a,b){return a>=this.displayMinX&&a<this.displayMaxX&&b>=this.displayMinY&&b<this.displayMaxY},"~N,~N");
j(c$,"isClippedXY",function(a,b,d){a=a+1>>1;return b<-a||b>=this.width+a||d<-a||d>=this.height+a},"~N,~N,~N");c(c$,"isClippedZ",function(a){return-2147483648!=a&&(a<this.slab||a>this.depth)},"~N");c(c$,"clipCode3",function(a,b,d){var c=0;0>a?c|=a<this.displayMinX2?-1:8:a>=this.width&&(c|=a>this.displayMaxX2?-1:4);0>b?c|=b<this.displayMinY2?-1:2:b>=this.height&&(c|=b>this.displayMaxY2?-1:1);d<this.slab?c|=32:d>this.depth&&(c|=16);return c},"~N,~N,~N");c(c$,"clipCode",function(a){var b=0;a<this.slab?
b|=32:a>this.depth&&(b|=16);return b},"~N");c(c$,"getFont3D",function(a){return JU.Font.createFont3D(0,0,a,a,this.apiPlatform,this.graphicsForMetrics)},"~N");c(c$,"getFont3DFS",function(a,b){return JU.Font.createFont3D(JU.Font.getFontFaceID(a),0,b,b,this.apiPlatform,this.graphicsForMetrics)},"~S,~N");c(c$,"getFontFidFS",function(a,b){return this.getFont3DFSS(a,"Bold",b).fid},"~S,~N");c(c$,"getFont3DFSS",function(a,b,d){b=JU.Font.getFontStyleID(b);0>b&&(b=0);return JU.Font.createFont3D(JU.Font.getFontFaceID(a),
b,d,d,this.apiPlatform,this.graphicsForMetrics)},"~S,~S,~N");c(c$,"getFont3DScaled",function(a,b){var d=a.fontSizeNominal*b;return d==a.fontSize?a:JU.Font.createFont3D(a.idFontFace,a.idFontStyle,d,a.fontSizeNominal,this.apiPlatform,this.graphicsForMetrics)},"JU.Font,~N");c(c$,"getFontFid",function(a){return this.getFont3D(a).fid},"~N");c(c$,"setBackgroundTransparent",function(){},"~B");c(c$,"setBackgroundArgb",function(a){this.bgcolor=a;this.setCel(this.shader.celOn)},"~N");c(c$,"setBackgroundImage",
function(a){this.backgroundImage=a},"~O");c(c$,"setWindowParameters",function(a,b,d){this.setWinParams(a,b,d)},"~N,~N,~B");c(c$,"setWinParams",function(a,b,d){this.newWindowWidth=a;this.newWindowHeight=b;this.newAntialiasing=d},"~N,~N,~B");c(c$,"setNewWindowParametersForExport",function(){this.windowWidth=this.newWindowWidth;this.windowHeight=this.newWindowHeight;this.setWidthHeight(!1)});c(c$,"setWidthHeight",function(a){this.width=this.windowWidth;this.height=this.windowHeight;a&&(this.width<<=
1,this.height<<=1);this.xLast=this.width-1;this.yLast=this.height-1;this.displayMinX=-(this.width>>1);this.displayMaxX=this.width-this.displayMinX;this.displayMinY=-(this.height>>1);this.displayMaxY=this.height-this.displayMinY;this.displayMinX2=this.displayMinX<<2;this.displayMaxX2=this.displayMaxX<<2;this.displayMinY2=this.displayMinY<<2;this.displayMaxY2=this.displayMaxY<<2;this.ht3=3*this.height;this.bufferSize=this.width*this.height},"~B");c(c$,"beginRendering",function(){},"JU.M3,~B,~B,~B");
c(c$,"endRendering",function(){});c(c$,"snapshotAnaglyphChannelBytes",function(){});c(c$,"getScreenImage",function(){return null},"~B");c(c$,"releaseScreenImage",function(){});c(c$,"applyAnaglygh",function(){},"J.c.STER,~A");c(c$,"setPass2",function(){return!1},"~B");c(c$,"destroy",function(){});c(c$,"clearFontCache",function(){});c(c$,"drawQuadrilateralBits",function(a,b,d,c,f,e){a.drawLineBits(b,b,d,c);a.drawLineBits(b,b,c,f);a.drawLineBits(b,b,f,e);a.drawLineBits(b,b,e,d)},"J.api.JmolRendererInterface,~N,JU.P3,JU.P3,JU.P3,JU.P3");
c(c$,"drawTriangleBits",function(a,b,d,c,f,e,h,j){1==(j&1)&&a.drawLineBits(d,f,b,c);2==(j&2)&&a.drawLineBits(f,h,c,e);4==(j&4)&&a.drawLineBits(h,d,e,b)},"J.api.JmolRendererInterface,JU.P3,~N,JU.P3,~N,JU.P3,~N,~N");c(c$,"plotImage",function(){},"~N,~N,~N,~O,J.api.JmolRendererInterface,~N,~N,~N");c(c$,"plotText",function(){},"~N,~N,~N,~N,~N,~S,JU.Font,J.api.JmolRendererInterface");c(c$,"renderBackground",function(){},"J.api.JmolRendererInterface");c(c$,"setFont",function(){},"JU.Font");c(c$,"setFontFid",
function(){},"~N");c(c$,"setColor",function(a){this.argbCurrent=this.argbNoisyUp=this.argbNoisyDn=a},"~N");c(c$,"setC",function(){return!0},"~N");c(c$,"isDirectedTowardsCamera",function(a){return 0>a||0<this.transformedVectors[a].z},"~N");c$.roundInt=c(c$,"roundInt",function(a){return a},"~N");c(c$,"clear",function(){});j(c$,"renderAllStrings",function(){},"~O");c(c$,"addRenderer",function(){},"~N");c$.getHermiteList=c(c$,"getHermiteList",function(a,b,d,c,f,e,h,j,l,n){var m=l+1;l-=1;var r=d.x,p=d.y,
q=d.z,s=c.x,t=c.y;c=c.z;var u=(s-b.x)*a/8,v=(t-b.y)*a/8;b=(c-b.z)*a/8;var w=(f.x-r)*a/8,A=(f.y-p)*a/8,B=(f.z-q)*a/8,D=(e.x-s)*a/8,G=(e.y-t)*a/8;a=(e.z-c)*a/8;h[j]=d;for(d=0;d<m;d++){var E=d/l;d==m-1&&(r=s,p=t,q=c,s=f.x,t=f.y,c=f.z,u=w,v=A,b=B,w=D,A=G,B=a,E-=1);var H=E*E,I=H*E;e=2*I-3*H+1;var K=-2*I+3*H,E=I-2*H+E,L=I-H,H=e*r+K*s+E*u+L*w,I=e*p+K*t+E*v+L*A;e=e*q+K*c+E*b+L*B;h[j+d]=n?JU.P3.new3(H,I,e):JU.V3.new3(H,I,e)}},"~N,JU.T3,JU.T3,JU.T3,JU.T3,JU.T3,~A,~N,~N,~B");c(c$,"setTextPosition",function(a){this.textY=
a},"~N");c(c$,"getTextPosition",function(){return this.textY});c(c$,"getTransformedVertexVectors",function(){return this.transformedVectors});c(c$,"getFont3DCurrent",function(){return this.currentFont});E(c$,"ENDCAPS_NONE",0,"ENDCAPS_HIDDEN",1,"ENDCAPS_FLAT",2,"ENDCAPS_SPHERICAL",3,"ENDCAPS_OPEN_TO_SPHERICAL",4,"ENDCAPS_FLAT_TO_SPHERICAL",5,"EXPORT_RAYTRACER",2,"EXPORT_CARTESIAN",1,"EXPORT_NOT",0,"yGT",1,"yLT",2,"xGT",4,"xLT",8,"zGT",16,"zLT",32,"HUGE",-1);c$.normixCount=c$.prototype.normixCount=
JU.Normix.getNormixCount()});s("JU");v(["J.api.JmolAppletInterface","$.JmolStatusListener"],"JU.GenericApplet","java.lang.Boolean java.net.URL java.util.Hashtable JU.Lst $.PT $.SB J.c.CBK J.i18n.GT JU.Logger JV.JC $.Viewer".split(" "),function(){c$=t(function(){this.isJS=!1;this.documentBase=this.codeBase=null;this.isSigned=!1;this.language=null;this.doTranslate=!0;this.mayScript=this.isStereoSlave=this.haveDocumentAccess=!1;this.vwrOptions=this.b$=this.viewer=this.gRight=this.statusTextarea=this.statusText=
this.statusForm=this.fullName=this.htmlName=null;this.haveNotifiedError=!1;this.appletObject=null;this.loading=this.isJNLP=!1;this.outputBuffer=this.syncId=null;p(this,arguments)},JU,"GenericApplet",null,[J.api.JmolAppletInterface,J.api.JmolStatusListener]);c(c$,"init",function(a){this.b$=new java.util.Hashtable;null==JU.GenericApplet.htRegistry&&(JU.GenericApplet.htRegistry=new java.util.Hashtable);this.appletObject=a;this.htmlName=JU.PT.split(""+this.getJmolParameter("name"),"_object")[0];this.syncId=
this.getJmolParameter("syncId");this.fullName=this.htmlName+"__"+this.syncId+"__";Zystem.out.println("Jmol JavaScript applet "+this.fullName+" initializing");a=this.getValue("logLevel",this.getBooleanValue("debug",!1)?"5":"4").charCodeAt(0)-48;4!=a&&Zystem.out.println("setting logLevel="+a+' -- To change, use script "set logLevel [0-5]"');JU.Logger.setLogLevel(a);J.i18n.GT.ignoreApplicationBundle();this.initOptions();JU.GenericApplet.checkIn(this.fullName,this.appletObject);this.initApplication()},
"~O");c(c$,"initApplication",function(){this.vwrOptions.put("applet",Boolean.TRUE);null==this.getJmolParameter("statusListener")&&this.vwrOptions.put("statusListener",this);this.language=this.getJmolParameter("language");null!=this.language&&this.vwrOptions.put("language",this.language);this.viewer=new JV.Viewer(this.vwrOptions);this.viewer.pushHoldRepaint();var a=this.getValueLowerCase("emulate","jmol");this.setStringProperty("defaults",a.equals("chime")?"RasMol":"Jmol");this.setStringProperty("backgroundColor",
this.getValue("bgcolor",this.getValue("boxbgcolor","black")));this.viewer.setBooleanProperty("frank",!0);this.loading=!0;for(var b,a=0,d=J.c.CBK.values();a<d.length&&((b=d[a])||1);a++)this.setValue(b.name()+"Callback",null);this.loading=!1;null!=this.language&&Zystem.out.print("requested language="+this.language+"; ");this.doTranslate=!"none".equals(this.language)&&this.getBooleanValue("doTranslate",!0);this.language=J.i18n.GT.getLanguage();Zystem.out.println("language="+this.language);if(null==this.b$.get(J.c.CBK.SCRIPT)&&
null==this.b$.get(J.c.CBK.ERROR)&&(null!=this.b$.get(J.c.CBK.MESSAGE)||null!=this.statusForm||null!=this.statusText))this.doTranslate&&null==this.getValue("doTranslate",null)&&(this.doTranslate=!1,JU.Logger.warn("Note -- Presence of message callback disables disable translation; to enable message translation use jmolSetTranslation(true) prior to jmolApplet()")),this.doTranslate&&JU.Logger.warn("Note -- Automatic language translation may affect parsing of message callbacks messages; use scriptCallback or errorCallback to process errors");
this.doTranslate||(J.i18n.GT.setDoTranslate(!1),JU.Logger.warn("Note -- language translation disabled"));this.getBooleanValue("popupMenu",!0)||this.viewer.getProperty("DATA_API","disablePopupMenu",null);b=this.getJmolParameter("menuFile");null!=b&&this.viewer.setMenu(b,!0);b=this.getValue("script","");a=this.getValue("loadInline",null);if(null==a){if(null!=(a=this.getValue("load",null)))b='load "'+a+'";'+b;a=null}this.viewer.popHoldRepaint("applet init");null!=a&&null!=this.viewer.loadInline(a)&&
(b="");0<b.length&&this.scriptProcessor(b,null,1);this.viewer.notifyStatusReady(!0)});j(c$,"destroy",function(){this.gRight=null;this.viewer.notifyStatusReady(!1);this.viewer=null;JU.GenericApplet.checkOut(this.fullName)});c(c$,"getBooleanValue",function(a,b){var d=this.getValue(a,b?"true":"");return d.equalsIgnoreCase("true")||d.equalsIgnoreCase("on")||d.equalsIgnoreCase("yes")},"~S,~B");c(c$,"getValue",function(a,b){var d=this.getJmolParameter(a);Zystem.out.println("Jmol getValue "+a+" "+d);return null==
d?b:d},"~S,~S");c(c$,"getValueLowerCase",function(a,b){var d=this.getValue(a,b);null!=d&&(d=d.trim().toLowerCase(),0==d.length&&(d=null));return d},"~S,~S");c(c$,"setValue",function(a,b){this.setStringProperty(a,this.getValue(a,b))},"~S,~S");c(c$,"setStringProperty",function(a,b){null!=b&&(JU.Logger.info(a+' = "'+b+'"'),this.viewer.setStringProperty(a,b))},"~S,~S");c(c$,"scriptProcessor",function(a,b,d){if(null==a||0==a.length)return"";switch(d){case 0:return a=this.viewer.scriptCheck(a),n(a,String)?
a:"";case 1:return null!=b?this.viewer.scriptWaitStatus(a,b).toString():this.viewer.scriptWait(a);default:return this.viewer.script(a)}},"~S,~S,~N");j(c$,"register",function(a,b){JU.GenericApplet.checkIn(a,b)},"~S,J.api.JmolSyncInterface");j(c$,"getJSpecViewProperty",function(){return null},"~S");c(c$,"syncScript",function(a){this.viewer.syncScript(a,"~",0)},"~S");j(c$,"handleEvent",function(a){return null==this.viewer?!1:this.viewer.processMouseEvent(a.id,a.x,a.y,a.modifiers,a.when)},"java.awt.Event");
j(c$,"getAppletInfo",function(){return J.i18n.GT.o(J.i18n.GT.$("Jmol Applet version {0} {1}.\n\nAn OpenScience project.\n\nSee http://www.jmol.org for more information"),u(-1,[JV.JC.version,JV.JC.date]))+"\nhtmlName = "+JU.PT.esc(this.htmlName)+"\nsyncId = "+JU.PT.esc(this.syncId)+"\ndocumentBase = "+JU.PT.esc(this.documentBase)+"\ncodeBase = "+JU.PT.esc(this.codeBase)});j(c$,"script",function(a){this.scriptNoWait(a)},"~S");j(c$,"scriptCheck",function(a){return null==a||0==a.length?"":this.scriptProcessor(a,
null,0)},"~S");j(c$,"scriptNoWait",function(a){return null==a||0==a.length?"":this.scriptProcessor(a,null,2)},"~S");c(c$,"scriptWait",function(a){return this.scriptWait(a,null)},"~S");c(c$,"scriptWait",function(a,b){if(null==a||0==a.length)return"";this.outputBuffer=null;return this.scriptProcessor(a,b,1)},"~S,~S");j(c$,"scriptWaitOutput",function(a){if(null==a||0==a.length)return"";this.outputBuffer=new JU.SB;this.viewer.scriptWaitStatus(a,"");a=null==this.outputBuffer?"":this.outputBuffer.toString();
this.outputBuffer=null;return a},"~S");j(c$,"getModelIndexFromId",function(a){return this.viewer.getModelIndexFromId(a)},"~S");j(c$,"getProperty",function(a,b){b||(b="");return this.viewer.getProperty(null,a,b)},"~S,~S");j(c$,"getPropertyAsString",function(a,b){b||(b="");return this.viewer.getProperty("readable",a,b).toString()},"~S,~S");j(c$,"getPropertyAsJSON",function(a,b){b||(b="");return this.viewer.getProperty("JSON",a,b).toString()},"~S,~S");j(c$,"loadInlineString",function(a,b,d){a=this.viewer.loadInlineAppend(a,
d);null==a&&this.script(b);return a},"~S,~S,~B");j(c$,"loadInlineArray",function(a,b,d){if(null==a||0==a.length)return null;a=this.viewer.loadInline(a,d);null==a&&this.script(b);return a},"~A,~S,~B");j(c$,"loadDOMNode",function(a){return this.viewer.openDOM(a)},"~O");c(c$,"loadInline",function(a){return this.loadInlineString(a,"",!1)},"~S");c(c$,"loadInline",function(a,b){return this.loadInlineString(a,b,!1)},"~S,~S");c(c$,"loadInline",function(a){return this.loadInlineArray(a,"",!1)},"~A");c(c$,
"loadInline",function(a,b){return this.loadInlineArray(a,b,!1)},"~A,~S");c(c$,"output",function(a){null!=this.outputBuffer&&null!=a&&this.outputBuffer.append(a).appendC("\n")},"~S");j(c$,"setCallbackFunction",function(a,b){if(a.equalsIgnoreCase("language"))this.consoleMessage(""),this.consoleMessage(null);else{var d=J.c.CBK.getCallback(a);null!=d&&(this.loading||d!==J.c.CBK.EVAL)?null==b?this.b$.remove(d):this.b$.put(d,b):this.consoleMessage("Available callbacks include: "+J.c.CBK.getNameList().$replace(";",
" ").trim())}},"~S,~S");c(c$,"consoleMessage",function(a){this.notifyCallback(J.c.CBK.ECHO,u(-1,["",a]))},"~S");j(c$,"notifyEnabled",function(a){switch(a){case J.c.CBK.SERVICE:return!1;case J.c.CBK.ECHO:case J.c.CBK.MESSAGE:case J.c.CBK.MEASURE:case J.c.CBK.PICK:case J.c.CBK.SYNC:return!0;case J.c.CBK.ANIMFRAME:case J.c.CBK.DRAGDROP:case J.c.CBK.ERROR:case J.c.CBK.EVAL:case J.c.CBK.IMAGE:case J.c.CBK.LOADSTRUCT:case J.c.CBK.SCRIPT:return!this.isJNLP}return null!=this.b$.get(a)},"J.c.CBK");c(c$,"notifyCallback",
function(a,b){var d=null==a?null:this.b$.get(a),c=null==a||null!=d&&(null==b||null==b[0]),f=!1;null!=b&&(b[0]=this.htmlName);var e=null==b||null==b[1]?null:b[1].toString();if(null!=a)switch(a){case J.c.CBK.APPLETREADY:b[3]=this.appletObject;break;case J.c.CBK.CLICK:"alert".equals(d)&&(e="x="+b[1]+" y="+b[2]+" action="+b[3]+" clickCount="+b[4]);break;case J.c.CBK.ANIMFRAME:var h=b[1],j=h[0],l=h[1],n=h[2],m=h[3],h=h[4],r=-2>=j,p=0>m?-1:1,q=0>h?-1:1;c&&(b=u(-1,[this.htmlName,Integer.$valueOf(Math.max(j,
-2-j)),Integer.$valueOf(l),Integer.$valueOf(n),Integer.$valueOf(Math.abs(m)),Integer.$valueOf(Math.abs(h)),Integer.$valueOf(r?1:0),Integer.$valueOf(p),Integer.$valueOf(q),b[2],b[3]]));break;case J.c.CBK.ECHO:l=(j=2==b.length)||1==b[2].intValue();c||(l&&(f=!0),c=!j&&null!=(d=this.b$.get(a=J.c.CBK.MESSAGE)));f||this.output(e);break;case J.c.CBK.LOADSTRUCT:j=b[4];if(null!=j){j=(0<=j.indexOf("NOTE:")?"":J.i18n.GT.$("File Error:"))+j;this.doShowStatus(j);this.notifyCallback(J.c.CBK.MESSAGE,u(-1,["",j]));
return}break;case J.c.CBK.MEASURE:c||(c=null!=(d=this.b$.get(a=J.c.CBK.MESSAGE)));j=b[3];0<=j.indexOf("Picked")||0<=j.indexOf("Sequence")?(this.doShowStatus(e),f=!0):0<=j.indexOf("Completed")&&(e=j+": "+e,f=!0);break;case J.c.CBK.MESSAGE:f=!c;c=(new Boolean(c&null!=e)).valueOf();f||this.output(e);break;case J.c.CBK.PICK:this.doShowStatus(e);f=!0;break;case J.c.CBK.SCRIPT:!(0<b[3].intValue())&&!c&&(c=null!=(d=this.b$.get(a=J.c.CBK.MESSAGE)));this.output(e);this.doShowStatus(e);break;case J.c.CBK.SYNC:this.sendScript(e,
b[2],!0,c);return}f&&(f=this.viewer.getProperty("DATA_API","getAppConsole",null),null!=f&&(f.notifyCallback(a,b),this.output(e),this.doSendJsTextareaStatus(e)));if(c&&this.mayScript)try{this.doSendCallback(d,b,e)}catch(s){if(H(s,Exception))this.haveNotifiedError||JU.Logger.debugging&&JU.Logger.debug(a.name()+"Callback call error to "+d+": "+s),this.haveNotifiedError=!0;else throw s;}},"J.c.CBK,~A");c(c$,"sendScript",function(a,b,d,c){if(c&&(a=this.notifySync(a,b),null==a||0==a.length||a.equals("0")))return"";
var f=new JU.Lst;JU.GenericApplet.findApplets(b,this.syncId,this.fullName,f);var e=f.size();if(0==e)return!c&&!b.equals("*")&&JU.Logger.error(this.fullName+" couldn't find applet "+b),"";b=d?null:new JU.SB;c=d&&a.equals("GET_GRAPHICS");var h=d&&a.equals("SET_GRAPHICS_OFF");c&&this.viewer.setStereo(!1,this.gRight=null);for(var j=0;j<e;j++){var l=f.get(j),n=JU.GenericApplet.htRegistry.get(l);JU.Logger.debugging&&JU.Logger.debug(this.fullName+" sending to "+l+": "+a);try{if(c||h)return this.viewer.setStereo(this.isStereoSlave=
c,this.gRight=n.setStereoGraphics(c)),"";d?n.syncScript(a):b.append(n.scriptWait(a,"output")).append("\n")}catch(m){if(H(m,Exception))l=this.htmlName+" couldn't send to "+l+": "+a+": "+m,JU.Logger.error(l),d||b.append(l);else throw m;}}return d?"":b.toString()},"~S,~S,~B,~B");c(c$,"notifySync",function(a,b){var d=this.b$.get(J.c.CBK.SYNC);if(!this.mayScript||null==d)return a;try{return this.doSendCallback(d,u(-1,[this.fullName,a,b]),null)}catch(c){if(H(c,Exception))this.haveNotifiedError||JU.Logger.debugging&&
JU.Logger.debug("syncCallback call error to "+d+": "+c),this.haveNotifiedError=!0;else throw c;}return a},"~S,~S");j(c$,"eval",function(a){var b=a.indexOf("\u0001");return 0<=b?this.sendScript(a.substring(b+1),a.substring(0,b),!1,!1):!this.haveDocumentAccess?"NO EVAL ALLOWED":null!=this.b$.get(J.c.CBK.EVAL)?(this.notifyCallback(J.c.CBK.EVAL,u(-1,[null,a])),""):this.doEval(a)},"~S");j(c$,"functionXY",function(a,b,d){return this.doFunctionXY(a,b,d)},"~S,~N,~N");j(c$,"functionXYZ",function(a,b,d,c){return this.doFunctionXYZ(a,
b,d,c)},"~S,~N,~N,~N");j(c$,"createImage",function(){return null},"~S,~S,~O,~N");j(c$,"getRegistryInfo",function(){JU.GenericApplet.checkIn(null,null);return JU.GenericApplet.htRegistry});j(c$,"showUrl",function(a){JU.Logger.debugging&&JU.Logger.debug("showUrl("+a+")");if(null!=a&&0<a.length)try{this.doShowDocument(new java.net.URL(aa("java.net.URL"),a,null))}catch(b){if(H(b,java.net.MalformedURLException))this.consoleMessage("Malformed URL:"+a);else throw b;}},"~S");j(c$,"resizeInnerPanel",function(){return B(-1,
[this.viewer.getScreenWidth(),this.viewer.getScreenHeight()])},"~S");c$.checkIn=c(c$,"checkIn",function(a,b){null!=a&&(JU.Logger.info("AppletRegistry.checkIn("+a+")"),JU.GenericApplet.htRegistry.put(a,b));if(JU.Logger.debugging)for(var d,c=JU.GenericApplet.htRegistry.entrySet().iterator();c.hasNext()&&((d=c.next())||1);){var f=d.getKey();JU.Logger.debug(f+" "+d.getValue())}},"~S,~O");c$.checkOut=c(c$,"checkOut",function(a){JU.GenericApplet.htRegistry.remove(a)},"~S");c$.findApplets=c(c$,"findApplets",
function(a,b,d,c){if(null!=a&&0<=a.indexOf(",")){var f=JU.PT.split(a,",");for(a=0;a<f.length;a++)JU.GenericApplet.findApplets(f[a],b,d,c)}else if(b="__"+b+"__",null==a||a.equals("*")||a.equals(">"))for(a=JU.GenericApplet.htRegistry.keySet().iterator();a.hasNext()&&((f=a.next())||1);)!f.equals(d)&&0<f.indexOf(b)&&c.addLast(f);else 0<=d.indexOf("_object")&&0>a.indexOf("_object")&&(a+="_object"),0>a.indexOf("__")&&(a+=b),JU.GenericApplet.htRegistry.containsKey(a)||(a="jmolApplet"+a),!a.equals(d)&&JU.GenericApplet.htRegistry.containsKey(a)&&
c.addLast(a)},"~S,~S,~S,JU.Lst");j(c$,"notifyAudioEnded",function(a){this.viewer.sm.notifyAudioStatus(a)},"~O");E(c$,"htRegistry",null,"SCRIPT_CHECK",0,"SCRIPT_WAIT",1,"SCRIPT_NOWAIT",2)});s("JU");v(["JU.AU"],"JU.Geodesic",["java.lang.NullPointerException","$.Short","java.util.Hashtable","JU.V3"],function(){c$=D(JU,"Geodesic");c$.getNeighborVertexesArrays=c(c$,"getNeighborVertexesArrays",function(){null==JU.Geodesic.vertexCounts&&JU.Geodesic.createGeodesic(3);return JU.Geodesic.neighborVertexesArrays});
c$.getVertexCount=c(c$,"getVertexCount",function(a){null==JU.Geodesic.vertexCounts&&JU.Geodesic.createGeodesic(3);return JU.Geodesic.vertexCounts[a]},"~N");c$.getVertexVectors=c(c$,"getVertexVectors",function(){null==JU.Geodesic.vertexCounts&&JU.Geodesic.createGeodesic(3);return JU.Geodesic.vertexVectors});c$.getVertexVector=c(c$,"getVertexVector",function(a){return JU.Geodesic.vertexVectors[a]},"~N");c$.getFaceVertexes=c(c$,"getFaceVertexes",function(a){return JU.Geodesic.faceVertexesArrays[a]},
"~N");c$.createGeodesic=c(c$,"createGeodesic",function(a){if(!(a<JU.Geodesic.currentLevel)){JU.Geodesic.currentLevel=a;var b=T(a+1,0);JU.Geodesic.neighborVertexesArrays=JU.AU.newShort2(a+1);JU.Geodesic.faceVertexesArrays=JU.AU.newShort2(a+1);JU.Geodesic.vertexVectors=Array(12);JU.Geodesic.vertexVectors[0]=JU.V3.new3(0,0,JU.Geodesic.halfRoot5);for(var d=0;5>d;++d)JU.Geodesic.vertexVectors[d+1]=JU.V3.new3(Math.cos(1.2566371*d),Math.sin(1.2566371*d),0.5),JU.Geodesic.vertexVectors[d+6]=JU.V3.new3(Math.cos(1.2566371*
d+0.62831855),Math.sin(1.2566371*d+0.62831855),-0.5);JU.Geodesic.vertexVectors[11]=JU.V3.new3(0,0,-JU.Geodesic.halfRoot5);for(d=12;0<=--d;)JU.Geodesic.vertexVectors[d].normalize();JU.Geodesic.faceVertexesArrays[0]=JU.Geodesic.faceVertexesIcosahedron;JU.Geodesic.neighborVertexesArrays[0]=JU.Geodesic.neighborVertexesIcosahedron;b[0]=12;for(d=0;d<a;++d)JU.Geodesic.quadruple(d,b);JU.Geodesic.vertexCounts=b}},"~N");c$.quadruple=c(c$,"quadruple",function(a,b){JU.Geodesic.htVertex=new java.util.Hashtable;
var d=JU.Geodesic.vertexVectors.length,c=JU.Geodesic.faceVertexesArrays[a],f=c.length,e=w(f/3),h=d+(d+e-2),e=4*e;JU.Geodesic.vertexVectors=JU.AU.arrayCopyObject(JU.Geodesic.vertexVectors,h);e=T(3*e,0);JU.Geodesic.faceVertexesArrays[a+1]=e;var j=T(6*h,0);JU.Geodesic.neighborVertexesArrays[a+1]=j;for(var l=j.length;0<=--l;)j[l]=-1;b[a+1]=h;JU.Geodesic.vertexNext=d;for(l=d=0;l<f;){var n=c[l++],m=c[l++],r=c[l++],p=JU.Geodesic.getVertex(n,m),q=JU.Geodesic.getVertex(m,r),s=JU.Geodesic.getVertex(r,n);e[d++]=
n;e[d++]=p;e[d++]=s;e[d++]=m;e[d++]=q;e[d++]=p;e[d++]=r;e[d++]=s;e[d++]=q;e[d++]=s;e[d++]=p;e[d++]=q;JU.Geodesic.addNeighboringVertexes(j,p,n);JU.Geodesic.addNeighboringVertexes(j,p,s);JU.Geodesic.addNeighboringVertexes(j,p,q);JU.Geodesic.addNeighboringVertexes(j,p,m);JU.Geodesic.addNeighboringVertexes(j,q,m);JU.Geodesic.addNeighboringVertexes(j,q,s);JU.Geodesic.addNeighboringVertexes(j,q,r);JU.Geodesic.addNeighboringVertexes(j,s,r);JU.Geodesic.addNeighboringVertexes(j,s,n)}f=JU.Geodesic.vertexVectors.length;
if(d!=e.length)throw new NullPointerException;if(JU.Geodesic.vertexNext!=h)throw new NullPointerException;for(l=0;12>l;++l)for(c=0;5>c;++c){d=j[6*l+c];if(0>d)throw new NullPointerException;if(d>=f)throw new NullPointerException;if(-1!=j[6*l+5])throw new NullPointerException;}for(l=72;l<j.length;++l){d=j[l];if(0>d)throw new NullPointerException;if(d>=f)throw new NullPointerException;}for(l=0;l<h;++l){f=0;for(c=j.length;0<=--c;)j[c]==l&&++f;if(12>l&&5!=f||12<=l&&6!=f)throw new NullPointerException;
f=0;for(c=e.length;0<=--c;)e[c]==l&&++f;if(12>l&&5!=f||12<=l&&6!=f)throw new NullPointerException;}JU.Geodesic.htVertex=null},"~N,~A");c$.addNeighboringVertexes=c(c$,"addNeighboringVertexes",function(a,b,d){for(var c=6*b,f=c+6;c<f;++c){if(a[c]==d)return;if(0>a[c]){a[c]=d;for(var e=6*d,h=e+6;e<h;++e){if(a[e]==b)return;if(0>a[e]){a[e]=b;return}}}}throw new NullPointerException;},"~A,~N,~N");c$.getVertex=c(c$,"getVertex",function(a,b){if(a>b){var d=a;a=b;b=d}var d=Integer.$valueOf((a<<16)+b),c=JU.Geodesic.htVertex.get(d);
if(null!=c)return c.shortValue();c=JU.Geodesic.vertexVectors[JU.Geodesic.vertexNext]=new JU.V3;c.add2(JU.Geodesic.vertexVectors[a],JU.Geodesic.vertexVectors[b]);c.normalize();JU.Geodesic.htVertex.put(d,Short.$valueOf(JU.Geodesic.vertexNext));return JU.Geodesic.vertexNext++},"~N,~N");c$.halfRoot5=c$.prototype.halfRoot5=0.5*Math.sqrt(5);E(c$,"oneFifth",1.2566371,"oneTenth",0.62831855,"faceVertexesIcosahedron",T(-1,[0,1,2,0,2,3,0,3,4,0,4,5,0,5,1,1,6,2,2,7,3,3,8,4,4,9,5,5,10,1,6,1,10,7,2,6,8,3,7,9,4,
8,10,5,9,11,6,10,11,7,6,11,8,7,11,9,8,11,10,9]),"neighborVertexesIcosahedron",T(-1,[1,2,3,4,5,-1,0,5,10,6,2,-1,0,1,6,7,3,-1,0,2,7,8,4,-1,0,3,8,9,5,-1,0,4,9,10,1,-1,1,10,11,7,2,-1,2,6,11,8,3,-1,3,7,11,9,4,-1,4,8,11,10,5,-1,5,9,11,6,1,-1,6,7,8,9,10,-1]),"standardLevel",3,"maxLevel",3,"vertexCounts",null,"vertexVectors",null,"faceVertexesArrays",null,"neighborVertexesArrays",null,"currentLevel",0,"vertexNext",0,"htVertex",null,"VALIDATE",!0)});s("JU");c$=t(function(){this.entryCount=0;this.entries=null;
p(this,arguments)},JU,"Int2IntHash");q(c$,function(a){this.entries=Array(a)},"~N");c(c$,"get",function(a){for(var b=this.entries,b=b[(a&2147483647)%b.length];null!=b;b=b.next)if(b.key==a)return b.value;return-2147483648},"~N");c(c$,"put",function(a,b){for(var d=this.entries,c=d.length,f=(a&2147483647)%c,e=d[f];null!=e;e=e.next)if(e.key==a){e.value=b;return}if(this.entryCount>c){for(var f=c,c=c+(c+1),h=Array(c),j=f;0<=--j;)for(e=d[j];null!=e;){var l=e,e=e.next,f=(l.key&2147483647)%c;l.next=h[f];h[f]=
l}d=this.entries=h;f=(a&2147483647)%c}d[f]=new JU.Int2IntHashEntry(a,b,d[f]);++this.entryCount},"~N,~N");c$=t(function(){this.value=this.key=0;this.next=null;p(this,arguments)},JU,"Int2IntHashEntry");q(c$,function(a,b,d){this.key=a;this.value=b;this.next=d},"~N,~N,JU.Int2IntHashEntry");s("JU");v(null,"JU.JSJSONParser","java.lang.Boolean $.Float java.util.Hashtable JU.JSONException $.Lst $.SB".split(" "),function(){c$=t(function(){this.str=null;this.len=this.index=0;this.asHashTable=!1;p(this,arguments)},
JU,"JSJSONParser");q(c$,function(){});c(c$,"parseMap",function(a,b){this.index=0;this.asHashTable=b;this.str=a;this.len=a.length;if("{"!=this.getChar())return null;this.returnChar();return this.getValue(!1)},"~S,~B");c(c$,"parse",function(a,b){this.index=0;this.asHashTable=b;this.str=a;this.len=a.length;return this.getValue(!1)},"~S,~B");c(c$,"next",function(){return this.index<this.len?this.str.charAt(this.index++):"\x00"});c(c$,"returnChar",function(){this.index--});c(c$,"getChar",function(){for(;;){var a=
this.next();if(0==a.charCodeAt(0)||" "<a)return a}});c(c$,"getValue",function(a){var b=this.index,d=this.getChar();switch(d){case "\x00":return null;case '"':case "'":return this.getString(d);case "{":if(!a)return this.getObject();d=String.fromCharCode(0);break;case "[":if(!a)return this.getArray();d=String.fromCharCode(0);break;default:for(this.returnChar();" "<=d&&0>"[,]{:}'\"".indexOf(d);)d=this.next();this.returnChar();a&&":"!=d&&(d=String.fromCharCode(0))}if(a&&0==d.charCodeAt(0))throw new JU.JSONException("invalid key");
b=this.str.substring(b,this.index).trim();if(!a){if(b.equals("true"))return Boolean.TRUE;if(b.equals("false"))return Boolean.FALSE;if(b.equals("null"))return this.asHashTable?b:null}d=b.charAt(0);if("0"<=d&&"9">=d||"-"==d)try{if(0>b.indexOf(".")&&0>b.indexOf("e")&&0>b.indexOf("E"))return new Integer(b);var c=Float.$valueOf(b);if(!c.isInfinite()&&!c.isNaN())return c}catch(f){if(!H(f,Exception))throw f;}Zystem.out.println("JSON parser cannot parse "+b);throw new JU.JSONException("invalid value");},
"~B");c(c$,"getString",function(a){for(var b,d=null,c=this.index;;){var f=this.index;switch(b=this.next()){case "\x00":case "\n":case "\r":throw this.syntaxError("Unterminated string");case "\\":switch(b=this.next()){case '"':case "'":case "\\":case "/":break;case "b":b="\b";break;case "t":b="\t";break;case "n":b="\n";break;case "f":b="\f";break;case "r":b="\r";break;case "u":var e=this.index;this.index+=4;try{b=String.fromCharCode(Integer.parseInt(this.str.substring(e,this.index),16))}catch(h){if(H(h,
Exception))throw this.syntaxError("Substring bounds error");throw h;}break;default:throw this.syntaxError("Illegal escape.");}break;default:if(b==a)return null==d?this.str.substring(c,f):d.toString()}this.index>f+1&&null==d&&(d=new JU.SB,d.append(this.str.substring(c,f)));null!=d&&d.appendC(b)}},"~S");c(c$,"getObject",function(){var a=this.asHashTable?new java.util.Hashtable:new java.util.HashMap,b=null;switch(this.getChar()){case "}":return a;case 0:throw new JU.JSONException("invalid object");}this.returnChar();
for(var d=!1;;)switch(!0==(d=!d)?b=this.getValue(!0).toString():a.put(b,this.getValue(!1)),this.getChar()){case "}":return a;case ":":if(d)continue;d=!0;case ",":if(!d)continue;default:throw this.syntaxError("Expected ',' or ':' or '}'");}});c(c$,"getArray",function(){var a=new JU.Lst;switch(this.getChar()){case "]":return a;case "\x00":throw new JU.JSONException("invalid array");}this.returnChar();for(var b=!1;;)switch(b?(a.addLast(null),b=!1):a.addLast(this.getValue(!1)),this.getChar()){case ",":switch(this.getChar()){case "]":return a;
case ",":b=!0;default:this.returnChar()}continue;case "]":return a;default:throw this.syntaxError("Expected ',' or ']'");}});c(c$,"syntaxError",function(a){return new JU.JSONException(a+" for "+this.str.substring(0,Math.min(this.index,this.len)))},"~S")});s("JU");v(["java.lang.RuntimeException"],"JU.JSONException",null,function(){c$=D(JU,"JSONException",RuntimeException)});s("JU");N(JU,"SimpleNode");s("JU");N(JU,"SimpleEdge");s("JU");v(["JU.SimpleNode"],"JU.Node",null,function(){N(JU,"Node",JU.SimpleNode)});
s("JU");v(["java.lang.Enum","JU.SimpleEdge"],"JU.Edge",null,function(){c$=t(function(){this.index=-1;this.order=0;p(this,arguments)},JU,"Edge",null,JU.SimpleEdge);c$.getArgbHbondType=c(c$,"getArgbHbondType",function(a){return JU.Edge.argbsHbondType[(a&30720)>>11]},"~N");c$.getBondOrderNumberFromOrder=c(c$,"getBondOrderNumberFromOrder",function(a){a&=-131073;switch(a){case 131071:case 65535:return"0";case 1025:case 1041:return"1";default:return JU.Edge.isOrderH(a)||JU.Edge.isAtropism(a)||0!=(a&256)?
"1":0!=(a&224)?(a>>5)+"."+(a&31):JU.Edge.EnumBondOrder.getNumberFromCode(a)}},"~N");c$.getCmlBondOrder=c(c$,"getCmlBondOrder",function(a){a=JU.Edge.getBondOrderNameFromOrder(a);switch(a.charAt(0)){case "s":case "d":case "t":return""+a.toUpperCase().charAt(0);case "a":return 0<=a.indexOf("Double")?"D":0<=a.indexOf("Single")?"S":"aromatic";case "p":return 0<=a.indexOf(" ")?a.substring(a.indexOf(" ")+1):"partial12"}return null},"~N");c$.getBondOrderNameFromOrder=c(c$,"getBondOrderNameFromOrder",function(a){a&=
-131073;switch(a){case 65535:case 131071:return"";case 1025:return"near";case 1041:return"far";case 32768:return JU.Edge.EnumBondOrder.STRUT.$$name;case 1:return JU.Edge.EnumBondOrder.SINGLE.$$name;case 2:return JU.Edge.EnumBondOrder.DOUBLE.$$name}return 0!=(a&224)?"partial "+JU.Edge.getBondOrderNumberFromOrder(a):JU.Edge.isOrderH(a)?JU.Edge.EnumBondOrder.H_REGULAR.$$name:65537==(a&65537)?(a=JU.Edge.getAtropismCode(a),"atropisomer_"+w(a/4)+a%4):0!=(a&256)?JU.Edge.EnumBondOrder.SINGLE.$$name:JU.Edge.EnumBondOrder.getNameFromCode(a)},
"~N");c$.getAtropismOrder=c(c$,"getAtropismOrder",function(a,b){return JU.Edge.getAtropismOrder12((a<<2)+b)},"~N,~N");c$.getAtropismOrder12=c(c$,"getAtropismOrder12",function(a){return a<<11|65537},"~N");c$.getAtropismCode=c(c$,"getAtropismCode",function(a){return a>>11&15},"~N");c$.getAtropismNode=c(c$,"getAtropismNode",function(a,b,d){a=a>>11+(d?0:2)&3;return b.getEdges()[a-1].getOtherNode(b)},"~N,JU.Node,~B");c$.isAtropism=c(c$,"isAtropism",function(a){return 65537==(a&65537)},"~N");c$.isOrderH=
c(c$,"isOrderH",function(a){return 0!=(a&30720)&&0==(a&65537)},"~N");c$.getPartialBondDotted=c(c$,"getPartialBondDotted",function(a){return a&31},"~N");c$.getPartialBondOrder=c(c$,"getPartialBondOrder",function(a){return(a&-131073)>>5},"~N");c$.getCovalentBondOrder=c(c$,"getCovalentBondOrder",function(a){if(0==(a&1023))return 0;a&=-131073;if(0!=(a&224))return JU.Edge.getPartialBondOrder(a);0!=(a&256)&&(a&=-257);0!=(a&248)&&(a=1);return a&7},"~N");c$.getBondOrderFromFloat=c(c$,"getBondOrderFromFloat",
function(a){switch(A(10*a)){case 10:return 1;case 5:case -10:return 33;case 15:return 515;case -15:return 66;case 20:return 2;case 25:return 97;case -25:return 100;case 30:return 3;case 40:return 4}return 131071},"~N");c$.getBondOrderFromString=c(c$,"getBondOrderFromString",function(a){var b=JU.Edge.EnumBondOrder.getCodeFromName(a);try{131071==b&&(14==a.length&&a.toLowerCase().startsWith("atropisomer_"))&&(b=JU.Edge.getAtropismOrder(Integer.parseInt(a.substring(12,13)),Integer.parseInt(a.substring(13,
14))))}catch(d){if(!H(d,NumberFormatException))throw d;}return b},"~S");j(c$,"getBondType",function(){return this.order});c(c$,"setCIPChirality",function(){},"~N");c(c$,"getCIPChirality",function(){return""},"~B");M(self.c$);c$=t(function(){this.code=0;this.$$name=this.number=null;p(this,arguments)},JU.Edge,"EnumBondOrder",Enum);q(c$,function(a,b,d){this.code=a;this.number=b;this.$$name=d},"~N,~S,~S");c$.getCodeFromName=c(c$,"getCodeFromName",function(a){for(var b,d=0,c=JU.Edge.EnumBondOrder.values();d<
c.length&&((b=c[d])||1);d++)if(b.$$name.equalsIgnoreCase(a))return b.code;return 131071},"~S");c$.getNameFromCode=c(c$,"getNameFromCode",function(a){for(var b,d=0,c=JU.Edge.EnumBondOrder.values();d<c.length&&((b=c[d])||1);d++)if(b.code==a)return b.$$name;return"?"},"~N");c$.getNumberFromCode=c(c$,"getNumberFromCode",function(a){for(var b,d=0,c=JU.Edge.EnumBondOrder.values();d<c.length&&((b=c[d])||1);d++)if(b.code==a)return b.number;return"?"},"~N");G(c$,"SINGLE",0,[1,"1","single"]);G(c$,"DOUBLE",
1,[2,"2","double"]);G(c$,"TRIPLE",2,[3,"3","triple"]);G(c$,"QUADRUPLE",3,[4,"4","quadruple"]);G(c$,"QUINTUPLE",4,[5,"5","quintuple"]);G(c$,"sextuple",5,[6,"6","sextuple"]);G(c$,"AROMATIC",6,[515,"1.5","aromatic"]);G(c$,"STRUT",7,[32768,"1","struts"]);G(c$,"H_REGULAR",8,[2048,"1","hbond"]);G(c$,"PARTIAL01",9,[33,"0.5","partial"]);G(c$,"PARTIAL12",10,[66,"1.5","partialDouble"]);G(c$,"PARTIAL23",11,[97,"2.5","partialTriple"]);G(c$,"PARTIAL32",12,[100,"2.5","partialTriple2"]);G(c$,"AROMATIC_SINGLE",13,
[513,"1","aromaticSingle"]);G(c$,"AROMATIC_DOUBLE",14,[514,"2","aromaticDouble"]);G(c$,"ATROPISOMER",15,[65537,"1","atropisomer"]);G(c$,"UNSPECIFIED",16,[17,"1","unspecified"]);c$=L();E(c$,"BOND_RENDER_SINGLE",65536,"TYPE_ATROPISOMER",65537,"TYPE_ATROPISOMER_REV",65538,"ATROPISOMER_SHIFT",11,"BOND_STEREO_MASK",1024,"BOND_STEREO_NEAR",1025,"BOND_STEREO_FAR",1041,"BOND_AROMATIC_MASK",512,"BOND_AROMATIC_SINGLE",513,"BOND_AROMATIC_DOUBLE",514,"BOND_AROMATIC",515,"BOND_SULFUR_MASK",256,"BOND_PARTIAL_MASK",
224,"BOND_PARTIAL01",33,"BOND_PARTIAL12",66,"BOND_PARTIAL23",97,"BOND_PARTIAL32",100,"BOND_COVALENT_MASK",1023,"BOND_COVALENT_SINGLE",1,"BOND_COVALENT_DOUBLE",2,"BOND_COVALENT_TRIPLE",3,"BOND_COVALENT_QUADRUPLE",4,"BOND_COVALENT_QUINTUPLE",5,"BOND_COVALENT_sextuple",6,"BOND_ORDER_UNSPECIFIED",17,"BOND_ORDER_ANY",65535,"BOND_ORDER_NULL",131071,"BOND_STRUT",32768,"BOND_PYMOL_NOMULT",65536,"BOND_PYMOL_MULT",98304,"BOND_NEW",131072,"BOND_HBOND_SHIFT",11,"BOND_HYDROGEN_MASK",30720,"BOND_H_REGULAR",2048,
"BOND_H_CALC_MASK",28672,"BOND_H_CALC",4096,"BOND_H_PLUS_2",6144,"BOND_H_PLUS_3",8192,"BOND_H_PLUS_4",10240,"BOND_H_PLUS_5",12288,"BOND_H_MINUS_3",14336,"BOND_H_MINUS_4",16384,"BOND_H_NUCLEOTIDE",18432,"argbsHbondType",B(-1,[4294928820,4294967040,4294967040,4294967295,4294902015,4294901760,4294944E3,4278255615,4278255360,4294934656]))});s("JU");v(["JU.Elements"],"JU.JmolMolecule",["java.lang.Character","java.util.Hashtable","JU.AU","$.BS","$.PT"],function(){c$=t(function(){this.nodes=null;this.nElements=
this.ac=this.firstAtomIndex=this.indexInModel=this.modelIndex=this.moleculeIndex=0;this.altElementCounts=this.elementCounts=null;this.altElementMax=this.elementNumberMax=0;this.atNos=this.atomList=this.mf=null;p(this,arguments)},JU,"JmolMolecule");O(c$,function(){this.elementCounts=B(JU.Elements.elementNumberMax,0);this.altElementCounts=B(JU.Elements.altElementMax,0)});q(c$,function(){});c$.getMolecules=c(c$,"getMolecules",function(a,b,d,c){var f=null,e=new JU.BS,h=-1,j=0,l=0,n=Array(4);null==c&&
(c=new JU.BS);for(var m=0;m<a.length;m++)if(!c.get(m)&&!e.get(m)){var r=a[m];null==r||r.isDeleted()?c.set(m):(r=r.getModelIndex(),r!=h&&(h=r,j=0,f=b[r]),e=JU.JmolMolecule.getBranchBitSet(a,m,f,d,-1,!0,!0),0<=e.nextSetBit(0)&&(n=JU.JmolMolecule.addMolecule(n,l++,a,m,e,r,j++,c)))}return JU.JmolMolecule.allocateArray(n,l)},"~A,~A,JU.Lst,JU.BS");c$.getBranchBitSet=c(c$,"getBranchBitSet",function(a,b,d,c,f,e,h){var j=JU.BS.newN(a.length);if(0>b)return j;0<=f&&d.clear(f);return JU.JmolMolecule.getCovalentlyConnectedBitSet(a,
a[b],d,e,h,c,j)?j:new JU.BS},"~A,~N,JU.BS,JU.Lst,~N,~B,~B");c$.addMolecule=c(c$,"addMolecule",function(a,b,d,c,f,e,h,j){j.or(f);b==a.length&&(a=JU.JmolMolecule.allocateArray(a,2*b+1));a[b]=JU.JmolMolecule.initialize(d,b,c,f,e,h);return a},"~A,~N,~A,~N,JU.BS,~N,~N,JU.BS");c$.getMolecularFormulaAtoms=c(c$,"getMolecularFormulaAtoms",function(a,b,d,c){var f=new JU.JmolMolecule;f.nodes=a;f.atomList=b;return f.getMolecularFormula(!1,d,c)},"~A,JU.BS,~A,~B");c(c$,"getMolecularFormula",function(a,b,d){return this.getMolecularFormulaImpl(a,
b,d)},"~B,~A,~B");c(c$,"getMolecularFormulaImpl",function(a,b,d){if(null!=this.mf)return this.mf;null==this.atomList&&(this.atomList=new JU.BS,this.atomList.setBits(0,null==this.atNos?this.nodes.length:this.atNos.length));this.elementCounts=B(JU.Elements.elementNumberMax,0);this.altElementCounts=B(JU.Elements.altElementMax,0);this.ac=this.atomList.cardinality();for(var c=this.nElements=0,f=this.atomList.nextSetBit(0);0<=f;f=this.atomList.nextSetBit(f+1),c++){var e,h=null;if(null==this.atNos){h=this.nodes[f];
if(null==h)continue;e=h.getAtomicAndIsotopeNumber()}else e=this.atNos[f];var j=null==b?1:A(8*b[c]);e<JU.Elements.elementNumberMax?(0==this.elementCounts[e]&&this.nElements++,this.elementCounts[e]+=j,this.elementNumberMax=Math.max(this.elementNumberMax,e),a&&(e=h.getImplicitHydrogenCount(),0<e&&(0==this.elementCounts[1]&&this.nElements++,this.elementCounts[1]+=e*j))):(e=JU.Elements.altElementIndexFromNumber(e),0==this.altElementCounts[e]&&this.nElements++,this.altElementCounts[e]+=j,this.altElementMax=
Math.max(this.altElementMax,e))}if(null!=b)for(f=1;f<=this.elementNumberMax;f++){var l=w(this.elementCounts[f]/8);if(8*l!=this.elementCounts[f])return"?";this.elementCounts[f]=l}if(d){b=2;for(a=!0;a;){b=1E5;for(f=1;f<=this.elementNumberMax;f++)if(0<(l=this.elementCounts[f])&&l<b)b=l;if(1==b)break;for(;1<b;b--){a=!0;for(f=1;f<=this.elementNumberMax&&a;f++)if(w((l=this.elementCounts[f])/b)*b!=l)a=!1;if(a){for(f=1;f<=this.elementNumberMax;f++)this.elementCounts[f]/=b;break}}}}a=l="";for(f=1;f<=this.elementNumberMax;f++)b=
this.elementCounts[f],0!=b&&(l+=a+JU.Elements.elementSymbolFromNumber(f)+" "+b,a=" ");for(f=1;f<=this.altElementMax;f++)b=this.altElementCounts[f],0!=b&&(l+=a+JU.Elements.elementSymbolFromNumber(JU.Elements.altElementNumberFromIndex(f))+" "+b,a=" ");return l},"~B,~A,~B");c$.initialize=c(c$,"initialize",function(a,b,d,c,f,e){var h=new JU.JmolMolecule;h.nodes=a;h.firstAtomIndex=d;h.atomList=c;h.ac=c.cardinality();h.moleculeIndex=b;h.modelIndex=f;h.indexInModel=e;return h},"~A,~N,~N,JU.BS,~N,~N");c$.getCovalentlyConnectedBitSet=
c(c$,"getCovalentlyConnectedBitSet",function(a,b,d,c,f,e,h){var j=b.getIndex();if(!d.get(j)||!f&&0<b.getBioStructureTypeName().length)return c;d.clear(j);if(null!=e&&!h.get(j))for(var l=e.size();0<=--l;){var n=e.get(l);if(n.get(j)){h.or(n);d.andNot(n);for(l=n.nextSetBit(0);0<=l;l=n.nextSetBit(l+1)){var m=a[l];null!=m&&(d.set(l),JU.JmolMolecule.getCovalentlyConnectedBitSet(a,m,d,c,f,e,h),d.clear(l))}break}}h.set(j);j=b.getEdges();if(null==j)return!0;for(l=j.length;0<=--l;)if(n=j[l],null!=n&&n.isCovalent()&&
!JU.JmolMolecule.getCovalentlyConnectedBitSet(a,n.getOtherNode(b),d,c,f,e,h))return!1;return!0},"~A,JU.Node,JU.BS,~B,~B,JU.Lst,JU.BS");c$.allocateArray=c(c$,"allocateArray",function(a,b){return b==a.length?a:JU.AU.arrayCopyObject(a,b)},"~A,~N");c$.getBitSetForMF=c(c$,"getBitSetForMF",function(a,b,d){var c=new java.util.Hashtable,f,e;d=JU.PT.rep(JU.PT.clean(d+"Z")," ","");for(var h=0,j=0,l=0,n=d.length;h<n;h++)if((e=Character.isDigit(f=d.charAt(h)))||0<h&&Character.isUpperCase(f)){var l=h,m=d.substring(j,
l).trim();if(e)for(;h<n&&Character.isDigit(d.charAt(h));)h++;j=h;c.put(m,B(-1,[e?JU.PT.parseInt(d.substring(l,j)):1]))}d=new JU.BS;for(h=b.nextSetBit(0);0<=h;h=b.nextSetBit(h+1))f=a[h].getElementSymbol(),f=c.get(f),null==f||1>f[0]--||d.set(h);var r;for(a=c.values().iterator();a.hasNext()&&((r=a.next())||1);)if(0<r[0])return new JU.BS;return d},"~A,JU.BS,~S")});s("JU");v(["java.util.Hashtable","JU.DefaultLogger"],"JU.Logger",["java.lang.Long"],function(){c$=D(JU,"Logger");c$.getProperty=c(c$,"getProperty",
function(a,b){try{var d=Zystem.getProperty("jmol.logger."+a,null);if(null!=d)return d.equalsIgnoreCase("true")}catch(c){if(!H(c,Exception))throw c;}return b},"~S,~B");c$.setLogger=c(c$,"setLogger",function(a){JU.Logger._logger=a;JU.Logger.debugging=JU.Logger.isActiveLevel(5)||JU.Logger.isActiveLevel(6);JU.Logger.debuggingHigh=JU.Logger.debugging&&JU.Logger._activeLevels[6]},"JU.LoggerInterface");c$.isActiveLevel=c(c$,"isActiveLevel",function(a){return null!=JU.Logger._logger&&0<=a&&7>a&&JU.Logger._activeLevels[a]},
"~N");c$.setActiveLevel=c(c$,"setActiveLevel",function(a,b){0>a&&(a=0);7<=a&&(a=6);JU.Logger._activeLevels[a]=b;JU.Logger.debugging=JU.Logger.isActiveLevel(5)||JU.Logger.isActiveLevel(6);JU.Logger.debuggingHigh=JU.Logger.debugging&&JU.Logger._activeLevels[6]},"~N,~B");c$.setLogLevel=c(c$,"setLogLevel",function(a){for(var b=7;0<=--b;)JU.Logger.setActiveLevel(b,b<=a)},"~N");c$.getLevel=c(c$,"getLevel",function(a){switch(a){case 6:return"DEBUGHIGH";case 5:return"DEBUG";case 4:return"INFO";case 3:return"WARN";
case 2:return"ERROR";case 1:return"FATAL"}return"????"},"~N");c$.logLevel=c(c$,"logLevel",function(){return JU.Logger._logLevel});c$.doLogLevel=c(c$,"doLogLevel",function(a){JU.Logger._logLevel=a},"~B");c$.debug=c(c$,"debug",function(a){if(JU.Logger.debugging)try{JU.Logger._logger.debug(a)}catch(b){}},"~S");c$.info=c(c$,"info",function(a){try{JU.Logger.isActiveLevel(4)&&JU.Logger._logger.info(a)}catch(b){}},"~S");c$.warn=c(c$,"warn",function(a){try{JU.Logger.isActiveLevel(3)&&JU.Logger._logger.warn(a)}catch(b){}},
"~S");c$.warnEx=c(c$,"warnEx",function(a,b){try{JU.Logger.isActiveLevel(3)&&JU.Logger._logger.warnEx(a,b)}catch(d){}},"~S,Throwable");c$.error=c(c$,"error",function(a){try{JU.Logger.isActiveLevel(2)&&JU.Logger._logger.error(a)}catch(b){}},"~S");c$.errorEx=c(c$,"errorEx",function(a,b){try{JU.Logger.isActiveLevel(2)&&JU.Logger._logger.errorEx(a,b)}catch(d){}},"~S,Throwable");c$.getLogLevel=c(c$,"getLogLevel",function(){for(var a=7;0<=--a;)if(JU.Logger.isActiveLevel(a))return a;return 0});c$.fatal=c(c$,
"fatal",function(a){try{JU.Logger.isActiveLevel(1)&&JU.Logger._logger.fatal(a)}catch(b){}},"~S");c$.fatalEx=c(c$,"fatalEx",function(a,b){try{JU.Logger.isActiveLevel(1)&&JU.Logger._logger.fatalEx(a,b)}catch(d){}},"~S,Throwable");c$.startTimer=c(c$,"startTimer",function(a){null!=a&&JU.Logger.htTiming.put(a,Long.$valueOf(Zystem.currentTimeMillis()))},"~S");c$.getTimerMsg=c(c$,"getTimerMsg",function(a,b){0==b&&(b=JU.Logger.getTimeFrom(a));return"Time for "+a+": "+b+" ms"},"~S,~N");c$.getTimeFrom=c(c$,
"getTimeFrom",function(a){var b;return null==a||null==(b=JU.Logger.htTiming.get(a))?-1:Zystem.currentTimeMillis()-b.longValue()},"~S");c$.checkTimer=c(c$,"checkTimer",function(a,b){var d=JU.Logger.getTimeFrom(a);0<=d&&!a.startsWith("(")&&JU.Logger.info(JU.Logger.getTimerMsg(a,d));b&&JU.Logger.startTimer(a);return d},"~S,~B");c$.checkMemory=c(c$,"checkMemory",function(){JU.Logger.info("Memory: Total-Free=0; Total=0; Free=0; Max=0")});c$._logger=c$.prototype._logger=new JU.DefaultLogger;E(c$,"LEVEL_FATAL",
1,"LEVEL_ERROR",2,"LEVEL_WARN",3,"LEVEL_INFO",4,"LEVEL_DEBUG",5,"LEVEL_DEBUGHIGH",6,"LEVEL_MAX",7,"_activeLevels",ga(7,!1),"_logLevel",!1,"debugging",!1,"debuggingHigh",!1);JU.Logger._activeLevels[6]=JU.Logger.getProperty("debugHigh",!1);JU.Logger._activeLevels[5]=JU.Logger.getProperty("debug",!1);JU.Logger._activeLevels[4]=JU.Logger.getProperty("info",!0);JU.Logger._activeLevels[3]=JU.Logger.getProperty("warn",!0);JU.Logger._activeLevels[2]=JU.Logger.getProperty("error",!0);JU.Logger._activeLevels[1]=
JU.Logger.getProperty("fatal",!0);JU.Logger._logLevel=JU.Logger.getProperty("logLevel",!1);JU.Logger.debugging=null!=JU.Logger._logger&&(JU.Logger._activeLevels[5]||JU.Logger._activeLevels[6]);JU.Logger.debuggingHigh=JU.Logger.debugging&&JU.Logger._activeLevels[6];c$.htTiming=c$.prototype.htTiming=new java.util.Hashtable});s("JU");N(JU,"LoggerInterface");s("JU");v(["JU.V3"],"JU.Measure","java.lang.Float javajs.api.Interface JU.Lst $.P3 $.P4 $.Quat".split(" "),function(){c$=D(JU,"Measure");c$.computeAngle=
c(c$,"computeAngle",function(a,b,d,c,f,e){c.sub2(a,b);f.sub2(d,b);a=c.angle(f);return e?a/0.017453292:a},"JU.T3,JU.T3,JU.T3,JU.V3,JU.V3,~B");c$.computeAngleABC=c(c$,"computeAngleABC",function(a,b,d,c){var f=new JU.V3,e=new JU.V3;return JU.Measure.computeAngle(a,b,d,f,e,c)},"JU.T3,JU.T3,JU.T3,~B");c$.computeTorsion=c(c$,"computeTorsion",function(a,b,d,c,f){var e=a.x-b.x,h=a.y-b.y;a=a.z-b.z;var j=d.x-b.x,l=d.y-b.y,n=d.z-b.z,m=d.x-c.x,r=d.y-c.y,p=d.z-c.z;c=h*n-a*l;b=a*j-e*n;var q=e*l-h*j;d=l*p-n*r;n=
n*m-j*p;j=j*r-l*m;m=1/(d*d+n*n+j*j);l=Math.sqrt(1/(c*c+b*b+q*q));m=Math.sqrt(m);c=(c*d+b*n+q*j)*l*m;1<c&&(c=1);-1>c&&(c=-1);c=Math.acos(c);e=e*d+h*n+a*j;h=Math.abs(e);c=0<e/h?c:-c;return f?c/0.017453292:c},"JU.T3,JU.T3,JU.T3,JU.T3,~B");c$.computeHelicalAxis=c(c$,"computeHelicalAxis",function(a,b,d){var c=new JU.V3;c.sub2(b,a);var f=d.getTheta(),e=d.getNormal(),h=c.dot(e);1E-4>Math.abs(h)&&(h=0);var j=new JU.V3;j.cross(c,e);0!=j.dot(j)&&j.normalize();var l=new JU.V3,n=JU.V3.newV(e);0==h&&(h=1.4E-45);
n.scale(h);l.sub2(n,c);l.scale(0.5);j.scale(0==f?0:l.length()/Math.tan(3.141592653589793*(f/2/180)));c=JU.V3.newV(j);0!=f&&c.add(l);l=JU.P3.newP(a);l.sub(c);1.4E-45!=h&&e.scale(h);j=JU.P3.newP(l);j.add(e);f=JU.Measure.computeTorsion(a,l,j,b,!0);if(Float.isNaN(f)||1E-4>c.length())f=d.getThetaDirectedV(e);a=Math.abs(0==f?0:360/f);h=Math.abs(1.4E-45==h?0:e.length()*(0==f?1:360/f));return u(-1,[l,e,c,JU.P3.new3(f,h,a),j])},"JU.P3,JU.P3,JU.Quat");c$.getPlaneThroughPoints=c(c$,"getPlaneThroughPoints",function(a,
b,d,c,f,e){a=JU.Measure.getNormalThroughPoints(a,b,d,c,f);e.set4(c.x,c.y,c.z,a);return e},"JU.T3,JU.T3,JU.T3,JU.V3,JU.V3,JU.P4");c$.getPlaneThroughPoint=c(c$,"getPlaneThroughPoint",function(a,b,d){d.set4(b.x,b.y,b.z,-b.dot(a))},"JU.T3,JU.V3,JU.P4");c$.distanceToPlane=c(c$,"distanceToPlane",function(a,b){return null==a?NaN:(a.dot(b)+a.w)/Math.sqrt(a.dot(a))},"JU.P4,JU.T3");c$.directedDistanceToPlane=c(c$,"directedDistanceToPlane",function(a,b,d){a=b.dot(a)+b.w;d=b.dot(d)+b.w;return Math.signum(d)*
a/Math.sqrt(b.dot(b))},"JU.P3,JU.P4,JU.P3");c$.distanceToPlaneD=c(c$,"distanceToPlaneD",function(a,b,d){return null==a?NaN:(a.dot(d)+a.w)/b},"JU.P4,~N,JU.P3");c$.distanceToPlaneV=c(c$,"distanceToPlaneV",function(a,b,d){return null==a?NaN:(a.dot(d)+b)/Math.sqrt(a.dot(a))},"JU.V3,~N,JU.P3");c$.calcNormalizedNormal=c(c$,"calcNormalizedNormal",function(a,b,d,c,f){f.sub2(b,a);c.sub2(d,a);c.cross(f,c);c.normalize()},"JU.T3,JU.T3,JU.T3,JU.T3,JU.T3");c$.getDirectedNormalThroughPoints=c(c$,"getDirectedNormalThroughPoints",
function(a,b,d,c,f,e){b=JU.Measure.getNormalThroughPoints(a,b,d,f,e);null!=c&&(d=JU.P3.newP(a),d.add(f),e=d.distance(c),d.sub2(a,f),e>d.distance(c)&&(f.scale(-1),b=-b));return b},"JU.T3,JU.T3,JU.T3,JU.T3,JU.V3,JU.V3");c$.getNormalThroughPoints=c(c$,"getNormalThroughPoints",function(a,b,d,c,f){JU.Measure.calcNormalizedNormal(a,b,d,c,f);f.setT(a);return-f.dot(c)},"JU.T3,JU.T3,JU.T3,JU.T3,JU.T3");c$.getPlaneProjection=c(c$,"getPlaneProjection",function(a,b,d,c){var f=JU.Measure.distanceToPlane(b,a);
c.set(b.x,b.y,b.z);c.normalize();c.scale(-f);d.add2(a,c)},"JU.P3,JU.P4,JU.P3,JU.V3");c$.getNormalFromCenter=c(c$,"getNormalFromCenter",function(a,b,d,c,f,e,h){b=JU.Measure.getNormalThroughPoints(b,d,c,e,h);a=0<JU.Measure.distanceToPlaneV(e,b,a);a==f&&e.scale(-1);return!a},"JU.P3,JU.P3,JU.P3,JU.P3,~B,JU.V3,JU.V3");c$.getNormalToLine=c(c$,"getNormalToLine",function(a,b,d){d.sub2(a,b);d.cross(d,JU.Measure.axisY);d.normalize();Float.isNaN(d.x)&&d.set(1,0,0)},"JU.P3,JU.P3,JU.V3");c$.getBisectingPlane=
c(c$,"getBisectingPlane",function(a,b,d,c,f){d.scaleAdd2(0.5,b,a);c.setT(b);c.normalize();JU.Measure.getPlaneThroughPoint(d,c,f)},"JU.P3,JU.V3,JU.T3,JU.V3,JU.P4");c$.projectOntoAxis=c(c$,"projectOntoAxis",function(a,b,d,c){c.sub2(a,b);var f=c.dot(d);a.scaleAdd2(f,d,b);c.sub2(a,b)},"JU.P3,JU.P3,JU.V3,JU.V3");c$.calcBestAxisThroughPoints=c(c$,"calcBestAxisThroughPoints",function(a,b,d,c,f){var e=a.length;b.setT(a[0]);d.sub2(a[e-1],b);d.normalize();JU.Measure.calcAveragePointN(a,e,b);for(var h=0;h++<
f&&0.001<JU.Measure.findAxis(a,e,b,d,c););a=JU.P3.newP(a[0]);JU.Measure.projectOntoAxis(a,b,d,c);b.setT(a)},"~A,JU.P3,JU.V3,JU.V3,~N");c$.findAxis=c(c$,"findAxis",function(a,b,d,c,f){for(var e=new JU.V3,h=new JU.V3,j=new JU.P3,l=new JU.P3,n=JU.V3.newV(c),m=0;0<=--b;)j.setT(a[b]),l.setT(j),JU.Measure.projectOntoAxis(l,d,c,f),h.sub2(j,l),h.cross(f,h),e.add(h),m+=f.lengthSquared();a=JU.V3.newV(e);a.scale(1/m);h.cross(a,c);c.add(h);c.normalize();h.sub2(c,n);return h.length()},"~A,~N,JU.P3,JU.V3,JU.V3");
c$.calcAveragePoint=c(c$,"calcAveragePoint",function(a,b,d){d.set((a.x+b.x)/2,(a.y+b.y)/2,(a.z+b.z)/2)},"JU.P3,JU.P3,JU.P3");c$.calcAveragePointN=c(c$,"calcAveragePointN",function(a,b,d){d.setT(a[0]);for(var c=1;c<b;c++)d.add(a[c]);d.scale(1/b)},"~A,~N,JU.P3");c$.transformPoints=c(c$,"transformPoints",function(a,b,d){for(var c=new JU.Lst,f=0;f<a.size();f++){var e=JU.P3.newP(a.get(f));e.sub(d);b.rotTrans(e);e.add(d);c.addLast(e)}return c},"JU.Lst,JU.M4,JU.P3");c$.isInTetrahedron=c(c$,"isInTetrahedron",
function(a,b,d,c,f,e,h,j,l){var n=0<=JU.Measure.distanceToPlane(JU.Measure.getPlaneThroughPoints(c,f,b,h,j,e),a);if(n!=0<=JU.Measure.distanceToPlane(JU.Measure.getPlaneThroughPoints(b,f,d,h,j,e),a)||n!=0<=JU.Measure.distanceToPlane(JU.Measure.getPlaneThroughPoints(d,f,c,h,j,e),a))return!1;a=JU.Measure.distanceToPlane(JU.Measure.getPlaneThroughPoints(b,d,c,h,j,e),a);if(l)return n==0<=a;f=JU.Measure.distanceToPlane(e,f);return 0>=f*a||Math.abs(f)>Math.abs(a)},"JU.P3,JU.P3,JU.P3,JU.P3,JU.P3,JU.P4,JU.V3,JU.V3,~B");
c$.getIntersectionPP=c(c$,"getIntersectionPP",function(a,b){var d=a.x,c=a.y,f=a.z,e=a.w,h=b.x,j=b.y,l=b.z,n=b.w,m=JU.V3.new3(d,c,f),r=JU.V3.new3(h,j,l),p=new JU.V3;p.cross(m,r);var m=Math.abs(p.x),r=Math.abs(p.y),q=Math.abs(p.z);switch(m>r?m>q?1:3:r>q?2:3){case 1:m=0;r=c*l-j*f;if(0.01>Math.abs(r))return null;f=(f*n-l*e)/r;d=(j*e-n*c)/r;break;case 2:r=d*l-h*f;if(0.01>Math.abs(r))return null;m=(f*n-l*e)/r;f=0;d=(h*e-n*d)/r;break;default:r=d*j-h*c;if(0.01>Math.abs(r))return null;m=(c*n-j*e)/r;f=(h*e-
n*d)/r;d=0}c=new JU.Lst;c.addLast(JU.P3.new3(m,f,d));p.normalize();c.addLast(p);return c},"JU.P4,JU.P4");c$.getIntersection=c(c$,"getIntersection",function(a,b,d,c,f,e){JU.Measure.getPlaneProjection(a,d,c,f);f.set(d.x,d.y,d.z);f.normalize();null==b&&(b=JU.V3.newV(f));d=b.dot(f);if(0.01>Math.abs(d))return null;e.sub2(c,a);c.scaleAdd2(e.dot(f)/d,b,a);return c},"JU.P3,JU.V3,JU.P4,JU.P3,JU.V3,JU.V3");c$.calculateQuaternionRotation=c(c$,"calculateQuaternionRotation",function(a,b){b[1]=NaN;var d=new JU.Quat,
c=a[0],f=a[1],e=c.length-1;if(2>e||c.length!=f.length)return d;for(var h=0,j=0,l=0,n=0,m=0,r=0,p=0,q=0,s=0,t=new JU.P3,u=new JU.P3,v=c[0],w=f[0],e=e+1;1<=--e;)t.sub2(c[e],v),u.sub2(f[e],w),h+=t.x*u.x,j+=t.x*u.y,l+=t.x*u.z,n+=t.y*u.x,m+=t.y*u.y,r+=t.y*u.z,p+=t.z*u.x,q+=t.z*u.y,s+=t.z*u.z;b[0]=JU.Measure.getRmsd(a,d);d=X(4,4,0);d[0][0]=h+m+s;d[0][1]=d[1][0]=r-q;d[0][2]=d[2][0]=p-l;d[0][3]=d[3][0]=j-n;d[1][1]=h-m-s;d[1][2]=d[2][1]=j+n;d[1][3]=d[3][1]=p+l;d[2][2]=-h+m-s;d[2][3]=d[3][2]=r+q;d[3][3]=-h-
m+s;h=javajs.api.Interface.getInterface("JU.Eigen").setM(d).getEigenvectorsFloatTransposed()[3];d=JU.Quat.newP4(JU.P4.new4(h[1],h[2],h[3],h[0]));b[1]=JU.Measure.getRmsd(a,d);return d},"~A,~A");c$.getTransformMatrix4=c(c$,"getTransformMatrix4",function(a,b,d,c){a=JU.Measure.getCenterAndPoints(a);var f=JU.Measure.getCenterAndPoints(b);b=K(2,0);var e=JU.Measure.calculateQuaternionRotation(u(-1,[a,f]),b).getMatrix();null==c?e.rotate(a[0]):c.setT(a[0]);c=JU.V3.newVsub(f[0],a[0]);d.setMV(e,c);return b[1]},
"JU.Lst,JU.Lst,JU.M4,JU.P3");c$.getCenterAndPoints=c(c$,"getCenterAndPoints",function(a){var b=a.size(),d=Array(b+1);d[0]=new JU.P3;if(0<b){for(var c=0;c<b;c++)d[0].add(d[c+1]=a.get(c));d[0].scale(1/b)}return d},"JU.Lst");c$.getRmsd=c(c$,"getRmsd",function(a,b){for(var d=0,c=a[0],f=a[1],e=c[0],h=f[0],j=c.length-1,l=new JU.P3,n=j+1;1<=--n;)l.sub2(c[n],e),b.transform2(l,l).add(h),d+=l.distanceSquared(f[n]);return Math.sqrt(d/j)},"~A,JU.Quat");E(c$,"radiansPerDegree",0.017453292519943295);c$.axisY=c$.prototype.axisY=
JU.V3.new3(0,1,0)});s("JU");v(null,"JU.MeshSurface","java.lang.Float JU.AU $.P3 J.api.Interface JU.BoxInfo $.C $.Geodesic $.TempArray".split(" "),function(){c$=t(function(){this.meshType=this.oabc=this.slicer=this.vwr=null;this.vc=0;this.surfaceAtoms=this.vertexSource=this.vvs=this.vs=null;this.pc=0;this.pis=null;this.haveQuads=this.isDrawPolygon=this.colorsExplicit=!1;this.colixBack=this.colix=0;this.isColorSolid=!0;this.normalsTemp=this.normals=this.vcs=this.pcs=this.altVertices=this.offset=null;
this.normixCount=this.normalCount=0;this.vertexSets=this.surfaceSet=this.mat4=this.bsPolygons=null;this.nSets=0;this.dataOnly=!1;this.vertexCount0=this.polygonCount0=this.iC=this.iB=this.iA=this.lastColix=this.lastColor=0;this.bsSlabGhost=this.bsSlabDisplay=null;this.slabColix=this.slabMeshType=0;this.slabOptions=this.bsDisplay=null;this.mergePolygonCount0=this.mergeVertexCount0=0;this.isMerged=!1;p(this,arguments)},JU,"MeshSurface");c(c$,"getMeshSlicer",function(){return null==this.slicer?this.slicer=
J.api.Interface.getInterface("JU.MeshSlicer",this.vwr,"script").set(this):this.slicer});c$.newMesh=c(c$,"newMesh",function(a,b,d,c,f,e){var h=new JU.MeshSurface;h.pis=c;a?h.altVertices=b:h.vs=b;h.vc=0==d?b.length:d;h.normals=f;h.normalCount=0==e&&null!=f?f.length:e;return h},"~B,~A,~N,~A,~A,~N");c(c$,"getVertices",function(){return null==this.altVertices?this.vs:this.altVertices});c(c$,"getFaces",function(){return this.pis});c(c$,"setColix",function(a){this.colix=a},"~N");c(c$,"setColixBack",function(a){this.colixBack=
a},"~N");c(c$,"addV",function(a,b){0==this.vc?this.vs=Array(25):this.vc==this.vs.length&&(this.vs=JU.AU.doubleLength(this.vs));this.vs[this.vc]=b?JU.P3.newP(a):a;return this.vc++},"JU.T3,~B");c(c$,"addTriangle",function(a,b,d){this.addPolygon(B(-1,[a,b,d]),null)},"~N,~N,~N");c(c$,"addQuad",function(a,b,d,c){this.haveQuads=!0;this.addPolygon(B(-1,[a,b,d,c]),null)},"~N,~N,~N,~N");c(c$,"setPolygonCount",function(a){this.pc=a;if(!(0>a)&&(null==this.pis||a>this.pis.length))this.pis=JU.AU.newInt2(a)},"~N");
c(c$,"addVCVal",function(a,b,d){0==this.vc?this.vvs=K(25,0):this.vc>=this.vvs.length&&(this.vvs=JU.AU.doubleLengthF(this.vvs));this.vvs[this.vc]=b;return this.addV(a,d)},"JU.T3,~N,~B");c(c$,"addTriangleCheck",function(a,b,d,c,f,e){return null==this.vs||null!=this.vvs&&(Float.isNaN(this.vvs[a])||Float.isNaN(this.vvs[b])||Float.isNaN(this.vvs[d]))||Float.isNaN(this.vs[a].x)||Float.isNaN(this.vs[b].x)||Float.isNaN(this.vs[d].x)?-1:this.addPolygonV3(a,b,d,c,f,e,null)},"~N,~N,~N,~N,~N,~N");c(c$,"addPolygonV3",
function(a,b,d,c,f,e,h){return this.dataOnly?this.addPolygon(B(-1,[a,b,d,c]),h):this.addPolygonC(B(-1,[a,b,d,c,f]),e,h,0>f)},"~N,~N,~N,~N,~N,~N,JU.BS");c(c$,"addPolygonC",function(a,b,d,c){if(0!=b){if(null==this.pcs||0==this.pc)this.lastColor=0;c?this.colorsExplicit=!0:(null==this.pcs?this.pcs=T(25,0):this.pc>=this.pcs.length&&(this.pcs=JU.AU.doubleLengthShort(this.pcs)),this.pcs[this.pc]=c?2047:b==this.lastColor?this.lastColix:this.lastColix=JU.C.getColix(this.lastColor=b))}return this.addPolygon(a,
d)},"~A,~N,JU.BS,~B");c(c$,"addPolygon",function(a,b){var d=this.pc;0==d?this.pis=JU.AU.newInt2(25):d==this.pis.length&&(this.pis=JU.AU.doubleLength(this.pis));null!=b&&b.set(d);this.pis[this.pc++]=a;return d},"~A,JU.BS");c(c$,"invalidatePolygons",function(){for(var a=this.pc;--a>=this.mergePolygonCount0;)if((null==this.bsSlabDisplay||this.bsSlabDisplay.get(a))&&null==this.setABC(a))this.pis[a]=null});c(c$,"setABC",function(a){if(null!=this.bsSlabDisplay&&!this.bsSlabDisplay.get(a)&&(null==this.bsSlabGhost||
!this.bsSlabGhost.get(a)))return null;a=this.pis[a];if(null==a||3>a.length)return null;this.iA=a[0];this.iB=a[1];this.iC=a[2];return null==this.vvs||!Float.isNaN(this.vvs[this.iA])&&!Float.isNaN(this.vvs[this.iB])&&!Float.isNaN(this.vvs[this.iC])?a:null},"~N");c(c$,"setTranslucentVertices",function(){},"JU.BS");c(c$,"getSlabColor",function(){return null==this.bsSlabGhost?null:JU.C.getHexCode(this.slabColix)});c(c$,"getSlabType",function(){return null!=this.bsSlabGhost&&1073742018==this.slabMeshType?
"mesh":null});c(c$,"resetSlab",function(){null!=this.slicer&&this.slicer.slabPolygons(JU.TempArray.getSlabObjectType(1073742333,null,!1,null),!1)});c(c$,"slabPolygonsList",function(a,b){this.getMeshSlicer();for(var d=0;d<a.size()&&this.slicer.slabPolygons(a.get(d),b);d++);},"JU.Lst,~B");c(c$,"slabBrillouin",function(){},"~A");c(c$,"getResolution",function(){return 0});c$.getSphereData=c(c$,"getSphereData",function(a){JU.Geodesic.createGeodesic(a);var b=JU.Geodesic.getVertexCount(a),d=JU.Geodesic.getFaceVertexes(a),
c=w(d.length/3);a=JU.AU.newInt2(c);for(var f=0,e=0;f<c;f++)a[f]=B(-1,[d[e++],d[e++],d[e++]]);d=Array(b);for(f=0;f<b;f++)d[f]=JU.Geodesic.getVertexVector(f);return JU.MeshSurface.newMesh(!0,d,0,a,d,0)},"~N");c(c$,"setBox",function(a,b){a.set(3.4028235E38,3.4028235E38,3.4028235E38);b.set(-3.4028235E38,-3.4028235E38,-3.4028235E38);for(var d=0;d<this.vc;d++){var c=this.vs[d];Float.isNaN(c.x)||JU.BoxInfo.addPoint(c,a,b,0)}},"JU.P3,JU.P3");c(c$,"setBoundingBox",function(){},"~A");c$.getSphericalInterpolationFraction=
c(c$,"getSphericalInterpolationFraction",function(a,b,d,c){b=Math.abs(a+b)/c;d=Math.abs(a+d)/c;a/=c;c=b*b;var f=c-d*d+1;return(f+(b<d?1:-1)*Math.sqrt(f*f+4*(a*a-c)))/2},"~N,~N,~N,~N");E(c$,"SEED_COUNT",25,"P_CHECK",3,"P_CONTOUR",4,"P_EXPLICIT_COLOR",4)});s("JU");v(["JU.Geodesic"],"JU.Normix",["JU.BS"],function(){c$=D(JU,"Normix");c$.getNormixCount=c(c$,"getNormixCount",function(){return 1<JU.Normix.normixCount?JU.Normix.normixCount:JU.Normix.normixCount=JU.Geodesic.getVertexCount(3)});c$.newVertexBitSet=
c(c$,"newVertexBitSet",function(){return JU.BS.newN(JU.Normix.getNormixCount())});c$.getVertexVectors=c(c$,"getVertexVectors",function(){null==JU.Normix.vertexVectors&&(JU.Normix.vertexVectors=JU.Geodesic.getVertexVectors());return JU.Normix.vertexVectors});c$.setInverseNormixes=c(c$,"setInverseNormixes",function(){if(null==JU.Normix.inverseNormixes){JU.Normix.getNormixCount();JU.Normix.getVertexVectors();JU.Normix.inverseNormixes=T(JU.Normix.normixCount,0);for(var a=new JU.BS,b=JU.Normix.normixCount;0<=
--b;){var d=JU.Normix.vertexVectors[b];JU.Normix.inverseNormixes[b]=JU.Normix.getNormix(-d.x,-d.y,-d.z,3,a)}}});c$.getInverseNormix=c(c$,"getInverseNormix",function(a){return JU.Normix.inverseNormixes[a]},"~N");c$.getNeighborVertexArrays=c(c$,"getNeighborVertexArrays",function(){null==JU.Normix.neighborVertexesArrays&&(JU.Normix.neighborVertexesArrays=JU.Geodesic.getNeighborVertexesArrays());return JU.Normix.neighborVertexesArrays});c$.getNormixV=c(c$,"getNormixV",function(a,b){return JU.Normix.getNormix(a.x,
a.y,a.z,3,b)},"JU.V3,JU.BS");c$.get2SidedNormix=c(c$,"get2SidedNormix",function(a,b){return~JU.Normix.getNormixV(a,b)},"JU.V3,JU.BS");c$.getNormix=c(c$,"getNormix",function(a,b,d,c,f){var e,h;0<=d?(e=0,h=d-1):(e=11,h=d- -1);f.clearAll();f.set(e);JU.Normix.getVertexVectors();JU.Normix.getNeighborVertexArrays();h=a*a+b*b+h*h;for(var j=0;j<=c;++j)for(var l=JU.Normix.neighborVertexesArrays[j],n=6*e,m=n+(12>e?5:6);--m>=n;){var r=l[m];if(!f.get(r)){f.set(r);var p=JU.Normix.vertexVectors[r],q;q=p.x-a;var s=
q*q;s>=h||(q=p.y-b,s+=q*q,s>=h||(q=p.z-d,s+=q*q,s>=h||(e=r,h=s)))}}return e},"~N,~N,~N,~N,JU.BS");E(c$,"NORMIX_GEODESIC_LEVEL",3,"normixCount",0,"vertexVectors",null,"inverseNormixes",null,"neighborVertexesArrays",null,"NORMIX_NULL",9999)});s("JU");v(null,"JU.Parser",["java.lang.Float","JU.PT"],function(){c$=D(JU,"Parser");c$.parseStringInfestedFloatArray=c(c$,"parseStringInfestedFloatArray",function(a,b,d){return JU.Parser.parseFloatArrayBsData(JU.PT.getTokens(a),b,d)},"~S,JU.BS,~A");c$.parseFloatArrayBsData=
c(c$,"parseFloatArrayBsData",function(a,b,d){for(var c=d.length,f=a.length,e=0,h=0,j=null!=b,l=j?b.nextSetBit(0):0;0<=l&&l<c&&e<f;l=j?b.nextSetBit(l+1):l+1){for(var n;Float.isNaN(n=JU.PT.parseFloat(a[e++]))&&e<f;);Float.isNaN(n)||(d[h=l]=n);if(e==f)break}return h+1},"~A,JU.BS,~A");c$.parseFloatArrayFromMatchAndField=c(c$,"parseFloatArrayFromMatchAndField",function(a,b,d,c,f,e,h,j,l){var n,m=-1,r=null!=f,p=JU.Parser.markLines(a,0<=a.indexOf("\n")?"\n":";");l=1>=l||l>=p.length?0:l-1;var q=0==l?0:p[l-
1],s=p.length;null==j&&(j=K(s-l,0));for(var t=j.length,u=0>=h?Math.max(e,d):Math.max(e+h,d+c)-1,v=null!=b;l<s;l++){var w=a.substring(q,p[l]).trim(),q=p[l],A=0>=h?JU.PT.getTokens(w):null;if(0>=h){if(A.length<u||Float.isNaN(n=JU.PT.parseFloat(A[e-1])))continue}else if(w.length<u||Float.isNaN(n=JU.PT.parseFloat(w.substring(e-1,e+h-1))))continue;if(r){w=JU.PT.parseInt(null==A?w.substring(d-1,d+c-1):A[d-1]);if(-2147483648==w||0>w||w>=t||0>(w=f[w]))continue;v&&b.set(w)}else{v?m=b.nextSetBit(m+1):m++;if(0>
m||m>=t)break;w=m}j[w]=n}return j},"~S,JU.BS,~N,~N,~A,~N,~N,~A,~N");c$.fixDataString=c(c$,"fixDataString",function(a){a=a.$replace(";",0>a.indexOf("\n")?"\n":" ");a=JU.PT.trim(a,"\n \t");a=JU.PT.rep(a,"\n ","\n");return a=JU.PT.rep(a,"\n\n","\n")},"~S");c$.markLines=c(c$,"markLines",function(a,b){for(var d=0,c=a.length;0<=--c;)a.charAt(c)==b&&d++;for(var c=B(d+1,0),f=d=0;0<=(f=a.indexOf(b,f));)c[d++]=++f;c[d]=a.length;return c},"~S,~S")});s("JU");v(["JU.P3"],"JU.Point3fi",null,function(){c$=t(function(){this.mi=
-1;this.sZ=this.sY=this.sX=this.i=0;this.sD=-1;p(this,arguments)},JU,"Point3fi",JU.P3,Cloneable);c(c$,"copy",function(){try{return this.clone()}catch(a){if(H(a,CloneNotSupportedException))return null;throw a;}})});s("JU");c$=t(function(){this.height=this.width=this.y=this.x=0;p(this,arguments)},JU,"Rectangle");c(c$,"contains",function(a,b){return a>=this.x&&b>=this.y&&a-this.x<this.width&&b-this.y<this.height},"~N,~N");s("JU");v(null,"JU.Rgb16",["JU.SB"],function(){c$=t(function(){this.b=this.g=this.r=
0;p(this,arguments)},JU,"Rgb16");q(c$,function(){});c$.newI=c(c$,"newI",function(a){var b=new JU.Rgb16;b.setInt(a);return b},"~N");c(c$,"setInt",function(a){this.r=a>>8&65280|128;this.g=a&65280|128;this.b=a<<8&65280|128},"~N");c(c$,"setRgb",function(a){this.r=a.r;this.g=a.g;this.b=a.b},"JU.Rgb16");c(c$,"diffDiv",function(a,b,d){this.r=w((a.r-b.r)/d);this.g=w((a.g-b.g)/d);this.b=w((a.b-b.b)/d)},"JU.Rgb16,JU.Rgb16,~N");c(c$,"setAndIncrement",function(a,b){this.r=a.r;a.r+=b.r;this.g=a.g;a.g+=b.g;this.b=
a.b;a.b+=b.b},"JU.Rgb16,JU.Rgb16");c(c$,"getArgb",function(){return 4278190080|this.r<<8&16711680|this.g&65280|this.b>>8});j(c$,"toString",function(){return(new JU.SB).append("Rgb16(").appendI(this.r).appendC(",").appendI(this.g).appendC(",").appendI(this.b).append(" -> ").appendI(this.r>>8&255).appendC(",").appendI(this.g>>8&255).appendC(",").appendI(this.b>>8&255).appendC(")").toString()})});s("JU");v(["JU.AU","$.V3"],"JU.Shader",["JU.CU","JU.C"],function(){c$=t(function(){this.zLight=this.yLight=
this.xLight=0;this.lightSource=null;this.specularOn=!0;this.usePhongExponent=!1;this.ambientPercent=45;this.diffusePercent=84;this.specularExponent=6;this.specularPercent=22;this.specularPower=40;this.phongExponent=64;this.specularFactor=this.intenseFraction=this.diffuseFactor=this.ambientFraction=0;this.ashadesGreyscale=this.ashades=null;this.celOn=!1;this.celPower=10;this.celZ=this.celRGB=0;this.useLight=!1;this.sphereShadeIndexes=null;this.seed=305419897;this.ellipsoidShades=this.sphereShapeCache=
null;this.nIn=this.nOut=0;p(this,arguments)},JU,"Shader");O(c$,function(){this.lightSource=new JU.V3;this.ambientFraction=this.ambientPercent/100;this.diffuseFactor=this.diffusePercent/100;this.intenseFraction=this.specularPower/100;this.specularFactor=this.specularPercent/100;this.ashades=JU.AU.newInt2(128);this.sphereShadeIndexes=Q(65536,0);this.sphereShapeCache=JU.AU.newInt2(128)});q(c$,function(){this.setLightSource(-1,-1,2.5)});c(c$,"setLightSource",function(a,b,d){this.lightSource.set(a,b,d);
this.lightSource.normalize();this.xLight=this.lightSource.x;this.yLight=this.lightSource.y;this.zLight=this.lightSource.z},"~N,~N,~N");c(c$,"setCel",function(a,b,d){a=a&&0!=b;d=JU.C.getArgb(JU.C.getBgContrast(d));d=4278190080==d?4278453252:-1==d?-2:d+1;this.celOn==a&&this.celRGB==d&&this.celPower==b||(this.celOn=a,this.celPower=b,this.useLight=!this.celOn||0<b,this.celZ=1-Math.pow(2,-Math.abs(b)/10),this.celRGB=d,this.flushCaches())},"~B,~N,~N");c(c$,"flushCaches",function(){this.checkShades(JU.C.colixMax);
for(var a=JU.C.colixMax;0<=--a;)this.ashades[a]=null;this.calcSphereShading();for(a=128;0<=--a;)this.sphereShapeCache[a]=null;this.ellipsoidShades=null});c(c$,"setLastColix",function(a,b){JU.C.allocateColix(a,!0);this.checkShades(2047);b&&JU.C.setLastGrey(a);this.ashades[2047]=this.getShades2(a,!1)},"~N,~B");c(c$,"getShades",function(a){this.checkShades(JU.C.colixMax);a&=-30721;var b=this.ashades[a];null==b&&(b=this.ashades[a]=this.getShades2(JU.C.argbs[a],!1));return b},"~N");c(c$,"getShadesG",function(a){this.checkShades(JU.C.colixMax);
a&=-30721;null==this.ashadesGreyscale&&(this.ashadesGreyscale=JU.AU.newInt2(this.ashades.length));var b=this.ashadesGreyscale[a];null==b&&(b=this.ashadesGreyscale[a]=this.getShades2(JU.C.argbs[a],!0));return b},"~N");c(c$,"checkShades",function(a){null!=this.ashades&&this.ashades.length>=a||(2047==a&&a++,this.ashades=JU.AU.arrayCopyII(this.ashades,a),null!=this.ashadesGreyscale&&(this.ashadesGreyscale=JU.AU.arrayCopyII(this.ashadesGreyscale,a)))},"~N");c(c$,"getShades2",function(a,b){var d=B(64,0);
if(0==a)return d;for(var c=a>>16&255,f=a>>8&255,e=a&255,h=0,j=0,l=0,n=this.ambientFraction;;)if(h=c*n+0.5,j=f*n+0.5,l=e*n+0.5,0<n&&4>h&&4>j&&4>l)c++,f++,e++,0.1>n&&(n+=0.1),a=JU.CU.rgb(w(Math.floor(c)),w(Math.floor(f)),w(Math.floor(e)));else break;var m=0,n=(1-n)/52,c=c*n,f=f*n,e=e*n;if(this.celOn){n=JU.CU.rgb(w(Math.floor(h)),w(Math.floor(j)),w(Math.floor(l)));if(0<=this.celPower)for(;32>m;++m)d[m]=n;j+=32*f;l+=32*e;for(n=JU.CU.rgb(w(Math.floor(h+32*c)),w(Math.floor(j)),w(Math.floor(l)));64>m;m++)d[m]=
n;d[0]=d[1]=this.celRGB}else{for(;52>m;++m)d[m]=JU.CU.rgb(w(Math.floor(h)),w(Math.floor(j)),w(Math.floor(l))),h+=c,j+=f,l+=e;d[m++]=a;n=this.intenseFraction/(64-m);c=(255.5-h)*n;f=(255.5-j)*n;for(e=(255.5-l)*n;64>m;m++)h+=c,j+=f,l+=e,d[m]=JU.CU.rgb(w(Math.floor(h)),w(Math.floor(j)),w(Math.floor(l)))}if(b)for(;0<=--m;)d[m]=JU.CU.toFFGGGfromRGB(d[m]);return d},"~N,~B");c(c$,"getShadeIndex",function(a,b,d){var c=Math.sqrt(a*a+b*b+d*d);return Math.round(63*this.getShadeF(a/c,b/c,d/c))},"~N,~N,~N");c(c$,
"getShadeB",function(a,b,d){return Math.round(63*this.getShadeF(a,b,d))},"~N,~N,~N");c(c$,"getShadeFp8",function(a,b,d){var c=Math.sqrt(a*a+b*b+d*d);return w(Math.floor(16128*this.getShadeF(a/c,b/c,d/c)))},"~N,~N,~N");c(c$,"getShadeF",function(a,b,d){b=this.useLight?a*this.xLight+b*this.yLight+d*this.zLight:d;if(0>=b)return 0;a=b*this.diffuseFactor;if(this.specularOn&&(b=2*b*d-this.zLight,0<b)){if(this.usePhongExponent)b=Math.pow(b,this.phongExponent);else for(var c=this.specularExponent;0<=--c&&
1E-4<b;)b*=b;a+=b*this.specularFactor}return this.celOn&&d<this.celZ?0:1<a?1:a},"~N,~N,~N");c(c$,"getShadeN",function(a,b,d,c){b=w(Math.floor(16128*this.getShadeF(a/c,b/c,d/c)));a=b>>8;if(!this.useLight)return a;(b&255)>this.nextRandom8Bit()&&++a;b=this.seed&65535;21845>b&&0<a?--a:43690<b&&63>a&&++a;return a},"~N,~N,~N,~N");c(c$,"calcSphereShading",function(){for(var a=-127.5,b=0;256>b;++a,++b)for(var d=-127.5,c=a*a,f=0;256>f;++d,++f){var e=0,h=16900-c-d*d;0<h&&(e=Math.sqrt(h),e=this.getShadeN(a,
d,e,130));this.sphereShadeIndexes[(f<<8)+b]=e}});c(c$,"nextRandom8Bit",function(){var a=this.seed;this.seed=a=(a<<16)+(a<<1)+a&2147483647;return a>>23});c(c$,"getEllipsoidShade",function(a,b,d,c,f){var e=f.m00*a+f.m01*b+f.m02*d+f.m03,h=f.m10*a+f.m11*b+f.m12*d+f.m13;a=f.m20*a+f.m21*b+f.m22*d+f.m23;c=Math.min(c/2,45)/Math.sqrt(e*e+h*h+a*a);e=A(-e*c);h=A(-h*c);c=A(a*c);if(a=-20>e||20<=e||-20>h||20<=h||0>c||40<=c){for(;0==e%2&&0==h%2&&0==c%2&&0<e+h+c;)e>>=1,h>>=1,c>>=1;a=-20>e||20<=e||-20>h||20<=h||0>
c||40<=c}a?this.nOut++:this.nIn++;return a?this.getShadeIndex(e,h,c):this.ellipsoidShades[e+20][h+20][c]},"~N,~N,~N,~N,JU.M4");c(c$,"createEllipsoidShades",function(){this.ellipsoidShades=Q(40,40,40,0);for(var a=0;40>a;a++)for(var b=0;40>b;b++)for(var d=0;40>d;d++)this.ellipsoidShades[a][b][d]=this.getShadeIndex(a-20,b-20,d)});E(c$,"SHADE_INDEX_MAX",64,"SHADE_INDEX_LAST",63,"SHADE_INDEX_NORMAL",52,"SHADE_INDEX_NOISY_LIMIT",56,"SLIM",20,"SDIM",40,"maxSphereCache",128)});s("JU");v(null,"JU.SimpleUnitCell",
"java.lang.Float JU.AU $.M4 $.P3 $.P4 $.PT $.V3 JU.Escape".split(" "),function(){c$=t(function(){this.matrixFractionalToCartesian=this.matrixCartesianToFractional=this.unitCellParams=null;this.dimension=this.c_=this.b_=this.a_=this.cB_=this.cA_=this.sinGamma=this.cosGamma=this.sinBeta=this.cosBeta=this.sinAlpha=this.cosAlpha=this.gamma=this.beta=this.alpha=this.c=this.b=this.a=this.nc=this.nb=this.na=this.volume=0;this.matrixFtoCNoOffset=this.matrixCtoFNoOffset=this.fractionalOrigin=null;p(this,arguments)},
JU,"SimpleUnitCell");c(c$,"isSupercell",function(){return 1<this.na||1<this.nb||1<this.nc});c$.isValid=c(c$,"isValid",function(a){return null!=a&&(0<a[0]||14<a.length&&!Float.isNaN(a[14]))},"~A");q(c$,function(){this.fractionalOrigin=new JU.P3});c$.newA=c(c$,"newA",function(a){var b=new JU.SimpleUnitCell;b.init(a);return b},"~A");c(c$,"init",function(a){null==a&&(a=K(-1,[1,1,1,90,90,90]));if(JU.SimpleUnitCell.isValid(a)){this.unitCellParams=JU.AU.arrayCopyF(a,a.length);var b=!1;this.a=a[0];this.b=
a[1];this.c=a[2];this.alpha=a[3];this.beta=a[4];this.gamma=a[5];-1==this.gamma&&(b=!0,this.gamma=120);var d=this.na=Math.max(1,25<=a.length&&!Float.isNaN(a[22])?A(a[22]):1),c=this.nb=Math.max(1,25<=a.length&&!Float.isNaN(a[23])?A(a[23]):1),f=this.nc=Math.max(1,25<=a.length&&!Float.isNaN(a[24])?A(a[24]):1);if(25<a.length&&!Float.isNaN(a[25]))var e=a[25],d=d*e,c=c*e,f=f*e;else d=c=f=1;if(0>=this.a){var e=JU.V3.new3(a[6],a[7],a[8]),h=JU.V3.new3(a[9],a[10],a[11]),j=JU.V3.new3(a[12],a[13],a[14]);this.setABC(e,
h,j);0>this.c&&(a=JU.AU.arrayCopyF(a,-1),0>this.b&&(h.set(0,0,1),h.cross(h,e),0.001>h.length()&&h.set(0,1,0),h.normalize(),a[9]=h.x,a[10]=h.y,a[11]=h.z),0>this.c&&(j.cross(e,h),j.normalize(),a[12]=j.x,a[13]=j.y,a[14]=j.z))}this.a*=d;0>=this.b?this.dimension=this.b=this.c=1:0>=this.c?(this.c=1,this.b*=c,this.dimension=2):(this.b*=c,this.c*=f,this.dimension=3);this.setCellParams();if(21<a.length&&!Float.isNaN(a[21])){b=K(16,0);for(e=0;16>e;e++){switch(e%4){case 0:h=d;break;case 1:h=c;break;case 2:h=
f;break;default:h=1}b[e]=a[6+e]*h}this.matrixCartesianToFractional=JU.M4.newA16(b);this.matrixCartesianToFractional.getTranslation(this.fractionalOrigin);this.matrixFractionalToCartesian=JU.M4.newM4(this.matrixCartesianToFractional).invert();1==a[0]&&this.setParamsFromMatrix()}else 14<a.length&&!Float.isNaN(a[14])?(e=this.matrixFractionalToCartesian=new JU.M4,e.setColumn4(0,a[6]*d,a[7]*d,a[8]*d,0),e.setColumn4(1,a[9]*c,a[10]*c,a[11]*c,0),e.setColumn4(2,a[12]*f,a[13]*f,a[14]*f,0)):(e=this.matrixFractionalToCartesian=
new JU.M4,b?(e.setColumn4(0,-this.b*this.cosGamma,-this.b*this.sinGamma,0,0),e.setColumn4(1,-this.b*this.cosGamma,this.b*this.sinGamma,0,0)):(e.setColumn4(0,this.a,0,0,0),e.setColumn4(1,this.b*this.cosGamma,this.b*this.sinGamma,0,0)),e.setColumn4(2,this.c*this.cosBeta,this.c*(this.cosAlpha-this.cosBeta*this.cosGamma)/this.sinGamma,this.volume/(this.a*this.b*this.sinGamma),0)),e.setColumn4(3,0,0,0,1),this.matrixCartesianToFractional=JU.M4.newM4(this.matrixFractionalToCartesian).invert();this.matrixCtoFNoOffset=
this.matrixCartesianToFractional;this.matrixFtoCNoOffset=this.matrixFractionalToCartesian}},"~A");c(c$,"setParamsFromMatrix",function(){var a=JU.V3.new3(1,0,0),b=JU.V3.new3(0,1,0),d=JU.V3.new3(0,0,1);this.matrixFractionalToCartesian.rotate(a);this.matrixFractionalToCartesian.rotate(b);this.matrixFractionalToCartesian.rotate(d);this.setABC(a,b,d);this.setCellParams()});c(c$,"setABC",function(a,b,d){this.a=a.length();this.b=b.length();this.c=d.length();0!=this.a&&(0==this.b?this.b=this.c=-1:0==this.c&&
(this.c=-1),this.alpha=0>this.b||0>this.c?90:b.angle(d)/0.017453292,this.beta=0>this.c?90:a.angle(d)/0.017453292,this.gamma=0>this.b?90:a.angle(b)/0.017453292)},"JU.V3,JU.V3,JU.V3");c(c$,"setCellParams",function(){this.cosAlpha=Math.cos(0.017453292*this.alpha);this.sinAlpha=Math.sin(0.017453292*this.alpha);this.cosBeta=Math.cos(0.017453292*this.beta);this.sinBeta=Math.sin(0.017453292*this.beta);this.cosGamma=Math.cos(0.017453292*this.gamma);this.sinGamma=Math.sin(0.017453292*this.gamma);var a=Math.sqrt(this.sinAlpha*
this.sinAlpha+this.sinBeta*this.sinBeta+this.sinGamma*this.sinGamma+2*this.cosAlpha*this.cosBeta*this.cosGamma-2);this.volume=this.a*this.b*this.c*a;this.cA_=(this.cosAlpha-this.cosBeta*this.cosGamma)/this.sinGamma;this.cB_=a/this.sinGamma;this.a_=this.b*this.c*this.sinAlpha/this.volume;this.b_=this.a*this.c*this.sinBeta/this.volume;this.c_=this.a*this.b*this.sinGamma/this.volume});c(c$,"getFractionalOrigin",function(){return this.fractionalOrigin});c(c$,"toSupercell",function(a){a.x/=this.na;a.y/=
this.nb;a.z/=this.nc;return a},"JU.P3");c(c$,"toCartesian",function(a,b){null!=this.matrixFractionalToCartesian&&(b?this.matrixFtoCNoOffset:this.matrixFractionalToCartesian).rotTrans(a)},"JU.T3,~B");c(c$,"toFractionalM",function(a){null!=this.matrixCartesianToFractional&&(a.mul(this.matrixFractionalToCartesian),a.mul2(this.matrixCartesianToFractional,a))},"JU.M4");c(c$,"toFractional",function(a,b){null!=this.matrixCartesianToFractional&&(b?this.matrixCtoFNoOffset:this.matrixCartesianToFractional).rotTrans(a)},
"JU.T3,~B");c(c$,"isPolymer",function(){return 1==this.dimension});c(c$,"isSlab",function(){return 2==this.dimension});c(c$,"getUnitCellParams",function(){return this.unitCellParams});c(c$,"getUnitCellAsArray",function(a){var b=this.matrixFractionalToCartesian;return a?K(-1,[b.m00,b.m10,b.m20,b.m01,b.m11,b.m21,b.m02,b.m12,b.m22]):K(-1,[this.a,this.b,this.c,this.alpha,this.beta,this.gamma,b.m00,b.m10,b.m20,b.m01,b.m11,b.m21,b.m02,b.m12,b.m22,this.dimension,this.volume])},"~B");c(c$,"getInfo",function(a){switch(a){case 0:return this.a;
case 1:return this.b;case 2:return this.c;case 3:return this.alpha;case 4:return this.beta;case 5:return this.gamma;case 6:return this.dimension}return NaN},"~N");c$.ijkToPoint3f=c(c$,"ijkToPoint3f",function(a,b,d,c){var f=1E9<a?1E3:1E6<a?100:10,e=f*f;d-=0<=d?w(5*f/10):d;b.x=w(a/e)%f+d;b.y=w(a%e/f)+d;b.z=(0==c?a%f:(-500==d?w(c/f):c)%f)+d},"~N,JU.P3,~N,~N");c$.escapeMultiplier=c(c$,"escapeMultiplier",function(a){if(n(a,JU.P4)){var b=1E3*w(Math.floor(a.x/1E3))+w(Math.floor(a.w/1E3))-1E3,d=1E3*w(Math.floor(a.y/
1E3))+w(Math.floor(a.w))%1E3;return"{"+b+" "+d+" "+a.z+"}"}return JU.Escape.eP(a)},"JU.T3");c$.getCellWeight=c(c$,"getCellWeight",function(a){var b=1;if(0.02>=a.x||0.98<=a.x)b/=2;if(0.02>=a.y||0.98<=a.y)b/=2;if(0.02>=a.z||0.98<=a.z)b/=2;return b},"JU.P3");c$.getReciprocal=c(c$,"getReciprocal",function(a,b,d){var c=Array(4),f=4==a.length?1:0;c[0]=1==f?JU.P3.newP(a[0]):new JU.P3;for(var e=0;3>e;e++)c[e+1]=new JU.P3,c[e+1].cross(a[(e+f)%3+f],a[(e+f+1)%3+f]),c[e+1].scale(d/a[e+f].dot(c[e+1]));if(null==
b)return c;for(e=0;4>e;e++)b[e]=c[e];return b},"~A,~A,~N");c$.setOabc=c(c$,"setOabc",function(a,b,d){if(null!=a&&(null==b&&(b=K(6,0)),a=JU.PT.split(a.$replace(",","="),"="),12<=a.length))for(var c=0;6>c;c++)b[c]=JU.PT.parseFloat(a[2*c+1]);if(null==d)return null;b=JU.SimpleUnitCell.newA(b).getUnitCellAsArray(!0);d[1].set(b[0],b[1],b[2]);d[2].set(b[3],b[4],b[5]);d[3].set(b[6],b[7],b[8]);return d},"~S,~A,~A");c$.setMinMaxLatticeParameters=c(c$,"setMinMaxLatticeParameters",function(a,b,d,c){if(d.x<=d.y&&
555<=d.y){var f=new JU.P3;JU.SimpleUnitCell.ijkToPoint3f(d.x,f,0,c);b.x=A(f.x);b.y=A(f.y);b.z=A(f.z);JU.SimpleUnitCell.ijkToPoint3f(d.y,f,1,c);d.x=A(f.x);d.y=A(f.y);d.z=A(f.z)}switch(a){case 1:b.y=0,d.y=1;case 2:b.z=0,d.z=1}},"~N,JU.P3i,JU.P3i,~N");j(c$,"toString",function(){return"["+this.a+" "+this.b+" "+this.c+" "+this.alpha+" "+this.beta+" "+this.gamma+"]"});E(c$,"toRadians",0.017453292,"INFO_DIMENSIONS",6,"INFO_GAMMA",5,"INFO_BETA",4,"INFO_ALPHA",3,"INFO_C",2,"INFO_B",1,"INFO_A",0,"SLOP",0.02,
"SLOP1",0.98)});s("JU");v(null,"JU.TempArray",["java.lang.Boolean","$.Float","JU.P3","$.P3i"],function(){c$=t(function(){this.freeEnum=this.lengthsFreeEnum=this.freeScreens=this.lengthsFreeScreens=this.freePoints=this.lengthsFreePoints=null;p(this,arguments)},JU,"TempArray");O(c$,function(){this.lengthsFreePoints=B(6,0);this.freePoints=Array(6);this.lengthsFreeScreens=B(6,0);this.freeScreens=Array(6);this.lengthsFreeEnum=B(2,0);this.freeEnum=Array(2)});q(c$,function(){});c(c$,"clear",function(){this.clearTempPoints();
this.clearTempScreens()});c$.findBestFit=c(c$,"findBestFit",function(a,b){for(var d=-1,c=2147483647,f=b.length;0<=--f;){var e=b[f];e>=a&&e<c&&(c=e,d=f)}0<=d&&(b[d]=0);return d},"~N,~A");c$.findShorter=c(c$,"findShorter",function(a,b){for(var d=b.length;0<=--d;)if(0==b[d])return b[d]=a,d;for(var c=0,f=b[0],d=b.length;0<--d;)b[d]<f&&(f=b[d],c=d);return f<a?(b[c]=a,c):-1},"~N,~A");c(c$,"clearTempPoints",function(){for(var a=0;6>a;a++)this.lengthsFreePoints[a]=0,this.freePoints[a]=null});c(c$,"allocTempPoints",
function(a){var b;b=JU.TempArray.findBestFit(a,this.lengthsFreePoints);if(0<b)b=this.freePoints[b];else for(b=Array(a);0<=--a;)b[a]=new JU.P3;return b},"~N");c(c$,"freeTempPoints",function(a){for(var b=0;b<this.freePoints.length;b++)if(this.freePoints[b]===a){this.lengthsFreePoints[b]=a.length;return}b=JU.TempArray.findShorter(a.length,this.lengthsFreePoints);0<=b&&(this.freePoints[b]=a)},"~A");c(c$,"clearTempScreens",function(){for(var a=0;6>a;a++)this.lengthsFreeScreens[a]=0,this.freeScreens[a]=
null});c(c$,"allocTempScreens",function(a){var b;b=JU.TempArray.findBestFit(a,this.lengthsFreeScreens);if(0<b)b=this.freeScreens[b];else for(b=Array(a);0<=--a;)b[a]=new JU.P3i;return b},"~N");c(c$,"freeTempScreens",function(a){for(var b=0;b<this.freeScreens.length;b++)if(this.freeScreens[b]===a){this.lengthsFreeScreens[b]=a.length;return}b=JU.TempArray.findShorter(a.length,this.lengthsFreeScreens);0<=b&&(this.freeScreens[b]=a)},"~A");c(c$,"allocTempEnum",function(a){var b=JU.TempArray.findBestFit(a,
this.lengthsFreeEnum);return 0<b?this.freeEnum[b]:Array(a)},"~N");c(c$,"freeTempEnum",function(a){for(var b=0;b<this.freeEnum.length;b++)if(this.freeEnum[b]===a){this.lengthsFreeEnum[b]=a.length;return}b=JU.TempArray.findShorter(a.length,this.lengthsFreeEnum);0<=b&&(this.freeEnum[b]=a)},"~A");c$.getSlabWithinRange=c(c$,"getSlabWithinRange",function(a,b){return u(-1,[Integer.$valueOf(1073742114),u(-1,[Float.$valueOf(a),Float.$valueOf(b)]),Boolean.FALSE,null])},"~N,~N");c$.getSlabObjectType=c(c$,"getSlabObjectType",
function(a,b,d,c){return u(-1,[Integer.$valueOf(a),b,Boolean.$valueOf(d),c])},"~N,~O,~B,~O");E(c$,"freePointsSize",6,"freeScreensSize",6,"freeEnumSize",2)});s("JU");v(null,"JU.Txt",["JU.P3","$.PT","JU.Escape"],function(){c$=D(JU,"Txt");c$.formatText=c(c$,"formatText",function(a,b){var d;if(0>b.indexOf("@{")&&0>b.indexOf("%{"))return b;var c=b,f=0<=c.indexOf("\\");f&&(c=JU.PT.rep(c,"\\%","\u0001"),c=JU.PT.rep(c,"\\@","\u0002"),f=!c.equals(b));for(var c=JU.PT.rep(c,"%{","@{"),e;0<=(d=c.indexOf("@{"));){d++;
var h=d+1;e=c.length;for(var j=1,l="\x00",p="\x00";0<j&&++d<e;){var m=c.charAt(d);if("\x00"!=l)"\\"==p?m="\x00":m==l&&(l="\x00"),p=m;else switch(m){case "'":case '"':l=m;break;case "{":j++;break;case "}":j--}}if(d>=e)return c;e=c.substring(h,d);if(0==e.length)return c;e=a.evaluateExpression(e);n(e,JU.P3)&&(e=JU.Escape.eP(e));c=c.substring(0,h-2)+e.toString()+c.substring(d+1)}f&&(c=JU.PT.rep(c,"\u0002","@"),c=JU.PT.rep(c,"\u0001","%"));return c},"J.api.JmolViewer,~S")});s("JU");v(["JU.V3"],"JU.Vibration",
["JU.P3"],function(){c$=t(function(){this.modDim=-1;this.modScale=NaN;this.showTrace=!1;this.trace=null;this.tracePt=0;p(this,arguments)},JU,"Vibration",JU.V3);c(c$,"setCalcPoint",function(a,b,d){switch(this.modDim){case -2:break;default:a.scaleAdd2(Math.cos(6.283185307179586*b.x)*d,this,a)}return a},"JU.T3,JU.T3,~N,~N");c(c$,"getInfo",function(a){a.put("vibVector",JU.V3.newV(this));a.put("vibType",-2==this.modDim?"spin":-1==this.modDim?"vib":"mod")},"java.util.Map");j(c$,"clone",function(){var a=
new JU.Vibration;a.setT(this);a.modDim=this.modDim;return a});c(c$,"setXYZ",function(a){this.setT(a)},"JU.T3");c(c$,"setType",function(a){this.modDim=a;return this},"~N");c(c$,"isNonzero",function(){return 0!=this.x||0!=this.y||0!=this.z});c(c$,"getOccupancy100",function(){return-2147483648},"~B");c(c$,"startTrace",function(a){this.trace=Array(a);this.tracePt=a},"~N");c(c$,"addTracePt",function(a,b){(null==this.trace||0==a||a!=this.trace.length)&&this.startTrace(a);if(null!=b&&2<a){if(0>=--this.tracePt){for(var d=
this.trace[this.trace.length-1],c=this.trace.length;1<=--c;)this.trace[c]=this.trace[c-1];this.trace[1]=d;this.tracePt=1}d=this.trace[this.tracePt];null==d&&(d=this.trace[this.tracePt]=new JU.P3);d.setT(b)}return this.trace},"~N,JU.Point3fi");E(c$,"twoPI",6.283185307179586,"TYPE_VIBRATION",-1,"TYPE_SPIN",-2)});s("JV");v(["J.api.EventManager","JU.Rectangle","JV.MouseState"],["JV.MotionPoint","$.ActionManager","$.Gesture"],"java.lang.Float JU.AU $.PT J.api.Interface J.i18n.GT JS.SV $.ScriptEval J.thread.HoverWatcherThread JU.BSUtil $.Escape $.Logger $.Point3fi JV.Viewer JV.binding.Binding $.JmolBinding".split(" "),
function(){c$=t(function(){this.vwr=null;this.isMultiTouch=this.haveMultiTouchInput=!1;this.predragBinding=this.rasmolBinding=this.dragBinding=this.pfaatBinding=this.jmolBinding=this.b=null;this.LEFT_DRAGGED=0;this.dragGesture=this.hoverWatcherThread=null;this.apm=1;this.pickingStyleSelect=this.pickingStyle=this.bondPickingMode=0;this.pickingStyleMeasure=5;this.rootPickingStyle=0;this.mouseDragFactor=this.gestureSwipeFactor=1;this.mouseWheelFactor=1.15;this.dragged=this.pressed=this.clicked=this.moved=
this.current=null;this.clickedCount=this.pressedCount=0;this.dragSelectedMode=this.labelMode=this.drawMode=!1;this.measuresEnabled=!0;this.hoverActive=this.haveSelection=!1;this.mp=null;this.dragAtomIndex=-1;this.rubberbandSelectionMode=this.mkBondPressed=!1;this.rectRubber=null;this.isAltKeyReleased=!0;this.isMultiTouchServer=this.isMultiTouchClient=this.keyProcessing=!1;this.clickAction=this.dragAction=this.pressAction=0;this.measurementQueued=null;this.selectionWorking=this.zoomTrigger=!1;p(this,
arguments)},JV,"ActionManager",null,J.api.EventManager);O(c$,function(){this.current=new JV.MouseState("current");this.moved=new JV.MouseState("moved");this.clicked=new JV.MouseState("clicked");this.pressed=new JV.MouseState("pressed");this.dragged=new JV.MouseState("dragged");this.rectRubber=new JU.Rectangle});c(c$,"setViewer",function(a){this.vwr=a;JV.Viewer.isJS||this.createActions();this.setBinding(this.jmolBinding=new JV.binding.JmolBinding);this.LEFT_DRAGGED=JV.binding.Binding.getMouseAction(1,
16,1);this.dragGesture=new JV.Gesture(20,a)},"JV.Viewer,~S");c(c$,"checkHover",function(){if(this.zoomTrigger)this.zoomTrigger=!1,8==this.vwr.currentCursor&&this.vwr.setCursor(0),this.vwr.setInMotion(!1);else if(!this.vwr.getInMotion(!0)&&!this.vwr.tm.spinOn&&!this.vwr.tm.navOn&&!this.vwr.checkObjectHovered(this.current.x,this.current.y)){var a=this.vwr.findNearestAtomIndex(this.current.x,this.current.y);if(!(0>a)){var b=2==this.apm&&this.bnd(JV.binding.Binding.getMouseAction(this.clickedCount,this.moved.modifiers,
1),[10]);this.vwr.hoverOn(a,b)}}});c(c$,"processMultitouchEvent",function(){},"~N,~N,~N,~N,JU.P3,~N");c(c$,"bind",function(a,b){var d=JV.ActionManager.getActionFromName(b),c=JV.binding.Binding.getMouseActionStr(a);0!=c&&(0<=d?this.b.bindAction(c,d):this.b.bindName(c,b))},"~S,~S");c(c$,"clearBindings",function(){this.setBinding(this.jmolBinding=new JV.binding.JmolBinding);this.rasmolBinding=this.dragBinding=this.pfaatBinding=null});c(c$,"unbindAction",function(a,b){if(null==a&&null==b)this.clearBindings();
else{var d=JV.ActionManager.getActionFromName(b),c=JV.binding.Binding.getMouseActionStr(a);0<=d?this.b.unbindAction(c,d):0!=c&&this.b.unbindName(c,b);null==b&&this.b.unbindUserAction(a)}},"~S,~S");c$.newAction=c(c$,"newAction",function(a,b,d){JV.ActionManager.actionInfo[a]=d;JV.ActionManager.actionNames[a]=b},"~N,~S,~S");c(c$,"createActions",function(){null==JV.ActionManager.actionInfo[0]&&(JV.ActionManager.newAction(0,"_assignNew",J.i18n.GT.o(J.i18n.GT.$("assign/new atom or bond (requires {0})"),
"set picking assignAtom_??/assignBond_?")),JV.ActionManager.newAction(1,"_center",J.i18n.GT.$("center")),JV.ActionManager.newAction(2,"_clickFrank",J.i18n.GT.$("pop up recent context menu (click on Jmol frank)")),JV.ActionManager.newAction(4,"_deleteAtom",J.i18n.GT.o(J.i18n.GT.$("delete atom (requires {0})"),"set picking DELETE ATOM")),JV.ActionManager.newAction(5,"_deleteBond",J.i18n.GT.o(J.i18n.GT.$("delete bond (requires {0})"),"set picking DELETE BOND")),JV.ActionManager.newAction(6,"_depth",
J.i18n.GT.o(J.i18n.GT.$("adjust depth (back plane; requires {0})"),"SLAB ON")),JV.ActionManager.newAction(7,"_dragAtom",J.i18n.GT.o(J.i18n.GT.$("move atom (requires {0})"),"set picking DRAGATOM")),JV.ActionManager.newAction(8,"_dragDrawObject",J.i18n.GT.o(J.i18n.GT.$("move whole DRAW object (requires {0})"),"set picking DRAW")),JV.ActionManager.newAction(9,"_dragDrawPoint",J.i18n.GT.o(J.i18n.GT.$("move specific DRAW point (requires {0})"),"set picking DRAW")),JV.ActionManager.newAction(10,"_dragLabel",
J.i18n.GT.o(J.i18n.GT.$("move label (requires {0})"),"set picking LABEL")),JV.ActionManager.newAction(11,"_dragMinimize",J.i18n.GT.o(J.i18n.GT.$("move atom and minimize molecule (requires {0})"),"set picking DRAGMINIMIZE")),JV.ActionManager.newAction(12,"_dragMinimizeMolecule",J.i18n.GT.o(J.i18n.GT.$("move and minimize molecule (requires {0})"),"set picking DRAGMINIMIZEMOLECULE")),JV.ActionManager.newAction(13,"_dragSelected",J.i18n.GT.o(J.i18n.GT.$("move selected atoms (requires {0})"),"set DRAGSELECTED")),
JV.ActionManager.newAction(14,"_dragZ",J.i18n.GT.o(J.i18n.GT.$("drag atoms in Z direction (requires {0})"),"set DRAGSELECTED")),JV.ActionManager.newAction(15,"_multiTouchSimulation",J.i18n.GT.$("simulate multi-touch using the mouse)")),JV.ActionManager.newAction(16,"_navTranslate",J.i18n.GT.o(J.i18n.GT.$("translate navigation point (requires {0} and {1})"),u(-1,["set NAVIGATIONMODE","set picking NAVIGATE"]))),JV.ActionManager.newAction(17,"_pickAtom",J.i18n.GT.$("pick an atom")),JV.ActionManager.newAction(3,
"_pickConnect",J.i18n.GT.o(J.i18n.GT.$("connect atoms (requires {0})"),"set picking CONNECT")),JV.ActionManager.newAction(18,"_pickIsosurface",J.i18n.GT.o(J.i18n.GT.$("pick an ISOSURFACE point (requires {0}"),"set DRAWPICKING")),JV.ActionManager.newAction(19,"_pickLabel",J.i18n.GT.o(J.i18n.GT.$("pick a label to toggle it hidden/displayed (requires {0})"),"set picking LABEL")),JV.ActionManager.newAction(20,"_pickMeasure",J.i18n.GT.o(J.i18n.GT.$("pick an atom to include it in a measurement (after starting a measurement or after {0})"),
"set picking DISTANCE/ANGLE/TORSION")),JV.ActionManager.newAction(21,"_pickNavigate",J.i18n.GT.o(J.i18n.GT.$("pick a point or atom to navigate to (requires {0})"),"set NAVIGATIONMODE")),JV.ActionManager.newAction(22,"_pickPoint",J.i18n.GT.o(J.i18n.GT.$("pick a DRAW point (for measurements) (requires {0}"),"set DRAWPICKING")),JV.ActionManager.newAction(23,"_popupMenu",J.i18n.GT.$("pop up the full context menu")),JV.ActionManager.newAction(24,"_reset",J.i18n.GT.$("reset (when clicked off the model)")),
JV.ActionManager.newAction(25,"_rotate",J.i18n.GT.$("rotate")),JV.ActionManager.newAction(26,"_rotateBranch",J.i18n.GT.o(J.i18n.GT.$("rotate branch around bond (requires {0})"),"set picking ROTATEBOND")),JV.ActionManager.newAction(27,"_rotateSelected",J.i18n.GT.o(J.i18n.GT.$("rotate selected atoms (requires {0})"),"set DRAGSELECTED")),JV.ActionManager.newAction(28,"_rotateZ",J.i18n.GT.$("rotate Z")),JV.ActionManager.newAction(29,"_rotateZorZoom",J.i18n.GT.$("rotate Z (horizontal motion of mouse) or zoom (vertical motion of mouse)")),
JV.ActionManager.newAction(30,"_select",J.i18n.GT.o(J.i18n.GT.$("select an atom (requires {0})"),"set pickingStyle EXTENDEDSELECT")),JV.ActionManager.newAction(31,"_selectAndDrag",J.i18n.GT.o(J.i18n.GT.$("select and drag atoms (requires {0})"),"set DRAGSELECTED")),JV.ActionManager.newAction(32,"_selectAndNot",J.i18n.GT.o(J.i18n.GT.$("unselect this group of atoms (requires {0})"),"set pickingStyle DRAG/EXTENDEDSELECT")),JV.ActionManager.newAction(33,"_selectNone",J.i18n.GT.o(J.i18n.GT.$("select NONE (requires {0})"),
"set pickingStyle EXTENDEDSELECT")),JV.ActionManager.newAction(34,"_selectOr",J.i18n.GT.o(J.i18n.GT.$("add this group of atoms to the set of selected atoms (requires {0})"),"set pickingStyle DRAG/EXTENDEDSELECT")),JV.ActionManager.newAction(35,"_selectToggle",J.i18n.GT.o(J.i18n.GT.$("toggle selection (requires {0})"),"set pickingStyle DRAG/EXTENDEDSELECT/RASMOL")),JV.ActionManager.newAction(36,"_selectToggleOr",J.i18n.GT.o(J.i18n.GT.$("if all are selected, unselect all, otherwise add this group of atoms to the set of selected atoms (requires {0})"),
"set pickingStyle DRAG")),JV.ActionManager.newAction(37,"_setMeasure",J.i18n.GT.$("pick an atom to initiate or conclude a measurement")),JV.ActionManager.newAction(38,"_slab",J.i18n.GT.o(J.i18n.GT.$("adjust slab (front plane; requires {0})"),"SLAB ON")),JV.ActionManager.newAction(39,"_slabAndDepth",J.i18n.GT.o(J.i18n.GT.$("move slab/depth window (both planes; requires {0})"),"SLAB ON")),JV.ActionManager.newAction(40,"_slideZoom",J.i18n.GT.$("zoom (along right edge of window)")),JV.ActionManager.newAction(41,
"_spinDrawObjectCCW",J.i18n.GT.o(J.i18n.GT.$("click on two points to spin around axis counterclockwise (requires {0})"),"set picking SPIN")),JV.ActionManager.newAction(42,"_spinDrawObjectCW",J.i18n.GT.o(J.i18n.GT.$("click on two points to spin around axis clockwise (requires {0})"),"set picking SPIN")),JV.ActionManager.newAction(43,"_stopMotion",J.i18n.GT.o(J.i18n.GT.$("stop motion (requires {0})"),"set waitForMoveTo FALSE")),JV.ActionManager.newAction(44,"_swipe",J.i18n.GT.$("spin model (swipe and release button and stop motion simultaneously)")),
JV.ActionManager.newAction(45,"_translate",J.i18n.GT.$("translate")),JV.ActionManager.newAction(46,"_wheelZoom",J.i18n.GT.$("zoom")))});c$.getActionName=c(c$,"getActionName",function(a){return a<JV.ActionManager.actionNames.length?JV.ActionManager.actionNames[a]:null},"~N");c$.getActionFromName=c(c$,"getActionFromName",function(a){for(var b=0;b<JV.ActionManager.actionNames.length;b++)if(JV.ActionManager.actionNames[b].equalsIgnoreCase(a))return b;return-1},"~S");c(c$,"getBindingInfo",function(a){return this.b.getBindingInfo(JV.ActionManager.actionInfo,
JV.ActionManager.actionNames,a)},"~S");c(c$,"setBinding",function(a){this.b=a},"JV.binding.Binding");c(c$,"bnd",function(a,b){for(var d=b.length;0<=--d;)if(this.b.isBound(a,b[d]))return!0;return!1},"~N,~A");c(c$,"isDrawOrLabelAction",function(a){return this.drawMode&&this.bnd(a,[8,9])||this.labelMode&&this.bnd(a,[10])},"~N");c(c$,"getBondPickingMode",function(){return this.bondPickingMode});c$.getPickingModeName=c(c$,"getPickingModeName",function(a){return 0>a||a>=JV.ActionManager.pickingModeNames.length?
"off":JV.ActionManager.pickingModeNames[a]},"~N");c$.getPickingMode=c(c$,"getPickingMode",function(a){for(var b=JV.ActionManager.pickingModeNames.length;0<=--b;)if(a.equalsIgnoreCase(JV.ActionManager.pickingModeNames[b]))return b;return-1},"~S");c$.getPickingStyleName=c(c$,"getPickingStyleName",function(a){return 0>a||a>=JV.ActionManager.pickingStyleNames.length?"toggle":JV.ActionManager.pickingStyleNames[a]},"~N");c$.getPickingStyleIndex=c(c$,"getPickingStyleIndex",function(a){for(var b=JV.ActionManager.pickingStyleNames.length;0<=
--b;)if(a.equalsIgnoreCase(JV.ActionManager.pickingStyleNames[b]))return b;return-1},"~S");c(c$,"getAtomPickingMode",function(){return this.apm});c(c$,"setPickingMode",function(a){var b=!1;switch(a){case -1:b=!0;this.bondPickingMode=35;a=1;this.vwr.setStringProperty("pickingStyle","toggle");this.vwr.setBooleanProperty("bondPicking",!1);break;case 35:case 34:case 33:case 8:this.vwr.setBooleanProperty("bondPicking",!0);this.bondPickingMode=a;this.resetMeasurement();return}b=(new Boolean(b|this.apm!=
a)).valueOf();this.apm=a;b&&this.resetMeasurement()},"~N");c(c$,"getPickingState",function(){var a=";set modelkitMode "+this.vwr.getBoolean(603983903)+";set picking "+JV.ActionManager.getPickingModeName(this.apm);32==this.apm&&(a+="_"+this.vwr.getModelkitProperty("atomType"));a+=";";0!=this.bondPickingMode&&(a+="set picking "+JV.ActionManager.getPickingModeName(this.bondPickingMode));33==this.bondPickingMode&&(a+="_"+this.vwr.getModelkitProperty("bondType"));return a+";"});c(c$,"getPickingStyle",
function(){return this.pickingStyle});c(c$,"setPickingStyle",function(a){this.pickingStyle=a;4<=a?(this.pickingStyleMeasure=a,this.resetMeasurement()):(3>a&&(this.rootPickingStyle=a),this.pickingStyleSelect=a);this.rubberbandSelectionMode=!1;switch(this.pickingStyleSelect){case 2:this.b.name.equals("extendedSelect")||this.setBinding(null==this.pfaatBinding?this.pfaatBinding=JV.binding.Binding.newBinding(this.vwr,"Pfaat"):this.pfaatBinding);break;case 3:this.b.name.equals("drag")||this.setBinding(null==
this.dragBinding?this.dragBinding=JV.binding.Binding.newBinding(this.vwr,"Drag"):this.dragBinding);this.rubberbandSelectionMode=!0;break;case 1:this.b.name.equals("selectOrToggle")||this.setBinding(null==this.rasmolBinding?this.rasmolBinding=JV.binding.Binding.newBinding(this.vwr,"Rasmol"):this.rasmolBinding);break;default:this.b!==this.jmolBinding&&this.setBinding(this.jmolBinding)}this.b.name.equals("drag")||(this.predragBinding=this.b)},"~N");c(c$,"setGestureSwipeFactor",function(a){this.gestureSwipeFactor=
a},"~N");c(c$,"setMouseDragFactor",function(a){this.mouseDragFactor=a},"~N");c(c$,"setMouseWheelFactor",function(a){this.mouseWheelFactor=a},"~N");c(c$,"isDraggedIsShiftDown",function(){return 0!=(this.dragged.modifiers&1)});c(c$,"setCurrent",function(a,b,d,c){this.vwr.hoverOff();this.current.set(a,b,d,c)},"~N,~N,~N,~N");c(c$,"getCurrentX",function(){return this.current.x});c(c$,"getCurrentY",function(){return this.current.y});c(c$,"setMouseMode",function(){this.drawMode=this.labelMode=!1;this.dragSelectedMode=
this.vwr.getDragSelected();this.measuresEnabled=!this.dragSelectedMode;if(!this.dragSelectedMode)switch(this.apm){default:return;case 32:this.measuresEnabled=!this.vwr.getModelkit(!1).isPickAtomAssignCharge();return;case 4:this.drawMode=!0;this.measuresEnabled=!1;break;case 2:this.labelMode=!0;this.measuresEnabled=!1;break;case 9:this.measuresEnabled=!1;break;case 19:case 22:case 20:case 21:this.measuresEnabled=!1;return}this.exitMeasurementMode(null)});c(c$,"clearMouseInfo",function(){this.pressedCount=
this.clickedCount=0;this.dragGesture.setAction(0,0);this.exitMeasurementMode(null)});c(c$,"setDragAtomIndex",function(a){this.dragAtomIndex=a;this.setAtomsPicked(JU.BSUtil.newAndSetBit(a),"Label picked for atomIndex = "+a)},"~N");c(c$,"isMTClient",function(){return this.isMultiTouchClient});c(c$,"isMTServer",function(){return this.isMultiTouchServer});c(c$,"dispose",function(){this.clear()});c(c$,"clear",function(){this.startHoverWatcher(!1);null!=this.predragBinding&&(this.b=this.predragBinding);
this.vwr.setPickingMode(null,1);this.vwr.setPickingStyle(null,this.rootPickingStyle);this.isAltKeyReleased=!0});c(c$,"startHoverWatcher",function(a){if(!this.vwr.isPreviewOnly)try{a?null==this.hoverWatcherThread&&(this.current.time=-1,this.hoverWatcherThread=new J.thread.HoverWatcherThread(this,this.current,this.moved,this.vwr)):null!=this.hoverWatcherThread&&(this.current.time=-1,this.hoverWatcherThread.interrupt(),this.hoverWatcherThread=null)}catch(b){if(!H(b,Exception))throw b;}},"~B");c(c$,"setModeMouse",
function(a){-1==a&&this.startHoverWatcher(!1)},"~N");j(c$,"keyPressed",function(a,b){if(this.keyProcessing)return!1;this.keyProcessing=!0;switch(a){case 18:this.dragSelectedMode&&this.isAltKeyReleased&&this.vwr.moveSelected(-2147483648,0,-2147483648,-2147483648,-2147483648,null,!1,!1,b);this.isAltKeyReleased=!1;this.moved.modifiers|=8;break;case 16:this.dragged.modifiers|=1;this.moved.modifiers|=1;break;case 17:this.moved.modifiers|=2;break;case 27:this.vwr.hoverOff();this.exitMeasurementMode("escape");
break;default:this.vwr.hoverOff()}var d=8464|this.moved.modifiers;!this.labelMode&&!this.b.isUserAction(d)&&this.checkMotionRotateZoom(d,this.current.x,0,0,!1);if(this.vwr.getBoolean(603979889))switch(a){case 38:case 40:case 37:case 39:case 32:case 46:this.vwr.navigate(a,b)}this.keyProcessing=!1;return!0},"~N,~N");j(c$,"keyTyped",function(){return!1},"~N,~N");j(c$,"keyReleased",function(a){switch(a){case 18:this.moved.modifiers&=-9;this.dragSelectedMode&&this.vwr.moveSelected(2147483647,0,-2147483648,
-2147483648,-2147483648,null,!1,!1,this.moved.modifiers);this.isAltKeyReleased=!0;break;case 16:this.moved.modifiers&=-2;break;case 17:this.moved.modifiers&=-3}0==this.moved.modifiers&&this.vwr.setCursor(0);if(this.vwr.getBoolean(603979889))switch(a){case 38:case 40:case 37:case 39:this.vwr.navigate(0,0)}},"~N");j(c$,"mouseEnterExit",function(a,b,d,c){this.vwr.tm.stereoDoubleDTI&&(b<<=1);this.setCurrent(a,b,d,0);c&&this.exitMeasurementMode("mouseExit")},"~N,~N,~N,~B");c(c$,"setMouseActions",function(a,
b,d){this.pressAction=JV.binding.Binding.getMouseAction(a,b,d?5:4);this.dragAction=JV.binding.Binding.getMouseAction(a,b,1);this.clickAction=JV.binding.Binding.getMouseAction(a,b,2)},"~N,~N,~B");j(c$,"mouseAction",function(a,b,d,c,f,e){if(this.vwr.getMouseEnabled())switch(JU.Logger.debuggingHigh&&(0!=a&&this.vwr.getBoolean(603979960))&&this.vwr.showString("mouse action: "+a+" "+e+" "+JV.binding.Binding.getMouseActionName(JV.binding.Binding.getMouseAction(f,e,a),!1),!1),this.vwr.tm.stereoDoubleDTI&&
(d<<=1),a){case 0:this.setCurrent(b,d,c,e);this.moved.setCurrent(this.current,0);if(null!=this.mp||this.hoverActive){this.clickAction=JV.binding.Binding.getMouseAction(this.clickedCount,e,0);this.checkClickAction(d,c,b,0);break}if(this.isZoomArea(d)){this.checkMotionRotateZoom(this.LEFT_DRAGGED,0,0,0,!1);break}8==this.vwr.currentCursor&&this.vwr.setCursor(0);break;case 4:this.setMouseMode();this.pressedCount=this.pressed.check(20,d,c,e,b,700)?this.pressedCount+1:1;1==this.pressedCount&&(this.vwr.checkInMotion(1),
this.setCurrent(b,d,c,e));this.pressAction=JV.binding.Binding.getMouseAction(this.pressedCount,e,4);this.vwr.setCursor(12);this.pressed.setCurrent(this.current,1);this.dragged.setCurrent(this.current,1);this.vwr.setFocus();this.dragGesture.setAction(this.dragAction,b);this.checkPressedAction(d,c,b);break;case 1:this.setMouseMode();this.setMouseActions(this.pressedCount,e,!1);a=d-this.dragged.x;f=c-this.dragged.y;this.setCurrent(b,d,c,e);this.dragged.setCurrent(this.current,-1);this.dragGesture.add(this.dragAction,
d,c,b);this.checkDragWheelAction(this.dragAction,d,c,a,f,b,1);break;case 5:this.setMouseActions(this.pressedCount,e,!0);this.setCurrent(b,d,c,e);this.vwr.spinXYBy(0,0,0);e=!this.pressed.check(10,d,c,e,b,9223372036854775E3);this.checkReleaseAction(d,c,b,e);break;case 3:if(this.vwr.isApplet&&!this.vwr.hasFocus())break;this.setCurrent(b,this.current.x,this.current.y,e);this.checkDragWheelAction(JV.binding.Binding.getMouseAction(0,e,3),this.current.x,this.current.y,0,c,b,3);break;case 2:this.setMouseMode();
this.clickedCount=1<f?f:this.clicked.check(10,0,0,e,b,700)?this.clickedCount+1:1;1==this.clickedCount&&this.setCurrent(b,d,c,e);this.setMouseActions(this.clickedCount,e,!1);this.clicked.setCurrent(this.current,this.clickedCount);this.vwr.setFocus();if(9!=this.apm&&this.bnd(JV.binding.Binding.getMouseAction(1,e,4),[31]))break;this.clickAction=JV.binding.Binding.getMouseAction(this.clickedCount,e,2);this.checkClickAction(d,c,b,this.clickedCount)}},"~N,~N,~N,~N,~N,~N");c(c$,"checkPressedAction",function(a,
b,d){var c=JV.binding.Binding.getButtonMods(this.pressAction),f=this.bnd(JV.binding.Binding.getMouseAction(1,c,4),[31]);if(0!=c){this.pressAction=this.vwr.notifyMouseClicked(a,b,this.pressAction,4);if(0==this.pressAction)return;c=JV.binding.Binding.getButtonMods(this.pressAction)}this.setMouseActions(this.pressedCount,c,!1);JU.Logger.debuggingHigh&&this.vwr.getBoolean(603979960)&&JU.Logger.debug(JV.binding.Binding.getMouseActionName(this.pressAction,!1));if(!this.isDrawOrLabelAction(this.dragAction)||
!this.vwr.checkObjectDragged(-2147483648,0,a,b,this.dragAction)){this.checkUserAction(this.pressAction,a,b,0,0,d,4);d=!1;switch(this.apm){case 32:d=this.bnd(this.clickAction,[0]);break;case 28:d=this.bnd(this.dragAction,[7,14]);break;case 26:case 36:case 37:case 27:d=this.bnd(this.dragAction,[7,14,27]);break;case 29:d=this.bnd(this.dragAction,[11,14]);break;case 30:d=this.bnd(this.dragAction,[11,14,27])}if(d){this.dragAtomIndex=this.vwr.findNearestAtomIndexMovable(a,b,!0);if(0<=this.dragAtomIndex&&
(32==this.apm||31==this.apm))34==this.bondPickingMode&&this.vwr.setModelkitProperty("bondAtomIndex",Integer.$valueOf(this.dragAtomIndex)),this.enterMeasurementMode(this.dragAtomIndex),this.mp.addPoint(this.dragAtomIndex,null,!1);a=this.vwr.getModelkitProperty("screenXY");this.mkBondPressed=null!=a&&this.pressed.inRange(10,a[0],a[1])}else this.bnd(this.pressAction,[23])?this.vwr.popupMenu(a,b,this.vwr.getBoolean(603983903)?"m":"j"):this.dragSelectedMode?(this.haveSelection=!f||0<=this.vwr.findNearestAtomIndexMovable(a,
b,!0))&&this.bnd(this.dragAction,[13,14])&&this.vwr.moveSelected(-2147483648,0,-2147483648,-2147483648,-2147483648,null,!1,!1,c):this.checkMotionRotateZoom(this.dragAction,a,0,0,!0)}},"~N,~N,~N");c(c$,"checkDragWheelAction",function(a,b,d,c,f,e,h){var j=JV.binding.Binding.getButtonMods(a);if(0!=j){var l=this.vwr.notifyMouseClicked(b,d,JV.binding.Binding.getMouseAction(this.pressedCount,j,h),h);if(0==l)return;0<l&&(a=l)}if(this.isRubberBandSelect(a))this.calcRectRubberBand(),this.vwr.refresh(3,"rubberBand selection");
else if(!this.checkUserAction(a,b,d,c,f,e,h))if(e=this.vwr.g.modelKitMode?this.vwr.getModelkit(!1).getRotateBondIndex():-1,0<=e&&(0<=this.dragAtomIndex||this.mkBondPressed||this.bnd(a,[26])))0<=this.dragAtomIndex&&(null==this.measurementQueued||0==this.measurementQueued.numSet?this.vwr.setPendingMeasurement(this.vwr.getModelkit(!1).setBondMeasure(e,this.measurementQueued=this.mp=this.getMP())):this.measurementQueued.refresh(null)),this.vwr.moveSelected(c,f,-2147483648,b,d,null,!1,!1,0<=this.dragAtomIndex?
0:16);else{e=null;if(0<=this.dragAtomIndex&&2!=this.apm)switch(this.apm){case 26:this.dragSelected(a,c,f,!0);return;case 36:case 37:case 27:case 30:e=this.vwr.ms.getAtoms(37==this.apm?1094717454:1094713360,JU.BSUtil.newAndSetBit(this.dragAtomIndex)),36==this.apm&&e.and(this.vwr.getAtomBitSet("ligand"));case 28:case 29:1==this.dragGesture.getPointCount()&&this.vwr.undoMoveActionClear(this.dragAtomIndex,2,!0);this.setMotion(13,!0);if(this.bnd(a,[27]))this.vwr.rotateSelected(this.getDegrees(c,!0),this.getDegrees(f,
!1),e);else{switch(this.apm){case 36:case 37:case 27:case 30:this.vwr.select(e,!1,0,!0)}this.vwr.moveAtomWithHydrogens(this.dragAtomIndex,c,f,this.bnd(a,[14])?-f:-2147483648,e)}return}if(0<=this.dragAtomIndex&&1==h&&this.bnd(this.clickAction,[0])&&32==this.apm)a=this.vwr.findNearestAtomIndexMovable(b,d,!1),0<=a?(null!=this.mp?this.mp.setCount(1):this.measuresEnabled&&this.enterMeasurementMode(a),this.addToMeasurement(a,null,!0),this.mp.colix=20):null!=this.mp&&(this.mp.setCount(1),this.mp.colix=23),
null!=this.mp&&(this.vwr.antialiased&&(b<<=1,d<<=1),this.mp.traceX=b,this.mp.traceY=d,this.vwr.refresh(3,"assignNew"));else if(!this.drawMode&&!this.labelMode&&this.bnd(a,[45]))this.vwr.translateXYBy(c,f);else if(this.dragSelectedMode&&this.haveSelection&&this.bnd(a,[13,27]))b=this.vwr.bsA().nextSetBit(0),0>b||(1==this.dragGesture.getPointCount()?this.vwr.undoMoveActionClear(b,2,!0):this.vwr.moveSelected(2147483647,0,-2147483648,-2147483648,-2147483648,null,!1,!1,j),this.dragSelected(a,c,f,!1));else{if(this.isDrawOrLabelAction(a)&&
(this.setMotion(13,!0),this.vwr.checkObjectDragged(this.dragged.x,this.dragged.y,b,d,a)))return;if(this.checkMotionRotateZoom(a,b,c,f,!0))this.vwr.tm.slabEnabled&&this.bnd(a,[39])?this.vwr.slabDepthByPixels(f):this.vwr.zoomBy(f);else if(this.bnd(a,[25]))this.vwr.rotateXYBy(this.getDegrees(c,!0),this.getDegrees(f,!1));else if(this.bnd(a,[29]))0==c&&1<Math.abs(f)?(this.setMotion(8,!0),this.vwr.zoomBy(f+(0<f?-1:1))):0==f&&1<Math.abs(c)&&(this.setMotion(13,!0),this.vwr.rotateZBy(-c+(0<c?1:-1),2147483647,
2147483647));else{if(this.vwr.tm.slabEnabled){if(this.bnd(a,[6])){this.vwr.depthByPixels(f);return}if(this.bnd(a,[38])){this.vwr.slabByPixels(f);return}if(this.bnd(a,[39])){this.vwr.slabDepthByPixels(f);return}}this.bnd(a,[46])?this.zoomByFactor(f,2147483647,2147483647):this.bnd(a,[28])&&(this.setMotion(13,!0),this.vwr.rotateZBy(-c,2147483647,2147483647))}}}},"~N,~N,~N,~N,~N,~N,~N");c(c$,"dragSelected",function(a,b,d,c){this.setMotion(13,!0);this.bnd(a,[27])&&this.vwr.getBoolean(603979785)?this.vwr.rotateSelected(this.getDegrees(b,
!0),this.getDegrees(d,!1),null):this.vwr.moveSelected(b,d,c&&this.bnd(a,[14])?-d:-2147483648,-2147483648,-2147483648,null,!0,!1,this.dragged.modifiers)},"~N,~N,~N,~B");c(c$,"checkReleaseAction",function(a,b,d,c){JU.Logger.debuggingHigh&&this.vwr.getBoolean(603979960)&&JU.Logger.debug(JV.binding.Binding.getMouseActionName(this.pressAction,!1));this.vwr.checkInMotion(0);this.vwr.setInMotion(!1);this.vwr.setCursor(0);this.dragGesture.add(this.dragAction,a,b,d);0<=this.dragAtomIndex&&(29==this.apm||30==
this.apm)&&this.minimize(!0);if(32==this.apm&&this.bnd(this.clickAction,[0]))null==this.mp||0>this.dragAtomIndex?this.exitMeasurementMode(null):34==this.bondPickingMode?(this.vwr.setModelkitProperty("bondAtomIndex",Integer.$valueOf(this.dragAtomIndex)),this.exitMeasurementMode(null)):this.assignNew();else if(this.dragAtomIndex=-1,this.mkBondPressed=!1,this.isRubberBandSelect(this.dragAction)&&this.selectRb(this.clickAction),this.rubberbandSelectionMode=this.b.name.equals("drag"),this.rectRubber.x=
2147483647,c&&this.vwr.notifyMouseClicked(a,b,JV.binding.Binding.getMouseAction(this.pressedCount,0,5),5),this.isDrawOrLabelAction(this.dragAction))this.vwr.checkObjectDragged(2147483647,0,a,b,this.dragAction);else if(this.haveSelection&&(this.dragSelectedMode&&this.bnd(this.dragAction,[13]))&&this.vwr.moveSelected(2147483647,0,-2147483648,-2147483648,-2147483648,null,!1,!1,this.dragged.modifiers),(!c||!this.checkUserAction(this.pressAction,a,b,0,0,d,5))&&this.vwr.getBoolean(603979780)&&this.bnd(this.dragAction,
[44]))a=this.getExitRate(),0<a&&this.vwr.spinXYBy(this.dragGesture.getDX(4,2),this.dragGesture.getDY(4,2),30*a*this.gestureSwipeFactor),this.vwr.g.logGestures&&this.vwr.log("$NOW$ swipe "+this.dragGesture+" "+a)},"~N,~N,~N,~B");c(c$,"checkClickAction",function(a,b,d,c){if(0<c){if(this.checkUserAction(this.clickAction,a,b,0,0,d,32768))return;this.clickAction=this.vwr.notifyMouseClicked(a,b,this.clickAction,32768);if(0==this.clickAction)return}JU.Logger.debuggingHigh&&this.vwr.getBoolean(603979960)&&
JU.Logger.debug(JV.binding.Binding.getMouseActionName(this.clickAction,!1));if(this.bnd(this.clickAction,[2])){if(this.vwr.frankClicked(a,b)){this.vwr.popupMenu(-a,b,"j");return}if(this.vwr.frankClickedModelKit(a,b)){this.vwr.popupMenu(10,0,"m");return}}d=null;var f=!1,e=!1,h=null;if(!this.drawMode&&(h=this.vwr.checkObjectClicked(a,b,this.clickAction),null!=h)){if(this.labelMode){this.pickLabel(h.get("atomIndex").intValue());return}f="bond".equals(h.get("type"));e="isosurface".equals(h.get("type"));
d=this.getPoint(h)}f&&(c=1);if(!(null!=d&&Float.isNaN(d.x))){var j=this.findNearestAtom(a,b,d,0<c);if(0==c&&32!=this.apm)null!=this.mp&&((null!=d||0==this.mp.getIndexOf(j))&&this.mp.addPoint(j,d,!1),this.mp.haveModified&&this.vwr.setPendingMeasurement(this.mp),this.vwr.refresh(3,"measurementPending"));else if(this.setMouseMode(),this.bnd(this.clickAction,[43])&&this.vwr.tm.stopMotion(),this.vwr.getBoolean(603979889)&&23==this.apm&&this.bnd(this.clickAction,[21]))this.vwr.navTranslatePercent(100*a/
this.vwr.getScreenWidth()-50,100*b/this.vwr.getScreenHeight()-50);else{if(f){if(this.bnd(this.clickAction,[34==this.bondPickingMode||33==this.bondPickingMode?0:5])){this.bondPicked(h.get("index").intValue());return}}else{if(e)return;if(32!=this.apm&&null!=this.mp&&this.bnd(this.clickAction,[20])){this.atomOrPointPicked(j,d);4==this.addToMeasurement(j,d,!1)&&this.toggleMeasurement();return}if(this.bnd(this.clickAction,[37])){null!=this.mp?(this.addToMeasurement(j,d,!0),this.toggleMeasurement()):!this.drawMode&&
(!this.labelMode&&!this.dragSelectedMode&&this.measuresEnabled)&&(this.enterMeasurementMode(j),this.addToMeasurement(j,d,!0));this.atomOrPointPicked(j,d);return}}this.isSelectAction(this.clickAction)?e||this.atomOrPointPicked(j,d):this.bnd(this.clickAction,[24])&&0>j&&this.reset()}}},"~N,~N,~N,~N");c(c$,"pickLabel",function(a){var b=this.vwr.ms.at[a].atomPropertyString(this.vwr,1825200146);2==this.pressedCount?(b=this.vwr.apiPlatform.prompt("Set label for atomIndex="+a,b,null,!1),null!=b&&(this.vwr.shm.setAtomLabel(b,
a),this.vwr.refresh(1,"label atom"))):this.setAtomsPicked(JU.BSUtil.newAndSetBit(a),"Label picked for atomIndex = "+a+": "+b)},"~N");c(c$,"checkUserAction",function(a,b,d,c,f,e,h){if(!this.b.isUserAction(a))return!1;for(var j=!1,l,n=this.b.getBindings(),m=a+"\t",r,p=n.keySet().iterator();p.hasNext()&&((r=p.next())||1);)if(0==r.indexOf(m)&&JU.AU.isAS(l=n.get(r))){var q=l[1],s=null;if(0<=q.indexOf("_ATOM")){var t=this.findNearestAtom(b,d,null,!0),q=JU.PT.rep(q,"_ATOM","({"+(0<=t?""+t:"")+"})");0<=t&&
(q=JU.PT.rep(q,"_POINT",JU.Escape.eP(this.vwr.ms.at[t])))}if(!this.drawMode&&(0<=q.indexOf("_POINT")||0<=q.indexOf("_OBJECT")||0<=q.indexOf("_BOND"))){t=this.vwr.checkObjectClicked(b,d,a);if(null!=t&&null!=(s=t.get("pt")))t.get("type").equals("bond")&&(q=JU.PT.rep(q,"_BOND","[{"+t.get("index")+"}]")),q=JU.PT.rep(q,"_POINT",JU.Escape.eP(s)),q=JU.PT.rep(q,"_OBJECT",JU.Escape.escapeMap(t));q=JU.PT.rep(q,"_BOND","[{}]");q=JU.PT.rep(q,"_OBJECT","{}")}q=JU.PT.rep(q,"_POINT","{}");q=JU.PT.rep(q,"_ACTION",
""+a);q=JU.PT.rep(q,"_X",""+b);q=JU.PT.rep(q,"_Y",""+(this.vwr.getScreenHeight()-d));q=JU.PT.rep(q,"_DELTAX",""+c);q=JU.PT.rep(q,"_DELTAY",""+f);q=JU.PT.rep(q,"_TIME",""+e);q=JU.PT.rep(q,"_MODE",""+h);q.startsWith("+:")&&(j=!0,q=q.substring(2));this.vwr.evalStringQuiet(q)}return!j},"~N,~N,~N,~N,~N,~N,~N");c(c$,"checkMotionRotateZoom",function(a,b,d,c,f){b=this.bnd(a,[40])&&this.isZoomArea(this.pressed.x);var e=this.bnd(a,[25]),h=this.bnd(a,[29]);if(!b&&!e&&!h)return!1;a=(d=h&&(0==d||Math.abs(c)>5*
Math.abs(d)))||this.isZoomArea(this.moved.x)||this.bnd(a,[46])?8:e||h?13:this.bnd(a,[1])?12:0;this.setMotion(a,f);return d||b},"~N,~N,~N,~N,~B");c(c$,"getExitRate",function(){var a=this.dragGesture.getTimeDifference(2);return this.isMultiTouch?80<a?0:this.dragGesture.getSpeedPixelsPerMillisecond(2,1):10<a?0:this.dragGesture.getSpeedPixelsPerMillisecond(4,2)});c(c$,"isRubberBandSelect",function(a){return this.rubberbandSelectionMode&&this.bnd(a&-8193|32768,[35,34,32])},"~N");c(c$,"getRubberBand",function(){return this.rubberbandSelectionMode&&
2147483647!=this.rectRubber.x?this.rectRubber:null});c(c$,"calcRectRubberBand",function(){var a=this.vwr.antialiased?2:1;this.current.x<this.pressed.x?(this.rectRubber.x=this.current.x*a,this.rectRubber.width=(this.pressed.x-this.current.x)*a):(this.rectRubber.x=this.pressed.x*a,this.rectRubber.width=(this.current.x-this.pressed.x)*a);this.current.y<this.pressed.y?(this.rectRubber.y=this.current.y*a,this.rectRubber.height=(this.pressed.y-this.current.y)*a):(this.rectRubber.y=this.pressed.y*a,this.rectRubber.height=
(this.current.y-this.pressed.y)*a)});c(c$,"getDegrees",function(a,b){return 180*(a/Math.min(500,b?this.vwr.getScreenWidth():this.vwr.getScreenHeight()))*this.mouseDragFactor},"~N,~B");c(c$,"isZoomArea",function(a){return a>98*this.vwr.getScreenWidth()*(this.vwr.tm.stereoDoubleFull||this.vwr.tm.stereoDoubleDTI?2:1)/100},"~N");c(c$,"getPoint",function(a){var b=new JU.Point3fi;b.setT(a.get("pt"));b.mi=a.get("modelIndex").intValue();return b},"java.util.Map");c(c$,"findNearestAtom",function(a,b,d,c){a=
this.drawMode||null!=d?-1:this.vwr.findNearestAtomIndexMovable(a,b,!1);return 0<=a&&(c||null==this.mp)&&!this.vwr.slm.isInSelectionSubset(a)?-1:a},"~N,~N,JU.Point3fi,~B");c(c$,"isSelectAction",function(a){return this.bnd(a,[17])||!this.drawMode&&!this.labelMode&&1==this.apm&&this.bnd(a,[1])||this.dragSelectedMode&&this.bnd(this.dragAction,[27,13])||this.bnd(a,[22,35,32,34,36,30])},"~N");c(c$,"enterMeasurementMode",function(a){this.vwr.setPicked(a,!0);this.vwr.setCursor(1);this.vwr.setPendingMeasurement(this.mp=
this.getMP());this.measurementQueued=this.mp},"~N");c(c$,"getMP",function(){return J.api.Interface.getInterface("JM.MeasurementPending",this.vwr,"mouse").set(this.vwr.ms)});c(c$,"addToMeasurement",function(a,b,d){if(-1==a&&null==b||null==this.mp)return this.exitMeasurementMode(null),0;var c=this.mp.count;-2147483648!=this.mp.traceX&&2==c&&this.mp.setCount(c=1);return 4==c&&!d?c:this.mp.addPoint(a,b,!0)},"~N,JU.Point3fi,~B");c(c$,"resetMeasurement",function(){this.exitMeasurementMode(null);this.measurementQueued=
this.getMP()});c(c$,"exitMeasurementMode",function(a){null!=this.mp&&(this.vwr.setPendingMeasurement(this.mp=null),this.vwr.setCursor(0),null!=a&&this.vwr.refresh(3,a))},"~S");c(c$,"getSequence",function(){var a=this.measurementQueued.getAtomIndex(1),b=this.measurementQueued.getAtomIndex(2);if(!(0>a||0>b))try{var d=this.vwr.getSmilesOpt(null,a,b,1048576,null);this.vwr.setStatusMeasuring("measureSequence",-2,d,0)}catch(c){if(H(c,Exception))JU.Logger.error(c.toString());else throw c;}});c(c$,"minimize",
function(a){var b=this.dragAtomIndex;a&&(this.dragAtomIndex=-1,this.mkBondPressed=!1);this.vwr.dragMinimizeAtom(b)},"~B");c(c$,"queueAtom",function(a,b){var d=this.measurementQueued.addPoint(a,b,!0);0<=a&&this.vwr.setStatusAtomPicked(a,"Atom #"+d+":"+this.vwr.getAtomInfo(a),null,!1);return d},"~N,JU.Point3fi");c(c$,"setMotion",function(a,b){switch(this.vwr.currentCursor){case 3:break;default:this.vwr.setCursor(a)}b&&this.vwr.setInMotion(!0)},"~N,~B");c(c$,"zoomByFactor",function(a,b,d){0!=a&&(this.setMotion(8,
!0),this.vwr.zoomByFactor(Math.pow(this.mouseWheelFactor,a),b,d),this.moved.setCurrent(this.current,0),this.vwr.setInMotion(!0),this.zoomTrigger=!0,this.startHoverWatcher(!0))},"~N,~N,~N");c(c$,"runScript",function(a){this.vwr.script(a)},"~S");c(c$,"atomOrPointPicked",function(a,b){if(0>a){this.resetMeasurement();if(this.bnd(this.clickAction,[33])){this.runScript("select none");return}if(5!=this.apm&&6!=this.apm)return}var d=2;switch(this.apm){case 28:case 29:return;case 0:return;case 25:case 24:case 8:var d=
8==this.apm,c=25==this.apm;if(!this.bnd(this.clickAction,[d?5:3]))return;if(null==this.measurementQueued||0==this.measurementQueued.count||2<this.measurementQueued.count)this.resetMeasurement(),this.enterMeasurementMode(a);this.addToMeasurement(a,b,!0);if(2!=this.queueAtom(a,b))return;d=d||this.measurementQueued.isConnected(this.vwr.ms.at,2)?" DELETE":c?"STRUTS":"";this.runScript("connect "+this.measurementQueued.getMeasurementScript(" ",!0)+d);this.resetMeasurement();return;case 21:d++;case 20:d++;
case 18:case 19:case 22:if(!this.bnd(this.clickAction,[20]))return;if(null==this.measurementQueued||0==this.measurementQueued.count||this.measurementQueued.count>d)this.resetMeasurement(),this.enterMeasurementMode(a);this.addToMeasurement(a,b,!0);this.queueAtom(a,b);c=this.measurementQueued.count;1==c&&this.vwr.setPicked(a,!0);if(c<d)return;22==this.apm?this.getSequence():(this.vwr.setStatusMeasuring("measurePicked",d,this.measurementQueued.getStringDetail(),this.measurementQueued.value),(18==this.apm||
4==this.pickingStyleMeasure)&&this.runScript("measure "+this.measurementQueued.getMeasurementScript(" ",!0)));this.resetMeasurement();return}d=null!=this.mp&&1!=this.apm?1:this.apm;switch(d){case 3:if(!this.bnd(this.clickAction,[17]))return;null==b?this.zoomTo(a):this.runScript("zoomTo "+JU.Escape.eP(b));return;case 5:case 6:this.bnd(this.clickAction,[17])&&this.checkTwoAtomAction(b,a)}if(null==b){switch(d){case 1:!this.drawMode&&!this.labelMode&&this.bnd(this.clickAction,[1])?this.zoomTo(a):this.bnd(this.clickAction,
[17])&&this.vwr.setStatusAtomPicked(a,null,null,!1);return;case 2:this.bnd(this.clickAction,[19])&&(this.runScript("set labeltoggle {atomindex="+a+"}"),this.pickLabel(a));return;case 31:this.bnd(this.clickAction,[0])&&(this.vwr.invertRingAt(a,!0),this.vwr.setStatusAtomPicked(a,"invert stereo for atomIndex="+a,null,!1));return;case 7:this.bnd(this.clickAction,[4])&&(d=JU.BSUtil.newAndSetBit(a),this.vwr.deleteAtoms(d,!1),this.vwr.setStatusAtomPicked(a,"deleted: "+JU.Escape.eBS(d),null,!1));return}d=
"atomindex="+a;switch(this.apm){default:return;case 9:this.selectAtoms(d);break;case 10:this.selectAtoms("within(group, "+d+")");break;case 11:this.selectAtoms("within(chain, "+d+")");break;case 13:this.selectAtoms("within(polymer, "+d+")");break;case 14:this.selectAtoms("within(structure, "+d+")");break;case 12:this.selectAtoms("within(molecule, "+d+")");break;case 16:this.selectAtoms("within(model, "+d+")");break;case 17:this.selectAtoms("visible and within(element, "+d+")");break;case 15:this.selectAtoms("visible and within(site, "+
d+")")}this.vwr.clearClickCount();this.vwr.setStatusAtomPicked(a,null,null,!1)}},"~N,JU.Point3fi");c(c$,"assignNew",function(){this.vwr.getModelkit(!1).handleAssignNew(this.pressed,this.dragged,this.mp,this.dragAtomIndex)||this.exitMeasurementMode("bond dropped");this.exitMeasurementMode(null)});c(c$,"bondPicked",function(a){33==this.bondPickingMode&&this.vwr.undoMoveActionClear(-1,4146,!0);switch(this.bondPickingMode){case 33:this.vwr.setModelkitProperty("scriptAssignBond",Integer.$valueOf(a));break;
case 8:this.vwr.deleteBonds(JU.BSUtil.newAndSetBit(a))}},"~N");c(c$,"checkTwoAtomAction",function(a,b){var d=5==this.apm;if(this.vwr.tm.spinOn||this.vwr.tm.navOn||null!=this.vwr.getPendingMeasurement())this.resetMeasurement(),this.vwr.tm.spinOn&&this.runScript("spin off");else{2<=this.measurementQueued.count&&this.resetMeasurement();var c=this.measurementQueued.count;if(1==c)if(null==a){if(this.measurementQueued.getAtomIndex(1)==b)return}else if(0==this.measurementQueued.getAtom(1).distance(a))return;
if(0<=b||null!=a)c=this.queueAtom(b,a);2>c?d?this.vwr.scriptStatus(1==c?J.i18n.GT.$("pick one more atom in order to spin the model around an axis"):J.i18n.GT.$("pick two atoms in order to spin the model around an axis")):this.vwr.scriptStatus(1==c?J.i18n.GT.$("pick one more atom in order to display the symmetry relationship"):J.i18n.GT.$("pick two atoms in order to display the symmetry relationship between them")):(c=this.measurementQueued.getMeasurementScript(" ",!1),d?this.runScript("spin"+c+" "+
this.vwr.getInt(553648157)):this.runScript("draw symop "+c+";show symop "+c))}},"JU.Point3fi,~N");c(c$,"reset",function(){this.runScript("!reset")});c(c$,"selectAtoms",function(a){if(!(null!=this.mp||this.selectionWorking)){this.selectionWorking=!0;var b=this.rubberbandSelectionMode||this.bnd(this.clickAction,[35])?"selected and not ("+a+") or (not selected) and ":this.bnd(this.clickAction,[32])?"selected and not ":this.bnd(this.clickAction,[34])?"selected or ":0==this.clickAction||this.bnd(this.clickAction,
[36])?"selected tog ":this.bnd(this.clickAction,[30])?"":null;if(null!=b)try{var d=this.vwr.getAtomBitSetEval(null,b+("("+a+")"));this.setAtomsPicked(d,"selected: "+JU.Escape.eBS(d));this.vwr.refresh(3,"selections set")}catch(c){if(!H(c,Exception))throw c;}this.selectionWorking=!1}},"~S");c(c$,"setAtomsPicked",function(a,b){this.vwr.select(a,!1,0,!1);this.vwr.setStatusAtomPicked(-1,b,null,!1)},"JU.BS,~S");c(c$,"selectRb",function(a){var b=this.vwr.ms.findAtomsInRectangle(this.rectRubber);0<b.length()&&
(b=JU.Escape.eBS(b),this.bnd(a,[34])?this.runScript("selectionHalos on;select selected or "+b):this.bnd(a,[32])?this.runScript("selectionHalos on;select selected and not "+b):this.runScript("selectionHalos on;select selected tog "+b));this.vwr.refresh(3,"mouseReleased")},"~N");c(c$,"toggleMeasurement",function(){if(null!=this.mp){var a=this.mp.count;2<=a&&4>=a&&this.runScript("!measure "+this.mp.getMeasurementScript(" ",!0));this.exitMeasurementMode(null)}});c(c$,"zoomTo",function(a){this.runScript("zoomTo (atomindex="+
a+")");this.vwr.setStatusAtomPicked(a,null,null,!1)},"~N");c(c$,"userActionEnabled",function(a){return this.vwr.isFunction(JV.ActionManager.getActionName(a).toLowerCase())},"~N");c(c$,"userAction",function(a,b){if(!this.userActionEnabled(a))return!1;var d=JS.ScriptEval.runUserAction(JV.ActionManager.getActionName(a),b,this.vwr);return!JS.SV.vF.equals(d)},"~N,~A");E(c$);c$.actionInfo=c$.prototype.actionInfo=
Array(47);c$.actionNames=c$.prototype.actionNames=Array(47);E(c$);JV.ActionManager.pickingModeNames="off identify label center draw spin symmetry deleteatom deletebond atom group chain molecule polymer structure site model element measure distance angle torsion sequence navigate connect struts dragselected dragmolecule dragatom dragminimize dragminimizemolecule invertstereo assignatom assignbond rotatebond identifybond dragligand dragmodel".$plit(" ");
E(c$,"pickingStyleNames",null);JV.ActionManager.pickingStyleNames="toggle selectOrToggle extendedSelect drag measure measureoff".$plit(" ");E(c$,"MAX_DOUBLE_CLICK_MILLIS",700,"MININUM_GESTURE_DELAY_MILLISECONDS",10,"SLIDE_ZOOM_X_PERCENT",98,"DEFAULT_MOUSE_DRAG_FACTOR",1,"DEFAULT_MOUSE_WHEEL_FACTOR",1.15,"DEFAULT_GESTURE_SWIPE_FACTOR",1,"XY_RANGE",10);c$=t(function(){this.time=this.y=this.x=this.index=0;p(this,arguments)},JV,"MotionPoint");c(c$,"set",function(a,b,d,c){this.index=a;this.x=b;this.y=
d;this.time=c},"~N,~N,~N,~N");j(c$,"toString",function(){return"[x = "+this.x+" y = "+this.y+" time = "+this.time+" ]"});c$=t(function(){this.action=0;this.nodes=null;this.time0=this.ptNext=0;this.vwr=null;p(this,arguments)},JV,"Gesture");q(c$,function(a,b){this.vwr=b;this.nodes=Array(a);for(var d=0;d<a;d++)this.nodes[d]=new JV.MotionPoint},"~N,JV.Viewer");c(c$,"setAction",function(a,b){this.action=a;this.ptNext=0;this.time0=b;for(var d=0;d<this.nodes.length;d++)this.nodes[d].index=-1},"~N,~N");c(c$,
"add",function(a,b,d,c){this.action=a;this.getNode(this.ptNext).set(this.ptNext,b,d,c-this.time0);this.ptNext++;return this.ptNext},"~N,~N,~N,~N");c(c$,"getTimeDifference",function(a){a=this.getPointCount2(a,0);if(2>a)return 0;var b=this.getNode(this.ptNext-1);a=this.getNode(this.ptNext-a);return b.time-a.time},"~N");c(c$,"getSpeedPixelsPerMillisecond",function(a,b){a=this.getPointCount2(a,b);if(2>a)return 0;var d=this.getNode(this.ptNext-1-b),c=this.getNode(this.ptNext-a-b),f=360*((d.x-c.x)/this.vwr.getScreenWidth()),
e=360*((d.y-c.y)/this.vwr.getScreenHeight());return Math.sqrt(f*f+e*e)/(d.time-c.time)},"~N,~N");c(c$,"getDX",function(a,b){a=this.getPointCount2(a,b);if(2>a)return 0;var d=this.getNode(this.ptNext-1-b),c=this.getNode(this.ptNext-a-b);return d.x-c.x},"~N,~N");c(c$,"getDY",function(a,b){a=this.getPointCount2(a,b);if(2>a)return 0;var d=this.getNode(this.ptNext-1-b),c=this.getNode(this.ptNext-a-b);return d.y-c.y},"~N,~N");c(c$,"getPointCount",function(){return this.ptNext});c(c$,"getPointCount2",function(a,
b){a>this.nodes.length-b&&(a=this.nodes.length-b);for(var d=a+1;0<=--d&&!(0<=this.getNode(this.ptNext-d-b).index););return d},"~N,~N");c(c$,"getNode",function(a){return this.nodes[(a+this.nodes.length+this.nodes.length)%this.nodes.length]},"~N");j(c$,"toString",function(){return 0==this.nodes.length?""+this:JV.binding.Binding.getMouseActionName(this.action,!1)+" nPoints = "+this.ptNext+" "+this.nodes[0]})});s("JV");v(["JU.BS"],"JV.AnimationManager",["J.api.Interface","JU.BSUtil"],function(){c$=t(function(){this.vwr=
this.animationThread=null;this.animationOn=!1;this.lastFrameDelayMs=this.firstFrameDelayMs=this.animationFps=0;this.bsVisibleModels=null;this.animationReplayMode=1073742070;this.animationFrames=this.bsDisplay=null;this.animationPaused=this.isMovie=!1;this.morphCount=this.caf=this.cmi=0;this.currentDirection=this.animationDirection=1;this.frameStep=this.lastFrameIndex=this.firstFrameIndex=0;this.backgroundModelIndex=-1;this.firstFrameDelay=this.currentMorphModel=0;this.lastFrameDelay=1;this.intAnimThread=
this.lastModelPainted=this.lastFramePainted=0;this.cai=-1;p(this,arguments)},JV,"AnimationManager");O(c$,function(){this.bsVisibleModels=new JU.BS});q(c$,function(a){this.vwr=a},"JV.Viewer");c(c$,"setAnimationOn",function(a){a!=this.animationOn&&(!a||this.vwr.headless?this.stopThread(!1):(this.vwr.tm.spinOn||this.vwr.refresh(3,"Anim:setAnimationOn"),this.setAnimationRange(-1,-1),this.resumeAnimation()))},"~B");c(c$,"stopThread",function(a){var b=!1;null!=this.animationThread&&(this.animationThread.interrupt(),
this.animationThread=null,b=!0);this.animationPaused=a;b&&!this.vwr.tm.spinOn&&this.vwr.refresh(3,"Viewer:setAnimationOff");this.animation(!1);this.vwr.setStatusFrameChanged(!1,!1)},"~B");c(c$,"setAnimationNext",function(){return this.setAnimationRelative(this.animationDirection)});c(c$,"currentIsLast",function(){return this.isMovie?this.lastFramePainted==this.caf:this.lastModelPainted==this.cmi});c(c$,"currentFrameIs",function(a){var b=this.cmi;return 0==this.morphCount?b==a:0.001>Math.abs(this.currentMorphModel-
a)},"~N");c(c$,"clear",function(){this.setMovie(null);this.initializePointers(0);this.setAnimationOn(!1);this.setModel(0,!0);this.currentDirection=1;this.cai=-1;this.setAnimationDirection(1);this.setAnimationFps(10);this.setAnimationReplayMode(1073742070,0,0);this.initializePointers(0)});c(c$,"getModelSpecial",function(a){switch(a){case -1:if(null!=this.animationFrames)return"1";a=this.firstFrameIndex;break;case 0:if(0<this.morphCount)return"-"+(1+this.currentMorphModel);a=this.cmi;break;case 1:if(null!=
this.animationFrames)return""+this.animationFrames.length;a=this.lastFrameIndex}return this.vwr.getModelNumberDotted(a)},"~N");c(c$,"setDisplay",function(a){this.bsDisplay=null==a||a.isEmpty()?null:JU.BSUtil.copy(a)},"JU.BS");c(c$,"setMorphCount",function(a){this.morphCount=this.isMovie?0:a},"~N");c(c$,"morph",function(a){var b=A(a);0.001>Math.abs(b-a)?a=b:0.999<Math.abs(b-a)&&(a=b+=1);a-=b;b-=1;if(0==a)this.currentMorphModel=b,this.setModel(b,!0);else{var d;this.setModel(b,!0);d=b+1;this.currentMorphModel=
b+a;d==b||(0>d||0>b)||this.vwr.ms.morphTrajectories(b,d,a)}},"~N");c(c$,"setModel",function(a,b){0>a&&this.stopThread(!1);var d=this.cmi,c=this.vwr.ms,f=null==c?0:c.mc;if(1==f)this.cmi=a=0;else if(0>a||a>=f)a=-1;var c=null,e=!1;if(this.cmi!=a){if(0<f){var f=this.vwr.ms,h=f.isJmolDataFrameForModel(a),j=f.isJmolDataFrameForModel(this.cmi);j&&f.setJmolDataFrame(null,-1,this.cmi);-1!=this.cmi&&this.vwr.saveModelOrientation();if(j||h)c=f.getJmolFrameType(a)+" "+a+" <--  "+this.cmi+" "+f.getJmolFrameType(this.cmi),
e=f.getJmolDataSourceFrame(a)==f.getJmolDataSourceFrame(this.cmi)}this.cmi=a;null!=c&&(0<=a&&this.vwr.restoreModelOrientation(a),e&&(0<=c.indexOf("quaternion")||0>c.indexOf("plot")&&0>c.indexOf("ramachandran")&&0>c.indexOf(" property "))&&this.vwr.restoreModelRotation(d))}this.setViewer(b)},"~N,~B");c(c$,"setBackgroundModelIndex",function(a){var b=this.vwr.ms;if(null==b||0>a||a>=b.mc)a=-1;this.backgroundModelIndex=a;0<=a&&this.vwr.ms.setTrajectory(a);this.vwr.setTainted(!0);this.setFrameRangeVisible()},
"~N");c(c$,"initializePointers",function(a){this.firstFrameIndex=0;this.lastFrameIndex=(0==a?0:this.getFrameCount())-1;this.frameStep=a;this.vwr.setFrameVariables()},"~N");c(c$,"setAnimationDirection",function(a){this.animationDirection=a},"~N");c(c$,"setAnimationFps",function(a){1>a&&(a=1);50<a&&(a=50);this.animationFps=a;this.vwr.setFrameVariables()},"~N");c(c$,"setAnimationReplayMode",function(a,b,d){this.firstFrameDelay=0<b?b:0;this.firstFrameDelayMs=A(1E3*this.firstFrameDelay);this.lastFrameDelay=
0<d?d:0;this.lastFrameDelayMs=A(1E3*this.lastFrameDelay);this.animationReplayMode=a;this.vwr.setFrameVariables()},"~N,~N,~N");c(c$,"setAnimationRange",function(a,b){var d=this.getFrameCount();0>a&&(a=0);0>b&&(b=d);a>=d&&(a=d-1);b>=d&&(b=d-1);this.currentMorphModel=this.firstFrameIndex=a;this.lastFrameIndex=b;this.frameStep=b<a?-1:1;this.rewindAnimation()},"~N,~N");c(c$,"pauseAnimation",function(){this.stopThread(!0)});c(c$,"reverseAnimation",function(){this.currentDirection=-this.currentDirection;
this.animationOn||this.resumeAnimation()});c(c$,"repaintDone",function(){this.lastModelPainted=this.cmi;this.lastFramePainted=this.caf});c(c$,"resumeAnimation",function(){0>this.cmi&&this.setAnimationRange(this.firstFrameIndex,this.lastFrameIndex);1>=this.getFrameCount()?this.animation(!1):(this.animation(!0),this.animationPaused=!1,null==this.animationThread&&(this.intAnimThread++,this.animationThread=J.api.Interface.getOption("thread.AnimationThread",this.vwr,"script"),this.animationThread.setManager(this,
this.vwr,B(-1,[this.firstFrameIndex,this.lastFrameIndex,this.intAnimThread])),this.animationThread.start()))});c(c$,"setAnimationLast",function(){this.setFrame(0<this.animationDirection?this.lastFrameIndex:this.firstFrameIndex)});c(c$,"rewindAnimation",function(){this.setFrame(0<this.animationDirection?this.firstFrameIndex:this.lastFrameIndex);this.currentDirection=1;this.vwr.setFrameVariables()});c(c$,"setAnimationPrevious",function(){return this.setAnimationRelative(-this.animationDirection)});
c(c$,"getAnimRunTimeSeconds",function(){var a=this.getFrameCount();if(this.firstFrameIndex==this.lastFrameIndex||0>this.lastFrameIndex||0>this.firstFrameIndex||this.lastFrameIndex>=a||this.firstFrameIndex>=a)return 0;for(var b=Math.min(this.firstFrameIndex,this.lastFrameIndex),a=Math.max(this.firstFrameIndex,this.lastFrameIndex),d=1*(a-b)/this.animationFps+this.firstFrameDelay+this.lastFrameDelay;b<=a;b++)d+=this.vwr.ms.getFrameDelayMs(this.modelIndexForFrame(b))/1E3;return d});c(c$,"setMovie",function(a){if(this.isMovie=
null!=a&&null==a.get("scripts")){this.animationFrames=a.get("frames");if(null==this.animationFrames||0==this.animationFrames.length)this.isMovie=!1;else if(this.caf=a.get("currentFrame").intValue(),0>this.caf||this.caf>=this.animationFrames.length)this.caf=0;this.setFrame(this.caf)}this.isMovie||(this.animationFrames=null);this.vwr.setBooleanProperty("_ismovie",this.isMovie);this.bsDisplay=null;this.currentMorphModel=this.morphCount=0;this.vwr.setFrameVariables()},"java.util.Map");c(c$,"modelIndexForFrame",
function(a){return this.isMovie?this.animationFrames[a]-1:a},"~N");c(c$,"getFrameCount",function(){return this.isMovie?this.animationFrames.length:this.vwr.ms.mc});c(c$,"setFrame",function(a){try{if(this.isMovie){var b=this.modelIndexForFrame(a);this.caf=a;a=b}else this.caf=a;this.setModel(a,!0)}catch(d){if(!H(d,Exception))throw d;}},"~N");c(c$,"getUnitCellAtomIndex",function(){return this.cai});c(c$,"setUnitCellAtomIndex",function(a){this.cai=a},"~N");c(c$,"setViewer",function(a){this.vwr.ms.setTrajectory(this.cmi);
this.vwr.tm.setFrameOffset(this.cmi);-1==this.cmi&&a&&this.setBackgroundModelIndex(-1);this.vwr.setTainted(!0);a=this.setFrameRangeVisible();this.vwr.setStatusFrameChanged(!1,!1);this.vwr.g.selectAllModels||this.setSelectAllSubset(2>a)},"~B");c(c$,"setSelectAllSubset",function(a){null!=this.vwr.ms&&this.vwr.slm.setSelectionSubset(a?this.vwr.ms.getModelAtomBitSetIncludingDeleted(this.cmi,!0):this.vwr.ms.getModelAtomBitSetIncludingDeletedBs(this.bsVisibleModels))},"~B");c(c$,"setFrameRangeVisible",
function(){var a=0;this.bsVisibleModels.clearAll();0<=this.backgroundModelIndex&&(this.bsVisibleModels.set(this.backgroundModelIndex),a=1);if(0<=this.cmi)return this.bsVisibleModels.set(this.cmi),++a;if(0==this.frameStep)return a;for(var b=0,a=0,d=this.firstFrameIndex;d!=this.lastFrameIndex;d+=this.frameStep){var c=this.modelIndexForFrame(d);this.vwr.ms.isJmolDataFrameForModel(c)||(this.bsVisibleModels.set(c),a++,b=d)}c=this.modelIndexForFrame(this.lastFrameIndex);if(this.firstFrameIndex==this.lastFrameIndex||
!this.vwr.ms.isJmolDataFrameForModel(c)||0==a)this.bsVisibleModels.set(c),0==a&&(this.firstFrameIndex=this.lastFrameIndex),a=0;1==a&&0>this.cmi&&this.setFrame(b);return a});c(c$,"animation",function(a){this.animationOn=a;this.vwr.setBooleanProperty("_animating",a)},"~B");c(c$,"setAnimationRelative",function(a){a=this.getFrameStep(a);var b=(this.isMovie?this.caf:this.cmi)+a,d=0,c=0,f;0<this.morphCount?(d=1/(this.morphCount+1),c=this.currentMorphModel+a*d,f=this.isNotInRange(c)):f=this.isNotInRange(b);
if(f)switch(this.animationReplayMode){case 1073742070:return!1;case 528411:c=b=this.animationDirection==this.currentDirection?this.firstFrameIndex:this.lastFrameIndex;break;case 1073742082:this.currentDirection=-this.currentDirection,b-=2*a,c-=2*a*d}if(1>this.morphCount){if(0>b||b>=this.getFrameCount())return!1;this.setFrame(b);return!0}this.morph(c+1);return!0},"~N");c(c$,"isNotInRange",function(a){var b=a-0.001;return b>this.firstFrameIndex&&b>this.lastFrameIndex||(b=a+0.001)<this.firstFrameIndex&&
b<this.lastFrameIndex},"~N");c(c$,"getFrameStep",function(a){return this.frameStep*a*this.currentDirection},"~N");E(c$,"FRAME_FIRST",-1,"FRAME_LAST",1,"MODEL_CURRENT",0)});s("JV.binding");v(["java.util.Hashtable"],"JV.binding.Binding","java.lang.Boolean java.util.Arrays JU.AU $.Lst $.PT $.SB J.api.Interface JU.Escape $.Logger".split(" "),function(){c$=t(function(){this.bindings=this.name=null;p(this,arguments)},JV.binding,"Binding");O(c$,function(){this.bindings=new java.util.Hashtable});c$.getMouseAction=
c(c$,"getMouseAction",function(a,b,d){2<a&&(a=2);switch(a){case 0:break;case 1:a=256;break;default:a=512}switch(d){case 4:d=4096;break;case 1:d=8192;break;case 5:d=16384;break;case 2:d=32768;break;case 3:d=32}return b&63|a|d},"~N,~N,~N");c$.getMouseActionStr=c(c$,"getMouseActionStr",function(a){if(null==a)return 0;var b=0;a=a.toUpperCase();b=0<=a.indexOf("MIDDLE")?8:0<=a.indexOf("RIGHT")?4:0<=a.indexOf("WHEEL")?32:16;0<=a.indexOf("DOWN")?b|=4096:0<=a.indexOf("DRAG")?b|=8192:0<=a.indexOf("UP")?b|=
16384:32!=b&&(b|=32768);32!=b&&0<=a.indexOf("DOUBLE")?b|=512:0<b&&(b|=256);288!=b&&0<=a.indexOf("ALT")&&(b|=8);0<=a.indexOf("CTRL")&&(b|=2);0<=a.indexOf("SHIFT")&&(b|=1);return b},"~S");c$.getButtonMods=c(c$,"getButtonMods",function(a){return a&63},"~N");c$.getClickCount=c(c$,"getClickCount",function(a){return(a&768)>>8},"~N");c$.getMouseActionName=c(c$,"getMouseActionName",function(a,b){var d=new JU.SB;if(0==a)return"";var c=JV.binding.Binding.includes(a,8)&&!JV.binding.Binding.includes(a,16)&&!JV.binding.Binding.includes(a,
4),f="      ".toCharArray();JV.binding.Binding.includes(a,2)&&(d.append("CTRL+"),f[5]="C");!c&&JV.binding.Binding.includes(a,8)&&(d.append("ALT+"),f[4]="A");JV.binding.Binding.includes(a,1)&&(d.append("SHIFT+"),f[3]="S");JV.binding.Binding.includes(a,16)?(f[2]="L",d.append("LEFT")):JV.binding.Binding.includes(a,4)?(f[2]="R",d.append("RIGHT")):c?(f[2]="M",d.append("MIDDLE")):JV.binding.Binding.includes(a,32)&&(f[2]="W",d.append("WHEEL"));JV.binding.Binding.includes(a,512)&&(d.append("+double"),f[1]=
"2");JV.binding.Binding.includes(a,4096)?(d.append("+down"),f[0]="1"):JV.binding.Binding.includes(a,8192)?(d.append("+drag"),f[0]="2"):JV.binding.Binding.includes(a,16384)?(d.append("+up"),f[0]="3"):JV.binding.Binding.includes(a,32768)&&(d.append("+click"),f[0]="4");return b?String.instantialize(f)+":"+d.toString():d.toString()},"~N,~B");c(c$,"getBindings",function(){return this.bindings});q(c$,function(){});c(c$,"bindAction",function(a,b){this.addBinding(a+"\t"+b,B(-1,[a,b]))},"~N,~N");c(c$,"bindName",
function(a,b){this.addBinding(a+"\t",Boolean.TRUE);this.addBinding(a+"\t"+b,u(-1,[JV.binding.Binding.getMouseActionName(a,!1),b]))},"~N,~S");c(c$,"unbindAction",function(a,b){0==a?this.unbindJmolAction(b):this.removeBinding(null,a+"\t"+b)},"~N,~N");c(c$,"unbindName",function(a,b){null==b?this.unbindMouseAction(a):this.removeBinding(null,a+"\t"+b)},"~N,~S");c(c$,"unbindJmolAction",function(a){var b=this.bindings.keySet().iterator();for(a="\t"+a;b.hasNext();){var d=b.next();d.endsWith(a)&&this.removeBinding(b,
d)}},"~N");c(c$,"addBinding",function(a,b){JU.Logger.debugging&&JU.Logger.debug("adding binding "+a+"\t==\t"+JU.Escape.e(b));this.bindings.put(a,b)},"~S,~O");c(c$,"removeBinding",function(a,b){JU.Logger.debugging&&JU.Logger.debug("removing binding "+b);null==a?this.bindings.remove(b):a.remove()},"java.util.Iterator,~S");c(c$,"unbindUserAction",function(a){var b=this.bindings.keySet().iterator();for(a="\t"+a;b.hasNext();){var d=b.next();d.endsWith(a)&&this.removeBinding(b,d)}},"~S");c(c$,"unbindMouseAction",
function(a){var b=this.bindings.keySet().iterator();for(a+="\t";b.hasNext();){var d=b.next();d.startsWith(a)&&this.removeBinding(b,d)}},"~N");c(c$,"isBound",function(a,b){return this.bindings.containsKey(a+"\t"+b)},"~N,~N");c(c$,"isUserAction",function(a){return this.bindings.containsKey(a+"\t")},"~N");c(c$,"getBindingInfo",function(a,b,d){var c=new JU.SB;d=null==d||d.equalsIgnoreCase("all")?null:d.toLowerCase();for(var f=Array(a.length),e=new JU.Lst,h,j=this.bindings.values().iterator();j.hasNext()&&
((h=j.next())||1);)if(!n(h,Boolean))if(JU.AU.isAS(h)){var l=h[0],p=h[1];(null==d||0<=d.indexOf("user")||0<=l.indexOf(d)||0<=p.indexOf(d))&&e.addLast(h)}else p=h,l=p[1],null==f[l]&&(f[l]=new JU.Lst),p=JV.binding.Binding.getMouseActionName(p[0],!0),(null==d||0<=(b[l]+";"+a[l]+";"+p).toLowerCase().indexOf(d))&&f[l].addLast(p);for(l=0;l<a.length;l++){var m;null==f[l]||0==(m=f[l].size())||this.addInfo(c,f[l].toArray(Array(m)),b[l],a[l])}for(l=0;l<e.size();l++)p=e.get(l),this.addInfo(c,u(-1,["USER:::"+
p[0]]),"user-defined",p[1]);return c.toString()},"~A,~A,~S");c(c$,"addInfo",function(a,b,d,c){java.util.Arrays.sort(b);JU.PT.leftJustify(a,"                      ",d);a.append("\t");d="";for(var f=a.length(),e=0;e<b.length;e++)a.append(d).append(b[e].substring(7)),d=", ";f=a.length()-f;20>f&&a.append("                 ".substring(0,20-f));a.append("\t").append(c).appendC("\n")},"JU.SB,~A,~S,~S");c$.includes=c(c$,"includes",function(a,b){return(a&b)==b},"~N,~N");c$.newBinding=c(c$,"newBinding",function(a,
b){return J.api.Interface.getInterface("JV.binding."+b+"Binding",a,"script")},"JV.Viewer,~S");E(c$,"LEFT",16,"MIDDLE",8,"RIGHT",4,"WHEEL",32,"ALT",8,"CTRL",2,"SHIFT",1,"CTRL_ALT",10,"CTRL_SHIFT",3,"MAC_COMMAND",20,"BUTTON_MASK",28,"BUTTON_MODIFIER_MASK",63,"SINGLE",256,"DOUBLE",512,"COUNT_MASK",768,"DOWN",4096,"DRAG",8192,"UP",16384,"CLICK",32768,"MODE_MASK",61440)});s("JV.binding");v(["JV.binding.JmolBinding"],"JV.binding.DragBinding",null,function(){c$=D(JV.binding,"DragBinding",JV.binding.JmolBinding);
q(c$,function(){I(this,JV.binding.DragBinding,[]);this.set("drag")});j(c$,"setSelectBindings",function(){this.bindAction(33040,30);this.bindAction(33041,35);this.bindAction(33048,34);this.bindAction(33049,32);this.bindAction(4368,31);this.bindAction(8464,13);this.bindAction(33040,17)})});s("JV.binding");v(["JV.binding.Binding"],"JV.binding.JmolBinding",null,function(){c$=D(JV.binding,"JmolBinding",JV.binding.Binding);q(c$,function(){I(this,JV.binding.JmolBinding,[]);this.set("toggle")});c(c$,"set",
function(a){this.name=a;this.setGeneralBindings();this.setSelectBindings()},"~S");c(c$,"setSelectBindings",function(){this.bindAction(33296,30);this.bindAction(33040,36)});c(c$,"setGeneralBindings",function(){this.bindAction(8474,45);this.bindAction(8454,45);this.bindAction(8721,45);this.bindAction(8712,45);this.bindAction(8464,25);this.bindAction(8720,25);this.bindAction(8472,28);this.bindAction(8453,28);this.bindAction(8465,29);this.bindAction(8456,29);this.bindAction(288,46);this.bindAction(8464,
40);this.bindAction(8464,16);this.bindAction(4370,23);this.bindAction(4356,23);this.bindAction(33040,2);this.bindAction(8467,38);this.bindAction(8723,6);this.bindAction(8475,39);this.bindAction(290,46);this.bindAction(289,46);this.bindAction(291,46);this.bindAction(290,38);this.bindAction(289,6);this.bindAction(291,39);this.bindAction(8464,44);this.bindAction(8464,41);this.bindAction(8465,42);this.bindAction(8473,13);this.bindAction(8465,14);this.bindAction(8472,27);this.bindAction(8465,26);this.bindAction(8464,
10);this.bindAction(8472,9);this.bindAction(8465,8);this.bindAction(33297,24);this.bindAction(33288,24);this.bindAction(33296,43);this.bindAction(8464,7);this.bindAction(8464,11);this.bindAction(8464,12);this.bindAction(33040,17);this.bindAction(33040,22);this.bindAction(33040,19);this.bindAction(33040,20);this.bindAction(33296,37);this.bindAction(33040,18);this.bindAction(33043,21);this.bindAction(33040,4);this.bindAction(33040,5);this.bindAction(33040,3);this.bindAction(33040,0);this.bindAction(33043,
1)})});s("JV");v(null,"JV.ColorManager","java.lang.Character $.Float JU.AU J.c.PAL JU.C $.ColorEncoder $.Elements $.Logger JV.JC".split(" "),function(){c$=t(function(){this.colorData=this.altArgbsCpk=this.argbsCpk=this.g3d=this.vwr=this.ce=null;this.isDefaultColorRasmol=!1;this.colixRubberband=22;this.colixBackgroundContrast=0;p(this,arguments)},JV,"ColorManager");q(c$,function(a,b){this.vwr=a;this.ce=new JU.ColorEncoder(null,a);this.g3d=b;this.argbsCpk=J.c.PAL.argbsCpk;this.altArgbsCpk=JU.AU.arrayCopyRangeI(JV.JC.altArgbsCpk,
0,-1)},"JV.Viewer,JU.GData");c(c$,"setDefaultColors",function(a){a?(this.isDefaultColorRasmol=!0,this.argbsCpk=JU.AU.arrayCopyI(JU.ColorEncoder.getRasmolScale(),-1)):(this.isDefaultColorRasmol=!1,this.argbsCpk=J.c.PAL.argbsCpk);this.altArgbsCpk=JU.AU.arrayCopyRangeI(JV.JC.altArgbsCpk,0,-1);this.ce.createColorScheme(a?"Rasmol=":"Jmol=",!0,!0);for(a=J.c.PAL.argbsCpk.length;0<=--a;)this.g3d.changeColixArgb(a,this.argbsCpk[a]);for(a=JV.JC.altArgbsCpk.length;0<=--a;)this.g3d.changeColixArgb(JU.Elements.elementNumberMax+
a,this.altArgbsCpk[a])},"~B");c(c$,"setRubberbandArgb",function(a){this.colixRubberband=0==a?0:JU.C.getColix(a)},"~N");c(c$,"setColixBackgroundContrast",function(a){this.colixBackgroundContrast=JU.C.getBgContrast(a)},"~N");c(c$,"getColixBondPalette",function(a,b){switch(b){case 19:return this.ce.getColorIndexFromPalette(a.getEnergy(),-2.5,-0.5,7,!1)}return 10},"JM.Bond,~N");c(c$,"getColixAtomPalette",function(a,b){var d=0,c;c=this.vwr.ms;switch(b){case 84:return null==this.colorData||a.i>=this.colorData.length||
Float.isNaN(this.colorData[a.i])?12:this.ce.getColorIndex(this.colorData[a.i]);case 0:case 1:c=a.getAtomicAndIsotopeNumber();if(c<JU.Elements.elementNumberMax)return this.g3d.getChangeableColix(c,this.argbsCpk[c]);d=c;c=JU.Elements.altElementIndexFromNumber(c);if(0<c)return this.g3d.getChangeableColix(JU.Elements.elementNumberMax+c,this.altArgbsCpk[c]);c=JU.Elements.getElementNumber(d);return this.g3d.getChangeableColix(c,this.argbsCpk[c]);case 2:return c=JU.ColorEncoder.quantize4(a.getPartialCharge(),
-1,1,JV.JC.PARTIAL_CHARGE_RANGE_SIZE),this.g3d.getChangeableColix(JV.JC.PARTIAL_CHARGE_COLIX_RED+c,JV.JC.argbsRwbScale[c]);case 3:return c=a.getFormalCharge()- -4,this.g3d.getChangeableColix(JV.JC.FORMAL_CHARGE_COLIX_RED+c,JV.JC.argbsFormalCharge[c]);case 68:case 5:return 68==b?(c=this.vwr.ms.getBfactor100Lo(),d=this.vwr.ms.getBfactor100Hi()):(c=0,d=1E4),this.ce.getColorIndexFromPalette(a.getBfactor100(),c,d,7,!1);case 86:return this.ce.getColorIndexFromPalette(a.group.getGroupParameter(1111490574),
-1,1,7,!1);case 70:return d=this.vwr.ms.getSurfaceDistanceMax(),this.ce.getColorIndexFromPalette(a.getSurfaceDistance100(),0,d,7,!1);case 24:return c=a.group.groupID,42<=c&&(c=24+"PGCATU".indexOf(Character.toUpperCase(a.group.group1))-1),this.ce.getColorIndexFromPalette(c,0,0,17,!1);case 8:return this.ce.getColorIndexFromPalette(a.group.groupID,0,0,5,!1);case 9:return this.ce.getColorIndexFromPalette(a.group.groupID,0,0,4,!1);case 75:return this.ce.getColorIndexFromPalette(a.group.selectedIndex,0,
a.group.chain.selectedGroupCount-1,1,!1);case 87:return c=this.vwr.ms.am[a.mi],this.ce.getColorIndexFromPalette(a.group.getBioPolymerIndexInModel(),0,(c.isBioModel?c.getBioPolymerCount():0)-1,1,!1);case 76:return this.ce.getColorIndexFromPalette(a.group.getSelectedMonomerIndex(),0,a.group.getSelectedMonomerCount()-1,1,!1);case 77:return this.ce.getColorIndexFromPalette(c.getMoleculeIndex(a.i,!0),0,c.getMoleculeCountInModel(a.mi)-1,0,!1);case 14:return d=a.mi,this.ce.getColorIndexFromPalette(c.getAltLocIndexInModel(d,
a.altloc),0,c.getAltLocCountInModel(d),0,!1);case 15:return d=a.mi,this.ce.getColorIndexFromPalette(c.getInsertionCodeIndexInModel(d,a.group.getInsertionCode()),0,c.getInsertionCountInModel(d),0,!1);case 16:c=a.getAtomicAndIsotopeNumber();d=this.getJmolOrRasmolArgb(c,1073741991);break;case 17:c=a.getAtomicAndIsotopeNumber();d=this.getJmolOrRasmolArgb(c,1073742116);break;case 7:d=a.group.getProteinStructureSubType().getColor();break;case 10:c=a.getChainID(),null==JU.ColorEncoder.argbsChainAtom&&(JU.ColorEncoder.argbsChainAtom=
this.getArgbs(1140850689),JU.ColorEncoder.argbsChainHetero=this.getArgbs(1612709894)),c=((0>c?0:256<=c?c-256:c)&31)%JU.ColorEncoder.argbsChainAtom.length,d=(a.isHetero()?JU.ColorEncoder.argbsChainHetero:JU.ColorEncoder.argbsChainAtom)[c]}return 0==d?22:JU.C.getColix(d)},"JM.Atom,~N");c(c$,"getArgbs",function(a){return this.vwr.getJBR().getArgbs(a)},"~N");c(c$,"getJmolOrRasmolArgb",function(a,b){switch(b){case 1073741991:if(a>=JU.Elements.elementNumberMax)break;return this.ce.getArgbFromPalette(a,
0,0,2);case 1073742116:if(a>=JU.Elements.elementNumberMax)break;return this.ce.getArgbFromPalette(a,0,0,3);default:return b}return JV.JC.altArgbsCpk[JU.Elements.altElementIndexFromNumber(a)]},"~N,~N");c(c$,"setElementArgb",function(a,b){if(1073741991==b&&this.argbsCpk===J.c.PAL.argbsCpk)return 0;b=this.getJmolOrRasmolArgb(a,b);this.argbsCpk===J.c.PAL.argbsCpk&&(this.argbsCpk=JU.AU.arrayCopyRangeI(J.c.PAL.argbsCpk,0,-1),this.altArgbsCpk=JU.AU.arrayCopyRangeI(JV.JC.altArgbsCpk,0,-1));if(a<JU.Elements.elementNumberMax){if(2147483647==
b)return JU.C.getColix(this.argbsCpk[a]);this.argbsCpk[a]=b;this.g3d.changeColixArgb(a,b);return 0}a=JU.Elements.altElementIndexFromNumber(a);this.altArgbsCpk[a]=b;this.g3d.changeColixArgb(JU.Elements.elementNumberMax+a,b);return 0},"~N,~N");c(c$,"getPropertyColorRange",function(){return this.ce.isReversed?K(-1,[this.ce.hi,this.ce.lo]):K(-1,[this.ce.lo,this.ce.hi])});c(c$,"setPropertyColorRangeData",function(a,b){this.colorData=a;this.ce.currentPalette=this.ce.createColorScheme(this.vwr.g.propertyColorScheme,
!0,!1);this.ce.hi=-3.4028235E38;this.ce.lo=3.4028235E38;if(null!=a){for(var c=null==b,g,f=c?a.length-1:b.nextSetBit(0);0<=f;f=c?f-1:b.nextSetBit(f+1))if(!Float.isNaN(g=a[f]))this.ce.hi=Math.max(this.ce.hi,g),this.ce.lo=Math.min(this.ce.lo,g);this.setPropertyColorRange(this.ce.lo,this.ce.hi)}},"~A,JU.BS");c(c$,"setPropertyColorRange",function(a,b){this.ce.setRange(a,b,a>b);JU.Logger.debugging&&JU.Logger.debug('ColorManager: color "'+this.ce.getCurrentColorSchemeName()+'" range '+a+" "+b)},"~N,~N");
c(c$,"setPropertyColorScheme",function(a,b,c){var g=0==a.length;g&&(a="=");var f=this.getPropertyColorRange();this.ce.currentPalette=this.ce.createColorScheme(a,!0,c);g||this.setPropertyColorRange(f[0],f[1]);this.ce.isTranslucent=b},"~S,~B,~B");c(c$,"getColorSchemeList",function(a){a=null==a||0==a.length?this.ce.currentPalette:this.ce.createColorScheme(a,!0,!1);return JU.ColorEncoder.getColorSchemeList(this.ce.getColorSchemeArray(a))},"~S");c(c$,"getColorEncoder",function(a){if(null==a||0==a.length)return this.ce;
var b=new JU.ColorEncoder(this.ce,this.vwr);b.currentPalette=b.createColorScheme(a,!1,!0);return 2147483647==b.currentPalette?null:b},"~S")});s("JV");v(["javajs.api.BytePoster","java.util.Hashtable"],"JV.FileManager","java.io.BufferedInputStream $.BufferedReader java.lang.Boolean java.net.URL $.URLEncoder java.util.Map JU.AU $.BArray $.Base64 $.LimitedLineReader $.Lst $.OC $.PT $.Rdr $.SB J.api.Interface J.io.FileReader JS.SV JU.Escape $.Logger JV.JC $.JmolAsyncException $.Viewer".split(" "),function(){c$=
t(function(){this.jzu=this.spartanDoc=this.vwr=null;this.pathForAllFiles="";this.nameAsGiven="zapped";this.lastFullPathName=this.fullPathName=null;this.lastNameAsGiven="zapped";this.spardirCache=this.pngjCache=this.cache=this.appletProxy=this.appletDocumentBaseURL=this.fileName=null;p(this,arguments)},JV,"FileManager",null,javajs.api.BytePoster);O(c$,function(){this.cache=new java.util.Hashtable});q(c$,function(a){this.vwr=a;this.clear()},"JV.Viewer");c(c$,"spartanUtil",function(){return null==this.spartanDoc?
this.spartanDoc=J.api.Interface.getInterface("J.adapter.readers.spartan.SpartanUtil",this.vwr,"fm getSpartanUtil()").set(this):this.spartanDoc});c(c$,"getJzu",function(){return null==this.jzu?this.jzu=J.api.Interface.getOption("io.JmolUtil",this.vwr,"file"):this.jzu});c(c$,"clear",function(){this.setFileInfo(u(-1,[this.vwr.getZapName()]));this.spardirCache=null});c(c$,"setLoadState",function(a){this.vwr.getPreserveState()&&a.put("loadState",this.vwr.g.getLoadState(a))},"java.util.Map");c(c$,"getPathForAllFiles",
function(){return this.pathForAllFiles});c(c$,"setPathForAllFiles",function(a){0<a.length&&(!a.endsWith("/")&&!a.endsWith("|"))&&(a+="/");return this.pathForAllFiles=a},"~S");c(c$,"setFileInfo",function(a){null==a?(this.fullPathName=this.lastFullPathName,this.nameAsGiven=this.lastNameAsGiven):(this.fullPathName=a[0],this.fileName=a[Math.min(1,a.length-1)],this.nameAsGiven=a[Math.min(2,a.length-1)],this.nameAsGiven.equals("zapped")||(this.lastNameAsGiven=this.nameAsGiven,this.lastFullPathName=this.fullPathName))},
"~A");c(c$,"getFileInfo",function(){return u(-1,[this.fullPathName,this.fileName,this.nameAsGiven])});c(c$,"getFullPathName",function(a){var b=null!=this.fullPathName?this.fullPathName:this.nameAsGiven;return!a||!b.equals("zapped")?b:null!=this.lastFullPathName?this.lastFullPathName:this.lastNameAsGiven},"~B");c(c$,"getFileName",function(){return null!=this.fileName?this.fileName:this.nameAsGiven});c(c$,"getAppletDocumentBase",function(){return null==this.appletDocumentBaseURL?"":this.appletDocumentBaseURL.toString()});
c(c$,"setAppletContext",function(a){try{Zystem.out.println('setting document base to "'+a+'"'),this.appletDocumentBaseURL=0==a.length?null:new java.net.URL(aa("java.net.URL"),a,null)}catch(b){if(H(b,java.net.MalformedURLException))Zystem.out.println("error setting document base to "+a);else throw b;}},"~S");c(c$,"setAppletProxy",function(a){this.appletProxy=null==a||0==a.length?null:a},"~S");c(c$,"createAtomSetCollectionFromFile",function(a,b,c){null==b.get("atomDataOnly")&&this.setLoadState(b);var g=
a;a=this.vwr.resolveDatabaseFormat(a);!g.equals(a)&&(0>g.indexOf("/")&&JV.Viewer.hasDatabasePrefix(g))&&b.put("dbName",g);a.endsWith("%2D%")&&(g=b.get("filter"),b.put("filter",(null==g?"":g)+"2D"),a=a.substring(0,a.length-4));var f=a.indexOf("::"),g=0<=f?a.substring(f+2):a,f=0<=f?a.substring(0,f):null;JU.Logger.info("\nFileManager.getAtomSetCollectionFromFile("+g+")"+(a.equals(g)?"":" //"+a));var e=this.getClassifiedName(g,!0);if(1==e.length)return e[0];a=e[0];e=e[1];b.put("fullPathName",(null==f?
"":f+"::")+JV.FileManager.fixDOSName(a));this.vwr.getBoolean(603979879)&&this.vwr.getBoolean(603979825)&&this.vwr.getChimeMessenger().update(a);b=new J.io.FileReader(this.vwr,e,a,g,f,null,b,c);b.run();return b.getAtomSetCollection()},"~S,java.util.Map,~B");c(c$,"createAtomSetCollectionFromFiles",function(a,b,c){this.setLoadState(b);for(var g=Array(a.length),f=Array(a.length),e=Array(a.length),h=0;h<a.length;h++){var j=a[h].indexOf("::"),l=0<=j?a[h].substring(j+2):a[h],j=0<=j?a[h].substring(0,j):null,
n=this.getClassifiedName(l,!0);if(1==n.length)return n[0];g[h]=n[0];a[h]=JV.FileManager.fixDOSName(n[0]);e[h]=j;f[h]=l}b.put("fullPathNames",g);b.put("fileTypes",e);a=this.newFilesReader(g,f,e,null,b,c);a.run();return a.getAtomSetCollection()},"~A,java.util.Map,~B");c(c$,"createAtomSetCollectionFromString",function(a,b,c){this.setLoadState(b);var g=0<=a.indexOf("Viewer.AddHydrogens")?this.getFileInfo():null;b=new J.io.FileReader(this.vwr,"string",null,null,null,JU.Rdr.getBR(a),b,c);b.run();null!=
g&&this.setFileInfo(g);!c&&!n(b.getAtomSetCollection(),String)&&this.setFileInfo(u(-1,["5\n\nC 0 0 0\nH .63 .63 .63\nH -.63 -.63 .63\nH -.63 .63 -.63\nH .63 -.63 -.63"===a?"Jmol Model Kit":"string"]));return b.getAtomSetCollection()},"~S,java.util.Map,~B");c(c$,"createAtomSeCollectionFromStrings",function(a,b,c,g){if(!c.containsKey("isData")){var f='"'+this.vwr.getDataSeparator()+'"',e='"'+(g?"append":"model")+' inline"',h=new JU.SB;h.append('set dataSeparator "~~~next file~~~";\ndata ').append(e);
for(var j=0;j<a.length;j++)0<j&&h.append("~~~next file~~~"),h.append(a[j]);h.append("end ").append(e).append(";set dataSeparator ").append(f);b.appendSB(h)}this.setLoadState(c);JU.Logger.info("FileManager.getAtomSetCollectionFromStrings(string[])");b=Array(a.length);f=Array(a.length);for(j=0;j<a.length;j++)b[j]="string["+j+"]",f[j]=JV.FileManager.newDataReader(this.vwr,a[j]);a=this.newFilesReader(b,b,null,f,c,g);a.run();return a.getAtomSetCollection()},"~A,JU.SB,java.util.Map,~B");c(c$,"createAtomSeCollectionFromArrayData",
function(a,b,c){JU.Logger.info("FileManager.getAtomSetCollectionFromArrayData(Vector)");for(var g=a.size(),f=Array(g),e=Array(g),h=0;h<g;h++)f[h]="String["+h+"]",e[h]=JV.FileManager.newDataReader(this.vwr,a.get(h));a=this.newFilesReader(f,f,null,e,b,c);a.run();return a.getAtomSetCollection()},"JU.Lst,java.util.Map,~B");c$.newDataReader=c(c$,"newDataReader",function(a,b){var c=n(b,String)?"String":JU.AU.isAS(b)?"Array":n(b,JU.Lst)?"List":null;return null==c?null:J.api.Interface.getInterface("JU."+
c+"DataReader",a,"file").setData(b)},"JV.Viewer,~O");c(c$,"newFilesReader",function(a,b,c,g,f,e){var h=J.api.Interface.getOption("io.FilesReader",this.vwr,"file");h.set(this,this.vwr,a,b,c,g,f,e);return h},"~A,~A,~A,~A,java.util.Map,~B");c(c$,"createAtomSetCollectionFromDOM",function(a,b){var c=J.api.Interface.getOption("io.DOMReader",this.vwr,"file");c.set(this,this.vwr,a,b);c.run();return c.getAtomSetCollection()},"~O,java.util.Map");c(c$,"createAtomSetCollectionFromReader",function(a,b,c,g){a=
new J.io.FileReader(this.vwr,b,a,null,null,c,g,!1);a.run();return a.getAtomSetCollection()},"~S,~S,~O,java.util.Map");c(c$,"getBufferedInputStream",function(a){a=this.getBufferedReaderOrErrorMessageFromName(a,Array(2),!0,!0);return n(a,java.io.BufferedInputStream)?a:null},"~S");c(c$,"getBufferedInputStreamOrErrorMessageFromName",function(a,b,c,g,f,e,h){var j=null,l=null;b=h&&null==f?b=this.getPngjOrDroppedBytes(b,a):null;try{h&&(0<=a.indexOf(".png")&&null==this.pngjCache&&!this.vwr.getBoolean(603979960))&&
(this.pngjCache=new java.util.Hashtable);if(null==b){var p=0<=a.indexOf("?POST?_PNGJBIN_"),m=p||0<=a.indexOf("?POST?_PNGJ_");if(0<a.indexOf("?POST?_PNG_")||m){var r=Array(1),q=this.vwr.getImageAsBytes(m?"PNGJ":"PNG",0,0,-1,r);if(null!=r[0])return r[0];p?(f=q,a=JU.PT.rep(a,"?_","=_")):a=(new JU.SB).append(a).append("=").appendSB(JU.Base64.getBase64(q)).toString()}var s=JU.OC.urlTypeIndex(a),p=0<=s,m=null;if(p&&0<=(s=a.indexOf("?POST?")))m=a.substring(s+6),a=a.substring(0,s);var t=null!=this.appletDocumentBaseURL;
if(t||p){t&&(p&&null!=this.appletProxy)&&(a=this.appletProxy+"?url="+this.urlEncode(a));var u=t?new java.net.URL(this.appletDocumentBaseURL,a,null):new java.net.URL(aa("java.net.URL"),a,null);if(g)return null;a=u.toString();c&&0>a.toLowerCase().indexOf("password")&&JU.Logger.info("FileManager opening url "+a);l=this.vwr.apiPlatform.getURLContents(u,f,m,!1);q=null;if(n(l,JU.SB)){a=l;if(e&&!JU.Rdr.isBase64(a))return JU.Rdr.getBR(a.toString());q=JU.Rdr.getBytesFromSB(a)}else JU.AU.isAB(l)&&(q=l);null!=
q&&(l=JU.Rdr.getBIS(q))}else if(!h||null==(b=this.cacheGet(a,!0)))c&&JU.Logger.info("FileManager opening file "+a),l=this.vwr.apiPlatform.getBufferedFileInputStream(a);if(n(l,String))return l}j=null==b?l:JU.Rdr.getBIS(b);g&&(j.close(),j=null);return j}catch(v){if(H(v,Exception))try{null!=j&&j.close()}catch(w){if(!H(w,java.io.IOException))throw w;}else throw v;}return""+v},"~S,~S,~B,~B,~A,~B,~B");c$.getBufferedReaderForResource=c(c$,"getBufferedReaderForResource",function(a,b,c,g){g=a.vwrOptions.get("codePath")+
c+g;if(a.async){a=a.fm.cacheGet(g,!1);if(null==a)throw new JV.JmolAsyncException(g);return JU.Rdr.getBufferedReader(JU.Rdr.getBIS(a),null)}return a.fm.getBufferedReaderOrErrorMessageFromName(g,u(-1,[null,null]),!1,!0)},"JV.Viewer,~O,~S,~S");c(c$,"urlEncode",function(a){try{return java.net.URLEncoder.encode(a,"utf-8")}catch(b){if(H(b,java.io.UnsupportedEncodingException))return a;throw b;}},"~S");c(c$,"getFullPathNameOrError",function(a,b,c){var g=this.getClassifiedName(JV.JC.fixProtocol(a),!0);if(null==
g||null==g[0]||2>g.length)return u(-1,[null,"cannot read file name: "+a]);a=g[0];g=JV.FileManager.fixDOSName(g[0]);a=JU.Rdr.getZipRoot(a);b=this.getBufferedInputStreamOrErrorMessageFromName(a,g,!1,!b,null,!1,!b);c[0]=g;n(b,String)&&(c[1]=b);return b},"~S,~B,~A");c(c$,"getBufferedReaderOrErrorMessageFromName",function(a,b,c,g){a=JV.JC.fixProtocol(a);var f=this.cacheGet(a,!1),e=JU.AU.isAB(f),h=e?f:null;if(a.startsWith("cache://")){if(null==f)return"cannot read "+a;if(e)h=f;else return JU.Rdr.getBR(f)}f=
this.getClassifiedName(a,!0);if(null==f)return"cannot read file name: "+a;null!=b&&(b[0]=JV.FileManager.fixDOSName(f[0]));return this.getUnzippedReaderOrStreamFromName(f[0],h,!1,c,!1,g,null)},"~S,~A,~B,~B");c(c$,"getUnzippedReaderOrStreamFromName",function(a,b,c,g,f,e,h){if(e&&null==b){var j=a.endsWith(".spt")?u(-1,[null,null,null]):0>a.indexOf(".spardir")?null:this.spartanUtil().getFileList(a,f);if(null!=j)return j}a=JV.JC.fixProtocol(a);null==b&&(null!=(b=this.getCachedPngjBytes(a))&&null!=h)&&
h.put("sourcePNGJ",Boolean.TRUE);e=a=a.$replace("#_DOCACHE_","");f=null;0<=a.indexOf("|")&&(f=JU.PT.split(a.$replace("\\","/"),"|"),null==b&&JU.Logger.info("FileManager opening zip "+a),a=f[0]);a=null==b?this.getBufferedInputStreamOrErrorMessageFromName(a,e,!0,!1,null,!g,!0):JU.AU.isAB(b)?JU.Rdr.getBIS(b):b;try{if(n(a,String)||n(a,java.io.BufferedReader))return a;JU.Rdr.isGzipS(a)?a=JU.Rdr.getUnzippedInputStream(this.vwr.getJzt(),a):JU.Rdr.isBZip2S(a)&&(a=JU.Rdr.getUnzippedInputStreamBZip2(this.vwr.getJzt(),
a));if(JU.Rdr.isTar(a))return j=this.vwr.getJzt().getZipFileDirectory(a,f,1,g),n(j,String)?JU.Rdr.getBR(j):j;if(g&&null==f)return a;if(JU.Rdr.isCompoundDocumentS(a)){var l=J.api.Interface.getInterface("JU.CompoundDocument",this.vwr,"file");l.setDocStream(this.vwr.getJzt(),a);var p=l.getAllDataFiles("Molecule","Input").toString();return g?JU.Rdr.getBIS(p.getBytes()):JU.Rdr.getBR(p)}if(JU.Rdr.isMessagePackS(a)||JU.Rdr.isPickleS(a))return a;a=JU.Rdr.getPngZipStream(a,!0);if(JU.Rdr.isZipS(a)){if(c)return this.vwr.getJzt().newZipInputStream(a);
j=this.vwr.getJzt().getZipFileDirectory(a,f,1,g);return n(j,String)?JU.Rdr.getBR(j):j}return g?a:JU.Rdr.getBufferedReader(a,null)}catch(m){if(H(m,Exception))return m.toString();throw m;}},"~S,~O,~B,~B,~B,~B,java.util.Map");c(c$,"getZipDirectory",function(a,b,c){a=this.getBufferedInputStreamOrErrorMessageFromName(a,a,!1,!1,null,!1,c);return this.vwr.getJzt().getZipDirectoryAndClose(a,b?"JmolManifest":null)},"~S,~B,~B");c(c$,"getFileAsBytes",function(a,b){if(null==a)return null;var c=a,g=null;0<=a.indexOf("|")&&
(g=JU.PT.split(a,"|"),a=g[0]);var f=null!=g?null:this.getPngjOrDroppedBytes(c,a);if(null==f){c=this.getBufferedInputStreamOrErrorMessageFromName(a,c,!1,!1,null,!1,!0);if(n(c,String))return"Error:"+c;try{f=null!=b||null==g||1>=g.length||!JU.Rdr.isZipS(c)&&!JU.Rdr.isPngZipStream(c)&&!JU.Rdr.isTar(c)?JU.Rdr.getStreamAsBytes(c,b):this.vwr.getJzt().getZipFileContentsAsBytes(c,g,1),c.close()}catch(e){if(H(e,Exception))return e.toString();throw e;}}if(null==b||!JU.AU.isAB(f))return f;b.write(f,0,-1);return f.length+
" bytes"},"~S,JU.OC");c(c$,"getFileAsMap",function(a,b){var c=new java.util.Hashtable,g;if(null==a){g=Array(1);var f=this.vwr.getImageAsBytes(b,-1,-1,-1,g);if(null!=g[0])return c.put("_ERROR_",g[0]),c;g=JU.Rdr.getBIS(f)}else{f=Array(2);g=this.getFullPathNameOrError(a,!0,f);if(n(g,String))return c.put("_ERROR_",g),c;if(!this.checkSecurity(f[0]))return c.put("_ERROR_","java.io. Security exception: cannot read file "+f[0]),c}try{this.vwr.getJzt().readFileAsMap(g,c,a)}catch(e){if(H(e,Exception))c.clear(),
c.put("_ERROR_",""+e);else throw e;}return c},"~S,~S");c(c$,"getFileDataAsString",function(a,b,c,g,f){a[1]="";var e=a[0];if(null==e)return!1;c=this.getBufferedReaderOrErrorMessageFromName(e,a,!1,c);if(n(c,String))return a[1]=c,!1;if(f&&!this.checkSecurity(a[0]))return a[1]="java.io. Security exception: cannot read file "+a[0],!1;try{return JU.Rdr.readAllAsString(c,b,g,a,1)}catch(h){if(H(h,Exception))return!1;throw h;}},"~A,~N,~B,~B,~B");c(c$,"checkSecurity",function(a){if(!a.startsWith("file:"))return!0;
var b=a.lastIndexOf("/");return a.lastIndexOf(":/")==b-1||0<=a.indexOf("/.")||a.lastIndexOf(".")<a.lastIndexOf("/")?!1:!0},"~S");c(c$,"loadImage",function(a,b,c){var g=null,f=null,e=null,h=null!=b&&b.startsWith("\u0001");if(h){if(b.equals("\u0001closeall\u0001null"))return this.vwr.loadImageData(Boolean.TRUE,"\u0001closeall","\u0001closeall",null);if("\u0001close".equals(a))return this.vwr.loadImageData(Boolean.FALSE,"\u0001close",b,null)}n(a,java.util.Map)&&(a=a.containsKey("_DATA_")?a.get("_DATA_"):
a.get("_IMAGE_"));n(a,JS.SV)&&(a=a.value);var j=n(a,String)?a:null,l=!1;null!=j&&j.startsWith(";base64,")?e=JU.Base64.decodeBase64(j):n(a,JU.BArray)?e=a.data:null==b||n(a,String)?(j=this.getClassifiedName(a,!0),f=null==j?"cannot read file name: "+a:JV.FileManager.fixDOSName(j[0]),null!=j&&(g=this.getImage(f,b,c)),l=null==g):g=a;null!=e&&(g=this.getImage(e,b,!0),l=!1);n(g,String)&&(f=g,g=null);!JV.Viewer.isJS&&(null!=g&&null!=e)&&(f=";base64,"+JU.Base64.getBase64(e).toString());return!JV.Viewer.isJS||
h&&null==f||!h&&null!=g?this.vwr.loadImageData(g,f,b,null):l},"~O,~S,~B");c(c$,"getImage",function(a,b,c){return this.getJzu().getImage(this.vwr,a,b,c)},"~O,~S,~B");c(c$,"getClassifiedName",function(a,b){if(null==a)return u(-1,[null]);var c=0<this.pathForAllFiles.length;if(a.startsWith("?")||a.startsWith("http://?")||a.startsWith("https://?")){if(!JV.Viewer.isJS&&null==(a=this.vwr.dialogAsk("Load",a,null)))return u(-1,[b?"#CANCELED#":null]);c=!1}var g=null,f=null,e=null;if(a.startsWith("cache://"))return e=
Array(3),e[0]=e[2]=a,e[1]=JV.FileManager.stripPath(e[0]),e;a=this.vwr.resolveDatabaseFormat(a);0>a.indexOf(":")&&0!=a.indexOf("/")&&(a=JV.FileManager.addDirectory(this.vwr.getDefaultDirectory(),a));if(null==this.appletDocumentBaseURL)if(0<=JU.OC.urlTypeIndex(a)||this.vwr.haveAccess(JV.Viewer.ACCESS.NONE)||this.vwr.haveAccess(JV.Viewer.ACCESS.READSPT)&&!a.endsWith(".spt")&&!a.endsWith("/"))try{f=new java.net.URL(aa("java.net.URL"),a,null)}catch(h){if(H(h,java.net.MalformedURLException))return u(-1,
[b?h.toString():null]);throw h;}else var g=this.vwr.apiPlatform.newFile(a),e=g.getFullPath(),j=g.getName(),e=u(-1,[null==e?j:e,j,null==e?j:"file:/"+e.$replace("\\","/")]);else try{if(1==a.indexOf(":\\")||1==a.indexOf(":/"))a="file:/"+a;f=new java.net.URL(this.appletDocumentBaseURL,a,null)}catch(l){if(H(l,java.net.MalformedURLException))return u(-1,[b?l.toString():null]);throw l;}null!=f&&(e=Array(3),e[0]=e[2]=f.toString(),e[1]=JV.FileManager.stripPath(e[0]));c&&(c=e[0],e[0]=this.pathForAllFiles+e[1],
JU.Logger.info("FileManager substituting "+c+" --\x3e "+e[0]));b&&JU.OC.isLocal(e[0])&&(c=e[0],null==g&&(c=JU.PT.trim(e[0].substring(e[0].indexOf(":")+1),"/")),g=c.length-e[1].length-1,0<g&&(c=c.substring(0,g),JV.FileManager.setLocalPath(this.vwr,c,!0)));return e},"~S,~B");c$.addDirectory=c(c$,"addDirectory",function(a,b){if(0==a.length)return b;var c=0<b.length?b.charAt(0):" ",g=a.toLowerCase();if((g.endsWith(".zip")||g.endsWith(".tar"))&&"|"!=c&&"/"!=c)a+="|";return a+("/"==c||"/"==c||"|"==(c=a.charAt(a.length-
1))||"/"==c?"":"/")+b},"~S,~S");c(c$,"getDefaultDirectory",function(a){a=this.getClassifiedName(a,!0);if(null==a)return"";a=JV.FileManager.fixPath(a[0]);return null==a?"":a.substring(0,a.lastIndexOf("/"))},"~S");c$.fixPath=c(c$,"fixPath",function(a){a=JV.FileManager.fixDOSName(a);a=JU.PT.rep(a,"/./","/");var b=a.lastIndexOf("//")+1;1>b&&(b=a.indexOf(":/")+1);1>b&&(b=a.indexOf("/"));if(0>b)return null;var c=a.substring(0,b);for(a=a.substring(b);0<=(b=a.lastIndexOf("/../"));){var g=a.substring(0,b).lastIndexOf("/");
if(0>g)return JU.PT.rep(c+a,"/../","/");a=a.substring(0,g)+a.substring(b+3)}0==a.length&&(a="/");return c+a},"~S");c(c$,"getFilePath",function(a,b,c){a=this.getClassifiedName(a,!1);return null==a||1==a.length?"":c?a[1]:b?a[2]:null==a[0]?"":JV.FileManager.fixDOSName(a[0])},"~S,~B,~B");c$.getLocalDirectory=c(c$,"getLocalDirectory",function(a,b){var c=a.getP(b?"currentLocalPath":"defaultDirectoryLocal");b&&0==c.length&&(c=a.getP("defaultDirectoryLocal"));if(0==c.length)return a.isApplet?null:a.apiPlatform.newFile(Zystem.getProperty("user.dir",
"."));a.isApplet&&0==c.indexOf("file:/")&&(c=c.substring(6));c=a.apiPlatform.newFile(c);try{return c.isDirectory()?c:c.getParentAsFile()}catch(g){if(H(g,Exception))return null;throw g;}},"JV.Viewer,~B");c$.setLocalPath=c(c$,"setLocalPath",function(a,b,c){for(;b.endsWith("/")||b.endsWith("\\");)b=b.substring(0,b.length-1);a.setStringProperty("currentLocalPath",b);c||a.setStringProperty("defaultDirectoryLocal",b)},"JV.Viewer,~S,~B");c$.getLocalPathForWritingFile=c(c$,"getLocalPathForWritingFile",function(a,
b){if(b.startsWith("http://")||b.startsWith("https://"))return b;b=JU.PT.rep(b,"?","");if(0==b.indexOf("file:/"))return b.substring(6);if(0==b.indexOf("/")||0<=b.indexOf(":"))return b;var c=null;try{c=JV.FileManager.getLocalDirectory(a,!1)}catch(g){if(!H(g,Exception))throw g;}return null==c?b:JV.FileManager.fixPath(c.toString()+"/"+b)},"JV.Viewer,~S");c$.fixDOSName=c(c$,"fixDOSName",function(a){return 0<=a.indexOf(":\\")?a.$replace("\\","/"):a},"~S");c$.stripPath=c(c$,"stripPath",function(a){var b=
Math.max(a.lastIndexOf("|"),a.lastIndexOf("/"));return a.substring(b+1)},"~S");c$.isScriptType=c(c$,"isScriptType",function(a){return JU.PT.isOneOf(a.toLowerCase().substring(a.lastIndexOf(".")+1),";pse;spt;png;pngj;jmol;zip;")},"~S");c$.determineSurfaceFileType=c(c$,"determineSurfaceFileType",function(a){var b=null;if(n(a,JU.Rdr.StreamReader)){var c=a.getStream();if(c.markSupported())try{c.mark(300);var g=Q(300,0);c.read(g,0,300);c.reset();if(131==(g[0]&255))return"BCifDensity";if(80==g[0]&&77==g[1]&&
1==g[2]&&0==g[3])return"Pmesh";if(77==g[208]&&65==g[209]&&80==g[210])return"Mrc";if(20==g[0]&&0==g[1]&&0==g[2]&&0==g[3])return"DelPhi";if(0==g[36]&&100==g[37])return"Dsn6"}catch(f){if(!H(f,java.io.IOException))throw f;}}c=null;try{c=new JU.LimitedLineReader(a,16E3),b=c.getHeader(0)}catch(e){if(!H(e,Exception))throw e;}if(null==c||null==b||0==b.length)return null;if(0<=b.indexOf("\x00")){if(131==b.charCodeAt(0))return"BCifDensity";if(0==b.indexOf("PM\u0001\x00"))return"Pmesh";if(208==b.indexOf("MAP "))return"Mrc";
if(0==b.indexOf("\u0014\x00\x00\x00"))return"DelPhi";if(37<b.length&&(0==b.charCodeAt(36)&&100==b.charCodeAt(37)||0==b.charCodeAt(36)&&100==b.charCodeAt(37)))return"Dsn6"}switch(b.charAt(0)){case "@":if(0==b.indexOf("@text"))return"Kinemage";break;case "#":if(0<=b.indexOf(".obj"))return"Obj";if(0<=b.indexOf("MSMS"))return"Msms";break;case "&":if(0==b.indexOf("&plot"))return"Jaguar";break;case "\r":case "\n":if(0<=b.indexOf("ZYX"))return"Xplor"}if(0<=b.indexOf("Here is your gzipped map"))return"UPPSALA"+
b;if(b.startsWith("data_SERVER"))return"CifDensity";if(b.startsWith("4MESHC"))return"Pmesh4";if(0<=b.indexOf("! nspins"))return"CastepDensity";if(0<=b.indexOf("<jvxl")&&0<=b.indexOf("<?xml"))return"JvxlXml";if(0<=b.indexOf("#JVXL+"))return"Jvxl+";if(0<=b.indexOf("#JVXL"))return"Jvxl";if(0<=b.indexOf("#JmolPmesh"))return"Pmesh";if(0<=b.indexOf("#obj")||0<=b.indexOf("#pmesh"))return"Obj";if(0<=b.indexOf("<efvet "))return"Efvet";if(0<=b.indexOf("usemtl"))return"Obj";if(0==b.indexOf("# object with"))return"Nff";
if(0<=b.indexOf("BEGIN_DATAGRID_3D")||0<=b.indexOf("BEGIN_BANDGRID_3D"))return"Xsf";if(0<=b.indexOf("tiles in x, y"))return"Ras3D";if(0<=b.indexOf(" 0.00000e+00 0.00000e+00      0      0\n"))return"Uhbd";b=c.readLineWithNewline();if(0==b.indexOf("object 1 class gridpositions counts"))return"Apbs";a=JU.PT.getTokens(b);var h=c.readLineWithNewline();if(2==a.length&&(3==JU.PT.parseInt(a[0])&&-2147483648!=JU.PT.parseInt(a[1]))&&(a=JU.PT.getTokens(h),3==a.length&&-2147483648!=JU.PT.parseInt(a[0])&&-2147483648!=
JU.PT.parseInt(a[1])&&-2147483648!=JU.PT.parseInt(a[2])))return"PltFormatted";g=c.readLineWithNewline();if(b.startsWith("v ")&&h.startsWith("v ")&&g.startsWith("v "))return"Obj";h=JU.PT.parseInt(g);if(-2147483648==h)return 0==g.indexOf("+")?"Jvxl+":null;a=JU.PT.getTokens(g);if(0<a[0].indexOf("."))return 60<=g.length||3!=a.length?null:"VaspChgcar";if(0<=h)return 4==a.length||5==a.length&&a[4].equals("1")?"Cube":null;for(a=4+-h;0<=--a;)if(null==(b=c.readLineWithNewline()))return null;b=JU.PT.parseInt(b);
return-2147483648==b?null:0>b?"Jvxl":"Cube"},"java.io.BufferedReader");c$.getManifestScriptPath=c(c$,"getManifestScriptPath",function(a){if(0<=a.indexOf("$SCRIPT_PATH$"))return"";var b=0<=a.indexOf("\n")?"\n":"\r";if(0<=a.indexOf(".spt")){a=JU.PT.split(a,b);for(b=a.length;0<=--b;)if(0<=a[b].indexOf(".spt"))return"|"+JU.PT.trim(a[b],"\r\n \t")}return null},"~S");c$.getFileReferences=c(c$,"getFileReferences",function(a,b,c){for(var g=0;g<JV.FileManager.scriptFilePrefixes.length;g++)for(var f=JV.FileManager.scriptFilePrefixes[g],
e=-1;0<=(e=a.indexOf(f,e+1));){var h=JV.FileManager.stripTypePrefix(JU.PT.getQuotedStringAt(a,e));0<=h.indexOf("\\u")&&(h=JU.Escape.unescapeUnicode(h));b.addLast(h);null!=c&&(0<=h.indexOf("\\u")&&(h=JU.Escape.unescapeUnicode(h)),c.addLast(h))}},"~S,JU.Lst,JU.Lst");c$.setScriptFileReferences=c(c$,"setScriptFileReferences",function(a,b,c,g){null!=b&&(a=JV.FileManager.setScriptFileRefs(a,b,!0));null!=c&&(a=JV.FileManager.setScriptFileRefs(a,c,!1));a=JU.PT.rep(a,'\u0001"','"');if(null!=g){for(;g.endsWith("/");)g=
g.substring(0,g.length-1);for(b=0;b<JV.FileManager.scriptFilePrefixes.length;b++)c=JV.FileManager.scriptFilePrefixes[b],a=JU.PT.rep(a,c+".",c+g)}return a},"~S,~S,~S,~S");c$.setScriptFileRefs=c(c$,"setScriptFileRefs",function(a,b,c){if(null==b)return a;var g=0==b.length,f=new JU.Lst;JV.FileManager.getFileReferences(a,f,null);for(var e=new JU.Lst,h=new JU.Lst,j=f.size(),l=0;l<j;l++){var n=f.get(l),m=n;if(c==JU.OC.isLocal(m)){var r=g?-1:m.indexOf("/"+b+"/");0<=r?m=m.substring(r+1):(r=m.lastIndexOf("/"),
0>r&&!g&&(m="/"+m),(0>r||g)&&r++,m=b+m.substring(r))}JU.Logger.info("FileManager substituting "+n+" --\x3e "+m);e.addLast('"'+n+'"');h.addLast('\u0001"'+m+'"')}return JU.PT.replaceStrings(a,e,h)},"~S,~S,~B");c(c$,"cachePut",function(a,b){a=JV.FileManager.fixDOSName(a);JU.Logger.debugging&&JU.Logger.debug("cachePut "+a);null==b||"".equals(b)?this.cache.remove(a):(this.cache.put(a,b),this.getCachedPngjBytes(a))},"~S,~O");c(c$,"cacheGet",function(a,b){a=JV.FileManager.fixDOSName(a);var c=a.indexOf("|");
0<=c&&!a.endsWith("##JmolSurfaceInfo##")&&(a=a.substring(0,c));a=this.getFilePath(a,!0,!1);c=null;(c=Jmol.Cache.get(a))||(c=this.cache.get(a));return b&&n(c,String)?null:c},"~S,~B");c(c$,"cacheClear",function(){JU.Logger.info("cache cleared");this.cache.clear();null!=this.pngjCache&&(this.pngjCache=null,JU.Logger.info("PNGJ cache cleared"))});c(c$,"cacheFileByNameAdd",function(a,b){if(null==a||!b&&a.equalsIgnoreCase(""))return this.cacheClear(),-1;var c;if(b){a=JV.JC.fixProtocol(this.vwr.resolveDatabaseFormat(a));
c=this.getFileAsBytes(a,null);if(n(c,String))return 0;this.cachePut(a,c)}else{if(a.endsWith("*"))return JU.AU.removeMapKeys(this.cache,a.substring(0,a.length-1));c=this.cache.remove(JV.FileManager.fixDOSName(a))}return null==c?0:(n(c,String),c.length)},"~S,~B");c(c$,"cacheList",function(){for(var a=new java.util.Hashtable,b,c=this.cache.entrySet().iterator();c.hasNext()&&((b=c.next())||1);)a.put(b.getKey(),Integer.$valueOf(JU.AU.isAB(b.getValue())?b.getValue().length:b.getValue().toString().length));
return a});c(c$,"getCanonicalName",function(a){var b=this.getClassifiedName(a,!0);return null==b?a:b[2]},"~S");c(c$,"recachePngjBytes",function(a,b){null!=this.pngjCache&&this.pngjCache.containsKey(a)&&(this.pngjCache.put(a,b),JU.Logger.info("PNGJ recaching "+a+" ("+b.length+")"))},"~S,~A");c(c$,"getPngjOrDroppedBytes",function(a,b){var c=this.getCachedPngjBytes(a);return null==c?this.cacheGet(b,!0):c},"~S,~S");c(c$,"getCachedPngjBytes",function(a){return null==a||null==this.pngjCache||0>a.indexOf(".png")?
null:this.getJzu().getCachedPngjBytes(this,a)},"~S");j(c$,"postByteArray",function(a,b){if(a.startsWith("cache://"))return this.cachePut(a,b),"OK "+b.length+"cached";var c=this.getBufferedInputStreamOrErrorMessageFromName(a,null,!1,!1,b,!1,!0);if(n(c,String))return c;try{c=JU.Rdr.getStreamAsBytes(c,null)}catch(g){if(H(g,java.io.IOException))try{c.close()}catch(f){if(!H(f,java.io.IOException))throw f;}else throw g;}return null==c?"":JU.Rdr.fixUTF(c)},"~S,~A");c$.isJmolType=c(c$,"isJmolType",function(a){return a.equals("PNG")||
a.equals("PNGJ")||a.equals("JMOL")||a.equals("ZIP")||a.equals("ZIPALL")},"~S");c$.isEmbeddable=c(c$,"isEmbeddable",function(a){var b=a.lastIndexOf(".");0<=b&&(a=a.substring(b+1));a=a.toUpperCase();return JV.FileManager.isJmolType(a)||JU.PT.isOneOf(a,";JPG;JPEG;POV;IDTF;")},"~S");c(c$,"getEmbeddedFileState",function(a,b,c){if(!JV.FileManager.isEmbeddable(a))return"";b=this.getZipDirectory(a,!1,b);if(0==b.length)return a=this.vwr.getFileAsString4(a,-1,!1,!0,!1,"file"),0>a.indexOf("**** Jmol Embedded Script ****")?
"":JV.FileManager.getEmbeddedScript(a);for(var g=0;g<b.length;g++)if(0<=b[g].indexOf(c))return a=u(-1,[a+"|"+b[g],null]),this.getFileDataAsString(a,-1,!1,!1,!1),a[1];return""},"~S,~B,~S");c$.stripTypePrefix=c(c$,"stripTypePrefix",function(a){var b=a.indexOf("::");return 0>b||20<=b?a:a.substring(b+2)},"~S");c$.getEmbeddedScript=c(c$,"getEmbeddedScript",function(a){if(null==a)return a;var b=a.indexOf("**** Jmol Embedded Script ****");if(0>b)return a;var c=a.lastIndexOf("/*",b),g=a.indexOf(("*"==a.charAt(c+
2)?"*":"")+"*/",b);for(0<=c&&g>=b&&(a=a.substring(b+30,g)+"\n");0<=(c=a.indexOf(" #Jmol...\x00"));)a=a.substring(0,c)+a.substring(c+10+4);JU.Logger.debugging&&JU.Logger.debug(a);return a},"~S");E(c$,"SIMULATION_PROTOCOL","http://SIMULATION/","DELPHI_BINARY_MAGIC_NUMBER","\u0014\x00\x00\x00","PMESH_BINARY_MAGIC_NUMBER","PM\u0001\x00","JPEG_CONTINUE_STRING"," #Jmol...\x00");c$.scriptFilePrefixes=c$.prototype.scriptFilePrefixes=u(-1,['/*file*/"','FILE0="','FILE1="'])});s("JV");v(["java.util.Hashtable",
"JU.P3","J.c.CBK"],"JV.GlobalSettings","java.lang.Boolean $.Float JU.DF $.PT $.SB J.c.STR JS.SV JU.Escape $.Logger JV.JC $.StateManager $.Viewer".split(" "),function(){c$=t(function(){this.htUserVariables=this.htPropertyFlagsRemoved=this.htBooleanParameterFlags=this.htNonbooleanParameterValues=this.vwr=null;this.zDepth=0;this.zShadePower=3;this.zSlab=50;this.slabByAtom=this.slabByMolecule=!1;this.appendNew=this.allowEmbeddedScripts=!0;this.appletProxy="";this.applySymmetryToBonds=!1;this.atomTypes=
"";this.autoBond=!0;this.axesOrientationRasmol=!1;this.bondRadiusMilliAngstroms=150;this.bondTolerance=0.45;this.defaultDirectory="";this.defaultStructureDSSP=!0;this.ptDefaultLattice=null;this.defaultLoadFilter=this.defaultLoadScript="";this.defaultDropScript="zap; load SYNC \"%FILE\";if (%ALLOWCARTOONS && _loadScript == '' && defaultLoadScript == '' && _filetype == 'Pdb') {if ({(protein or nucleic)&*/1.1} && {*/1.1}[1].groupindex != {*/1.1}[0].groupindex){select protein or nucleic;cartoons only;}if ({visible && cartoons > 0}){color structure}else{wireframe -0.1};if (!{visible}){spacefill 23%};select *}";
this.forceAutoBond=!1;this.fractionalRelative=!0;this.inlineNewlineChar="|";this.resolverResolver=this.macroDirectory=this.pubChemFormat=this.nihResolverFormat=this.smilesUrlFormat=this.nmrPredictFormat=this.nmrUrlFormat=this.pdbLoadLigandFormat=this.pdbLoadFormat=this.loadFormat=null;this.checkCIR=!1;this.minBondDistance=0.4;this.minPixelSelRadius=6;this.pdbSequential=this.pdbGetHeader=this.pdbAddHydrogens=!1;this.percentVdwAtom=23;this.smallMoleculeMaxAtoms=4E4;this.minimizationMaxAtoms=200;this.smartAromatic=
!0;this.legacyJavaFloat=this.legacyHAddition=this.legacyAutoBonding=this.zeroBasedXyzRasmol=!1;this.modulateOccupancy=this.jmolInJSpecView=!0;this.dotSolvent=this.allowMoveAtoms=this.allowRotateSelected=!1;this.defaultTorsionLabel=this.defaultDistanceLabel=this.defaultAngleLabel="%VALUE %UNITS";this.measureAllModels=this.justifyMeasurements=!1;this.minimizationSteps=100;this.minimizationRefresh=!0;this.minimizationSilent=!1;this.minimizationCriterion=0.001;this.infoFontSize=20;this.antialiasDisplay=
!1;this.displayCellParameters=this.antialiasTranslucent=this.imageState=this.antialiasImages=!0;this.dotsSelectedOnly=!1;this.dotSurface=!0;this.dotDensity=3;this.meshScale=this.dotScale=1;this.isosurfaceKey=this.greyscaleRendering=!1;this.isosurfacePropertySmoothing=!0;this.isosurfacePropertySmoothingPower=7;this.platformSpeed=10;this.repaintWaitMs=1E3;this.showHiddenSelectionHalos=!1;this.showMeasurements=this.showKeyStrokes=!0;this.showTiming=!1;this.zoomLarge=!0;this.zoomHeight=!1;this.backgroundImageFileName=
null;this.hbondsBackbone=this.bondModeOr=this.partialDots=!1;this.hbondsAngleMinimum=90;this.hbondNODistanceMaximum=3.25;this.hbondHXDistanceMaximum=2.5;this.hbondsRasmol=!0;this.hbondsSolid=!1;this.modeMultipleBond=2;this.showMultipleBonds=this.showHydrogens=!0;this.ssbondsBackbone=!1;this.multipleBondSpacing=-1;this.multipleBondRadiusFactor=0;this.multipleBondBananas=!1;this.nboCharges=!0;this.cartoonRockets=this.cartoonBaseEdges=!1;this.cartoonBlockHeight=0.5;this.cipRule6Full=this.chainCaseSensitive=
this.cartoonRibose=this.cartoonLadders=this.cartoonFancy=this.cartoonSteps=this.cartoonBlocks=!1;this.hermiteLevel=0;this.rangeSelected=this.highResolutionFlag=!1;this.rasmolHeteroSetting=this.rasmolHydrogenSetting=!0;this.ribbonAspectRatio=16;this.rocketBarrels=this.ribbonBorder=!1;this.sheetSmoothing=1;this.translucent=this.traceAlpha=!0;this.twistedSheets=!1;this.allowAudio=this.autoplayMovie=!0;this.allowGestures=!1;this.allowMultiTouch=this.allowModelkit=!0;this.hiddenLinesDashed=this.allowKeyStrokes=
!1;this.animationFps=10;this.atomPicking=!0;this.autoFps=!1;this.axesMode=603979809;this.axesScale=2;this.axesOffset=0;this.starWidth=0.05;this.bondPicking=!1;this.dataSeparator="~~~";this.debugScript=!1;this.defaultDrawArrowScale=0.5;this.defaultLabelXYZ="%a";this.defaultLabelPDB="%m%r";this.defaultTranslucent=0.5;this.delayMaximumMs=0;this.dipoleScale=1;this.drawFontSize=14;this.drawPicking=this.drawHover=this.dragSelected=this.disablePopupMenu=!1;this.dsspCalcHydrogen=!0;this.energyUnits="kJ";
this.exportScale=0;this.helpPath="https://chemapps.stolaf.edu/jmol/docs/index.htm";this.fontScaling=!1;this.fontCaching=!0;this.forceField="MMFF";this.helixStep=1;this.hideNameInPopup=!1;this.hoverDelayMs=500;this.loadAtomDataTolerance=0.01;this.logGestures=this.logCommands=!1;this.measureDistanceUnits="nanometers";this.measurementLabels=!0;this.monitorEnergy=this.messageStyleChime=!1;this.modulationScale=1;this.multiProcessor=!0;this.particleRadius=20;this.pickingSpinRate=10;this.pickLabel="";this.pointGroupDistanceTolerance=
0.2;this.pointGroupLinearTolerance=8;this.preserveState=!0;this.propertyColorScheme="roygb";this.quaternionFrame="p";this.saveProteinStructureState=!0;this.showModVecs=!1;this.showUnitCellDetails=!0;this.solventProbeRadius=1.2;this.scriptDelay=0;this.statusReporting=this.selectAllModels=!0;this.strandCountForStrands=5;this.strandCountForMeshRibbon=7;this.strutSpacing=6;this.strutLengthMaximum=7;this.strutDefaultRadius=0.3;this.strutsMultiple=!1;this.waitForMoveTo=this.useScriptQueue=this.useNumberLocalization=
this.useMinimizationThread=!0;this.noDelay=!1;this.vectorScale=1;this.vectorsCentered=this.vectorSymmetry=!1;this.vectorTrail=0;this.vibrationScale=this.vibrationPeriod=1;this.navigationPeriodic=this.navigationMode=this.hideNavigationPoint=this.wireframeRotation=!1;this.navigationSpeed=5;this.showNavigationPointAlways=!1;this.stereoState=null;this.modelKitMode=!1;this.objMad10=this.objStateOn=this.objColors=null;this.ellipsoidFill=this.ellipsoidArrows=this.ellipsoidArcs=this.ellipsoidDots=this.ellipsoidAxes=
!1;this.ellipsoidBall=!0;this.ellipsoidDotCount=200;this.ellipsoidAxisDiameter=0.02;this.testFlag4=this.testFlag3=this.testFlag2=this.testFlag1=!1;this.structureList=null;this.haveSetStructureList=!1;this.bondingVersion=0;p(this,arguments)},JV,"GlobalSettings");O(c$,function(){this.htUserVariables=new java.util.Hashtable;this.ptDefaultLattice=new JU.P3;this.objColors=B(7,0);this.objStateOn=ga(7,!1);this.objMad10=B(7,0);this.structureList=new java.util.Hashtable;this.structureList.put(J.c.STR.TURN,
K(-1,[30,90,-15,95]));this.structureList.put(J.c.STR.SHEET,K(-1,[-180,-10,70,180,-180,-45,-180,-130,140,180,90,180]));this.structureList.put(J.c.STR.HELIX,K(-1,[-160,0,-100,45]))});q(c$,function(a,b,c){this.vwr=a;this.htNonbooleanParameterValues=new java.util.Hashtable;this.htBooleanParameterFlags=new java.util.Hashtable;this.htPropertyFlagsRemoved=new java.util.Hashtable;this.loadFormat=this.pdbLoadFormat=JV.JC.databases.get("pdb");this.pdbLoadLigandFormat=JV.JC.databases.get("ligand");this.nmrUrlFormat=
JV.JC.databases.get("nmr");this.nmrPredictFormat=JV.JC.databases.get("nmrdb");this.pubChemFormat=JV.JC.databases.get("pubchem");this.resolverResolver=JV.JC.databases.get("resolverresolver");this.macroDirectory="https://chemapps.stolaf.edu/jmol/macros";null!=b&&(c||(this.setO("_pngjFile",b.getParameter("_pngjFile",!1)),this.htUserVariables=b.htUserVariables),this.debugScript=b.debugScript,this.disablePopupMenu=b.disablePopupMenu,this.messageStyleChime=b.messageStyleChime,this.defaultDirectory=b.defaultDirectory,
this.autoplayMovie=b.autoplayMovie,this.allowAudio=b.allowAudio,this.allowGestures=b.allowGestures,this.allowModelkit=b.allowModelkit,this.allowMultiTouch=b.allowMultiTouch,this.allowKeyStrokes=b.allowKeyStrokes,this.legacyAutoBonding=b.legacyAutoBonding,this.legacyHAddition=b.legacyHAddition,this.legacyJavaFloat=b.legacyJavaFloat,this.bondingVersion=b.bondingVersion,this.platformSpeed=b.platformSpeed,this.useScriptQueue=b.useScriptQueue,this.showTiming=b.showTiming,this.wireframeRotation=b.wireframeRotation,
this.testFlag1=b.testFlag1,this.testFlag2=b.testFlag2,this.testFlag3=b.testFlag3,this.testFlag4=b.testFlag4,this.nihResolverFormat=b.nihResolverFormat);null==this.nihResolverFormat&&(this.nihResolverFormat=JV.JC.databases.get("nci"));this.setCIR(this.nihResolverFormat);var g;c=0;for(var f=J.c.CBK.values();c<f.length&&((g=f[c])||1);c++)this.resetValue(g.name()+"Callback",b);this.setF("cameraDepth",3);this.setI("contextDepthMax",100);this.setI("depth",0);this.setF("gestureSwipeFactor",1);this.setB("hideNotSelected",
!1);this.setI("historyLevel",0);this.setO("hoverLabel","");this.setB("isKiosk",a.isKiosk());this.setO("logFile",a.getLogFileName());this.setI("logLevel",JU.Logger.getLogLevel());this.setF("mouseWheelFactor",1.15);this.setF("mouseDragFactor",1);this.setI("navFps",10);this.setI("navigationDepth",0);this.setI("navigationSlab",0);this.setI("navX",0);this.setI("navY",0);this.setI("navZ",0);this.setO("pathForAllFiles","");this.setB("perspectiveDepth",!0);this.setI("perspectiveModel",11);this.setO("picking",
"identify");this.setO("pickingStyle","toggle");this.setB("refreshing",!0);this.setI("rotationRadius",0);this.setI("scaleAngstromsPerInch",0);this.setI("scriptReportingLevel",0);this.setB("selectionHalos",!1);this.setB("showaxes",!1);this.setB("showboundbox",!1);this.setB("showfrank",!1);this.setB("showUnitcell",!1);this.setI("slab",100);this.setB("slabEnabled",!1);this.setF("slabrange",0);this.setI("spinX",0);this.setI("spinY",30);this.setI("spinZ",0);this.setI("spinFps",30);this.setF("visualRange",
5);this.setI("stereoDegrees",-5);this.setB("syncScript",a.sm.syncingScripts);this.setB("syncMouse",a.sm.syncingMouse);this.setB("syncStereo",a.sm.stereoSync);this.setB("windowCentered",!0);this.setB("zoomEnabled",!0);this.setI("_version",JV.JC.versionInt);this.setO("_versionDate",JV.Viewer.getJmolVersion());this.setB("axesWindow",!0);this.setB("axesMolecular",!1);this.setB("axesPosition",!1);this.setB("axesUnitcell",!1);this.setI("backgroundModel",0);this.setB("colorRasmol",!1);this.setO("currentLocalPath",
"");this.setO("defaultLattice","{0 0 0}");this.setO("defaultColorScheme","Jmol");this.setO("defaultDirectoryLocal","");this.setO("defaults","Jmol");this.setO("defaultVDW","Jmol");this.setO("exportDrivers","Idtf;Maya;Povray;Vrml;X3d;Stl;Tachyon;Obj");this.setI("propertyAtomNumberColumnCount",0);this.setI("propertyAtomNumberField",0);this.setI("propertyDataColumnCount",0);this.setI("propertyDataField",0);this.setB("undo",!0);this.setB("allowEmbeddedScripts",this.allowEmbeddedScripts);this.setB("allowGestures",
this.allowGestures);this.setB("allowKeyStrokes",this.allowKeyStrokes);this.setB("allowModelkit",this.allowModelkit);this.setB("allowMultiTouch",this.allowMultiTouch);this.setB("allowRotateSelected",this.allowRotateSelected);this.setB("allowMoveAtoms",this.allowMoveAtoms);this.setI("animationFps",this.animationFps);this.setB("antialiasImages",this.antialiasImages);this.setB("antialiasDisplay",this.antialiasDisplay);this.setB("antialiasTranslucent",this.antialiasTranslucent);this.setB("appendNew",this.appendNew);
this.setO("appletProxy",this.appletProxy);this.setB("applySymmetryToBonds",this.applySymmetryToBonds);this.setB("atomPicking",this.atomPicking);this.setO("atomTypes",this.atomTypes);this.setB("autoBond",this.autoBond);this.setB("autoFps",this.autoFps);this.setI("axesMode",603979808==this.axesMode?2:603979804==this.axesMode?1:0);this.setF("axesScale",this.axesScale);this.setF("axesOffset",this.axesOffset);this.setB("axesOrientationRasmol",this.axesOrientationRasmol);this.setF("cartoonBlockHeight",
this.cartoonBlockHeight);this.setB("cartoonBlocks",this.cartoonBlocks);this.setB("cartoonSteps",this.cartoonSteps);this.setB("bondModeOr",this.bondModeOr);this.setB("bondPicking",this.bondPicking);this.setI("bondRadiusMilliAngstroms",this.bondRadiusMilliAngstroms);this.setF("bondTolerance",this.bondTolerance);this.setB("cartoonBaseEdges",this.cartoonBaseEdges);this.setB("cartoonFancy",this.cartoonFancy);this.setB("cartoonLadders",this.cartoonLadders);this.setB("cartoonLadders",this.cartoonRibose);
this.setB("cartoonRockets",this.cartoonRockets);this.setB("chainCaseSensitive",this.chainCaseSensitive);this.setB("cipRule6Full",this.cipRule6Full);this.setI("bondingVersion",this.bondingVersion);this.setO("dataSeparator",this.dataSeparator);this.setB("debugScript",this.debugScript);this.setO("defaultAngleLabel",this.defaultAngleLabel);this.setF("defaultDrawArrowScale",this.defaultDrawArrowScale);this.setO("defaultDirectory",this.defaultDirectory);this.setO("defaultDistanceLabel",this.defaultDistanceLabel);
this.setO("defaultDropScript",this.defaultDropScript);this.setO("defaultLabelPDB",this.defaultLabelPDB);this.setO("defaultLabelXYZ",this.defaultLabelXYZ);this.setO("defaultLoadFilter",this.defaultLoadFilter);this.setO("defaultLoadScript",this.defaultLoadScript);this.setB("defaultStructureDSSP",this.defaultStructureDSSP);this.setO("defaultTorsionLabel",this.defaultTorsionLabel);this.setF("defaultTranslucent",this.defaultTranslucent);this.setI("delayMaximumMs",this.delayMaximumMs);this.setF("dipoleScale",
this.dipoleScale);this.setB("disablePopupMenu",this.disablePopupMenu);this.setB("displayCellParameters",this.displayCellParameters);this.setI("dotDensity",this.dotDensity);this.setI("dotScale",this.dotScale);this.setB("dotsSelectedOnly",this.dotsSelectedOnly);this.setB("dotSurface",this.dotSurface);this.setB("dragSelected",this.dragSelected);this.setB("drawHover",this.drawHover);this.setF("drawFontSize",this.drawFontSize);this.setB("drawPicking",this.drawPicking);this.setB("dsspCalculateHydrogenAlways",
this.dsspCalcHydrogen);this.setB("ellipsoidArcs",this.ellipsoidArcs);this.setB("ellipsoidArrows",this.ellipsoidArrows);this.setB("ellipsoidAxes",this.ellipsoidAxes);this.setF("ellipsoidAxisDiameter",this.ellipsoidAxisDiameter);this.setB("ellipsoidBall",this.ellipsoidBall);this.setI("ellipsoidDotCount",this.ellipsoidDotCount);this.setB("ellipsoidDots",this.ellipsoidDots);this.setB("ellipsoidFill",this.ellipsoidFill);this.setO("energyUnits",this.energyUnits);this.setF("exportScale",this.exportScale);
this.setB("fontScaling",this.fontScaling);this.setB("fontCaching",this.fontCaching);this.setB("forceAutoBond",this.forceAutoBond);this.setO("forceField",this.forceField);this.setB("fractionalRelative",this.fractionalRelative);this.setF("particleRadius",this.particleRadius);this.setB("greyscaleRendering",this.greyscaleRendering);this.setF("hbondsAngleMinimum",this.hbondsAngleMinimum);this.setF("hbondHXDistanceMaximum",this.hbondHXDistanceMaximum);this.setF("hbondsDistanceMaximum",this.hbondNODistanceMaximum);
this.setB("hbondsBackbone",this.hbondsBackbone);this.setB("hbondsRasmol",this.hbondsRasmol);this.setB("hbondsSolid",this.hbondsSolid);this.setI("helixStep",this.helixStep);this.setO("helpPath",this.helpPath);this.setI("hermiteLevel",this.hermiteLevel);this.setB("hideNameInPopup",this.hideNameInPopup);this.setB("hideNavigationPoint",this.hideNavigationPoint);this.setB("hiddenLinesDashed",this.hiddenLinesDashed);this.setB("highResolution",this.highResolutionFlag);this.setF("hoverDelay",this.hoverDelayMs/
1E3);this.setB("imageState",this.imageState);this.setI("infoFontSize",this.infoFontSize);this.setB("isosurfaceKey",this.isosurfaceKey);this.setB("isosurfacePropertySmoothing",this.isosurfacePropertySmoothing);this.setI("isosurfacePropertySmoothingPower",this.isosurfacePropertySmoothingPower);this.setB("jmolInJSpecView",this.jmolInJSpecView);this.setB("justifyMeasurements",this.justifyMeasurements);this.setB("legacyAutoBonding",this.legacyAutoBonding);this.setB("legacyHAddition",this.legacyHAddition);
this.setB("legacyJavaFloat",this.legacyJavaFloat);this.setF("loadAtomDataTolerance",this.loadAtomDataTolerance);this.setO("loadFormat",this.loadFormat);this.setO("loadLigandFormat",this.pdbLoadLigandFormat);this.setB("logCommands",this.logCommands);this.setB("logGestures",this.logGestures);this.setB("measureAllModels",this.measureAllModels);this.setB("measurementLabels",this.measurementLabels);this.setO("measurementUnits",this.measureDistanceUnits);this.setI("meshScale",this.meshScale);this.setB("messageStyleChime",
this.messageStyleChime);this.setF("minBondDistance",this.minBondDistance);this.setI("minPixelSelRadius",this.minPixelSelRadius);this.setI("minimizationSteps",this.minimizationSteps);this.setB("minimizationRefresh",this.minimizationRefresh);this.setB("minimizationSilent",this.minimizationSilent);this.setF("minimizationCriterion",this.minimizationCriterion);this.setB("modelKitMode",this.modelKitMode);this.setF("modulationScale",this.modulationScale);this.setB("monitorEnergy",this.monitorEnergy);this.setF("multipleBondRadiusFactor",
this.multipleBondRadiusFactor);this.setB("multipleBondBananas",this.multipleBondBananas);this.setF("multipleBondSpacing",this.multipleBondSpacing);this.setB("multiProcessor",this.multiProcessor&&1<JV.Viewer.nProcessors);this.setB("navigationMode",this.navigationMode);this.setB("navigationPeriodic",this.navigationPeriodic);this.setF("navigationSpeed",this.navigationSpeed);this.setB("nboCharges",this.nboCharges);this.setB("noDelay",this.noDelay);this.setO("nmrPredictFormat",this.nmrPredictFormat);this.setO("nmrUrlFormat",
this.nmrUrlFormat);this.setB("partialDots",this.partialDots);this.setB("pdbAddHydrogens",this.pdbAddHydrogens);this.setB("pdbGetHeader",this.pdbGetHeader);this.setB("pdbSequential",this.pdbSequential);this.setI("percentVdwAtom",this.percentVdwAtom);this.setI("pickingSpinRate",this.pickingSpinRate);this.setO("pickLabel",this.pickLabel);this.setI("platformSpeed",this.platformSpeed);this.setF("pointGroupLinearTolerance",this.pointGroupLinearTolerance);this.setF("pointGroupDistanceTolerance",this.pointGroupDistanceTolerance);
this.setB("preserveState",this.preserveState);this.setO("propertyColorScheme",this.propertyColorScheme);this.setO("quaternionFrame",this.quaternionFrame);this.setB("rangeSelected",this.rangeSelected);this.setI("repaintWaitMs",this.repaintWaitMs);this.setI("ribbonAspectRatio",this.ribbonAspectRatio);this.setB("ribbonBorder",this.ribbonBorder);this.setB("rocketBarrels",this.rocketBarrels);this.setB("saveProteinStructureState",this.saveProteinStructureState);this.setB("scriptqueue",this.useScriptQueue);
this.setB("selectAllModels",this.selectAllModels);this.setB("selectHetero",this.rasmolHeteroSetting);this.setB("selectHydrogen",this.rasmolHydrogenSetting);this.setF("sheetSmoothing",this.sheetSmoothing);this.setB("showHiddenSelectionHalos",this.showHiddenSelectionHalos);this.setB("showHydrogens",this.showHydrogens);this.setB("showKeyStrokes",this.showKeyStrokes);this.setB("showMeasurements",this.showMeasurements);this.setB("showModulationVectors",this.showModVecs);this.setB("showMultipleBonds",this.showMultipleBonds);
this.setB("showNavigationPointAlways",this.showNavigationPointAlways);this.setI("showScript",this.scriptDelay);this.setB("showtiming",this.showTiming);this.setB("slabByMolecule",this.slabByMolecule);this.setB("slabByAtom",this.slabByAtom);this.setB("smartAromatic",this.smartAromatic);this.setI("minimizationMaxAtoms",this.minimizationMaxAtoms);this.setI("smallMoleculeMaxAtoms",this.smallMoleculeMaxAtoms);this.setO("smilesUrlFormat",this.smilesUrlFormat);this.setO("macroDirectory",this.macroDirectory);
this.setO("nihResolverFormat",this.nihResolverFormat);this.setO("pubChemFormat",this.pubChemFormat);this.setB("showUnitCellDetails",this.showUnitCellDetails);this.setB("solventProbe",this.dotSolvent);this.setF("solventProbeRadius",this.solventProbeRadius);this.setB("ssbondsBackbone",this.ssbondsBackbone);this.setF("starWidth",this.starWidth);this.setB("statusReporting",this.statusReporting);this.setI("strandCount",this.strandCountForStrands);this.setI("strandCountForStrands",this.strandCountForStrands);
this.setI("strandCountForMeshRibbon",this.strandCountForMeshRibbon);this.setF("strutDefaultRadius",this.strutDefaultRadius);this.setF("strutLengthMaximum",this.strutLengthMaximum);this.setI("strutSpacing",this.strutSpacing);this.setB("strutsMultiple",this.strutsMultiple);this.setB("testFlag1",this.testFlag1);this.setB("testFlag2",this.testFlag2);this.setB("testFlag3",this.testFlag3);this.setB("testFlag4",this.testFlag4);this.setB("traceAlpha",this.traceAlpha);this.setB("translucent",this.translucent);
this.setB("twistedSheets",this.twistedSheets);this.setB("useMinimizationThread",this.useMinimizationThread);this.setB("useNumberLocalization",this.useNumberLocalization);this.setB("vectorsCentered",this.vectorsCentered);this.setF("vectorScale",this.vectorScale);this.setB("vectorSymmetry",this.vectorSymmetry);this.setI("vectorTrail",this.vectorTrail);this.setF("vibrationPeriod",this.vibrationPeriod);this.setF("vibrationScale",this.vibrationScale);this.setB("waitForMoveTo",this.waitForMoveTo);this.setB("wireframeRotation",
this.wireframeRotation);this.setI("zDepth",this.zDepth);this.setB("zeroBasedXyzRasmol",this.zeroBasedXyzRasmol);this.setB("zoomHeight",this.zoomHeight);this.setB("zoomLarge",this.zoomLarge);this.setI("zShadePower",this.zShadePower);this.setI("zSlab",this.zSlab)},"JV.Viewer,JV.GlobalSettings,~B");c(c$,"clear",function(){for(var a=this.htUserVariables.keySet().iterator();a.hasNext();){var b=a.next();("@"==b.charAt(0)||b.startsWith("site_"))&&a.remove()}this.vwr.setPicked(-1,!1);this.setI("_atomhovered",
-1);this.setO("_pickinfo","");this.setB("selectionhalos",!1);this.setB("hidenotselected",!1);this.setB("measurementlabels",this.measurementLabels=!0);this.setB("drawHover",this.drawHover=!1);this.vwr.stm.saveScene("DELETE",null)});c(c$,"setUnits",function(a){var b=this.measureDistanceUnits,c=this.energyUnits;a.equalsIgnoreCase("angstroms")?this.measureDistanceUnits="angstroms":a.equalsIgnoreCase("nanometers")||a.equalsIgnoreCase("nm")?this.measureDistanceUnits="nanometers":a.equalsIgnoreCase("picometers")||
a.equalsIgnoreCase("pm")?this.measureDistanceUnits="picometers":a.equalsIgnoreCase("bohr")||a.equalsIgnoreCase("au")?this.measureDistanceUnits="au":a.equalsIgnoreCase("vanderwaals")||a.equalsIgnoreCase("vdw")?this.measureDistanceUnits="vdw":a.toLowerCase().endsWith("hz")||a.toLowerCase().endsWith("khz")?this.measureDistanceUnits=a.toLowerCase():a.equalsIgnoreCase("kj")?this.energyUnits="kJ":a.equalsIgnoreCase("kcal")&&(this.energyUnits="kcal");b.equalsIgnoreCase(this.measureDistanceUnits)?c.equalsIgnoreCase(this.energyUnits)||
this.setO("energyUnits",this.energyUnits):this.setO("measurementUnits",this.measureDistanceUnits)},"~S");c(c$,"isJmolVariable",function(a){return"_"==a.charAt(0)||this.htNonbooleanParameterValues.containsKey(a=a.toLowerCase())||this.htBooleanParameterFlags.containsKey(a)||0<=JV.GlobalSettings.unreportedProperties.indexOf(";"+a+";")},"~S");c(c$,"resetValue",function(a,b){this.setO(a,null==b?"":b.getParameter(a,!0))},"~S,JV.GlobalSettings");c(c$,"setB",function(a,b){a=a.toLowerCase();this.htNonbooleanParameterValues.containsKey(a)||
this.htBooleanParameterFlags.put(a,b?Boolean.TRUE:Boolean.FALSE)},"~S,~B");c(c$,"setI",function(a,b){2147483647!=b&&this.setO(a,Integer.$valueOf(b))},"~S,~N");c(c$,"setF",function(a,b){Float.isNaN(b)||this.setO(a,Float.$valueOf(b))},"~S,~N");c(c$,"setO",function(a,b){a=a.toLowerCase();null==b||this.htBooleanParameterFlags.containsKey(a)||this.htNonbooleanParameterValues.put(a,b)},"~S,~O");c(c$,"removeParam",function(a){a=a.toLowerCase();this.htBooleanParameterFlags.containsKey(a)?(this.htBooleanParameterFlags.remove(a),
this.htPropertyFlagsRemoved.containsKey(a)||this.htPropertyFlagsRemoved.put(a,Boolean.FALSE)):this.htNonbooleanParameterValues.containsKey(a)&&this.htNonbooleanParameterValues.remove(a)},"~S");c(c$,"setUserVariable",function(a,b){null!=b&&(a=a.toLowerCase(),this.htUserVariables.put(a,b.setName(a)));return b},"~S,JS.SV");c(c$,"unsetUserVariable",function(a){a.equals("all")||a.equals("variables")?(this.htUserVariables.clear(),JU.Logger.info("all user-defined variables deleted")):this.htUserVariables.containsKey(a)&&
(JU.Logger.info("variable "+a+" deleted"),this.htUserVariables.remove(a))},"~S");c(c$,"removeUserVariable",function(a){this.htUserVariables.remove(a)},"~S");c(c$,"getUserVariable",function(a){if(null==a)return null;a=a.toLowerCase();return this.htUserVariables.get(a)},"~S");c(c$,"getParameterEscaped",function(a,b){a=a.toLowerCase();if(this.htNonbooleanParameterValues.containsKey(a)){var c=this.htNonbooleanParameterValues.get(a);return JV.StateManager.varClip(a,JU.Escape.e(c),b)}return this.htBooleanParameterFlags.containsKey(a)?
this.htBooleanParameterFlags.get(a).toString():this.htUserVariables.containsKey(a)?this.htUserVariables.get(a).escape():this.htPropertyFlagsRemoved.containsKey(a)?"false":"<not defined>"},"~S,~N");c(c$,"getParameter",function(a,b){var c=this.getParam(a,!1);return null==c&&b?"":c},"~S,~B");c(c$,"getAndSetNewVariable",function(a,b){if(null==a||0==a.length)a="x";var c=this.getParam(a,!0);return null==c&&b&&"_"!=a.charAt(0)?this.setUserVariable(a,JS.SV.newV(4,"")):JS.SV.getVariable(c)},"~S,~B");c(c$,
"getParam",function(a,b){a=a.toLowerCase();if(a.equals("_memory")){var c=JU.DF.formatDecimal(0,1)+"/"+JU.DF.formatDecimal(0,1);this.htNonbooleanParameterValues.put("_memory",c)}return this.htNonbooleanParameterValues.containsKey(a)?this.htNonbooleanParameterValues.get(a):this.htBooleanParameterFlags.containsKey(a)?this.htBooleanParameterFlags.get(a):this.htPropertyFlagsRemoved.containsKey(a)?Boolean.FALSE:this.htUserVariables.containsKey(a)?(c=this.htUserVariables.get(a),b?c:JS.SV.oValue(c)):null},
"~S,~B");c(c$,"getVariableList",function(){return JV.StateManager.getVariableList(this.htUserVariables,0,!0,!1)});c(c$,"setStructureList",function(a,b){this.haveSetStructureList=!0;this.structureList.put(b,a)},"~A,J.c.STR");c(c$,"getStructureList",function(){return this.structureList});c$.doReportProperty=c(c$,"doReportProperty",function(a){return"_"!=a.charAt(0)&&0>JV.GlobalSettings.unreportedProperties.indexOf(";"+a+";")},"~S");c(c$,"getAllVariables",function(){var a=new java.util.Hashtable;a.putAll(this.htBooleanParameterFlags);
a.putAll(this.htNonbooleanParameterValues);a.putAll(this.htUserVariables);return a});c(c$,"getLoadState",function(a){var b=new JU.SB;this.app(b,"set allowEmbeddedScripts false");this.allowEmbeddedScripts&&this.setB("allowEmbeddedScripts",!0);this.app(b,"set appendNew "+this.appendNew);this.app(b,"set appletProxy "+JU.PT.esc(this.appletProxy));this.app(b,"set applySymmetryToBonds "+this.applySymmetryToBonds);0<this.atomTypes.length&&this.app(b,"set atomTypes "+JU.PT.esc(this.atomTypes));this.app(b,
"set autoBond "+this.autoBond);this.axesOrientationRasmol&&this.app(b,"set axesOrientationRasmol true");this.app(b,"set bondRadiusMilliAngstroms "+this.bondRadiusMilliAngstroms);this.app(b,"set bondTolerance "+this.bondTolerance);this.app(b,"set defaultLattice "+JU.Escape.eP(this.ptDefaultLattice));this.app(b,"set defaultLoadFilter "+JU.PT.esc(this.defaultLoadFilter));this.app(b,'set defaultLoadScript ""');0<this.defaultLoadScript.length&&this.setO("defaultLoadScript",this.defaultLoadScript);this.app(b,
"set defaultStructureDssp "+this.defaultStructureDSSP);var c=this.vwr.getDefaultVdwNameOrData(-2147483648,null,null);this.app(b,"set defaultVDW "+c);c.equals("User")&&this.app(b,this.vwr.getDefaultVdwNameOrData(2147483647,null,null));this.app(b,"set forceAutoBond "+this.forceAutoBond);this.app(b,"#set defaultDirectory "+JU.PT.esc(this.defaultDirectory));this.app(b,"#set loadFormat "+JU.PT.esc(this.loadFormat));this.app(b,"#set loadLigandFormat "+JU.PT.esc(this.pdbLoadLigandFormat));this.app(b,"#set smilesUrlFormat "+
JU.PT.esc(this.smilesUrlFormat));this.app(b,"#set nihResolverFormat "+JU.PT.esc(this.nihResolverFormat));this.app(b,"#set pubChemFormat "+JU.PT.esc(this.pubChemFormat));this.app(b,"set bondingVersion "+this.bondingVersion);this.app(b,"set legacyAutoBonding "+this.legacyAutoBonding);this.app(b,"set legacyAutoBonding "+this.legacyAutoBonding);this.app(b,"set legacyHAddition "+this.legacyHAddition);this.app(b,"set legacyJavaFloat "+this.legacyJavaFloat);this.app(b,"set minBondDistance "+this.minBondDistance);
this.app(b,"set minimizationCriterion  "+this.minimizationCriterion);this.app(b,"set minimizationSteps  "+this.minimizationSteps);this.app(b,"set multipleBondBananas false");this.app(b,"set pdbAddHydrogens "+(null!=a&&a.get("pdbNoHydrogens")!==Boolean.TRUE?this.pdbAddHydrogens:!1));this.app(b,"set pdbGetHeader "+this.pdbGetHeader);this.app(b,"set pdbSequential "+this.pdbSequential);this.app(b,"set percentVdwAtom "+this.percentVdwAtom);this.app(b,"set smallMoleculeMaxAtoms "+this.smallMoleculeMaxAtoms);
this.app(b,"set smartAromatic "+this.smartAromatic);this.zeroBasedXyzRasmol&&this.app(b,"set zeroBasedXyzRasmol true");return b.toString()},"java.util.Map");c(c$,"app",function(a,b){0!=b.length&&a.append("  ").append(b).append(";\n")},"JU.SB,~S");c(c$,"setCIR",function(a){if(!(null==a||a.equals(this.nihResolverFormat)&&null!=this.smilesUrlFormat)){var b=a.indexOf("/structure");0<b&&(this.nihResolverFormat=a.substring(0,b+10),this.smilesUrlFormat=this.nihResolverFormat+"/%FILE/file?format=sdf&get3d=true",
Zystem.out.println("CIR resolver set to "+this.nihResolverFormat))}},"~S");c$.unreportedProperties=c$.prototype.unreportedProperties=(";ambientpercent;animationfps;antialiasdisplay;antialiasimages;antialiastranslucent;appendnew;axescolor;axesposition;axesmolecular;axesorientationrasmol;axesunitcell;axeswindow;axis1color;axis2color;axis3color;backgroundcolor;backgroundmodel;bondsymmetryatoms;boundboxcolor;cameradepth;bondingversion;ciprule6full;contextdepthmax;debug;debugscript;defaultlatttice;defaults;defaultdropscript;diffusepercent;;exportdrivers;exportscale;_filecaching;_filecache;fontcaching;fontscaling;forcefield;language;hbondsDistanceMaximum;hbondsangleminimum;jmolinJSV;legacyautobonding;legacyhaddition;legacyjavafloat;loglevel;logfile;loggestures;logcommands;measurestylechime;loadformat;loadligandformat;macrodirectory;mkaddhydrogens;minimizationmaxatoms;smilesurlformat;pubchemformat;nihresolverformat;edsurlformat;edsurlcutoff;multiprocessor;navigationmode;;nodelay;pathforallfiles;perspectivedepth;phongexponent;perspectivemodel;platformspeed;preservestate;refreshing;repaintwaitms;rotationradius;selectallmodels;showaxes;showaxis1;showaxis2;showaxis3;showboundbox;showfrank;showtiming;showunitcell;slabenabled;slab;slabrange;depth;zshade;zshadepower;specular;specularexponent;specularpercent;celshading;celshadingpower;specularpower;stateversion;statusreporting;stereo;stereostate;vibrationperiod;unitcellcolor;visualrange;windowcentered;zerobasedxyzrasmol;zoomenabled;mousedragfactor;mousewheelfactor;scriptqueue;scriptreportinglevel;syncscript;syncmouse;syncstereo;defaultdirectory;currentlocalpath;defaultdirectorylocal;ambient;bonds;colorrasmol;diffuse;fractionalrelative;frank;hetero;hidenotselected;hoverlabel;hydrogen;languagetranslation;measurementunits;navigationdepth;navigationslab;picking;pickingstyle;propertycolorschemeoverload;radius;rgbblue;rgbgreen;rgbred;scaleangstromsperinch;selectionhalos;showscript;showselections;solvent;strandcount;spinx;spiny;spinz;spinfps;navx;navy;navz;navfps;"+
J.c.CBK.getNameList()+";undo;atompicking;drawpicking;bondpicking;pickspinrate;picklabel;modelkitmode;autoplaymovie;allowaudio;allowgestures;allowkeystrokes;allowmultitouch;allowmodelkit;dodrop;hovered;historylevel;imagestate;iskiosk;useminimizationthread;checkcir;resolverresolver;showkeystrokes;saveproteinstructurestate;testflag1;testflag2;testflag3;testflag4;selecthetero;selecthydrogen;;").toLowerCase()});s("JV");v(["java.util.Hashtable","JU.SB","$.V3","JU.Elements"],"JV.JC",["JU.PT","JU.Logger"],
function(){c$=D(JV,"JC");c$.getNBOTypeFromName=c(c$,"getNBOTypeFromName",function(a){a=";AO;;;;PNAO;;NAO;;;PNHO;;NHO;;;PNBO;;NBO;;;PNLMO;NLMO;;MO;;;;NO;;;;;;;;;;PRNBO;RNBO;;;;;;;;".indexOf(";"+a+";");return 0>a?a:w(a/6)+31},"~S");c$.getCIPChiralityName=c(c$,"getCIPChiralityName",function(a){switch(a){case 13:return"Z";case 5:return"z";case 14:return"E";case 6:return"e";case 17:return"M";case 18:return"P";case 1:return"R";case 2:return"S";case 9:return"r";case 10:return"s";case 25:return"m";case 26:return"p";
case 7:return"?";default:return""}},"~N");c$.getCIPRuleName=c(c$,"getCIPRuleName",function(a){return JV.JC.ruleNames[a]},"~N");c$.getCIPChiralityCode=c(c$,"getCIPChiralityCode",function(a){switch(a){case "Z":return 13;case "z":return 5;case "E":return 14;case "e":return 6;case "R":return 1;case "S":return 2;case "r":return 9;case "s":return 10;case "?":return 7;default:return 0}},"~S");c$.resolveDataBase=c(c$,"resolveDataBase",function(a,b,c){if(null==c){if(null==(c=JV.JC.databases.get(a.toLowerCase())))return null;
var d=b.indexOf("/");0>d&&(a.equals("pubchem")?b="name/"+b:a.equals("nci")&&(b+="/file?format=sdf&get3d=true"));c.startsWith("'")&&(d=b.indexOf("."),a=0<d?JU.PT.parseInt(b.substring(d+1)):0,0<d&&(b=b.substring(0,d)),c=JU.PT.rep(c,"%n",""+a))}else 0<=b.indexOf(".")&&0<=c.indexOf("%FILE.")&&(c=c.substring(0,c.indexOf("%FILE")));if(0<=c.indexOf("%c")){d=1;for(a=b.length;d<=a;d++)0<=c.indexOf("%c"+d)&&(c=JU.PT.rep(c,"%c"+d,b.substring(d-1,d).toLowerCase()))}return 0<=c.indexOf("%FILE")?JU.PT.rep(c,"%FILE",
b):0<=c.indexOf("%file")?JU.PT.rep(c,"%file",b.toLowerCase()):c+b},"~S,~S,~S");c$.fixProtocol=c(c$,"fixProtocol",function(a){var b=null==a?null:0==a.indexOf("http://www.rcsb.org/pdb/files/")?JV.JC.resolveDataBase(0<=a.indexOf("/ligand/")?"ligand":"pdb",a.substring(a.lastIndexOf("/")+1),null):0==a.indexOf("http://www.ebi")||0==a.indexOf("http://pubchem")||0==a.indexOf("http://cactus")||0==a.indexOf("http://www.materialsproject")?"https://"+a.substring(7):a;b!==a&&JU.Logger.info("JC.fixProtocol "+a+
" --\x3e "+b);return b},"~S");c$.embedScript=c(c$,"embedScript",function(a){return"\n/****** Jmol Embedded Script **** \n"+a+"\n**/"},"~S");c$.getShapeVisibilityFlag=c(c$,"getShapeVisibilityFlag",function(a){return 16<<Math.min(a,26)},"~N");c$.shapeTokenIndex=c(c$,"shapeTokenIndex",function(a){switch(a){case 1140850689:case 1073741859:return 0;case 1677721602:case 659488:return 1;case 1613238294:return 2;case 1611141176:return 3;case 659482:return 4;case 1825200146:return 5;case 1745489939:case 537006096:return 6;
case 1112152076:return 7;case 1112152070:return 8;case 1114249217:return 9;case 1112152078:return 10;case 1112152066:return 11;case 1649022989:return 12;case 1112152071:return 13;case 1112152073:return 14;case 1112152074:return 15;case 1112150019:return 16;case 135175:return 17;case 135198:return 18;case 1112150021:return 19;case 1112150020:return 20;case 1275203608:return 21;case 135174:return 23;case 135176:return 22;case 135180:return 24;case 134353926:return 25;case 135182:return 26;case 1073877010:return 27;
case 1073877011:return 28;case 135188:return 29;case 135190:return 30;case 537022465:return 31;case 1611272194:return 34;case 1678381065:return 32;case 1814695966:return 33;case 544771:return 35;case 1611272202:return 36}return-1},"~N");c$.getShapeClassName=c(c$,"getShapeClassName",function(a,b){return 0>a?JV.JC.shapeClassBases[~a]:"J."+(b?"render":"shape")+(9<=a&&16>a?"bio.":16<=a&&23>a?"special.":24<=a&&30>a?"surface.":23==a?"cgo.":".")+JV.JC.shapeClassBases[a]},"~N,~B");c$.getEchoName=c(c$,"getEchoName",
function(a){return JV.JC.echoNames[a]},"~N");c$.setZPosition=c(c$,"setZPosition",function(a,b){return a&-49|b},"~N,~N");c$.setPointer=c(c$,"setPointer",function(a,b){return a&-4|b},"~N,~N");c$.getPointer=c(c$,"getPointer",function(a){return a&3},"~N");c$.getPointerName=c(c$,"getPointerName",function(a){return 0==(a&1)?"":0<(a&2)?"background":"on"},"~N");c$.isOffsetAbsolute=c(c$,"isOffsetAbsolute",function(a){return 0!=(a&64)},"~N");c$.getOffset=c(c$,"getOffset",function(a,b,c){a=Math.min(Math.max(a,
-500),500);b=Math.min(Math.max(b,-500),500);var d=(a&1023)<<21|(b&1023)<<11|(c?64:0);if(d==JV.JC.LABEL_DEFAULT_OFFSET)d=0;else if(!c&&(0==a||0==b))d|=256;return d},"~N,~N,~B");c$.getXOffset=c(c$,"getXOffset",function(a){if(0==a)return 4;a=a>>21&1023;return 500<a?a-1023-1:a},"~N");c$.getYOffset=c(c$,"getYOffset",function(a){if(0==a)return 4;a=a>>11&1023;return 500<a?a-1023-1:a},"~N");c$.getAlignment=c(c$,"getAlignment",function(a){return a&12},"~N");c$.setHorizAlignment=c(c$,"setHorizAlignment",function(a,
b){return a&-13|b},"~N,~N");c$.getHorizAlignmentName=c(c$,"getHorizAlignmentName",function(a){return JV.JC.hAlignNames[a>>2&3]},"~N");c$.isSmilesCanonical=c(c$,"isSmilesCanonical",function(a){return null!=a&&JU.PT.isOneOf(a.toLowerCase(),";/cactvs///;/cactus///;/nci///;/canonical///;")},"~S");c$.getServiceCommand=c(c$,"getServiceCommand",function(a){return 7>a.length?-1:"JSPECVIPEAKS: SELECT:JSVSTR:H1SIMULC13SIMUNBO:MODNBO:RUNNBO:VIENBO:SEANBO:CONNONESIM".indexOf(a.substring(0,7).toUpperCase())},
"~S");c$.getUnitIDFlags=c(c$,"getUnitIDFlags",function(a){var b=14;0==a.indexOf("-")&&(0<a.indexOf("m")&&(b|=1),0>a.indexOf("a")&&(b^=4),0<a.indexOf("t")&&(b|=16));return b},"~S");E(c$,"NBO_TYPES",";AO;;;;PNAO;;NAO;;;PNHO;;NHO;;;PNBO;;NBO;;;PNLMO;NLMO;;MO;;;;NO;;;;;;;;;;PRNBO;RNBO;;;;;;;;","CIP_CHIRALITY_UNKNOWN",0,"CIP_CHIRALITY_R_FLAG",1,"CIP_CHIRALITY_S_FLAG",2,"CIP_CHIRALITY_CANTDETERMINE",7,"CIP_CHIRALITY_NONE",3,"CIP_CHIRALITY_EZ_FLAG",4,"CIP_CHIRALITY_PSEUDO_FLAG",8,"CIP_CHIRALITY_AXIAL_FLAG",
16,"CIP_CHIRALITY_NAME_MASK",224,"CIP_CHIRALITY_NAME_OFFSET",5,"CIP_CHIRALITY_seqCis_FLAG",5,"CIP_CHIRALITY_seqTrans_FLAG",6,"CIP_CHIRALITY_seqcis_FLAG",13,"CIP_CHIRALITY_seqtrans_FLAG",14,"CIP_CHIRALITY_M_FLAG",17,"CIP_CHIRALITY_P_FLAG",18,"CIP_CHIRALITY_r_FLAG",9,"CIP_CHIRALITY_s_FLAG",10,"CIP_CHIRALITY_m_FLAG",25,"CIP_CHIRALITY_p_FLAG",26,"ruleNames",u(-1," 1a 1b 2 3 4a 4b 4c 5 6".split(" ")),"PDB_ANNOTATIONS",";dssr;rna3d;dom;val;","CACTUS_FILE_TYPES",";alc;cdxml;cerius;charmm;cif;cml;ctx;gjf;gromacs;hyperchem;jme;maestro;mol;mol2;sybyl2;mrv;pdb;sdf;sdf3000;sln;smiles;xyz",
"defaultMacroDirectory","https://chemapps.stolaf.edu/jmol/macros","databaseArray",u(-1,"aflowbin;http://aflowlib.mems.duke.edu/users/jmolers/binary_new/%FILE.aflow_binary;aflow;http://aflowlib.mems.duke.edu/users/jmolers/binary_new/%FILE.aflow_binary;ams;'http://rruff.geo.arizona.edu/AMS/viewJmol.php?'+(0+'%file'==0? 'mineral':('%file'.length==7? 'amcsd':'id'))+'=%file&action=showcif#_DOCACHE_';dssr;http://dssr-jmol.x3dna.org/report.php?id=%FILE&opts=--json=ebi;dssrModel;http://dssr-jmol.x3dna.org/report.php?POST?opts=--json=ebi&model=;iucr;http://scripts.iucr.org/cgi-bin/sendcif_yard?%FILE;cod;http://www.crystallography.net/cod/cif/%c1/%c2%c3/%c4%c5/%FILE.cif;nmr;https://www.nmrdb.org/new_predictor?POST?molfile=;nmrdb;https://www.nmrdb.org/service/predictor?POST?molfile=;nmrdb13;https://www.nmrdb.org/service/jsmol13c?POST?molfile=;magndata;http://webbdcrista1.ehu.es/magndata/mcif/%FILE.mcif;rna3d;http://rna.bgsu.edu/rna3dhub/%TYPE/download/%FILE;mmtf;https://mmtf.rcsb.org/v1.0/full/%FILE;chebi;https://www.ebi.ac.uk/chebi/saveStructure.do?defaultImage=true&chebiId=%file%2D%;ligand;https://files.rcsb.org/ligands/download/%FILE.cif;mp;https://www.materialsproject.org/materials/mp-%FILE/cif#_DOCACHE_;nci;https://cactus.nci.nih.gov/chemical/structure;pdb;https://files.rcsb.org/download/%FILE.pdb;pdb0;https://files.rcsb.org/download/%FILE.pdb;pdbe;https://www.ebi.ac.uk/pdbe/entry-files/download/%FILE.cif;pdbe2;https://www.ebi.ac.uk/pdbe/static/entry/%FILE_updated.cif;pubchem;https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/%FILE/SDF?record_type=3d;map;https://www.ebi.ac.uk/pdbe/api/%TYPE/%FILE?pretty=false&metadata=true;pdbemap;https://www.ebi.ac.uk/pdbe/coordinates/files/%file.ccp4;pdbemapdiff;https://www.ebi.ac.uk/pdbe/coordinates/files/%file_diff.ccp4;pdbemapserver;https://www.ebi.ac.uk/pdbe/densities/x-ray/%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif;pdbemapdiffserver;https://www.ebi.ac.uk/pdbe/densities/x-ray/%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif&diff=1;emdbmap;https://www.ebi.ac.uk/pdbe/densities/emd/emd-%file/cell?detail=6&space=cartesian&encoding=bcif;emdbquery;https://www.ebi.ac.uk/emdb/api/search/fitted_pdbs:%file?fl=emdb_id,map_contour_level_value&wt=csv;emdbmapserver;https://www.ebi.ac.uk/pdbe/densities/emd/emd-%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif;resolverResolver;https://chemapps.stolaf.edu/resolver".split(";")),
"legacyResolver","cactus.nci.nih.gov/chemical/structure");c$.databases=c$.prototype.databases=new java.util.Hashtable;for(var a=0;a<JV.JC.databaseArray.length;a+=2)JV.JC.databases.put(JV.JC.databaseArray[a].toLowerCase(),JV.JC.databaseArray[a+1]);E(c$,"macros",u(-1,"aflow;https://chemapps.stolaf.edu/jmol/macros/AFLOW.spt;AFLOW macros;bz;https://chemapps.stolaf.edu/jmol/macros/bz.spt;Brillouin Zone/Wigner-Seitz macros;topology;https://chemapps.stolaf.edu/jmol/macros/topology.spt;Topology CIF macros;topond;https://chemapps.stolaf.edu/jmol/macros/topond.spt;CRYSTAL/TOPOND macros;crystal;https://chemapps.stolaf.edu/jmol/macros/crystal.spt;CRYSTAL macros".split(";")),
"copyright","(C) 2015 Jmol Development","version",null,"majorVersion",null,"date",null,"versionInt",0);var b=null,d=null,b=Jmol.___JmolVersion,d=Jmol.___JmolDate;null!=d&&(d=d.substring(7,23));JV.JC.version=null!=b?b:"(Unknown_version)";JV.JC.majorVersion=null!=b?b:"(Unknown_version)";JV.JC.date=null!=d?d:"(Unknown_date)";b=-1;try{var g=JV.JC.version,d="",a=g.indexOf(".");0>a&&(b=1E5*Integer.parseInt(g),g=null);if(null!=g&&(b=1E5*Integer.parseInt(d=g.substring(0,a)),g=g.substring(a+1),a=g.indexOf("."),
0>a&&(b+=1E3*Integer.parseInt(g),g=null),null!=g)){var f=g.substring(0,a);JV.JC.majorVersion=d+("."+f);b+=1E3*Integer.parseInt(f);g=g.substring(a+1);a=g.indexOf("_");0<=a&&(g=g.substring(0,a));a=g.indexOf(" ");0<=a&&(g=g.substring(0,a));b+=Integer.parseInt(g)}}catch(e){if(!H(e,NumberFormatException))throw e;}JV.JC.versionInt=b;E(c$,"officialRelease",!1,"DEFAULT_HELP_PATH","https://chemapps.stolaf.edu/jmol/docs/index.htm","STATE_VERSION_STAMP","# Jmol state version ","EMBEDDED_SCRIPT_TAG","**** Jmol Embedded Script ****",
"NOTE_SCRIPT_FILE","NOTE: file recognized as a script file: ","SCRIPT_EDITOR_IGNORE","\u0001## EDITOR_IGNORE ##","REPAINT_IGNORE","\u0001## REPAINT_IGNORE ##","LOAD_ATOM_DATA_TYPES",";xyz;vxyz;vibration;temperature;occupancy;partialcharge;","radiansPerDegree",0.017453292519943295,"allowedQuaternionFrames","RC;RP;a;b;c;n;p;q;x;","EXPORT_DRIVER_LIST","Idtf;Maya;Povray;Vrml;X3d;Stl;Tachyon;Obj");c$.center=c$.prototype.center=JU.V3.new3(0,0,0);c$.axisX=c$.prototype.axisX=JU.V3.new3(1,0,0);c$.axisY=c$.prototype.axisY=
JU.V3.new3(0,1,0);c$.axisZ=c$.prototype.axisZ=JU.V3.new3(0,0,1);c$.axisNX=c$.prototype.axisNX=JU.V3.new3(-1,0,0);c$.axisNY=c$.prototype.axisNY=JU.V3.new3(0,-1,0);c$.axisNZ=c$.prototype.axisNZ=JU.V3.new3(0,0,-1);c$.unitAxisVectors=c$.prototype.unitAxisVectors=u(-1,[JV.JC.axisX,JV.JC.axisY,JV.JC.axisZ,JV.JC.axisNX,JV.JC.axisNY,JV.JC.axisNZ]);E(c$,"XY_ZTOP",100,"DEFAULT_PERCENT_VDW_ATOM",23,"DEFAULT_BOND_RADIUS",0.15,"DEFAULT_BOND_MILLIANGSTROM_RADIUS",$(150),"DEFAULT_STRUT_RADIUS",0.3,"DEFAULT_BOND_TOLERANCE",
0.45,"DEFAULT_MIN_BOND_DISTANCE",0.4,"DEFAULT_MAX_CONNECT_DISTANCE",1E8,"DEFAULT_MIN_CONNECT_DISTANCE",0.1,"MINIMIZE_FIXED_RANGE",5,"ENC_CALC_MAX_DIST",3,"ENV_CALC_MAX_LEVEL",3,"MOUSE_NONE",-1,"MULTIBOND_NEVER",0,"MULTIBOND_WIREFRAME",1,"MULTIBOND_NOTSMALL",2,"MULTIBOND_ALWAYS",3,"MAXIMUM_AUTO_BOND_COUNT",20,"madMultipleBondSmallMaximum",500,"ANGSTROMS_PER_BOHR",0.5291772,"altArgbsCpk",B(-1,[4294907027,4290750118,4294967088,4283897743,4294967232,4294967200,4292401368,4283453520,4282400832,4279259216]),
"argbsFormalCharge",B(-1,[4294901760,4294918208,4294934656,4294951104,4294967295,4292401407,4290032895,4287664383,4285295871,4282927359,4280558847,4278190335]),"argbsRwbScale",B(-1,[4294901760,4294905872,4294909984,4294914096,4294918208,4294922320,4294926432,4294930544,4294934656,4294938768,4294942880,4294946992,4294951104,4294955216,4294959328,4294967295,4292927743,4291875071,4290822399,4289769727,4288717055,4287664383,4286611711,4285559039,4284506367,4283453695,4282401023,4281348351,4280295679,
4279243007,4278190335]));c$.FORMAL_CHARGE_COLIX_RED=c$.prototype.FORMAL_CHARGE_COLIX_RED=JU.Elements.elementSymbols.length+JV.JC.altArgbsCpk.length;c$.PARTIAL_CHARGE_COLIX_RED=c$.prototype.PARTIAL_CHARGE_COLIX_RED=JV.JC.FORMAL_CHARGE_COLIX_RED+JV.JC.argbsFormalCharge.length;c$.PARTIAL_CHARGE_RANGE_SIZE=c$.prototype.PARTIAL_CHARGE_RANGE_SIZE=JV.JC.argbsRwbScale.length;E(c$,"argbsRoygbScale",B(-1,[4294901760,4294909952,4294918144,4294926336,4294934528,4294942720,4294950912,4294959104,4294963200,4294967040,
4293980160,4292935424,4290838272,4288741120,4286643968,4284546816,4282449664,4280352512,4278255360,4278255392,4278255424,4278255456,4278255488,4278255520,4278255552,4278255584,4278255615,4278247679,4278239487,4278231295,4278223103,4278214911,4278206719,4278198527,4278190335]),"argbsIsosurfacePositive",4283441312,"argbsIsosurfaceNegative",4288684112,"ATOMID_AMINO_NITROGEN",1,"ATOMID_ALPHA_CARBON",2,"ATOMID_CARBONYL_CARBON",3,"ATOMID_CARBONYL_OXYGEN",4,"ATOMID_O1",5,"ATOMID_ALPHA_ONLY_MASK",4,"ATOMID_PROTEIN_MASK",
14,"ATOMID_O5_PRIME",6,"ATOMID_C5_PRIME",7,"ATOMID_C4_PRIME",8,"ATOMID_C3_PRIME",9,"ATOMID_O3_PRIME",10,"ATOMID_C2_PRIME",11,"ATOMID_C1_PRIME",12,"ATOMID_O4_PRIME",78,"ATOMID_NUCLEIC_MASK",8128,"ATOMID_NUCLEIC_PHOSPHORUS",13,"ATOMID_PHOSPHORUS_ONLY_MASK",8192,"ATOMID_DISTINGUISHING_ATOM_MAX",14,"ATOMID_CARBONYL_OD1",14,"ATOMID_CARBONYL_OD2",15,"ATOMID_CARBONYL_OE1",16,"ATOMID_CARBONYL_OE2",17,"ATOMID_N1",32,"ATOMID_C2",33,"ATOMID_N3",34,"ATOMID_C4",35,"ATOMID_C5",36,"ATOMID_C6",37,"ATOMID_O2",38,
"ATOMID_N7",39,"ATOMID_C8",40,"ATOMID_N9",41,"ATOMID_N4",42,"ATOMID_N2",43,"ATOMID_N6",44,"ATOMID_C5M",45,"ATOMID_O6",46,"ATOMID_O4",47,"ATOMID_S4",48,"ATOMID_C7",49,"ATOMID_TERMINATING_OXT",64,"ATOMID_H5T_TERMINUS",72,"ATOMID_O5T_TERMINUS",73,"ATOMID_O1P",74,"ATOMID_OP1",75,"ATOMID_O2P",76,"ATOMID_OP2",77,"ATOMID_O2_PRIME",79,"ATOMID_H3T_TERMINUS",88,"ATOMID_HO3_PRIME",89,"ATOMID_HO5_PRIME",90,"PURINE_MASK",153957,"PYRIMIDINE_MASK",108186,"DNA_MASK",65480,"RNA_MASK",196663,"GROUPID_ARGININE",2,"GROUPID_ASPARAGINE",
3,"GROUPID_ASPARTATE",4,"GROUPID_CYSTEINE",5,"GROUPID_GLUTAMINE",6,"GROUPID_GLUTAMATE",7,"GROUPID_HISTIDINE",9,"GROUPID_LYSINE",12,"GROUPID_PROLINE",15,"GROUPID_TRYPTOPHAN",19,"GROUPID_AMINO_MAX",24,"GROUPID_NUCLEIC_MAX",42,"GROUPID_WATER",42,"GROUPID_SOLVENT_MIN",45,"GROUPID_ION_MIN",46,"GROUPID_ION_MAX",48,"predefinedVariable",u(-1,"@_1H _H & !(_2H,_3H);@_12C _C & !(_13C,_14C);@_14N _N & !(_15N);@solvent water, (_g>=45 & _g<48);@ligand _g=0|!(_g<46,protein,nucleic,water);@turn structure=1;@sheet structure=2;@helix structure=3;@helix310 substructure=7;@helixalpha substructure=8;@helixpi substructure=9;@bulges within(dssr,'bulges');@coaxStacks within(dssr,'coaxStacks');@hairpins within(dssr,'hairpins');@hbonds within(dssr,'hbonds');@helices within(dssr,'helices');@iloops within(dssr,'iloops');@isoCanonPairs within(dssr,'isoCanonPairs');@junctions within(dssr,'junctions');@kissingLoops within(dssr,'kissingLoops');@multiplets within(dssr,'multiplets');@nonStack within(dssr,'nonStack');@nts within(dssr,'nts');@pairs within(dssr,'pairs');@ssSegments within(dssr,'ssSegments');@stacks within(dssr,'stacks');@stems within(dssr,'stems')".split(";")),
"predefinedStatic",u(-1,"@amino _g>0 & _g<=23;@acidic asp,glu;@basic arg,his,lys;@charged acidic,basic;@negative acidic;@positive basic;@neutral amino&!(acidic,basic);@polar amino&!hydrophobic;@peptide protein&within(chain,monomer>1)&!within(chain,monomer>12);@cyclic his,phe,pro,trp,tyr;@acyclic amino&!cyclic;@aliphatic ala,gly,ile,leu,val;@aromatic his,phe,trp,tyr;@cystine within(group,(cys,cyx)&atomname=sg&connected((cys,cyx)&atomname=sg));@buried ala,cys,ile,leu,met,phe,trp,val;@surface amino&!buried;@hydrophobic ala,gly,ile,leu,met,phe,pro,trp,tyr,val;@mainchain backbone;@small ala,gly,ser;@medium asn,asp,cys,pro,thr,val;@large arg,glu,gln,his,ile,leu,lys,met,phe,trp,tyr;@c nucleic & ([C] or [DC] or within(group,_a=42));@g nucleic & ([G] or [DG] or within(group,_a=43));@cg c,g;@a nucleic & ([A] or [DA] or within(group,_a=44));@t nucleic & ([T] or [DT] or within(group,_a=45 | _a=49));@at a,t;@i nucleic & ([I] or [DI] or within(group,_a=46) & !g);@u nucleic & ([U] or [DU] or within(group,_a=47) & !t);@tu nucleic & within(group,_a=48);@ions _g>=46&_g<48;@alpha _a=2;@_bb protein&(_a>=1&_a<6|_a=64) | nucleic&(_a>=6&_a<14|_a>=73&&_a<=79||_a==99||_a=100);@backbone _bb | _H && connected(single, _bb);@spine protein&_a>=1&_a<4|nucleic&(_a>=6&_a<11|_a=13);@sidechain (protein,nucleic) & !backbone;@base nucleic & !backbone;@dynamic_flatring search('[a]');@nonmetal _H,_He,_B,_C,_N,_O,_F,_Ne,_Si,_P,_S,_Cl,_Ar,_As,_Se,_Br,_Kr,_Te,_I,_Xe,_At,_Rn;@metal !nonmetal && !_Xx;@alkaliMetal _Li,_Na,_K,_Rb,_Cs,_Fr;@alkalineEarth _Be,_Mg,_Ca,_Sr,_Ba,_Ra;@nobleGas _He,_Ne,_Ar,_Kr,_Xe,_Rn;@metalloid _B,_Si,_Ge,_As,_Sb,_Te;@transitionMetal elemno>=21&elemno<=30|elemno=57|elemno=89|elemno>=39&elemno<=48|elemno>=72&elemno<=80|elemno>=104&elemno<=112;@lanthanide elemno>57&elemno<=71;@actinide elemno>89&elemno<=103".split(";")),
"MODELKIT_ZAP_STRING","5\n\nC 0 0 0\nH .63 .63 .63\nH -.63 -.63 .63\nH -.63 .63 -.63\nH .63 -.63 -.63","MODELKIT_ZAP_TITLE","Jmol Model Kit","ZAP_TITLE","zapped","ADD_HYDROGEN_TITLE","Viewer.AddHydrogens","DEFAULT_FONTFACE","SansSerif","DEFAULT_FONTSTYLE","Plain","MEASURE_DEFAULT_FONTSIZE",18,"AXES_DEFAULT_FONTSIZE",16,"SHAPE_BALLS",0,"SHAPE_STICKS",1,"SHAPE_HSTICKS",2,"SHAPE_SSSTICKS",3,"SHAPE_STRUTS",4,"SHAPE_LABELS",5,"SHAPE_MEASURES",6,"SHAPE_STARS",7,"SHAPE_MIN_HAS_SETVIS",8,"SHAPE_HALOS",8,
"SHAPE_MIN_SECONDARY",9,"SHAPE_BACKBONE",9,"SHAPE_TRACE",10,"SHAPE_CARTOON",11,"SHAPE_STRANDS",12,"SHAPE_MESHRIBBON",13,"SHAPE_RIBBONS",14,"SHAPE_ROCKETS",15,"SHAPE_MAX_SECONDARY",16,"SHAPE_MIN_SPECIAL",16,"SHAPE_DOTS",16,"SHAPE_DIPOLES",17,"SHAPE_VECTORS",18,"SHAPE_GEOSURFACE",19,"SHAPE_ELLIPSOIDS",20,"SHAPE_MAX_SIZE_ZERO_ON_RESTRICT",21,"SHAPE_MIN_HAS_ID",21,"SHAPE_POLYHEDRA",21,"SHAPE_DRAW",22,"SHAPE_MAX_SPECIAL",23,"SHAPE_CGO",23,"SHAPE_MIN_SURFACE",24,"SHAPE_ISOSURFACE",24,"SHAPE_CONTACT",25,
"SHAPE_LCAOCARTOON",26,"SHAPE_LAST_ATOM_VIS_FLAG",26,"SHAPE_MO",27,"SHAPE_NBO",28,"SHAPE_PMESH",29,"SHAPE_PLOT3D",30,"SHAPE_MAX_SURFACE",30,"SHAPE_MAX_MESH_COLLECTION",30,"SHAPE_ECHO",31,"SHAPE_MAX_HAS_ID",32,"SHAPE_BBCAGE",32,"SHAPE_MAX_HAS_SETVIS",33,"SHAPE_UCCAGE",33,"SHAPE_AXES",34,"SHAPE_HOVER",35,"SHAPE_FRANK",36,"SHAPE_MAX",37,"VIS_BOND_FLAG",32,"VIS_BALLS_FLAG",16,"VIS_LABEL_FLAG",512,"VIS_BACKBONE_FLAG",8192,"VIS_CARTOON_FLAG",32768,"ALPHA_CARBON_VISIBILITY_FLAG",1040384,"shapeClassBases",
u(-1,"Balls Sticks Hsticks Sssticks Struts Labels Measures Stars Halos Backbone Trace Cartoon Strands MeshRibbon Ribbons Rockets Dots Dipoles Vectors GeoSurface Ellipsoids Polyhedra Draw CGO Isosurface Contact LcaoCartoon MolecularOrbital NBO Pmesh Plot3D Echo Bbcage Uccage Axes Hover Frank".split(" ")),"SCRIPT_COMPLETED","Script completed","JPEG_EXTENSIONS",";jpg;jpeg;jpg64;jpeg64;");c$.IMAGE_TYPES=c$.prototype.IMAGE_TYPES=";jpg;jpeg;jpg64;jpeg64;gif;gift;pdf;ppm;png;pngj;pngt;";c$.IMAGE_OR_SCENE=
c$.prototype.IMAGE_OR_SCENE=";jpg;jpeg;jpg64;jpeg64;gif;gift;pdf;ppm;png;pngj;pngt;scene;";E(c$,"LABEL_MINIMUM_FONTSIZE",6,"LABEL_MAXIMUM_FONTSIZE",63,"LABEL_DEFAULT_FONTSIZE",13,"LABEL_DEFAULT_X_OFFSET",4,"LABEL_DEFAULT_Y_OFFSET",4,"LABEL_OFFSET_MAX",500,"LABEL_OFFSET_MASK",1023,"LABEL_FLAGY_OFFSET_SHIFT",11,"LABEL_FLAGX_OFFSET_SHIFT",21,"LABEL_FLAGS",63,"LABEL_POINTER_FLAGS",3,"LABEL_POINTER_NONE",0,"LABEL_POINTER_ON",1,"LABEL_POINTER_BACKGROUND",2,"TEXT_ALIGN_SHIFT",2,"TEXT_ALIGN_FLAGS",12,"TEXT_ALIGN_NONE",
0,"TEXT_ALIGN_LEFT",4,"TEXT_ALIGN_CENTER",8,"TEXT_ALIGN_RIGHT",12,"LABEL_ZPOS_FLAGS",48,"LABEL_ZPOS_GROUP",16,"LABEL_ZPOS_FRONT",32,"LABEL_EXPLICIT",64,"LABEL_CENTERED",256,"LABEL_DEFAULT_OFFSET",8396800,"ECHO_TOP",0,"ECHO_BOTTOM",1,"ECHO_MIDDLE",2,"ECHO_XY",3,"ECHO_XYZ",4,"echoNames",u(-1,["top","bottom","middle","xy","xyz"]),"hAlignNames",u(-1,["","left","center","right"]),"SMILES_TYPE_SMILES",1,"SMILES_TYPE_SMARTS",2,"SMILES_TYPE_OPENSMILES",5,"SMILES_TYPE_OPENSMARTS",7,"SMILES_FIRST_MATCH_ONLY",
8,"SMILES_NO_AROMATIC",16,"SMILES_IGNORE_STEREOCHEMISTRY",32,"SMILES_INVERT_STEREOCHEMISTRY",64,"SMILES_MAP_UNIQUE",128,"SMILES_AROMATIC_DEFINED",128,"SMILES_AROMATIC_STRICT",256,"SMILES_AROMATIC_DOUBLE",512,"SMILES_AROMATIC_MMFF94",768,"SMILES_AROMATIC_PLANAR",1024,"SMILES_IGNORE_ATOM_CLASS",2048,"SMILES_GEN_EXPLICIT_H_ALL",4096,"SMILES_GEN_EXPLICIT_H2_ONLY",8192,"SMILES_GEN_TOPOLOGY",16384,"SMILES_GEN_POLYHEDRAL",65536,"SMILES_GEN_ATOM_COMMENT",131072,"SMILES_GEN_BIO",1048576,"SMILES_GEN_BIO_ALLOW_UNMATCHED_RINGS",
3145728,"SMILES_GEN_BIO_COV_CROSSLINK",5242880,"SMILES_GEN_BIO_HH_CROSSLINK",9437184,"SMILES_GEN_BIO_COMMENT",17825792,"SMILES_GEN_BIO_NOCOMMENTS",34603008,"SMILES_GROUP_BY_MODEL",67108864,"SMILES_2D",134217728,"JSV_NOT",-1,"JSV_SEND_JDXMOL",0,"JSV_SETPEAKS",7,"JSV_SELECT",14,"JSV_STRUCTURE",21,"JSV_SEND_H1SIMULATE",28,"JSV_SEND_C13SIMULATE",35,"NBO_MODEL",42,"NBO_RUN",49,"NBO_VIEW",56,"NBO_SEARCH",63,"NBO_CONFIG",70,"JSV_CLOSE",77,"READER_NOT_FOUND","File reader was not found:","UNITID_MODEL",1,
"UNITID_RESIDUE",2,"UNITID_ATOM",4,"UNITID_INSCODE",8,"UNITID_TRIM",16,"DEFAULT_DRAG_DROP_SCRIPT","zap; load SYNC \"%FILE\";if (%ALLOWCARTOONS && _loadScript == '' && defaultLoadScript == '' && _filetype == 'Pdb') {if ({(protein or nucleic)&*/1.1} && {*/1.1}[1].groupindex != {*/1.1}[0].groupindex){select protein or nucleic;cartoons only;}if ({visible && cartoons > 0}){color structure}else{wireframe -0.1};if (!{visible}){spacefill 23%};select *}")});s("JV");v(["java.io.IOException"],"JV.JmolAsyncException",
null,function(){c$=t(function(){this.fileName=null;p(this,arguments)},JV,"JmolAsyncException",java.io.IOException);q(c$,function(a){I(this,JV.JmolAsyncException,[]);this.fileName=a},"~S");c(c$,"getFileName",function(){return this.fileName})});s("JV");v(null,"JV.ModelManager",["JM.ModelLoader"],function(){c$=t(function(){this.fileName=this.modelSetPathName=this.modelSet=this.vwr=null;p(this,arguments)},JV,"ModelManager");q(c$,function(a){this.vwr=a},"JV.Viewer");c(c$,"zap",function(){this.modelSetPathName=
this.fileName=null;new JM.ModelLoader(this.vwr,this.vwr.getZapName(),null,null,null,null)});c(c$,"createModelSet",function(a,b,c,g,f,e){var h=null;e?(h=this.modelSet.modelSetName,h.equals("zapped")?h=null:0>h.indexOf(" (modified)")&&(h+=" (modified)")):null==g?this.zap():(this.modelSetPathName=a,this.fileName=b);null!=g&&(null==h&&(h=this.vwr.getModelAdapter().getAtomSetCollectionName(g),null!=h&&(h=h.trim(),0==h.length&&(h=null)),null==h&&(h=JV.ModelManager.reduceFilename(b))),new JM.ModelLoader(this.vwr,
h,c,g,e?this.modelSet:null,f));0==this.modelSet.ac&&!this.modelSet.getMSInfoB("isPyMOL")&&this.zap()},"~S,~S,JU.SB,~O,JU.BS,~B");c$.reduceFilename=c(c$,"reduceFilename",function(a){if(null==a)return null;var b=a.indexOf(".");0<b&&(a=a.substring(0,b));24<a.length&&(a=a.substring(0,20)+" ...");return a},"~S");c(c$,"createAtomDataSet",function(a,b){return JM.ModelLoader.createAtomDataSet(this.vwr,this.modelSet,b,a,this.vwr.bsA())},"~O,~N")});s("JV");c$=t(function(){this.y=this.x=-1E3;this.modifiers=
0;this.time=-1;this.name=null;p(this,arguments)},JV,"MouseState");q(c$,function(a){this.name=a},"~S");c(c$,"set",function(a,b,c,g){this.time=a;this.x=b;this.y=c;this.modifiers=g},"~N,~N,~N,~N");c(c$,"setCurrent",function(a,b){this.time=a.time;2>b&&(this.x=a.x,this.y=a.y);this.modifiers=a.modifiers},"JV.MouseState,~N");c(c$,"inRange",function(a,b,c){return Math.abs(this.x-b)<=a&&Math.abs(this.y-c)<=a},"~N,~N,~N");c(c$,"check",function(a,b,c,g,f,e){return this.modifiers==g&&(2147483647<=e?this.inRange(a,
b,c):f-this.time<e&&20<f-this.time)},"~N,~N,~N,~N,~N,~N");c(c$,"is",function(a){return a.x==this.x&&a.y==this.y&&a.time==this.time},"JV.MouseState");E(c$,"MIN_DELAY_MS",20);s("JV");v(["JU.BS"],"JV.SelectionManager",["JU.AU","J.i18n.GT","JU.BSUtil"],function(){c$=t(function(){this.noneSelected=this.bsDeleted=this.bsSubset=this.bsFixed=this.bsSelection=this.bsHidden=this.listeners=this.vwr=null;this.empty=1;this.hideNotSelected=!1;this.bsTemp=null;p(this,arguments)},JV,"SelectionManager");O(c$,function(){this.listeners=
[];this.bsHidden=new JU.BS;this.bsSelection=new JU.BS;this.bsFixed=new JU.BS;this.bsTemp=new JU.BS});q(c$,function(a){this.vwr=a},"JV.Viewer");c(c$,"processDeletedModelAtoms",function(a){JU.BSUtil.deleteBits(this.bsDeleted,a);JU.BSUtil.deleteBits(this.bsSubset,a);JU.BSUtil.deleteBits(this.bsFixed,a);JU.BSUtil.deleteBits(this.bsHidden,a);var b=JU.BSUtil.copy(this.bsSelection);JU.BSUtil.deleteBits(b,a);this.setSelectionSet(b,0);this.selectionChanged(!1)},"JU.BS");c(c$,"clear",function(){this.clearSelection(!0);
this.setSelectionSubset(null);this.hide(null,null,0,!0);this.bsDeleted=null;this.setMotionFixedAtoms(null)});c(c$,"display",function(a,b,c,g){switch(c){default:c=null==this.bsSubset?null:JU.BSUtil.andNot(JU.BSUtil.copy(this.bsHidden),this.bsSubset);var f=a.getModelAtomBitSetIncludingDeleted(-1,!1);this.bsHidden.or(f);null!=c&&(this.bsHidden.and(this.bsSubset),this.bsHidden.or(c));case 1275069441:null!=b&&this.bsHidden.andNot(b);break;case 1073742119:null!=b&&this.bsHidden.or(b)}JU.BSUtil.andNot(this.bsHidden,
this.bsDeleted);a.setBsHidden(this.bsHidden);g||this.vwr.reportSelection(J.i18n.GT.i(J.i18n.GT.$("{0} atoms hidden"),this.bsHidden.cardinality()))},"JM.ModelSet,JU.BS,~N,~B");c(c$,"hide",function(a,b,c,g){var f=0==c||null==this.bsSubset?null:JU.BSUtil.andNot(JU.BSUtil.copy(this.bsHidden),this.bsSubset);JV.SelectionManager.setBitSet(this.bsHidden,b,c);null!=f&&this.bsHidden.or(f);null!=a&&a.setBsHidden(this.bsHidden);g||this.vwr.reportSelection(J.i18n.GT.i(J.i18n.GT.$("{0} atoms hidden"),this.bsHidden.cardinality()))},
"JM.ModelSet,JU.BS,~N,~B");c(c$,"setSelectionSet",function(a,b){JV.SelectionManager.setBitSet(this.bsSelection,a,b);this.empty=-1},"JU.BS,~N");c$.setBitSet=c(c$,"setBitSet",function(a,b,c){switch(c){default:a.clearAll();case 1275069441:null!=b&&a.or(b);break;case 1073742119:null!=b&&a.andNot(b)}},"JU.BS,JU.BS,~N");c(c$,"getHiddenSet",function(){return this.bsHidden});c(c$,"getHideNotSelected",function(){return this.hideNotSelected});c(c$,"setHideNotSelected",function(a){(this.hideNotSelected=a)&&
this.selectionChanged(!1)},"~B");c(c$,"isSelected",function(a){return 0<=a&&this.bsSelection.get(a)},"~N");c(c$,"select",function(a,b,c){null==a?this.selectAll(!0):this.setSelectionSet(a,b);this.vwr.getBoolean(1612709900)||this.excludeSelectionSet(this.vwr.ms.getAtoms(1612709900,null));this.vwr.getBoolean(1612709894)||this.excludeSelectionSet(this.vwr.ms.getAtoms(1612709894,null));this.selectionChanged(!1);if((a=this.vwr.getBoolean(603979879))||!c)b=this.getSelectionCount(),a?this.vwr.getChimeMessenger().reportSelection(b):
c||this.vwr.reportSelection(J.i18n.GT.i(J.i18n.GT.$("{0} atoms selected"),b))},"JU.BS,~N,~B");c(c$,"selectAll",function(a){var b=this.vwr.ms.ac;for(this.empty=0==b?1:0;0<=--b;)this.bsSelection.set(b);JU.BSUtil.andNot(this.bsSelection,this.bsDeleted);this.selectionChanged(a)},"~B");c(c$,"clearSelection",function(a){this.setHideNotSelected(!1);this.bsSelection.clearAll();this.empty=1;this.selectionChanged(a)},"~B");c(c$,"isAtomSelected",function(a){return((null==this.bsSubset||this.bsSubset.get(a))&&
null==this.bsDeleted||!this.bsDeleted.get(a))&&this.bsSelection.get(a)},"~N");c(c$,"setSelectedAtom",function(a,b){0>a?this.selectionChanged(!0):null!=this.bsSubset&&!this.bsSubset.get(a)||null!=this.bsDeleted&&this.bsDeleted.get(a)||(this.bsSelection.setBitTo(a,b),this.empty=b?0:-1)},"~N,~B");c(c$,"setSelectionSubset",function(a){this.bsSubset=a},"JU.BS");c(c$,"isInSelectionSubset",function(a){return 0>a||null==this.bsSubset||this.bsSubset.get(a)},"~N");c(c$,"invertSelection",function(){JU.BSUtil.invertInPlace(this.bsSelection,
this.vwr.ms.ac);this.empty=0<this.bsSelection.length()?0:1;this.selectionChanged(!1)});c(c$,"excludeSelectionSet",function(a){null==a||1==this.empty||(this.bsSelection.andNot(a),this.empty=-1)},"JU.BS");c(c$,"getSelectionCount",function(){if(1==this.empty)return 0;this.empty=1;var a;null==this.bsSubset?a=this.bsSelection:(this.bsTemp.clearAll(),this.bsTemp.or(this.bsSubset),this.bsTemp.and(this.bsSelection),a=this.bsTemp);a=a.cardinality();0<a&&(this.empty=0);return a});c(c$,"addListener",function(a){for(var b=
this.listeners.length;0<=--b;)if(this.listeners[b]===a){this.listeners[b]=null;break}for(var c=this.listeners.length,b=c;0<=--b;)if(null==this.listeners[b]){this.listeners[b]=a;return}this.listeners=0==this.listeners.length?Array(1):JU.AU.doubleLength(this.listeners);this.listeners[c]=a},"J.api.JmolSelectionListener");c(c$,"selectionChanged",function(a){this.hideNotSelected&&this.hide(this.vwr.ms,JU.BSUtil.copyInvert(this.bsSelection,this.vwr.ms.ac),0,a);if(!(a||0==this.listeners.length))for(a=this.listeners.length;0<=
--a;)null!=this.listeners[a]&&this.listeners[a].selectionChanged(this.bsSelection)},"~B");c(c$,"deleteAtoms",function(a){var b=JU.BSUtil.copy(a);null==this.bsDeleted?this.bsDeleted=b:(b.andNot(this.bsDeleted),this.bsDeleted.or(a));this.bsHidden.andNot(this.bsDeleted);this.bsSelection.andNot(this.bsDeleted);return b.cardinality()},"JU.BS");c(c$,"getSelectedAtoms",function(){if(null==this.bsSubset)return this.bsSelection;var a=JU.BSUtil.copy(this.bsSelection);a.and(this.bsSubset);return a});c(c$,"getSelectedAtomsNoSubset",
function(){return JU.BSUtil.copy(this.bsSelection)});c(c$,"excludeAtoms",function(a,b){null!=this.bsDeleted&&a.andNot(this.bsDeleted);!b&&null!=this.bsSubset&&(a=JU.BSUtil.copy(a)).and(this.bsSubset);return a},"JU.BS,~B");c(c$,"setMotionFixedAtoms",function(a){this.bsFixed.clearAll();null!=a&&this.bsFixed.or(a)},"JU.BS");c(c$,"getMotionFixedAtoms",function(){return this.bsFixed});E(c$,"TRUE",1,"FALSE",0,"UNKNOWN",-1)});s("JV");v(null,"JV.ShapeManager","java.lang.Boolean JU.BS $.P3 J.api.Interface J.c.PAL $.VDW JM.Atom JU.BSUtil JV.JC".split(" "),
function(){c$=t(function(){this.navMinMax=this.bsSlabbedInternal=this.bsRenderableAtoms=this.vwr=this.shapes=this.ms=null;p(this,arguments)},JV,"ShapeManager");O(c$,function(){this.navMinMax=B(4,0)});q(c$,function(a){this.vwr=a;this.bsRenderableAtoms=new JU.BS;this.bsSlabbedInternal=new JU.BS},"JV.Viewer");c(c$,"findNearestShapeAtomIndex",function(a,b,c,g){if(null!=this.shapes)for(var f=0;f<this.shapes.length&&null==c[0];++f)null!=this.shapes[f]&&this.shapes[f].findNearestAtomIndex(a,b,c,g)},"~N,~N,~A,JU.BS");
c(c$,"getShapePropertyIndex",function(a,b,c){if(null==this.shapes||null==this.shapes[a])return null;this.vwr.setShapeErrorState(a,"get "+b);a=this.shapes[a].getProperty(b,c);this.vwr.setShapeErrorState(-1,null);return a},"~N,~S,~N");c(c$,"getShapePropertyData",function(a,b,c){if(null==this.shapes||null==this.shapes[a])return!1;this.vwr.setShapeErrorState(a,"get "+b);a=this.shapes[a].getPropertyData(b,c);this.vwr.setShapeErrorState(-1,null);return a},"~N,~S,~A");c(c$,"getShapeIdFromObjectName",function(a){if(null!=
this.shapes)for(var b=16;30>b;++b)if(null!=this.shapes[b]&&0<=this.shapes[b].getIndexFromName(a))return b;return null!=this.shapes[6]&&0<=this.shapes[6].getIndexFromName(a)?6:-1},"~S");c(c$,"loadDefaultShapes",function(a){this.ms=a;if(null!=this.shapes)for(var b=0;b<this.shapes.length;++b)null!=this.shapes[b]&&this.shapes[b].setModelSet(a);this.loadShape(0);this.loadShape(1)},"JM.ModelSet");c(c$,"loadShape",function(a){if(null==this.shapes)return null;if(null!=this.shapes[a])return this.shapes[a];
if(2==a||3==a||4==a)return null;var b=JV.JC.getShapeClassName(a,!1);if(null==(b=J.api.Interface.getInterface(b,this.vwr,"shape")))return null;this.vwr.setShapeErrorState(a,"allocate");b.initializeShape(this.vwr,this.ms,a);this.vwr.setShapeErrorState(-1,null);return this.shapes[a]=b},"~N");c(c$,"notifyAtomPositionsChanged",function(a,b,c){var g=Integer.$valueOf(a);a=this.vwr.getModelUndeletedAtomsBitSet(a);for(var f=0;37>f;f++)null!=this.shapes[f]&&this.setShapePropertyBs(f,"refreshTrajectories",u(-1,
[g,b,c]),a)},"~N,JU.BS,JU.M4");c(c$,"releaseShape",function(a){null!=this.shapes&&(this.shapes[a]=null)},"~N");c(c$,"resetShapes",function(){this.shapes=Array(37)});c(c$,"setShapeSizeBs",function(a,b,c,g){if(null!=this.shapes){if(null==g&&(1!=a||2147483647!=b))g=this.vwr.bsA();null!=c&&(0!=c.value&&c.vdwType===J.c.VDW.TEMP)&&this.ms.getBfactor100Lo();this.vwr.setShapeErrorState(a,"set size");(null==c?0!=b:0!=c.value)&&this.loadShape(a);null!=this.shapes[a]&&this.shapes[a].setShapeSizeRD(b,c,g);this.vwr.setShapeErrorState(-1,
null)}},"~N,~N,J.atomdata.RadiusData,JU.BS");c(c$,"setLabel",function(a,b){if(null==a){if(null==this.shapes[5])return}else this.loadShape(5),this.setShapeSizeBs(5,0,null,b);this.setShapePropertyBs(5,"label",a,b)},"~O,JU.BS");c(c$,"setShapePropertyBs",function(a,b,c,g){null==this.shapes||null==this.shapes[a]||(null==g&&(g=this.vwr.bsA()),this.vwr.setShapeErrorState(a,"set "+b),this.shapes[a].setProperty(b.intern(),c,g),this.vwr.setShapeErrorState(-1,null))},"~N,~S,~O,JU.BS");c(c$,"checkFrankclicked",
function(a,b){var c=this.shapes[36];return null!=c&&c.wasClicked(a,b)},"~N,~N");c(c$,"checkObjectClicked",function(a,b,c,g,f){var e,h=null;if(2==this.vwr.getPickingMode())return this.shapes[5].checkObjectClicked(a,b,c,g,!1);if(0!=c&&this.vwr.getBondsPickable()&&null!=(h=this.shapes[1].checkObjectClicked(a,b,c,g,!1)))return h;for(var j=0;j<JV.ShapeManager.clickableMax;j++)if(null!=(e=this.shapes[JV.ShapeManager.hoverable[j]])&&null!=(h=e.checkObjectClicked(a,b,c,g,f)))return h;return null},"~N,~N,~N,JU.BS,~B");
c(c$,"checkObjectDragged",function(a,b,c,g,f,e,h){for(var j=!1,l=0<h?h+1:37;!j&&h<l;++h)null!=this.shapes[h]&&(j=this.shapes[h].checkObjectDragged(a,b,c,g,f,e));return j},"~N,~N,~N,~N,~N,JU.BS,~N");c(c$,"checkObjectHovered",function(a,b,c,g){var f=this.shapes[1];if(g&&null!=f&&f.checkObjectHovered(a,b,c))return!0;for(g=0;g<JV.ShapeManager.hoverable.length;g++)if(f=this.shapes[JV.ShapeManager.hoverable[g]],null!=f&&f.checkObjectHovered(a,b,c))return!0;return!1},"~N,~N,JU.BS,~B");c(c$,"deleteShapeAtoms",
function(a,b){if(null!=this.shapes)for(var c=0;37>c;c++)null!=this.shapes[c]&&this.setShapePropertyBs(c,"deleteModelAtoms",a,b)},"~A,JU.BS");c(c$,"deleteVdwDependentShapes",function(a){null==a&&(a=this.vwr.bsA());null!=this.shapes[24]&&this.shapes[24].setProperty("deleteVdw",null,a);null!=this.shapes[25]&&this.shapes[25].setProperty("deleteVdw",null,a)},"JU.BS");c(c$,"getAtomShapeValue",function(a,b,c){a=JV.JC.shapeTokenIndex(a);if(0>a||null==this.shapes[a])return 0;c=this.shapes[a].getSize(c);if(0==
c){if(0==(b.shapeVisibilityFlags&this.shapes[a].vf))return 0;c=this.shapes[a].getSizeG(b)}return c/2E3},"~N,JM.Group,~N");c(c$,"replaceGroup",function(a,b){if(null!=this.shapes)for(var c=9;16>c;c++)null!=this.shapes[c]&&this.shapes[c].replaceGroup(a,b)},"JM.Group,JM.Group");c(c$,"getObjectMap",function(a,b){if(null!=this.shapes)for(var c=Boolean.$valueOf(b),g=16;30>g;++g)this.getShapePropertyData(g,"getNames",u(-1,[a,c]))},"java.util.Map,~B");c(c$,"getProperty",function(a){return a.equals("getShapes")?
this.shapes:null},"~O");c(c$,"getShape",function(a){return null==this.shapes?null:this.shapes[a]},"~N");c(c$,"resetBioshapes",function(a){if(null!=this.shapes)for(var b=0;b<this.shapes.length;++b)null!=this.shapes[b]&&this.shapes[b].isBioShape&&(this.shapes[b].setModelSet(this.ms),this.shapes[b].setShapeSizeRD(0,null,a),this.shapes[b].setProperty("color",J.c.PAL.NONE,a))},"JU.BS");c(c$,"setAtomLabel",function(a,b){null!=this.shapes&&this.shapes[5].setProperty("label:"+a,Integer.$valueOf(b),null)},
"~S,~N");c(c$,"setModelVisibility",function(){var a=this.shapes;if(!(null==a||null==a[0])){for(var b=this.vwr.getVisibleFramesBitSet(),c=8;33>c;c++)null!=a[c]&&a[c].setModelVisibilityFlags(b);var a=this.vwr.getBoolean(603979922),g=this.vwr.slm.bsDeleted,f=this.ms.at;this.ms.clearVisibleSets();if(0<f.length)for(c=this.ms.ac;0<=--c;){var e=f[c];null!=e&&(e.shapeVisibilityFlags&=-64);if(!(null!=g&&g.get(c))&&b.get(e.mi)){var h=1;if(!this.ms.isAtomHidden(c)&&(a||1!=e.getElementNumber()))h|=8,0!=e.madAtom&&
(h|=16),e.setShapeVisibility(h,!0)}}this.setShapeVis()}});c(c$,"setShapeVis",function(){for(var a=0;37>a;++a){var b=this.shapes[a];null!=b&&b.setAtomClickability()}});c(c$,"finalizeAtoms",function(a,b){var c=this.vwr,g=c.tm;b&&c.finalizeTransformParameters();if(null!=a){var f=this.ms.getAtomSetCenter(a),e=new JU.P3;g.transformPt3f(f,e);e.add(g.ptOffset);g.unTransformPoint(e,e);e.sub(f);c.setAtomCoordsRelative(e,a);g.ptOffset.set(0,0,0);g.bsSelectedAtoms=null}f=this.bsRenderableAtoms;this.ms.getAtomsInFrame(f);
var h=this.ms.vibrations,j=null!=h&&g.vibrationOn,l=null!=this.ms.bsModulated&&null!=this.ms.occupancies,n=this.ms.at,m,r=!1,p=this.bsSlabbedInternal;p.clearAll();for(var q=f.nextSetBit(0);0<=q;q=f.nextSetBit(q+1)){var e=n[q],s=j&&e.hasVibration()?g.transformPtVib(e,h[q]):g.transformPt(e);1==s.z&&(g.internalSlab&&g.xyzIsSlabbedInternal(e))&&p.set(q);e.sX=s.x;e.sY=s.y;e.sZ=s.z;var t=Math.abs(e.madAtom);t==JM.Atom.MAD_GLOBAL&&(t=A(2E3*c.getFloat(1140850689)));e.sD=$(c.tm.scaleToScreen(s.z,t));if(l&&
null!=h[q]&&-2147483648!=(m=h[q].getOccupancy100(j)))r=!0,e.setShapeVisibility(2,!1),0<=m&&50>m?e.setShapeVisibility(24,!1):e.setShapeVisibility(8|(0<e.madAtom?16:0),!0),this.ms.occupancies[e.i]=Math.abs(m)}r&&this.setShapeVis();m=c.gdata;if(g.slabEnabled){q=c.getBoolean(603979940);g=c.getBoolean(603979939);h=m.slab;j=m.depth;if(q){l=this.ms.getMolecules();r=this.ms.getMoleculeCountInModel(-1);for(q=0;q<r;q++)if(p=l[q],s=0,e=p.firstAtomIndex,f.get(e)){for(;s<p.ac&&!m.isClippedZ(n[e].sZ-(n[e].sD>>
1));s++,e++);if(s!=p.ac){e=p.firstAtomIndex;for(s=0;s<p.ac;s++)f.clear(e),n[e++].sZ=0}}}for(q=f.nextSetBit(0);0<=q;q=f.nextSetBit(q+1))if(e=n[q],m.isClippedZ(e.sZ-(g?n[q].sD>>1:0)))e.setClickable(0),l=w((g?-1:1)*e.sD/2),(e.sZ+l<h||e.sZ-l>j||!m.isInDisplayRange(e.sX,e.sY))&&f.clear(q)}if(0==this.ms.ac||!c.getShowNavigationPoint())return null;c=2147483647;m=-2147483648;g=2147483647;h=-2147483648;for(q=f.nextSetBit(0);0<=q;q=f.nextSetBit(q+1))e=n[q],e.sX<c&&(c=e.sX),e.sX>m&&(m=e.sX),e.sY<g&&(g=e.sY),
e.sY>h&&(h=e.sY);this.navMinMax[0]=c;this.navMinMax[1]=m;this.navMinMax[2]=g;this.navMinMax[3]=h;return this.navMinMax},"JU.BS,~B");c(c$,"setModelSet",function(a){this.ms=this.vwr.ms=a},"JM.ModelSet");c(c$,"checkInheritedShapes",function(){null!=this.shapes[24]&&this.setShapePropertyBs(24,"remapInherited",null,null)});c(c$,"restrictSelected",function(a,b){var c=this.vwr.slm.getSelectedAtomsNoSubset();if(b){this.vwr.slm.invertSelection();var g=this.vwr.slm.bsSubset;null!=g&&(c=this.vwr.slm.getSelectedAtomsNoSubset(),
c.and(g),this.vwr.select(c,!1,0,!0),JU.BSUtil.invertInPlace(c,this.vwr.ms.ac),c.and(g))}JU.BSUtil.andNot(c,this.vwr.slm.bsDeleted);g=this.vwr.getBoolean(603979812);a||this.vwr.setBooleanProperty("bondModeOr",!0);this.setShapeSizeBs(1,0,null,null);this.setShapePropertyBs(1,"type",Integer.$valueOf(32768),null);this.setShapeSizeBs(1,0,null,null);this.setShapePropertyBs(1,"type",Integer.$valueOf(1023),null);for(var f=this.vwr.bsA(),e=21;0<=--e;)6!=e&&null!=this.getShape(e)&&this.setShapeSizeBs(e,0,null,
f);null!=this.getShape(21)&&this.setShapePropertyBs(21,"off",f,null);this.setLabel(null,f);a||this.vwr.setBooleanProperty("bondModeOr",g);this.vwr.select(c,!1,0,!0)},"~B,~B");E(c$,"hoverable",B(-1,[31,20,25,24,22,36]));c$.clickableMax=c$.prototype.clickableMax=JV.ShapeManager.hoverable.length-1});s("JV");v(["java.util.Hashtable"],["JV.Connection","$.Scene","$.StateManager","$.Connections"],["java.util.Arrays","JU.BS","$.SB","JM.Orientation","JU.BSUtil"],function(){c$=t(function(){this.saved=this.vwr=
null;this.lastCoordinates=this.lastShape=this.lastState=this.lastSelected=this.lastScene=this.lastConnections=this.lastContext=this.lastOrientation="";p(this,arguments)},JV,"StateManager");O(c$,function(){this.saved=new java.util.Hashtable});c$.getVariableList=c(c$,"getVariableList",function(a,b,c,g){var f=new JU.SB,e=0,h=Array(a.size()),j;for(a=a.entrySet().iterator();a.hasNext()&&((j=a.next())||1);){var l=j.getKey(),n=j.getValue();if((c||!l.startsWith("site_"))&&(!g||"@"==l.charAt(0)))h[e++]=l+
("@"==l.charAt(0)?" "+n.asString():" = "+JV.StateManager.varClip(l,n.escape(),b))}java.util.Arrays.sort(h,0,e);for(b=0;b<e;b++)null!=h[b]&&f.append("  ").append(h[b]).append(";\n");0==e&&!g&&f.append("# --no global user variables defined--;\n");return f.toString()},"java.util.Map,~N,~B,~B");c$.getObjectIdFromName=c(c$,"getObjectIdFromName",function(a){if(null==a)return-1;a="background axis1      axis2      axis3      boundbox   unitcell   frank      ".indexOf(a.toLowerCase());return 0>a?a:w(a/11)},
"~S");c$.getObjectNameFromId=c(c$,"getObjectNameFromId",function(a){return 0>a||7<=a?null:"background axis1      axis2      axis3      boundbox   unitcell   frank      ".substring(11*a,11*a+11).trim()},"~N");q(c$,function(a){this.vwr=a},"JV.Viewer");c(c$,"clear",function(a){this.vwr.setShowAxes(!1);this.vwr.setShowBbcage(!1);this.vwr.setShowUnitCell(!1);a.clear()},"JV.GlobalSettings");c(c$,"resetLighting",function(){this.vwr.setIntProperty("ambientPercent",45);this.vwr.setIntProperty("celShadingPower",
10);this.vwr.setIntProperty("diffusePercent",84);this.vwr.setIntProperty("phongExponent",64);this.vwr.setIntProperty("specularExponent",6);this.vwr.setIntProperty("specularPercent",22);this.vwr.setIntProperty("specularPower",40);this.vwr.setIntProperty("zDepth",0);this.vwr.setIntProperty("zShadePower",3);this.vwr.setIntProperty("zSlab",50);this.vwr.setBooleanProperty("specular",!0);this.vwr.setBooleanProperty("celShading",!1);this.vwr.setBooleanProperty("zshade",!1)});c(c$,"setCrystallographicDefaults",
function(){this.vwr.setAxesMode(603979808);this.vwr.setShowAxes(!0);this.vwr.setShowUnitCell(!0);this.vwr.setBooleanProperty("perspectiveDepth",!1)});c(c$,"setCommonDefaults",function(){this.vwr.setBooleanProperty("perspectiveDepth",!0);this.vwr.setFloatProperty("bondTolerance",0.45);this.vwr.setFloatProperty("minBondDistance",0.4);this.vwr.setIntProperty("bondingVersion",0);this.vwr.setBooleanProperty("translucent",!0)});c(c$,"setJmolDefaults",function(){this.setCommonDefaults();this.vwr.setStringProperty("defaultColorScheme",
"Jmol");this.vwr.setBooleanProperty("axesOrientationRasmol",!1);this.vwr.setBooleanProperty("zeroBasedXyzRasmol",!1);this.vwr.setIntProperty("percentVdwAtom",23);this.vwr.setIntProperty("bondRadiusMilliAngstroms",150);this.vwr.setVdwStr("auto")});c(c$,"setRasMolDefaults",function(){this.setCommonDefaults();this.vwr.setStringProperty("defaultColorScheme","RasMol");this.vwr.setBooleanProperty("axesOrientationRasmol",!0);this.vwr.setBooleanProperty("zeroBasedXyzRasmol",!0);this.vwr.setIntProperty("percentVdwAtom",
0);this.vwr.setIntProperty("bondRadiusMilliAngstroms",1);this.vwr.setVdwStr("Rasmol")});c(c$,"setPyMOLDefaults",function(){this.setCommonDefaults();this.vwr.setStringProperty("measurementUnits","ANGSTROMS");this.vwr.setBooleanProperty("zoomHeight",!0)});c$.getNoCase=c(c$,"getNoCase",function(a,b){for(var c,g=a.entrySet().iterator();g.hasNext()&&((c=g.next())||1);)if(c.getKey().equalsIgnoreCase(b))return c.getValue();return null},"java.util.Map,~S");c(c$,"listSavedStates",function(){for(var a="",b,
c=this.saved.keySet().iterator();c.hasNext()&&((b=c.next())||1);)a+="\n"+b;return a});c(c$,"deleteSavedType",function(a){for(var b=this.saved.keySet().iterator();b.hasNext();)b.next().startsWith(a)&&b.remove()},"~S");c(c$,"deleteSaved",function(a){for(var b=this.saved.keySet().iterator();b.hasNext();){var c=b.next();(c.startsWith(a)||c.endsWith("_"+a)&&c.indexOf("_")==c.lastIndexOf("_"+a))&&b.remove()}},"~S");c(c$,"saveSelection",function(a,b){a.equalsIgnoreCase("DELETE")?this.deleteSavedType("Selected_"):
(a=this.lastSelected="Selected_"+a,this.saved.put(a,JU.BSUtil.copy(b)))},"~S,JU.BS");c(c$,"restoreSelection",function(a){a=JV.StateManager.getNoCase(this.saved,0<a.length?"Selected_"+a:this.lastSelected);if(null==a)return this.vwr.select(new JU.BS,!1,0,!1),!1;this.vwr.select(a,!1,0,!1);return!0},"~S");c(c$,"saveState",function(a){a.equalsIgnoreCase("DELETE")?this.deleteSavedType("State_"):(a=this.lastState="State_"+a,this.saved.put(a,this.vwr.getStateInfo()))},"~S");c(c$,"getSavedState",function(a){a=
JV.StateManager.getNoCase(this.saved,0<a.length?"State_"+a:this.lastState);return null==a?"":a},"~S");c(c$,"saveStructure",function(a){a.equalsIgnoreCase("DELETE")?this.deleteSavedType("Shape_"):(a=this.lastShape="Shape_"+a,this.saved.put(a,this.vwr.getStructureState()))},"~S");c(c$,"getSavedStructure",function(a){a=JV.StateManager.getNoCase(this.saved,0<a.length?"Shape_"+a:this.lastShape);return null==a?"":a},"~S");c(c$,"saveCoordinates",function(a,b){a.equalsIgnoreCase("DELETE")?this.deleteSavedType("Coordinates_"):
(a=this.lastCoordinates="Coordinates_"+a,this.saved.put(a,this.vwr.getCoordinateState(b)))},"~S,JU.BS");c(c$,"getSavedCoordinates",function(a){a=JV.StateManager.getNoCase(this.saved,0<a.length?"Coordinates_"+a:this.lastCoordinates);return null==a?"":a},"~S");c(c$,"getOrientation",function(){return new JM.Orientation(this.vwr,!1,null)});c(c$,"getSavedOrientationText",function(a){var b;if(null!=a)return b=this.getOrientationFor(a),null==b?"":b.getMoveToText(!0);a=new JU.SB;for(var c=this.saved.entrySet().iterator();c.hasNext()&&
((b=c.next())||1);)b.getKey().startsWith("Orientation_")&&a.append(b.getValue().getMoveToText(!0));return a.toString()},"~S");c(c$,"saveScene",function(a,b){if(a.equalsIgnoreCase("DELETE"))this.deleteSavedType("Scene_");else{var c=new JV.Scene(b);c.saveName=this.lastScene="Scene_"+a;this.saved.put(c.saveName,c)}},"~S,java.util.Map");c(c$,"restoreScene",function(a,b){var c=JV.StateManager.getNoCase(this.saved,0<a.length?"Scene_"+a:this.lastScene);return null!=c&&c.restore(this.vwr,b)},"~S,~N");c(c$,
"saveOrientation",function(a,b){if(a.equalsIgnoreCase("DELETE"))this.deleteSavedType("Orientation_");else{var c=new JM.Orientation(this.vwr,a.equalsIgnoreCase("default"),b);c.saveName=this.lastOrientation="Orientation_"+a;this.saved.put(c.saveName,c)}},"~S,~A");c(c$,"restoreOrientation",function(a,b,c){a=this.getOrientationFor(a);return null!=a&&a.restore(b,c)},"~S,~N,~B");c(c$,"getOrientationFor",function(a){return JV.StateManager.getNoCase(this.saved,0<a.length?"Orientation_"+a:this.lastOrientation)},
"~S");c(c$,"saveContext",function(a,b){a.equalsIgnoreCase("DELETE")?this.deleteSavedType("Context_"):this.saved.put(this.lastContext="Context_"+a,b)},"~S,~O");c(c$,"getContext",function(a){return this.saved.get(0==a.length?this.lastContext:"Context_"+a)},"~S");c(c$,"saveBonds",function(a){if(a.equalsIgnoreCase("DELETE"))this.deleteSavedType("Bonds_");else{var b=new JV.Connections(this.vwr);b.saveName=this.lastConnections="Bonds_"+a;this.saved.put(b.saveName,b)}},"~S");c(c$,"restoreBonds",function(a){this.vwr.clearModelDependentObjects();
a=JV.StateManager.getNoCase(this.saved,0<a.length?"Bonds_"+a:this.lastConnections);return null!=a&&a.restore()},"~S");c$.varClip=c(c$,"varClip",function(a,b,c){0<c&&b.length>c&&(b=b.substring(0,c)+" #...more ("+b.length+" bytes -- use SHOW "+a+" or MESSAGE @"+a+" to view)");return b},"~S,~S,~N");E(c$,"OBJ_BACKGROUND",0,"OBJ_AXIS1",1,"OBJ_AXIS2",2,"OBJ_AXIS3",3,"OBJ_BOUNDBOX",4,"OBJ_UNITCELL",5,"OBJ_FRANK",6,"OBJ_MAX",7,"objectNameList","background axis1      axis2      axis3      boundbox   unitcell   frank      ");
c$=t(function(){this.scene=this.saveName=null;p(this,arguments)},JV,"Scene");q(c$,function(a){this.scene=a},"java.util.Map");c(c$,"restore",function(a,b){var c=this.scene.get("generator");null!=c&&c.generateScene(this.scene);c=this.scene.get("pymolView");return null!=c&&a.tm.moveToPyMOL(a.eval,b,c)},"JV.Viewer,~N");c$=t(function(){this.saveName=null;this.bondCount=0;this.vwr=this.connections=null;p(this,arguments)},JV,"Connections");q(c$,function(a){var b=a.ms;if(null!=b){this.vwr=a;this.bondCount=
b.bondCount;this.connections=Array(this.bondCount+1);a=b.bo;for(b=this.bondCount;0<=--b;){var c=a[b];this.connections[b]=new JV.Connection(c.atom1.i,c.atom2.i,c.mad,c.colix,c.order,c.getEnergy(),c.shapeVisibilityFlags)}}},"JV.Viewer");c(c$,"restore",function(){var a=this.vwr.ms;if(null==a)return!1;a.deleteAllBonds();for(var b=this.bondCount;0<=--b;){var c=this.connections[b],g=a.ac;c.atomIndex1>=g||c.atomIndex2>=g||(g=a.bondAtoms(a.at[c.atomIndex1],a.at[c.atomIndex2],c.order,c.mad,null,c.energy,!1,
!0),g.colix=c.colix,g.shapeVisibilityFlags=c.shapeVisibilityFlags)}for(b=a.bondCount;0<=--b;)a.bo[b].index=b;this.vwr.setShapeProperty(1,"reportAll",null);return!0});c$=t(function(){this.shapeVisibilityFlags=this.energy=this.order=this.colix=this.mad=this.atomIndex2=this.atomIndex1=0;p(this,arguments)},JV,"Connection");q(c$,function(a,b,c,g,f,e,h){this.atomIndex1=a;this.atomIndex2=b;this.mad=c;this.colix=g;this.order=f;this.energy=e;this.shapeVisibilityFlags=h},"~N,~N,~N,~N,~N,~N,~N")});s("JV");v(["java.util.Hashtable"],
"JV.StatusManager","java.lang.Boolean $.Float JU.Lst $.PT J.api.Interface J.c.CBK JS.SV JU.Logger JV.JC".split(" "),function(){c$=t(function(){this.cbl=this.jsl=this.vwr=null;this.statusList="";this.allowStatusReporting=!1;this.messageQueue=null;this.statusPtr=0;this.imageMap=this.jmolScriptCallbacks=null;this.minSyncRepeatMs=100;this.stereoSync=this.syncDisabled=this.isSynced=this.drivingSync=this.syncingMouse=this.syncingScripts=!1;this.qualityPNG=this.qualityJPG=-1;this.audios=this.imageType=null;
p(this,arguments)},JV,"StatusManager");O(c$,function(){this.messageQueue=new java.util.Hashtable;this.jmolScriptCallbacks=new java.util.Hashtable});q(c$,function(a){this.vwr=a},"JV.Viewer");c(c$,"recordStatus",function(a){return this.allowStatusReporting&&0<this.statusList.length&&(this.statusList.equals("all")||0<=this.statusList.indexOf(a))},"~S");c(c$,"setStatusChanged",function(a,b,c,g){if(this.recordStatus(a)){var f=new JU.Lst;f.addLast(Integer.$valueOf(++this.statusPtr));f.addLast(a);f.addLast(Integer.$valueOf(b));
f.addLast(c);b=g?null:this.messageQueue.get(a);null==b?this.messageQueue.put(a,b=new JU.Lst):b.size()==JV.StatusManager.MAXIMUM_QUEUE_LENGTH&&b.removeItemAt(0);b.addLast(f)}},"~S,~N,~O,~B");c(c$,"getStatusChanged",function(a){var b=0<a.length&&"-"==a.charAt(0),c=0<a.length&&"+"==a.charAt(0),g=!1;b?this.statusList=JU.PT.rep(this.statusList,a.substring(1,a.length),""):(a=JU.PT.rep(a,"+",""),this.statusList.equals(a)||c&&0<=this.statusList.indexOf(a)?g=!0:(c||(this.statusList=""),this.statusList+=a,
JU.Logger.debugging&&JU.Logger.debug("StatusManager messageQueue = "+this.statusList)));a=new JU.Lst;if(g)for(var f,g=this.messageQueue.entrySet().iterator();g.hasNext()&&((f=g.next())||1);)a.addLast(f.getValue());this.messageQueue.clear();this.statusPtr=0;return a},"~S");c(c$,"jmolScriptCallback",function(a){var b=this.jmolScriptCallbacks.get(a.name());null!=b&&this.vwr.evalStringQuietSync(b,!0,!1);this.jmolScriptCallbacks.containsKey("SYNC:"+a.name())&&(b="SYNC");return b},"J.c.CBK");c(c$,"setCallbackFunction",
function(a,b){if(null!=J.c.CBK.getCallback(a)){var c=J.c.CBK.getCallback(a).name();JU.Logger.info("StatusManager callback set for "+a+" f="+b+" cb="+c);if(null!=b&&b.startsWith("SYNC:"))if(b.toLowerCase().trim().equals("sync:off"))this.jmolScriptCallbacks.remove("SYNC:"+c),JU.Logger.info("SYNC callback for "+c+" deactivated");else{this.jmolScriptCallbacks.put("SYNC:"+c,"_");JU.Logger.info("SYNC callback for "+c+" activated");return}else{var g="",f=null==b?0:(g=b.toLowerCase()).startsWith("script:")?
7:g.startsWith("jmolscript:")?11:0;0==f?this.jmolScriptCallbacks.remove(c):this.jmolScriptCallbacks.put(c,b.substring(f).trim())}}null!=this.cbl&&this.cbl.setCallbackFunction(a,b)},"~S,~S");c(c$,"notifyEnabled",function(a){return null!=this.cbl&&this.cbl.notifyEnabled(a)},"J.c.CBK");c(c$,"setStatusAppletReady",function(a,b){var c=b?this.jmolScriptCallback(J.c.CBK.APPLETREADY):null;this.notifyEnabled(J.c.CBK.APPLETREADY)&&this.cbl.notifyCallback(J.c.CBK.APPLETREADY,u(-1,[c,a,Boolean.$valueOf(b),null]))},
"~S,~B");c(c$,"setStatusAtomMoved",function(a){var b=this.jmolScriptCallback(J.c.CBK.ATOMMOVED);this.setStatusChanged("atomMoved",-1,a,!1);this.notifyEnabled(J.c.CBK.ATOMMOVED)&&this.cbl.notifyCallback(J.c.CBK.ATOMMOVED,u(-1,[b,a]))},"JU.BS");c(c$,"setStatusAtomPicked",function(a,b,c){var g=this.jmolScriptCallback(J.c.CBK.PICK);JU.Logger.info("setStatusAtomPicked("+a+","+b+")");this.setStatusChanged("atomPicked",a,b,!1);this.notifyEnabled(J.c.CBK.PICK)&&this.cbl.notifyCallback(J.c.CBK.PICK,u(-1,[g,
b,Integer.$valueOf(a),c]))},"~N,~S,java.util.Map");c(c$,"setStatusClicked",function(a,b,c,g,f){var e=this.jmolScriptCallback(J.c.CBK.CLICK);if(!this.notifyEnabled(J.c.CBK.CLICK))return c;f=B(-1,[c,f]);this.cbl.notifyCallback(J.c.CBK.CLICK,u(-1,[e,Integer.$valueOf(a),Integer.$valueOf(b),Integer.$valueOf(c),Integer.$valueOf(g),f]));return f[0]},"~N,~N,~N,~N,~N");c(c$,"setStatusResized",function(a,b){var c=this.jmolScriptCallback(J.c.CBK.RESIZE);this.notifyEnabled(J.c.CBK.RESIZE)&&this.cbl.notifyCallback(J.c.CBK.RESIZE,
u(-1,[c,Integer.$valueOf(a),Integer.$valueOf(b)]))},"~N,~N");c(c$,"haveHoverCallback",function(){return this.jmolScriptCallbacks.containsKey(J.c.CBK.HOVER)||this.notifyEnabled(J.c.CBK.HOVER)});c(c$,"setStatusAtomHovered",function(a,b){var c=this.jmolScriptCallback(J.c.CBK.HOVER);this.notifyEnabled(J.c.CBK.HOVER)&&this.cbl.notifyCallback(J.c.CBK.HOVER,u(-1,[c,b,Integer.$valueOf(a)]))},"~N,~S");c(c$,"setStatusObjectHovered",function(a,b,c){var g=this.jmolScriptCallback(J.c.CBK.HOVER);this.notifyEnabled(J.c.CBK.HOVER)&&
this.cbl.notifyCallback(J.c.CBK.HOVER,u(-1,[g,b,Integer.$valueOf(-1),a,Float.$valueOf(c.x),Float.$valueOf(c.y),Float.$valueOf(c.z)]))},"~S,~S,JU.T3");c(c$,"showImage",function(a,b){var c=JU.PT.split(a,"\u0001");a=2>c.length?"Jmol":3>c.length||c[2].equals("null")?c[1].substring(c[1].lastIndexOf("/")+1):c[2];c=this.jmolScriptCallback(J.c.CBK.IMAGE);this.notifyEnabled(J.c.CBK.IMAGE)&&this.cbl.notifyCallback(J.c.CBK.IMAGE,u(-1,[c,a,b]));if(Boolean.TRUE.equals(b)){if(null!=this.imageMap){for(var c=new JU.Lst,
g,f=this.imageMap.keySet().iterator();f.hasNext()&&((g=f.next())||1);)c.addLast(g);for(g=c.size();0<=--g;)this.imageMap.get(c.get(g)).closeMe()}}else null==this.imageMap&&(this.imageMap=new java.util.Hashtable),g=this.imageMap.get(a),Boolean.FALSE.equals(b)?null!=g&&g.closeMe():(null==g&&null!=b&&(g=this.vwr.apiPlatform.getImageDialog(a,this.imageMap)),null!=g&&(null==b?g.closeMe():g.setImage(b)))},"~S,~O");c(c$,"setFileLoadStatus",function(a,b,c,g,f,e,h){null==a&&"resetUndo".equals(b)&&(b=this.vwr.getProperty("DATA_API",
"getAppConsole",null),null!=b&&b.zap(),b=this.vwr.getZapName());this.setStatusChanged("fileLoaded",f,a,!1);null!=g&&this.setStatusChanged("fileLoadError",f,g,!1);var j=this.jmolScriptCallback(J.c.CBK.LOADSTRUCT);e&&this.notifyEnabled(J.c.CBK.LOADSTRUCT)&&(e=this.vwr.getP("_smilesString"),0!=e.length&&(b=e),this.cbl.notifyCallback(J.c.CBK.LOADSTRUCT,u(-1,[j,a,b,c,g,Integer.$valueOf(f),this.vwr.getP("_modelNumber"),this.vwr.getModelNumberDotted(this.vwr.ms.mc-1),h])))},"~S,~S,~S,~S,~N,~B,Boolean");
c(c$,"setStatusModelKit",function(a){var b=1==a?"ON":"OFF";this.setStatusChanged("modelkit",a,b,!1);a=this.jmolScriptCallback(J.c.CBK.MODELKIT);this.notifyEnabled(J.c.CBK.MODELKIT)&&this.cbl.notifyCallback(J.c.CBK.MODELKIT,u(-1,[a,b]))},"~N");c(c$,"setStatusFrameChanged",function(a,b,c,g,f,e,h){if(null!=this.vwr.ms){var j=this.vwr.am.animationOn,l=j?-2-f:f;this.setStatusChanged("frameChanged",l,0<=f?this.vwr.getModelNumberDotted(f):"",!1);var n=this.jmolScriptCallback(J.c.CBK.ANIMFRAME);this.notifyEnabled(J.c.CBK.ANIMFRAME)&&
this.cbl.notifyCallback(J.c.CBK.ANIMFRAME,u(-1,[n,B(-1,[l,a,b,c,g,f]),h,Float.$valueOf(e)]));!j&&!this.vwr.isJSNoAWT&&this.vwr.checkMenuUpdate()}},"~N,~N,~N,~N,~N,~N,~S");c(c$,"setStatusDragDropped",function(a,b,c,g){this.setStatusChanged("dragDrop",0,"",!1);var f=this.jmolScriptCallback(J.c.CBK.DRAGDROP);if(!this.notifyEnabled(J.c.CBK.DRAGDROP))return!1;this.cbl.notifyCallback(J.c.CBK.DRAGDROP,u(-1,[f,Integer.$valueOf(a),Integer.$valueOf(b),Integer.$valueOf(c),g]));return!0},"~N,~N,~N,~S");c(c$,
"setScriptEcho",function(a,b){if(null!=a){this.setStatusChanged("scriptEcho",0,a,!1);var c=this.jmolScriptCallback(J.c.CBK.ECHO);this.notifyEnabled(J.c.CBK.ECHO)&&this.cbl.notifyCallback(J.c.CBK.ECHO,u(-1,[c,a,Integer.$valueOf(b?1:0)]))}},"~S,~B");c(c$,"setStatusMeasuring",function(a,b,c,g){this.setStatusChanged(a,b,c,!1);var f=null;a.equals("measureCompleted")?(JU.Logger.info("measurement["+b+"] = "+c),f=this.jmolScriptCallback(J.c.CBK.MEASURE)):a.equals("measurePicked")&&(this.setStatusChanged("measurePicked",
b,c,!1),JU.Logger.info("measurePicked "+b+" "+c));this.notifyEnabled(J.c.CBK.MEASURE)&&this.cbl.notifyCallback(J.c.CBK.MEASURE,u(-1,[f,c,Integer.$valueOf(b),a,Float.$valueOf(g)]))},"~S,~N,~S,~N");c(c$,"notifyError",function(a,b,c){var g=this.jmolScriptCallback(J.c.CBK.ERROR);this.notifyEnabled(J.c.CBK.ERROR)&&this.cbl.notifyCallback(J.c.CBK.ERROR,u(-1,[g,a,b,this.vwr.getShapeErrorState(),c]))},"~S,~S,~S");c(c$,"notifyMinimizationStatus",function(a,b,c,g,f){var e=this.jmolScriptCallback(J.c.CBK.MINIMIZATION);
this.notifyEnabled(J.c.CBK.MINIMIZATION)&&this.cbl.notifyCallback(J.c.CBK.MINIMIZATION,u(-1,[e,a,b,c,g,f]))},"~S,Integer,Float,Float,~S");c(c$,"setScriptStatus",function(a,b,c,g){if(-1>c)a=-2-c,this.setStatusChanged("scriptStarted",a,b,!1),a="script "+a+" started";else if(null==a)return;var f=0==c?this.jmolScriptCallback(J.c.CBK.SCRIPT):null,e="Script completed"===a;if(this.recordStatus("script")){var h=null!=g;this.setStatusChanged(h?"scriptError":"scriptStatus",0,a,!1);if(h||e)this.setStatusChanged("scriptTerminated",
1,"Jmol script terminated"+(h?" unsuccessfully: "+a:" successfully"),!1)}e&&(this.vwr.getBoolean(603979879)&&this.vwr.getBoolean(603979825))&&(a=this.vwr.getChimeMessenger().scriptCompleted(this,b,g));b=u(-1,[f,a,b,Integer.$valueOf(e?-1:c),g]);this.notifyEnabled(J.c.CBK.SCRIPT)&&this.cbl.notifyCallback(J.c.CBK.SCRIPT,b);this.processScript(b)},"~S,~S,~N,~S");c(c$,"processScript",function(a){var b=a[3].intValue();this.vwr.notifyScriptEditor(b,a);0==b&&this.vwr.sendConsoleMessage(null==a[1]?null:a[1].toString())},
"~A");c(c$,"doSync",function(){return this.isSynced&&this.drivingSync&&!this.syncDisabled});c(c$,"setSync",function(a){this.syncingMouse?null!=a&&this.syncSend(a,"*",0):this.syncingScripts||this.syncSend("!"+this.vwr.tm.getMoveToText(this.minSyncRepeatMs/1E3,!1),"*",0)},"~S");c(c$,"setSyncDriver",function(a){this.stereoSync&&4!=a&&(this.syncSend("SET_GRAPHICS_OFF","*",0),this.stereoSync=!1);switch(a){case 4:if(!this.syncDisabled)return;this.syncDisabled=!1;break;case 3:this.syncDisabled=!0;break;
case 5:this.stereoSync=this.isSynced=this.drivingSync=!0;break;case 1:this.isSynced=this.drivingSync=!0;break;case 2:this.drivingSync=!1;this.isSynced=!0;break;default:this.isSynced=this.drivingSync=!1}JU.Logger.debugging&&JU.Logger.debug(this.vwr.appletName+" sync mode="+a+"; synced? "+this.isSynced+"; driving? "+this.drivingSync+"; disabled? "+this.syncDisabled)},"~N");c(c$,"syncSend",function(a,b,c){return 0!=c||this.notifyEnabled(J.c.CBK.SYNC)?(a=u(-1,[null,a,b,Integer.$valueOf(c)]),null!=this.cbl&&
this.cbl.notifyCallback(J.c.CBK.SYNC,a),a[0]):null},"~S,~O,~N");c(c$,"modifySend",function(a,b,c,g){var f=this.jmolScriptCallback(J.c.CBK.STRUCTUREMODIFIED);this.notifyEnabled(J.c.CBK.STRUCTUREMODIFIED)&&this.cbl.notifyCallback(J.c.CBK.STRUCTUREMODIFIED,u(-1,[f,Integer.$valueOf(c),Integer.$valueOf(a),Integer.$valueOf(b),g]))},"~N,~N,~N,~S");c(c$,"processService",function(a){var b=a.get("service");if(null==b)return null;if(n(b,JS.SV)){var b=new java.util.Hashtable,c;for(a=a.entrySet().iterator();a.hasNext()&&
((c=a.next())||1);)b.put(c.getKey(),JS.SV.oValue(c.getValue()));a=b}this.notifyEnabled(J.c.CBK.SERVICE)&&this.cbl.notifyCallback(J.c.CBK.SERVICE,u(-1,[null,a]));return a},"java.util.Map");c(c$,"getSyncMode",function(){return!this.isSynced?0:this.drivingSync?1:2});c(c$,"showUrl",function(a){null!=this.jsl&&this.jsl.showUrl(a)},"~S");c(c$,"clearConsole",function(){this.vwr.sendConsoleMessage(null);null!=this.jsl&&this.cbl.notifyCallback(J.c.CBK.MESSAGE,null)});c(c$,"functionXY",function(a,b,c){return null==
this.jsl?K(Math.abs(b),Math.abs(c),0):this.jsl.functionXY(a,b,c)},"~S,~N,~N");c(c$,"functionXYZ",function(a,b,c,g){return null==this.jsl?K(Math.abs(b),Math.abs(c),Math.abs(c),0):this.jsl.functionXYZ(a,b,c,g)},"~S,~N,~N,~N");c(c$,"jsEval",function(a){return null==this.jsl?"":this.jsl.eval(a)},"~S");c(c$,"createImage",function(a,b,c,g,f){return null==this.jsl?null:this.jsl.createImage(a,b,null==c?g:c,f)},"~S,~S,~S,~A,~N");c(c$,"getRegistryInfo",function(){return null==this.jsl?null:this.jsl.getRegistryInfo()});
c(c$,"dialogAsk",function(a,b,c){var g=a.equals("Save Image"),f=J.api.Interface.getOption("dialog.Dialog",this.vwr,"status");if(null==f)return null;f.setupUI(!1);g&&f.setImageInfo(this.qualityJPG,this.qualityPNG,this.imageType);a=f.getFileNameFromDialog(this.vwr,a,b);g&&null!=a&&(this.qualityJPG=f.getQuality("JPG"),this.qualityPNG=f.getQuality("PNG"),g=f.getType(),null!=c&&(c.put("qualityJPG",Integer.$valueOf(this.qualityJPG)),c.put("qualityPNG",Integer.$valueOf(this.qualityPNG)),null!=g&&c.put("dialogImageType",
g)),null!=g&&(this.imageType=g));return a},"~S,~S,java.util.Map");c(c$,"getJspecViewProperties",function(a){return null==this.jsl?null:this.jsl.getJSpecViewProperty(null==a||0==a.length?"":":"+a)},"~S");c(c$,"resizeInnerPanel",function(a,b){return null==this.jsl||a==this.vwr.getScreenWidth()&&b==this.vwr.getScreenHeight()?B(-1,[a,b]):this.jsl.resizeInnerPanel("preferredWidthHeight "+a+" "+b+";")},"~N,~N");c(c$,"resizeInnerPanelString",function(a){null!=this.jsl&&this.jsl.resizeInnerPanel(a)},"~S");
c(c$,"registerAudio",function(a,b){this.stopAudio(a);null==this.audios&&(this.audios=new java.util.Hashtable);null==b?this.audios.remove(a):this.audios.put(a,b.get("audioPlayer"))},"~S,java.util.Map");c(c$,"stopAudio",function(a){null!=this.audios&&(a=this.audios.get(a),null!=a&&a.action("kill"))},"~S");c(c$,"playAudio",function(a){if(this.vwr.getBoolean(603979797))try{var b=null==a?"close":a.get("action"),c=null==a?null:a.get("id");if(null!=b&&0<b.length){if(null==c||0==c.length){if(null==this.audios||
this.audios.isEmpty())return;if(b.equals("close"))for(var g,f=this.audios.keySet().iterator();f.hasNext()&&((g=f.next())||1);){var e=this.audios.remove(g);e.action("close")}return}e=this.audios.get(c);if(null!=e){e.action(b);return}}try{J.api.Interface.getInterface("JU.JmolAudio",this.vwr,"script").playAudio(this.vwr,a)}catch(h){if(H(h,Exception))JU.Logger.info(h.getMessage());else throw h;}}catch(j){}else null!=a&&(a.put("status","close"),JU.Logger.info("allowAudio is set false"),this.notifyAudioStatus(a))},
"java.util.Map");c(c$,"notifyAudioStatus",function(a){var b=a.get("status");Zystem.out.println(b);var c=a.get(b);null!=c&&this.vwr.script(c);"ended"===b&&this.registerAudio(a.get("id"),null);b=this.jmolScriptCallback(J.c.CBK.AUDIO);this.notifyEnabled(J.c.CBK.AUDIO)&&this.cbl.notifyCallback(J.c.CBK.AUDIO,u(-1,[b,a]))},"java.util.Map");c(c$,"syncScript",function(a,b,c){if("GET_GRAPHICS".equalsIgnoreCase(a))this.setSyncDriver(5),this.syncSend(a,b,0),this.vwr.setBooleanProperty("_syncMouse",!1),this.vwr.setBooleanProperty("_syncScript",
!1);else{"=".equals(b)&&(b="~",this.setSyncDriver(2));var g="~".equals(b);if(0<c||!g&&!".".equals(b))if(this.syncSend(a,b,c),!"*".equals(b)||a.startsWith("{"))return;if(a.equalsIgnoreCase("on")||a.equalsIgnoreCase("true"))this.setSyncDriver(1);else if(a.equalsIgnoreCase("off")||a.equalsIgnoreCase("false"))this.setSyncDriver(0);else if(a.equalsIgnoreCase("slave"))this.setSyncDriver(2);else if(b=this.getSyncMode(),0!=b)if(1!=b&&(g=!1),JU.Logger.debugging&&JU.Logger.debug(this.vwr.htmlName+" syncing with script: "+
a),g&&this.setSyncDriver(3),0!=a.indexOf("Mouse: ")){b=JV.JC.getServiceCommand(a);switch(b){case 70:case 42:case 49:case 56:case 63:this.syncSend(a,".",c);return;case 0:case 77:case 28:case 35:if(g)return;case 21:case 7:case 14:if(null==(a=this.vwr.getJSV().processSync(a,b)))return}this.vwr.evalStringQuietSync(a,!0,!1)}else this.mouseScript(a),g&&this.vwr.setSyncDriver(4)}},"~S,~S,~N");c(c$,"mouseScript",function(a){var b=JU.PT.getTokens(a),c=b[1];try{switch(c=(c.toLowerCase()+"...............").substring(0,
15),"zoombyfactor...zoomby.........rotatezby......rotatexyby.....translatexyby..rotatemolecule.spinxyby.......rotatearcball..".indexOf(c)){case 0:switch(b.length){case 3:this.vwr.zoomByFactor(JU.PT.parseFloat(b[2]),2147483647,2147483647);return;case 5:this.vwr.zoomByFactor(JU.PT.parseFloat(b[2]),JU.PT.parseInt(b[3]),JU.PT.parseInt(b[4]));return}break;case 15:switch(b.length){case 3:this.vwr.zoomBy(JU.PT.parseInt(b[2]));return}break;case 30:switch(b.length){case 3:this.vwr.rotateZBy(JU.PT.parseInt(b[2]),
2147483647,2147483647);return;case 5:this.vwr.rotateZBy(JU.PT.parseInt(b[2]),JU.PT.parseInt(b[3]),JU.PT.parseInt(b[4]))}break;case 45:this.vwr.rotateXYBy(JU.PT.parseFloat(b[2]),JU.PT.parseFloat(b[3]));return;case 60:this.vwr.translateXYBy(JU.PT.parseInt(b[2]),JU.PT.parseInt(b[3]));return;case 75:this.vwr.rotateSelected(JU.PT.parseFloat(b[2]),JU.PT.parseFloat(b[3]),null);return;case 90:this.vwr.spinXYBy(JU.PT.parseInt(b[2]),JU.PT.parseInt(b[3]),JU.PT.parseFloat(b[4]));return;case 105:this.vwr.rotateXYBy(JU.PT.parseInt(b[2]),
JU.PT.parseInt(b[3]));return}}catch(g){if(!H(g,Exception))throw g;}this.vwr.showString("error reading SYNC command: "+a,!1)},"~S");E(c$,"MAXIMUM_QUEUE_LENGTH",16,"SYNC_OFF",0,"SYNC_DRIVER",1,"SYNC_SLAVE",2,"SYNC_DISABLE",3,"SYNC_ENABLE",4,"SYNC_STEREO",5)});s("JV");v("JU.A4 $.M3 $.M4 $.P3 $.P3i $.V3 J.c.STER JU.Point3fi".split(" "),"JV.TransformManager","java.lang.Boolean $.Float java.util.Hashtable JU.P4 $.Quat $.SB J.api.Interface JU.Escape $.Logger".split(" "),function(){c$=t(function(){this.spinThread=
this.vibrationThread=this.movetoThread=this.vwr=null;this.perspectiveModel=11;this.modelRadiusPixels=this.modelRadius=this.modelCenterOffset=this.cameraDistanceFromCenter=this.aperatureAngle=this.referencePlaneOffset=this.cameraScaleFactor=0;this.navigationShiftXY=this.navigationOffset=this.navigationCenter=null;this.navigationDepthPercent=0;this.vectorTemp=this.matrixTemp=null;this.haveNotifiedNaN=!1;this.spinX=0;this.spinY=30;this.spinZ=0;this.spinFps=30;this.navZ=this.navY=this.navX=0;this.navFps=
NaN;this.doTransform4D=this.isSpinSelected=this.isSpinFixed=this.isSpinInternal=!1;this.rotationCenterDefault=this.perspectiveShiftXY=this.perspectiveOffset=this.fixedRotationCenter=this.fixedRotationOffset=null;this.rotationRadiusDefault=0;this.internalRotationCenter=this.internalTranslation=this.internalRotationAxis=this.fixedRotationAxis=null;this.internalRotationAngle=0;this.rotationAxis=this.pointT2=this.vectorT2=this.vectorT=this.axisangleT=this.matrixTemp4=this.matrixTemp3=this.matrixRotate=
null;this.rotationRate=0;this.cameraSetting=this.camera=this.fixedTranslation=null;this.yTranslationFraction=this.xTranslationFraction=0.5;this.previousY=this.previousX=this.prevZoomSetting=0;this.zoomEnabled=!0;this.zmPctSet=this.zmPct=100;this.zoomRatio=0;this.internalSlab=this.zShadeEnabled=this.slabEnabled=!1;this.depthValue=this.slabValue=this.depthPercentSetting=this.slabPercentSetting=0;this.zSlabPercentSetting=50;this.zDepthPercentSetting=0;this.zSlabPoint=null;this.slabRange=this.zDepthValue=
this.zSlabValue=0;this.depthPlane=this.slabPlane=null;this.perspectiveDepth=!0;this.scale3D=!1;this.cameraDepthSetting=this.cameraDepth=3;this.visualRangeAngstroms=0;this.cameraDistance=1E3;this.scale3DAngstromsPerInch=this.scaleDefaultPixelsPerAngstrom=this.scalePixelsPerAngstrom=this.screenPixelCount=this.height=this.width=0;this.zoomHeight=this.useZoomLarge=this.antialias=!1;this.screenHeight=this.screenWidth=0;this.ptVibTemp=this.iScrPt=this.fScrPt=this.matrixTransformInv=this.matrixTransform=
null;this.navigating=!1;this.defaultMode=this.mode=0;this.matrixTest=this.aaTest1=this.ptTest3=this.ptTest2=this.ptTest1=this.untransformedPoint=null;this.vibrationOn=this.spinIsGesture=this.navOn=this.spinOn=!1;this.vibrationScale=this.vibrationPeriodMs=this.vibrationPeriod=0;this.stereoColors=this.stereoMode=this.vibrationT=null;this.stereoDoubleFull=this.stereoDoubleDTI=!1;this.stereoDegrees=NaN;this.stereoRadians=0;this.stereoFrame=!1;this.matrixStereo=null;this.windowCentered=!1;this.ptOffset=
this.bsSelectedAtoms=this.bsFrameOffsets=this.frameOffsets=this.frameOffset=null;this.navMode=1;this.zoomFactor=3.4028235E38;this.navigationSlabOffset=0;this.nav=null;p(this,arguments)},JV,"TransformManager");O(c$,function(){this.navigationCenter=new JU.P3;this.navigationOffset=new JU.P3;this.navigationShiftXY=new JU.P3;this.matrixTemp=new JU.M4;this.vectorTemp=new JU.V3;this.fixedRotationOffset=new JU.P3;this.fixedRotationCenter=new JU.P3;this.perspectiveOffset=new JU.P3;this.perspectiveShiftXY=
new JU.P3;this.rotationCenterDefault=new JU.P3;this.fixedRotationAxis=new JU.A4;this.internalRotationAxis=new JU.A4;this.internalRotationCenter=JU.P3.new3(0,0,0);this.matrixRotate=new JU.M3;this.matrixTemp3=new JU.M3;this.matrixTemp4=new JU.M4;this.axisangleT=new JU.A4;this.vectorT=new JU.V3;this.vectorT2=new JU.V3;this.pointT2=new JU.P3;this.rotationAxis=new JU.V3;this.fixedTranslation=new JU.P3;this.camera=new JU.P3;this.cameraSetting=new JU.P3;this.matrixTransform=new JU.M4;this.matrixTransformInv=
new JU.M4;this.fScrPt=new JU.P3;this.iScrPt=new JU.P3i;this.ptVibTemp=new JU.Point3fi;this.untransformedPoint=new JU.P3;this.ptTest1=new JU.P3;this.ptTest2=new JU.P3;this.ptTest3=new JU.P3;this.aaTest1=new JU.A4;this.matrixTest=new JU.M3;this.vibrationT=new JU.P3;this.stereoMode=J.c.STER.NONE;this.matrixStereo=new JU.M3;this.frameOffset=new JU.P3;this.ptOffset=new JU.P3});q(c$,function(){});c$.getTransformManager=c(c$,"getTransformManager",function(a,b,c,g){g=g?J.api.Interface.getInterface("JV.TransformManager4D",
a,"tm"):new JV.TransformManager;g.vwr=a;g.setScreenParameters(b,c,!0,!1,!0,!0);return g},"JV.Viewer,~N,~N,~B");c(c$,"setDefaultPerspective",function(){this.setCameraDepthPercent(3,!0);this.setPerspectiveDepth(!0);this.setStereoDegrees(-5);this.visualRangeAngstroms=5;this.setSpinOff();this.setVibrationPeriod(0)});c(c$,"homePosition",function(a){a&&this.setSpinOff();this.setNavOn(!1);this.navFps=10;this.navX=this.navY=this.navZ=0;this.rotationCenterDefault.setT(this.vwr.getBoundBoxCenter());this.setFixedRotationCenter(this.rotationCenterDefault);
this.rotationRadiusDefault=this.setRotationRadius(0,!0);this.windowCentered=!0;this.setRotationCenterAndRadiusXYZ(null,!0);this.resetRotation();a=this.vwr.ms.getInfoM("defaultOrientationMatrix");null!=a&&this.setRotation(a);this.setZoomEnabled(!0);this.zoomToPercent(this.vwr.g.modelKitMode?50:100);this.zmPct=this.zmPctSet;this.slabReset();this.resetFitToScreen(!0);this.vwr.isJmolDataFrame()?this.fixedRotationCenter.set(0,0,0):this.vwr.g.axesOrientationRasmol&&this.matrixRotate.setAsXRotation(3.141592653589793);
this.vwr.stm.saveOrientation("default",null);1==this.mode&&this.setNavigationMode(!0)},"~B");c(c$,"setRotation",function(a){a.isRotation()?this.matrixRotate.setM3(a):this.resetRotation()},"JU.M3");c(c$,"resetRotation",function(){this.matrixRotate.setScale(1)});c(c$,"clearThreads",function(){this.clearVibration();this.clearSpin();this.setNavOn(!1);this.stopMotion()});c(c$,"clear",function(){this.fixedRotationCenter.set(0,0,0);this.navigating=!1;this.zSlabPoint=this.depthPlane=this.slabPlane=null;this.resetNavigationPoint(!0)});
c(c$,"setFixedRotationCenter",function(a){null!=a&&this.fixedRotationCenter.setT(a)},"JU.T3");c(c$,"setRotationPointXY",function(a){a=this.transformPt(a);this.fixedTranslation.set(a.x,a.y,0)},"JU.P3");c(c$,"spinXYBy",function(a,b,c){if(0==a&&0==b)null!=this.spinThread&&this.spinIsGesture&&this.clearSpin();else{this.clearSpin();var g=JU.P3.newP(this.fixedRotationCenter),f=new JU.P3;this.transformPt3f(g,f);a=JU.P3.new3(-b,a,0);a.add(f);this.unTransformPoint(a,a);this.vwr.setInMotion(!1);this.rotateAboutPointsInternal(null,
a,g,10*c,NaN,!1,!0,null,!0,null,null,null,null)}},"~N,~N,~N");c(c$,"rotateXYBy",function(a,b,c){this.rotate3DBall(a,b,c)},"~N,~N,JU.BS");c(c$,"rotateZBy",function(a,b,c){2147483647!=b&&2147483647!=c&&this.resetXYCenter(b,c);this.rotateZRadians(a/57.29577951308232)},"~N,~N,~N");c(c$,"applyRotation",function(a,b,c,g,f,e){null==c?this.matrixRotate.mul2(a,this.matrixRotate):(this.vwr.moveAtoms(e,a,this.matrixRotate,g,this.internalRotationCenter,b,c,f),null!=g&&this.internalRotationCenter.add(g))},"JU.M3,~B,JU.BS,JU.V3,~B,JU.M4");
c(c$,"rotate3DBall",function(a,b,c){this.matrixTemp3.setAsBallRotation(0.017453292,-b,-a)&&this.applyRotation(this.matrixTemp3,!1,c,null,!1,null)},"~N,~N,JU.BS");c(c$,"rotateXRadians",function(a,b){this.applyRotation(this.matrixTemp3.setAsXRotation(a),!1,b,null,!1,null)},"~N,JU.BS");c(c$,"rotateYRadians",function(a,b){this.applyRotation(this.matrixTemp3.setAsYRotation(a),!1,b,null,!1,null)},"~N,JU.BS");c(c$,"rotateZRadians",function(a){this.applyRotation(this.matrixTemp3.setAsZRotation(a),!1,null,
null,!1,null)},"~N");c(c$,"rotateAxisAngle",function(a,b){this.axisangleT.setVA(a,b);this.rotateAxisAngle2(this.axisangleT,null)},"JU.V3,~N");c(c$,"rotateAxisAngle2",function(a,b){this.applyRotation(this.matrixTemp3.setAA(a),!1,b,null,!1,null)},"JU.A4,JU.BS");c(c$,"rotateAxisAngleAtCenter",function(a,b,c,g,f,e,h){null!=b&&this.moveRotationCenter(b,!0);e&&this.setSpinOff();this.setNavOn(!1);if(this.vwr.headless){if(e&&3.4028235E38==f)return!1;e=!1}if(Float.isNaN(g)||0==g||0==f)return!1;null!=b&&this.setRotationPointXY(b);
this.setFixedRotationCenter(b);this.rotationAxis.setT(c);this.rotationRate=g;if(e)return this.fixedRotationAxis.setVA(c,0.017453292*g),this.isSpinInternal=!1,this.isSpinFixed=!0,this.isSpinSelected=null!=h,this.setSpin(a,!0,f,null,null,h,!1),3.4028235E38!=f;a=0.017453292*f;this.fixedRotationAxis.setVA(c,f);this.rotateAxisAngleRadiansFixed(a,h);return!0},"J.api.JmolScriptEvaluator,JU.P3,JU.V3,~N,~N,~B,JU.BS");c(c$,"rotateAxisAngleRadiansFixed",function(a,b){this.axisangleT.setAA(this.fixedRotationAxis);
this.axisangleT.angle=a;this.rotateAxisAngle2(this.axisangleT,b)},"~N,JU.BS");c(c$,"rotateAboutPointsInternal",function(a,b,c,g,f,e,h,j,l,n,m,r,p){h&&this.setSpinOff();this.setNavOn(!1);if(null==r&&(null==n||0.001>n.length())&&(h?Float.isNaN(g)||0==g:0==f))return!1;var q=null;null==r&&(q=JU.V3.newVsub(c,b),e&&q.scale(-1),this.internalRotationCenter.setT(b),this.rotationAxis.setT(q),this.internalTranslation=null==n?null:JU.V3.newV(n));b=null!=j;if(h)return null==r?(0==f&&(f=NaN),Float.isNaN(f)?this.rotationRate=
g:(p=w(Math.abs(f)/Math.abs(g)*this.spinFps+0.5),this.rotationRate=f/p*this.spinFps,null!=n&&this.internalTranslation.scale(1/p)),this.internalRotationAxis.setVA(q,0.017453292*(Float.isNaN(this.rotationRate)?0:this.rotationRate)),this.isSpinInternal=!0,this.isSpinFixed=!1,this.isSpinSelected=b):f=g,this.setSpin(a,!0,f,m,r,j,l),!Float.isNaN(f);a=0.017453292*f;this.internalRotationAxis.setVA(q,a);this.rotateAxisAngleRadiansInternal(a,j,p);return!1},"J.api.JmolScriptEvaluator,JU.T3,JU.T3,~N,~N,~B,~B,JU.BS,~B,JU.V3,JU.Lst,~A,JU.M4");
c(c$,"rotateAxisAngleRadiansInternal",function(a,b,c){this.internalRotationAngle=a;this.vectorT.set(this.internalRotationAxis.x,this.internalRotationAxis.y,this.internalRotationAxis.z);this.matrixRotate.rotate2(this.vectorT,this.vectorT2);this.axisangleT.setVA(this.vectorT2,a);this.applyRotation(this.matrixTemp3.setAA(this.axisangleT),!0,b,this.internalTranslation,1E6<a,c);null==b&&this.getNewFixedRotationCenter()},"~N,JU.BS,JU.M4");c(c$,"getNewFixedRotationCenter",function(){this.axisangleT.setAA(this.internalRotationAxis);
this.axisangleT.angle=-this.internalRotationAngle;this.matrixTemp4.setToAA(this.axisangleT);this.vectorT.setT(this.internalRotationCenter);this.pointT2.sub2(this.fixedRotationCenter,this.vectorT);var a=this.matrixTemp4.rotTrans2(this.pointT2,new JU.P3);a.add(this.vectorT);this.setRotationCenterAndRadiusXYZ(a,!1)});c(c$,"setTranslationFractions",function(){this.xTranslationFraction=this.fixedTranslation.x/this.width;this.yTranslationFraction=this.fixedTranslation.y/this.height});c(c$,"centerAt",function(a,
b,c){null==c?this.translateXYBy(a,b):(this.windowCentered&&this.vwr.setBooleanProperty("windowCentered",!1),this.fixedTranslation.x=a,this.fixedTranslation.y=b,this.setFixedRotationCenter(c))},"~N,~N,JU.P3");c(c$,"percentToPixels",function(a,b){switch(a){case "x":return w(Math.floor(b/100*this.width));case "y":return w(Math.floor(b/100*this.height));case "z":return w(Math.floor(b/100*this.screenPixelCount))}return 0},"~S,~N");c(c$,"angstromsToPixels",function(a){return w(Math.floor(this.scalePixelsPerAngstrom*
a))},"~N");c(c$,"translateXYBy",function(a,b){this.fixedTranslation.x+=a;this.fixedTranslation.y+=b;this.setTranslationFractions()},"~N,~N");c(c$,"setCamera",function(a,b){this.cameraSetting.set(a,b,0==a&&0==b?0:1)},"~N,~N");c(c$,"translateToPercent",function(a,b){switch(a){case "x":this.xTranslationFraction=0.5+b/100;this.fixedTranslation.x=this.width*this.xTranslationFraction;break;case "y":this.yTranslationFraction=0.5+b/100;this.fixedTranslation.y=this.height*this.yTranslationFraction;break;case "z":1==
this.mode&&this.setNavigationDepthPercent(b)}},"~S,~N");c(c$,"getTranslationXPercent",function(){return 0==this.width?0:100*(this.fixedTranslation.x-this.width/2)/this.width});c(c$,"getTranslationYPercent",function(){return 0==this.height?0:100*(this.fixedTranslation.y-this.height/2)/this.height});c(c$,"getTranslationScript",function(){var a="",b=this.getTranslationXPercent();0!=b&&(a+="translate x "+b+";");b=this.getTranslationYPercent();0!=b&&(a+="translate y "+b+";");return a});c(c$,"getOrientationText",
function(a,b){switch(a){case 4129:return this.getMoveToText(1,!1);case 1073742132:var c=this.getRotationQ();b&&(c=c.inv());return c.toString();case 1073742178:var c=new JU.SB,g=this.getTranslationXPercent();JV.TransformManager.truncate2(c,b?-g:g);g=this.getTranslationYPercent();JV.TransformManager.truncate2(c,b?-g:g);return c.toString();default:return this.getMoveToText(1,!0)+"\n#OR\n"+this.getRotateZyzText(!0)}},"~N,~B");c(c$,"getRotationQ",function(){return JU.Quat.newM(this.matrixRotate)});c(c$,
"getOrientationInfo",function(){var a=new java.util.Hashtable;a.put("moveTo",this.getMoveToText(1,!1));a.put("center","center "+this.getCenterText());a.put("centerPt",this.fixedRotationCenter);var b=new JU.A4;b.setM(this.matrixRotate);a.put("axisAngle",b);a.put("quaternion",this.getRotationQ().toPoint4f());a.put("rotationMatrix",this.matrixRotate);a.put("rotateZYZ",this.getRotateZyzText(!1));a.put("rotateXYZ",this.getRotateXyzText());a.put("transXPercent",Float.$valueOf(this.getTranslationXPercent()));
a.put("transYPercent",Float.$valueOf(this.getTranslationYPercent()));a.put("zoom",Float.$valueOf(this.zmPct));a.put("modelRadius",Float.$valueOf(this.modelRadius));1==this.mode&&(a.put("navigationCenter","navigate center "+JU.Escape.eP(this.navigationCenter)),a.put("navigationOffsetXPercent",Float.$valueOf(this.getNavigationOffsetPercent("X"))),a.put("navigationOffsetYPercent",Float.$valueOf(this.getNavigationOffsetPercent("Y"))),a.put("navigationDepthPercent",Float.$valueOf(this.navigationDepthPercent)));
return a});c(c$,"getRotation",function(a){a.setM3(this.matrixRotate)},"JU.M3");c(c$,"setZoomHeight",function(a,b){this.zoomHeight=a;this.scaleFitToScreen(!1,b,!1,!0)},"~B,~B");c(c$,"zoomBy",function(a){20<a?a=20:-20>a&&(a=-20);var b=a*this.zmPctSet/50;0==b&&(b=0<a?1:0>b?-1:0);this.zoomRatio=(b+this.zmPctSet)/this.zmPctSet;this.zmPctSet+=b},"~N");c(c$,"zoomByFactor",function(a,b,c){!(0>=a)&&this.zoomEnabled&&(1!=this.mode?(this.zoomRatio=a,this.zmPctSet*=a,this.resetXYCenter(b,c)):this.getNav()&&this.nav.zoomByFactor(a,
b,c))},"~N,~N,~N");c(c$,"zoomToPercent",function(a){this.zmPctSet=a;this.zoomRatio=0},"~N");c(c$,"translateZBy",function(a){a>=this.screenPixelCount||(a=this.scalePixelsPerAngstrom/(1-1*a/this.screenPixelCount),a>=this.screenPixelCount||(a=100*(a/this.scaleDefaultPixelsPerAngstrom),this.zoomRatio=a/this.zmPctSet,this.zmPctSet=a))},"~N");c(c$,"resetXYCenter",function(a,b){if(!(2147483647==a||2147483647==b)){this.windowCentered&&this.vwr.setBooleanProperty("windowCentered",!1);var c=new JU.P3;this.transformPt3f(this.fixedRotationCenter,
c);c.set(a,b,c.z);this.unTransformPoint(c,c);this.fixedTranslation.set(a,b,0);this.setFixedRotationCenter(c)}},"~N,~N");c(c$,"zoomByPercent",function(a){var b=a*this.zmPctSet/100;0==b&&(b=0>a?-1:1);this.zoomRatio=(b+this.zmPctSet)/this.zmPctSet;this.zmPctSet+=b},"~N");c(c$,"setScaleAngstromsPerInch",function(a){if(this.scale3D=0<a)this.scale3DAngstromsPerInch=a;this.perspectiveDepth=!this.scale3D},"~N");c(c$,"setSlabRange",function(a){this.slabRange=a},"~N");c(c$,"setSlabEnabled",function(a){this.vwr.g.setB("slabEnabled",
this.slabEnabled=a)},"~B");c(c$,"setZShadeEnabled",function(a){this.zShadeEnabled=a;this.vwr.g.setB("zShade",a)},"~B");c(c$,"setZoomEnabled",function(a){this.zoomEnabled=a;this.vwr.g.setB("zoomEnabled",a)},"~B");c(c$,"slabReset",function(){this.slabToPercent(100);this.depthToPercent(0);this.slabPlane=this.depthPlane=null;this.setSlabEnabled(!1);this.setZShadeEnabled(!1);this.slabDepthChanged()});c(c$,"getSlabPercentSetting",function(){return this.slabPercentSetting});c(c$,"slabDepthChanged",function(){this.vwr.g.setI("slab",
this.slabPercentSetting);this.vwr.g.setI("depth",this.depthPercentSetting);this.finalizeTransformParameters()});c(c$,"slabByPercentagePoints",function(a){this.slabPlane=null;if(!(0>a?this.slabPercentSetting<=Math.max(0,this.depthPercentSetting):100<=this.slabPercentSetting))this.slabPercentSetting+=a,this.slabDepthChanged(),this.depthPercentSetting>=this.slabPercentSetting&&(this.depthPercentSetting=this.slabPercentSetting-1)},"~N");c(c$,"depthByPercentagePoints",function(a){this.depthPlane=null;
if(!(0>a?0>=this.depthPercentSetting:this.depthPercentSetting>=Math.min(100,this.slabPercentSetting)))this.depthPercentSetting+=a,this.slabPercentSetting<=this.depthPercentSetting&&(this.slabPercentSetting=this.depthPercentSetting+1),this.slabDepthChanged()},"~N");c(c$,"slabDepthByPercentagePoints",function(a){this.depthPlane=this.slabPlane=null;if(!(0>a?this.slabPercentSetting<=Math.max(0,this.depthPercentSetting):this.depthPercentSetting>=Math.min(100,this.slabPercentSetting)))this.slabPercentSetting+=
a,this.depthPercentSetting+=a,this.slabDepthChanged()},"~N");c(c$,"slabToPercent",function(a){this.slabPlane=null;this.vwr.setFloatProperty("slabRange",0);this.slabPercentSetting=a;this.depthPercentSetting>=this.slabPercentSetting&&(this.depthPercentSetting=this.slabPercentSetting-1);this.slabDepthChanged()},"~N");c(c$,"depthToPercent",function(a){this.depthPlane=null;this.vwr.g.setI("depth",a);this.depthPercentSetting=a;this.slabPercentSetting<=this.depthPercentSetting&&(this.slabPercentSetting=
this.depthPercentSetting+1);this.slabDepthChanged()},"~N");c(c$,"zSlabToPercent",function(a){this.zSlabPercentSetting=a;this.zDepthPercentSetting>this.zSlabPercentSetting&&(this.zDepthPercentSetting=a)},"~N");c(c$,"zDepthToPercent",function(a){this.zDepthPercentSetting=a;this.zDepthPercentSetting>this.zSlabPercentSetting&&(this.zSlabPercentSetting=a)},"~N");c(c$,"slabInternal",function(a,b){b?(this.depthPlane=a,this.depthPercentSetting=0):(this.slabPlane=a,this.slabPercentSetting=100);this.slabDepthChanged()},
"JU.P4,~B");c(c$,"setSlabDepthInternal",function(a){a?this.depthPlane=null:this.slabPlane=null;this.finalizeTransformParameters();this.slabInternal(this.getSlabDepthPlane(a),a)},"~B");c(c$,"getSlabDepthPlane",function(a){if(a){if(null!=this.depthPlane)return this.depthPlane}else if(null!=this.slabPlane)return this.slabPlane;var b=this.matrixTransform;return JU.P4.new4(-b.m20,-b.m21,-b.m22,-b.m23+(a?this.depthValue:this.slabValue))},"~B");c(c$,"getCameraFactors",function(){this.aperatureAngle=360*
Math.atan2(this.screenPixelCount/2,this.referencePlaneOffset)/3.141592653589793;this.cameraDistanceFromCenter=this.referencePlaneOffset/this.scalePixelsPerAngstrom;var a=JU.P3.new3(w(this.screenWidth/2),w(this.screenHeight/2),this.referencePlaneOffset);this.unTransformPoint(a,a);var b=JU.P3.new3(w(this.screenWidth/2),w(this.screenHeight/2),0);this.unTransformPoint(b,b);b.sub(this.fixedRotationCenter);var c=JU.P3.new3(w(this.screenWidth/2),w(this.screenHeight/2),this.cameraDistanceFromCenter*this.scalePixelsPerAngstrom);
this.unTransformPoint(c,c);c.sub(this.fixedRotationCenter);b.add(c);return u(-1,[a,b,this.fixedRotationCenter,JU.P3.new3(this.cameraDistanceFromCenter,this.aperatureAngle,this.scalePixelsPerAngstrom)])});c(c$,"setPerspectiveDepth",function(a){this.perspectiveDepth!=a&&(this.perspectiveDepth=a,this.vwr.g.setB("perspectiveDepth",a),this.resetFitToScreen(!1))},"~B");c(c$,"getPerspectiveDepth",function(){return this.perspectiveDepth});c(c$,"setCameraDepthPercent",function(a,b){this.resetNavigationPoint(b);
var c=0>a?-a/100:a;0!=c&&(this.cameraDepthSetting=c,this.vwr.g.setF("cameraDepth",this.cameraDepthSetting),this.cameraDepth=NaN)},"~N,~B");c(c$,"getCameraDepth",function(){return this.cameraDepthSetting});c(c$,"setScreenParameters0",function(a,b,c,g,f,e){2147483647!=a&&(this.screenWidth=a,this.screenHeight=b,this.useZoomLarge=c,this.width=(this.antialias=g)?2*a:a,this.height=g?2*b:b,this.scaleFitToScreen(!1,c,f,e))},"~N,~N,~B,~B,~B,~B");c(c$,"setAntialias",function(a){var b=this.antialias!=a;this.width=
(this.antialias=a)?2*this.screenWidth:this.screenWidth;this.height=this.antialias?2*this.screenHeight:this.screenHeight;b&&this.scaleFitToScreen(!1,this.useZoomLarge,!1,!1)},"~B");c(c$,"defaultScaleToScreen",function(a){return this.screenPixelCount/2/a},"~N");c(c$,"resetFitToScreen",function(a){this.scaleFitToScreen(a,this.vwr.g.zoomLarge,!0,!0)},"~B");c(c$,"scaleFitToScreen",function(a,b,c,g){0==this.width||0==this.height?this.screenPixelCount=1:(this.fixedTranslation.set(this.width*(a?0.5:this.xTranslationFraction),
this.height*(a?0.5:this.yTranslationFraction),0),this.setTranslationFractions(),a&&this.camera.set(0,0,0),g&&this.resetNavigationPoint(c),this.zoomHeight&&(b=this.height>this.width),this.screenPixelCount=b==this.height>this.width?this.height:this.width);2<this.screenPixelCount&&(this.screenPixelCount-=2);this.scaleDefaultPixelsPerAngstrom=this.defaultScaleToScreen(this.modelRadius)},"~B,~B,~B,~B");c(c$,"scaleToScreen",function(a,b){if(0==b||2>a)return 0;var c=this.scaleToPerspective(a,b*this.scalePixelsPerAngstrom/
1E3);return 0<c?c:1},"~N,~N");c(c$,"unscaleToScreen",function(a,b){var c=b/this.scalePixelsPerAngstrom;return this.perspectiveDepth?c/this.getPerspectiveFactor(a):c},"~N,~N");c(c$,"scaleToPerspective",function(a,b){return this.perspectiveDepth?b*this.getPerspectiveFactor(a):b},"~N,~N");c(c$,"setNavigationMode",function(a){this.mode=a?1:this.defaultMode;this.resetNavigationPoint(!0)},"~B");c(c$,"isNavigating",function(){return this.navigating||this.navOn});c(c$,"finalizeTransformParameters",function(){this.haveNotifiedNaN=
!1;this.fixedRotationOffset.setT(this.fixedTranslation);this.camera.setT(this.cameraSetting);this.internalSlab=this.slabEnabled&&(null!=this.slabPlane||null!=this.depthPlane);var a=this.getZoomSetting();this.zmPct!=a&&(this.zmPct=a,this.vwr.g.fontCaching||this.vwr.gdata.clearFontCache());this.calcCameraFactors();this.calcTransformMatrix();1==this.mode?this.calcNavigationPoint():this.calcSlabAndDepthValues()});c(c$,"getZoomSetting",function(){5>this.zmPctSet&&(this.zmPctSet=5);2E5<this.zmPctSet&&(this.zmPctSet=
2E5);return this.zoomEnabled||1==this.mode?this.zmPctSet:100});c(c$,"calcSlabAndDepthValues",function(){this.slabValue=1>this.slabRange?this.zValueFromPercent(this.slabPercentSetting):w(Math.floor(this.modelCenterOffset*this.slabRange/(2*this.modelRadius)*(this.zmPctSet/100)));this.depthValue=this.zValueFromPercent(this.depthPercentSetting);this.zSlabPercentSetting==this.zDepthPercentSetting?(this.zSlabValue=this.slabValue,this.zDepthValue=this.depthValue):(this.zSlabValue=this.zValueFromPercent(this.zSlabPercentSetting),
this.zDepthValue=this.zValueFromPercent(this.zDepthPercentSetting));if(null!=this.zSlabPoint)try{this.transformPt3f(this.zSlabPoint,this.pointT2),this.zSlabValue=A(this.pointT2.z)}catch(a){if(!H(a,Exception))throw a;}this.vwr.g.setO("_slabPlane",JU.Escape.eP4(this.getSlabDepthPlane(!1)));this.vwr.g.setO("_depthPlane",JU.Escape.eP4(this.getSlabDepthPlane(!0)));this.slabEnabled||(this.slabValue=0,this.depthValue=2147483647)});c(c$,"zValueFromPercent",function(a){return w(Math.floor((1-a/50)*this.modelRadiusPixels+
this.modelCenterOffset))},"~N");c(c$,"calcTransformMatrix",function(){this.matrixTransform.setIdentity();this.vectorTemp.sub2(this.frameOffset,this.fixedRotationCenter);this.matrixTransform.setTranslation(this.vectorTemp);this.matrixTemp.setToM3(this.stereoFrame?this.matrixStereo:this.matrixRotate);this.matrixTransform.mul2(this.matrixTemp,this.matrixTransform);this.matrixTemp.setIdentity();this.matrixTemp.m00=this.matrixTemp.m11=this.matrixTemp.m22=this.scalePixelsPerAngstrom;this.matrixTemp.m11=
this.matrixTemp.m22=-this.scalePixelsPerAngstrom;this.matrixTransform.mul2(this.matrixTemp,this.matrixTransform);this.matrixTransform.m23+=this.modelCenterOffset;try{this.matrixTransformInv.setM4(this.matrixTransform).invert()}catch(a){if(H(a,Exception))Zystem.out.println("ERROR INVERTING matrixTransform!");else throw a;}});c(c$,"rotatePoint",function(a,b){this.matrixRotate.rotate2(a,b);b.y=-b.y},"JU.T3,JU.T3");c(c$,"getScreenTemp",function(a){this.matrixTransform.rotTrans2(a,this.fScrPt)},"JU.T3");
c(c$,"transformPtScr",function(a,b){b.setT(this.transformPt(a))},"JU.T3,JU.P3i");c(c$,"transformPtScrT3",function(a,b){this.transformPt(a);b.setT(this.fScrPt)},"JU.T3,JU.T3");c(c$,"transformPt3f",function(a,b){this.applyPerspective(a,a);b.setT(this.fScrPt)},"JU.T3,JU.P3");c(c$,"transformPtNoClip",function(a,b){this.applyPerspective(a,null);b.setT(this.fScrPt)},"JU.T3,JU.T3");c(c$,"transformPt",function(a){return this.applyPerspective(a,this.internalSlab?a:null)},"JU.T3");c(c$,"transformPtVib",function(a,
b){this.ptVibTemp.setT(a);return this.applyPerspective(this.getVibrationPoint(b,this.ptVibTemp,NaN),a)},"JU.P3,JU.Vibration");c(c$,"getVibrationPoint",function(a,b,c){return a.setCalcPoint(b,this.vibrationT,Float.isNaN(c)?this.vibrationScale:c,this.vwr.g.modulationScale)},"JU.Vibration,JU.T3,~N");c(c$,"transformPt2D",function(a){-3.4028235E38==a.z?(this.iScrPt.x=w(Math.floor(a.x/100*this.screenWidth)),this.iScrPt.y=w(Math.floor((1-a.y/100)*this.screenHeight))):(this.iScrPt.x=A(a.x),this.iScrPt.y=
this.screenHeight-A(a.y));this.antialias&&(this.iScrPt.x<<=1,this.iScrPt.y<<=1);this.matrixTransform.rotTrans2(this.fixedRotationCenter,this.fScrPt);this.iScrPt.z=A(this.fScrPt.z);return this.iScrPt},"JU.T3");c(c$,"applyPerspective",function(a,b){this.getScreenTemp(a);var c=this.fScrPt.z;Float.isNaN(c)?(!this.haveNotifiedNaN&&JU.Logger.debugging&&JU.Logger.debug("NaN seen in TransformPoint"),this.haveNotifiedNaN=!0,c=this.fScrPt.z=1):0>=c&&(c=this.fScrPt.z=1);switch(this.mode){case 1:this.fScrPt.x-=
this.navigationShiftXY.x;this.fScrPt.y-=this.navigationShiftXY.y;break;case 2:this.fScrPt.x+=this.perspectiveShiftXY.x,this.fScrPt.y+=this.perspectiveShiftXY.y}this.perspectiveDepth&&(c=this.getPerspectiveFactor(c),this.fScrPt.x*=c,this.fScrPt.y*=c);switch(this.mode){case 1:this.fScrPt.x+=this.navigationOffset.x;this.fScrPt.y+=this.navigationOffset.y;break;case 2:this.fScrPt.x-=this.perspectiveShiftXY.x,this.fScrPt.y-=this.perspectiveShiftXY.y;case 0:this.fScrPt.x+=this.fixedRotationOffset.x,this.fScrPt.y+=
this.fixedRotationOffset.y}Float.isNaN(this.fScrPt.x)&&!this.haveNotifiedNaN&&(JU.Logger.debugging&&JU.Logger.debug("NaN found in transformPoint "),this.haveNotifiedNaN=!0);this.iScrPt.set(A(this.fScrPt.x),A(this.fScrPt.y),A(this.fScrPt.z));null!=b&&this.xyzIsSlabbedInternal(b)&&(this.fScrPt.z=this.iScrPt.z=1);return this.iScrPt},"JU.T3,JU.T3");c(c$,"xyzIsSlabbedInternal",function(a){return null!=this.slabPlane&&0<a.x*this.slabPlane.x+a.y*this.slabPlane.y+a.z*this.slabPlane.z+this.slabPlane.w||null!=
this.depthPlane&&0>a.x*this.depthPlane.x+a.y*this.depthPlane.y+a.z*this.depthPlane.z+this.depthPlane.w},"JU.T3");c(c$,"move",function(a,b,c,g,f,e,h){this.movetoThread=J.api.Interface.getOption("thread.MoveToThread",this.vwr,"tm");this.movetoThread.setManager(this,this.vwr,u(-1,[b,g,K(-1,[c,f,e,h])]));0<e&&this.movetoThread.setEval(a);this.movetoThread.run()},"J.api.JmolScriptEvaluator,JU.V3,~N,JU.V3,~N,~N,~N");c(c$,"isInPosition",function(a,b){if(Float.isNaN(b))return!0;this.aaTest1.setVA(a,b/57.29577951308232);
this.ptTest1.set(4.321,1.23456,3.14159);this.getRotation(this.matrixTest);this.matrixTest.rotate2(this.ptTest1,this.ptTest2);this.matrixTest.setAA(this.aaTest1).rotate2(this.ptTest1,this.ptTest3);return 0.1>this.ptTest3.distance(this.ptTest2)},"JU.V3,~N");c(c$,"moveToPyMOL",function(a,b,c){var g=JU.M3.newA9(c);g.invert();var f=c[9],e=-c[10],h=-c[11],j=JU.P3.new3(c[12],c[13],c[14]),l=c[15],n=c[16],m=c[17];this.setPerspectiveDepth(!(0<=m));var m=Math.abs(m)/2,p=Math.tan(3.141592653589793*m/180),m=h*
p,p=0.5/p-0.5,q=50/m;0<l&&(l=50+A((h-l)*q),h=50+A((h-n)*q),this.setSlabEnabled(!0),this.slabToPercent(l),this.depthToPercent(h),21==c.length&&(n=0!=c[18],q=0!=c[19],c=c[20],this.setZShadeEnabled(n),n&&(q?this.vwr.setIntProperty("zSlab",A(Math.min(100,l+c*(h-l)))):this.vwr.setIntProperty("zSlab",A((l+h)/2)),this.vwr.setIntProperty("zDepth",h))));this.moveTo(a,b,j,null,0,g,100,NaN,NaN,m,null,NaN,NaN,NaN,p,f,e);return!0},"J.api.JmolScriptEvaluator,~N,~A");c(c$,"moveTo",function(a,b,c,g,f,e,h,j,l,n,m,
p,q,s,t,v,w){if(null==e)if(e=new JU.M3,g=JU.V3.newV(g),Float.isNaN(f))e.m00=NaN;else if(0.01>f&&-0.01<f)e.setScale(1);else{if(0==g.x&&0==g.y&&0==g.z)return;var A=new JU.A4;A.setVA(g,f/57.29577951308232);e.setAA(A)}v==this.cameraSetting.x&&(v=NaN);w==this.cameraSetting.y&&(w=NaN);t==this.cameraDepth&&(t=NaN);Float.isNaN(v)||(j=50*v/n/this.width*this.screenPixelCount);Float.isNaN(w)||(l=50*w/n/this.height*this.screenPixelCount);f=null==c?this.scaleDefaultPixelsPerAngstrom:this.defaultScaleToScreen(n);
if(0>=b)this.setAll(c,e,m,h,j,l,n,f,s,p,q,t,v,w),this.vwr.moveUpdate(b),this.vwr.finalizeTransformParameters();else try{null==this.movetoThread&&(this.movetoThread=J.api.Interface.getOption("thread.MoveToThread",this.vwr,"tm"));var B=this.movetoThread.setManager(this,this.vwr,u(-1,[c,e,m,K(-1,[b,h,j,l,n,f,s,p,q,t,v,w])]));0>=B||this.vwr.g.waitForMoveTo?(0<B&&this.movetoThread.setEval(a),this.movetoThread.run(),this.vwr.isSingleThreaded||(this.movetoThread=null)):this.movetoThread.start()}catch(D){if(!H(D,
Exception))throw D;}},"J.api.JmolScriptEvaluator,~N,JU.P3,JU.T3,~N,JU.M3,~N,~N,~N,~N,JU.P3,~N,~N,~N,~N,~N,~N");c(c$,"setAll",function(a,b,c,g,f,e,h,j,l,n,m,p,q,s){Float.isNaN(b.m00)||this.setRotation(b);null!=a&&this.moveRotationCenter(a,!this.windowCentered);null!=c&&1==this.mode&&this.navigationCenter.setT(c);Float.isNaN(p)||this.setCameraDepthPercent(p,!1);!Float.isNaN(q)&&!Float.isNaN(s)&&this.setCamera(q,s);Float.isNaN(g)||this.zoomToPercent(g);Float.isNaN(h)||(this.modelRadius=h);Float.isNaN(j)||
(this.scaleDefaultPixelsPerAngstrom=j);!Float.isNaN(f)&&!Float.isNaN(e)&&(this.translateToPercent("x",f),this.translateToPercent("y",e));1==this.mode&&(!Float.isNaN(n)&&!Float.isNaN(m)&&this.navTranslatePercentOrTo(0,n,m),Float.isNaN(l)||this.setNavigationDepthPercent(l))},"JU.P3,JU.M3,JU.P3,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N,~N");c(c$,"stopMotion",function(){this.movetoThread=null});c(c$,"getRotationText",function(){this.axisangleT.setM(this.matrixRotate);var a=57.29577951308232*this.axisangleT.angle,
b=new JU.SB;this.vectorT.set(this.axisangleT.x,this.axisangleT.y,this.axisangleT.z);if(0.01>a)return"{0 0 1 0}";this.vectorT.normalize();this.vectorT.scale(1E3);b.append("{");JV.TransformManager.truncate0(b,this.vectorT.x);JV.TransformManager.truncate0(b,this.vectorT.y);JV.TransformManager.truncate0(b,this.vectorT.z);JV.TransformManager.truncate2(b,a);b.append("}");return b.toString()});c(c$,"getMoveToText",function(a,b){this.finalizeTransformParameters();var c=new JU.SB;c.append("moveto ");b&&c.append("/* time, axisAngle */ ");
c.appendF(a);c.append(" ").append(this.getRotationText());b&&c.append(" /* zoom, translation */ ");JV.TransformManager.truncate2(c,this.zmPctSet);JV.TransformManager.truncate2(c,this.getTranslationXPercent());JV.TransformManager.truncate2(c,this.getTranslationYPercent());c.append(" ");b&&c.append(" /* center, rotationRadius */ ");c.append(this.getCenterText());c.append(" ").appendF(this.modelRadius);c.append(this.getNavigationText(b));b&&c.append(" /* cameraDepth, cameraX, cameraY */ ");JV.TransformManager.truncate2(c,
this.cameraDepth);JV.TransformManager.truncate2(c,this.cameraSetting.x);JV.TransformManager.truncate2(c,this.cameraSetting.y);c.append(";");return c.toString()},"~N,~B");c(c$,"getCenterText",function(){return JU.Escape.eP(this.fixedRotationCenter)});c(c$,"getRotateXyzText",function(){var a=new JU.SB,b=this.matrixRotate.m20,c=-(57.29577951308232*Math.asin(b)),g;0.999<b||-0.999>b?(b=-(57.29577951308232*Math.atan2(this.matrixRotate.m12,this.matrixRotate.m11)),g=0):(b=57.29577951308232*Math.atan2(this.matrixRotate.m21,
this.matrixRotate.m22),g=57.29577951308232*Math.atan2(this.matrixRotate.m10,this.matrixRotate.m00));a.append("reset");a.append(";center ").append(this.getCenterText());0!=b&&(a.append("; rotate x"),JV.TransformManager.truncate2(a,b));0!=c&&(a.append("; rotate y"),JV.TransformManager.truncate2(a,c));0!=g&&(a.append("; rotate z"),JV.TransformManager.truncate2(a,g));a.append(";");this.addZoomTranslationNavigationText(a);return a.toString()});c(c$,"addZoomTranslationNavigationText",function(a){100!=this.zmPct&&
(a.append(" zoom"),JV.TransformManager.truncate2(a,this.zmPct),a.append(";"));var b=this.getTranslationXPercent();0!=b&&(a.append(" translate x"),JV.TransformManager.truncate2(a,b),a.append(";"));b=this.getTranslationYPercent();0!=b&&(a.append(" translate y"),JV.TransformManager.truncate2(a,b),a.append(";"));if(this.modelRadius!=this.rotationRadiusDefault||10==this.modelRadius)a.append(" set rotationRadius"),JV.TransformManager.truncate2(a,this.modelRadius),a.append(";");1==this.mode&&(a.append("navigate 0 center ").append(JU.Escape.eP(this.navigationCenter)),
a.append(";navigate 0 translate"),JV.TransformManager.truncate2(a,this.getNavigationOffsetPercent("X")),JV.TransformManager.truncate2(a,this.getNavigationOffsetPercent("Y")),a.append(";navigate 0 depth "),JV.TransformManager.truncate2(a,this.navigationDepthPercent),a.append(";"))},"JU.SB");c(c$,"getRotateZyzText",function(a){var b=new JU.SB,c=this.vwr.ms.getInfoM("defaultOrientationMatrix");null==c?c=this.matrixRotate:(c=JU.M3.newM3(c),c.invert(),c.mul2(this.matrixRotate,c));var g=c.m22,f=57.29577951308232*
Math.acos(g);0.999<g||-0.999>g?(g=57.29577951308232*Math.atan2(c.m10,c.m11),c=0):(g=57.29577951308232*Math.atan2(c.m21,-c.m20),c=57.29577951308232*Math.atan2(c.m12,c.m02));0!=g&&(0!=f&&0!=c&&a)&&b.append("#Follows Z-Y-Z convention for Euler angles\n");b.append("reset");b.append(";center ").append(this.getCenterText());0!=g&&(b.append("; rotate z"),JV.TransformManager.truncate2(b,g));0!=f&&(b.append("; rotate y"),JV.TransformManager.truncate2(b,f));0!=c&&(b.append("; rotate z"),JV.TransformManager.truncate2(b,
c));b.append(";");this.addZoomTranslationNavigationText(b);return b.toString()},"~B");c$.truncate0=c(c$,"truncate0",function(a,b){a.appendC(" ");a.appendI(Math.round(b))},"JU.SB,~N");c$.truncate2=c(c$,"truncate2",function(a,b){a.appendC(" ");a.appendF(Math.round(100*b)/100)},"JU.SB,~N");c(c$,"setSpinXYZ",function(a,b,c){Float.isNaN(a)||(this.spinX=a);Float.isNaN(b)||(this.spinY=b);Float.isNaN(c)||(this.spinZ=c);(this.isSpinInternal||this.isSpinFixed)&&this.clearSpin()},"~N,~N,~N");c(c$,"setSpinFps",
function(a){0>=a?a=1:50<a&&(a=50);this.spinFps=a},"~N");c(c$,"setNavXYZ",function(a,b,c){Float.isNaN(a)||(this.navX=a);Float.isNaN(b)||(this.navY=b);Float.isNaN(c)||(this.navZ=c)},"~N,~N,~N");c(c$,"clearSpin",function(){this.setSpinOff();this.setNavOn(!1);this.isSpinFixed=this.isSpinInternal=!1});c(c$,"setSpinOn",function(){this.setSpin(null,!0,3.4028235E38,null,null,null,!1)});c(c$,"setSpinOff",function(){this.setSpin(null,!1,3.4028235E38,null,null,null,!1)});c(c$,"setSpin",function(a,b,c,g,f,e,
h){this.navOn&&b&&this.setNavOn(!1);this.spinOn!=b&&(this.spinOn=b,this.vwr.g.setB("_spinning",b),b?null==this.spinThread&&(this.spinThread=J.api.Interface.getOption("thread.SpinThread",this.vwr,"tm"),this.spinThread.setManager(this,this.vwr,u(-1,[Float.$valueOf(c),g,f,e,h?Boolean.TRUE:null])),this.spinIsGesture=h,Float.isNaN(c)||3.4028235E38==c||!this.vwr.g.waitForMoveTo?this.spinThread.start():(this.spinThread.setEval(a),this.spinThread.run())):null!=this.spinThread&&(this.spinThread.reset(),this.spinThread=
null))},"J.api.JmolScriptEvaluator,~B,~N,JU.Lst,~A,JU.BS,~B");c(c$,"setNavOn",function(a){if(!Float.isNaN(this.navFps)){var b=this.navOn;a&&this.spinOn&&this.setSpin(null,!1,0,null,null,null,!1);this.navOn=a;this.vwr.g.setB("_navigating",a);a||this.navInterrupt();a?(0==this.navX&&(0==this.navY&&0==this.navZ)&&(this.navZ=1),0==this.navFps&&(this.navFps=10),null==this.spinThread&&(this.spinThread=J.api.Interface.getOption("thread.SpinThread",this.vwr,"tm"),this.spinThread.setManager(this,this.vwr,null),
this.spinThread.start())):b&&null!=this.spinThread&&(this.spinThread.interrupt(),this.spinThread=null)}},"~B");c(c$,"setVibrationScale",function(a){this.vibrationScale=a},"~N");c(c$,"setVibrationPeriod",function(a){if(Float.isNaN(a))a=this.vibrationPeriod;else if(0==a)this.vibrationPeriodMs=this.vibrationPeriod=0;else{this.vibrationPeriod=Math.abs(a);this.vibrationPeriodMs=A(1E3*this.vibrationPeriod);if(0<a)return;a=-a}this.setVibrationOn(0<a&&0<=this.vwr.ms.getLastVibrationVector(this.vwr.am.cmi,
0))},"~N");c(c$,"setVibrationT",function(a){this.vibrationT.x=a;0==this.vibrationScale&&(this.vibrationScale=this.vwr.g.vibrationScale)},"~N");c(c$,"isVibrationOn",function(){return this.vibrationOn});c(c$,"setVibrationOn",function(a){a?1>this.vwr.ms.mc?(this.vibrationOn=!1,this.vibrationT.x=0):(null==this.vibrationThread&&(this.vibrationThread=J.api.Interface.getOption("thread.VibrationThread",this.vwr,"tm"),this.vibrationThread.setManager(this,this.vwr,null),this.vibrationThread.start()),this.vibrationOn=
!0):(null!=this.vibrationThread&&(this.vibrationThread.interrupt(),this.vibrationThread=null),this.vibrationOn=!1,this.vibrationT.x=0)},"~B");c(c$,"clearVibration",function(){this.setVibrationOn(!1);this.vibrationScale=0});c(c$,"setStereoMode2",function(a){this.stereoMode=J.c.STER.CUSTOM;this.stereoColors=a},"~A");c(c$,"setStereoMode",function(a){this.stereoColors=null;this.stereoMode=a;this.stereoDoubleDTI=a===J.c.STER.DTI;this.stereoDoubleFull=a===J.c.STER.DOUBLE},"J.c.STER");c(c$,"setStereoDegrees",
function(a){this.stereoDegrees=a;this.stereoRadians=0.017453292*a},"~N");c(c$,"getStereoRotationMatrix",function(a){this.stereoFrame=a;if(!a)return this.matrixRotate;this.matrixTemp3.setAsYRotation(-this.stereoRadians);this.matrixStereo.mul2(this.matrixTemp3,this.matrixRotate);return this.matrixStereo},"~B");c(c$,"isWindowCentered",function(){return this.windowCentered});c(c$,"setWindowCentered",function(a){this.windowCentered=a;this.resetNavigationPoint(!0)},"~B");c(c$,"setRotationRadius",function(a,
b){a=this.modelRadius=0>=a?this.vwr.ms.calcRotationRadius(this.vwr.am.cmi,this.fixedRotationCenter,!0):a;b&&this.vwr.setRotationRadius(a,!1);return a},"~N,~B");c(c$,"setRotationCenterAndRadiusXYZ",function(a,b){this.resetNavigationPoint(!1);null==a?(this.setFixedRotationCenter(this.rotationCenterDefault),this.modelRadius=this.rotationRadiusDefault):(this.setFixedRotationCenter(a),b&&this.windowCentered&&(this.modelRadius=this.vwr.ms.calcRotationRadius(this.vwr.am.cmi,this.fixedRotationCenter,!0)))},
"JU.T3,~B");c(c$,"setNewRotationCenter",function(a,b){null==a&&(a=this.rotationCenterDefault);this.windowCentered?(this.translateToPercent("x",0),this.translateToPercent("y",0),this.setRotationCenterAndRadiusXYZ(a,!0),b&&this.resetFitToScreen(!0)):this.moveRotationCenter(a,!0)},"JU.P3,~B");c(c$,"moveRotationCenter",function(a,b){this.setRotationCenterAndRadiusXYZ(a,!1);b&&this.setRotationPointXY(this.fixedRotationCenter)},"JU.P3,~B");c(c$,"setCenter",function(){this.setRotationCenterAndRadiusXYZ(this.fixedRotationCenter,
!0)});c(c$,"setCenterAt",function(a,b){var c=JU.P3.newP(b);switch(a){case 1073741826:break;case 96:c.add(this.vwr.ms.getAverageAtomPoint());break;case 1678381065:c.add(this.vwr.getBoundBoxCenter());break;default:c.setT(this.rotationCenterDefault)}this.setRotationCenterAndRadiusXYZ(c,!0);this.resetFitToScreen(!0)},"~N,JU.P3");c(c$,"setFrameOffset",function(a){null==this.frameOffsets||0>a||a>=this.frameOffsets.length?this.frameOffset.set(0,0,0):this.frameOffset.setT(this.frameOffsets[a])},"~N");c(c$,
"setSelectedTranslation",function(a,b,c){this.bsSelectedAtoms=a;switch(b){case "X":case "x":this.ptOffset.x+=c;break;case "Y":case "y":this.ptOffset.y+=c;break;case "Z":case "z":this.ptOffset.z+=c}},"JU.BS,~S,~N");c(c$,"setNavFps",function(a){this.navFps=a},"~N");c(c$,"calcCameraFactors",function(){Float.isNaN(this.cameraDepth)&&(this.cameraDepth=this.cameraDepthSetting,this.zoomFactor=3.4028235E38);this.cameraDistance=this.cameraDepth*this.screenPixelCount;this.referencePlaneOffset=this.cameraDistance+
this.screenPixelCount/2;this.scalePixelsPerAngstrom=this.scale3D&&!this.perspectiveDepth&&1!=this.mode?72/this.scale3DAngstromsPerInch*(this.antialias?2:1):this.screenPixelCount/this.visualRangeAngstroms;1!=this.mode&&(this.mode=0==this.camera.z?0:2);this.perspectiveShiftXY.set(0==this.camera.z?0:100*(this.camera.x*this.scalePixelsPerAngstrom/this.screenWidth),0==this.camera.z?0:100*(this.camera.y*this.scalePixelsPerAngstrom/this.screenHeight),0);this.modelRadiusPixels=this.modelRadius*this.scalePixelsPerAngstrom;
var a=2*this.modelRadius/this.visualRangeAngstroms*this.referencePlaneOffset;if(1==this.mode)this.calcNavCameraFactors(a);else{this.zoomFactor=3.4028235E38;this.modelCenterOffset=this.referencePlaneOffset;if(!this.scale3D||this.perspectiveDepth)this.scalePixelsPerAngstrom*=this.modelCenterOffset/a*this.zmPct/100;this.modelRadiusPixels=this.modelRadius*this.scalePixelsPerAngstrom}});c(c$,"calcNavCameraFactors",function(a){3.4028235E38==this.zoomFactor?(1E4<this.zmPct&&(this.zmPct=1E4),this.modelCenterOffset=
100*a/this.zmPct):this.prevZoomSetting!=this.zmPctSet&&(this.modelCenterOffset=0==this.zoomRatio?100*a/this.zmPctSet:this.modelCenterOffset+(1-this.zoomRatio)*this.referencePlaneOffset,this.navMode=-1);this.prevZoomSetting=this.zmPctSet;this.zoomFactor=this.modelCenterOffset/this.referencePlaneOffset;this.zmPct=0==this.zoomFactor?1E4:100*(a/this.modelCenterOffset)},"~N");c(c$,"getPerspectiveFactor",function(a){return 0>=a?this.referencePlaneOffset:this.referencePlaneOffset/a},"~N");c(c$,"unTransformPoint",
function(a,b){this.untransformedPoint.setT(a);switch(this.mode){case 1:this.untransformedPoint.x-=this.navigationOffset.x;this.untransformedPoint.y-=this.navigationOffset.y;break;case 2:this.fScrPt.x+=this.perspectiveShiftXY.x,this.fScrPt.y+=this.perspectiveShiftXY.y;case 0:this.untransformedPoint.x-=this.fixedRotationOffset.x,this.untransformedPoint.y-=this.fixedRotationOffset.y}if(this.perspectiveDepth){var c=this.getPerspectiveFactor(this.untransformedPoint.z);this.untransformedPoint.x/=c;this.untransformedPoint.y/=
c}switch(this.mode){case 1:this.untransformedPoint.x+=this.navigationShiftXY.x;this.untransformedPoint.y+=this.navigationShiftXY.y;break;case 2:this.untransformedPoint.x-=this.perspectiveShiftXY.x,this.untransformedPoint.y-=this.perspectiveShiftXY.y}this.matrixTransformInv.rotTrans2(this.untransformedPoint,b)},"JU.T3,JU.T3");c(c$,"resetNavigationPoint",function(a){5>this.zmPct&&1!=this.mode?(this.perspectiveDepth=!0,this.mode=1):(1==this.mode?(this.navMode=1,this.slabPercentSetting=0,this.perspectiveDepth=
!0):a&&(this.slabPercentSetting=100),this.vwr.setFloatProperty("slabRange",0),a&&this.setSlabEnabled(1==this.mode),this.zoomFactor=3.4028235E38,this.zmPctSet=this.zmPct)},"~B");c(c$,"setNavigatePt",function(a){this.navigationCenter.setT(a);this.navMode=3;this.navigating=!0;this.finalizeTransformParameters();this.navigating=!1},"JU.P3");c(c$,"setNavigationSlabOffsetPercent",function(a){this.vwr.g.setF("navigationSlab",a);this.calcCameraFactors();this.navigationSlabOffset=a/50*this.modelRadiusPixels},
"~N");c(c$,"getNavigationOffset",function(){this.transformPt3f(this.navigationCenter,this.navigationOffset);return this.navigationOffset});c(c$,"getNavPtHeight",function(){return this.height/2});c(c$,"getNavigationOffsetPercent",function(a){this.getNavigationOffset();return 0==this.width||0==this.height?0:"X"==a?100*(this.navigationOffset.x-this.width/2)/this.width:100*(this.navigationOffset.y-this.getNavPtHeight())/this.height},"~S");c(c$,"getNavigationText",function(a){a=a?" /* navigation center, translation, depth */ ":
" ";if(1!=this.mode)return a+"{0 0 0} 0 0 0";this.getNavigationOffset();return a+JU.Escape.eP(this.navigationCenter)+" "+this.getNavigationOffsetPercent("X")+" "+this.getNavigationOffsetPercent("Y")+" "+this.navigationDepthPercent},"~B");c(c$,"setScreenParameters",function(a,b,c,g,f,e){var h=1==this.mode?JU.P3.newP(this.navigationCenter):null,j=JU.P3.newP(this.navigationOffset);j.x/=this.width;j.y/=this.height;this.setScreenParameters0(a,b,c,g,f,e);null!=h&&(this.navigationCenter.setT(h),this.navTranslatePercentOrTo(-1,
j.x*this.width,j.y*this.height),this.setNavigatePt(h))},"~N,~N,~B,~B,~B,~B");c(c$,"navInterrupt",function(){null!=this.nav&&this.nav.interrupt()});c(c$,"getNav",function(){if(null!=this.nav)return!0;this.nav=J.api.Interface.getOption("navigate.Navigator",this.vwr,"tm");if(null==this.nav)return!1;this.nav.set(this,this.vwr);return!0});c(c$,"navigateList",function(a,b){this.getNav()&&this.nav.navigateList(a,b)},"J.api.JmolScriptEvaluator,JU.Lst");c(c$,"navigateAxis",function(a,b){this.getNav()&&this.nav.navigateAxis(a,
b)},"JU.V3,~N");c(c$,"setNavigationOffsetRelative",function(){this.getNav()&&this.nav.setNavigationOffsetRelative()});c(c$,"navigateKey",function(a,b){this.getNav()&&this.nav.navigateKey(a,b)},"~N,~N");c(c$,"setNavigationDepthPercent",function(a){this.getNav()&&this.nav.setNavigationDepthPercent(a)},"~N");c(c$,"navTranslatePercentOrTo",function(a,b,c){this.getNav()&&this.nav.navTranslatePercentOrTo(a,b,c)},"~N,~N,~N");c(c$,"calcNavigationPoint",function(){this.getNav()&&this.nav.calcNavigationPoint()});
c(c$,"getNavigationState",function(){return 1==this.mode&&this.getNav()?this.nav.getNavigationState():""});E(c$,"DEFAULT_SPIN_Y",30,"DEFAULT_SPIN_FPS",30,"DEFAULT_NAV_FPS",10,"DEFAULT_VISUAL_RANGE",5,"DEFAULT_STEREO_DEGREES",-5,"MODE_STANDARD",0,"MODE_NAVIGATION",1,"MODE_PERSPECTIVE_PYMOL",2,"DEFAULT_PERSPECTIVE_MODEL",11,"DEFAULT_PERSPECTIVE_DEPTH",!0,"DEFAULT_CAMERA_DEPTH",3,"degreesPerRadian",57.29577951308232,"MAXIMUM_ZOOM_PERCENTAGE",2E5,"MAXIMUM_ZOOM_PERSPECTIVE_DEPTH",1E4,"NAV_MODE_IGNORE",
-2,"NAV_MODE_ZOOMED",-1,"NAV_MODE_NONE",0,"NAV_MODE_RESET",1,"NAV_MODE_NEWXY",2,"NAV_MODE_NEWXYZ",3,"NAV_MODE_NEWZ",4)});s("JV");v(["java.lang.Enum","J.api.JmolViewer","$.PlatformViewer","J.atomdata.AtomDataServer","java.util.Hashtable"],"JV.Viewer","java.io.BufferedInputStream $.BufferedReader $.Reader java.lang.Boolean $.Character $.Float $.NullPointerException java.util.Arrays JU.AU $.BS $.CU $.DF $.Lst $.P3 $.P3i $.PT $.Quat $.Rdr $.SB J.adapter.smarter.SmarterJmolAdapter J.api.Interface $.JmolAppConsoleInterface J.atomdata.RadiusData J.c.FIL $.STER $.VDW J.i18n.GT JM.ModelSet JS.SV $.T J.thread.TimeoutThread JU.BSUtil $.C $.CommandHistory $.Elements $.Escape $.GData $.JmolMolecule $.Logger $.Parser $.TempArray JV.ActionManager $.AnimationManager $.ColorManager $.FileManager $.GlobalSettings $.JC $.ModelManager $.SelectionManager $.ShapeManager $.StateManager $.StatusManager $.TransformManager JV.binding.Binding".split(" "),
function(){c$=t(function(){this.queueOnHold=this.isSingleThreaded=this.isWebGL=this.isJSNoAWT=this.haveDisplay=this.autoExit=this.testAsync=!1;this.fullName="";this.fm=this.ms=this.definedAtomSets=this.compiler=null;this.mustRender=this.listCommands=this.isSyntaxCheck=this.isSyntaxAndFileCheck=this.isJNLP=this.isApplet=!1;this.appletName=this.htmlName="";this.tryPt=0;this.insertedCommand="";this.tm=this.sm=this.g=this.rm=this.slm=this.shm=this.dm=this.cm=this.am=this.acm=this.html5Applet=this.gdata=
null;this.logFilePath=this.syncId="";this.useCommandThread=this.noGraphicsAllowed=this.multiTouch=this.isSilent=this.isSignedAppletLocal=this.isSignedApplet=this.isPrintOnly=this.allowScripting=!1;this.tempArray=this.eval=this.scm=this.stm=this.mm=this.commandHistory=this.access=this.modelAdapter=this.display=this.vwrOptions=this.commandOptions=null;this.async=this.allowArrayDotNotation=!1;this.executor=null;this.screenHeight=this.screenWidth=0;this.errorMessageUntranslated=this.errorMessage=this.chainList=
this.chainMap=this.rd=this.defaultVdw=this.actionStatesRedo=this.actionStates=null;this.privateKey=0;this.dataOnly=!1;this.maximumSize=2147483647;this.gRight=null;this.isStereoSlave=!1;this.imageFontScaling=1;this.antialiased=!1;this.prevFrame=-2147483648;this.prevMorphModel=0;this.haveJDX=!1;this.jzt=this.outputManager=this.jsv=null;this.headless=this.isPreviewOnly=!1;this.lastData=this.jsc=this.smilesMatcher=this.minimizer=this.dssrParser=this.annotationParser=this.ligandModelSet=this.ligandModels=
this.mouse=this.movableBitSet=null;this.motionEventNumber=0;this.inMotion=!1;this.refreshing=!0;this.axesAreTainted=!1;this.jsParams=this.captureParams=null;this.cirChecked=!1;this.hoverAtomIndex=-1;this.hoverText=null;this.hoverLabel="%U";this.hoverEnabled=!0;this.currentCursor=0;this.ptTemp=null;this.selectionHalosEnabled=!1;this.noFrankEcho=this.frankOn=!0;this.scriptEditorVisible=!1;this.pm=this.headlessImageParams=this.modelkit=this.jmolpopup=this.scriptEditor=this.appConsole=null;this.isTainted=
!0;this.showSelected=this.movingSelected=!1;this.atomHighlighted=-1;this.creatingImage=!1;this.userVdwMars=this.userVdws=this.bsUserVdws=null;this.currentShapeID=-1;this.localFunctions=this.currentShapeState=null;this.$isKiosk=!1;this.displayLoadErrors=!0;this.$isParallel=!1;this.stateScriptVersionInt=2147483647;this.timeouts=this.jsExporter3D=null;this.chainCaseSpecified=!1;this.macros=this.nboParser=this.triangulator=this.jsonParser=this.jcm=this.jbr=this.logFileName=this.nmrCalculation=null;this.consoleFontScale=
1;p(this,arguments)},JV,"Viewer",J.api.JmolViewer,[J.atomdata.AtomDataServer,J.api.PlatformViewer]);c(c$,"finalize",function(){JU.Logger.debugging&&JU.Logger.debug("vwr finalize "+this);W(this,JV.Viewer,"finalize",[])});c(c$,"setInsertedCommand",function(a){this.insertedCommand=a},"~S");c$.getJmolVersion=j(c$,"getJmolVersion",function(){return null==JV.Viewer.version_date?JV.Viewer.version_date=JV.JC.version+"  "+JV.JC.date:JV.Viewer.version_date});c$.allocateViewer=c(c$,"allocateViewer",function(a,
b,c,g,f,e,h,j){var l=new java.util.Hashtable;l.put("display",a);l.put("adapter",b);l.put("statusListener",h);l.put("platform",j);l.put("options",e);l.put("fullName",c);l.put("documentBase",g);l.put("codeBase",f);return new JV.Viewer(l)},"~O,J.api.JmolAdapter,~S,java.net.URL,java.net.URL,~S,J.api.JmolStatusListener,J.api.GenericPlatform");q(c$,function(a){I(this,JV.Viewer,[]);this.commandHistory=new JU.CommandHistory;this.rd=new J.atomdata.RadiusData(null,0,null,null);this.defaultVdw=J.c.VDW.JMOL;
this.localFunctions=new java.util.Hashtable;this.privateKey=Math.random();this.actionStates=new JU.Lst;this.actionStatesRedo=new JU.Lst;this.chainMap=new java.util.Hashtable;this.chainList=new JU.Lst;this.setOptions(a)},"java.util.Map");c(c$,"haveAccess",function(a){return this.access===a},"JV.Viewer.ACCESS");j(c$,"getModelAdapter",function(){return null==this.modelAdapter?this.modelAdapter=new J.adapter.smarter.SmarterJmolAdapter:this.modelAdapter});j(c$,"getSmartsMatch",function(a,b){null==b&&(b=
this.bsA());return this.getSmilesMatcher().getSubstructureSet(a,this.ms.at,this.ms.ac,b,2)},"~S,JU.BS");c(c$,"getSmartsMatchForNodes",function(a,b){return this.getSmilesMatcher().getSubstructureSet(a,b,b.length,null,2)},"~S,~A");c(c$,"getSmartsMap",function(a,b,c){null==b&&(b=this.bsA());0==c&&(c=2);return this.getSmilesMatcher().getCorrelationMaps(a,this.ms.at,this.ms.ac,b,c)},"~S,JU.BS,~N");c(c$,"setOptions",function(a){this.vwrOptions=a;JU.Logger.debugging&&JU.Logger.debug("Viewer constructor "+
this);this.modelAdapter=a.get("adapter");var b=a.get("statusListener");this.fullName=a.get("fullName");null==this.fullName&&(this.fullName="");var c=a.get("codePath");null==c&&(c="../java/");JV.Viewer.appletCodeBase=c.toString();JV.Viewer.appletIdiomaBase=JV.Viewer.appletCodeBase.substring(0,JV.Viewer.appletCodeBase.lastIndexOf("/",JV.Viewer.appletCodeBase.length-2)+1)+"idioma";c=a.get("documentBase");JV.Viewer.appletDocumentBase=null==c?"":c.toString();c=a.get("options");this.commandOptions=null==
c?"":c.toString();(a.containsKey("debug")||0<=this.commandOptions.indexOf("-debug"))&&JU.Logger.setLogLevel(5);this.isApplet&&a.containsKey("maximumSize")&&this.setMaximumSize(a.get("maximumSize").intValue());(this.isJNLP=this.checkOption2("isJNLP","-jnlp"))&&JU.Logger.info("setting JNLP mode TRUE");this.isApplet=(this.isSignedApplet=this.isJNLP||this.checkOption2("signedApplet","-signed"))||this.checkOption2("applet","-applet");this.allowScripting=!this.checkOption2("noscripting","-noscripting");
var g=this.fullName.indexOf("__");this.htmlName=0>g?this.fullName:this.fullName.substring(0,g);this.appletName=JU.PT.split(this.htmlName+"_","_")[0];this.syncId=0>g?"":this.fullName.substring(g+2,this.fullName.length-2);this.access=this.checkOption2("access:READSPT","-r")?JV.Viewer.ACCESS.READSPT:this.checkOption2("access:NONE","-R")?JV.Viewer.ACCESS.NONE:JV.Viewer.ACCESS.ALL;(this.isPreviewOnly=a.containsKey("previewOnly"))&&a.remove("previewOnly");this.isPrintOnly=this.checkOption2("printOnly",
"-p");this.dataOnly=this.checkOption2("isDataOnly","\x00");this.autoExit=this.checkOption2("exit","-x");c=a.get("platform");g="unknown";null==c&&(c=this.commandOptions.contains("platform=")?this.commandOptions.substring(this.commandOptions.indexOf("platform=")+9):"J.awt.Platform");if(n(c,String)){g=c;this.isWebGL=0<=g.indexOf(".awtjs.");JV.Viewer.isJS=this.isJSNoAWT=this.isWebGL||0<=g.indexOf(".awtjs2d.");this.async=!this.dataOnly&&!this.autoExit&&(this.testAsync||JV.Viewer.isJS&&a.containsKey("async"));
var f=c=null,e="?";self.Jmol&&(f=Jmol,c=Jmol._applets[this.htmlName.split("_object")[0]],e=Jmol._version);null!=e&&(this.html5Applet=c,JV.Viewer.jmolObject=f,JV.Viewer.strJavaVersion=e,JV.Viewer.strJavaVendor="Java2Script "+(this.isWebGL?"(WebGL)":"(HTML5)"));c=J.api.Interface.getInterface(g,this,"setOptions")}this.apiPlatform=c;this.display=a.get("display");this.isSingleThreaded=this.apiPlatform.isSingleThreaded();this.noGraphicsAllowed=this.checkOption2("noDisplay","-n");this.headless=this.apiPlatform.isHeadless();
this.haveDisplay=this.isWebGL||null!=this.display&&!this.noGraphicsAllowed&&!this.headless&&!this.dataOnly;this.noGraphicsAllowed=(new Boolean(this.noGraphicsAllowed&null==this.display)).valueOf();this.headless=(new Boolean(this.headless|this.noGraphicsAllowed)).valueOf();this.haveDisplay?(this.mustRender=!0,this.multiTouch=this.checkOption2("multiTouch","-multitouch"),this.isWebGL||(this.display=document.getElementById(this.display))):this.display=null;this.apiPlatform.setViewer(this,this.display);
c=a.get("graphicsAdapter");null==c&&!this.isWebGL&&(c=J.api.Interface.getOption("g3d.Graphics3D",this,"setOptions"));this.gdata=null==c&&(this.isWebGL||!JV.Viewer.isJS)?new JU.GData:c;this.gdata.initialize(this,this.apiPlatform);this.stm=new JV.StateManager(this);this.cm=new JV.ColorManager(this,this.gdata);this.sm=new JV.StatusManager(this);g=a.containsKey("4DMouse");this.tm=JV.TransformManager.getTransformManager(this,2147483647,0,g);this.slm=new JV.SelectionManager(this);this.haveDisplay&&(this.acm=
this.multiTouch?J.api.Interface.getOption("multitouch.ActionManagerMT",null,null):new JV.ActionManager,this.acm.setViewer(this,this.commandOptions+"-multitouch-"+a.get("multiTouch")),this.mouse=this.apiPlatform.getMouseManager(this.privateKey,this.display),this.multiTouch&&!this.checkOption2("-simulated","-simulated")&&this.apiPlatform.setTransparentCursor(this.display));this.mm=new JV.ModelManager(this);this.shm=new JV.ShapeManager(this);this.tempArray=new JU.TempArray;this.am=new JV.AnimationManager(this);
c=a.get("repaintManager");null==c&&(c=J.api.Interface.getOption("render.RepaintManager",this,"setOptions"));if(JV.Viewer.isJS||null!=c&&!c.equals(""))(this.rm=c).set(this,this.shm);this.ms=new JM.ModelSet(this,null);this.initialize(!0,!1);this.fm=new JV.FileManager(this);this.definedAtomSets=new java.util.Hashtable;this.setJmolStatusListener(b);this.isApplet?(JU.Logger.info("vwrOptions: \n"+JU.Escape.escapeMap(this.vwrOptions)),b=this.vwrOptions.get("documentLocation"),!JV.Viewer.isJS&&(null!=b&&
b.startsWith("file:/"))&&(b=b.substring(0,b.substring(0,(b+"?").indexOf("?")).lastIndexOf("/")),JU.Logger.info("setting current directory to "+b),this.cd(b)),b=JV.Viewer.appletDocumentBase,g=b.indexOf("#"),0<=g&&(b=b.substring(0,g)),g=b.lastIndexOf("?"),0<=g&&(b=b.substring(0,g)),g=b.lastIndexOf("/"),0<=g&&(b=b.substring(0,g)),JV.Viewer.jsDocumentBase=b,this.fm.setAppletContext(JV.Viewer.appletDocumentBase),b=a.get("appletProxy"),null!=b&&this.setStringProperty("appletProxy",b),this.isSignedApplet?
(this.logFilePath=JU.PT.rep(JV.Viewer.appletCodeBase,"file://",""),this.logFilePath=JU.PT.rep(this.logFilePath,"file:/",""),0<=this.logFilePath.indexOf("//")?this.logFilePath=null:this.isSignedAppletLocal=!0):JV.Viewer.isJS||(this.logFilePath=null),new J.i18n.GT(this,a.get("language")),JV.Viewer.isJS&&this.acm.createActions()):(this.gdata.setBackgroundTransparent(this.checkOption2("backgroundTransparent","-b")),(this.isSilent=this.checkOption2("silent","-i"))&&JU.Logger.setLogLevel(3),this.headless&&
!this.isSilent&&JU.Logger.info("Operating headless display="+this.display+" nographicsallowed="+this.noGraphicsAllowed),this.isSyntaxCheck=(this.isSyntaxAndFileCheck=this.checkOption2("checkLoad","-C"))||this.checkOption2("check","-c"),this.listCommands=this.checkOption2("listCommands","-l"),this.cd("."),this.headless&&(this.headlessImageParams=a.get("headlessImage"),c=a.get("headlistMaxTimeMs"),null==c&&(c=Integer.$valueOf(6E4)),this.setTimeout(""+Math.random(),c.intValue(),"exitJmol")));this.useCommandThread=
!this.headless&&this.checkOption2("useCommandThread","-threaded");this.setStartupBooleans();this.setIntProperty("_nProcessors",JV.Viewer.nProcessors);this.isSilent||JU.Logger.info("(C) 2015 Jmol Development\nJmol Version: "+JV.Viewer.getJmolVersion()+"\njava.vendor: "+JV.Viewer.strJavaVendor+"\njava.version: "+JV.Viewer.strJavaVersion+"\nos.name: "+JV.Viewer.strOSName+"\nAccess: "+this.access+"\nmemory: "+this.getP("_memory")+"\nprocessors available: "+JV.Viewer.nProcessors+"\nuseCommandThread: "+
this.useCommandThread+(!this.isApplet?"":"\nappletId:"+this.htmlName+(this.isSignedApplet?" (signed)":"")));this.zap(!1,!0,!1);this.g.setO("language",J.i18n.GT.getLanguage());this.g.setO("_hoverLabel",this.hoverLabel);this.stm.setJmolDefaults();JU.Elements.covalentVersion=1;this.allowArrayDotNotation=!0;this.allowScripting&&this.getScriptManager()},"java.util.Map");c(c$,"setMaximumSize",function(a){this.maximumSize=Math.max(a,100)},"~N");c(c$,"setStereo",function(a,b){this.isStereoSlave=a;this.gRight=
b},"~B,~O");c(c$,"getMenu",function(a){this.getPopupMenu();return a.equals("\x00")?(this.popupMenu(this.screenWidth-120,0,"j"),"OK"):null==this.jmolpopup?"":this.jmolpopup.jpiGetMenuAsString("Jmol version "+JV.Viewer.getJmolVersion()+"|_GET_MENU|"+a)},"~S");j(c$,"resizeInnerPanel",function(a,b){if(!this.autoExit&&this.haveDisplay)return this.sm.resizeInnerPanel(a,b);this.setScreenDimension(a,b);return B(-1,[this.screenWidth,this.screenHeight])},"~N,~N");j(c$,"setScreenDimension",function(a,b){b=Math.min(b,
this.maximumSize);a=Math.min(a,this.maximumSize);this.tm.stereoDoubleFull&&(a=w((a+1)/2));this.screenWidth==a&&this.screenHeight==b||this.resizeImage(a,b,!1,!1,!0)},"~N,~N");c(c$,"resizeImage",function(a,b,c,g,f){if(c||!this.creatingImage){var e=this.antialiased;this.antialiased=f?this.g.antialiasDisplay&&this.checkMotionRendering(603979786):c&&!g?this.g.antialiasImages:!1;!g&&(!c&&(0<a||e!=this.antialiased))&&this.setShapeProperty(5,"clearBoxes",null);this.imageFontScaling=(this.antialiased?2:1)*
(f||this.tm.scale3D||0>=a?1:1*(this.g.zoomLarge==b>a?b:a)/this.getScreenDim());0<a?(this.screenWidth=a,this.screenHeight=b,c||(this.g.setI("_width",a),this.g.setI("_height",b))):(a=0==this.screenWidth?this.screenWidth=500:this.screenWidth,b=0==this.screenHeight?this.screenHeight=500:this.screenHeight);this.tm.setScreenParameters(a,b,c||f?this.g.zoomLarge:!1,this.antialiased,!1,!1);this.gdata.setWindowParameters(a,b,this.antialiased);0<a&&!c&&this.setStatusResized(a,b)}},"~N,~N,~B,~B,~B");j(c$,"getScreenWidth",
function(){return this.screenWidth});j(c$,"getScreenHeight",function(){return this.screenHeight});c(c$,"getScreenDim",function(){return this.g.zoomLarge==this.screenHeight>this.screenWidth?this.screenHeight:this.screenWidth});c(c$,"setWidthHeightVar",function(){this.g.setI("_width",this.screenWidth);this.g.setI("_height",this.screenHeight)});c(c$,"getBoundBoxCenterX",function(){return w(this.screenWidth/2)});c(c$,"getBoundBoxCenterY",function(){return w(this.screenHeight/2)});c(c$,"updateWindow",
function(a,b){if(!this.refreshing||this.creatingImage)return this.refreshing?!1:!JV.Viewer.isJS;(this.isTainted||this.tm.slabEnabled)&&this.setModelVisibility();this.isTainted=!1;null!=this.rm&&0!=a&&this.setScreenDimension(a,b);return!0},"~N,~N");c(c$,"getImage",function(a,b){var c=null;try{this.beginRendering(a,b),this.render(),this.gdata.endRendering(),c=this.gdata.getScreenImage(b)}catch(g){if(H(g,Error))this.gdata.getScreenImage(b),this.handleError(g,!1),this.setErrorMessage("Error during rendering: "+
g,null);else if(H(g,Exception))Zystem.out.println("render error"+g);else throw g;}return c},"~B,~B");c(c$,"beginRendering",function(a,b){this.gdata.beginRendering(this.tm.getStereoRotationMatrix(a),this.g.translucent,b,!this.checkMotionRendering(603979967))},"~B,~B");c(c$,"render",function(){if(!(null==this.mm.modelSet||!this.mustRender||!this.refreshing&&!this.creatingImage||null==this.rm)){var a=this.antialiased&&this.g.antialiasTranslucent,b=this.shm.finalizeAtoms(this.tm.bsSelectedAtoms,!0);this.isWebGL?
(this.rm.renderExport(this.gdata,this.ms,this.jsParams),this.notifyViewerRepaintDone()):(this.rm.render(this.gdata,this.ms,!0,b),this.gdata.setPass2(a)&&(this.tm.setAntialias(a),this.rm.render(this.gdata,this.ms,!1,null),this.tm.setAntialias(this.antialiased)))}});c(c$,"drawImage",function(a,b,c,g,f){null!=a&&null!=b&&this.apiPlatform.drawImage(a,b,c,g,this.screenWidth,this.screenHeight,f);this.gdata.releaseScreenImage()},"~O,~O,~N,~N,~B");c(c$,"getScreenImage",function(){return this.getScreenImageBuffer(null,
!0)});j(c$,"getScreenImageBuffer",function(a,b){if(this.isWebGL)return b?this.apiPlatform.allocateRgbImage(0,0,null,0,!1,!0):null;var c=this.tm.stereoDoubleFull||this.tm.stereoDoubleDTI,g=this.tm.stereoMode.isBiColor(),f=null==a&&c;g?(this.beginRendering(!0,b),this.render(),this.gdata.endRendering(),this.gdata.snapshotAnaglyphChannelBytes(),this.beginRendering(!1,b),this.render(),this.gdata.endRendering(),this.gdata.applyAnaglygh(this.tm.stereoMode,this.tm.stereoColors),g=this.gdata.getScreenImage(b)):
g=this.getImage(c,b);var e=null;f&&(e=this.apiPlatform.newBufferedImage(g,this.tm.stereoDoubleDTI?this.screenWidth:this.screenWidth<<1,this.screenHeight),a=this.apiPlatform.getGraphics(e));null!=a&&(c&&(this.tm.stereoMode===J.c.STER.DTI?(this.drawImage(a,g,this.screenWidth>>1,0,!0),g=this.getImage(!1,!1),this.drawImage(a,g,0,0,!0),a=null):(this.drawImage(a,g,this.screenWidth,0,!1),g=this.getImage(!1,!1))),null!=a&&this.drawImage(a,g,0,0,!1));return f?e:g},"~O,~B");c(c$,"evalStringWaitStatusQueued",
function(a,b,c,g,f){return 0==b.indexOf("JSCONSOLE")?(this.html5Applet._showInfo(0>b.indexOf("CLOSE")),0<=b.indexOf("CLEAR")&&this.html5Applet._clearConsole(),null):null==this.getScriptManager()?null:this.scm.evalStringWaitStatusQueued(a,b,c,g,f)},"~S,~S,~S,~B,~B");c(c$,"popupMenu",function(a,b,c){if(this.haveDisplay&&this.refreshing&&!this.isPreviewOnly&&!this.g.disablePopupMenu)switch(c){case "j":try{this.getPopupMenu(),this.jmolpopup.jpiShow(a,b)}catch(g){JU.Logger.info(g.toString()),this.g.disablePopupMenu=
!0}break;case "a":case "b":case "m":if(null==this.getModelkit(!0))break;this.modelkit.showMenu(a,b)}},"~N,~N,~S");c(c$,"getModelkit",function(a){null==this.modelkit?(this.modelkit=J.api.Interface.getInterface("J.modelkit.ModelKit",this,"script")).setMenu(this.apiPlatform.getMenuPopup(null,"m")):a&&this.modelkit.updateMenu();return this.modelkit},"~B");c(c$,"getPopupMenu",function(){if(this.g.disablePopupMenu)return null;if(null==this.jmolpopup&&(this.jmolpopup=this.allowScripting?this.apiPlatform.getMenuPopup(this.menuStructure,
"j"):null,null==this.jmolpopup))return this.async||(this.g.disablePopupMenu=!0),null;this.isJSNoAWT&&this.checkMenuUpdate();return this.jmolpopup.jpiGetMenuAsObject()});j(c$,"setMenu",function(a,b){b&&JU.Logger.info("Setting menu "+(0==a.length?"to Jmol defaults":"from file "+a));0==a.length?a=null:b&&(a=this.getFileAsString3(a,!1,null));this.getProperty("DATA_API","setMenu",a);this.sm.setCallbackFunction("menu",a)},"~S,~B");c(c$,"setStatusFrameChanged",function(a){a&&(this.prevFrame=-2147483648);
this.tm.setVibrationPeriod(NaN);var b=this.am.firstFrameIndex,c=this.am.lastFrameIndex,g=this.am.isMovie;a=this.am.cmi;b==c&&!g&&(a=b);var f=this.getModelFileNumber(a),e=this.am.cmi,h=f,j=f%1E6,l=g?b:this.getModelFileNumber(b),n=g?c:this.getModelFileNumber(c),m;g?m=""+(e+1):0==h?(m=this.getModelNumberDotted(b),b!=c&&(m+=" - "+this.getModelNumberDotted(c)),w(l/1E6)==w(n/1E6)&&(h=l)):m=this.getModelNumberDotted(a);0!=h&&(h=1E6>h?1:w(h/1E6));g||(this.g.setI("_currentFileNumber",h),this.g.setI("_currentModelNumberInFile",
j));b=this.am.currentMorphModel;this.g.setI("_currentFrame",e);this.g.setI("_morphCount",this.am.morphCount);this.g.setF("_currentMorphFrame",b);this.g.setI("_frameID",f);this.g.setI("_modelIndex",a);this.g.setO("_modelNumber",m);this.g.setO("_modelName",0>a?"":this.getModelName(a));f=0>a?"":this.ms.getModelTitle(a);this.g.setO("_modelTitle",null==f?"":f);this.g.setO("_modelFile",0>a?"":this.ms.getModelFileName(a));this.g.setO("_modelType",0>a?"":this.ms.getModelFileType(a));e==this.prevFrame&&b==
this.prevMorphModel||(this.prevFrame=e,this.prevMorphModel=b,f=this.getModelName(e),g?f=""+(""===f?e+1:this.am.caf+1)+": "+f:(g=""+this.getModelNumberDotted(e),f.equals(g)||(f=g+": "+f)),this.sm.setStatusFrameChanged(h,j,0>this.am.animationDirection?-l:l,0>this.am.currentDirection?-n:n,e,b,f),this.doHaveJDX()&&this.getJSV().setModel(a),JV.Viewer.isJS&&this.updateJSView(a,-1))},"~B,~B");c(c$,"doHaveJDX",function(){return this.haveJDX||(this.haveJDX=this.getBooleanProperty("_jspecview"))});c(c$,"getJSV",
function(){null==this.jsv&&(this.jsv=J.api.Interface.getOption("jsv.JSpecView",this,"script"),this.jsv.setViewer(this));return this.jsv});c(c$,"getJDXBaseModelIndex",function(a){return!this.doHaveJDX()?a:this.getJSV().getBaseModelIndex(a)},"~N");c(c$,"getJspecViewProperties",function(a){a=this.sm.getJspecViewProperties(""+a);null!=a&&(this.haveJDX=!0);return a},"~O");c(c$,"scriptEcho",function(a){JU.Logger.isActiveLevel(4)&&(JV.Viewer.isJS&&Zystem.out.println(a),this.sm.setScriptEcho(a,this.isScriptQueued()),
this.listCommands&&(null!=a&&0==a.indexOf("$["))&&JU.Logger.info(a))},"~S");c(c$,"isScriptQueued",function(){return null!=this.scm&&this.scm.isScriptQueued()});c(c$,"notifyError",function(a,b,c){this.g.setO("_errormessage",c);this.sm.notifyError(a,b,c)},"~S,~S,~S");c(c$,"jsEval",function(a){return""+this.sm.jsEval(a)},"~S");c(c$,"jsEvalSV",function(a){return JS.SV.getVariable(JV.Viewer.isJS?this.sm.jsEval(a):this.jsEval(a))},"~S");c(c$,"setFileLoadStatus",function(a,b,c,g,f,e){this.setErrorMessage(f,
null);this.g.setI("_loadPoint",a.getCode());var h=a!==J.c.FIL.CREATING_MODELSET;h&&this.setStatusFrameChanged(!1,!1);this.sm.setFileLoadStatus(b,c,g,f,a.getCode(),h,e);h&&(this.doHaveJDX()&&this.getJSV().setModel(this.am.cmi),JV.Viewer.isJS&&this.updateJSView(this.am.cmi,-2))},"J.c.FIL,~S,~S,~S,~S,Boolean");c(c$,"getZapName",function(){return this.g.modelKitMode?"Jmol Model Kit":"zapped"});c(c$,"setStatusMeasuring",function(a,b,c,g){this.sm.setStatusMeasuring(a,b,c,g)},"~S,~N,~S,~N");c(c$,"notifyMinimizationStatus",
function(){var a=this.getP("_minimizationStep"),b=this.getP("_minimizationForceField");this.sm.notifyMinimizationStatus(this.getP("_minimizationStatus"),n(a,String)?Integer.$valueOf(0):a,this.getP("_minimizationEnergy"),a.toString().equals("0")?Float.$valueOf(0):this.getP("_minimizationEnergyDiff"),b)});c(c$,"setStatusAtomPicked",function(a,b,c,g){g&&this.setSelectionSet(JU.BSUtil.newAndSetBit(a));null==b&&(b=this.g.pickLabel,b=0==b.length?this.getAtomInfoXYZ(a,this.g.messageStyleChime):this.ms.getAtomInfo(a,
b,this.ptTemp));this.setPicked(a,!1);0>a&&(g=this.getPendingMeasurement(),null!=g&&(b=b.substring(0,b.length-1)+',"'+g.getString()+'"]'));this.g.setO("_pickinfo",b);this.sm.setStatusAtomPicked(a,b,c);0>a||(1==this.sm.getSyncMode()&&this.doHaveJDX()&&this.getJSV().atomPicked(a),JV.Viewer.isJS&&this.updateJSView(this.ms.at[a].mi,a))},"~N,~S,java.util.Map,~B");j(c$,"getProperty",function(a,b,c){if(!"DATA_API".equals(a))return this.getPropertyManager().getProperty(a,b,c);switch("scriptCheck.........consoleText.........scriptEditor........scriptEditorState...getAppConsole.......getScriptEditor.....setMenu.............spaceGroupInfo......disablePopupMenu....defaultDirectory....getPopupMenu........shapeManager........getPreference.......".indexOf(b)){case 0:return this.scriptCheckRet(c,
!0);case 20:return null==this.appConsole?"":this.appConsole.getText();case 40:return this.showEditor(c),null;case 60:return this.scriptEditorVisible=c.booleanValue(),null;case 80:return this.$isKiosk?this.appConsole=null:n(c,J.api.JmolAppConsoleInterface)?this.appConsole=c:null!=c&&!c.booleanValue()?this.appConsole=null:null==this.appConsole&&(null!=c&&c.booleanValue())&&(JV.Viewer.isJS&&(this.appConsole=J.api.Interface.getOption("consolejs.AppletConsole",this,"script")),null!=this.appConsole&&this.appConsole.start(this)),
this.scriptEditor=JV.Viewer.isJS||null==this.appConsole?null:this.appConsole.getScriptEditor(),this.appConsole;case 100:return null==this.appConsole&&(null!=c&&c.booleanValue())&&(this.getProperty("DATA_API","getAppConsole",Boolean.TRUE),this.scriptEditor=null==this.appConsole?null:this.appConsole.getScriptEditor()),this.scriptEditor;case 120:return null!=this.jmolpopup&&this.jmolpopup.jpiDispose(),this.jmolpopup=null,this.menuStructure=c;case 140:return this.getSymTemp().getSpaceGroupInfo(this.ms,
null,-1,!1,null);case 160:return this.g.disablePopupMenu=!0,null;case 180:return this.g.defaultDirectory;case 200:return n(c,String)?this.getMenu(c):this.getPopupMenu();case 220:return this.shm.getProperty(c);case 240:return this.sm.syncSend("getPreference",c,1)}JU.Logger.error("ERROR in getProperty DATA_API: "+b);return null},"~S,~S,~O");c(c$,"notifyMouseClicked",function(a,b,c,g){var f=JV.binding.Binding.getButtonMods(c),e=JV.binding.Binding.getClickCount(c);this.g.setI("_mouseX",a);this.g.setI("_mouseY",
this.screenHeight-b);this.g.setI("_mouseAction",c);this.g.setI("_mouseModifiers",f);this.g.setI("_clickCount",e);return this.sm.setStatusClicked(a,this.screenHeight-b,c,e,g)},"~N,~N,~N,~N");c(c$,"getOutputManager",function(){return null!=this.outputManager?this.outputManager:(this.outputManager=J.api.Interface.getInterface("JV.OutputManager"+(JV.Viewer.isJS?"JS":"Awt"),this,"file")).setViewer(this,this.privateKey)});c(c$,"getJzt",function(){return null==this.jzt?this.jzt=J.api.Interface.getInterface("JU.ZipTools",
this,"zip"):this.jzt});c(c$,"readFileAsMap",function(a,b,c){this.getJzt().readFileAsMap(a,b,c)},"java.io.BufferedInputStream,java.util.Map,~S");c(c$,"getZipDirectoryAsString",function(a){a=this.fm.getBufferedInputStreamOrErrorMessageFromName(a,a,!1,!1,null,!1,!0);return this.getJzt().getZipDirectoryAsStringAndClose(a)},"~S");j(c$,"getImageAsBytes",function(a,b,c,g,f){return this.getOutputManager().getImageAsBytes(a,b,c,g,f)},"~S,~N,~N,~N,~A");j(c$,"releaseScreenImage",function(){this.gdata.releaseScreenImage()});
c(c$,"setDisplay",function(a){this.display=a;this.apiPlatform.setViewer(this,a)},"~O");c(c$,"newMeasurementData",function(a,b){return J.api.Interface.getInterface("JM.MeasurementData",this,"script").init(a,this,b)},"~S,JU.Lst");c(c$,"getDataManager",function(){return null==this.dm?this.dm=J.api.Interface.getInterface("JV.DataManager",this,"script").set(this):this.dm});c(c$,"getScriptManager",function(){if(this.allowScripting&&null==this.scm){this.scm=J.api.Interface.getInterface("JS.ScriptManager",
this,"setOptions");if(JV.Viewer.isJS&&null==this.scm)throw new NullPointerException;if(null==this.scm)return this.allowScripting=!1,null;this.eval=this.scm.setViewer(this);this.useCommandThread&&this.scm.startCommandWatcher(!0)}return this.scm});c(c$,"checkOption2",function(a,b){return this.vwrOptions.containsKey(a)&&!this.vwrOptions.get(a).toString().equals("false")||0<=this.commandOptions.indexOf(b)},"~S,~S");c(c$,"setStartupBooleans",function(){this.setBooleanProperty("_applet",this.isApplet);
this.setBooleanProperty("_jspecview",!1);this.setBooleanProperty("_signedApplet",this.isSignedApplet);this.setBooleanProperty("_headless",this.headless);this.setStringProperty("_restrict",'"'+this.access+'"');this.setBooleanProperty("_useCommandThread",this.useCommandThread)});c(c$,"getExportDriverList",function(){return this.haveAccess(JV.Viewer.ACCESS.ALL)?this.g.getParameter("exportDrivers",!0):""});j(c$,"dispose",function(){this.gRight=null;null!=this.mouse&&(this.acm.dispose(),this.mouse.dispose(),
this.mouse=null);this.clearScriptQueue();this.clearThreads();this.haltScriptExecution();null!=this.scm&&this.scm.clear(!0);this.gdata.destroy();null!=this.jmolpopup&&this.jmolpopup.jpiDispose();null!=this.modelkit&&this.modelkit.dispose();try{null!=this.appConsole&&(this.appConsole.dispose(),this.appConsole=null),null!=this.scriptEditor&&(this.scriptEditor.dispose(),this.scriptEditor=null)}catch(a){if(!H(a,Exception))throw a;}});c(c$,"reset",function(a){this.ms.calcBoundBoxDimensions(null,1);this.axesAreTainted=
!0;this.tm.homePosition(a);this.ms.setCrystallographicDefaults()?this.stm.setCrystallographicDefaults():this.setAxesMode(603979809);this.prevFrame=-2147483648;this.tm.spinOn||this.setSync()},"~B");j(c$,"homePosition",function(){this.evalString("reset spin")});c(c$,"initialize",function(a,b){this.g=new JV.GlobalSettings(this,this.g,a);this.setStartupBooleans();this.setWidthHeightVar();this.haveDisplay&&(this.g.setB("_is2D",JV.Viewer.isJS&&!this.isWebGL),this.g.setB("_multiTouchClient",this.acm.isMTClient()),
this.g.setB("_multiTouchServer",this.acm.isMTServer()));this.cm.setDefaultColors(!1);this.setObjectColor("background","black");this.setObjectColor("axis1","red");this.setObjectColor("axis2","green");this.setObjectColor("axis3","blue");this.am.setAnimationOn(!1);this.am.setAnimationFps(this.g.animationFps);this.sm.playAudio(null);this.sm.allowStatusReporting=this.g.statusReporting;this.setBooleanProperty("antialiasDisplay",b?!0:this.g.antialiasDisplay);this.stm.resetLighting();this.tm.setDefaultPerspective()},
"~B,~B");c(c$,"saveModelOrientation",function(){this.ms.saveModelOrientation(this.am.cmi,this.stm.getOrientation())});c(c$,"restoreModelOrientation",function(a){a=this.ms.getModelOrientation(a);null!=a&&a.restore(-1,!0)},"~N");c(c$,"restoreModelRotation",function(a){a=this.ms.getModelOrientation(a);null!=a&&a.restore(-1,!1)},"~N");c(c$,"getGLmolView",function(){var a=this.tm,b=a.fixedRotationCenter,c=a.getRotationQ();return{center:b,quaternion:c,xtrans:a.xTranslationFraction,ytrans:a.yTranslationFraction,
scale:a.scalePixelsPerAngstrom,zoom:a.zmPctSet,cameraDistance:a.cameraDistance,pixelCount:a.screenPixelCount,perspective:a.perspectiveDepth,width:a.width,height:a.height}});c(c$,"setRotationRadius",function(a,b){b&&(a=this.tm.setRotationRadius(a,!1));this.ms.setRotationRadius(this.am.cmi,a)&&this.g.setF("rotationRadius",a)},"~N,~B");c(c$,"setCenterBitSet",function(a,b){this.isJmolDataFrame()||this.tm.setNewRotationCenter(0<JU.BSUtil.cardinalityOf(a)?this.ms.getAtomSetCenter(a):null,b)},"JU.BS,~B");
c(c$,"setNewRotationCenter",function(a){this.isJmolDataFrame()||this.tm.setNewRotationCenter(a,!0)},"JU.P3");c(c$,"navigate",function(a,b){this.isJmolDataFrame()||(this.tm.navigateKey(a,b),!this.tm.vibrationOn&&0!=a&&this.refresh(1,"Viewer:navigate()"))},"~N,~N");c(c$,"move",function(a,b,c,g,f,e,h){this.tm.move(a,b,c,g,f,e,h);this.moveUpdate(e)},"J.api.JmolScriptEvaluator,JU.V3,~N,JU.V3,~N,~N,~N");c(c$,"moveTo",function(a,b,c,g,f,e,h,j,l,n,m,p,q,s,t,u,v){this.haveDisplay||(b=0);this.setTainted(!0);
this.tm.moveTo(a,b,c,g,f,e,h,j,l,n,m,p,q,s,t,u,v)},"J.api.JmolScriptEvaluator,~N,JU.P3,JU.V3,~N,JU.M3,~N,~N,~N,~N,JU.P3,~N,~N,~N,~N,~N,~N");c(c$,"moveUpdate",function(a){0<a?this.requestRepaintAndWait("moveUpdate"):0==a&&this.setSync()},"~N");c(c$,"navigatePt",function(a){this.tm.setNavigatePt(a);this.setSync()},"JU.P3");c(c$,"navigateAxis",function(a,b){this.tm.navigateAxis(a,b);this.setSync()},"JU.V3,~N");c(c$,"navTranslatePercent",function(a,b){this.isJmolDataFrame()||(this.tm.navTranslatePercentOrTo(0,
a,b),this.setSync())},"~N,~N");c(c$,"zoomBy",function(a){this.tm.zoomBy(a);this.refresh(2,this.sm.syncingMouse?"Mouse: zoomBy "+a:"")},"~N");c(c$,"zoomByFactor",function(a,b,c){this.tm.zoomByFactor(a,b,c);this.refresh(2,!this.sm.syncingMouse?"":"Mouse: zoomByFactor "+a+(2147483647==b?"":" "+b+" "+c))},"~N,~N,~N");c(c$,"rotateXYBy",function(a,b){this.tm.rotateXYBy(a,b,null);this.refresh(2,this.sm.syncingMouse?"Mouse: rotateXYBy "+a+" "+b:"")},"~N,~N");c(c$,"spinXYBy",function(a,b,c){this.tm.spinXYBy(a,
b,c);0==a&&0==b||this.refresh(2,this.sm.syncingMouse?"Mouse: spinXYBy "+a+" "+b+" "+c:"")},"~N,~N,~N");c(c$,"rotateZBy",function(a,b,c){this.tm.rotateZBy(a,b,c);this.refresh(2,this.sm.syncingMouse?"Mouse: rotateZBy "+a+(2147483647==b?"":" "+b+" "+c):"")},"~N,~N,~N");c(c$,"rotateSelected",function(a,b,c){this.isJmolDataFrame()||(this.tm.rotateXYBy(a,b,this.setMovableBitSet(c,!0)),this.refreshMeasures(!0),this.refresh(2,this.sm.syncingMouse?"Mouse: rotateMolecule "+a+" "+b:""))},"~N,~N,JU.BS");c(c$,
"setMovableBitSet",function(a,b){null==a&&(a=this.bsA());a=JU.BSUtil.copy(a);JU.BSUtil.andNot(a,this.getMotionFixedAtoms());b&&!this.g.allowMoveAtoms&&(a=this.ms.getMoleculeBitSet(a));return this.movableBitSet=a},"JU.BS,~B");c(c$,"translateXYBy",function(a,b){this.tm.translateXYBy(a,b);this.refresh(2,this.sm.syncingMouse?"Mouse: translateXYBy "+a+" "+b:"")},"~N,~N");j(c$,"rotateFront",function(){this.tm.resetRotation();this.refresh(1,"Viewer:rotateFront()")});c(c$,"translate",function(a,b,c,g){var f=
"\x00"==c?A(b):"%"==c?this.tm.percentToPixels(a,b):this.tm.angstromsToPixels(b*("n"==c?10:1));if(null!=g){if(0==f)return;this.tm.setSelectedTranslation(g,a,f)}else switch(a){case "X":case "x":"\x00"==c?this.tm.translateToPercent("x",b):this.tm.translateXYBy(f,0);break;case "Y":case "y":"\x00"==c?this.tm.translateToPercent("y",b):this.tm.translateXYBy(0,f);break;case "Z":case "z":"\x00"==c?this.tm.translateToPercent("z",b):this.tm.translateZBy(f)}this.refresh(1,"Viewer:translate()")},"~S,~N,~S,JU.BS");
c(c$,"slabByPixels",function(a){this.tm.slabByPercentagePoints(a);this.refresh(3,"slabByPixels")},"~N");c(c$,"depthByPixels",function(a){this.tm.depthByPercentagePoints(a);this.refresh(3,"depthByPixels")},"~N");c(c$,"slabDepthByPixels",function(a){this.tm.slabDepthByPercentagePoints(a);this.refresh(3,"slabDepthByPixels")},"~N");c(c$,"finalizeTransformParameters",function(){this.tm.finalizeTransformParameters();this.gdata.setSlabAndZShade(this.tm.slabValue,this.tm.depthValue,this.tm.zShadeEnabled?
this.tm.zSlabValue:2147483647,this.tm.zDepthValue,this.g.zShadePower)});c(c$,"getScalePixelsPerAngstrom",function(a){return this.tm.scalePixelsPerAngstrom*(a||!this.antialiased?1:0.5)},"~B");c(c$,"setSpin",function(a,b){if(JU.PT.isOneOf(a,";x;y;z;fps;X;Y;Z;FPS;")){var c="x;y;z;fps;X;Y;Z;FPS".indexOf(a);switch(c){case 0:this.tm.setSpinXYZ(b,NaN,NaN);break;case 2:this.tm.setSpinXYZ(NaN,b,NaN);break;case 4:this.tm.setSpinXYZ(NaN,NaN,b);break;default:this.tm.setSpinFps(b);break;case 10:this.tm.setNavXYZ(b,
NaN,NaN);break;case 12:this.tm.setNavXYZ(NaN,b,NaN);break;case 14:this.tm.setNavXYZ(NaN,NaN,b);break;case 16:this.tm.setNavFps(b)}this.g.setI((10>c?"spin":"nav")+a,b)}},"~S,~N");c(c$,"getSpinState",function(){return this.getStateCreator().getSpinState(!1)});c(c$,"getOrientationText",function(a,b,c){switch(a){case 1312817669:case 1814695966:case 1073741864:case 1111492629:case 1111492630:case 1111492631:case 134221850:null==c&&(c=this.bsA());if(c.isEmpty())return 1312817669==a?"0":1814695966==a?null:
new JU.Quat;c=this.ms.getBoundBoxOrientation(a,c);return"best"===b&&1312817669!=a?c.div(this.tm.getRotationQ()):c;case 1073742034:return this.stm.getSavedOrientationText(b);default:return this.tm.getOrientationText(a,"best"===b)}},"~N,~S,JU.BS");c(c$,"getCurrentColorRange",function(){return this.cm.getPropertyColorRange()});c(c$,"setDefaultColors",function(a){this.cm.setDefaultColors(a);this.g.setB("colorRasmol",a);this.g.setO("defaultColorScheme",a?"rasmol":"jmol")},"~B");c(c$,"setElementArgb",function(a,
b){this.g.setO("=color "+JU.Elements.elementNameFromNumber(a),JU.Escape.escapeColor(b));this.cm.setElementArgb(a,b)},"~N,~N");j(c$,"setVectorScale",function(a){this.g.setF("vectorScale",a);this.g.vectorScale=a},"~N");j(c$,"setVibrationScale",function(a){this.tm.setVibrationScale(a);this.g.vibrationScale=a;this.g.setF("vibrationScale",a)},"~N");j(c$,"setVibrationPeriod",function(a){this.tm.setVibrationPeriod(a);a=Math.abs(a);this.g.vibrationPeriod=a;this.g.setF("vibrationPeriod",a)},"~N");c(c$,"setObjectColor",
function(a,b){null==b||0==b.length||this.setObjectArgb(a,JU.CU.getArgbFromString(b))},"~S,~S");c(c$,"setObjectVisibility",function(a,b){var c=JV.StateManager.getObjectIdFromName(a);0<=c&&this.setShapeProperty(c,"display",b?Boolean.TRUE:Boolean.FALSE)},"~S,~B");c(c$,"setObjectArgb",function(a,b){var c=JV.StateManager.getObjectIdFromName(a);if(0>c)a.equalsIgnoreCase("axes")&&(this.setObjectArgb("axis1",b),this.setObjectArgb("axis2",b),this.setObjectArgb("axis3",b));else{this.g.objColors[c]=b;switch(c){case 0:this.gdata.setBackgroundArgb(b),
this.cm.setColixBackgroundContrast(b)}this.g.setO(a+"Color",JU.Escape.escapeColor(b))}},"~S,~N");c(c$,"setBackgroundImage",function(a,b){this.g.backgroundImageFileName=a;this.gdata.setBackgroundImage(b)},"~S,~O");c(c$,"getObjectColix",function(a){a=this.g.objColors[a];return 0==a?this.cm.colixBackgroundContrast:JU.C.getColix(a)},"~N");j(c$,"setColorBackground",function(a){this.setObjectColor("background",a)},"~S");j(c$,"getBackgroundArgb",function(){return this.g.objColors[0]});c(c$,"setObjectMad10",
function(a,b,c){var g=JV.StateManager.getObjectIdFromName(b.equalsIgnoreCase("axes")?"axis":b);if(!(0>g)){if(-2==c||-4==c){var f=c+3;c=this.getObjectMad10(g);0==c&&(c=f)}this.g.setB("show"+b,0!=c);this.g.objStateOn[g]=0!=c;0!=c&&(this.g.objMad10[g]=c,this.setShapeSize(a,c,null))}},"~N,~S,~N");c(c$,"getObjectMad10",function(a){return this.g.objStateOn[a]?this.g.objMad10[a]:0},"~N");c(c$,"setPropertyColorScheme",function(a,b,c){this.g.propertyColorScheme=a;a.startsWith("translucent ")&&(b=!0,a=a.substring(12).trim());
this.cm.setPropertyColorScheme(a,b,c)},"~S,~B,~B");c(c$,"getLightingState",function(){return this.getStateCreator().getLightingState(!0)});c(c$,"getColorPointForPropertyValue",function(a){return JU.CU.colorPtFromInt(this.gdata.getColorArgbOrGray(this.cm.ce.getColorIndex(a)),null)},"~N");c(c$,"select",function(a,b,c,g){b&&(a=this.getUndeletedGroupAtomBits(a));this.slm.select(a,c,g);this.shm.setShapeSizeBs(1,2147483647,null,null)},"JU.BS,~B,~N,~B");j(c$,"setSelectionSet",function(a){this.select(a,!1,
0,!0)},"JU.BS");c(c$,"selectBonds",function(a){this.shm.setShapeSizeBs(1,2147483647,null,a)},"JU.BS");c(c$,"displayAtoms",function(a,b,c,g,f){c&&(a=this.getUndeletedGroupAtomBits(a));b?this.slm.display(this.ms,a,g,f):this.slm.hide(this.ms,a,g,f)},"JU.BS,~B,~B,~N,~B");c(c$,"getUndeletedGroupAtomBits",function(a){a=this.ms.getAtoms(1086324742,a);JU.BSUtil.andNot(a,this.slm.bsDeleted);return a},"JU.BS");c(c$,"reportSelection",function(a){this.selectionHalosEnabled&&this.setTainted(!0);(this.isScriptQueued()||
this.g.debugScript)&&this.scriptStatus(a)},"~S");c(c$,"clearAtomSets",function(){this.slm.setSelectionSubset(null);this.definedAtomSets.clear();this.haveDisplay&&this.acm.exitMeasurementMode("clearAtomSets")});c(c$,"getDefinedAtomSet",function(a){a=this.definedAtomSets.get(a.toLowerCase());return n(a,JU.BS)?a:new JU.BS},"~S");j(c$,"selectAll",function(){this.slm.selectAll(!1)});j(c$,"clearSelection",function(){this.slm.clearSelection(!0);this.g.setB("hideNotSelected",!1)});c(c$,"bsA",function(){return this.slm.getSelectedAtoms()});
j(c$,"addSelectionListener",function(a){this.slm.addListener(a)},"J.api.JmolSelectionListener");j(c$,"removeSelectionListener",function(a){this.slm.addListener(a)},"J.api.JmolSelectionListener");c(c$,"getAtomBitSetEval",function(a,b){return this.allowScripting?this.getScriptManager().getAtomBitSetEval(a,b):new JU.BS},"J.api.JmolScriptEvaluator,~O");c(c$,"processTwoPointGesture",function(a){this.mouse.processTwoPointGesture(a)},"~A");c(c$,"processMouseEvent",function(a,b,c,g,f){return this.mouse.processEvent(a,
b,c,g,f)},"~N,~N,~N,~N,~N");c(c$,"getRubberBandSelection",function(){return this.haveDisplay?this.acm.getRubberBand():null});c(c$,"isBound",function(a,b){return this.haveDisplay&&this.acm.bnd(a,[b])},"~N,~N");c(c$,"getCursorX",function(){return this.haveDisplay?this.acm.getCurrentX():0});c(c$,"getCursorY",function(){return this.haveDisplay?this.acm.getCurrentY():0});c(c$,"getDefaultDirectory",function(){return this.g.defaultDirectory});c(c$,"getLocalUrl",function(a){return this.apiPlatform.getLocalUrl(a)},
"~S");c(c$,"getFileAsString",function(a){return this.getAsciiFileOrNull(a)},"~S");j(c$,"getBufferedInputStream",function(a){return this.fm.getBufferedInputStream(a)},"~S");c(c$,"setLoadParameters",function(a,b){null==a&&(a=new java.util.Hashtable);a.put("vwr",this);0<this.g.atomTypes.length&&a.put("atomTypes",this.g.atomTypes);a.containsKey("lattice")||a.put("lattice",this.g.ptDefaultLattice);this.g.applySymmetryToBonds&&a.put("applySymmetryToBonds",Boolean.TRUE);this.g.pdbGetHeader&&a.put("getHeader",
Boolean.TRUE);this.g.pdbSequential&&a.put("isSequential",Boolean.TRUE);this.g.legacyJavaFloat&&a.put("legacyJavaFloat",Boolean.TRUE);a.put("stateScriptVersionInt",Integer.$valueOf(this.stateScriptVersionInt));if(!a.containsKey("filter")){var c=this.g.defaultLoadFilter;0<c.length&&a.put("filter",c)}c=b&&!this.g.appendNew&&0<this.ms.ac;a.put("baseAtomIndex",Integer.$valueOf(b?this.ms.ac:0));a.put("baseBondIndex",Integer.$valueOf(b?this.ms.bondCount:0));a.put("baseModelIndex",Integer.$valueOf(0==this.ms.ac?
0:this.ms.mc+(c?-1:0)));c&&a.put("merging",Boolean.TRUE);return a},"java.util.Map,~B");j(c$,"openFileAsyncSpecial",function(a,b){this.getScriptManager().openFileAsync(a,b,!1)},"~S,~N");c(c$,"openFileDropped",function(a,b){this.getScriptManager().openFileAsync(a,16,b)},"~S,~B");j(c$,"openFile",function(a){this.zap(!0,!0,!1);return this.loadModelFromFileRepaint(null,a,null,null)},"~S");j(c$,"openFiles",function(a){this.zap(!0,!0,!1);return this.loadModelFromFileRepaint(null,null,a,null)},"~A");c(c$,
"openReader",function(a,b,c){this.zap(!0,!0,!1);return this.loadModelFromFileRepaint(a,b,null,c)},"~S,~S,~O");j(c$,"openDOM",function(a){this.zap(!0,!0,!1);return this.loadModelFromFileRepaint("?","?",null,a)},"~O");c(c$,"loadModelFromFileRepaint",function(a,b,c,g){a=this.loadModelFromFile(a,b,c,g,!1,null,null,null,0," ");this.refresh(1,"loadModelFromFileRepaint");return a},"~S,~S,~A,~O");c(c$,"loadModelFromFile",function(a,b,c,g,f,e,h,j,l,p){null==e&&(e=this.setLoadParameters(null,f));0!=l&&e.put("dataType",
JS.T.nameOf(l));" "!==p&&e.put("concatenate",Boolean.TRUE);var m=this.fm.getFileInfo();if(null!=c){if(null==h){h=(new JU.SB).append("load files");for(a=0;a<c.length;a++)h.append(0==a||null==p?" ":p).append("/*file*/$FILENAME"+(a+1)+"$");0<j.length()&&h.append(" /*options*/ ").append(j.toString())}a=Zystem.currentTimeMillis();j=this.fm.createAtomSetCollectionFromFiles(c,this.setLoadParameters(e,f),f);a=Zystem.currentTimeMillis()-a;JU.Logger.info("openFiles("+c.length+") "+a+" ms");c=e.get("fullPathNames");
p=e.get("fileTypes");h=h.toString();for(a=0;a<c.length;a++)b=c[a],null!=p&&null!=p[a]&&(b=p[a]+"::"+b),h=JU.PT.rep(h,"$FILENAME"+(a+1)+"$",JU.PT.esc(JV.FileManager.fixDOSName(b)));h=(new JU.SB).append(h)}else null==g?(null==h&&(h=(new JU.SB).append("load /*file*/$FILENAME$")),j=this.openFileFull(b,f,e,h)):j=n(g,java.io.Reader)||n(g,java.io.BufferedInputStream)||JU.AU.isAB(g)?this.fm.createAtomSetCollectionFromReader(a,b,g,this.setLoadParameters(e,f)):this.fm.createAtomSetCollectionFromDOM(g,this.setLoadParameters(e,
f));if(0!=l)return this.fm.setFileInfo(m),this.loadAtomDataAndReturnError(j,l);if(e.containsKey("isData"))return j;null!=h&&!n(j,String)&&(b=e.get("fullPathName"),null==b&&(b=""),e.containsKey("loadScript")&&(h=e.get("loadScript")),e.put("loadScript",h=(new JU.SB).append(JU.PT.rep(h.toString(),"$FILENAME$",JU.PT.esc(JV.FileManager.fixDOSName(b))))));return this.createModelSetAndReturnError(j,f,h,e)},"~S,~S,~A,~O,~B,java.util.Map,JU.SB,JU.SB,~N,~S");c(c$,"setLigandModel",function(a,b){null==this.ligandModels&&
(this.ligandModels=new java.util.Hashtable);this.ligandModels.put(a,b)},"~S,~S");c(c$,"getLigandModel",function(a,b,c,g){if(null==a){if(null!=this.ligandModelSet)for(a=this.ligandModels.entrySet().iterator();a.hasNext();){var f=a.next();n(f.getValue(),Boolean)&&a.remove()}return null}a=a.$replace("\\","/");var e=b.equals("ligand_");a=0<=a.indexOf("/cif")?a:e?a.toUpperCase():a.substring(a.lastIndexOf("/")+1);null==this.ligandModelSet&&(this.ligandModelSet=new java.util.Hashtable);this.ligandModelSet.put(a,
Boolean.TRUE);null==this.ligandModels&&(this.ligandModels=new java.util.Hashtable);var h=a.indexOf("|");0<=h&&(a=a.substring(a.indexOf("|")+1));var j=null==g?this.ligandModels.get(a):null,f=null;if(n(j,Boolean))return null;if(null==j&&(null==g||0<=h))j=this.ligandModels.get(a+c);var h=!1,l=null==j;if(l){if(e){f=this.setLoadFormat("#"+a,"#",!1);if(0==f.length)return null;this.scriptEcho("fetching "+f);j=this.getFileAsString3(f,!1,null)}else this.scriptEcho("fetching "+b),j=this.getFileAsString3(b,
!1,null),g=null==g?-1:j.indexOf(g),0<=g&&(j=j.substring(0,g));(h=0==j.indexOf("java."))||this.ligandModels.put(a+c,j)}if(!e)return l||this.scriptEcho(b+" loaded from cache"),j;!h&&n(j,String)&&(b=j,0!=b.length&&(c=new java.util.Hashtable,c.put("modelOnly",Boolean.TRUE),j=this.getModelAdapter().getAtomSetCollectionReader("ligand",null,JU.Rdr.getBR(b),c),h=n(j,String),h||(j=this.getModelAdapter().getAtomSetCollection(j),h=n(j,String),null!=f&&!h&&this.scriptEcho(this.getModelAdapter().getAtomSetCollectionAuxiliaryInfo(j).get("modelLoadNote")))));
return h?(this.scriptEcho(j.toString()),this.ligandModels.put(a,Boolean.FALSE),null):j},"~S,~S,~S,~S");c(c$,"openFileFull",function(a,b,c,g){if(null==a||a.equals("String[]"))return null;var f,e="openFile("+a+")";JU.Logger.startTimer(e);c=this.setLoadParameters(c,b);var h=a.startsWith("@"),j=c.containsKey("fileData");0==a.indexOf("$")&&c.put("smilesString",a.substring(1));var l=a.equals("string")||a.equals("Jmol Model Kit");f=null;if(j){if(f=c.get("fileData"),c.containsKey("isData"))return e=this.loadInlineScript(f,
"\x00",b,c),this.lastData=this.g.preserveState?this.getDataManager().createFileData(f):null,e}else if(l){f=this.ms.getInlineData(-1);if(null==f)if(this.g.modelKitMode)f="5\n\nC 0 0 0\nH .63 .63 .63\nH -.63 -.63 .63\nH -.63 .63 -.63\nH .63 -.63 -.63";else return"cannot find string data";null!=g&&c.put("loadScript",g=(new JU.SB).append(JU.PT.rep(g.toString(),"/*file*/$FILENAME$",'/*data*/data "model inline"\n'+f+'end "model inline"')))}null!=f?(b||this.zap(!0,!1,!1),!h&&(!j||l)&&this.getStateCreator().getInlineData(g,
f,b,c.get("appendToModelIndex"),this.g.defaultLoadFilter),f=this.fm.createAtomSetCollectionFromString(f,c,b)):f=this.fm.createAtomSetCollectionFromFile(a,c,b);JU.Logger.checkTimer(e,!1);return f},"~S,~B,java.util.Map,JU.SB");j(c$,"openStringInline",function(a){a=this.openStringInlineParamsAppend(a,null,!1);this.refresh(1,"openStringInline");return a},"~S");c(c$,"loadInline",function(a){return this.loadInlineScriptRepaint(a,this.g.inlineNewlineChar,!1)},"~S");c(c$,"loadInline",function(a,b){return this.loadInlineScriptRepaint(a,
b,!1)},"~S,~S");j(c$,"loadInlineAppend",function(a,b){return this.loadInlineScriptRepaint(a,"\x00",b)},"~S,~B");c(c$,"loadInlineScriptRepaint",function(a,b,c){a=this.loadInlineScript(a,b,c,null);this.refresh(1,"loadInlineScript");return a},"~S,~S,~B");c(c$,"loadInline",function(a){return this.loadInline(a,!1)},"~A");c(c$,"loadInline",function(a,b){if(null==a||0==a.length)return null;var c=this.openStringsInlineParamsAppend(a,new java.util.Hashtable,b);this.refresh(1,"loadInline String[]");return c},
"~A,~B");c(c$,"loadInline",function(a,b){if(null==a||0==a.size())return null;b||this.zap(!0,!1,!1);for(var c=new JU.Lst,g=0;g<a.size();g++)c.addLast(a.get(g));c=this.fm.createAtomSeCollectionFromArrayData(c,this.setLoadParameters(null,b),b);c=this.createModelSetAndReturnError(c,b,null,new java.util.Hashtable);this.refresh(1,"loadInline");return c},"java.util.List,~B");c(c$,"loadInlineScript",function(a,b,c,g){if(null==a||0==a.length)return null;a=JV.Viewer.fixInlineString(a,b);0!=b.charCodeAt(0)&&
JU.Logger.info("loading model inline, "+a.length+" bytes, with newLine character "+b.charCodeAt(0)+" isAppend="+c);JU.Logger.debugging&&JU.Logger.debug(a);b=this.getDataSeparator();var f;if(null!=b&&""!==b&&0<=(f=a.indexOf(b))&&0>a.indexOf("# Jmol state")){for(var e=2;0<=(f=a.indexOf(b,f+1));)e++;var h=Array(e),j=0,l=0;for(f=0;f<e;f++)j=a.indexOf(b,l),0>j&&(j=a.length),h[f]=a.substring(l,j),l=j+b.length;return this.openStringsInlineParamsAppend(h,g,c)}return this.openStringInlineParamsAppend(a,g,
c)},"~S,~S,~B,java.util.Map");c$.fixInlineString=c(c$,"fixInlineString",function(a,b){var c;0<=a.indexOf("\\/n")&&(a=JU.PT.rep(a,"\n",""),a=JU.PT.rep(a,"\\/n","\n"),b=String.fromCharCode(0));if(0!=b.charCodeAt(0)&&"\n"!=b){var g=0<=a.indexOf("\n"),f=a.length;for(c=0;c<f&&" "==a.charAt(c);++c);c<f&&a.charAt(c)==b&&(a=a.substring(c+1));a=g?JU.PT.rep(a,""+b,""):a.$replace(b,"\n")}return a},"~S,~S");c(c$,"openStringInlineParamsAppend",function(a,b,c){var g=this.getModelAdapter().getFileTypeName(JU.Rdr.getBR(a));
if(null==g)return"unknown file type";if(g.equals("spt"))return"cannot open script inline";b=this.setLoadParameters(b,c);var g=b.get("loadScript"),f=b.containsKey("isData");null==g&&(g=new JU.SB);c||this.zap(!0,!1,!1);f||this.getStateCreator().getInlineData(g,a,c,b.get("appendToModelIndex"),this.g.defaultLoadFilter);a=this.fm.createAtomSetCollectionFromString(a,b,c);return this.createModelSetAndReturnError(a,c,g,b)},"~S,java.util.Map,~B");c(c$,"openStringsInlineParamsAppend",function(a,b,c){var g=
new JU.SB;c||this.zap(!0,!1,!1);a=this.fm.createAtomSeCollectionFromStrings(a,g,this.setLoadParameters(b,c),c);return this.createModelSetAndReturnError(a,c,g,b)},"~A,java.util.Map,~B");c(c$,"getInlineChar",function(){return this.g.inlineNewlineChar});c(c$,"getDataSeparator",function(){return this.g.getParameter("dataseparator",!0)});c(c$,"createModelSetAndReturnError",function(a,b,c,g){JU.Logger.startTimer("creating model");var f=this.fm.getFullPathName(!1),e=this.fm.getFileName();null==c&&(this.setBooleanProperty("preserveState",
!1),c=(new JU.SB).append('load "???"'));if(n(a,String))return this.setFileLoadStatus(J.c.FIL.NOT_LOADED,f,null,null,a,null),this.displayLoadErrors&&(!b&&!a.equals("#CANCELED#")&&!a.startsWith(JV.JC.READER_NOT_FOUND))&&this.zapMsg(a),a;b?this.clearAtomSets():this.g.modelKitMode&&!e.equals("Jmol Model Kit")&&this.setModelKitMode(!1);this.setFileLoadStatus(J.c.FIL.CREATING_MODELSET,f,e,null,null,null);this.pushHoldRepaintWhy("createModelSet");this.setErrorMessage(null,null);try{var h=new JU.BS;this.mm.createModelSet(f,
e,c,a,h,b);if(0<h.cardinality()){var j=this.ms.getInfoM("jmolscript");if(this.ms.getMSInfoB("doMinimize"))try{var l=g.get("eval"),p=this.getAtomBitSet("_C & connected(3) & !connected(double)");p.and(h);0<=p.nextSetBit(0)&&h.or(this.addHydrogens(p,21));this.minimize(l,2147483647,0,h,null,0,29)}catch(m){if(!H(m,Exception))throw m;}else this.addHydrogens(h,5);null!=j&&this.ms.msInfo.put("jmolscript",j)}this.initializeModel(b)}catch(r){if(H(r,Error))this.handleError(r,!0),a=this.getShapeErrorState(),
a="ERROR creating model: "+r+(0==a.length?"":"|"+a),this.zapMsg(a),this.setErrorMessage(a,null);else throw r;}this.popHoldRepaint("createModelSet \u0001## REPAINT_IGNORE ##");a=this.getErrorMessage();this.setFileLoadStatus(J.c.FIL.CREATED,f,e,this.ms.modelSetName,a,g.get("async"));b&&(this.selectAll(),this.setTainted(!0),this.axesAreTainted=!0);JU.Logger.checkTimer("creating model",!1);Zystem.gc();return a},"~O,~B,JU.SB,java.util.Map");c(c$,"loadAtomDataAndReturnError",function(a,b){if(n(a,String))return a;
this.setErrorMessage(null,null);try{var c=this.mm.createAtomDataSet(a,b);switch(b){case 1145047050:null!=c&&this.runScriptCautiously(c);break;case 4166:this.setStatusFrameChanged(!0,!1);break;case 1648363544:this.shm.deleteVdwDependentShapes(null)}}catch(g){if(H(g,Error))this.handleError(g,!0),c=this.getShapeErrorState(),c="ERROR adding atom data: "+g+(0==c.length?"":"|"+c),this.zapMsg(c),this.setErrorMessage(c,null),this.setParallel(!1);else throw g;}return this.getErrorMessage()},"~O,~N");c(c$,
"getCurrentFileAsString",function(a){var b=this.fm.getFullPathName(!1);return b.equals("string")||b.equals("Jmol Model Kit")?this.ms.getInlineData(this.am.cmi):b.equals("String[]")?b:"JSNode"===b?"<DOM NODE>":this.getFileAsString4(b,-1,!0,!1,!1,a)},"~S");c(c$,"getFullPathNameOrError",function(a){var b=Array(2);this.fm.getFullPathNameOrError(a,!1,b);return b},"~S");c(c$,"getFileAsString3",function(a,b,c){return this.getFileAsString4(a,-1,!1,!1,b,c)},"~S,~B,~S");c(c$,"getFileAsString4",function(a,b,
c,g,f,e){if(null==a)return this.getCurrentFileAsString(e);a=u(-1,[a,null]);this.fm.getFileDataAsString(a,b,c,g,f);return a[1]},"~S,~N,~B,~B,~B,~S");c(c$,"getAsciiFileOrNull",function(a){a=u(-1,[a,null]);return this.fm.getFileDataAsString(a,-1,!1,!1,!1)?a[1]:null},"~S");c(c$,"autoCalculate",function(a,b){switch(a){case 1111490575:this.ms.getSurfaceDistanceMax();break;case 1111490574:this.ms.calculateStraightnessAll();break;case 1111490587:this.ms.calculateDssrProperty(b)}},"~N,~S");c(c$,"calculateStraightness",
function(){this.ms.haveStraightness=!1;this.ms.calculateStraightnessAll()});c(c$,"calculateSurface",function(a,b){null==a&&(a=this.bsA());if(3.4028235E38==b||-1==b)this.ms.addStateScript("calculate surfaceDistance "+(3.4028235E38==b?"FROM":"WITHIN"),null,a,null,"",!1,!0);return this.ms.calculateSurface(a,b)},"JU.BS,~N");c(c$,"getStructureList",function(){return this.g.getStructureList()});c(c$,"setStructureList",function(a,b){this.g.setStructureList(a,b);this.ms.setStructureList(this.getStructureList())},
"~A,J.c.STR");c(c$,"calculateStructures",function(a,b,c,g){null==a&&(a=this.bsA());return this.ms.calculateStructures(a,b,!this.am.animationOn,this.g.dsspCalcHydrogen,c,g)},"JU.BS,~B,~B,~N");c(c$,"getAnnotationParser",function(a){return a?null==this.dssrParser?this.dssrParser=J.api.Interface.getOption("dssx.DSSR1",this,"script"):this.dssrParser:null==this.annotationParser?this.annotationParser=J.api.Interface.getOption("dssx.AnnotationParser",this,"script"):this.annotationParser},"~B");j(c$,"getSelectedAtomIterator",
function(a,b,c,g){return this.ms.getSelectedAtomIterator(a,b,c,!1,g)},"JU.BS,~B,~B,~B");j(c$,"setIteratorForAtom",function(a,b,c){this.ms.setIteratorForAtom(a,-1,b,c,null)},"J.api.AtomIndexIterator,~N,~N");j(c$,"setIteratorForPoint",function(a,b,c,g){this.ms.setIteratorForPoint(a,b,c,g)},"J.api.AtomIndexIterator,~N,JU.T3,~N");j(c$,"fillAtomData",function(a,b){a.programInfo="Jmol Version "+JV.Viewer.getJmolVersion();a.fileName=this.fm.getFileName();this.ms.fillAtomData(a,b)},"J.atomdata.AtomData,~N");
c(c$,"addStateScript",function(a,b,c){return this.ms.addStateScript(a,null,null,null,null,b,c)},"~S,~B,~B");c(c$,"getMinimizer",function(a){return null==this.minimizer&&a?(this.minimizer=J.api.Interface.getInterface("JM.Minimizer",this,"script")).setProperty("vwr",this):this.minimizer},"~B");c(c$,"getSmilesMatcher",function(){return null==this.smilesMatcher?this.smilesMatcher=J.api.Interface.getInterface("JS.SmilesMatcher",this,"script"):this.smilesMatcher});c(c$,"clearModelDependentObjects",function(){this.setFrameOffsets(null,
!1);this.stopMinimization();this.smilesMatcher=this.minimizer=null});c(c$,"zap",function(a,b,c){this.clearThreads();null==this.mm.modelSet?this.mm.zap():(this.ligandModelSet=null,this.clearModelDependentObjects(),this.fm.clear(),this.clearRepaintManager(-1),this.am.clear(),this.tm.clear(),this.slm.clear(),this.clearAllMeasurements(),this.clearMinimization(),this.gdata.clear(),this.mm.zap(),null!=this.scm&&this.scm.clear(!1),null!=this.nmrCalculation&&this.getNMRCalculation().setChemicalShiftReference(null,
0),this.haveDisplay&&(this.mouse.clear(),this.clearTimeouts(),this.acm.clear()),this.stm.clear(this.g),this.tempArray.clear(),this.chainMap.clear(),this.chainList.clear(),this.chainCaseSpecified=!1,this.definedAtomSets.clear(),this.lastData=null,null!=this.dm&&this.dm.clear(),this.setBooleanProperty("legacyjavafloat",!1),b&&(c&&this.g.removeParam("_pngjFile"),c&&this.g.modelKitMode&&this.loadDefaultModelKitModel(null),this.undoClear()),Zystem.gc());this.initializeModel(!1);a&&this.setFileLoadStatus(J.c.FIL.ZAPPED,
null,b?"resetUndo":this.getZapName(),null,null,null);JU.Logger.debugging&&JU.Logger.checkMemory()},"~B,~B,~B");c(c$,"loadDefaultModelKitModel",function(a){this.openStringInlineParamsAppend(this.getModelkit(!1).getDefaultModel(),a,!0);this.setRotationRadius(5,!0);this.setStringProperty("picking","assignAtom_C");this.setStringProperty("picking","assignBond_p")},"java.util.Map");c(c$,"zapMsg",function(a){this.zap(!0,!0,!1);this.echoMessage(a)},"~S");c(c$,"echoMessage",function(a){this.shm.loadShape(31);
this.setShapeProperty(31,"font",this.getFont3D("SansSerif","Plain",20));this.setShapeProperty(31,"target","error");this.setShapeProperty(31,"text",a)},"~S");c(c$,"initializeModel",function(a){this.clearThreads();a?this.am.initializePointers(1):(this.reset(!0),this.selectAll(),null!=this.modelkit&&this.modelkit.initializeForModel(),this.movingSelected=!1,this.slm.noneSelected=Boolean.FALSE,this.setHoverEnabled(!0),this.setSelectionHalosEnabled(!1),this.tm.setCenter(),this.am.initializePointers(1),
this.setBooleanProperty("multipleBondBananas",!1),this.ms.getMSInfoB("isPyMOL")||(this.clearAtomSets(),this.setCurrentModelIndex(0)),this.setBackgroundModelIndex(-1),this.setFrankOn(this.getShowFrank()),this.startHoverWatcher(!0),this.setTainted(!0),this.finalizeTransformParameters())},"~B");c(c$,"startHoverWatcher",function(a){a&&this.inMotion||(!this.haveDisplay||a&&(!this.hoverEnabled&&!this.sm.haveHoverCallback()||this.am.animationOn))||this.acm.startHoverWatcher(a)},"~B");j(c$,"getModelSetPathName",
function(){return this.mm.modelSetPathName});j(c$,"getModelSetFileName",function(){return null==this.mm.fileName?this.getZapName():this.mm.fileName});c(c$,"getUnitCellInfoText",function(){var a=this.getCurrentUnitCell();return null==a?"not applicable":a.getUnitCellInfo()});c(c$,"getUnitCellInfo",function(a){var b=this.getCurrentUnitCell();return null==b?NaN:b.getUnitCellInfoType(a)},"~N");c(c$,"getV0abc",function(a,b){var c=0>a?this.getCurrentUnitCell():this.getUnitCell(a);return null==c?null:c.getV0abc(b)},
"~N,~O");c(c$,"getCurrentUnitCell",function(){var a=this.am.getUnitCellAtomIndex();return 0<=a?this.ms.getUnitCellForAtom(a):this.getUnitCell(this.am.cmi)});c(c$,"getUnitCell",function(a){if(0<=a)return this.ms.getUnitCell(a);a=this.getVisibleFramesBitSet();for(var b=null,c=a.nextSetBit(0);0<=c;c=a.nextSetBit(c+1)){var g=this.ms.getUnitCell(c);if(null!=g)if(null==b)b=g;else if(!b.unitCellEquals(g))return null}return b},"~N");c(c$,"getPolymerPointsAndVectors",function(a,b){this.ms.getPolymerPointsAndVectors(a,
b,this.g.traceAlpha,this.g.sheetSmoothing)},"JU.BS,JU.Lst");c(c$,"getHybridizationAndAxes",function(a,b,c,g){return this.ms.getHybridizationAndAxes(a,0,b,c,g,!0,!0,!1)},"~N,JU.V3,JU.V3,~S");c(c$,"getAllAtoms",function(){return this.getModelUndeletedAtomsBitSet(-1)});c(c$,"getFrameAtoms",function(){return this.getModelUndeletedAtomsBitSetBs(this.getVisibleFramesBitSet())});j(c$,"getVisibleFramesBitSet",function(){var a=JU.BSUtil.copy(this.am.bsVisibleModels);null!=this.ms.trajectory&&this.ms.trajectory.selectDisplayed(a);
return a});c(c$,"getModelUndeletedAtomsBitSet",function(a){return this.slm.excludeAtoms(this.ms.getModelAtomBitSetIncludingDeleted(a,!0),!1)},"~N");c(c$,"getModelUndeletedAtomsBitSetBs",function(a){return this.slm.excludeAtoms(this.ms.getModelAtomBitSetIncludingDeletedBs(a),!1)},"JU.BS");j(c$,"getBoundBoxCenter",function(){return this.ms.getBoundBoxCenter(this.am.cmi)});c(c$,"calcBoundBoxDimensions",function(a,b){this.ms.calcBoundBoxDimensions(a,b);this.axesAreTainted=!0},"JU.BS,~N");j(c$,"getBoundBoxCornerVector",
function(){return this.ms.getBoundBoxCornerVector()});j(c$,"getModelSetProperties",function(){return this.ms.modelSetProperties});j(c$,"getModelProperties",function(a){return this.ms.am[a].properties},"~N");c(c$,"getModelForAtomIndex",function(a){return this.ms.am[this.ms.at[a].mi]},"~N");j(c$,"getModelSetAuxiliaryInfo",function(){return this.ms.getAuxiliaryInfo(null)});j(c$,"getModelNumber",function(a){return 0>a?a:this.ms.getModelNumber(a)},"~N");c(c$,"getModelFileNumber",function(a){return 0>a?
0:this.ms.modelFileNumbers[a]},"~N");j(c$,"getModelNumberDotted",function(a){return 0>a?"0":this.ms.getModelNumberDotted(a)},"~N");j(c$,"getModelName",function(a){return this.ms.getModelName(a)},"~N");c(c$,"modelHasVibrationVectors",function(a){return 0<=this.ms.getLastVibrationVector(a,4166)},"~N");c(c$,"getBondsForSelectedAtoms",function(a){return this.ms.getBondsForSelectedAtoms(a,this.g.bondModeOr||1==JU.BSUtil.cardinalityOf(a))},"JU.BS");c(c$,"frankClicked",function(a,b){return!this.g.disablePopupMenu&&
this.getShowFrank()&&this.shm.checkFrankclicked(a,b)},"~N,~N");c(c$,"frankClickedModelKit",function(a,b){return!this.g.disablePopupMenu&&this.g.modelKitMode&&0<=a&&0<=b&&40>a&&104>b},"~N,~N");j(c$,"findNearestAtomIndex",function(a,b){return this.findNearestAtomIndexMovable(a,b,!1)},"~N,~N");c(c$,"findNearestAtomIndexMovable",function(a,b,c){return!this.g.atomPicking?-1:this.ms.findNearestAtomIndex(a,b,c?this.slm.getMotionFixedAtoms():null,this.g.minPixelSelRadius)},"~N,~N,~B");c(c$,"toCartesian",
function(a,b){var c=this.getCurrentUnitCell();null!=c&&(c.toCartesian(a,b),this.g.legacyJavaFloat||JU.PT.fixPtFloats(a,1E4))},"JU.T3,~B");c(c$,"toFractional",function(a,b){var c=this.getCurrentUnitCell();null!=c&&(c.toFractional(a,b),this.g.legacyJavaFloat||JU.PT.fixPtFloats(a,1E5))},"JU.T3,~B");c(c$,"toUnitCell",function(a,b){var c=this.getCurrentUnitCell();null!=c&&c.toUnitCell(a,b)},"JU.P3,JU.P3");c(c$,"setCurrentCage",function(a){a=u(-1,[a,null]);this.shm.getShapePropertyData(24,"unitCell",a);
this.ms.setModelCage(this.am.cmi,a[1])},"~S");c(c$,"addUnitCellOffset",function(a){var b=this.getCurrentUnitCell();null!=b&&a.add(b.getCartesianOffset())},"JU.P3");c(c$,"setAtomData",function(a,b,c,g){this.ms.setAtomData(a,b,c,g);2==a&&this.checkCoordinatesChanged();this.refreshMeasures(!0)},"~N,~S,~S,~B");j(c$,"setCenterSelected",function(){this.setCenterBitSet(this.bsA(),!0)});c(c$,"setApplySymmetryToBonds",function(a){this.g.applySymmetryToBonds=a},"~B");j(c$,"setBondTolerance",function(a){this.g.setF("bondTolerance",
a);this.g.bondTolerance=a},"~N");j(c$,"setMinBondDistance",function(a){this.g.setF("minBondDistance",a);this.g.minBondDistance=a},"~N");c(c$,"getAtomsNearPt",function(a,b){var c=new JU.BS;this.ms.getAtomsWithin(a,b,c,-1);return c},"~N,JU.P3");c(c$,"getBranchBitSet",function(a,b,c){return 0>a||a>=this.ms.ac?new JU.BS:JU.JmolMolecule.getBranchBitSet(this.ms.at,a,this.getModelUndeletedAtomsBitSet(this.ms.at[a].mi),null,b,c,!0)},"~N,~N,~B");j(c$,"getElementsPresentBitSet",function(a){return this.ms.getElementsPresentBitSet(a)},
"~N");c(c$,"getFileHeader",function(){return this.ms.getFileHeader(this.am.cmi)});c(c$,"getFileData",function(){return this.ms.getFileData(this.am.cmi)});c(c$,"getCifData",function(a){return this.readCifData(this.ms.getModelFileName(a),this.ms.getModelFileType(a).toUpperCase())},"~N");c(c$,"readCifData",function(a,b){var c=null==a?this.ms.getModelFileName(this.am.cmi):a;if(null==b&&null!=c&&0<=c.toUpperCase().indexOf("BCIF")){c=this.fm.getBufferedInputStream(c);try{return J.api.Interface.getInterface("JU.MessagePackReader",
this,"script").getMapForStream(c)}catch(g){if(H(g,Exception))return g.printStackTrace(),new java.util.Hashtable;throw g;}}c=null==a||0==a.length?this.getCurrentFileAsString("script"):this.getFileAsString3(a,!1,null);if(null==c||2>c.length)return null;c=JU.Rdr.getBR(c);null==b&&(b=this.getModelAdapter().getFileTypeName(c));return null==b?null:this.readCifData(null,c,b)},"~S,~S");c(c$,"readCifData",function(a,b,c){null==b&&(b=this.getFileAsString(a));a=n(b,java.io.BufferedReader)?b:JU.Rdr.getBR(b);
return JU.Rdr.readCifData(J.api.Interface.getInterface("Cif2".equals(c)?"J.adapter.readers.cif.Cif2DataParser":"JU.CifDataParser",this,"script"),a)},"~S,~O,~S");c(c$,"getStateCreator",function(){null==this.jsc&&(this.jsc=J.api.Interface.getInterface("JV.StateCreator",this,"script")).setViewer(this);return this.jsc});c(c$,"getWrappedStateScript",function(){return this.getOutputManager().getWrappedState(null,null,null,null)});j(c$,"getStateInfo",function(){return this.getStateInfo3(null,0,0)});c(c$,
"getStateInfo3",function(a,b,c){return this.g.preserveState?this.getStateCreator().getStateScript(a,b,c):""},"~S,~N,~N");c(c$,"getStructureState",function(){return this.getStateCreator().getModelState(null,!1,!0)});c(c$,"getCoordinateState",function(a){return this.getStateCreator().getAtomicPropertyState(2,a)},"JU.BS");c(c$,"setCurrentColorRange",function(a){var b=this.getDataObj(a,null,1);a=null==b?null:this.getDataObj(a,null,-1)[2];null!=a&&this.g.rangeSelected&&a.and(this.bsA());this.cm.setPropertyColorRangeData(b,
a)},"~S");c(c$,"setData",function(a,b,c,g,f,e,h){this.getDataManager().setData(a,this.lastData=b,c,this.ms.ac,g,f,e,h)},"~S,~A,~N,~N,~N,~N,~N");c(c$,"getDataObj",function(a,b,c){return null==a&&-2==c?this.lastData:this.getDataManager().getData(a,b,c)},"~S,JU.BS,~N");c(c$,"autoHbond",function(a,b,c){null==a&&(a=b=this.bsA());return this.ms.autoHbond(a,b,c)},"JU.BS,JU.BS,~B");c(c$,"getDefaultMeasurementLabel",function(a){switch(a){case 2:return this.g.defaultDistanceLabel;case 3:return this.g.defaultAngleLabel;
default:return this.g.defaultTorsionLabel}},"~N");j(c$,"getMeasurementCount",function(){var a=this.getShapePropertyAsInt(6,"count");return 0>=a?0:a});j(c$,"getMeasurementStringValue",function(a){return""+this.shm.getShapePropertyIndex(6,"stringValue",a)},"~N");c(c$,"getMeasurementInfoAsString",function(){return this.getShapeProperty(6,"infostring")});j(c$,"getMeasurementCountPlusIndices",function(a){return this.shm.getShapePropertyIndex(6,"countPlusIndices",a)},"~N");c(c$,"setPendingMeasurement",
function(a){this.shm.loadShape(6);this.setShapeProperty(6,"pending",a)},"JM.MeasurementPending");c(c$,"getPendingMeasurement",function(){return this.getShapeProperty(6,"pending")});c(c$,"clearAllMeasurements",function(){this.setShapeProperty(6,"clear",null)});j(c$,"clearMeasurements",function(){this.evalString("measures delete")});c(c$,"setAnimation",function(a){switch(a){case 1073742098:this.am.reverseAnimation();case 1073742096:case 4143:this.am.animationOn||this.am.resumeAnimation();break;case 20487:this.am.animationOn&&
!this.am.animationPaused&&this.am.pauseAnimation();break;case 1073742037:this.am.setAnimationNext();break;case 1073742108:this.am.setAnimationPrevious();break;case 1073741942:case 1073742125:this.am.rewindAnimation();break;case 1073741993:this.am.setAnimationLast()}},"~N");j(c$,"setAnimationFps",function(a){this.am.setAnimationFps(a)},"~N");c(c$,"setAnimationMode",function(a){a.equalsIgnoreCase("once")?this.am.setAnimationReplayMode(1073742070,0,0):a.equalsIgnoreCase("loop")?this.am.setAnimationReplayMode(528411,
1,1):a.startsWith("pal")&&this.am.setAnimationReplayMode(1073742082,1,1)},"~S");c(c$,"setAnimationOn",function(a){a!=this.am.animationOn&&this.am.setAnimationOn(a)},"~B");c(c$,"setAnimationRange",function(a,b){this.am.setAnimationRange(a,b)},"~N,~N");c(c$,"defineAtomSets",function(a){this.definedAtomSets.putAll(a)},"java.util.Map");c(c$,"setAnimDisplay",function(a){this.am.setDisplay(a);this.am.animationOn||this.am.morph(this.am.currentMorphModel+1)},"JU.BS");c(c$,"setCurrentModelIndex",function(a){-2147483648==
a?(this.prevFrame=-2147483648,this.setCurrentModelIndexClear(this.am.cmi,!0)):this.am.setModel(a,!0)},"~N");c(c$,"getTrajectoryState",function(){return null==this.ms.trajectory?"":this.ms.trajectory.getState()});c(c$,"setFrameOffsets",function(a,b){this.tm.bsFrameOffsets=null;b?this.clearModelDependentObjects():this.tm.bsFrameOffsets=a;this.tm.frameOffsets=this.ms.getFrameOffsets(a,b)},"JU.BS,~B");c(c$,"setCurrentModelIndexClear",function(a,b){this.am.setModel(a,b)},"~N,~B");c(c$,"haveFileSet",function(){return 1<
this.ms.mc&&2E6<this.getModelNumber(2147483647)});c(c$,"setBackgroundModelIndex",function(a){this.am.setBackgroundModelIndex(a);this.g.setO("backgroundModel",this.ms.getModelNumberDotted(a))},"~N");c(c$,"setFrameVariables",function(){this.g.setO("animationMode",JS.T.nameOf(this.am.animationReplayMode));this.g.setI("animationFps",this.am.animationFps);this.g.setO("_firstFrame",this.am.getModelSpecial(-1));this.g.setO("_lastFrame",this.am.getModelSpecial(1));this.g.setF("_animTimeSec",this.am.getAnimRunTimeSeconds());
this.g.setB("_animMovie",this.am.isMovie)});c(c$,"getInMotion",function(a){return this.inMotion||a&&this.am.animationOn},"~B");j(c$,"getMotionEventNumber",function(){return this.motionEventNumber});j(c$,"setInMotion",function(a){(new Boolean(this.inMotion^a)).valueOf()&&(this.inMotion=a,this.resizeImage(0,0,!1,!1,!0),a?(this.startHoverWatcher(!1),++this.motionEventNumber):(this.startHoverWatcher(!0),this.refresh(3,"vwr setInMotion "+a)))},"~B");c(c$,"setRefreshing",function(a){this.refreshing=a},
"~B");c(c$,"getRefreshing",function(){return this.refreshing});j(c$,"pushHoldRepaint",function(){this.pushHoldRepaintWhy(null)});c(c$,"pushHoldRepaintWhy",function(a){null!=this.rm&&this.rm.pushHoldRepaint(a)},"~S");j(c$,"popHoldRepaint",function(a){null!=this.rm&&this.rm.popHoldRepaint(0>a.indexOf("\u0001## REPAINT_IGNORE ##"),a)},"~S");j(c$,"refresh",function(a,b){if(!(null==this.rm||!this.refreshing||6==a&&this.getInMotion(!0)||!this.isWebGL&&7==a)){if(this.isWebGL)switch(a){case 1:case 2:case 7:this.tm.finalizeTransformParameters();
if(null==this.html5Applet)return;this.html5Applet._refresh();if(7==a)return}else this.rm.repaintIfReady("refresh "+a+" "+b);this.sm.doSync()&&this.sm.setSync(2==a?b:null)}},"~N,~S");c(c$,"requestRepaintAndWait",function(a){null!=this.rm&&(this.haveDisplay?(this.rm.requestRepaintAndWait(a),this.setSync()):(this.setModelVisibility(),this.shm.finalizeAtoms(null,!0)))},"~S");c(c$,"clearShapeRenderers",function(){this.clearRepaintManager(-1)});c(c$,"isRepaintPending",function(){return null==this.rm?!1:
this.rm.isRepaintPending()});j(c$,"notifyViewerRepaintDone",function(){null!=this.rm&&this.rm.repaintDone();this.am.repaintDone()});c(c$,"areAxesTainted",function(){var a=this.axesAreTainted;this.axesAreTainted=!1;return a});j(c$,"generateOutputForExport",function(a){return this.noGraphicsAllowed||null==this.rm?null:this.getOutputManager().getOutputFromExport(a)},"java.util.Map");c(c$,"clearRepaintManager",function(a){null!=this.rm&&this.rm.clear(a)},"~N");c(c$,"renderScreenImage",function(a,b,c){this.renderScreenImageStereo(a,
!1,b,c)},"~O,~N,~N");c(c$,"renderScreenImageStereo",function(a,b,c,g){this.updateWindow(c,g)&&(!b||null==this.gRight?this.getScreenImageBuffer(a,!1):(this.drawImage(this.gRight,this.getImage(!0,!1),0,0,this.tm.stereoDoubleDTI),this.drawImage(a,this.getImage(!1,!1),0,0,this.tm.stereoDoubleDTI)));null!=this.captureParams&&Boolean.FALSE!==this.captureParams.get("captureEnabled")&&(this.captureParams.remove("imagePixels"),a=this.captureParams.get("endTime").longValue(),0<a&&Zystem.currentTimeMillis()+
50>a&&this.captureParams.put("captureMode","end"),this.processWriteOrCapture(this.captureParams));this.notifyViewerRepaintDone()},"~O,~B,~N,~N");c(c$,"updateJS",function(){this.isWebGL?(null==this.jsParams&&(this.jsParams=new java.util.Hashtable,this.jsParams.put("type","JS")),this.updateWindow(0,0)&&this.render(),this.notifyViewerRepaintDone()):this.isStereoSlave||this.renderScreenImageStereo(this.apiPlatform.getGraphics(null),!0,0,0)});c(c$,"updateJSView",function(a,b){if(null!=this.html5Applet){var c=
this.html5Applet,g=!0;(g=null!=c&&null!=c._viewSet)&&this.html5Applet._atomPickedCallback(a,b)}},"~N,~N");j(c$,"evalFile",function(a){return this.allowScripting&&null!=this.getScriptManager()?this.scm.evalFile(a):null},"~S");c(c$,"getInsertedCommand",function(){var a=this.insertedCommand;this.insertedCommand="";JU.Logger.debugging&&""!==a&&JU.Logger.debug("inserting: "+a);return a});j(c$,"script",function(a){return this.evalStringQuietSync(a,!1,!0)},"~S");j(c$,"evalString",function(a){return this.evalStringQuietSync(a,
!1,!0)},"~S");j(c$,"evalStringQuiet",function(a){return this.evalStringQuietSync(a,!0,!0)},"~S");c(c$,"evalStringQuietSync",function(a,b,c){return null==this.getScriptManager()?null:this.scm.evalStringQuietSync(a,b,c)},"~S,~B,~B");c(c$,"clearScriptQueue",function(){null!=this.scm&&this.scm.clearQueue()});c(c$,"setScriptQueue",function(a){(this.g.useScriptQueue=a)||this.clearScriptQueue()},"~B");j(c$,"checkHalt",function(a,b){return null!=this.scm&&this.scm.checkHalt(a,b)},"~S,~B");j(c$,"scriptWait",
function(a){return this.evalWait("JSON",a,"+scriptStarted,+scriptStatus,+scriptEcho,+scriptTerminated")},"~S");j(c$,"scriptWaitStatus",function(a,b){return this.evalWait("object",a,b)},"~S,~S");c(c$,"evalWait",function(a,b,c){if(null==this.getScriptManager())return null;this.scm.waitForQueue();var g=J.i18n.GT.setDoTranslate(!1);a=this.evalStringWaitStatusQueued(a,b,c,!1,!1);J.i18n.GT.setDoTranslate(g);return a},"~S,~S,~S");c(c$,"exitJmol",function(){if(!this.isApplet||this.isJNLP){if(null!=this.headlessImageParams)try{this.headless&&
this.outputToFile(this.headlessImageParams)}catch(a){if(!H(a,Exception))throw a;}JU.Logger.debugging&&JU.Logger.debug("exitJmol -- exiting");Zystem.out.flush();Zystem.exit(0)}});c(c$,"scriptCheckRet",function(a,b){return null==this.getScriptManager()?null:this.scm.scriptCheckRet(a,b)},"~S,~B");j(c$,"scriptCheck",function(a){return this.scriptCheckRet(a,!1)},"~S");j(c$,"isScriptExecuting",function(){return null!=this.eval&&this.eval.isExecuting()});j(c$,"haltScriptExecution",function(){null!=this.eval&&
(this.eval.haltExecution(),this.eval.stopScriptThreads());this.setStringPropertyTok("pathForAllFiles",545259572,"");this.clearTimeouts()});c(c$,"pauseScriptExecution",function(){null!=this.eval&&this.eval.pauseExecution(!0)});c(c$,"resolveDatabaseFormat",function(a){return JV.Viewer.hasDatabasePrefix(a)||0<=a.indexOf("cactus.nci.nih.gov/chemical/structure")?this.setLoadFormat(a,a.charAt(0),!1):a},"~S");c$.hasDatabasePrefix=c(c$,"hasDatabasePrefix",function(a){return 0!=a.length&&JV.Viewer.isDatabaseCode(a.charAt(0))},
"~S");c$.isDatabaseCode=c(c$,"isDatabaseCode",function(a){return"*"==a||"$"==a||"="==a||":"==a},"~S");c(c$,"setLoadFormat",function(a,b,c){var g=null,f=a.substring(1);switch(b){case "c":return a;case "h":return this.checkCIR(!1),this.g.nihResolverFormat+a.substring(a.indexOf("/structure")+10);case "=":if(a.startsWith("=="))f=f.substring(1);else if(0<f.indexOf("/"))try{var e=f.indexOf("/"),h=f.substring(0,e),f=JV.JC.resolveDataBase(h,f.substring(e+1),null);null!=f&&f.startsWith("'")&&(f=this.evaluateExpression(f).toString());
return null==f||0==f.length?a:f}catch(j){if(H(j,Exception))return a;throw j;}else{if(f.endsWith(".mmtf"))return f=f.substring(0,f.indexOf(".mmtf")),JV.JC.resolveDataBase("mmtf",f.toUpperCase(),null);g=this.g.loadFormat}case "#":return null==g&&(g=this.g.pdbLoadLigandFormat),JV.JC.resolveDataBase(null,f,g);case "*":e=a.lastIndexOf("/");if(a.startsWith("*dom/"))return f=a.substring(e+1),g=4<e?a.substring(5):"mappings",JU.PT.rep(JV.JC.resolveDataBase("map",f,null),"%TYPE",g);if(a.startsWith("*val/"))return f=
a.substring(e+1),g=4<e?a.substring(5):"validation/outliers/all",JU.PT.rep(JV.JC.resolveDataBase("map",f,null),"%TYPE",g);if(a.startsWith("*rna3d/"))return f=a.substring(e+1),g=6<e?a.substring(6):"loops",JU.PT.rep(JV.JC.resolveDataBase("rna3d",f,null),"%TYPE",g);if(a.startsWith("*dssr--"))return f=a.substring(e+1),f=JV.JC.resolveDataBase("dssr",f,null),f+"%20"+JU.PT.rep(a.substring(5,e)," ","%20");if(a.startsWith("*dssr/"))return f=a.substring(e+1),JV.JC.resolveDataBase("dssr",f,null);if(a.startsWith("*dssr1/"))return f=
a.substring(e+1),JV.JC.resolveDataBase("dssr1",f,null);b="pdbe";5==f.length&&"*"==f.charAt(4)&&(b="pdbe2",f=f.substring(0,4));return JV.JC.resolveDataBase(b,f,null);case ":":g=this.g.pubChemFormat;if(f.equals(""))try{f="smiles:"+this.getOpenSmiles(this.bsA())}catch(l){if(!H(l,Exception))throw l;}b=f.toLowerCase();e=-2147483648;try{e=Integer.parseInt(f)}catch(n){if(!H(n,Exception))throw n;}-2147483648!=e?f="cid/"+e:b.startsWith("smiles:")?(g+="?POST?smiles="+f.substring(7),f="smiles"):f.startsWith("cid:")||
f.startsWith("inchikey:")||f.startsWith("cas:")?f=f.$replace(":","/"):(b.startsWith("name:")&&(f=f.substring(5)),f="name/"+JU.PT.escapeUrl(f));return JU.PT.formatStringS(g,"FILE",f);case "$":if(this.checkCIR(!1),a.equals("$"))try{f=this.getOpenSmiles(this.bsA())}catch(m){if(!H(m,Exception))throw m;}else if(a.startsWith("$$")){f=f.substring(1);if(0==f.length)try{f=this.getOpenSmiles(this.bsA())}catch(p){if(!H(p,Exception))throw p;}g=JU.PT.rep(this.g.smilesUrlFormat,"get3d=true","get3d=false");return JU.PT.formatStringS(g,
"FILE",JU.PT.escapeUrl(f))}case "M":case "N":case "2":case "I":case "K":case "S":case "T":case "/":f=JU.PT.escapeUrl(f);switch(b){case "M":case "N":g=this.g.nihResolverFormat+"/%FILE/names";break;case "2":g=this.g.nihResolverFormat+"/%FILE/image";break;case "I":case "T":g=this.g.nihResolverFormat+"/%FILE/stdinchi";break;case "K":g=this.g.nihResolverFormat+"/%FILE/inchikey";break;case "S":g=this.g.nihResolverFormat+"/%FILE/stdinchikey";break;case "/":g=this.g.nihResolverFormat+"/%FILE/";break;default:g=
this.g.smilesUrlFormat}return(c?"MOL3D::":"")+JU.PT.formatStringS(g,"FILE",f);case "?":case "-":case "_":(g=f.startsWith("*")||f.startsWith("="))&&(f=f.substring(1));a=null;e=f.indexOf(".");0<=e&&(a=f.substring(e+1),f=f.substring(0,e));(c=f.startsWith("density"))&&(f="em"+f.substring(7));if(f.equals("emdb")||f.equals("em"))f+="/";f.startsWith("em/")&&(f="emdb"+f.substring(2));if(f.startsWith("emdb/")){f=f.substring(5);0==f.length?f="=":f.startsWith("*")&&(f="="+f.substring(1));h="#-sigma=10";if(f.startsWith("=")){f=
f.equals("=")?this.getPdbID():f.substring(1);if(null==f||"?"==b)break;var e=JV.JC.resolveDataBase("emdbquery",f,null),q=this.fm.cacheGet(e,!1);if(null==q){this.showString("retrieving "+e,!1);q=this.getFileAsString(e);if(null==q){this.showString("EM retrieve failed for "+f,!1);if(!c)return null;q="FAILED"}else this.showString(q,!1);this.fm.cachePut(e,q)}e=q.indexOf("EMD-");if(0<=e)f=q.substring(e+4),e=f.indexOf("\n"),0<e&&(f=f.substring(0,e)),e=f.indexOf(","),0<e&&(h="#-cutoff="+f.substring(e+1),f=
f.substring(0,e));else{if(!c)return null;h=null}}if(null!=h)return JV.JC.resolveDataBase("emdbmap"+("-"==b?"server":""),f,null)+h}f=JV.JC.resolveDataBase((g?"pdbemapdiff":"pdbemap")+("-"==b?"server":""),f,null);"cif".equals(a)&&(f=f.$replace("bcif","cif"))}return f},"~S,~S,~B");c(c$,"checkCIR",function(a){if(!this.cirChecked||a){try{this.g.removeParam("_cirStatus");var b=this.getModelSetAuxiliaryInfo();b.remove("cirInfo");var c=this.parseJSONMap(this.getFileAsString(this.g.resolverResolver));b.put("cirInfo",
c);this.ms.msInfo=b;var g=c.get("status");this.g.setO("_cirStatus",g);this.g.setCIR(c.get("rfc6570Template"));Zystem.out.println("Viewer.checkCIR _.cirInfo.status = "+g)}catch(f){Zystem.out.println("Viewer.checkCIR failed at "+this.g.resolverResolver+": "+f)}this.cirChecked=!0}},"~B");c(c$,"getStandardLabelFormat",function(a){switch(a){default:case 0:return"%[identify]";case 1:return this.g.defaultLabelXYZ;case 2:return this.g.defaultLabelPDB}},"~N");c(c$,"getAdditionalHydrogens",function(a,b,c){null==
a&&(a=this.bsA());var g=B(1,0);a=this.ms.calculateHydrogens(a,g,b,c);g=Array(g[0]);for(c=b=0;b<a.length;b++)if(null!=a[b])for(var f=0;f<a[b].length;f++)g[c++]=a[b][f];return g},"JU.BS,JU.Lst,~N");j(c$,"setMarBond",function(a){this.g.bondRadiusMilliAngstroms=a;this.g.setI("bondRadiusMilliAngstroms",a);this.setShapeSize(1,2*a,JU.BSUtil.setAll(this.ms.ac))},"~N");c(c$,"setHoverLabel",function(a){this.shm.loadShape(35);this.setShapeProperty(35,"label",a);this.setHoverEnabled(null!=a);this.g.setO("_hoverLabel",
this.hoverLabel=a);!this.hoverEnabled&&!this.sm.haveHoverCallback()&&this.startHoverWatcher(!1)},"~S");c(c$,"setHoverEnabled",function(a){this.hoverEnabled=a;this.g.setB("_hoverEnabled",a)},"~B");c(c$,"hoverOn",function(a,b){this.g.removeParam("_objecthovered");this.g.setI("_atomhovered",a);this.g.setO("_hoverLabel",this.hoverLabel);this.g.setUserVariable("hovered",JS.SV.getVariable(JU.BSUtil.newAndSetBit(a)));this.sm.haveHoverCallback()&&this.sm.setStatusAtomHovered(a,this.getAtomInfoXYZ(a,!1));
if(this.hoverEnabled&&!(null!=this.eval&&this.isScriptExecuting()||a==this.hoverAtomIndex||0==this.g.hoverDelayMs||!this.slm.isInSelectionSubset(a))){var c=b?J.i18n.GT.$("Drag to move label"):this.g.modelKitMode&&null!=this.modelkit?this.modelkit.setProperty("hoverLabel",Integer.$valueOf(a)):null;this.shm.loadShape(35);null!=c&&(!b||this.ms.at[a].isVisible(512))&&this.setShapeProperty(35,"specialLabel",c);this.setShapeProperty(35,"text",this.hoverText=null);this.setShapeProperty(35,"target",Integer.$valueOf(this.hoverAtomIndex=
a));this.refresh(3,"hover on atom")}},"~N,~B");c(c$,"hoverOnPt",function(a,b,c,g,f){null!=this.eval&&this.isScriptExecuting()||(this.g.setO("_hoverLabel",c),null!=g&&null!=f&&(this.g.setO("_objecthovered",g),this.g.setI("_atomhovered",-1),this.g.setUserVariable("hovered",JS.SV.getVariable(f)),this.sm.haveHoverCallback()&&this.sm.setStatusObjectHovered(g,c,f)),this.hoverEnabled&&(this.shm.loadShape(35),this.setShapeProperty(35,"xy",JU.P3i.new3(a,b,0)),this.setShapeProperty(35,"target",null),this.setShapeProperty(35,
"specialLabel",null),this.setShapeProperty(35,"text",c),this.hoverAtomIndex=-1,this.hoverText=c,this.refresh(3,"hover on point")))},"~N,~N,~S,~S,JU.T3");c(c$,"hoverOff",function(){try{if(this.g.modelKitMode&&34!=this.acm.getBondPickingMode()&&this.highlight(null),this.hoverEnabled){var a=null!=this.hoverText||0<=this.hoverAtomIndex;0<=this.hoverAtomIndex&&(this.setShapeProperty(35,"target",null),this.hoverAtomIndex=-1);null!=this.hoverText&&(this.setShapeProperty(35,"text",null),this.hoverText=null);
this.setShapeProperty(35,"specialLabel",null);a&&this.refresh(3,"hover off")}}catch(b){if(!H(b,Exception))throw b;}});j(c$,"setDebugScript",function(a){this.g.debugScript=a;this.g.setB("debugScript",a);null!=this.eval&&this.eval.setDebugging()},"~B");c(c$,"clearClickCount",function(){this.setTainted(!0)});c(c$,"setCursor",function(a){if(!this.$isKiosk&&!(this.currentCursor==a||this.multiTouch||!this.haveDisplay))this.apiPlatform.setCursor(this.currentCursor=a,this.display)},"~N");c(c$,"setPickingMode",
function(a,b){if(this.haveDisplay){this.showSelected=!1;var c=null;if(null!=a){var g=a.indexOf("_");0<=g&&(c=a.substring(g+1),a=a.substring(0,g));b=JV.ActionManager.getPickingMode(a)}0>b&&(b=1);this.acm.setPickingMode(b);this.g.setO("picking",JV.ActionManager.getPickingModeName(this.acm.getAtomPickingMode()));if(!(null==c||0==c.length))switch(c=Character.toUpperCase(c.charAt(0))+(1==c.length?"":c.substring(1,2)),b){case 32:this.getModelkit(!1).setProperty("atomType",c);break;case 33:this.getModelkit(!1).setProperty("bondType",
c);break;default:JU.Logger.error("Bad picking mode: "+a+"_"+c)}}},"~S,~N");c(c$,"getPickingMode",function(){return this.haveDisplay?this.acm.getAtomPickingMode():0});c(c$,"setPickingStyle",function(a,b){this.haveDisplay&&(null!=a&&(b=JV.ActionManager.getPickingStyleIndex(a)),0>b&&(b=0),this.acm.setPickingStyle(b),this.g.setO("pickingStyle",JV.ActionManager.getPickingStyleName(this.acm.getPickingStyle())))},"~S,~N");c(c$,"getDrawHover",function(){return this.haveDisplay&&this.g.drawHover});c(c$,"getAtomInfo",
function(a){null==this.ptTemp&&(this.ptTemp=new JU.P3);return 0<=a?this.ms.getAtomInfo(a,null,this.ptTemp):this.shm.getShapePropertyIndex(6,"pointInfo",-a)},"~N");c(c$,"getAtomInfoXYZ",function(a,b){var c=this.ms.at[a];if(b)return this.getChimeMessenger().getInfoXYZ(c);null==this.ptTemp&&(this.ptTemp=new JU.P3);return c.getIdentityXYZ(!0,this.ptTemp)},"~N,~B");c(c$,"setSync",function(){this.sm.doSync()&&this.sm.setSync(null)});j(c$,"setJmolCallbackListener",function(a){this.sm.cbl=a},"J.api.JmolCallbackListener");
j(c$,"setJmolStatusListener",function(a){this.sm.cbl=this.sm.jsl=a},"J.api.JmolStatusListener");c(c$,"getStatusChanged",function(a){return null==a?null:this.sm.getStatusChanged(a)},"~S");c(c$,"menuEnabled",function(){return!this.g.disablePopupMenu&&null!=this.getPopupMenu()});c(c$,"setStatusDragDropped",function(a,b,c,g){0==a&&(this.g.setO("_fileDropped",g),this.g.setUserVariable("doDrop",JS.SV.vT));return!this.sm.setStatusDragDropped(a,b,c,g)||this.getP("doDrop").toString().equals("true")},"~N,~N,~N,~S");
c(c$,"setStatusResized",function(a,b){this.sm.setStatusResized(a,b)},"~N,~N");c(c$,"scriptStatus",function(a){this.setScriptStatus(a,"",0,null)},"~S");c(c$,"scriptStatusMsg",function(a,b){this.setScriptStatus(a,b,0,null)},"~S,~S");c(c$,"setScriptStatus",function(a,b,c,g){this.sm.setScriptStatus(a,b,c,g)},"~S,~S,~N,~S");j(c$,"showUrl",function(a){if(null!=a){if(0>a.indexOf(":")){var b=this.fm.getAppletDocumentBase();""===b&&(b=this.fm.getFullPathName(!1));0<=b.indexOf("/")?b=b.substring(0,b.lastIndexOf("/")+
1):0<=b.indexOf("\\")&&(b=b.substring(0,b.lastIndexOf("\\")+1));a=b+a}JU.Logger.info("showUrl:"+a);this.sm.showUrl(a)}},"~S");c(c$,"setMeshCreator",function(a){this.shm.loadShape(24);this.setShapeProperty(24,"meshCreator",a)},"~O");c(c$,"showConsole",function(a){if(this.haveDisplay)try{null==this.appConsole&&a&&this.getConsole(),this.appConsole.setVisible(!0)}catch(b){}},"~B");c(c$,"getConsole",function(){this.getProperty("DATA_API","getAppConsole",Boolean.TRUE);return this.appConsole});j(c$,"getParameter",
function(a){return this.getP(a)},"~S");c(c$,"getP",function(a){return this.g.getParameter(a,!0)},"~S");c(c$,"getPOrNull",function(a){return this.g.getParameter(a,!1)},"~S");c(c$,"unsetProperty",function(a){a=a.toLowerCase();(a.equals("all")||a.equals("variables"))&&this.fm.setPathForAllFiles("");this.g.unsetUserVariable(a)},"~S");j(c$,"notifyStatusReady",function(a){Zystem.out.println("Jmol applet "+this.fullName+(a?" ready":" destroyed"));a||this.dispose();this.sm.setStatusAppletReady(this.fullName,
a)},"~B");j(c$,"getBooleanProperty",function(a){a=a.toLowerCase();if(this.g.htBooleanParameterFlags.containsKey(a))return this.g.htBooleanParameterFlags.get(a).booleanValue();if(a.endsWith("p!")){if(null==this.acm)return!1;var b=this.acm.getPickingState().toLowerCase();a=a.substring(0,a.length-2)+";";return 0<=b.indexOf(a)}if(a.equalsIgnoreCase("executionPaused"))return null!=this.eval&&this.eval.isPaused();if(a.equalsIgnoreCase("executionStepping"))return null!=this.eval&&this.eval.isStepping();
if(a.equalsIgnoreCase("haveBFactors"))return null!=this.ms.getBFactors();if(a.equalsIgnoreCase("colorRasmol"))return this.cm.isDefaultColorRasmol;if(a.equalsIgnoreCase("frank"))return this.getShowFrank();if(a.equalsIgnoreCase("spinOn"))return this.tm.spinOn;if(a.equalsIgnoreCase("isNavigating"))return this.tm.isNavigating();if(a.equalsIgnoreCase("showSelections"))return this.selectionHalosEnabled;if(this.g.htUserVariables.containsKey(a)){b=this.g.getUserVariable(a);if(1073742335==b.tok)return!0;if(1073742334==
b.tok)return!1}JU.Logger.error("vwr.getBooleanProperty("+a+") - unrecognized");return!1},"~S");j(c$,"getInt",function(a){switch(a){case 553648132:return this.am.animationFps;case 553648141:return this.g.dotDensity;case 553648142:return this.g.dotScale;case 553648144:return this.g.helixStep;case 553648147:return this.g.infoFontSize;case 553648150:return this.g.meshScale;case 553648153:return this.g.minPixelSelRadius;case 553648154:return this.g.percentVdwAtom;case 553648157:return this.g.pickingSpinRate;
case 553648166:return this.g.ribbonAspectRatio;case 536870922:return this.g.scriptDelay;case 553648152:return this.g.minimizationMaxAtoms;case 553648170:return this.g.smallMoleculeMaxAtoms;case 553648184:return this.g.strutSpacing;case 553648185:return this.g.vectorTrail}JU.Logger.error("viewer.getInt("+JS.T.nameOf(a)+") - not listed");return 0},"~N");c(c$,"getDelayMaximumMs",function(){return this.haveDisplay?this.g.delayMaximumMs:1});c(c$,"getHermiteLevel",function(){return this.tm.spinOn&&0<this.g.hermiteLevel?
0:this.g.hermiteLevel});c(c$,"getHoverDelay",function(){return this.g.modelKitMode?20:this.g.hoverDelayMs});j(c$,"getBoolean",function(a){switch(a){case 603979891:return this.g.nboCharges;case 603979856:return this.g.hiddenLinesDashed;case 1073742086:return this.ms.getMSInfoB("isPDB");case 603979802:return this.g.autoplayMovie;case 603979797:return!this.headless&&this.g.allowAudio;case 603979780:return this.g.allowGestures;case 603979784:return this.g.allowMultiTouch;case 603979785:return this.g.allowRotateSelected;
case 603979792:return this.g.appendNew;case 603979794:return this.g.applySymmetryToBonds;case 603979796:return this.g.atomPicking;case 603979798:return this.g.autoBond;case 603979800:return this.g.autoFps;case 603979806:return this.g.axesOrientationRasmol;case 603979811:return this.g.cartoonSteps;case 603979810:return this.g.cartoonBlocks;case 603979821:return this.g.checkCIR;case 603979812:return this.g.bondModeOr;case 603979815:return this.g.cartoonBaseEdges;case 603979816:return this.g.cartoonFancy;
case 603979817:return this.g.cartoonLadders;case 603979818:return this.g.cartoonRibose;case 603979819:return this.g.cartoonRockets;case 603979822:return this.g.chainCaseSensitive||this.chainCaseSpecified;case 603979823:return this.g.cipRule6Full;case 603979825:return this.g.debugScript;case 603979826:return this.g.defaultStructureDSSP;case 603979827:return this.g.disablePopupMenu;case 603979828:return this.g.displayCellParameters;case 603979830:return this.g.dotSurface;case 603979829:return this.g.dotsSelectedOnly;
case 603979833:return this.g.drawPicking;case 603979844:return this.g.fontCaching;case 603979845:return this.g.fontScaling;case 603979846:return this.g.forceAutoBond;case 603979848:return!1;case 603979850:return this.g.greyscaleRendering;case 603979852:return this.g.hbondsBackbone;case 603979853:return this.g.hbondsRasmol;case 603979854:return this.g.hbondsSolid;case 1612709894:return this.g.rasmolHeteroSetting;case 603979858:return this.g.hideNameInPopup;case 603979864:return this.g.highResolutionFlag;
case 1612709900:return this.g.rasmolHydrogenSetting;case 603979867:return this.g.isosurfaceKey;case 603979869:return this.g.jmolInJSpecView;case 603979870:return this.g.justifyMeasurements;case 603979872:return this.g.legacyAutoBonding;case 603979873:return this.g.legacyHAddition;case 603979874:return this.g.legacyJavaFloat;case 603979876:return this.g.logGestures;case 603979877:return this.g.measureAllModels;case 603979878:return this.g.measurementLabels;case 603979879:return this.g.messageStyleChime;
case 603983903:return this.g.modelKitMode;case 603979886:return this.g.multipleBondBananas;case 603979889:return this.g.navigationMode;case 603979890:return this.g.navigationPeriodic;case 603979893:return this.g.partialDots;case 603979894:return this.g.pdbAddHydrogens;case 603979896:return this.g.pdbSequential;case 603979898:return this.g.preserveState;case 603979900:return this.refreshing;case 603979901:return this.g.ribbonBorder;case 603979902:return this.g.rocketBarrels;case 603979892:return this.g.noDelay;
case 603979906:return this.g.selectAllModels;case 603979920:return this.g.showHiddenSelectionHalos;case 603979922:return this.g.showHydrogens;case 603979926:return this.g.showMeasurements;case 603979927:return this.g.showModVecs;case 603979928:return this.g.showMultipleBonds;case 603979934:return this.g.showTiming;case 603979937:return this.g.showUnitCellDetails;case 603979939:return this.g.slabByAtom;case 603979940:return this.g.slabByMolecule;case 603979944:return this.g.smartAromatic;case 603979948:return this.g.dotSolvent;
case 603979952:return this.g.ssbondsBackbone;case 603979955:return this.g.strutsMultiple;case 603979960:return this.g.testFlag1;case 603979962:return this.g.testFlag2;case 603979964:return this.g.testFlag3;case 603979965:return this.g.testFlag4;case 603979966:return this.g.traceAlpha;case 603979967:return this.g.translucent;case 603979968:return this.g.twistedSheets;case 603979972:return this.g.vectorsCentered;case 603979973:return this.g.vectorSymmetry;case 603979975:return this.g.waitForMoveTo;
case 603979978:return this.g.zeroBasedXyzRasmol}JU.Logger.error("viewer.getBoolean("+JS.T.nameOf(a)+") - not listed");return!1},"~N");c(c$,"allowEmbeddedScripts",function(){return this.g.allowEmbeddedScripts&&!this.isPreviewOnly});c(c$,"getDragSelected",function(){return this.g.dragSelected&&!this.g.modelKitMode});c(c$,"getBondsPickable",function(){return this.g.bondPicking||this.g.modelKitMode&&this.getModelkitProperty("isMolecular")===Boolean.TRUE});c(c$,"useMinimizationThread",function(){return this.g.useMinimizationThread&&
!this.autoExit});j(c$,"getFloat",function(a){switch(a){case 1140850689:return this.g.particleRadius;case 570425345:return this.g.axesOffset;case 570425346:return this.g.axesScale;case 570425348:return this.g.bondTolerance;case 570425353:return this.g.defaultTranslucent;case 570425352:return this.g.defaultDrawArrowScale;case 570425354:return this.g.dipoleScale;case 570425355:return this.g.drawFontSize;case 570425357:return this.g.exportScale;case 570425359:return this.g.hbondsAngleMinimum;case 570425361:return this.g.hbondHXDistanceMaximum;
case 570425360:return this.g.hbondNODistanceMaximum;case 570425363:return this.g.loadAtomDataTolerance;case 570425364:return this.g.minBondDistance;case 1275072532:return this.g.modulationScale;case 570425370:return this.g.multipleBondSpacing;case 570425369:return this.g.multipleBondRadiusFactor;case 570425374:return this.g.navigationSpeed;case 570425382:return this.g.pointGroupDistanceTolerance;case 570425384:return this.g.pointGroupLinearTolerance;case 570425388:return this.tm.modelRadius;case 570425392:return this.g.sheetSmoothing;
case 570425394:return this.g.solventProbeRadius;case 570425403:return this.g.starWidth;case 570425406:return this.g.strutDefaultRadius;case 570425408:return this.g.strutLengthMaximum;case 1648361473:return this.g.vectorScale;case 570425412:return this.g.vibrationPeriod;case 570425347:return this.g.cartoonBlockHeight}JU.Logger.error("viewer.getFloat("+JS.T.nameOf(a)+") - not listed");return 0},"~N");j(c$,"setStringProperty",function(a,b){if(!(null==b||null==a||0==a.length))if("_"==a.charAt(0))this.g.setO(a,
b);else{var c=JS.T.getTokFromName(a);switch(JS.T.getParamType(c)){case 603979776:this.setBooleanPropertyTok(a,c,JS.SV.newV(4,b).asBoolean());break;case 553648128:this.setIntPropertyTok(a,c,JS.SV.newV(4,b).asInt());break;case 570425344:this.setFloatPropertyTok(a,c,JU.PT.parseFloat(b));break;default:this.setStringPropertyTok(a,c,b)}}},"~S,~S");c(c$,"setStringPropertyTok",function(a,b,c){switch(b){case 545259567:this.g.macroDirectory=c=null==c||0==c.length?"https://chemapps.stolaf.edu/jmol/macros":c;
this.macros=null;break;case 545259570:this.g.nihResolverFormat=c;break;case 545259521:this.setAnimationMode(c);return;case 545259569:this.g.nmrPredictFormat=c;break;case 545259548:this.g.defaultDropScript=c;break;case 545259572:c=this.fm.setPathForAllFiles(c);break;case 545259558:this.setUnits(c,!1);return;case 545259560:this.g.forceField=c="UFF".equalsIgnoreCase(c)?"UFF":"UFF2D".equalsIgnoreCase(c)?"UFF2D":"MMFF2D".equalsIgnoreCase(c)?"MMFF2D":"MMFF";this.minimizer=null;break;case 545259571:this.g.nmrUrlFormat=
c;break;case 545259568:this.setUnits(c,!0);return;case 545259565:this.g.pdbLoadLigandFormat=c;break;case 545259543:this.g.defaultLabelPDB=c;break;case 545259544:this.g.defaultLabelXYZ=c;break;case 545259549:this.g.defaultLoadFilter=c;break;case 545259566:c=this.getOutputManager().setLogFile(c);if(null==c)return;break;case 545259559:break;case 545259524:this.g.atomTypes=c;break;case 545259538:break;case 545259576:this.g.pickLabel=c;break;case 545259580:this.g.quaternionFrame=2==c.length&&c.startsWith("R")?
c.substring(0,2):""+(c.toLowerCase()+"p").charAt(0);JU.PT.isOneOf(this.g.quaternionFrame,"RC;RP;a;b;c;n;p;q;x;")||(this.g.quaternionFrame="p");this.ms.haveStraightness=!1;break;case 545259555:this.setVdwStr(c);return;case 545259563:new J.i18n.GT(this,c);b=J.i18n.GT.getLanguage();this.modelkit=null;null!=this.jmolpopup&&(this.jmolpopup.jpiDispose(),this.jmolpopup=null,this.getPopupMenu());this.sm.setCallbackFunction("language",b);c=J.i18n.GT.getLanguage();break;case 545259564:this.g.loadFormat=c;break;
case 545259534:this.setObjectColor("background",c);return;case 545259528:this.setObjectColor("axis1",c);return;case 545259530:this.setObjectColor("axis2",c);return;case 545259532:this.setObjectColor("axis3",c);return;case 545259536:this.setObjectColor("boundbox",c);return;case 545259586:this.setObjectColor("unitcell",c);return;case 545259578:this.setPropertyColorScheme(c,!1,!1);break;case 545259562:this.shm.loadShape(35);this.setShapeProperty(35,"atomLabel",c);break;case 545259547:this.g.defaultDistanceLabel=
c;break;case 545259542:this.g.defaultAngleLabel=c;break;case 545259554:this.g.defaultTorsionLabel=c;break;case 545259550:this.g.defaultLoadScript=c;break;case 545259522:this.fm.setAppletProxy(c);break;case 545259546:null==c&&(c="");c=c.$replace("\\","/");this.g.defaultDirectory=c;break;case 545259561:this.g.helpPath=c;break;case 545259552:!c.equalsIgnoreCase("RasMol")&&!c.equalsIgnoreCase("PyMOL")&&(c="Jmol");this.setDefaultsType(c);break;case 545259545:this.setDefaultColors(c.equalsIgnoreCase("rasmol"));
return;case 545259573:this.setPickingMode(c,0);return;case 545259574:this.setPickingStyle(c,0);return;case 545259540:break;default:if(a.toLowerCase().endsWith("callback")){this.sm.setCallbackFunction(a,0==c.length||c.equalsIgnoreCase("none")?null:c);break}if(!this.g.htNonbooleanParameterValues.containsKey(a.toLowerCase())){this.g.setUserVariable(a,JS.SV.newV(4,c));return}}this.g.setO(a,c)},"~S,~N,~S");j(c$,"setFloatProperty",function(a,b){if(!Float.isNaN(b)&&!(null==a||0==a.length))if("_"==a.charAt(0))this.g.setF(a,
b);else{var c=JS.T.getTokFromName(a);switch(JS.T.getParamType(c)){case 545259520:this.setStringPropertyTok(a,c,""+b);break;case 603979776:this.setBooleanPropertyTok(a,c,0!=b);break;case 553648128:this.setIntPropertyTok(a,c,A(b));break;default:this.setFloatPropertyTok(a,c,b)}}},"~S,~N");c(c$,"setFloatPropertyTok",function(a,b,c){switch(b){case 570425347:this.g.cartoonBlockHeight=c;break;case 570425366:this.ms.setModulation(null,!1,null,!1);this.g.modulationScale=c=Math.max(0.1,c);this.ms.setModulation(null,
!0,null,!1);break;case 570425381:this.g.particleRadius=Math.abs(c);break;case 570425355:this.g.drawFontSize=c;break;case 570425357:this.g.exportScale=c;break;case 570425403:this.g.starWidth=c;break;case 570425369:this.g.multipleBondRadiusFactor=c;break;case 570425370:this.g.multipleBondSpacing=c;break;case 570425393:this.tm.setSlabRange(c);break;case 570425365:this.g.minimizationCriterion=c;break;case 570425358:this.haveDisplay&&this.acm.setGestureSwipeFactor(c);break;case 570425367:this.haveDisplay&&
this.acm.setMouseDragFactor(c);break;case 570425368:this.haveDisplay&&this.acm.setMouseWheelFactor(c);break;case 570425408:this.g.strutLengthMaximum=c;break;case 570425406:this.g.strutDefaultRadius=c;break;case 570425376:this.setSpin("X",A(c));break;case 570425378:this.setSpin("Y",A(c));break;case 570425380:this.setSpin("Z",A(c));break;case 570425371:if(Float.isNaN(c))return;this.setSpin("FPS",A(c));break;case 570425363:this.g.loadAtomDataTolerance=c;break;case 570425359:this.g.hbondsAngleMinimum=
c;break;case 570425361:this.g.hbondHXDistanceMaximum=c;break;case 570425360:this.g.hbondNODistanceMaximum=c;break;case 570425382:this.g.pointGroupDistanceTolerance=c;break;case 570425384:this.g.pointGroupLinearTolerance=c;break;case 570425356:this.g.ellipsoidAxisDiameter=c;break;case 570425398:this.setSpin("x",A(c));break;case 570425400:this.setSpin("y",A(c));break;case 570425402:this.setSpin("z",A(c));break;case 570425396:this.setSpin("fps",A(c));break;case 570425352:this.g.defaultDrawArrowScale=
c;break;case 570425353:this.g.defaultTranslucent=c;break;case 570425345:this.setAxesScale(b,c);break;case 570425346:this.setAxesScale(b,c);break;case 570425416:this.tm.visualRangeAngstroms=c;this.refresh(1,"set visualRange");break;case 570425372:this.setNavigationDepthPercent(c);break;case 570425374:this.g.navigationSpeed=c;break;case 570425373:this.tm.setNavigationSlabOffsetPercent(c);break;case 570425350:this.tm.setCameraDepthPercent(c,!1);this.refresh(1,"set cameraDepth");return;case 570425388:this.setRotationRadius(c,
!0);return;case 570425362:this.g.hoverDelayMs=A(1E3*c);break;case 570425392:this.g.sheetSmoothing=c;break;case 570425354:c=JV.Viewer.checkFloatRange(c,-10,10);this.g.dipoleScale=c;break;case 570425404:this.tm.setStereoDegrees(c);break;case 1648361473:this.setVectorScale(c);return;case 570425412:this.setVibrationPeriod(c);return;case 570425414:this.setVibrationScale(c);return;case 570425348:this.setBondTolerance(c);return;case 570425364:this.setMinBondDistance(c);return;case 570425390:this.tm.setScaleAngstromsPerInch(c);
break;case 570425394:c=JV.Viewer.checkFloatRange(c,0,10);this.g.solventProbeRadius=c;break;default:if(!this.g.htNonbooleanParameterValues.containsKey(a.toLowerCase())){this.g.setUserVariable(a,JS.SV.newF(c));return}}this.g.setF(a,c)},"~S,~N,~N");j(c$,"setIntProperty",function(a,b){if(!(-2147483648==b||null==a||0==a.length))if("_"==a.charAt(0))this.g.setI(a,b);else{var c=JS.T.getTokFromName(a);switch(JS.T.getParamType(c)){case 545259520:this.setStringPropertyTok(a,c,""+b);break;case 603979776:this.setBooleanPropertyTok(a,
c,0!=b);break;case 570425344:this.setFloatPropertyTok(a,c,b);break;default:this.setIntPropertyTok(a,c,b)}}},"~S,~N");c(c$,"setIntPropertyTok",function(a,b,c){switch(b){case 553648152:this.g.minimizationMaxAtoms=c;break;case 553648147:this.g.infoFontSize=Math.max(0,c);break;case 553648167:case 553648146:case 553648168:c=this.eval.setStatic(b,c);break;case 553648185:this.g.vectorTrail=c;break;case 553648138:c=0==c?0:1;this.g.bondingVersion=JU.Elements.bondingVersion=c;break;case 553648137:this.gdata.setCelPower(c);
break;case 553648129:this.gdata.setAmbientOcclusion(c);break;case 553648158:this.g.platformSpeed=Math.min(Math.max(c,0),10);break;case 553648150:this.g.meshScale=c;break;case 553648153:this.g.minPixelSelRadius=c;break;case 553648148:this.g.isosurfacePropertySmoothingPower=c;break;case 553648165:this.g.repaintWaitMs=c;break;case 553648170:this.g.smallMoleculeMaxAtoms=c;break;case 553648151:this.g.minimizationSteps=c;break;case 553648184:this.g.strutSpacing=c;break;case 553648156:c=JV.Viewer.checkIntRange(c,
0,1E3);this.gdata.setPhongExponent(c);break;case 553648144:this.g.helixStep=c;this.ms.haveStraightness=!1;break;case 553648142:this.g.dotScale=c;break;case 553648141:this.g.dotDensity=c;break;case 553648139:this.g.delayMaximumMs=c;break;case 553648149:JU.Logger.setLogLevel(c);JU.Logger.info("logging level set to "+c);this.g.setI("logLevel",c);null!=this.eval&&this.eval.setDebugging();return;case 553648134:this.setAxesMode(2==c?603979808:1==c?603979804:603979809);return;case 553648178:this.setStrandCount(0,
c);return;case 553648182:this.setStrandCount(12,c);return;case 553648180:this.setStrandCount(13,c);return;case 553648155:return;case 536870922:this.g.scriptDelay=c;break;case 553648176:c=0>c?JV.Viewer.checkIntRange(c,-10,-1):JV.Viewer.checkIntRange(c,0,100);this.gdata.setSpecularPower(c);break;case 553648172:c=JV.Viewer.checkIntRange(-c,-10,-1);this.gdata.setSpecularPower(c);break;case 553648136:this.setMarBond(c);return;case 536870924:this.setBooleanPropertyTok(a,b,1==c);return;case 553648174:c=
JV.Viewer.checkIntRange(c,0,100);this.gdata.setSpecularPercent(c);break;case 553648140:c=JV.Viewer.checkIntRange(c,0,100);this.gdata.setDiffusePercent(c);break;case 553648130:c=JV.Viewer.checkIntRange(c,0,100);this.gdata.setAmbientPercent(c);break;case 553648186:this.tm.zDepthToPercent(c);break;case 553648188:this.tm.zSlabToPercent(c);break;case 554176526:this.tm.depthToPercent(c);break;case 554176565:this.tm.slabToPercent(c);break;case 553648190:this.g.zShadePower=c=Math.max(c,0);break;case 553648166:this.g.ribbonAspectRatio=
c;break;case 553648157:this.g.pickingSpinRate=1>c?1:c;break;case 553648132:this.setAnimationFps(c);return;case 553648154:this.setPercentVdwAtom(c);break;case 553648145:this.g.hermiteLevel=c;break;case 553648143:case 553648160:case 553648159:case 553648162:case 553648164:break;default:if(!this.g.htNonbooleanParameterValues.containsKey(a)){this.g.setUserVariable(a,JS.SV.newI(c));return}}this.g.setI(a,c)},"~S,~N,~N");c$.checkIntRange=c(c$,"checkIntRange",function(a,b,c){return a<b?b:a>c?c:a},"~N,~N,~N");
c$.checkFloatRange=c(c$,"checkFloatRange",function(a,b,c){return a<b?b:a>c?c:a},"~N,~N,~N");j(c$,"setBooleanProperty",function(a,b){if(!(null==a||0==a.length))if("_"==a.charAt(0))this.g.setB(a,b);else{var c=JS.T.getTokFromName(a);switch(JS.T.getParamType(c)){case 545259520:this.setStringPropertyTok(a,c,"");break;case 553648128:this.setIntPropertyTok(a,c,b?1:0);break;case 570425344:this.setFloatPropertyTok(a,c,b?1:0);break;default:this.setBooleanPropertyTok(a,c,b)}}},"~S,~B");c(c$,"setBooleanPropertyTok",
function(a,b,c){var g=!0;switch(b){case 603979821:(this.g.checkCIR=c)&&this.checkCIR(!0);break;case 603979823:this.g.cipRule6Full=c;break;case 603979802:this.g.autoplayMovie=c;break;case 603979797:c=!1;this.g.allowAudio=c;break;case 603979892:this.g.noDelay=c;break;case 603979891:this.g.nboCharges=c;break;case 603979856:this.g.hiddenLinesDashed=c;break;case 603979886:this.g.multipleBondBananas=c;break;case 603979884:this.g.modulateOccupancy=c;break;case 603979874:this.g.legacyJavaFloat=c;break;case 603979927:this.g.showModVecs=
c;break;case 603979937:this.g.showUnitCellDetails=c;break;case 603979848:g=!1;break;case 603979972:this.g.vectorsCentered=c;break;case 603979810:this.g.cartoonBlocks=c;break;case 603979811:this.g.cartoonSteps=c;break;case 603979818:this.g.cartoonRibose=c;break;case 603979837:this.g.ellipsoidArrows=c;break;case 603979967:this.g.translucent=c;break;case 603979817:this.g.cartoonLadders=c;break;case 603979968:b=this.g.twistedSheets;this.g.twistedSheets=c;b!=c&&this.checkCoordinatesChanged();break;case 603979820:this.gdata.setCel(c);
break;case 603979816:this.g.cartoonFancy=c;break;case 603979934:this.g.showTiming=c;break;case 603979973:this.g.vectorSymmetry=c;break;case 603979867:this.g.isosurfaceKey=c;break;case 603979893:this.g.partialDots=c;break;case 603979872:this.g.legacyAutoBonding=c;break;case 603979826:this.g.defaultStructureDSSP=c;break;case 603979834:this.g.dsspCalcHydrogen=c;break;case 603979782:(this.g.allowModelkit=c)||this.setModelKitMode(!1);break;case 603983903:this.setModelKitMode(c);break;case 603979887:this.g.multiProcessor=
c&&1<JV.Viewer.nProcessors;break;case 603979885:this.g.monitorEnergy=c;break;case 603979853:this.g.hbondsRasmol=c;break;case 603979880:this.g.minimizationRefresh=c;break;case 603979881:this.g.minimizationSilent=c;break;case 603979866:c&&(this.$isKiosk=!0,this.g.disablePopupMenu=!0,null!=this.display&&this.apiPlatform.setTransparentCursor(this.display));break;case 603979975:this.g.waitForMoveTo=c;break;case 603979875:this.g.logCommands=!0;break;case 603979876:this.g.logGestures=!0;break;case 603979784:this.g.allowMultiTouch=
c;break;case 603979898:this.g.preserveState=c;this.ms.setPreserveState(c);this.undoClear();break;case 603979955:this.g.strutsMultiple=c;break;case 603979842:break;case 603979939:this.g.slabByAtom=c;break;case 603979940:this.g.slabByMolecule=c;break;case 603979903:this.g.saveProteinStructureState=c;break;case 603979780:this.g.allowGestures=c;break;case 603979865:this.g.imageState=c;break;case 603979970:this.g.useMinimizationThread=c;break;case 603979781:this.g.allowKeyStrokes=c;break;case 603979831:this.g.dragSelected=
c;this.showSelected=!1;break;case 603979924:this.g.showKeyStrokes=c;break;case 603979844:this.g.fontCaching=c;break;case 603979796:this.g.atomPicking=c;break;case 603979814:this.highlight(null);this.g.bondPicking=c;break;case 603979906:(this.g.selectAllModels=c)?this.slm.setSelectionSubset(null):this.am.setSelectAllSubset(!1);break;case 603979879:this.g.messageStyleChime=c;break;case 603979896:this.g.pdbSequential=c;break;case 603979894:this.g.pdbAddHydrogens=c;break;case 603979895:this.g.pdbGetHeader=
c;break;case 603979838:this.g.ellipsoidAxes=c;break;case 603979836:this.g.ellipsoidArcs=c;break;case 603979839:this.g.ellipsoidBall=c;break;case 603979840:this.g.ellipsoidDots=c;break;case 603979841:this.g.ellipsoidFill=c;break;case 603979845:this.g.fontScaling=c;break;case 603979956:this.setSyncTarget(0,c);break;case 603979958:this.setSyncTarget(1,c);break;case 603979977:this.g.wireframeRotation=c;break;case 603979868:this.g.isosurfacePropertySmoothing=c;break;case 603979833:this.g.drawPicking=c;
break;case 603979786:case 603979790:case 603979788:this.setAntialias(b,c);break;case 603979944:this.g.smartAromatic=c;break;case 603979794:this.setApplySymmetryToBonds(c);break;case 603979792:this.g.appendNew=c;break;case 603979800:this.g.autoFps=c;break;case 603979971:JU.DF.setUseNumberLocalization(this.g.useNumberLocalization=c);break;case 603979918:case 1611272202:a="showFrank";this.setFrankOn(c);break;case 1612709912:a="solventProbe";this.g.dotSolvent=c;break;case 603979948:this.g.dotSolvent=
c;break;case 603979785:this.g.allowRotateSelected=c;break;case 603979783:this.g.allowMoveAtoms=c;this.showSelected=!1;break;case 536870922:this.setIntPropertyTok("showScript",b,c?1:0);return;case 603979778:this.g.allowEmbeddedScripts=c;break;case 603979890:this.g.navigationPeriodic=c;break;case 603979984:this.tm.setZShadeEnabled(c);return;case 603979832:this.haveDisplay&&(this.g.drawHover=c);break;case 603979889:this.setNavigationMode(c);break;case 603979888:return;case 603979860:this.g.hideNavigationPoint=
c;break;case 603979930:this.g.showNavigationPointAlways=c;break;case 603979900:this.setRefreshing(c);break;case 603979869:this.g.jmolInJSpecView=c;break;case 603979870:this.g.justifyMeasurements=c;break;case 603979952:this.g.ssbondsBackbone=c;break;case 603979852:this.g.hbondsBackbone=c;break;case 603979854:this.g.hbondsSolid=c;break;case 536870924:this.gdata.setSpecular(c);break;case 603979942:this.tm.setSlabEnabled(c);return;case 603979980:this.tm.setZoomEnabled(c);return;case 603979864:this.g.highResolutionFlag=
c;break;case 603979966:this.g.traceAlpha=c;break;case 603979983:this.g.zoomLarge=c;this.tm.setZoomHeight(this.g.zoomHeight,c);break;case 603979982:this.g.zoomHeight=c;this.tm.setZoomHeight(c,this.g.zoomLarge);break;case 603979871:J.i18n.GT.setDoTranslate(c);break;case 603979862:this.slm.setHideNotSelected(c);break;case 603979904:this.setScriptQueue(c);break;case 603979830:this.g.dotSurface=c;break;case 603979829:this.g.dotsSelectedOnly=c;break;case 1611141171:this.setSelectionHalosEnabled(c);break;
case 603979910:this.g.rasmolHydrogenSetting=c;break;case 603979908:this.g.rasmolHeteroSetting=c;break;case 603979928:this.g.showMultipleBonds=c;break;case 603979920:this.g.showHiddenSelectionHalos=c;break;case 603979976:this.tm.setWindowCentered(c);break;case 603979828:this.g.displayCellParameters=c;break;case 603979960:this.g.testFlag1=c;break;case 603979962:this.g.testFlag2=c;break;case 603979964:this.g.testFlag3=c;break;case 603979965:this.jmolTest();this.g.testFlag4=c;break;case 603979901:this.g.ribbonBorder=
c;break;case 603979815:this.g.cartoonBaseEdges=c;break;case 603979819:this.g.cartoonRockets=c;break;case 603979902:this.g.rocketBarrels=c;break;case 603979850:this.gdata.setGreyscaleMode(this.g.greyscaleRendering=c);break;case 603979878:this.g.measurementLabels=c;break;case 603979809:case 603979804:case 603979808:this.setAxesMode(b);return;case 603979806:this.setAxesOrientationRasmol(c);return;case 603979824:this.setStringPropertyTok("defaultcolorscheme",545259545,c?"rasmol":"jmol");return;case 603979825:this.setDebugScript(c);
return;case 603979897:this.setPerspectiveDepth(c);return;case 603979798:this.setAutoBond(c);return;case 603979914:this.setShowAxes(c);return;case 603979916:this.setShowBbcage(c);return;case 603979922:this.setShowHydrogens(c);return;case 603979926:this.setShowMeasurements(c);return;case 603979936:this.setShowUnitCell(c);return;case 603979812:g=!1;this.g.bondModeOr=c;break;case 603979978:g=!1;this.g.zeroBasedXyzRasmol=c;this.reset(!0);break;case 603979899:g=!1;this.g.rangeSelected=c;break;case 603979877:g=
!1;this.g.measureAllModels=c;break;case 603979954:g=!1;this.sm.allowStatusReporting=c;break;case 603979822:g=!1;this.g.chainCaseSensitive=c;break;case 603979858:g=!1;this.g.hideNameInPopup=c;break;case 603979827:g=!1;this.g.disablePopupMenu=c;break;case 603979846:g=!1;this.g.forceAutoBond=c;break;default:if(!this.g.htBooleanParameterFlags.containsKey(a.toLowerCase())){this.g.setUserVariable(a,JS.SV.getBoolean(c));return}}this.g.setB(a,c);g&&this.setTainted(!0)},"~S,~N,~B");c(c$,"setModelKitMode",
function(a){if(null!=this.acm&&this.allowScripting){if(a||this.g.modelKitMode)this.setPickingMode(null,a?33:1),this.setPickingMode(null,a?32:1);var b=this.g.modelKitMode!=a;this.g.modelKitMode=a;this.g.setB("modelkitmode",a);this.highlight(null);a?(this.setNavigationMode(!1),this.selectAll(),this.setStringProperty("picking","assignAtom_C"),this.setStringProperty("picking","assignBond_p"),this.isApplet||this.popupMenu(10,0,"m"),b&&this.sm.setStatusModelKit(1),this.g.modelKitMode=!0,0==this.ms.ac?this.zap(!1,
!0,!0):0<=this.am.cmi&&this.getModelUndeletedAtomsBitSet(this.am.cmi).isEmpty()&&(a=new java.util.Hashtable,a.put("appendToModelIndex",Integer.$valueOf(this.am.cmi)),this.loadDefaultModelKitModel(a))):(this.acm.setPickingMode(-1),this.setStringProperty("pickingStyle","toggle"),this.setBooleanProperty("bondPicking",!1),b&&this.sm.setStatusModelKit(0))}},"~B");c(c$,"setSmilesString",function(a){null==a?this.g.removeParam("_smilesString"):this.g.setO("_smilesString",a)},"~S");c(c$,"removeUserVariable",
function(a){this.g.removeUserVariable(a);a.endsWith("callback")&&this.sm.setCallbackFunction(a,null)},"~S");c(c$,"jmolTest",function(){});c(c$,"showParameter",function(a,b,c){c=""+this.g.getParameterEscaped(a,c);(b||0>c.indexOf("<not defined>"))&&this.showString(a+" = "+c,!1)},"~S,~B,~N");c(c$,"showString",function(a,b){!JV.Viewer.isJS&&(this.isScriptQueued()&&(!this.isSilent||b)&&!"\x00".equals(a))&&JU.Logger.warn(a);this.scriptEcho(a)},"~S,~B");c(c$,"getAllSettings",function(a){return this.getStateCreator().getAllSettings(a)},
"~S");c(c$,"getBindingInfo",function(a){return this.haveDisplay?this.acm.getBindingInfo(a):""},"~S");c(c$,"getIsosurfacePropertySmoothing",function(a){return a?this.g.isosurfacePropertySmoothingPower:this.g.isosurfacePropertySmoothing?1:0},"~B");c(c$,"setNavigationDepthPercent",function(a){this.tm.setNavigationDepthPercent(a);this.refresh(1,"set navigationDepth")},"~N");c(c$,"getShowNavigationPoint",function(){return!this.g.navigationMode?!1:this.tm.isNavigating()&&!this.g.hideNavigationPoint||this.g.showNavigationPointAlways||
this.getInMotion(!0)});j(c$,"setPerspectiveDepth",function(a){this.tm.setPerspectiveDepth(a)},"~B");j(c$,"setAxesOrientationRasmol",function(a){this.g.setB("axesOrientationRasmol",a);this.g.axesOrientationRasmol=a;this.reset(!0)},"~B");c(c$,"setAxesScale",function(a,b){b=JV.Viewer.checkFloatRange(b,-100,100);570425345==a?this.g.axesOffset=b:this.g.axesScale=b;this.axesAreTainted=!0},"~N,~N");c(c$,"setAxesMode",function(a){this.g.axesMode=a;this.axesAreTainted=!0;switch(a){case 603979808:this.g.removeParam("axesmolecular");
this.g.removeParam("axeswindow");this.g.setB("axesUnitcell",!0);a=2;break;case 603979804:this.g.removeParam("axesunitcell");this.g.removeParam("axeswindow");this.g.setB("axesMolecular",!0);a=1;break;case 603979809:this.g.removeParam("axesunitcell"),this.g.removeParam("axesmolecular"),this.g.setB("axesWindow",!0),a=0}this.g.setI("axesMode",a)},"~N");c(c$,"getSelectionHalosEnabled",function(){return this.selectionHalosEnabled});c(c$,"setSelectionHalosEnabled",function(a){this.selectionHalosEnabled!=
a&&(this.g.setB("selectionHalos",a),this.shm.loadShape(8),this.selectionHalosEnabled=a)},"~B");c(c$,"getShowSelectedOnce",function(){var a=this.showSelected;this.showSelected=!1;return a});c(c$,"setStrandCount",function(a,b){b=JV.Viewer.checkIntRange(b,0,20);switch(a){case 12:this.g.strandCountForStrands=b;break;case 13:this.g.strandCountForMeshRibbon=b;break;default:this.g.strandCountForStrands=b,this.g.strandCountForMeshRibbon=b}this.g.setI("strandCount",b);this.g.setI("strandCountForStrands",this.g.strandCountForStrands);
this.g.setI("strandCountForMeshRibbon",this.g.strandCountForMeshRibbon)},"~N,~N");c(c$,"getStrandCount",function(a){return 12==a?this.g.strandCountForStrands:this.g.strandCountForMeshRibbon},"~N");c(c$,"setNavigationMode",function(a){this.g.navigationMode=a;this.tm.setNavigationMode(a)},"~B");j(c$,"setAutoBond",function(a){this.g.setB("autobond",a);this.g.autoBond=a},"~B");c(c$,"makeConnections",function(a,b,c,g,f,e,h,j,l,n){this.clearModelDependentObjects();this.clearMinimization();return this.ms.makeConnections(a,
b,c,g,f,e,h,j,l,n)},"~N,~N,~N,~N,JU.BS,JU.BS,JU.BS,~B,~B,~N");j(c$,"rebond",function(){this.rebondState(!1)});c(c$,"rebondState",function(a){this.clearModelDependentObjects();this.ms.deleteAllBonds();a=a&&this.g.legacyAutoBonding;this.ms.autoBondBs4(null,null,null,null,this.getMadBond(),a);this.addStateScript(a?"set legacyAutoBonding TRUE;connect;set legacyAutoBonding FALSE;":"connect;",!1,!0)},"~B");j(c$,"setPercentVdwAtom",function(a){this.g.setI("percentVdwAtom",a);this.g.percentVdwAtom=a;this.rd.value=
a/100;this.rd.factorType=J.atomdata.RadiusData.EnumType.FACTOR;this.rd.vdwType=J.c.VDW.AUTO;this.shm.setShapeSizeBs(0,0,this.rd,null)},"~N");j(c$,"getMadBond",function(){return 2*this.g.bondRadiusMilliAngstroms});j(c$,"setShowHydrogens",function(a){this.g.setB("showHydrogens",a);this.g.showHydrogens=a},"~B");c(c$,"setShowBbcage",function(a){this.setObjectMad10(32,"boundbox",a?-4:0);this.g.setB("showBoundBox",a)},"~B");c(c$,"getShowBbcage",function(){return 0!=this.getObjectMad10(4)});c(c$,"setShowUnitCell",
function(a){this.setObjectMad10(33,"unitcell",a?-2:0);this.g.setB("showUnitCell",a)},"~B");c(c$,"getShowUnitCell",function(){return 0!=this.getObjectMad10(5)});c(c$,"setShowAxes",function(a){this.setObjectMad10(34,"axes",a?-2:0);this.g.setB("showAxes",a)},"~B");c(c$,"getShowAxes",function(){return 0!=this.getObjectMad10(1)});j(c$,"setFrankOn",function(a){this.isPreviewOnly&&(a=!1);this.frankOn=a;this.setObjectMad10(36,"frank",a?1:0)},"~B");c(c$,"getShowFrank",function(){return this.isPreviewOnly||
this.isApplet&&this.creatingImage?!1:this.isSignedApplet&&!this.isSignedAppletLocal&&!JV.Viewer.isJS||this.frankOn});j(c$,"setShowMeasurements",function(a){this.g.setB("showMeasurements",a);this.g.showMeasurements=a},"~B");c(c$,"setUnits",function(a,b){this.g.setUnits(a);b&&(this.g.setUnits(a),this.setShapeProperty(6,"reformatDistances",null))},"~S,~B");j(c$,"setRasmolDefaults",function(){this.setDefaultsType("RasMol")});j(c$,"setJmolDefaults",function(){this.setDefaultsType("Jmol")});c(c$,"setDefaultsType",
function(a){a.equalsIgnoreCase("RasMol")?this.stm.setRasMolDefaults():a.equalsIgnoreCase("PyMOL")?this.stm.setPyMOLDefaults():(this.stm.setJmolDefaults(),this.setIntProperty("bondingVersion",0),this.shm.setShapeSizeBs(0,0,this.rd,this.getAllAtoms()))},"~S");c(c$,"setAntialias",function(a,b){var c=!1;switch(a){case 603979786:c=this.g.antialiasDisplay!=b;this.g.antialiasDisplay=b;break;case 603979790:c=this.g.antialiasTranslucent!=b;this.g.antialiasTranslucent=b;break;case 603979788:this.g.antialiasImages=
b;return}c&&(this.resizeImage(0,0,!1,!1,!0),this.refresh(3,"Viewer:setAntialias()"))},"~N,~B");c(c$,"allocTempPoints",function(a){return this.tempArray.allocTempPoints(a)},"~N");c(c$,"freeTempPoints",function(a){this.tempArray.freeTempPoints(a)},"~A");c(c$,"allocTempScreens",function(a){return this.tempArray.allocTempScreens(a)},"~N");c(c$,"freeTempScreens",function(a){this.tempArray.freeTempScreens(a)},"~A");c(c$,"allocTempEnum",function(a){return this.tempArray.allocTempEnum(a)},"~N");c(c$,"freeTempEnum",
function(a){this.tempArray.freeTempEnum(a)},"~A");c(c$,"getFont3D",function(a,b,c){return this.gdata.getFont3DFSS(a,b,c)},"~S,~S,~N");c(c$,"getAtomGroupQuaternions",function(a,b){return this.ms.getAtomGroupQuaternions(a,b,this.getQuaternionFrame())},"JU.BS,~N");c(c$,"setStereoMode",function(a,b,c){this.setFloatProperty("stereoDegrees",c);this.setBooleanProperty("greyscaleRendering",b.isBiColor());null!=a?this.tm.setStereoMode2(a):this.tm.setStereoMode(b)},"~A,J.c.STER,~N");c(c$,"getChimeInfo",function(a){return this.getPropertyManager().getChimeInfo(a,
this.bsA())},"~N");c(c$,"getModelFileInfo",function(){return this.getPropertyManager().getModelFileInfo(this.getVisibleFramesBitSet())});c(c$,"getModelFileInfoAll",function(){return this.getPropertyManager().getModelFileInfo(null)});c(c$,"showEditor",function(a){var b=this.getProperty("DATA_API","getScriptEditor",Boolean.TRUE);null!=b&&b.show(a)},"~A");c(c$,"getPropertyManager",function(){null==this.pm&&(this.pm=J.api.Interface.getInterface("JV.PropertyManager",this,"prop")).setViewer(this);return this.pm});
c(c$,"setTainted",function(a){this.isTainted=this.axesAreTainted=a&&(this.refreshing||this.creatingImage)},"~B");c(c$,"checkObjectClicked",function(a,b,c){return this.shm.checkObjectClicked(a,b,c,this.getVisibleFramesBitSet(),this.g.drawPicking)},"~N,~N,~N");c(c$,"checkObjectHovered",function(a,b){return 0<=a&&null!=this.shm&&this.shm.checkObjectHovered(a,b,this.getVisibleFramesBitSet(),this.getBondsPickable())},"~N,~N");c(c$,"checkObjectDragged",function(a,b,c,g,f){var e=0;switch(this.getPickingMode()){case 2:e=
5;break;case 4:e=22}return this.shm.checkObjectDragged(a,b,c,g,f,this.getVisibleFramesBitSet(),e)?(this.refresh(1,"checkObjectDragged"),22==e&&this.scriptEcho(this.getShapeProperty(22,"command")),!0):!1},"~N,~N,~N,~N,~N");c(c$,"rotateAxisAngleAtCenter",function(a,b,c,g,f,e,h){(a=this.tm.rotateAxisAngleAtCenter(a,b,c,g,f,e,h))&&this.setSync();return a},"J.api.JmolScriptEvaluator,JU.P3,JU.V3,~N,~N,~B,JU.BS");c(c$,"rotateAboutPointsInternal",function(a,b,c,g,f,e,h,j,l,n,m){null==a&&(a=this.eval);if(this.headless){if(e&&
3.4028235E38==f)return!1;e=!1}(a=this.tm.rotateAboutPointsInternal(a,b,c,g,f,!1,e,h,!1,j,l,n,m))&&this.setSync();return a},"J.api.JmolScriptEvaluator,JU.P3,JU.P3,~N,~N,~B,JU.BS,JU.V3,JU.Lst,~A,JU.M4");c(c$,"startSpinningAxis",function(a,b,c){this.tm.spinOn||this.tm.navOn?(this.tm.setSpinOff(),this.tm.setNavOn(!1)):this.tm.rotateAboutPointsInternal(null,a,b,this.g.pickingSpinRate,3.4028235E38,c,!0,null,!1,null,null,null,null)},"JU.T3,JU.T3,~B");c(c$,"getModelDipole",function(){return this.ms.getModelDipole(this.am.cmi)});
c(c$,"calculateMolecularDipole",function(a){try{return this.ms.calculateMolecularDipole(this.am.cmi,a)}catch(b){if(H(b,JV.JmolAsyncException))return null!=this.eval&&this.eval.loadFileResourceAsync(b.getFileName()),null;throw b;}},"JU.BS");c(c$,"setDefaultLattice",function(a){Float.isNaN(a.x+a.y+a.z)||this.g.ptDefaultLattice.setT(a);this.g.setO("defaultLattice",JU.Escape.eP(a))},"JU.P3");c(c$,"getDefaultLattice",function(){return this.g.ptDefaultLattice});c(c$,"getModelExtract",function(a,b,c,g){return this.getPropertyManager().getModelExtract(this.getAtomBitSet(a),
b,c,g,!1)},"~O,~B,~B,~S");j(c$,"getData",function(a,b){return this.getModelFileData(a,b,!0)},"~S,~S");c(c$,"getModelFileData",function(a,b,c){return this.getPropertyManager().getAtomData(a,b,c)},"~S,~S,~B");c(c$,"getModelCml",function(a,b,c,g){return this.getPropertyManager().getModelCml(a,b,c,g,!1)},"JU.BS,~N,~B,~B");c(c$,"getPdbAtomData",function(a,b,c,g){return this.getPropertyManager().getPdbAtomData(null==a?this.bsA():a,b,c,g,!1)},"JU.BS,JU.OC,~B,~B");c(c$,"isJmolDataFrame",function(){return this.ms.isJmolDataFrameForModel(this.am.cmi)});
c(c$,"setFrameTitle",function(a,b){this.ms.setFrameTitle(JU.BSUtil.newAndSetBit(a),b)},"~N,~S");c(c$,"setFrameTitleObj",function(a){this.shm.loadShape(31);this.ms.setFrameTitle(this.getVisibleFramesBitSet(),a)},"~O");c(c$,"getFrameTitle",function(){return this.ms.getFrameTitle(this.am.cmi)});c(c$,"setAtomProperty",function(a,b,c,g,f,e,h){1648363544==b&&this.shm.deleteVdwDependentShapes(a);this.clearMinimization();this.ms.setAtomProperty(a,b,c,g,f,e,h);switch(b){case 1111492609:case 1111492610:case 1111492611:case 1111492612:case 1111492613:case 1111492614:case 1111490577:case 1111490578:case 1111490579:case 1086326789:this.refreshMeasures(!0)}},
"JU.BS,~N,~N,~N,~S,~A,~A");c(c$,"checkCoordinatesChanged",function(){this.ms.recalculatePositionDependentQuantities(null,null);this.refreshMeasures(!0)});c(c$,"setAtomCoords",function(a,b,c){a.isEmpty()||(this.ms.setAtomCoords(a,b,c),this.checkMinimization(),this.sm.setStatusAtomMoved(a))},"JU.BS,~N,~O");c(c$,"setAtomCoordsRelative",function(a,b){null==b&&(b=this.bsA());b.isEmpty()||(this.ms.setAtomCoordsRelative(a,b),this.checkMinimization(),this.sm.setStatusAtomMoved(b))},"JU.T3,JU.BS");c(c$,"invertAtomCoordPt",
function(a,b){this.ms.invertSelected(a,null,-1,b);this.checkMinimization();this.sm.setStatusAtomMoved(b)},"JU.P3,JU.BS");c(c$,"invertAtomCoordPlane",function(a,b){this.ms.invertSelected(null,a,-1,b);this.checkMinimization();this.sm.setStatusAtomMoved(b)},"JU.P4,JU.BS");c(c$,"invertRingAt",function(a,b){var c=this.getAtomBitSet("connected(atomIndex="+a+") and !within(SMARTS,'[r50,R]')"),g=c.cardinality();switch(g){case 0:case 1:return;case 3:case 4:for(var f=B(g,0),e=B(g,0),h=0,j=c.nextSetBit(0);0<=
j;j=c.nextSetBit(j+1),h++)f[h]=this.getBranchBitSet(j,a,!0).cardinality(),e[h]=j;for(h=0;h<g-2;h++){for(var l=-2147483648,n=0,j=0;j<g;j++)f[j]>=l&&c.get(e[j])&&(n=e[j],l=f[j]);c.clear(n)}}b&&this.undoMoveActionClear(a,2,!0);this.invertSelected(null,null,a,c);b&&this.setStatusAtomPicked(a,"inverted: "+JU.Escape.eBS(c),null,!1)},"~N,~B");c(c$,"invertSelected",function(a,b,c,g){null==g&&(g=this.bsA());0!=g.cardinality()&&(this.ms.invertSelected(a,b,c,g),this.checkMinimization(),this.sm.setStatusAtomMoved(g))},
"JU.P3,JU.P4,~N,JU.BS");c(c$,"moveAtoms",function(a,b,c,g,f,e,h,j){h.isEmpty()||(this.ms.moveAtoms(a,b,c,g,h,f,e,j),this.checkMinimization(),this.sm.setStatusAtomMoved(h))},"JU.M4,JU.M3,JU.M3,JU.V3,JU.P3,~B,JU.BS,~B");c(c$,"moveSelected",function(a,b,c,g,f,e,h,j,l){0!=c&&(-2147483648==g&&this.setModelKitRotateBondIndex(-2147483648),this.isJmolDataFrame()||(-2147483648==a?(this.showSelected=!0,this.movableBitSet=this.setMovableBitSet(null,!j),this.shm.loadShape(8),this.refresh(6,"moveSelected")):2147483647==
a?this.showSelected&&(this.showSelected=!1,this.movableBitSet=null,this.refresh(6,"moveSelected")):this.movingSelected||(this.movingSelected=!0,this.stopMinimization(),-2147483648!=g&&null!=this.modelkit&&null!=this.modelkit.getProperty("rotateBondIndex")?this.modelkit.actionRotateBond(a,b,g,f,0!=(l&16)):(e=this.setMovableBitSet(e,!j),e.isEmpty()||(h?(g=this.ms.getAtomSetCenter(e),this.tm.finalizeTransformParameters(),f=this.g.antialiasDisplay?2:1,h=this.tm.transformPt(g),a=-2147483648!=c?JU.P3.new3(h.x,
h.y,h.z+c+0.5):JU.P3.new3(h.x+a*f+0.5,h.y+b*f+0.5,h.z),b=new JU.P3,this.tm.unTransformPoint(a,b),b.sub(g),this.setAtomCoordsRelative(b,e)):this.tm.rotateXYBy(a,b,e))),this.refresh(2,""),this.movingSelected=!1)))},"~N,~N,~N,~N,~N,JU.BS,~B,~B,~N");c(c$,"highlightBond",function(a,b,c,g){if(this.hoverEnabled){var f=null;if(0<=a){b=this.ms.bo[a];f=b.atom2.i;if(!this.ms.isAtomInLastModel(f))return;f=JU.BSUtil.newAndSetBit(f);f.set(b.atom1.i)}this.highlight(f);this.setModelkitProperty("bondIndex",Integer.$valueOf(a));
this.setModelkitProperty("screenXY",B(-1,[c,g]));a=this.setModelkitProperty("hoverLabel",Integer.$valueOf(-2-a));null!=a&&this.hoverOnPt(c,g,a,null,null);this.refresh(3,"highlightBond")}},"~N,~N,~N,~N");c(c$,"highlight",function(a){this.atomHighlighted=null!=a&&1==a.cardinality()?a.nextSetBit(0):-1;null==a?this.setCursor(0):(this.shm.loadShape(8),this.setCursor(12));this.setModelkitProperty("highlight",a);this.setShapeProperty(8,"highlight",a)},"JU.BS");c(c$,"refreshMeasures",function(a){this.setShapeProperty(6,
"refresh",null);a&&this.stopMinimization()},"~B");c(c$,"functionXY",function(a,b,c){var g=null;if(0==a.indexOf("file:"))g=this.getFileAsString3(a.substring(5),!1,null);else if(0!=a.indexOf("data2d_"))return this.sm.functionXY(a,b,c);b=Math.abs(b);c=Math.abs(c);if(null==g){a=this.getDataObj(a,null,2);if(null!=a)return a;g=""}a=K(b,c,0);var f=K(b*c,0);JU.Parser.parseStringInfestedFloatArray(g,null,f);for(var e=g=0;g<b;g++)for(var h=0;h<c;h++)a[g][h]=f[e++];return a},"~S,~N,~N");c(c$,"functionXYZ",function(a,
b,c,g){var f=null;if(0==a.indexOf("file:"))f=this.getFileAsString3(a.substring(5),!1,null);else if(0!=a.indexOf("data3d_"))return this.sm.functionXYZ(a,b,c,g);b=Math.abs(b);c=Math.abs(c);g=Math.abs(g);if(null==f){a=this.getDataObj(a,null,2);if(null!=a)return a;f=""}a=K(b,c,g,0);var e=K(b*c*g,0);JU.Parser.parseStringInfestedFloatArray(f,null,e);for(var h=f=0;f<b;f++)for(var j=0;j<c;j++)for(var l=0;l<g;l++)a[f][j][l]=e[h++];return a},"~S,~N,~N,~N");j(c$,"extractMolData",function(a){if(null==a){a=this.am.cmi;
if(0>a||0==this.ms.ac)return null;a=this.getModelNumberDotted(a)}return this.getModelExtract(a,!0,!1,"V2000")},"~S");c(c$,"getNMRPredict",function(a){a=a.toUpperCase();if(a.equals("H")||a.equals("1H")||a.equals(""))a="H1";else if(a.equals("C")||a.equals("13C"))a="C13";if(!a.equals("NONE")){if(!a.equals("C13")&&!a.equals("H1"))return"Type must be H1 or C13";var b=this.getModelExtract("selected",!0,!1,"V2000"),c=b.indexOf("\n");if(0>c)return null;b="Jmol "+JV.Viewer.version_date+b.substring(c);if(this.isApplet)return this.showUrl(this.g.nmrUrlFormat+
b),"opening "+this.g.nmrUrlFormat}this.syncScript("true","*",0);this.syncScript(a+"Simulate:",".",0);return"sending request to JSpecView"},"~S");c(c$,"getHelp",function(a){0>this.g.helpPath.indexOf("?")?(0<a.length&&0!=a.indexOf("?")&&(a="?search="+JU.PT.rep(a," ","%20")),a+=(0==a.length?"?ver=":"&ver=")+JV.JC.majorVersion):a="&"+a;this.showUrl(this.g.helpPath+a)},"~S");c(c$,"getChemicalInfo",function(a,b,c){b=b.toLowerCase();var g="/";switch(";inchi;inchikey;stdinchi;stdinchikey;name;image;drawing;names;".indexOf(";"+
b+";")){case 0:g="I";break;case 6:g="K";break;case 15:g="T";break;case 24:g="S";break;case 36:g="M";break;case 41:case 47:g="2";break;case 55:g="N"}var f=this.setLoadFormat("_"+a,g,!1);if("2"==g)return this.fm.loadImage(f,"\u0001"+a,!1),f;"/"==g&&(f=JU.PT.isOneOf(b,";alc;cdxml;cerius;charmm;cif;cml;ctx;gjf;gromacs;hyperchem;jme;maestro;mol;mol2;sybyl2;mrv;pdb;sdf;sdf3000;sln;smiles;xyz")?f+("file?format="+b):f+JU.PT.rep(b," ","%20"));f=this.getFileAsString4(f,-1,!1,!1,!1,"file");"M"==g&&0<f.indexOf("\n")?
f=f.substring(0,f.indexOf("\n")):b.equals("jme")&&(f=this.getPropertyManager().fixJMEFormalCharges(c,f));return f},"~S,~S,JU.BS");c(c$,"addCommand",function(a){!this.autoExit&&(this.haveDisplay&&this.getPreserveState())&&this.commandHistory.addCommand(JU.PT.replaceAllCharacters(a,"\r\n\t"," "))},"~S");c(c$,"pushState",function(){!this.autoExit&&(this.haveDisplay&&this.getPreserveState())&&this.commandHistory.pushState(this.getStateInfo())});c(c$,"popState",function(){if(!this.autoExit&&this.haveDisplay&&
this.getPreserveState()){var a=this.commandHistory.popState();null!=a&&this.evalStringQuiet(a)}});c(c$,"removeCommand",function(){return this.commandHistory.removeCommand()});j(c$,"getSetHistory",function(a){return this.commandHistory.getSetHistory(a)},"~N");c(c$,"historyFind",function(a,b){return this.commandHistory.find(a,b)},"~S,~N");c(c$,"setHistory",function(a){this.commandHistory.getSetHistory(-2147483648);this.commandHistory.addCommand(this.getFileAsString4(a,-1,!1,!1,!0,null))},"~S");c(c$,
"getOutputChannel",function(a,b){return this.getOutputManager().getOutputChannel(a,b)},"~S,~A");j(c$,"writeTextFile",function(a,b){var c=new java.util.Hashtable;c.put("fileName",a);c.put("type","txt");c.put("text",b);return this.outputToFile(c)},"~S,~S");j(c$,"clipImageOrPasteText",function(a){return!this.haveAccess(JV.Viewer.ACCESS.ALL)?"no":this.getOutputManager().clipImageOrPasteText(a)},"~S");j(c$,"getClipboardText",function(){if(!this.haveAccess(JV.Viewer.ACCESS.ALL))return"no";try{return this.getOutputManager().getClipboardText()}catch(a){if(H(a,
Error))return J.i18n.GT.$("clipboard is not accessible -- use signed applet");throw a;}});c(c$,"processWriteOrCapture",function(a){return this.getOutputManager().processWriteOrCapture(a)},"java.util.Map");c(c$,"createZip",function(a,b,c){var g=this.getStateInfo(),f=c.get("data");null!=a&&c.put("fileName",a);c.put("type",b);c.put("text",g);n(f,Array)?c.put("scripts",f):n(f,JU.Lst)&&c.put("imageData",f);return this.getOutputManager().outputToFile(c)},"~S,~S,java.util.Map");j(c$,"outputToFile",function(a){return this.getOutputManager().outputToFile(a)},
"java.util.Map");c(c$,"setSyncTarget",function(a,b){switch(a){case 0:this.sm.syncingMouse=b;break;case 1:this.sm.syncingScripts=b;break;case 2:this.sm.syncSend(b?"GET_GRAPHICS":"SET_GRAPHICS_OFF","*",0);Float.isNaN(this.tm.stereoDegrees)&&this.setFloatProperty("stereoDegrees",-5);b&&(this.setBooleanProperty("_syncMouse",!1),this.setBooleanProperty("_syncScript",!1));return}!this.sm.syncingScripts&&!this.sm.syncingMouse&&this.setSync()},"~N,~B");j(c$,"syncScript",function(a,b,c){this.sm.syncScript(a,
b,c)},"~S,~S,~N");j(c$,"getModelIndexFromId",function(a){return this.ms.getModelIndexFromId(a)},"~S");c(c$,"setSyncDriver",function(a){this.sm.setSyncDriver(a)},"~N");c(c$,"setProteinType",function(a,b){this.ms.setProteinType(null==b?this.bsA():b,a)},"J.c.STR,JU.BS");c(c$,"getVanderwaalsMar",function(a){return this.defaultVdw===J.c.VDW.USER?this.userVdwMars[a]:JU.Elements.getVanderwaalsMar(a,this.defaultVdw)},"~N");c(c$,"getVanderwaalsMarType",function(a,b){if(null==b)b=this.defaultVdw;else switch(b){case J.c.VDW.AUTO:case J.c.VDW.AUTO_BABEL:case J.c.VDW.AUTO_JMOL:case J.c.VDW.AUTO_RASMOL:this.defaultVdw!==
J.c.VDW.AUTO&&(b=this.defaultVdw)}b===J.c.VDW.USER&&null==this.bsUserVdws&&(b=J.c.VDW.JMOL);return b===J.c.VDW.USER?this.userVdwMars[a&127]:JU.Elements.getVanderwaalsMar(a,b)},"~N,J.c.VDW");c(c$,"setVdwStr",function(a){a=J.c.VDW.getVdwType(a);null==a&&(a=J.c.VDW.AUTO);switch(a){case J.c.VDW.JMOL:case J.c.VDW.BABEL:case J.c.VDW.RASMOL:case J.c.VDW.AUTO:case J.c.VDW.USER:break;default:a=J.c.VDW.JMOL}a!==this.defaultVdw&&(a===J.c.VDW.USER&&null==this.bsUserVdws)&&this.setUserVdw(this.defaultVdw);this.defaultVdw=
a;this.g.setO("defaultVDW",a.getVdwLabel())},"~S");c(c$,"setUserVdw",function(a){this.userVdwMars=B(JU.Elements.elementNumberMax,0);this.userVdws=K(JU.Elements.elementNumberMax,0);this.bsUserVdws=new JU.BS;a===J.c.VDW.USER&&(a=J.c.VDW.JMOL);for(var b=1;b<JU.Elements.elementNumberMax;b++)this.userVdwMars[b]=JU.Elements.getVanderwaalsMar(b,a),this.userVdws[b]=this.userVdwMars[b]/1E3},"J.c.VDW");c(c$,"getDefaultVdwNameOrData",function(a,b,c){switch(a){case -2147483648:return this.defaultVdw.getVdwLabel();
case 2147483647:if(null==(c=this.bsUserVdws))return"";b=J.c.VDW.USER}if(null==b||b===J.c.VDW.AUTO)b=this.defaultVdw;b===J.c.VDW.USER&&null==this.bsUserVdws&&this.setUserVdw(this.defaultVdw);return this.getDataManager().getDefaultVdwNameOrData(b,c)},"~N,J.c.VDW,JU.BS");c(c$,"deleteAtoms",function(a,b){var c=null==a?-1:a.nextSetBit(0);if(0>c)return 0;this.clearModelDependentObjects();var g=this.ms.at[c];if(null==g)return 0;var f=g.mi;return!b?(this.sm.modifySend(c,g.mi,4,"deleting atom "+g.getAtomName()),
this.ms.deleteAtoms(a),g=this.slm.deleteAtoms(a),this.setTainted(!0),this.sm.modifySend(c,f,-4,"OK"),g):this.deleteModels(f,a)},"JU.BS,~B");c(c$,"deleteModels",function(a,b){this.clearModelDependentObjects();this.sm.modifySend(-1,a,5,"deleting model "+this.getModelNumberDotted(a));var c=this.am.cmi;this.setCurrentModelIndexClear(0,!1);this.am.setAnimationOn(!1);var g=JU.BSUtil.copy(this.slm.bsDeleted),f=null==b?JU.BSUtil.newAndSetBit(a):this.ms.getModelBS(b,!1),f=this.ms.deleteModels(f);if(null==
f)return this.setCurrentModelIndexClear(c,!1),0;this.slm.processDeletedModelAtoms(f);null!=this.eval&&this.eval.deleteAtomsInVariables(f);this.setAnimationRange(0,0);this.clearRepaintManager(-1);this.am.clear();this.am.initializePointers(1);this.setCurrentModelIndexClear(1<this.ms.mc?-1:0,1<this.ms.mc);this.hoverAtomIndex=-1;this.setFileLoadStatus(J.c.FIL.DELETED,null,null,null,null,null);this.refreshMeasures(!0);null!=g&&f.andNot(g);this.sm.modifySend(-1,a,-5,"OK");return JU.BSUtil.cardinalityOf(f)},
"~N,JU.BS");c(c$,"deleteBonds",function(a){var b=this.ms.bo[a.nextSetBit(0)].atom1.mi;this.sm.modifySend(-1,b,2,"delete bonds "+JU.Escape.eBond(a));this.ms.deleteBonds(a,!1);this.sm.modifySend(-1,b,-2,"OK")},"JU.BS");c(c$,"deleteModelAtoms",function(a,b,c,g){this.sm.modifySend(-1,a,1,"delete atoms "+JU.Escape.eBS(g));JU.BSUtil.deleteBits(this.tm.bsFrameOffsets,g);this.getDataManager().deleteModelAtoms(b,c,g);this.sm.modifySend(-1,a,-1,"OK")},"~N,~N,~N,JU.BS");c(c$,"getQuaternionFrame",function(){return this.g.quaternionFrame.charAt(2==
this.g.quaternionFrame.length?1:0)});c(c$,"loadImageData",function(a,b,c,g){null==a&&null!=b&&this.scriptEcho(b);null==c?this.setBackgroundImage(null==a?null:b,a):c.startsWith("\u0001")?this.sm.showImage(c,a):c.startsWith("\x00")?null!=a&&this.setWindowDimensions(K(-1,[this.apiPlatform.getImageWidth(a),this.apiPlatform.getImageHeight(a)])):(this.shm.loadShape(31),this.setShapeProperty(31,"text",b),null!=a&&this.setShapeProperty(31,"image",a));JV.Viewer.isJS&&null!=g&&(g.mustResumeEval=!0,this.eval.resumeEval(g));
return!1},"~O,~S,~S,~O");c(c$,"cd",function(a){null==a?a=".":0==a.length&&(this.setStringProperty("defaultDirectory",""),a=".");a=this.fm.getDefaultDirectory(a+(a.equals("=")?"":a.endsWith("/")?"X.spt":"/X.spt"));0<a.length&&this.setStringProperty("defaultDirectory",a);this.fm.getFilePath(a+"/",!0,!1).startsWith("file:/")&&JV.FileManager.setLocalPath(this,a,!1);return a},"~S");c(c$,"setErrorMessage",function(a,b){this.errorMessageUntranslated=b;null!=a&&this.eval.stopScriptThreads();return this.errorMessage=
a},"~S,~S");j(c$,"getErrorMessage",function(){return this.errorMessage});j(c$,"getErrorMessageUn",function(){return null==this.errorMessageUntranslated?this.errorMessage:this.errorMessageUntranslated});c(c$,"setShapeErrorState",function(a,b){this.currentShapeID=a;this.currentShapeState=b},"~N,~S");c(c$,"getShapeErrorState",function(){if(0>this.currentShapeID)return"";this.shm.releaseShape(this.currentShapeID);this.clearRepaintManager(this.currentShapeID);return JV.JC.getShapeClassName(this.currentShapeID,
!1)+" "+this.currentShapeState});c(c$,"handleError",function(a,b){try{b&&this.zapMsg(""+a),this.undoClear(),0==JU.Logger.getLogLevel()&&JU.Logger.setLogLevel(4),this.setCursor(0),this.setBooleanProperty("refreshing",!0),this.fm.setPathForAllFiles(""),JU.Logger.error("vwr handling error condition: "+a+"  "),this.notifyError("Error","doClear="+b+"; "+a,""+a)}catch(c){try{JU.Logger.error("Could not notify error "+a+": due to "+c)}catch(g){}}},"Error,~B");c(c$,"getFunctions",function(a){return a?JV.Viewer.staticFunctions:
this.localFunctions},"~B");c(c$,"removeFunction",function(a){a=a.toLowerCase();null!=this.getFunction(a)&&(JV.Viewer.staticFunctions.remove(a),this.localFunctions.remove(a))},"~S");c(c$,"getFunction",function(a){if(null==a)return null;a=(JV.Viewer.isStaticFunction(a)?JV.Viewer.staticFunctions:this.localFunctions).get(a);return null==a||null==a.geTokens()?null:a},"~S");c$.isStaticFunction=c(c$,"isStaticFunction",function(a){return a.startsWith("static_")},"~S");c(c$,"isFunction",function(a){return(JV.Viewer.isStaticFunction(a)?
JV.Viewer.staticFunctions:this.localFunctions).containsKey(a)},"~S");c(c$,"clearFunctions",function(){JV.Viewer.staticFunctions.clear();this.localFunctions.clear()});c(c$,"addFunction",function(a){var b=a.getName();(JV.Viewer.isStaticFunction(b)?JV.Viewer.staticFunctions:this.localFunctions).put(b,a)},"J.api.JmolScriptFunction");c(c$,"getFunctionCalls",function(a){return this.getStateCreator().getFunctionCalls(a)},"~S");c(c$,"checkPrivateKey",function(a){return a==this.privateKey},"~N");c(c$,"bindAction",
function(a,b){this.haveDisplay&&this.acm.bind(a,b)},"~S,~S");c(c$,"unBindAction",function(a,b){this.haveDisplay&&this.acm.unbindAction(a,b)},"~S,~S");c(c$,"calculateStruts",function(a,b){return this.ms.calculateStruts(null==a?this.bsA():a,null==b?this.bsA():b)},"JU.BS,JU.BS");c(c$,"getPreserveState",function(){return this.g.preserveState&&null!=this.scm});c(c$,"isKiosk",function(){return this.$isKiosk});c(c$,"hasFocus",function(){return this.haveDisplay&&(this.$isKiosk||this.apiPlatform.hasFocus(this.display))});
c(c$,"setFocus",function(){this.haveDisplay&&!this.apiPlatform.hasFocus(this.display)&&this.apiPlatform.requestFocusInWindow(this.display)});c(c$,"stopMinimization",function(){null!=this.minimizer&&this.minimizer.setProperty("stop",null)});c(c$,"clearMinimization",function(){null!=this.minimizer&&this.minimizer.setProperty("clear",null)});c(c$,"getMinimizationInfo",function(){return null==this.minimizer?"":this.minimizer.getProperty("log",0)});c(c$,"checkMinimization",function(){this.refreshMeasures(!0);
if(this.g.monitorEnergy){try{this.minimize(null,0,0,this.getFrameAtoms(),null,0,1)}catch(a){if(!H(a,Exception))throw a;}this.echoMessage(this.getP("_minimizationForceField")+" Energy = "+this.getP("_minimizationEnergy"))}});c(c$,"minimize",function(a,b,c,g,f,e,h){var j=1==(h&1),l=4==(h&4),n=0==(h&16),m=8==(h&8),p=this.g.forceField,q=this.getFrameAtoms();null==g?g=this.getModelUndeletedAtomsBitSet(this.getVisibleFramesBitSet().nextSetBit(0)):l||g.and(q);l&&(this.getAuxiliaryInfoForAtoms(g).put("dimension",
"3D"),q=g);0>=e&&(e=5);f=JU.BSUtil.copy(null==f?this.slm.getMotionFixedAtoms():f);var s=0<f.cardinality();s&&g.andNot(f);e=n?new JU.BS:this.ms.getAtomsWithinRadius(e,g,!0,null);e.andNot(g);s?f.and(e):f=e;f.and(q);h|=(s?2:0)|(this.getBooleanProperty("minimizationSilent")?1:0);if(!l||!this.getBoolean(603979962)){if(l)try{j||JU.Logger.info("Minimizing "+g.cardinality()+" atoms"),this.getMinimizer(!0).minimize(b,c,g,f,h,"UFF")}catch(t){if(H(t,Exception))JU.Logger.error("Minimization error: "+t.toString()),
t.printStackTrace();else throw t;}m&&(m=this.addHydrogens(g,h),l||g.or(m));m=g.cardinality();if(p.equals("MMFF")&&m>this.g.minimizationMaxAtoms)this.scriptStatusMsg("Too many atoms for minimization ("+m+">"+this.g.minimizationMaxAtoms+"); use 'set minimizationMaxAtoms' to increase this limit","minimization: too many atoms");else try{j||JU.Logger.info("Minimizing "+g.cardinality()+" atoms"),this.getMinimizer(!0).minimize(b,c,g,f,h,l?"MMFF":p),l&&(this.g.forceField="MMFF",this.setHydrogens(g),this.showString("Minimized by Jmol",
!1))}catch(u){if(H(u,JV.JmolAsyncException))null!=a&&a.loadFileResourceAsync(u.getFileName());else if(H(u,Exception))a=u,JU.Logger.error("Minimization error: "+a.toString()),a.printStackTrace();else throw u;}}},"J.api.JmolScriptEvaluator,~N,~N,JU.BS,JU.BS,~N,~N");c(c$,"setHydrogens",function(a){for(var b=B(1,0),b=this.ms.calculateHydrogens(a,b,null,2052),c=a.nextSetBit(0);0<=c;c=a.nextSetBit(c+1)){var g=b[c];if(!(null==g||0==g.length))for(var f=this.ms.at[c],e=f.bonds,h=0,j=0,l=f.getBondCount();h<
l;h++){var n=e[h].getOtherAtom(f);if(1==n.getAtomicAndIsotopeNumber()){var m=g[j++];this.ms.setAtomCoord(n.i,m.x,m.y,m.z)}}}this.ms.resetMolecules()},"JU.BS");c(c$,"setMotionFixedAtoms",function(a){this.slm.setMotionFixedAtoms(a)},"JU.BS");c(c$,"getMotionFixedAtoms",function(){return this.slm.getMotionFixedAtoms()});c(c$,"getAtomicPropertyState",function(a,b,c,g,f){this.getStateCreator().getAtomicPropertyStateBuffer(a,b,c,g,f)},"JU.SB,~N,JU.BS,~S,~A");c(c$,"getCenterAndPoints",function(a,b){return this.ms.getCenterAndPoints(a,
b)},"JU.Lst,~B");c(c$,"writeFileData",function(a,b,c,g){return this.getOutputManager().writeFileData(a,b,c,g)},"~S,~S,~N,~A");c(c$,"getPdbData",function(a,b,c,g,f,e){return this.getPropertyManager().getPdbData(a,b,null==c?this.bsA():c,g,f,e)},"~N,~S,JU.BS,~A,JU.OC,~B");c(c$,"getGroupsWithin",function(a,b){return this.ms.getGroupsWithin(a,b)},"~N,JU.BS");c(c$,"setShapeSize",function(a,b,c){null==c&&(c=this.bsA());this.shm.setShapeSizeBs(a,b,null,c)},"~N,~N,JU.BS");c(c$,"setShapeProperty",function(a,
b,c){0<=a&&this.shm.setShapePropertyBs(a,b,c,null)},"~N,~S,~O");c(c$,"getShapeProperty",function(a,b){return this.shm.getShapePropertyIndex(a,b,-2147483648)},"~N,~S");c(c$,"getShapePropertyAsInt",function(a,b){var c=this.getShapeProperty(a,b);return null==c||!n(c,Integer)?-2147483648:c.intValue()},"~N,~S");c(c$,"setModelVisibility",function(){null!=this.shm&&this.shm.setModelVisibility()});c(c$,"resetShapes",function(a){this.shm.resetShapes();a&&(this.shm.loadDefaultShapes(this.ms),this.clearRepaintManager(-1))},
"~B");c(c$,"setParallel",function(a){return this.$isParallel=this.g.multiProcessor&&a},"~B");c(c$,"isParallel",function(){return this.g.multiProcessor&&this.$isParallel});c(c$,"undoClear",function(){this.actionStates.clear();this.actionStatesRedo.clear()});c(c$,"undoMoveAction",function(a,b){this.getStateCreator().undoMoveAction(a,b)},"~N,~N");c(c$,"undoMoveActionClear",function(a,b,c){this.g.preserveState&&this.getStateCreator().undoMoveActionClear(a,b,c)},"~N,~N,~B");c(c$,"moveAtomWithHydrogens",
function(a,b,c,g,f){this.stopMinimization();if(null==f){var e=this.ms.at[a];f=JU.BSUtil.newAndSetBit(a);a=e.bonds;if(null!=a)for(var h=0;h<a.length;h++){var j=a[h].getOtherAtom(e);1==j.getElementNumber()&&f.set(j.i)}}this.moveSelected(b,c,g,-2147483648,-2147483648,f,!0,!0,0)},"~N,~N,~N,~N,JU.BS");c(c$,"isModelPDB",function(a){return this.ms.am[a].isBioModel},"~N");j(c$,"deleteMeasurement",function(a){this.setShapeProperty(6,"delete",Integer.$valueOf(a))},"~N");j(c$,"getSmiles",function(a){return this.getSmilesOpt(a,
-1,-1,null==a&&JU.Logger.debugging?131072:0,null)},"JU.BS");j(c$,"getOpenSmiles",function(a){return this.getSmilesOpt(a,-1,-1,5|(null==a&&JU.Logger.debugging?131072:0),"/openstrict///")},"JU.BS");c(c$,"getBioSmiles",function(a){return this.getSmilesOpt(a,-1,-1,24117248|(JU.Logger.debugging?131072:0),null)},"JU.BS");c(c$,"getSmilesOpt",function(a,b,c,g,f){var e=17825792==(g&17825792)?JV.Viewer.getJmolVersion()+" "+this.getModelName(this.am.cmi):f,h=this.ms.at;null==a&&(0>b||0>c?a=this.bsA():(1048576==
(g&1048576)&&(b>c&&(a=b,b=c,c=a),b=h[b].group.firstAtomIndex,c=h[c].group.lastAtomIndex),a=new JU.BS,a.setBits(b,c+1)));g|=this.isModel2D(a)?134217728:0;b=this.getSmilesMatcher();return JV.JC.isSmilesCanonical(f)?(g=b.getSmiles(h,this.ms.ac,a,"/noAromatic/",g),this.getChemicalInfo(g,"smiles",null).trim()):b.getSmiles(h,this.ms.ac,a,e,g)},"JU.BS,~N,~N,~N,~S");c(c$,"isModel2D",function(a){a=this.getModelForAtomIndex(a.nextSetBit(0));return null!=a&&"2D".equals(a.auxiliaryInfo.get("dimension"))},"JU.BS");
c(c$,"alert",function(a){this.prompt(a,null,null,!0)},"~S");c(c$,"prompt",function(a,b,c,g){return this.$isKiosk?"null":this.apiPlatform.prompt(a,b,c,g)},"~S,~S,~A,~B");c(c$,"dialogAsk",function(a,b){return prompt(a,b)},"~S,~S,java.util.Map");c(c$,"initializeExporter",function(a){var b=a.get("type").equals("JS");if(b){if(null!=this.jsExporter3D)return this.jsExporter3D.initializeOutput(this,this.privateKey,a),this.jsExporter3D}else{var c=a.get("fileName"),g=a.get("fullPath"),c=this.getOutputChannel(c,
g);if(null==c)return null;a.put("outputChannel",c)}c=J.api.Interface.getOption("export.Export3D",this,"export");if(null==c)return null;a=c.initializeExporter(this,this.privateKey,this.gdata,a);b&&null!=a&&(this.jsExporter3D=c);return null==a?null:c},"java.util.Map");c(c$,"getMouseEnabled",function(){return this.refreshing&&!this.creatingImage});j(c$,"calcAtomsMinMax",function(a,b){this.ms.calcAtomsMinMax(a,b)},"JU.BS,JU.BoxInfo");c(c$,"getObjectMap",function(a,b){switch(b){case "{":null!=this.getScriptManager()&&
(null!=this.definedAtomSets&&a.putAll(this.definedAtomSets),JS.T.getTokensType(a,2097152));break;case "$":case "0":this.shm.getObjectMap(a,"$"==b)}},"java.util.Map,~S");c(c$,"setPicked",function(a,b){var c=null,g=null;0<=a&&(b&&this.setPicked(-1,!1),this.g.setI("_atompicked",a),c=this.g.getParam("picked",!0),g=this.g.getParam("pickedList",!0));if(null==c||10!=c.tok)c=JS.SV.newV(10,new JU.BS),g=JS.SV.getVariableList(new JU.Lst),this.g.setUserVariable("picked",c),this.g.setUserVariable("pickedList",
g);0>a||(JS.SV.getBitSet(c,!1).set(a),c=g.pushPop(null,null),10==c.tok&&g.pushPop(null,c),(10!=c.tok||!c.value.get(a))&&g.pushPop(null,JS.SV.newV(10,JU.BSUtil.newAndSetBit(a))))},"~N,~B");j(c$,"runScript",function(a){return""+this.evaluateExpression(u(-1,[u(-1,[JS.T.tokenScript,JS.T.tokenLeftParen,JS.SV.newS(a),JS.T.tokenRightParen])]))},"~S");j(c$,"runScriptCautiously",function(a){var b=new JU.SB;try{if(null==this.getScriptManager())return null;this.eval.runScriptBuffer(a,b,!1)}catch(c){if(H(c,Exception))return this.eval.getErrorMessage();
throw c;}return b.toString()},"~S");c(c$,"setFrameDelayMs",function(a){this.ms.setFrameDelayMs(a,this.getVisibleFramesBitSet())},"~N");c(c$,"getBaseModelBitSet",function(){return this.ms.getModelAtomBitSetIncludingDeleted(this.getJDXBaseModelIndex(this.am.cmi),!0)});c(c$,"clearTimeouts",function(){null!=this.timeouts&&J.thread.TimeoutThread.clear(this.timeouts)});c(c$,"setTimeout",function(a,b,c){this.haveDisplay&&(!this.headless&&!this.autoExit)&&(null==a?this.clearTimeouts():(null==this.timeouts&&
(this.timeouts=new java.util.Hashtable),J.thread.TimeoutThread.setTimeout(this,this.timeouts,a,b,c)))},"~S,~N,~S");c(c$,"triggerTimeout",function(a){this.haveDisplay&&null!=this.timeouts&&J.thread.TimeoutThread.trigger(this.timeouts,a)},"~S");c(c$,"clearTimeout",function(a){this.setTimeout(a,0,null)},"~S");c(c$,"showTimeout",function(a){return this.haveDisplay?J.thread.TimeoutThread.showTimeout(this.timeouts,a):""},"~S");c(c$,"getOrCalcPartialCharges",function(a,b){null==a&&(a=this.bsA());a=JU.BSUtil.copy(a);
JU.BSUtil.andNot(a,b);JU.BSUtil.andNot(a,this.ms.bsPartialCharges);a.isEmpty()||this.calculatePartialCharges(a);return this.ms.getPartialCharges()},"JU.BS,JU.BS");c(c$,"calculatePartialCharges",function(a){if(null==a||a.isEmpty())a=this.getFrameAtoms();a.isEmpty()||(JU.Logger.info("Calculating MMFF94 partial charges for "+a.cardinality()+" atoms"),this.getMinimizer(!0).calculatePartialCharges(this.ms,a,null))},"JU.BS");c(c$,"setCurrentModelID",function(a){var b=this.am.cmi;0<=b&&this.ms.setInfo(b,
"modelID",a)},"~S");c(c$,"cacheClear",function(){this.fm.cacheClear();this.ligandModels=this.ligandModelSet=null;this.ms.clearCache()});c(c$,"cachePut",function(a,b){JU.Logger.info("Viewer cachePut "+a);this.fm.cachePut(a,b)},"~S,~O");c(c$,"cacheFileByName",function(a,b){return null==a?(this.cacheClear(),-1):this.fm.cacheFileByNameAdd(a,b)},"~S,~B");c(c$,"clearThreads",function(){null!=this.eval&&this.eval.stopScriptThreads();this.stopMinimization();this.tm.clearThreads();this.setAnimationOn(!1)});
c(c$,"getEvalContextAndHoldQueue",function(a){if(null==a||!JV.Viewer.isJS&&!this.testAsync)return null;a.pushContextDown("getEvalContextAndHoldQueue");a=a.getThisContext();a.setMustResume();this.queueOnHold=a.isJSThread=!0;return a},"J.api.JmolScriptEvaluator");c(c$,"getDefaultPropertyParam",function(a){return this.getPropertyManager().getDefaultPropertyParam(a)},"~N");c(c$,"getPropertyNumber",function(a){return this.getPropertyManager().getPropertyNumber(a)},"~S");c(c$,"checkPropertyParameter",function(a){return this.getPropertyManager().checkPropertyParameter(a)},
"~S");c(c$,"extractProperty",function(a,b,c){return this.getPropertyManager().extractProperty(a,b,c,null,!1)},"~O,~O,~N");c(c$,"addHydrogens",function(a,b){var c=1==(b&1),g=4==(b&4),f=null==a;null==a&&(a=this.getModelUndeletedAtomsBitSet(this.getVisibleFramesBitSet().length()-1));var e=new JU.BS;if(a.isEmpty())return e;var h=new JU.Lst,f=this.getAdditionalHydrogens(a,h,b|(f?256:0)),j=!1,j=this.g.appendNew;if(0<f.length){this.clearModelDependentObjects();try{e=g?this.ms.addHydrogens(h,f):this.addHydrogensInline(a,
h,f)}catch(l){if(H(l,Exception))Zystem.out.println(l.toString());else throw l;}j&&(this.g.appendNew=!0)}c||this.scriptStatus(J.i18n.GT.i(J.i18n.GT.$("{0} hydrogens added"),f.length));return e},"JU.BS,~N");c(c$,"addHydrogensInline",function(a,b,c){return null==this.getScriptManager()?null:this.scm.addHydrogensInline(a,b,c)},"JU.BS,JU.Lst,~A");j(c$,"evalFunctionFloat",function(a,b,c){return null==this.getScriptManager()?0:this.eval.evalFunctionFloat(a,b,c)},"~O,~O,~A");c(c$,"evalParallel",function(a,
b){this.displayLoadErrors=!1;var c=null!=this.getScriptManager()&&this.eval.evalParallel(a,null==b?this.shm:b);this.displayLoadErrors=!0;return c},"JS.ScriptContext,JV.ShapeManager");j(c$,"evaluateExpression",function(a){return null==this.getScriptManager()?null:this.eval.evaluateExpression(a,!1,!1)},"~O");j(c$,"evaluateExpressionAsVariable",function(a){return null==this.getScriptManager()?null:this.eval.evaluateExpression(a,!0,!1)},"~O");c(c$,"getAtomBitSet",function(a){if(n(a,JU.BS))return this.slm.excludeAtoms(a,
!1);this.getScriptManager();return this.getAtomBitSetEval(this.eval,a)},"~O");c(c$,"getScriptContext",function(a){return null==this.getScriptManager()?null:this.eval.getScriptContext(a)},"~S");c(c$,"getAtomDefs",function(a){var b=new JU.Lst,c;for(a=a.entrySet().iterator();a.hasNext()&&((c=a.next())||1);)n(c.getValue(),JU.BS)&&b.addLast("{"+c.getKey()+"} <"+c.getValue().cardinality()+" atoms>\n");c=b.size();a=Array(c);b.toArray(a);java.util.Arrays.sort(a);for(var b=new JU.SB,g=0;g<c;g++)b.append(a[g]);
return b.append("\n").toString()},"java.util.Map");c(c$,"setCGO",function(a){this.shm.loadShape(23);this.shm.setShapePropertyBs(23,"setCGO",a,null)},"JU.Lst");c(c$,"setModelSet",function(a){this.ms=this.mm.modelSet=a},"JM.ModelSet");c(c$,"setObjectProp",function(a,b){this.getScriptManager();null==a&&(a="*");return null==this.eval?null:this.eval.setObjectPropSafe(a,b)},"~S,~N");c(c$,"setDihedrals",function(a,b,c){null==b&&(b=this.ms.getBsBranches(a));this.ms.setDihedrals(a,b,c)},"~A,~A,~N");c(c$,"getChainID",
function(a,b){var c=this.chainMap.get(a);if(null!=c)return c.intValue();var g=a.charCodeAt(0);if(1<a.length)g=300+this.chainList.size();else if((b||this.chainCaseSpecified)&&97<=g&&122>=g)g+=159;if(256<=g){c=this.chainMap.get(a);if(null!=c)return c.intValue();this.chainCaseSpecified=(new Boolean(this.chainCaseSpecified|b)).valueOf();this.chainList.addLast(a)}c=Integer.$valueOf(g);this.chainMap.put(c,a);this.chainMap.put(a,c);return g},"~S,~B");c(c$,"getChainIDStr",function(a){return this.chainMap.get(Integer.$valueOf(a))},
"~N");c(c$,"getScriptQueueInfo",function(){return null!=this.scm&&this.scm.isQueueProcessing()?Boolean.TRUE:Boolean.FALSE});c(c$,"getNMRCalculation",function(){return null==this.nmrCalculation?(this.nmrCalculation=J.api.Interface.getOption("quantum.NMRCalculation",this,"script")).setViewer(this):this.nmrCalculation});c(c$,"getDistanceUnits",function(a){null==a&&(a=this.getDefaultMeasurementLabel(2));var b=a.indexOf("//");return 0>b?this.g.measureDistanceUnits:a.substring(b+2)},"~S");c(c$,"calculateFormalCharges",
function(a){return this.ms.fixFormalCharges(null==a?this.bsA():a)},"JU.BS");c(c$,"setModulation",function(a,b,c,g){g&&this.g.setO("_modt",JU.Escape.eP(c));this.ms.setModulation(null==a?this.getAllAtoms():a,b,c,g);this.refreshMeasures(!0)},"JU.BS,~B,JU.P3,~B");c(c$,"checkInMotion",function(a){switch(a){case 0:this.setTimeout("_SET_IN_MOTION_",0,null);break;case 1:this.inMotion||this.setTimeout("_SET_IN_MOTION_",2*this.g.hoverDelayMs,"!setInMotion");break;case 2:this.setInMotion(!0),this.refresh(3,
"timeoutThread set in motion")}},"~N");c(c$,"checkMotionRendering",function(a){if(!this.getInMotion(!0)&&!this.tm.spinOn&&!this.tm.vibrationOn&&!this.am.animationOn)return!0;if(this.g.wireframeRotation)return!1;var b=0;switch(a){case 1677721602:case 1140850689:b=2;break;case 1112150020:b=3;break;case 1112150021:b=4;break;case 1112152066:b=5;break;case 1073742018:b=6;break;case 603979967:b=7;break;case 603979786:b=8}return this.g.platformSpeed>=b},"~N");c(c$,"openExportChannel",function(a,b,c){return this.getOutputManager().openOutputChannel(a,
b,c,!1)},"~N,~S,~B");j(c$,"log",function(a){null!=a&&this.getOutputManager().logToFile(a)},"~S");c(c$,"getLogFileName",function(){return null==this.logFileName?"":this.logFileName});c(c$,"getCommands",function(a,b,c){return this.getStateCreator().getCommands(a,b,c)},"java.util.Map,java.util.Map,~S");c(c$,"allowCapture",function(){return!this.isApplet||this.isSignedApplet});c(c$,"compileExpr",function(a){var b=null==this.getScriptManager()?null:this.eval.evaluateExpression(a,!1,!0);return n(b,Array)?
b:u(-1,[JS.T.o(4,a)])},"~S");c(c$,"checkSelect",function(a,b){return null!=this.getScriptManager()&&this.eval.checkSelect(a,b)},"java.util.Map,~A");c(c$,"getAnnotationInfo",function(a,b,c){return this.getAnnotationParser(1111490587==c).getAnnotationInfo(this,a,b,c,this.am.cmi)},"JS.SV,~S,~N");c(c$,"getAtomValidation",function(a,b){return this.getAnnotationParser(!1).getAtomValidation(this,a,b)},"~S,JM.Atom");c(c$,"dragMinimizeAtom",function(a){this.stopMinimization();var b=this.getMotionFixedAtoms().isEmpty()?
this.ms.getAtoms(this.ms.isAtomPDB(a)?1086324742:1094713360,JU.BSUtil.newAndSetBit(a)):JU.BSUtil.setAll(this.ms.ac);try{this.minimize(null,2147483647,0,b,null,0,0)}catch(c){if(H(c,Exception)){if(this.async){var g=(R("JV.Viewer$1")?0:JV.Viewer.$Viewer$1$(),P(JV.Viewer$1,this,Ha("me",this,"iAtom",a)));setTimeout(function(){g.run()},100)}}else throw c;}},"~N");c(c$,"getJBR",function(){return null==this.jbr?this.jbr=J.api.Interface.getInterface("JM.BioResolver",this,"file").setViewer(this):this.jbr});
c(c$,"checkMenuUpdate",function(){null!=this.jmolpopup&&this.jmolpopup.jpiUpdateComputedMenus()});c(c$,"getChimeMessenger",function(){return null==this.jcm?this.jcm=J.api.Interface.getInterface("JV.ChimeMessenger",this,"script").set(this):this.jcm});c(c$,"getAuxiliaryInfoForAtoms",function(a){return this.ms.getAuxiliaryInfo(this.ms.getModelBS(this.getAtomBitSet(a),!1))},"~O");c(c$,"getJSJSONParser",function(){return null==this.jsonParser?this.jsonParser=J.api.Interface.getInterface("JU.JSJSONParser",
this,"script"):this.jsonParser});c(c$,"parseJSON",function(a){return null==a?null:(a=a.trim()).startsWith("{")?this.parseJSONMap(a):this.parseJSONArray(a)},"~S");c(c$,"parseJSONMap",function(a){return this.getJSJSONParser().parseMap(a,!0)},"~S");c(c$,"parseJSONArray",function(a){return this.getJSJSONParser().parse(a,!0)},"~S");c(c$,"getSymTemp",function(){return J.api.Interface.getSymmetry(this,"ms")});c(c$,"setWindowDimensions",function(a){this.resizeInnerPanel(A(a[0]),A(a[1]))},"~A");c(c$,"getTriangulator",
function(){return null==this.triangulator?this.triangulator=J.api.Interface.getUtil("Triangulator",this,"script"):this.triangulator});c(c$,"getCurrentModelAuxInfo",function(){return 0<=this.am.cmi?this.ms.getModelAuxiliaryInfo(this.am.cmi):null});c(c$,"startNBO",function(a){var b=new java.util.Hashtable;b.put("service","nbo");b.put("action","showPanel");b.put("options",a);this.sm.processService(b)},"~S");c(c$,"startPlugin",function(a){"nbo".equalsIgnoreCase(a)&&this.startNBO("all")},"~S");c(c$,"connectNBO",
function(a){0>this.am.cmi||this.getNBOParser().connectNBO(this.am.cmi,a)},"~S");c(c$,"getNBOParser",function(){return null==this.nboParser?this.nboParser=J.api.Interface.getInterface("J.adapter.readers.quantum.NBOParser",this,"script").set(this):this.nboParser});c(c$,"getNBOAtomLabel",function(a){return this.getNBOParser().getNBOAtomLabel(a)},"JM.Atom");c(c$,"calculateChirality",function(a){null==a&&(a=this.bsA());return this.ms.calculateChiralityForAtoms(a,!0)},"JU.BS");c(c$,"getSubstructureSetArray",
function(a,b,c){return this.getSmilesMatcher().getSubstructureSetArray(a,this.ms.at,this.ms.ac,b,null,c)},"~S,JU.BS,~N");c(c$,"getSubstructureSetArrayForNodes",function(a,b,c){return this.getSmilesMatcher().getSubstructureSetArray(a,b,b.length,null,null,c)},"~S,~A,~N");c(c$,"getSmilesAtoms",function(a){return this.getSmilesMatcher().getAtoms(a)},"~S");c(c$,"calculateChiralityForSmiles",function(a){try{return J.api.Interface.getSymmetry(this,"ms").calculateCIPChiralityForSmiles(this,a)}catch(b){if(H(b,
Exception))return null;throw b;}},"~S");c(c$,"getPdbID",function(){return this.ms.getInfo(this.am.cmi,"isPDB")===Boolean.TRUE?this.ms.getInfo(this.am.cmi,"pdbID"):null});c(c$,"getModelInfo",function(a){return this.ms.getInfo(this.am.cmi,a)},"~S");c(c$,"notifyScriptEditor",function(a,b){null!=this.scriptEditor&&this.scriptEditor.notify(a,b)},"~N,~A");c(c$,"sendConsoleMessage",function(a){null!=this.appConsole&&this.appConsole.sendConsoleMessage(a)},"~S");c(c$,"getModelkitProperty",function(a){return null==
this.modelkit?null:this.modelkit.getProperty(a)},"~S");c(c$,"setModelkitProperty",function(a,b){return null==this.modelkit?null:this.modelkit.setProperty(a,b)},"~S,~O");c(c$,"getSymmetryInfo",function(a,b,c,g,f,e,h,j,l,n,m){try{return this.getSymTemp().getSymmetryInfoAtom(this.ms,a,b,c,g,f,e,j,h,l,n,m)}catch(p){if(H(p,Exception))return Zystem.out.println("Exception in Viewer.getSymmetryInfo: "+p),JV.Viewer.isJS||p.printStackTrace(),null;throw p;}},"~N,~S,~N,JU.P3,JU.P3,JU.P3,~N,~S,~N,~N,~N");c(c$,
"setModelKitRotateBondIndex",function(a){null!=this.modelkit&&this.modelkit.setProperty("rotateBondIndex",Integer.$valueOf(a))},"~N");c(c$,"getMacro",function(a){if(null==this.macros||this.macros.isEmpty())try{var b=this.getAsciiFileOrNull(this.g.macroDirectory+"/macros.json");this.macros=this.parseJSON(b)}catch(c){if(H(c,Exception))this.macros=new java.util.Hashtable;else throw c;}if(null==a){var b=new JU.SB,g;for(a=this.macros.keySet().iterator();a.hasNext()&&((g=a.next())||1);){var f=this.macros.get(g);
b.append(g).append("\t").appendO(f).append("\n")}return b.toString()}a=a.toLowerCase();return this.macros.containsKey(a)?this.macros.get(a).get("path").toString():null},"~S");c(c$,"getInchi",function(a,b,c){try{var g=this.apiPlatform.getInChI();if(null==a&&null==b)return"";null!=b&&(b.startsWith("$")||b.startsWith(":")?b=this.getFileAsString4(b,-1,!1,!1,!0,"script"):!b.startsWith("InChI=")&&0>b.indexOf(" ")&&(b=this.getFileAsString4("$"+b,-1,!1,!1,!0,"script")));return g.getInchi(this,a,b,c)}catch(f){return""}},
"JU.BS,~S,~S");c(c$,"getConsoleFontScale",function(){return this.consoleFontScale});c(c$,"setConsoleFontScale",function(a){this.consoleFontScale=a},"~N");c(c$,"confirm",function(a,b){return this.apiPlatform.confirm(a,b)},"~S,~S");M(self.c$);c$=D(JV.Viewer,"ACCESS",Enum);G(c$,"NONE",0,[]);G(c$,"READSPT",1,[]);G(c$,"ALL",2,[]);c$=L();c$.$Viewer$1$=function(){M(self.c$);c$=da(JV,"Viewer$1",null,Runnable);j(c$,"run",function(){this.f$.me.dragMinimizeAtom(this.f$.iAtom)});c$=L()};E(c$,"nullDeletedAtoms",
!1);self.Jmol&&Jmol.extend&&Jmol.extend("vwr",JV.Viewer.prototype);E(c$,"isJS",!1,"isSwingJS",!1,"appletDocumentBase","","appletCodeBase","","appletIdiomaBase",null,"jsDocumentBase","","jmolObject",null);c$.strJavaVendor=c$.prototype.strJavaVendor="Java: "+Zystem.getProperty("java.vendor","j2s");c$.strOSName=c$.prototype.strOSName=Zystem.getProperty("os.name","");c$.strJavaVersion=c$.prototype.strJavaVersion="Java "+Zystem.getProperty("java.version","");E(c$,"version_date",null,"REFRESH_REPAINT",
1,"REFRESH_SYNC",2,"REFRESH_SYNC_MASK",3,"REFRESH_REPAINT_NO_MOTION_ONLY",6,"REFRESH_SEND_WEBGL_NEW_ORIENTATION",7,"SYNC_GRAPHICS_MESSAGE","GET_GRAPHICS","SYNC_NO_GRAPHICS_MESSAGE","SET_GRAPHICS_OFF");c$.staticFunctions=c$.prototype.staticFunctions=new java.util.Hashtable;E(c$,"MIN_SILENT",1,"MIN_HAVE_FIXED",2,"MIN_QUICK",4,"MIN_ADDH",8,"MIN_NO_RANGE",16,"nProcessors",1)})}ea._coreLoaded=!0})(Clazz,Clazz.getClassName,Clazz.newLongArray,Clazz.doubleToByte,Clazz.doubleToInt,Clazz.doubleToLong,Clazz.declarePackage,
Clazz.instanceOf,Clazz.load,Clazz.instantialize,Clazz.decorateAsClass,Clazz.floatToInt,Clazz.floatToLong,Clazz.makeConstructor,Clazz.defineEnumConstant,Clazz.exceptionOf,Clazz.newIntArray,Clazz.defineStatics,Clazz.newFloatArray,Clazz.declareType,Clazz.prepareFields,Clazz.superConstructor,Clazz.newByteArray,Clazz.declareInterface,Clazz.p0p,Clazz.pu$h,Clazz.newShortArray,Clazz.innerTypeInstance,Clazz.isClassDefined,Clazz.prepareCallback,Clazz.newArray,Clazz.castNullAs,Clazz.floatToShort,Clazz.superCall,
Clazz.decorateAsType,Clazz.newBooleanArray,Clazz.newCharArray,Clazz.implementOf,Clazz.newDoubleArray,Clazz.overrideConstructor,Clazz.clone,Clazz.doubleToShort,Clazz.getInheritedLevel,Clazz.getParamsType,Clazz.isAF,Clazz.isAB,Clazz.isAI,Clazz.isAS,Clazz.isASS,Clazz.isAP,Clazz.isAFloat,Clazz.isAII,Clazz.isAFF,Clazz.isAFFF,Clazz.tryToSearchAndExecute,Clazz.getStackTrace,Clazz.inheritArgs,Clazz.alert,Clazz.defineMethod,Clazz.overrideMethod,Clazz.declareAnonymous,Clazz.cloneFinals);
